/* Generated by Cython 0.29.28 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [
            "/home/zsherman/anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/core/include/numpy/arrayobject.h",
            "/home/zsherman/anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/core/include/numpy/arrayscalars.h",
            "/home/zsherman/anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/core/include/numpy/ndarrayobject.h",
            "/home/zsherman/anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/core/include/numpy/ndarraytypes.h",
            "/home/zsherman/anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/core/include/numpy/ufuncobject.h"
        ],
        "include_dirs": [
            "/home/zsherman/anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/core/include"
        ],
        "name": "pyart.io._sigmetfile",
        "sources": [
            "pyart/io/_sigmetfile.pyx"
        ]
    },
    "module_name": "pyart.io._sigmetfile"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_28"
#define CYTHON_HEX_VERSION 0x001D1CF0
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0 || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 0
  #elif !defined(CYTHON_FAST_THREAD_STATE)
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL (PY_VERSION_HEX < 0x030B00A1)
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 0
  #elif !defined(CYTHON_USE_EXC_INFO_STACK)
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_MAJOR_VERSION < 3
    #include "longintrepr.h"
  #endif
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_DefaultClassType PyType_Type
#if PY_VERSION_HEX >= 0x030B00A1
    static CYTHON_INLINE PyCodeObject* __Pyx_PyCode_New(int a, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
        PyObject *kwds=NULL, *argcount=NULL, *posonlyargcount=NULL, *kwonlyargcount=NULL;
        PyObject *nlocals=NULL, *stacksize=NULL, *flags=NULL, *replace=NULL, *call_result=NULL, *empty=NULL;
        const char *fn_cstr=NULL;
        const char *name_cstr=NULL;
        PyCodeObject* co=NULL;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        if (!(kwds=PyDict_New())) goto end;
        if (!(argcount=PyLong_FromLong(a))) goto end;
        if (PyDict_SetItemString(kwds, "co_argcount", argcount) != 0) goto end;
        if (!(posonlyargcount=PyLong_FromLong(0))) goto end;
        if (PyDict_SetItemString(kwds, "co_posonlyargcount", posonlyargcount) != 0) goto end;
        if (!(kwonlyargcount=PyLong_FromLong(k))) goto end;
        if (PyDict_SetItemString(kwds, "co_kwonlyargcount", kwonlyargcount) != 0) goto end;
        if (!(nlocals=PyLong_FromLong(l))) goto end;
        if (PyDict_SetItemString(kwds, "co_nlocals", nlocals) != 0) goto end;
        if (!(stacksize=PyLong_FromLong(s))) goto end;
        if (PyDict_SetItemString(kwds, "co_stacksize", stacksize) != 0) goto end;
        if (!(flags=PyLong_FromLong(f))) goto end;
        if (PyDict_SetItemString(kwds, "co_flags", flags) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_code", code) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_consts", c) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_names", n) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_varnames", v) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_freevars", fv) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_cellvars", cell) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_linetable", lnos) != 0) goto end;
        if (!(fn_cstr=PyUnicode_AsUTF8AndSize(fn, NULL))) goto end;
        if (!(name_cstr=PyUnicode_AsUTF8AndSize(name, NULL))) goto end;
        if (!(co = PyCode_NewEmpty(fn_cstr, name_cstr, fline))) goto end;
        if (!(replace = PyObject_GetAttrString((PyObject*)co, "replace"))) goto cleanup_code_too;
        if (!(empty = PyTuple_New(0))) goto cleanup_code_too; // unfortunately __pyx_empty_tuple isn't available here
        if (!(call_result = PyObject_Call(replace, empty, kwds))) goto cleanup_code_too;
        Py_XDECREF((PyObject*)co);
        co = (PyCodeObject*)call_result;
        call_result = NULL;
        if (0) {
            cleanup_code_too:
            Py_XDECREF((PyObject*)co);
            co = NULL;
        }
        end:
        Py_XDECREF(kwds);
        Py_XDECREF(argcount);
        Py_XDECREF(posonlyargcount);
        Py_XDECREF(kwonlyargcount);
        Py_XDECREF(nlocals);
        Py_XDECREF(stacksize);
        Py_XDECREF(replace);
        Py_XDECREF(call_result);
        Py_XDECREF(empty);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return co;
    }
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if defined(PyUnicode_IS_READY)
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #else
  #define __Pyx_PyUnicode_READY(op)       (0)
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if defined(PyUnicode_IS_READY) && defined(PyUnicode_GET_SIZE)
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
  #endif
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsHash_t
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__pyart__io___sigmetfile
#define __PYX_HAVE_API__pyart__io___sigmetfile
/* Early includes */
#include <string.h>
#include <stdio.h>
#include "numpy/arrayobject.h"
#include "numpy/ndarrayobject.h"
#include "numpy/ndarraytypes.h"
#include "numpy/arrayscalars.h"
#include "numpy/ufuncobject.h"

    /* NumPy API declarations from "numpy/__init__.pxd" */
    
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;

/* Header.proto */
#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif defined(_Complex_I)
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif


static const char *__pyx_f[] = {
  "pyart/io/_sigmetfile.pyx",
  "stringsource",
  "__init__.pxd",
  "type.pxd",
};
/* BufferFormatStructs.proto */
#define IS_UNSIGNED(type) (((type) -1) > 0)
struct __Pyx_StructField_;
#define __PYX_BUF_FLAGS_PACKED_STRUCT (1 << 0)
typedef struct {
  const char* name;
  struct __Pyx_StructField_* fields;
  size_t size;
  size_t arraysize[8];
  int ndim;
  char typegroup;
  char is_unsigned;
  int flags;
} __Pyx_TypeInfo;
typedef struct __Pyx_StructField_ {
  __Pyx_TypeInfo* type;
  const char* name;
  size_t offset;
} __Pyx_StructField;
typedef struct {
  __Pyx_StructField* field;
  size_t parent_offset;
} __Pyx_BufFmt_StackElem;
typedef struct {
  __Pyx_StructField root;
  __Pyx_BufFmt_StackElem* head;
  size_t fmt_offset;
  size_t new_count, enc_count;
  size_t struct_alignment;
  int is_complex;
  char enc_type;
  char new_packmode;
  char enc_packmode;
  char is_valid_array;
} __Pyx_BufFmt_Context;


/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":690
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 */
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":691
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
 */
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":692
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * #ctypedef npy_int96      int96_t
 */
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":693
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_int96      int96_t
 * #ctypedef npy_int128     int128_t
 */
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":697
 * #ctypedef npy_int128     int128_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 */
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":698
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
 */
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":699
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * #ctypedef npy_uint96     uint96_t
 */
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":700
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_uint96     uint96_t
 * #ctypedef npy_uint128    uint128_t
 */
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":704
 * #ctypedef npy_uint128    uint128_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
 */
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":705
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
 */
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":714
 * # The int types are mapped a bit surprising --
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t
 */
typedef npy_long __pyx_t_5numpy_int_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":715
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   longlong_t
 * 
 */
typedef npy_longlong __pyx_t_5numpy_long_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":716
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_ulong      uint_t
 */
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":718
 * ctypedef npy_longlong   longlong_t
 * 
 * ctypedef npy_ulong      uint_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t
 */
typedef npy_ulong __pyx_t_5numpy_uint_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":719
 * 
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 */
typedef npy_ulonglong __pyx_t_5numpy_ulong_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":720
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
 */
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":722
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
 */
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":723
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
 */
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":725
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
 */
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":726
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
 */
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":727
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cfloat      cfloat_t
 */
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;
/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif
static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);

/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif
static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);


/*--- Type declarations ---*/
struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":729
 * ctypedef npy_longdouble longdouble_t
 * 
 * ctypedef npy_cfloat      cfloat_t             # <<<<<<<<<<<<<<
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t
 */
typedef npy_cfloat __pyx_t_5numpy_cfloat_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":730
 * 
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t             # <<<<<<<<<<<<<<
 * ctypedef npy_clongdouble clongdouble_t
 * 
 */
typedef npy_cdouble __pyx_t_5numpy_cdouble_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":731
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cdouble     complex_t
 */
typedef npy_clongdouble __pyx_t_5numpy_clongdouble_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":733
 * ctypedef npy_clongdouble clongdouble_t
 * 
 * ctypedef npy_cdouble     complex_t             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 */
typedef npy_cdouble __pyx_t_5numpy_complex_t;
struct __pyx_opt_args_5pyart_2io_11_sigmetfile_10SigmetFile__incr_rbuf_pos;

/* "pyart/io/_sigmetfile.pyx":374
 *         return 0
 * 
 *     cdef int _incr_rbuf_pos(self, int incr=1):             # <<<<<<<<<<<<<<
 *         """
 *         Increment the record buffer position, load a new record if needed.
 */
struct __pyx_opt_args_5pyart_2io_11_sigmetfile_10SigmetFile__incr_rbuf_pos {
  int __pyx_n;
  int incr;
};

/* "pyart/io/_sigmetfile.pyx":16
 * 
 * 
 * cdef class SigmetFile:             # <<<<<<<<<<<<<<
 *     """
 *     A class for accessing data from Sigmet (IRIS) product files.
 */
struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile {
  PyObject_HEAD
  struct __pyx_vtabstruct_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_vtab;
  PyObject *debug;
  PyObject *product_hdr;
  PyObject *ingest_header;
  PyObject *ingest_data_headers;
  PyObject *data_types;
  PyObject *data_type_names;
  PyObject *ndata_types;
  PyObject *_fh;
  PyObject *_raw_product_bhdrs;
  PyArrayObject *_rbuf;
  __pyx_t_5numpy_int16_t *_rbuf_p;
  int _rbuf_pos;
  int _record_number;
};



struct __pyx_vtabstruct_5pyart_2io_11_sigmetfile_SigmetFile {
  int (*_get_ray)(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *, int, PyArrayObject *);
  int (*_incr_rbuf_pos)(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *, struct __pyx_opt_args_5pyart_2io_11_sigmetfile_10SigmetFile__incr_rbuf_pos *__pyx_optional_args);
  int (*_load_record)(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *);
};
static struct __pyx_vtabstruct_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_vtabptr_5pyart_2io_11_sigmetfile_SigmetFile;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
#if CYTHON_FAST_PYCALL
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif // CYTHON_FAST_PYCALL
#endif

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_AddObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* PyObjectGetMethod.proto */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* append.proto */
static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x);

/* PyFloatBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyFloat_TrueDivideCObj(PyObject *op1, PyObject *op2, double floatval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyFloat_TrueDivideCObj(op1, op2, floatval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceTrueDivide(op1, op2) : PyNumber_TrueDivide(op1, op2))
#endif

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_SubtractObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_SubtractObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceSubtract(op1, op2) : PyNumber_Subtract(op1, op2))
#endif

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_TrueDivideObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_TrueDivideObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceTrueDivide(op1, op2) : PyNumber_TrueDivide(op1, op2))
#endif

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddCObj(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_AddCObj(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* PyIntCompare.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* IsLittleEndian.proto */
static CYTHON_INLINE int __Pyx_Is_Little_Endian(void);

/* BufferFormatCheck.proto */
static const char* __Pyx_BufFmt_CheckString(__Pyx_BufFmt_Context* ctx, const char* ts);
static void __Pyx_BufFmt_Init(__Pyx_BufFmt_Context* ctx,
                              __Pyx_BufFmt_StackElem* stack,
                              __Pyx_TypeInfo* type);

/* BufferGetAndValidate.proto */
#define __Pyx_GetBufferAndValidate(buf, obj, dtype, flags, nd, cast, stack)\
    ((obj == Py_None || obj == NULL) ?\
    (__Pyx_ZeroBuffer(buf), 0) :\
    __Pyx__GetBufferAndValidate(buf, obj, dtype, flags, nd, cast, stack))
static int  __Pyx__GetBufferAndValidate(Py_buffer* buf, PyObject* obj,
    __Pyx_TypeInfo* dtype, int flags, int nd, int cast, __Pyx_BufFmt_StackElem* stack);
static void __Pyx_ZeroBuffer(Py_buffer* buf);
static CYTHON_INLINE void __Pyx_SafeReleaseBuffer(Py_buffer* info);
static Py_ssize_t __Pyx_minusones[] = { -1, -1, -1, -1, -1, -1, -1, -1 };
static Py_ssize_t __Pyx_zeros[] = { 0, 0, 0, 0, 0, 0, 0, 0 };

/* BufferIndexError.proto */
static void __Pyx_RaiseBufferIndexError(int axis);

#define __Pyx_BufPtrStrided1d(type, buf, i0, s0) (type)((char*)buf + i0 * s0)
/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AndObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_AndObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceAnd(op1, op2) : PyNumber_And(op1, op2))
#endif

/* PyFloatBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyFloat_TrueDivideObjC(PyObject *op1, PyObject *op2, double floatval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyFloat_TrueDivideObjC(op1, op2, floatval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceTrueDivide(op1, op2) : PyNumber_TrueDivide(op1, op2))
#endif

/* PyFloatBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyFloat_SubtractObjC(PyObject *op1, PyObject *op2, double floatval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyFloat_SubtractObjC(op1, op2, floatval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceSubtract(op1, op2) : PyNumber_Subtract(op1, op2))
#endif

/* SliceObject.proto */
#define __Pyx_PyObject_DelSlice(obj, cstart, cstop, py_start, py_stop, py_slice, has_cstart, has_cstop, wraparound)\
    __Pyx_PyObject_SetSlice(obj, (PyObject*)NULL, cstart, cstop, py_start, py_stop, py_slice, has_cstart, has_cstop, wraparound)
static CYTHON_INLINE int __Pyx_PyObject_SetSlice(
        PyObject* obj, PyObject* value, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* PyFloatBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyFloat_SubtractCObj(PyObject *op1, PyObject *op2, double floatval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyFloat_SubtractCObj(op1, op2, floatval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceSubtract(op1, op2) : PyNumber_Subtract(op1, op2))
#endif

#define __Pyx_BufPtrStrided2d(type, buf, i0, s0, i1, s1) (type)((char*)buf + i0 * s0 + i1 * s1)
/* PyIntCompare.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_NeObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* BufferStructDeclare.proto */
typedef struct {
  Py_ssize_t shape, strides, suboffsets;
} __Pyx_Buf_DimInfo;
typedef struct {
  size_t refcount;
  Py_buffer pybuffer;
} __Pyx_Buffer;
typedef struct {
  __Pyx_Buffer *rcbuffer;
  char *data;
  __Pyx_Buf_DimInfo diminfo[8];
} __Pyx_LocalBuf_ND;

#if PY_MAJOR_VERSION < 3
    static int __Pyx_GetBuffer(PyObject *obj, Py_buffer *view, int flags);
    static void __Pyx_ReleaseBuffer(Py_buffer *view);
#else
    #define __Pyx_GetBuffer PyObject_GetBuffer
    #define __Pyx_ReleaseBuffer PyBuffer_Release
#endif


/* GCCDiagnostics.proto */
#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* RealImag.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if defined(__cplusplus) && CYTHON_CCOMPLEX\
        && (defined(_WIN32) || defined(__clang__) || (defined(__GNUC__) && (__GNUC__ >= 5 || __GNUC__ == 4 && __GNUC_MINOR__ >= 4 )) || __cplusplus >= 201103)
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_float(a, b)   ((a)==(b))
    #define __Pyx_c_sum_float(a, b)  ((a)+(b))
    #define __Pyx_c_diff_float(a, b) ((a)-(b))
    #define __Pyx_c_prod_float(a, b) ((a)*(b))
    #define __Pyx_c_quot_float(a, b) ((a)/(b))
    #define __Pyx_c_neg_float(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_float(z) ((z)==(float)0)
    #define __Pyx_c_conj_float(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (::std::abs(z))
        #define __Pyx_c_pow_float(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_float(z) ((z)==0)
    #define __Pyx_c_conj_float(z)    (conjf(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (cabsf(z))
        #define __Pyx_c_pow_float(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_double(a, b)   ((a)==(b))
    #define __Pyx_c_sum_double(a, b)  ((a)+(b))
    #define __Pyx_c_diff_double(a, b) ((a)-(b))
    #define __Pyx_c_prod_double(a, b) ((a)*(b))
    #define __Pyx_c_quot_double(a, b) ((a)/(b))
    #define __Pyx_c_neg_double(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_double(z) ((z)==(double)0)
    #define __Pyx_c_conj_double(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (::std::abs(z))
        #define __Pyx_c_pow_double(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_double(z) ((z)==0)
    #define __Pyx_c_conj_double(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (cabs(z))
        #define __Pyx_c_pow_double(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_short(short value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static int __pyx_f_5pyart_2io_11_sigmetfile_10SigmetFile__get_ray(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, int __pyx_v_nbins, PyArrayObject *__pyx_v_out); /* proto*/
static int __pyx_f_5pyart_2io_11_sigmetfile_10SigmetFile__incr_rbuf_pos(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, struct __pyx_opt_args_5pyart_2io_11_sigmetfile_10SigmetFile__incr_rbuf_pos *__pyx_optional_args); /* proto*/
static int __pyx_f_5pyart_2io_11_sigmetfile_10SigmetFile__load_record(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self); /* proto*/

/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'numpy' */

/* Module declarations from 'numpy' */
static PyTypeObject *__pyx_ptype_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_5numpy_flatiter = 0;
static PyTypeObject *__pyx_ptype_5numpy_broadcast = 0;
static PyTypeObject *__pyx_ptype_5numpy_ndarray = 0;
static PyTypeObject *__pyx_ptype_5numpy_generic = 0;
static PyTypeObject *__pyx_ptype_5numpy_number = 0;
static PyTypeObject *__pyx_ptype_5numpy_integer = 0;
static PyTypeObject *__pyx_ptype_5numpy_signedinteger = 0;
static PyTypeObject *__pyx_ptype_5numpy_unsignedinteger = 0;
static PyTypeObject *__pyx_ptype_5numpy_inexact = 0;
static PyTypeObject *__pyx_ptype_5numpy_floating = 0;
static PyTypeObject *__pyx_ptype_5numpy_complexfloating = 0;
static PyTypeObject *__pyx_ptype_5numpy_flexible = 0;
static PyTypeObject *__pyx_ptype_5numpy_character = 0;
static PyTypeObject *__pyx_ptype_5numpy_ufunc = 0;

/* Module declarations from 'cython' */

/* Module declarations from 'pyart.io._sigmetfile' */
static PyTypeObject *__pyx_ptype_5pyart_2io_11_sigmetfile_SigmetFile = 0;
static PyObject *__pyx_f_5pyart_2io_11_sigmetfile__mask_gates_not_collected(PyArrayObject *, PyArrayObject *); /*proto*/
static __Pyx_TypeInfo __Pyx_TypeInfo_nn___pyx_t_5numpy_int16_t = { "int16_t", NULL, sizeof(__pyx_t_5numpy_int16_t), { 0 }, 0, IS_UNSIGNED(__pyx_t_5numpy_int16_t) ? 'U' : 'I', IS_UNSIGNED(__pyx_t_5numpy_int16_t), 0 };
static __Pyx_TypeInfo __Pyx_TypeInfo_nn___pyx_t_5numpy_uint8_t = { "uint8_t", NULL, sizeof(__pyx_t_5numpy_uint8_t), { 0 }, 0, IS_UNSIGNED(__pyx_t_5numpy_uint8_t) ? 'U' : 'I', IS_UNSIGNED(__pyx_t_5numpy_uint8_t), 0 };
#define __Pyx_MODULE_NAME "pyart.io._sigmetfile"
extern int __pyx_module_is_main_pyart__io___sigmetfile;
int __pyx_module_is_main_pyart__io___sigmetfile = 0;

/* Implementation of 'pyart.io._sigmetfile' */
static PyObject *__pyx_builtin_open;
static PyObject *__pyx_builtin_xrange;
static PyObject *__pyx_builtin_enumerate;
static PyObject *__pyx_builtin_sum;
static PyObject *__pyx_builtin_print;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_zip;
static PyObject *__pyx_builtin_ImportError;
static const char __pyx_k_2[] = "2";
static const char __pyx_k_B[] = "B";
static const char __pyx_k_H[] = "H";
static const char __pyx_k_I[] = "I";
static const char __pyx_k_b[] = "b";
static const char __pyx_k_c[] = "c";
static const char __pyx_k_d[] = "d";
static const char __pyx_k_f[] = "f";
static const char __pyx_k_h[] = "h";
static const char __pyx_k_i[] = "i";
static const char __pyx_k_l[] = "l";
static const char __pyx_k_1s[] = "1s";
static const char __pyx_k_2s[] = "2s";
static const char __pyx_k_4s[] = "4s";
static const char __pyx_k_6s[] = "6s";
static const char __pyx_k_8s[] = "8s";
static const char __pyx_k_i4[] = "i4";
static const char __pyx_k_ma[] = "ma";
static const char __pyx_k_np[] = "np";
static const char __pyx_k_rb[] = "rb";
static const char __pyx_k_12s[] = "12s";
static const char __pyx_k_16s[] = "16s";
static const char __pyx_k_18s[] = "18s";
static const char __pyx_k_24s[] = "24s";
static const char __pyx_k_28s[] = "28s";
static const char __pyx_k_32s[] = "32s";
static const char __pyx_k_36s[] = "36s";
static const char __pyx_k_40s[] = "40s";
static const char __pyx_k_48s[] = "48s";
static const char __pyx_k_52s[] = "52s";
static const char __pyx_k_80s[] = "80s";
static const char __pyx_k_94s[] = "94s";
static const char __pyx_k_DBT[] = "DBT";
static const char __pyx_k_DBZ[] = "DBZ";
static const char __pyx_k_KDP[] = "KDP";
static const char __pyx_k_RAW[] = "RAW";
static const char __pyx_k_SQI[] = "SQI";
static const char __pyx_k_VEL[] = "VEL";
static const char __pyx_k_ZDR[] = "ZDR";
static const char __pyx_k__15[] = "";
static const char __pyx_k_az0[] = "az0";
static const char __pyx_k_az1[] = "az1";
static const char __pyx_k_bit[] = "bit";
static const char __pyx_k_day[] = "day";
static const char __pyx_k_dic[] = "dic";
static const char __pyx_k_el0[] = "el0";
static const char __pyx_k_el1[] = "el1";
static const char __pyx_k_exp[] = "exp";
static const char __pyx_k_fmt[] = "fmt";
static const char __pyx_k_idh[] = "idh";
static const char __pyx_k_key[] = "key";
static const char __pyx_k_mod[] = "mod";
static const char __pyx_k_out[] = "out";
static const char __pyx_k_prf[] = "prf";
static const char __pyx_k_sum[] = "sum";
static const char __pyx_k_zip[] = "zip";
static const char __pyx_k_112s[] = "112s";
static const char __pyx_k_115s[] = "115s";
static const char __pyx_k_120s[] = "120s";
static const char __pyx_k_128s[] = "128s";
static const char __pyx_k_136s[] = "136s";
static const char __pyx_k_160s[] = "160s";
static const char __pyx_k_184s[] = "184s";
static const char __pyx_k_198s[] = "198s";
static const char __pyx_k_200s[] = "200s";
static const char __pyx_k_204s[] = "204s";
static const char __pyx_k_208s[] = "208s";
static const char __pyx_k_228s[] = "228s";
static const char __pyx_k_256s[] = "256s";
static const char __pyx_k_308s[] = "308s";
static const char __pyx_k_320s[] = "320s";
static const char __pyx_k_480s[] = "480s";
static const char __pyx_k_720s[] = "720s";
static const char __pyx_k_732s[] = "732s";
static const char __pyx_k_920s[] = "920s";
static const char __pyx_k_BIN1[] = "BIN1";
static const char __pyx_k_BIN2[] = "BIN2";
static const char __pyx_k_BIN4[] = "BIN4";
static const char __pyx_k_CSP8[] = "CSP8";
static const char __pyx_k_DBT2[] = "DBT2";
static const char __pyx_k_DBZ2[] = "DBZ2";
static const char __pyx_k_DBZC[] = "DBZC";
static const char __pyx_k_FLT4[] = "FLT4";
static const char __pyx_k_FLT8[] = "FLT8";
static const char __pyx_k_KDP2[] = "KDP2";
static const char __pyx_k_LDRH[] = "LDRH";
static const char __pyx_k_LDRV[] = "LDRV";
static const char __pyx_k_LOG8[] = "LOG8";
static const char __pyx_k_PHIH[] = "PHIH";
static const char __pyx_k_PHIV[] = "PHIV";
static const char __pyx_k_PMI8[] = "PMI8";
static const char __pyx_k_RHOH[] = "RHOH";
static const char __pyx_k_RHOV[] = "RHOV";
static const char __pyx_k_SNR8[] = "SNR8";
static const char __pyx_k_SQI2[] = "SQI2";
static const char __pyx_k_USER[] = "USER";
static const char __pyx_k_VEL2[] = "VEL2";
static const char __pyx_k_VELC[] = "VELC";
static const char __pyx_k_VIL2[] = "VIL2";
static const char __pyx_k_XHDR[] = "XHDR";
static const char __pyx_k_ZDR2[] = "ZDR2";
static const char __pyx_k_ZDRC[] = "ZDRC";
static const char __pyx_k_bin2[] = "bin2";
static const char __pyx_k_bin4[] = "bin4";
static const char __pyx_k_copy[] = "copy";
static const char __pyx_k_data[] = "data";
static const char __pyx_k_flag[] = "flag";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_mask[] = "mask";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_nbin[] = "nbin";
static const char __pyx_k_ones[] = "ones";
static const char __pyx_k_open[] = "open";
static const char __pyx_k_read[] = "read";
static const char __pyx_k_sqrt[] = "sqrt";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_time[] = "time";
static const char __pyx_k_view[] = "view";
static const char __pyx_k_warn[] = "warn";
static const char __pyx_k_year[] = "year";
static const char __pyx_k_2612s[] = "2612s";
static const char __pyx_k_CSP16[] = "CSP16";
static const char __pyx_k_DBTE8[] = "DBTE8";
static const char __pyx_k_DBTV8[] = "DBTV8";
static const char __pyx_k_DBZC2[] = "DBZC2";
static const char __pyx_k_DBZE8[] = "DBZE8";
static const char __pyx_k_DBZV8[] = "DBZV8";
static const char __pyx_k_HDIR2[] = "HDIR2";
static const char __pyx_k_HVEL2[] = "HVEL2";
static const char __pyx_k_LDRH2[] = "LDRH2";
static const char __pyx_k_LDRV2[] = "LDRV2";
static const char __pyx_k_LOG16[] = "LOG16";
static const char __pyx_k_OTHER[] = "OTHER";
static const char __pyx_k_PHIDP[] = "PHIDP";
static const char __pyx_k_PHIH2[] = "PHIH2";
static const char __pyx_k_PHIV2[] = "PHIV2";
static const char __pyx_k_PMI16[] = "PMI16";
static const char __pyx_k_RHOH2[] = "RHOH2";
static const char __pyx_k_RHOHV[] = "RHOHV";
static const char __pyx_k_RHOV2[] = "RHOV2";
static const char __pyx_k_SHEAR[] = "SHEAR";
static const char __pyx_k_SINT1[] = "SINT1";
static const char __pyx_k_SINT2[] = "SINT2";
static const char __pyx_k_SINT4[] = "SINT4";
static const char __pyx_k_SNR16[] = "SNR16";
static const char __pyx_k_TIME2[] = "TIME2";
static const char __pyx_k_UINT1[] = "UINT1";
static const char __pyx_k_UINT2[] = "UINT2";
static const char __pyx_k_UINT4[] = "UINT4";
static const char __pyx_k_USER2[] = "USER2";
static const char __pyx_k_VELC2[] = "VELC2";
static const char __pyx_k_VIR16[] = "VIR16";
static const char __pyx_k_VVEL2[] = "VVEL2";
static const char __pyx_k_WIDTH[] = "WIDTH";
static const char __pyx_k_ZDRC2[] = "ZDRC2";
static const char __pyx_k_bool8[] = "bool8";
static const char __pyx_k_close[] = "close";
static const char __pyx_k_debug[] = "debug";
static const char __pyx_k_dtype[] = "dtype";
static const char __pyx_k_empty[] = "empty";
static const char __pyx_k_flags[] = "flags";
static const char __pyx_k_gparm[] = "gparm";
static const char __pyx_k_int16[] = "int16";
static const char __pyx_k_int32[] = "int32";
static const char __pyx_k_istep[] = "istep";
static const char __pyx_k_month[] = "month";
static const char __pyx_k_nbins[] = "nbins";
static const char __pyx_k_ndata[] = "ndata";
static const char __pyx_k_nrays[] = "nrays";
static const char __pyx_k_numpy[] = "numpy";
static const char __pyx_k_power[] = "power";
static const char __pyx_k_print[] = "print";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_shape[] = "shape";
static const char __pyx_k_uint8[] = "uint8";
static const char __pyx_k_word0[] = "word0";
static const char __pyx_k_word1[] = "word1";
static const char __pyx_k_word2[] = "word2";
static const char __pyx_k_word3[] = "word3";
static const char __pyx_k_AXDIL2[] = "AXDIL2";
static const char __pyx_k_DBTE16[] = "DBTE16";
static const char __pyx_k_DBTV16[] = "DBTV16";
static const char __pyx_k_DBZE16[] = "DBZE16";
static const char __pyx_k_DBZV16[] = "DBZV16";
static const char __pyx_k_HCLASS[] = "HCLASS";
static const char __pyx_k_HEIGHT[] = "HEIGHT";
static const char __pyx_k_PHIDP2[] = "PHIDP2";
static const char __pyx_k_RHOHV2[] = "RHOHV2";
static const char __pyx_k_TURB16[] = "TURB16";
static const char __pyx_k_VILD16[] = "VILD16";
static const char __pyx_k_WIDTH2[] = "WIDTH2";
static const char __pyx_k_append[] = "append";
static const char __pyx_k_flags2[] = "flags2";
static const char __pyx_k_iflags[] = "iflags";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_istart[] = "istart";
static const char __pyx_k_number[] = "number";
static const char __pyx_k_offset[] = "offset";
static const char __pyx_k_record[] = "record";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_shrink[] = "shrink";
static const char __pyx_k_string[] = "string";
static const char __pyx_k_struct[] = "struct";
static const char __pyx_k_uint16[] = "uint16";
static const char __pyx_k_unpack[] = "unpack";
static const char __pyx_k_x_size[] = "x_size";
static const char __pyx_k_xrange[] = "xrange";
static const char __pyx_k_y_size[] = "y_size";
static const char __pyx_k_z_size[] = "z_size";
static const char __pyx_k_2_uint8[] = "(2,) uint8";
static const char __pyx_k_ALBEDO8[] = "ALBEDO8";
static const char __pyx_k_DEFORM2[] = "DEFORM2";
static const char __pyx_k_HCLASS2[] = "HCLASS2";
static const char __pyx_k_MESSAGE[] = "MESSAGE";
static const char __pyx_k_float32[] = "float32";
static const char __pyx_k_icolcnt[] = "icolcnt";
static const char __pyx_k_reshape[] = "reshape";
static const char __pyx_k_seconds[] = "seconds";
static const char __pyx_k_spare_0[] = "spare_0";
static const char __pyx_k_spare_1[] = "spare_1";
static const char __pyx_k_spare_2[] = "spare_2";
static const char __pyx_k_spare_3[] = "spare_3";
static const char __pyx_k_spare_4[] = "spare_4";
static const char __pyx_k_tz_name[] = "tz_name";
static const char __pyx_k_x_scale[] = "x_scale";
static const char __pyx_k_y_scale[] = "y_scale";
static const char __pyx_k_z_scale[] = "z_scale";
static const char __pyx_k_ALBEDO16[] = "ALBEDO16";
static const char __pyx_k_DIVERGE2[] = "DIVERGE2";
static const char __pyx_k_FLIQUID2[] = "FLIQUID2";
static const char __pyx_k_UINT16_T[] = "UINT16_T";
static const char __pyx_k_comments[] = "comments";
static const char __pyx_k_datetime[] = "datetime";
static const char __pyx_k_dsp_type[] = "dsp_type";
static const char __pyx_k_filename[] = "filename";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_latitude[] = "latitude";
static const char __pyx_k_ldr_bias[] = "ldr_bias";
static const char __pyx_k_like_dbt[] = "like_dbt";
static const char __pyx_k_like_sqi[] = "like_sqi";
static const char __pyx_k_prf_flag[] = "prf_flag";
static const char __pyx_k_raw_data[] = "raw_data";
static const char __pyx_k_reserved[] = "reserved";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_warnings[] = "warnings";
static const char __pyx_k_zdr_bias[] = "zdr_bias";
static const char __pyx_k_RAINRATE2[] = "RAINRATE2";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_UNKNOWN_6[] = "UNKNOWN_6";
static const char __pyx_k_YMDS_TIME[] = "YMDS_TIME";
static const char __pyx_k_azimuth_0[] = "azimuth_0";
static const char __pyx_k_azimuth_1[] = "azimuth_1";
static const char __pyx_k_data_type[] = "data_type";
static const char __pyx_k_enumerate[] = "enumerate";
static const char __pyx_k_full_xhdr[] = "full_xhdr";
static const char __pyx_k_get_sweep[] = "_get_sweep";
static const char __pyx_k_like_dbt2[] = "like_dbt2";
static const char __pyx_k_like_sqi2[] = "like_sqi2";
static const char __pyx_k_longitude[] = "longitude";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_site_name[] = "site_name";
static const char __pyx_k_skip_time[] = "skip_time";
static const char __pyx_k_stop_time[] = "stop_time";
static const char __pyx_k_structure[] = "structure";
static const char __pyx_k_task_name[] = "task_name";
static const char __pyx_k_times_run[] = "times_run";
static const char __pyx_k_SigmetFile[] = "SigmetFile";
static const char __pyx_k_UNKNOWN_29[] = "UNKNOWN_29";
static const char __pyx_k_UNKNOWN_30[] = "UNKNOWN_30";
static const char __pyx_k_UNKNOWN_31[] = "UNKNOWN_31";
static const char __pyx_k_UNKNOWN_81[] = "UNKNOWN_81";
static const char __pyx_k_UNKNOWN_82[] = "UNKNOWN_82";
static const char __pyx_k_UNKNOWN_83[] = "UNKNOWN_83";
static const char __pyx_k_UNKNOWN_84[] = "UNKNOWN_84";
static const char __pyx_k_UNKNOWN_85[] = "UNKNOWN_85";
static const char __pyx_k_UNKNOWN_86[] = "UNKNOWN_86";
static const char __pyx_k_UNKNOWN_87[] = "UNKNOWN_87";
static const char __pyx_k_UNKNOWN_88[] = "UNKNOWN_88";
static const char __pyx_k_UNKNOWN_89[] = "UNKNOWN_89";
static const char __pyx_k_UNKNOWN_90[] = "UNKNOWN_90";
static const char __pyx_k_UNKNOWN_91[] = "UNKNOWN_91";
static const char __pyx_k_UNKNOWN_92[] = "UNKNOWN_92";
static const char __pyx_k_UNKNOWN_93[] = "UNKNOWN_93";
static const char __pyx_k_UNKNOWN_94[] = "UNKNOWN_94";
static const char __pyx_k_UNKNOWN_95[] = "UNKNOWN_95";
static const char __pyx_k_UNKNOWN_96[] = "UNKNOWN_96";
static const char __pyx_k_UNKNOWN_97[] = "UNKNOWN_97";
static const char __pyx_k_UNKNOWN_98[] = "UNKNOWN_98";
static const char __pyx_k_UNKNOWN_99[] = "UNKNOWN_99";
static const char __pyx_k_cluttermap[] = "cluttermap";
static const char __pyx_k_data_types[] = "data_types";
static const char __pyx_k_empty_like[] = "empty_like";
static const char __pyx_k_fill_value[] = "fill_value";
static const char __pyx_k_frombuffer[] = "frombuffer";
static const char __pyx_k_input_mask[] = "input_mask";
static const char __pyx_k_is_bit_set[] = "_is_bit_set";
static const char __pyx_k_ldr_offset[] = "ldr_offset";
static const char __pyx_k_low_prf_hz[] = "low_prf_hz";
static const char __pyx_k_major_mode[] = "major_mode";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_ray_number[] = "ray_number";
static const char __pyx_k_start_time[] = "start_time";
static const char __pyx_k_task_state[] = "task_state";
static const char __pyx_k_total_size[] = "total_size";
static const char __pyx_k_unpack_key[] = "_unpack_key";
static const char __pyx_k_wavelength[] = "wavelength";
static const char __pyx_k_x_location[] = "x_location";
static const char __pyx_k_x_smoother[] = "x_smoother";
static const char __pyx_k_y_location[] = "y_location";
static const char __pyx_k_y_smoother[] = "y_smoother";
static const char __pyx_k_z_location[] = "z_location";
static const char __pyx_k_zdr_offset[] = "zdr_offset";
static const char __pyx_k_zeros_like[] = "zeros_like";
static const char __pyx_k_ImportError[] = "ImportError";
static const char __pyx_k_PRODUCT_END[] = "PRODUCT_END";
static const char __pyx_k_PRODUCT_HDR[] = "PRODUCT_HDR";
static const char __pyx_k_RECORD_SIZE[] = "RECORD_SIZE";
static const char __pyx_k_UNKNOWN_100[] = "UNKNOWN_100";
static const char __pyx_k_UNKNOWN_101[] = "UNKNOWN_101";
static const char __pyx_k_UNKNOWN_102[] = "UNKNOWN_102";
static const char __pyx_k_UNKNOWN_103[] = "UNKNOWN_103";
static const char __pyx_k_UNKNOWN_104[] = "UNKNOWN_104";
static const char __pyx_k_UNKNOWN_105[] = "UNKNOWN_105";
static const char __pyx_k_UNKNOWN_106[] = "UNKNOWN_106";
static const char __pyx_k_UNKNOWN_107[] = "UNKNOWN_107";
static const char __pyx_k_UNKNOWN_108[] = "UNKNOWN_108";
static const char __pyx_k_UNKNOWN_109[] = "UNKNOWN_109";
static const char __pyx_k_UNKNOWN_110[] = "UNKNOWN_110";
static const char __pyx_k_UNKNOWN_112[] = "UNKNOWN_112";
static const char __pyx_k_UNKNOWN_113[] = "UNKNOWN_113";
static const char __pyx_k_UNKNOWN_114[] = "UNKNOWN_114";
static const char __pyx_k_UNKNOWN_115[] = "UNKNOWN_115";
static const char __pyx_k_UNKNOWN_116[] = "UNKNOWN_116";
static const char __pyx_k_UNKNOWN_117[] = "UNKNOWN_117";
static const char __pyx_k_UNKNOWN_118[] = "UNKNOWN_118";
static const char __pyx_k_UNKNOWN_119[] = "UNKNOWN_119";
static const char __pyx_k_UNKNOWN_120[] = "UNKNOWN_120";
static const char __pyx_k_UNKNOWN_121[] = "UNKNOWN_121";
static const char __pyx_k_UNKNOWN_122[] = "UNKNOWN_122";
static const char __pyx_k_UNKNOWN_123[] = "UNKNOWN_123";
static const char __pyx_k_UNKNOWN_124[] = "UNKNOWN_124";
static const char __pyx_k_UNKNOWN_125[] = "UNKNOWN_125";
static const char __pyx_k_UNKNOWN_126[] = "UNKNOWN_126";
static const char __pyx_k_UNKNOWN_127[] = "UNKNOWN_127";
static const char __pyx_k_attenuation[] = "attenuation";
static const char __pyx_k_bit_per_bin[] = "bit_per_bin";
static const char __pyx_k_elevation_0[] = "elevation_0";
static const char __pyx_k_elevation_1[] = "elevation_1";
static const char __pyx_k_fixed_angle[] = "fixed_angle";
static const char __pyx_k_height_site[] = "height_site";
static const char __pyx_k_ingest_time[] = "ingest_time";
static const char __pyx_k_mask_word_0[] = "mask_word_0";
static const char __pyx_k_mask_word_1[] = "mask_word_1";
static const char __pyx_k_mask_word_2[] = "mask_word_2";
static const char __pyx_k_mask_word_3[] = "mask_word_3";
static const char __pyx_k_mask_word_4[] = "mask_word_4";
static const char __pyx_k_ndata_types[] = "ndata_types";
static const char __pyx_k_number_bins[] = "number_bins";
static const char __pyx_k_product_end[] = "product_end";
static const char __pyx_k_product_hdr[] = "product_hdr";
static const char __pyx_k_pulse_width[] = "pulse_width";
static const char __pyx_k_ray_headers[] = "ray_headers";
static const char __pyx_k_sample_size[] = "sample_size";
static const char __pyx_k_velocity_up[] = "velocity_up";
static const char __pyx_k_zr_constant[] = "zr_constant";
static const char __pyx_k_zr_exponent[] = "zr_exponent";
static const char __pyx_k_azimuth_list[] = "azimuth_list";
static const char __pyx_k_earth_radius[] = "earth_radius";
static const char __pyx_k_fault_status[] = "fault_status";
static const char __pyx_k_gparam_bytes[] = "gparam_bytes";
static const char __pyx_k_height_radar[] = "height_radar";
static const char __pyx_k_ilevel_seams[] = "ilevel_seams";
static const char __pyx_k_iris_version[] = "iris_version";
static const char __pyx_k_last_run_day[] = "last_run_day";
static const char __pyx_k_masked_array[] = "masked_array";
static const char __pyx_k_milliseconds[] = "milliseconds";
static const char __pyx_k_number_files[] = "number_files";
static const char __pyx_k_number_tasks[] = "number_tasks";
static const char __pyx_k_polarization[] = "polarization";
static const char __pyx_k_product_name[] = "product_name";
static const char __pyx_k_radar_height[] = "radar_height";
static const char __pyx_k_samples_used[] = "samples_used";
static const char __pyx_k_sweep_number[] = "sweep_number";
static const char __pyx_k_trigger_rate[] = "trigger_rate";
static const char __pyx_k_DSP_DATA_MASK[] = "DSP_DATA_MASK";
static const char __pyx_k_INGEST_HEADER[] = "INGEST_HEADER";
static const char __pyx_k_RAW_PROD_BHDR[] = "RAW_PROD_BHDR";
static const char __pyx_k_TASK_DSP_INFO[] = "TASK_DSP_INFO";
static const char __pyx_k_TASK_END_INFO[] = "TASK_END_INFO";
static const char __pyx_k_TEMPERATURE16[] = "TEMPERATURE16";
static const char __pyx_k_bin2_to_angle[] = "bin2_to_angle";
static const char __pyx_k_bin4_to_angle[] = "bin4_to_angle";
static const char __pyx_k_comment_bytes[] = "comment_bytes";
static const char __pyx_k_first_azimuth[] = "first_azimuth";
static const char __pyx_k_ground_height[] = "ground_height";
static const char __pyx_k_hardware_site[] = "hardware_site";
static const char __pyx_k_ingest_header[] = "ingest_header";
static const char __pyx_k_last_run_time[] = "last_run_time";
static const char __pyx_k_maximum_range[] = "maximum_range";
static const char __pyx_k_melting_layer[] = "melting_layer";
static const char __pyx_k_melting_level[] = "melting_level";
static const char __pyx_k_number_ingest[] = "number_ingest";
static const char __pyx_k_number_sweeps[] = "number_sweeps";
static const char __pyx_k_playback_flag[] = "playback_flag";
static const char __pyx_k_record_number[] = "record_number";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_sqi_threshold[] = "sqi_threshold";
static const char __pyx_k_task_dsp_info[] = "task_dsp_info";
static const char __pyx_k_task_dsp_mode[] = "task_dsp_mode";
static const char __pyx_k_task_end_info[] = "task_end_info";
static const char __pyx_k_velocity_east[] = "velocity_east";
static const char __pyx_k_TASK_MISC_INFO[] = "TASK_MISC_INFO";
static const char __pyx_k_TASK_SCAN_INFO[] = "TASK_SCAN_INFO";
static const char __pyx_k_altitude_radar[] = "altitude_radar";
static const char __pyx_k_clutter_filter[] = "clutter_filter";
static const char __pyx_k_data_type_name[] = "data_type_name";
static const char __pyx_k_dual_prf_delay[] = "dual_prf_delay";
static const char __pyx_k_elevation_list[] = "elevation_list";
static const char __pyx_k_format_version[] = "format_version";
static const char __pyx_k_iset_and_scale[] = "iset_and_scale";
static const char __pyx_k_last_bin_range[] = "last_bin_range";
static const char __pyx_k_latitude_radar[] = "latitude_radar";
static const char __pyx_k_local_timezone[] = "local_timezone";
static const char __pyx_k_multi_prf_flag[] = "multi_prf_flag";
static const char __pyx_k_product_config[] = "product_config";
static const char __pyx_k_task_data_time[] = "task_data_time";
static const char __pyx_k_task_misc_info[] = "task_misc_info";
static const char __pyx_k_task_scan_info[] = "task_scan_info";
static const char __pyx_k_transmit_power[] = "transmit_power";
static const char __pyx_k_velocity_north[] = "velocity_north";
static const char __pyx_k_COLOR_SCALE_DEF[] = "COLOR_SCALE_DEF";
static const char __pyx_k_TASK_CALIB_INFO[] = "TASK_CALIB_INFO";
static const char __pyx_k_TASK_RANGE_INFO[] = "TASK_RANGE_INFO";
static const char __pyx_k_TASK_SCHED_INFO[] = "TASK_SCHED_INFO";
static const char __pyx_k_cluttermap_flag[] = "cluttermap_flag";
static const char __pyx_k_color_scale_def[] = "color_scale_def";
static const char __pyx_k_first_bin_range[] = "first_bin_range";
static const char __pyx_k_first_elevation[] = "first_elevation";
static const char __pyx_k_first_ray_index[] = "first_ray_index";
static const char __pyx_k_gas_attenuation[] = "gas_attenuation";
static const char __pyx_k_generation_time[] = "generation_time";
static const char __pyx_k_input_data_type[] = "input_data_type";
static const char __pyx_k_longitude_radar[] = "longitude_radar";
static const char __pyx_k_mean_wind_speed[] = "mean_wind_speed";
static const char __pyx_k_number_elements[] = "number_elements";
static const char __pyx_k_power_threshold[] = "power_threshold";
static const char __pyx_k_projection_name[] = "projection_name";
static const char __pyx_k_projection_type[] = "projection_type";
static const char __pyx_k_radial_smoother[] = "radial_smoother";
static const char __pyx_k_ray_header_mask[] = "ray_header_mask";
static const char __pyx_k_resolution_rays[] = "resolution_rays";
static const char __pyx_k_scheduling_code[] = "scheduling_code";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_step_input_bins[] = "step_input_bins";
static const char __pyx_k_task_calib_info[] = "task_calib_info";
static const char __pyx_k_task_range_info[] = "task_range_info";
static const char __pyx_k_task_sched_info[] = "task_sched_info";
static const char __pyx_k_tcf_cal_flags_1[] = "tcf_cal_flags_1";
static const char __pyx_k_tcf_cal_flags_2[] = "tcf_cal_flags_2";
static const char __pyx_k_STRUCTURE_HEADER[] = "STRUCTURE_HEADER";
static const char __pyx_k_customer_storage[] = "customer_storage";
static const char __pyx_k_file_ingest_time[] = "file_ingest_time";
static const char __pyx_k_first_ray_offset[] = "first_ray_offset";
static const char __pyx_k_ingest_site_name[] = "ingest_site_name_";
static const char __pyx_k_inverse_flatting[] = "inverse_flatting";
static const char __pyx_k_playback_version[] = "playback_version";
static const char __pyx_k_ray_header_bytes[] = "ray_header_bytes";
static const char __pyx_k_step_output_bins[] = "step_output_bins";
static const char __pyx_k_structure_header[] = "structure_header";
static const char __pyx_k_sweep_start_time[] = "sweep_start_time";
static const char __pyx_k_task_description[] = "task_description";
static const char __pyx_k_tr_serial_number[] = "tr_serial_number";
static const char __pyx_k_trucation_height[] = "trucation_height";
static const char __pyx_k_unpack_structure[] = "_unpack_structure";
static const char __pyx_k_SIGMET_DATA_TYPES[] = "SIGMET_DATA_TYPES";
static const char __pyx_k_agc_feedback_code[] = "agc_feedback_code";
static const char __pyx_k_antenna_offset_up[] = "antenna_offset_up";
static const char __pyx_k_antenna_scan_mode[] = "antenna_scan_mode";
static const char __pyx_k_gain_control_flag[] = "gain_control_flag";
static const char __pyx_k_low_prf_factional[] = "low_prf_factional";
static const char __pyx_k_minor_task_suffix[] = "minor_task_suffix";
static const char __pyx_k_number_input_bins[] = "number_input_bins";
static const char __pyx_k_number_log_filter[] = "number_log_filter";
static const char __pyx_k_number_rays_sweep[] = "number_rays_sweep";
static const char __pyx_k_parse_ray_headers[] = "_parse_ray_headers";
static const char __pyx_k_polarization_type[] = "polarization_type";
static const char __pyx_k_product_type_code[] = "product_type_code";
static const char __pyx_k_self__rbuf_pos_is[] = "self._rbuf_pos is";
static const char __pyx_k_sweep_ingest_time[] = "sweep_ingest_time";
static const char __pyx_k_task_major_number[] = "task_major_number";
static const char __pyx_k_task_minor_number[] = "task_minor_number";
static const char __pyx_k_truncation_height[] = "truncation_height";
static const char __pyx_k_INGEST_DATA_HEADER[] = "INGEST_DATA_HEADER";
static const char __pyx_k_Reading_ray_i_of_i[] = "Reading ray: %i of %i";
static const char __pyx_k_TASK_CONFIGURATION[] = "TASK_CONFIGURATION";
static const char __pyx_k_TASK_PPI_SCAN_INFO[] = "TASK_PPI_SCAN_INFO";
static const char __pyx_k_TASK_RHI_SCAN_INFO[] = "TASK_RHI_SCAN_INFO";
static const char __pyx_k_antenna_offset_bow[] = "antenna_offset_bow";
static const char __pyx_k_bytes_in_structure[] = "bytes_in_structure";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_configuration_name[] = "configuration_name";
static const char __pyx_k_left_azimuth_limit[] = "left_azimuth_limit";
static const char __pyx_k_number_output_bins[] = "number_output_bins";
static const char __pyx_k_reciever_bandwidth[] = "reciever_bandwidth";
static const char __pyx_k_reflectivity_slope[] = "reflectivity_slope";
static const char __pyx_k_task_configuration[] = "task_configuration";
static const char __pyx_k_time_used_last_run[] = "time_used_last_run";
static const char __pyx_k_unpack_product_hdr[] = "_unpack_product_hdr";
static const char __pyx_k_vertical_beamwidth[] = "vertical_beamwidth";
static const char __pyx_k_TASK_DSP_MODE_BATCH[] = "TASK_DSP_MODE_BATCH";
static const char __pyx_k_TASK_FILE_SCAN_INFO[] = "TASK_FILE_SCAN_INFO";
static const char __pyx_k_clutter_filter_name[] = "clutter_filter_name";
static const char __pyx_k_convert_sigmet_data[] = "convert_sigmet_data";
static const char __pyx_k_ingest_iris_version[] = "ingest_iris_version";
static const char __pyx_k_latitude_projection[] = "latitude_projection";
static const char __pyx_k_low_prf_sample_size[] = "low_prf_sample_size";
static const char __pyx_k_mean_wind_direction[] = "mean_wind_direction";
static const char __pyx_k_nexrad_clutter_skip[] = "nexrad_clutter_skip";
static const char __pyx_k_right_azimuth_limit[] = "right_azimuth_limit";
static const char __pyx_k_standard_parallel_1[] = "standard_parallel_1";
static const char __pyx_k_standard_parallel_2[] = "standard_parallel_2";
static const char __pyx_k_zdr_threshold_flags[] = "zdr_threshold_flags";
static const char __pyx_k_INGEST_CONFIGURATION[] = "INGEST_CONFIGURATION";
static const char __pyx_k_data_types_from_mask[] = "_data_types_from_mask";
static const char __pyx_k_determine_data_types[] = "_determine_data_types";
static const char __pyx_k_extended_header_type[] = "extended_header_type";
static const char __pyx_k_horizontal_beamwidth[] = "horizontal_beamwidth";
static const char __pyx_k_ingest_configuration[] = "ingest_configuration";
static const char __pyx_k_ingest_hardware_name[] = "ingest_hardware_name_";
static const char __pyx_k_iris_version_created[] = "iris_version_created";
static const char __pyx_k_log_filter_first_bin[] = "log_filter_first_bin";
static const char __pyx_k_longitude_projection[] = "longitude_projection";
static const char __pyx_k_number_linear_filter[] = "number_linear_filter";
static const char __pyx_k_pyart_io__sigmetfile[] = "pyart.io._sigmetfile";
static const char __pyx_k_seconds_between_runs[] = "seconds_between_runs";
static const char __pyx_k_structure_identifier[] = "structure_identifier";
static const char __pyx_k_unpack_ingest_header[] = "_unpack_ingest_header";
static const char __pyx_k_unpack_raw_prod_bhdr[] = "_unpack_raw_prod_bhdr";
static const char __pyx_k_PRODUCT_CONFIGURATION[] = "PRODUCT_CONFIGURATION";
static const char __pyx_k_TASK_MANUAL_SCAN_INFO[] = "TASK_MANUAL_SCAN_INFO";
static const char __pyx_k_lower_elevation_limit[] = "lower_elevation_limit";
static const char __pyx_k_product_configuration[] = "product_configuration";
static const char __pyx_k_signal_processor_type[] = "signal_processor_type";
static const char __pyx_k_upper_elevation_limit[] = "upper_elevation_limit";
static const char __pyx_k_width_threshold_flags[] = "width_threshold_flags";
static const char __pyx_k_current_data_type_mask[] = "current_data_type_mask";
static const char __pyx_k_custom_ray_header_name[] = "custom_ray_header_name";
static const char __pyx_k_product_specific_bytes[] = "product_specific_bytes";
static const char __pyx_k_vertical_current_noise[] = "vertical_current_noise";
static const char __pyx_k_volume_scan_start_time[] = "volume_scan_start_time";
static const char __pyx_k_Finished_loading_record[] = "Finished loading record:";
static const char __pyx_k_GMT_minute_offset_local[] = "GMT_minute_offset_local";
static const char __pyx_k_linear_filter_first_bin[] = "linear_filter_first_bin";
static const char __pyx_k_low_prf_range_averaging[] = "low_prf_range_averaging";
static const char __pyx_k_number_rays_file_actual[] = "number_rays_file_actual";
static const char __pyx_k_number_sweeps_completed[] = "number_sweeps_completed";
static const char __pyx_k_original_data_type_mask[] = "original_data_type_mask";
static const char __pyx_k_product_sequence_number[] = "product_sequence_number";
static const char __pyx_k_start_first_sector_flag[] = "start_first_sector_flag";
static const char __pyx_k_variable_range_bin_flag[] = "variable_range_bin_flag";
static const char __pyx_k_vertical_i0_calibration[] = "vertical_i0_calibration";
static const char __pyx_k_vertical_radar_constant[] = "vertical_radar_constant";
static const char __pyx_k_antenna_offset_starboard[] = "antenna_offset_starboard";
static const char __pyx_k_gmt_offset_minutes_local[] = "gmt_offset_minutes_local";
static const char __pyx_k_horizontal_current_noise[] = "horizontal_current_noise";
static const char __pyx_k_nexrad_clutter_threshold[] = "nexrad_clutter_threshold";
static const char __pyx_k_number_task_config_table[] = "number_task_config_table";
static const char __pyx_k_pyart_io__sigmetfile_pyx[] = "pyart/io/_sigmetfile.pyx";
static const char __pyx_k_range_bin_averaging_flag[] = "range_bin_averaging_flag";
static const char __pyx_k_reflectivity_calibration[] = "reflectivity_calibration";
static const char __pyx_k_start_first_section_flag[] = "start_first_section_flag";
static const char __pyx_k_task_scan_type_scan_info[] = "task_scan_type_scan_info";
static const char __pyx_k_velocity_threshold_flags[] = "velocity_threshold_flags";
static const char __pyx_k_extended_ray_header_bytes[] = "extended_ray_header_bytes";
static const char __pyx_k_horizontal_calibration_i0[] = "horizontal_calibration_i0";
static const char __pyx_k_horizontal_i0_calibration[] = "horizontal_i0_calibration";
static const char __pyx_k_horizontal_radar_constant[] = "horizontal_radar_constant";
static const char __pyx_k_number_rays_file_expected[] = "number_rays_file_expected";
static const char __pyx_k_tranmitter_phase_sequence[] = "tranmitter_phase_sequence";
static const char __pyx_k_unpack_ingest_data_header[] = "_unpack_ingest_data_header";
static const char __pyx_k_width_unfolding_threshold[] = "width_unfolding_threshold";
static const char __pyx_k_GMT_minute_offset_standard[] = "GMT_minute_offset_standard";
static const char __pyx_k_angular_resolution_desired[] = "angular_resolution_desired";
static const char __pyx_k_unpack_ingest_data_headers[] = "_unpack_ingest_data_headers";
static const char __pyx_k_vertical_noise_calibration[] = "vertical_noise_calibration";
static const char __pyx_k_gmt_offset_minutes_standard[] = "gmt_offset_minutes_standard";
static const char __pyx_k_clutter_correction_threshold[] = "clutter_correction_threshold";
static const char __pyx_k_horizontal_calibration_noise[] = "horizontal_calibration_noise";
static const char __pyx_k_horizontal_noise_calibration[] = "horizontal_noise_calibration";
static const char __pyx_k_radar_height_above_reference[] = "radar_height_above_reference";
static const char __pyx_k_reflectivity_noise_threshold[] = "reflectivity_noise_threshold";
static const char __pyx_k_task_configuration_file_name[] = "task_configuration_file_name";
static const char __pyx_k_velocity_unfolding_threshold[] = "velocity_unfolding_threshold";
static const char __pyx_k_extended_product_header_offset[] = "extended_product_header_offset";
static const char __pyx_k_A_class_and_supporting_function[] = "\nA class and supporting functions for reading Sigmet (raw format) files.\n\n";
static const char __pyx_k_numpy_core_multiarray_failed_to[] = "numpy.core.multiarray failed to import";
static const char __pyx_k_File_truncated_or_corrupt_i_of_i[] = "File truncated or corrupt, %i of %i sweeps read";
static const char __pyx_k_Unknown_type_s_returning_raw_dat[] = "Unknown type: %s, returning raw data";
static const char __pyx_k_corrected_reflectivity_threshold[] = "corrected_reflectivity_threshold_flags";
static const char __pyx_k_numpy_core_umath_failed_to_impor[] = "numpy.core.umath failed to import";
static const char __pyx_k_reflectivity_unfolding_threshold[] = "reflectivity_unfolding_threshold";
static const char __pyx_k_self__rbuf_p_cannot_be_converted[] = "self._rbuf_p cannot be converted to a Python object for pickling";
static const char __pyx_k_uncorrected_reflectivity_thresho[] = "uncorrected_reflectivity_threshold_flags";
static PyObject *__pyx_kp_u_112s;
static PyObject *__pyx_kp_u_115s;
static PyObject *__pyx_kp_u_120s;
static PyObject *__pyx_kp_u_128s;
static PyObject *__pyx_kp_u_12s;
static PyObject *__pyx_kp_u_136s;
static PyObject *__pyx_kp_u_160s;
static PyObject *__pyx_kp_u_16s;
static PyObject *__pyx_kp_u_184s;
static PyObject *__pyx_kp_u_18s;
static PyObject *__pyx_kp_u_198s;
static PyObject *__pyx_kp_u_1s;
static PyObject *__pyx_kp_u_2;
static PyObject *__pyx_kp_u_200s;
static PyObject *__pyx_kp_u_204s;
static PyObject *__pyx_kp_u_208s;
static PyObject *__pyx_kp_u_228s;
static PyObject *__pyx_kp_u_24s;
static PyObject *__pyx_kp_u_256s;
static PyObject *__pyx_kp_u_2612s;
static PyObject *__pyx_kp_u_28s;
static PyObject *__pyx_kp_u_2_uint8;
static PyObject *__pyx_kp_u_2s;
static PyObject *__pyx_kp_u_308s;
static PyObject *__pyx_kp_u_320s;
static PyObject *__pyx_kp_u_32s;
static PyObject *__pyx_kp_u_36s;
static PyObject *__pyx_kp_u_40s;
static PyObject *__pyx_kp_u_480s;
static PyObject *__pyx_kp_u_48s;
static PyObject *__pyx_kp_u_4s;
static PyObject *__pyx_kp_u_52s;
static PyObject *__pyx_kp_u_6s;
static PyObject *__pyx_kp_u_720s;
static PyObject *__pyx_kp_u_732s;
static PyObject *__pyx_kp_u_80s;
static PyObject *__pyx_kp_u_8s;
static PyObject *__pyx_kp_u_920s;
static PyObject *__pyx_kp_u_94s;
static PyObject *__pyx_n_u_ALBEDO16;
static PyObject *__pyx_n_u_ALBEDO8;
static PyObject *__pyx_n_u_AXDIL2;
static PyObject *__pyx_n_u_B;
static PyObject *__pyx_n_s_BIN1;
static PyObject *__pyx_n_s_BIN2;
static PyObject *__pyx_n_s_BIN4;
static PyObject *__pyx_n_s_COLOR_SCALE_DEF;
static PyObject *__pyx_n_u_CSP16;
static PyObject *__pyx_n_u_CSP8;
static PyObject *__pyx_n_u_DBT;
static PyObject *__pyx_n_u_DBT2;
static PyObject *__pyx_n_u_DBTE16;
static PyObject *__pyx_n_u_DBTE8;
static PyObject *__pyx_n_u_DBTV16;
static PyObject *__pyx_n_u_DBTV8;
static PyObject *__pyx_n_u_DBZ;
static PyObject *__pyx_n_u_DBZ2;
static PyObject *__pyx_n_u_DBZC;
static PyObject *__pyx_n_u_DBZC2;
static PyObject *__pyx_n_u_DBZE16;
static PyObject *__pyx_n_u_DBZE8;
static PyObject *__pyx_n_u_DBZV16;
static PyObject *__pyx_n_u_DBZV8;
static PyObject *__pyx_n_u_DEFORM2;
static PyObject *__pyx_n_u_DIVERGE2;
static PyObject *__pyx_n_s_DSP_DATA_MASK;
static PyObject *__pyx_n_u_FLIQUID2;
static PyObject *__pyx_n_s_FLT4;
static PyObject *__pyx_n_s_FLT8;
static PyObject *__pyx_kp_u_File_truncated_or_corrupt_i_of_i;
static PyObject *__pyx_kp_u_Finished_loading_record;
static PyObject *__pyx_n_u_GMT_minute_offset_local;
static PyObject *__pyx_n_u_GMT_minute_offset_standard;
static PyObject *__pyx_n_u_H;
static PyObject *__pyx_n_u_HCLASS;
static PyObject *__pyx_n_u_HCLASS2;
static PyObject *__pyx_n_u_HDIR2;
static PyObject *__pyx_n_u_HEIGHT;
static PyObject *__pyx_n_u_HVEL2;
static PyObject *__pyx_n_u_I;
static PyObject *__pyx_n_s_INGEST_CONFIGURATION;
static PyObject *__pyx_n_s_INGEST_DATA_HEADER;
static PyObject *__pyx_n_s_INGEST_HEADER;
static PyObject *__pyx_n_s_ImportError;
static PyObject *__pyx_n_u_KDP;
static PyObject *__pyx_n_u_KDP2;
static PyObject *__pyx_n_u_LDRH;
static PyObject *__pyx_n_u_LDRH2;
static PyObject *__pyx_n_u_LDRV;
static PyObject *__pyx_n_u_LDRV2;
static PyObject *__pyx_n_u_LOG16;
static PyObject *__pyx_n_u_LOG8;
static PyObject *__pyx_n_s_MESSAGE;
static PyObject *__pyx_n_u_OTHER;
static PyObject *__pyx_n_u_PHIDP;
static PyObject *__pyx_n_u_PHIDP2;
static PyObject *__pyx_n_u_PHIH;
static PyObject *__pyx_n_u_PHIH2;
static PyObject *__pyx_n_u_PHIV;
static PyObject *__pyx_n_u_PHIV2;
static PyObject *__pyx_n_u_PMI16;
static PyObject *__pyx_n_u_PMI8;
static PyObject *__pyx_n_s_PRODUCT_CONFIGURATION;
static PyObject *__pyx_n_s_PRODUCT_END;
static PyObject *__pyx_n_s_PRODUCT_HDR;
static PyObject *__pyx_n_u_RAINRATE2;
static PyObject *__pyx_n_u_RAW;
static PyObject *__pyx_n_s_RAW_PROD_BHDR;
static PyObject *__pyx_n_s_RECORD_SIZE;
static PyObject *__pyx_n_u_RHOH;
static PyObject *__pyx_n_u_RHOH2;
static PyObject *__pyx_n_u_RHOHV;
static PyObject *__pyx_n_u_RHOHV2;
static PyObject *__pyx_n_u_RHOV;
static PyObject *__pyx_n_u_RHOV2;
static PyObject *__pyx_kp_u_Reading_ray_i_of_i;
static PyObject *__pyx_n_u_SHEAR;
static PyObject *__pyx_n_s_SIGMET_DATA_TYPES;
static PyObject *__pyx_n_s_SINT1;
static PyObject *__pyx_n_s_SINT2;
static PyObject *__pyx_n_s_SINT4;
static PyObject *__pyx_n_u_SNR16;
static PyObject *__pyx_n_u_SNR8;
static PyObject *__pyx_n_u_SQI;
static PyObject *__pyx_n_u_SQI2;
static PyObject *__pyx_n_s_STRUCTURE_HEADER;
static PyObject *__pyx_n_s_SigmetFile;
static PyObject *__pyx_n_s_TASK_CALIB_INFO;
static PyObject *__pyx_n_s_TASK_CONFIGURATION;
static PyObject *__pyx_n_s_TASK_DSP_INFO;
static PyObject *__pyx_n_s_TASK_DSP_MODE_BATCH;
static PyObject *__pyx_n_s_TASK_END_INFO;
static PyObject *__pyx_n_s_TASK_FILE_SCAN_INFO;
static PyObject *__pyx_n_s_TASK_MANUAL_SCAN_INFO;
static PyObject *__pyx_n_s_TASK_MISC_INFO;
static PyObject *__pyx_n_s_TASK_PPI_SCAN_INFO;
static PyObject *__pyx_n_s_TASK_RANGE_INFO;
static PyObject *__pyx_n_s_TASK_RHI_SCAN_INFO;
static PyObject *__pyx_n_s_TASK_SCAN_INFO;
static PyObject *__pyx_n_s_TASK_SCHED_INFO;
static PyObject *__pyx_n_u_TEMPERATURE16;
static PyObject *__pyx_n_u_TIME2;
static PyObject *__pyx_n_u_TURB16;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_n_s_UINT1;
static PyObject *__pyx_n_s_UINT16_T;
static PyObject *__pyx_n_s_UINT2;
static PyObject *__pyx_n_s_UINT4;
static PyObject *__pyx_n_u_UNKNOWN_100;
static PyObject *__pyx_n_u_UNKNOWN_101;
static PyObject *__pyx_n_u_UNKNOWN_102;
static PyObject *__pyx_n_u_UNKNOWN_103;
static PyObject *__pyx_n_u_UNKNOWN_104;
static PyObject *__pyx_n_u_UNKNOWN_105;
static PyObject *__pyx_n_u_UNKNOWN_106;
static PyObject *__pyx_n_u_UNKNOWN_107;
static PyObject *__pyx_n_u_UNKNOWN_108;
static PyObject *__pyx_n_u_UNKNOWN_109;
static PyObject *__pyx_n_u_UNKNOWN_110;
static PyObject *__pyx_n_u_UNKNOWN_112;
static PyObject *__pyx_n_u_UNKNOWN_113;
static PyObject *__pyx_n_u_UNKNOWN_114;
static PyObject *__pyx_n_u_UNKNOWN_115;
static PyObject *__pyx_n_u_UNKNOWN_116;
static PyObject *__pyx_n_u_UNKNOWN_117;
static PyObject *__pyx_n_u_UNKNOWN_118;
static PyObject *__pyx_n_u_UNKNOWN_119;
static PyObject *__pyx_n_u_UNKNOWN_120;
static PyObject *__pyx_n_u_UNKNOWN_121;
static PyObject *__pyx_n_u_UNKNOWN_122;
static PyObject *__pyx_n_u_UNKNOWN_123;
static PyObject *__pyx_n_u_UNKNOWN_124;
static PyObject *__pyx_n_u_UNKNOWN_125;
static PyObject *__pyx_n_u_UNKNOWN_126;
static PyObject *__pyx_n_u_UNKNOWN_127;
static PyObject *__pyx_n_u_UNKNOWN_29;
static PyObject *__pyx_n_u_UNKNOWN_30;
static PyObject *__pyx_n_u_UNKNOWN_31;
static PyObject *__pyx_n_u_UNKNOWN_6;
static PyObject *__pyx_n_u_UNKNOWN_81;
static PyObject *__pyx_n_u_UNKNOWN_82;
static PyObject *__pyx_n_u_UNKNOWN_83;
static PyObject *__pyx_n_u_UNKNOWN_84;
static PyObject *__pyx_n_u_UNKNOWN_85;
static PyObject *__pyx_n_u_UNKNOWN_86;
static PyObject *__pyx_n_u_UNKNOWN_87;
static PyObject *__pyx_n_u_UNKNOWN_88;
static PyObject *__pyx_n_u_UNKNOWN_89;
static PyObject *__pyx_n_u_UNKNOWN_90;
static PyObject *__pyx_n_u_UNKNOWN_91;
static PyObject *__pyx_n_u_UNKNOWN_92;
static PyObject *__pyx_n_u_UNKNOWN_93;
static PyObject *__pyx_n_u_UNKNOWN_94;
static PyObject *__pyx_n_u_UNKNOWN_95;
static PyObject *__pyx_n_u_UNKNOWN_96;
static PyObject *__pyx_n_u_UNKNOWN_97;
static PyObject *__pyx_n_u_UNKNOWN_98;
static PyObject *__pyx_n_u_UNKNOWN_99;
static PyObject *__pyx_n_u_USER;
static PyObject *__pyx_n_u_USER2;
static PyObject *__pyx_kp_u_Unknown_type_s_returning_raw_dat;
static PyObject *__pyx_n_u_VEL;
static PyObject *__pyx_n_u_VEL2;
static PyObject *__pyx_n_u_VELC;
static PyObject *__pyx_n_u_VELC2;
static PyObject *__pyx_n_u_VIL2;
static PyObject *__pyx_n_u_VILD16;
static PyObject *__pyx_n_u_VIR16;
static PyObject *__pyx_n_u_VVEL2;
static PyObject *__pyx_n_u_WIDTH;
static PyObject *__pyx_n_u_WIDTH2;
static PyObject *__pyx_n_u_XHDR;
static PyObject *__pyx_n_s_YMDS_TIME;
static PyObject *__pyx_n_u_ZDR;
static PyObject *__pyx_n_u_ZDR2;
static PyObject *__pyx_n_u_ZDRC;
static PyObject *__pyx_n_u_ZDRC2;
static PyObject *__pyx_kp_u__15;
static PyObject *__pyx_n_u_agc_feedback_code;
static PyObject *__pyx_n_u_altitude_radar;
static PyObject *__pyx_n_u_angular_resolution_desired;
static PyObject *__pyx_n_u_antenna_offset_bow;
static PyObject *__pyx_n_u_antenna_offset_starboard;
static PyObject *__pyx_n_u_antenna_offset_up;
static PyObject *__pyx_n_u_antenna_scan_mode;
static PyObject *__pyx_n_s_append;
static PyObject *__pyx_n_u_attenuation;
static PyObject *__pyx_n_s_az0;
static PyObject *__pyx_n_s_az1;
static PyObject *__pyx_n_u_azimuth_0;
static PyObject *__pyx_n_u_azimuth_1;
static PyObject *__pyx_n_u_azimuth_list;
static PyObject *__pyx_n_u_b;
static PyObject *__pyx_n_s_bin2;
static PyObject *__pyx_n_s_bin2_to_angle;
static PyObject *__pyx_n_s_bin4;
static PyObject *__pyx_n_s_bin4_to_angle;
static PyObject *__pyx_n_s_bit;
static PyObject *__pyx_n_u_bit_per_bin;
static PyObject *__pyx_n_s_bool8;
static PyObject *__pyx_n_u_bytes_in_structure;
static PyObject *__pyx_n_u_c;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_close;
static PyObject *__pyx_n_u_clutter_correction_threshold;
static PyObject *__pyx_n_u_clutter_filter;
static PyObject *__pyx_n_u_clutter_filter_name;
static PyObject *__pyx_n_u_cluttermap;
static PyObject *__pyx_n_u_cluttermap_flag;
static PyObject *__pyx_n_u_color_scale_def;
static PyObject *__pyx_n_u_comment_bytes;
static PyObject *__pyx_n_u_comments;
static PyObject *__pyx_n_u_configuration_name;
static PyObject *__pyx_n_s_convert_sigmet_data;
static PyObject *__pyx_n_s_copy;
static PyObject *__pyx_n_u_corrected_reflectivity_threshold;
static PyObject *__pyx_n_u_current_data_type_mask;
static PyObject *__pyx_n_u_custom_ray_header_name;
static PyObject *__pyx_n_u_customer_storage;
static PyObject *__pyx_n_u_d;
static PyObject *__pyx_n_s_data;
static PyObject *__pyx_n_s_data_type;
static PyObject *__pyx_n_u_data_type;
static PyObject *__pyx_n_s_data_type_name;
static PyObject *__pyx_n_s_data_types;
static PyObject *__pyx_n_s_data_types_from_mask;
static PyObject *__pyx_n_s_datetime;
static PyObject *__pyx_n_u_day;
static PyObject *__pyx_n_s_debug;
static PyObject *__pyx_n_s_determine_data_types;
static PyObject *__pyx_n_s_dic;
static PyObject *__pyx_n_u_dsp_type;
static PyObject *__pyx_n_s_dtype;
static PyObject *__pyx_n_u_dual_prf_delay;
static PyObject *__pyx_n_u_earth_radius;
static PyObject *__pyx_n_s_el0;
static PyObject *__pyx_n_s_el1;
static PyObject *__pyx_n_u_elevation_0;
static PyObject *__pyx_n_u_elevation_1;
static PyObject *__pyx_n_u_elevation_list;
static PyObject *__pyx_n_s_empty;
static PyObject *__pyx_n_s_empty_like;
static PyObject *__pyx_n_s_enumerate;
static PyObject *__pyx_n_s_exp;
static PyObject *__pyx_n_u_extended_header_type;
static PyObject *__pyx_n_u_extended_product_header_offset;
static PyObject *__pyx_n_u_extended_ray_header_bytes;
static PyObject *__pyx_n_u_f;
static PyObject *__pyx_n_u_fault_status;
static PyObject *__pyx_n_u_file_ingest_time;
static PyObject *__pyx_n_s_filename;
static PyObject *__pyx_n_u_filename;
static PyObject *__pyx_n_s_fill_value;
static PyObject *__pyx_n_u_first_azimuth;
static PyObject *__pyx_n_u_first_bin_range;
static PyObject *__pyx_n_u_first_elevation;
static PyObject *__pyx_n_u_first_ray_index;
static PyObject *__pyx_n_u_first_ray_offset;
static PyObject *__pyx_n_u_fixed_angle;
static PyObject *__pyx_n_u_flag;
static PyObject *__pyx_n_u_flags;
static PyObject *__pyx_n_u_flags2;
static PyObject *__pyx_n_u_float32;
static PyObject *__pyx_n_s_fmt;
static PyObject *__pyx_n_u_format_version;
static PyObject *__pyx_n_s_frombuffer;
static PyObject *__pyx_n_s_full_xhdr;
static PyObject *__pyx_n_u_gain_control_flag;
static PyObject *__pyx_n_u_gas_attenuation;
static PyObject *__pyx_n_u_generation_time;
static PyObject *__pyx_n_s_get_sweep;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_u_gmt_offset_minutes_local;
static PyObject *__pyx_n_u_gmt_offset_minutes_standard;
static PyObject *__pyx_n_u_gparam_bytes;
static PyObject *__pyx_n_u_gparm;
static PyObject *__pyx_n_u_ground_height;
static PyObject *__pyx_n_u_h;
static PyObject *__pyx_n_u_hardware_site;
static PyObject *__pyx_n_u_height_radar;
static PyObject *__pyx_n_u_height_site;
static PyObject *__pyx_n_u_horizontal_beamwidth;
static PyObject *__pyx_n_u_horizontal_calibration_i0;
static PyObject *__pyx_n_u_horizontal_calibration_noise;
static PyObject *__pyx_n_u_horizontal_current_noise;
static PyObject *__pyx_n_u_horizontal_i0_calibration;
static PyObject *__pyx_n_u_horizontal_noise_calibration;
static PyObject *__pyx_n_u_horizontal_radar_constant;
static PyObject *__pyx_n_s_i;
static PyObject *__pyx_n_u_i;
static PyObject *__pyx_n_u_i4;
static PyObject *__pyx_n_u_icolcnt;
static PyObject *__pyx_n_s_idh;
static PyObject *__pyx_n_u_iflags;
static PyObject *__pyx_n_u_ilevel_seams;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_ingest_configuration;
static PyObject *__pyx_n_u_ingest_configuration;
static PyObject *__pyx_n_u_ingest_hardware_name;
static PyObject *__pyx_n_s_ingest_header;
static PyObject *__pyx_n_u_ingest_iris_version;
static PyObject *__pyx_n_u_ingest_site_name;
static PyObject *__pyx_n_u_ingest_time;
static PyObject *__pyx_n_u_input_data_type;
static PyObject *__pyx_n_u_input_mask;
static PyObject *__pyx_n_u_int16;
static PyObject *__pyx_n_u_int32;
static PyObject *__pyx_n_u_inverse_flatting;
static PyObject *__pyx_n_u_iris_version;
static PyObject *__pyx_n_u_iris_version_created;
static PyObject *__pyx_n_s_is_bit_set;
static PyObject *__pyx_n_u_iset_and_scale;
static PyObject *__pyx_n_u_istart;
static PyObject *__pyx_n_u_istep;
static PyObject *__pyx_n_s_key;
static PyObject *__pyx_n_s_l;
static PyObject *__pyx_n_u_last_bin_range;
static PyObject *__pyx_n_u_last_run_day;
static PyObject *__pyx_n_u_last_run_time;
static PyObject *__pyx_n_u_latitude;
static PyObject *__pyx_n_u_latitude_projection;
static PyObject *__pyx_n_u_latitude_radar;
static PyObject *__pyx_n_u_ldr_bias;
static PyObject *__pyx_n_u_ldr_offset;
static PyObject *__pyx_n_u_left_azimuth_limit;
static PyObject *__pyx_n_s_like_dbt;
static PyObject *__pyx_n_s_like_dbt2;
static PyObject *__pyx_n_s_like_sqi;
static PyObject *__pyx_n_s_like_sqi2;
static PyObject *__pyx_n_u_linear_filter_first_bin;
static PyObject *__pyx_n_u_local_timezone;
static PyObject *__pyx_n_u_log_filter_first_bin;
static PyObject *__pyx_n_u_longitude;
static PyObject *__pyx_n_u_longitude_projection;
static PyObject *__pyx_n_u_longitude_radar;
static PyObject *__pyx_n_u_low_prf_factional;
static PyObject *__pyx_n_u_low_prf_hz;
static PyObject *__pyx_n_u_low_prf_range_averaging;
static PyObject *__pyx_n_u_low_prf_sample_size;
static PyObject *__pyx_n_u_lower_elevation_limit;
static PyObject *__pyx_n_s_ma;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_u_major_mode;
static PyObject *__pyx_n_s_mask;
static PyObject *__pyx_n_u_mask_word_0;
static PyObject *__pyx_n_u_mask_word_1;
static PyObject *__pyx_n_u_mask_word_2;
static PyObject *__pyx_n_u_mask_word_3;
static PyObject *__pyx_n_u_mask_word_4;
static PyObject *__pyx_n_s_masked_array;
static PyObject *__pyx_n_u_maximum_range;
static PyObject *__pyx_n_u_mean_wind_direction;
static PyObject *__pyx_n_u_mean_wind_speed;
static PyObject *__pyx_n_u_melting_layer;
static PyObject *__pyx_n_u_melting_level;
static PyObject *__pyx_n_u_milliseconds;
static PyObject *__pyx_n_u_minor_task_suffix;
static PyObject *__pyx_n_s_mod;
static PyObject *__pyx_n_u_month;
static PyObject *__pyx_n_u_multi_prf_flag;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_nbin;
static PyObject *__pyx_n_s_nbins;
static PyObject *__pyx_n_u_nbins;
static PyObject *__pyx_n_s_ndata;
static PyObject *__pyx_n_s_ndata_types;
static PyObject *__pyx_n_u_nexrad_clutter_skip;
static PyObject *__pyx_n_u_nexrad_clutter_threshold;
static PyObject *__pyx_n_s_np;
static PyObject *__pyx_n_s_nrays;
static PyObject *__pyx_n_s_number;
static PyObject *__pyx_n_u_number_bins;
static PyObject *__pyx_n_u_number_elements;
static PyObject *__pyx_n_u_number_files;
static PyObject *__pyx_n_u_number_ingest;
static PyObject *__pyx_n_u_number_input_bins;
static PyObject *__pyx_n_u_number_linear_filter;
static PyObject *__pyx_n_u_number_log_filter;
static PyObject *__pyx_n_u_number_output_bins;
static PyObject *__pyx_n_u_number_rays_file_actual;
static PyObject *__pyx_n_u_number_rays_file_expected;
static PyObject *__pyx_n_u_number_rays_sweep;
static PyObject *__pyx_n_u_number_sweeps;
static PyObject *__pyx_n_u_number_sweeps_completed;
static PyObject *__pyx_n_u_number_task_config_table;
static PyObject *__pyx_n_u_number_tasks;
static PyObject *__pyx_n_s_numpy;
static PyObject *__pyx_kp_u_numpy_core_multiarray_failed_to;
static PyObject *__pyx_kp_u_numpy_core_umath_failed_to_impor;
static PyObject *__pyx_n_s_offset;
static PyObject *__pyx_n_s_ones;
static PyObject *__pyx_n_s_open;
static PyObject *__pyx_n_u_original_data_type_mask;
static PyObject *__pyx_n_s_out;
static PyObject *__pyx_n_s_parse_ray_headers;
static PyObject *__pyx_n_u_playback_flag;
static PyObject *__pyx_n_u_playback_version;
static PyObject *__pyx_n_u_polarization;
static PyObject *__pyx_n_u_polarization_type;
static PyObject *__pyx_n_s_power;
static PyObject *__pyx_n_u_power_threshold;
static PyObject *__pyx_n_u_prf;
static PyObject *__pyx_n_s_prf_flag;
static PyObject *__pyx_n_u_prf_flag;
static PyObject *__pyx_n_s_print;
static PyObject *__pyx_n_s_product_config;
static PyObject *__pyx_n_u_product_configuration;
static PyObject *__pyx_n_s_product_end;
static PyObject *__pyx_n_u_product_end;
static PyObject *__pyx_n_s_product_hdr;
static PyObject *__pyx_n_u_product_name;
static PyObject *__pyx_n_u_product_sequence_number;
static PyObject *__pyx_n_u_product_specific_bytes;
static PyObject *__pyx_n_u_product_type_code;
static PyObject *__pyx_n_u_projection_name;
static PyObject *__pyx_n_u_projection_type;
static PyObject *__pyx_n_u_pulse_width;
static PyObject *__pyx_n_s_pyart_io__sigmetfile;
static PyObject *__pyx_kp_s_pyart_io__sigmetfile_pyx;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_u_radar_height;
static PyObject *__pyx_n_u_radar_height_above_reference;
static PyObject *__pyx_n_u_radial_smoother;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_u_range_bin_averaging_flag;
static PyObject *__pyx_n_s_raw_data;
static PyObject *__pyx_n_u_ray_header_bytes;
static PyObject *__pyx_n_u_ray_header_mask;
static PyObject *__pyx_n_s_ray_headers;
static PyObject *__pyx_n_u_ray_number;
static PyObject *__pyx_n_u_rb;
static PyObject *__pyx_n_s_read;
static PyObject *__pyx_n_u_read;
static PyObject *__pyx_n_u_reciever_bandwidth;
static PyObject *__pyx_n_s_record;
static PyObject *__pyx_n_u_record_number;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_u_reflectivity_calibration;
static PyObject *__pyx_n_u_reflectivity_noise_threshold;
static PyObject *__pyx_n_u_reflectivity_slope;
static PyObject *__pyx_n_u_reflectivity_unfolding_threshold;
static PyObject *__pyx_n_u_reserved;
static PyObject *__pyx_n_s_reshape;
static PyObject *__pyx_n_u_resolution_rays;
static PyObject *__pyx_n_u_right_azimuth_limit;
static PyObject *__pyx_n_u_sample_size;
static PyObject *__pyx_n_u_samples_used;
static PyObject *__pyx_n_u_scheduling_code;
static PyObject *__pyx_n_u_seconds;
static PyObject *__pyx_n_u_seconds_between_runs;
static PyObject *__pyx_kp_s_self__rbuf_p_cannot_be_converted;
static PyObject *__pyx_kp_u_self__rbuf_pos_is;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_shape;
static PyObject *__pyx_n_s_shrink;
static PyObject *__pyx_n_u_signal_processor_type;
static PyObject *__pyx_n_u_site_name;
static PyObject *__pyx_n_u_skip_time;
static PyObject *__pyx_n_u_spare_0;
static PyObject *__pyx_n_u_spare_1;
static PyObject *__pyx_n_u_spare_2;
static PyObject *__pyx_n_u_spare_3;
static PyObject *__pyx_n_u_spare_4;
static PyObject *__pyx_n_u_sqi_threshold;
static PyObject *__pyx_n_s_sqrt;
static PyObject *__pyx_n_u_standard_parallel_1;
static PyObject *__pyx_n_u_standard_parallel_2;
static PyObject *__pyx_n_u_start_first_section_flag;
static PyObject *__pyx_n_u_start_first_sector_flag;
static PyObject *__pyx_n_u_start_time;
static PyObject *__pyx_n_u_step_input_bins;
static PyObject *__pyx_n_u_step_output_bins;
static PyObject *__pyx_n_u_stop_time;
static PyObject *__pyx_n_s_string;
static PyObject *__pyx_n_s_struct;
static PyObject *__pyx_n_s_structure;
static PyObject *__pyx_n_u_structure_header;
static PyObject *__pyx_n_u_structure_identifier;
static PyObject *__pyx_n_s_sum;
static PyObject *__pyx_n_u_sweep_ingest_time;
static PyObject *__pyx_n_u_sweep_number;
static PyObject *__pyx_n_u_sweep_start_time;
static PyObject *__pyx_n_u_task_calib_info;
static PyObject *__pyx_n_s_task_configuration;
static PyObject *__pyx_n_u_task_configuration;
static PyObject *__pyx_n_u_task_configuration_file_name;
static PyObject *__pyx_n_u_task_data_time;
static PyObject *__pyx_n_u_task_description;
static PyObject *__pyx_n_s_task_dsp_info;
static PyObject *__pyx_n_u_task_dsp_info;
static PyObject *__pyx_n_u_task_dsp_mode;
static PyObject *__pyx_n_s_task_end_info;
static PyObject *__pyx_n_u_task_end_info;
static PyObject *__pyx_n_u_task_major_number;
static PyObject *__pyx_n_u_task_minor_number;
static PyObject *__pyx_n_u_task_misc_info;
static PyObject *__pyx_n_u_task_name;
static PyObject *__pyx_n_u_task_range_info;
static PyObject *__pyx_n_u_task_scan_info;
static PyObject *__pyx_n_u_task_scan_type_scan_info;
static PyObject *__pyx_n_u_task_sched_info;
static PyObject *__pyx_n_u_task_state;
static PyObject *__pyx_n_u_tcf_cal_flags_1;
static PyObject *__pyx_n_u_tcf_cal_flags_2;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_time;
static PyObject *__pyx_n_u_time;
static PyObject *__pyx_n_u_time_used_last_run;
static PyObject *__pyx_n_u_times_run;
static PyObject *__pyx_n_u_total_size;
static PyObject *__pyx_n_u_tr_serial_number;
static PyObject *__pyx_n_u_tranmitter_phase_sequence;
static PyObject *__pyx_n_u_transmit_power;
static PyObject *__pyx_n_u_trigger_rate;
static PyObject *__pyx_n_u_trucation_height;
static PyObject *__pyx_n_u_truncation_height;
static PyObject *__pyx_n_u_tz_name;
static PyObject *__pyx_n_u_uint16;
static PyObject *__pyx_n_s_uint8;
static PyObject *__pyx_n_u_uncorrected_reflectivity_thresho;
static PyObject *__pyx_n_s_unpack;
static PyObject *__pyx_n_s_unpack_ingest_data_header;
static PyObject *__pyx_n_s_unpack_ingest_data_headers;
static PyObject *__pyx_n_s_unpack_ingest_header;
static PyObject *__pyx_n_s_unpack_key;
static PyObject *__pyx_n_s_unpack_product_hdr;
static PyObject *__pyx_n_s_unpack_raw_prod_bhdr;
static PyObject *__pyx_n_s_unpack_structure;
static PyObject *__pyx_n_u_upper_elevation_limit;
static PyObject *__pyx_n_u_variable_range_bin_flag;
static PyObject *__pyx_n_u_velocity_east;
static PyObject *__pyx_n_u_velocity_north;
static PyObject *__pyx_n_u_velocity_threshold_flags;
static PyObject *__pyx_n_u_velocity_unfolding_threshold;
static PyObject *__pyx_n_u_velocity_up;
static PyObject *__pyx_n_u_vertical_beamwidth;
static PyObject *__pyx_n_u_vertical_current_noise;
static PyObject *__pyx_n_u_vertical_i0_calibration;
static PyObject *__pyx_n_u_vertical_noise_calibration;
static PyObject *__pyx_n_u_vertical_radar_constant;
static PyObject *__pyx_n_s_view;
static PyObject *__pyx_n_u_volume_scan_start_time;
static PyObject *__pyx_n_s_warn;
static PyObject *__pyx_n_s_warnings;
static PyObject *__pyx_n_u_wavelength;
static PyObject *__pyx_n_u_width_threshold_flags;
static PyObject *__pyx_n_u_width_unfolding_threshold;
static PyObject *__pyx_n_s_word0;
static PyObject *__pyx_n_s_word1;
static PyObject *__pyx_n_s_word2;
static PyObject *__pyx_n_s_word3;
static PyObject *__pyx_n_u_x_location;
static PyObject *__pyx_n_u_x_scale;
static PyObject *__pyx_n_u_x_size;
static PyObject *__pyx_n_u_x_smoother;
static PyObject *__pyx_n_s_xrange;
static PyObject *__pyx_n_u_y_location;
static PyObject *__pyx_n_u_y_scale;
static PyObject *__pyx_n_u_y_size;
static PyObject *__pyx_n_u_y_smoother;
static PyObject *__pyx_n_u_year;
static PyObject *__pyx_n_u_z_location;
static PyObject *__pyx_n_u_z_scale;
static PyObject *__pyx_n_u_z_size;
static PyObject *__pyx_n_u_zdr_bias;
static PyObject *__pyx_n_u_zdr_offset;
static PyObject *__pyx_n_u_zdr_threshold_flags;
static PyObject *__pyx_n_s_zeros_like;
static PyObject *__pyx_n_s_zip;
static PyObject *__pyx_n_u_zr_constant;
static PyObject *__pyx_n_u_zr_exponent;
static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile___init__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_filename, PyObject *__pyx_v_debug); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_2_determine_data_types(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_4close(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_6read_data(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_full_xhdr); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_8_get_sweep(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_full_xhdr, PyObject *__pyx_v_raw_data); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_5debug___get__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_5debug_2__set__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_5debug_4__del__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_11product_hdr___get__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_11product_hdr_2__set__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_11product_hdr_4__del__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_13ingest_header___get__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_13ingest_header_2__set__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_13ingest_header_4__del__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_19ingest_data_headers___get__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_19ingest_data_headers_2__set__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_19ingest_data_headers_4__del__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_10data_types___get__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_10data_types_2__set__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_10data_types_4__del__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_15data_type_names___get__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_15data_type_names_2__set__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_15data_type_names_4__del__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_11ndata_types___get__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_11ndata_types_2__set__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_11ndata_types_4__del__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_3_fh___get__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_3_fh_2__set__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_3_fh_4__del__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_18_raw_product_bhdrs___get__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_18_raw_product_bhdrs_2__set__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_18_raw_product_bhdrs_4__del__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_9_rbuf_pos___get__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_9_rbuf_pos_2__set__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_14_record_number___get__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_14_record_number_2__set__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_10__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_12__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile__data_types_from_mask(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_word0, PyObject *__pyx_v_word1, PyObject *__pyx_v_word2, PyObject *__pyx_v_word3); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_2_is_bit_set(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_number, PyObject *__pyx_v_bit); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_4_parse_ray_headers(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_ray_headers); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_6convert_sigmet_data(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_data_type, PyObject *__pyx_v_data, PyObject *__pyx_v_nbins); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_8bin2_to_angle(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_bin2); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10bin4_to_angle(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_bin4); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_12_unpack_structure(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_string, PyObject *__pyx_v_structure); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_14_unpack_key(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_dic, PyObject *__pyx_v_key, PyObject *__pyx_v_structure); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_16_unpack_ingest_data_headers(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_record, PyObject *__pyx_v_ndata_types); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_18_unpack_ingest_data_header(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_record, PyObject *__pyx_v_number); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_20_unpack_raw_prod_bhdr(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_record); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_22_unpack_product_hdr(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_record); /* proto */
static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_24_unpack_ingest_header(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_record); /* proto */
static PyObject *__pyx_tp_new_5pyart_2io_11_sigmetfile_SigmetFile(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_float_1_;
static PyObject *__pyx_float_2_;
static PyObject *__pyx_float_16_;
static PyObject *__pyx_float_64_;
static PyObject *__pyx_float_0_25;
static PyObject *__pyx_float_100_;
static PyObject *__pyx_float_126_;
static PyObject *__pyx_float_127_;
static PyObject *__pyx_float_128_;
static PyObject *__pyx_float_129_;
static PyObject *__pyx_float_180_;
static PyObject *__pyx_float_253_;
static PyObject *__pyx_float_254_;
static PyObject *__pyx_float_256_;
static PyObject *__pyx_float_360_;
static PyObject *__pyx_float_600_;
static PyObject *__pyx_float_32768_;
static PyObject *__pyx_float_65533_;
static PyObject *__pyx_float_65534_;
static PyObject *__pyx_float_neg_0_25;
static PyObject *__pyx_float_neg_9999_0;
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_2;
static PyObject *__pyx_int_3;
static PyObject *__pyx_int_4;
static PyObject *__pyx_int_5;
static PyObject *__pyx_int_6;
static PyObject *__pyx_int_7;
static PyObject *__pyx_int_8;
static PyObject *__pyx_int_9;
static PyObject *__pyx_int_10;
static PyObject *__pyx_int_11;
static PyObject *__pyx_int_12;
static PyObject *__pyx_int_13;
static PyObject *__pyx_int_14;
static PyObject *__pyx_int_15;
static PyObject *__pyx_int_16;
static PyObject *__pyx_int_17;
static PyObject *__pyx_int_18;
static PyObject *__pyx_int_19;
static PyObject *__pyx_int_20;
static PyObject *__pyx_int_21;
static PyObject *__pyx_int_22;
static PyObject *__pyx_int_23;
static PyObject *__pyx_int_24;
static PyObject *__pyx_int_25;
static PyObject *__pyx_int_26;
static PyObject *__pyx_int_27;
static PyObject *__pyx_int_28;
static PyObject *__pyx_int_29;
static PyObject *__pyx_int_30;
static PyObject *__pyx_int_31;
static PyObject *__pyx_int_32;
static PyObject *__pyx_int_33;
static PyObject *__pyx_int_34;
static PyObject *__pyx_int_35;
static PyObject *__pyx_int_36;
static PyObject *__pyx_int_37;
static PyObject *__pyx_int_38;
static PyObject *__pyx_int_39;
static PyObject *__pyx_int_40;
static PyObject *__pyx_int_41;
static PyObject *__pyx_int_42;
static PyObject *__pyx_int_43;
static PyObject *__pyx_int_44;
static PyObject *__pyx_int_45;
static PyObject *__pyx_int_46;
static PyObject *__pyx_int_47;
static PyObject *__pyx_int_48;
static PyObject *__pyx_int_49;
static PyObject *__pyx_int_50;
static PyObject *__pyx_int_51;
static PyObject *__pyx_int_52;
static PyObject *__pyx_int_53;
static PyObject *__pyx_int_54;
static PyObject *__pyx_int_55;
static PyObject *__pyx_int_56;
static PyObject *__pyx_int_57;
static PyObject *__pyx_int_58;
static PyObject *__pyx_int_59;
static PyObject *__pyx_int_60;
static PyObject *__pyx_int_61;
static PyObject *__pyx_int_62;
static PyObject *__pyx_int_63;
static PyObject *__pyx_int_64;
static PyObject *__pyx_int_65;
static PyObject *__pyx_int_66;
static PyObject *__pyx_int_67;
static PyObject *__pyx_int_68;
static PyObject *__pyx_int_69;
static PyObject *__pyx_int_70;
static PyObject *__pyx_int_71;
static PyObject *__pyx_int_72;
static PyObject *__pyx_int_73;
static PyObject *__pyx_int_74;
static PyObject *__pyx_int_75;
static PyObject *__pyx_int_76;
static PyObject *__pyx_int_77;
static PyObject *__pyx_int_78;
static PyObject *__pyx_int_79;
static PyObject *__pyx_int_80;
static PyObject *__pyx_int_81;
static PyObject *__pyx_int_82;
static PyObject *__pyx_int_83;
static PyObject *__pyx_int_84;
static PyObject *__pyx_int_85;
static PyObject *__pyx_int_86;
static PyObject *__pyx_int_87;
static PyObject *__pyx_int_88;
static PyObject *__pyx_int_89;
static PyObject *__pyx_int_90;
static PyObject *__pyx_int_91;
static PyObject *__pyx_int_92;
static PyObject *__pyx_int_93;
static PyObject *__pyx_int_94;
static PyObject *__pyx_int_95;
static PyObject *__pyx_int_96;
static PyObject *__pyx_int_97;
static PyObject *__pyx_int_98;
static PyObject *__pyx_int_99;
static PyObject *__pyx_int_100;
static PyObject *__pyx_int_101;
static PyObject *__pyx_int_102;
static PyObject *__pyx_int_103;
static PyObject *__pyx_int_104;
static PyObject *__pyx_int_105;
static PyObject *__pyx_int_106;
static PyObject *__pyx_int_107;
static PyObject *__pyx_int_108;
static PyObject *__pyx_int_109;
static PyObject *__pyx_int_110;
static PyObject *__pyx_int_111;
static PyObject *__pyx_int_112;
static PyObject *__pyx_int_113;
static PyObject *__pyx_int_114;
static PyObject *__pyx_int_115;
static PyObject *__pyx_int_116;
static PyObject *__pyx_int_117;
static PyObject *__pyx_int_118;
static PyObject *__pyx_int_119;
static PyObject *__pyx_int_120;
static PyObject *__pyx_int_121;
static PyObject *__pyx_int_122;
static PyObject *__pyx_int_123;
static PyObject *__pyx_int_124;
static PyObject *__pyx_int_125;
static PyObject *__pyx_int_126;
static PyObject *__pyx_int_127;
static PyObject *__pyx_int_128;
static PyObject *__pyx_int_255;
static PyObject *__pyx_int_640;
static PyObject *__pyx_int_4884;
static PyObject *__pyx_int_6144;
static PyObject *__pyx_int_65536;
static PyObject *__pyx_int_4294967296;
static PyObject *__pyx_int_neg_1;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_slice__2;
static PyObject *__pyx_slice__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_slice__12;
static PyObject *__pyx_slice__13;
static PyObject *__pyx_slice__16;
static PyObject *__pyx_slice__17;
static PyObject *__pyx_slice__18;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__20;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__23;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_tuple__29;
static PyObject *__pyx_tuple__31;
static PyObject *__pyx_tuple__33;
static PyObject *__pyx_tuple__35;
static PyObject *__pyx_tuple__37;
static PyObject *__pyx_tuple__39;
static PyObject *__pyx_tuple__41;
static PyObject *__pyx_tuple__43;
static PyObject *__pyx_tuple__45;
static PyObject *__pyx_tuple__47;
static PyObject *__pyx_tuple__48;
static PyObject *__pyx_tuple__49;
static PyObject *__pyx_tuple__50;
static PyObject *__pyx_tuple__51;
static PyObject *__pyx_tuple__52;
static PyObject *__pyx_tuple__53;
static PyObject *__pyx_tuple__54;
static PyObject *__pyx_tuple__55;
static PyObject *__pyx_tuple__56;
static PyObject *__pyx_tuple__57;
static PyObject *__pyx_tuple__58;
static PyObject *__pyx_tuple__59;
static PyObject *__pyx_tuple__60;
static PyObject *__pyx_tuple__61;
static PyObject *__pyx_tuple__62;
static PyObject *__pyx_tuple__63;
static PyObject *__pyx_tuple__64;
static PyObject *__pyx_tuple__65;
static PyObject *__pyx_tuple__66;
static PyObject *__pyx_tuple__67;
static PyObject *__pyx_tuple__68;
static PyObject *__pyx_tuple__69;
static PyObject *__pyx_tuple__70;
static PyObject *__pyx_tuple__71;
static PyObject *__pyx_tuple__72;
static PyObject *__pyx_tuple__73;
static PyObject *__pyx_tuple__74;
static PyObject *__pyx_tuple__75;
static PyObject *__pyx_tuple__76;
static PyObject *__pyx_tuple__77;
static PyObject *__pyx_tuple__78;
static PyObject *__pyx_tuple__79;
static PyObject *__pyx_tuple__80;
static PyObject *__pyx_tuple__81;
static PyObject *__pyx_tuple__82;
static PyObject *__pyx_tuple__83;
static PyObject *__pyx_tuple__84;
static PyObject *__pyx_tuple__85;
static PyObject *__pyx_tuple__86;
static PyObject *__pyx_tuple__87;
static PyObject *__pyx_tuple__88;
static PyObject *__pyx_tuple__89;
static PyObject *__pyx_tuple__90;
static PyObject *__pyx_tuple__91;
static PyObject *__pyx_tuple__92;
static PyObject *__pyx_tuple__93;
static PyObject *__pyx_tuple__94;
static PyObject *__pyx_tuple__95;
static PyObject *__pyx_tuple__96;
static PyObject *__pyx_tuple__97;
static PyObject *__pyx_tuple__98;
static PyObject *__pyx_tuple__99;
static PyObject *__pyx_tuple__100;
static PyObject *__pyx_tuple__101;
static PyObject *__pyx_tuple__102;
static PyObject *__pyx_tuple__103;
static PyObject *__pyx_tuple__104;
static PyObject *__pyx_tuple__105;
static PyObject *__pyx_tuple__106;
static PyObject *__pyx_tuple__107;
static PyObject *__pyx_tuple__108;
static PyObject *__pyx_tuple__109;
static PyObject *__pyx_tuple__110;
static PyObject *__pyx_tuple__111;
static PyObject *__pyx_tuple__112;
static PyObject *__pyx_tuple__113;
static PyObject *__pyx_tuple__114;
static PyObject *__pyx_tuple__115;
static PyObject *__pyx_tuple__116;
static PyObject *__pyx_tuple__117;
static PyObject *__pyx_tuple__118;
static PyObject *__pyx_tuple__119;
static PyObject *__pyx_tuple__120;
static PyObject *__pyx_tuple__121;
static PyObject *__pyx_tuple__122;
static PyObject *__pyx_tuple__123;
static PyObject *__pyx_tuple__124;
static PyObject *__pyx_tuple__125;
static PyObject *__pyx_tuple__126;
static PyObject *__pyx_tuple__127;
static PyObject *__pyx_tuple__128;
static PyObject *__pyx_tuple__129;
static PyObject *__pyx_tuple__130;
static PyObject *__pyx_tuple__131;
static PyObject *__pyx_tuple__132;
static PyObject *__pyx_tuple__133;
static PyObject *__pyx_tuple__134;
static PyObject *__pyx_tuple__135;
static PyObject *__pyx_tuple__136;
static PyObject *__pyx_codeobj__22;
static PyObject *__pyx_codeobj__24;
static PyObject *__pyx_codeobj__26;
static PyObject *__pyx_codeobj__28;
static PyObject *__pyx_codeobj__30;
static PyObject *__pyx_codeobj__32;
static PyObject *__pyx_codeobj__34;
static PyObject *__pyx_codeobj__36;
static PyObject *__pyx_codeobj__38;
static PyObject *__pyx_codeobj__40;
static PyObject *__pyx_codeobj__42;
static PyObject *__pyx_codeobj__44;
static PyObject *__pyx_codeobj__46;
/* Late includes */

/* "pyart/io/_sigmetfile.pyx":57
 *     cdef public int _rbuf_pos, _record_number
 * 
 *     def __init__(self, filename, debug=False):             # <<<<<<<<<<<<<<
 *         """ initalize the object. """
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5pyart_2io_11_sigmetfile_10SigmetFile___init__[] = " initalize the object. ";
#if CYTHON_COMPILING_IN_CPYTHON
struct wrapperbase __pyx_wrapperbase_5pyart_2io_11_sigmetfile_10SigmetFile___init__;
#endif
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_filename = 0;
  PyObject *__pyx_v_debug = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_filename,&__pyx_n_s_debug,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_filename)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_debug);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 57, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_filename = values[0];
    __pyx_v_debug = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 57, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._sigmetfile.SigmetFile.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile___init__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self), __pyx_v_filename, __pyx_v_debug);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile___init__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_filename, PyObject *__pyx_v_debug) {
  PyObject *__pyx_v_fh = NULL;
  PyObject *__pyx_7genexpr__pyx_v_i = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  Py_ssize_t __pyx_t_9;
  PyObject *(*__pyx_t_10)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pyart/io/_sigmetfile.pyx":60
 *         """ initalize the object. """
 * 
 *         self.debug = debug             # <<<<<<<<<<<<<<
 * 
 *         # open the file
 */
  __Pyx_INCREF(__pyx_v_debug);
  __Pyx_GIVEREF(__pyx_v_debug);
  __Pyx_GOTREF(__pyx_v_self->debug);
  __Pyx_DECREF(__pyx_v_self->debug);
  __pyx_v_self->debug = __pyx_v_debug;

  /* "pyart/io/_sigmetfile.pyx":63
 * 
 *         # open the file
 *         if hasattr(filename, 'read'):             # <<<<<<<<<<<<<<
 *             fh = filename
 *         else:
 */
  __pyx_t_1 = __Pyx_HasAttr(__pyx_v_filename, __pyx_n_u_read); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 63, __pyx_L1_error)
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "pyart/io/_sigmetfile.pyx":64
 *         # open the file
 *         if hasattr(filename, 'read'):
 *             fh = filename             # <<<<<<<<<<<<<<
 *         else:
 *             fh = open(filename, 'rb')
 */
    __Pyx_INCREF(__pyx_v_filename);
    __pyx_v_fh = __pyx_v_filename;

    /* "pyart/io/_sigmetfile.pyx":63
 * 
 *         # open the file
 *         if hasattr(filename, 'read'):             # <<<<<<<<<<<<<<
 *             fh = filename
 *         else:
 */
    goto __pyx_L3;
  }

  /* "pyart/io/_sigmetfile.pyx":66
 *             fh = filename
 *         else:
 *             fh = open(filename, 'rb')             # <<<<<<<<<<<<<<
 * 
 *         # read the headers from the first 2 records.
 */
  /*else*/ {
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 66, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_filename);
    __Pyx_GIVEREF(__pyx_v_filename);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_filename);
    __Pyx_INCREF(__pyx_n_u_rb);
    __Pyx_GIVEREF(__pyx_n_u_rb);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_n_u_rb);
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_open, __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 66, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_fh = __pyx_t_4;
    __pyx_t_4 = 0;
  }
  __pyx_L3:;

  /* "pyart/io/_sigmetfile.pyx":69
 * 
 *         # read the headers from the first 2 records.
 *         self.product_hdr = _unpack_product_hdr(fh.read(RECORD_SIZE))             # <<<<<<<<<<<<<<
 *         self.ingest_header = _unpack_ingest_header(fh.read(RECORD_SIZE))
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_unpack_product_hdr); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 69, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_fh, __pyx_n_s_read); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 69, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_RECORD_SIZE); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 69, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
    }
  }
  __pyx_t_5 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_8, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 69, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 69, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->product_hdr);
  __Pyx_DECREF(__pyx_v_self->product_hdr);
  __pyx_v_self->product_hdr = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "pyart/io/_sigmetfile.pyx":70
 *         # read the headers from the first 2 records.
 *         self.product_hdr = _unpack_product_hdr(fh.read(RECORD_SIZE))
 *         self.ingest_header = _unpack_ingest_header(fh.read(RECORD_SIZE))             # <<<<<<<<<<<<<<
 * 
 *         # determine data types contained in the file
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_unpack_ingest_header); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_fh, __pyx_n_s_read); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_RECORD_SIZE); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
    }
  }
  __pyx_t_5 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_8, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->ingest_header);
  __Pyx_DECREF(__pyx_v_self->ingest_header);
  __pyx_v_self->ingest_header = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "pyart/io/_sigmetfile.pyx":73
 * 
 *         # determine data types contained in the file
 *         self.data_types = self._determine_data_types()             # <<<<<<<<<<<<<<
 *         self.ndata_types = len(self.data_types)
 *         self.data_type_names = [SIGMET_DATA_TYPES[i] for i in self.data_types]
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_determine_data_types); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_4 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->data_types);
  __Pyx_DECREF(__pyx_v_self->data_types);
  __pyx_v_self->data_types = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "pyart/io/_sigmetfile.pyx":74
 *         # determine data types contained in the file
 *         self.data_types = self._determine_data_types()
 *         self.ndata_types = len(self.data_types)             # <<<<<<<<<<<<<<
 *         self.data_type_names = [SIGMET_DATA_TYPES[i] for i in self.data_types]
 * 
 */
  __pyx_t_4 = __pyx_v_self->data_types;
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_9 = PyObject_Length(__pyx_t_4); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyInt_FromSsize_t(__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->ndata_types);
  __Pyx_DECREF(__pyx_v_self->ndata_types);
  __pyx_v_self->ndata_types = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "pyart/io/_sigmetfile.pyx":75
 *         self.data_types = self._determine_data_types()
 *         self.ndata_types = len(self.data_types)
 *         self.data_type_names = [SIGMET_DATA_TYPES[i] for i in self.data_types]             # <<<<<<<<<<<<<<
 * 
 *         # set attributes
 */
  { /* enter inner scope */
    __pyx_t_4 = PyList_New(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 75, __pyx_L6_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (likely(PyList_CheckExact(__pyx_v_self->data_types)) || PyTuple_CheckExact(__pyx_v_self->data_types)) {
      __pyx_t_3 = __pyx_v_self->data_types; __Pyx_INCREF(__pyx_t_3); __pyx_t_9 = 0;
      __pyx_t_10 = NULL;
    } else {
      __pyx_t_9 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_v_self->data_types); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 75, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_10 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 75, __pyx_L6_error)
    }
    for (;;) {
      if (likely(!__pyx_t_10)) {
        if (likely(PyList_CheckExact(__pyx_t_3))) {
          if (__pyx_t_9 >= PyList_GET_SIZE(__pyx_t_3)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_9); __Pyx_INCREF(__pyx_t_5); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 75, __pyx_L6_error)
          #else
          __pyx_t_5 = PySequence_ITEM(__pyx_t_3, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 75, __pyx_L6_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        } else {
          if (__pyx_t_9 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_9); __Pyx_INCREF(__pyx_t_5); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 75, __pyx_L6_error)
          #else
          __pyx_t_5 = PySequence_ITEM(__pyx_t_3, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 75, __pyx_L6_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        }
      } else {
        __pyx_t_5 = __pyx_t_10(__pyx_t_3);
        if (unlikely(!__pyx_t_5)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 75, __pyx_L6_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_5);
      }
      __Pyx_XDECREF_SET(__pyx_7genexpr__pyx_v_i, __pyx_t_5);
      __pyx_t_5 = 0;
      __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_SIGMET_DATA_TYPES); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 75, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = __Pyx_PyObject_GetItem(__pyx_t_5, __pyx_7genexpr__pyx_v_i); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 75, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_4, (PyObject*)__pyx_t_6))) __PYX_ERR(0, 75, __pyx_L6_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_7genexpr__pyx_v_i); __pyx_7genexpr__pyx_v_i = 0;
    goto __pyx_L9_exit_scope;
    __pyx_L6_error:;
    __Pyx_XDECREF(__pyx_7genexpr__pyx_v_i); __pyx_7genexpr__pyx_v_i = 0;
    goto __pyx_L1_error;
    __pyx_L9_exit_scope:;
  } /* exit inner scope */
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->data_type_names);
  __Pyx_DECREF(__pyx_v_self->data_type_names);
  __pyx_v_self->data_type_names = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "pyart/io/_sigmetfile.pyx":78
 * 
 *         # set attributes
 *         self.ingest_data_headers = None             # <<<<<<<<<<<<<<
 *         self._fh = fh
 *         self._record_number = 2
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->ingest_data_headers);
  __Pyx_DECREF(__pyx_v_self->ingest_data_headers);
  __pyx_v_self->ingest_data_headers = Py_None;

  /* "pyart/io/_sigmetfile.pyx":79
 *         # set attributes
 *         self.ingest_data_headers = None
 *         self._fh = fh             # <<<<<<<<<<<<<<
 *         self._record_number = 2
 *         self._raw_product_bhdrs = []
 */
  __Pyx_INCREF(__pyx_v_fh);
  __Pyx_GIVEREF(__pyx_v_fh);
  __Pyx_GOTREF(__pyx_v_self->_fh);
  __Pyx_DECREF(__pyx_v_self->_fh);
  __pyx_v_self->_fh = __pyx_v_fh;

  /* "pyart/io/_sigmetfile.pyx":80
 *         self.ingest_data_headers = None
 *         self._fh = fh
 *         self._record_number = 2             # <<<<<<<<<<<<<<
 *         self._raw_product_bhdrs = []
 * 
 */
  __pyx_v_self->_record_number = 2;

  /* "pyart/io/_sigmetfile.pyx":81
 *         self._fh = fh
 *         self._record_number = 2
 *         self._raw_product_bhdrs = []             # <<<<<<<<<<<<<<
 * 
 *     def _determine_data_types(self):
 */
  __pyx_t_4 = PyList_New(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->_raw_product_bhdrs);
  __Pyx_DECREF(__pyx_v_self->_raw_product_bhdrs);
  __pyx_v_self->_raw_product_bhdrs = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "pyart/io/_sigmetfile.pyx":57
 *     cdef public int _rbuf_pos, _record_number
 * 
 *     def __init__(self, filename, debug=False):             # <<<<<<<<<<<<<<
 *         """ initalize the object. """
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("pyart.io._sigmetfile.SigmetFile.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_fh);
  __Pyx_XDECREF(__pyx_7genexpr__pyx_v_i);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_sigmetfile.pyx":83
 *         self._raw_product_bhdrs = []
 * 
 *     def _determine_data_types(self):             # <<<<<<<<<<<<<<
 *         """ Determine the available data types in the file. """
 *         # determine the available fields
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_3_determine_data_types(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5pyart_2io_11_sigmetfile_10SigmetFile_2_determine_data_types[] = " Determine the available data types in the file. ";
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_3_determine_data_types(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_determine_data_types (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_2_determine_data_types(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_2_determine_data_types(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self) {
  PyObject *__pyx_v_task_config = NULL;
  PyObject *__pyx_v_task_dsp_info = NULL;
  PyObject *__pyx_v_word0 = NULL;
  PyObject *__pyx_v_word1 = NULL;
  PyObject *__pyx_v_word2 = NULL;
  PyObject *__pyx_v_word3 = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_determine_data_types", 0);

  /* "pyart/io/_sigmetfile.pyx":86
 *         """ Determine the available data types in the file. """
 *         # determine the available fields
 *         task_config = self.ingest_header['task_configuration']             # <<<<<<<<<<<<<<
 *         task_dsp_info = task_config['task_dsp_info']
 *         word0 = task_dsp_info['current_data_type_mask']['mask_word_0']
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_self->ingest_header, __pyx_n_u_task_configuration); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_task_config = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":87
 *         # determine the available fields
 *         task_config = self.ingest_header['task_configuration']
 *         task_dsp_info = task_config['task_dsp_info']             # <<<<<<<<<<<<<<
 *         word0 = task_dsp_info['current_data_type_mask']['mask_word_0']
 *         word1 = task_dsp_info['current_data_type_mask']['mask_word_1']
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_task_config, __pyx_n_u_task_dsp_info); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 87, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_task_dsp_info = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":88
 *         task_config = self.ingest_header['task_configuration']
 *         task_dsp_info = task_config['task_dsp_info']
 *         word0 = task_dsp_info['current_data_type_mask']['mask_word_0']             # <<<<<<<<<<<<<<
 *         word1 = task_dsp_info['current_data_type_mask']['mask_word_1']
 *         word2 = task_dsp_info['current_data_type_mask']['mask_word_2']
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_task_dsp_info, __pyx_n_u_current_data_type_mask); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 88, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_mask_word_0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 88, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_word0 = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pyart/io/_sigmetfile.pyx":89
 *         task_dsp_info = task_config['task_dsp_info']
 *         word0 = task_dsp_info['current_data_type_mask']['mask_word_0']
 *         word1 = task_dsp_info['current_data_type_mask']['mask_word_1']             # <<<<<<<<<<<<<<
 *         word2 = task_dsp_info['current_data_type_mask']['mask_word_2']
 *         word3 = task_dsp_info['current_data_type_mask']['mask_word_3']
 */
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_v_task_dsp_info, __pyx_n_u_current_data_type_mask); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 89, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_mask_word_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 89, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_word1 = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":90
 *         word0 = task_dsp_info['current_data_type_mask']['mask_word_0']
 *         word1 = task_dsp_info['current_data_type_mask']['mask_word_1']
 *         word2 = task_dsp_info['current_data_type_mask']['mask_word_2']             # <<<<<<<<<<<<<<
 *         word3 = task_dsp_info['current_data_type_mask']['mask_word_3']
 *         return _data_types_from_mask(word0, word1, word2, word3)
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_task_dsp_info, __pyx_n_u_current_data_type_mask); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_mask_word_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_word2 = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pyart/io/_sigmetfile.pyx":91
 *         word1 = task_dsp_info['current_data_type_mask']['mask_word_1']
 *         word2 = task_dsp_info['current_data_type_mask']['mask_word_2']
 *         word3 = task_dsp_info['current_data_type_mask']['mask_word_3']             # <<<<<<<<<<<<<<
 *         return _data_types_from_mask(word0, word1, word2, word3)
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_v_task_dsp_info, __pyx_n_u_current_data_type_mask); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_mask_word_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_word3 = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":92
 *         word2 = task_dsp_info['current_data_type_mask']['mask_word_2']
 *         word3 = task_dsp_info['current_data_type_mask']['mask_word_3']
 *         return _data_types_from_mask(word0, word1, word2, word3)             # <<<<<<<<<<<<<<
 * 
 *     def close(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_data_types_from_mask); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[5] = {__pyx_t_3, __pyx_v_word0, __pyx_v_word1, __pyx_v_word2, __pyx_v_word3};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 4+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 92, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[5] = {__pyx_t_3, __pyx_v_word0, __pyx_v_word1, __pyx_v_word2, __pyx_v_word3};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 4+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 92, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(4+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 92, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_word0);
    __Pyx_GIVEREF(__pyx_v_word0);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_word0);
    __Pyx_INCREF(__pyx_v_word1);
    __Pyx_GIVEREF(__pyx_v_word1);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_word1);
    __Pyx_INCREF(__pyx_v_word2);
    __Pyx_GIVEREF(__pyx_v_word2);
    PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_4, __pyx_v_word2);
    __Pyx_INCREF(__pyx_v_word3);
    __Pyx_GIVEREF(__pyx_v_word3);
    PyTuple_SET_ITEM(__pyx_t_5, 3+__pyx_t_4, __pyx_v_word3);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 92, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_sigmetfile.pyx":83
 *         self._raw_product_bhdrs = []
 * 
 *     def _determine_data_types(self):             # <<<<<<<<<<<<<<
 *         """ Determine the available data types in the file. """
 *         # determine the available fields
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyart.io._sigmetfile.SigmetFile._determine_data_types", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_task_config);
  __Pyx_XDECREF(__pyx_v_task_dsp_info);
  __Pyx_XDECREF(__pyx_v_word0);
  __Pyx_XDECREF(__pyx_v_word1);
  __Pyx_XDECREF(__pyx_v_word2);
  __Pyx_XDECREF(__pyx_v_word3);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_sigmetfile.pyx":94
 *         return _data_types_from_mask(word0, word1, word2, word3)
 * 
 *     def close(self):             # <<<<<<<<<<<<<<
 *         """ Close the file. """
 *         self._fh.close()
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_5close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5pyart_2io_11_sigmetfile_10SigmetFile_4close[] = " Close the file. ";
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_5close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("close (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_4close(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_4close(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("close", 0);

  /* "pyart/io/_sigmetfile.pyx":96
 *     def close(self):
 *         """ Close the file. """
 *         self._fh.close()             # <<<<<<<<<<<<<<
 * 
 *     def read_data(self, full_xhdr=False):
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->_fh, __pyx_n_s_close); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 96, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 96, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":94
 *         return _data_types_from_mask(word0, word1, word2, word3)
 * 
 *     def close(self):             # <<<<<<<<<<<<<<
 *         """ Close the file. """
 *         self._fh.close()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pyart.io._sigmetfile.SigmetFile.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_sigmetfile.pyx":98
 *         self._fh.close()
 * 
 *     def read_data(self, full_xhdr=False):             # <<<<<<<<<<<<<<
 *         """
 *         Read all data from the file.
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_7read_data(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5pyart_2io_11_sigmetfile_10SigmetFile_6read_data[] = "\n        Read all data from the file.\n\n        Parameters\n        ----------\n        full_xhdr : bool\n            True to return the full extended headers if they exist padded with\n            ones.  False will return a length 1 extended header converted to\n            int32.  This is useful when the file contains a customer specified\n            extended header (for example aircraft radar).\n\n        Returns\n        -------\n        data : dict of ndarrays\n            Data arrays of shape=(nsweeps, nrays, nbins) for each data type.\n            Indexed by data type name (str).\n        metadata : dict of dicts\n            Arrays of 'azimuth_0', 'azimuth_1', 'elevation_0', 'elevation_1',\n            'nbins', and 'time' for each data type.  Indexed by data type name\n            (str).  Rays which were not collected are marked with a value of\n            -1 in the 'nbins' array.\n\n        ";
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_7read_data(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_full_xhdr = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("read_data (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_full_xhdr,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_full_xhdr);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "read_data") < 0)) __PYX_ERR(0, 98, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_full_xhdr = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("read_data", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 98, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._sigmetfile.SigmetFile.read_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_6read_data(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self), __pyx_v_full_xhdr);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_6read_data(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_full_xhdr) {
  PyObject *__pyx_v_nsweeps = NULL;
  PyObject *__pyx_v_nbins = NULL;
  PyObject *__pyx_v_nrays = NULL;
  PyObject *__pyx_v_shape = NULL;
  PyObject *__pyx_v_data = NULL;
  PyObject *__pyx_v_metadata = NULL;
  PyObject *__pyx_v_name = NULL;
  PyObject *__pyx_v_header_dic = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_v_ingest_data_hdrs = NULL;
  PyObject *__pyx_v_sweep_data = NULL;
  PyObject *__pyx_v_sweep_metadata = NULL;
  PyObject *__pyx_v_mess = NULL;
  PyObject *__pyx_v_k = NULL;
  PyObject *__pyx_v_j = NULL;
  PyObject *__pyx_v_temp = NULL;
  PyObject *__pyx_v_az0 = NULL;
  PyObject *__pyx_v_el0 = NULL;
  PyObject *__pyx_v_az1 = NULL;
  PyObject *__pyx_v_el1 = NULL;
  PyObject *__pyx_v_ray_nbins = NULL;
  PyObject *__pyx_v_ray_time = NULL;
  PyObject *__pyx_v_prf_flag = NULL;
  PyObject *__pyx_v_wavelength_cm = NULL;
  PyObject *__pyx_v_prt_value = NULL;
  PyObject *__pyx_v_task_config = NULL;
  PyObject *__pyx_v_multi_prf_flag = NULL;
  PyObject *__pyx_v_multiplier = NULL;
  PyObject *__pyx_v_nyquist = NULL;
  PyObject *__pyx_8genexpr1__pyx_v_name = NULL;
  PyObject *__pyx_8genexpr2__pyx_v_name = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_t_10;
  PyObject *(*__pyx_t_11)(PyObject *);
  Py_ssize_t __pyx_t_12;
  PyObject *(*__pyx_t_13)(PyObject *);
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  int __pyx_t_21;
  PyObject *__pyx_t_22 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("read_data", 0);

  /* "pyart/io/_sigmetfile.pyx":124
 * 
 *         # determine size of data
 *         nsweeps = self.ingest_header['task_configuration'][             # <<<<<<<<<<<<<<
 *             'task_scan_info']['number_sweeps']
 *         nbins = self.product_hdr['product_end']['number_bins']
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_self->ingest_header, __pyx_n_u_task_configuration); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_task_scan_info); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":125
 *         # determine size of data
 *         nsweeps = self.ingest_header['task_configuration'][
 *             'task_scan_info']['number_sweeps']             # <<<<<<<<<<<<<<
 *         nbins = self.product_hdr['product_end']['number_bins']
 *         nrays = self.ingest_header['ingest_configuration'][
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_number_sweeps); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_nsweeps = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":126
 *         nsweeps = self.ingest_header['task_configuration'][
 *             'task_scan_info']['number_sweeps']
 *         nbins = self.product_hdr['product_end']['number_bins']             # <<<<<<<<<<<<<<
 *         nrays = self.ingest_header['ingest_configuration'][
 *             'number_rays_sweep']
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_self->product_hdr, __pyx_n_u_product_end); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_number_bins); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_nbins = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pyart/io/_sigmetfile.pyx":127
 *             'task_scan_info']['number_sweeps']
 *         nbins = self.product_hdr['product_end']['number_bins']
 *         nrays = self.ingest_header['ingest_configuration'][             # <<<<<<<<<<<<<<
 *             'number_rays_sweep']
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_v_self->ingest_header, __pyx_n_u_ingest_configuration); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_number_rays_sweep); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_nrays = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":131
 * 
 *         # create empty outputs
 *         shape = (nsweeps, nrays, nbins)             # <<<<<<<<<<<<<<
 *         data = dict([(name, np.ma.empty(shape, dtype='float32'))
 *                     for name in self.data_type_names])
 */
  __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 131, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_nsweeps);
  __Pyx_GIVEREF(__pyx_v_nsweeps);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_nsweeps);
  __Pyx_INCREF(__pyx_v_nrays);
  __Pyx_GIVEREF(__pyx_v_nrays);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_nrays);
  __Pyx_INCREF(__pyx_v_nbins);
  __Pyx_GIVEREF(__pyx_v_nbins);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_nbins);
  __pyx_v_shape = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":132
 *         # create empty outputs
 *         shape = (nsweeps, nrays, nbins)
 *         data = dict([(name, np.ma.empty(shape, dtype='float32'))             # <<<<<<<<<<<<<<
 *                     for name in self.data_type_names])
 *         if 'XHDR' in self.data_type_names:
 */
  { /* enter inner scope */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 132, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_1);

    /* "pyart/io/_sigmetfile.pyx":133
 *         shape = (nsweeps, nrays, nbins)
 *         data = dict([(name, np.ma.empty(shape, dtype='float32'))
 *                     for name in self.data_type_names])             # <<<<<<<<<<<<<<
 *         if 'XHDR' in self.data_type_names:
 *             if full_xhdr:
 */
    if (likely(PyList_CheckExact(__pyx_v_self->data_type_names)) || PyTuple_CheckExact(__pyx_v_self->data_type_names)) {
      __pyx_t_2 = __pyx_v_self->data_type_names; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
      __pyx_t_4 = NULL;
    } else {
      __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_self->data_type_names); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 133, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 133, __pyx_L5_error)
    }
    for (;;) {
      if (likely(!__pyx_t_4)) {
        if (likely(PyList_CheckExact(__pyx_t_2))) {
          if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 133, __pyx_L5_error)
          #else
          __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 133, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        } else {
          if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 133, __pyx_L5_error)
          #else
          __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 133, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        }
      } else {
        __pyx_t_5 = __pyx_t_4(__pyx_t_2);
        if (unlikely(!__pyx_t_5)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 133, __pyx_L5_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_5);
      }
      __Pyx_XDECREF_SET(__pyx_8genexpr1__pyx_v_name, __pyx_t_5);
      __pyx_t_5 = 0;

      /* "pyart/io/_sigmetfile.pyx":132
 *         # create empty outputs
 *         shape = (nsweeps, nrays, nbins)
 *         data = dict([(name, np.ma.empty(shape, dtype='float32'))             # <<<<<<<<<<<<<<
 *                     for name in self.data_type_names])
 *         if 'XHDR' in self.data_type_names:
 */
      __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 132, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_ma); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 132, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_empty); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 132, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 132, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_INCREF(__pyx_v_shape);
      __Pyx_GIVEREF(__pyx_v_shape);
      PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_shape);
      __pyx_t_7 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 132, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_7);
      if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_dtype, __pyx_n_u_float32) < 0) __PYX_ERR(0, 132, __pyx_L5_error)
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_6, __pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 132, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 132, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_INCREF(__pyx_8genexpr1__pyx_v_name);
      __Pyx_GIVEREF(__pyx_8genexpr1__pyx_v_name);
      PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_8genexpr1__pyx_v_name);
      __Pyx_GIVEREF(__pyx_t_8);
      PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_8);
      __pyx_t_8 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_7))) __PYX_ERR(0, 132, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "pyart/io/_sigmetfile.pyx":133
 *         shape = (nsweeps, nrays, nbins)
 *         data = dict([(name, np.ma.empty(shape, dtype='float32'))
 *                     for name in self.data_type_names])             # <<<<<<<<<<<<<<
 *         if 'XHDR' in self.data_type_names:
 *             if full_xhdr:
 */
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_name); __pyx_8genexpr1__pyx_v_name = 0;
    goto __pyx_L8_exit_scope;
    __pyx_L5_error:;
    __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_name); __pyx_8genexpr1__pyx_v_name = 0;
    goto __pyx_L1_error;
    __pyx_L8_exit_scope:;
  } /* exit inner scope */

  /* "pyart/io/_sigmetfile.pyx":132
 *         # create empty outputs
 *         shape = (nsweeps, nrays, nbins)
 *         data = dict([(name, np.ma.empty(shape, dtype='float32'))             # <<<<<<<<<<<<<<
 *                     for name in self.data_type_names])
 *         if 'XHDR' in self.data_type_names:
 */
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyDict_Type)), __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_data = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyart/io/_sigmetfile.pyx":134
 *         data = dict([(name, np.ma.empty(shape, dtype='float32'))
 *                     for name in self.data_type_names])
 *         if 'XHDR' in self.data_type_names:             # <<<<<<<<<<<<<<
 *             if full_xhdr:
 *                 data['XHDR'] = np.ones(shape, dtype='int16')
 */
  __pyx_t_9 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_XHDR, __pyx_v_self->data_type_names, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 134, __pyx_L1_error)
  __pyx_t_10 = (__pyx_t_9 != 0);
  if (__pyx_t_10) {

    /* "pyart/io/_sigmetfile.pyx":135
 *                     for name in self.data_type_names])
 *         if 'XHDR' in self.data_type_names:
 *             if full_xhdr:             # <<<<<<<<<<<<<<
 *                 data['XHDR'] = np.ones(shape, dtype='int16')
 *             else:
 */
    __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_v_full_xhdr); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 135, __pyx_L1_error)
    if (__pyx_t_10) {

      /* "pyart/io/_sigmetfile.pyx":136
 *         if 'XHDR' in self.data_type_names:
 *             if full_xhdr:
 *                 data['XHDR'] = np.ones(shape, dtype='int16')             # <<<<<<<<<<<<<<
 *             else:
 *                 data['XHDR'] = np.ones((nsweeps, nrays, 1), dtype='int32')
 */
      __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 136, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_ones); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 136, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 136, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_INCREF(__pyx_v_shape);
      __Pyx_GIVEREF(__pyx_v_shape);
      PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_shape);
      __pyx_t_7 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 136, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_dtype, __pyx_n_u_int16) < 0) __PYX_ERR(0, 136, __pyx_L1_error)
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, __pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 136, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(PyDict_SetItem(__pyx_v_data, __pyx_n_u_XHDR, __pyx_t_8) < 0)) __PYX_ERR(0, 136, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "pyart/io/_sigmetfile.pyx":135
 *                     for name in self.data_type_names])
 *         if 'XHDR' in self.data_type_names:
 *             if full_xhdr:             # <<<<<<<<<<<<<<
 *                 data['XHDR'] = np.ones(shape, dtype='int16')
 *             else:
 */
      goto __pyx_L10;
    }

    /* "pyart/io/_sigmetfile.pyx":138
 *                 data['XHDR'] = np.ones(shape, dtype='int16')
 *             else:
 *                 data['XHDR'] = np.ones((nsweeps, nrays, 1), dtype='int32')             # <<<<<<<<<<<<<<
 * 
 *         metadata = {}
 */
    /*else*/ {
      __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_n_s_np); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 138, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_ones); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 138, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 138, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_INCREF(__pyx_v_nsweeps);
      __Pyx_GIVEREF(__pyx_v_nsweeps);
      PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_v_nsweeps);
      __Pyx_INCREF(__pyx_v_nrays);
      __Pyx_GIVEREF(__pyx_v_nrays);
      PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_v_nrays);
      __Pyx_INCREF(__pyx_int_1);
      __Pyx_GIVEREF(__pyx_int_1);
      PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_int_1);
      __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 138, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GIVEREF(__pyx_t_8);
      PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_8);
      __pyx_t_8 = 0;
      __pyx_t_8 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 138, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_dtype, __pyx_n_u_int32) < 0) __PYX_ERR(0, 138, __pyx_L1_error)
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_2, __pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 138, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(PyDict_SetItem(__pyx_v_data, __pyx_n_u_XHDR, __pyx_t_1) < 0)) __PYX_ERR(0, 138, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __pyx_L10:;

    /* "pyart/io/_sigmetfile.pyx":134
 *         data = dict([(name, np.ma.empty(shape, dtype='float32'))
 *                     for name in self.data_type_names])
 *         if 'XHDR' in self.data_type_names:             # <<<<<<<<<<<<<<
 *             if full_xhdr:
 *                 data['XHDR'] = np.ones(shape, dtype='int16')
 */
  }

  /* "pyart/io/_sigmetfile.pyx":140
 *                 data['XHDR'] = np.ones((nsweeps, nrays, 1), dtype='int32')
 * 
 *         metadata = {}             # <<<<<<<<<<<<<<
 *         for name in self.data_type_names:
 *             header_dic = {
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 140, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_metadata = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":141
 * 
 *         metadata = {}
 *         for name in self.data_type_names:             # <<<<<<<<<<<<<<
 *             header_dic = {
 *                 'azimuth_0': np.empty((nsweeps, nrays), dtype='float32'),
 */
  if (likely(PyList_CheckExact(__pyx_v_self->data_type_names)) || PyTuple_CheckExact(__pyx_v_self->data_type_names)) {
    __pyx_t_1 = __pyx_v_self->data_type_names; __Pyx_INCREF(__pyx_t_1); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_self->data_type_names); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 141, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 141, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_8 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_8); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 141, __pyx_L1_error)
        #else
        __pyx_t_8 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 141, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_8 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_8); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 141, __pyx_L1_error)
        #else
        __pyx_t_8 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 141, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        #endif
      }
    } else {
      __pyx_t_8 = __pyx_t_4(__pyx_t_1);
      if (unlikely(!__pyx_t_8)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 141, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_8);
    }
    __Pyx_XDECREF_SET(__pyx_v_name, __pyx_t_8);
    __pyx_t_8 = 0;

    /* "pyart/io/_sigmetfile.pyx":143
 *         for name in self.data_type_names:
 *             header_dic = {
 *                 'azimuth_0': np.empty((nsweeps, nrays), dtype='float32'),             # <<<<<<<<<<<<<<
 *                 'elevation_0': np.empty((nsweeps, nrays), dtype='float32'),
 *                 'azimuth_1': np.empty((nsweeps, nrays), dtype='float32'),
 */
    __pyx_t_8 = __Pyx_PyDict_NewPresized(7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 143, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 143, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 143, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 143, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_nsweeps);
    __Pyx_GIVEREF(__pyx_v_nsweeps);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_nsweeps);
    __Pyx_INCREF(__pyx_v_nrays);
    __Pyx_GIVEREF(__pyx_v_nrays);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_nrays);
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 143, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 143, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_n_u_float32) < 0) __PYX_ERR(0, 143, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_6, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 143, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (PyDict_SetItem(__pyx_t_8, __pyx_n_u_azimuth_0, __pyx_t_5) < 0) __PYX_ERR(0, 143, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "pyart/io/_sigmetfile.pyx":144
 *             header_dic = {
 *                 'azimuth_0': np.empty((nsweeps, nrays), dtype='float32'),
 *                 'elevation_0': np.empty((nsweeps, nrays), dtype='float32'),             # <<<<<<<<<<<<<<
 *                 'azimuth_1': np.empty((nsweeps, nrays), dtype='float32'),
 *                 'elevation_1': np.empty((nsweeps, nrays), dtype='float32'),
 */
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 144, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_empty); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 144, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 144, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_v_nsweeps);
    __Pyx_GIVEREF(__pyx_v_nsweeps);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_nsweeps);
    __Pyx_INCREF(__pyx_v_nrays);
    __Pyx_GIVEREF(__pyx_v_nrays);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_v_nrays);
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 144, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 144, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_dtype, __pyx_n_u_float32) < 0) __PYX_ERR(0, 144, __pyx_L1_error)
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, __pyx_t_5); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 144, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (PyDict_SetItem(__pyx_t_8, __pyx_n_u_elevation_0, __pyx_t_7) < 0) __PYX_ERR(0, 143, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "pyart/io/_sigmetfile.pyx":145
 *                 'azimuth_0': np.empty((nsweeps, nrays), dtype='float32'),
 *                 'elevation_0': np.empty((nsweeps, nrays), dtype='float32'),
 *                 'azimuth_1': np.empty((nsweeps, nrays), dtype='float32'),             # <<<<<<<<<<<<<<
 *                 'elevation_1': np.empty((nsweeps, nrays), dtype='float32'),
 *                 'nbins': np.empty((nsweeps, nrays), dtype='int16'),
 */
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_np); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 145, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_empty); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 145, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 145, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_INCREF(__pyx_v_nsweeps);
    __Pyx_GIVEREF(__pyx_v_nsweeps);
    PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_v_nsweeps);
    __Pyx_INCREF(__pyx_v_nrays);
    __Pyx_GIVEREF(__pyx_v_nrays);
    PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_v_nrays);
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 145, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 145, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_dtype, __pyx_n_u_float32) < 0) __PYX_ERR(0, 145, __pyx_L1_error)
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_6, __pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 145, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (PyDict_SetItem(__pyx_t_8, __pyx_n_u_azimuth_1, __pyx_t_2) < 0) __PYX_ERR(0, 143, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyart/io/_sigmetfile.pyx":146
 *                 'elevation_0': np.empty((nsweeps, nrays), dtype='float32'),
 *                 'azimuth_1': np.empty((nsweeps, nrays), dtype='float32'),
 *                 'elevation_1': np.empty((nsweeps, nrays), dtype='float32'),             # <<<<<<<<<<<<<<
 *                 'nbins': np.empty((nsweeps, nrays), dtype='int16'),
 *                 'time': np.empty((nsweeps, nrays), dtype='uint16'),
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 146, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 146, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 146, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_nsweeps);
    __Pyx_GIVEREF(__pyx_v_nsweeps);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_nsweeps);
    __Pyx_INCREF(__pyx_v_nrays);
    __Pyx_GIVEREF(__pyx_v_nrays);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_nrays);
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 146, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 146, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_n_u_float32) < 0) __PYX_ERR(0, 146, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_6, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 146, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (PyDict_SetItem(__pyx_t_8, __pyx_n_u_elevation_1, __pyx_t_5) < 0) __PYX_ERR(0, 143, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "pyart/io/_sigmetfile.pyx":147
 *                 'azimuth_1': np.empty((nsweeps, nrays), dtype='float32'),
 *                 'elevation_1': np.empty((nsweeps, nrays), dtype='float32'),
 *                 'nbins': np.empty((nsweeps, nrays), dtype='int16'),             # <<<<<<<<<<<<<<
 *                 'time': np.empty((nsweeps, nrays), dtype='uint16'),
 *                 'prf_flag': np.empty((nsweeps, nrays), dtype='int16')}
 */
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 147, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_empty); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 147, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 147, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_v_nsweeps);
    __Pyx_GIVEREF(__pyx_v_nsweeps);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_nsweeps);
    __Pyx_INCREF(__pyx_v_nrays);
    __Pyx_GIVEREF(__pyx_v_nrays);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_v_nrays);
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 147, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 147, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_dtype, __pyx_n_u_int16) < 0) __PYX_ERR(0, 147, __pyx_L1_error)
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, __pyx_t_5); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 147, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (PyDict_SetItem(__pyx_t_8, __pyx_n_u_nbins, __pyx_t_7) < 0) __PYX_ERR(0, 143, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "pyart/io/_sigmetfile.pyx":148
 *                 'elevation_1': np.empty((nsweeps, nrays), dtype='float32'),
 *                 'nbins': np.empty((nsweeps, nrays), dtype='int16'),
 *                 'time': np.empty((nsweeps, nrays), dtype='uint16'),             # <<<<<<<<<<<<<<
 *                 'prf_flag': np.empty((nsweeps, nrays), dtype='int16')}
 *             metadata[name] = header_dic
 */
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_np); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 148, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_empty); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 148, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 148, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_INCREF(__pyx_v_nsweeps);
    __Pyx_GIVEREF(__pyx_v_nsweeps);
    PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_v_nsweeps);
    __Pyx_INCREF(__pyx_v_nrays);
    __Pyx_GIVEREF(__pyx_v_nrays);
    PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_v_nrays);
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 148, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 148, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_dtype, __pyx_n_u_uint16) < 0) __PYX_ERR(0, 148, __pyx_L1_error)
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_6, __pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 148, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (PyDict_SetItem(__pyx_t_8, __pyx_n_u_time, __pyx_t_2) < 0) __PYX_ERR(0, 143, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyart/io/_sigmetfile.pyx":149
 *                 'nbins': np.empty((nsweeps, nrays), dtype='int16'),
 *                 'time': np.empty((nsweeps, nrays), dtype='uint16'),
 *                 'prf_flag': np.empty((nsweeps, nrays), dtype='int16')}             # <<<<<<<<<<<<<<
 *             metadata[name] = header_dic
 * 
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 149, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 149, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 149, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_nsweeps);
    __Pyx_GIVEREF(__pyx_v_nsweeps);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_nsweeps);
    __Pyx_INCREF(__pyx_v_nrays);
    __Pyx_GIVEREF(__pyx_v_nrays);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_nrays);
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 149, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 149, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_n_u_int16) < 0) __PYX_ERR(0, 149, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_6, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 149, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (PyDict_SetItem(__pyx_t_8, __pyx_n_u_prf_flag, __pyx_t_5) < 0) __PYX_ERR(0, 143, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF_SET(__pyx_v_header_dic, ((PyObject*)__pyx_t_8));
    __pyx_t_8 = 0;

    /* "pyart/io/_sigmetfile.pyx":150
 *                 'time': np.empty((nsweeps, nrays), dtype='uint16'),
 *                 'prf_flag': np.empty((nsweeps, nrays), dtype='int16')}
 *             metadata[name] = header_dic             # <<<<<<<<<<<<<<
 * 
 *         self.ingest_data_headers = dict([(name, []) for name in
 */
    if (unlikely(PyDict_SetItem(__pyx_v_metadata, __pyx_v_name, __pyx_v_header_dic) < 0)) __PYX_ERR(0, 150, __pyx_L1_error)

    /* "pyart/io/_sigmetfile.pyx":141
 * 
 *         metadata = {}
 *         for name in self.data_type_names:             # <<<<<<<<<<<<<<
 *             header_dic = {
 *                 'azimuth_0': np.empty((nsweeps, nrays), dtype='float32'),
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":152
 *             metadata[name] = header_dic
 * 
 *         self.ingest_data_headers = dict([(name, []) for name in             # <<<<<<<<<<<<<<
 *                                          self.data_type_names])
 * 
 */
  { /* enter inner scope */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 152, __pyx_L15_error)
    __Pyx_GOTREF(__pyx_t_1);

    /* "pyart/io/_sigmetfile.pyx":153
 * 
 *         self.ingest_data_headers = dict([(name, []) for name in
 *                                          self.data_type_names])             # <<<<<<<<<<<<<<
 * 
 *         self._raw_product_bhdrs = []
 */
    if (likely(PyList_CheckExact(__pyx_v_self->data_type_names)) || PyTuple_CheckExact(__pyx_v_self->data_type_names)) {
      __pyx_t_8 = __pyx_v_self->data_type_names; __Pyx_INCREF(__pyx_t_8); __pyx_t_3 = 0;
      __pyx_t_4 = NULL;
    } else {
      __pyx_t_3 = -1; __pyx_t_8 = PyObject_GetIter(__pyx_v_self->data_type_names); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 153, __pyx_L15_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_4 = Py_TYPE(__pyx_t_8)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 153, __pyx_L15_error)
    }
    for (;;) {
      if (likely(!__pyx_t_4)) {
        if (likely(PyList_CheckExact(__pyx_t_8))) {
          if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_8)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyList_GET_ITEM(__pyx_t_8, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 153, __pyx_L15_error)
          #else
          __pyx_t_5 = PySequence_ITEM(__pyx_t_8, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 153, __pyx_L15_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        } else {
          if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_8)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_8, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 153, __pyx_L15_error)
          #else
          __pyx_t_5 = PySequence_ITEM(__pyx_t_8, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 153, __pyx_L15_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        }
      } else {
        __pyx_t_5 = __pyx_t_4(__pyx_t_8);
        if (unlikely(!__pyx_t_5)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 153, __pyx_L15_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_5);
      }
      __Pyx_XDECREF_SET(__pyx_8genexpr2__pyx_v_name, __pyx_t_5);
      __pyx_t_5 = 0;

      /* "pyart/io/_sigmetfile.pyx":152
 *             metadata[name] = header_dic
 * 
 *         self.ingest_data_headers = dict([(name, []) for name in             # <<<<<<<<<<<<<<
 *                                          self.data_type_names])
 * 
 */
      __pyx_t_5 = PyList_New(0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 152, __pyx_L15_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 152, __pyx_L15_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_INCREF(__pyx_8genexpr2__pyx_v_name);
      __Pyx_GIVEREF(__pyx_8genexpr2__pyx_v_name);
      PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_8genexpr2__pyx_v_name);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_5);
      __pyx_t_5 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_2))) __PYX_ERR(0, 152, __pyx_L15_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_8genexpr2__pyx_v_name); __pyx_8genexpr2__pyx_v_name = 0;
    goto __pyx_L18_exit_scope;
    __pyx_L15_error:;
    __Pyx_XDECREF(__pyx_8genexpr2__pyx_v_name); __pyx_8genexpr2__pyx_v_name = 0;
    goto __pyx_L1_error;
    __pyx_L18_exit_scope:;
  } /* exit inner scope */
  __pyx_t_8 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyDict_Type)), __pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 152, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GIVEREF(__pyx_t_8);
  __Pyx_GOTREF(__pyx_v_self->ingest_data_headers);
  __Pyx_DECREF(__pyx_v_self->ingest_data_headers);
  __pyx_v_self->ingest_data_headers = __pyx_t_8;
  __pyx_t_8 = 0;

  /* "pyart/io/_sigmetfile.pyx":155
 *                                          self.data_type_names])
 * 
 *         self._raw_product_bhdrs = []             # <<<<<<<<<<<<<<
 * 
 *         # read in data sweep by sweep
 */
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 155, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_8);
  __Pyx_GOTREF(__pyx_v_self->_raw_product_bhdrs);
  __Pyx_DECREF(__pyx_v_self->_raw_product_bhdrs);
  __pyx_v_self->_raw_product_bhdrs = __pyx_t_8;
  __pyx_t_8 = 0;

  /* "pyart/io/_sigmetfile.pyx":158
 * 
 *         # read in data sweep by sweep
 *         for i in xrange(nsweeps):             # <<<<<<<<<<<<<<
 *             ingest_data_hdrs, sweep_data, sweep_metadata = self._get_sweep(
 *                 full_xhdr=full_xhdr)
 */
  __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_builtin_xrange, __pyx_v_nsweeps); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 158, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (likely(PyList_CheckExact(__pyx_t_8)) || PyTuple_CheckExact(__pyx_t_8)) {
    __pyx_t_1 = __pyx_t_8; __Pyx_INCREF(__pyx_t_1); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 158, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 158, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_8 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_8); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 158, __pyx_L1_error)
        #else
        __pyx_t_8 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 158, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_8 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_8); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 158, __pyx_L1_error)
        #else
        __pyx_t_8 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 158, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        #endif
      }
    } else {
      __pyx_t_8 = __pyx_t_4(__pyx_t_1);
      if (unlikely(!__pyx_t_8)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 158, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_8);
    }
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_8);
    __pyx_t_8 = 0;

    /* "pyart/io/_sigmetfile.pyx":159
 *         # read in data sweep by sweep
 *         for i in xrange(nsweeps):
 *             ingest_data_hdrs, sweep_data, sweep_metadata = self._get_sweep(             # <<<<<<<<<<<<<<
 *                 full_xhdr=full_xhdr)
 * 
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_sweep); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 159, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);

    /* "pyart/io/_sigmetfile.pyx":160
 *         for i in xrange(nsweeps):
 *             ingest_data_hdrs, sweep_data, sweep_metadata = self._get_sweep(
 *                 full_xhdr=full_xhdr)             # <<<<<<<<<<<<<<
 * 
 *             # check for a truncated file, return sweep(s) read up until error
 */
    __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 160, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_full_xhdr, __pyx_v_full_xhdr) < 0) __PYX_ERR(0, 160, __pyx_L1_error)

    /* "pyart/io/_sigmetfile.pyx":159
 *         # read in data sweep by sweep
 *         for i in xrange(nsweeps):
 *             ingest_data_hdrs, sweep_data, sweep_metadata = self._get_sweep(             # <<<<<<<<<<<<<<
 *                 full_xhdr=full_xhdr)
 * 
 */
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 159, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if ((likely(PyTuple_CheckExact(__pyx_t_5))) || (PyList_CheckExact(__pyx_t_5))) {
      PyObject* sequence = __pyx_t_5;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 3)) {
        if (size > 3) __Pyx_RaiseTooManyValuesError(3);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 159, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_8 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_6 = PyTuple_GET_ITEM(sequence, 2); 
      } else {
        __pyx_t_2 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_8 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_6 = PyList_GET_ITEM(sequence, 2); 
      }
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_6);
      #else
      __pyx_t_2 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 159, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_8 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 159, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_6 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 159, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      #endif
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_7 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 159, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_11 = Py_TYPE(__pyx_t_7)->tp_iternext;
      index = 0; __pyx_t_2 = __pyx_t_11(__pyx_t_7); if (unlikely(!__pyx_t_2)) goto __pyx_L21_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_2);
      index = 1; __pyx_t_8 = __pyx_t_11(__pyx_t_7); if (unlikely(!__pyx_t_8)) goto __pyx_L21_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_8);
      index = 2; __pyx_t_6 = __pyx_t_11(__pyx_t_7); if (unlikely(!__pyx_t_6)) goto __pyx_L21_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_11(__pyx_t_7), 3) < 0) __PYX_ERR(0, 159, __pyx_L1_error)
      __pyx_t_11 = NULL;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L22_unpacking_done;
      __pyx_L21_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_11 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 159, __pyx_L1_error)
      __pyx_L22_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_ingest_data_hdrs, __pyx_t_2);
    __pyx_t_2 = 0;
    __Pyx_XDECREF_SET(__pyx_v_sweep_data, __pyx_t_8);
    __pyx_t_8 = 0;
    __Pyx_XDECREF_SET(__pyx_v_sweep_metadata, __pyx_t_6);
    __pyx_t_6 = 0;

    /* "pyart/io/_sigmetfile.pyx":163
 * 
 *             # check for a truncated file, return sweep(s) read up until error
 *             if ingest_data_hdrs is None:             # <<<<<<<<<<<<<<
 * 
 *                 mess = ('File truncated or corrupt, %i of %i sweeps read' %
 */
    __pyx_t_10 = (__pyx_v_ingest_data_hdrs == Py_None);
    __pyx_t_9 = (__pyx_t_10 != 0);
    if (__pyx_t_9) {

      /* "pyart/io/_sigmetfile.pyx":166
 * 
 *                 mess = ('File truncated or corrupt, %i of %i sweeps read' %
 *                         (i, nsweeps))             # <<<<<<<<<<<<<<
 *                 warnings.warn(mess)
 * 
 */
      __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 166, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_v_i);
      __Pyx_GIVEREF(__pyx_v_i);
      PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_i);
      __Pyx_INCREF(__pyx_v_nsweeps);
      __Pyx_GIVEREF(__pyx_v_nsweeps);
      PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_v_nsweeps);

      /* "pyart/io/_sigmetfile.pyx":165
 *             if ingest_data_hdrs is None:
 * 
 *                 mess = ('File truncated or corrupt, %i of %i sweeps read' %             # <<<<<<<<<<<<<<
 *                         (i, nsweeps))
 *                 warnings.warn(mess)
 */
      __pyx_t_6 = PyUnicode_Format(__pyx_kp_u_File_truncated_or_corrupt_i_of_i, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 165, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_v_mess = ((PyObject*)__pyx_t_6);
      __pyx_t_6 = 0;

      /* "pyart/io/_sigmetfile.pyx":167
 *                 mess = ('File truncated or corrupt, %i of %i sweeps read' %
 *                         (i, nsweeps))
 *                 warnings.warn(mess)             # <<<<<<<<<<<<<<
 * 
 *                 for name in self.data_type_names:
 */
      __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_warnings); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 167, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_warn); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 167, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_8))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_8);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_8, function);
        }
      }
      __pyx_t_6 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_5, __pyx_v_mess) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_v_mess);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 167, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "pyart/io/_sigmetfile.pyx":169
 *                 warnings.warn(mess)
 * 
 *                 for name in self.data_type_names:             # <<<<<<<<<<<<<<
 *                     data[name] = data[name][:i]
 *                     for k in metadata[name]:
 */
      if (likely(PyList_CheckExact(__pyx_v_self->data_type_names)) || PyTuple_CheckExact(__pyx_v_self->data_type_names)) {
        __pyx_t_6 = __pyx_v_self->data_type_names; __Pyx_INCREF(__pyx_t_6); __pyx_t_12 = 0;
        __pyx_t_13 = NULL;
      } else {
        __pyx_t_12 = -1; __pyx_t_6 = PyObject_GetIter(__pyx_v_self->data_type_names); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 169, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_13 = Py_TYPE(__pyx_t_6)->tp_iternext; if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 169, __pyx_L1_error)
      }
      for (;;) {
        if (likely(!__pyx_t_13)) {
          if (likely(PyList_CheckExact(__pyx_t_6))) {
            if (__pyx_t_12 >= PyList_GET_SIZE(__pyx_t_6)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_8 = PyList_GET_ITEM(__pyx_t_6, __pyx_t_12); __Pyx_INCREF(__pyx_t_8); __pyx_t_12++; if (unlikely(0 < 0)) __PYX_ERR(0, 169, __pyx_L1_error)
            #else
            __pyx_t_8 = PySequence_ITEM(__pyx_t_6, __pyx_t_12); __pyx_t_12++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 169, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_8);
            #endif
          } else {
            if (__pyx_t_12 >= PyTuple_GET_SIZE(__pyx_t_6)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_8 = PyTuple_GET_ITEM(__pyx_t_6, __pyx_t_12); __Pyx_INCREF(__pyx_t_8); __pyx_t_12++; if (unlikely(0 < 0)) __PYX_ERR(0, 169, __pyx_L1_error)
            #else
            __pyx_t_8 = PySequence_ITEM(__pyx_t_6, __pyx_t_12); __pyx_t_12++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 169, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_8);
            #endif
          }
        } else {
          __pyx_t_8 = __pyx_t_13(__pyx_t_6);
          if (unlikely(!__pyx_t_8)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(0, 169, __pyx_L1_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_8);
        }
        __Pyx_XDECREF_SET(__pyx_v_name, __pyx_t_8);
        __pyx_t_8 = 0;

        /* "pyart/io/_sigmetfile.pyx":170
 * 
 *                 for name in self.data_type_names:
 *                     data[name] = data[name][:i]             # <<<<<<<<<<<<<<
 *                     for k in metadata[name]:
 *                         metadata[name][k] = metadata[name][k][:i]
 */
        __pyx_t_8 = __Pyx_PyDict_GetItem(__pyx_v_data, __pyx_v_name); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 170, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_5 = __Pyx_PyObject_GetSlice(__pyx_t_8, 0, 0, NULL, &__pyx_v_i, NULL, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 170, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(PyDict_SetItem(__pyx_v_data, __pyx_v_name, __pyx_t_5) < 0)) __PYX_ERR(0, 170, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

        /* "pyart/io/_sigmetfile.pyx":171
 *                 for name in self.data_type_names:
 *                     data[name] = data[name][:i]
 *                     for k in metadata[name]:             # <<<<<<<<<<<<<<
 *                         metadata[name][k] = metadata[name][k][:i]
 *                 return data, metadata
 */
        __pyx_t_5 = __Pyx_PyDict_GetItem(__pyx_v_metadata, __pyx_v_name); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 171, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        if (likely(PyList_CheckExact(__pyx_t_5)) || PyTuple_CheckExact(__pyx_t_5)) {
          __pyx_t_8 = __pyx_t_5; __Pyx_INCREF(__pyx_t_8); __pyx_t_14 = 0;
          __pyx_t_15 = NULL;
        } else {
          __pyx_t_14 = -1; __pyx_t_8 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 171, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_15 = Py_TYPE(__pyx_t_8)->tp_iternext; if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 171, __pyx_L1_error)
        }
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        for (;;) {
          if (likely(!__pyx_t_15)) {
            if (likely(PyList_CheckExact(__pyx_t_8))) {
              if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_8)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_5 = PyList_GET_ITEM(__pyx_t_8, __pyx_t_14); __Pyx_INCREF(__pyx_t_5); __pyx_t_14++; if (unlikely(0 < 0)) __PYX_ERR(0, 171, __pyx_L1_error)
              #else
              __pyx_t_5 = PySequence_ITEM(__pyx_t_8, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 171, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_5);
              #endif
            } else {
              if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_8)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_8, __pyx_t_14); __Pyx_INCREF(__pyx_t_5); __pyx_t_14++; if (unlikely(0 < 0)) __PYX_ERR(0, 171, __pyx_L1_error)
              #else
              __pyx_t_5 = PySequence_ITEM(__pyx_t_8, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 171, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_5);
              #endif
            }
          } else {
            __pyx_t_5 = __pyx_t_15(__pyx_t_8);
            if (unlikely(!__pyx_t_5)) {
              PyObject* exc_type = PyErr_Occurred();
              if (exc_type) {
                if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                else __PYX_ERR(0, 171, __pyx_L1_error)
              }
              break;
            }
            __Pyx_GOTREF(__pyx_t_5);
          }
          __Pyx_XDECREF_SET(__pyx_v_k, __pyx_t_5);
          __pyx_t_5 = 0;

          /* "pyart/io/_sigmetfile.pyx":172
 *                     data[name] = data[name][:i]
 *                     for k in metadata[name]:
 *                         metadata[name][k] = metadata[name][k][:i]             # <<<<<<<<<<<<<<
 *                 return data, metadata
 * 
 */
          __pyx_t_5 = __Pyx_PyDict_GetItem(__pyx_v_metadata, __pyx_v_name); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 172, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_t_5, __pyx_v_k); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 172, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __pyx_t_5 = __Pyx_PyObject_GetSlice(__pyx_t_2, 0, 0, NULL, &__pyx_v_i, NULL, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 172, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __pyx_t_2 = __Pyx_PyDict_GetItem(__pyx_v_metadata, __pyx_v_name); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 172, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          if (unlikely(PyObject_SetItem(__pyx_t_2, __pyx_v_k, __pyx_t_5) < 0)) __PYX_ERR(0, 172, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

          /* "pyart/io/_sigmetfile.pyx":171
 *                 for name in self.data_type_names:
 *                     data[name] = data[name][:i]
 *                     for k in metadata[name]:             # <<<<<<<<<<<<<<
 *                         metadata[name][k] = metadata[name][k][:i]
 *                 return data, metadata
 */
        }
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

        /* "pyart/io/_sigmetfile.pyx":169
 *                 warnings.warn(mess)
 * 
 *                 for name in self.data_type_names:             # <<<<<<<<<<<<<<
 *                     data[name] = data[name][:i]
 *                     for k in metadata[name]:
 */
      }
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "pyart/io/_sigmetfile.pyx":173
 *                     for k in metadata[name]:
 *                         metadata[name][k] = metadata[name][k][:i]
 *                 return data, metadata             # <<<<<<<<<<<<<<
 * 
 *             for j, name in enumerate(self.data_type_names):
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 173, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_INCREF(__pyx_v_data);
      __Pyx_GIVEREF(__pyx_v_data);
      PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_data);
      __Pyx_INCREF(__pyx_v_metadata);
      __Pyx_GIVEREF(__pyx_v_metadata);
      PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_metadata);
      __pyx_r = __pyx_t_6;
      __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;

      /* "pyart/io/_sigmetfile.pyx":163
 * 
 *             # check for a truncated file, return sweep(s) read up until error
 *             if ingest_data_hdrs is None:             # <<<<<<<<<<<<<<
 * 
 *                 mess = ('File truncated or corrupt, %i of %i sweeps read' %
 */
    }

    /* "pyart/io/_sigmetfile.pyx":175
 *                 return data, metadata
 * 
 *             for j, name in enumerate(self.data_type_names):             # <<<<<<<<<<<<<<
 *                 temp = sweep_metadata[j]
 *                 (az0, el0, az1, el1, ray_nbins, ray_time, prf_flag) = temp
 */
    __Pyx_INCREF(__pyx_int_0);
    __pyx_t_6 = __pyx_int_0;
    if (likely(PyList_CheckExact(__pyx_v_self->data_type_names)) || PyTuple_CheckExact(__pyx_v_self->data_type_names)) {
      __pyx_t_8 = __pyx_v_self->data_type_names; __Pyx_INCREF(__pyx_t_8); __pyx_t_12 = 0;
      __pyx_t_13 = NULL;
    } else {
      __pyx_t_12 = -1; __pyx_t_8 = PyObject_GetIter(__pyx_v_self->data_type_names); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 175, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_13 = Py_TYPE(__pyx_t_8)->tp_iternext; if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 175, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_13)) {
        if (likely(PyList_CheckExact(__pyx_t_8))) {
          if (__pyx_t_12 >= PyList_GET_SIZE(__pyx_t_8)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyList_GET_ITEM(__pyx_t_8, __pyx_t_12); __Pyx_INCREF(__pyx_t_5); __pyx_t_12++; if (unlikely(0 < 0)) __PYX_ERR(0, 175, __pyx_L1_error)
          #else
          __pyx_t_5 = PySequence_ITEM(__pyx_t_8, __pyx_t_12); __pyx_t_12++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 175, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        } else {
          if (__pyx_t_12 >= PyTuple_GET_SIZE(__pyx_t_8)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_8, __pyx_t_12); __Pyx_INCREF(__pyx_t_5); __pyx_t_12++; if (unlikely(0 < 0)) __PYX_ERR(0, 175, __pyx_L1_error)
          #else
          __pyx_t_5 = PySequence_ITEM(__pyx_t_8, __pyx_t_12); __pyx_t_12++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 175, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        }
      } else {
        __pyx_t_5 = __pyx_t_13(__pyx_t_8);
        if (unlikely(!__pyx_t_5)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 175, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_5);
      }
      __Pyx_XDECREF_SET(__pyx_v_name, __pyx_t_5);
      __pyx_t_5 = 0;
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_XDECREF_SET(__pyx_v_j, __pyx_t_6);
      __pyx_t_5 = __Pyx_PyInt_AddObjC(__pyx_t_6, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 175, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_6);
      __pyx_t_6 = __pyx_t_5;
      __pyx_t_5 = 0;

      /* "pyart/io/_sigmetfile.pyx":176
 * 
 *             for j, name in enumerate(self.data_type_names):
 *                 temp = sweep_metadata[j]             # <<<<<<<<<<<<<<
 *                 (az0, el0, az1, el1, ray_nbins, ray_time, prf_flag) = temp
 * 
 */
      __pyx_t_5 = __Pyx_PyObject_GetItem(__pyx_v_sweep_metadata, __pyx_v_j); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 176, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_XDECREF_SET(__pyx_v_temp, __pyx_t_5);
      __pyx_t_5 = 0;

      /* "pyart/io/_sigmetfile.pyx":177
 *             for j, name in enumerate(self.data_type_names):
 *                 temp = sweep_metadata[j]
 *                 (az0, el0, az1, el1, ray_nbins, ray_time, prf_flag) = temp             # <<<<<<<<<<<<<<
 * 
 *                 data[name][i] = sweep_data[j]
 */
      if ((likely(PyTuple_CheckExact(__pyx_v_temp))) || (PyList_CheckExact(__pyx_v_temp))) {
        PyObject* sequence = __pyx_v_temp;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 7)) {
          if (size > 7) __Pyx_RaiseTooManyValuesError(7);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 177, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1); 
          __pyx_t_7 = PyTuple_GET_ITEM(sequence, 2); 
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 3); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 4); 
          __pyx_t_18 = PyTuple_GET_ITEM(sequence, 5); 
          __pyx_t_19 = PyTuple_GET_ITEM(sequence, 6); 
        } else {
          __pyx_t_5 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_2 = PyList_GET_ITEM(sequence, 1); 
          __pyx_t_7 = PyList_GET_ITEM(sequence, 2); 
          __pyx_t_16 = PyList_GET_ITEM(sequence, 3); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 4); 
          __pyx_t_18 = PyList_GET_ITEM(sequence, 5); 
          __pyx_t_19 = PyList_GET_ITEM(sequence, 6); 
        }
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        __Pyx_INCREF(__pyx_t_18);
        __Pyx_INCREF(__pyx_t_19);
        #else
        {
          Py_ssize_t i;
          PyObject** temps[7] = {&__pyx_t_5,&__pyx_t_2,&__pyx_t_7,&__pyx_t_16,&__pyx_t_17,&__pyx_t_18,&__pyx_t_19};
          for (i=0; i < 7; i++) {
            PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 177, __pyx_L1_error)
            __Pyx_GOTREF(item);
            *(temps[i]) = item;
          }
        }
        #endif
      } else {
        Py_ssize_t index = -1;
        PyObject** temps[7] = {&__pyx_t_5,&__pyx_t_2,&__pyx_t_7,&__pyx_t_16,&__pyx_t_17,&__pyx_t_18,&__pyx_t_19};
        __pyx_t_20 = PyObject_GetIter(__pyx_v_temp); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 177, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_20);
        __pyx_t_11 = Py_TYPE(__pyx_t_20)->tp_iternext;
        for (index=0; index < 7; index++) {
          PyObject* item = __pyx_t_11(__pyx_t_20); if (unlikely(!item)) goto __pyx_L30_unpacking_failed;
          __Pyx_GOTREF(item);
          *(temps[index]) = item;
        }
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_11(__pyx_t_20), 7) < 0) __PYX_ERR(0, 177, __pyx_L1_error)
        __pyx_t_11 = NULL;
        __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
        goto __pyx_L31_unpacking_done;
        __pyx_L30_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
        __pyx_t_11 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 177, __pyx_L1_error)
        __pyx_L31_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_az0, __pyx_t_5);
      __pyx_t_5 = 0;
      __Pyx_XDECREF_SET(__pyx_v_el0, __pyx_t_2);
      __pyx_t_2 = 0;
      __Pyx_XDECREF_SET(__pyx_v_az1, __pyx_t_7);
      __pyx_t_7 = 0;
      __Pyx_XDECREF_SET(__pyx_v_el1, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_ray_nbins, __pyx_t_17);
      __pyx_t_17 = 0;
      __Pyx_XDECREF_SET(__pyx_v_ray_time, __pyx_t_18);
      __pyx_t_18 = 0;
      __Pyx_XDECREF_SET(__pyx_v_prf_flag, __pyx_t_19);
      __pyx_t_19 = 0;

      /* "pyart/io/_sigmetfile.pyx":179
 *                 (az0, el0, az1, el1, ray_nbins, ray_time, prf_flag) = temp
 * 
 *                 data[name][i] = sweep_data[j]             # <<<<<<<<<<<<<<
 *                 metadata[name]['azimuth_0'][i] = az0
 *                 metadata[name]['azimuth_1'][i] = az1
 */
      __pyx_t_19 = __Pyx_PyObject_GetItem(__pyx_v_sweep_data, __pyx_v_j); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 179, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_19);
      __pyx_t_18 = __Pyx_PyDict_GetItem(__pyx_v_data, __pyx_v_name); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 179, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_18);
      if (unlikely(PyObject_SetItem(__pyx_t_18, __pyx_v_i, __pyx_t_19) < 0)) __PYX_ERR(0, 179, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
      __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;

      /* "pyart/io/_sigmetfile.pyx":180
 * 
 *                 data[name][i] = sweep_data[j]
 *                 metadata[name]['azimuth_0'][i] = az0             # <<<<<<<<<<<<<<
 *                 metadata[name]['azimuth_1'][i] = az1
 *                 metadata[name]['elevation_0'][i] = el0
 */
      __pyx_t_19 = __Pyx_PyDict_GetItem(__pyx_v_metadata, __pyx_v_name); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 180, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_19);
      __pyx_t_18 = __Pyx_PyObject_Dict_GetItem(__pyx_t_19, __pyx_n_u_azimuth_0); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 180, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_18);
      __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
      if (unlikely(PyObject_SetItem(__pyx_t_18, __pyx_v_i, __pyx_v_az0) < 0)) __PYX_ERR(0, 180, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;

      /* "pyart/io/_sigmetfile.pyx":181
 *                 data[name][i] = sweep_data[j]
 *                 metadata[name]['azimuth_0'][i] = az0
 *                 metadata[name]['azimuth_1'][i] = az1             # <<<<<<<<<<<<<<
 *                 metadata[name]['elevation_0'][i] = el0
 *                 metadata[name]['elevation_1'][i] = el1
 */
      __pyx_t_18 = __Pyx_PyDict_GetItem(__pyx_v_metadata, __pyx_v_name); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 181, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_18);
      __pyx_t_19 = __Pyx_PyObject_Dict_GetItem(__pyx_t_18, __pyx_n_u_azimuth_1); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 181, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_19);
      __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
      if (unlikely(PyObject_SetItem(__pyx_t_19, __pyx_v_i, __pyx_v_az1) < 0)) __PYX_ERR(0, 181, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;

      /* "pyart/io/_sigmetfile.pyx":182
 *                 metadata[name]['azimuth_0'][i] = az0
 *                 metadata[name]['azimuth_1'][i] = az1
 *                 metadata[name]['elevation_0'][i] = el0             # <<<<<<<<<<<<<<
 *                 metadata[name]['elevation_1'][i] = el1
 *                 metadata[name]['nbins'][i] = ray_nbins
 */
      __pyx_t_19 = __Pyx_PyDict_GetItem(__pyx_v_metadata, __pyx_v_name); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 182, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_19);
      __pyx_t_18 = __Pyx_PyObject_Dict_GetItem(__pyx_t_19, __pyx_n_u_elevation_0); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 182, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_18);
      __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
      if (unlikely(PyObject_SetItem(__pyx_t_18, __pyx_v_i, __pyx_v_el0) < 0)) __PYX_ERR(0, 182, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;

      /* "pyart/io/_sigmetfile.pyx":183
 *                 metadata[name]['azimuth_1'][i] = az1
 *                 metadata[name]['elevation_0'][i] = el0
 *                 metadata[name]['elevation_1'][i] = el1             # <<<<<<<<<<<<<<
 *                 metadata[name]['nbins'][i] = ray_nbins
 *                 metadata[name]['time'][i] = ray_time
 */
      __pyx_t_18 = __Pyx_PyDict_GetItem(__pyx_v_metadata, __pyx_v_name); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 183, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_18);
      __pyx_t_19 = __Pyx_PyObject_Dict_GetItem(__pyx_t_18, __pyx_n_u_elevation_1); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 183, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_19);
      __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
      if (unlikely(PyObject_SetItem(__pyx_t_19, __pyx_v_i, __pyx_v_el1) < 0)) __PYX_ERR(0, 183, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;

      /* "pyart/io/_sigmetfile.pyx":184
 *                 metadata[name]['elevation_0'][i] = el0
 *                 metadata[name]['elevation_1'][i] = el1
 *                 metadata[name]['nbins'][i] = ray_nbins             # <<<<<<<<<<<<<<
 *                 metadata[name]['time'][i] = ray_time
 *                 metadata[name]['prf_flag'][i] = prf_flag
 */
      __pyx_t_19 = __Pyx_PyDict_GetItem(__pyx_v_metadata, __pyx_v_name); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 184, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_19);
      __pyx_t_18 = __Pyx_PyObject_Dict_GetItem(__pyx_t_19, __pyx_n_u_nbins); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 184, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_18);
      __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
      if (unlikely(PyObject_SetItem(__pyx_t_18, __pyx_v_i, __pyx_v_ray_nbins) < 0)) __PYX_ERR(0, 184, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;

      /* "pyart/io/_sigmetfile.pyx":185
 *                 metadata[name]['elevation_1'][i] = el1
 *                 metadata[name]['nbins'][i] = ray_nbins
 *                 metadata[name]['time'][i] = ray_time             # <<<<<<<<<<<<<<
 *                 metadata[name]['prf_flag'][i] = prf_flag
 *                 self.ingest_data_headers[name].append(ingest_data_hdrs[j])
 */
      __pyx_t_18 = __Pyx_PyDict_GetItem(__pyx_v_metadata, __pyx_v_name); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 185, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_18);
      __pyx_t_19 = __Pyx_PyObject_Dict_GetItem(__pyx_t_18, __pyx_n_u_time); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 185, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_19);
      __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
      if (unlikely(PyObject_SetItem(__pyx_t_19, __pyx_v_i, __pyx_v_ray_time) < 0)) __PYX_ERR(0, 185, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;

      /* "pyart/io/_sigmetfile.pyx":186
 *                 metadata[name]['nbins'][i] = ray_nbins
 *                 metadata[name]['time'][i] = ray_time
 *                 metadata[name]['prf_flag'][i] = prf_flag             # <<<<<<<<<<<<<<
 *                 self.ingest_data_headers[name].append(ingest_data_hdrs[j])
 * 
 */
      __pyx_t_19 = __Pyx_PyDict_GetItem(__pyx_v_metadata, __pyx_v_name); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 186, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_19);
      __pyx_t_18 = __Pyx_PyObject_Dict_GetItem(__pyx_t_19, __pyx_n_u_prf_flag); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 186, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_18);
      __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
      if (unlikely(PyObject_SetItem(__pyx_t_18, __pyx_v_i, __pyx_v_prf_flag) < 0)) __PYX_ERR(0, 186, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;

      /* "pyart/io/_sigmetfile.pyx":187
 *                 metadata[name]['time'][i] = ray_time
 *                 metadata[name]['prf_flag'][i] = prf_flag
 *                 self.ingest_data_headers[name].append(ingest_data_hdrs[j])             # <<<<<<<<<<<<<<
 * 
 *         # scale 1-byte velocity by the Nyquist (section 4.3.29)
 */
      __pyx_t_18 = __Pyx_PyObject_GetItem(__pyx_v_self->ingest_data_headers, __pyx_v_name); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 187, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_18);
      __pyx_t_19 = __Pyx_PyObject_GetItem(__pyx_v_ingest_data_hdrs, __pyx_v_j); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 187, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_19);
      __pyx_t_21 = __Pyx_PyObject_Append(__pyx_t_18, __pyx_t_19); if (unlikely(__pyx_t_21 == ((int)-1))) __PYX_ERR(0, 187, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
      __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;

      /* "pyart/io/_sigmetfile.pyx":175
 *                 return data, metadata
 * 
 *             for j, name in enumerate(self.data_type_names):             # <<<<<<<<<<<<<<
 *                 temp = sweep_metadata[j]
 *                 (az0, el0, az1, el1, ray_nbins, ray_time, prf_flag) = temp
 */
    }
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "pyart/io/_sigmetfile.pyx":158
 * 
 *         # read in data sweep by sweep
 *         for i in xrange(nsweeps):             # <<<<<<<<<<<<<<
 *             ingest_data_hdrs, sweep_data, sweep_metadata = self._get_sweep(
 *                 full_xhdr=full_xhdr)
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":193
 *         # product_hdr does not need to be accessed at lower abstraction
 *         # layers.
 *         if 'VEL' in self.data_type_names:             # <<<<<<<<<<<<<<
 *             wavelength_cm = self.product_hdr['product_end']['wavelength']
 *             prt_value = 1. / self.product_hdr['product_end']['prf']
 */
  __pyx_t_9 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_VEL, __pyx_v_self->data_type_names, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 193, __pyx_L1_error)
  __pyx_t_10 = (__pyx_t_9 != 0);
  if (__pyx_t_10) {

    /* "pyart/io/_sigmetfile.pyx":194
 *         # layers.
 *         if 'VEL' in self.data_type_names:
 *             wavelength_cm = self.product_hdr['product_end']['wavelength']             # <<<<<<<<<<<<<<
 *             prt_value = 1. / self.product_hdr['product_end']['prf']
 *             task_config = self.ingest_header['task_configuration']
 */
    __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_self->product_hdr, __pyx_n_u_product_end); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 194, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_wavelength); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 194, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_wavelength_cm = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "pyart/io/_sigmetfile.pyx":195
 *         if 'VEL' in self.data_type_names:
 *             wavelength_cm = self.product_hdr['product_end']['wavelength']
 *             prt_value = 1. / self.product_hdr['product_end']['prf']             # <<<<<<<<<<<<<<
 *             task_config = self.ingest_header['task_configuration']
 *             multi_prf_flag = task_config['task_dsp_info']['multi_prf_flag']
 */
    __pyx_t_6 = __Pyx_PyObject_Dict_GetItem(__pyx_v_self->product_hdr, __pyx_n_u_product_end); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 195, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_6, __pyx_n_u_prf); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 195, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyFloat_TrueDivideCObj(__pyx_float_1_, __pyx_t_1, 1., 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 195, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_prt_value = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "pyart/io/_sigmetfile.pyx":196
 *             wavelength_cm = self.product_hdr['product_end']['wavelength']
 *             prt_value = 1. / self.product_hdr['product_end']['prf']
 *             task_config = self.ingest_header['task_configuration']             # <<<<<<<<<<<<<<
 *             multi_prf_flag = task_config['task_dsp_info']['multi_prf_flag']
 *             if multi_prf_flag > 3 or multi_prf_flag < 0:
 */
    __pyx_t_6 = __Pyx_PyObject_Dict_GetItem(__pyx_v_self->ingest_header, __pyx_n_u_task_configuration); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 196, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_v_task_config = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "pyart/io/_sigmetfile.pyx":197
 *             prt_value = 1. / self.product_hdr['product_end']['prf']
 *             task_config = self.ingest_header['task_configuration']
 *             multi_prf_flag = task_config['task_dsp_info']['multi_prf_flag']             # <<<<<<<<<<<<<<
 *             if multi_prf_flag > 3 or multi_prf_flag < 0:
 *                 multiplier = 1  # multiplier not defined in IRIS manual
 */
    __pyx_t_6 = __Pyx_PyObject_Dict_GetItem(__pyx_v_task_config, __pyx_n_u_task_dsp_info); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 197, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_6, __pyx_n_u_multi_prf_flag); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 197, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_multi_prf_flag = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "pyart/io/_sigmetfile.pyx":198
 *             task_config = self.ingest_header['task_configuration']
 *             multi_prf_flag = task_config['task_dsp_info']['multi_prf_flag']
 *             if multi_prf_flag > 3 or multi_prf_flag < 0:             # <<<<<<<<<<<<<<
 *                 multiplier = 1  # multiplier not defined in IRIS manual
 *             else:
 */
    __pyx_t_1 = PyObject_RichCompare(__pyx_v_multi_prf_flag, __pyx_int_3, Py_GT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 198, __pyx_L1_error)
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 198, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (!__pyx_t_9) {
    } else {
      __pyx_t_10 = __pyx_t_9;
      goto __pyx_L34_bool_binop_done;
    }
    __pyx_t_1 = PyObject_RichCompare(__pyx_v_multi_prf_flag, __pyx_int_0, Py_LT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 198, __pyx_L1_error)
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 198, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_10 = __pyx_t_9;
    __pyx_L34_bool_binop_done:;
    if (__pyx_t_10) {

      /* "pyart/io/_sigmetfile.pyx":199
 *             multi_prf_flag = task_config['task_dsp_info']['multi_prf_flag']
 *             if multi_prf_flag > 3 or multi_prf_flag < 0:
 *                 multiplier = 1  # multiplier not defined in IRIS manual             # <<<<<<<<<<<<<<
 *             else:
 *                 multiplier = [1, 2, 3, 4][multi_prf_flag]
 */
      __Pyx_INCREF(__pyx_int_1);
      __pyx_v_multiplier = __pyx_int_1;

      /* "pyart/io/_sigmetfile.pyx":198
 *             task_config = self.ingest_header['task_configuration']
 *             multi_prf_flag = task_config['task_dsp_info']['multi_prf_flag']
 *             if multi_prf_flag > 3 or multi_prf_flag < 0:             # <<<<<<<<<<<<<<
 *                 multiplier = 1  # multiplier not defined in IRIS manual
 *             else:
 */
      goto __pyx_L33;
    }

    /* "pyart/io/_sigmetfile.pyx":201
 *                 multiplier = 1  # multiplier not defined in IRIS manual
 *             else:
 *                 multiplier = [1, 2, 3, 4][multi_prf_flag]             # <<<<<<<<<<<<<<
 *             nyquist = wavelength_cm / (10000.0 * 4.0 * prt_value) * multiplier
 *             data['VEL'] *= nyquist
 */
    /*else*/ {
      __pyx_t_1 = PyList_New(4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 201, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_INCREF(__pyx_int_1);
      __Pyx_GIVEREF(__pyx_int_1);
      PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_1);
      __Pyx_INCREF(__pyx_int_2);
      __Pyx_GIVEREF(__pyx_int_2);
      PyList_SET_ITEM(__pyx_t_1, 1, __pyx_int_2);
      __Pyx_INCREF(__pyx_int_3);
      __Pyx_GIVEREF(__pyx_int_3);
      PyList_SET_ITEM(__pyx_t_1, 2, __pyx_int_3);
      __Pyx_INCREF(__pyx_int_4);
      __Pyx_GIVEREF(__pyx_int_4);
      PyList_SET_ITEM(__pyx_t_1, 3, __pyx_int_4);
      __pyx_t_6 = __Pyx_PyObject_GetItem(__pyx_t_1, __pyx_v_multi_prf_flag); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 201, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_v_multiplier = __pyx_t_6;
      __pyx_t_6 = 0;
    }
    __pyx_L33:;

    /* "pyart/io/_sigmetfile.pyx":202
 *             else:
 *                 multiplier = [1, 2, 3, 4][multi_prf_flag]
 *             nyquist = wavelength_cm / (10000.0 * 4.0 * prt_value) * multiplier             # <<<<<<<<<<<<<<
 *             data['VEL'] *= nyquist
 *         # scale 1-byte width by the Nyquist
 */
    __pyx_t_6 = PyFloat_FromDouble((10000.0 * 4.0)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 202, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_1 = PyNumber_Multiply(__pyx_t_6, __pyx_v_prt_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 202, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyNumber_Divide(__pyx_v_wavelength_cm, __pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 202, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyNumber_Multiply(__pyx_t_6, __pyx_v_multiplier); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 202, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_nyquist = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "pyart/io/_sigmetfile.pyx":203
 *                 multiplier = [1, 2, 3, 4][multi_prf_flag]
 *             nyquist = wavelength_cm / (10000.0 * 4.0 * prt_value) * multiplier
 *             data['VEL'] *= nyquist             # <<<<<<<<<<<<<<
 *         # scale 1-byte width by the Nyquist
 *         if 'WIDTH' in self.data_type_names:
 */
    __Pyx_INCREF(__pyx_n_u_VEL);
    __pyx_t_22 = __pyx_n_u_VEL;
    __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_data, __pyx_t_22); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 203, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = PyNumber_InPlaceMultiply(__pyx_t_1, __pyx_v_nyquist); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 203, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(PyDict_SetItem(__pyx_v_data, __pyx_t_22, __pyx_t_6) < 0)) __PYX_ERR(0, 203, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;

    /* "pyart/io/_sigmetfile.pyx":193
 *         # product_hdr does not need to be accessed at lower abstraction
 *         # layers.
 *         if 'VEL' in self.data_type_names:             # <<<<<<<<<<<<<<
 *             wavelength_cm = self.product_hdr['product_end']['wavelength']
 *             prt_value = 1. / self.product_hdr['product_end']['prf']
 */
  }

  /* "pyart/io/_sigmetfile.pyx":205
 *             data['VEL'] *= nyquist
 *         # scale 1-byte width by the Nyquist
 *         if 'WIDTH' in self.data_type_names:             # <<<<<<<<<<<<<<
 *             # The IRIS Programmer's Manual indicates 1-byte width format data
 *             # should be scaled by the unambiguous velocity, twice the nyquist,
 */
  __pyx_t_10 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_WIDTH, __pyx_v_self->data_type_names, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 205, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_10 != 0);
  if (__pyx_t_9) {

    /* "pyart/io/_sigmetfile.pyx":211
 *             # nyquist.  Therefore to agree with these two packages the width
 *             # is scaled by the nyquist.
 *             wavelength_cm = self.product_hdr['product_end']['wavelength']             # <<<<<<<<<<<<<<
 *             prt_value = 1. / self.product_hdr['product_end']['prf']
 *             nyquist = wavelength_cm / (10000.0 * 4.0 * prt_value)
 */
    __pyx_t_6 = __Pyx_PyObject_Dict_GetItem(__pyx_v_self->product_hdr, __pyx_n_u_product_end); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 211, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_6, __pyx_n_u_wavelength); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 211, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF_SET(__pyx_v_wavelength_cm, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "pyart/io/_sigmetfile.pyx":212
 *             # is scaled by the nyquist.
 *             wavelength_cm = self.product_hdr['product_end']['wavelength']
 *             prt_value = 1. / self.product_hdr['product_end']['prf']             # <<<<<<<<<<<<<<
 *             nyquist = wavelength_cm / (10000.0 * 4.0 * prt_value)
 *             data['WIDTH'] *= nyquist
 */
    __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_self->product_hdr, __pyx_n_u_product_end); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 212, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_prf); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 212, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyFloat_TrueDivideCObj(__pyx_float_1_, __pyx_t_6, 1., 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 212, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF_SET(__pyx_v_prt_value, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "pyart/io/_sigmetfile.pyx":213
 *             wavelength_cm = self.product_hdr['product_end']['wavelength']
 *             prt_value = 1. / self.product_hdr['product_end']['prf']
 *             nyquist = wavelength_cm / (10000.0 * 4.0 * prt_value)             # <<<<<<<<<<<<<<
 *             data['WIDTH'] *= nyquist
 * 
 */
    __pyx_t_1 = PyFloat_FromDouble((10000.0 * 4.0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 213, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = PyNumber_Multiply(__pyx_t_1, __pyx_v_prt_value); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 213, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyNumber_Divide(__pyx_v_wavelength_cm, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 213, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF_SET(__pyx_v_nyquist, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "pyart/io/_sigmetfile.pyx":214
 *             prt_value = 1. / self.product_hdr['product_end']['prf']
 *             nyquist = wavelength_cm / (10000.0 * 4.0 * prt_value)
 *             data['WIDTH'] *= nyquist             # <<<<<<<<<<<<<<
 * 
 *         return data, metadata
 */
    __Pyx_INCREF(__pyx_n_u_WIDTH);
    __pyx_t_22 = __pyx_n_u_WIDTH;
    __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_data, __pyx_t_22); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 214, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = PyNumber_InPlaceMultiply(__pyx_t_1, __pyx_v_nyquist); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 214, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(PyDict_SetItem(__pyx_v_data, __pyx_t_22, __pyx_t_6) < 0)) __PYX_ERR(0, 214, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;

    /* "pyart/io/_sigmetfile.pyx":205
 *             data['VEL'] *= nyquist
 *         # scale 1-byte width by the Nyquist
 *         if 'WIDTH' in self.data_type_names:             # <<<<<<<<<<<<<<
 *             # The IRIS Programmer's Manual indicates 1-byte width format data
 *             # should be scaled by the unambiguous velocity, twice the nyquist,
 */
  }

  /* "pyart/io/_sigmetfile.pyx":216
 *             data['WIDTH'] *= nyquist
 * 
 *         return data, metadata             # <<<<<<<<<<<<<<
 * 
 *     def _get_sweep(self, full_xhdr=False, raw_data=False):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 216, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_v_data);
  __Pyx_GIVEREF(__pyx_v_data);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_data);
  __Pyx_INCREF(__pyx_v_metadata);
  __Pyx_GIVEREF(__pyx_v_metadata);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_metadata);
  __pyx_r = __pyx_t_6;
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "pyart/io/_sigmetfile.pyx":98
 *         self._fh.close()
 * 
 *     def read_data(self, full_xhdr=False):             # <<<<<<<<<<<<<<
 *         """
 *         Read all data from the file.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_XDECREF(__pyx_t_19);
  __Pyx_XDECREF(__pyx_t_20);
  __Pyx_XDECREF(__pyx_t_22);
  __Pyx_AddTraceback("pyart.io._sigmetfile.SigmetFile.read_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_nsweeps);
  __Pyx_XDECREF(__pyx_v_nbins);
  __Pyx_XDECREF(__pyx_v_nrays);
  __Pyx_XDECREF(__pyx_v_shape);
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XDECREF(__pyx_v_metadata);
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XDECREF(__pyx_v_header_dic);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XDECREF(__pyx_v_ingest_data_hdrs);
  __Pyx_XDECREF(__pyx_v_sweep_data);
  __Pyx_XDECREF(__pyx_v_sweep_metadata);
  __Pyx_XDECREF(__pyx_v_mess);
  __Pyx_XDECREF(__pyx_v_k);
  __Pyx_XDECREF(__pyx_v_j);
  __Pyx_XDECREF(__pyx_v_temp);
  __Pyx_XDECREF(__pyx_v_az0);
  __Pyx_XDECREF(__pyx_v_el0);
  __Pyx_XDECREF(__pyx_v_az1);
  __Pyx_XDECREF(__pyx_v_el1);
  __Pyx_XDECREF(__pyx_v_ray_nbins);
  __Pyx_XDECREF(__pyx_v_ray_time);
  __Pyx_XDECREF(__pyx_v_prf_flag);
  __Pyx_XDECREF(__pyx_v_wavelength_cm);
  __Pyx_XDECREF(__pyx_v_prt_value);
  __Pyx_XDECREF(__pyx_v_task_config);
  __Pyx_XDECREF(__pyx_v_multi_prf_flag);
  __Pyx_XDECREF(__pyx_v_multiplier);
  __Pyx_XDECREF(__pyx_v_nyquist);
  __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_name);
  __Pyx_XDECREF(__pyx_8genexpr2__pyx_v_name);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_sigmetfile.pyx":218
 *         return data, metadata
 * 
 *     def _get_sweep(self, full_xhdr=False, raw_data=False):             # <<<<<<<<<<<<<<
 *         """
 *         Get the data and metadata from the next sweep.
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_9_get_sweep(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5pyart_2io_11_sigmetfile_10SigmetFile_8_get_sweep[] = "\n        Get the data and metadata from the next sweep.\n\n        If the file ends early None is returned for all values.\n\n        Parameters\n        ----------\n        full_xhdr : bool\n            True to return the full extended headers if they exist padded with\n            ones.  False will return a length 1 extended header converted to\n            int32.  This is useful when the file contains a customer specified\n            extended header (for example aircraft radar).\n        raw_data : bool, optional\n            True to return the raw_data for the given sweep, False to\n            convert the data to floating point representation.\n\n        Returns\n        -------\n        ingest_data_headers : list of dict\n            List of ingest_data_header structures for each data type.\n        sweep_data : list of arrays\n            Sweep data for each data types in the order they appear in the\n            file.\n        sweep_metadata : list of tuples\n            Sweep metadata for each data type in the same order as sweep_data.\n\n        ";
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_9_get_sweep(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_full_xhdr = 0;
  PyObject *__pyx_v_raw_data = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_sweep (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_full_xhdr,&__pyx_n_s_raw_data,0};
    PyObject* values[2] = {0,0};
    values[0] = ((PyObject *)Py_False);
    values[1] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_full_xhdr);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_raw_data);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_get_sweep") < 0)) __PYX_ERR(0, 218, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_full_xhdr = values[0];
    __pyx_v_raw_data = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_get_sweep", 0, 0, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 218, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._sigmetfile.SigmetFile._get_sweep", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_8_get_sweep(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self), __pyx_v_full_xhdr, __pyx_v_raw_data);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_8_get_sweep(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_full_xhdr, PyObject *__pyx_v_raw_data) {
  PyObject *__pyx_v_lead_record = NULL;
  PyObject *__pyx_v_raw_prod_bhdr = NULL;
  PyObject *__pyx_v_ingest_data_headers = NULL;
  PyObject *__pyx_v_nray_data_types = NULL;
  PyObject *__pyx_v_nrays = NULL;
  PyObject *__pyx_v_nbins = NULL;
  PyObject *__pyx_v_raw_sweep_data = NULL;
  PyObject *__pyx_v_ray_i = NULL;
  PyObject *__pyx_v_sweep_data = NULL;
  PyObject *__pyx_v_sweep_metadata = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_v_data_type = NULL;
  PyObject *__pyx_8genexpr3__pyx_v_d = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  PyObject *(*__pyx_t_10)(PyObject *);
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_sweep", 0);

  /* "pyart/io/_sigmetfile.pyx":248
 * 
 *         # get the next record
 *         lead_record = self._fh.read(RECORD_SIZE)             # <<<<<<<<<<<<<<
 *         self._record_number += 1
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->_fh, __pyx_n_s_read); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_RECORD_SIZE); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_lead_record = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":249
 *         # get the next record
 *         lead_record = self._fh.read(RECORD_SIZE)
 *         self._record_number += 1             # <<<<<<<<<<<<<<
 * 
 *         # check if the file ended early, if so return Nones
 */
  __pyx_v_self->_record_number = (__pyx_v_self->_record_number + 1);

  /* "pyart/io/_sigmetfile.pyx":252
 * 
 *         # check if the file ended early, if so return Nones
 *         if len(lead_record) != RECORD_SIZE:             # <<<<<<<<<<<<<<
 *             return None, None, None
 * 
 */
  __pyx_t_5 = PyObject_Length(__pyx_v_lead_record); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 252, __pyx_L1_error)
  __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 252, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_RECORD_SIZE); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 252, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_t_2, Py_NE); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 252, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 252, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_6) {

    /* "pyart/io/_sigmetfile.pyx":253
 *         # check if the file ended early, if so return Nones
 *         if len(lead_record) != RECORD_SIZE:
 *             return None, None, None             # <<<<<<<<<<<<<<
 * 
 *         # unpack structures
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_tuple_);
    __pyx_r = __pyx_tuple_;
    goto __pyx_L0;

    /* "pyart/io/_sigmetfile.pyx":252
 * 
 *         # check if the file ended early, if so return Nones
 *         if len(lead_record) != RECORD_SIZE:             # <<<<<<<<<<<<<<
 *             return None, None, None
 * 
 */
  }

  /* "pyart/io/_sigmetfile.pyx":256
 * 
 *         # unpack structures
 *         raw_prod_bhdr = _unpack_raw_prod_bhdr(lead_record)             # <<<<<<<<<<<<<<
 *         self._raw_product_bhdrs.append([raw_prod_bhdr])
 *         ingest_data_headers = _unpack_ingest_data_headers(
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_raw_prod_bhdr); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_1, __pyx_v_lead_record) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_lead_record);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_raw_prod_bhdr = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pyart/io/_sigmetfile.pyx":257
 *         # unpack structures
 *         raw_prod_bhdr = _unpack_raw_prod_bhdr(lead_record)
 *         self._raw_product_bhdrs.append([raw_prod_bhdr])             # <<<<<<<<<<<<<<
 *         ingest_data_headers = _unpack_ingest_data_headers(
 *             lead_record, self.ndata_types)
 */
  __pyx_t_3 = PyList_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_raw_prod_bhdr);
  __Pyx_GIVEREF(__pyx_v_raw_prod_bhdr);
  PyList_SET_ITEM(__pyx_t_3, 0, __pyx_v_raw_prod_bhdr);
  __pyx_t_7 = __Pyx_PyObject_Append(__pyx_v_self->_raw_product_bhdrs, __pyx_t_3); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pyart/io/_sigmetfile.pyx":258
 *         raw_prod_bhdr = _unpack_raw_prod_bhdr(lead_record)
 *         self._raw_product_bhdrs.append([raw_prod_bhdr])
 *         ingest_data_headers = _unpack_ingest_data_headers(             # <<<<<<<<<<<<<<
 *             lead_record, self.ndata_types)
 *         if ingest_data_headers is None:
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_ingest_data_headers); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 258, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "pyart/io/_sigmetfile.pyx":259
 *         self._raw_product_bhdrs.append([raw_prod_bhdr])
 *         ingest_data_headers = _unpack_ingest_data_headers(
 *             lead_record, self.ndata_types)             # <<<<<<<<<<<<<<
 *         if ingest_data_headers is None:
 *             return None, None, None
 */
  __pyx_t_1 = NULL;
  __pyx_t_8 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_8 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_lead_record, __pyx_v_self->ndata_types};
    __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 258, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_lead_record, __pyx_v_self->ndata_types};
    __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 258, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else
  #endif
  {
    __pyx_t_4 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 258, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__pyx_t_1) {
      __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1); __pyx_t_1 = NULL;
    }
    __Pyx_INCREF(__pyx_v_lead_record);
    __Pyx_GIVEREF(__pyx_v_lead_record);
    PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_8, __pyx_v_lead_record);
    __Pyx_INCREF(__pyx_v_self->ndata_types);
    __Pyx_GIVEREF(__pyx_v_self->ndata_types);
    PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_8, __pyx_v_self->ndata_types);
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 258, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_ingest_data_headers = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pyart/io/_sigmetfile.pyx":260
 *         ingest_data_headers = _unpack_ingest_data_headers(
 *             lead_record, self.ndata_types)
 *         if ingest_data_headers is None:             # <<<<<<<<<<<<<<
 *             return None, None, None
 * 
 */
  __pyx_t_6 = (__pyx_v_ingest_data_headers == Py_None);
  __pyx_t_9 = (__pyx_t_6 != 0);
  if (__pyx_t_9) {

    /* "pyart/io/_sigmetfile.pyx":261
 *             lead_record, self.ndata_types)
 *         if ingest_data_headers is None:
 *             return None, None, None             # <<<<<<<<<<<<<<
 * 
 *         # determine size of data
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_tuple_);
    __pyx_r = __pyx_tuple_;
    goto __pyx_L0;

    /* "pyart/io/_sigmetfile.pyx":260
 *         ingest_data_headers = _unpack_ingest_data_headers(
 *             lead_record, self.ndata_types)
 *         if ingest_data_headers is None:             # <<<<<<<<<<<<<<
 *             return None, None, None
 * 
 */
  }

  /* "pyart/io/_sigmetfile.pyx":264
 * 
 *         # determine size of data
 *         nray_data_types = [d['number_rays_file_expected']             # <<<<<<<<<<<<<<
 *                            for d in ingest_data_headers]
 *         nrays = sum(nray_data_types)    # total rays
 */
  { /* enter inner scope */
    __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 264, __pyx_L7_error)
    __Pyx_GOTREF(__pyx_t_3);

    /* "pyart/io/_sigmetfile.pyx":265
 *         # determine size of data
 *         nray_data_types = [d['number_rays_file_expected']
 *                            for d in ingest_data_headers]             # <<<<<<<<<<<<<<
 *         nrays = sum(nray_data_types)    # total rays
 *         nbins = self.product_hdr['product_end']['number_bins']
 */
    if (likely(PyList_CheckExact(__pyx_v_ingest_data_headers)) || PyTuple_CheckExact(__pyx_v_ingest_data_headers)) {
      __pyx_t_2 = __pyx_v_ingest_data_headers; __Pyx_INCREF(__pyx_t_2); __pyx_t_5 = 0;
      __pyx_t_10 = NULL;
    } else {
      __pyx_t_5 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_ingest_data_headers); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 265, __pyx_L7_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_10 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 265, __pyx_L7_error)
    }
    for (;;) {
      if (likely(!__pyx_t_10)) {
        if (likely(PyList_CheckExact(__pyx_t_2))) {
          if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_5); __Pyx_INCREF(__pyx_t_4); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 265, __pyx_L7_error)
          #else
          __pyx_t_4 = PySequence_ITEM(__pyx_t_2, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 265, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        } else {
          if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_5); __Pyx_INCREF(__pyx_t_4); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 265, __pyx_L7_error)
          #else
          __pyx_t_4 = PySequence_ITEM(__pyx_t_2, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 265, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        }
      } else {
        __pyx_t_4 = __pyx_t_10(__pyx_t_2);
        if (unlikely(!__pyx_t_4)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 265, __pyx_L7_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_4);
      }
      __Pyx_XDECREF_SET(__pyx_8genexpr3__pyx_v_d, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "pyart/io/_sigmetfile.pyx":264
 * 
 *         # determine size of data
 *         nray_data_types = [d['number_rays_file_expected']             # <<<<<<<<<<<<<<
 *                            for d in ingest_data_headers]
 *         nrays = sum(nray_data_types)    # total rays
 */
      __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_8genexpr3__pyx_v_d, __pyx_n_u_number_rays_file_expected); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 264, __pyx_L7_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_t_4))) __PYX_ERR(0, 264, __pyx_L7_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyart/io/_sigmetfile.pyx":265
 *         # determine size of data
 *         nray_data_types = [d['number_rays_file_expected']
 *                            for d in ingest_data_headers]             # <<<<<<<<<<<<<<
 *         nrays = sum(nray_data_types)    # total rays
 *         nbins = self.product_hdr['product_end']['number_bins']
 */
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_8genexpr3__pyx_v_d); __pyx_8genexpr3__pyx_v_d = 0;
    goto __pyx_L10_exit_scope;
    __pyx_L7_error:;
    __Pyx_XDECREF(__pyx_8genexpr3__pyx_v_d); __pyx_8genexpr3__pyx_v_d = 0;
    goto __pyx_L1_error;
    __pyx_L10_exit_scope:;
  } /* exit inner scope */
  __pyx_v_nray_data_types = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "pyart/io/_sigmetfile.pyx":266
 *         nray_data_types = [d['number_rays_file_expected']
 *                            for d in ingest_data_headers]
 *         nrays = sum(nray_data_types)    # total rays             # <<<<<<<<<<<<<<
 *         nbins = self.product_hdr['product_end']['number_bins']
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_sum, __pyx_v_nray_data_types); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 266, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_nrays = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pyart/io/_sigmetfile.pyx":267
 *                            for d in ingest_data_headers]
 *         nrays = sum(nray_data_types)    # total rays
 *         nbins = self.product_hdr['product_end']['number_bins']             # <<<<<<<<<<<<<<
 * 
 *         # prepare to read rays
 */
  __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_v_self->product_hdr, __pyx_n_u_product_end); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_3, __pyx_n_u_number_bins); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_nbins = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pyart/io/_sigmetfile.pyx":270
 * 
 *         # prepare to read rays
 *         self._rbuf = np.frombuffer(lead_record, dtype='int16')             # <<<<<<<<<<<<<<
 *         self._rbuf_p = <np.int16_t*>self._rbuf.data
 *         self._rbuf_pos = int((12 + 76 * self.ndata_types) / 2) - 1
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_frombuffer); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_lead_record);
  __Pyx_GIVEREF(__pyx_v_lead_record);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_lead_record);
  __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_n_u_int16) < 0) __PYX_ERR(0, 270, __pyx_L1_error)
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_rbuf);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_rbuf));
  __pyx_v_self->_rbuf = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":271
 *         # prepare to read rays
 *         self._rbuf = np.frombuffer(lead_record, dtype='int16')
 *         self._rbuf_p = <np.int16_t*>self._rbuf.data             # <<<<<<<<<<<<<<
 *         self._rbuf_pos = int((12 + 76 * self.ndata_types) / 2) - 1
 *         # set data initially to ones so that missing data can be better
 */
  __pyx_v_self->_rbuf_p = ((__pyx_t_5numpy_int16_t *)__pyx_v_self->_rbuf->data);

  /* "pyart/io/_sigmetfile.pyx":272
 *         self._rbuf = np.frombuffer(lead_record, dtype='int16')
 *         self._rbuf_p = <np.int16_t*>self._rbuf.data
 *         self._rbuf_pos = int((12 + 76 * self.ndata_types) / 2) - 1             # <<<<<<<<<<<<<<
 *         # set data initially to ones so that missing data can be better
 *         # seen when debugging
 */
  __pyx_t_1 = PyNumber_Multiply(__pyx_int_76, __pyx_v_self->ndata_types); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_AddCObj(__pyx_int_12, __pyx_t_1, 12, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyInt_TrueDivideObjC(__pyx_t_4, __pyx_int_2, 2, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyNumber_Int(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyInt_SubtractObjC(__pyx_t_4, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_8 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_8 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 272, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->_rbuf_pos = __pyx_t_8;

  /* "pyart/io/_sigmetfile.pyx":275
 *         # set data initially to ones so that missing data can be better
 *         # seen when debugging
 *         raw_sweep_data = np.ones((nrays, nbins + 6), dtype='int16')             # <<<<<<<<<<<<<<
 * 
 *         # get the raw data ray-by-ray
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_ones); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyInt_AddObjC(__pyx_v_nbins, __pyx_int_6, 6, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_nrays);
  __Pyx_GIVEREF(__pyx_v_nrays);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_nrays);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_n_u_int16) < 0) __PYX_ERR(0, 275, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_raw_sweep_data = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pyart/io/_sigmetfile.pyx":278
 * 
 *         # get the raw data ray-by-ray
 *         for ray_i in xrange(nrays):             # <<<<<<<<<<<<<<
 *             if self.debug:
 *                 print("Reading ray: %i of %i" % (ray_i, nrays))
 */
  __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_xrange, __pyx_v_nrays); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 278, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
    __pyx_t_2 = __pyx_t_3; __Pyx_INCREF(__pyx_t_2); __pyx_t_5 = 0;
    __pyx_t_10 = NULL;
  } else {
    __pyx_t_5 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 278, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_10 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 278, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  for (;;) {
    if (likely(!__pyx_t_10)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_3 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_5); __Pyx_INCREF(__pyx_t_3); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 278, __pyx_L1_error)
        #else
        __pyx_t_3 = PySequence_ITEM(__pyx_t_2, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 278, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        #endif
      } else {
        if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_5); __Pyx_INCREF(__pyx_t_3); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 278, __pyx_L1_error)
        #else
        __pyx_t_3 = PySequence_ITEM(__pyx_t_2, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 278, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        #endif
      }
    } else {
      __pyx_t_3 = __pyx_t_10(__pyx_t_2);
      if (unlikely(!__pyx_t_3)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 278, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_XDECREF_SET(__pyx_v_ray_i, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "pyart/io/_sigmetfile.pyx":279
 *         # get the raw data ray-by-ray
 *         for ray_i in xrange(nrays):
 *             if self.debug:             # <<<<<<<<<<<<<<
 *                 print("Reading ray: %i of %i" % (ray_i, nrays))
 *                 print("self._rbuf_pos is", self._rbuf_pos)
 */
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_v_self->debug); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 279, __pyx_L1_error)
    if (__pyx_t_9) {

      /* "pyart/io/_sigmetfile.pyx":280
 *         for ray_i in xrange(nrays):
 *             if self.debug:
 *                 print("Reading ray: %i of %i" % (ray_i, nrays))             # <<<<<<<<<<<<<<
 *                 print("self._rbuf_pos is", self._rbuf_pos)
 *             if self._get_ray(nbins, raw_sweep_data[ray_i]):
 */
      __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 280, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_v_ray_i);
      __Pyx_GIVEREF(__pyx_v_ray_i);
      PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_ray_i);
      __Pyx_INCREF(__pyx_v_nrays);
      __Pyx_GIVEREF(__pyx_v_nrays);
      PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_nrays);
      __pyx_t_1 = PyUnicode_Format(__pyx_kp_u_Reading_ray_i_of_i, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 280, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 280, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "pyart/io/_sigmetfile.pyx":281
 *             if self.debug:
 *                 print("Reading ray: %i of %i" % (ray_i, nrays))
 *                 print("self._rbuf_pos is", self._rbuf_pos)             # <<<<<<<<<<<<<<
 *             if self._get_ray(nbins, raw_sweep_data[ray_i]):
 *                 return None, None, None
 */
      __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->_rbuf_pos); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 281, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 281, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_INCREF(__pyx_kp_u_self__rbuf_pos_is);
      __Pyx_GIVEREF(__pyx_kp_u_self__rbuf_pos_is);
      PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_kp_u_self__rbuf_pos_is);
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
      __pyx_t_3 = 0;
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 281, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "pyart/io/_sigmetfile.pyx":279
 *         # get the raw data ray-by-ray
 *         for ray_i in xrange(nrays):
 *             if self.debug:             # <<<<<<<<<<<<<<
 *                 print("Reading ray: %i of %i" % (ray_i, nrays))
 *                 print("self._rbuf_pos is", self._rbuf_pos)
 */
    }

    /* "pyart/io/_sigmetfile.pyx":282
 *                 print("Reading ray: %i of %i" % (ray_i, nrays))
 *                 print("self._rbuf_pos is", self._rbuf_pos)
 *             if self._get_ray(nbins, raw_sweep_data[ray_i]):             # <<<<<<<<<<<<<<
 *                 return None, None, None
 * 
 */
    __pyx_t_8 = __Pyx_PyInt_As_int(__pyx_v_nbins); if (unlikely((__pyx_t_8 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 282, __pyx_L1_error)
    __pyx_t_3 = __Pyx_PyObject_GetItem(__pyx_v_raw_sweep_data, __pyx_v_ray_i); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 282, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 282, __pyx_L1_error)
    __pyx_t_9 = (((struct __pyx_vtabstruct_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self->__pyx_vtab)->_get_ray(__pyx_v_self, __pyx_t_8, ((PyArrayObject *)__pyx_t_3)) != 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "pyart/io/_sigmetfile.pyx":283
 *                 print("self._rbuf_pos is", self._rbuf_pos)
 *             if self._get_ray(nbins, raw_sweep_data[ray_i]):
 *                 return None, None, None             # <<<<<<<<<<<<<<
 * 
 *         # return raw data if requested
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_tuple_);
      __pyx_r = __pyx_tuple_;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      goto __pyx_L0;

      /* "pyart/io/_sigmetfile.pyx":282
 *                 print("Reading ray: %i of %i" % (ray_i, nrays))
 *                 print("self._rbuf_pos is", self._rbuf_pos)
 *             if self._get_ray(nbins, raw_sweep_data[ray_i]):             # <<<<<<<<<<<<<<
 *                 return None, None, None
 * 
 */
    }

    /* "pyart/io/_sigmetfile.pyx":278
 * 
 *         # get the raw data ray-by-ray
 *         for ray_i in xrange(nrays):             # <<<<<<<<<<<<<<
 *             if self.debug:
 *                 print("Reading ray: %i of %i" % (ray_i, nrays))
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyart/io/_sigmetfile.pyx":286
 * 
 *         # return raw data if requested
 *         if raw_data:             # <<<<<<<<<<<<<<
 *             return ingest_data_headers, raw_sweep_data
 * 
 */
  __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_v_raw_data); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 286, __pyx_L1_error)
  if (__pyx_t_9) {

    /* "pyart/io/_sigmetfile.pyx":287
 *         # return raw data if requested
 *         if raw_data:
 *             return ingest_data_headers, raw_sweep_data             # <<<<<<<<<<<<<<
 * 
 *         # convert the data and parse the metadata
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 287, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_ingest_data_headers);
    __Pyx_GIVEREF(__pyx_v_ingest_data_headers);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_ingest_data_headers);
    __Pyx_INCREF(__pyx_v_raw_sweep_data);
    __Pyx_GIVEREF(__pyx_v_raw_sweep_data);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_raw_sweep_data);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "pyart/io/_sigmetfile.pyx":286
 * 
 *         # return raw data if requested
 *         if raw_data:             # <<<<<<<<<<<<<<
 *             return ingest_data_headers, raw_sweep_data
 * 
 */
  }

  /* "pyart/io/_sigmetfile.pyx":290
 * 
 *         # convert the data and parse the metadata
 *         sweep_data = []             # <<<<<<<<<<<<<<
 *         sweep_metadata = []
 *         for i, data_type in enumerate(self.data_types):
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 290, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_sweep_data = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyart/io/_sigmetfile.pyx":291
 *         # convert the data and parse the metadata
 *         sweep_data = []
 *         sweep_metadata = []             # <<<<<<<<<<<<<<
 *         for i, data_type in enumerate(self.data_types):
 *             if data_type == 0 and full_xhdr:
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 291, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_sweep_metadata = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyart/io/_sigmetfile.pyx":292
 *         sweep_data = []
 *         sweep_metadata = []
 *         for i, data_type in enumerate(self.data_types):             # <<<<<<<<<<<<<<
 *             if data_type == 0 and full_xhdr:
 *                 sweep_data.append(raw_sweep_data[i::self.ndata_types, 6:])
 */
  __Pyx_INCREF(__pyx_int_0);
  __pyx_t_2 = __pyx_int_0;
  if (likely(PyList_CheckExact(__pyx_v_self->data_types)) || PyTuple_CheckExact(__pyx_v_self->data_types)) {
    __pyx_t_3 = __pyx_v_self->data_types; __Pyx_INCREF(__pyx_t_3); __pyx_t_5 = 0;
    __pyx_t_10 = NULL;
  } else {
    __pyx_t_5 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_v_self->data_types); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 292, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_10 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 292, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_10)) {
      if (likely(PyList_CheckExact(__pyx_t_3))) {
        if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_5); __Pyx_INCREF(__pyx_t_1); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 292, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_3, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 292, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_5); __Pyx_INCREF(__pyx_t_1); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 292, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_3, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 292, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_10(__pyx_t_3);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 292, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_data_type, __pyx_t_1);
    __pyx_t_1 = 0;
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_2);
    __pyx_t_1 = __Pyx_PyInt_AddObjC(__pyx_t_2, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 292, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2);
    __pyx_t_2 = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "pyart/io/_sigmetfile.pyx":293
 *         sweep_metadata = []
 *         for i, data_type in enumerate(self.data_types):
 *             if data_type == 0 and full_xhdr:             # <<<<<<<<<<<<<<
 *                 sweep_data.append(raw_sweep_data[i::self.ndata_types, 6:])
 *             else:
 */
    __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_data_type, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 293, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 293, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_6) {
    } else {
      __pyx_t_9 = __pyx_t_6;
      goto __pyx_L19_bool_binop_done;
    }
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_full_xhdr); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 293, __pyx_L1_error)
    __pyx_t_9 = __pyx_t_6;
    __pyx_L19_bool_binop_done:;
    if (__pyx_t_9) {

      /* "pyart/io/_sigmetfile.pyx":294
 *         for i, data_type in enumerate(self.data_types):
 *             if data_type == 0 and full_xhdr:
 *                 sweep_data.append(raw_sweep_data[i::self.ndata_types, 6:])             # <<<<<<<<<<<<<<
 *             else:
 *                 sweep_data.append(convert_sigmet_data(
 */
      __pyx_t_1 = PySlice_New(__pyx_v_i, Py_None, __pyx_v_self->ndata_types); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 294, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 294, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_1);
      PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
      __Pyx_INCREF(__pyx_slice__2);
      __Pyx_GIVEREF(__pyx_slice__2);
      PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_slice__2);
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_raw_sweep_data, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 294, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_7 = __Pyx_PyList_Append(__pyx_v_sweep_data, __pyx_t_1); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 294, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "pyart/io/_sigmetfile.pyx":293
 *         sweep_metadata = []
 *         for i, data_type in enumerate(self.data_types):
 *             if data_type == 0 and full_xhdr:             # <<<<<<<<<<<<<<
 *                 sweep_data.append(raw_sweep_data[i::self.ndata_types, 6:])
 *             else:
 */
      goto __pyx_L18;
    }

    /* "pyart/io/_sigmetfile.pyx":296
 *                 sweep_data.append(raw_sweep_data[i::self.ndata_types, 6:])
 *             else:
 *                 sweep_data.append(convert_sigmet_data(             # <<<<<<<<<<<<<<
 *                     data_type, raw_sweep_data[i::self.ndata_types, 6:],
 *                     raw_sweep_data[i::self.ndata_types, 4]))
 */
    /*else*/ {
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_convert_sigmet_data); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 296, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);

      /* "pyart/io/_sigmetfile.pyx":297
 *             else:
 *                 sweep_data.append(convert_sigmet_data(
 *                     data_type, raw_sweep_data[i::self.ndata_types, 6:],             # <<<<<<<<<<<<<<
 *                     raw_sweep_data[i::self.ndata_types, 4]))
 *             sweep_metadata.append(_parse_ray_headers(
 */
      __pyx_t_11 = PySlice_New(__pyx_v_i, Py_None, __pyx_v_self->ndata_types); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 297, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_12 = PyTuple_New(2); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 297, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_GIVEREF(__pyx_t_11);
      PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_11);
      __Pyx_INCREF(__pyx_slice__2);
      __Pyx_GIVEREF(__pyx_slice__2);
      PyTuple_SET_ITEM(__pyx_t_12, 1, __pyx_slice__2);
      __pyx_t_11 = 0;
      __pyx_t_11 = __Pyx_PyObject_GetItem(__pyx_v_raw_sweep_data, __pyx_t_12); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 297, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

      /* "pyart/io/_sigmetfile.pyx":298
 *                 sweep_data.append(convert_sigmet_data(
 *                     data_type, raw_sweep_data[i::self.ndata_types, 6:],
 *                     raw_sweep_data[i::self.ndata_types, 4]))             # <<<<<<<<<<<<<<
 *             sweep_metadata.append(_parse_ray_headers(
 *                 raw_sweep_data[i::self.ndata_types, :6]))
 */
      __pyx_t_12 = PySlice_New(__pyx_v_i, Py_None, __pyx_v_self->ndata_types); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 298, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_13 = PyTuple_New(2); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 298, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_GIVEREF(__pyx_t_12);
      PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_12);
      __Pyx_INCREF(__pyx_int_4);
      __Pyx_GIVEREF(__pyx_int_4);
      PyTuple_SET_ITEM(__pyx_t_13, 1, __pyx_int_4);
      __pyx_t_12 = 0;
      __pyx_t_12 = __Pyx_PyObject_GetItem(__pyx_v_raw_sweep_data, __pyx_t_13); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 298, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_13 = NULL;
      __pyx_t_8 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_13)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_13);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
          __pyx_t_8 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_v_data_type, __pyx_t_11, __pyx_t_12};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 296, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_v_data_type, __pyx_t_11, __pyx_t_12};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 296, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      } else
      #endif
      {
        __pyx_t_14 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 296, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_14);
        if (__pyx_t_13) {
          __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_13); __pyx_t_13 = NULL;
        }
        __Pyx_INCREF(__pyx_v_data_type);
        __Pyx_GIVEREF(__pyx_v_data_type);
        PyTuple_SET_ITEM(__pyx_t_14, 0+__pyx_t_8, __pyx_v_data_type);
        __Pyx_GIVEREF(__pyx_t_11);
        PyTuple_SET_ITEM(__pyx_t_14, 1+__pyx_t_8, __pyx_t_11);
        __Pyx_GIVEREF(__pyx_t_12);
        PyTuple_SET_ITEM(__pyx_t_14, 2+__pyx_t_8, __pyx_t_12);
        __pyx_t_11 = 0;
        __pyx_t_12 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_14, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 296, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyart/io/_sigmetfile.pyx":296
 *                 sweep_data.append(raw_sweep_data[i::self.ndata_types, 6:])
 *             else:
 *                 sweep_data.append(convert_sigmet_data(             # <<<<<<<<<<<<<<
 *                     data_type, raw_sweep_data[i::self.ndata_types, 6:],
 *                     raw_sweep_data[i::self.ndata_types, 4]))
 */
      __pyx_t_7 = __Pyx_PyList_Append(__pyx_v_sweep_data, __pyx_t_1); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 296, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __pyx_L18:;

    /* "pyart/io/_sigmetfile.pyx":299
 *                     data_type, raw_sweep_data[i::self.ndata_types, 6:],
 *                     raw_sweep_data[i::self.ndata_types, 4]))
 *             sweep_metadata.append(_parse_ray_headers(             # <<<<<<<<<<<<<<
 *                 raw_sweep_data[i::self.ndata_types, :6]))
 *         return ingest_data_headers, sweep_data, sweep_metadata
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_parse_ray_headers); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 299, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);

    /* "pyart/io/_sigmetfile.pyx":300
 *                     raw_sweep_data[i::self.ndata_types, 4]))
 *             sweep_metadata.append(_parse_ray_headers(
 *                 raw_sweep_data[i::self.ndata_types, :6]))             # <<<<<<<<<<<<<<
 *         return ingest_data_headers, sweep_data, sweep_metadata
 * 
 */
    __pyx_t_14 = PySlice_New(__pyx_v_i, Py_None, __pyx_v_self->ndata_types); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 300, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __pyx_t_12 = PyTuple_New(2); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 300, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_GIVEREF(__pyx_t_14);
    PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_14);
    __Pyx_INCREF(__pyx_slice__3);
    __Pyx_GIVEREF(__pyx_slice__3);
    PyTuple_SET_ITEM(__pyx_t_12, 1, __pyx_slice__3);
    __pyx_t_14 = 0;
    __pyx_t_14 = __Pyx_PyObject_GetItem(__pyx_v_raw_sweep_data, __pyx_t_12); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 300, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __pyx_t_12 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_12)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_12);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_1 = (__pyx_t_12) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_12, __pyx_t_14) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_14);
    __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 299, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "pyart/io/_sigmetfile.pyx":299
 *                     data_type, raw_sweep_data[i::self.ndata_types, 6:],
 *                     raw_sweep_data[i::self.ndata_types, 4]))
 *             sweep_metadata.append(_parse_ray_headers(             # <<<<<<<<<<<<<<
 *                 raw_sweep_data[i::self.ndata_types, :6]))
 *         return ingest_data_headers, sweep_data, sweep_metadata
 */
    __pyx_t_7 = __Pyx_PyList_Append(__pyx_v_sweep_metadata, __pyx_t_1); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 299, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyart/io/_sigmetfile.pyx":292
 *         sweep_data = []
 *         sweep_metadata = []
 *         for i, data_type in enumerate(self.data_types):             # <<<<<<<<<<<<<<
 *             if data_type == 0 and full_xhdr:
 *                 sweep_data.append(raw_sweep_data[i::self.ndata_types, 6:])
 */
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyart/io/_sigmetfile.pyx":301
 *             sweep_metadata.append(_parse_ray_headers(
 *                 raw_sweep_data[i::self.ndata_types, :6]))
 *         return ingest_data_headers, sweep_data, sweep_metadata             # <<<<<<<<<<<<<<
 * 
 *     @cython.wraparound(False)
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 301, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_ingest_data_headers);
  __Pyx_GIVEREF(__pyx_v_ingest_data_headers);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_ingest_data_headers);
  __Pyx_INCREF(__pyx_v_sweep_data);
  __Pyx_GIVEREF(__pyx_v_sweep_data);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_sweep_data);
  __Pyx_INCREF(__pyx_v_sweep_metadata);
  __Pyx_GIVEREF(__pyx_v_sweep_metadata);
  PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_sweep_metadata);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "pyart/io/_sigmetfile.pyx":218
 *         return data, metadata
 * 
 *     def _get_sweep(self, full_xhdr=False, raw_data=False):             # <<<<<<<<<<<<<<
 *         """
 *         Get the data and metadata from the next sweep.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_AddTraceback("pyart.io._sigmetfile.SigmetFile._get_sweep", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_lead_record);
  __Pyx_XDECREF(__pyx_v_raw_prod_bhdr);
  __Pyx_XDECREF(__pyx_v_ingest_data_headers);
  __Pyx_XDECREF(__pyx_v_nray_data_types);
  __Pyx_XDECREF(__pyx_v_nrays);
  __Pyx_XDECREF(__pyx_v_nbins);
  __Pyx_XDECREF(__pyx_v_raw_sweep_data);
  __Pyx_XDECREF(__pyx_v_ray_i);
  __Pyx_XDECREF(__pyx_v_sweep_data);
  __Pyx_XDECREF(__pyx_v_sweep_metadata);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XDECREF(__pyx_v_data_type);
  __Pyx_XDECREF(__pyx_8genexpr3__pyx_v_d);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_sigmetfile.pyx":304
 * 
 *     @cython.wraparound(False)
 *     cdef int _get_ray(self, int nbins, np.ndarray[np.int16_t, ndim=1] out):             # <<<<<<<<<<<<<<
 *         """
 *         Get the next ray, loading new records as needed.
 */

static int __pyx_f_5pyart_2io_11_sigmetfile_10SigmetFile__get_ray(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, int __pyx_v_nbins, PyArrayObject *__pyx_v_out) {
  short __pyx_v_compression_code;
  int __pyx_v_words;
  int __pyx_v_remain;
  int __pyx_v_out_pos;
  int __pyx_v_first_end;
  int __pyx_v_i;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_out;
  __Pyx_Buffer __pyx_pybuffer_out;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  struct __pyx_opt_args_5pyart_2io_11_sigmetfile_10SigmetFile__incr_rbuf_pos __pyx_t_7;
  short __pyx_t_8;
  short __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_ray", 0);
  __pyx_pybuffer_out.pybuffer.buf = NULL;
  __pyx_pybuffer_out.refcount = 0;
  __pyx_pybuffernd_out.data = NULL;
  __pyx_pybuffernd_out.rcbuffer = &__pyx_pybuffer_out;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_out.rcbuffer->pybuffer, (PyObject*)__pyx_v_out, &__Pyx_TypeInfo_nn___pyx_t_5numpy_int16_t, PyBUF_FORMAT| PyBUF_STRIDES| PyBUF_WRITABLE, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 304, __pyx_L1_error)
  }
  __pyx_pybuffernd_out.diminfo[0].strides = __pyx_pybuffernd_out.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_out.diminfo[0].shape = __pyx_pybuffernd_out.rcbuffer->pybuffer.shape[0];

  /* "pyart/io/_sigmetfile.pyx":324
 *         cdef int words, remain, out_pos, first_end, i
 * 
 *         if self._incr_rbuf_pos():             # <<<<<<<<<<<<<<
 *             return -1   # failed read
 *         compression_code = self._rbuf_p[self._rbuf_pos]
 */
  __pyx_t_1 = (((struct __pyx_vtabstruct_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self->__pyx_vtab)->_incr_rbuf_pos(__pyx_v_self, NULL) != 0);
  if (__pyx_t_1) {

    /* "pyart/io/_sigmetfile.pyx":325
 * 
 *         if self._incr_rbuf_pos():
 *             return -1   # failed read             # <<<<<<<<<<<<<<
 *         compression_code = self._rbuf_p[self._rbuf_pos]
 *         out_pos = 0
 */
    __pyx_r = -1;
    goto __pyx_L0;

    /* "pyart/io/_sigmetfile.pyx":324
 *         cdef int words, remain, out_pos, first_end, i
 * 
 *         if self._incr_rbuf_pos():             # <<<<<<<<<<<<<<
 *             return -1   # failed read
 *         compression_code = self._rbuf_p[self._rbuf_pos]
 */
  }

  /* "pyart/io/_sigmetfile.pyx":326
 *         if self._incr_rbuf_pos():
 *             return -1   # failed read
 *         compression_code = self._rbuf_p[self._rbuf_pos]             # <<<<<<<<<<<<<<
 *         out_pos = 0
 * 
 */
  __pyx_v_compression_code = (__pyx_v_self->_rbuf_p[__pyx_v_self->_rbuf_pos]);

  /* "pyart/io/_sigmetfile.pyx":327
 *             return -1   # failed read
 *         compression_code = self._rbuf_p[self._rbuf_pos]
 *         out_pos = 0             # <<<<<<<<<<<<<<
 * 
 *         if compression_code == 1:
 */
  __pyx_v_out_pos = 0;

  /* "pyart/io/_sigmetfile.pyx":329
 *         out_pos = 0
 * 
 *         if compression_code == 1:             # <<<<<<<<<<<<<<
 *             # mark ray as missing by setting numbers of bins to -1
 *             out[4] = -1
 */
  __pyx_t_1 = ((__pyx_v_compression_code == 1) != 0);
  if (__pyx_t_1) {

    /* "pyart/io/_sigmetfile.pyx":331
 *         if compression_code == 1:
 *             # mark ray as missing by setting numbers of bins to -1
 *             out[4] = -1             # <<<<<<<<<<<<<<
 *             return 0
 * 
 */
    __pyx_t_2 = 4;
    __pyx_t_3 = -1;
    if (__pyx_t_2 < 0) {
      __pyx_t_3 = 0;
    } else if (unlikely(__pyx_t_2 >= __pyx_pybuffernd_out.diminfo[0].shape)) __pyx_t_3 = 0;
    if (unlikely(__pyx_t_3 != -1)) {
      __Pyx_RaiseBufferIndexError(__pyx_t_3);
      __PYX_ERR(0, 331, __pyx_L1_error)
    }
    *__Pyx_BufPtrStrided1d(__pyx_t_5numpy_int16_t *, __pyx_pybuffernd_out.rcbuffer->pybuffer.buf, __pyx_t_2, __pyx_pybuffernd_out.diminfo[0].strides) = -1;

    /* "pyart/io/_sigmetfile.pyx":332
 *             # mark ray as missing by setting numbers of bins to -1
 *             out[4] = -1
 *             return 0             # <<<<<<<<<<<<<<
 * 
 *         while compression_code != 1:
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "pyart/io/_sigmetfile.pyx":329
 *         out_pos = 0
 * 
 *         if compression_code == 1:             # <<<<<<<<<<<<<<
 *             # mark ray as missing by setting numbers of bins to -1
 *             out[4] = -1
 */
  }

  /* "pyart/io/_sigmetfile.pyx":334
 *             return 0
 * 
 *         while compression_code != 1:             # <<<<<<<<<<<<<<
 * 
 *             if self._incr_rbuf_pos():
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_compression_code != 1) != 0);
    if (!__pyx_t_1) break;

    /* "pyart/io/_sigmetfile.pyx":336
 *         while compression_code != 1:
 * 
 *             if self._incr_rbuf_pos():             # <<<<<<<<<<<<<<
 *                 return -1   # failed read
 *             if compression_code < 0:
 */
    __pyx_t_1 = (((struct __pyx_vtabstruct_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self->__pyx_vtab)->_incr_rbuf_pos(__pyx_v_self, NULL) != 0);
    if (__pyx_t_1) {

      /* "pyart/io/_sigmetfile.pyx":337
 * 
 *             if self._incr_rbuf_pos():
 *                 return -1   # failed read             # <<<<<<<<<<<<<<
 *             if compression_code < 0:
 *                 words = compression_code + 32768    # last 7 bits give size
 */
      __pyx_r = -1;
      goto __pyx_L0;

      /* "pyart/io/_sigmetfile.pyx":336
 *         while compression_code != 1:
 * 
 *             if self._incr_rbuf_pos():             # <<<<<<<<<<<<<<
 *                 return -1   # failed read
 *             if compression_code < 0:
 */
    }

    /* "pyart/io/_sigmetfile.pyx":338
 *             if self._incr_rbuf_pos():
 *                 return -1   # failed read
 *             if compression_code < 0:             # <<<<<<<<<<<<<<
 *                 words = compression_code + 32768    # last 7 bits give size
 *                 if self._rbuf_pos + words <= 3072:
 */
    __pyx_t_1 = ((__pyx_v_compression_code < 0) != 0);
    if (__pyx_t_1) {

      /* "pyart/io/_sigmetfile.pyx":339
 *                 return -1   # failed read
 *             if compression_code < 0:
 *                 words = compression_code + 32768    # last 7 bits give size             # <<<<<<<<<<<<<<
 *                 if self._rbuf_pos + words <= 3072:
 *                     # all compressed data is in the current record
 */
      __pyx_v_words = (__pyx_v_compression_code + 0x8000);

      /* "pyart/io/_sigmetfile.pyx":340
 *             if compression_code < 0:
 *                 words = compression_code + 32768    # last 7 bits give size
 *                 if self._rbuf_pos + words <= 3072:             # <<<<<<<<<<<<<<
 *                     # all compressed data is in the current record
 *                     for i in range(words):
 */
      __pyx_t_1 = (((__pyx_v_self->_rbuf_pos + __pyx_v_words) <= 0xC00) != 0);
      if (__pyx_t_1) {

        /* "pyart/io/_sigmetfile.pyx":342
 *                 if self._rbuf_pos + words <= 3072:
 *                     # all compressed data is in the current record
 *                     for i in range(words):             # <<<<<<<<<<<<<<
 *                         out[out_pos + i] = self._rbuf_p[self._rbuf_pos + i]
 *                     if self._incr_rbuf_pos(words):
 */
        __pyx_t_3 = __pyx_v_words;
        __pyx_t_4 = __pyx_t_3;
        for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
          __pyx_v_i = __pyx_t_5;

          /* "pyart/io/_sigmetfile.pyx":343
 *                     # all compressed data is in the current record
 *                     for i in range(words):
 *                         out[out_pos + i] = self._rbuf_p[self._rbuf_pos + i]             # <<<<<<<<<<<<<<
 *                     if self._incr_rbuf_pos(words):
 *                         return -1   # failed read
 */
          __pyx_t_2 = (__pyx_v_out_pos + __pyx_v_i);
          __pyx_t_6 = -1;
          if (__pyx_t_2 < 0) {
            __pyx_t_6 = 0;
          } else if (unlikely(__pyx_t_2 >= __pyx_pybuffernd_out.diminfo[0].shape)) __pyx_t_6 = 0;
          if (unlikely(__pyx_t_6 != -1)) {
            __Pyx_RaiseBufferIndexError(__pyx_t_6);
            __PYX_ERR(0, 343, __pyx_L1_error)
          }
          *__Pyx_BufPtrStrided1d(__pyx_t_5numpy_int16_t *, __pyx_pybuffernd_out.rcbuffer->pybuffer.buf, __pyx_t_2, __pyx_pybuffernd_out.diminfo[0].strides) = (__pyx_v_self->_rbuf_p[(__pyx_v_self->_rbuf_pos + __pyx_v_i)]);
        }

        /* "pyart/io/_sigmetfile.pyx":344
 *                     for i in range(words):
 *                         out[out_pos + i] = self._rbuf_p[self._rbuf_pos + i]
 *                     if self._incr_rbuf_pos(words):             # <<<<<<<<<<<<<<
 *                         return -1   # failed read
 *                     out_pos += words
 */
        __pyx_t_7.__pyx_n = 1;
        __pyx_t_7.incr = __pyx_v_words;
        __pyx_t_3 = ((struct __pyx_vtabstruct_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self->__pyx_vtab)->_incr_rbuf_pos(__pyx_v_self, &__pyx_t_7); 
        __pyx_t_1 = (__pyx_t_3 != 0);
        if (__pyx_t_1) {

          /* "pyart/io/_sigmetfile.pyx":345
 *                         out[out_pos + i] = self._rbuf_p[self._rbuf_pos + i]
 *                     if self._incr_rbuf_pos(words):
 *                         return -1   # failed read             # <<<<<<<<<<<<<<
 *                     out_pos += words
 *                 else:
 */
          __pyx_r = -1;
          goto __pyx_L0;

          /* "pyart/io/_sigmetfile.pyx":344
 *                     for i in range(words):
 *                         out[out_pos + i] = self._rbuf_p[self._rbuf_pos + i]
 *                     if self._incr_rbuf_pos(words):             # <<<<<<<<<<<<<<
 *                         return -1   # failed read
 *                     out_pos += words
 */
        }

        /* "pyart/io/_sigmetfile.pyx":346
 *                     if self._incr_rbuf_pos(words):
 *                         return -1   # failed read
 *                     out_pos += words             # <<<<<<<<<<<<<<
 *                 else:
 *                     # data is split between current and next record
 */
        __pyx_v_out_pos = (__pyx_v_out_pos + __pyx_v_words);

        /* "pyart/io/_sigmetfile.pyx":340
 *             if compression_code < 0:
 *                 words = compression_code + 32768    # last 7 bits give size
 *                 if self._rbuf_pos + words <= 3072:             # <<<<<<<<<<<<<<
 *                     # all compressed data is in the current record
 *                     for i in range(words):
 */
        goto __pyx_L9;
      }

      /* "pyart/io/_sigmetfile.pyx":350
 *                     # data is split between current and next record
 *                     # store data from current record
 *                     remain = words - (3072 - self._rbuf_pos)             # <<<<<<<<<<<<<<
 *                     first_end = out_pos + words - remain
 *                     for i in range(first_end - out_pos):
 */
      /*else*/ {
        __pyx_v_remain = (__pyx_v_words - (0xC00 - __pyx_v_self->_rbuf_pos));

        /* "pyart/io/_sigmetfile.pyx":351
 *                     # store data from current record
 *                     remain = words - (3072 - self._rbuf_pos)
 *                     first_end = out_pos + words - remain             # <<<<<<<<<<<<<<
 *                     for i in range(first_end - out_pos):
 *                         out[out_pos + i] = self._rbuf_p[self._rbuf_pos + i]
 */
        __pyx_v_first_end = ((__pyx_v_out_pos + __pyx_v_words) - __pyx_v_remain);

        /* "pyart/io/_sigmetfile.pyx":352
 *                     remain = words - (3072 - self._rbuf_pos)
 *                     first_end = out_pos + words - remain
 *                     for i in range(first_end - out_pos):             # <<<<<<<<<<<<<<
 *                         out[out_pos + i] = self._rbuf_p[self._rbuf_pos + i]
 * 
 */
        __pyx_t_3 = (__pyx_v_first_end - __pyx_v_out_pos);
        __pyx_t_4 = __pyx_t_3;
        for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
          __pyx_v_i = __pyx_t_5;

          /* "pyart/io/_sigmetfile.pyx":353
 *                     first_end = out_pos + words - remain
 *                     for i in range(first_end - out_pos):
 *                         out[out_pos + i] = self._rbuf_p[self._rbuf_pos + i]             # <<<<<<<<<<<<<<
 * 
 *                     # read data from next record and store
 */
          __pyx_t_2 = (__pyx_v_out_pos + __pyx_v_i);
          __pyx_t_6 = -1;
          if (__pyx_t_2 < 0) {
            __pyx_t_6 = 0;
          } else if (unlikely(__pyx_t_2 >= __pyx_pybuffernd_out.diminfo[0].shape)) __pyx_t_6 = 0;
          if (unlikely(__pyx_t_6 != -1)) {
            __Pyx_RaiseBufferIndexError(__pyx_t_6);
            __PYX_ERR(0, 353, __pyx_L1_error)
          }
          *__Pyx_BufPtrStrided1d(__pyx_t_5numpy_int16_t *, __pyx_pybuffernd_out.rcbuffer->pybuffer.buf, __pyx_t_2, __pyx_pybuffernd_out.diminfo[0].strides) = (__pyx_v_self->_rbuf_p[(__pyx_v_self->_rbuf_pos + __pyx_v_i)]);
        }

        /* "pyart/io/_sigmetfile.pyx":356
 * 
 *                     # read data from next record and store
 *                     self._load_record()             # <<<<<<<<<<<<<<
 *                     for i in range(out_pos + words - first_end):
 *                         out[first_end + i] = self._rbuf_p[self._rbuf_pos + i]
 */
        (void)(((struct __pyx_vtabstruct_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self->__pyx_vtab)->_load_record(__pyx_v_self));

        /* "pyart/io/_sigmetfile.pyx":357
 *                     # read data from next record and store
 *                     self._load_record()
 *                     for i in range(out_pos + words - first_end):             # <<<<<<<<<<<<<<
 *                         out[first_end + i] = self._rbuf_p[self._rbuf_pos + i]
 * 
 */
        __pyx_t_3 = ((__pyx_v_out_pos + __pyx_v_words) - __pyx_v_first_end);
        __pyx_t_4 = __pyx_t_3;
        for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
          __pyx_v_i = __pyx_t_5;

          /* "pyart/io/_sigmetfile.pyx":358
 *                     self._load_record()
 *                     for i in range(out_pos + words - first_end):
 *                         out[first_end + i] = self._rbuf_p[self._rbuf_pos + i]             # <<<<<<<<<<<<<<
 * 
 *                     if self._incr_rbuf_pos(remain):
 */
          __pyx_t_2 = (__pyx_v_first_end + __pyx_v_i);
          __pyx_t_6 = -1;
          if (__pyx_t_2 < 0) {
            __pyx_t_6 = 0;
          } else if (unlikely(__pyx_t_2 >= __pyx_pybuffernd_out.diminfo[0].shape)) __pyx_t_6 = 0;
          if (unlikely(__pyx_t_6 != -1)) {
            __Pyx_RaiseBufferIndexError(__pyx_t_6);
            __PYX_ERR(0, 358, __pyx_L1_error)
          }
          *__Pyx_BufPtrStrided1d(__pyx_t_5numpy_int16_t *, __pyx_pybuffernd_out.rcbuffer->pybuffer.buf, __pyx_t_2, __pyx_pybuffernd_out.diminfo[0].strides) = (__pyx_v_self->_rbuf_p[(__pyx_v_self->_rbuf_pos + __pyx_v_i)]);
        }

        /* "pyart/io/_sigmetfile.pyx":360
 *                         out[first_end + i] = self._rbuf_p[self._rbuf_pos + i]
 * 
 *                     if self._incr_rbuf_pos(remain):             # <<<<<<<<<<<<<<
 *                         return -1   # failed read
 *                     out_pos += words
 */
        __pyx_t_7.__pyx_n = 1;
        __pyx_t_7.incr = __pyx_v_remain;
        __pyx_t_3 = ((struct __pyx_vtabstruct_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self->__pyx_vtab)->_incr_rbuf_pos(__pyx_v_self, &__pyx_t_7); 
        __pyx_t_1 = (__pyx_t_3 != 0);
        if (__pyx_t_1) {

          /* "pyart/io/_sigmetfile.pyx":361
 * 
 *                     if self._incr_rbuf_pos(remain):
 *                         return -1   # failed read             # <<<<<<<<<<<<<<
 *                     out_pos += words
 *             else:
 */
          __pyx_r = -1;
          goto __pyx_L0;

          /* "pyart/io/_sigmetfile.pyx":360
 *                         out[first_end + i] = self._rbuf_p[self._rbuf_pos + i]
 * 
 *                     if self._incr_rbuf_pos(remain):             # <<<<<<<<<<<<<<
 *                         return -1   # failed read
 *                     out_pos += words
 */
        }

        /* "pyart/io/_sigmetfile.pyx":362
 *                     if self._incr_rbuf_pos(remain):
 *                         return -1   # failed read
 *                     out_pos += words             # <<<<<<<<<<<<<<
 *             else:
 *                 # add zeros to out
 */
        __pyx_v_out_pos = (__pyx_v_out_pos + __pyx_v_words);
      }
      __pyx_L9:;

      /* "pyart/io/_sigmetfile.pyx":338
 *             if self._incr_rbuf_pos():
 *                 return -1   # failed read
 *             if compression_code < 0:             # <<<<<<<<<<<<<<
 *                 words = compression_code + 32768    # last 7 bits give size
 *                 if self._rbuf_pos + words <= 3072:
 */
      goto __pyx_L8;
    }

    /* "pyart/io/_sigmetfile.pyx":365
 *             else:
 *                 # add zeros to out
 *                 if compression_code + out_pos > nbins + 6:             # <<<<<<<<<<<<<<
 *                     return -1   # file is corrupt
 *                 for i in range(compression_code):
 */
    /*else*/ {
      __pyx_t_1 = (((__pyx_v_compression_code + __pyx_v_out_pos) > (__pyx_v_nbins + 6)) != 0);
      if (__pyx_t_1) {

        /* "pyart/io/_sigmetfile.pyx":366
 *                 # add zeros to out
 *                 if compression_code + out_pos > nbins + 6:
 *                     return -1   # file is corrupt             # <<<<<<<<<<<<<<
 *                 for i in range(compression_code):
 *                     out[out_pos + i] = 0
 */
        __pyx_r = -1;
        goto __pyx_L0;

        /* "pyart/io/_sigmetfile.pyx":365
 *             else:
 *                 # add zeros to out
 *                 if compression_code + out_pos > nbins + 6:             # <<<<<<<<<<<<<<
 *                     return -1   # file is corrupt
 *                 for i in range(compression_code):
 */
      }

      /* "pyart/io/_sigmetfile.pyx":367
 *                 if compression_code + out_pos > nbins + 6:
 *                     return -1   # file is corrupt
 *                 for i in range(compression_code):             # <<<<<<<<<<<<<<
 *                     out[out_pos + i] = 0
 *                 out_pos += compression_code
 */
      __pyx_t_8 = __pyx_v_compression_code;
      __pyx_t_9 = __pyx_t_8;
      for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_9; __pyx_t_3+=1) {
        __pyx_v_i = __pyx_t_3;

        /* "pyart/io/_sigmetfile.pyx":368
 *                     return -1   # file is corrupt
 *                 for i in range(compression_code):
 *                     out[out_pos + i] = 0             # <<<<<<<<<<<<<<
 *                 out_pos += compression_code
 *             compression_code = self._rbuf_p[self._rbuf_pos]
 */
        __pyx_t_2 = (__pyx_v_out_pos + __pyx_v_i);
        __pyx_t_4 = -1;
        if (__pyx_t_2 < 0) {
          __pyx_t_4 = 0;
        } else if (unlikely(__pyx_t_2 >= __pyx_pybuffernd_out.diminfo[0].shape)) __pyx_t_4 = 0;
        if (unlikely(__pyx_t_4 != -1)) {
          __Pyx_RaiseBufferIndexError(__pyx_t_4);
          __PYX_ERR(0, 368, __pyx_L1_error)
        }
        *__Pyx_BufPtrStrided1d(__pyx_t_5numpy_int16_t *, __pyx_pybuffernd_out.rcbuffer->pybuffer.buf, __pyx_t_2, __pyx_pybuffernd_out.diminfo[0].strides) = 0;
      }

      /* "pyart/io/_sigmetfile.pyx":369
 *                 for i in range(compression_code):
 *                     out[out_pos + i] = 0
 *                 out_pos += compression_code             # <<<<<<<<<<<<<<
 *             compression_code = self._rbuf_p[self._rbuf_pos]
 * 
 */
      __pyx_v_out_pos = (__pyx_v_out_pos + __pyx_v_compression_code);
    }
    __pyx_L8:;

    /* "pyart/io/_sigmetfile.pyx":370
 *                     out[out_pos + i] = 0
 *                 out_pos += compression_code
 *             compression_code = self._rbuf_p[self._rbuf_pos]             # <<<<<<<<<<<<<<
 * 
 *         return 0
 */
    __pyx_v_compression_code = (__pyx_v_self->_rbuf_p[__pyx_v_self->_rbuf_pos]);
  }

  /* "pyart/io/_sigmetfile.pyx":372
 *             compression_code = self._rbuf_p[self._rbuf_pos]
 * 
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     cdef int _incr_rbuf_pos(self, int incr=1):
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/io/_sigmetfile.pyx":304
 * 
 *     @cython.wraparound(False)
 *     cdef int _get_ray(self, int nbins, np.ndarray[np.int16_t, ndim=1] out):             # <<<<<<<<<<<<<<
 *         """
 *         Get the next ray, loading new records as needed.
 */

  /* function exit code */
  __pyx_L1_error:;
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_out.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_WriteUnraisable("pyart.io._sigmetfile.SigmetFile._get_ray", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_out.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_sigmetfile.pyx":374
 *         return 0
 * 
 *     cdef int _incr_rbuf_pos(self, int incr=1):             # <<<<<<<<<<<<<<
 *         """
 *         Increment the record buffer position, load a new record if needed.
 */

static int __pyx_f_5pyart_2io_11_sigmetfile_10SigmetFile__incr_rbuf_pos(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, struct __pyx_opt_args_5pyart_2io_11_sigmetfile_10SigmetFile__incr_rbuf_pos *__pyx_optional_args) {
  int __pyx_v_incr = ((int)1);
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("_incr_rbuf_pos", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_incr = __pyx_optional_args->incr;
    }
  }

  /* "pyart/io/_sigmetfile.pyx":378
 *         Increment the record buffer position, load a new record if needed.
 *         """
 *         self._rbuf_pos += incr             # <<<<<<<<<<<<<<
 *         if self._rbuf_pos >= 3072:
 *             if self._load_record():
 */
  __pyx_v_self->_rbuf_pos = (__pyx_v_self->_rbuf_pos + __pyx_v_incr);

  /* "pyart/io/_sigmetfile.pyx":379
 *         """
 *         self._rbuf_pos += incr
 *         if self._rbuf_pos >= 3072:             # <<<<<<<<<<<<<<
 *             if self._load_record():
 *                 return -1   # failed read
 */
  __pyx_t_1 = ((__pyx_v_self->_rbuf_pos >= 0xC00) != 0);
  if (__pyx_t_1) {

    /* "pyart/io/_sigmetfile.pyx":380
 *         self._rbuf_pos += incr
 *         if self._rbuf_pos >= 3072:
 *             if self._load_record():             # <<<<<<<<<<<<<<
 *                 return -1   # failed read
 *         return 0
 */
    __pyx_t_1 = (((struct __pyx_vtabstruct_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self->__pyx_vtab)->_load_record(__pyx_v_self) != 0);
    if (__pyx_t_1) {

      /* "pyart/io/_sigmetfile.pyx":381
 *         if self._rbuf_pos >= 3072:
 *             if self._load_record():
 *                 return -1   # failed read             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
      __pyx_r = -1;
      goto __pyx_L0;

      /* "pyart/io/_sigmetfile.pyx":380
 *         self._rbuf_pos += incr
 *         if self._rbuf_pos >= 3072:
 *             if self._load_record():             # <<<<<<<<<<<<<<
 *                 return -1   # failed read
 *         return 0
 */
    }

    /* "pyart/io/_sigmetfile.pyx":379
 *         """
 *         self._rbuf_pos += incr
 *         if self._rbuf_pos >= 3072:             # <<<<<<<<<<<<<<
 *             if self._load_record():
 *                 return -1   # failed read
 */
  }

  /* "pyart/io/_sigmetfile.pyx":382
 *             if self._load_record():
 *                 return -1   # failed read
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     cdef int _load_record(self):
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/io/_sigmetfile.pyx":374
 *         return 0
 * 
 *     cdef int _incr_rbuf_pos(self, int incr=1):             # <<<<<<<<<<<<<<
 *         """
 *         Increment the record buffer position, load a new record if needed.
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_sigmetfile.pyx":384
 *         return 0
 * 
 *     cdef int _load_record(self):             # <<<<<<<<<<<<<<
 *         """ Load the next record. returns -1 on fail, 0 if success. """
 *         record = self._fh.read(RECORD_SIZE)
 */

static int __pyx_f_5pyart_2io_11_sigmetfile_10SigmetFile__load_record(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self) {
  PyObject *__pyx_v_record = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_load_record", 0);

  /* "pyart/io/_sigmetfile.pyx":386
 *     cdef int _load_record(self):
 *         """ Load the next record. returns -1 on fail, 0 if success. """
 *         record = self._fh.read(RECORD_SIZE)             # <<<<<<<<<<<<<<
 *         if len(record) != RECORD_SIZE:
 *             return -1   # failed read
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->_fh, __pyx_n_s_read); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 386, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_RECORD_SIZE); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 386, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 386, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_record = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":387
 *         """ Load the next record. returns -1 on fail, 0 if success. """
 *         record = self._fh.read(RECORD_SIZE)
 *         if len(record) != RECORD_SIZE:             # <<<<<<<<<<<<<<
 *             return -1   # failed read
 *         self._record_number += 1
 */
  __pyx_t_5 = PyObject_Length(__pyx_v_record); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 387, __pyx_L1_error)
  __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 387, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_RECORD_SIZE); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 387, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_t_2, Py_NE); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 387, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 387, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_6) {

    /* "pyart/io/_sigmetfile.pyx":388
 *         record = self._fh.read(RECORD_SIZE)
 *         if len(record) != RECORD_SIZE:
 *             return -1   # failed read             # <<<<<<<<<<<<<<
 *         self._record_number += 1
 *         if self.debug:
 */
    __pyx_r = -1;
    goto __pyx_L0;

    /* "pyart/io/_sigmetfile.pyx":387
 *         """ Load the next record. returns -1 on fail, 0 if success. """
 *         record = self._fh.read(RECORD_SIZE)
 *         if len(record) != RECORD_SIZE:             # <<<<<<<<<<<<<<
 *             return -1   # failed read
 *         self._record_number += 1
 */
  }

  /* "pyart/io/_sigmetfile.pyx":389
 *         if len(record) != RECORD_SIZE:
 *             return -1   # failed read
 *         self._record_number += 1             # <<<<<<<<<<<<<<
 *         if self.debug:
 *             print("Finished loading record:", self._record_number)
 */
  __pyx_v_self->_record_number = (__pyx_v_self->_record_number + 1);

  /* "pyart/io/_sigmetfile.pyx":390
 *             return -1   # failed read
 *         self._record_number += 1
 *         if self.debug:             # <<<<<<<<<<<<<<
 *             print("Finished loading record:", self._record_number)
 *         self._raw_product_bhdrs[-1].append(_unpack_raw_prod_bhdr(record))
 */
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_self->debug); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 390, __pyx_L1_error)
  if (__pyx_t_6) {

    /* "pyart/io/_sigmetfile.pyx":391
 *         self._record_number += 1
 *         if self.debug:
 *             print("Finished loading record:", self._record_number)             # <<<<<<<<<<<<<<
 *         self._raw_product_bhdrs[-1].append(_unpack_raw_prod_bhdr(record))
 *         self._rbuf = np.frombuffer(record, dtype='int16')
 */
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->_record_number); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 391, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 391, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_kp_u_Finished_loading_record);
    __Pyx_GIVEREF(__pyx_kp_u_Finished_loading_record);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_kp_u_Finished_loading_record);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 391, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pyart/io/_sigmetfile.pyx":390
 *             return -1   # failed read
 *         self._record_number += 1
 *         if self.debug:             # <<<<<<<<<<<<<<
 *             print("Finished loading record:", self._record_number)
 *         self._raw_product_bhdrs[-1].append(_unpack_raw_prod_bhdr(record))
 */
  }

  /* "pyart/io/_sigmetfile.pyx":392
 *         if self.debug:
 *             print("Finished loading record:", self._record_number)
 *         self._raw_product_bhdrs[-1].append(_unpack_raw_prod_bhdr(record))             # <<<<<<<<<<<<<<
 *         self._rbuf = np.frombuffer(record, dtype='int16')
 *         self._rbuf_pos = 6
 */
  __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_self->_raw_product_bhdrs, -1L, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 392, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_unpack_raw_prod_bhdr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 392, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_4, __pyx_v_record) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v_record);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 392, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = __Pyx_PyObject_Append(__pyx_t_3, __pyx_t_2); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 392, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyart/io/_sigmetfile.pyx":393
 *             print("Finished loading record:", self._record_number)
 *         self._raw_product_bhdrs[-1].append(_unpack_raw_prod_bhdr(record))
 *         self._rbuf = np.frombuffer(record, dtype='int16')             # <<<<<<<<<<<<<<
 *         self._rbuf_pos = 6
 *         self._rbuf_p = <np.int16_t*>self._rbuf.data
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 393, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_frombuffer); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 393, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 393, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_record);
  __Pyx_GIVEREF(__pyx_v_record);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_record);
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 393, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_n_u_int16) < 0) __PYX_ERR(0, 393, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 393, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 393, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->_rbuf);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_rbuf));
  __pyx_v_self->_rbuf = ((PyArrayObject *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "pyart/io/_sigmetfile.pyx":394
 *         self._raw_product_bhdrs[-1].append(_unpack_raw_prod_bhdr(record))
 *         self._rbuf = np.frombuffer(record, dtype='int16')
 *         self._rbuf_pos = 6             # <<<<<<<<<<<<<<
 *         self._rbuf_p = <np.int16_t*>self._rbuf.data
 *         return 0
 */
  __pyx_v_self->_rbuf_pos = 6;

  /* "pyart/io/_sigmetfile.pyx":395
 *         self._rbuf = np.frombuffer(record, dtype='int16')
 *         self._rbuf_pos = 6
 *         self._rbuf_p = <np.int16_t*>self._rbuf.data             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
  __pyx_v_self->_rbuf_p = ((__pyx_t_5numpy_int16_t *)__pyx_v_self->_rbuf->data);

  /* "pyart/io/_sigmetfile.pyx":396
 *         self._rbuf_pos = 6
 *         self._rbuf_p = <np.int16_t*>self._rbuf.data
 *         return 0             # <<<<<<<<<<<<<<
 * 
 * # functions used by the SigmetFile class
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/io/_sigmetfile.pyx":384
 *         return 0
 * 
 *     cdef int _load_record(self):             # <<<<<<<<<<<<<<
 *         """ Load the next record. returns -1 on fail, 0 if success. """
 *         record = self._fh.read(RECORD_SIZE)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("pyart.io._sigmetfile.SigmetFile._load_record", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_record);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_sigmetfile.pyx":49
 * 
 *     """
 *     cdef public debug, product_hdr, ingest_header, ingest_data_headers, \             # <<<<<<<<<<<<<<
 *         data_types, data_type_names, ndata_types,
 *     cdef public _fh, _raw_product_bhdrs
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_5debug_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_5debug_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_5debug___get__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_5debug___get__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->debug);
  __pyx_r = __pyx_v_self->debug;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_5debug_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_5debug_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_5debug_2__set__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_5debug_2__set__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->debug);
  __Pyx_DECREF(__pyx_v_self->debug);
  __pyx_v_self->debug = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_5debug_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_5debug_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_5debug_4__del__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_5debug_4__del__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->debug);
  __Pyx_DECREF(__pyx_v_self->debug);
  __pyx_v_self->debug = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_11product_hdr_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_11product_hdr_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_11product_hdr___get__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_11product_hdr___get__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->product_hdr);
  __pyx_r = __pyx_v_self->product_hdr;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_11product_hdr_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_11product_hdr_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_11product_hdr_2__set__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_11product_hdr_2__set__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->product_hdr);
  __Pyx_DECREF(__pyx_v_self->product_hdr);
  __pyx_v_self->product_hdr = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_11product_hdr_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_11product_hdr_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_11product_hdr_4__del__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_11product_hdr_4__del__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->product_hdr);
  __Pyx_DECREF(__pyx_v_self->product_hdr);
  __pyx_v_self->product_hdr = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_13ingest_header_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_13ingest_header_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_13ingest_header___get__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_13ingest_header___get__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->ingest_header);
  __pyx_r = __pyx_v_self->ingest_header;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_13ingest_header_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_13ingest_header_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_13ingest_header_2__set__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_13ingest_header_2__set__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->ingest_header);
  __Pyx_DECREF(__pyx_v_self->ingest_header);
  __pyx_v_self->ingest_header = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_13ingest_header_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_13ingest_header_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_13ingest_header_4__del__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_13ingest_header_4__del__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->ingest_header);
  __Pyx_DECREF(__pyx_v_self->ingest_header);
  __pyx_v_self->ingest_header = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_19ingest_data_headers_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_19ingest_data_headers_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_19ingest_data_headers___get__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_19ingest_data_headers___get__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->ingest_data_headers);
  __pyx_r = __pyx_v_self->ingest_data_headers;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_19ingest_data_headers_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_19ingest_data_headers_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_19ingest_data_headers_2__set__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_19ingest_data_headers_2__set__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->ingest_data_headers);
  __Pyx_DECREF(__pyx_v_self->ingest_data_headers);
  __pyx_v_self->ingest_data_headers = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_19ingest_data_headers_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_19ingest_data_headers_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_19ingest_data_headers_4__del__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_19ingest_data_headers_4__del__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->ingest_data_headers);
  __Pyx_DECREF(__pyx_v_self->ingest_data_headers);
  __pyx_v_self->ingest_data_headers = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_sigmetfile.pyx":50
 *     """
 *     cdef public debug, product_hdr, ingest_header, ingest_data_headers, \
 *         data_types, data_type_names, ndata_types,             # <<<<<<<<<<<<<<
 *     cdef public _fh, _raw_product_bhdrs
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_10data_types_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_10data_types_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_10data_types___get__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_10data_types___get__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->data_types);
  __pyx_r = __pyx_v_self->data_types;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_10data_types_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_10data_types_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_10data_types_2__set__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_10data_types_2__set__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->data_types);
  __Pyx_DECREF(__pyx_v_self->data_types);
  __pyx_v_self->data_types = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_10data_types_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_10data_types_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_10data_types_4__del__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_10data_types_4__del__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->data_types);
  __Pyx_DECREF(__pyx_v_self->data_types);
  __pyx_v_self->data_types = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_15data_type_names_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_15data_type_names_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_15data_type_names___get__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_15data_type_names___get__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->data_type_names);
  __pyx_r = __pyx_v_self->data_type_names;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_15data_type_names_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_15data_type_names_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_15data_type_names_2__set__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_15data_type_names_2__set__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->data_type_names);
  __Pyx_DECREF(__pyx_v_self->data_type_names);
  __pyx_v_self->data_type_names = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_15data_type_names_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_15data_type_names_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_15data_type_names_4__del__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_15data_type_names_4__del__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->data_type_names);
  __Pyx_DECREF(__pyx_v_self->data_type_names);
  __pyx_v_self->data_type_names = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_11ndata_types_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_11ndata_types_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_11ndata_types___get__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_11ndata_types___get__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->ndata_types);
  __pyx_r = __pyx_v_self->ndata_types;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_11ndata_types_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_11ndata_types_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_11ndata_types_2__set__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_11ndata_types_2__set__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->ndata_types);
  __Pyx_DECREF(__pyx_v_self->ndata_types);
  __pyx_v_self->ndata_types = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_11ndata_types_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_11ndata_types_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_11ndata_types_4__del__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_11ndata_types_4__del__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->ndata_types);
  __Pyx_DECREF(__pyx_v_self->ndata_types);
  __pyx_v_self->ndata_types = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_sigmetfile.pyx":51
 *     cdef public debug, product_hdr, ingest_header, ingest_data_headers, \
 *         data_types, data_type_names, ndata_types,
 *     cdef public _fh, _raw_product_bhdrs             # <<<<<<<<<<<<<<
 * 
 *     cdef np.ndarray _rbuf
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_3_fh_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_3_fh_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_3_fh___get__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_3_fh___get__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_fh);
  __pyx_r = __pyx_v_self->_fh;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_3_fh_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_3_fh_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_3_fh_2__set__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_3_fh_2__set__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->_fh);
  __Pyx_DECREF(__pyx_v_self->_fh);
  __pyx_v_self->_fh = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_3_fh_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_3_fh_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_3_fh_4__del__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_3_fh_4__del__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_fh);
  __Pyx_DECREF(__pyx_v_self->_fh);
  __pyx_v_self->_fh = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_18_raw_product_bhdrs_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_18_raw_product_bhdrs_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_18_raw_product_bhdrs___get__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_18_raw_product_bhdrs___get__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_raw_product_bhdrs);
  __pyx_r = __pyx_v_self->_raw_product_bhdrs;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_18_raw_product_bhdrs_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_18_raw_product_bhdrs_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_18_raw_product_bhdrs_2__set__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_18_raw_product_bhdrs_2__set__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->_raw_product_bhdrs);
  __Pyx_DECREF(__pyx_v_self->_raw_product_bhdrs);
  __pyx_v_self->_raw_product_bhdrs = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_18_raw_product_bhdrs_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_18_raw_product_bhdrs_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_18_raw_product_bhdrs_4__del__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_18_raw_product_bhdrs_4__del__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_raw_product_bhdrs);
  __Pyx_DECREF(__pyx_v_self->_raw_product_bhdrs);
  __pyx_v_self->_raw_product_bhdrs = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_sigmetfile.pyx":55
 *     cdef np.ndarray _rbuf
 *     cdef np.int16_t * _rbuf_p   # hack for fast indexing of _rbuf
 *     cdef public int _rbuf_pos, _record_number             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, filename, debug=False):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_9_rbuf_pos_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_9_rbuf_pos_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_9_rbuf_pos___get__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_9_rbuf_pos___get__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_rbuf_pos); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._sigmetfile.SigmetFile._rbuf_pos.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_9_rbuf_pos_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_9_rbuf_pos_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_9_rbuf_pos_2__set__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_9_rbuf_pos_2__set__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 55, __pyx_L1_error)
  __pyx_v_self->_rbuf_pos = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyart.io._sigmetfile.SigmetFile._rbuf_pos.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_14_record_number_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_14_record_number_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_14_record_number___get__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_14_record_number___get__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_record_number); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._sigmetfile.SigmetFile._record_number.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_14_record_number_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_14_record_number_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_14_record_number_2__set__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_14_record_number_2__set__(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 55, __pyx_L1_error)
  __pyx_v_self->_record_number = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyart.io._sigmetfile.SigmetFile._record_number.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._rbuf_p cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_10__reduce_cython__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_10__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._rbuf_p cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._rbuf_p cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._rbuf_p cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._sigmetfile.SigmetFile.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._rbuf_p cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._rbuf_p cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_12__setstate_cython__(((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10SigmetFile_12__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self._rbuf_p cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._rbuf_p cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._rbuf_p cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._rbuf_p cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._sigmetfile.SigmetFile.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_sigmetfile.pyx":401
 * 
 * 
 * def _data_types_from_mask(word0, word1, word2, word3):             # <<<<<<<<<<<<<<
 *     """
 *     Return a list of the data types from the words in the data_type mask.
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_1_data_types_from_mask(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5pyart_2io_11_sigmetfile__data_types_from_mask[] = "\n    Return a list of the data types from the words in the data_type mask.\n    ";
static PyMethodDef __pyx_mdef_5pyart_2io_11_sigmetfile_1_data_types_from_mask = {"_data_types_from_mask", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5pyart_2io_11_sigmetfile_1_data_types_from_mask, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5pyart_2io_11_sigmetfile__data_types_from_mask};
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_1_data_types_from_mask(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_word0 = 0;
  PyObject *__pyx_v_word1 = 0;
  PyObject *__pyx_v_word2 = 0;
  PyObject *__pyx_v_word3 = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_data_types_from_mask (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_word0,&__pyx_n_s_word1,&__pyx_n_s_word2,&__pyx_n_s_word3,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_word0)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_word1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_data_types_from_mask", 1, 4, 4, 1); __PYX_ERR(0, 401, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_word2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_data_types_from_mask", 1, 4, 4, 2); __PYX_ERR(0, 401, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_word3)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_data_types_from_mask", 1, 4, 4, 3); __PYX_ERR(0, 401, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_data_types_from_mask") < 0)) __PYX_ERR(0, 401, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_word0 = values[0];
    __pyx_v_word1 = values[1];
    __pyx_v_word2 = values[2];
    __pyx_v_word3 = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_data_types_from_mask", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 401, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._sigmetfile._data_types_from_mask", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile__data_types_from_mask(__pyx_self, __pyx_v_word0, __pyx_v_word1, __pyx_v_word2, __pyx_v_word3);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile__data_types_from_mask(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_word0, PyObject *__pyx_v_word1, PyObject *__pyx_v_word2, PyObject *__pyx_v_word3) {
  PyObject *__pyx_v_data_types = NULL;
  long __pyx_8genexpr4__pyx_v_i;
  long __pyx_8genexpr5__pyx_v_i;
  long __pyx_8genexpr6__pyx_v_i;
  long __pyx_8genexpr7__pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  long __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_data_types_from_mask", 0);

  /* "pyart/io/_sigmetfile.pyx":405
 *     Return a list of the data types from the words in the data_type mask.
 *     """
 *     data_types = [i for i in range(32) if _is_bit_set(word0, i)]             # <<<<<<<<<<<<<<
 *     data_types += [i+32 for i in range(32) if _is_bit_set(word1, i)]
 *     data_types += [i+64 for i in range(32) if _is_bit_set(word2, i)]
 */
  { /* enter inner scope */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 405, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    for (__pyx_t_2 = 0; __pyx_t_2 < 32; __pyx_t_2+=1) {
      __pyx_8genexpr4__pyx_v_i = __pyx_t_2;
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_is_bit_set); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 405, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = __Pyx_PyInt_From_long(__pyx_8genexpr4__pyx_v_i); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 405, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = NULL;
      __pyx_t_7 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
          __pyx_t_7 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_v_word0, __pyx_t_5};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 405, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_v_word0, __pyx_t_5};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 405, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      {
        __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 405, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        if (__pyx_t_6) {
          __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
        }
        __Pyx_INCREF(__pyx_v_word0);
        __Pyx_GIVEREF(__pyx_v_word0);
        PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_v_word0);
        __Pyx_GIVEREF(__pyx_t_5);
        PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_5);
        __pyx_t_5 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_8, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 405, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 405, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_9) {
        __pyx_t_3 = __Pyx_PyInt_From_long(__pyx_8genexpr4__pyx_v_i); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 405, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_3))) __PYX_ERR(0, 405, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      }
    }
  } /* exit inner scope */
  __pyx_v_data_types = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":406
 *     """
 *     data_types = [i for i in range(32) if _is_bit_set(word0, i)]
 *     data_types += [i+32 for i in range(32) if _is_bit_set(word1, i)]             # <<<<<<<<<<<<<<
 *     data_types += [i+64 for i in range(32) if _is_bit_set(word2, i)]
 *     data_types += [i+96 for i in range(32) if _is_bit_set(word3, i)]
 */
  { /* enter inner scope */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 406, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    for (__pyx_t_2 = 0; __pyx_t_2 < 32; __pyx_t_2+=1) {
      __pyx_8genexpr5__pyx_v_i = __pyx_t_2;
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_is_bit_set); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 406, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_8 = __Pyx_PyInt_From_long(__pyx_8genexpr5__pyx_v_i); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 406, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_5 = NULL;
      __pyx_t_7 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
          __pyx_t_7 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_word1, __pyx_t_8};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 406, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_word1, __pyx_t_8};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 406, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 406, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (__pyx_t_5) {
          __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        }
        __Pyx_INCREF(__pyx_v_word1);
        __Pyx_GIVEREF(__pyx_v_word1);
        PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_7, __pyx_v_word1);
        __Pyx_GIVEREF(__pyx_t_8);
        PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_7, __pyx_t_8);
        __pyx_t_8 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 406, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 406, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_9) {
        __pyx_t_3 = __Pyx_PyInt_From_long((__pyx_8genexpr5__pyx_v_i + 32)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 406, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_3))) __PYX_ERR(0, 406, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      }
    }
  } /* exit inner scope */
  __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_data_types, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 406, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF_SET(__pyx_v_data_types, ((PyObject*)__pyx_t_3));
  __pyx_t_3 = 0;

  /* "pyart/io/_sigmetfile.pyx":407
 *     data_types = [i for i in range(32) if _is_bit_set(word0, i)]
 *     data_types += [i+32 for i in range(32) if _is_bit_set(word1, i)]
 *     data_types += [i+64 for i in range(32) if _is_bit_set(word2, i)]             # <<<<<<<<<<<<<<
 *     data_types += [i+96 for i in range(32) if _is_bit_set(word3, i)]
 *     return data_types
 */
  { /* enter inner scope */
    __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 407, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    for (__pyx_t_2 = 0; __pyx_t_2 < 32; __pyx_t_2+=1) {
      __pyx_8genexpr6__pyx_v_i = __pyx_t_2;
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_is_bit_set); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 407, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_6 = __Pyx_PyInt_From_long(__pyx_8genexpr6__pyx_v_i); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 407, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_8 = NULL;
      __pyx_t_7 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
          __pyx_t_7 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_v_word2, __pyx_t_6};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 407, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_v_word2, __pyx_t_6};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 407, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      {
        __pyx_t_5 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 407, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        if (__pyx_t_8) {
          __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_8); __pyx_t_8 = NULL;
        }
        __Pyx_INCREF(__pyx_v_word2);
        __Pyx_GIVEREF(__pyx_v_word2);
        PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_7, __pyx_v_word2);
        __Pyx_GIVEREF(__pyx_t_6);
        PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_7, __pyx_t_6);
        __pyx_t_6 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 407, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 407, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (__pyx_t_9) {
        __pyx_t_1 = __Pyx_PyInt_From_long((__pyx_8genexpr6__pyx_v_i + 64)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 407, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_t_1))) __PYX_ERR(0, 407, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      }
    }
  } /* exit inner scope */
  __pyx_t_1 = PyNumber_InPlaceAdd(__pyx_v_data_types, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 407, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF_SET(__pyx_v_data_types, ((PyObject*)__pyx_t_1));
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":408
 *     data_types += [i+32 for i in range(32) if _is_bit_set(word1, i)]
 *     data_types += [i+64 for i in range(32) if _is_bit_set(word2, i)]
 *     data_types += [i+96 for i in range(32) if _is_bit_set(word3, i)]             # <<<<<<<<<<<<<<
 *     return data_types
 * 
 */
  { /* enter inner scope */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 408, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    for (__pyx_t_2 = 0; __pyx_t_2 < 32; __pyx_t_2+=1) {
      __pyx_8genexpr7__pyx_v_i = __pyx_t_2;
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_is_bit_set); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 408, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = __Pyx_PyInt_From_long(__pyx_8genexpr7__pyx_v_i); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 408, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = NULL;
      __pyx_t_7 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
          __pyx_t_7 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_v_word3, __pyx_t_5};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 408, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_v_word3, __pyx_t_5};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 408, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      {
        __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 408, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        if (__pyx_t_6) {
          __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
        }
        __Pyx_INCREF(__pyx_v_word3);
        __Pyx_GIVEREF(__pyx_v_word3);
        PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_v_word3);
        __Pyx_GIVEREF(__pyx_t_5);
        PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_5);
        __pyx_t_5 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_8, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 408, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 408, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_9) {
        __pyx_t_3 = __Pyx_PyInt_From_long((__pyx_8genexpr7__pyx_v_i + 96)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 408, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_3))) __PYX_ERR(0, 408, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      }
    }
  } /* exit inner scope */
  __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_data_types, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF_SET(__pyx_v_data_types, ((PyObject*)__pyx_t_3));
  __pyx_t_3 = 0;

  /* "pyart/io/_sigmetfile.pyx":409
 *     data_types += [i+64 for i in range(32) if _is_bit_set(word2, i)]
 *     data_types += [i+96 for i in range(32) if _is_bit_set(word3, i)]
 *     return data_types             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_data_types);
  __pyx_r = __pyx_v_data_types;
  goto __pyx_L0;

  /* "pyart/io/_sigmetfile.pyx":401
 * 
 * 
 * def _data_types_from_mask(word0, word1, word2, word3):             # <<<<<<<<<<<<<<
 *     """
 *     Return a list of the data types from the words in the data_type mask.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("pyart.io._sigmetfile._data_types_from_mask", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_data_types);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_sigmetfile.pyx":412
 * 
 * 
 * def _is_bit_set(number, bit):             # <<<<<<<<<<<<<<
 *     """ Return True if bit is set in number. """
 *     return number >> bit & 1 == 1
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_3_is_bit_set(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5pyart_2io_11_sigmetfile_2_is_bit_set[] = " Return True if bit is set in number. ";
static PyMethodDef __pyx_mdef_5pyart_2io_11_sigmetfile_3_is_bit_set = {"_is_bit_set", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5pyart_2io_11_sigmetfile_3_is_bit_set, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5pyart_2io_11_sigmetfile_2_is_bit_set};
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_3_is_bit_set(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_number = 0;
  PyObject *__pyx_v_bit = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_is_bit_set (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_number,&__pyx_n_s_bit,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_number)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_bit)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_is_bit_set", 1, 2, 2, 1); __PYX_ERR(0, 412, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_is_bit_set") < 0)) __PYX_ERR(0, 412, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_number = values[0];
    __pyx_v_bit = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_is_bit_set", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 412, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._sigmetfile._is_bit_set", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_2_is_bit_set(__pyx_self, __pyx_v_number, __pyx_v_bit);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_2_is_bit_set(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_number, PyObject *__pyx_v_bit) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_is_bit_set", 0);

  /* "pyart/io/_sigmetfile.pyx":414
 * def _is_bit_set(number, bit):
 *     """ Return True if bit is set in number. """
 *     return number >> bit & 1 == 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyNumber_Rshift(__pyx_v_number, __pyx_v_bit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_AndObjC(__pyx_t_1, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_t_2, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_sigmetfile.pyx":412
 * 
 * 
 * def _is_bit_set(number, bit):             # <<<<<<<<<<<<<<
 *     """ Return True if bit is set in number. """
 *     return number >> bit & 1 == 1
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyart.io._sigmetfile._is_bit_set", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_sigmetfile.pyx":417
 * 
 * 
 * def _parse_ray_headers(ray_headers):             # <<<<<<<<<<<<<<
 *     """
 *     Parse the metadata from Sigmet ray headers.
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_5_parse_ray_headers(PyObject *__pyx_self, PyObject *__pyx_v_ray_headers); /*proto*/
static char __pyx_doc_5pyart_2io_11_sigmetfile_4_parse_ray_headers[] = "\n    Parse the metadata from Sigmet ray headers.\n\n    Parameters\n    ----------\n    ray_headers : array, shape=(..., 6)\n        Ray headers to parse.\n\n    Returns\n    -------\n    az0 : array\n        Azimuth angles (in degrees) at beginning of the rays.\n    el0 : array\n        Elevation angles at the beginning of the rays.\n    az1 : array\n        Azimuth angles at the end of the rays.\n    el1 : array\n        Elevation angles at the end of the rays.\n    nbins : array\n        Number of bins in the rays.\n    time : array\n        Seconds since the start of the sweep for the rays.\n    prf_flag : array\n        Numerical indication of what PRF was used, 0 for high, 1 for low.\n        Not applicable if dual-PRF is not used during collection.\n\n    ";
static PyMethodDef __pyx_mdef_5pyart_2io_11_sigmetfile_5_parse_ray_headers = {"_parse_ray_headers", (PyCFunction)__pyx_pw_5pyart_2io_11_sigmetfile_5_parse_ray_headers, METH_O, __pyx_doc_5pyart_2io_11_sigmetfile_4_parse_ray_headers};
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_5_parse_ray_headers(PyObject *__pyx_self, PyObject *__pyx_v_ray_headers) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_parse_ray_headers (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_4_parse_ray_headers(__pyx_self, ((PyObject *)__pyx_v_ray_headers));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_4_parse_ray_headers(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_ray_headers) {
  PyObject *__pyx_v_az0 = NULL;
  PyObject *__pyx_v_el0 = NULL;
  PyObject *__pyx_v_az1 = NULL;
  PyObject *__pyx_v_el1 = NULL;
  PyObject *__pyx_v_nbins = NULL;
  PyObject *__pyx_v_time = NULL;
  PyObject *__pyx_v_prf_flag = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_parse_ray_headers", 0);

  /* "pyart/io/_sigmetfile.pyx":445
 * 
 *     """
 *     az0 = bin2_to_angle(ray_headers.view('uint16')[..., 0])             # <<<<<<<<<<<<<<
 *     el0 = bin2_to_angle(ray_headers.view('uint16')[..., 1])
 *     az1 = bin2_to_angle(ray_headers.view('uint16')[..., 2])
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_bin2_to_angle); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_ray_headers, __pyx_n_s_view); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_n_u_uint16) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_n_u_uint16);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetItem(__pyx_t_3, __pyx_tuple__6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_az0 = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":446
 *     """
 *     az0 = bin2_to_angle(ray_headers.view('uint16')[..., 0])
 *     el0 = bin2_to_angle(ray_headers.view('uint16')[..., 1])             # <<<<<<<<<<<<<<
 *     az1 = bin2_to_angle(ray_headers.view('uint16')[..., 2])
 *     el1 = bin2_to_angle(ray_headers.view('uint16')[..., 3])
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_bin2_to_angle); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 446, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_ray_headers, __pyx_n_s_view); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 446, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_4 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_n_u_uint16) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_n_u_uint16);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 446, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetItem(__pyx_t_4, __pyx_tuple__7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 446, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_el0 = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":447
 *     az0 = bin2_to_angle(ray_headers.view('uint16')[..., 0])
 *     el0 = bin2_to_angle(ray_headers.view('uint16')[..., 1])
 *     az1 = bin2_to_angle(ray_headers.view('uint16')[..., 2])             # <<<<<<<<<<<<<<
 *     el1 = bin2_to_angle(ray_headers.view('uint16')[..., 3])
 *     nbins = ray_headers.view('int16')[..., 4]
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_bin2_to_angle); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_ray_headers, __pyx_n_s_view); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_n_u_uint16) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_n_u_uint16);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetItem(__pyx_t_3, __pyx_tuple__8); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_az1 = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":448
 *     el0 = bin2_to_angle(ray_headers.view('uint16')[..., 1])
 *     az1 = bin2_to_angle(ray_headers.view('uint16')[..., 2])
 *     el1 = bin2_to_angle(ray_headers.view('uint16')[..., 3])             # <<<<<<<<<<<<<<
 *     nbins = ray_headers.view('int16')[..., 4]
 *     time = ray_headers.view('uint16')[..., 5]
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_bin2_to_angle); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 448, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_ray_headers, __pyx_n_s_view); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 448, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_4 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_n_u_uint16) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_n_u_uint16);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 448, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetItem(__pyx_t_4, __pyx_tuple__9); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 448, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 448, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_el1 = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":449
 *     az1 = bin2_to_angle(ray_headers.view('uint16')[..., 2])
 *     el1 = bin2_to_angle(ray_headers.view('uint16')[..., 3])
 *     nbins = ray_headers.view('int16')[..., 4]             # <<<<<<<<<<<<<<
 *     time = ray_headers.view('uint16')[..., 5]
 *     prf_flag = np.mod(ray_headers.view('int16')[..., 0], 2)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_ray_headers, __pyx_n_s_view); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_n_u_int16) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_n_u_int16);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_t_1, __pyx_tuple__10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_nbins = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pyart/io/_sigmetfile.pyx":450
 *     el1 = bin2_to_angle(ray_headers.view('uint16')[..., 3])
 *     nbins = ray_headers.view('int16')[..., 4]
 *     time = ray_headers.view('uint16')[..., 5]             # <<<<<<<<<<<<<<
 *     prf_flag = np.mod(ray_headers.view('int16')[..., 0], 2)
 *     return (az0, el0, az1, el1, nbins, time, prf_flag)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_ray_headers, __pyx_n_s_view); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_3, __pyx_n_u_uint16) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_n_u_uint16);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_t_2, __pyx_tuple__11); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_time = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":451
 *     nbins = ray_headers.view('int16')[..., 4]
 *     time = ray_headers.view('uint16')[..., 5]
 *     prf_flag = np.mod(ray_headers.view('int16')[..., 0], 2)             # <<<<<<<<<<<<<<
 *     return (az0, el0, az1, el1, nbins, time, prf_flag)
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 451, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_mod); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 451, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_ray_headers, __pyx_n_s_view); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 451, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_n_u_int16) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_n_u_int16);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 451, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetItem(__pyx_t_2, __pyx_tuple__6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 451, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_t_4, __pyx_int_2};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 451, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_t_4, __pyx_int_2};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 451, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 451, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_2) {
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2); __pyx_t_2 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_6, __pyx_t_4);
    __Pyx_INCREF(__pyx_int_2);
    __Pyx_GIVEREF(__pyx_int_2);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_6, __pyx_int_2);
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 451, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_prf_flag = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":452
 *     time = ray_headers.view('uint16')[..., 5]
 *     prf_flag = np.mod(ray_headers.view('int16')[..., 0], 2)
 *     return (az0, el0, az1, el1, nbins, time, prf_flag)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyTuple_New(7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 452, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_az0);
  __Pyx_GIVEREF(__pyx_v_az0);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_az0);
  __Pyx_INCREF(__pyx_v_el0);
  __Pyx_GIVEREF(__pyx_v_el0);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_el0);
  __Pyx_INCREF(__pyx_v_az1);
  __Pyx_GIVEREF(__pyx_v_az1);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_az1);
  __Pyx_INCREF(__pyx_v_el1);
  __Pyx_GIVEREF(__pyx_v_el1);
  PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_v_el1);
  __Pyx_INCREF(__pyx_v_nbins);
  __Pyx_GIVEREF(__pyx_v_nbins);
  PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_v_nbins);
  __Pyx_INCREF(__pyx_v_time);
  __Pyx_GIVEREF(__pyx_v_time);
  PyTuple_SET_ITEM(__pyx_t_1, 5, __pyx_v_time);
  __Pyx_INCREF(__pyx_v_prf_flag);
  __Pyx_GIVEREF(__pyx_v_prf_flag);
  PyTuple_SET_ITEM(__pyx_t_1, 6, __pyx_v_prf_flag);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_sigmetfile.pyx":417
 * 
 * 
 * def _parse_ray_headers(ray_headers):             # <<<<<<<<<<<<<<
 *     """
 *     Parse the metadata from Sigmet ray headers.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyart.io._sigmetfile._parse_ray_headers", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_az0);
  __Pyx_XDECREF(__pyx_v_el0);
  __Pyx_XDECREF(__pyx_v_az1);
  __Pyx_XDECREF(__pyx_v_el1);
  __Pyx_XDECREF(__pyx_v_nbins);
  __Pyx_XDECREF(__pyx_v_time);
  __Pyx_XDECREF(__pyx_v_prf_flag);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_sigmetfile.pyx":596
 * # file. Rewriting the convertions/masking in Cython does not seem to improved
 * # performance likely since most of the routines are already vectorized.
 * def convert_sigmet_data(data_type, data, nbins):             # <<<<<<<<<<<<<<
 *     """ Convert sigmet data. """
 *     out = np.empty_like(data, dtype='float32')
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_7convert_sigmet_data(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5pyart_2io_11_sigmetfile_6convert_sigmet_data[] = " Convert sigmet data. ";
static PyMethodDef __pyx_mdef_5pyart_2io_11_sigmetfile_7convert_sigmet_data = {"convert_sigmet_data", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5pyart_2io_11_sigmetfile_7convert_sigmet_data, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5pyart_2io_11_sigmetfile_6convert_sigmet_data};
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_7convert_sigmet_data(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data_type = 0;
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_nbins = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("convert_sigmet_data (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data_type,&__pyx_n_s_data,&__pyx_n_s_nbins,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("convert_sigmet_data", 1, 3, 3, 1); __PYX_ERR(0, 596, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_nbins)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("convert_sigmet_data", 1, 3, 3, 2); __PYX_ERR(0, 596, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "convert_sigmet_data") < 0)) __PYX_ERR(0, 596, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_data_type = values[0];
    __pyx_v_data = values[1];
    __pyx_v_nbins = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("convert_sigmet_data", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 596, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._sigmetfile.convert_sigmet_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_6convert_sigmet_data(__pyx_self, __pyx_v_data_type, __pyx_v_data, __pyx_v_nbins);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_6convert_sigmet_data(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_data_type, PyObject *__pyx_v_data, PyObject *__pyx_v_nbins) {
  PyObject *__pyx_v_out = NULL;
  PyObject *__pyx_v_mask = NULL;
  PyObject *__pyx_v_data_type_name = NULL;
  PyObject *__pyx_v_like_dbt2 = NULL;
  PyObject *__pyx_v_like_sqi = NULL;
  PyObject *__pyx_v_like_sqi2 = NULL;
  PyObject *__pyx_v_like_dbt = NULL;
  PyObject *__pyx_v_nrays = NULL;
  PyObject *__pyx_v_nbin = NULL;
  PyObject *__pyx_v_ndata = NULL;
  PyObject *__pyx_v_exp = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("convert_sigmet_data", 0);

  /* "pyart/io/_sigmetfile.pyx":598
 * def convert_sigmet_data(data_type, data, nbins):
 *     """ Convert sigmet data. """
 *     out = np.empty_like(data, dtype='float32')             # <<<<<<<<<<<<<<
 *     mask = np.zeros_like(data, dtype=np.bool8)
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 598, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_empty_like); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 598, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 598, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_data);
  __Pyx_GIVEREF(__pyx_v_data);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_data);
  __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 598, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_n_u_float32) < 0) __PYX_ERR(0, 598, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 598, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_out = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "pyart/io/_sigmetfile.pyx":599
 *     """ Convert sigmet data. """
 *     out = np.empty_like(data, dtype='float32')
 *     mask = np.zeros_like(data, dtype=np.bool8)             # <<<<<<<<<<<<<<
 * 
 *     data_type_name = SIGMET_DATA_TYPES[data_type]
 */
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 599, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros_like); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 599, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 599, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_data);
  __Pyx_GIVEREF(__pyx_v_data);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_data);
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 599, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 599, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_bool8); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 599, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_5) < 0) __PYX_ERR(0, 599, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 599, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_mask = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "pyart/io/_sigmetfile.pyx":601
 *     mask = np.zeros_like(data, dtype=np.bool8)
 * 
 *     data_type_name = SIGMET_DATA_TYPES[data_type]             # <<<<<<<<<<<<<<
 * 
 *     like_dbt2 = [
 */
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_SIGMET_DATA_TYPES); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 601, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_t_5, __pyx_v_data_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 601, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_data_type_name = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":603
 *     data_type_name = SIGMET_DATA_TYPES[data_type]
 * 
 *     like_dbt2 = [             # <<<<<<<<<<<<<<
 *         'DBT2',     # 2-byte Reflectivity Format, section 4.3.4
 *         'DBZ2',     # " "
 */
  __pyx_t_1 = PyList_New(17); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 603, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_u_DBT2);
  __Pyx_GIVEREF(__pyx_n_u_DBT2);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_u_DBT2);
  __Pyx_INCREF(__pyx_n_u_DBZ2);
  __Pyx_GIVEREF(__pyx_n_u_DBZ2);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_u_DBZ2);
  __Pyx_INCREF(__pyx_n_u_KDP2);
  __Pyx_GIVEREF(__pyx_n_u_KDP2);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_u_KDP2);
  __Pyx_INCREF(__pyx_n_u_LDRH2);
  __Pyx_GIVEREF(__pyx_n_u_LDRH2);
  PyList_SET_ITEM(__pyx_t_1, 3, __pyx_n_u_LDRH2);
  __Pyx_INCREF(__pyx_n_u_LDRV2);
  __Pyx_GIVEREF(__pyx_n_u_LDRV2);
  PyList_SET_ITEM(__pyx_t_1, 4, __pyx_n_u_LDRV2);
  __Pyx_INCREF(__pyx_n_u_VEL2);
  __Pyx_GIVEREF(__pyx_n_u_VEL2);
  PyList_SET_ITEM(__pyx_t_1, 5, __pyx_n_u_VEL2);
  __Pyx_INCREF(__pyx_n_u_VELC2);
  __Pyx_GIVEREF(__pyx_n_u_VELC2);
  PyList_SET_ITEM(__pyx_t_1, 6, __pyx_n_u_VELC2);
  __Pyx_INCREF(__pyx_n_u_ZDR2);
  __Pyx_GIVEREF(__pyx_n_u_ZDR2);
  PyList_SET_ITEM(__pyx_t_1, 7, __pyx_n_u_ZDR2);
  __Pyx_INCREF(__pyx_n_u_DBZC2);
  __Pyx_GIVEREF(__pyx_n_u_DBZC2);
  PyList_SET_ITEM(__pyx_t_1, 8, __pyx_n_u_DBZC2);
  __Pyx_INCREF(__pyx_n_u_ZDRC2);
  __Pyx_GIVEREF(__pyx_n_u_ZDRC2);
  PyList_SET_ITEM(__pyx_t_1, 9, __pyx_n_u_ZDRC2);
  __Pyx_INCREF(__pyx_n_u_DBTV16);
  __Pyx_GIVEREF(__pyx_n_u_DBTV16);
  PyList_SET_ITEM(__pyx_t_1, 10, __pyx_n_u_DBTV16);
  __Pyx_INCREF(__pyx_n_u_DBZV16);
  __Pyx_GIVEREF(__pyx_n_u_DBZV16);
  PyList_SET_ITEM(__pyx_t_1, 11, __pyx_n_u_DBZV16);
  __Pyx_INCREF(__pyx_n_u_SNR16);
  __Pyx_GIVEREF(__pyx_n_u_SNR16);
  PyList_SET_ITEM(__pyx_t_1, 12, __pyx_n_u_SNR16);
  __Pyx_INCREF(__pyx_n_u_DBTE16);
  __Pyx_GIVEREF(__pyx_n_u_DBTE16);
  PyList_SET_ITEM(__pyx_t_1, 13, __pyx_n_u_DBTE16);
  __Pyx_INCREF(__pyx_n_u_DBZE16);
  __Pyx_GIVEREF(__pyx_n_u_DBZE16);
  PyList_SET_ITEM(__pyx_t_1, 14, __pyx_n_u_DBZE16);
  __Pyx_INCREF(__pyx_n_u_LOG16);
  __Pyx_GIVEREF(__pyx_n_u_LOG16);
  PyList_SET_ITEM(__pyx_t_1, 15, __pyx_n_u_LOG16);
  __Pyx_INCREF(__pyx_n_u_CSP16);
  __Pyx_GIVEREF(__pyx_n_u_CSP16);
  PyList_SET_ITEM(__pyx_t_1, 16, __pyx_n_u_CSP16);
  __pyx_v_like_dbt2 = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":623
 *     ]
 * 
 *     like_sqi = [             # <<<<<<<<<<<<<<
 *         'RHOH',     # 1-byte Rho Format, section 4.3.21
 *         'RHOV',     # " "
 */
  __pyx_t_1 = PyList_New(5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 623, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_u_RHOH);
  __Pyx_GIVEREF(__pyx_n_u_RHOH);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_u_RHOH);
  __Pyx_INCREF(__pyx_n_u_RHOV);
  __Pyx_GIVEREF(__pyx_n_u_RHOV);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_u_RHOV);
  __Pyx_INCREF(__pyx_n_u_RHOHV);
  __Pyx_GIVEREF(__pyx_n_u_RHOHV);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_u_RHOHV);
  __Pyx_INCREF(__pyx_n_u_SQI);
  __Pyx_GIVEREF(__pyx_n_u_SQI);
  PyList_SET_ITEM(__pyx_t_1, 3, __pyx_n_u_SQI);
  __Pyx_INCREF(__pyx_n_u_PMI8);
  __Pyx_GIVEREF(__pyx_n_u_PMI8);
  PyList_SET_ITEM(__pyx_t_1, 4, __pyx_n_u_PMI8);
  __pyx_v_like_sqi = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":631
 *     ]
 * 
 *     like_sqi2 = [             # <<<<<<<<<<<<<<
 *         'RHOV2',    # 2-byte Rho Format, section 4.3.22
 *         'RHOH2',    # " "
 */
  __pyx_t_1 = PyList_New(5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 631, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_u_RHOV2);
  __Pyx_GIVEREF(__pyx_n_u_RHOV2);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_u_RHOV2);
  __Pyx_INCREF(__pyx_n_u_RHOH2);
  __Pyx_GIVEREF(__pyx_n_u_RHOH2);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_u_RHOH2);
  __Pyx_INCREF(__pyx_n_u_RHOHV2);
  __Pyx_GIVEREF(__pyx_n_u_RHOHV2);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_u_RHOHV2);
  __Pyx_INCREF(__pyx_n_u_SQI2);
  __Pyx_GIVEREF(__pyx_n_u_SQI2);
  PyList_SET_ITEM(__pyx_t_1, 3, __pyx_n_u_SQI2);
  __Pyx_INCREF(__pyx_n_u_PMI16);
  __Pyx_GIVEREF(__pyx_n_u_PMI16);
  PyList_SET_ITEM(__pyx_t_1, 4, __pyx_n_u_PMI16);
  __pyx_v_like_sqi2 = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":639
 *     ]
 * 
 *     like_dbt = [             # <<<<<<<<<<<<<<
 *         'DBT',      # 1-bytes Reflectivity Format, section 4.3.3
 *         'DBZ',      # " "
 */
  __pyx_t_1 = PyList_New(9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 639, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_u_DBT);
  __Pyx_GIVEREF(__pyx_n_u_DBT);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_u_DBT);
  __Pyx_INCREF(__pyx_n_u_DBZ);
  __Pyx_GIVEREF(__pyx_n_u_DBZ);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_u_DBZ);
  __Pyx_INCREF(__pyx_n_u_DBTV8);
  __Pyx_GIVEREF(__pyx_n_u_DBTV8);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_u_DBTV8);
  __Pyx_INCREF(__pyx_n_u_DBZV8);
  __Pyx_GIVEREF(__pyx_n_u_DBZV8);
  PyList_SET_ITEM(__pyx_t_1, 3, __pyx_n_u_DBZV8);
  __Pyx_INCREF(__pyx_n_u_SNR8);
  __Pyx_GIVEREF(__pyx_n_u_SNR8);
  PyList_SET_ITEM(__pyx_t_1, 4, __pyx_n_u_SNR8);
  __Pyx_INCREF(__pyx_n_u_DBTE8);
  __Pyx_GIVEREF(__pyx_n_u_DBTE8);
  PyList_SET_ITEM(__pyx_t_1, 5, __pyx_n_u_DBTE8);
  __Pyx_INCREF(__pyx_n_u_DBZE8);
  __Pyx_GIVEREF(__pyx_n_u_DBZE8);
  PyList_SET_ITEM(__pyx_t_1, 6, __pyx_n_u_DBZE8);
  __Pyx_INCREF(__pyx_n_u_LOG8);
  __Pyx_GIVEREF(__pyx_n_u_LOG8);
  PyList_SET_ITEM(__pyx_t_1, 7, __pyx_n_u_LOG8);
  __Pyx_INCREF(__pyx_n_u_CSP8);
  __Pyx_GIVEREF(__pyx_n_u_CSP8);
  PyList_SET_ITEM(__pyx_t_1, 8, __pyx_n_u_CSP8);
  __pyx_v_like_dbt = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":651
 *     ]
 * 
 *     if data_type_name in like_dbt2:             # <<<<<<<<<<<<<<
 *         # value = (N - 32768) / 100.
 *         # 0 : no data available (mask)
 */
  __pyx_t_6 = (__Pyx_PySequence_ContainsTF(__pyx_v_data_type_name, __pyx_v_like_dbt2, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 651, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  if (__pyx_t_7) {

    /* "pyart/io/_sigmetfile.pyx":655
 *         # 0 : no data available (mask)
 *         # 65535 Reserved for area not scanned in product file (nothing)
 *         out[:] = (data.view('uint16') - 32768.) / 100.             # <<<<<<<<<<<<<<
 *         mask[data.view('uint16') == 0] = True
 * 
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_data, __pyx_n_s_view); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 655, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_4, __pyx_n_u_uint16) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_n_u_uint16);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 655, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyFloat_SubtractObjC(__pyx_t_1, __pyx_float_32768_, 32768., 0, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 655, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_5, __pyx_float_100_, 100., 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 655, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (__Pyx_PyObject_SetSlice(__pyx_v_out, __pyx_t_1, 0, 0, NULL, NULL, &__pyx_slice__12, 0, 0, 1) < 0) __PYX_ERR(0, 655, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyart/io/_sigmetfile.pyx":656
 *         # 65535 Reserved for area not scanned in product file (nothing)
 *         out[:] = (data.view('uint16') - 32768.) / 100.
 *         mask[data.view('uint16') == 0] = True             # <<<<<<<<<<<<<<
 * 
 *     elif data_type_name in like_sqi2:
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_data, __pyx_n_s_view); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 656, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_4, __pyx_n_u_uint16) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_n_u_uint16);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 656, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyInt_EqObjC(__pyx_t_1, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 656, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(PyObject_SetItem(__pyx_v_mask, __pyx_t_5, Py_True) < 0)) __PYX_ERR(0, 656, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "pyart/io/_sigmetfile.pyx":651
 *     ]
 * 
 *     if data_type_name in like_dbt2:             # <<<<<<<<<<<<<<
 *         # value = (N - 32768) / 100.
 *         # 0 : no data available (mask)
 */
    goto __pyx_L3;
  }

  /* "pyart/io/_sigmetfile.pyx":658
 *         mask[data.view('uint16') == 0] = True
 * 
 *     elif data_type_name in like_sqi2:             # <<<<<<<<<<<<<<
 *         # value = (N - 1) / 65533
 *         # 0 : no data available (mask)
 */
  __pyx_t_7 = (__Pyx_PySequence_ContainsTF(__pyx_v_data_type_name, __pyx_v_like_sqi2, Py_EQ)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 658, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_7 != 0);
  if (__pyx_t_6) {

    /* "pyart/io/_sigmetfile.pyx":662
 *         # 0 : no data available (mask)
 *         # 65535 Area not scanned
 *         out[:] = (data.view('uint16') - 1.) / 65533.             # <<<<<<<<<<<<<<
 *         mask[data.view('uint16') == 0] = True
 * 
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_data, __pyx_n_s_view); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 662, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_5 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_4, __pyx_n_u_uint16) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_n_u_uint16);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 662, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyFloat_SubtractObjC(__pyx_t_5, __pyx_float_1_, 1., 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 662, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_1, __pyx_float_65533_, 65533., 0, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 662, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__Pyx_PyObject_SetSlice(__pyx_v_out, __pyx_t_5, 0, 0, NULL, NULL, &__pyx_slice__12, 0, 0, 1) < 0) __PYX_ERR(0, 662, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "pyart/io/_sigmetfile.pyx":663
 *         # 65535 Area not scanned
 *         out[:] = (data.view('uint16') - 1.) / 65533.
 *         mask[data.view('uint16') == 0] = True             # <<<<<<<<<<<<<<
 * 
 *     elif data_type_name == 'WIDTH2':
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_data, __pyx_n_s_view); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 663, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_5 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_4, __pyx_n_u_uint16) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_n_u_uint16);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 663, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_t_5, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 663, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(PyObject_SetItem(__pyx_v_mask, __pyx_t_1, Py_True) < 0)) __PYX_ERR(0, 663, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyart/io/_sigmetfile.pyx":658
 *         mask[data.view('uint16') == 0] = True
 * 
 *     elif data_type_name in like_sqi2:             # <<<<<<<<<<<<<<
 *         # value = (N - 1) / 65533
 *         # 0 : no data available (mask)
 */
    goto __pyx_L3;
  }

  /* "pyart/io/_sigmetfile.pyx":665
 *         mask[data.view('uint16') == 0] = True
 * 
 *     elif data_type_name == 'WIDTH2':             # <<<<<<<<<<<<<<
 *         # DB_WIDTH2, 11, Width (2 byte)
 *         # 2-byte Width Format, section 4.3.36
 */
  __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_v_data_type_name, __pyx_n_u_WIDTH2, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 665, __pyx_L1_error)
  if (__pyx_t_6) {

    /* "pyart/io/_sigmetfile.pyx":668
 *         # DB_WIDTH2, 11, Width (2 byte)
 *         # 2-byte Width Format, section 4.3.36
 *         out[:] = data.view('uint16') / 100.             # <<<<<<<<<<<<<<
 *         mask[data.view('uint16') == 0] = True
 * 
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_data, __pyx_n_s_view); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 668, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_4, __pyx_n_u_uint16) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_n_u_uint16);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 668, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_1, __pyx_float_100_, 100., 0, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 668, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__Pyx_PyObject_SetSlice(__pyx_v_out, __pyx_t_5, 0, 0, NULL, NULL, &__pyx_slice__12, 0, 0, 1) < 0) __PYX_ERR(0, 668, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "pyart/io/_sigmetfile.pyx":669
 *         # 2-byte Width Format, section 4.3.36
 *         out[:] = data.view('uint16') / 100.
 *         mask[data.view('uint16') == 0] = True             # <<<<<<<<<<<<<<
 * 
 *     elif data_type_name == 'PHIDP2':
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_data, __pyx_n_s_view); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 669, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_5 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_4, __pyx_n_u_uint16) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_n_u_uint16);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 669, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_t_5, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 669, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(PyObject_SetItem(__pyx_v_mask, __pyx_t_1, Py_True) < 0)) __PYX_ERR(0, 669, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyart/io/_sigmetfile.pyx":665
 *         mask[data.view('uint16') == 0] = True
 * 
 *     elif data_type_name == 'WIDTH2':             # <<<<<<<<<<<<<<
 *         # DB_WIDTH2, 11, Width (2 byte)
 *         # 2-byte Width Format, section 4.3.36
 */
    goto __pyx_L3;
  }

  /* "pyart/io/_sigmetfile.pyx":671
 *         mask[data.view('uint16') == 0] = True
 * 
 *     elif data_type_name == 'PHIDP2':             # <<<<<<<<<<<<<<
 *         # DB_PHIDP2, 24, PhiDP (Differential Phase) (2 byte)
 *         # 2-byte PhiDP format, section 4.3.19
 */
  __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_v_data_type_name, __pyx_n_u_PHIDP2, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 671, __pyx_L1_error)
  if (__pyx_t_6) {

    /* "pyart/io/_sigmetfile.pyx":674
 *         # DB_PHIDP2, 24, PhiDP (Differential Phase) (2 byte)
 *         # 2-byte PhiDP format, section 4.3.19
 *         out[:] = 360. * (data.view('uint16') - 1.) / 65534.             # <<<<<<<<<<<<<<
 *         mask[data.view('uint16') == 0] = True
 * 
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_data, __pyx_n_s_view); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 674, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_4, __pyx_n_u_uint16) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_n_u_uint16);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 674, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyFloat_SubtractObjC(__pyx_t_1, __pyx_float_1_, 1., 0, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 674, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyNumber_Multiply(__pyx_float_360_, __pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 674, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_1, __pyx_float_65534_, 65534., 0, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 674, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__Pyx_PyObject_SetSlice(__pyx_v_out, __pyx_t_5, 0, 0, NULL, NULL, &__pyx_slice__12, 0, 0, 1) < 0) __PYX_ERR(0, 674, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "pyart/io/_sigmetfile.pyx":675
 *         # 2-byte PhiDP format, section 4.3.19
 *         out[:] = 360. * (data.view('uint16') - 1.) / 65534.
 *         mask[data.view('uint16') == 0] = True             # <<<<<<<<<<<<<<
 * 
 *     elif data_type_name == 'HCLASS2':
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_data, __pyx_n_s_view); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 675, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_5 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_4, __pyx_n_u_uint16) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_n_u_uint16);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 675, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_t_5, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 675, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(PyObject_SetItem(__pyx_v_mask, __pyx_t_1, Py_True) < 0)) __PYX_ERR(0, 675, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyart/io/_sigmetfile.pyx":671
 *         mask[data.view('uint16') == 0] = True
 * 
 *     elif data_type_name == 'PHIDP2':             # <<<<<<<<<<<<<<
 *         # DB_PHIDP2, 24, PhiDP (Differential Phase) (2 byte)
 *         # 2-byte PhiDP format, section 4.3.19
 */
    goto __pyx_L3;
  }

  /* "pyart/io/_sigmetfile.pyx":677
 *         mask[data.view('uint16') == 0] = True
 * 
 *     elif data_type_name == 'HCLASS2':             # <<<<<<<<<<<<<<
 *         # DB_HCLASS2, 56, Hydrometeor class (2 byte)
 *         # 2-byte HydroClass Format, section 4.3.9
 */
  __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_v_data_type_name, __pyx_n_u_HCLASS2, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 677, __pyx_L1_error)
  if (__pyx_t_6) {

    /* "pyart/io/_sigmetfile.pyx":680
 *         # DB_HCLASS2, 56, Hydrometeor class (2 byte)
 *         # 2-byte HydroClass Format, section 4.3.9
 *         out[:] = data.view('uint16')             # <<<<<<<<<<<<<<
 * 
 *     elif data_type_name == 'XHDR':
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_data, __pyx_n_s_view); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 680, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_4, __pyx_n_u_uint16) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_n_u_uint16);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 680, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (__Pyx_PyObject_SetSlice(__pyx_v_out, __pyx_t_1, 0, 0, NULL, NULL, &__pyx_slice__12, 0, 0, 1) < 0) __PYX_ERR(0, 680, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyart/io/_sigmetfile.pyx":677
 *         mask[data.view('uint16') == 0] = True
 * 
 *     elif data_type_name == 'HCLASS2':             # <<<<<<<<<<<<<<
 *         # DB_HCLASS2, 56, Hydrometeor class (2 byte)
 *         # 2-byte HydroClass Format, section 4.3.9
 */
    goto __pyx_L3;
  }

  /* "pyart/io/_sigmetfile.pyx":682
 *         out[:] = data.view('uint16')
 * 
 *     elif data_type_name == 'XHDR':             # <<<<<<<<<<<<<<
 *         # Extended Headers, 0
 *         # extended_header_v0, _v1, _v2, section 4.2.8-4.2.10
 */
  __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_v_data_type_name, __pyx_n_u_XHDR, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 682, __pyx_L1_error)
  if (__pyx_t_6) {

    /* "pyart/io/_sigmetfile.pyx":686
 *         # extended_header_v0, _v1, _v2, section 4.2.8-4.2.10
 *         # Here we return an array with the times in milliseconds.
 *         return data[..., :2].copy().view('i4')             # <<<<<<<<<<<<<<
 * 
 *     # one byte data types
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyObject_GetItem(__pyx_v_data, __pyx_tuple__14); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 686, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_copy); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 686, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_5 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 686, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_view); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 686, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_n_u_i4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_n_u_i4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 686, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "pyart/io/_sigmetfile.pyx":682
 *         out[:] = data.view('uint16')
 * 
 *     elif data_type_name == 'XHDR':             # <<<<<<<<<<<<<<
 *         # Extended Headers, 0
 *         # extended_header_v0, _v1, _v2, section 4.2.8-4.2.10
 */
  }

  /* "pyart/io/_sigmetfile.pyx":689
 * 
 *     # one byte data types
 *     elif data_type_name[-1] != '2':             # <<<<<<<<<<<<<<
 *         # make a view of left half of the data as uint8,
 *         # this is the actual ray data collected, the right half is blank.
 */
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_data_type_name, -1L, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 689, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_kp_u_2, Py_NE)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 689, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_6) {

    /* "pyart/io/_sigmetfile.pyx":692
 *         # make a view of left half of the data as uint8,
 *         # this is the actual ray data collected, the right half is blank.
 *         nrays, nbin = data.shape             # <<<<<<<<<<<<<<
 *         ndata = data.view('(2,) uint8').reshape(nrays, -1)[:, :nbin]
 * 
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_data, __pyx_n_s_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 692, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
      PyObject* sequence = __pyx_t_1;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 692, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_5 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_3 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_5 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      #else
      __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 692, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 692, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_4 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 692, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_8 = Py_TYPE(__pyx_t_4)->tp_iternext;
      index = 0; __pyx_t_3 = __pyx_t_8(__pyx_t_4); if (unlikely(!__pyx_t_3)) goto __pyx_L4_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_3);
      index = 1; __pyx_t_5 = __pyx_t_8(__pyx_t_4); if (unlikely(!__pyx_t_5)) goto __pyx_L4_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_5);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_8(__pyx_t_4), 2) < 0) __PYX_ERR(0, 692, __pyx_L1_error)
      __pyx_t_8 = NULL;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      goto __pyx_L5_unpacking_done;
      __pyx_L4_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_8 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 692, __pyx_L1_error)
      __pyx_L5_unpacking_done:;
    }
    __pyx_v_nrays = __pyx_t_3;
    __pyx_t_3 = 0;
    __pyx_v_nbin = __pyx_t_5;
    __pyx_t_5 = 0;

    /* "pyart/io/_sigmetfile.pyx":693
 *         # this is the actual ray data collected, the right half is blank.
 *         nrays, nbin = data.shape
 *         ndata = data.view('(2,) uint8').reshape(nrays, -1)[:, :nbin]             # <<<<<<<<<<<<<<
 * 
 *         if data_type_name in like_dbt:
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_data, __pyx_n_s_view); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 693, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_5 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_kp_u_2_uint8) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_u_2_uint8);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 693, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_reshape); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 693, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    __pyx_t_9 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_9 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_nrays, __pyx_int_neg_1};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 693, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_nrays, __pyx_int_neg_1};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 693, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 693, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_INCREF(__pyx_v_nrays);
      __Pyx_GIVEREF(__pyx_v_nrays);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_9, __pyx_v_nrays);
      __Pyx_INCREF(__pyx_int_neg_1);
      __Pyx_GIVEREF(__pyx_int_neg_1);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_9, __pyx_int_neg_1);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 693, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PySlice_New(Py_None, __pyx_v_nbin, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 693, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 693, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_slice__12);
    __Pyx_GIVEREF(__pyx_slice__12);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_slice__12);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_GetItem(__pyx_t_1, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 693, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_ndata = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "pyart/io/_sigmetfile.pyx":695
 *         ndata = data.view('(2,) uint8').reshape(nrays, -1)[:, :nbin]
 * 
 *         if data_type_name in like_dbt:             # <<<<<<<<<<<<<<
 *             # DB_DBT, 1, Total Power (1 byte)
 *             # 1-byte Reflectivity Format, section 4.3.3
 */
    __pyx_t_6 = (__Pyx_PySequence_ContainsTF(__pyx_v_data_type_name, __pyx_v_like_dbt, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 695, __pyx_L1_error)
    __pyx_t_7 = (__pyx_t_6 != 0);
    if (__pyx_t_7) {

      /* "pyart/io/_sigmetfile.pyx":698
 *             # DB_DBT, 1, Total Power (1 byte)
 *             # 1-byte Reflectivity Format, section 4.3.3
 *             out[:] = (ndata - 64.) / 2.             # <<<<<<<<<<<<<<
 *             mask[ndata == 0] = True
 * 
 */
      __pyx_t_3 = __Pyx_PyFloat_SubtractObjC(__pyx_v_ndata, __pyx_float_64_, 64., 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 698, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_3, __pyx_float_2_, 2., 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 698, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__Pyx_PyObject_SetSlice(__pyx_v_out, __pyx_t_4, 0, 0, NULL, NULL, &__pyx_slice__12, 0, 0, 1) < 0) __PYX_ERR(0, 698, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyart/io/_sigmetfile.pyx":699
 *             # 1-byte Reflectivity Format, section 4.3.3
 *             out[:] = (ndata - 64.) / 2.
 *             mask[ndata == 0] = True             # <<<<<<<<<<<<<<
 * 
 *         elif data_type_name in like_sqi:
 */
      __pyx_t_4 = __Pyx_PyInt_EqObjC(__pyx_v_ndata, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 699, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (unlikely(PyObject_SetItem(__pyx_v_mask, __pyx_t_4, Py_True) < 0)) __PYX_ERR(0, 699, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyart/io/_sigmetfile.pyx":695
 *         ndata = data.view('(2,) uint8').reshape(nrays, -1)[:, :nbin]
 * 
 *         if data_type_name in like_dbt:             # <<<<<<<<<<<<<<
 *             # DB_DBT, 1, Total Power (1 byte)
 *             # 1-byte Reflectivity Format, section 4.3.3
 */
      goto __pyx_L6;
    }

    /* "pyart/io/_sigmetfile.pyx":701
 *             mask[ndata == 0] = True
 * 
 *         elif data_type_name in like_sqi:             # <<<<<<<<<<<<<<
 *             # value = sqrt((N - 1) / 253)
 *             # 0 : no data available (mask)
 */
    __pyx_t_7 = (__Pyx_PySequence_ContainsTF(__pyx_v_data_type_name, __pyx_v_like_sqi, Py_EQ)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 701, __pyx_L1_error)
    __pyx_t_6 = (__pyx_t_7 != 0);
    if (__pyx_t_6) {

      /* "pyart/io/_sigmetfile.pyx":705
 *             # 0 : no data available (mask)
 *             # 255 Area not scanned
 *             out[:] = np.sqrt((ndata - 1.) / 253.)             # <<<<<<<<<<<<<<
 *             mask[ndata == 0] = True
 *             mask[ndata == 255] = True
 */
      __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 705, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_sqrt); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 705, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = __Pyx_PyFloat_SubtractObjC(__pyx_v_ndata, __pyx_float_1_, 1., 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 705, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_3, __pyx_float_253_, 253., 0, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 705, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
        }
      }
      __pyx_t_4 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_3, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_5);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 705, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (__Pyx_PyObject_SetSlice(__pyx_v_out, __pyx_t_4, 0, 0, NULL, NULL, &__pyx_slice__12, 0, 0, 1) < 0) __PYX_ERR(0, 705, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyart/io/_sigmetfile.pyx":706
 *             # 255 Area not scanned
 *             out[:] = np.sqrt((ndata - 1.) / 253.)
 *             mask[ndata == 0] = True             # <<<<<<<<<<<<<<
 *             mask[ndata == 255] = True
 * 
 */
      __pyx_t_4 = __Pyx_PyInt_EqObjC(__pyx_v_ndata, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 706, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (unlikely(PyObject_SetItem(__pyx_v_mask, __pyx_t_4, Py_True) < 0)) __PYX_ERR(0, 706, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyart/io/_sigmetfile.pyx":707
 *             out[:] = np.sqrt((ndata - 1.) / 253.)
 *             mask[ndata == 0] = True
 *             mask[ndata == 255] = True             # <<<<<<<<<<<<<<
 * 
 *         elif data_type_name == 'VEL':
 */
      __pyx_t_4 = __Pyx_PyInt_EqObjC(__pyx_v_ndata, __pyx_int_255, 0xFF, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 707, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (unlikely(PyObject_SetItem(__pyx_v_mask, __pyx_t_4, Py_True) < 0)) __PYX_ERR(0, 707, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyart/io/_sigmetfile.pyx":701
 *             mask[ndata == 0] = True
 * 
 *         elif data_type_name in like_sqi:             # <<<<<<<<<<<<<<
 *             # value = sqrt((N - 1) / 253)
 *             # 0 : no data available (mask)
 */
      goto __pyx_L6;
    }

    /* "pyart/io/_sigmetfile.pyx":709
 *             mask[ndata == 255] = True
 * 
 *         elif data_type_name == 'VEL':             # <<<<<<<<<<<<<<
 *             # VEL, 3, Velocity (1 byte)
 *             # 1-byte Velocity Format, section 4.3.29
 */
    __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_v_data_type_name, __pyx_n_u_VEL, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 709, __pyx_L1_error)
    if (__pyx_t_6) {

      /* "pyart/io/_sigmetfile.pyx":714
 *             # Note that this data should be multiplied by Nyquist,
 *             # this is done in the get_data method of the SigmetFile class.
 *             out[:] = (ndata - 128.) / 127.             # <<<<<<<<<<<<<<
 *             mask[ndata == 0] = True
 * 
 */
      __pyx_t_4 = __Pyx_PyFloat_SubtractObjC(__pyx_v_ndata, __pyx_float_128_, 128., 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 714, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_1 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_4, __pyx_float_127_, 127., 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 714, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__Pyx_PyObject_SetSlice(__pyx_v_out, __pyx_t_1, 0, 0, NULL, NULL, &__pyx_slice__12, 0, 0, 1) < 0) __PYX_ERR(0, 714, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "pyart/io/_sigmetfile.pyx":715
 *             # this is done in the get_data method of the SigmetFile class.
 *             out[:] = (ndata - 128.) / 127.
 *             mask[ndata == 0] = True             # <<<<<<<<<<<<<<
 * 
 *         elif data_type_name == 'WIDTH':
 */
      __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_ndata, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 715, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (unlikely(PyObject_SetItem(__pyx_v_mask, __pyx_t_1, Py_True) < 0)) __PYX_ERR(0, 715, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "pyart/io/_sigmetfile.pyx":709
 *             mask[ndata == 255] = True
 * 
 *         elif data_type_name == 'VEL':             # <<<<<<<<<<<<<<
 *             # VEL, 3, Velocity (1 byte)
 *             # 1-byte Velocity Format, section 4.3.29
 */
      goto __pyx_L6;
    }

    /* "pyart/io/_sigmetfile.pyx":717
 *             mask[ndata == 0] = True
 * 
 *         elif data_type_name == 'WIDTH':             # <<<<<<<<<<<<<<
 *             # WIDTH, 4, Width (1 byte)
 *             # 1-byte Width format, section 4.3.25
 */
    __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_v_data_type_name, __pyx_n_u_WIDTH, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 717, __pyx_L1_error)
    if (__pyx_t_6) {

      /* "pyart/io/_sigmetfile.pyx":722
 *             # Note that this data should be multiplied by the unambiguous
 *             # velocity
 *             out[:] = ndata / 256.             # <<<<<<<<<<<<<<
 *             mask[ndata == 0] = True
 * 
 */
      __pyx_t_1 = __Pyx_PyFloat_TrueDivideObjC(__pyx_v_ndata, __pyx_float_256_, 256., 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 722, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__Pyx_PyObject_SetSlice(__pyx_v_out, __pyx_t_1, 0, 0, NULL, NULL, &__pyx_slice__12, 0, 0, 1) < 0) __PYX_ERR(0, 722, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "pyart/io/_sigmetfile.pyx":723
 *             # velocity
 *             out[:] = ndata / 256.
 *             mask[ndata == 0] = True             # <<<<<<<<<<<<<<
 * 
 *         elif data_type_name == 'ZDR':
 */
      __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_ndata, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 723, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (unlikely(PyObject_SetItem(__pyx_v_mask, __pyx_t_1, Py_True) < 0)) __PYX_ERR(0, 723, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "pyart/io/_sigmetfile.pyx":717
 *             mask[ndata == 0] = True
 * 
 *         elif data_type_name == 'WIDTH':             # <<<<<<<<<<<<<<
 *             # WIDTH, 4, Width (1 byte)
 *             # 1-byte Width format, section 4.3.25
 */
      goto __pyx_L6;
    }

    /* "pyart/io/_sigmetfile.pyx":725
 *             mask[ndata == 0] = True
 * 
 *         elif data_type_name == 'ZDR':             # <<<<<<<<<<<<<<
 *             # ZDR, 5, Differential reflectivity (1 byte)
 *             # 1-byte ZDR format, section 4.3.37
 */
    __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_v_data_type_name, __pyx_n_u_ZDR, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 725, __pyx_L1_error)
    if (__pyx_t_6) {

      /* "pyart/io/_sigmetfile.pyx":728
 *             # ZDR, 5, Differential reflectivity (1 byte)
 *             # 1-byte ZDR format, section 4.3.37
 *             out[:] = (ndata - 128.) / 16.             # <<<<<<<<<<<<<<
 *             mask[ndata == 0] = True
 * 
 */
      __pyx_t_1 = __Pyx_PyFloat_SubtractObjC(__pyx_v_ndata, __pyx_float_128_, 128., 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 728, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_4 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_1, __pyx_float_16_, 16., 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 728, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (__Pyx_PyObject_SetSlice(__pyx_v_out, __pyx_t_4, 0, 0, NULL, NULL, &__pyx_slice__12, 0, 0, 1) < 0) __PYX_ERR(0, 728, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyart/io/_sigmetfile.pyx":729
 *             # 1-byte ZDR format, section 4.3.37
 *             out[:] = (ndata - 128.) / 16.
 *             mask[ndata == 0] = True             # <<<<<<<<<<<<<<
 * 
 *         elif data_type_name == 'KDP':
 */
      __pyx_t_4 = __Pyx_PyInt_EqObjC(__pyx_v_ndata, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 729, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (unlikely(PyObject_SetItem(__pyx_v_mask, __pyx_t_4, Py_True) < 0)) __PYX_ERR(0, 729, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyart/io/_sigmetfile.pyx":725
 *             mask[ndata == 0] = True
 * 
 *         elif data_type_name == 'ZDR':             # <<<<<<<<<<<<<<
 *             # ZDR, 5, Differential reflectivity (1 byte)
 *             # 1-byte ZDR format, section 4.3.37
 */
      goto __pyx_L6;
    }

    /* "pyart/io/_sigmetfile.pyx":731
 *             mask[ndata == 0] = True
 * 
 *         elif data_type_name == 'KDP':             # <<<<<<<<<<<<<<
 *             # KDP, 14, KDP (Differential phase) (1 byte)
 *             # 1-byte KDP format, section 4.3.12
 */
    __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_v_data_type_name, __pyx_n_u_KDP, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 731, __pyx_L1_error)
    if (__pyx_t_6) {

      /* "pyart/io/_sigmetfile.pyx":738
 * 
 *             # above 128 use positive value equation
 *             exp = np.power(600., (ndata[ndata > 128] - 129.) / 126.)             # <<<<<<<<<<<<<<
 *             out[ndata > 128] = 0.25 * exp
 *             # below 128, use negative value equation
 */
      __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 738, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_power); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 738, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = PyObject_RichCompare(__pyx_v_ndata, __pyx_int_128, Py_GT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 738, __pyx_L1_error)
      __pyx_t_3 = __Pyx_PyObject_GetItem(__pyx_v_ndata, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 738, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyFloat_SubtractObjC(__pyx_t_3, __pyx_float_129_, 129., 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 738, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_1, __pyx_float_126_, 126., 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 738, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = NULL;
      __pyx_t_9 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_1)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
          __pyx_t_9 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_float_600_, __pyx_t_3};
        __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 738, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_float_600_, __pyx_t_3};
        __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 738, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else
      #endif
      {
        __pyx_t_2 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 738, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (__pyx_t_1) {
          __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1); __pyx_t_1 = NULL;
        }
        __Pyx_INCREF(__pyx_float_600_);
        __Pyx_GIVEREF(__pyx_float_600_);
        PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_9, __pyx_float_600_);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_9, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_2, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 738, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_v_exp = __pyx_t_4;
      __pyx_t_4 = 0;

      /* "pyart/io/_sigmetfile.pyx":739
 *             # above 128 use positive value equation
 *             exp = np.power(600., (ndata[ndata > 128] - 129.) / 126.)
 *             out[ndata > 128] = 0.25 * exp             # <<<<<<<<<<<<<<
 *             # below 128, use negative value equation
 *             exp = np.power(600., (127. - ndata[ndata < 128]) / 126.)
 */
      __pyx_t_4 = PyNumber_Multiply(__pyx_float_0_25, __pyx_v_exp); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 739, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = PyObject_RichCompare(__pyx_v_ndata, __pyx_int_128, Py_GT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 739, __pyx_L1_error)
      if (unlikely(PyObject_SetItem(__pyx_v_out, __pyx_t_5, __pyx_t_4) < 0)) __PYX_ERR(0, 739, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyart/io/_sigmetfile.pyx":741
 *             out[ndata > 128] = 0.25 * exp
 *             # below 128, use negative value equation
 *             exp = np.power(600., (127. - ndata[ndata < 128]) / 126.)             # <<<<<<<<<<<<<<
 *             out[ndata < 128] = -0.25 * exp
 *             # equal to 128, zero
 */
      __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 741, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_power); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 741, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = PyObject_RichCompare(__pyx_v_ndata, __pyx_int_128, Py_LT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 741, __pyx_L1_error)
      __pyx_t_3 = __Pyx_PyObject_GetItem(__pyx_v_ndata, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 741, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyFloat_SubtractCObj(__pyx_float_127_, __pyx_t_3, 127., 0, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 741, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_5, __pyx_float_126_, 126., 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 741, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = NULL;
      __pyx_t_9 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
          __pyx_t_9 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_float_600_, __pyx_t_3};
        __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 741, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_float_600_, __pyx_t_3};
        __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 741, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else
      #endif
      {
        __pyx_t_1 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 741, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        if (__pyx_t_5) {
          __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_5); __pyx_t_5 = NULL;
        }
        __Pyx_INCREF(__pyx_float_600_);
        __Pyx_GIVEREF(__pyx_float_600_);
        PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_9, __pyx_float_600_);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_9, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 741, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF_SET(__pyx_v_exp, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "pyart/io/_sigmetfile.pyx":742
 *             # below 128, use negative value equation
 *             exp = np.power(600., (127. - ndata[ndata < 128]) / 126.)
 *             out[ndata < 128] = -0.25 * exp             # <<<<<<<<<<<<<<
 *             # equal to 128, zero
 *             out[ndata == 128] = 0
 */
      __pyx_t_4 = PyNumber_Multiply(__pyx_float_neg_0_25, __pyx_v_exp); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 742, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = PyObject_RichCompare(__pyx_v_ndata, __pyx_int_128, Py_LT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 742, __pyx_L1_error)
      if (unlikely(PyObject_SetItem(__pyx_v_out, __pyx_t_2, __pyx_t_4) < 0)) __PYX_ERR(0, 742, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyart/io/_sigmetfile.pyx":744
 *             out[ndata < 128] = -0.25 * exp
 *             # equal to 128, zero
 *             out[ndata == 128] = 0             # <<<<<<<<<<<<<<
 * 
 *             mask[ndata == 0] = True
 */
      __pyx_t_4 = __Pyx_PyInt_EqObjC(__pyx_v_ndata, __pyx_int_128, 0x80, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 744, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (unlikely(PyObject_SetItem(__pyx_v_out, __pyx_t_4, __pyx_int_0) < 0)) __PYX_ERR(0, 744, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyart/io/_sigmetfile.pyx":746
 *             out[ndata == 128] = 0
 * 
 *             mask[ndata == 0] = True             # <<<<<<<<<<<<<<
 *             mask[ndata == 255] = True
 * 
 */
      __pyx_t_4 = __Pyx_PyInt_EqObjC(__pyx_v_ndata, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 746, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (unlikely(PyObject_SetItem(__pyx_v_mask, __pyx_t_4, Py_True) < 0)) __PYX_ERR(0, 746, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyart/io/_sigmetfile.pyx":747
 * 
 *             mask[ndata == 0] = True
 *             mask[ndata == 255] = True             # <<<<<<<<<<<<<<
 * 
 *         elif data_type_name == 'PHIDP':
 */
      __pyx_t_4 = __Pyx_PyInt_EqObjC(__pyx_v_ndata, __pyx_int_255, 0xFF, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 747, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (unlikely(PyObject_SetItem(__pyx_v_mask, __pyx_t_4, Py_True) < 0)) __PYX_ERR(0, 747, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyart/io/_sigmetfile.pyx":731
 *             mask[ndata == 0] = True
 * 
 *         elif data_type_name == 'KDP':             # <<<<<<<<<<<<<<
 *             # KDP, 14, KDP (Differential phase) (1 byte)
 *             # 1-byte KDP format, section 4.3.12
 */
      goto __pyx_L6;
    }

    /* "pyart/io/_sigmetfile.pyx":749
 *             mask[ndata == 255] = True
 * 
 *         elif data_type_name == 'PHIDP':             # <<<<<<<<<<<<<<
 *             # PHIDP, 16, PhiDP(Differential phase) (1 byte)
 *             # 1-byte PhiDP format, section 4.3.18
 */
    __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_v_data_type_name, __pyx_n_u_PHIDP, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 749, __pyx_L1_error)
    if (__pyx_t_6) {

      /* "pyart/io/_sigmetfile.pyx":752
 *             # PHIDP, 16, PhiDP(Differential phase) (1 byte)
 *             # 1-byte PhiDP format, section 4.3.18
 *             out[:] = 180. * ((ndata - 1.) / 254.)             # <<<<<<<<<<<<<<
 *             mask[ndata == 0] = True
 *             mask[ndata == 255] = True
 */
      __pyx_t_4 = __Pyx_PyFloat_SubtractObjC(__pyx_v_ndata, __pyx_float_1_, 1., 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 752, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_4, __pyx_float_254_, 254., 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 752, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = PyNumber_Multiply(__pyx_float_180_, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 752, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (__Pyx_PyObject_SetSlice(__pyx_v_out, __pyx_t_4, 0, 0, NULL, NULL, &__pyx_slice__12, 0, 0, 1) < 0) __PYX_ERR(0, 752, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyart/io/_sigmetfile.pyx":753
 *             # 1-byte PhiDP format, section 4.3.18
 *             out[:] = 180. * ((ndata - 1.) / 254.)
 *             mask[ndata == 0] = True             # <<<<<<<<<<<<<<
 *             mask[ndata == 255] = True
 * 
 */
      __pyx_t_4 = __Pyx_PyInt_EqObjC(__pyx_v_ndata, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 753, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (unlikely(PyObject_SetItem(__pyx_v_mask, __pyx_t_4, Py_True) < 0)) __PYX_ERR(0, 753, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyart/io/_sigmetfile.pyx":754
 *             out[:] = 180. * ((ndata - 1.) / 254.)
 *             mask[ndata == 0] = True
 *             mask[ndata == 255] = True             # <<<<<<<<<<<<<<
 * 
 *         elif data_type_name == "HCLASS":
 */
      __pyx_t_4 = __Pyx_PyInt_EqObjC(__pyx_v_ndata, __pyx_int_255, 0xFF, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 754, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (unlikely(PyObject_SetItem(__pyx_v_mask, __pyx_t_4, Py_True) < 0)) __PYX_ERR(0, 754, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyart/io/_sigmetfile.pyx":749
 *             mask[ndata == 255] = True
 * 
 *         elif data_type_name == 'PHIDP':             # <<<<<<<<<<<<<<
 *             # PHIDP, 16, PhiDP(Differential phase) (1 byte)
 *             # 1-byte PhiDP format, section 4.3.18
 */
      goto __pyx_L6;
    }

    /* "pyart/io/_sigmetfile.pyx":756
 *             mask[ndata == 255] = True
 * 
 *         elif data_type_name == "HCLASS":             # <<<<<<<<<<<<<<
 *             # HCLASS, 55, Hydrometeor class (1 byte)
 *             # 1-byte HydroClass format, section 4.3.8
 */
    __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_v_data_type_name, __pyx_n_u_HCLASS, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 756, __pyx_L1_error)
    if (__pyx_t_6) {

      /* "pyart/io/_sigmetfile.pyx":759
 *             # HCLASS, 55, Hydrometeor class (1 byte)
 *             # 1-byte HydroClass format, section 4.3.8
 *             out[:] = ndata[:]             # <<<<<<<<<<<<<<
 *             mask[ndata == 0] = True     # No data available
 *             mask[ndata == 255] = True   # Area not scanned
 */
      __pyx_t_4 = __Pyx_PyObject_GetSlice(__pyx_v_ndata, 0, 0, NULL, NULL, &__pyx_slice__12, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 759, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__Pyx_PyObject_SetSlice(__pyx_v_out, __pyx_t_4, 0, 0, NULL, NULL, &__pyx_slice__12, 0, 0, 1) < 0) __PYX_ERR(0, 759, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyart/io/_sigmetfile.pyx":760
 *             # 1-byte HydroClass format, section 4.3.8
 *             out[:] = ndata[:]
 *             mask[ndata == 0] = True     # No data available             # <<<<<<<<<<<<<<
 *             mask[ndata == 255] = True   # Area not scanned
 * 
 */
      __pyx_t_4 = __Pyx_PyInt_EqObjC(__pyx_v_ndata, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 760, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (unlikely(PyObject_SetItem(__pyx_v_mask, __pyx_t_4, Py_True) < 0)) __PYX_ERR(0, 760, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyart/io/_sigmetfile.pyx":761
 *             out[:] = ndata[:]
 *             mask[ndata == 0] = True     # No data available
 *             mask[ndata == 255] = True   # Area not scanned             # <<<<<<<<<<<<<<
 * 
 *         else:
 */
      __pyx_t_4 = __Pyx_PyInt_EqObjC(__pyx_v_ndata, __pyx_int_255, 0xFF, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 761, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (unlikely(PyObject_SetItem(__pyx_v_mask, __pyx_t_4, Py_True) < 0)) __PYX_ERR(0, 761, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyart/io/_sigmetfile.pyx":756
 *             mask[ndata == 255] = True
 * 
 *         elif data_type_name == "HCLASS":             # <<<<<<<<<<<<<<
 *             # HCLASS, 55, Hydrometeor class (1 byte)
 *             # 1-byte HydroClass format, section 4.3.8
 */
      goto __pyx_L6;
    }

    /* "pyart/io/_sigmetfile.pyx":765
 *         else:
 *             # TODO implement conversions for addition 1-byte formats
 *             warnings.warn('Unknown type: %s, returning raw data' % data_type)             # <<<<<<<<<<<<<<
 *             out[:] = np.ma.masked_array(data)
 *             return out
 */
    /*else*/ {
      __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_warnings); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 765, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_warn); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 765, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyUnicode_FormatSafe(__pyx_kp_u_Unknown_type_s_returning_raw_dat, __pyx_v_data_type); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 765, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
        }
      }
      __pyx_t_4 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_2);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 765, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyart/io/_sigmetfile.pyx":766
 *             # TODO implement conversions for addition 1-byte formats
 *             warnings.warn('Unknown type: %s, returning raw data' % data_type)
 *             out[:] = np.ma.masked_array(data)             # <<<<<<<<<<<<<<
 *             return out
 *     else:
 */
      __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 766, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_ma); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 766, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_masked_array); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 766, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
        }
      }
      __pyx_t_4 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_2, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v_data);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 766, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (__Pyx_PyObject_SetSlice(__pyx_v_out, __pyx_t_4, 0, 0, NULL, NULL, &__pyx_slice__12, 0, 0, 1) < 0) __PYX_ERR(0, 766, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyart/io/_sigmetfile.pyx":767
 *             warnings.warn('Unknown type: %s, returning raw data' % data_type)
 *             out[:] = np.ma.masked_array(data)
 *             return out             # <<<<<<<<<<<<<<
 *     else:
 *         # TODO implement conversions for additional formats.
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_v_out);
      __pyx_r = __pyx_v_out;
      goto __pyx_L0;
    }
    __pyx_L6:;

    /* "pyart/io/_sigmetfile.pyx":689
 * 
 *     # one byte data types
 *     elif data_type_name[-1] != '2':             # <<<<<<<<<<<<<<
 *         # make a view of left half of the data as uint8,
 *         # this is the actual ray data collected, the right half is blank.
 */
    goto __pyx_L3;
  }

  /* "pyart/io/_sigmetfile.pyx":770
 *     else:
 *         # TODO implement conversions for additional formats.
 *         warnings.warn('Unknown type: %s, returning raw data' % data_type)             # <<<<<<<<<<<<<<
 *         out[:] = data
 *         return np.ma.masked_array(out)
 */
  /*else*/ {
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_warnings); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 770, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_warn); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 770, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyUnicode_FormatSafe(__pyx_kp_u_Unknown_type_s_returning_raw_dat, __pyx_v_data_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 770, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_4 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_t_1) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_1);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 770, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "pyart/io/_sigmetfile.pyx":771
 *         # TODO implement conversions for additional formats.
 *         warnings.warn('Unknown type: %s, returning raw data' % data_type)
 *         out[:] = data             # <<<<<<<<<<<<<<
 *         return np.ma.masked_array(out)
 * 
 */
    if (__Pyx_PyObject_SetSlice(__pyx_v_out, __pyx_v_data, 0, 0, NULL, NULL, &__pyx_slice__12, 0, 0, 1) < 0) __PYX_ERR(0, 771, __pyx_L1_error)

    /* "pyart/io/_sigmetfile.pyx":772
 *         warnings.warn('Unknown type: %s, returning raw data' % data_type)
 *         out[:] = data
 *         return np.ma.masked_array(out)             # <<<<<<<<<<<<<<
 * 
 *     # mask any gates which are beyond the number of gates in that ray.
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 772, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_ma); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 772, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_masked_array); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 772, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_4 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_1, __pyx_v_out) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_out);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 772, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }
  __pyx_L3:;

  /* "pyart/io/_sigmetfile.pyx":775
 * 
 *     # mask any gates which are beyond the number of gates in that ray.
 *     _mask_gates_not_collected(mask.view(np.uint8), nbins)             # <<<<<<<<<<<<<<
 * 
 *     return np.ma.masked_array(out, mask=mask, fill_value=-9999.0,
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_mask, __pyx_n_s_view); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 775, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 775, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_uint8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 775, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_4 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_1, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 775, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 775, __pyx_L1_error)
  if (!(likely(((__pyx_v_nbins) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_nbins, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 775, __pyx_L1_error)
  __pyx_t_2 = __pyx_f_5pyart_2io_11_sigmetfile__mask_gates_not_collected(((PyArrayObject *)__pyx_t_4), ((PyArrayObject *)__pyx_v_nbins)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 775, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyart/io/_sigmetfile.pyx":777
 *     _mask_gates_not_collected(mask.view(np.uint8), nbins)
 * 
 *     return np.ma.masked_array(out, mask=mask, fill_value=-9999.0,             # <<<<<<<<<<<<<<
 *                               shrink=False)
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 777, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_ma); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 777, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_masked_array); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 777, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 777, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_out);
  __Pyx_GIVEREF(__pyx_v_out);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_out);
  __pyx_t_3 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 777, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_mask, __pyx_v_mask) < 0) __PYX_ERR(0, 777, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_fill_value, __pyx_float_neg_9999_0) < 0) __PYX_ERR(0, 777, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":778
 * 
 *     return np.ma.masked_array(out, mask=mask, fill_value=-9999.0,
 *                               shrink=False)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shrink, Py_False) < 0) __PYX_ERR(0, 777, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":777
 *     _mask_gates_not_collected(mask.view(np.uint8), nbins)
 * 
 *     return np.ma.masked_array(out, mask=mask, fill_value=-9999.0,             # <<<<<<<<<<<<<<
 *                               shrink=False)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 777, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_sigmetfile.pyx":596
 * # file. Rewriting the convertions/masking in Cython does not seem to improved
 * # performance likely since most of the routines are already vectorized.
 * def convert_sigmet_data(data_type, data, nbins):             # <<<<<<<<<<<<<<
 *     """ Convert sigmet data. """
 *     out = np.empty_like(data, dtype='float32')
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyart.io._sigmetfile.convert_sigmet_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_out);
  __Pyx_XDECREF(__pyx_v_mask);
  __Pyx_XDECREF(__pyx_v_data_type_name);
  __Pyx_XDECREF(__pyx_v_like_dbt2);
  __Pyx_XDECREF(__pyx_v_like_sqi);
  __Pyx_XDECREF(__pyx_v_like_sqi2);
  __Pyx_XDECREF(__pyx_v_like_dbt);
  __Pyx_XDECREF(__pyx_v_nrays);
  __Pyx_XDECREF(__pyx_v_nbin);
  __Pyx_XDECREF(__pyx_v_ndata);
  __Pyx_XDECREF(__pyx_v_exp);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_sigmetfile.pyx":782
 * 
 * @cython.boundscheck(False)
 * cdef _mask_gates_not_collected(             # <<<<<<<<<<<<<<
 *         np.ndarray[np.uint8_t, ndim=2] mask,
 *         np.ndarray[np.int16_t, ndim=1] nbins):
 */

static PyObject *__pyx_f_5pyart_2io_11_sigmetfile__mask_gates_not_collected(PyArrayObject *__pyx_v_mask, PyArrayObject *__pyx_v_nbins) {
  int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_v_nrays;
  int __pyx_v_nbin;
  int __pyx_v_full_nbins;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_mask;
  __Pyx_Buffer __pyx_pybuffer_mask;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_nbins;
  __Pyx_Buffer __pyx_pybuffer_nbins;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_mask_gates_not_collected", 0);
  __pyx_pybuffer_mask.pybuffer.buf = NULL;
  __pyx_pybuffer_mask.refcount = 0;
  __pyx_pybuffernd_mask.data = NULL;
  __pyx_pybuffernd_mask.rcbuffer = &__pyx_pybuffer_mask;
  __pyx_pybuffer_nbins.pybuffer.buf = NULL;
  __pyx_pybuffer_nbins.refcount = 0;
  __pyx_pybuffernd_nbins.data = NULL;
  __pyx_pybuffernd_nbins.rcbuffer = &__pyx_pybuffer_nbins;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_mask.rcbuffer->pybuffer, (PyObject*)__pyx_v_mask, &__Pyx_TypeInfo_nn___pyx_t_5numpy_uint8_t, PyBUF_FORMAT| PyBUF_STRIDES| PyBUF_WRITABLE, 2, 0, __pyx_stack) == -1)) __PYX_ERR(0, 782, __pyx_L1_error)
  }
  __pyx_pybuffernd_mask.diminfo[0].strides = __pyx_pybuffernd_mask.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_mask.diminfo[0].shape = __pyx_pybuffernd_mask.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_mask.diminfo[1].strides = __pyx_pybuffernd_mask.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_mask.diminfo[1].shape = __pyx_pybuffernd_mask.rcbuffer->pybuffer.shape[1];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_nbins.rcbuffer->pybuffer, (PyObject*)__pyx_v_nbins, &__Pyx_TypeInfo_nn___pyx_t_5numpy_int16_t, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 782, __pyx_L1_error)
  }
  __pyx_pybuffernd_nbins.diminfo[0].strides = __pyx_pybuffernd_nbins.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_nbins.diminfo[0].shape = __pyx_pybuffernd_nbins.rcbuffer->pybuffer.shape[0];

  /* "pyart/io/_sigmetfile.pyx":787
 *     """ Add gates not collected (beyond nbin) to the mask. """
 *     cdef int i, j, nrays, nbin, full_nbins
 *     nrays = mask.shape[0]             # <<<<<<<<<<<<<<
 *     full_nbins = mask.shape[1]
 *     for i in range(nrays):
 */
  __pyx_v_nrays = (__pyx_v_mask->dimensions[0]);

  /* "pyart/io/_sigmetfile.pyx":788
 *     cdef int i, j, nrays, nbin, full_nbins
 *     nrays = mask.shape[0]
 *     full_nbins = mask.shape[1]             # <<<<<<<<<<<<<<
 *     for i in range(nrays):
 *         nbin = nbins[i]
 */
  __pyx_v_full_nbins = (__pyx_v_mask->dimensions[1]);

  /* "pyart/io/_sigmetfile.pyx":789
 *     nrays = mask.shape[0]
 *     full_nbins = mask.shape[1]
 *     for i in range(nrays):             # <<<<<<<<<<<<<<
 *         nbin = nbins[i]
 *         for j in range(nbin, full_nbins):
 */
  __pyx_t_1 = __pyx_v_nrays;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "pyart/io/_sigmetfile.pyx":790
 *     full_nbins = mask.shape[1]
 *     for i in range(nrays):
 *         nbin = nbins[i]             # <<<<<<<<<<<<<<
 *         for j in range(nbin, full_nbins):
 *             mask[i, j] = 1
 */
    __pyx_t_4 = __pyx_v_i;
    if (__pyx_t_4 < 0) __pyx_t_4 += __pyx_pybuffernd_nbins.diminfo[0].shape;
    __pyx_v_nbin = (*__Pyx_BufPtrStrided1d(__pyx_t_5numpy_int16_t *, __pyx_pybuffernd_nbins.rcbuffer->pybuffer.buf, __pyx_t_4, __pyx_pybuffernd_nbins.diminfo[0].strides));

    /* "pyart/io/_sigmetfile.pyx":791
 *     for i in range(nrays):
 *         nbin = nbins[i]
 *         for j in range(nbin, full_nbins):             # <<<<<<<<<<<<<<
 *             mask[i, j] = 1
 *     return
 */
    __pyx_t_5 = __pyx_v_full_nbins;
    __pyx_t_6 = __pyx_t_5;
    for (__pyx_t_7 = __pyx_v_nbin; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
      __pyx_v_j = __pyx_t_7;

      /* "pyart/io/_sigmetfile.pyx":792
 *         nbin = nbins[i]
 *         for j in range(nbin, full_nbins):
 *             mask[i, j] = 1             # <<<<<<<<<<<<<<
 *     return
 * 
 */
      __pyx_t_4 = __pyx_v_i;
      __pyx_t_8 = __pyx_v_j;
      if (__pyx_t_4 < 0) __pyx_t_4 += __pyx_pybuffernd_mask.diminfo[0].shape;
      if (__pyx_t_8 < 0) __pyx_t_8 += __pyx_pybuffernd_mask.diminfo[1].shape;
      *__Pyx_BufPtrStrided2d(__pyx_t_5numpy_uint8_t *, __pyx_pybuffernd_mask.rcbuffer->pybuffer.buf, __pyx_t_4, __pyx_pybuffernd_mask.diminfo[0].strides, __pyx_t_8, __pyx_pybuffernd_mask.diminfo[1].strides) = 1;
    }
  }

  /* "pyart/io/_sigmetfile.pyx":793
 *         for j in range(nbin, full_nbins):
 *             mask[i, j] = 1
 *     return             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "pyart/io/_sigmetfile.pyx":782
 * 
 * @cython.boundscheck(False)
 * cdef _mask_gates_not_collected(             # <<<<<<<<<<<<<<
 *         np.ndarray[np.uint8_t, ndim=2] mask,
 *         np.ndarray[np.int16_t, ndim=1] nbins):
 */

  /* function exit code */
  __pyx_L1_error:;
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_mask.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_nbins.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("pyart.io._sigmetfile._mask_gates_not_collected", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_mask.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_nbins.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_sigmetfile.pyx":796
 * 
 * 
 * def bin2_to_angle(bin2):             # <<<<<<<<<<<<<<
 *     """ Return an angle from Sigmet bin2 encoded value (or array). """
 *     return 360. * bin2 / 65536
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_9bin2_to_angle(PyObject *__pyx_self, PyObject *__pyx_v_bin2); /*proto*/
static char __pyx_doc_5pyart_2io_11_sigmetfile_8bin2_to_angle[] = " Return an angle from Sigmet bin2 encoded value (or array). ";
static PyMethodDef __pyx_mdef_5pyart_2io_11_sigmetfile_9bin2_to_angle = {"bin2_to_angle", (PyCFunction)__pyx_pw_5pyart_2io_11_sigmetfile_9bin2_to_angle, METH_O, __pyx_doc_5pyart_2io_11_sigmetfile_8bin2_to_angle};
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_9bin2_to_angle(PyObject *__pyx_self, PyObject *__pyx_v_bin2) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("bin2_to_angle (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_8bin2_to_angle(__pyx_self, ((PyObject *)__pyx_v_bin2));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_8bin2_to_angle(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_bin2) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("bin2_to_angle", 0);

  /* "pyart/io/_sigmetfile.pyx":798
 * def bin2_to_angle(bin2):
 *     """ Return an angle from Sigmet bin2 encoded value (or array). """
 *     return 360. * bin2 / 65536             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyNumber_Multiply(__pyx_float_360_, __pyx_v_bin2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 798, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_TrueDivideObjC(__pyx_t_1, __pyx_int_65536, 0x10000, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 798, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "pyart/io/_sigmetfile.pyx":796
 * 
 * 
 * def bin2_to_angle(bin2):             # <<<<<<<<<<<<<<
 *     """ Return an angle from Sigmet bin2 encoded value (or array). """
 *     return 360. * bin2 / 65536
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyart.io._sigmetfile.bin2_to_angle", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_sigmetfile.pyx":801
 * 
 * 
 * def bin4_to_angle(bin4):             # <<<<<<<<<<<<<<
 *     """ Return an angle from Sigmet bin4 encoded value (or array). """
 *     return 360. * bin4 / 4294967296
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_11bin4_to_angle(PyObject *__pyx_self, PyObject *__pyx_v_bin4); /*proto*/
static char __pyx_doc_5pyart_2io_11_sigmetfile_10bin4_to_angle[] = " Return an angle from Sigmet bin4 encoded value (or array). ";
static PyMethodDef __pyx_mdef_5pyart_2io_11_sigmetfile_11bin4_to_angle = {"bin4_to_angle", (PyCFunction)__pyx_pw_5pyart_2io_11_sigmetfile_11bin4_to_angle, METH_O, __pyx_doc_5pyart_2io_11_sigmetfile_10bin4_to_angle};
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_11bin4_to_angle(PyObject *__pyx_self, PyObject *__pyx_v_bin4) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("bin4_to_angle (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_10bin4_to_angle(__pyx_self, ((PyObject *)__pyx_v_bin4));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_10bin4_to_angle(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_bin4) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("bin4_to_angle", 0);

  /* "pyart/io/_sigmetfile.pyx":803
 * def bin4_to_angle(bin4):
 *     """ Return an angle from Sigmet bin4 encoded value (or array). """
 *     return 360. * bin4 / 4294967296             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyNumber_Multiply(__pyx_float_360_, __pyx_v_bin4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 803, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyNumber_Divide(__pyx_t_1, __pyx_int_4294967296); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 803, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "pyart/io/_sigmetfile.pyx":801
 * 
 * 
 * def bin4_to_angle(bin4):             # <<<<<<<<<<<<<<
 *     """ Return an angle from Sigmet bin4 encoded value (or array). """
 *     return 360. * bin4 / 4294967296
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyart.io._sigmetfile.bin4_to_angle", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_sigmetfile.pyx":811
 * 
 * 
 * def _unpack_structure(string, structure):             # <<<<<<<<<<<<<<
 *     """ Unpack a structure """
 *     fmt = ''.join([i[1] for i in structure])
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_13_unpack_structure(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5pyart_2io_11_sigmetfile_12_unpack_structure[] = " Unpack a structure ";
static PyMethodDef __pyx_mdef_5pyart_2io_11_sigmetfile_13_unpack_structure = {"_unpack_structure", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5pyart_2io_11_sigmetfile_13_unpack_structure, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5pyart_2io_11_sigmetfile_12_unpack_structure};
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_13_unpack_structure(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_string = 0;
  PyObject *__pyx_v_structure = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_unpack_structure (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_string,&__pyx_n_s_structure,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_string)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_structure)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_unpack_structure", 1, 2, 2, 1); __PYX_ERR(0, 811, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_unpack_structure") < 0)) __PYX_ERR(0, 811, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_string = values[0];
    __pyx_v_structure = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_unpack_structure", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 811, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._sigmetfile._unpack_structure", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_12_unpack_structure(__pyx_self, __pyx_v_string, __pyx_v_structure);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_12_unpack_structure(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_string, PyObject *__pyx_v_structure) {
  PyObject *__pyx_v_fmt = NULL;
  PyObject *__pyx_v_l = NULL;
  PyObject *__pyx_8genexpr8__pyx_v_i = NULL;
  PyObject *__pyx_8genexpr9__pyx_v_i = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_unpack_structure", 0);

  /* "pyart/io/_sigmetfile.pyx":813
 * def _unpack_structure(string, structure):
 *     """ Unpack a structure """
 *     fmt = ''.join([i[1] for i in structure])             # <<<<<<<<<<<<<<
 *     l = struct.unpack(fmt, string)
 *     return dict(zip([i[0] for i in structure], l))
 */
  { /* enter inner scope */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 813, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (likely(PyList_CheckExact(__pyx_v_structure)) || PyTuple_CheckExact(__pyx_v_structure)) {
      __pyx_t_2 = __pyx_v_structure; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
      __pyx_t_4 = NULL;
    } else {
      __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_structure); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 813, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 813, __pyx_L5_error)
    }
    for (;;) {
      if (likely(!__pyx_t_4)) {
        if (likely(PyList_CheckExact(__pyx_t_2))) {
          if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 813, __pyx_L5_error)
          #else
          __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 813, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        } else {
          if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 813, __pyx_L5_error)
          #else
          __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 813, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        }
      } else {
        __pyx_t_5 = __pyx_t_4(__pyx_t_2);
        if (unlikely(!__pyx_t_5)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 813, __pyx_L5_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_5);
      }
      __Pyx_XDECREF_SET(__pyx_8genexpr8__pyx_v_i, __pyx_t_5);
      __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_GetItemInt(__pyx_8genexpr8__pyx_v_i, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 813, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_5))) __PYX_ERR(0, 813, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_8genexpr8__pyx_v_i); __pyx_8genexpr8__pyx_v_i = 0;
    goto __pyx_L8_exit_scope;
    __pyx_L5_error:;
    __Pyx_XDECREF(__pyx_8genexpr8__pyx_v_i); __pyx_8genexpr8__pyx_v_i = 0;
    goto __pyx_L1_error;
    __pyx_L8_exit_scope:;
  } /* exit inner scope */
  __pyx_t_2 = PyUnicode_Join(__pyx_kp_u__15, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 813, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_fmt = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyart/io/_sigmetfile.pyx":814
 *     """ Unpack a structure """
 *     fmt = ''.join([i[1] for i in structure])
 *     l = struct.unpack(fmt, string)             # <<<<<<<<<<<<<<
 *     return dict(zip([i[0] for i in structure], l))
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_struct); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 814, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_unpack); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 814, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_5)) {
    PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_fmt, __pyx_v_string};
    __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 814, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
    PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_fmt, __pyx_v_string};
    __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 814, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 814, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_1) {
      __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_1); __pyx_t_1 = NULL;
    }
    __Pyx_INCREF(__pyx_v_fmt);
    __Pyx_GIVEREF(__pyx_v_fmt);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_v_fmt);
    __Pyx_INCREF(__pyx_v_string);
    __Pyx_GIVEREF(__pyx_v_string);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_v_string);
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 814, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_l = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pyart/io/_sigmetfile.pyx":815
 *     fmt = ''.join([i[1] for i in structure])
 *     l = struct.unpack(fmt, string)
 *     return dict(zip([i[0] for i in structure], l))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  { /* enter inner scope */
    __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 815, __pyx_L11_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (likely(PyList_CheckExact(__pyx_v_structure)) || PyTuple_CheckExact(__pyx_v_structure)) {
      __pyx_t_5 = __pyx_v_structure; __Pyx_INCREF(__pyx_t_5); __pyx_t_3 = 0;
      __pyx_t_4 = NULL;
    } else {
      __pyx_t_3 = -1; __pyx_t_5 = PyObject_GetIter(__pyx_v_structure); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 815, __pyx_L11_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_4 = Py_TYPE(__pyx_t_5)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 815, __pyx_L11_error)
    }
    for (;;) {
      if (likely(!__pyx_t_4)) {
        if (likely(PyList_CheckExact(__pyx_t_5))) {
          if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_5)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_7 = PyList_GET_ITEM(__pyx_t_5, __pyx_t_3); __Pyx_INCREF(__pyx_t_7); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 815, __pyx_L11_error)
          #else
          __pyx_t_7 = PySequence_ITEM(__pyx_t_5, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 815, __pyx_L11_error)
          __Pyx_GOTREF(__pyx_t_7);
          #endif
        } else {
          if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_5)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_5, __pyx_t_3); __Pyx_INCREF(__pyx_t_7); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 815, __pyx_L11_error)
          #else
          __pyx_t_7 = PySequence_ITEM(__pyx_t_5, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 815, __pyx_L11_error)
          __Pyx_GOTREF(__pyx_t_7);
          #endif
        }
      } else {
        __pyx_t_7 = __pyx_t_4(__pyx_t_5);
        if (unlikely(!__pyx_t_7)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 815, __pyx_L11_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_7);
      }
      __Pyx_XDECREF_SET(__pyx_8genexpr9__pyx_v_i, __pyx_t_7);
      __pyx_t_7 = 0;
      __pyx_t_7 = __Pyx_GetItemInt(__pyx_8genexpr9__pyx_v_i, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 815, __pyx_L11_error)
      __Pyx_GOTREF(__pyx_t_7);
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_2, (PyObject*)__pyx_t_7))) __PYX_ERR(0, 815, __pyx_L11_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_8genexpr9__pyx_v_i); __pyx_8genexpr9__pyx_v_i = 0;
    goto __pyx_L14_exit_scope;
    __pyx_L11_error:;
    __Pyx_XDECREF(__pyx_8genexpr9__pyx_v_i); __pyx_8genexpr9__pyx_v_i = 0;
    goto __pyx_L1_error;
    __pyx_L14_exit_scope:;
  } /* exit inner scope */
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 815, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_l);
  __Pyx_GIVEREF(__pyx_v_l);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_v_l);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 815, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyDict_Type)), __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 815, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "pyart/io/_sigmetfile.pyx":811
 * 
 * 
 * def _unpack_structure(string, structure):             # <<<<<<<<<<<<<<
 *     """ Unpack a structure """
 *     fmt = ''.join([i[1] for i in structure])
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("pyart.io._sigmetfile._unpack_structure", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_fmt);
  __Pyx_XDECREF(__pyx_v_l);
  __Pyx_XDECREF(__pyx_8genexpr8__pyx_v_i);
  __Pyx_XDECREF(__pyx_8genexpr9__pyx_v_i);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_sigmetfile.pyx":818
 * 
 * 
 * def _unpack_key(dic, key, structure):             # <<<<<<<<<<<<<<
 *     """ Unpack a key. """
 *     dic[key] = _unpack_structure(dic[key], structure)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_15_unpack_key(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5pyart_2io_11_sigmetfile_14_unpack_key[] = " Unpack a key. ";
static PyMethodDef __pyx_mdef_5pyart_2io_11_sigmetfile_15_unpack_key = {"_unpack_key", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5pyart_2io_11_sigmetfile_15_unpack_key, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5pyart_2io_11_sigmetfile_14_unpack_key};
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_15_unpack_key(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_dic = 0;
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_structure = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_unpack_key (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_dic,&__pyx_n_s_key,&__pyx_n_s_structure,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dic)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_key)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_unpack_key", 1, 3, 3, 1); __PYX_ERR(0, 818, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_structure)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_unpack_key", 1, 3, 3, 2); __PYX_ERR(0, 818, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_unpack_key") < 0)) __PYX_ERR(0, 818, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_dic = values[0];
    __pyx_v_key = values[1];
    __pyx_v_structure = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_unpack_key", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 818, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._sigmetfile._unpack_key", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_14_unpack_key(__pyx_self, __pyx_v_dic, __pyx_v_key, __pyx_v_structure);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_14_unpack_key(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_dic, PyObject *__pyx_v_key, PyObject *__pyx_v_structure) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_unpack_key", 0);

  /* "pyart/io/_sigmetfile.pyx":820
 * def _unpack_key(dic, key, structure):
 *     """ Unpack a key. """
 *     dic[key] = _unpack_structure(dic[key], structure)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_structure); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 820, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetItem(__pyx_v_dic, __pyx_v_key); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 820, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_3, __pyx_v_structure};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 820, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_3, __pyx_v_structure};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 820, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 820, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_structure);
    __Pyx_GIVEREF(__pyx_v_structure);
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_structure);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 820, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(PyObject_SetItem(__pyx_v_dic, __pyx_v_key, __pyx_t_1) < 0)) __PYX_ERR(0, 820, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":818
 * 
 * 
 * def _unpack_key(dic, key, structure):             # <<<<<<<<<<<<<<
 *     """ Unpack a key. """
 *     dic[key] = _unpack_structure(dic[key], structure)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pyart.io._sigmetfile._unpack_key", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_sigmetfile.pyx":823
 * 
 * 
 * def _unpack_ingest_data_headers(record, ndata_types):             # <<<<<<<<<<<<<<
 *     """
 *     Unpack one or more ingest_data_header from a record.
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_17_unpack_ingest_data_headers(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5pyart_2io_11_sigmetfile_16_unpack_ingest_data_headers[] = "\n    Unpack one or more ingest_data_header from a record.\n\n    Returns a list of dictionaries or None when an error occurs.\n\n    ";
static PyMethodDef __pyx_mdef_5pyart_2io_11_sigmetfile_17_unpack_ingest_data_headers = {"_unpack_ingest_data_headers", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5pyart_2io_11_sigmetfile_17_unpack_ingest_data_headers, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5pyart_2io_11_sigmetfile_16_unpack_ingest_data_headers};
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_17_unpack_ingest_data_headers(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_record = 0;
  PyObject *__pyx_v_ndata_types = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_unpack_ingest_data_headers (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_record,&__pyx_n_s_ndata_types,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_record)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_ndata_types)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_unpack_ingest_data_headers", 1, 2, 2, 1); __PYX_ERR(0, 823, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_unpack_ingest_data_headers") < 0)) __PYX_ERR(0, 823, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_record = values[0];
    __pyx_v_ndata_types = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_unpack_ingest_data_headers", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 823, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._sigmetfile._unpack_ingest_data_headers", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_16_unpack_ingest_data_headers(__pyx_self, __pyx_v_record, __pyx_v_ndata_types);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_16_unpack_ingest_data_headers(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_record, PyObject *__pyx_v_ndata_types) {
  PyObject *__pyx_v_idh = NULL;
  PyObject *__pyx_9genexpr10__pyx_v_i = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *(*__pyx_t_5)(PyObject *);
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_unpack_ingest_data_headers", 0);

  /* "pyart/io/_sigmetfile.pyx":830
 * 
 *     """
 *     idh = [_unpack_ingest_data_header(record, i) for i in range(ndata_types)]             # <<<<<<<<<<<<<<
 *     if None in idh:
 *         return None
 */
  { /* enter inner scope */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 830, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_v_ndata_types); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 830, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
      __pyx_t_3 = __pyx_t_2; __Pyx_INCREF(__pyx_t_3); __pyx_t_4 = 0;
      __pyx_t_5 = NULL;
    } else {
      __pyx_t_4 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 830, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 830, __pyx_L5_error)
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    for (;;) {
      if (likely(!__pyx_t_5)) {
        if (likely(PyList_CheckExact(__pyx_t_3))) {
          if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_3)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_4); __Pyx_INCREF(__pyx_t_2); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 830, __pyx_L5_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 830, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        } else {
          if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_4); __Pyx_INCREF(__pyx_t_2); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 830, __pyx_L5_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 830, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        }
      } else {
        __pyx_t_2 = __pyx_t_5(__pyx_t_3);
        if (unlikely(!__pyx_t_2)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 830, __pyx_L5_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_XDECREF_SET(__pyx_9genexpr10__pyx_v_i, __pyx_t_2);
      __pyx_t_2 = 0;
      __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_unpack_ingest_data_header); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 830, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = NULL;
      __pyx_t_8 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
          __pyx_t_8 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_v_record, __pyx_9genexpr10__pyx_v_i};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 830, __pyx_L5_error)
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_v_record, __pyx_9genexpr10__pyx_v_i};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 830, __pyx_L5_error)
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_9 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 830, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_9);
        if (__pyx_t_7) {
          __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
        }
        __Pyx_INCREF(__pyx_v_record);
        __Pyx_GIVEREF(__pyx_v_record);
        PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_v_record);
        __Pyx_INCREF(__pyx_9genexpr10__pyx_v_i);
        __Pyx_GIVEREF(__pyx_9genexpr10__pyx_v_i);
        PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_9genexpr10__pyx_v_i);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_9, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 830, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      }
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_2))) __PYX_ERR(0, 830, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_9genexpr10__pyx_v_i); __pyx_9genexpr10__pyx_v_i = 0;
    goto __pyx_L8_exit_scope;
    __pyx_L5_error:;
    __Pyx_XDECREF(__pyx_9genexpr10__pyx_v_i); __pyx_9genexpr10__pyx_v_i = 0;
    goto __pyx_L1_error;
    __pyx_L8_exit_scope:;
  } /* exit inner scope */
  __pyx_v_idh = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":831
 *     """
 *     idh = [_unpack_ingest_data_header(record, i) for i in range(ndata_types)]
 *     if None in idh:             # <<<<<<<<<<<<<<
 *         return None
 *     else:
 */
  __pyx_t_10 = (__Pyx_PySequence_ContainsTF(Py_None, __pyx_v_idh, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 831, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_10 != 0);
  if (__pyx_t_11) {

    /* "pyart/io/_sigmetfile.pyx":832
 *     idh = [_unpack_ingest_data_header(record, i) for i in range(ndata_types)]
 *     if None in idh:
 *         return None             # <<<<<<<<<<<<<<
 *     else:
 *         return idh
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "pyart/io/_sigmetfile.pyx":831
 *     """
 *     idh = [_unpack_ingest_data_header(record, i) for i in range(ndata_types)]
 *     if None in idh:             # <<<<<<<<<<<<<<
 *         return None
 *     else:
 */
  }

  /* "pyart/io/_sigmetfile.pyx":834
 *         return None
 *     else:
 *         return idh             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_idh);
    __pyx_r = __pyx_v_idh;
    goto __pyx_L0;
  }

  /* "pyart/io/_sigmetfile.pyx":823
 * 
 * 
 * def _unpack_ingest_data_headers(record, ndata_types):             # <<<<<<<<<<<<<<
 *     """
 *     Unpack one or more ingest_data_header from a record.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("pyart.io._sigmetfile._unpack_ingest_data_headers", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_idh);
  __Pyx_XDECREF(__pyx_9genexpr10__pyx_v_i);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_sigmetfile.pyx":837
 * 
 * 
 * def _unpack_ingest_data_header(record, number):             # <<<<<<<<<<<<<<
 *     """
 *     Unpack a single ingest_data_header from record.  Return None on error.
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_19_unpack_ingest_data_header(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5pyart_2io_11_sigmetfile_18_unpack_ingest_data_header[] = "\n    Unpack a single ingest_data_header from record.  Return None on error.\n    ";
static PyMethodDef __pyx_mdef_5pyart_2io_11_sigmetfile_19_unpack_ingest_data_header = {"_unpack_ingest_data_header", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5pyart_2io_11_sigmetfile_19_unpack_ingest_data_header, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5pyart_2io_11_sigmetfile_18_unpack_ingest_data_header};
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_19_unpack_ingest_data_header(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_record = 0;
  PyObject *__pyx_v_number = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_unpack_ingest_data_header (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_record,&__pyx_n_s_number,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_record)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_number)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_unpack_ingest_data_header", 1, 2, 2, 1); __PYX_ERR(0, 837, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_unpack_ingest_data_header") < 0)) __PYX_ERR(0, 837, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_record = values[0];
    __pyx_v_number = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_unpack_ingest_data_header", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 837, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._sigmetfile._unpack_ingest_data_header", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_18_unpack_ingest_data_header(__pyx_self, __pyx_v_record, __pyx_v_number);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_18_unpack_ingest_data_header(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_record, PyObject *__pyx_v_number) {
  PyObject *__pyx_v_offset = NULL;
  PyObject *__pyx_v_string = NULL;
  PyObject *__pyx_v_idh = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_unpack_ingest_data_header", 0);

  /* "pyart/io/_sigmetfile.pyx":841
 *     Unpack a single ingest_data_header from record.  Return None on error.
 *     """
 *     offset = 12 + 76 * number             # <<<<<<<<<<<<<<
 *     string = record[offset:offset + 76]
 *     idh = _unpack_structure(string, INGEST_DATA_HEADER)
 */
  __pyx_t_1 = PyNumber_Multiply(__pyx_int_76, __pyx_v_number); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 841, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_AddCObj(__pyx_int_12, __pyx_t_1, 12, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 841, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_offset = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pyart/io/_sigmetfile.pyx":842
 *     """
 *     offset = 12 + 76 * number
 *     string = record[offset:offset + 76]             # <<<<<<<<<<<<<<
 *     idh = _unpack_structure(string, INGEST_DATA_HEADER)
 *     _unpack_key(idh, 'structure_header', STRUCTURE_HEADER)
 */
  __pyx_t_2 = __Pyx_PyInt_AddObjC(__pyx_v_offset, __pyx_int_76, 76, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 842, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetSlice(__pyx_v_record, 0, 0, &__pyx_v_offset, &__pyx_t_2, NULL, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 842, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_string = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":843
 *     offset = 12 + 76 * number
 *     string = record[offset:offset + 76]
 *     idh = _unpack_structure(string, INGEST_DATA_HEADER)             # <<<<<<<<<<<<<<
 *     _unpack_key(idh, 'structure_header', STRUCTURE_HEADER)
 *     _unpack_key(idh, 'sweep_start_time', YMDS_TIME)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_structure); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 843, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_INGEST_DATA_HEADER); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 843, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_string, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 843, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_string, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 843, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 843, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(__pyx_v_string);
    __Pyx_GIVEREF(__pyx_v_string);
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_string);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 843, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_idh = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":844
 *     string = record[offset:offset + 76]
 *     idh = _unpack_structure(string, INGEST_DATA_HEADER)
 *     _unpack_key(idh, 'structure_header', STRUCTURE_HEADER)             # <<<<<<<<<<<<<<
 *     _unpack_key(idh, 'sweep_start_time', YMDS_TIME)
 *     if idh['structure_header']['structure_identifier'] != 24:
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 844, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_STRUCTURE_HEADER); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 844, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_idh, __pyx_n_u_structure_header, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 844, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_idh, __pyx_n_u_structure_header, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 844, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  {
    __pyx_t_4 = PyTuple_New(3+__pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 844, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_idh);
    __Pyx_GIVEREF(__pyx_v_idh);
    PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_v_idh);
    __Pyx_INCREF(__pyx_n_u_structure_header);
    __Pyx_GIVEREF(__pyx_n_u_structure_header);
    PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_n_u_structure_header);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 2+__pyx_t_5, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 844, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":845
 *     idh = _unpack_structure(string, INGEST_DATA_HEADER)
 *     _unpack_key(idh, 'structure_header', STRUCTURE_HEADER)
 *     _unpack_key(idh, 'sweep_start_time', YMDS_TIME)             # <<<<<<<<<<<<<<
 *     if idh['structure_header']['structure_identifier'] != 24:
 *         return None
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 845, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_YMDS_TIME); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 845, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_v_idh, __pyx_n_u_sweep_start_time, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 845, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_v_idh, __pyx_n_u_sweep_start_time, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 845, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_3 = PyTuple_New(3+__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 845, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__pyx_t_6) {
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
    }
    __Pyx_INCREF(__pyx_v_idh);
    __Pyx_GIVEREF(__pyx_v_idh);
    PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_v_idh);
    __Pyx_INCREF(__pyx_n_u_sweep_start_time);
    __Pyx_GIVEREF(__pyx_n_u_sweep_start_time);
    PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_n_u_sweep_start_time);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 2+__pyx_t_5, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 845, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":846
 *     _unpack_key(idh, 'structure_header', STRUCTURE_HEADER)
 *     _unpack_key(idh, 'sweep_start_time', YMDS_TIME)
 *     if idh['structure_header']['structure_identifier'] != 24:             # <<<<<<<<<<<<<<
 *         return None
 *     return idh
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_idh, __pyx_n_u_structure_header); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 846, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_structure_identifier); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 846, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyInt_NeObjC(__pyx_t_2, __pyx_int_24, 24, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 846, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 846, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_7) {

    /* "pyart/io/_sigmetfile.pyx":847
 *     _unpack_key(idh, 'sweep_start_time', YMDS_TIME)
 *     if idh['structure_header']['structure_identifier'] != 24:
 *         return None             # <<<<<<<<<<<<<<
 *     return idh
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "pyart/io/_sigmetfile.pyx":846
 *     _unpack_key(idh, 'structure_header', STRUCTURE_HEADER)
 *     _unpack_key(idh, 'sweep_start_time', YMDS_TIME)
 *     if idh['structure_header']['structure_identifier'] != 24:             # <<<<<<<<<<<<<<
 *         return None
 *     return idh
 */
  }

  /* "pyart/io/_sigmetfile.pyx":848
 *     if idh['structure_header']['structure_identifier'] != 24:
 *         return None
 *     return idh             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_idh);
  __pyx_r = __pyx_v_idh;
  goto __pyx_L0;

  /* "pyart/io/_sigmetfile.pyx":837
 * 
 * 
 * def _unpack_ingest_data_header(record, number):             # <<<<<<<<<<<<<<
 *     """
 *     Unpack a single ingest_data_header from record.  Return None on error.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pyart.io._sigmetfile._unpack_ingest_data_header", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_offset);
  __Pyx_XDECREF(__pyx_v_string);
  __Pyx_XDECREF(__pyx_v_idh);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_sigmetfile.pyx":851
 * 
 * 
 * def _unpack_raw_prod_bhdr(record):             # <<<<<<<<<<<<<<
 *     """ Return a dict with the unpacked raw_prod_bhdr from a record. """
 *     return _unpack_structure(record[:12], RAW_PROD_BHDR)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_21_unpack_raw_prod_bhdr(PyObject *__pyx_self, PyObject *__pyx_v_record); /*proto*/
static char __pyx_doc_5pyart_2io_11_sigmetfile_20_unpack_raw_prod_bhdr[] = " Return a dict with the unpacked raw_prod_bhdr from a record. ";
static PyMethodDef __pyx_mdef_5pyart_2io_11_sigmetfile_21_unpack_raw_prod_bhdr = {"_unpack_raw_prod_bhdr", (PyCFunction)__pyx_pw_5pyart_2io_11_sigmetfile_21_unpack_raw_prod_bhdr, METH_O, __pyx_doc_5pyart_2io_11_sigmetfile_20_unpack_raw_prod_bhdr};
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_21_unpack_raw_prod_bhdr(PyObject *__pyx_self, PyObject *__pyx_v_record) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_unpack_raw_prod_bhdr (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_20_unpack_raw_prod_bhdr(__pyx_self, ((PyObject *)__pyx_v_record));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_20_unpack_raw_prod_bhdr(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_record) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_unpack_raw_prod_bhdr", 0);

  /* "pyart/io/_sigmetfile.pyx":853
 * def _unpack_raw_prod_bhdr(record):
 *     """ Return a dict with the unpacked raw_prod_bhdr from a record. """
 *     return _unpack_structure(record[:12], RAW_PROD_BHDR)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_structure); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 853, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetSlice(__pyx_v_record, 0, 12, NULL, NULL, &__pyx_slice__16, 0, 1, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 853, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_RAW_PROD_BHDR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 853, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 853, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 853, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 853, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 853, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_sigmetfile.pyx":851
 * 
 * 
 * def _unpack_raw_prod_bhdr(record):             # <<<<<<<<<<<<<<
 *     """ Return a dict with the unpacked raw_prod_bhdr from a record. """
 *     return _unpack_structure(record[:12], RAW_PROD_BHDR)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("pyart.io._sigmetfile._unpack_raw_prod_bhdr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_sigmetfile.pyx":856
 * 
 * 
 * def _unpack_product_hdr(record):             # <<<<<<<<<<<<<<
 *     """
 *     Return a dict with the unpacked product_hdr from the first record.
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_23_unpack_product_hdr(PyObject *__pyx_self, PyObject *__pyx_v_record); /*proto*/
static char __pyx_doc_5pyart_2io_11_sigmetfile_22_unpack_product_hdr[] = "\n    Return a dict with the unpacked product_hdr from the first record.\n    ";
static PyMethodDef __pyx_mdef_5pyart_2io_11_sigmetfile_23_unpack_product_hdr = {"_unpack_product_hdr", (PyCFunction)__pyx_pw_5pyart_2io_11_sigmetfile_23_unpack_product_hdr, METH_O, __pyx_doc_5pyart_2io_11_sigmetfile_22_unpack_product_hdr};
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_23_unpack_product_hdr(PyObject *__pyx_self, PyObject *__pyx_v_record) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_unpack_product_hdr (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_22_unpack_product_hdr(__pyx_self, ((PyObject *)__pyx_v_record));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_22_unpack_product_hdr(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_record) {
  PyObject *__pyx_v_product_hdr = NULL;
  PyObject *__pyx_v_product_config = NULL;
  PyObject *__pyx_v_product_end = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_unpack_product_hdr", 0);

  /* "pyart/io/_sigmetfile.pyx":862
 * 
 *     # unpack the product_hdr structure from the first record
 *     product_hdr = _unpack_structure(record[:640], PRODUCT_HDR)             # <<<<<<<<<<<<<<
 * 
 *     # product_hdr substructure
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_structure); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 862, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetSlice(__pyx_v_record, 0, 0x280, NULL, NULL, &__pyx_slice__17, 0, 1, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 862, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_PRODUCT_HDR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 862, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 862, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 862, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 862, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 862, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_product_hdr = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":865
 * 
 *     # product_hdr substructure
 *     _unpack_key(product_hdr, 'structure_header', STRUCTURE_HEADER)             # <<<<<<<<<<<<<<
 *     _unpack_key(product_hdr, 'product_configuration',
 *                 PRODUCT_CONFIGURATION)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 865, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_STRUCTURE_HEADER); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 865, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_product_hdr, __pyx_n_u_structure_header, __pyx_t_7};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 865, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_product_hdr, __pyx_n_u_structure_header, __pyx_t_7};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 865, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  {
    __pyx_t_3 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 865, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(__pyx_v_product_hdr);
    __Pyx_GIVEREF(__pyx_v_product_hdr);
    PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_6, __pyx_v_product_hdr);
    __Pyx_INCREF(__pyx_n_u_structure_header);
    __Pyx_GIVEREF(__pyx_n_u_structure_header);
    PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_6, __pyx_n_u_structure_header);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_3, 2+__pyx_t_6, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 865, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":866
 *     # product_hdr substructure
 *     _unpack_key(product_hdr, 'structure_header', STRUCTURE_HEADER)
 *     _unpack_key(product_hdr, 'product_configuration',             # <<<<<<<<<<<<<<
 *                 PRODUCT_CONFIGURATION)
 *     _unpack_key(product_hdr, 'product_end', PRODUCT_END)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 866, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "pyart/io/_sigmetfile.pyx":867
 *     _unpack_key(product_hdr, 'structure_header', STRUCTURE_HEADER)
 *     _unpack_key(product_hdr, 'product_configuration',
 *                 PRODUCT_CONFIGURATION)             # <<<<<<<<<<<<<<
 *     _unpack_key(product_hdr, 'product_end', PRODUCT_END)
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_PRODUCT_CONFIGURATION); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 867, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_product_hdr, __pyx_n_u_product_configuration, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 866, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_product_hdr, __pyx_n_u_product_configuration, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 866, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_4 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 866, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__pyx_t_7) {
      __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_7); __pyx_t_7 = NULL;
    }
    __Pyx_INCREF(__pyx_v_product_hdr);
    __Pyx_GIVEREF(__pyx_v_product_hdr);
    PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_6, __pyx_v_product_hdr);
    __Pyx_INCREF(__pyx_n_u_product_configuration);
    __Pyx_GIVEREF(__pyx_n_u_product_configuration);
    PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_6, __pyx_n_u_product_configuration);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 2+__pyx_t_6, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 866, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":868
 *     _unpack_key(product_hdr, 'product_configuration',
 *                 PRODUCT_CONFIGURATION)
 *     _unpack_key(product_hdr, 'product_end', PRODUCT_END)             # <<<<<<<<<<<<<<
 * 
 *     # product_config substructure
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 868, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_PRODUCT_END); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 868, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_product_hdr, __pyx_n_u_product_end, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 868, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_product_hdr, __pyx_n_u_product_end, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 868, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 868, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_product_hdr);
    __Pyx_GIVEREF(__pyx_v_product_hdr);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_v_product_hdr);
    __Pyx_INCREF(__pyx_n_u_product_end);
    __Pyx_GIVEREF(__pyx_n_u_product_end);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_n_u_product_end);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 2+__pyx_t_6, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 868, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":871
 * 
 *     # product_config substructure
 *     product_config = product_hdr['product_configuration']             # <<<<<<<<<<<<<<
 *     _unpack_key(product_config, 'structure_header', STRUCTURE_HEADER)
 *     _unpack_key(product_config, 'generation_time', YMDS_TIME)
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_product_hdr, __pyx_n_u_product_configuration); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 871, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_product_config = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":872
 *     # product_config substructure
 *     product_config = product_hdr['product_configuration']
 *     _unpack_key(product_config, 'structure_header', STRUCTURE_HEADER)             # <<<<<<<<<<<<<<
 *     _unpack_key(product_config, 'generation_time', YMDS_TIME)
 *     _unpack_key(product_config, 'sweep_ingest_time', YMDS_TIME)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 872, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_STRUCTURE_HEADER); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 872, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_product_config, __pyx_n_u_structure_header, __pyx_t_7};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 872, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_product_config, __pyx_n_u_structure_header, __pyx_t_7};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 872, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  {
    __pyx_t_3 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 872, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(__pyx_v_product_config);
    __Pyx_GIVEREF(__pyx_v_product_config);
    PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_6, __pyx_v_product_config);
    __Pyx_INCREF(__pyx_n_u_structure_header);
    __Pyx_GIVEREF(__pyx_n_u_structure_header);
    PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_6, __pyx_n_u_structure_header);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_3, 2+__pyx_t_6, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 872, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":873
 *     product_config = product_hdr['product_configuration']
 *     _unpack_key(product_config, 'structure_header', STRUCTURE_HEADER)
 *     _unpack_key(product_config, 'generation_time', YMDS_TIME)             # <<<<<<<<<<<<<<
 *     _unpack_key(product_config, 'sweep_ingest_time', YMDS_TIME)
 *     _unpack_key(product_config, 'file_ingest_time', YMDS_TIME)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 873, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_YMDS_TIME); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 873, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_product_config, __pyx_n_u_generation_time, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 873, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_product_config, __pyx_n_u_generation_time, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 873, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_4 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 873, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__pyx_t_7) {
      __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_7); __pyx_t_7 = NULL;
    }
    __Pyx_INCREF(__pyx_v_product_config);
    __Pyx_GIVEREF(__pyx_v_product_config);
    PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_6, __pyx_v_product_config);
    __Pyx_INCREF(__pyx_n_u_generation_time);
    __Pyx_GIVEREF(__pyx_n_u_generation_time);
    PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_6, __pyx_n_u_generation_time);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 2+__pyx_t_6, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 873, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":874
 *     _unpack_key(product_config, 'structure_header', STRUCTURE_HEADER)
 *     _unpack_key(product_config, 'generation_time', YMDS_TIME)
 *     _unpack_key(product_config, 'sweep_ingest_time', YMDS_TIME)             # <<<<<<<<<<<<<<
 *     _unpack_key(product_config, 'file_ingest_time', YMDS_TIME)
 *     _unpack_key(product_config, 'color_scale_def', COLOR_SCALE_DEF)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 874, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_YMDS_TIME); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 874, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_product_config, __pyx_n_u_sweep_ingest_time, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 874, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_product_config, __pyx_n_u_sweep_ingest_time, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 874, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 874, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_product_config);
    __Pyx_GIVEREF(__pyx_v_product_config);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_v_product_config);
    __Pyx_INCREF(__pyx_n_u_sweep_ingest_time);
    __Pyx_GIVEREF(__pyx_n_u_sweep_ingest_time);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_n_u_sweep_ingest_time);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 2+__pyx_t_6, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 874, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":875
 *     _unpack_key(product_config, 'generation_time', YMDS_TIME)
 *     _unpack_key(product_config, 'sweep_ingest_time', YMDS_TIME)
 *     _unpack_key(product_config, 'file_ingest_time', YMDS_TIME)             # <<<<<<<<<<<<<<
 *     _unpack_key(product_config, 'color_scale_def', COLOR_SCALE_DEF)
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 875, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_YMDS_TIME); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 875, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_product_config, __pyx_n_u_file_ingest_time, __pyx_t_7};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 875, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_product_config, __pyx_n_u_file_ingest_time, __pyx_t_7};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 875, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  {
    __pyx_t_3 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 875, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(__pyx_v_product_config);
    __Pyx_GIVEREF(__pyx_v_product_config);
    PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_6, __pyx_v_product_config);
    __Pyx_INCREF(__pyx_n_u_file_ingest_time);
    __Pyx_GIVEREF(__pyx_n_u_file_ingest_time);
    PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_6, __pyx_n_u_file_ingest_time);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_3, 2+__pyx_t_6, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 875, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":876
 *     _unpack_key(product_config, 'sweep_ingest_time', YMDS_TIME)
 *     _unpack_key(product_config, 'file_ingest_time', YMDS_TIME)
 *     _unpack_key(product_config, 'color_scale_def', COLOR_SCALE_DEF)             # <<<<<<<<<<<<<<
 * 
 *     # product_end substructure
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 876, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_COLOR_SCALE_DEF); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 876, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_product_config, __pyx_n_u_color_scale_def, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 876, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_product_config, __pyx_n_u_color_scale_def, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 876, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_4 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 876, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__pyx_t_7) {
      __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_7); __pyx_t_7 = NULL;
    }
    __Pyx_INCREF(__pyx_v_product_config);
    __Pyx_GIVEREF(__pyx_v_product_config);
    PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_6, __pyx_v_product_config);
    __Pyx_INCREF(__pyx_n_u_color_scale_def);
    __Pyx_GIVEREF(__pyx_n_u_color_scale_def);
    PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_6, __pyx_n_u_color_scale_def);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 2+__pyx_t_6, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 876, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":879
 * 
 *     # product_end substructure
 *     product_end = product_hdr['product_end']             # <<<<<<<<<<<<<<
 *     _unpack_key(product_end, 'ingest_time', YMDS_TIME)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_product_hdr, __pyx_n_u_product_end); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 879, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_product_end = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":880
 *     # product_end substructure
 *     product_end = product_hdr['product_end']
 *     _unpack_key(product_end, 'ingest_time', YMDS_TIME)             # <<<<<<<<<<<<<<
 * 
 *     return product_hdr
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 880, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_YMDS_TIME); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 880, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_product_end, __pyx_n_u_ingest_time, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 880, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_product_end, __pyx_n_u_ingest_time, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 880, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 880, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_product_end);
    __Pyx_GIVEREF(__pyx_v_product_end);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_v_product_end);
    __Pyx_INCREF(__pyx_n_u_ingest_time);
    __Pyx_GIVEREF(__pyx_n_u_ingest_time);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_n_u_ingest_time);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 2+__pyx_t_6, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 880, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":882
 *     _unpack_key(product_end, 'ingest_time', YMDS_TIME)
 * 
 *     return product_hdr             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_product_hdr);
  __pyx_r = __pyx_v_product_hdr;
  goto __pyx_L0;

  /* "pyart/io/_sigmetfile.pyx":856
 * 
 * 
 * def _unpack_product_hdr(record):             # <<<<<<<<<<<<<<
 *     """
 *     Return a dict with the unpacked product_hdr from the first record.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("pyart.io._sigmetfile._unpack_product_hdr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_product_hdr);
  __Pyx_XDECREF(__pyx_v_product_config);
  __Pyx_XDECREF(__pyx_v_product_end);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_sigmetfile.pyx":885
 * 
 * 
 * def _unpack_ingest_header(record):             # <<<<<<<<<<<<<<
 *     """
 *     Return a dict with the unpacked ingest_header from the second record.
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_25_unpack_ingest_header(PyObject *__pyx_self, PyObject *__pyx_v_record); /*proto*/
static char __pyx_doc_5pyart_2io_11_sigmetfile_24_unpack_ingest_header[] = "\n    Return a dict with the unpacked ingest_header from the second record.\n    ";
static PyMethodDef __pyx_mdef_5pyart_2io_11_sigmetfile_25_unpack_ingest_header = {"_unpack_ingest_header", (PyCFunction)__pyx_pw_5pyart_2io_11_sigmetfile_25_unpack_ingest_header, METH_O, __pyx_doc_5pyart_2io_11_sigmetfile_24_unpack_ingest_header};
static PyObject *__pyx_pw_5pyart_2io_11_sigmetfile_25_unpack_ingest_header(PyObject *__pyx_self, PyObject *__pyx_v_record) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_unpack_ingest_header (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_11_sigmetfile_24_unpack_ingest_header(__pyx_self, ((PyObject *)__pyx_v_record));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_11_sigmetfile_24_unpack_ingest_header(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_record) {
  PyObject *__pyx_v_ingest_header = NULL;
  PyObject *__pyx_v_ingest_configuration = NULL;
  PyObject *__pyx_v_task_configuration = NULL;
  PyObject *__pyx_v_task_dsp_info = NULL;
  PyObject *__pyx_v_task_end_info = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_unpack_ingest_header", 0);

  /* "pyart/io/_sigmetfile.pyx":891
 * 
 *     # unpack the ingest_header structure from the second_record
 *     ingest_header = _unpack_structure(record[:4884], INGEST_HEADER)             # <<<<<<<<<<<<<<
 * 
 *     # ingest_header substructure
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_structure); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 891, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetSlice(__pyx_v_record, 0, 0x1314, NULL, NULL, &__pyx_slice__18, 0, 1, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 891, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_INGEST_HEADER); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 891, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 891, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 891, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 891, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 891, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_ingest_header = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":894
 * 
 *     # ingest_header substructure
 *     _unpack_key(ingest_header, 'structure_header', STRUCTURE_HEADER)             # <<<<<<<<<<<<<<
 *     _unpack_key(ingest_header, 'ingest_configuration',
 *                 INGEST_CONFIGURATION)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 894, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_STRUCTURE_HEADER); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 894, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_ingest_header, __pyx_n_u_structure_header, __pyx_t_7};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 894, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_ingest_header, __pyx_n_u_structure_header, __pyx_t_7};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 894, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  {
    __pyx_t_3 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 894, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(__pyx_v_ingest_header);
    __Pyx_GIVEREF(__pyx_v_ingest_header);
    PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_6, __pyx_v_ingest_header);
    __Pyx_INCREF(__pyx_n_u_structure_header);
    __Pyx_GIVEREF(__pyx_n_u_structure_header);
    PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_6, __pyx_n_u_structure_header);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_3, 2+__pyx_t_6, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 894, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":895
 *     # ingest_header substructure
 *     _unpack_key(ingest_header, 'structure_header', STRUCTURE_HEADER)
 *     _unpack_key(ingest_header, 'ingest_configuration',             # <<<<<<<<<<<<<<
 *                 INGEST_CONFIGURATION)
 *     _unpack_key(ingest_header, 'task_configuration', TASK_CONFIGURATION)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 895, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "pyart/io/_sigmetfile.pyx":896
 *     _unpack_key(ingest_header, 'structure_header', STRUCTURE_HEADER)
 *     _unpack_key(ingest_header, 'ingest_configuration',
 *                 INGEST_CONFIGURATION)             # <<<<<<<<<<<<<<
 *     _unpack_key(ingest_header, 'task_configuration', TASK_CONFIGURATION)
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_INGEST_CONFIGURATION); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 896, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_ingest_header, __pyx_n_u_ingest_configuration, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 895, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_ingest_header, __pyx_n_u_ingest_configuration, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 895, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_4 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 895, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__pyx_t_7) {
      __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_7); __pyx_t_7 = NULL;
    }
    __Pyx_INCREF(__pyx_v_ingest_header);
    __Pyx_GIVEREF(__pyx_v_ingest_header);
    PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_6, __pyx_v_ingest_header);
    __Pyx_INCREF(__pyx_n_u_ingest_configuration);
    __Pyx_GIVEREF(__pyx_n_u_ingest_configuration);
    PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_6, __pyx_n_u_ingest_configuration);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 2+__pyx_t_6, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 895, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":897
 *     _unpack_key(ingest_header, 'ingest_configuration',
 *                 INGEST_CONFIGURATION)
 *     _unpack_key(ingest_header, 'task_configuration', TASK_CONFIGURATION)             # <<<<<<<<<<<<<<
 * 
 *     # ingest_configuration substructure
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 897, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_TASK_CONFIGURATION); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 897, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_ingest_header, __pyx_n_u_task_configuration, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 897, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_ingest_header, __pyx_n_u_task_configuration, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 897, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 897, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_ingest_header);
    __Pyx_GIVEREF(__pyx_v_ingest_header);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_v_ingest_header);
    __Pyx_INCREF(__pyx_n_u_task_configuration);
    __Pyx_GIVEREF(__pyx_n_u_task_configuration);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_n_u_task_configuration);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 2+__pyx_t_6, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 897, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":900
 * 
 *     # ingest_configuration substructure
 *     ingest_configuration = ingest_header['ingest_configuration']             # <<<<<<<<<<<<<<
 *     _unpack_key(ingest_configuration, 'volume_scan_start_time', YMDS_TIME)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_ingest_header, __pyx_n_u_ingest_configuration); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 900, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_ingest_configuration = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":901
 *     # ingest_configuration substructure
 *     ingest_configuration = ingest_header['ingest_configuration']
 *     _unpack_key(ingest_configuration, 'volume_scan_start_time', YMDS_TIME)             # <<<<<<<<<<<<<<
 * 
 *     # task_configuration substructure
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 901, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_YMDS_TIME); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 901, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_ingest_configuration, __pyx_n_u_volume_scan_start_time, __pyx_t_7};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 901, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_ingest_configuration, __pyx_n_u_volume_scan_start_time, __pyx_t_7};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 901, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  {
    __pyx_t_3 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 901, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(__pyx_v_ingest_configuration);
    __Pyx_GIVEREF(__pyx_v_ingest_configuration);
    PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_6, __pyx_v_ingest_configuration);
    __Pyx_INCREF(__pyx_n_u_volume_scan_start_time);
    __Pyx_GIVEREF(__pyx_n_u_volume_scan_start_time);
    PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_6, __pyx_n_u_volume_scan_start_time);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_3, 2+__pyx_t_6, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 901, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":904
 * 
 *     # task_configuration substructure
 *     task_configuration = ingest_header['task_configuration']             # <<<<<<<<<<<<<<
 *     _unpack_key(task_configuration, 'structure_header', STRUCTURE_HEADER)
 *     _unpack_key(task_configuration, 'task_sched_info', TASK_SCHED_INFO)
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_ingest_header, __pyx_n_u_task_configuration); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 904, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_task_configuration = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":905
 *     # task_configuration substructure
 *     task_configuration = ingest_header['task_configuration']
 *     _unpack_key(task_configuration, 'structure_header', STRUCTURE_HEADER)             # <<<<<<<<<<<<<<
 *     _unpack_key(task_configuration, 'task_sched_info', TASK_SCHED_INFO)
 *     _unpack_key(task_configuration, 'task_dsp_info', TASK_DSP_INFO)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 905, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_STRUCTURE_HEADER); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 905, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_task_configuration, __pyx_n_u_structure_header, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 905, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_task_configuration, __pyx_n_u_structure_header, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 905, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_4 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 905, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__pyx_t_7) {
      __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_7); __pyx_t_7 = NULL;
    }
    __Pyx_INCREF(__pyx_v_task_configuration);
    __Pyx_GIVEREF(__pyx_v_task_configuration);
    PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_6, __pyx_v_task_configuration);
    __Pyx_INCREF(__pyx_n_u_structure_header);
    __Pyx_GIVEREF(__pyx_n_u_structure_header);
    PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_6, __pyx_n_u_structure_header);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 2+__pyx_t_6, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 905, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":906
 *     task_configuration = ingest_header['task_configuration']
 *     _unpack_key(task_configuration, 'structure_header', STRUCTURE_HEADER)
 *     _unpack_key(task_configuration, 'task_sched_info', TASK_SCHED_INFO)             # <<<<<<<<<<<<<<
 *     _unpack_key(task_configuration, 'task_dsp_info', TASK_DSP_INFO)
 *     _unpack_key(task_configuration, 'task_calib_info', TASK_CALIB_INFO)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 906, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_TASK_SCHED_INFO); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 906, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_task_configuration, __pyx_n_u_task_sched_info, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 906, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_task_configuration, __pyx_n_u_task_sched_info, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 906, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 906, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_task_configuration);
    __Pyx_GIVEREF(__pyx_v_task_configuration);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_v_task_configuration);
    __Pyx_INCREF(__pyx_n_u_task_sched_info);
    __Pyx_GIVEREF(__pyx_n_u_task_sched_info);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_n_u_task_sched_info);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 2+__pyx_t_6, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 906, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":907
 *     _unpack_key(task_configuration, 'structure_header', STRUCTURE_HEADER)
 *     _unpack_key(task_configuration, 'task_sched_info', TASK_SCHED_INFO)
 *     _unpack_key(task_configuration, 'task_dsp_info', TASK_DSP_INFO)             # <<<<<<<<<<<<<<
 *     _unpack_key(task_configuration, 'task_calib_info', TASK_CALIB_INFO)
 *     _unpack_key(task_configuration, 'task_range_info', TASK_RANGE_INFO)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 907, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_TASK_DSP_INFO); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 907, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_task_configuration, __pyx_n_u_task_dsp_info, __pyx_t_7};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 907, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_task_configuration, __pyx_n_u_task_dsp_info, __pyx_t_7};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 907, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  {
    __pyx_t_3 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 907, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(__pyx_v_task_configuration);
    __Pyx_GIVEREF(__pyx_v_task_configuration);
    PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_6, __pyx_v_task_configuration);
    __Pyx_INCREF(__pyx_n_u_task_dsp_info);
    __Pyx_GIVEREF(__pyx_n_u_task_dsp_info);
    PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_6, __pyx_n_u_task_dsp_info);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_3, 2+__pyx_t_6, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 907, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":908
 *     _unpack_key(task_configuration, 'task_sched_info', TASK_SCHED_INFO)
 *     _unpack_key(task_configuration, 'task_dsp_info', TASK_DSP_INFO)
 *     _unpack_key(task_configuration, 'task_calib_info', TASK_CALIB_INFO)             # <<<<<<<<<<<<<<
 *     _unpack_key(task_configuration, 'task_range_info', TASK_RANGE_INFO)
 *     _unpack_key(task_configuration, 'task_scan_info', TASK_SCAN_INFO)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 908, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_TASK_CALIB_INFO); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 908, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_task_configuration, __pyx_n_u_task_calib_info, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 908, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_task_configuration, __pyx_n_u_task_calib_info, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 908, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_4 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 908, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__pyx_t_7) {
      __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_7); __pyx_t_7 = NULL;
    }
    __Pyx_INCREF(__pyx_v_task_configuration);
    __Pyx_GIVEREF(__pyx_v_task_configuration);
    PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_6, __pyx_v_task_configuration);
    __Pyx_INCREF(__pyx_n_u_task_calib_info);
    __Pyx_GIVEREF(__pyx_n_u_task_calib_info);
    PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_6, __pyx_n_u_task_calib_info);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 2+__pyx_t_6, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 908, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":909
 *     _unpack_key(task_configuration, 'task_dsp_info', TASK_DSP_INFO)
 *     _unpack_key(task_configuration, 'task_calib_info', TASK_CALIB_INFO)
 *     _unpack_key(task_configuration, 'task_range_info', TASK_RANGE_INFO)             # <<<<<<<<<<<<<<
 *     _unpack_key(task_configuration, 'task_scan_info', TASK_SCAN_INFO)
 *     _unpack_key(task_configuration, 'task_misc_info', TASK_MISC_INFO)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 909, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_TASK_RANGE_INFO); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 909, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_task_configuration, __pyx_n_u_task_range_info, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 909, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_task_configuration, __pyx_n_u_task_range_info, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 909, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 909, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_task_configuration);
    __Pyx_GIVEREF(__pyx_v_task_configuration);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_v_task_configuration);
    __Pyx_INCREF(__pyx_n_u_task_range_info);
    __Pyx_GIVEREF(__pyx_n_u_task_range_info);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_n_u_task_range_info);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 2+__pyx_t_6, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 909, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":910
 *     _unpack_key(task_configuration, 'task_calib_info', TASK_CALIB_INFO)
 *     _unpack_key(task_configuration, 'task_range_info', TASK_RANGE_INFO)
 *     _unpack_key(task_configuration, 'task_scan_info', TASK_SCAN_INFO)             # <<<<<<<<<<<<<<
 *     _unpack_key(task_configuration, 'task_misc_info', TASK_MISC_INFO)
 *     _unpack_key(task_configuration, 'task_end_info', TASK_END_INFO)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 910, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_TASK_SCAN_INFO); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 910, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_task_configuration, __pyx_n_u_task_scan_info, __pyx_t_7};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 910, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_task_configuration, __pyx_n_u_task_scan_info, __pyx_t_7};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 910, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  {
    __pyx_t_3 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 910, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(__pyx_v_task_configuration);
    __Pyx_GIVEREF(__pyx_v_task_configuration);
    PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_6, __pyx_v_task_configuration);
    __Pyx_INCREF(__pyx_n_u_task_scan_info);
    __Pyx_GIVEREF(__pyx_n_u_task_scan_info);
    PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_6, __pyx_n_u_task_scan_info);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_3, 2+__pyx_t_6, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 910, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":911
 *     _unpack_key(task_configuration, 'task_range_info', TASK_RANGE_INFO)
 *     _unpack_key(task_configuration, 'task_scan_info', TASK_SCAN_INFO)
 *     _unpack_key(task_configuration, 'task_misc_info', TASK_MISC_INFO)             # <<<<<<<<<<<<<<
 *     _unpack_key(task_configuration, 'task_end_info', TASK_END_INFO)
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 911, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_TASK_MISC_INFO); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 911, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_task_configuration, __pyx_n_u_task_misc_info, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 911, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_task_configuration, __pyx_n_u_task_misc_info, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 911, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_4 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 911, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__pyx_t_7) {
      __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_7); __pyx_t_7 = NULL;
    }
    __Pyx_INCREF(__pyx_v_task_configuration);
    __Pyx_GIVEREF(__pyx_v_task_configuration);
    PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_6, __pyx_v_task_configuration);
    __Pyx_INCREF(__pyx_n_u_task_misc_info);
    __Pyx_GIVEREF(__pyx_n_u_task_misc_info);
    PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_6, __pyx_n_u_task_misc_info);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 2+__pyx_t_6, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 911, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":912
 *     _unpack_key(task_configuration, 'task_scan_info', TASK_SCAN_INFO)
 *     _unpack_key(task_configuration, 'task_misc_info', TASK_MISC_INFO)
 *     _unpack_key(task_configuration, 'task_end_info', TASK_END_INFO)             # <<<<<<<<<<<<<<
 * 
 *     # task_dsp_info substructure
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 912, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_TASK_END_INFO); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 912, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_task_configuration, __pyx_n_u_task_end_info, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 912, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_task_configuration, __pyx_n_u_task_end_info, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 912, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 912, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_task_configuration);
    __Pyx_GIVEREF(__pyx_v_task_configuration);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_v_task_configuration);
    __Pyx_INCREF(__pyx_n_u_task_end_info);
    __Pyx_GIVEREF(__pyx_n_u_task_end_info);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_n_u_task_end_info);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 2+__pyx_t_6, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 912, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":915
 * 
 *     # task_dsp_info substructure
 *     task_dsp_info = task_configuration['task_dsp_info']             # <<<<<<<<<<<<<<
 *     _unpack_key(task_dsp_info, 'current_data_type_mask', DSP_DATA_MASK)
 *     _unpack_key(task_dsp_info, 'original_data_type_mask', DSP_DATA_MASK)
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_task_configuration, __pyx_n_u_task_dsp_info); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 915, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_task_dsp_info = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":916
 *     # task_dsp_info substructure
 *     task_dsp_info = task_configuration['task_dsp_info']
 *     _unpack_key(task_dsp_info, 'current_data_type_mask', DSP_DATA_MASK)             # <<<<<<<<<<<<<<
 *     _unpack_key(task_dsp_info, 'original_data_type_mask', DSP_DATA_MASK)
 *     _unpack_key(task_dsp_info, 'task_dsp_mode', TASK_DSP_MODE_BATCH)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 916, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_DSP_DATA_MASK); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 916, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_task_dsp_info, __pyx_n_u_current_data_type_mask, __pyx_t_7};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 916, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_task_dsp_info, __pyx_n_u_current_data_type_mask, __pyx_t_7};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 916, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  {
    __pyx_t_3 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 916, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(__pyx_v_task_dsp_info);
    __Pyx_GIVEREF(__pyx_v_task_dsp_info);
    PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_6, __pyx_v_task_dsp_info);
    __Pyx_INCREF(__pyx_n_u_current_data_type_mask);
    __Pyx_GIVEREF(__pyx_n_u_current_data_type_mask);
    PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_6, __pyx_n_u_current_data_type_mask);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_3, 2+__pyx_t_6, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 916, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":917
 *     task_dsp_info = task_configuration['task_dsp_info']
 *     _unpack_key(task_dsp_info, 'current_data_type_mask', DSP_DATA_MASK)
 *     _unpack_key(task_dsp_info, 'original_data_type_mask', DSP_DATA_MASK)             # <<<<<<<<<<<<<<
 *     _unpack_key(task_dsp_info, 'task_dsp_mode', TASK_DSP_MODE_BATCH)
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 917, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_DSP_DATA_MASK); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 917, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_task_dsp_info, __pyx_n_u_original_data_type_mask, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 917, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_task_dsp_info, __pyx_n_u_original_data_type_mask, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 917, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_4 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 917, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__pyx_t_7) {
      __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_7); __pyx_t_7 = NULL;
    }
    __Pyx_INCREF(__pyx_v_task_dsp_info);
    __Pyx_GIVEREF(__pyx_v_task_dsp_info);
    PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_6, __pyx_v_task_dsp_info);
    __Pyx_INCREF(__pyx_n_u_original_data_type_mask);
    __Pyx_GIVEREF(__pyx_n_u_original_data_type_mask);
    PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_6, __pyx_n_u_original_data_type_mask);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 2+__pyx_t_6, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 917, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":918
 *     _unpack_key(task_dsp_info, 'current_data_type_mask', DSP_DATA_MASK)
 *     _unpack_key(task_dsp_info, 'original_data_type_mask', DSP_DATA_MASK)
 *     _unpack_key(task_dsp_info, 'task_dsp_mode', TASK_DSP_MODE_BATCH)             # <<<<<<<<<<<<<<
 * 
 *     # task_scan_info substructure
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 918, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_TASK_DSP_MODE_BATCH); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 918, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_task_dsp_info, __pyx_n_u_task_dsp_mode, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 918, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_task_dsp_info, __pyx_n_u_task_dsp_mode, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 918, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 918, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_task_dsp_info);
    __Pyx_GIVEREF(__pyx_v_task_dsp_info);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_v_task_dsp_info);
    __Pyx_INCREF(__pyx_n_u_task_dsp_mode);
    __Pyx_GIVEREF(__pyx_n_u_task_dsp_mode);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_n_u_task_dsp_mode);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 2+__pyx_t_6, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 918, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":928
 * 
 *     # task_end_info substructure
 *     task_end_info = task_configuration['task_end_info']             # <<<<<<<<<<<<<<
 *     _unpack_key(task_end_info, 'task_data_time', YMDS_TIME)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_task_configuration, __pyx_n_u_task_end_info); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 928, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_task_end_info = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":929
 *     # task_end_info substructure
 *     task_end_info = task_configuration['task_end_info']
 *     _unpack_key(task_end_info, 'task_data_time', YMDS_TIME)             # <<<<<<<<<<<<<<
 * 
 *     return ingest_header
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_unpack_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 929, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_YMDS_TIME); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 929, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_task_end_info, __pyx_n_u_task_data_time, __pyx_t_7};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 929, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_task_end_info, __pyx_n_u_task_data_time, __pyx_t_7};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 929, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  {
    __pyx_t_3 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 929, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(__pyx_v_task_end_info);
    __Pyx_GIVEREF(__pyx_v_task_end_info);
    PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_6, __pyx_v_task_end_info);
    __Pyx_INCREF(__pyx_n_u_task_data_time);
    __Pyx_GIVEREF(__pyx_n_u_task_data_time);
    PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_6, __pyx_n_u_task_data_time);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_3, 2+__pyx_t_6, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 929, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":931
 *     _unpack_key(task_end_info, 'task_data_time', YMDS_TIME)
 * 
 *     return ingest_header             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_ingest_header);
  __pyx_r = __pyx_v_ingest_header;
  goto __pyx_L0;

  /* "pyart/io/_sigmetfile.pyx":885
 * 
 * 
 * def _unpack_ingest_header(record):             # <<<<<<<<<<<<<<
 *     """
 *     Return a dict with the unpacked ingest_header from the second record.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("pyart.io._sigmetfile._unpack_ingest_header", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ingest_header);
  __Pyx_XDECREF(__pyx_v_ingest_configuration);
  __Pyx_XDECREF(__pyx_v_task_configuration);
  __Pyx_XDECREF(__pyx_v_task_dsp_info);
  __Pyx_XDECREF(__pyx_v_task_end_info);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":735
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":736
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 736, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":735
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":738
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":739
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 739, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":738
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":741
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":742
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 742, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":741
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":744
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":745
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 745, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":744
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":747
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":748
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 748, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":747
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":750
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyDataType_SHAPE(PyArray_Descr *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("PyDataType_SHAPE", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":751
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  __pyx_t_1 = (PyDataType_HASSUBARRAY(__pyx_v_d) != 0);
  if (__pyx_t_1) {

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":752
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape             # <<<<<<<<<<<<<<
 *     else:
 *         return ()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject*)__pyx_v_d->subarray->shape));
    __pyx_r = ((PyObject*)__pyx_v_d->subarray->shape);
    goto __pyx_L0;

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":751
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  }

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":754
 *         return <tuple>d.subarray.shape
 *     else:
 *         return ()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_empty_tuple);
    __pyx_r = __pyx_empty_tuple;
    goto __pyx_L0;
  }

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":750
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":929
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_array_base", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":930
 * 
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!             # <<<<<<<<<<<<<<
 *     PyArray_SetBaseObject(arr, base)
 * 
 */
  Py_INCREF(__pyx_v_base);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":931
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
 */
  (void)(PyArray_SetBaseObject(__pyx_v_arr, __pyx_v_base));

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":929
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":933
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_v_base;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_array_base", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":934
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)             # <<<<<<<<<<<<<<
 *     if base is NULL:
 *         return None
 */
  __pyx_v_base = PyArray_BASE(__pyx_v_arr);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":935
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  __pyx_t_1 = ((__pyx_v_base == NULL) != 0);
  if (__pyx_t_1) {

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":936
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     return <object>base
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":935
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  }

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":937
 *     if base is NULL:
 *         return None
 *     return <object>base             # <<<<<<<<<<<<<<
 * 
 * # Versions of the import_* functions which are more suitable for
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_base));
  __pyx_r = ((PyObject *)__pyx_v_base);
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":933
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":941
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_array(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_array", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":942
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":943
 * cdef inline int import_array() except -1:
 *     try:
 *         __pyx_import_array()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")
 */
      __pyx_t_4 = _import_array(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 943, __pyx_L3_error)

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":942
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":944
 *     try:
 *         __pyx_import_array()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 944, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":945
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__19, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 945, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 945, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":942
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":941
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":947
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_umath(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_umath", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":948
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":949
 * cdef inline int import_umath() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 949, __pyx_L3_error)

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":948
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":950
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 950, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":951
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__20, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 951, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 951, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":948
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":947
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":953
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_ufunc(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_ufunc", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":954
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":955
 * cdef inline int import_ufunc() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 955, __pyx_L3_error)

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":954
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":956
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 956, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":957
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef extern from *:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__20, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 957, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 957, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":954
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":953
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":967
 * 
 * 
 * cdef inline bint is_timedelta64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.timedelta64)`
 */

static CYTHON_INLINE int __pyx_f_5numpy_is_timedelta64_object(PyObject *__pyx_v_obj) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_timedelta64_object", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":979
 *     bool
 *     """
 *     return PyObject_TypeCheck(obj, &PyTimedeltaArrType_Type)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = PyObject_TypeCheck(__pyx_v_obj, (&PyTimedeltaArrType_Type));
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":967
 * 
 * 
 * cdef inline bint is_timedelta64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.timedelta64)`
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":982
 * 
 * 
 * cdef inline bint is_datetime64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.datetime64)`
 */

static CYTHON_INLINE int __pyx_f_5numpy_is_datetime64_object(PyObject *__pyx_v_obj) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_datetime64_object", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":994
 *     bool
 *     """
 *     return PyObject_TypeCheck(obj, &PyDatetimeArrType_Type)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = PyObject_TypeCheck(__pyx_v_obj, (&PyDatetimeArrType_Type));
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":982
 * 
 * 
 * cdef inline bint is_datetime64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.datetime64)`
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":997
 * 
 * 
 * cdef inline npy_datetime get_datetime64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy datetime64 object
 */

static CYTHON_INLINE npy_datetime __pyx_f_5numpy_get_datetime64_value(PyObject *__pyx_v_obj) {
  npy_datetime __pyx_r;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":1004
 *     also needed.  That can be found using `get_datetime64_unit`.
 *     """
 *     return (<PyDatetimeScalarObject*>obj).obval             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((PyDatetimeScalarObject *)__pyx_v_obj)->obval;
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":997
 * 
 * 
 * cdef inline npy_datetime get_datetime64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy datetime64 object
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":1007
 * 
 * 
 * cdef inline npy_timedelta get_timedelta64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy timedelta64 object
 */

static CYTHON_INLINE npy_timedelta __pyx_f_5numpy_get_timedelta64_value(PyObject *__pyx_v_obj) {
  npy_timedelta __pyx_r;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":1011
 *     returns the int64 value underlying scalar numpy timedelta64 object
 *     """
 *     return (<PyTimedeltaScalarObject*>obj).obval             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((PyTimedeltaScalarObject *)__pyx_v_obj)->obval;
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":1007
 * 
 * 
 * cdef inline npy_timedelta get_timedelta64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy timedelta64 object
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":1014
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
 */

static CYTHON_INLINE NPY_DATETIMEUNIT __pyx_f_5numpy_get_datetime64_unit(PyObject *__pyx_v_obj) {
  NPY_DATETIMEUNIT __pyx_r;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":1018
 *     returns the unit part of the dtype for a numpy datetime64 object.
 *     """
 *     return <NPY_DATETIMEUNIT>(<PyDatetimeScalarObject*>obj).obmeta.base             # <<<<<<<<<<<<<<
 */
  __pyx_r = ((NPY_DATETIMEUNIT)((PyDatetimeScalarObject *)__pyx_v_obj)->obmeta.base);
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":1014
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}
static struct __pyx_vtabstruct_5pyart_2io_11_sigmetfile_SigmetFile __pyx_vtable_5pyart_2io_11_sigmetfile_SigmetFile;

static PyObject *__pyx_tp_new_5pyart_2io_11_sigmetfile_SigmetFile(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)o);
  p->__pyx_vtab = __pyx_vtabptr_5pyart_2io_11_sigmetfile_SigmetFile;
  p->debug = Py_None; Py_INCREF(Py_None);
  p->product_hdr = Py_None; Py_INCREF(Py_None);
  p->ingest_header = Py_None; Py_INCREF(Py_None);
  p->ingest_data_headers = Py_None; Py_INCREF(Py_None);
  p->data_types = Py_None; Py_INCREF(Py_None);
  p->data_type_names = Py_None; Py_INCREF(Py_None);
  p->ndata_types = Py_None; Py_INCREF(Py_None);
  p->_fh = Py_None; Py_INCREF(Py_None);
  p->_raw_product_bhdrs = Py_None; Py_INCREF(Py_None);
  p->_rbuf = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5pyart_2io_11_sigmetfile_SigmetFile(PyObject *o) {
  struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *p = (struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->debug);
  Py_CLEAR(p->product_hdr);
  Py_CLEAR(p->ingest_header);
  Py_CLEAR(p->ingest_data_headers);
  Py_CLEAR(p->data_types);
  Py_CLEAR(p->data_type_names);
  Py_CLEAR(p->ndata_types);
  Py_CLEAR(p->_fh);
  Py_CLEAR(p->_raw_product_bhdrs);
  Py_CLEAR(p->_rbuf);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_5pyart_2io_11_sigmetfile_SigmetFile(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *p = (struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)o;
  if (p->debug) {
    e = (*v)(p->debug, a); if (e) return e;
  }
  if (p->product_hdr) {
    e = (*v)(p->product_hdr, a); if (e) return e;
  }
  if (p->ingest_header) {
    e = (*v)(p->ingest_header, a); if (e) return e;
  }
  if (p->ingest_data_headers) {
    e = (*v)(p->ingest_data_headers, a); if (e) return e;
  }
  if (p->data_types) {
    e = (*v)(p->data_types, a); if (e) return e;
  }
  if (p->data_type_names) {
    e = (*v)(p->data_type_names, a); if (e) return e;
  }
  if (p->ndata_types) {
    e = (*v)(p->ndata_types, a); if (e) return e;
  }
  if (p->_fh) {
    e = (*v)(p->_fh, a); if (e) return e;
  }
  if (p->_raw_product_bhdrs) {
    e = (*v)(p->_raw_product_bhdrs, a); if (e) return e;
  }
  if (p->_rbuf) {
    e = (*v)(((PyObject *)p->_rbuf), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5pyart_2io_11_sigmetfile_SigmetFile(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *p = (struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *)o;
  tmp = ((PyObject*)p->debug);
  p->debug = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->product_hdr);
  p->product_hdr = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->ingest_header);
  p->ingest_header = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->ingest_data_headers);
  p->ingest_data_headers = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->data_types);
  p->data_types = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->data_type_names);
  p->data_type_names = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->ndata_types);
  p->ndata_types = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_fh);
  p->_fh = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_raw_product_bhdrs);
  p->_raw_product_bhdrs = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_rbuf);
  p->_rbuf = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_5pyart_2io_11_sigmetfile_10SigmetFile_debug(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_5debug_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_11_sigmetfile_10SigmetFile_debug(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_5debug_3__set__(o, v);
  }
  else {
    return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_5debug_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5pyart_2io_11_sigmetfile_10SigmetFile_product_hdr(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_11product_hdr_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_11_sigmetfile_10SigmetFile_product_hdr(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_11product_hdr_3__set__(o, v);
  }
  else {
    return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_11product_hdr_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5pyart_2io_11_sigmetfile_10SigmetFile_ingest_header(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_13ingest_header_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_11_sigmetfile_10SigmetFile_ingest_header(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_13ingest_header_3__set__(o, v);
  }
  else {
    return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_13ingest_header_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5pyart_2io_11_sigmetfile_10SigmetFile_ingest_data_headers(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_19ingest_data_headers_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_11_sigmetfile_10SigmetFile_ingest_data_headers(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_19ingest_data_headers_3__set__(o, v);
  }
  else {
    return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_19ingest_data_headers_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5pyart_2io_11_sigmetfile_10SigmetFile_data_types(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_10data_types_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_11_sigmetfile_10SigmetFile_data_types(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_10data_types_3__set__(o, v);
  }
  else {
    return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_10data_types_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5pyart_2io_11_sigmetfile_10SigmetFile_data_type_names(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_15data_type_names_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_11_sigmetfile_10SigmetFile_data_type_names(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_15data_type_names_3__set__(o, v);
  }
  else {
    return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_15data_type_names_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5pyart_2io_11_sigmetfile_10SigmetFile_ndata_types(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_11ndata_types_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_11_sigmetfile_10SigmetFile_ndata_types(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_11ndata_types_3__set__(o, v);
  }
  else {
    return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_11ndata_types_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5pyart_2io_11_sigmetfile_10SigmetFile__fh(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_3_fh_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_11_sigmetfile_10SigmetFile__fh(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_3_fh_3__set__(o, v);
  }
  else {
    return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_3_fh_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5pyart_2io_11_sigmetfile_10SigmetFile__raw_product_bhdrs(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_18_raw_product_bhdrs_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_11_sigmetfile_10SigmetFile__raw_product_bhdrs(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_18_raw_product_bhdrs_3__set__(o, v);
  }
  else {
    return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_18_raw_product_bhdrs_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5pyart_2io_11_sigmetfile_10SigmetFile__rbuf_pos(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_9_rbuf_pos_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_11_sigmetfile_10SigmetFile__rbuf_pos(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_9_rbuf_pos_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_11_sigmetfile_10SigmetFile__record_number(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_14_record_number_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_11_sigmetfile_10SigmetFile__record_number(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_14_record_number_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_5pyart_2io_11_sigmetfile_SigmetFile[] = {
  {"_determine_data_types", (PyCFunction)__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_3_determine_data_types, METH_NOARGS, __pyx_doc_5pyart_2io_11_sigmetfile_10SigmetFile_2_determine_data_types},
  {"close", (PyCFunction)__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_5close, METH_NOARGS, __pyx_doc_5pyart_2io_11_sigmetfile_10SigmetFile_4close},
  {"read_data", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_7read_data, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5pyart_2io_11_sigmetfile_10SigmetFile_6read_data},
  {"_get_sweep", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_9_get_sweep, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5pyart_2io_11_sigmetfile_10SigmetFile_8_get_sweep},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_11__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_13__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5pyart_2io_11_sigmetfile_SigmetFile[] = {
  {(char *)"debug", __pyx_getprop_5pyart_2io_11_sigmetfile_10SigmetFile_debug, __pyx_setprop_5pyart_2io_11_sigmetfile_10SigmetFile_debug, (char *)0, 0},
  {(char *)"product_hdr", __pyx_getprop_5pyart_2io_11_sigmetfile_10SigmetFile_product_hdr, __pyx_setprop_5pyart_2io_11_sigmetfile_10SigmetFile_product_hdr, (char *)0, 0},
  {(char *)"ingest_header", __pyx_getprop_5pyart_2io_11_sigmetfile_10SigmetFile_ingest_header, __pyx_setprop_5pyart_2io_11_sigmetfile_10SigmetFile_ingest_header, (char *)0, 0},
  {(char *)"ingest_data_headers", __pyx_getprop_5pyart_2io_11_sigmetfile_10SigmetFile_ingest_data_headers, __pyx_setprop_5pyart_2io_11_sigmetfile_10SigmetFile_ingest_data_headers, (char *)0, 0},
  {(char *)"data_types", __pyx_getprop_5pyart_2io_11_sigmetfile_10SigmetFile_data_types, __pyx_setprop_5pyart_2io_11_sigmetfile_10SigmetFile_data_types, (char *)0, 0},
  {(char *)"data_type_names", __pyx_getprop_5pyart_2io_11_sigmetfile_10SigmetFile_data_type_names, __pyx_setprop_5pyart_2io_11_sigmetfile_10SigmetFile_data_type_names, (char *)0, 0},
  {(char *)"ndata_types", __pyx_getprop_5pyart_2io_11_sigmetfile_10SigmetFile_ndata_types, __pyx_setprop_5pyart_2io_11_sigmetfile_10SigmetFile_ndata_types, (char *)0, 0},
  {(char *)"_fh", __pyx_getprop_5pyart_2io_11_sigmetfile_10SigmetFile__fh, __pyx_setprop_5pyart_2io_11_sigmetfile_10SigmetFile__fh, (char *)0, 0},
  {(char *)"_raw_product_bhdrs", __pyx_getprop_5pyart_2io_11_sigmetfile_10SigmetFile__raw_product_bhdrs, __pyx_setprop_5pyart_2io_11_sigmetfile_10SigmetFile__raw_product_bhdrs, (char *)0, 0},
  {(char *)"_rbuf_pos", __pyx_getprop_5pyart_2io_11_sigmetfile_10SigmetFile__rbuf_pos, __pyx_setprop_5pyart_2io_11_sigmetfile_10SigmetFile__rbuf_pos, (char *)0, 0},
  {(char *)"_record_number", __pyx_getprop_5pyart_2io_11_sigmetfile_10SigmetFile__record_number, __pyx_setprop_5pyart_2io_11_sigmetfile_10SigmetFile__record_number, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pyart_2io_11_sigmetfile_SigmetFile = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyart.io._sigmetfile.SigmetFile", /*tp_name*/
  sizeof(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pyart_2io_11_sigmetfile_SigmetFile, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    A class for accessing data from Sigmet (IRIS) product files.\n\n    Parameters\n    ----------\n    filename : str\n        Filename or file-like object.\n\n    Attributes\n    ----------\n    debug : bool\n        Set True to print out debugging information, False otherwise.\n    product_hdr : dict\n        Product_hdr structure.\n    ingest_header : dict\n        Ingest_header structure.\n    ingest_data_headers : list of dict\n        Ingest_data_header structures for each data type.  Indexed by the\n        data type name (str).  None when data has not yet been read.\n    data_types : list\n        List of data types (int) in the file.\n    data_type_names : list\n        List of data type names (stR) in the file.\n    ndata_types : int\n        Number of data types in the file.\n    _fh : file\n        Open file being read.\n    _raw_product_bhdrs : list\n        List of raw_product_bhdr structure dictionaries seperated by sweep.\n        None when data has not yet been read.\n\n    ", /*tp_doc*/
  __pyx_tp_traverse_5pyart_2io_11_sigmetfile_SigmetFile, /*tp_traverse*/
  __pyx_tp_clear_5pyart_2io_11_sigmetfile_SigmetFile, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pyart_2io_11_sigmetfile_SigmetFile, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5pyart_2io_11_sigmetfile_SigmetFile, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5pyart_2io_11_sigmetfile_10SigmetFile_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pyart_2io_11_sigmetfile_SigmetFile, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec__sigmetfile(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec__sigmetfile},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "_sigmetfile",
    __pyx_k_A_class_and_supporting_function, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_u_112s, __pyx_k_112s, sizeof(__pyx_k_112s), 0, 1, 0, 0},
  {&__pyx_kp_u_115s, __pyx_k_115s, sizeof(__pyx_k_115s), 0, 1, 0, 0},
  {&__pyx_kp_u_120s, __pyx_k_120s, sizeof(__pyx_k_120s), 0, 1, 0, 0},
  {&__pyx_kp_u_128s, __pyx_k_128s, sizeof(__pyx_k_128s), 0, 1, 0, 0},
  {&__pyx_kp_u_12s, __pyx_k_12s, sizeof(__pyx_k_12s), 0, 1, 0, 0},
  {&__pyx_kp_u_136s, __pyx_k_136s, sizeof(__pyx_k_136s), 0, 1, 0, 0},
  {&__pyx_kp_u_160s, __pyx_k_160s, sizeof(__pyx_k_160s), 0, 1, 0, 0},
  {&__pyx_kp_u_16s, __pyx_k_16s, sizeof(__pyx_k_16s), 0, 1, 0, 0},
  {&__pyx_kp_u_184s, __pyx_k_184s, sizeof(__pyx_k_184s), 0, 1, 0, 0},
  {&__pyx_kp_u_18s, __pyx_k_18s, sizeof(__pyx_k_18s), 0, 1, 0, 0},
  {&__pyx_kp_u_198s, __pyx_k_198s, sizeof(__pyx_k_198s), 0, 1, 0, 0},
  {&__pyx_kp_u_1s, __pyx_k_1s, sizeof(__pyx_k_1s), 0, 1, 0, 0},
  {&__pyx_kp_u_2, __pyx_k_2, sizeof(__pyx_k_2), 0, 1, 0, 0},
  {&__pyx_kp_u_200s, __pyx_k_200s, sizeof(__pyx_k_200s), 0, 1, 0, 0},
  {&__pyx_kp_u_204s, __pyx_k_204s, sizeof(__pyx_k_204s), 0, 1, 0, 0},
  {&__pyx_kp_u_208s, __pyx_k_208s, sizeof(__pyx_k_208s), 0, 1, 0, 0},
  {&__pyx_kp_u_228s, __pyx_k_228s, sizeof(__pyx_k_228s), 0, 1, 0, 0},
  {&__pyx_kp_u_24s, __pyx_k_24s, sizeof(__pyx_k_24s), 0, 1, 0, 0},
  {&__pyx_kp_u_256s, __pyx_k_256s, sizeof(__pyx_k_256s), 0, 1, 0, 0},
  {&__pyx_kp_u_2612s, __pyx_k_2612s, sizeof(__pyx_k_2612s), 0, 1, 0, 0},
  {&__pyx_kp_u_28s, __pyx_k_28s, sizeof(__pyx_k_28s), 0, 1, 0, 0},
  {&__pyx_kp_u_2_uint8, __pyx_k_2_uint8, sizeof(__pyx_k_2_uint8), 0, 1, 0, 0},
  {&__pyx_kp_u_2s, __pyx_k_2s, sizeof(__pyx_k_2s), 0, 1, 0, 0},
  {&__pyx_kp_u_308s, __pyx_k_308s, sizeof(__pyx_k_308s), 0, 1, 0, 0},
  {&__pyx_kp_u_320s, __pyx_k_320s, sizeof(__pyx_k_320s), 0, 1, 0, 0},
  {&__pyx_kp_u_32s, __pyx_k_32s, sizeof(__pyx_k_32s), 0, 1, 0, 0},
  {&__pyx_kp_u_36s, __pyx_k_36s, sizeof(__pyx_k_36s), 0, 1, 0, 0},
  {&__pyx_kp_u_40s, __pyx_k_40s, sizeof(__pyx_k_40s), 0, 1, 0, 0},
  {&__pyx_kp_u_480s, __pyx_k_480s, sizeof(__pyx_k_480s), 0, 1, 0, 0},
  {&__pyx_kp_u_48s, __pyx_k_48s, sizeof(__pyx_k_48s), 0, 1, 0, 0},
  {&__pyx_kp_u_4s, __pyx_k_4s, sizeof(__pyx_k_4s), 0, 1, 0, 0},
  {&__pyx_kp_u_52s, __pyx_k_52s, sizeof(__pyx_k_52s), 0, 1, 0, 0},
  {&__pyx_kp_u_6s, __pyx_k_6s, sizeof(__pyx_k_6s), 0, 1, 0, 0},
  {&__pyx_kp_u_720s, __pyx_k_720s, sizeof(__pyx_k_720s), 0, 1, 0, 0},
  {&__pyx_kp_u_732s, __pyx_k_732s, sizeof(__pyx_k_732s), 0, 1, 0, 0},
  {&__pyx_kp_u_80s, __pyx_k_80s, sizeof(__pyx_k_80s), 0, 1, 0, 0},
  {&__pyx_kp_u_8s, __pyx_k_8s, sizeof(__pyx_k_8s), 0, 1, 0, 0},
  {&__pyx_kp_u_920s, __pyx_k_920s, sizeof(__pyx_k_920s), 0, 1, 0, 0},
  {&__pyx_kp_u_94s, __pyx_k_94s, sizeof(__pyx_k_94s), 0, 1, 0, 0},
  {&__pyx_n_u_ALBEDO16, __pyx_k_ALBEDO16, sizeof(__pyx_k_ALBEDO16), 0, 1, 0, 1},
  {&__pyx_n_u_ALBEDO8, __pyx_k_ALBEDO8, sizeof(__pyx_k_ALBEDO8), 0, 1, 0, 1},
  {&__pyx_n_u_AXDIL2, __pyx_k_AXDIL2, sizeof(__pyx_k_AXDIL2), 0, 1, 0, 1},
  {&__pyx_n_u_B, __pyx_k_B, sizeof(__pyx_k_B), 0, 1, 0, 1},
  {&__pyx_n_s_BIN1, __pyx_k_BIN1, sizeof(__pyx_k_BIN1), 0, 0, 1, 1},
  {&__pyx_n_s_BIN2, __pyx_k_BIN2, sizeof(__pyx_k_BIN2), 0, 0, 1, 1},
  {&__pyx_n_s_BIN4, __pyx_k_BIN4, sizeof(__pyx_k_BIN4), 0, 0, 1, 1},
  {&__pyx_n_s_COLOR_SCALE_DEF, __pyx_k_COLOR_SCALE_DEF, sizeof(__pyx_k_COLOR_SCALE_DEF), 0, 0, 1, 1},
  {&__pyx_n_u_CSP16, __pyx_k_CSP16, sizeof(__pyx_k_CSP16), 0, 1, 0, 1},
  {&__pyx_n_u_CSP8, __pyx_k_CSP8, sizeof(__pyx_k_CSP8), 0, 1, 0, 1},
  {&__pyx_n_u_DBT, __pyx_k_DBT, sizeof(__pyx_k_DBT), 0, 1, 0, 1},
  {&__pyx_n_u_DBT2, __pyx_k_DBT2, sizeof(__pyx_k_DBT2), 0, 1, 0, 1},
  {&__pyx_n_u_DBTE16, __pyx_k_DBTE16, sizeof(__pyx_k_DBTE16), 0, 1, 0, 1},
  {&__pyx_n_u_DBTE8, __pyx_k_DBTE8, sizeof(__pyx_k_DBTE8), 0, 1, 0, 1},
  {&__pyx_n_u_DBTV16, __pyx_k_DBTV16, sizeof(__pyx_k_DBTV16), 0, 1, 0, 1},
  {&__pyx_n_u_DBTV8, __pyx_k_DBTV8, sizeof(__pyx_k_DBTV8), 0, 1, 0, 1},
  {&__pyx_n_u_DBZ, __pyx_k_DBZ, sizeof(__pyx_k_DBZ), 0, 1, 0, 1},
  {&__pyx_n_u_DBZ2, __pyx_k_DBZ2, sizeof(__pyx_k_DBZ2), 0, 1, 0, 1},
  {&__pyx_n_u_DBZC, __pyx_k_DBZC, sizeof(__pyx_k_DBZC), 0, 1, 0, 1},
  {&__pyx_n_u_DBZC2, __pyx_k_DBZC2, sizeof(__pyx_k_DBZC2), 0, 1, 0, 1},
  {&__pyx_n_u_DBZE16, __pyx_k_DBZE16, sizeof(__pyx_k_DBZE16), 0, 1, 0, 1},
  {&__pyx_n_u_DBZE8, __pyx_k_DBZE8, sizeof(__pyx_k_DBZE8), 0, 1, 0, 1},
  {&__pyx_n_u_DBZV16, __pyx_k_DBZV16, sizeof(__pyx_k_DBZV16), 0, 1, 0, 1},
  {&__pyx_n_u_DBZV8, __pyx_k_DBZV8, sizeof(__pyx_k_DBZV8), 0, 1, 0, 1},
  {&__pyx_n_u_DEFORM2, __pyx_k_DEFORM2, sizeof(__pyx_k_DEFORM2), 0, 1, 0, 1},
  {&__pyx_n_u_DIVERGE2, __pyx_k_DIVERGE2, sizeof(__pyx_k_DIVERGE2), 0, 1, 0, 1},
  {&__pyx_n_s_DSP_DATA_MASK, __pyx_k_DSP_DATA_MASK, sizeof(__pyx_k_DSP_DATA_MASK), 0, 0, 1, 1},
  {&__pyx_n_u_FLIQUID2, __pyx_k_FLIQUID2, sizeof(__pyx_k_FLIQUID2), 0, 1, 0, 1},
  {&__pyx_n_s_FLT4, __pyx_k_FLT4, sizeof(__pyx_k_FLT4), 0, 0, 1, 1},
  {&__pyx_n_s_FLT8, __pyx_k_FLT8, sizeof(__pyx_k_FLT8), 0, 0, 1, 1},
  {&__pyx_kp_u_File_truncated_or_corrupt_i_of_i, __pyx_k_File_truncated_or_corrupt_i_of_i, sizeof(__pyx_k_File_truncated_or_corrupt_i_of_i), 0, 1, 0, 0},
  {&__pyx_kp_u_Finished_loading_record, __pyx_k_Finished_loading_record, sizeof(__pyx_k_Finished_loading_record), 0, 1, 0, 0},
  {&__pyx_n_u_GMT_minute_offset_local, __pyx_k_GMT_minute_offset_local, sizeof(__pyx_k_GMT_minute_offset_local), 0, 1, 0, 1},
  {&__pyx_n_u_GMT_minute_offset_standard, __pyx_k_GMT_minute_offset_standard, sizeof(__pyx_k_GMT_minute_offset_standard), 0, 1, 0, 1},
  {&__pyx_n_u_H, __pyx_k_H, sizeof(__pyx_k_H), 0, 1, 0, 1},
  {&__pyx_n_u_HCLASS, __pyx_k_HCLASS, sizeof(__pyx_k_HCLASS), 0, 1, 0, 1},
  {&__pyx_n_u_HCLASS2, __pyx_k_HCLASS2, sizeof(__pyx_k_HCLASS2), 0, 1, 0, 1},
  {&__pyx_n_u_HDIR2, __pyx_k_HDIR2, sizeof(__pyx_k_HDIR2), 0, 1, 0, 1},
  {&__pyx_n_u_HEIGHT, __pyx_k_HEIGHT, sizeof(__pyx_k_HEIGHT), 0, 1, 0, 1},
  {&__pyx_n_u_HVEL2, __pyx_k_HVEL2, sizeof(__pyx_k_HVEL2), 0, 1, 0, 1},
  {&__pyx_n_u_I, __pyx_k_I, sizeof(__pyx_k_I), 0, 1, 0, 1},
  {&__pyx_n_s_INGEST_CONFIGURATION, __pyx_k_INGEST_CONFIGURATION, sizeof(__pyx_k_INGEST_CONFIGURATION), 0, 0, 1, 1},
  {&__pyx_n_s_INGEST_DATA_HEADER, __pyx_k_INGEST_DATA_HEADER, sizeof(__pyx_k_INGEST_DATA_HEADER), 0, 0, 1, 1},
  {&__pyx_n_s_INGEST_HEADER, __pyx_k_INGEST_HEADER, sizeof(__pyx_k_INGEST_HEADER), 0, 0, 1, 1},
  {&__pyx_n_s_ImportError, __pyx_k_ImportError, sizeof(__pyx_k_ImportError), 0, 0, 1, 1},
  {&__pyx_n_u_KDP, __pyx_k_KDP, sizeof(__pyx_k_KDP), 0, 1, 0, 1},
  {&__pyx_n_u_KDP2, __pyx_k_KDP2, sizeof(__pyx_k_KDP2), 0, 1, 0, 1},
  {&__pyx_n_u_LDRH, __pyx_k_LDRH, sizeof(__pyx_k_LDRH), 0, 1, 0, 1},
  {&__pyx_n_u_LDRH2, __pyx_k_LDRH2, sizeof(__pyx_k_LDRH2), 0, 1, 0, 1},
  {&__pyx_n_u_LDRV, __pyx_k_LDRV, sizeof(__pyx_k_LDRV), 0, 1, 0, 1},
  {&__pyx_n_u_LDRV2, __pyx_k_LDRV2, sizeof(__pyx_k_LDRV2), 0, 1, 0, 1},
  {&__pyx_n_u_LOG16, __pyx_k_LOG16, sizeof(__pyx_k_LOG16), 0, 1, 0, 1},
  {&__pyx_n_u_LOG8, __pyx_k_LOG8, sizeof(__pyx_k_LOG8), 0, 1, 0, 1},
  {&__pyx_n_s_MESSAGE, __pyx_k_MESSAGE, sizeof(__pyx_k_MESSAGE), 0, 0, 1, 1},
  {&__pyx_n_u_OTHER, __pyx_k_OTHER, sizeof(__pyx_k_OTHER), 0, 1, 0, 1},
  {&__pyx_n_u_PHIDP, __pyx_k_PHIDP, sizeof(__pyx_k_PHIDP), 0, 1, 0, 1},
  {&__pyx_n_u_PHIDP2, __pyx_k_PHIDP2, sizeof(__pyx_k_PHIDP2), 0, 1, 0, 1},
  {&__pyx_n_u_PHIH, __pyx_k_PHIH, sizeof(__pyx_k_PHIH), 0, 1, 0, 1},
  {&__pyx_n_u_PHIH2, __pyx_k_PHIH2, sizeof(__pyx_k_PHIH2), 0, 1, 0, 1},
  {&__pyx_n_u_PHIV, __pyx_k_PHIV, sizeof(__pyx_k_PHIV), 0, 1, 0, 1},
  {&__pyx_n_u_PHIV2, __pyx_k_PHIV2, sizeof(__pyx_k_PHIV2), 0, 1, 0, 1},
  {&__pyx_n_u_PMI16, __pyx_k_PMI16, sizeof(__pyx_k_PMI16), 0, 1, 0, 1},
  {&__pyx_n_u_PMI8, __pyx_k_PMI8, sizeof(__pyx_k_PMI8), 0, 1, 0, 1},
  {&__pyx_n_s_PRODUCT_CONFIGURATION, __pyx_k_PRODUCT_CONFIGURATION, sizeof(__pyx_k_PRODUCT_CONFIGURATION), 0, 0, 1, 1},
  {&__pyx_n_s_PRODUCT_END, __pyx_k_PRODUCT_END, sizeof(__pyx_k_PRODUCT_END), 0, 0, 1, 1},
  {&__pyx_n_s_PRODUCT_HDR, __pyx_k_PRODUCT_HDR, sizeof(__pyx_k_PRODUCT_HDR), 0, 0, 1, 1},
  {&__pyx_n_u_RAINRATE2, __pyx_k_RAINRATE2, sizeof(__pyx_k_RAINRATE2), 0, 1, 0, 1},
  {&__pyx_n_u_RAW, __pyx_k_RAW, sizeof(__pyx_k_RAW), 0, 1, 0, 1},
  {&__pyx_n_s_RAW_PROD_BHDR, __pyx_k_RAW_PROD_BHDR, sizeof(__pyx_k_RAW_PROD_BHDR), 0, 0, 1, 1},
  {&__pyx_n_s_RECORD_SIZE, __pyx_k_RECORD_SIZE, sizeof(__pyx_k_RECORD_SIZE), 0, 0, 1, 1},
  {&__pyx_n_u_RHOH, __pyx_k_RHOH, sizeof(__pyx_k_RHOH), 0, 1, 0, 1},
  {&__pyx_n_u_RHOH2, __pyx_k_RHOH2, sizeof(__pyx_k_RHOH2), 0, 1, 0, 1},
  {&__pyx_n_u_RHOHV, __pyx_k_RHOHV, sizeof(__pyx_k_RHOHV), 0, 1, 0, 1},
  {&__pyx_n_u_RHOHV2, __pyx_k_RHOHV2, sizeof(__pyx_k_RHOHV2), 0, 1, 0, 1},
  {&__pyx_n_u_RHOV, __pyx_k_RHOV, sizeof(__pyx_k_RHOV), 0, 1, 0, 1},
  {&__pyx_n_u_RHOV2, __pyx_k_RHOV2, sizeof(__pyx_k_RHOV2), 0, 1, 0, 1},
  {&__pyx_kp_u_Reading_ray_i_of_i, __pyx_k_Reading_ray_i_of_i, sizeof(__pyx_k_Reading_ray_i_of_i), 0, 1, 0, 0},
  {&__pyx_n_u_SHEAR, __pyx_k_SHEAR, sizeof(__pyx_k_SHEAR), 0, 1, 0, 1},
  {&__pyx_n_s_SIGMET_DATA_TYPES, __pyx_k_SIGMET_DATA_TYPES, sizeof(__pyx_k_SIGMET_DATA_TYPES), 0, 0, 1, 1},
  {&__pyx_n_s_SINT1, __pyx_k_SINT1, sizeof(__pyx_k_SINT1), 0, 0, 1, 1},
  {&__pyx_n_s_SINT2, __pyx_k_SINT2, sizeof(__pyx_k_SINT2), 0, 0, 1, 1},
  {&__pyx_n_s_SINT4, __pyx_k_SINT4, sizeof(__pyx_k_SINT4), 0, 0, 1, 1},
  {&__pyx_n_u_SNR16, __pyx_k_SNR16, sizeof(__pyx_k_SNR16), 0, 1, 0, 1},
  {&__pyx_n_u_SNR8, __pyx_k_SNR8, sizeof(__pyx_k_SNR8), 0, 1, 0, 1},
  {&__pyx_n_u_SQI, __pyx_k_SQI, sizeof(__pyx_k_SQI), 0, 1, 0, 1},
  {&__pyx_n_u_SQI2, __pyx_k_SQI2, sizeof(__pyx_k_SQI2), 0, 1, 0, 1},
  {&__pyx_n_s_STRUCTURE_HEADER, __pyx_k_STRUCTURE_HEADER, sizeof(__pyx_k_STRUCTURE_HEADER), 0, 0, 1, 1},
  {&__pyx_n_s_SigmetFile, __pyx_k_SigmetFile, sizeof(__pyx_k_SigmetFile), 0, 0, 1, 1},
  {&__pyx_n_s_TASK_CALIB_INFO, __pyx_k_TASK_CALIB_INFO, sizeof(__pyx_k_TASK_CALIB_INFO), 0, 0, 1, 1},
  {&__pyx_n_s_TASK_CONFIGURATION, __pyx_k_TASK_CONFIGURATION, sizeof(__pyx_k_TASK_CONFIGURATION), 0, 0, 1, 1},
  {&__pyx_n_s_TASK_DSP_INFO, __pyx_k_TASK_DSP_INFO, sizeof(__pyx_k_TASK_DSP_INFO), 0, 0, 1, 1},
  {&__pyx_n_s_TASK_DSP_MODE_BATCH, __pyx_k_TASK_DSP_MODE_BATCH, sizeof(__pyx_k_TASK_DSP_MODE_BATCH), 0, 0, 1, 1},
  {&__pyx_n_s_TASK_END_INFO, __pyx_k_TASK_END_INFO, sizeof(__pyx_k_TASK_END_INFO), 0, 0, 1, 1},
  {&__pyx_n_s_TASK_FILE_SCAN_INFO, __pyx_k_TASK_FILE_SCAN_INFO, sizeof(__pyx_k_TASK_FILE_SCAN_INFO), 0, 0, 1, 1},
  {&__pyx_n_s_TASK_MANUAL_SCAN_INFO, __pyx_k_TASK_MANUAL_SCAN_INFO, sizeof(__pyx_k_TASK_MANUAL_SCAN_INFO), 0, 0, 1, 1},
  {&__pyx_n_s_TASK_MISC_INFO, __pyx_k_TASK_MISC_INFO, sizeof(__pyx_k_TASK_MISC_INFO), 0, 0, 1, 1},
  {&__pyx_n_s_TASK_PPI_SCAN_INFO, __pyx_k_TASK_PPI_SCAN_INFO, sizeof(__pyx_k_TASK_PPI_SCAN_INFO), 0, 0, 1, 1},
  {&__pyx_n_s_TASK_RANGE_INFO, __pyx_k_TASK_RANGE_INFO, sizeof(__pyx_k_TASK_RANGE_INFO), 0, 0, 1, 1},
  {&__pyx_n_s_TASK_RHI_SCAN_INFO, __pyx_k_TASK_RHI_SCAN_INFO, sizeof(__pyx_k_TASK_RHI_SCAN_INFO), 0, 0, 1, 1},
  {&__pyx_n_s_TASK_SCAN_INFO, __pyx_k_TASK_SCAN_INFO, sizeof(__pyx_k_TASK_SCAN_INFO), 0, 0, 1, 1},
  {&__pyx_n_s_TASK_SCHED_INFO, __pyx_k_TASK_SCHED_INFO, sizeof(__pyx_k_TASK_SCHED_INFO), 0, 0, 1, 1},
  {&__pyx_n_u_TEMPERATURE16, __pyx_k_TEMPERATURE16, sizeof(__pyx_k_TEMPERATURE16), 0, 1, 0, 1},
  {&__pyx_n_u_TIME2, __pyx_k_TIME2, sizeof(__pyx_k_TIME2), 0, 1, 0, 1},
  {&__pyx_n_u_TURB16, __pyx_k_TURB16, sizeof(__pyx_k_TURB16), 0, 1, 0, 1},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_n_s_UINT1, __pyx_k_UINT1, sizeof(__pyx_k_UINT1), 0, 0, 1, 1},
  {&__pyx_n_s_UINT16_T, __pyx_k_UINT16_T, sizeof(__pyx_k_UINT16_T), 0, 0, 1, 1},
  {&__pyx_n_s_UINT2, __pyx_k_UINT2, sizeof(__pyx_k_UINT2), 0, 0, 1, 1},
  {&__pyx_n_s_UINT4, __pyx_k_UINT4, sizeof(__pyx_k_UINT4), 0, 0, 1, 1},
  {&__pyx_n_u_UNKNOWN_100, __pyx_k_UNKNOWN_100, sizeof(__pyx_k_UNKNOWN_100), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_101, __pyx_k_UNKNOWN_101, sizeof(__pyx_k_UNKNOWN_101), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_102, __pyx_k_UNKNOWN_102, sizeof(__pyx_k_UNKNOWN_102), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_103, __pyx_k_UNKNOWN_103, sizeof(__pyx_k_UNKNOWN_103), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_104, __pyx_k_UNKNOWN_104, sizeof(__pyx_k_UNKNOWN_104), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_105, __pyx_k_UNKNOWN_105, sizeof(__pyx_k_UNKNOWN_105), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_106, __pyx_k_UNKNOWN_106, sizeof(__pyx_k_UNKNOWN_106), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_107, __pyx_k_UNKNOWN_107, sizeof(__pyx_k_UNKNOWN_107), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_108, __pyx_k_UNKNOWN_108, sizeof(__pyx_k_UNKNOWN_108), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_109, __pyx_k_UNKNOWN_109, sizeof(__pyx_k_UNKNOWN_109), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_110, __pyx_k_UNKNOWN_110, sizeof(__pyx_k_UNKNOWN_110), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_112, __pyx_k_UNKNOWN_112, sizeof(__pyx_k_UNKNOWN_112), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_113, __pyx_k_UNKNOWN_113, sizeof(__pyx_k_UNKNOWN_113), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_114, __pyx_k_UNKNOWN_114, sizeof(__pyx_k_UNKNOWN_114), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_115, __pyx_k_UNKNOWN_115, sizeof(__pyx_k_UNKNOWN_115), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_116, __pyx_k_UNKNOWN_116, sizeof(__pyx_k_UNKNOWN_116), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_117, __pyx_k_UNKNOWN_117, sizeof(__pyx_k_UNKNOWN_117), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_118, __pyx_k_UNKNOWN_118, sizeof(__pyx_k_UNKNOWN_118), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_119, __pyx_k_UNKNOWN_119, sizeof(__pyx_k_UNKNOWN_119), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_120, __pyx_k_UNKNOWN_120, sizeof(__pyx_k_UNKNOWN_120), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_121, __pyx_k_UNKNOWN_121, sizeof(__pyx_k_UNKNOWN_121), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_122, __pyx_k_UNKNOWN_122, sizeof(__pyx_k_UNKNOWN_122), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_123, __pyx_k_UNKNOWN_123, sizeof(__pyx_k_UNKNOWN_123), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_124, __pyx_k_UNKNOWN_124, sizeof(__pyx_k_UNKNOWN_124), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_125, __pyx_k_UNKNOWN_125, sizeof(__pyx_k_UNKNOWN_125), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_126, __pyx_k_UNKNOWN_126, sizeof(__pyx_k_UNKNOWN_126), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_127, __pyx_k_UNKNOWN_127, sizeof(__pyx_k_UNKNOWN_127), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_29, __pyx_k_UNKNOWN_29, sizeof(__pyx_k_UNKNOWN_29), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_30, __pyx_k_UNKNOWN_30, sizeof(__pyx_k_UNKNOWN_30), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_31, __pyx_k_UNKNOWN_31, sizeof(__pyx_k_UNKNOWN_31), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_6, __pyx_k_UNKNOWN_6, sizeof(__pyx_k_UNKNOWN_6), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_81, __pyx_k_UNKNOWN_81, sizeof(__pyx_k_UNKNOWN_81), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_82, __pyx_k_UNKNOWN_82, sizeof(__pyx_k_UNKNOWN_82), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_83, __pyx_k_UNKNOWN_83, sizeof(__pyx_k_UNKNOWN_83), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_84, __pyx_k_UNKNOWN_84, sizeof(__pyx_k_UNKNOWN_84), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_85, __pyx_k_UNKNOWN_85, sizeof(__pyx_k_UNKNOWN_85), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_86, __pyx_k_UNKNOWN_86, sizeof(__pyx_k_UNKNOWN_86), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_87, __pyx_k_UNKNOWN_87, sizeof(__pyx_k_UNKNOWN_87), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_88, __pyx_k_UNKNOWN_88, sizeof(__pyx_k_UNKNOWN_88), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_89, __pyx_k_UNKNOWN_89, sizeof(__pyx_k_UNKNOWN_89), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_90, __pyx_k_UNKNOWN_90, sizeof(__pyx_k_UNKNOWN_90), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_91, __pyx_k_UNKNOWN_91, sizeof(__pyx_k_UNKNOWN_91), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_92, __pyx_k_UNKNOWN_92, sizeof(__pyx_k_UNKNOWN_92), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_93, __pyx_k_UNKNOWN_93, sizeof(__pyx_k_UNKNOWN_93), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_94, __pyx_k_UNKNOWN_94, sizeof(__pyx_k_UNKNOWN_94), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_95, __pyx_k_UNKNOWN_95, sizeof(__pyx_k_UNKNOWN_95), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_96, __pyx_k_UNKNOWN_96, sizeof(__pyx_k_UNKNOWN_96), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_97, __pyx_k_UNKNOWN_97, sizeof(__pyx_k_UNKNOWN_97), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_98, __pyx_k_UNKNOWN_98, sizeof(__pyx_k_UNKNOWN_98), 0, 1, 0, 1},
  {&__pyx_n_u_UNKNOWN_99, __pyx_k_UNKNOWN_99, sizeof(__pyx_k_UNKNOWN_99), 0, 1, 0, 1},
  {&__pyx_n_u_USER, __pyx_k_USER, sizeof(__pyx_k_USER), 0, 1, 0, 1},
  {&__pyx_n_u_USER2, __pyx_k_USER2, sizeof(__pyx_k_USER2), 0, 1, 0, 1},
  {&__pyx_kp_u_Unknown_type_s_returning_raw_dat, __pyx_k_Unknown_type_s_returning_raw_dat, sizeof(__pyx_k_Unknown_type_s_returning_raw_dat), 0, 1, 0, 0},
  {&__pyx_n_u_VEL, __pyx_k_VEL, sizeof(__pyx_k_VEL), 0, 1, 0, 1},
  {&__pyx_n_u_VEL2, __pyx_k_VEL2, sizeof(__pyx_k_VEL2), 0, 1, 0, 1},
  {&__pyx_n_u_VELC, __pyx_k_VELC, sizeof(__pyx_k_VELC), 0, 1, 0, 1},
  {&__pyx_n_u_VELC2, __pyx_k_VELC2, sizeof(__pyx_k_VELC2), 0, 1, 0, 1},
  {&__pyx_n_u_VIL2, __pyx_k_VIL2, sizeof(__pyx_k_VIL2), 0, 1, 0, 1},
  {&__pyx_n_u_VILD16, __pyx_k_VILD16, sizeof(__pyx_k_VILD16), 0, 1, 0, 1},
  {&__pyx_n_u_VIR16, __pyx_k_VIR16, sizeof(__pyx_k_VIR16), 0, 1, 0, 1},
  {&__pyx_n_u_VVEL2, __pyx_k_VVEL2, sizeof(__pyx_k_VVEL2), 0, 1, 0, 1},
  {&__pyx_n_u_WIDTH, __pyx_k_WIDTH, sizeof(__pyx_k_WIDTH), 0, 1, 0, 1},
  {&__pyx_n_u_WIDTH2, __pyx_k_WIDTH2, sizeof(__pyx_k_WIDTH2), 0, 1, 0, 1},
  {&__pyx_n_u_XHDR, __pyx_k_XHDR, sizeof(__pyx_k_XHDR), 0, 1, 0, 1},
  {&__pyx_n_s_YMDS_TIME, __pyx_k_YMDS_TIME, sizeof(__pyx_k_YMDS_TIME), 0, 0, 1, 1},
  {&__pyx_n_u_ZDR, __pyx_k_ZDR, sizeof(__pyx_k_ZDR), 0, 1, 0, 1},
  {&__pyx_n_u_ZDR2, __pyx_k_ZDR2, sizeof(__pyx_k_ZDR2), 0, 1, 0, 1},
  {&__pyx_n_u_ZDRC, __pyx_k_ZDRC, sizeof(__pyx_k_ZDRC), 0, 1, 0, 1},
  {&__pyx_n_u_ZDRC2, __pyx_k_ZDRC2, sizeof(__pyx_k_ZDRC2), 0, 1, 0, 1},
  {&__pyx_kp_u__15, __pyx_k__15, sizeof(__pyx_k__15), 0, 1, 0, 0},
  {&__pyx_n_u_agc_feedback_code, __pyx_k_agc_feedback_code, sizeof(__pyx_k_agc_feedback_code), 0, 1, 0, 1},
  {&__pyx_n_u_altitude_radar, __pyx_k_altitude_radar, sizeof(__pyx_k_altitude_radar), 0, 1, 0, 1},
  {&__pyx_n_u_angular_resolution_desired, __pyx_k_angular_resolution_desired, sizeof(__pyx_k_angular_resolution_desired), 0, 1, 0, 1},
  {&__pyx_n_u_antenna_offset_bow, __pyx_k_antenna_offset_bow, sizeof(__pyx_k_antenna_offset_bow), 0, 1, 0, 1},
  {&__pyx_n_u_antenna_offset_starboard, __pyx_k_antenna_offset_starboard, sizeof(__pyx_k_antenna_offset_starboard), 0, 1, 0, 1},
  {&__pyx_n_u_antenna_offset_up, __pyx_k_antenna_offset_up, sizeof(__pyx_k_antenna_offset_up), 0, 1, 0, 1},
  {&__pyx_n_u_antenna_scan_mode, __pyx_k_antenna_scan_mode, sizeof(__pyx_k_antenna_scan_mode), 0, 1, 0, 1},
  {&__pyx_n_s_append, __pyx_k_append, sizeof(__pyx_k_append), 0, 0, 1, 1},
  {&__pyx_n_u_attenuation, __pyx_k_attenuation, sizeof(__pyx_k_attenuation), 0, 1, 0, 1},
  {&__pyx_n_s_az0, __pyx_k_az0, sizeof(__pyx_k_az0), 0, 0, 1, 1},
  {&__pyx_n_s_az1, __pyx_k_az1, sizeof(__pyx_k_az1), 0, 0, 1, 1},
  {&__pyx_n_u_azimuth_0, __pyx_k_azimuth_0, sizeof(__pyx_k_azimuth_0), 0, 1, 0, 1},
  {&__pyx_n_u_azimuth_1, __pyx_k_azimuth_1, sizeof(__pyx_k_azimuth_1), 0, 1, 0, 1},
  {&__pyx_n_u_azimuth_list, __pyx_k_azimuth_list, sizeof(__pyx_k_azimuth_list), 0, 1, 0, 1},
  {&__pyx_n_u_b, __pyx_k_b, sizeof(__pyx_k_b), 0, 1, 0, 1},
  {&__pyx_n_s_bin2, __pyx_k_bin2, sizeof(__pyx_k_bin2), 0, 0, 1, 1},
  {&__pyx_n_s_bin2_to_angle, __pyx_k_bin2_to_angle, sizeof(__pyx_k_bin2_to_angle), 0, 0, 1, 1},
  {&__pyx_n_s_bin4, __pyx_k_bin4, sizeof(__pyx_k_bin4), 0, 0, 1, 1},
  {&__pyx_n_s_bin4_to_angle, __pyx_k_bin4_to_angle, sizeof(__pyx_k_bin4_to_angle), 0, 0, 1, 1},
  {&__pyx_n_s_bit, __pyx_k_bit, sizeof(__pyx_k_bit), 0, 0, 1, 1},
  {&__pyx_n_u_bit_per_bin, __pyx_k_bit_per_bin, sizeof(__pyx_k_bit_per_bin), 0, 1, 0, 1},
  {&__pyx_n_s_bool8, __pyx_k_bool8, sizeof(__pyx_k_bool8), 0, 0, 1, 1},
  {&__pyx_n_u_bytes_in_structure, __pyx_k_bytes_in_structure, sizeof(__pyx_k_bytes_in_structure), 0, 1, 0, 1},
  {&__pyx_n_u_c, __pyx_k_c, sizeof(__pyx_k_c), 0, 1, 0, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_close, __pyx_k_close, sizeof(__pyx_k_close), 0, 0, 1, 1},
  {&__pyx_n_u_clutter_correction_threshold, __pyx_k_clutter_correction_threshold, sizeof(__pyx_k_clutter_correction_threshold), 0, 1, 0, 1},
  {&__pyx_n_u_clutter_filter, __pyx_k_clutter_filter, sizeof(__pyx_k_clutter_filter), 0, 1, 0, 1},
  {&__pyx_n_u_clutter_filter_name, __pyx_k_clutter_filter_name, sizeof(__pyx_k_clutter_filter_name), 0, 1, 0, 1},
  {&__pyx_n_u_cluttermap, __pyx_k_cluttermap, sizeof(__pyx_k_cluttermap), 0, 1, 0, 1},
  {&__pyx_n_u_cluttermap_flag, __pyx_k_cluttermap_flag, sizeof(__pyx_k_cluttermap_flag), 0, 1, 0, 1},
  {&__pyx_n_u_color_scale_def, __pyx_k_color_scale_def, sizeof(__pyx_k_color_scale_def), 0, 1, 0, 1},
  {&__pyx_n_u_comment_bytes, __pyx_k_comment_bytes, sizeof(__pyx_k_comment_bytes), 0, 1, 0, 1},
  {&__pyx_n_u_comments, __pyx_k_comments, sizeof(__pyx_k_comments), 0, 1, 0, 1},
  {&__pyx_n_u_configuration_name, __pyx_k_configuration_name, sizeof(__pyx_k_configuration_name), 0, 1, 0, 1},
  {&__pyx_n_s_convert_sigmet_data, __pyx_k_convert_sigmet_data, sizeof(__pyx_k_convert_sigmet_data), 0, 0, 1, 1},
  {&__pyx_n_s_copy, __pyx_k_copy, sizeof(__pyx_k_copy), 0, 0, 1, 1},
  {&__pyx_n_u_corrected_reflectivity_threshold, __pyx_k_corrected_reflectivity_threshold, sizeof(__pyx_k_corrected_reflectivity_threshold), 0, 1, 0, 1},
  {&__pyx_n_u_current_data_type_mask, __pyx_k_current_data_type_mask, sizeof(__pyx_k_current_data_type_mask), 0, 1, 0, 1},
  {&__pyx_n_u_custom_ray_header_name, __pyx_k_custom_ray_header_name, sizeof(__pyx_k_custom_ray_header_name), 0, 1, 0, 1},
  {&__pyx_n_u_customer_storage, __pyx_k_customer_storage, sizeof(__pyx_k_customer_storage), 0, 1, 0, 1},
  {&__pyx_n_u_d, __pyx_k_d, sizeof(__pyx_k_d), 0, 1, 0, 1},
  {&__pyx_n_s_data, __pyx_k_data, sizeof(__pyx_k_data), 0, 0, 1, 1},
  {&__pyx_n_s_data_type, __pyx_k_data_type, sizeof(__pyx_k_data_type), 0, 0, 1, 1},
  {&__pyx_n_u_data_type, __pyx_k_data_type, sizeof(__pyx_k_data_type), 0, 1, 0, 1},
  {&__pyx_n_s_data_type_name, __pyx_k_data_type_name, sizeof(__pyx_k_data_type_name), 0, 0, 1, 1},
  {&__pyx_n_s_data_types, __pyx_k_data_types, sizeof(__pyx_k_data_types), 0, 0, 1, 1},
  {&__pyx_n_s_data_types_from_mask, __pyx_k_data_types_from_mask, sizeof(__pyx_k_data_types_from_mask), 0, 0, 1, 1},
  {&__pyx_n_s_datetime, __pyx_k_datetime, sizeof(__pyx_k_datetime), 0, 0, 1, 1},
  {&__pyx_n_u_day, __pyx_k_day, sizeof(__pyx_k_day), 0, 1, 0, 1},
  {&__pyx_n_s_debug, __pyx_k_debug, sizeof(__pyx_k_debug), 0, 0, 1, 1},
  {&__pyx_n_s_determine_data_types, __pyx_k_determine_data_types, sizeof(__pyx_k_determine_data_types), 0, 0, 1, 1},
  {&__pyx_n_s_dic, __pyx_k_dic, sizeof(__pyx_k_dic), 0, 0, 1, 1},
  {&__pyx_n_u_dsp_type, __pyx_k_dsp_type, sizeof(__pyx_k_dsp_type), 0, 1, 0, 1},
  {&__pyx_n_s_dtype, __pyx_k_dtype, sizeof(__pyx_k_dtype), 0, 0, 1, 1},
  {&__pyx_n_u_dual_prf_delay, __pyx_k_dual_prf_delay, sizeof(__pyx_k_dual_prf_delay), 0, 1, 0, 1},
  {&__pyx_n_u_earth_radius, __pyx_k_earth_radius, sizeof(__pyx_k_earth_radius), 0, 1, 0, 1},
  {&__pyx_n_s_el0, __pyx_k_el0, sizeof(__pyx_k_el0), 0, 0, 1, 1},
  {&__pyx_n_s_el1, __pyx_k_el1, sizeof(__pyx_k_el1), 0, 0, 1, 1},
  {&__pyx_n_u_elevation_0, __pyx_k_elevation_0, sizeof(__pyx_k_elevation_0), 0, 1, 0, 1},
  {&__pyx_n_u_elevation_1, __pyx_k_elevation_1, sizeof(__pyx_k_elevation_1), 0, 1, 0, 1},
  {&__pyx_n_u_elevation_list, __pyx_k_elevation_list, sizeof(__pyx_k_elevation_list), 0, 1, 0, 1},
  {&__pyx_n_s_empty, __pyx_k_empty, sizeof(__pyx_k_empty), 0, 0, 1, 1},
  {&__pyx_n_s_empty_like, __pyx_k_empty_like, sizeof(__pyx_k_empty_like), 0, 0, 1, 1},
  {&__pyx_n_s_enumerate, __pyx_k_enumerate, sizeof(__pyx_k_enumerate), 0, 0, 1, 1},
  {&__pyx_n_s_exp, __pyx_k_exp, sizeof(__pyx_k_exp), 0, 0, 1, 1},
  {&__pyx_n_u_extended_header_type, __pyx_k_extended_header_type, sizeof(__pyx_k_extended_header_type), 0, 1, 0, 1},
  {&__pyx_n_u_extended_product_header_offset, __pyx_k_extended_product_header_offset, sizeof(__pyx_k_extended_product_header_offset), 0, 1, 0, 1},
  {&__pyx_n_u_extended_ray_header_bytes, __pyx_k_extended_ray_header_bytes, sizeof(__pyx_k_extended_ray_header_bytes), 0, 1, 0, 1},
  {&__pyx_n_u_f, __pyx_k_f, sizeof(__pyx_k_f), 0, 1, 0, 1},
  {&__pyx_n_u_fault_status, __pyx_k_fault_status, sizeof(__pyx_k_fault_status), 0, 1, 0, 1},
  {&__pyx_n_u_file_ingest_time, __pyx_k_file_ingest_time, sizeof(__pyx_k_file_ingest_time), 0, 1, 0, 1},
  {&__pyx_n_s_filename, __pyx_k_filename, sizeof(__pyx_k_filename), 0, 0, 1, 1},
  {&__pyx_n_u_filename, __pyx_k_filename, sizeof(__pyx_k_filename), 0, 1, 0, 1},
  {&__pyx_n_s_fill_value, __pyx_k_fill_value, sizeof(__pyx_k_fill_value), 0, 0, 1, 1},
  {&__pyx_n_u_first_azimuth, __pyx_k_first_azimuth, sizeof(__pyx_k_first_azimuth), 0, 1, 0, 1},
  {&__pyx_n_u_first_bin_range, __pyx_k_first_bin_range, sizeof(__pyx_k_first_bin_range), 0, 1, 0, 1},
  {&__pyx_n_u_first_elevation, __pyx_k_first_elevation, sizeof(__pyx_k_first_elevation), 0, 1, 0, 1},
  {&__pyx_n_u_first_ray_index, __pyx_k_first_ray_index, sizeof(__pyx_k_first_ray_index), 0, 1, 0, 1},
  {&__pyx_n_u_first_ray_offset, __pyx_k_first_ray_offset, sizeof(__pyx_k_first_ray_offset), 0, 1, 0, 1},
  {&__pyx_n_u_fixed_angle, __pyx_k_fixed_angle, sizeof(__pyx_k_fixed_angle), 0, 1, 0, 1},
  {&__pyx_n_u_flag, __pyx_k_flag, sizeof(__pyx_k_flag), 0, 1, 0, 1},
  {&__pyx_n_u_flags, __pyx_k_flags, sizeof(__pyx_k_flags), 0, 1, 0, 1},
  {&__pyx_n_u_flags2, __pyx_k_flags2, sizeof(__pyx_k_flags2), 0, 1, 0, 1},
  {&__pyx_n_u_float32, __pyx_k_float32, sizeof(__pyx_k_float32), 0, 1, 0, 1},
  {&__pyx_n_s_fmt, __pyx_k_fmt, sizeof(__pyx_k_fmt), 0, 0, 1, 1},
  {&__pyx_n_u_format_version, __pyx_k_format_version, sizeof(__pyx_k_format_version), 0, 1, 0, 1},
  {&__pyx_n_s_frombuffer, __pyx_k_frombuffer, sizeof(__pyx_k_frombuffer), 0, 0, 1, 1},
  {&__pyx_n_s_full_xhdr, __pyx_k_full_xhdr, sizeof(__pyx_k_full_xhdr), 0, 0, 1, 1},
  {&__pyx_n_u_gain_control_flag, __pyx_k_gain_control_flag, sizeof(__pyx_k_gain_control_flag), 0, 1, 0, 1},
  {&__pyx_n_u_gas_attenuation, __pyx_k_gas_attenuation, sizeof(__pyx_k_gas_attenuation), 0, 1, 0, 1},
  {&__pyx_n_u_generation_time, __pyx_k_generation_time, sizeof(__pyx_k_generation_time), 0, 1, 0, 1},
  {&__pyx_n_s_get_sweep, __pyx_k_get_sweep, sizeof(__pyx_k_get_sweep), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_u_gmt_offset_minutes_local, __pyx_k_gmt_offset_minutes_local, sizeof(__pyx_k_gmt_offset_minutes_local), 0, 1, 0, 1},
  {&__pyx_n_u_gmt_offset_minutes_standard, __pyx_k_gmt_offset_minutes_standard, sizeof(__pyx_k_gmt_offset_minutes_standard), 0, 1, 0, 1},
  {&__pyx_n_u_gparam_bytes, __pyx_k_gparam_bytes, sizeof(__pyx_k_gparam_bytes), 0, 1, 0, 1},
  {&__pyx_n_u_gparm, __pyx_k_gparm, sizeof(__pyx_k_gparm), 0, 1, 0, 1},
  {&__pyx_n_u_ground_height, __pyx_k_ground_height, sizeof(__pyx_k_ground_height), 0, 1, 0, 1},
  {&__pyx_n_u_h, __pyx_k_h, sizeof(__pyx_k_h), 0, 1, 0, 1},
  {&__pyx_n_u_hardware_site, __pyx_k_hardware_site, sizeof(__pyx_k_hardware_site), 0, 1, 0, 1},
  {&__pyx_n_u_height_radar, __pyx_k_height_radar, sizeof(__pyx_k_height_radar), 0, 1, 0, 1},
  {&__pyx_n_u_height_site, __pyx_k_height_site, sizeof(__pyx_k_height_site), 0, 1, 0, 1},
  {&__pyx_n_u_horizontal_beamwidth, __pyx_k_horizontal_beamwidth, sizeof(__pyx_k_horizontal_beamwidth), 0, 1, 0, 1},
  {&__pyx_n_u_horizontal_calibration_i0, __pyx_k_horizontal_calibration_i0, sizeof(__pyx_k_horizontal_calibration_i0), 0, 1, 0, 1},
  {&__pyx_n_u_horizontal_calibration_noise, __pyx_k_horizontal_calibration_noise, sizeof(__pyx_k_horizontal_calibration_noise), 0, 1, 0, 1},
  {&__pyx_n_u_horizontal_current_noise, __pyx_k_horizontal_current_noise, sizeof(__pyx_k_horizontal_current_noise), 0, 1, 0, 1},
  {&__pyx_n_u_horizontal_i0_calibration, __pyx_k_horizontal_i0_calibration, sizeof(__pyx_k_horizontal_i0_calibration), 0, 1, 0, 1},
  {&__pyx_n_u_horizontal_noise_calibration, __pyx_k_horizontal_noise_calibration, sizeof(__pyx_k_horizontal_noise_calibration), 0, 1, 0, 1},
  {&__pyx_n_u_horizontal_radar_constant, __pyx_k_horizontal_radar_constant, sizeof(__pyx_k_horizontal_radar_constant), 0, 1, 0, 1},
  {&__pyx_n_s_i, __pyx_k_i, sizeof(__pyx_k_i), 0, 0, 1, 1},
  {&__pyx_n_u_i, __pyx_k_i, sizeof(__pyx_k_i), 0, 1, 0, 1},
  {&__pyx_n_u_i4, __pyx_k_i4, sizeof(__pyx_k_i4), 0, 1, 0, 1},
  {&__pyx_n_u_icolcnt, __pyx_k_icolcnt, sizeof(__pyx_k_icolcnt), 0, 1, 0, 1},
  {&__pyx_n_s_idh, __pyx_k_idh, sizeof(__pyx_k_idh), 0, 0, 1, 1},
  {&__pyx_n_u_iflags, __pyx_k_iflags, sizeof(__pyx_k_iflags), 0, 1, 0, 1},
  {&__pyx_n_u_ilevel_seams, __pyx_k_ilevel_seams, sizeof(__pyx_k_ilevel_seams), 0, 1, 0, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_ingest_configuration, __pyx_k_ingest_configuration, sizeof(__pyx_k_ingest_configuration), 0, 0, 1, 1},
  {&__pyx_n_u_ingest_configuration, __pyx_k_ingest_configuration, sizeof(__pyx_k_ingest_configuration), 0, 1, 0, 1},
  {&__pyx_n_u_ingest_hardware_name, __pyx_k_ingest_hardware_name, sizeof(__pyx_k_ingest_hardware_name), 0, 1, 0, 1},
  {&__pyx_n_s_ingest_header, __pyx_k_ingest_header, sizeof(__pyx_k_ingest_header), 0, 0, 1, 1},
  {&__pyx_n_u_ingest_iris_version, __pyx_k_ingest_iris_version, sizeof(__pyx_k_ingest_iris_version), 0, 1, 0, 1},
  {&__pyx_n_u_ingest_site_name, __pyx_k_ingest_site_name, sizeof(__pyx_k_ingest_site_name), 0, 1, 0, 1},
  {&__pyx_n_u_ingest_time, __pyx_k_ingest_time, sizeof(__pyx_k_ingest_time), 0, 1, 0, 1},
  {&__pyx_n_u_input_data_type, __pyx_k_input_data_type, sizeof(__pyx_k_input_data_type), 0, 1, 0, 1},
  {&__pyx_n_u_input_mask, __pyx_k_input_mask, sizeof(__pyx_k_input_mask), 0, 1, 0, 1},
  {&__pyx_n_u_int16, __pyx_k_int16, sizeof(__pyx_k_int16), 0, 1, 0, 1},
  {&__pyx_n_u_int32, __pyx_k_int32, sizeof(__pyx_k_int32), 0, 1, 0, 1},
  {&__pyx_n_u_inverse_flatting, __pyx_k_inverse_flatting, sizeof(__pyx_k_inverse_flatting), 0, 1, 0, 1},
  {&__pyx_n_u_iris_version, __pyx_k_iris_version, sizeof(__pyx_k_iris_version), 0, 1, 0, 1},
  {&__pyx_n_u_iris_version_created, __pyx_k_iris_version_created, sizeof(__pyx_k_iris_version_created), 0, 1, 0, 1},
  {&__pyx_n_s_is_bit_set, __pyx_k_is_bit_set, sizeof(__pyx_k_is_bit_set), 0, 0, 1, 1},
  {&__pyx_n_u_iset_and_scale, __pyx_k_iset_and_scale, sizeof(__pyx_k_iset_and_scale), 0, 1, 0, 1},
  {&__pyx_n_u_istart, __pyx_k_istart, sizeof(__pyx_k_istart), 0, 1, 0, 1},
  {&__pyx_n_u_istep, __pyx_k_istep, sizeof(__pyx_k_istep), 0, 1, 0, 1},
  {&__pyx_n_s_key, __pyx_k_key, sizeof(__pyx_k_key), 0, 0, 1, 1},
  {&__pyx_n_s_l, __pyx_k_l, sizeof(__pyx_k_l), 0, 0, 1, 1},
  {&__pyx_n_u_last_bin_range, __pyx_k_last_bin_range, sizeof(__pyx_k_last_bin_range), 0, 1, 0, 1},
  {&__pyx_n_u_last_run_day, __pyx_k_last_run_day, sizeof(__pyx_k_last_run_day), 0, 1, 0, 1},
  {&__pyx_n_u_last_run_time, __pyx_k_last_run_time, sizeof(__pyx_k_last_run_time), 0, 1, 0, 1},
  {&__pyx_n_u_latitude, __pyx_k_latitude, sizeof(__pyx_k_latitude), 0, 1, 0, 1},
  {&__pyx_n_u_latitude_projection, __pyx_k_latitude_projection, sizeof(__pyx_k_latitude_projection), 0, 1, 0, 1},
  {&__pyx_n_u_latitude_radar, __pyx_k_latitude_radar, sizeof(__pyx_k_latitude_radar), 0, 1, 0, 1},
  {&__pyx_n_u_ldr_bias, __pyx_k_ldr_bias, sizeof(__pyx_k_ldr_bias), 0, 1, 0, 1},
  {&__pyx_n_u_ldr_offset, __pyx_k_ldr_offset, sizeof(__pyx_k_ldr_offset), 0, 1, 0, 1},
  {&__pyx_n_u_left_azimuth_limit, __pyx_k_left_azimuth_limit, sizeof(__pyx_k_left_azimuth_limit), 0, 1, 0, 1},
  {&__pyx_n_s_like_dbt, __pyx_k_like_dbt, sizeof(__pyx_k_like_dbt), 0, 0, 1, 1},
  {&__pyx_n_s_like_dbt2, __pyx_k_like_dbt2, sizeof(__pyx_k_like_dbt2), 0, 0, 1, 1},
  {&__pyx_n_s_like_sqi, __pyx_k_like_sqi, sizeof(__pyx_k_like_sqi), 0, 0, 1, 1},
  {&__pyx_n_s_like_sqi2, __pyx_k_like_sqi2, sizeof(__pyx_k_like_sqi2), 0, 0, 1, 1},
  {&__pyx_n_u_linear_filter_first_bin, __pyx_k_linear_filter_first_bin, sizeof(__pyx_k_linear_filter_first_bin), 0, 1, 0, 1},
  {&__pyx_n_u_local_timezone, __pyx_k_local_timezone, sizeof(__pyx_k_local_timezone), 0, 1, 0, 1},
  {&__pyx_n_u_log_filter_first_bin, __pyx_k_log_filter_first_bin, sizeof(__pyx_k_log_filter_first_bin), 0, 1, 0, 1},
  {&__pyx_n_u_longitude, __pyx_k_longitude, sizeof(__pyx_k_longitude), 0, 1, 0, 1},
  {&__pyx_n_u_longitude_projection, __pyx_k_longitude_projection, sizeof(__pyx_k_longitude_projection), 0, 1, 0, 1},
  {&__pyx_n_u_longitude_radar, __pyx_k_longitude_radar, sizeof(__pyx_k_longitude_radar), 0, 1, 0, 1},
  {&__pyx_n_u_low_prf_factional, __pyx_k_low_prf_factional, sizeof(__pyx_k_low_prf_factional), 0, 1, 0, 1},
  {&__pyx_n_u_low_prf_hz, __pyx_k_low_prf_hz, sizeof(__pyx_k_low_prf_hz), 0, 1, 0, 1},
  {&__pyx_n_u_low_prf_range_averaging, __pyx_k_low_prf_range_averaging, sizeof(__pyx_k_low_prf_range_averaging), 0, 1, 0, 1},
  {&__pyx_n_u_low_prf_sample_size, __pyx_k_low_prf_sample_size, sizeof(__pyx_k_low_prf_sample_size), 0, 1, 0, 1},
  {&__pyx_n_u_lower_elevation_limit, __pyx_k_lower_elevation_limit, sizeof(__pyx_k_lower_elevation_limit), 0, 1, 0, 1},
  {&__pyx_n_s_ma, __pyx_k_ma, sizeof(__pyx_k_ma), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_u_major_mode, __pyx_k_major_mode, sizeof(__pyx_k_major_mode), 0, 1, 0, 1},
  {&__pyx_n_s_mask, __pyx_k_mask, sizeof(__pyx_k_mask), 0, 0, 1, 1},
  {&__pyx_n_u_mask_word_0, __pyx_k_mask_word_0, sizeof(__pyx_k_mask_word_0), 0, 1, 0, 1},
  {&__pyx_n_u_mask_word_1, __pyx_k_mask_word_1, sizeof(__pyx_k_mask_word_1), 0, 1, 0, 1},
  {&__pyx_n_u_mask_word_2, __pyx_k_mask_word_2, sizeof(__pyx_k_mask_word_2), 0, 1, 0, 1},
  {&__pyx_n_u_mask_word_3, __pyx_k_mask_word_3, sizeof(__pyx_k_mask_word_3), 0, 1, 0, 1},
  {&__pyx_n_u_mask_word_4, __pyx_k_mask_word_4, sizeof(__pyx_k_mask_word_4), 0, 1, 0, 1},
  {&__pyx_n_s_masked_array, __pyx_k_masked_array, sizeof(__pyx_k_masked_array), 0, 0, 1, 1},
  {&__pyx_n_u_maximum_range, __pyx_k_maximum_range, sizeof(__pyx_k_maximum_range), 0, 1, 0, 1},
  {&__pyx_n_u_mean_wind_direction, __pyx_k_mean_wind_direction, sizeof(__pyx_k_mean_wind_direction), 0, 1, 0, 1},
  {&__pyx_n_u_mean_wind_speed, __pyx_k_mean_wind_speed, sizeof(__pyx_k_mean_wind_speed), 0, 1, 0, 1},
  {&__pyx_n_u_melting_layer, __pyx_k_melting_layer, sizeof(__pyx_k_melting_layer), 0, 1, 0, 1},
  {&__pyx_n_u_melting_level, __pyx_k_melting_level, sizeof(__pyx_k_melting_level), 0, 1, 0, 1},
  {&__pyx_n_u_milliseconds, __pyx_k_milliseconds, sizeof(__pyx_k_milliseconds), 0, 1, 0, 1},
  {&__pyx_n_u_minor_task_suffix, __pyx_k_minor_task_suffix, sizeof(__pyx_k_minor_task_suffix), 0, 1, 0, 1},
  {&__pyx_n_s_mod, __pyx_k_mod, sizeof(__pyx_k_mod), 0, 0, 1, 1},
  {&__pyx_n_u_month, __pyx_k_month, sizeof(__pyx_k_month), 0, 1, 0, 1},
  {&__pyx_n_u_multi_prf_flag, __pyx_k_multi_prf_flag, sizeof(__pyx_k_multi_prf_flag), 0, 1, 0, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_nbin, __pyx_k_nbin, sizeof(__pyx_k_nbin), 0, 0, 1, 1},
  {&__pyx_n_s_nbins, __pyx_k_nbins, sizeof(__pyx_k_nbins), 0, 0, 1, 1},
  {&__pyx_n_u_nbins, __pyx_k_nbins, sizeof(__pyx_k_nbins), 0, 1, 0, 1},
  {&__pyx_n_s_ndata, __pyx_k_ndata, sizeof(__pyx_k_ndata), 0, 0, 1, 1},
  {&__pyx_n_s_ndata_types, __pyx_k_ndata_types, sizeof(__pyx_k_ndata_types), 0, 0, 1, 1},
  {&__pyx_n_u_nexrad_clutter_skip, __pyx_k_nexrad_clutter_skip, sizeof(__pyx_k_nexrad_clutter_skip), 0, 1, 0, 1},
  {&__pyx_n_u_nexrad_clutter_threshold, __pyx_k_nexrad_clutter_threshold, sizeof(__pyx_k_nexrad_clutter_threshold), 0, 1, 0, 1},
  {&__pyx_n_s_np, __pyx_k_np, sizeof(__pyx_k_np), 0, 0, 1, 1},
  {&__pyx_n_s_nrays, __pyx_k_nrays, sizeof(__pyx_k_nrays), 0, 0, 1, 1},
  {&__pyx_n_s_number, __pyx_k_number, sizeof(__pyx_k_number), 0, 0, 1, 1},
  {&__pyx_n_u_number_bins, __pyx_k_number_bins, sizeof(__pyx_k_number_bins), 0, 1, 0, 1},
  {&__pyx_n_u_number_elements, __pyx_k_number_elements, sizeof(__pyx_k_number_elements), 0, 1, 0, 1},
  {&__pyx_n_u_number_files, __pyx_k_number_files, sizeof(__pyx_k_number_files), 0, 1, 0, 1},
  {&__pyx_n_u_number_ingest, __pyx_k_number_ingest, sizeof(__pyx_k_number_ingest), 0, 1, 0, 1},
  {&__pyx_n_u_number_input_bins, __pyx_k_number_input_bins, sizeof(__pyx_k_number_input_bins), 0, 1, 0, 1},
  {&__pyx_n_u_number_linear_filter, __pyx_k_number_linear_filter, sizeof(__pyx_k_number_linear_filter), 0, 1, 0, 1},
  {&__pyx_n_u_number_log_filter, __pyx_k_number_log_filter, sizeof(__pyx_k_number_log_filter), 0, 1, 0, 1},
  {&__pyx_n_u_number_output_bins, __pyx_k_number_output_bins, sizeof(__pyx_k_number_output_bins), 0, 1, 0, 1},
  {&__pyx_n_u_number_rays_file_actual, __pyx_k_number_rays_file_actual, sizeof(__pyx_k_number_rays_file_actual), 0, 1, 0, 1},
  {&__pyx_n_u_number_rays_file_expected, __pyx_k_number_rays_file_expected, sizeof(__pyx_k_number_rays_file_expected), 0, 1, 0, 1},
  {&__pyx_n_u_number_rays_sweep, __pyx_k_number_rays_sweep, sizeof(__pyx_k_number_rays_sweep), 0, 1, 0, 1},
  {&__pyx_n_u_number_sweeps, __pyx_k_number_sweeps, sizeof(__pyx_k_number_sweeps), 0, 1, 0, 1},
  {&__pyx_n_u_number_sweeps_completed, __pyx_k_number_sweeps_completed, sizeof(__pyx_k_number_sweeps_completed), 0, 1, 0, 1},
  {&__pyx_n_u_number_task_config_table, __pyx_k_number_task_config_table, sizeof(__pyx_k_number_task_config_table), 0, 1, 0, 1},
  {&__pyx_n_u_number_tasks, __pyx_k_number_tasks, sizeof(__pyx_k_number_tasks), 0, 1, 0, 1},
  {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
  {&__pyx_kp_u_numpy_core_multiarray_failed_to, __pyx_k_numpy_core_multiarray_failed_to, sizeof(__pyx_k_numpy_core_multiarray_failed_to), 0, 1, 0, 0},
  {&__pyx_kp_u_numpy_core_umath_failed_to_impor, __pyx_k_numpy_core_umath_failed_to_impor, sizeof(__pyx_k_numpy_core_umath_failed_to_impor), 0, 1, 0, 0},
  {&__pyx_n_s_offset, __pyx_k_offset, sizeof(__pyx_k_offset), 0, 0, 1, 1},
  {&__pyx_n_s_ones, __pyx_k_ones, sizeof(__pyx_k_ones), 0, 0, 1, 1},
  {&__pyx_n_s_open, __pyx_k_open, sizeof(__pyx_k_open), 0, 0, 1, 1},
  {&__pyx_n_u_original_data_type_mask, __pyx_k_original_data_type_mask, sizeof(__pyx_k_original_data_type_mask), 0, 1, 0, 1},
  {&__pyx_n_s_out, __pyx_k_out, sizeof(__pyx_k_out), 0, 0, 1, 1},
  {&__pyx_n_s_parse_ray_headers, __pyx_k_parse_ray_headers, sizeof(__pyx_k_parse_ray_headers), 0, 0, 1, 1},
  {&__pyx_n_u_playback_flag, __pyx_k_playback_flag, sizeof(__pyx_k_playback_flag), 0, 1, 0, 1},
  {&__pyx_n_u_playback_version, __pyx_k_playback_version, sizeof(__pyx_k_playback_version), 0, 1, 0, 1},
  {&__pyx_n_u_polarization, __pyx_k_polarization, sizeof(__pyx_k_polarization), 0, 1, 0, 1},
  {&__pyx_n_u_polarization_type, __pyx_k_polarization_type, sizeof(__pyx_k_polarization_type), 0, 1, 0, 1},
  {&__pyx_n_s_power, __pyx_k_power, sizeof(__pyx_k_power), 0, 0, 1, 1},
  {&__pyx_n_u_power_threshold, __pyx_k_power_threshold, sizeof(__pyx_k_power_threshold), 0, 1, 0, 1},
  {&__pyx_n_u_prf, __pyx_k_prf, sizeof(__pyx_k_prf), 0, 1, 0, 1},
  {&__pyx_n_s_prf_flag, __pyx_k_prf_flag, sizeof(__pyx_k_prf_flag), 0, 0, 1, 1},
  {&__pyx_n_u_prf_flag, __pyx_k_prf_flag, sizeof(__pyx_k_prf_flag), 0, 1, 0, 1},
  {&__pyx_n_s_print, __pyx_k_print, sizeof(__pyx_k_print), 0, 0, 1, 1},
  {&__pyx_n_s_product_config, __pyx_k_product_config, sizeof(__pyx_k_product_config), 0, 0, 1, 1},
  {&__pyx_n_u_product_configuration, __pyx_k_product_configuration, sizeof(__pyx_k_product_configuration), 0, 1, 0, 1},
  {&__pyx_n_s_product_end, __pyx_k_product_end, sizeof(__pyx_k_product_end), 0, 0, 1, 1},
  {&__pyx_n_u_product_end, __pyx_k_product_end, sizeof(__pyx_k_product_end), 0, 1, 0, 1},
  {&__pyx_n_s_product_hdr, __pyx_k_product_hdr, sizeof(__pyx_k_product_hdr), 0, 0, 1, 1},
  {&__pyx_n_u_product_name, __pyx_k_product_name, sizeof(__pyx_k_product_name), 0, 1, 0, 1},
  {&__pyx_n_u_product_sequence_number, __pyx_k_product_sequence_number, sizeof(__pyx_k_product_sequence_number), 0, 1, 0, 1},
  {&__pyx_n_u_product_specific_bytes, __pyx_k_product_specific_bytes, sizeof(__pyx_k_product_specific_bytes), 0, 1, 0, 1},
  {&__pyx_n_u_product_type_code, __pyx_k_product_type_code, sizeof(__pyx_k_product_type_code), 0, 1, 0, 1},
  {&__pyx_n_u_projection_name, __pyx_k_projection_name, sizeof(__pyx_k_projection_name), 0, 1, 0, 1},
  {&__pyx_n_u_projection_type, __pyx_k_projection_type, sizeof(__pyx_k_projection_type), 0, 1, 0, 1},
  {&__pyx_n_u_pulse_width, __pyx_k_pulse_width, sizeof(__pyx_k_pulse_width), 0, 1, 0, 1},
  {&__pyx_n_s_pyart_io__sigmetfile, __pyx_k_pyart_io__sigmetfile, sizeof(__pyx_k_pyart_io__sigmetfile), 0, 0, 1, 1},
  {&__pyx_kp_s_pyart_io__sigmetfile_pyx, __pyx_k_pyart_io__sigmetfile_pyx, sizeof(__pyx_k_pyart_io__sigmetfile_pyx), 0, 0, 1, 0},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_u_radar_height, __pyx_k_radar_height, sizeof(__pyx_k_radar_height), 0, 1, 0, 1},
  {&__pyx_n_u_radar_height_above_reference, __pyx_k_radar_height_above_reference, sizeof(__pyx_k_radar_height_above_reference), 0, 1, 0, 1},
  {&__pyx_n_u_radial_smoother, __pyx_k_radial_smoother, sizeof(__pyx_k_radial_smoother), 0, 1, 0, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_u_range_bin_averaging_flag, __pyx_k_range_bin_averaging_flag, sizeof(__pyx_k_range_bin_averaging_flag), 0, 1, 0, 1},
  {&__pyx_n_s_raw_data, __pyx_k_raw_data, sizeof(__pyx_k_raw_data), 0, 0, 1, 1},
  {&__pyx_n_u_ray_header_bytes, __pyx_k_ray_header_bytes, sizeof(__pyx_k_ray_header_bytes), 0, 1, 0, 1},
  {&__pyx_n_u_ray_header_mask, __pyx_k_ray_header_mask, sizeof(__pyx_k_ray_header_mask), 0, 1, 0, 1},
  {&__pyx_n_s_ray_headers, __pyx_k_ray_headers, sizeof(__pyx_k_ray_headers), 0, 0, 1, 1},
  {&__pyx_n_u_ray_number, __pyx_k_ray_number, sizeof(__pyx_k_ray_number), 0, 1, 0, 1},
  {&__pyx_n_u_rb, __pyx_k_rb, sizeof(__pyx_k_rb), 0, 1, 0, 1},
  {&__pyx_n_s_read, __pyx_k_read, sizeof(__pyx_k_read), 0, 0, 1, 1},
  {&__pyx_n_u_read, __pyx_k_read, sizeof(__pyx_k_read), 0, 1, 0, 1},
  {&__pyx_n_u_reciever_bandwidth, __pyx_k_reciever_bandwidth, sizeof(__pyx_k_reciever_bandwidth), 0, 1, 0, 1},
  {&__pyx_n_s_record, __pyx_k_record, sizeof(__pyx_k_record), 0, 0, 1, 1},
  {&__pyx_n_u_record_number, __pyx_k_record_number, sizeof(__pyx_k_record_number), 0, 1, 0, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_u_reflectivity_calibration, __pyx_k_reflectivity_calibration, sizeof(__pyx_k_reflectivity_calibration), 0, 1, 0, 1},
  {&__pyx_n_u_reflectivity_noise_threshold, __pyx_k_reflectivity_noise_threshold, sizeof(__pyx_k_reflectivity_noise_threshold), 0, 1, 0, 1},
  {&__pyx_n_u_reflectivity_slope, __pyx_k_reflectivity_slope, sizeof(__pyx_k_reflectivity_slope), 0, 1, 0, 1},
  {&__pyx_n_u_reflectivity_unfolding_threshold, __pyx_k_reflectivity_unfolding_threshold, sizeof(__pyx_k_reflectivity_unfolding_threshold), 0, 1, 0, 1},
  {&__pyx_n_u_reserved, __pyx_k_reserved, sizeof(__pyx_k_reserved), 0, 1, 0, 1},
  {&__pyx_n_s_reshape, __pyx_k_reshape, sizeof(__pyx_k_reshape), 0, 0, 1, 1},
  {&__pyx_n_u_resolution_rays, __pyx_k_resolution_rays, sizeof(__pyx_k_resolution_rays), 0, 1, 0, 1},
  {&__pyx_n_u_right_azimuth_limit, __pyx_k_right_azimuth_limit, sizeof(__pyx_k_right_azimuth_limit), 0, 1, 0, 1},
  {&__pyx_n_u_sample_size, __pyx_k_sample_size, sizeof(__pyx_k_sample_size), 0, 1, 0, 1},
  {&__pyx_n_u_samples_used, __pyx_k_samples_used, sizeof(__pyx_k_samples_used), 0, 1, 0, 1},
  {&__pyx_n_u_scheduling_code, __pyx_k_scheduling_code, sizeof(__pyx_k_scheduling_code), 0, 1, 0, 1},
  {&__pyx_n_u_seconds, __pyx_k_seconds, sizeof(__pyx_k_seconds), 0, 1, 0, 1},
  {&__pyx_n_u_seconds_between_runs, __pyx_k_seconds_between_runs, sizeof(__pyx_k_seconds_between_runs), 0, 1, 0, 1},
  {&__pyx_kp_s_self__rbuf_p_cannot_be_converted, __pyx_k_self__rbuf_p_cannot_be_converted, sizeof(__pyx_k_self__rbuf_p_cannot_be_converted), 0, 0, 1, 0},
  {&__pyx_kp_u_self__rbuf_pos_is, __pyx_k_self__rbuf_pos_is, sizeof(__pyx_k_self__rbuf_pos_is), 0, 1, 0, 0},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_shape, __pyx_k_shape, sizeof(__pyx_k_shape), 0, 0, 1, 1},
  {&__pyx_n_s_shrink, __pyx_k_shrink, sizeof(__pyx_k_shrink), 0, 0, 1, 1},
  {&__pyx_n_u_signal_processor_type, __pyx_k_signal_processor_type, sizeof(__pyx_k_signal_processor_type), 0, 1, 0, 1},
  {&__pyx_n_u_site_name, __pyx_k_site_name, sizeof(__pyx_k_site_name), 0, 1, 0, 1},
  {&__pyx_n_u_skip_time, __pyx_k_skip_time, sizeof(__pyx_k_skip_time), 0, 1, 0, 1},
  {&__pyx_n_u_spare_0, __pyx_k_spare_0, sizeof(__pyx_k_spare_0), 0, 1, 0, 1},
  {&__pyx_n_u_spare_1, __pyx_k_spare_1, sizeof(__pyx_k_spare_1), 0, 1, 0, 1},
  {&__pyx_n_u_spare_2, __pyx_k_spare_2, sizeof(__pyx_k_spare_2), 0, 1, 0, 1},
  {&__pyx_n_u_spare_3, __pyx_k_spare_3, sizeof(__pyx_k_spare_3), 0, 1, 0, 1},
  {&__pyx_n_u_spare_4, __pyx_k_spare_4, sizeof(__pyx_k_spare_4), 0, 1, 0, 1},
  {&__pyx_n_u_sqi_threshold, __pyx_k_sqi_threshold, sizeof(__pyx_k_sqi_threshold), 0, 1, 0, 1},
  {&__pyx_n_s_sqrt, __pyx_k_sqrt, sizeof(__pyx_k_sqrt), 0, 0, 1, 1},
  {&__pyx_n_u_standard_parallel_1, __pyx_k_standard_parallel_1, sizeof(__pyx_k_standard_parallel_1), 0, 1, 0, 1},
  {&__pyx_n_u_standard_parallel_2, __pyx_k_standard_parallel_2, sizeof(__pyx_k_standard_parallel_2), 0, 1, 0, 1},
  {&__pyx_n_u_start_first_section_flag, __pyx_k_start_first_section_flag, sizeof(__pyx_k_start_first_section_flag), 0, 1, 0, 1},
  {&__pyx_n_u_start_first_sector_flag, __pyx_k_start_first_sector_flag, sizeof(__pyx_k_start_first_sector_flag), 0, 1, 0, 1},
  {&__pyx_n_u_start_time, __pyx_k_start_time, sizeof(__pyx_k_start_time), 0, 1, 0, 1},
  {&__pyx_n_u_step_input_bins, __pyx_k_step_input_bins, sizeof(__pyx_k_step_input_bins), 0, 1, 0, 1},
  {&__pyx_n_u_step_output_bins, __pyx_k_step_output_bins, sizeof(__pyx_k_step_output_bins), 0, 1, 0, 1},
  {&__pyx_n_u_stop_time, __pyx_k_stop_time, sizeof(__pyx_k_stop_time), 0, 1, 0, 1},
  {&__pyx_n_s_string, __pyx_k_string, sizeof(__pyx_k_string), 0, 0, 1, 1},
  {&__pyx_n_s_struct, __pyx_k_struct, sizeof(__pyx_k_struct), 0, 0, 1, 1},
  {&__pyx_n_s_structure, __pyx_k_structure, sizeof(__pyx_k_structure), 0, 0, 1, 1},
  {&__pyx_n_u_structure_header, __pyx_k_structure_header, sizeof(__pyx_k_structure_header), 0, 1, 0, 1},
  {&__pyx_n_u_structure_identifier, __pyx_k_structure_identifier, sizeof(__pyx_k_structure_identifier), 0, 1, 0, 1},
  {&__pyx_n_s_sum, __pyx_k_sum, sizeof(__pyx_k_sum), 0, 0, 1, 1},
  {&__pyx_n_u_sweep_ingest_time, __pyx_k_sweep_ingest_time, sizeof(__pyx_k_sweep_ingest_time), 0, 1, 0, 1},
  {&__pyx_n_u_sweep_number, __pyx_k_sweep_number, sizeof(__pyx_k_sweep_number), 0, 1, 0, 1},
  {&__pyx_n_u_sweep_start_time, __pyx_k_sweep_start_time, sizeof(__pyx_k_sweep_start_time), 0, 1, 0, 1},
  {&__pyx_n_u_task_calib_info, __pyx_k_task_calib_info, sizeof(__pyx_k_task_calib_info), 0, 1, 0, 1},
  {&__pyx_n_s_task_configuration, __pyx_k_task_configuration, sizeof(__pyx_k_task_configuration), 0, 0, 1, 1},
  {&__pyx_n_u_task_configuration, __pyx_k_task_configuration, sizeof(__pyx_k_task_configuration), 0, 1, 0, 1},
  {&__pyx_n_u_task_configuration_file_name, __pyx_k_task_configuration_file_name, sizeof(__pyx_k_task_configuration_file_name), 0, 1, 0, 1},
  {&__pyx_n_u_task_data_time, __pyx_k_task_data_time, sizeof(__pyx_k_task_data_time), 0, 1, 0, 1},
  {&__pyx_n_u_task_description, __pyx_k_task_description, sizeof(__pyx_k_task_description), 0, 1, 0, 1},
  {&__pyx_n_s_task_dsp_info, __pyx_k_task_dsp_info, sizeof(__pyx_k_task_dsp_info), 0, 0, 1, 1},
  {&__pyx_n_u_task_dsp_info, __pyx_k_task_dsp_info, sizeof(__pyx_k_task_dsp_info), 0, 1, 0, 1},
  {&__pyx_n_u_task_dsp_mode, __pyx_k_task_dsp_mode, sizeof(__pyx_k_task_dsp_mode), 0, 1, 0, 1},
  {&__pyx_n_s_task_end_info, __pyx_k_task_end_info, sizeof(__pyx_k_task_end_info), 0, 0, 1, 1},
  {&__pyx_n_u_task_end_info, __pyx_k_task_end_info, sizeof(__pyx_k_task_end_info), 0, 1, 0, 1},
  {&__pyx_n_u_task_major_number, __pyx_k_task_major_number, sizeof(__pyx_k_task_major_number), 0, 1, 0, 1},
  {&__pyx_n_u_task_minor_number, __pyx_k_task_minor_number, sizeof(__pyx_k_task_minor_number), 0, 1, 0, 1},
  {&__pyx_n_u_task_misc_info, __pyx_k_task_misc_info, sizeof(__pyx_k_task_misc_info), 0, 1, 0, 1},
  {&__pyx_n_u_task_name, __pyx_k_task_name, sizeof(__pyx_k_task_name), 0, 1, 0, 1},
  {&__pyx_n_u_task_range_info, __pyx_k_task_range_info, sizeof(__pyx_k_task_range_info), 0, 1, 0, 1},
  {&__pyx_n_u_task_scan_info, __pyx_k_task_scan_info, sizeof(__pyx_k_task_scan_info), 0, 1, 0, 1},
  {&__pyx_n_u_task_scan_type_scan_info, __pyx_k_task_scan_type_scan_info, sizeof(__pyx_k_task_scan_type_scan_info), 0, 1, 0, 1},
  {&__pyx_n_u_task_sched_info, __pyx_k_task_sched_info, sizeof(__pyx_k_task_sched_info), 0, 1, 0, 1},
  {&__pyx_n_u_task_state, __pyx_k_task_state, sizeof(__pyx_k_task_state), 0, 1, 0, 1},
  {&__pyx_n_u_tcf_cal_flags_1, __pyx_k_tcf_cal_flags_1, sizeof(__pyx_k_tcf_cal_flags_1), 0, 1, 0, 1},
  {&__pyx_n_u_tcf_cal_flags_2, __pyx_k_tcf_cal_flags_2, sizeof(__pyx_k_tcf_cal_flags_2), 0, 1, 0, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_time, __pyx_k_time, sizeof(__pyx_k_time), 0, 0, 1, 1},
  {&__pyx_n_u_time, __pyx_k_time, sizeof(__pyx_k_time), 0, 1, 0, 1},
  {&__pyx_n_u_time_used_last_run, __pyx_k_time_used_last_run, sizeof(__pyx_k_time_used_last_run), 0, 1, 0, 1},
  {&__pyx_n_u_times_run, __pyx_k_times_run, sizeof(__pyx_k_times_run), 0, 1, 0, 1},
  {&__pyx_n_u_total_size, __pyx_k_total_size, sizeof(__pyx_k_total_size), 0, 1, 0, 1},
  {&__pyx_n_u_tr_serial_number, __pyx_k_tr_serial_number, sizeof(__pyx_k_tr_serial_number), 0, 1, 0, 1},
  {&__pyx_n_u_tranmitter_phase_sequence, __pyx_k_tranmitter_phase_sequence, sizeof(__pyx_k_tranmitter_phase_sequence), 0, 1, 0, 1},
  {&__pyx_n_u_transmit_power, __pyx_k_transmit_power, sizeof(__pyx_k_transmit_power), 0, 1, 0, 1},
  {&__pyx_n_u_trigger_rate, __pyx_k_trigger_rate, sizeof(__pyx_k_trigger_rate), 0, 1, 0, 1},
  {&__pyx_n_u_trucation_height, __pyx_k_trucation_height, sizeof(__pyx_k_trucation_height), 0, 1, 0, 1},
  {&__pyx_n_u_truncation_height, __pyx_k_truncation_height, sizeof(__pyx_k_truncation_height), 0, 1, 0, 1},
  {&__pyx_n_u_tz_name, __pyx_k_tz_name, sizeof(__pyx_k_tz_name), 0, 1, 0, 1},
  {&__pyx_n_u_uint16, __pyx_k_uint16, sizeof(__pyx_k_uint16), 0, 1, 0, 1},
  {&__pyx_n_s_uint8, __pyx_k_uint8, sizeof(__pyx_k_uint8), 0, 0, 1, 1},
  {&__pyx_n_u_uncorrected_reflectivity_thresho, __pyx_k_uncorrected_reflectivity_thresho, sizeof(__pyx_k_uncorrected_reflectivity_thresho), 0, 1, 0, 1},
  {&__pyx_n_s_unpack, __pyx_k_unpack, sizeof(__pyx_k_unpack), 0, 0, 1, 1},
  {&__pyx_n_s_unpack_ingest_data_header, __pyx_k_unpack_ingest_data_header, sizeof(__pyx_k_unpack_ingest_data_header), 0, 0, 1, 1},
  {&__pyx_n_s_unpack_ingest_data_headers, __pyx_k_unpack_ingest_data_headers, sizeof(__pyx_k_unpack_ingest_data_headers), 0, 0, 1, 1},
  {&__pyx_n_s_unpack_ingest_header, __pyx_k_unpack_ingest_header, sizeof(__pyx_k_unpack_ingest_header), 0, 0, 1, 1},
  {&__pyx_n_s_unpack_key, __pyx_k_unpack_key, sizeof(__pyx_k_unpack_key), 0, 0, 1, 1},
  {&__pyx_n_s_unpack_product_hdr, __pyx_k_unpack_product_hdr, sizeof(__pyx_k_unpack_product_hdr), 0, 0, 1, 1},
  {&__pyx_n_s_unpack_raw_prod_bhdr, __pyx_k_unpack_raw_prod_bhdr, sizeof(__pyx_k_unpack_raw_prod_bhdr), 0, 0, 1, 1},
  {&__pyx_n_s_unpack_structure, __pyx_k_unpack_structure, sizeof(__pyx_k_unpack_structure), 0, 0, 1, 1},
  {&__pyx_n_u_upper_elevation_limit, __pyx_k_upper_elevation_limit, sizeof(__pyx_k_upper_elevation_limit), 0, 1, 0, 1},
  {&__pyx_n_u_variable_range_bin_flag, __pyx_k_variable_range_bin_flag, sizeof(__pyx_k_variable_range_bin_flag), 0, 1, 0, 1},
  {&__pyx_n_u_velocity_east, __pyx_k_velocity_east, sizeof(__pyx_k_velocity_east), 0, 1, 0, 1},
  {&__pyx_n_u_velocity_north, __pyx_k_velocity_north, sizeof(__pyx_k_velocity_north), 0, 1, 0, 1},
  {&__pyx_n_u_velocity_threshold_flags, __pyx_k_velocity_threshold_flags, sizeof(__pyx_k_velocity_threshold_flags), 0, 1, 0, 1},
  {&__pyx_n_u_velocity_unfolding_threshold, __pyx_k_velocity_unfolding_threshold, sizeof(__pyx_k_velocity_unfolding_threshold), 0, 1, 0, 1},
  {&__pyx_n_u_velocity_up, __pyx_k_velocity_up, sizeof(__pyx_k_velocity_up), 0, 1, 0, 1},
  {&__pyx_n_u_vertical_beamwidth, __pyx_k_vertical_beamwidth, sizeof(__pyx_k_vertical_beamwidth), 0, 1, 0, 1},
  {&__pyx_n_u_vertical_current_noise, __pyx_k_vertical_current_noise, sizeof(__pyx_k_vertical_current_noise), 0, 1, 0, 1},
  {&__pyx_n_u_vertical_i0_calibration, __pyx_k_vertical_i0_calibration, sizeof(__pyx_k_vertical_i0_calibration), 0, 1, 0, 1},
  {&__pyx_n_u_vertical_noise_calibration, __pyx_k_vertical_noise_calibration, sizeof(__pyx_k_vertical_noise_calibration), 0, 1, 0, 1},
  {&__pyx_n_u_vertical_radar_constant, __pyx_k_vertical_radar_constant, sizeof(__pyx_k_vertical_radar_constant), 0, 1, 0, 1},
  {&__pyx_n_s_view, __pyx_k_view, sizeof(__pyx_k_view), 0, 0, 1, 1},
  {&__pyx_n_u_volume_scan_start_time, __pyx_k_volume_scan_start_time, sizeof(__pyx_k_volume_scan_start_time), 0, 1, 0, 1},
  {&__pyx_n_s_warn, __pyx_k_warn, sizeof(__pyx_k_warn), 0, 0, 1, 1},
  {&__pyx_n_s_warnings, __pyx_k_warnings, sizeof(__pyx_k_warnings), 0, 0, 1, 1},
  {&__pyx_n_u_wavelength, __pyx_k_wavelength, sizeof(__pyx_k_wavelength), 0, 1, 0, 1},
  {&__pyx_n_u_width_threshold_flags, __pyx_k_width_threshold_flags, sizeof(__pyx_k_width_threshold_flags), 0, 1, 0, 1},
  {&__pyx_n_u_width_unfolding_threshold, __pyx_k_width_unfolding_threshold, sizeof(__pyx_k_width_unfolding_threshold), 0, 1, 0, 1},
  {&__pyx_n_s_word0, __pyx_k_word0, sizeof(__pyx_k_word0), 0, 0, 1, 1},
  {&__pyx_n_s_word1, __pyx_k_word1, sizeof(__pyx_k_word1), 0, 0, 1, 1},
  {&__pyx_n_s_word2, __pyx_k_word2, sizeof(__pyx_k_word2), 0, 0, 1, 1},
  {&__pyx_n_s_word3, __pyx_k_word3, sizeof(__pyx_k_word3), 0, 0, 1, 1},
  {&__pyx_n_u_x_location, __pyx_k_x_location, sizeof(__pyx_k_x_location), 0, 1, 0, 1},
  {&__pyx_n_u_x_scale, __pyx_k_x_scale, sizeof(__pyx_k_x_scale), 0, 1, 0, 1},
  {&__pyx_n_u_x_size, __pyx_k_x_size, sizeof(__pyx_k_x_size), 0, 1, 0, 1},
  {&__pyx_n_u_x_smoother, __pyx_k_x_smoother, sizeof(__pyx_k_x_smoother), 0, 1, 0, 1},
  {&__pyx_n_s_xrange, __pyx_k_xrange, sizeof(__pyx_k_xrange), 0, 0, 1, 1},
  {&__pyx_n_u_y_location, __pyx_k_y_location, sizeof(__pyx_k_y_location), 0, 1, 0, 1},
  {&__pyx_n_u_y_scale, __pyx_k_y_scale, sizeof(__pyx_k_y_scale), 0, 1, 0, 1},
  {&__pyx_n_u_y_size, __pyx_k_y_size, sizeof(__pyx_k_y_size), 0, 1, 0, 1},
  {&__pyx_n_u_y_smoother, __pyx_k_y_smoother, sizeof(__pyx_k_y_smoother), 0, 1, 0, 1},
  {&__pyx_n_u_year, __pyx_k_year, sizeof(__pyx_k_year), 0, 1, 0, 1},
  {&__pyx_n_u_z_location, __pyx_k_z_location, sizeof(__pyx_k_z_location), 0, 1, 0, 1},
  {&__pyx_n_u_z_scale, __pyx_k_z_scale, sizeof(__pyx_k_z_scale), 0, 1, 0, 1},
  {&__pyx_n_u_z_size, __pyx_k_z_size, sizeof(__pyx_k_z_size), 0, 1, 0, 1},
  {&__pyx_n_u_zdr_bias, __pyx_k_zdr_bias, sizeof(__pyx_k_zdr_bias), 0, 1, 0, 1},
  {&__pyx_n_u_zdr_offset, __pyx_k_zdr_offset, sizeof(__pyx_k_zdr_offset), 0, 1, 0, 1},
  {&__pyx_n_u_zdr_threshold_flags, __pyx_k_zdr_threshold_flags, sizeof(__pyx_k_zdr_threshold_flags), 0, 1, 0, 1},
  {&__pyx_n_s_zeros_like, __pyx_k_zeros_like, sizeof(__pyx_k_zeros_like), 0, 0, 1, 1},
  {&__pyx_n_s_zip, __pyx_k_zip, sizeof(__pyx_k_zip), 0, 0, 1, 1},
  {&__pyx_n_u_zr_constant, __pyx_k_zr_constant, sizeof(__pyx_k_zr_constant), 0, 1, 0, 1},
  {&__pyx_n_u_zr_exponent, __pyx_k_zr_exponent, sizeof(__pyx_k_zr_exponent), 0, 1, 0, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_open = __Pyx_GetBuiltinName(__pyx_n_s_open); if (!__pyx_builtin_open) __PYX_ERR(0, 66, __pyx_L1_error)
  #if PY_MAJOR_VERSION >= 3
  __pyx_builtin_xrange = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_xrange) __PYX_ERR(0, 158, __pyx_L1_error)
  #else
  __pyx_builtin_xrange = __Pyx_GetBuiltinName(__pyx_n_s_xrange); if (!__pyx_builtin_xrange) __PYX_ERR(0, 158, __pyx_L1_error)
  #endif
  __pyx_builtin_enumerate = __Pyx_GetBuiltinName(__pyx_n_s_enumerate); if (!__pyx_builtin_enumerate) __PYX_ERR(0, 175, __pyx_L1_error)
  __pyx_builtin_sum = __Pyx_GetBuiltinName(__pyx_n_s_sum); if (!__pyx_builtin_sum) __PYX_ERR(0, 266, __pyx_L1_error)
  __pyx_builtin_print = __Pyx_GetBuiltinName(__pyx_n_s_print); if (!__pyx_builtin_print) __PYX_ERR(0, 280, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 342, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(1, 2, __pyx_L1_error)
  __pyx_builtin_zip = __Pyx_GetBuiltinName(__pyx_n_s_zip); if (!__pyx_builtin_zip) __PYX_ERR(0, 815, __pyx_L1_error)
  __pyx_builtin_ImportError = __Pyx_GetBuiltinName(__pyx_n_s_ImportError); if (!__pyx_builtin_ImportError) __PYX_ERR(2, 945, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "pyart/io/_sigmetfile.pyx":253
 *         # check if the file ended early, if so return Nones
 *         if len(lead_record) != RECORD_SIZE:
 *             return None, None, None             # <<<<<<<<<<<<<<
 * 
 *         # unpack structures
 */
  __pyx_tuple_ = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple_)) __PYX_ERR(0, 253, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "pyart/io/_sigmetfile.pyx":294
 *         for i, data_type in enumerate(self.data_types):
 *             if data_type == 0 and full_xhdr:
 *                 sweep_data.append(raw_sweep_data[i::self.ndata_types, 6:])             # <<<<<<<<<<<<<<
 *             else:
 *                 sweep_data.append(convert_sigmet_data(
 */
  __pyx_slice__2 = PySlice_New(__pyx_int_6, Py_None, Py_None); if (unlikely(!__pyx_slice__2)) __PYX_ERR(0, 294, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__2);
  __Pyx_GIVEREF(__pyx_slice__2);

  /* "pyart/io/_sigmetfile.pyx":300
 *                     raw_sweep_data[i::self.ndata_types, 4]))
 *             sweep_metadata.append(_parse_ray_headers(
 *                 raw_sweep_data[i::self.ndata_types, :6]))             # <<<<<<<<<<<<<<
 *         return ingest_data_headers, sweep_data, sweep_metadata
 * 
 */
  __pyx_slice__3 = PySlice_New(Py_None, __pyx_int_6, Py_None); if (unlikely(!__pyx_slice__3)) __PYX_ERR(0, 300, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__3);
  __Pyx_GIVEREF(__pyx_slice__3);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._rbuf_p cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._rbuf_p cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_s_self__rbuf_p_cannot_be_converted); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "(tree fragment)":4
 *     raise TypeError("self._rbuf_p cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._rbuf_p cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_s_self__rbuf_p_cannot_be_converted); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "pyart/io/_sigmetfile.pyx":445
 * 
 *     """
 *     az0 = bin2_to_angle(ray_headers.view('uint16')[..., 0])             # <<<<<<<<<<<<<<
 *     el0 = bin2_to_angle(ray_headers.view('uint16')[..., 1])
 *     az1 = bin2_to_angle(ray_headers.view('uint16')[..., 2])
 */
  __pyx_tuple__6 = PyTuple_Pack(2, Py_Ellipsis, __pyx_int_0); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "pyart/io/_sigmetfile.pyx":446
 *     """
 *     az0 = bin2_to_angle(ray_headers.view('uint16')[..., 0])
 *     el0 = bin2_to_angle(ray_headers.view('uint16')[..., 1])             # <<<<<<<<<<<<<<
 *     az1 = bin2_to_angle(ray_headers.view('uint16')[..., 2])
 *     el1 = bin2_to_angle(ray_headers.view('uint16')[..., 3])
 */
  __pyx_tuple__7 = PyTuple_Pack(2, Py_Ellipsis, __pyx_int_1); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(0, 446, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "pyart/io/_sigmetfile.pyx":447
 *     az0 = bin2_to_angle(ray_headers.view('uint16')[..., 0])
 *     el0 = bin2_to_angle(ray_headers.view('uint16')[..., 1])
 *     az1 = bin2_to_angle(ray_headers.view('uint16')[..., 2])             # <<<<<<<<<<<<<<
 *     el1 = bin2_to_angle(ray_headers.view('uint16')[..., 3])
 *     nbins = ray_headers.view('int16')[..., 4]
 */
  __pyx_tuple__8 = PyTuple_Pack(2, Py_Ellipsis, __pyx_int_2); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "pyart/io/_sigmetfile.pyx":448
 *     el0 = bin2_to_angle(ray_headers.view('uint16')[..., 1])
 *     az1 = bin2_to_angle(ray_headers.view('uint16')[..., 2])
 *     el1 = bin2_to_angle(ray_headers.view('uint16')[..., 3])             # <<<<<<<<<<<<<<
 *     nbins = ray_headers.view('int16')[..., 4]
 *     time = ray_headers.view('uint16')[..., 5]
 */
  __pyx_tuple__9 = PyTuple_Pack(2, Py_Ellipsis, __pyx_int_3); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(0, 448, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "pyart/io/_sigmetfile.pyx":449
 *     az1 = bin2_to_angle(ray_headers.view('uint16')[..., 2])
 *     el1 = bin2_to_angle(ray_headers.view('uint16')[..., 3])
 *     nbins = ray_headers.view('int16')[..., 4]             # <<<<<<<<<<<<<<
 *     time = ray_headers.view('uint16')[..., 5]
 *     prf_flag = np.mod(ray_headers.view('int16')[..., 0], 2)
 */
  __pyx_tuple__10 = PyTuple_Pack(2, Py_Ellipsis, __pyx_int_4); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(0, 449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "pyart/io/_sigmetfile.pyx":450
 *     el1 = bin2_to_angle(ray_headers.view('uint16')[..., 3])
 *     nbins = ray_headers.view('int16')[..., 4]
 *     time = ray_headers.view('uint16')[..., 5]             # <<<<<<<<<<<<<<
 *     prf_flag = np.mod(ray_headers.view('int16')[..., 0], 2)
 *     return (az0, el0, az1, el1, nbins, time, prf_flag)
 */
  __pyx_tuple__11 = PyTuple_Pack(2, Py_Ellipsis, __pyx_int_5); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "pyart/io/_sigmetfile.pyx":655
 *         # 0 : no data available (mask)
 *         # 65535 Reserved for area not scanned in product file (nothing)
 *         out[:] = (data.view('uint16') - 32768.) / 100.             # <<<<<<<<<<<<<<
 *         mask[data.view('uint16') == 0] = True
 * 
 */
  __pyx_slice__12 = PySlice_New(Py_None, Py_None, Py_None); if (unlikely(!__pyx_slice__12)) __PYX_ERR(0, 655, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__12);
  __Pyx_GIVEREF(__pyx_slice__12);

  /* "pyart/io/_sigmetfile.pyx":686
 *         # extended_header_v0, _v1, _v2, section 4.2.8-4.2.10
 *         # Here we return an array with the times in milliseconds.
 *         return data[..., :2].copy().view('i4')             # <<<<<<<<<<<<<<
 * 
 *     # one byte data types
 */
  __pyx_slice__13 = PySlice_New(Py_None, __pyx_int_2, Py_None); if (unlikely(!__pyx_slice__13)) __PYX_ERR(0, 686, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__13);
  __Pyx_GIVEREF(__pyx_slice__13);
  __pyx_tuple__14 = PyTuple_Pack(2, Py_Ellipsis, __pyx_slice__13); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(0, 686, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);

  /* "pyart/io/_sigmetfile.pyx":853
 * def _unpack_raw_prod_bhdr(record):
 *     """ Return a dict with the unpacked raw_prod_bhdr from a record. """
 *     return _unpack_structure(record[:12], RAW_PROD_BHDR)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_slice__16 = PySlice_New(Py_None, __pyx_int_12, Py_None); if (unlikely(!__pyx_slice__16)) __PYX_ERR(0, 853, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__16);
  __Pyx_GIVEREF(__pyx_slice__16);

  /* "pyart/io/_sigmetfile.pyx":862
 * 
 *     # unpack the product_hdr structure from the first record
 *     product_hdr = _unpack_structure(record[:640], PRODUCT_HDR)             # <<<<<<<<<<<<<<
 * 
 *     # product_hdr substructure
 */
  __pyx_slice__17 = PySlice_New(Py_None, __pyx_int_640, Py_None); if (unlikely(!__pyx_slice__17)) __PYX_ERR(0, 862, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__17);
  __Pyx_GIVEREF(__pyx_slice__17);

  /* "pyart/io/_sigmetfile.pyx":891
 * 
 *     # unpack the ingest_header structure from the second_record
 *     ingest_header = _unpack_structure(record[:4884], INGEST_HEADER)             # <<<<<<<<<<<<<<
 * 
 *     # ingest_header substructure
 */
  __pyx_slice__18 = PySlice_New(Py_None, __pyx_int_4884, Py_None); if (unlikely(!__pyx_slice__18)) __PYX_ERR(0, 891, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__18);
  __Pyx_GIVEREF(__pyx_slice__18);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":945
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
  __pyx_tuple__19 = PyTuple_Pack(1, __pyx_kp_u_numpy_core_multiarray_failed_to); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(2, 945, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":951
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
  __pyx_tuple__20 = PyTuple_Pack(1, __pyx_kp_u_numpy_core_umath_failed_to_impor); if (unlikely(!__pyx_tuple__20)) __PYX_ERR(2, 951, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__20);
  __Pyx_GIVEREF(__pyx_tuple__20);

  /* "pyart/io/_sigmetfile.pyx":401
 * 
 * 
 * def _data_types_from_mask(word0, word1, word2, word3):             # <<<<<<<<<<<<<<
 *     """
 *     Return a list of the data types from the words in the data_type mask.
 */
  __pyx_tuple__21 = PyTuple_Pack(9, __pyx_n_s_word0, __pyx_n_s_word1, __pyx_n_s_word2, __pyx_n_s_word3, __pyx_n_s_data_types, __pyx_n_s_i, __pyx_n_s_i, __pyx_n_s_i, __pyx_n_s_i); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(0, 401, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);
  __pyx_codeobj__22 = (PyObject*)__Pyx_PyCode_New(4, 0, 9, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__21, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_pyart_io__sigmetfile_pyx, __pyx_n_s_data_types_from_mask, 401, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__22)) __PYX_ERR(0, 401, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":412
 * 
 * 
 * def _is_bit_set(number, bit):             # <<<<<<<<<<<<<<
 *     """ Return True if bit is set in number. """
 *     return number >> bit & 1 == 1
 */
  __pyx_tuple__23 = PyTuple_Pack(2, __pyx_n_s_number, __pyx_n_s_bit); if (unlikely(!__pyx_tuple__23)) __PYX_ERR(0, 412, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__23);
  __Pyx_GIVEREF(__pyx_tuple__23);
  __pyx_codeobj__24 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__23, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_pyart_io__sigmetfile_pyx, __pyx_n_s_is_bit_set, 412, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__24)) __PYX_ERR(0, 412, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":417
 * 
 * 
 * def _parse_ray_headers(ray_headers):             # <<<<<<<<<<<<<<
 *     """
 *     Parse the metadata from Sigmet ray headers.
 */
  __pyx_tuple__25 = PyTuple_Pack(8, __pyx_n_s_ray_headers, __pyx_n_s_az0, __pyx_n_s_el0, __pyx_n_s_az1, __pyx_n_s_el1, __pyx_n_s_nbins, __pyx_n_s_time, __pyx_n_s_prf_flag); if (unlikely(!__pyx_tuple__25)) __PYX_ERR(0, 417, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);
  __pyx_codeobj__26 = (PyObject*)__Pyx_PyCode_New(1, 0, 8, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__25, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_pyart_io__sigmetfile_pyx, __pyx_n_s_parse_ray_headers, 417, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__26)) __PYX_ERR(0, 417, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":596
 * # file. Rewriting the convertions/masking in Cython does not seem to improved
 * # performance likely since most of the routines are already vectorized.
 * def convert_sigmet_data(data_type, data, nbins):             # <<<<<<<<<<<<<<
 *     """ Convert sigmet data. """
 *     out = np.empty_like(data, dtype='float32')
 */
  __pyx_tuple__27 = PyTuple_Pack(14, __pyx_n_s_data_type, __pyx_n_s_data, __pyx_n_s_nbins, __pyx_n_s_out, __pyx_n_s_mask, __pyx_n_s_data_type_name, __pyx_n_s_like_dbt2, __pyx_n_s_like_sqi, __pyx_n_s_like_sqi2, __pyx_n_s_like_dbt, __pyx_n_s_nrays, __pyx_n_s_nbin, __pyx_n_s_ndata, __pyx_n_s_exp); if (unlikely(!__pyx_tuple__27)) __PYX_ERR(0, 596, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__27);
  __Pyx_GIVEREF(__pyx_tuple__27);
  __pyx_codeobj__28 = (PyObject*)__Pyx_PyCode_New(3, 0, 14, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__27, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_pyart_io__sigmetfile_pyx, __pyx_n_s_convert_sigmet_data, 596, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__28)) __PYX_ERR(0, 596, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":796
 * 
 * 
 * def bin2_to_angle(bin2):             # <<<<<<<<<<<<<<
 *     """ Return an angle from Sigmet bin2 encoded value (or array). """
 *     return 360. * bin2 / 65536
 */
  __pyx_tuple__29 = PyTuple_Pack(1, __pyx_n_s_bin2); if (unlikely(!__pyx_tuple__29)) __PYX_ERR(0, 796, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__29);
  __Pyx_GIVEREF(__pyx_tuple__29);
  __pyx_codeobj__30 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__29, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_pyart_io__sigmetfile_pyx, __pyx_n_s_bin2_to_angle, 796, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__30)) __PYX_ERR(0, 796, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":801
 * 
 * 
 * def bin4_to_angle(bin4):             # <<<<<<<<<<<<<<
 *     """ Return an angle from Sigmet bin4 encoded value (or array). """
 *     return 360. * bin4 / 4294967296
 */
  __pyx_tuple__31 = PyTuple_Pack(1, __pyx_n_s_bin4); if (unlikely(!__pyx_tuple__31)) __PYX_ERR(0, 801, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__31);
  __Pyx_GIVEREF(__pyx_tuple__31);
  __pyx_codeobj__32 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__31, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_pyart_io__sigmetfile_pyx, __pyx_n_s_bin4_to_angle, 801, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__32)) __PYX_ERR(0, 801, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":811
 * 
 * 
 * def _unpack_structure(string, structure):             # <<<<<<<<<<<<<<
 *     """ Unpack a structure """
 *     fmt = ''.join([i[1] for i in structure])
 */
  __pyx_tuple__33 = PyTuple_Pack(6, __pyx_n_s_string, __pyx_n_s_structure, __pyx_n_s_fmt, __pyx_n_s_l, __pyx_n_s_i, __pyx_n_s_i); if (unlikely(!__pyx_tuple__33)) __PYX_ERR(0, 811, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__33);
  __Pyx_GIVEREF(__pyx_tuple__33);
  __pyx_codeobj__34 = (PyObject*)__Pyx_PyCode_New(2, 0, 6, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__33, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_pyart_io__sigmetfile_pyx, __pyx_n_s_unpack_structure, 811, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__34)) __PYX_ERR(0, 811, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":818
 * 
 * 
 * def _unpack_key(dic, key, structure):             # <<<<<<<<<<<<<<
 *     """ Unpack a key. """
 *     dic[key] = _unpack_structure(dic[key], structure)
 */
  __pyx_tuple__35 = PyTuple_Pack(3, __pyx_n_s_dic, __pyx_n_s_key, __pyx_n_s_structure); if (unlikely(!__pyx_tuple__35)) __PYX_ERR(0, 818, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__35);
  __Pyx_GIVEREF(__pyx_tuple__35);
  __pyx_codeobj__36 = (PyObject*)__Pyx_PyCode_New(3, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__35, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_pyart_io__sigmetfile_pyx, __pyx_n_s_unpack_key, 818, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__36)) __PYX_ERR(0, 818, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":823
 * 
 * 
 * def _unpack_ingest_data_headers(record, ndata_types):             # <<<<<<<<<<<<<<
 *     """
 *     Unpack one or more ingest_data_header from a record.
 */
  __pyx_tuple__37 = PyTuple_Pack(4, __pyx_n_s_record, __pyx_n_s_ndata_types, __pyx_n_s_idh, __pyx_n_s_i); if (unlikely(!__pyx_tuple__37)) __PYX_ERR(0, 823, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__37);
  __Pyx_GIVEREF(__pyx_tuple__37);
  __pyx_codeobj__38 = (PyObject*)__Pyx_PyCode_New(2, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__37, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_pyart_io__sigmetfile_pyx, __pyx_n_s_unpack_ingest_data_headers, 823, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__38)) __PYX_ERR(0, 823, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":837
 * 
 * 
 * def _unpack_ingest_data_header(record, number):             # <<<<<<<<<<<<<<
 *     """
 *     Unpack a single ingest_data_header from record.  Return None on error.
 */
  __pyx_tuple__39 = PyTuple_Pack(5, __pyx_n_s_record, __pyx_n_s_number, __pyx_n_s_offset, __pyx_n_s_string, __pyx_n_s_idh); if (unlikely(!__pyx_tuple__39)) __PYX_ERR(0, 837, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__39);
  __Pyx_GIVEREF(__pyx_tuple__39);
  __pyx_codeobj__40 = (PyObject*)__Pyx_PyCode_New(2, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__39, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_pyart_io__sigmetfile_pyx, __pyx_n_s_unpack_ingest_data_header, 837, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__40)) __PYX_ERR(0, 837, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":851
 * 
 * 
 * def _unpack_raw_prod_bhdr(record):             # <<<<<<<<<<<<<<
 *     """ Return a dict with the unpacked raw_prod_bhdr from a record. """
 *     return _unpack_structure(record[:12], RAW_PROD_BHDR)
 */
  __pyx_tuple__41 = PyTuple_Pack(1, __pyx_n_s_record); if (unlikely(!__pyx_tuple__41)) __PYX_ERR(0, 851, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__41);
  __Pyx_GIVEREF(__pyx_tuple__41);
  __pyx_codeobj__42 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__41, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_pyart_io__sigmetfile_pyx, __pyx_n_s_unpack_raw_prod_bhdr, 851, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__42)) __PYX_ERR(0, 851, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":856
 * 
 * 
 * def _unpack_product_hdr(record):             # <<<<<<<<<<<<<<
 *     """
 *     Return a dict with the unpacked product_hdr from the first record.
 */
  __pyx_tuple__43 = PyTuple_Pack(4, __pyx_n_s_record, __pyx_n_s_product_hdr, __pyx_n_s_product_config, __pyx_n_s_product_end); if (unlikely(!__pyx_tuple__43)) __PYX_ERR(0, 856, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__43);
  __Pyx_GIVEREF(__pyx_tuple__43);
  __pyx_codeobj__44 = (PyObject*)__Pyx_PyCode_New(1, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__43, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_pyart_io__sigmetfile_pyx, __pyx_n_s_unpack_product_hdr, 856, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__44)) __PYX_ERR(0, 856, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":885
 * 
 * 
 * def _unpack_ingest_header(record):             # <<<<<<<<<<<<<<
 *     """
 *     Return a dict with the unpacked ingest_header from the second record.
 */
  __pyx_tuple__45 = PyTuple_Pack(6, __pyx_n_s_record, __pyx_n_s_ingest_header, __pyx_n_s_ingest_configuration, __pyx_n_s_task_configuration, __pyx_n_s_task_dsp_info, __pyx_n_s_task_end_info); if (unlikely(!__pyx_tuple__45)) __PYX_ERR(0, 885, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__45);
  __Pyx_GIVEREF(__pyx_tuple__45);
  __pyx_codeobj__46 = (PyObject*)__Pyx_PyCode_New(1, 0, 6, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__45, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_pyart_io__sigmetfile_pyx, __pyx_n_s_unpack_ingest_header, 885, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__46)) __PYX_ERR(0, 885, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":961
 * # 640 bytes: product_hdr (section 4.2.25, page 47)
 * PRODUCT_HDR = (
 *     ('structure_header', '12s'),        # 12 bytes             # <<<<<<<<<<<<<<
 *     ('product_configuration', '320s'),  # 320 bytes
 *     ('product_end', '308s'),            # 308 bytes
 */
  __pyx_tuple__47 = PyTuple_Pack(2, __pyx_n_u_structure_header, __pyx_kp_u_12s); if (unlikely(!__pyx_tuple__47)) __PYX_ERR(0, 961, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__47);
  __Pyx_GIVEREF(__pyx_tuple__47);

  /* "pyart/io/_sigmetfile.pyx":962
 * PRODUCT_HDR = (
 *     ('structure_header', '12s'),        # 12 bytes
 *     ('product_configuration', '320s'),  # 320 bytes             # <<<<<<<<<<<<<<
 *     ('product_end', '308s'),            # 308 bytes
 * )
 */
  __pyx_tuple__48 = PyTuple_Pack(2, __pyx_n_u_product_configuration, __pyx_kp_u_320s); if (unlikely(!__pyx_tuple__48)) __PYX_ERR(0, 962, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__48);
  __Pyx_GIVEREF(__pyx_tuple__48);

  /* "pyart/io/_sigmetfile.pyx":963
 *     ('structure_header', '12s'),        # 12 bytes
 *     ('product_configuration', '320s'),  # 320 bytes
 *     ('product_end', '308s'),            # 308 bytes             # <<<<<<<<<<<<<<
 * )
 * 
 */
  __pyx_tuple__49 = PyTuple_Pack(2, __pyx_n_u_product_end, __pyx_kp_u_308s); if (unlikely(!__pyx_tuple__49)) __PYX_ERR(0, 963, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__49);
  __Pyx_GIVEREF(__pyx_tuple__49);

  /* "pyart/io/_sigmetfile.pyx":961
 * # 640 bytes: product_hdr (section 4.2.25, page 47)
 * PRODUCT_HDR = (
 *     ('structure_header', '12s'),        # 12 bytes             # <<<<<<<<<<<<<<
 *     ('product_configuration', '320s'),  # 320 bytes
 *     ('product_end', '308s'),            # 308 bytes
 */
  __pyx_tuple__50 = PyTuple_Pack(3, __pyx_tuple__47, __pyx_tuple__48, __pyx_tuple__49); if (unlikely(!__pyx_tuple__50)) __PYX_ERR(0, 961, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__50);
  __Pyx_GIVEREF(__pyx_tuple__50);

  /* "pyart/io/_sigmetfile.pyx":981
 *     ('scheduling_code', UINT2),
 *     ('seconds_between_runs', SINT4),
 *     ('generation_time', '12s'),     # 12 bytes: ymds_time             # <<<<<<<<<<<<<<
 *     ('sweep_ingest_time', '12s'),   # 12 bytes: ymds_time
 *     ('file_ingest_time', '12s'),    # 12 bytes: ymds_time
 */
  __pyx_tuple__51 = PyTuple_Pack(2, __pyx_n_u_generation_time, __pyx_kp_u_12s); if (unlikely(!__pyx_tuple__51)) __PYX_ERR(0, 981, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__51);
  __Pyx_GIVEREF(__pyx_tuple__51);

  /* "pyart/io/_sigmetfile.pyx":982
 *     ('seconds_between_runs', SINT4),
 *     ('generation_time', '12s'),     # 12 bytes: ymds_time
 *     ('sweep_ingest_time', '12s'),   # 12 bytes: ymds_time             # <<<<<<<<<<<<<<
 *     ('file_ingest_time', '12s'),    # 12 bytes: ymds_time
 *     ('spare_0', '6s'),              # 6 bytes
 */
  __pyx_tuple__52 = PyTuple_Pack(2, __pyx_n_u_sweep_ingest_time, __pyx_kp_u_12s); if (unlikely(!__pyx_tuple__52)) __PYX_ERR(0, 982, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__52);
  __Pyx_GIVEREF(__pyx_tuple__52);

  /* "pyart/io/_sigmetfile.pyx":983
 *     ('generation_time', '12s'),     # 12 bytes: ymds_time
 *     ('sweep_ingest_time', '12s'),   # 12 bytes: ymds_time
 *     ('file_ingest_time', '12s'),    # 12 bytes: ymds_time             # <<<<<<<<<<<<<<
 *     ('spare_0', '6s'),              # 6 bytes
 *     ('product_name', '12s'),
 */
  __pyx_tuple__53 = PyTuple_Pack(2, __pyx_n_u_file_ingest_time, __pyx_kp_u_12s); if (unlikely(!__pyx_tuple__53)) __PYX_ERR(0, 983, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__53);
  __Pyx_GIVEREF(__pyx_tuple__53);

  /* "pyart/io/_sigmetfile.pyx":984
 *     ('sweep_ingest_time', '12s'),   # 12 bytes: ymds_time
 *     ('file_ingest_time', '12s'),    # 12 bytes: ymds_time
 *     ('spare_0', '6s'),              # 6 bytes             # <<<<<<<<<<<<<<
 *     ('product_name', '12s'),
 *     ('task_name', '12s'),
 */
  __pyx_tuple__54 = PyTuple_Pack(2, __pyx_n_u_spare_0, __pyx_kp_u_6s); if (unlikely(!__pyx_tuple__54)) __PYX_ERR(0, 984, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__54);
  __Pyx_GIVEREF(__pyx_tuple__54);

  /* "pyart/io/_sigmetfile.pyx":985
 *     ('file_ingest_time', '12s'),    # 12 bytes: ymds_time
 *     ('spare_0', '6s'),              # 6 bytes
 *     ('product_name', '12s'),             # <<<<<<<<<<<<<<
 *     ('task_name', '12s'),
 *     ('flag', UINT2),
 */
  __pyx_tuple__55 = PyTuple_Pack(2, __pyx_n_u_product_name, __pyx_kp_u_12s); if (unlikely(!__pyx_tuple__55)) __PYX_ERR(0, 985, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__55);
  __Pyx_GIVEREF(__pyx_tuple__55);

  /* "pyart/io/_sigmetfile.pyx":986
 *     ('spare_0', '6s'),              # 6 bytes
 *     ('product_name', '12s'),
 *     ('task_name', '12s'),             # <<<<<<<<<<<<<<
 *     ('flag', UINT2),
 *     ('x_scale', SINT4),
 */
  __pyx_tuple__56 = PyTuple_Pack(2, __pyx_n_u_task_name, __pyx_kp_u_12s); if (unlikely(!__pyx_tuple__56)) __PYX_ERR(0, 986, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__56);
  __Pyx_GIVEREF(__pyx_tuple__56);

  /* "pyart/io/_sigmetfile.pyx":999
 *     ('maximum_range', SINT4),
 *     ('data_type', UINT2),
 *     ('projection_name', '12s'),             # <<<<<<<<<<<<<<
 *     ('input_data_type', UINT2),
 *     ('projection_type', UINT1),
 */
  __pyx_tuple__57 = PyTuple_Pack(2, __pyx_n_u_projection_name, __pyx_kp_u_12s); if (unlikely(!__pyx_tuple__57)) __PYX_ERR(0, 999, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__57);
  __Pyx_GIVEREF(__pyx_tuple__57);

  /* "pyart/io/_sigmetfile.pyx":1002
 *     ('input_data_type', UINT2),
 *     ('projection_type', UINT1),
 *     ('spare_1', '1s'),              # 1 bytes             # <<<<<<<<<<<<<<
 *     ('radial_smoother', SINT2),
 *     ('times_run', SINT2),
 */
  __pyx_tuple__58 = PyTuple_Pack(2, __pyx_n_u_spare_1, __pyx_kp_u_1s); if (unlikely(!__pyx_tuple__58)) __PYX_ERR(0, 1002, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__58);
  __Pyx_GIVEREF(__pyx_tuple__58);

  /* "pyart/io/_sigmetfile.pyx":1009
 *     ('x_smoother', SINT2),
 *     ('y_smoother', SINT2),
 *     ('product_specific_bytes', '80s'),  # 80 bytes:             # <<<<<<<<<<<<<<
 *     ('minor_task_suffix', '16s'),
 *     ('spare_2', '12s'),             # 12 bytes
 */
  __pyx_tuple__59 = PyTuple_Pack(2, __pyx_n_u_product_specific_bytes, __pyx_kp_u_80s); if (unlikely(!__pyx_tuple__59)) __PYX_ERR(0, 1009, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__59);
  __Pyx_GIVEREF(__pyx_tuple__59);

  /* "pyart/io/_sigmetfile.pyx":1010
 *     ('y_smoother', SINT2),
 *     ('product_specific_bytes', '80s'),  # 80 bytes:
 *     ('minor_task_suffix', '16s'),             # <<<<<<<<<<<<<<
 *     ('spare_2', '12s'),             # 12 bytes
 *     ('color_scale_def', '48s')      # 48 bytes: color_scale_def
 */
  __pyx_tuple__60 = PyTuple_Pack(2, __pyx_n_u_minor_task_suffix, __pyx_kp_u_16s); if (unlikely(!__pyx_tuple__60)) __PYX_ERR(0, 1010, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__60);
  __Pyx_GIVEREF(__pyx_tuple__60);

  /* "pyart/io/_sigmetfile.pyx":1011
 *     ('product_specific_bytes', '80s'),  # 80 bytes:
 *     ('minor_task_suffix', '16s'),
 *     ('spare_2', '12s'),             # 12 bytes             # <<<<<<<<<<<<<<
 *     ('color_scale_def', '48s')      # 48 bytes: color_scale_def
 * )
 */
  __pyx_tuple__61 = PyTuple_Pack(2, __pyx_n_u_spare_2, __pyx_kp_u_12s); if (unlikely(!__pyx_tuple__61)) __PYX_ERR(0, 1011, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__61);
  __Pyx_GIVEREF(__pyx_tuple__61);

  /* "pyart/io/_sigmetfile.pyx":1012
 *     ('minor_task_suffix', '16s'),
 *     ('spare_2', '12s'),             # 12 bytes
 *     ('color_scale_def', '48s')      # 48 bytes: color_scale_def             # <<<<<<<<<<<<<<
 * )
 * 
 */
  __pyx_tuple__62 = PyTuple_Pack(2, __pyx_n_u_color_scale_def, __pyx_kp_u_48s); if (unlikely(!__pyx_tuple__62)) __PYX_ERR(0, 1012, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__62);
  __Pyx_GIVEREF(__pyx_tuple__62);

  /* "pyart/io/_sigmetfile.pyx":1031
 *     ('icolcnt', SINT2),
 *     ('iset_and_scale', UINT2),
 *     ('ilevel_seams', '32s')     # 32 bytes: UINT2[16]             # <<<<<<<<<<<<<<
 * )
 * 
 */
  __pyx_tuple__63 = PyTuple_Pack(2, __pyx_n_u_ilevel_seams, __pyx_kp_u_32s); if (unlikely(!__pyx_tuple__63)) __PYX_ERR(0, 1031, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__63);
  __Pyx_GIVEREF(__pyx_tuple__63);

  /* "pyart/io/_sigmetfile.pyx":1036
 * # 308 bytes : product_end (section 4.2.24)
 * PRODUCT_END = (
 *     ('site_name', '16s'),             # <<<<<<<<<<<<<<
 *     ('iris_version_created', '8s'),
 *     ('ingest_iris_version', '8s'),
 */
  __pyx_tuple__64 = PyTuple_Pack(2, __pyx_n_u_site_name, __pyx_kp_u_16s); if (unlikely(!__pyx_tuple__64)) __PYX_ERR(0, 1036, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__64);
  __Pyx_GIVEREF(__pyx_tuple__64);

  /* "pyart/io/_sigmetfile.pyx":1037
 * PRODUCT_END = (
 *     ('site_name', '16s'),
 *     ('iris_version_created', '8s'),             # <<<<<<<<<<<<<<
 *     ('ingest_iris_version', '8s'),
 *     ('ingest_time', '12s'),         # 12 bytes: ymds_time
 */
  __pyx_tuple__65 = PyTuple_Pack(2, __pyx_n_u_iris_version_created, __pyx_kp_u_8s); if (unlikely(!__pyx_tuple__65)) __PYX_ERR(0, 1037, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__65);
  __Pyx_GIVEREF(__pyx_tuple__65);

  /* "pyart/io/_sigmetfile.pyx":1038
 *     ('site_name', '16s'),
 *     ('iris_version_created', '8s'),
 *     ('ingest_iris_version', '8s'),             # <<<<<<<<<<<<<<
 *     ('ingest_time', '12s'),         # 12 bytes: ymds_time
 *     ('spare_0', '28s'),             # 28 bytes
 */
  __pyx_tuple__66 = PyTuple_Pack(2, __pyx_n_u_ingest_iris_version, __pyx_kp_u_8s); if (unlikely(!__pyx_tuple__66)) __PYX_ERR(0, 1038, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__66);
  __Pyx_GIVEREF(__pyx_tuple__66);

  /* "pyart/io/_sigmetfile.pyx":1039
 *     ('iris_version_created', '8s'),
 *     ('ingest_iris_version', '8s'),
 *     ('ingest_time', '12s'),         # 12 bytes: ymds_time             # <<<<<<<<<<<<<<
 *     ('spare_0', '28s'),             # 28 bytes
 *     ('GMT_minute_offset_local', SINT2),
 */
  __pyx_tuple__67 = PyTuple_Pack(2, __pyx_n_u_ingest_time, __pyx_kp_u_12s); if (unlikely(!__pyx_tuple__67)) __PYX_ERR(0, 1039, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__67);
  __Pyx_GIVEREF(__pyx_tuple__67);

  /* "pyart/io/_sigmetfile.pyx":1040
 *     ('ingest_iris_version', '8s'),
 *     ('ingest_time', '12s'),         # 12 bytes: ymds_time
 *     ('spare_0', '28s'),             # 28 bytes             # <<<<<<<<<<<<<<
 *     ('GMT_minute_offset_local', SINT2),
 *     ('ingest_hardware_name_', '16s'),
 */
  __pyx_tuple__68 = PyTuple_Pack(2, __pyx_n_u_spare_0, __pyx_kp_u_28s); if (unlikely(!__pyx_tuple__68)) __PYX_ERR(0, 1040, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__68);
  __Pyx_GIVEREF(__pyx_tuple__68);

  /* "pyart/io/_sigmetfile.pyx":1042
 *     ('spare_0', '28s'),             # 28 bytes
 *     ('GMT_minute_offset_local', SINT2),
 *     ('ingest_hardware_name_', '16s'),             # <<<<<<<<<<<<<<
 *     ('ingest_site_name_', '16s'),
 *     ('GMT_minute_offset_standard', SINT2),
 */
  __pyx_tuple__69 = PyTuple_Pack(2, __pyx_n_u_ingest_hardware_name, __pyx_kp_u_16s); if (unlikely(!__pyx_tuple__69)) __PYX_ERR(0, 1042, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__69);
  __Pyx_GIVEREF(__pyx_tuple__69);

  /* "pyart/io/_sigmetfile.pyx":1043
 *     ('GMT_minute_offset_local', SINT2),
 *     ('ingest_hardware_name_', '16s'),
 *     ('ingest_site_name_', '16s'),             # <<<<<<<<<<<<<<
 *     ('GMT_minute_offset_standard', SINT2),
 *     ('latitude', BIN4),
 */
  __pyx_tuple__70 = PyTuple_Pack(2, __pyx_n_u_ingest_site_name, __pyx_kp_u_16s); if (unlikely(!__pyx_tuple__70)) __PYX_ERR(0, 1043, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__70);
  __Pyx_GIVEREF(__pyx_tuple__70);

  /* "pyart/io/_sigmetfile.pyx":1054
 *     ('trigger_rate', UINT2),
 *     ('samples_used', SINT2),
 *     ('clutter_filter', '12s'),             # <<<<<<<<<<<<<<
 *     ('number_linear_filter', UINT2),
 *     ('wavelength', SINT4),
 */
  __pyx_tuple__71 = PyTuple_Pack(2, __pyx_n_u_clutter_filter, __pyx_kp_u_12s); if (unlikely(!__pyx_tuple__71)) __PYX_ERR(0, 1054, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__71);
  __Pyx_GIVEREF(__pyx_tuple__71);

  /* "pyart/io/_sigmetfile.pyx":1074
 *     ('tcf_cal_flags_1', UINT16_T),
 *     ('tcf_cal_flags_2', UINT16_T),
 *     ('spare_1', '18s'),             # 18 bytes             # <<<<<<<<<<<<<<
 *     ('standard_parallel_1', BIN4),
 *     ('standard_parallel_2', BIN4),
 */
  __pyx_tuple__72 = PyTuple_Pack(2, __pyx_n_u_spare_1, __pyx_kp_u_18s); if (unlikely(!__pyx_tuple__72)) __PYX_ERR(0, 1074, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__72);
  __Pyx_GIVEREF(__pyx_tuple__72);

  /* "pyart/io/_sigmetfile.pyx":1086
 *     ('longitude_projection', BIN4),
 *     ('product_sequence_number', SINT2),
 *     ('spare_2', '32s'),             # 32 bytes             # <<<<<<<<<<<<<<
 *     ('melting_level', SINT2),
 *     ('radar_height_above_reference', SINT2),
 */
  __pyx_tuple__73 = PyTuple_Pack(2, __pyx_n_u_spare_2, __pyx_kp_u_32s); if (unlikely(!__pyx_tuple__73)) __PYX_ERR(0, 1086, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__73);
  __Pyx_GIVEREF(__pyx_tuple__73);

  /* "pyart/io/_sigmetfile.pyx":1092
 *     ('mean_wind_speed', UINT1),
 *     ('mean_wind_direction', BIN1),
 *     ('spare_3', '2s'),              # 2 bytes             # <<<<<<<<<<<<<<
 *     ('tz_name', '8s'),
 *     ('extended_product_header_offset', UINT4),
 */
  __pyx_tuple__74 = PyTuple_Pack(2, __pyx_n_u_spare_3, __pyx_kp_u_2s); if (unlikely(!__pyx_tuple__74)) __PYX_ERR(0, 1092, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__74);
  __Pyx_GIVEREF(__pyx_tuple__74);

  /* "pyart/io/_sigmetfile.pyx":1093
 *     ('mean_wind_direction', BIN1),
 *     ('spare_3', '2s'),              # 2 bytes
 *     ('tz_name', '8s'),             # <<<<<<<<<<<<<<
 *     ('extended_product_header_offset', UINT4),
 *     ('spare_4', '4s'),              # 4 bytes
 */
  __pyx_tuple__75 = PyTuple_Pack(2, __pyx_n_u_tz_name, __pyx_kp_u_8s); if (unlikely(!__pyx_tuple__75)) __PYX_ERR(0, 1093, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__75);
  __Pyx_GIVEREF(__pyx_tuple__75);

  /* "pyart/io/_sigmetfile.pyx":1095
 *     ('tz_name', '8s'),
 *     ('extended_product_header_offset', UINT4),
 *     ('spare_4', '4s'),              # 4 bytes             # <<<<<<<<<<<<<<
 * )
 * 
 */
  __pyx_tuple__76 = PyTuple_Pack(2, __pyx_n_u_spare_4, __pyx_kp_u_4s); if (unlikely(!__pyx_tuple__76)) __PYX_ERR(0, 1095, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__76);
  __Pyx_GIVEREF(__pyx_tuple__76);

  /* "pyart/io/_sigmetfile.pyx":1101
 * INGEST_HEADER = (
 *     ('structure_header', '12s'),        # 12 bytes: structure_header
 *     ('ingest_configuration', '480s'),   # 480 bytes: ingest_configuration             # <<<<<<<<<<<<<<
 *     ('task_configuration', '2612s'),    # 2612 bytes: task_configuration
 *     ('spare_0', '732s'),                # 732 bytes
 */
  __pyx_tuple__77 = PyTuple_Pack(2, __pyx_n_u_ingest_configuration, __pyx_kp_u_480s); if (unlikely(!__pyx_tuple__77)) __PYX_ERR(0, 1101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__77);
  __Pyx_GIVEREF(__pyx_tuple__77);

  /* "pyart/io/_sigmetfile.pyx":1102
 *     ('structure_header', '12s'),        # 12 bytes: structure_header
 *     ('ingest_configuration', '480s'),   # 480 bytes: ingest_configuration
 *     ('task_configuration', '2612s'),    # 2612 bytes: task_configuration             # <<<<<<<<<<<<<<
 *     ('spare_0', '732s'),                # 732 bytes
 *     ('gparm', '128s'),                  # 128 bytes
 */
  __pyx_tuple__78 = PyTuple_Pack(2, __pyx_n_u_task_configuration, __pyx_kp_u_2612s); if (unlikely(!__pyx_tuple__78)) __PYX_ERR(0, 1102, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__78);
  __Pyx_GIVEREF(__pyx_tuple__78);

  /* "pyart/io/_sigmetfile.pyx":1103
 *     ('ingest_configuration', '480s'),   # 480 bytes: ingest_configuration
 *     ('task_configuration', '2612s'),    # 2612 bytes: task_configuration
 *     ('spare_0', '732s'),                # 732 bytes             # <<<<<<<<<<<<<<
 *     ('gparm', '128s'),                  # 128 bytes
 *     ('reserved', '920s'),               # 920 bytes
 */
  __pyx_tuple__79 = PyTuple_Pack(2, __pyx_n_u_spare_0, __pyx_kp_u_732s); if (unlikely(!__pyx_tuple__79)) __PYX_ERR(0, 1103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__79);
  __Pyx_GIVEREF(__pyx_tuple__79);

  /* "pyart/io/_sigmetfile.pyx":1104
 *     ('task_configuration', '2612s'),    # 2612 bytes: task_configuration
 *     ('spare_0', '732s'),                # 732 bytes
 *     ('gparm', '128s'),                  # 128 bytes             # <<<<<<<<<<<<<<
 *     ('reserved', '920s'),               # 920 bytes
 * )
 */
  __pyx_tuple__80 = PyTuple_Pack(2, __pyx_n_u_gparm, __pyx_kp_u_128s); if (unlikely(!__pyx_tuple__80)) __PYX_ERR(0, 1104, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__80);
  __Pyx_GIVEREF(__pyx_tuple__80);

  /* "pyart/io/_sigmetfile.pyx":1105
 *     ('spare_0', '732s'),                # 732 bytes
 *     ('gparm', '128s'),                  # 128 bytes
 *     ('reserved', '920s'),               # 920 bytes             # <<<<<<<<<<<<<<
 * )
 * 
 */
  __pyx_tuple__81 = PyTuple_Pack(2, __pyx_n_u_reserved, __pyx_kp_u_920s); if (unlikely(!__pyx_tuple__81)) __PYX_ERR(0, 1105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__81);
  __Pyx_GIVEREF(__pyx_tuple__81);

  /* "pyart/io/_sigmetfile.pyx":1100
 * # 4884 bytes ingest_header Structure (section 4.2.16, page 40)
 * INGEST_HEADER = (
 *     ('structure_header', '12s'),        # 12 bytes: structure_header             # <<<<<<<<<<<<<<
 *     ('ingest_configuration', '480s'),   # 480 bytes: ingest_configuration
 *     ('task_configuration', '2612s'),    # 2612 bytes: task_configuration
 */
  __pyx_tuple__82 = PyTuple_Pack(6, __pyx_tuple__47, __pyx_tuple__77, __pyx_tuple__78, __pyx_tuple__79, __pyx_tuple__80, __pyx_tuple__81); if (unlikely(!__pyx_tuple__82)) __PYX_ERR(0, 1100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__82);
  __Pyx_GIVEREF(__pyx_tuple__82);

  /* "pyart/io/_sigmetfile.pyx":1110
 * # 480 bytes ingest_configuration Structure (section 4.2.14, page 38)
 * INGEST_CONFIGURATION = (
 *     ('filename', '80s'),             # <<<<<<<<<<<<<<
 *     ('number_files', SINT2),
 *     ('number_sweeps_completed', SINT2),
 */
  __pyx_tuple__83 = PyTuple_Pack(2, __pyx_n_u_filename, __pyx_kp_u_80s); if (unlikely(!__pyx_tuple__83)) __PYX_ERR(0, 1110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__83);
  __Pyx_GIVEREF(__pyx_tuple__83);

  /* "pyart/io/_sigmetfile.pyx":1114
 *     ('number_sweeps_completed', SINT2),
 *     ('total_size', SINT4),
 *     ('volume_scan_start_time', '12s'),  # 12 bytes: ymds_time             # <<<<<<<<<<<<<<
 *     ('spare_0', '12s'),                 # 12 bytes
 *     ('ray_header_bytes', SINT2),
 */
  __pyx_tuple__84 = PyTuple_Pack(2, __pyx_n_u_volume_scan_start_time, __pyx_kp_u_12s); if (unlikely(!__pyx_tuple__84)) __PYX_ERR(0, 1114, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__84);
  __Pyx_GIVEREF(__pyx_tuple__84);

  /* "pyart/io/_sigmetfile.pyx":1115
 *     ('total_size', SINT4),
 *     ('volume_scan_start_time', '12s'),  # 12 bytes: ymds_time
 *     ('spare_0', '12s'),                 # 12 bytes             # <<<<<<<<<<<<<<
 *     ('ray_header_bytes', SINT2),
 *     ('extended_ray_header_bytes', SINT2),
 */
  __pyx_tuple__85 = PyTuple_Pack(2, __pyx_n_u_spare_0, __pyx_kp_u_12s); if (unlikely(!__pyx_tuple__85)) __PYX_ERR(0, 1115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__85);
  __Pyx_GIVEREF(__pyx_tuple__85);

  /* "pyart/io/_sigmetfile.pyx":1120
 *     ('number_task_config_table', SINT2),
 *     ('playback_version', SINT2),
 *     ('spare_1', '4s'),                  # 4 bytes             # <<<<<<<<<<<<<<
 *     ('iris_version', '8s'),
 *     ('hardware_site', '16s'),
 */
  __pyx_tuple__86 = PyTuple_Pack(2, __pyx_n_u_spare_1, __pyx_kp_u_4s); if (unlikely(!__pyx_tuple__86)) __PYX_ERR(0, 1120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__86);
  __Pyx_GIVEREF(__pyx_tuple__86);

  /* "pyart/io/_sigmetfile.pyx":1121
 *     ('playback_version', SINT2),
 *     ('spare_1', '4s'),                  # 4 bytes
 *     ('iris_version', '8s'),             # <<<<<<<<<<<<<<
 *     ('hardware_site', '16s'),
 *     ('gmt_offset_minutes_local', SINT2),
 */
  __pyx_tuple__87 = PyTuple_Pack(2, __pyx_n_u_iris_version, __pyx_kp_u_8s); if (unlikely(!__pyx_tuple__87)) __PYX_ERR(0, 1121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__87);
  __Pyx_GIVEREF(__pyx_tuple__87);

  /* "pyart/io/_sigmetfile.pyx":1122
 *     ('spare_1', '4s'),                  # 4 bytes
 *     ('iris_version', '8s'),
 *     ('hardware_site', '16s'),             # <<<<<<<<<<<<<<
 *     ('gmt_offset_minutes_local', SINT2),
 *     ('site_name', '16s'),
 */
  __pyx_tuple__88 = PyTuple_Pack(2, __pyx_n_u_hardware_site, __pyx_kp_u_16s); if (unlikely(!__pyx_tuple__88)) __PYX_ERR(0, 1122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__88);
  __Pyx_GIVEREF(__pyx_tuple__88);

  /* "pyart/io/_sigmetfile.pyx":1143
 *     ('fault_status', UINT4),
 *     ('melting_layer', SINT2),
 *     ('spare_2', '2s'),              # 2 bytes             # <<<<<<<<<<<<<<
 *     ('local_timezone', '8s'),
 *     ('flags', UINT4),
 */
  __pyx_tuple__89 = PyTuple_Pack(2, __pyx_n_u_spare_2, __pyx_kp_u_2s); if (unlikely(!__pyx_tuple__89)) __PYX_ERR(0, 1143, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__89);
  __Pyx_GIVEREF(__pyx_tuple__89);

  /* "pyart/io/_sigmetfile.pyx":1144
 *     ('melting_layer', SINT2),
 *     ('spare_2', '2s'),              # 2 bytes
 *     ('local_timezone', '8s'),             # <<<<<<<<<<<<<<
 *     ('flags', UINT4),
 *     ('configuration_name', '16s'),
 */
  __pyx_tuple__90 = PyTuple_Pack(2, __pyx_n_u_local_timezone, __pyx_kp_u_8s); if (unlikely(!__pyx_tuple__90)) __PYX_ERR(0, 1144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__90);
  __Pyx_GIVEREF(__pyx_tuple__90);

  /* "pyart/io/_sigmetfile.pyx":1146
 *     ('local_timezone', '8s'),
 *     ('flags', UINT4),
 *     ('configuration_name', '16s'),             # <<<<<<<<<<<<<<
 *     ('spare_3', '228s')
 * )
 */
  __pyx_tuple__91 = PyTuple_Pack(2, __pyx_n_u_configuration_name, __pyx_kp_u_16s); if (unlikely(!__pyx_tuple__91)) __PYX_ERR(0, 1146, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__91);
  __Pyx_GIVEREF(__pyx_tuple__91);

  /* "pyart/io/_sigmetfile.pyx":1147
 *     ('flags', UINT4),
 *     ('configuration_name', '16s'),
 *     ('spare_3', '228s')             # <<<<<<<<<<<<<<
 * )
 * 
 */
  __pyx_tuple__92 = PyTuple_Pack(2, __pyx_n_u_spare_3, __pyx_kp_u_228s); if (unlikely(!__pyx_tuple__92)) __PYX_ERR(0, 1147, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__92);
  __Pyx_GIVEREF(__pyx_tuple__92);

  /* "pyart/io/_sigmetfile.pyx":1153
 * TASK_CONFIGURATION = (
 *     ('structure_header', '12s'),    # 12 bytes: structure_header
 *     ('task_sched_info', '120s'),    # 120 bytes: task_sched_info             # <<<<<<<<<<<<<<
 *     ('task_dsp_info', '320s'),      # 320 bytes: task_dsp_info
 *     ('task_calib_info', '320s'),    # 320 bytes: task_calib_info
 */
  __pyx_tuple__93 = PyTuple_Pack(2, __pyx_n_u_task_sched_info, __pyx_kp_u_120s); if (unlikely(!__pyx_tuple__93)) __PYX_ERR(0, 1153, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__93);
  __Pyx_GIVEREF(__pyx_tuple__93);

  /* "pyart/io/_sigmetfile.pyx":1154
 *     ('structure_header', '12s'),    # 12 bytes: structure_header
 *     ('task_sched_info', '120s'),    # 120 bytes: task_sched_info
 *     ('task_dsp_info', '320s'),      # 320 bytes: task_dsp_info             # <<<<<<<<<<<<<<
 *     ('task_calib_info', '320s'),    # 320 bytes: task_calib_info
 *     ('task_range_info', '160s'),    # 160 bytes: task_range_info
 */
  __pyx_tuple__94 = PyTuple_Pack(2, __pyx_n_u_task_dsp_info, __pyx_kp_u_320s); if (unlikely(!__pyx_tuple__94)) __PYX_ERR(0, 1154, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__94);
  __Pyx_GIVEREF(__pyx_tuple__94);

  /* "pyart/io/_sigmetfile.pyx":1155
 *     ('task_sched_info', '120s'),    # 120 bytes: task_sched_info
 *     ('task_dsp_info', '320s'),      # 320 bytes: task_dsp_info
 *     ('task_calib_info', '320s'),    # 320 bytes: task_calib_info             # <<<<<<<<<<<<<<
 *     ('task_range_info', '160s'),    # 160 bytes: task_range_info
 *     ('task_scan_info', '320s'),     # 320 bytes: task_scan_info
 */
  __pyx_tuple__95 = PyTuple_Pack(2, __pyx_n_u_task_calib_info, __pyx_kp_u_320s); if (unlikely(!__pyx_tuple__95)) __PYX_ERR(0, 1155, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__95);
  __Pyx_GIVEREF(__pyx_tuple__95);

  /* "pyart/io/_sigmetfile.pyx":1156
 *     ('task_dsp_info', '320s'),      # 320 bytes: task_dsp_info
 *     ('task_calib_info', '320s'),    # 320 bytes: task_calib_info
 *     ('task_range_info', '160s'),    # 160 bytes: task_range_info             # <<<<<<<<<<<<<<
 *     ('task_scan_info', '320s'),     # 320 bytes: task_scan_info
 *     ('task_misc_info', '320s'),     # 320 bytes: task_misc_info
 */
  __pyx_tuple__96 = PyTuple_Pack(2, __pyx_n_u_task_range_info, __pyx_kp_u_160s); if (unlikely(!__pyx_tuple__96)) __PYX_ERR(0, 1156, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__96);
  __Pyx_GIVEREF(__pyx_tuple__96);

  /* "pyart/io/_sigmetfile.pyx":1157
 *     ('task_calib_info', '320s'),    # 320 bytes: task_calib_info
 *     ('task_range_info', '160s'),    # 160 bytes: task_range_info
 *     ('task_scan_info', '320s'),     # 320 bytes: task_scan_info             # <<<<<<<<<<<<<<
 *     ('task_misc_info', '320s'),     # 320 bytes: task_misc_info
 *     ('task_end_info', '320s'),      # 320 bytes: task_end_info
 */
  __pyx_tuple__97 = PyTuple_Pack(2, __pyx_n_u_task_scan_info, __pyx_kp_u_320s); if (unlikely(!__pyx_tuple__97)) __PYX_ERR(0, 1157, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__97);
  __Pyx_GIVEREF(__pyx_tuple__97);

  /* "pyart/io/_sigmetfile.pyx":1158
 *     ('task_range_info', '160s'),    # 160 bytes: task_range_info
 *     ('task_scan_info', '320s'),     # 320 bytes: task_scan_info
 *     ('task_misc_info', '320s'),     # 320 bytes: task_misc_info             # <<<<<<<<<<<<<<
 *     ('task_end_info', '320s'),      # 320 bytes: task_end_info
 *     ('comments', '720s'),
 */
  __pyx_tuple__98 = PyTuple_Pack(2, __pyx_n_u_task_misc_info, __pyx_kp_u_320s); if (unlikely(!__pyx_tuple__98)) __PYX_ERR(0, 1158, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__98);
  __Pyx_GIVEREF(__pyx_tuple__98);

  /* "pyart/io/_sigmetfile.pyx":1159
 *     ('task_scan_info', '320s'),     # 320 bytes: task_scan_info
 *     ('task_misc_info', '320s'),     # 320 bytes: task_misc_info
 *     ('task_end_info', '320s'),      # 320 bytes: task_end_info             # <<<<<<<<<<<<<<
 *     ('comments', '720s'),
 * )
 */
  __pyx_tuple__99 = PyTuple_Pack(2, __pyx_n_u_task_end_info, __pyx_kp_u_320s); if (unlikely(!__pyx_tuple__99)) __PYX_ERR(0, 1159, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__99);
  __Pyx_GIVEREF(__pyx_tuple__99);

  /* "pyart/io/_sigmetfile.pyx":1160
 *     ('task_misc_info', '320s'),     # 320 bytes: task_misc_info
 *     ('task_end_info', '320s'),      # 320 bytes: task_end_info
 *     ('comments', '720s'),             # <<<<<<<<<<<<<<
 * )
 * 
 */
  __pyx_tuple__100 = PyTuple_Pack(2, __pyx_n_u_comments, __pyx_kp_u_720s); if (unlikely(!__pyx_tuple__100)) __PYX_ERR(0, 1160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__100);
  __Pyx_GIVEREF(__pyx_tuple__100);

  /* "pyart/io/_sigmetfile.pyx":1152
 * # 2612 bytes: task_configuration Structure (section 4.2.50, page 61)
 * TASK_CONFIGURATION = (
 *     ('structure_header', '12s'),    # 12 bytes: structure_header             # <<<<<<<<<<<<<<
 *     ('task_sched_info', '120s'),    # 120 bytes: task_sched_info
 *     ('task_dsp_info', '320s'),      # 320 bytes: task_dsp_info
 */
  __pyx_tuple__101 = PyTuple_Pack(9, __pyx_tuple__47, __pyx_tuple__93, __pyx_tuple__94, __pyx_tuple__95, __pyx_tuple__96, __pyx_tuple__97, __pyx_tuple__98, __pyx_tuple__99, __pyx_tuple__100); if (unlikely(!__pyx_tuple__101)) __PYX_ERR(0, 1152, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__101);
  __Pyx_GIVEREF(__pyx_tuple__101);

  /* "pyart/io/_sigmetfile.pyx":1172
 *     ('last_run_day', SINT4),
 *     ('flag', UINT2),
 *     ('spare_0', '94s'),             # <<<<<<<<<<<<<<
 * )
 * 
 */
  __pyx_tuple__102 = PyTuple_Pack(2, __pyx_n_u_spare_0, __pyx_kp_u_94s); if (unlikely(!__pyx_tuple__102)) __PYX_ERR(0, 1172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__102);
  __Pyx_GIVEREF(__pyx_tuple__102);

  /* "pyart/io/_sigmetfile.pyx":1179
 *     ('major_mode', UINT2),
 *     ('dsp_type', UINT2),
 *     ('current_data_type_mask', '24s'),      # 24 bytes: dsp_data_mask             # <<<<<<<<<<<<<<
 *     ('original_data_type_mask', '24s'),     # 24 bytes: dsp_data_mask
 *     ('task_dsp_mode', '32s'),               # 32 bytes: task_dsp_mode_batch
 */
  __pyx_tuple__103 = PyTuple_Pack(2, __pyx_n_u_current_data_type_mask, __pyx_kp_u_24s); if (unlikely(!__pyx_tuple__103)) __PYX_ERR(0, 1179, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__103);
  __Pyx_GIVEREF(__pyx_tuple__103);

  /* "pyart/io/_sigmetfile.pyx":1180
 *     ('dsp_type', UINT2),
 *     ('current_data_type_mask', '24s'),      # 24 bytes: dsp_data_mask
 *     ('original_data_type_mask', '24s'),     # 24 bytes: dsp_data_mask             # <<<<<<<<<<<<<<
 *     ('task_dsp_mode', '32s'),               # 32 bytes: task_dsp_mode_batch
 *     ('spare_0', '52s'),
 */
  __pyx_tuple__104 = PyTuple_Pack(2, __pyx_n_u_original_data_type_mask, __pyx_kp_u_24s); if (unlikely(!__pyx_tuple__104)) __PYX_ERR(0, 1180, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__104);
  __Pyx_GIVEREF(__pyx_tuple__104);

  /* "pyart/io/_sigmetfile.pyx":1181
 *     ('current_data_type_mask', '24s'),      # 24 bytes: dsp_data_mask
 *     ('original_data_type_mask', '24s'),     # 24 bytes: dsp_data_mask
 *     ('task_dsp_mode', '32s'),               # 32 bytes: task_dsp_mode_batch             # <<<<<<<<<<<<<<
 *     ('spare_0', '52s'),
 *     ('prf', SINT4),
 */
  __pyx_tuple__105 = PyTuple_Pack(2, __pyx_n_u_task_dsp_mode, __pyx_kp_u_32s); if (unlikely(!__pyx_tuple__105)) __PYX_ERR(0, 1181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__105);
  __Pyx_GIVEREF(__pyx_tuple__105);

  /* "pyart/io/_sigmetfile.pyx":1182
 *     ('original_data_type_mask', '24s'),     # 24 bytes: dsp_data_mask
 *     ('task_dsp_mode', '32s'),               # 32 bytes: task_dsp_mode_batch
 *     ('spare_0', '52s'),             # <<<<<<<<<<<<<<
 *     ('prf', SINT4),
 *     ('pulse_width', SINT4),
 */
  __pyx_tuple__106 = PyTuple_Pack(2, __pyx_n_u_spare_0, __pyx_kp_u_52s); if (unlikely(!__pyx_tuple__106)) __PYX_ERR(0, 1182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__106);
  __Pyx_GIVEREF(__pyx_tuple__106);

  /* "pyart/io/_sigmetfile.pyx":1190
 *     ('sample_size', SINT2),
 *     ('gain_control_flag', UINT2),
 *     ('clutter_filter_name', '12s'),             # <<<<<<<<<<<<<<
 *     ('linear_filter_first_bin', UINT1),
 *     ('log_filter_first_bin', UINT1),
 */
  __pyx_tuple__107 = PyTuple_Pack(2, __pyx_n_u_clutter_filter_name, __pyx_kp_u_12s); if (unlikely(!__pyx_tuple__107)) __PYX_ERR(0, 1190, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__107);
  __Pyx_GIVEREF(__pyx_tuple__107);

  /* "pyart/io/_sigmetfile.pyx":1199
 *     ('ray_header_mask', UINT4),
 *     ('playback_flag', UINT2),
 *     ('spare_1', '2s'),             # <<<<<<<<<<<<<<
 *     ('custom_ray_header_name', '16s'),
 *     ('spare_2', '120s')
 */
  __pyx_tuple__108 = PyTuple_Pack(2, __pyx_n_u_spare_1, __pyx_kp_u_2s); if (unlikely(!__pyx_tuple__108)) __PYX_ERR(0, 1199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__108);
  __Pyx_GIVEREF(__pyx_tuple__108);

  /* "pyart/io/_sigmetfile.pyx":1200
 *     ('playback_flag', UINT2),
 *     ('spare_1', '2s'),
 *     ('custom_ray_header_name', '16s'),             # <<<<<<<<<<<<<<
 *     ('spare_2', '120s')
 * )
 */
  __pyx_tuple__109 = PyTuple_Pack(2, __pyx_n_u_custom_ray_header_name, __pyx_kp_u_16s); if (unlikely(!__pyx_tuple__109)) __PYX_ERR(0, 1200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__109);
  __Pyx_GIVEREF(__pyx_tuple__109);

  /* "pyart/io/_sigmetfile.pyx":1201
 *     ('spare_1', '2s'),
 *     ('custom_ray_header_name', '16s'),
 *     ('spare_2', '120s')             # <<<<<<<<<<<<<<
 * )
 * 
 */
  __pyx_tuple__110 = PyTuple_Pack(2, __pyx_n_u_spare_2, __pyx_kp_u_120s); if (unlikely(!__pyx_tuple__110)) __PYX_ERR(0, 1201, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__110);
  __Pyx_GIVEREF(__pyx_tuple__110);

  /* "pyart/io/_sigmetfile.pyx":1223
 *     ('velocity_unfolding_threshold', SINT2),
 *     ('width_unfolding_threshold', SINT2),
 *     ('spare_0', '18s'),             # <<<<<<<<<<<<<<
 * )
 * 
 */
  __pyx_tuple__111 = PyTuple_Pack(2, __pyx_n_u_spare_0, __pyx_kp_u_18s); if (unlikely(!__pyx_tuple__111)) __PYX_ERR(0, 1223, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__111);
  __Pyx_GIVEREF(__pyx_tuple__111);

  /* "pyart/io/_sigmetfile.pyx":1233
 *     ('sqi_threshold', SINT2),
 *     ('power_threshold', SINT2),
 *     ('spare_0', '8s'),             # <<<<<<<<<<<<<<
 *     ('reflectivity_calibration', SINT2),
 *     ('uncorrected_reflectivity_threshold_flags', UINT2),
 */
  __pyx_tuple__112 = PyTuple_Pack(2, __pyx_n_u_spare_0, __pyx_kp_u_8s); if (unlikely(!__pyx_tuple__112)) __PYX_ERR(0, 1233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__112);
  __Pyx_GIVEREF(__pyx_tuple__112);

  /* "pyart/io/_sigmetfile.pyx":1240
 *     ('width_threshold_flags', UINT2),
 *     ('zdr_threshold_flags', UINT2),
 *     ('spare_1', '6s'),             # <<<<<<<<<<<<<<
 *     ('flags', UINT2),
 *     ('spare_2', '2s'),
 */
  __pyx_tuple__113 = PyTuple_Pack(2, __pyx_n_u_spare_1, __pyx_kp_u_6s); if (unlikely(!__pyx_tuple__113)) __PYX_ERR(0, 1240, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__113);
  __Pyx_GIVEREF(__pyx_tuple__113);

  /* "pyart/io/_sigmetfile.pyx":1255
 *     ('reciever_bandwidth', UINT2),
 *     ('flags2', UINT16_T),
 *     ('spare_3', '256s'),             # <<<<<<<<<<<<<<
 * )
 * 
 */
  __pyx_tuple__114 = PyTuple_Pack(2, __pyx_n_u_spare_3, __pyx_kp_u_256s); if (unlikely(!__pyx_tuple__114)) __PYX_ERR(0, 1255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__114);
  __Pyx_GIVEREF(__pyx_tuple__114);

  /* "pyart/io/_sigmetfile.pyx":1268
 *     ('variable_range_bin_flag', UINT2),
 *     ('range_bin_averaging_flag', SINT2),
 *     ('spare_0', '136s'),             # <<<<<<<<<<<<<<
 * )
 * 
 */
  __pyx_tuple__115 = PyTuple_Pack(2, __pyx_n_u_spare_0, __pyx_kp_u_136s); if (unlikely(!__pyx_tuple__115)) __PYX_ERR(0, 1268, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__115);
  __Pyx_GIVEREF(__pyx_tuple__115);

  /* "pyart/io/_sigmetfile.pyx":1275
 *     ('antenna_scan_mode', UINT2),
 *     ('angular_resolution_desired', SINT2),
 *     ('spare_0', '2s'),             # <<<<<<<<<<<<<<
 *     ('number_sweeps', SINT2),
 *     ('task_scan_type_scan_info', '200s'),   # 200 bytes: task_foo_scan_info
 */
  __pyx_tuple__116 = PyTuple_Pack(2, __pyx_n_u_spare_0, __pyx_kp_u_2s); if (unlikely(!__pyx_tuple__116)) __PYX_ERR(0, 1275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__116);
  __Pyx_GIVEREF(__pyx_tuple__116);

  /* "pyart/io/_sigmetfile.pyx":1277
 *     ('spare_0', '2s'),
 *     ('number_sweeps', SINT2),
 *     ('task_scan_type_scan_info', '200s'),   # 200 bytes: task_foo_scan_info             # <<<<<<<<<<<<<<
 *     ('spare_1', '112s'),
 * )
 */
  __pyx_tuple__117 = PyTuple_Pack(2, __pyx_n_u_task_scan_type_scan_info, __pyx_kp_u_200s); if (unlikely(!__pyx_tuple__117)) __PYX_ERR(0, 1277, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__117);
  __Pyx_GIVEREF(__pyx_tuple__117);

  /* "pyart/io/_sigmetfile.pyx":1278
 *     ('number_sweeps', SINT2),
 *     ('task_scan_type_scan_info', '200s'),   # 200 bytes: task_foo_scan_info
 *     ('spare_1', '112s'),             # <<<<<<<<<<<<<<
 * )
 * 
 */
  __pyx_tuple__118 = PyTuple_Pack(2, __pyx_n_u_spare_1, __pyx_kp_u_112s); if (unlikely(!__pyx_tuple__118)) __PYX_ERR(0, 1278, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__118);
  __Pyx_GIVEREF(__pyx_tuple__118);

  /* "pyart/io/_sigmetfile.pyx":1285
 *     ('lower_elevation_limit', UINT2),
 *     ('upper_elevation_limit', UINT2),
 *     ('azimuth_list', '80s'),            # UINT2[40]             # <<<<<<<<<<<<<<
 *     ('spare_0', '115s'),
 *     ('start_first_sector_flag', 'c'),   # unknown type
 */
  __pyx_tuple__119 = PyTuple_Pack(2, __pyx_n_u_azimuth_list, __pyx_kp_u_80s); if (unlikely(!__pyx_tuple__119)) __PYX_ERR(0, 1285, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__119);
  __Pyx_GIVEREF(__pyx_tuple__119);

  /* "pyart/io/_sigmetfile.pyx":1286
 *     ('upper_elevation_limit', UINT2),
 *     ('azimuth_list', '80s'),            # UINT2[40]
 *     ('spare_0', '115s'),             # <<<<<<<<<<<<<<
 *     ('start_first_sector_flag', 'c'),   # unknown type
 * )
 */
  __pyx_tuple__120 = PyTuple_Pack(2, __pyx_n_u_spare_0, __pyx_kp_u_115s); if (unlikely(!__pyx_tuple__120)) __PYX_ERR(0, 1286, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__120);
  __Pyx_GIVEREF(__pyx_tuple__120);

  /* "pyart/io/_sigmetfile.pyx":1287
 *     ('azimuth_list', '80s'),            # UINT2[40]
 *     ('spare_0', '115s'),
 *     ('start_first_sector_flag', 'c'),   # unknown type             # <<<<<<<<<<<<<<
 * )
 * 
 */
  __pyx_tuple__121 = PyTuple_Pack(2, __pyx_n_u_start_first_sector_flag, __pyx_n_u_c); if (unlikely(!__pyx_tuple__121)) __PYX_ERR(0, 1287, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__121);
  __Pyx_GIVEREF(__pyx_tuple__121);

  /* "pyart/io/_sigmetfile.pyx":1294
 *     ('left_azimuth_limit', BIN2),
 *     ('right_azimuth_limit', BIN2),
 *     ('elevation_list', '80s'),          # UINT2[40]             # <<<<<<<<<<<<<<
 *     ('spare_0', '115s'),
 *     ('start_first_section_flag', 'c'),  # unknown type
 */
  __pyx_tuple__122 = PyTuple_Pack(2, __pyx_n_u_elevation_list, __pyx_kp_u_80s); if (unlikely(!__pyx_tuple__122)) __PYX_ERR(0, 1294, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__122);
  __Pyx_GIVEREF(__pyx_tuple__122);

  /* "pyart/io/_sigmetfile.pyx":1296
 *     ('elevation_list', '80s'),          # UINT2[40]
 *     ('spare_0', '115s'),
 *     ('start_first_section_flag', 'c'),  # unknown type             # <<<<<<<<<<<<<<
 * )
 * 
 */
  __pyx_tuple__123 = PyTuple_Pack(2, __pyx_n_u_start_first_section_flag, __pyx_n_u_c); if (unlikely(!__pyx_tuple__123)) __PYX_ERR(0, 1296, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__123);
  __Pyx_GIVEREF(__pyx_tuple__123);

  /* "pyart/io/_sigmetfile.pyx":1303
 *     ('first_azimuth', UINT2),
 *     ('first_elevation', UINT2),
 *     ('filename', '12s'),             # <<<<<<<<<<<<<<
 *     ('spare_0', '184s'),
 * )
 */
  __pyx_tuple__124 = PyTuple_Pack(2, __pyx_n_u_filename, __pyx_kp_u_12s); if (unlikely(!__pyx_tuple__124)) __PYX_ERR(0, 1303, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__124);
  __Pyx_GIVEREF(__pyx_tuple__124);

  /* "pyart/io/_sigmetfile.pyx":1304
 *     ('first_elevation', UINT2),
 *     ('filename', '12s'),
 *     ('spare_0', '184s'),             # <<<<<<<<<<<<<<
 * )
 * 
 */
  __pyx_tuple__125 = PyTuple_Pack(2, __pyx_n_u_spare_0, __pyx_kp_u_184s); if (unlikely(!__pyx_tuple__125)) __PYX_ERR(0, 1304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__125);
  __Pyx_GIVEREF(__pyx_tuple__125);

  /* "pyart/io/_sigmetfile.pyx":1310
 * TASK_MANUAL_SCAN_INFO = (
 *     ('flags', UINT2),
 *     ('spare_0', '198s'),             # <<<<<<<<<<<<<<
 * )
 * 
 */
  __pyx_tuple__126 = PyTuple_Pack(2, __pyx_n_u_spare_0, __pyx_kp_u_198s); if (unlikely(!__pyx_tuple__126)) __PYX_ERR(0, 1310, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__126);
  __Pyx_GIVEREF(__pyx_tuple__126);

  /* "pyart/io/_sigmetfile.pyx":1316
 * TASK_MISC_INFO = (
 *     ('wavelength', SINT4),
 *     ('tr_serial_number', '16s'),             # <<<<<<<<<<<<<<
 *     ('transmit_power', SINT4),
 *     ('flags', UINT2),
 */
  __pyx_tuple__127 = PyTuple_Pack(2, __pyx_n_u_tr_serial_number, __pyx_kp_u_16s); if (unlikely(!__pyx_tuple__127)) __PYX_ERR(0, 1316, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__127);
  __Pyx_GIVEREF(__pyx_tuple__127);

  /* "pyart/io/_sigmetfile.pyx":1322
 *     ('trucation_height', SINT4),
 *     ('spare_0', '18s'),             # 18 bytes
 *     ('spare_1', '12s'),             # 12 bytes             # <<<<<<<<<<<<<<
 *     ('comment_bytes', SINT2),
 *     ('horizontal_beamwidth', BIN4),
 */
  __pyx_tuple__128 = PyTuple_Pack(2, __pyx_n_u_spare_1, __pyx_kp_u_12s); if (unlikely(!__pyx_tuple__128)) __PYX_ERR(0, 1322, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__128);
  __Pyx_GIVEREF(__pyx_tuple__128);

  /* "pyart/io/_sigmetfile.pyx":1326
 *     ('horizontal_beamwidth', BIN4),
 *     ('vertical_beamwidth', BIN4),
 *     ('customer_storage', '40s'),    # 40 bytes, uint4[10]             # <<<<<<<<<<<<<<
 *     ('spare_2', '208s'),
 * )
 */
  __pyx_tuple__129 = PyTuple_Pack(2, __pyx_n_u_customer_storage, __pyx_kp_u_40s); if (unlikely(!__pyx_tuple__129)) __PYX_ERR(0, 1326, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__129);
  __Pyx_GIVEREF(__pyx_tuple__129);

  /* "pyart/io/_sigmetfile.pyx":1327
 *     ('vertical_beamwidth', BIN4),
 *     ('customer_storage', '40s'),    # 40 bytes, uint4[10]
 *     ('spare_2', '208s'),             # <<<<<<<<<<<<<<
 * )
 * 
 */
  __pyx_tuple__130 = PyTuple_Pack(2, __pyx_n_u_spare_2, __pyx_kp_u_208s); if (unlikely(!__pyx_tuple__130)) __PYX_ERR(0, 1327, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__130);
  __Pyx_GIVEREF(__pyx_tuple__130);

  /* "pyart/io/_sigmetfile.pyx":1334
 *     ('task_major_number', SINT2),
 *     ('task_minor_number', SINT2),
 *     ('task_configuration_file_name', '12s'),             # <<<<<<<<<<<<<<
 *     ('task_description', '80s'),
 *     ('number_tasks', SINT4),
 */
  __pyx_tuple__131 = PyTuple_Pack(2, __pyx_n_u_task_configuration_file_name, __pyx_kp_u_12s); if (unlikely(!__pyx_tuple__131)) __PYX_ERR(0, 1334, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__131);
  __Pyx_GIVEREF(__pyx_tuple__131);

  /* "pyart/io/_sigmetfile.pyx":1335
 *     ('task_minor_number', SINT2),
 *     ('task_configuration_file_name', '12s'),
 *     ('task_description', '80s'),             # <<<<<<<<<<<<<<
 *     ('number_tasks', SINT4),
 *     ('task_state', UINT2),
 */
  __pyx_tuple__132 = PyTuple_Pack(2, __pyx_n_u_task_description, __pyx_kp_u_80s); if (unlikely(!__pyx_tuple__132)) __PYX_ERR(0, 1335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__132);
  __Pyx_GIVEREF(__pyx_tuple__132);

  /* "pyart/io/_sigmetfile.pyx":1339
 *     ('task_state', UINT2),
 *     ('spare_0', '2s'),
 *     ('task_data_time', '12s'),      # 12 bytes: ymds_time             # <<<<<<<<<<<<<<
 *     ('spare_1', '204s'),
 * )
 */
  __pyx_tuple__133 = PyTuple_Pack(2, __pyx_n_u_task_data_time, __pyx_kp_u_12s); if (unlikely(!__pyx_tuple__133)) __PYX_ERR(0, 1339, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__133);
  __Pyx_GIVEREF(__pyx_tuple__133);

  /* "pyart/io/_sigmetfile.pyx":1340
 *     ('spare_0', '2s'),
 *     ('task_data_time', '12s'),      # 12 bytes: ymds_time
 *     ('spare_1', '204s'),             # <<<<<<<<<<<<<<
 * )
 * 
 */
  __pyx_tuple__134 = PyTuple_Pack(2, __pyx_n_u_spare_1, __pyx_kp_u_204s); if (unlikely(!__pyx_tuple__134)) __PYX_ERR(0, 1340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__134);
  __Pyx_GIVEREF(__pyx_tuple__134);

  /* "pyart/io/_sigmetfile.pyx":1357
 * INGEST_DATA_HEADER = (
 *     ('structure_header', '12s'),    # 12 bytes: structure_header
 *     ('sweep_start_time', '12s'),    # 12 bytes: ymds_time             # <<<<<<<<<<<<<<
 *     ('sweep_number', SINT2),
 *     ('number_rays_sweep', SINT2),
 */
  __pyx_tuple__135 = PyTuple_Pack(2, __pyx_n_u_sweep_start_time, __pyx_kp_u_12s); if (unlikely(!__pyx_tuple__135)) __PYX_ERR(0, 1357, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__135);
  __Pyx_GIVEREF(__pyx_tuple__135);

  /* "pyart/io/_sigmetfile.pyx":1366
 *     ('bit_per_bin', SINT2),
 *     ('data_type', UINT2),
 *     ('spare_0', '36s')      # 36 bytes             # <<<<<<<<<<<<<<
 * )
 */
  __pyx_tuple__136 = PyTuple_Pack(2, __pyx_n_u_spare_0, __pyx_kp_u_36s); if (unlikely(!__pyx_tuple__136)) __PYX_ERR(0, 1366, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__136);
  __Pyx_GIVEREF(__pyx_tuple__136);
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_float_1_ = PyFloat_FromDouble(1.); if (unlikely(!__pyx_float_1_)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_2_ = PyFloat_FromDouble(2.); if (unlikely(!__pyx_float_2_)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_ = PyFloat_FromDouble(16.); if (unlikely(!__pyx_float_16_)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_64_ = PyFloat_FromDouble(64.); if (unlikely(!__pyx_float_64_)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_0_25 = PyFloat_FromDouble(0.25); if (unlikely(!__pyx_float_0_25)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_100_ = PyFloat_FromDouble(100.); if (unlikely(!__pyx_float_100_)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_126_ = PyFloat_FromDouble(126.); if (unlikely(!__pyx_float_126_)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_127_ = PyFloat_FromDouble(127.); if (unlikely(!__pyx_float_127_)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_128_ = PyFloat_FromDouble(128.); if (unlikely(!__pyx_float_128_)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_129_ = PyFloat_FromDouble(129.); if (unlikely(!__pyx_float_129_)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_180_ = PyFloat_FromDouble(180.); if (unlikely(!__pyx_float_180_)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_253_ = PyFloat_FromDouble(253.); if (unlikely(!__pyx_float_253_)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_254_ = PyFloat_FromDouble(254.); if (unlikely(!__pyx_float_254_)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_256_ = PyFloat_FromDouble(256.); if (unlikely(!__pyx_float_256_)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_360_ = PyFloat_FromDouble(360.); if (unlikely(!__pyx_float_360_)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_600_ = PyFloat_FromDouble(600.); if (unlikely(!__pyx_float_600_)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32768_ = PyFloat_FromDouble(32768.); if (unlikely(!__pyx_float_32768_)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_65533_ = PyFloat_FromDouble(65533.); if (unlikely(!__pyx_float_65533_)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_65534_ = PyFloat_FromDouble(65534.); if (unlikely(!__pyx_float_65534_)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_neg_0_25 = PyFloat_FromDouble(-0.25); if (unlikely(!__pyx_float_neg_0_25)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_neg_9999_0 = PyFloat_FromDouble(-9999.0); if (unlikely(!__pyx_float_neg_9999_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_3 = PyInt_FromLong(3); if (unlikely(!__pyx_int_3)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_4 = PyInt_FromLong(4); if (unlikely(!__pyx_int_4)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_5 = PyInt_FromLong(5); if (unlikely(!__pyx_int_5)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_6 = PyInt_FromLong(6); if (unlikely(!__pyx_int_6)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_7 = PyInt_FromLong(7); if (unlikely(!__pyx_int_7)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_8 = PyInt_FromLong(8); if (unlikely(!__pyx_int_8)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_9 = PyInt_FromLong(9); if (unlikely(!__pyx_int_9)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_10 = PyInt_FromLong(10); if (unlikely(!__pyx_int_10)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_11 = PyInt_FromLong(11); if (unlikely(!__pyx_int_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_12 = PyInt_FromLong(12); if (unlikely(!__pyx_int_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_13 = PyInt_FromLong(13); if (unlikely(!__pyx_int_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_14 = PyInt_FromLong(14); if (unlikely(!__pyx_int_14)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_15 = PyInt_FromLong(15); if (unlikely(!__pyx_int_15)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_16 = PyInt_FromLong(16); if (unlikely(!__pyx_int_16)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_17 = PyInt_FromLong(17); if (unlikely(!__pyx_int_17)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_18 = PyInt_FromLong(18); if (unlikely(!__pyx_int_18)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_19 = PyInt_FromLong(19); if (unlikely(!__pyx_int_19)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_20 = PyInt_FromLong(20); if (unlikely(!__pyx_int_20)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_21 = PyInt_FromLong(21); if (unlikely(!__pyx_int_21)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_22 = PyInt_FromLong(22); if (unlikely(!__pyx_int_22)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_23 = PyInt_FromLong(23); if (unlikely(!__pyx_int_23)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_24 = PyInt_FromLong(24); if (unlikely(!__pyx_int_24)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_25 = PyInt_FromLong(25); if (unlikely(!__pyx_int_25)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_26 = PyInt_FromLong(26); if (unlikely(!__pyx_int_26)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_27 = PyInt_FromLong(27); if (unlikely(!__pyx_int_27)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_28 = PyInt_FromLong(28); if (unlikely(!__pyx_int_28)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_29 = PyInt_FromLong(29); if (unlikely(!__pyx_int_29)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_30 = PyInt_FromLong(30); if (unlikely(!__pyx_int_30)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_31 = PyInt_FromLong(31); if (unlikely(!__pyx_int_31)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_32 = PyInt_FromLong(32); if (unlikely(!__pyx_int_32)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_33 = PyInt_FromLong(33); if (unlikely(!__pyx_int_33)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_34 = PyInt_FromLong(34); if (unlikely(!__pyx_int_34)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_35 = PyInt_FromLong(35); if (unlikely(!__pyx_int_35)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_36 = PyInt_FromLong(36); if (unlikely(!__pyx_int_36)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_37 = PyInt_FromLong(37); if (unlikely(!__pyx_int_37)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_38 = PyInt_FromLong(38); if (unlikely(!__pyx_int_38)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_39 = PyInt_FromLong(39); if (unlikely(!__pyx_int_39)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_40 = PyInt_FromLong(40); if (unlikely(!__pyx_int_40)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_41 = PyInt_FromLong(41); if (unlikely(!__pyx_int_41)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_42 = PyInt_FromLong(42); if (unlikely(!__pyx_int_42)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_43 = PyInt_FromLong(43); if (unlikely(!__pyx_int_43)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_44 = PyInt_FromLong(44); if (unlikely(!__pyx_int_44)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_45 = PyInt_FromLong(45); if (unlikely(!__pyx_int_45)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_46 = PyInt_FromLong(46); if (unlikely(!__pyx_int_46)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_47 = PyInt_FromLong(47); if (unlikely(!__pyx_int_47)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_48 = PyInt_FromLong(48); if (unlikely(!__pyx_int_48)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_49 = PyInt_FromLong(49); if (unlikely(!__pyx_int_49)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_50 = PyInt_FromLong(50); if (unlikely(!__pyx_int_50)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_51 = PyInt_FromLong(51); if (unlikely(!__pyx_int_51)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_52 = PyInt_FromLong(52); if (unlikely(!__pyx_int_52)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_53 = PyInt_FromLong(53); if (unlikely(!__pyx_int_53)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_54 = PyInt_FromLong(54); if (unlikely(!__pyx_int_54)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_55 = PyInt_FromLong(55); if (unlikely(!__pyx_int_55)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_56 = PyInt_FromLong(56); if (unlikely(!__pyx_int_56)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_57 = PyInt_FromLong(57); if (unlikely(!__pyx_int_57)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_58 = PyInt_FromLong(58); if (unlikely(!__pyx_int_58)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_59 = PyInt_FromLong(59); if (unlikely(!__pyx_int_59)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_60 = PyInt_FromLong(60); if (unlikely(!__pyx_int_60)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_61 = PyInt_FromLong(61); if (unlikely(!__pyx_int_61)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_62 = PyInt_FromLong(62); if (unlikely(!__pyx_int_62)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_63 = PyInt_FromLong(63); if (unlikely(!__pyx_int_63)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_64 = PyInt_FromLong(64); if (unlikely(!__pyx_int_64)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_65 = PyInt_FromLong(65); if (unlikely(!__pyx_int_65)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_66 = PyInt_FromLong(66); if (unlikely(!__pyx_int_66)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_67 = PyInt_FromLong(67); if (unlikely(!__pyx_int_67)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_68 = PyInt_FromLong(68); if (unlikely(!__pyx_int_68)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_69 = PyInt_FromLong(69); if (unlikely(!__pyx_int_69)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_70 = PyInt_FromLong(70); if (unlikely(!__pyx_int_70)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_71 = PyInt_FromLong(71); if (unlikely(!__pyx_int_71)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_72 = PyInt_FromLong(72); if (unlikely(!__pyx_int_72)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_73 = PyInt_FromLong(73); if (unlikely(!__pyx_int_73)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_74 = PyInt_FromLong(74); if (unlikely(!__pyx_int_74)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_75 = PyInt_FromLong(75); if (unlikely(!__pyx_int_75)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_76 = PyInt_FromLong(76); if (unlikely(!__pyx_int_76)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_77 = PyInt_FromLong(77); if (unlikely(!__pyx_int_77)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_78 = PyInt_FromLong(78); if (unlikely(!__pyx_int_78)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_79 = PyInt_FromLong(79); if (unlikely(!__pyx_int_79)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_80 = PyInt_FromLong(80); if (unlikely(!__pyx_int_80)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_81 = PyInt_FromLong(81); if (unlikely(!__pyx_int_81)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_82 = PyInt_FromLong(82); if (unlikely(!__pyx_int_82)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_83 = PyInt_FromLong(83); if (unlikely(!__pyx_int_83)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_84 = PyInt_FromLong(84); if (unlikely(!__pyx_int_84)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_85 = PyInt_FromLong(85); if (unlikely(!__pyx_int_85)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_86 = PyInt_FromLong(86); if (unlikely(!__pyx_int_86)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_87 = PyInt_FromLong(87); if (unlikely(!__pyx_int_87)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_88 = PyInt_FromLong(88); if (unlikely(!__pyx_int_88)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_89 = PyInt_FromLong(89); if (unlikely(!__pyx_int_89)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_90 = PyInt_FromLong(90); if (unlikely(!__pyx_int_90)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_91 = PyInt_FromLong(91); if (unlikely(!__pyx_int_91)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_92 = PyInt_FromLong(92); if (unlikely(!__pyx_int_92)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_93 = PyInt_FromLong(93); if (unlikely(!__pyx_int_93)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_94 = PyInt_FromLong(94); if (unlikely(!__pyx_int_94)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_95 = PyInt_FromLong(95); if (unlikely(!__pyx_int_95)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_96 = PyInt_FromLong(96); if (unlikely(!__pyx_int_96)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_97 = PyInt_FromLong(97); if (unlikely(!__pyx_int_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_98 = PyInt_FromLong(98); if (unlikely(!__pyx_int_98)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_99 = PyInt_FromLong(99); if (unlikely(!__pyx_int_99)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_100 = PyInt_FromLong(100); if (unlikely(!__pyx_int_100)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_101 = PyInt_FromLong(101); if (unlikely(!__pyx_int_101)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_102 = PyInt_FromLong(102); if (unlikely(!__pyx_int_102)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_103 = PyInt_FromLong(103); if (unlikely(!__pyx_int_103)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_104 = PyInt_FromLong(104); if (unlikely(!__pyx_int_104)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_105 = PyInt_FromLong(105); if (unlikely(!__pyx_int_105)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_106 = PyInt_FromLong(106); if (unlikely(!__pyx_int_106)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_107 = PyInt_FromLong(107); if (unlikely(!__pyx_int_107)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_108 = PyInt_FromLong(108); if (unlikely(!__pyx_int_108)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_109 = PyInt_FromLong(109); if (unlikely(!__pyx_int_109)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_110 = PyInt_FromLong(110); if (unlikely(!__pyx_int_110)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_111 = PyInt_FromLong(111); if (unlikely(!__pyx_int_111)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_112 = PyInt_FromLong(112); if (unlikely(!__pyx_int_112)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_113 = PyInt_FromLong(113); if (unlikely(!__pyx_int_113)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_114 = PyInt_FromLong(114); if (unlikely(!__pyx_int_114)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_115 = PyInt_FromLong(115); if (unlikely(!__pyx_int_115)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_116 = PyInt_FromLong(116); if (unlikely(!__pyx_int_116)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_117 = PyInt_FromLong(117); if (unlikely(!__pyx_int_117)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_118 = PyInt_FromLong(118); if (unlikely(!__pyx_int_118)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_119 = PyInt_FromLong(119); if (unlikely(!__pyx_int_119)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_120 = PyInt_FromLong(120); if (unlikely(!__pyx_int_120)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_121 = PyInt_FromLong(121); if (unlikely(!__pyx_int_121)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_122 = PyInt_FromLong(122); if (unlikely(!__pyx_int_122)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_123 = PyInt_FromLong(123); if (unlikely(!__pyx_int_123)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_124 = PyInt_FromLong(124); if (unlikely(!__pyx_int_124)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_125 = PyInt_FromLong(125); if (unlikely(!__pyx_int_125)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_126 = PyInt_FromLong(126); if (unlikely(!__pyx_int_126)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_127 = PyInt_FromLong(127); if (unlikely(!__pyx_int_127)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_128 = PyInt_FromLong(128); if (unlikely(!__pyx_int_128)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_255 = PyInt_FromLong(255); if (unlikely(!__pyx_int_255)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_640 = PyInt_FromLong(640); if (unlikely(!__pyx_int_640)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_4884 = PyInt_FromLong(4884); if (unlikely(!__pyx_int_4884)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_6144 = PyInt_FromLong(6144); if (unlikely(!__pyx_int_6144)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_65536 = PyInt_FromLong(65536L); if (unlikely(!__pyx_int_65536)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_4294967296 = PyInt_FromString((char *)"4294967296", 0, 0); if (unlikely(!__pyx_int_4294967296)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_1 = PyInt_FromLong(-1); if (unlikely(!__pyx_int_neg_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_5pyart_2io_11_sigmetfile_SigmetFile = &__pyx_vtable_5pyart_2io_11_sigmetfile_SigmetFile;
  __pyx_vtable_5pyart_2io_11_sigmetfile_SigmetFile._get_ray = (int (*)(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *, int, PyArrayObject *))__pyx_f_5pyart_2io_11_sigmetfile_10SigmetFile__get_ray;
  __pyx_vtable_5pyart_2io_11_sigmetfile_SigmetFile._incr_rbuf_pos = (int (*)(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *, struct __pyx_opt_args_5pyart_2io_11_sigmetfile_10SigmetFile__incr_rbuf_pos *__pyx_optional_args))__pyx_f_5pyart_2io_11_sigmetfile_10SigmetFile__incr_rbuf_pos;
  __pyx_vtable_5pyart_2io_11_sigmetfile_SigmetFile._load_record = (int (*)(struct __pyx_obj_5pyart_2io_11_sigmetfile_SigmetFile *))__pyx_f_5pyart_2io_11_sigmetfile_10SigmetFile__load_record;
  if (PyType_Ready(&__pyx_type_5pyart_2io_11_sigmetfile_SigmetFile) < 0) __PYX_ERR(0, 16, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_5pyart_2io_11_sigmetfile_SigmetFile.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_5pyart_2io_11_sigmetfile_SigmetFile.tp_dictoffset && __pyx_type_5pyart_2io_11_sigmetfile_SigmetFile.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_5pyart_2io_11_sigmetfile_SigmetFile.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  #if CYTHON_COMPILING_IN_CPYTHON
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)&__pyx_type_5pyart_2io_11_sigmetfile_SigmetFile, "__init__"); if (unlikely(!wrapper)) __PYX_ERR(0, 16, __pyx_L1_error)
    if (Py_TYPE(wrapper) == &PyWrapperDescr_Type) {
      __pyx_wrapperbase_5pyart_2io_11_sigmetfile_10SigmetFile___init__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_5pyart_2io_11_sigmetfile_10SigmetFile___init__.doc = __pyx_doc_5pyart_2io_11_sigmetfile_10SigmetFile___init__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_5pyart_2io_11_sigmetfile_10SigmetFile___init__;
    }
  }
  #endif
  if (__Pyx_SetVtable(__pyx_type_5pyart_2io_11_sigmetfile_SigmetFile.tp_dict, __pyx_vtabptr_5pyart_2io_11_sigmetfile_SigmetFile) < 0) __PYX_ERR(0, 16, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_SigmetFile, (PyObject *)&__pyx_type_5pyart_2io_11_sigmetfile_SigmetFile) < 0) __PYX_ERR(0, 16, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5pyart_2io_11_sigmetfile_SigmetFile) < 0) __PYX_ERR(0, 16, __pyx_L1_error)
  __pyx_ptype_5pyart_2io_11_sigmetfile_SigmetFile = &__pyx_type_5pyart_2io_11_sigmetfile_SigmetFile;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("numpy"); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_5numpy_dtype = __Pyx_ImportType(__pyx_t_1, "numpy", "dtype", sizeof(PyArray_Descr), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_dtype) __PYX_ERR(2, 200, __pyx_L1_error)
  __pyx_ptype_5numpy_flatiter = __Pyx_ImportType(__pyx_t_1, "numpy", "flatiter", sizeof(PyArrayIterObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_flatiter) __PYX_ERR(2, 223, __pyx_L1_error)
  __pyx_ptype_5numpy_broadcast = __Pyx_ImportType(__pyx_t_1, "numpy", "broadcast", sizeof(PyArrayMultiIterObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_broadcast) __PYX_ERR(2, 227, __pyx_L1_error)
  __pyx_ptype_5numpy_ndarray = __Pyx_ImportType(__pyx_t_1, "numpy", "ndarray", sizeof(PyArrayObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_ndarray) __PYX_ERR(2, 239, __pyx_L1_error)
  __pyx_ptype_5numpy_generic = __Pyx_ImportType(__pyx_t_1, "numpy", "generic", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_generic) __PYX_ERR(2, 771, __pyx_L1_error)
  __pyx_ptype_5numpy_number = __Pyx_ImportType(__pyx_t_1, "numpy", "number", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_number) __PYX_ERR(2, 773, __pyx_L1_error)
  __pyx_ptype_5numpy_integer = __Pyx_ImportType(__pyx_t_1, "numpy", "integer", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_integer) __PYX_ERR(2, 775, __pyx_L1_error)
  __pyx_ptype_5numpy_signedinteger = __Pyx_ImportType(__pyx_t_1, "numpy", "signedinteger", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_signedinteger) __PYX_ERR(2, 777, __pyx_L1_error)
  __pyx_ptype_5numpy_unsignedinteger = __Pyx_ImportType(__pyx_t_1, "numpy", "unsignedinteger", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_unsignedinteger) __PYX_ERR(2, 779, __pyx_L1_error)
  __pyx_ptype_5numpy_inexact = __Pyx_ImportType(__pyx_t_1, "numpy", "inexact", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_inexact) __PYX_ERR(2, 781, __pyx_L1_error)
  __pyx_ptype_5numpy_floating = __Pyx_ImportType(__pyx_t_1, "numpy", "floating", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_floating) __PYX_ERR(2, 783, __pyx_L1_error)
  __pyx_ptype_5numpy_complexfloating = __Pyx_ImportType(__pyx_t_1, "numpy", "complexfloating", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_complexfloating) __PYX_ERR(2, 785, __pyx_L1_error)
  __pyx_ptype_5numpy_flexible = __Pyx_ImportType(__pyx_t_1, "numpy", "flexible", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_flexible) __PYX_ERR(2, 787, __pyx_L1_error)
  __pyx_ptype_5numpy_character = __Pyx_ImportType(__pyx_t_1, "numpy", "character", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_character) __PYX_ERR(2, 789, __pyx_L1_error)
  __pyx_ptype_5numpy_ufunc = __Pyx_ImportType(__pyx_t_1, "numpy", "ufunc", sizeof(PyUFuncObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_ufunc) __PYX_ERR(2, 827, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC init_sigmetfile(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC init_sigmetfile(void)
#else
__Pyx_PyMODINIT_FUNC PyInit__sigmetfile(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit__sigmetfile(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec__sigmetfile(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  PyObject *__pyx_t_24 = NULL;
  PyObject *__pyx_t_25 = NULL;
  PyObject *__pyx_t_26 = NULL;
  PyObject *__pyx_t_27 = NULL;
  PyObject *__pyx_t_28 = NULL;
  PyObject *__pyx_t_29 = NULL;
  PyObject *__pyx_t_30 = NULL;
  PyObject *__pyx_t_31 = NULL;
  PyObject *__pyx_t_32 = NULL;
  PyObject *__pyx_t_33 = NULL;
  PyObject *__pyx_t_34 = NULL;
  PyObject *__pyx_t_35 = NULL;
  PyObject *__pyx_t_36 = NULL;
  PyObject *__pyx_t_37 = NULL;
  PyObject *__pyx_t_38 = NULL;
  PyObject *__pyx_t_39 = NULL;
  PyObject *__pyx_t_40 = NULL;
  PyObject *__pyx_t_41 = NULL;
  PyObject *__pyx_t_42 = NULL;
  PyObject *__pyx_t_43 = NULL;
  PyObject *__pyx_t_44 = NULL;
  PyObject *__pyx_t_45 = NULL;
  PyObject *__pyx_t_46 = NULL;
  PyObject *__pyx_t_47 = NULL;
  PyObject *__pyx_t_48 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module '_sigmetfile' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit__sigmetfile(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("_sigmetfile", __pyx_methods, __pyx_k_A_class_and_supporting_function, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_pyart__io___sigmetfile) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "pyart.io._sigmetfile")) {
      if (unlikely(PyDict_SetItemString(modules, "pyart.io._sigmetfile", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely(__Pyx_modinit_type_init_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_type_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "pyart/io/_sigmetfile.pyx":5
 * 
 * """
 * import struct             # <<<<<<<<<<<<<<
 * import datetime
 * import warnings
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_struct, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_struct, __pyx_t_1) < 0) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":6
 * """
 * import struct
 * import datetime             # <<<<<<<<<<<<<<
 * import warnings
 * 
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_datetime, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_datetime, __pyx_t_1) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":7
 * import struct
 * import datetime
 * import warnings             # <<<<<<<<<<<<<<
 * 
 * import numpy as np
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_warnings, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_warnings, __pyx_t_1) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":9
 * import warnings
 * 
 * import numpy as np             # <<<<<<<<<<<<<<
 * cimport numpy as np
 * cimport cython
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_np, __pyx_t_1) < 0) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":13
 * cimport cython
 * 
 * RECORD_SIZE = 6144      # Raw product file blocked into 6144 byte records             # <<<<<<<<<<<<<<
 * 
 * 
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_RECORD_SIZE, __pyx_int_6144) < 0) __PYX_ERR(0, 13, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":401
 * 
 * 
 * def _data_types_from_mask(word0, word1, word2, word3):             # <<<<<<<<<<<<<<
 *     """
 *     Return a list of the data types from the words in the data_type mask.
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5pyart_2io_11_sigmetfile_1_data_types_from_mask, NULL, __pyx_n_s_pyart_io__sigmetfile); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 401, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_data_types_from_mask, __pyx_t_1) < 0) __PYX_ERR(0, 401, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":412
 * 
 * 
 * def _is_bit_set(number, bit):             # <<<<<<<<<<<<<<
 *     """ Return True if bit is set in number. """
 *     return number >> bit & 1 == 1
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5pyart_2io_11_sigmetfile_3_is_bit_set, NULL, __pyx_n_s_pyart_io__sigmetfile); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 412, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_is_bit_set, __pyx_t_1) < 0) __PYX_ERR(0, 412, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":417
 * 
 * 
 * def _parse_ray_headers(ray_headers):             # <<<<<<<<<<<<<<
 *     """
 *     Parse the metadata from Sigmet ray headers.
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5pyart_2io_11_sigmetfile_5_parse_ray_headers, NULL, __pyx_n_s_pyart_io__sigmetfile); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 417, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_parse_ray_headers, __pyx_t_1) < 0) __PYX_ERR(0, 417, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":461
 * # Data type constants, table 13, section 4.8
 * SIGMET_DATA_TYPES = {
 *     0: 'XHDR',             # <<<<<<<<<<<<<<
 *     1: 'DBT',
 *     2: 'DBZ',
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(128); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 461, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_0, __pyx_n_u_XHDR) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_1, __pyx_n_u_DBT) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_2, __pyx_n_u_DBZ) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_3, __pyx_n_u_VEL) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_4, __pyx_n_u_WIDTH) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_5, __pyx_n_u_ZDR) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_6, __pyx_n_u_UNKNOWN_6) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_7, __pyx_n_u_DBZC) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_8, __pyx_n_u_DBT2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_9, __pyx_n_u_DBZ2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_10, __pyx_n_u_VEL2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_11, __pyx_n_u_WIDTH2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_12, __pyx_n_u_ZDR2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_13, __pyx_n_u_RAINRATE2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_14, __pyx_n_u_KDP) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_15, __pyx_n_u_KDP2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_16, __pyx_n_u_PHIDP) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_17, __pyx_n_u_VELC) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_18, __pyx_n_u_SQI) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_19, __pyx_n_u_RHOHV) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_20, __pyx_n_u_RHOHV2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_21, __pyx_n_u_DBZC2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_22, __pyx_n_u_VELC2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_23, __pyx_n_u_SQI2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_24, __pyx_n_u_PHIDP2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_25, __pyx_n_u_LDRH) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_26, __pyx_n_u_LDRH2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_27, __pyx_n_u_LDRV) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_28, __pyx_n_u_LDRV2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_29, __pyx_n_u_UNKNOWN_29) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_30, __pyx_n_u_UNKNOWN_30) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_31, __pyx_n_u_UNKNOWN_31) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_32, __pyx_n_u_HEIGHT) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_33, __pyx_n_u_VIL2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_34, __pyx_n_u_RAW) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_35, __pyx_n_u_SHEAR) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_36, __pyx_n_u_DIVERGE2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_37, __pyx_n_u_FLIQUID2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_38, __pyx_n_u_USER) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_39, __pyx_n_u_OTHER) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_40, __pyx_n_u_DEFORM2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_41, __pyx_n_u_VVEL2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_42, __pyx_n_u_HVEL2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_43, __pyx_n_u_HDIR2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_44, __pyx_n_u_AXDIL2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_45, __pyx_n_u_TIME2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_46, __pyx_n_u_RHOH) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_47, __pyx_n_u_RHOH2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_48, __pyx_n_u_RHOV) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_49, __pyx_n_u_RHOV2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_50, __pyx_n_u_PHIH) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_51, __pyx_n_u_PHIH2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_52, __pyx_n_u_PHIV) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_53, __pyx_n_u_PHIV2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_54, __pyx_n_u_USER2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_55, __pyx_n_u_HCLASS) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_56, __pyx_n_u_HCLASS2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_57, __pyx_n_u_ZDRC) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_58, __pyx_n_u_ZDRC2) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_59, __pyx_n_u_TEMPERATURE16) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_60, __pyx_n_u_VIR16) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_61, __pyx_n_u_DBTV8) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_62, __pyx_n_u_DBTV16) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_63, __pyx_n_u_DBZV8) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_64, __pyx_n_u_DBZV16) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_65, __pyx_n_u_SNR8) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_66, __pyx_n_u_SNR16) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_67, __pyx_n_u_ALBEDO8) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_68, __pyx_n_u_ALBEDO16) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_69, __pyx_n_u_VILD16) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_70, __pyx_n_u_TURB16) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_71, __pyx_n_u_DBTE8) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_72, __pyx_n_u_DBTE16) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_73, __pyx_n_u_DBZE8) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_74, __pyx_n_u_DBZE16) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_75, __pyx_n_u_PMI8) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_76, __pyx_n_u_PMI16) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_77, __pyx_n_u_LOG8) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_78, __pyx_n_u_LOG16) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_79, __pyx_n_u_CSP8) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_80, __pyx_n_u_CSP16) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_81, __pyx_n_u_UNKNOWN_81) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_82, __pyx_n_u_UNKNOWN_82) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_83, __pyx_n_u_UNKNOWN_83) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_84, __pyx_n_u_UNKNOWN_84) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_85, __pyx_n_u_UNKNOWN_85) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_86, __pyx_n_u_UNKNOWN_86) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_87, __pyx_n_u_UNKNOWN_87) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_88, __pyx_n_u_UNKNOWN_88) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_89, __pyx_n_u_UNKNOWN_89) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_90, __pyx_n_u_UNKNOWN_90) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_91, __pyx_n_u_UNKNOWN_91) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_92, __pyx_n_u_UNKNOWN_92) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_93, __pyx_n_u_UNKNOWN_93) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_94, __pyx_n_u_UNKNOWN_94) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_95, __pyx_n_u_UNKNOWN_95) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_96, __pyx_n_u_UNKNOWN_96) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_97, __pyx_n_u_UNKNOWN_97) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_98, __pyx_n_u_UNKNOWN_98) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_99, __pyx_n_u_UNKNOWN_99) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_100, __pyx_n_u_UNKNOWN_100) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_101, __pyx_n_u_UNKNOWN_101) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_102, __pyx_n_u_UNKNOWN_102) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_103, __pyx_n_u_UNKNOWN_103) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_104, __pyx_n_u_UNKNOWN_104) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_105, __pyx_n_u_UNKNOWN_105) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_106, __pyx_n_u_UNKNOWN_106) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_107, __pyx_n_u_UNKNOWN_107) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_108, __pyx_n_u_UNKNOWN_108) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_109, __pyx_n_u_UNKNOWN_109) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_110, __pyx_n_u_UNKNOWN_110) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_111, __pyx_n_u_UNKNOWN_110) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_112, __pyx_n_u_UNKNOWN_112) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_113, __pyx_n_u_UNKNOWN_113) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_114, __pyx_n_u_UNKNOWN_114) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_115, __pyx_n_u_UNKNOWN_115) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_116, __pyx_n_u_UNKNOWN_116) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_117, __pyx_n_u_UNKNOWN_117) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_118, __pyx_n_u_UNKNOWN_118) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_119, __pyx_n_u_UNKNOWN_119) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_120, __pyx_n_u_UNKNOWN_120) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_121, __pyx_n_u_UNKNOWN_121) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_122, __pyx_n_u_UNKNOWN_122) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_123, __pyx_n_u_UNKNOWN_123) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_124, __pyx_n_u_UNKNOWN_124) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_125, __pyx_n_u_UNKNOWN_125) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_126, __pyx_n_u_UNKNOWN_126) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_127, __pyx_n_u_UNKNOWN_127) < 0) __PYX_ERR(0, 461, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SIGMET_DATA_TYPES, __pyx_t_1) < 0) __PYX_ERR(0, 460, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":596
 * # file. Rewriting the convertions/masking in Cython does not seem to improved
 * # performance likely since most of the routines are already vectorized.
 * def convert_sigmet_data(data_type, data, nbins):             # <<<<<<<<<<<<<<
 *     """ Convert sigmet data. """
 *     out = np.empty_like(data, dtype='float32')
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5pyart_2io_11_sigmetfile_7convert_sigmet_data, NULL, __pyx_n_s_pyart_io__sigmetfile); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 596, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_convert_sigmet_data, __pyx_t_1) < 0) __PYX_ERR(0, 596, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":796
 * 
 * 
 * def bin2_to_angle(bin2):             # <<<<<<<<<<<<<<
 *     """ Return an angle from Sigmet bin2 encoded value (or array). """
 *     return 360. * bin2 / 65536
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5pyart_2io_11_sigmetfile_9bin2_to_angle, NULL, __pyx_n_s_pyart_io__sigmetfile); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 796, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_bin2_to_angle, __pyx_t_1) < 0) __PYX_ERR(0, 796, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":801
 * 
 * 
 * def bin4_to_angle(bin4):             # <<<<<<<<<<<<<<
 *     """ Return an angle from Sigmet bin4 encoded value (or array). """
 *     return 360. * bin4 / 4294967296
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5pyart_2io_11_sigmetfile_11bin4_to_angle, NULL, __pyx_n_s_pyart_io__sigmetfile); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 801, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_bin4_to_angle, __pyx_t_1) < 0) __PYX_ERR(0, 801, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":811
 * 
 * 
 * def _unpack_structure(string, structure):             # <<<<<<<<<<<<<<
 *     """ Unpack a structure """
 *     fmt = ''.join([i[1] for i in structure])
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5pyart_2io_11_sigmetfile_13_unpack_structure, NULL, __pyx_n_s_pyart_io__sigmetfile); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 811, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_unpack_structure, __pyx_t_1) < 0) __PYX_ERR(0, 811, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":818
 * 
 * 
 * def _unpack_key(dic, key, structure):             # <<<<<<<<<<<<<<
 *     """ Unpack a key. """
 *     dic[key] = _unpack_structure(dic[key], structure)
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5pyart_2io_11_sigmetfile_15_unpack_key, NULL, __pyx_n_s_pyart_io__sigmetfile); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 818, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_unpack_key, __pyx_t_1) < 0) __PYX_ERR(0, 818, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":823
 * 
 * 
 * def _unpack_ingest_data_headers(record, ndata_types):             # <<<<<<<<<<<<<<
 *     """
 *     Unpack one or more ingest_data_header from a record.
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5pyart_2io_11_sigmetfile_17_unpack_ingest_data_headers, NULL, __pyx_n_s_pyart_io__sigmetfile); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 823, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_unpack_ingest_data_headers, __pyx_t_1) < 0) __PYX_ERR(0, 823, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":837
 * 
 * 
 * def _unpack_ingest_data_header(record, number):             # <<<<<<<<<<<<<<
 *     """
 *     Unpack a single ingest_data_header from record.  Return None on error.
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5pyart_2io_11_sigmetfile_19_unpack_ingest_data_header, NULL, __pyx_n_s_pyart_io__sigmetfile); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 837, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_unpack_ingest_data_header, __pyx_t_1) < 0) __PYX_ERR(0, 837, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":851
 * 
 * 
 * def _unpack_raw_prod_bhdr(record):             # <<<<<<<<<<<<<<
 *     """ Return a dict with the unpacked raw_prod_bhdr from a record. """
 *     return _unpack_structure(record[:12], RAW_PROD_BHDR)
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5pyart_2io_11_sigmetfile_21_unpack_raw_prod_bhdr, NULL, __pyx_n_s_pyart_io__sigmetfile); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 851, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_unpack_raw_prod_bhdr, __pyx_t_1) < 0) __PYX_ERR(0, 851, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":856
 * 
 * 
 * def _unpack_product_hdr(record):             # <<<<<<<<<<<<<<
 *     """
 *     Return a dict with the unpacked product_hdr from the first record.
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5pyart_2io_11_sigmetfile_23_unpack_product_hdr, NULL, __pyx_n_s_pyart_io__sigmetfile); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 856, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_unpack_product_hdr, __pyx_t_1) < 0) __PYX_ERR(0, 856, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":885
 * 
 * 
 * def _unpack_ingest_header(record):             # <<<<<<<<<<<<<<
 *     """
 *     Return a dict with the unpacked ingest_header from the second record.
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5pyart_2io_11_sigmetfile_25_unpack_ingest_header, NULL, __pyx_n_s_pyart_io__sigmetfile); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 885, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_unpack_ingest_header, __pyx_t_1) < 0) __PYX_ERR(0, 885, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":939
 * 
 * # scalar defitions, section 4.1, table 7, and corresponding
 * SINT1 = 'b'             # <<<<<<<<<<<<<<
 * UINT1 = 'B'
 * SINT2 = 'h'
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SINT1, __pyx_n_u_b) < 0) __PYX_ERR(0, 939, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":940
 * # scalar defitions, section 4.1, table 7, and corresponding
 * SINT1 = 'b'
 * UINT1 = 'B'             # <<<<<<<<<<<<<<
 * SINT2 = 'h'
 * UINT2 = 'H'
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_UINT1, __pyx_n_u_B) < 0) __PYX_ERR(0, 940, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":941
 * SINT1 = 'b'
 * UINT1 = 'B'
 * SINT2 = 'h'             # <<<<<<<<<<<<<<
 * UINT2 = 'H'
 * SINT4 = 'i'
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SINT2, __pyx_n_u_h) < 0) __PYX_ERR(0, 941, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":942
 * UINT1 = 'B'
 * SINT2 = 'h'
 * UINT2 = 'H'             # <<<<<<<<<<<<<<
 * SINT4 = 'i'
 * UINT4 = 'I'
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_UINT2, __pyx_n_u_H) < 0) __PYX_ERR(0, 942, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":943
 * SINT2 = 'h'
 * UINT2 = 'H'
 * SINT4 = 'i'             # <<<<<<<<<<<<<<
 * UINT4 = 'I'
 * FLT4 = 'f'
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SINT4, __pyx_n_u_i) < 0) __PYX_ERR(0, 943, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":944
 * UINT2 = 'H'
 * SINT4 = 'i'
 * UINT4 = 'I'             # <<<<<<<<<<<<<<
 * FLT4 = 'f'
 * FLT8 = 'd'
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_UINT4, __pyx_n_u_I) < 0) __PYX_ERR(0, 944, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":945
 * SINT4 = 'i'
 * UINT4 = 'I'
 * FLT4 = 'f'             # <<<<<<<<<<<<<<
 * FLT8 = 'd'
 * BIN1 = 'B'
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_FLT4, __pyx_n_u_f) < 0) __PYX_ERR(0, 945, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":946
 * UINT4 = 'I'
 * FLT4 = 'f'
 * FLT8 = 'd'             # <<<<<<<<<<<<<<
 * BIN1 = 'B'
 * BIN2 = 'H'      # these values need to be decoded with _bin2_to_angle
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_FLT8, __pyx_n_u_d) < 0) __PYX_ERR(0, 946, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":947
 * FLT4 = 'f'
 * FLT8 = 'd'
 * BIN1 = 'B'             # <<<<<<<<<<<<<<
 * BIN2 = 'H'      # these values need to be decoded with _bin2_to_angle
 * BIN4 = 'I'      # these values need to be decoded with _bin4_to_angle
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BIN1, __pyx_n_u_B) < 0) __PYX_ERR(0, 947, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":948
 * FLT8 = 'd'
 * BIN1 = 'B'
 * BIN2 = 'H'      # these values need to be decoded with _bin2_to_angle             # <<<<<<<<<<<<<<
 * BIN4 = 'I'      # these values need to be decoded with _bin4_to_angle
 * MESSAGE = 'I'
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BIN2, __pyx_n_u_H) < 0) __PYX_ERR(0, 948, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":949
 * BIN1 = 'B'
 * BIN2 = 'H'      # these values need to be decoded with _bin2_to_angle
 * BIN4 = 'I'      # these values need to be decoded with _bin4_to_angle             # <<<<<<<<<<<<<<
 * MESSAGE = 'I'
 * UINT16_T = 'H'
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BIN4, __pyx_n_u_I) < 0) __PYX_ERR(0, 949, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":950
 * BIN2 = 'H'      # these values need to be decoded with _bin2_to_angle
 * BIN4 = 'I'      # these values need to be decoded with _bin4_to_angle
 * MESSAGE = 'I'             # <<<<<<<<<<<<<<
 * UINT16_T = 'H'
 * 
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_MESSAGE, __pyx_n_u_I) < 0) __PYX_ERR(0, 950, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":951
 * BIN4 = 'I'      # these values need to be decoded with _bin4_to_angle
 * MESSAGE = 'I'
 * UINT16_T = 'H'             # <<<<<<<<<<<<<<
 * 
 * # structures are taken from Vaisala PROGRAMMER'S MANUAL IRIS
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_UINT16_T, __pyx_n_u_H) < 0) __PYX_ERR(0, 951, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":961
 * # 640 bytes: product_hdr (section 4.2.25, page 47)
 * PRODUCT_HDR = (
 *     ('structure_header', '12s'),        # 12 bytes             # <<<<<<<<<<<<<<
 *     ('product_configuration', '320s'),  # 320 bytes
 *     ('product_end', '308s'),            # 308 bytes
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PRODUCT_HDR, __pyx_tuple__50) < 0) __PYX_ERR(0, 960, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":968
 * # 12 bytes : structure_header (section 4.2.47)
 * STRUCTURE_HEADER = (
 *     ('structure_identifier', SINT2),             # <<<<<<<<<<<<<<
 *     ('format_version', SINT2),
 *     ('bytes_in_structure', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 968, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 968, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_structure_identifier);
  __Pyx_GIVEREF(__pyx_n_u_structure_identifier);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_structure_identifier);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":969
 * STRUCTURE_HEADER = (
 *     ('structure_identifier', SINT2),
 *     ('format_version', SINT2),             # <<<<<<<<<<<<<<
 *     ('bytes_in_structure', SINT4),
 *     ('reserved', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 969, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 969, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_n_u_format_version);
  __Pyx_GIVEREF(__pyx_n_u_format_version);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_n_u_format_version);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":970
 *     ('structure_identifier', SINT2),
 *     ('format_version', SINT2),
 *     ('bytes_in_structure', SINT4),             # <<<<<<<<<<<<<<
 *     ('reserved', SINT2),
 *     ('flag', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 970, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 970, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_n_u_bytes_in_structure);
  __Pyx_GIVEREF(__pyx_n_u_bytes_in_structure);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_n_u_bytes_in_structure);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":971
 *     ('format_version', SINT2),
 *     ('bytes_in_structure', SINT4),
 *     ('reserved', SINT2),             # <<<<<<<<<<<<<<
 *     ('flag', SINT2),
 * )
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 971, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 971, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_n_u_reserved);
  __Pyx_GIVEREF(__pyx_n_u_reserved);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_n_u_reserved);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":972
 *     ('bytes_in_structure', SINT4),
 *     ('reserved', SINT2),
 *     ('flag', SINT2),             # <<<<<<<<<<<<<<
 * )
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 972, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 972, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_n_u_flag);
  __Pyx_GIVEREF(__pyx_n_u_flag);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_n_u_flag);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":968
 * # 12 bytes : structure_header (section 4.2.47)
 * STRUCTURE_HEADER = (
 *     ('structure_identifier', SINT2),             # <<<<<<<<<<<<<<
 *     ('format_version', SINT2),
 *     ('bytes_in_structure', SINT4),
 */
  __pyx_t_1 = PyTuple_New(5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 968, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_t_6);
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_STRUCTURE_HEADER, __pyx_t_1) < 0) __PYX_ERR(0, 967, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":978
 * PRODUCT_CONFIGURATION = (
 *     ('structure_header', '12s'),    # 12 bytes: structure_header
 *     ('product_type_code', UINT2),             # <<<<<<<<<<<<<<
 *     ('scheduling_code', UINT2),
 *     ('seconds_between_runs', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 978, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 978, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_n_u_product_type_code);
  __Pyx_GIVEREF(__pyx_n_u_product_type_code);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_n_u_product_type_code);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":979
 *     ('structure_header', '12s'),    # 12 bytes: structure_header
 *     ('product_type_code', UINT2),
 *     ('scheduling_code', UINT2),             # <<<<<<<<<<<<<<
 *     ('seconds_between_runs', SINT4),
 *     ('generation_time', '12s'),     # 12 bytes: ymds_time
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 979, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 979, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_n_u_scheduling_code);
  __Pyx_GIVEREF(__pyx_n_u_scheduling_code);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_n_u_scheduling_code);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":980
 *     ('product_type_code', UINT2),
 *     ('scheduling_code', UINT2),
 *     ('seconds_between_runs', SINT4),             # <<<<<<<<<<<<<<
 *     ('generation_time', '12s'),     # 12 bytes: ymds_time
 *     ('sweep_ingest_time', '12s'),   # 12 bytes: ymds_time
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 980, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 980, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_n_u_seconds_between_runs);
  __Pyx_GIVEREF(__pyx_n_u_seconds_between_runs);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_n_u_seconds_between_runs);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":987
 *     ('product_name', '12s'),
 *     ('task_name', '12s'),
 *     ('flag', UINT2),             # <<<<<<<<<<<<<<
 *     ('x_scale', SINT4),
 *     ('y_scale', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 987, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 987, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_n_u_flag);
  __Pyx_GIVEREF(__pyx_n_u_flag);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_n_u_flag);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":988
 *     ('task_name', '12s'),
 *     ('flag', UINT2),
 *     ('x_scale', SINT4),             # <<<<<<<<<<<<<<
 *     ('y_scale', SINT4),
 *     ('z_scale', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 988, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 988, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_x_scale);
  __Pyx_GIVEREF(__pyx_n_u_x_scale);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_x_scale);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":989
 *     ('flag', UINT2),
 *     ('x_scale', SINT4),
 *     ('y_scale', SINT4),             # <<<<<<<<<<<<<<
 *     ('z_scale', SINT4),
 *     ('x_size', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 989, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 989, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_INCREF(__pyx_n_u_y_scale);
  __Pyx_GIVEREF(__pyx_n_u_y_scale);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_n_u_y_scale);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":990
 *     ('x_scale', SINT4),
 *     ('y_scale', SINT4),
 *     ('z_scale', SINT4),             # <<<<<<<<<<<<<<
 *     ('x_size', SINT4),
 *     ('y_size', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 990, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 990, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_INCREF(__pyx_n_u_z_scale);
  __Pyx_GIVEREF(__pyx_n_u_z_scale);
  PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_n_u_z_scale);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":991
 *     ('y_scale', SINT4),
 *     ('z_scale', SINT4),
 *     ('x_size', SINT4),             # <<<<<<<<<<<<<<
 *     ('y_size', SINT4),
 *     ('z_size', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 991, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 991, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_INCREF(__pyx_n_u_x_size);
  __Pyx_GIVEREF(__pyx_n_u_x_size);
  PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_n_u_x_size);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":992
 *     ('z_scale', SINT4),
 *     ('x_size', SINT4),
 *     ('y_size', SINT4),             # <<<<<<<<<<<<<<
 *     ('z_size', SINT4),
 *     ('x_location', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 992, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_10 = PyTuple_New(2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 992, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_INCREF(__pyx_n_u_y_size);
  __Pyx_GIVEREF(__pyx_n_u_y_size);
  PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_n_u_y_size);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":993
 *     ('x_size', SINT4),
 *     ('y_size', SINT4),
 *     ('z_size', SINT4),             # <<<<<<<<<<<<<<
 *     ('x_location', SINT4),
 *     ('y_location', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 993, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_11 = PyTuple_New(2); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 993, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_INCREF(__pyx_n_u_z_size);
  __Pyx_GIVEREF(__pyx_n_u_z_size);
  PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_n_u_z_size);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":994
 *     ('y_size', SINT4),
 *     ('z_size', SINT4),
 *     ('x_location', SINT4),             # <<<<<<<<<<<<<<
 *     ('y_location', SINT4),
 *     ('z_location', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 994, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_12 = PyTuple_New(2); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 994, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_INCREF(__pyx_n_u_x_location);
  __Pyx_GIVEREF(__pyx_n_u_x_location);
  PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_n_u_x_location);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_12, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":995
 *     ('z_size', SINT4),
 *     ('x_location', SINT4),
 *     ('y_location', SINT4),             # <<<<<<<<<<<<<<
 *     ('z_location', SINT4),
 *     ('maximum_range', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 995, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_13 = PyTuple_New(2); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 995, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_INCREF(__pyx_n_u_y_location);
  __Pyx_GIVEREF(__pyx_n_u_y_location);
  PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_n_u_y_location);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_13, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":996
 *     ('x_location', SINT4),
 *     ('y_location', SINT4),
 *     ('z_location', SINT4),             # <<<<<<<<<<<<<<
 *     ('maximum_range', SINT4),
 *     ('data_type', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 996, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_14 = PyTuple_New(2); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 996, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_INCREF(__pyx_n_u_z_location);
  __Pyx_GIVEREF(__pyx_n_u_z_location);
  PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_n_u_z_location);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_14, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":997
 *     ('y_location', SINT4),
 *     ('z_location', SINT4),
 *     ('maximum_range', SINT4),             # <<<<<<<<<<<<<<
 *     ('data_type', UINT2),
 *     ('projection_name', '12s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 997, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_15 = PyTuple_New(2); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 997, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __Pyx_INCREF(__pyx_n_u_maximum_range);
  __Pyx_GIVEREF(__pyx_n_u_maximum_range);
  PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_n_u_maximum_range);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_15, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":998
 *     ('z_location', SINT4),
 *     ('maximum_range', SINT4),
 *     ('data_type', UINT2),             # <<<<<<<<<<<<<<
 *     ('projection_name', '12s'),
 *     ('input_data_type', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 998, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_16 = PyTuple_New(2); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 998, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);
  __Pyx_INCREF(__pyx_n_u_data_type);
  __Pyx_GIVEREF(__pyx_n_u_data_type);
  PyTuple_SET_ITEM(__pyx_t_16, 0, __pyx_n_u_data_type);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_16, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1000
 *     ('data_type', UINT2),
 *     ('projection_name', '12s'),
 *     ('input_data_type', UINT2),             # <<<<<<<<<<<<<<
 *     ('projection_type', UINT1),
 *     ('spare_1', '1s'),              # 1 bytes
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1000, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_17 = PyTuple_New(2); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1000, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_17);
  __Pyx_INCREF(__pyx_n_u_input_data_type);
  __Pyx_GIVEREF(__pyx_n_u_input_data_type);
  PyTuple_SET_ITEM(__pyx_t_17, 0, __pyx_n_u_input_data_type);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_17, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1001
 *     ('projection_name', '12s'),
 *     ('input_data_type', UINT2),
 *     ('projection_type', UINT1),             # <<<<<<<<<<<<<<
 *     ('spare_1', '1s'),              # 1 bytes
 *     ('radial_smoother', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1001, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_18 = PyTuple_New(2); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 1001, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_18);
  __Pyx_INCREF(__pyx_n_u_projection_type);
  __Pyx_GIVEREF(__pyx_n_u_projection_type);
  PyTuple_SET_ITEM(__pyx_t_18, 0, __pyx_n_u_projection_type);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_18, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1003
 *     ('projection_type', UINT1),
 *     ('spare_1', '1s'),              # 1 bytes
 *     ('radial_smoother', SINT2),             # <<<<<<<<<<<<<<
 *     ('times_run', SINT2),
 *     ('zr_constant', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1003, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_19 = PyTuple_New(2); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 1003, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_19);
  __Pyx_INCREF(__pyx_n_u_radial_smoother);
  __Pyx_GIVEREF(__pyx_n_u_radial_smoother);
  PyTuple_SET_ITEM(__pyx_t_19, 0, __pyx_n_u_radial_smoother);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_19, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1004
 *     ('spare_1', '1s'),              # 1 bytes
 *     ('radial_smoother', SINT2),
 *     ('times_run', SINT2),             # <<<<<<<<<<<<<<
 *     ('zr_constant', SINT4),
 *     ('zr_exponent', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1004, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_20 = PyTuple_New(2); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 1004, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_20);
  __Pyx_INCREF(__pyx_n_u_times_run);
  __Pyx_GIVEREF(__pyx_n_u_times_run);
  PyTuple_SET_ITEM(__pyx_t_20, 0, __pyx_n_u_times_run);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_20, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1005
 *     ('radial_smoother', SINT2),
 *     ('times_run', SINT2),
 *     ('zr_constant', SINT4),             # <<<<<<<<<<<<<<
 *     ('zr_exponent', SINT4),
 *     ('x_smoother', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1005, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_21 = PyTuple_New(2); if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 1005, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_21);
  __Pyx_INCREF(__pyx_n_u_zr_constant);
  __Pyx_GIVEREF(__pyx_n_u_zr_constant);
  PyTuple_SET_ITEM(__pyx_t_21, 0, __pyx_n_u_zr_constant);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_21, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1006
 *     ('times_run', SINT2),
 *     ('zr_constant', SINT4),
 *     ('zr_exponent', SINT4),             # <<<<<<<<<<<<<<
 *     ('x_smoother', SINT2),
 *     ('y_smoother', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1006, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_22 = PyTuple_New(2); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 1006, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_22);
  __Pyx_INCREF(__pyx_n_u_zr_exponent);
  __Pyx_GIVEREF(__pyx_n_u_zr_exponent);
  PyTuple_SET_ITEM(__pyx_t_22, 0, __pyx_n_u_zr_exponent);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_22, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1007
 *     ('zr_constant', SINT4),
 *     ('zr_exponent', SINT4),
 *     ('x_smoother', SINT2),             # <<<<<<<<<<<<<<
 *     ('y_smoother', SINT2),
 *     ('product_specific_bytes', '80s'),  # 80 bytes:
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1007, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_23 = PyTuple_New(2); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 1007, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_23);
  __Pyx_INCREF(__pyx_n_u_x_smoother);
  __Pyx_GIVEREF(__pyx_n_u_x_smoother);
  PyTuple_SET_ITEM(__pyx_t_23, 0, __pyx_n_u_x_smoother);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_23, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1008
 *     ('zr_exponent', SINT4),
 *     ('x_smoother', SINT2),
 *     ('y_smoother', SINT2),             # <<<<<<<<<<<<<<
 *     ('product_specific_bytes', '80s'),  # 80 bytes:
 *     ('minor_task_suffix', '16s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1008, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_24 = PyTuple_New(2); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 1008, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_24);
  __Pyx_INCREF(__pyx_n_u_y_smoother);
  __Pyx_GIVEREF(__pyx_n_u_y_smoother);
  PyTuple_SET_ITEM(__pyx_t_24, 0, __pyx_n_u_y_smoother);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_24, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":977
 * # 320 bytes: product_configuration (section 4.2.23, page 43) 320 bytes
 * PRODUCT_CONFIGURATION = (
 *     ('structure_header', '12s'),    # 12 bytes: structure_header             # <<<<<<<<<<<<<<
 *     ('product_type_code', UINT2),
 *     ('scheduling_code', UINT2),
 */
  __pyx_t_1 = PyTuple_New(36); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 977, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_tuple__47);
  __Pyx_GIVEREF(__pyx_tuple__47);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_tuple__47);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_t_4);
  __Pyx_INCREF(__pyx_tuple__51);
  __Pyx_GIVEREF(__pyx_tuple__51);
  PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_tuple__51);
  __Pyx_INCREF(__pyx_tuple__52);
  __Pyx_GIVEREF(__pyx_tuple__52);
  PyTuple_SET_ITEM(__pyx_t_1, 5, __pyx_tuple__52);
  __Pyx_INCREF(__pyx_tuple__53);
  __Pyx_GIVEREF(__pyx_tuple__53);
  PyTuple_SET_ITEM(__pyx_t_1, 6, __pyx_tuple__53);
  __Pyx_INCREF(__pyx_tuple__54);
  __Pyx_GIVEREF(__pyx_tuple__54);
  PyTuple_SET_ITEM(__pyx_t_1, 7, __pyx_tuple__54);
  __Pyx_INCREF(__pyx_tuple__55);
  __Pyx_GIVEREF(__pyx_tuple__55);
  PyTuple_SET_ITEM(__pyx_t_1, 8, __pyx_tuple__55);
  __Pyx_INCREF(__pyx_tuple__56);
  __Pyx_GIVEREF(__pyx_tuple__56);
  PyTuple_SET_ITEM(__pyx_t_1, 9, __pyx_tuple__56);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_1, 10, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 11, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_1, 12, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_1, 13, __pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_9);
  PyTuple_SET_ITEM(__pyx_t_1, 14, __pyx_t_9);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_1, 15, __pyx_t_10);
  __Pyx_GIVEREF(__pyx_t_11);
  PyTuple_SET_ITEM(__pyx_t_1, 16, __pyx_t_11);
  __Pyx_GIVEREF(__pyx_t_12);
  PyTuple_SET_ITEM(__pyx_t_1, 17, __pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_13);
  PyTuple_SET_ITEM(__pyx_t_1, 18, __pyx_t_13);
  __Pyx_GIVEREF(__pyx_t_14);
  PyTuple_SET_ITEM(__pyx_t_1, 19, __pyx_t_14);
  __Pyx_GIVEREF(__pyx_t_15);
  PyTuple_SET_ITEM(__pyx_t_1, 20, __pyx_t_15);
  __Pyx_GIVEREF(__pyx_t_16);
  PyTuple_SET_ITEM(__pyx_t_1, 21, __pyx_t_16);
  __Pyx_INCREF(__pyx_tuple__57);
  __Pyx_GIVEREF(__pyx_tuple__57);
  PyTuple_SET_ITEM(__pyx_t_1, 22, __pyx_tuple__57);
  __Pyx_GIVEREF(__pyx_t_17);
  PyTuple_SET_ITEM(__pyx_t_1, 23, __pyx_t_17);
  __Pyx_GIVEREF(__pyx_t_18);
  PyTuple_SET_ITEM(__pyx_t_1, 24, __pyx_t_18);
  __Pyx_INCREF(__pyx_tuple__58);
  __Pyx_GIVEREF(__pyx_tuple__58);
  PyTuple_SET_ITEM(__pyx_t_1, 25, __pyx_tuple__58);
  __Pyx_GIVEREF(__pyx_t_19);
  PyTuple_SET_ITEM(__pyx_t_1, 26, __pyx_t_19);
  __Pyx_GIVEREF(__pyx_t_20);
  PyTuple_SET_ITEM(__pyx_t_1, 27, __pyx_t_20);
  __Pyx_GIVEREF(__pyx_t_21);
  PyTuple_SET_ITEM(__pyx_t_1, 28, __pyx_t_21);
  __Pyx_GIVEREF(__pyx_t_22);
  PyTuple_SET_ITEM(__pyx_t_1, 29, __pyx_t_22);
  __Pyx_GIVEREF(__pyx_t_23);
  PyTuple_SET_ITEM(__pyx_t_1, 30, __pyx_t_23);
  __Pyx_GIVEREF(__pyx_t_24);
  PyTuple_SET_ITEM(__pyx_t_1, 31, __pyx_t_24);
  __Pyx_INCREF(__pyx_tuple__59);
  __Pyx_GIVEREF(__pyx_tuple__59);
  PyTuple_SET_ITEM(__pyx_t_1, 32, __pyx_tuple__59);
  __Pyx_INCREF(__pyx_tuple__60);
  __Pyx_GIVEREF(__pyx_tuple__60);
  PyTuple_SET_ITEM(__pyx_t_1, 33, __pyx_tuple__60);
  __Pyx_INCREF(__pyx_tuple__61);
  __Pyx_GIVEREF(__pyx_tuple__61);
  PyTuple_SET_ITEM(__pyx_t_1, 34, __pyx_tuple__61);
  __Pyx_INCREF(__pyx_tuple__62);
  __Pyx_GIVEREF(__pyx_tuple__62);
  PyTuple_SET_ITEM(__pyx_t_1, 35, __pyx_tuple__62);
  __pyx_t_6 = 0;
  __pyx_t_5 = 0;
  __pyx_t_4 = 0;
  __pyx_t_3 = 0;
  __pyx_t_2 = 0;
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  __pyx_t_9 = 0;
  __pyx_t_10 = 0;
  __pyx_t_11 = 0;
  __pyx_t_12 = 0;
  __pyx_t_13 = 0;
  __pyx_t_14 = 0;
  __pyx_t_15 = 0;
  __pyx_t_16 = 0;
  __pyx_t_17 = 0;
  __pyx_t_18 = 0;
  __pyx_t_19 = 0;
  __pyx_t_20 = 0;
  __pyx_t_21 = 0;
  __pyx_t_22 = 0;
  __pyx_t_23 = 0;
  __pyx_t_24 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PRODUCT_CONFIGURATION, __pyx_t_1) < 0) __PYX_ERR(0, 976, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1017
 * # 12 bytes: ymds_time Structure (section 4.2.76, page 72)
 * YMDS_TIME = (
 *     ('seconds', SINT4),             # <<<<<<<<<<<<<<
 *     ('milliseconds', UINT2),    # milliseconds in lowest 10 bits,
 *     ('year', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1017, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_24 = PyTuple_New(2); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 1017, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_24);
  __Pyx_INCREF(__pyx_n_u_seconds);
  __Pyx_GIVEREF(__pyx_n_u_seconds);
  PyTuple_SET_ITEM(__pyx_t_24, 0, __pyx_n_u_seconds);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_24, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1018
 * YMDS_TIME = (
 *     ('seconds', SINT4),
 *     ('milliseconds', UINT2),    # milliseconds in lowest 10 bits,             # <<<<<<<<<<<<<<
 *     ('year', SINT2),
 *     ('month', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1018, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_23 = PyTuple_New(2); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 1018, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_23);
  __Pyx_INCREF(__pyx_n_u_milliseconds);
  __Pyx_GIVEREF(__pyx_n_u_milliseconds);
  PyTuple_SET_ITEM(__pyx_t_23, 0, __pyx_n_u_milliseconds);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_23, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1019
 *     ('seconds', SINT4),
 *     ('milliseconds', UINT2),    # milliseconds in lowest 10 bits,
 *     ('year', SINT2),             # <<<<<<<<<<<<<<
 *     ('month', SINT2),
 *     ('day', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1019, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_22 = PyTuple_New(2); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 1019, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_22);
  __Pyx_INCREF(__pyx_n_u_year);
  __Pyx_GIVEREF(__pyx_n_u_year);
  PyTuple_SET_ITEM(__pyx_t_22, 0, __pyx_n_u_year);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_22, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1020
 *     ('milliseconds', UINT2),    # milliseconds in lowest 10 bits,
 *     ('year', SINT2),
 *     ('month', SINT2),             # <<<<<<<<<<<<<<
 *     ('day', SINT2),
 * )
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1020, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_21 = PyTuple_New(2); if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 1020, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_21);
  __Pyx_INCREF(__pyx_n_u_month);
  __Pyx_GIVEREF(__pyx_n_u_month);
  PyTuple_SET_ITEM(__pyx_t_21, 0, __pyx_n_u_month);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_21, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1021
 *     ('year', SINT2),
 *     ('month', SINT2),
 *     ('day', SINT2),             # <<<<<<<<<<<<<<
 * )
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1021, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_20 = PyTuple_New(2); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 1021, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_20);
  __Pyx_INCREF(__pyx_n_u_day);
  __Pyx_GIVEREF(__pyx_n_u_day);
  PyTuple_SET_ITEM(__pyx_t_20, 0, __pyx_n_u_day);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_20, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1017
 * # 12 bytes: ymds_time Structure (section 4.2.76, page 72)
 * YMDS_TIME = (
 *     ('seconds', SINT4),             # <<<<<<<<<<<<<<
 *     ('milliseconds', UINT2),    # milliseconds in lowest 10 bits,
 *     ('year', SINT2),
 */
  __pyx_t_1 = PyTuple_New(5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1017, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_24);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_24);
  __Pyx_GIVEREF(__pyx_t_23);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_23);
  __Pyx_GIVEREF(__pyx_t_22);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_t_22);
  __Pyx_GIVEREF(__pyx_t_21);
  PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_t_21);
  __Pyx_GIVEREF(__pyx_t_20);
  PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_t_20);
  __pyx_t_24 = 0;
  __pyx_t_23 = 0;
  __pyx_t_22 = 0;
  __pyx_t_21 = 0;
  __pyx_t_20 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_YMDS_TIME, __pyx_t_1) < 0) __PYX_ERR(0, 1016, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1026
 * # 48 bytes: color_scale_def (section 4.2.5, page 34)
 * COLOR_SCALE_DEF = (
 *     ('iflags', UINT4),             # <<<<<<<<<<<<<<
 *     ('istart', SINT4),
 *     ('istep', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1026, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_20 = PyTuple_New(2); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 1026, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_20);
  __Pyx_INCREF(__pyx_n_u_iflags);
  __Pyx_GIVEREF(__pyx_n_u_iflags);
  PyTuple_SET_ITEM(__pyx_t_20, 0, __pyx_n_u_iflags);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_20, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1027
 * COLOR_SCALE_DEF = (
 *     ('iflags', UINT4),
 *     ('istart', SINT4),             # <<<<<<<<<<<<<<
 *     ('istep', SINT4),
 *     ('icolcnt', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1027, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_21 = PyTuple_New(2); if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 1027, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_21);
  __Pyx_INCREF(__pyx_n_u_istart);
  __Pyx_GIVEREF(__pyx_n_u_istart);
  PyTuple_SET_ITEM(__pyx_t_21, 0, __pyx_n_u_istart);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_21, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1028
 *     ('iflags', UINT4),
 *     ('istart', SINT4),
 *     ('istep', SINT4),             # <<<<<<<<<<<<<<
 *     ('icolcnt', SINT2),
 *     ('iset_and_scale', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1028, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_22 = PyTuple_New(2); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 1028, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_22);
  __Pyx_INCREF(__pyx_n_u_istep);
  __Pyx_GIVEREF(__pyx_n_u_istep);
  PyTuple_SET_ITEM(__pyx_t_22, 0, __pyx_n_u_istep);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_22, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1029
 *     ('istart', SINT4),
 *     ('istep', SINT4),
 *     ('icolcnt', SINT2),             # <<<<<<<<<<<<<<
 *     ('iset_and_scale', UINT2),
 *     ('ilevel_seams', '32s')     # 32 bytes: UINT2[16]
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1029, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_23 = PyTuple_New(2); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 1029, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_23);
  __Pyx_INCREF(__pyx_n_u_icolcnt);
  __Pyx_GIVEREF(__pyx_n_u_icolcnt);
  PyTuple_SET_ITEM(__pyx_t_23, 0, __pyx_n_u_icolcnt);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_23, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1030
 *     ('istep', SINT4),
 *     ('icolcnt', SINT2),
 *     ('iset_and_scale', UINT2),             # <<<<<<<<<<<<<<
 *     ('ilevel_seams', '32s')     # 32 bytes: UINT2[16]
 * )
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1030, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_24 = PyTuple_New(2); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 1030, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_24);
  __Pyx_INCREF(__pyx_n_u_iset_and_scale);
  __Pyx_GIVEREF(__pyx_n_u_iset_and_scale);
  PyTuple_SET_ITEM(__pyx_t_24, 0, __pyx_n_u_iset_and_scale);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_24, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1026
 * # 48 bytes: color_scale_def (section 4.2.5, page 34)
 * COLOR_SCALE_DEF = (
 *     ('iflags', UINT4),             # <<<<<<<<<<<<<<
 *     ('istart', SINT4),
 *     ('istep', SINT4),
 */
  __pyx_t_1 = PyTuple_New(6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1026, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_20);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_20);
  __Pyx_GIVEREF(__pyx_t_21);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_21);
  __Pyx_GIVEREF(__pyx_t_22);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_t_22);
  __Pyx_GIVEREF(__pyx_t_23);
  PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_t_23);
  __Pyx_GIVEREF(__pyx_t_24);
  PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_t_24);
  __Pyx_INCREF(__pyx_tuple__63);
  __Pyx_GIVEREF(__pyx_tuple__63);
  PyTuple_SET_ITEM(__pyx_t_1, 5, __pyx_tuple__63);
  __pyx_t_20 = 0;
  __pyx_t_21 = 0;
  __pyx_t_22 = 0;
  __pyx_t_23 = 0;
  __pyx_t_24 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_COLOR_SCALE_DEF, __pyx_t_1) < 0) __PYX_ERR(0, 1025, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1041
 *     ('ingest_time', '12s'),         # 12 bytes: ymds_time
 *     ('spare_0', '28s'),             # 28 bytes
 *     ('GMT_minute_offset_local', SINT2),             # <<<<<<<<<<<<<<
 *     ('ingest_hardware_name_', '16s'),
 *     ('ingest_site_name_', '16s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1041, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_24 = PyTuple_New(2); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 1041, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_24);
  __Pyx_INCREF(__pyx_n_u_GMT_minute_offset_local);
  __Pyx_GIVEREF(__pyx_n_u_GMT_minute_offset_local);
  PyTuple_SET_ITEM(__pyx_t_24, 0, __pyx_n_u_GMT_minute_offset_local);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_24, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1044
 *     ('ingest_hardware_name_', '16s'),
 *     ('ingest_site_name_', '16s'),
 *     ('GMT_minute_offset_standard', SINT2),             # <<<<<<<<<<<<<<
 *     ('latitude', BIN4),
 *     ('longitude', BIN4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1044, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_23 = PyTuple_New(2); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 1044, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_23);
  __Pyx_INCREF(__pyx_n_u_GMT_minute_offset_standard);
  __Pyx_GIVEREF(__pyx_n_u_GMT_minute_offset_standard);
  PyTuple_SET_ITEM(__pyx_t_23, 0, __pyx_n_u_GMT_minute_offset_standard);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_23, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1045
 *     ('ingest_site_name_', '16s'),
 *     ('GMT_minute_offset_standard', SINT2),
 *     ('latitude', BIN4),             # <<<<<<<<<<<<<<
 *     ('longitude', BIN4),
 *     ('ground_height', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_BIN4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1045, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_22 = PyTuple_New(2); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 1045, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_22);
  __Pyx_INCREF(__pyx_n_u_latitude);
  __Pyx_GIVEREF(__pyx_n_u_latitude);
  PyTuple_SET_ITEM(__pyx_t_22, 0, __pyx_n_u_latitude);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_22, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1046
 *     ('GMT_minute_offset_standard', SINT2),
 *     ('latitude', BIN4),
 *     ('longitude', BIN4),             # <<<<<<<<<<<<<<
 *     ('ground_height', SINT2),
 *     ('radar_height', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_BIN4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1046, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_21 = PyTuple_New(2); if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 1046, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_21);
  __Pyx_INCREF(__pyx_n_u_longitude);
  __Pyx_GIVEREF(__pyx_n_u_longitude);
  PyTuple_SET_ITEM(__pyx_t_21, 0, __pyx_n_u_longitude);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_21, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1047
 *     ('latitude', BIN4),
 *     ('longitude', BIN4),
 *     ('ground_height', SINT2),             # <<<<<<<<<<<<<<
 *     ('radar_height', SINT2),
 *     ('prf', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1047, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_20 = PyTuple_New(2); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 1047, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_20);
  __Pyx_INCREF(__pyx_n_u_ground_height);
  __Pyx_GIVEREF(__pyx_n_u_ground_height);
  PyTuple_SET_ITEM(__pyx_t_20, 0, __pyx_n_u_ground_height);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_20, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1048
 *     ('longitude', BIN4),
 *     ('ground_height', SINT2),
 *     ('radar_height', SINT2),             # <<<<<<<<<<<<<<
 *     ('prf', SINT4),
 *     ('pulse_width', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1048, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_19 = PyTuple_New(2); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 1048, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_19);
  __Pyx_INCREF(__pyx_n_u_radar_height);
  __Pyx_GIVEREF(__pyx_n_u_radar_height);
  PyTuple_SET_ITEM(__pyx_t_19, 0, __pyx_n_u_radar_height);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_19, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1049
 *     ('ground_height', SINT2),
 *     ('radar_height', SINT2),
 *     ('prf', SINT4),             # <<<<<<<<<<<<<<
 *     ('pulse_width', SINT4),
 *     ('signal_processor_type', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1049, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_18 = PyTuple_New(2); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 1049, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_18);
  __Pyx_INCREF(__pyx_n_u_prf);
  __Pyx_GIVEREF(__pyx_n_u_prf);
  PyTuple_SET_ITEM(__pyx_t_18, 0, __pyx_n_u_prf);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_18, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1050
 *     ('radar_height', SINT2),
 *     ('prf', SINT4),
 *     ('pulse_width', SINT4),             # <<<<<<<<<<<<<<
 *     ('signal_processor_type', UINT2),
 *     ('trigger_rate', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1050, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_17 = PyTuple_New(2); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1050, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_17);
  __Pyx_INCREF(__pyx_n_u_pulse_width);
  __Pyx_GIVEREF(__pyx_n_u_pulse_width);
  PyTuple_SET_ITEM(__pyx_t_17, 0, __pyx_n_u_pulse_width);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_17, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1051
 *     ('prf', SINT4),
 *     ('pulse_width', SINT4),
 *     ('signal_processor_type', UINT2),             # <<<<<<<<<<<<<<
 *     ('trigger_rate', UINT2),
 *     ('samples_used', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1051, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_16 = PyTuple_New(2); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1051, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);
  __Pyx_INCREF(__pyx_n_u_signal_processor_type);
  __Pyx_GIVEREF(__pyx_n_u_signal_processor_type);
  PyTuple_SET_ITEM(__pyx_t_16, 0, __pyx_n_u_signal_processor_type);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_16, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1052
 *     ('pulse_width', SINT4),
 *     ('signal_processor_type', UINT2),
 *     ('trigger_rate', UINT2),             # <<<<<<<<<<<<<<
 *     ('samples_used', SINT2),
 *     ('clutter_filter', '12s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1052, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_15 = PyTuple_New(2); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1052, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __Pyx_INCREF(__pyx_n_u_trigger_rate);
  __Pyx_GIVEREF(__pyx_n_u_trigger_rate);
  PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_n_u_trigger_rate);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_15, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1053
 *     ('signal_processor_type', UINT2),
 *     ('trigger_rate', UINT2),
 *     ('samples_used', SINT2),             # <<<<<<<<<<<<<<
 *     ('clutter_filter', '12s'),
 *     ('number_linear_filter', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1053, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_14 = PyTuple_New(2); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1053, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_INCREF(__pyx_n_u_samples_used);
  __Pyx_GIVEREF(__pyx_n_u_samples_used);
  PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_n_u_samples_used);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_14, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1055
 *     ('samples_used', SINT2),
 *     ('clutter_filter', '12s'),
 *     ('number_linear_filter', UINT2),             # <<<<<<<<<<<<<<
 *     ('wavelength', SINT4),
 *     ('truncation_height', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1055, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_13 = PyTuple_New(2); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1055, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_INCREF(__pyx_n_u_number_linear_filter);
  __Pyx_GIVEREF(__pyx_n_u_number_linear_filter);
  PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_n_u_number_linear_filter);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_13, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1056
 *     ('clutter_filter', '12s'),
 *     ('number_linear_filter', UINT2),
 *     ('wavelength', SINT4),             # <<<<<<<<<<<<<<
 *     ('truncation_height', SINT4),
 *     ('first_bin_range', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1056, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_12 = PyTuple_New(2); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1056, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_INCREF(__pyx_n_u_wavelength);
  __Pyx_GIVEREF(__pyx_n_u_wavelength);
  PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_n_u_wavelength);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_12, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1057
 *     ('number_linear_filter', UINT2),
 *     ('wavelength', SINT4),
 *     ('truncation_height', SINT4),             # <<<<<<<<<<<<<<
 *     ('first_bin_range', SINT4),
 *     ('last_bin_range', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1057, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_11 = PyTuple_New(2); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1057, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_INCREF(__pyx_n_u_truncation_height);
  __Pyx_GIVEREF(__pyx_n_u_truncation_height);
  PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_n_u_truncation_height);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1058
 *     ('wavelength', SINT4),
 *     ('truncation_height', SINT4),
 *     ('first_bin_range', SINT4),             # <<<<<<<<<<<<<<
 *     ('last_bin_range', SINT4),
 *     ('number_bins', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1058, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_10 = PyTuple_New(2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1058, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_INCREF(__pyx_n_u_first_bin_range);
  __Pyx_GIVEREF(__pyx_n_u_first_bin_range);
  PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_n_u_first_bin_range);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1059
 *     ('truncation_height', SINT4),
 *     ('first_bin_range', SINT4),
 *     ('last_bin_range', SINT4),             # <<<<<<<<<<<<<<
 *     ('number_bins', SINT4),
 *     ('flag', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1059, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1059, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_INCREF(__pyx_n_u_last_bin_range);
  __Pyx_GIVEREF(__pyx_n_u_last_bin_range);
  PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_n_u_last_bin_range);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1060
 *     ('first_bin_range', SINT4),
 *     ('last_bin_range', SINT4),
 *     ('number_bins', SINT4),             # <<<<<<<<<<<<<<
 *     ('flag', UINT2),
 *     ('number_ingest', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1060, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1060, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_INCREF(__pyx_n_u_number_bins);
  __Pyx_GIVEREF(__pyx_n_u_number_bins);
  PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_n_u_number_bins);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1061
 *     ('last_bin_range', SINT4),
 *     ('number_bins', SINT4),
 *     ('flag', UINT2),             # <<<<<<<<<<<<<<
 *     ('number_ingest', SINT2),
 *     ('polarization', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1061, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1061, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_INCREF(__pyx_n_u_flag);
  __Pyx_GIVEREF(__pyx_n_u_flag);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_n_u_flag);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1062
 *     ('number_bins', SINT4),
 *     ('flag', UINT2),
 *     ('number_ingest', SINT2),             # <<<<<<<<<<<<<<
 *     ('polarization', UINT2),
 *     ('horizontal_calibration_i0', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1062, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1062, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_number_ingest);
  __Pyx_GIVEREF(__pyx_n_u_number_ingest);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_number_ingest);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1063
 *     ('flag', UINT2),
 *     ('number_ingest', SINT2),
 *     ('polarization', UINT2),             # <<<<<<<<<<<<<<
 *     ('horizontal_calibration_i0', SINT2),
 *     ('horizontal_calibration_noise', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1063, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1063, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_n_u_polarization);
  __Pyx_GIVEREF(__pyx_n_u_polarization);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_n_u_polarization);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1064
 *     ('number_ingest', SINT2),
 *     ('polarization', UINT2),
 *     ('horizontal_calibration_i0', SINT2),             # <<<<<<<<<<<<<<
 *     ('horizontal_calibration_noise', SINT2),
 *     ('horizontal_radar_constant', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1064, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1064, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_n_u_horizontal_calibration_i0);
  __Pyx_GIVEREF(__pyx_n_u_horizontal_calibration_i0);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_n_u_horizontal_calibration_i0);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1065
 *     ('polarization', UINT2),
 *     ('horizontal_calibration_i0', SINT2),
 *     ('horizontal_calibration_noise', SINT2),             # <<<<<<<<<<<<<<
 *     ('horizontal_radar_constant', SINT2),
 *     ('reciever_bandwidth', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1065, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1065, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_n_u_horizontal_calibration_noise);
  __Pyx_GIVEREF(__pyx_n_u_horizontal_calibration_noise);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_n_u_horizontal_calibration_noise);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1066
 *     ('horizontal_calibration_i0', SINT2),
 *     ('horizontal_calibration_noise', SINT2),
 *     ('horizontal_radar_constant', SINT2),             # <<<<<<<<<<<<<<
 *     ('reciever_bandwidth', UINT2),
 *     ('horizontal_current_noise', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1066, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1066, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_n_u_horizontal_radar_constant);
  __Pyx_GIVEREF(__pyx_n_u_horizontal_radar_constant);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_n_u_horizontal_radar_constant);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1067
 *     ('horizontal_calibration_noise', SINT2),
 *     ('horizontal_radar_constant', SINT2),
 *     ('reciever_bandwidth', UINT2),             # <<<<<<<<<<<<<<
 *     ('horizontal_current_noise', SINT2),
 *     ('vertical_current_noise', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1067, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_25 = PyTuple_New(2); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 1067, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_25);
  __Pyx_INCREF(__pyx_n_u_reciever_bandwidth);
  __Pyx_GIVEREF(__pyx_n_u_reciever_bandwidth);
  PyTuple_SET_ITEM(__pyx_t_25, 0, __pyx_n_u_reciever_bandwidth);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_25, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1068
 *     ('horizontal_radar_constant', SINT2),
 *     ('reciever_bandwidth', UINT2),
 *     ('horizontal_current_noise', SINT2),             # <<<<<<<<<<<<<<
 *     ('vertical_current_noise', SINT2),
 *     ('ldr_offset', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1068, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_26 = PyTuple_New(2); if (unlikely(!__pyx_t_26)) __PYX_ERR(0, 1068, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_26);
  __Pyx_INCREF(__pyx_n_u_horizontal_current_noise);
  __Pyx_GIVEREF(__pyx_n_u_horizontal_current_noise);
  PyTuple_SET_ITEM(__pyx_t_26, 0, __pyx_n_u_horizontal_current_noise);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_26, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1069
 *     ('reciever_bandwidth', UINT2),
 *     ('horizontal_current_noise', SINT2),
 *     ('vertical_current_noise', SINT2),             # <<<<<<<<<<<<<<
 *     ('ldr_offset', SINT2),
 *     ('zdr_offset', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1069, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_27 = PyTuple_New(2); if (unlikely(!__pyx_t_27)) __PYX_ERR(0, 1069, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_27);
  __Pyx_INCREF(__pyx_n_u_vertical_current_noise);
  __Pyx_GIVEREF(__pyx_n_u_vertical_current_noise);
  PyTuple_SET_ITEM(__pyx_t_27, 0, __pyx_n_u_vertical_current_noise);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_27, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1070
 *     ('horizontal_current_noise', SINT2),
 *     ('vertical_current_noise', SINT2),
 *     ('ldr_offset', SINT2),             # <<<<<<<<<<<<<<
 *     ('zdr_offset', SINT2),
 *     ('tcf_cal_flags_1', UINT16_T),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1070, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_28 = PyTuple_New(2); if (unlikely(!__pyx_t_28)) __PYX_ERR(0, 1070, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_28);
  __Pyx_INCREF(__pyx_n_u_ldr_offset);
  __Pyx_GIVEREF(__pyx_n_u_ldr_offset);
  PyTuple_SET_ITEM(__pyx_t_28, 0, __pyx_n_u_ldr_offset);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_28, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1071
 *     ('vertical_current_noise', SINT2),
 *     ('ldr_offset', SINT2),
 *     ('zdr_offset', SINT2),             # <<<<<<<<<<<<<<
 *     ('tcf_cal_flags_1', UINT16_T),
 *     ('tcf_cal_flags_2', UINT16_T),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1071, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_29 = PyTuple_New(2); if (unlikely(!__pyx_t_29)) __PYX_ERR(0, 1071, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_29);
  __Pyx_INCREF(__pyx_n_u_zdr_offset);
  __Pyx_GIVEREF(__pyx_n_u_zdr_offset);
  PyTuple_SET_ITEM(__pyx_t_29, 0, __pyx_n_u_zdr_offset);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_29, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1072
 *     ('ldr_offset', SINT2),
 *     ('zdr_offset', SINT2),
 *     ('tcf_cal_flags_1', UINT16_T),             # <<<<<<<<<<<<<<
 *     ('tcf_cal_flags_2', UINT16_T),
 *     ('spare_1', '18s'),             # 18 bytes
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT16_T); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1072, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_30 = PyTuple_New(2); if (unlikely(!__pyx_t_30)) __PYX_ERR(0, 1072, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_30);
  __Pyx_INCREF(__pyx_n_u_tcf_cal_flags_1);
  __Pyx_GIVEREF(__pyx_n_u_tcf_cal_flags_1);
  PyTuple_SET_ITEM(__pyx_t_30, 0, __pyx_n_u_tcf_cal_flags_1);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_30, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1073
 *     ('zdr_offset', SINT2),
 *     ('tcf_cal_flags_1', UINT16_T),
 *     ('tcf_cal_flags_2', UINT16_T),             # <<<<<<<<<<<<<<
 *     ('spare_1', '18s'),             # 18 bytes
 *     ('standard_parallel_1', BIN4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT16_T); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1073, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_31 = PyTuple_New(2); if (unlikely(!__pyx_t_31)) __PYX_ERR(0, 1073, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_31);
  __Pyx_INCREF(__pyx_n_u_tcf_cal_flags_2);
  __Pyx_GIVEREF(__pyx_n_u_tcf_cal_flags_2);
  PyTuple_SET_ITEM(__pyx_t_31, 0, __pyx_n_u_tcf_cal_flags_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_31, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1075
 *     ('tcf_cal_flags_2', UINT16_T),
 *     ('spare_1', '18s'),             # 18 bytes
 *     ('standard_parallel_1', BIN4),             # <<<<<<<<<<<<<<
 *     ('standard_parallel_2', BIN4),
 *     ('earth_radius', UINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_BIN4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1075, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_32 = PyTuple_New(2); if (unlikely(!__pyx_t_32)) __PYX_ERR(0, 1075, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_32);
  __Pyx_INCREF(__pyx_n_u_standard_parallel_1);
  __Pyx_GIVEREF(__pyx_n_u_standard_parallel_1);
  PyTuple_SET_ITEM(__pyx_t_32, 0, __pyx_n_u_standard_parallel_1);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_32, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1076
 *     ('spare_1', '18s'),             # 18 bytes
 *     ('standard_parallel_1', BIN4),
 *     ('standard_parallel_2', BIN4),             # <<<<<<<<<<<<<<
 *     ('earth_radius', UINT4),
 *     ('inverse_flatting', UINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_BIN4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1076, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_33 = PyTuple_New(2); if (unlikely(!__pyx_t_33)) __PYX_ERR(0, 1076, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_33);
  __Pyx_INCREF(__pyx_n_u_standard_parallel_2);
  __Pyx_GIVEREF(__pyx_n_u_standard_parallel_2);
  PyTuple_SET_ITEM(__pyx_t_33, 0, __pyx_n_u_standard_parallel_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_33, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1077
 *     ('standard_parallel_1', BIN4),
 *     ('standard_parallel_2', BIN4),
 *     ('earth_radius', UINT4),             # <<<<<<<<<<<<<<
 *     ('inverse_flatting', UINT4),
 *     ('fault_status', UINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1077, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_34 = PyTuple_New(2); if (unlikely(!__pyx_t_34)) __PYX_ERR(0, 1077, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_34);
  __Pyx_INCREF(__pyx_n_u_earth_radius);
  __Pyx_GIVEREF(__pyx_n_u_earth_radius);
  PyTuple_SET_ITEM(__pyx_t_34, 0, __pyx_n_u_earth_radius);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_34, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1078
 *     ('standard_parallel_2', BIN4),
 *     ('earth_radius', UINT4),
 *     ('inverse_flatting', UINT4),             # <<<<<<<<<<<<<<
 *     ('fault_status', UINT4),
 *     ('input_mask', UINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1078, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_35 = PyTuple_New(2); if (unlikely(!__pyx_t_35)) __PYX_ERR(0, 1078, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_35);
  __Pyx_INCREF(__pyx_n_u_inverse_flatting);
  __Pyx_GIVEREF(__pyx_n_u_inverse_flatting);
  PyTuple_SET_ITEM(__pyx_t_35, 0, __pyx_n_u_inverse_flatting);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_35, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1079
 *     ('earth_radius', UINT4),
 *     ('inverse_flatting', UINT4),
 *     ('fault_status', UINT4),             # <<<<<<<<<<<<<<
 *     ('input_mask', UINT4),
 *     ('number_log_filter', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1079, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_36 = PyTuple_New(2); if (unlikely(!__pyx_t_36)) __PYX_ERR(0, 1079, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_36);
  __Pyx_INCREF(__pyx_n_u_fault_status);
  __Pyx_GIVEREF(__pyx_n_u_fault_status);
  PyTuple_SET_ITEM(__pyx_t_36, 0, __pyx_n_u_fault_status);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_36, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1080
 *     ('inverse_flatting', UINT4),
 *     ('fault_status', UINT4),
 *     ('input_mask', UINT4),             # <<<<<<<<<<<<<<
 *     ('number_log_filter', UINT2),
 *     ('cluttermap', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1080, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_37 = PyTuple_New(2); if (unlikely(!__pyx_t_37)) __PYX_ERR(0, 1080, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_37);
  __Pyx_INCREF(__pyx_n_u_input_mask);
  __Pyx_GIVEREF(__pyx_n_u_input_mask);
  PyTuple_SET_ITEM(__pyx_t_37, 0, __pyx_n_u_input_mask);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_37, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1081
 *     ('fault_status', UINT4),
 *     ('input_mask', UINT4),
 *     ('number_log_filter', UINT2),             # <<<<<<<<<<<<<<
 *     ('cluttermap', UINT2),
 *     ('latitude_projection', BIN4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1081, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_38 = PyTuple_New(2); if (unlikely(!__pyx_t_38)) __PYX_ERR(0, 1081, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_38);
  __Pyx_INCREF(__pyx_n_u_number_log_filter);
  __Pyx_GIVEREF(__pyx_n_u_number_log_filter);
  PyTuple_SET_ITEM(__pyx_t_38, 0, __pyx_n_u_number_log_filter);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_38, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1082
 *     ('input_mask', UINT4),
 *     ('number_log_filter', UINT2),
 *     ('cluttermap', UINT2),             # <<<<<<<<<<<<<<
 *     ('latitude_projection', BIN4),
 *     ('longitude_projection', BIN4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1082, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_39 = PyTuple_New(2); if (unlikely(!__pyx_t_39)) __PYX_ERR(0, 1082, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_39);
  __Pyx_INCREF(__pyx_n_u_cluttermap);
  __Pyx_GIVEREF(__pyx_n_u_cluttermap);
  PyTuple_SET_ITEM(__pyx_t_39, 0, __pyx_n_u_cluttermap);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_39, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1083
 *     ('number_log_filter', UINT2),
 *     ('cluttermap', UINT2),
 *     ('latitude_projection', BIN4),             # <<<<<<<<<<<<<<
 *     ('longitude_projection', BIN4),
 *     ('product_sequence_number', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_BIN4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1083, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_40 = PyTuple_New(2); if (unlikely(!__pyx_t_40)) __PYX_ERR(0, 1083, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_40);
  __Pyx_INCREF(__pyx_n_u_latitude_projection);
  __Pyx_GIVEREF(__pyx_n_u_latitude_projection);
  PyTuple_SET_ITEM(__pyx_t_40, 0, __pyx_n_u_latitude_projection);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_40, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1084
 *     ('cluttermap', UINT2),
 *     ('latitude_projection', BIN4),
 *     ('longitude_projection', BIN4),             # <<<<<<<<<<<<<<
 *     ('product_sequence_number', SINT2),
 *     ('spare_2', '32s'),             # 32 bytes
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_BIN4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1084, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_41 = PyTuple_New(2); if (unlikely(!__pyx_t_41)) __PYX_ERR(0, 1084, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_41);
  __Pyx_INCREF(__pyx_n_u_longitude_projection);
  __Pyx_GIVEREF(__pyx_n_u_longitude_projection);
  PyTuple_SET_ITEM(__pyx_t_41, 0, __pyx_n_u_longitude_projection);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_41, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1085
 *     ('latitude_projection', BIN4),
 *     ('longitude_projection', BIN4),
 *     ('product_sequence_number', SINT2),             # <<<<<<<<<<<<<<
 *     ('spare_2', '32s'),             # 32 bytes
 *     ('melting_level', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1085, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_42 = PyTuple_New(2); if (unlikely(!__pyx_t_42)) __PYX_ERR(0, 1085, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_42);
  __Pyx_INCREF(__pyx_n_u_product_sequence_number);
  __Pyx_GIVEREF(__pyx_n_u_product_sequence_number);
  PyTuple_SET_ITEM(__pyx_t_42, 0, __pyx_n_u_product_sequence_number);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_42, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1087
 *     ('product_sequence_number', SINT2),
 *     ('spare_2', '32s'),             # 32 bytes
 *     ('melting_level', SINT2),             # <<<<<<<<<<<<<<
 *     ('radar_height_above_reference', SINT2),
 *     ('number_elements', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1087, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_43 = PyTuple_New(2); if (unlikely(!__pyx_t_43)) __PYX_ERR(0, 1087, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_43);
  __Pyx_INCREF(__pyx_n_u_melting_level);
  __Pyx_GIVEREF(__pyx_n_u_melting_level);
  PyTuple_SET_ITEM(__pyx_t_43, 0, __pyx_n_u_melting_level);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_43, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1088
 *     ('spare_2', '32s'),             # 32 bytes
 *     ('melting_level', SINT2),
 *     ('radar_height_above_reference', SINT2),             # <<<<<<<<<<<<<<
 *     ('number_elements', SINT2),
 *     ('mean_wind_speed', UINT1),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1088, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_44 = PyTuple_New(2); if (unlikely(!__pyx_t_44)) __PYX_ERR(0, 1088, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_44);
  __Pyx_INCREF(__pyx_n_u_radar_height_above_reference);
  __Pyx_GIVEREF(__pyx_n_u_radar_height_above_reference);
  PyTuple_SET_ITEM(__pyx_t_44, 0, __pyx_n_u_radar_height_above_reference);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_44, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1089
 *     ('melting_level', SINT2),
 *     ('radar_height_above_reference', SINT2),
 *     ('number_elements', SINT2),             # <<<<<<<<<<<<<<
 *     ('mean_wind_speed', UINT1),
 *     ('mean_wind_direction', BIN1),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1089, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_45 = PyTuple_New(2); if (unlikely(!__pyx_t_45)) __PYX_ERR(0, 1089, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_45);
  __Pyx_INCREF(__pyx_n_u_number_elements);
  __Pyx_GIVEREF(__pyx_n_u_number_elements);
  PyTuple_SET_ITEM(__pyx_t_45, 0, __pyx_n_u_number_elements);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_45, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1090
 *     ('radar_height_above_reference', SINT2),
 *     ('number_elements', SINT2),
 *     ('mean_wind_speed', UINT1),             # <<<<<<<<<<<<<<
 *     ('mean_wind_direction', BIN1),
 *     ('spare_3', '2s'),              # 2 bytes
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1090, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_46 = PyTuple_New(2); if (unlikely(!__pyx_t_46)) __PYX_ERR(0, 1090, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_46);
  __Pyx_INCREF(__pyx_n_u_mean_wind_speed);
  __Pyx_GIVEREF(__pyx_n_u_mean_wind_speed);
  PyTuple_SET_ITEM(__pyx_t_46, 0, __pyx_n_u_mean_wind_speed);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_46, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1091
 *     ('number_elements', SINT2),
 *     ('mean_wind_speed', UINT1),
 *     ('mean_wind_direction', BIN1),             # <<<<<<<<<<<<<<
 *     ('spare_3', '2s'),              # 2 bytes
 *     ('tz_name', '8s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_BIN1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1091, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_47 = PyTuple_New(2); if (unlikely(!__pyx_t_47)) __PYX_ERR(0, 1091, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_47);
  __Pyx_INCREF(__pyx_n_u_mean_wind_direction);
  __Pyx_GIVEREF(__pyx_n_u_mean_wind_direction);
  PyTuple_SET_ITEM(__pyx_t_47, 0, __pyx_n_u_mean_wind_direction);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_47, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1094
 *     ('spare_3', '2s'),              # 2 bytes
 *     ('tz_name', '8s'),
 *     ('extended_product_header_offset', UINT4),             # <<<<<<<<<<<<<<
 *     ('spare_4', '4s'),              # 4 bytes
 * )
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1094, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_48 = PyTuple_New(2); if (unlikely(!__pyx_t_48)) __PYX_ERR(0, 1094, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_48);
  __Pyx_INCREF(__pyx_n_u_extended_product_header_offset);
  __Pyx_GIVEREF(__pyx_n_u_extended_product_header_offset);
  PyTuple_SET_ITEM(__pyx_t_48, 0, __pyx_n_u_extended_product_header_offset);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_48, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1036
 * # 308 bytes : product_end (section 4.2.24)
 * PRODUCT_END = (
 *     ('site_name', '16s'),             # <<<<<<<<<<<<<<
 *     ('iris_version_created', '8s'),
 *     ('ingest_iris_version', '8s'),
 */
  __pyx_t_1 = PyTuple_New(60); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1036, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_tuple__64);
  __Pyx_GIVEREF(__pyx_tuple__64);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_tuple__64);
  __Pyx_INCREF(__pyx_tuple__65);
  __Pyx_GIVEREF(__pyx_tuple__65);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_tuple__65);
  __Pyx_INCREF(__pyx_tuple__66);
  __Pyx_GIVEREF(__pyx_tuple__66);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_tuple__66);
  __Pyx_INCREF(__pyx_tuple__67);
  __Pyx_GIVEREF(__pyx_tuple__67);
  PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_tuple__67);
  __Pyx_INCREF(__pyx_tuple__68);
  __Pyx_GIVEREF(__pyx_tuple__68);
  PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_tuple__68);
  __Pyx_GIVEREF(__pyx_t_24);
  PyTuple_SET_ITEM(__pyx_t_1, 5, __pyx_t_24);
  __Pyx_INCREF(__pyx_tuple__69);
  __Pyx_GIVEREF(__pyx_tuple__69);
  PyTuple_SET_ITEM(__pyx_t_1, 6, __pyx_tuple__69);
  __Pyx_INCREF(__pyx_tuple__70);
  __Pyx_GIVEREF(__pyx_tuple__70);
  PyTuple_SET_ITEM(__pyx_t_1, 7, __pyx_tuple__70);
  __Pyx_GIVEREF(__pyx_t_23);
  PyTuple_SET_ITEM(__pyx_t_1, 8, __pyx_t_23);
  __Pyx_GIVEREF(__pyx_t_22);
  PyTuple_SET_ITEM(__pyx_t_1, 9, __pyx_t_22);
  __Pyx_GIVEREF(__pyx_t_21);
  PyTuple_SET_ITEM(__pyx_t_1, 10, __pyx_t_21);
  __Pyx_GIVEREF(__pyx_t_20);
  PyTuple_SET_ITEM(__pyx_t_1, 11, __pyx_t_20);
  __Pyx_GIVEREF(__pyx_t_19);
  PyTuple_SET_ITEM(__pyx_t_1, 12, __pyx_t_19);
  __Pyx_GIVEREF(__pyx_t_18);
  PyTuple_SET_ITEM(__pyx_t_1, 13, __pyx_t_18);
  __Pyx_GIVEREF(__pyx_t_17);
  PyTuple_SET_ITEM(__pyx_t_1, 14, __pyx_t_17);
  __Pyx_GIVEREF(__pyx_t_16);
  PyTuple_SET_ITEM(__pyx_t_1, 15, __pyx_t_16);
  __Pyx_GIVEREF(__pyx_t_15);
  PyTuple_SET_ITEM(__pyx_t_1, 16, __pyx_t_15);
  __Pyx_GIVEREF(__pyx_t_14);
  PyTuple_SET_ITEM(__pyx_t_1, 17, __pyx_t_14);
  __Pyx_INCREF(__pyx_tuple__71);
  __Pyx_GIVEREF(__pyx_tuple__71);
  PyTuple_SET_ITEM(__pyx_t_1, 18, __pyx_tuple__71);
  __Pyx_GIVEREF(__pyx_t_13);
  PyTuple_SET_ITEM(__pyx_t_1, 19, __pyx_t_13);
  __Pyx_GIVEREF(__pyx_t_12);
  PyTuple_SET_ITEM(__pyx_t_1, 20, __pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_11);
  PyTuple_SET_ITEM(__pyx_t_1, 21, __pyx_t_11);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_1, 22, __pyx_t_10);
  __Pyx_GIVEREF(__pyx_t_9);
  PyTuple_SET_ITEM(__pyx_t_1, 23, __pyx_t_9);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_1, 24, __pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_1, 25, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 26, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_1, 27, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_1, 28, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_1, 29, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_1, 30, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_25);
  PyTuple_SET_ITEM(__pyx_t_1, 31, __pyx_t_25);
  __Pyx_GIVEREF(__pyx_t_26);
  PyTuple_SET_ITEM(__pyx_t_1, 32, __pyx_t_26);
  __Pyx_GIVEREF(__pyx_t_27);
  PyTuple_SET_ITEM(__pyx_t_1, 33, __pyx_t_27);
  __Pyx_GIVEREF(__pyx_t_28);
  PyTuple_SET_ITEM(__pyx_t_1, 34, __pyx_t_28);
  __Pyx_GIVEREF(__pyx_t_29);
  PyTuple_SET_ITEM(__pyx_t_1, 35, __pyx_t_29);
  __Pyx_GIVEREF(__pyx_t_30);
  PyTuple_SET_ITEM(__pyx_t_1, 36, __pyx_t_30);
  __Pyx_GIVEREF(__pyx_t_31);
  PyTuple_SET_ITEM(__pyx_t_1, 37, __pyx_t_31);
  __Pyx_INCREF(__pyx_tuple__72);
  __Pyx_GIVEREF(__pyx_tuple__72);
  PyTuple_SET_ITEM(__pyx_t_1, 38, __pyx_tuple__72);
  __Pyx_GIVEREF(__pyx_t_32);
  PyTuple_SET_ITEM(__pyx_t_1, 39, __pyx_t_32);
  __Pyx_GIVEREF(__pyx_t_33);
  PyTuple_SET_ITEM(__pyx_t_1, 40, __pyx_t_33);
  __Pyx_GIVEREF(__pyx_t_34);
  PyTuple_SET_ITEM(__pyx_t_1, 41, __pyx_t_34);
  __Pyx_GIVEREF(__pyx_t_35);
  PyTuple_SET_ITEM(__pyx_t_1, 42, __pyx_t_35);
  __Pyx_GIVEREF(__pyx_t_36);
  PyTuple_SET_ITEM(__pyx_t_1, 43, __pyx_t_36);
  __Pyx_GIVEREF(__pyx_t_37);
  PyTuple_SET_ITEM(__pyx_t_1, 44, __pyx_t_37);
  __Pyx_GIVEREF(__pyx_t_38);
  PyTuple_SET_ITEM(__pyx_t_1, 45, __pyx_t_38);
  __Pyx_GIVEREF(__pyx_t_39);
  PyTuple_SET_ITEM(__pyx_t_1, 46, __pyx_t_39);
  __Pyx_GIVEREF(__pyx_t_40);
  PyTuple_SET_ITEM(__pyx_t_1, 47, __pyx_t_40);
  __Pyx_GIVEREF(__pyx_t_41);
  PyTuple_SET_ITEM(__pyx_t_1, 48, __pyx_t_41);
  __Pyx_GIVEREF(__pyx_t_42);
  PyTuple_SET_ITEM(__pyx_t_1, 49, __pyx_t_42);
  __Pyx_INCREF(__pyx_tuple__73);
  __Pyx_GIVEREF(__pyx_tuple__73);
  PyTuple_SET_ITEM(__pyx_t_1, 50, __pyx_tuple__73);
  __Pyx_GIVEREF(__pyx_t_43);
  PyTuple_SET_ITEM(__pyx_t_1, 51, __pyx_t_43);
  __Pyx_GIVEREF(__pyx_t_44);
  PyTuple_SET_ITEM(__pyx_t_1, 52, __pyx_t_44);
  __Pyx_GIVEREF(__pyx_t_45);
  PyTuple_SET_ITEM(__pyx_t_1, 53, __pyx_t_45);
  __Pyx_GIVEREF(__pyx_t_46);
  PyTuple_SET_ITEM(__pyx_t_1, 54, __pyx_t_46);
  __Pyx_GIVEREF(__pyx_t_47);
  PyTuple_SET_ITEM(__pyx_t_1, 55, __pyx_t_47);
  __Pyx_INCREF(__pyx_tuple__74);
  __Pyx_GIVEREF(__pyx_tuple__74);
  PyTuple_SET_ITEM(__pyx_t_1, 56, __pyx_tuple__74);
  __Pyx_INCREF(__pyx_tuple__75);
  __Pyx_GIVEREF(__pyx_tuple__75);
  PyTuple_SET_ITEM(__pyx_t_1, 57, __pyx_tuple__75);
  __Pyx_GIVEREF(__pyx_t_48);
  PyTuple_SET_ITEM(__pyx_t_1, 58, __pyx_t_48);
  __Pyx_INCREF(__pyx_tuple__76);
  __Pyx_GIVEREF(__pyx_tuple__76);
  PyTuple_SET_ITEM(__pyx_t_1, 59, __pyx_tuple__76);
  __pyx_t_24 = 0;
  __pyx_t_23 = 0;
  __pyx_t_22 = 0;
  __pyx_t_21 = 0;
  __pyx_t_20 = 0;
  __pyx_t_19 = 0;
  __pyx_t_18 = 0;
  __pyx_t_17 = 0;
  __pyx_t_16 = 0;
  __pyx_t_15 = 0;
  __pyx_t_14 = 0;
  __pyx_t_13 = 0;
  __pyx_t_12 = 0;
  __pyx_t_11 = 0;
  __pyx_t_10 = 0;
  __pyx_t_9 = 0;
  __pyx_t_8 = 0;
  __pyx_t_7 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_25 = 0;
  __pyx_t_26 = 0;
  __pyx_t_27 = 0;
  __pyx_t_28 = 0;
  __pyx_t_29 = 0;
  __pyx_t_30 = 0;
  __pyx_t_31 = 0;
  __pyx_t_32 = 0;
  __pyx_t_33 = 0;
  __pyx_t_34 = 0;
  __pyx_t_35 = 0;
  __pyx_t_36 = 0;
  __pyx_t_37 = 0;
  __pyx_t_38 = 0;
  __pyx_t_39 = 0;
  __pyx_t_40 = 0;
  __pyx_t_41 = 0;
  __pyx_t_42 = 0;
  __pyx_t_43 = 0;
  __pyx_t_44 = 0;
  __pyx_t_45 = 0;
  __pyx_t_46 = 0;
  __pyx_t_47 = 0;
  __pyx_t_48 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PRODUCT_END, __pyx_t_1) < 0) __PYX_ERR(0, 1035, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1100
 * # 4884 bytes ingest_header Structure (section 4.2.16, page 40)
 * INGEST_HEADER = (
 *     ('structure_header', '12s'),        # 12 bytes: structure_header             # <<<<<<<<<<<<<<
 *     ('ingest_configuration', '480s'),   # 480 bytes: ingest_configuration
 *     ('task_configuration', '2612s'),    # 2612 bytes: task_configuration
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_INGEST_HEADER, __pyx_tuple__82) < 0) __PYX_ERR(0, 1099, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":1111
 * INGEST_CONFIGURATION = (
 *     ('filename', '80s'),
 *     ('number_files', SINT2),             # <<<<<<<<<<<<<<
 *     ('number_sweeps_completed', SINT2),
 *     ('total_size', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_48 = PyTuple_New(2); if (unlikely(!__pyx_t_48)) __PYX_ERR(0, 1111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_48);
  __Pyx_INCREF(__pyx_n_u_number_files);
  __Pyx_GIVEREF(__pyx_n_u_number_files);
  PyTuple_SET_ITEM(__pyx_t_48, 0, __pyx_n_u_number_files);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_48, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1112
 *     ('filename', '80s'),
 *     ('number_files', SINT2),
 *     ('number_sweeps_completed', SINT2),             # <<<<<<<<<<<<<<
 *     ('total_size', SINT4),
 *     ('volume_scan_start_time', '12s'),  # 12 bytes: ymds_time
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_47 = PyTuple_New(2); if (unlikely(!__pyx_t_47)) __PYX_ERR(0, 1112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_47);
  __Pyx_INCREF(__pyx_n_u_number_sweeps_completed);
  __Pyx_GIVEREF(__pyx_n_u_number_sweeps_completed);
  PyTuple_SET_ITEM(__pyx_t_47, 0, __pyx_n_u_number_sweeps_completed);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_47, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1113
 *     ('number_files', SINT2),
 *     ('number_sweeps_completed', SINT2),
 *     ('total_size', SINT4),             # <<<<<<<<<<<<<<
 *     ('volume_scan_start_time', '12s'),  # 12 bytes: ymds_time
 *     ('spare_0', '12s'),                 # 12 bytes
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1113, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_46 = PyTuple_New(2); if (unlikely(!__pyx_t_46)) __PYX_ERR(0, 1113, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_46);
  __Pyx_INCREF(__pyx_n_u_total_size);
  __Pyx_GIVEREF(__pyx_n_u_total_size);
  PyTuple_SET_ITEM(__pyx_t_46, 0, __pyx_n_u_total_size);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_46, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1116
 *     ('volume_scan_start_time', '12s'),  # 12 bytes: ymds_time
 *     ('spare_0', '12s'),                 # 12 bytes
 *     ('ray_header_bytes', SINT2),             # <<<<<<<<<<<<<<
 *     ('extended_ray_header_bytes', SINT2),
 *     ('number_task_config_table', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_45 = PyTuple_New(2); if (unlikely(!__pyx_t_45)) __PYX_ERR(0, 1116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_45);
  __Pyx_INCREF(__pyx_n_u_ray_header_bytes);
  __Pyx_GIVEREF(__pyx_n_u_ray_header_bytes);
  PyTuple_SET_ITEM(__pyx_t_45, 0, __pyx_n_u_ray_header_bytes);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_45, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1117
 *     ('spare_0', '12s'),                 # 12 bytes
 *     ('ray_header_bytes', SINT2),
 *     ('extended_ray_header_bytes', SINT2),             # <<<<<<<<<<<<<<
 *     ('number_task_config_table', SINT2),
 *     ('playback_version', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_44 = PyTuple_New(2); if (unlikely(!__pyx_t_44)) __PYX_ERR(0, 1117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_44);
  __Pyx_INCREF(__pyx_n_u_extended_ray_header_bytes);
  __Pyx_GIVEREF(__pyx_n_u_extended_ray_header_bytes);
  PyTuple_SET_ITEM(__pyx_t_44, 0, __pyx_n_u_extended_ray_header_bytes);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_44, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1118
 *     ('ray_header_bytes', SINT2),
 *     ('extended_ray_header_bytes', SINT2),
 *     ('number_task_config_table', SINT2),             # <<<<<<<<<<<<<<
 *     ('playback_version', SINT2),
 *     ('spare_1', '4s'),                  # 4 bytes
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_43 = PyTuple_New(2); if (unlikely(!__pyx_t_43)) __PYX_ERR(0, 1118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_43);
  __Pyx_INCREF(__pyx_n_u_number_task_config_table);
  __Pyx_GIVEREF(__pyx_n_u_number_task_config_table);
  PyTuple_SET_ITEM(__pyx_t_43, 0, __pyx_n_u_number_task_config_table);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_43, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1119
 *     ('extended_ray_header_bytes', SINT2),
 *     ('number_task_config_table', SINT2),
 *     ('playback_version', SINT2),             # <<<<<<<<<<<<<<
 *     ('spare_1', '4s'),                  # 4 bytes
 *     ('iris_version', '8s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1119, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_42 = PyTuple_New(2); if (unlikely(!__pyx_t_42)) __PYX_ERR(0, 1119, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_42);
  __Pyx_INCREF(__pyx_n_u_playback_version);
  __Pyx_GIVEREF(__pyx_n_u_playback_version);
  PyTuple_SET_ITEM(__pyx_t_42, 0, __pyx_n_u_playback_version);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_42, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1123
 *     ('iris_version', '8s'),
 *     ('hardware_site', '16s'),
 *     ('gmt_offset_minutes_local', SINT2),             # <<<<<<<<<<<<<<
 *     ('site_name', '16s'),
 *     ('gmt_offset_minutes_standard', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_41 = PyTuple_New(2); if (unlikely(!__pyx_t_41)) __PYX_ERR(0, 1123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_41);
  __Pyx_INCREF(__pyx_n_u_gmt_offset_minutes_local);
  __Pyx_GIVEREF(__pyx_n_u_gmt_offset_minutes_local);
  PyTuple_SET_ITEM(__pyx_t_41, 0, __pyx_n_u_gmt_offset_minutes_local);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_41, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1125
 *     ('gmt_offset_minutes_local', SINT2),
 *     ('site_name', '16s'),
 *     ('gmt_offset_minutes_standard', SINT2),             # <<<<<<<<<<<<<<
 *     ('latitude_radar', BIN4),
 *     ('longitude_radar', BIN4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_40 = PyTuple_New(2); if (unlikely(!__pyx_t_40)) __PYX_ERR(0, 1125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_40);
  __Pyx_INCREF(__pyx_n_u_gmt_offset_minutes_standard);
  __Pyx_GIVEREF(__pyx_n_u_gmt_offset_minutes_standard);
  PyTuple_SET_ITEM(__pyx_t_40, 0, __pyx_n_u_gmt_offset_minutes_standard);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_40, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1126
 *     ('site_name', '16s'),
 *     ('gmt_offset_minutes_standard', SINT2),
 *     ('latitude_radar', BIN4),             # <<<<<<<<<<<<<<
 *     ('longitude_radar', BIN4),
 *     ('height_site', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_BIN4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_39 = PyTuple_New(2); if (unlikely(!__pyx_t_39)) __PYX_ERR(0, 1126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_39);
  __Pyx_INCREF(__pyx_n_u_latitude_radar);
  __Pyx_GIVEREF(__pyx_n_u_latitude_radar);
  PyTuple_SET_ITEM(__pyx_t_39, 0, __pyx_n_u_latitude_radar);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_39, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1127
 *     ('gmt_offset_minutes_standard', SINT2),
 *     ('latitude_radar', BIN4),
 *     ('longitude_radar', BIN4),             # <<<<<<<<<<<<<<
 *     ('height_site', SINT2),
 *     ('height_radar', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_BIN4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_38 = PyTuple_New(2); if (unlikely(!__pyx_t_38)) __PYX_ERR(0, 1127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_38);
  __Pyx_INCREF(__pyx_n_u_longitude_radar);
  __Pyx_GIVEREF(__pyx_n_u_longitude_radar);
  PyTuple_SET_ITEM(__pyx_t_38, 0, __pyx_n_u_longitude_radar);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_38, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1128
 *     ('latitude_radar', BIN4),
 *     ('longitude_radar', BIN4),
 *     ('height_site', SINT2),             # <<<<<<<<<<<<<<
 *     ('height_radar', SINT2),
 *     ('resolution_rays', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_37 = PyTuple_New(2); if (unlikely(!__pyx_t_37)) __PYX_ERR(0, 1128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_37);
  __Pyx_INCREF(__pyx_n_u_height_site);
  __Pyx_GIVEREF(__pyx_n_u_height_site);
  PyTuple_SET_ITEM(__pyx_t_37, 0, __pyx_n_u_height_site);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_37, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1129
 *     ('longitude_radar', BIN4),
 *     ('height_site', SINT2),
 *     ('height_radar', SINT2),             # <<<<<<<<<<<<<<
 *     ('resolution_rays', UINT2),
 *     ('first_ray_index', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_36 = PyTuple_New(2); if (unlikely(!__pyx_t_36)) __PYX_ERR(0, 1129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_36);
  __Pyx_INCREF(__pyx_n_u_height_radar);
  __Pyx_GIVEREF(__pyx_n_u_height_radar);
  PyTuple_SET_ITEM(__pyx_t_36, 0, __pyx_n_u_height_radar);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_36, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1130
 *     ('height_site', SINT2),
 *     ('height_radar', SINT2),
 *     ('resolution_rays', UINT2),             # <<<<<<<<<<<<<<
 *     ('first_ray_index', UINT2),
 *     ('number_rays_sweep', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1130, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_35 = PyTuple_New(2); if (unlikely(!__pyx_t_35)) __PYX_ERR(0, 1130, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_35);
  __Pyx_INCREF(__pyx_n_u_resolution_rays);
  __Pyx_GIVEREF(__pyx_n_u_resolution_rays);
  PyTuple_SET_ITEM(__pyx_t_35, 0, __pyx_n_u_resolution_rays);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_35, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1131
 *     ('height_radar', SINT2),
 *     ('resolution_rays', UINT2),
 *     ('first_ray_index', UINT2),             # <<<<<<<<<<<<<<
 *     ('number_rays_sweep', UINT2),
 *     ('gparam_bytes', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1131, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_34 = PyTuple_New(2); if (unlikely(!__pyx_t_34)) __PYX_ERR(0, 1131, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_34);
  __Pyx_INCREF(__pyx_n_u_first_ray_index);
  __Pyx_GIVEREF(__pyx_n_u_first_ray_index);
  PyTuple_SET_ITEM(__pyx_t_34, 0, __pyx_n_u_first_ray_index);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_34, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1132
 *     ('resolution_rays', UINT2),
 *     ('first_ray_index', UINT2),
 *     ('number_rays_sweep', UINT2),             # <<<<<<<<<<<<<<
 *     ('gparam_bytes', SINT2),
 *     ('altitude_radar', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_33 = PyTuple_New(2); if (unlikely(!__pyx_t_33)) __PYX_ERR(0, 1132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_33);
  __Pyx_INCREF(__pyx_n_u_number_rays_sweep);
  __Pyx_GIVEREF(__pyx_n_u_number_rays_sweep);
  PyTuple_SET_ITEM(__pyx_t_33, 0, __pyx_n_u_number_rays_sweep);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_33, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1133
 *     ('first_ray_index', UINT2),
 *     ('number_rays_sweep', UINT2),
 *     ('gparam_bytes', SINT2),             # <<<<<<<<<<<<<<
 *     ('altitude_radar', SINT4),
 *     ('velocity_east', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1133, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_32 = PyTuple_New(2); if (unlikely(!__pyx_t_32)) __PYX_ERR(0, 1133, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_32);
  __Pyx_INCREF(__pyx_n_u_gparam_bytes);
  __Pyx_GIVEREF(__pyx_n_u_gparam_bytes);
  PyTuple_SET_ITEM(__pyx_t_32, 0, __pyx_n_u_gparam_bytes);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_32, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1134
 *     ('number_rays_sweep', UINT2),
 *     ('gparam_bytes', SINT2),
 *     ('altitude_radar', SINT4),             # <<<<<<<<<<<<<<
 *     ('velocity_east', SINT4),
 *     ('velocity_north', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1134, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_31 = PyTuple_New(2); if (unlikely(!__pyx_t_31)) __PYX_ERR(0, 1134, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_31);
  __Pyx_INCREF(__pyx_n_u_altitude_radar);
  __Pyx_GIVEREF(__pyx_n_u_altitude_radar);
  PyTuple_SET_ITEM(__pyx_t_31, 0, __pyx_n_u_altitude_radar);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_31, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1135
 *     ('gparam_bytes', SINT2),
 *     ('altitude_radar', SINT4),
 *     ('velocity_east', SINT4),             # <<<<<<<<<<<<<<
 *     ('velocity_north', SINT4),
 *     ('velocity_up', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1135, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_30 = PyTuple_New(2); if (unlikely(!__pyx_t_30)) __PYX_ERR(0, 1135, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_30);
  __Pyx_INCREF(__pyx_n_u_velocity_east);
  __Pyx_GIVEREF(__pyx_n_u_velocity_east);
  PyTuple_SET_ITEM(__pyx_t_30, 0, __pyx_n_u_velocity_east);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_30, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1136
 *     ('altitude_radar', SINT4),
 *     ('velocity_east', SINT4),
 *     ('velocity_north', SINT4),             # <<<<<<<<<<<<<<
 *     ('velocity_up', SINT4),
 *     ('antenna_offset_starboard', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1136, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_29 = PyTuple_New(2); if (unlikely(!__pyx_t_29)) __PYX_ERR(0, 1136, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_29);
  __Pyx_INCREF(__pyx_n_u_velocity_north);
  __Pyx_GIVEREF(__pyx_n_u_velocity_north);
  PyTuple_SET_ITEM(__pyx_t_29, 0, __pyx_n_u_velocity_north);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_29, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1137
 *     ('velocity_east', SINT4),
 *     ('velocity_north', SINT4),
 *     ('velocity_up', SINT4),             # <<<<<<<<<<<<<<
 *     ('antenna_offset_starboard', SINT4),
 *     ('antenna_offset_bow', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_28 = PyTuple_New(2); if (unlikely(!__pyx_t_28)) __PYX_ERR(0, 1137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_28);
  __Pyx_INCREF(__pyx_n_u_velocity_up);
  __Pyx_GIVEREF(__pyx_n_u_velocity_up);
  PyTuple_SET_ITEM(__pyx_t_28, 0, __pyx_n_u_velocity_up);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_28, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1138
 *     ('velocity_north', SINT4),
 *     ('velocity_up', SINT4),
 *     ('antenna_offset_starboard', SINT4),             # <<<<<<<<<<<<<<
 *     ('antenna_offset_bow', SINT4),
 *     ('antenna_offset_up', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1138, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_27 = PyTuple_New(2); if (unlikely(!__pyx_t_27)) __PYX_ERR(0, 1138, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_27);
  __Pyx_INCREF(__pyx_n_u_antenna_offset_starboard);
  __Pyx_GIVEREF(__pyx_n_u_antenna_offset_starboard);
  PyTuple_SET_ITEM(__pyx_t_27, 0, __pyx_n_u_antenna_offset_starboard);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_27, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1139
 *     ('velocity_up', SINT4),
 *     ('antenna_offset_starboard', SINT4),
 *     ('antenna_offset_bow', SINT4),             # <<<<<<<<<<<<<<
 *     ('antenna_offset_up', SINT4),
 *     ('fault_status', UINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1139, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_26 = PyTuple_New(2); if (unlikely(!__pyx_t_26)) __PYX_ERR(0, 1139, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_26);
  __Pyx_INCREF(__pyx_n_u_antenna_offset_bow);
  __Pyx_GIVEREF(__pyx_n_u_antenna_offset_bow);
  PyTuple_SET_ITEM(__pyx_t_26, 0, __pyx_n_u_antenna_offset_bow);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_26, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1140
 *     ('antenna_offset_starboard', SINT4),
 *     ('antenna_offset_bow', SINT4),
 *     ('antenna_offset_up', SINT4),             # <<<<<<<<<<<<<<
 *     ('fault_status', UINT4),
 *     ('melting_layer', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1140, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_25 = PyTuple_New(2); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 1140, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_25);
  __Pyx_INCREF(__pyx_n_u_antenna_offset_up);
  __Pyx_GIVEREF(__pyx_n_u_antenna_offset_up);
  PyTuple_SET_ITEM(__pyx_t_25, 0, __pyx_n_u_antenna_offset_up);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_25, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1141
 *     ('antenna_offset_bow', SINT4),
 *     ('antenna_offset_up', SINT4),
 *     ('fault_status', UINT4),             # <<<<<<<<<<<<<<
 *     ('melting_layer', SINT2),
 *     ('spare_2', '2s'),              # 2 bytes
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_n_u_fault_status);
  __Pyx_GIVEREF(__pyx_n_u_fault_status);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_n_u_fault_status);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1142
 *     ('antenna_offset_up', SINT4),
 *     ('fault_status', UINT4),
 *     ('melting_layer', SINT2),             # <<<<<<<<<<<<<<
 *     ('spare_2', '2s'),              # 2 bytes
 *     ('local_timezone', '8s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1142, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1142, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_n_u_melting_layer);
  __Pyx_GIVEREF(__pyx_n_u_melting_layer);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_n_u_melting_layer);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1145
 *     ('spare_2', '2s'),              # 2 bytes
 *     ('local_timezone', '8s'),
 *     ('flags', UINT4),             # <<<<<<<<<<<<<<
 *     ('configuration_name', '16s'),
 *     ('spare_3', '228s')
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1145, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1145, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_n_u_flags);
  __Pyx_GIVEREF(__pyx_n_u_flags);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_n_u_flags);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1110
 * # 480 bytes ingest_configuration Structure (section 4.2.14, page 38)
 * INGEST_CONFIGURATION = (
 *     ('filename', '80s'),             # <<<<<<<<<<<<<<
 *     ('number_files', SINT2),
 *     ('number_sweeps_completed', SINT2),
 */
  __pyx_t_1 = PyTuple_New(38); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_tuple__83);
  __Pyx_GIVEREF(__pyx_tuple__83);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_tuple__83);
  __Pyx_GIVEREF(__pyx_t_48);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_48);
  __Pyx_GIVEREF(__pyx_t_47);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_t_47);
  __Pyx_GIVEREF(__pyx_t_46);
  PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_t_46);
  __Pyx_INCREF(__pyx_tuple__84);
  __Pyx_GIVEREF(__pyx_tuple__84);
  PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_tuple__84);
  __Pyx_INCREF(__pyx_tuple__85);
  __Pyx_GIVEREF(__pyx_tuple__85);
  PyTuple_SET_ITEM(__pyx_t_1, 5, __pyx_tuple__85);
  __Pyx_GIVEREF(__pyx_t_45);
  PyTuple_SET_ITEM(__pyx_t_1, 6, __pyx_t_45);
  __Pyx_GIVEREF(__pyx_t_44);
  PyTuple_SET_ITEM(__pyx_t_1, 7, __pyx_t_44);
  __Pyx_GIVEREF(__pyx_t_43);
  PyTuple_SET_ITEM(__pyx_t_1, 8, __pyx_t_43);
  __Pyx_GIVEREF(__pyx_t_42);
  PyTuple_SET_ITEM(__pyx_t_1, 9, __pyx_t_42);
  __Pyx_INCREF(__pyx_tuple__86);
  __Pyx_GIVEREF(__pyx_tuple__86);
  PyTuple_SET_ITEM(__pyx_t_1, 10, __pyx_tuple__86);
  __Pyx_INCREF(__pyx_tuple__87);
  __Pyx_GIVEREF(__pyx_tuple__87);
  PyTuple_SET_ITEM(__pyx_t_1, 11, __pyx_tuple__87);
  __Pyx_INCREF(__pyx_tuple__88);
  __Pyx_GIVEREF(__pyx_tuple__88);
  PyTuple_SET_ITEM(__pyx_t_1, 12, __pyx_tuple__88);
  __Pyx_GIVEREF(__pyx_t_41);
  PyTuple_SET_ITEM(__pyx_t_1, 13, __pyx_t_41);
  __Pyx_INCREF(__pyx_tuple__64);
  __Pyx_GIVEREF(__pyx_tuple__64);
  PyTuple_SET_ITEM(__pyx_t_1, 14, __pyx_tuple__64);
  __Pyx_GIVEREF(__pyx_t_40);
  PyTuple_SET_ITEM(__pyx_t_1, 15, __pyx_t_40);
  __Pyx_GIVEREF(__pyx_t_39);
  PyTuple_SET_ITEM(__pyx_t_1, 16, __pyx_t_39);
  __Pyx_GIVEREF(__pyx_t_38);
  PyTuple_SET_ITEM(__pyx_t_1, 17, __pyx_t_38);
  __Pyx_GIVEREF(__pyx_t_37);
  PyTuple_SET_ITEM(__pyx_t_1, 18, __pyx_t_37);
  __Pyx_GIVEREF(__pyx_t_36);
  PyTuple_SET_ITEM(__pyx_t_1, 19, __pyx_t_36);
  __Pyx_GIVEREF(__pyx_t_35);
  PyTuple_SET_ITEM(__pyx_t_1, 20, __pyx_t_35);
  __Pyx_GIVEREF(__pyx_t_34);
  PyTuple_SET_ITEM(__pyx_t_1, 21, __pyx_t_34);
  __Pyx_GIVEREF(__pyx_t_33);
  PyTuple_SET_ITEM(__pyx_t_1, 22, __pyx_t_33);
  __Pyx_GIVEREF(__pyx_t_32);
  PyTuple_SET_ITEM(__pyx_t_1, 23, __pyx_t_32);
  __Pyx_GIVEREF(__pyx_t_31);
  PyTuple_SET_ITEM(__pyx_t_1, 24, __pyx_t_31);
  __Pyx_GIVEREF(__pyx_t_30);
  PyTuple_SET_ITEM(__pyx_t_1, 25, __pyx_t_30);
  __Pyx_GIVEREF(__pyx_t_29);
  PyTuple_SET_ITEM(__pyx_t_1, 26, __pyx_t_29);
  __Pyx_GIVEREF(__pyx_t_28);
  PyTuple_SET_ITEM(__pyx_t_1, 27, __pyx_t_28);
  __Pyx_GIVEREF(__pyx_t_27);
  PyTuple_SET_ITEM(__pyx_t_1, 28, __pyx_t_27);
  __Pyx_GIVEREF(__pyx_t_26);
  PyTuple_SET_ITEM(__pyx_t_1, 29, __pyx_t_26);
  __Pyx_GIVEREF(__pyx_t_25);
  PyTuple_SET_ITEM(__pyx_t_1, 30, __pyx_t_25);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_1, 31, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_1, 32, __pyx_t_5);
  __Pyx_INCREF(__pyx_tuple__89);
  __Pyx_GIVEREF(__pyx_tuple__89);
  PyTuple_SET_ITEM(__pyx_t_1, 33, __pyx_tuple__89);
  __Pyx_INCREF(__pyx_tuple__90);
  __Pyx_GIVEREF(__pyx_tuple__90);
  PyTuple_SET_ITEM(__pyx_t_1, 34, __pyx_tuple__90);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_1, 35, __pyx_t_4);
  __Pyx_INCREF(__pyx_tuple__91);
  __Pyx_GIVEREF(__pyx_tuple__91);
  PyTuple_SET_ITEM(__pyx_t_1, 36, __pyx_tuple__91);
  __Pyx_INCREF(__pyx_tuple__92);
  __Pyx_GIVEREF(__pyx_tuple__92);
  PyTuple_SET_ITEM(__pyx_t_1, 37, __pyx_tuple__92);
  __pyx_t_48 = 0;
  __pyx_t_47 = 0;
  __pyx_t_46 = 0;
  __pyx_t_45 = 0;
  __pyx_t_44 = 0;
  __pyx_t_43 = 0;
  __pyx_t_42 = 0;
  __pyx_t_41 = 0;
  __pyx_t_40 = 0;
  __pyx_t_39 = 0;
  __pyx_t_38 = 0;
  __pyx_t_37 = 0;
  __pyx_t_36 = 0;
  __pyx_t_35 = 0;
  __pyx_t_34 = 0;
  __pyx_t_33 = 0;
  __pyx_t_32 = 0;
  __pyx_t_31 = 0;
  __pyx_t_30 = 0;
  __pyx_t_29 = 0;
  __pyx_t_28 = 0;
  __pyx_t_27 = 0;
  __pyx_t_26 = 0;
  __pyx_t_25 = 0;
  __pyx_t_6 = 0;
  __pyx_t_5 = 0;
  __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_INGEST_CONFIGURATION, __pyx_t_1) < 0) __PYX_ERR(0, 1109, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1152
 * # 2612 bytes: task_configuration Structure (section 4.2.50, page 61)
 * TASK_CONFIGURATION = (
 *     ('structure_header', '12s'),    # 12 bytes: structure_header             # <<<<<<<<<<<<<<
 *     ('task_sched_info', '120s'),    # 120 bytes: task_sched_info
 *     ('task_dsp_info', '320s'),      # 320 bytes: task_dsp_info
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_TASK_CONFIGURATION, __pyx_tuple__101) < 0) __PYX_ERR(0, 1151, __pyx_L1_error)

  /* "pyart/io/_sigmetfile.pyx":1165
 * # 120 bytes: task_sched_info Structure (section 4.2.61, page 65)
 * TASK_SCHED_INFO = (
 *     ('start_time', SINT4),             # <<<<<<<<<<<<<<
 *     ('stop_time', SINT4),
 *     ('skip_time', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1165, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1165, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_n_u_start_time);
  __Pyx_GIVEREF(__pyx_n_u_start_time);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_n_u_start_time);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1166
 * TASK_SCHED_INFO = (
 *     ('start_time', SINT4),
 *     ('stop_time', SINT4),             # <<<<<<<<<<<<<<
 *     ('skip_time', SINT4),
 *     ('last_run_time', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_n_u_stop_time);
  __Pyx_GIVEREF(__pyx_n_u_stop_time);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_n_u_stop_time);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1167
 *     ('start_time', SINT4),
 *     ('stop_time', SINT4),
 *     ('skip_time', SINT4),             # <<<<<<<<<<<<<<
 *     ('last_run_time', SINT4),
 *     ('time_used_last_run', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_n_u_skip_time);
  __Pyx_GIVEREF(__pyx_n_u_skip_time);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_n_u_skip_time);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1168
 *     ('stop_time', SINT4),
 *     ('skip_time', SINT4),
 *     ('last_run_time', SINT4),             # <<<<<<<<<<<<<<
 *     ('time_used_last_run', SINT4),
 *     ('last_run_day', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1168, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_25 = PyTuple_New(2); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 1168, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_25);
  __Pyx_INCREF(__pyx_n_u_last_run_time);
  __Pyx_GIVEREF(__pyx_n_u_last_run_time);
  PyTuple_SET_ITEM(__pyx_t_25, 0, __pyx_n_u_last_run_time);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_25, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1169
 *     ('skip_time', SINT4),
 *     ('last_run_time', SINT4),
 *     ('time_used_last_run', SINT4),             # <<<<<<<<<<<<<<
 *     ('last_run_day', SINT4),
 *     ('flag', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1169, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_26 = PyTuple_New(2); if (unlikely(!__pyx_t_26)) __PYX_ERR(0, 1169, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_26);
  __Pyx_INCREF(__pyx_n_u_time_used_last_run);
  __Pyx_GIVEREF(__pyx_n_u_time_used_last_run);
  PyTuple_SET_ITEM(__pyx_t_26, 0, __pyx_n_u_time_used_last_run);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_26, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1170
 *     ('last_run_time', SINT4),
 *     ('time_used_last_run', SINT4),
 *     ('last_run_day', SINT4),             # <<<<<<<<<<<<<<
 *     ('flag', UINT2),
 *     ('spare_0', '94s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_27 = PyTuple_New(2); if (unlikely(!__pyx_t_27)) __PYX_ERR(0, 1170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_27);
  __Pyx_INCREF(__pyx_n_u_last_run_day);
  __Pyx_GIVEREF(__pyx_n_u_last_run_day);
  PyTuple_SET_ITEM(__pyx_t_27, 0, __pyx_n_u_last_run_day);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_27, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1171
 *     ('time_used_last_run', SINT4),
 *     ('last_run_day', SINT4),
 *     ('flag', UINT2),             # <<<<<<<<<<<<<<
 *     ('spare_0', '94s'),
 * )
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_28 = PyTuple_New(2); if (unlikely(!__pyx_t_28)) __PYX_ERR(0, 1171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_28);
  __Pyx_INCREF(__pyx_n_u_flag);
  __Pyx_GIVEREF(__pyx_n_u_flag);
  PyTuple_SET_ITEM(__pyx_t_28, 0, __pyx_n_u_flag);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_28, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1165
 * # 120 bytes: task_sched_info Structure (section 4.2.61, page 65)
 * TASK_SCHED_INFO = (
 *     ('start_time', SINT4),             # <<<<<<<<<<<<<<
 *     ('stop_time', SINT4),
 *     ('skip_time', SINT4),
 */
  __pyx_t_1 = PyTuple_New(8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1165, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_25);
  PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_t_25);
  __Pyx_GIVEREF(__pyx_t_26);
  PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_t_26);
  __Pyx_GIVEREF(__pyx_t_27);
  PyTuple_SET_ITEM(__pyx_t_1, 5, __pyx_t_27);
  __Pyx_GIVEREF(__pyx_t_28);
  PyTuple_SET_ITEM(__pyx_t_1, 6, __pyx_t_28);
  __Pyx_INCREF(__pyx_tuple__102);
  __Pyx_GIVEREF(__pyx_tuple__102);
  PyTuple_SET_ITEM(__pyx_t_1, 7, __pyx_tuple__102);
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_25 = 0;
  __pyx_t_26 = 0;
  __pyx_t_27 = 0;
  __pyx_t_28 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_TASK_SCHED_INFO, __pyx_t_1) < 0) __PYX_ERR(0, 1164, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1177
 * # 320 bytes: task_dsp_info Structure (section 4.2.51, page 61)
 * TASK_DSP_INFO = (
 *     ('major_mode', UINT2),             # <<<<<<<<<<<<<<
 *     ('dsp_type', UINT2),
 *     ('current_data_type_mask', '24s'),      # 24 bytes: dsp_data_mask
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_28 = PyTuple_New(2); if (unlikely(!__pyx_t_28)) __PYX_ERR(0, 1177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_28);
  __Pyx_INCREF(__pyx_n_u_major_mode);
  __Pyx_GIVEREF(__pyx_n_u_major_mode);
  PyTuple_SET_ITEM(__pyx_t_28, 0, __pyx_n_u_major_mode);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_28, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1178
 * TASK_DSP_INFO = (
 *     ('major_mode', UINT2),
 *     ('dsp_type', UINT2),             # <<<<<<<<<<<<<<
 *     ('current_data_type_mask', '24s'),      # 24 bytes: dsp_data_mask
 *     ('original_data_type_mask', '24s'),     # 24 bytes: dsp_data_mask
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1178, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_27 = PyTuple_New(2); if (unlikely(!__pyx_t_27)) __PYX_ERR(0, 1178, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_27);
  __Pyx_INCREF(__pyx_n_u_dsp_type);
  __Pyx_GIVEREF(__pyx_n_u_dsp_type);
  PyTuple_SET_ITEM(__pyx_t_27, 0, __pyx_n_u_dsp_type);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_27, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1183
 *     ('task_dsp_mode', '32s'),               # 32 bytes: task_dsp_mode_batch
 *     ('spare_0', '52s'),
 *     ('prf', SINT4),             # <<<<<<<<<<<<<<
 *     ('pulse_width', SINT4),
 *     ('multi_prf_flag', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1183, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_26 = PyTuple_New(2); if (unlikely(!__pyx_t_26)) __PYX_ERR(0, 1183, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_26);
  __Pyx_INCREF(__pyx_n_u_prf);
  __Pyx_GIVEREF(__pyx_n_u_prf);
  PyTuple_SET_ITEM(__pyx_t_26, 0, __pyx_n_u_prf);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_26, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1184
 *     ('spare_0', '52s'),
 *     ('prf', SINT4),
 *     ('pulse_width', SINT4),             # <<<<<<<<<<<<<<
 *     ('multi_prf_flag', UINT2),
 *     ('dual_prf_delay', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_25 = PyTuple_New(2); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 1184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_25);
  __Pyx_INCREF(__pyx_n_u_pulse_width);
  __Pyx_GIVEREF(__pyx_n_u_pulse_width);
  PyTuple_SET_ITEM(__pyx_t_25, 0, __pyx_n_u_pulse_width);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_25, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1185
 *     ('prf', SINT4),
 *     ('pulse_width', SINT4),
 *     ('multi_prf_flag', UINT2),             # <<<<<<<<<<<<<<
 *     ('dual_prf_delay', SINT2),
 *     ('agc_feedback_code', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_n_u_multi_prf_flag);
  __Pyx_GIVEREF(__pyx_n_u_multi_prf_flag);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_n_u_multi_prf_flag);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1186
 *     ('pulse_width', SINT4),
 *     ('multi_prf_flag', UINT2),
 *     ('dual_prf_delay', SINT2),             # <<<<<<<<<<<<<<
 *     ('agc_feedback_code', UINT2),
 *     ('sample_size', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1186, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1186, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_n_u_dual_prf_delay);
  __Pyx_GIVEREF(__pyx_n_u_dual_prf_delay);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_n_u_dual_prf_delay);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1187
 *     ('multi_prf_flag', UINT2),
 *     ('dual_prf_delay', SINT2),
 *     ('agc_feedback_code', UINT2),             # <<<<<<<<<<<<<<
 *     ('sample_size', SINT2),
 *     ('gain_control_flag', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_n_u_agc_feedback_code);
  __Pyx_GIVEREF(__pyx_n_u_agc_feedback_code);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_n_u_agc_feedback_code);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1188
 *     ('dual_prf_delay', SINT2),
 *     ('agc_feedback_code', UINT2),
 *     ('sample_size', SINT2),             # <<<<<<<<<<<<<<
 *     ('gain_control_flag', UINT2),
 *     ('clutter_filter_name', '12s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1188, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_29 = PyTuple_New(2); if (unlikely(!__pyx_t_29)) __PYX_ERR(0, 1188, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_29);
  __Pyx_INCREF(__pyx_n_u_sample_size);
  __Pyx_GIVEREF(__pyx_n_u_sample_size);
  PyTuple_SET_ITEM(__pyx_t_29, 0, __pyx_n_u_sample_size);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_29, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1189
 *     ('agc_feedback_code', UINT2),
 *     ('sample_size', SINT2),
 *     ('gain_control_flag', UINT2),             # <<<<<<<<<<<<<<
 *     ('clutter_filter_name', '12s'),
 *     ('linear_filter_first_bin', UINT1),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_30 = PyTuple_New(2); if (unlikely(!__pyx_t_30)) __PYX_ERR(0, 1189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_30);
  __Pyx_INCREF(__pyx_n_u_gain_control_flag);
  __Pyx_GIVEREF(__pyx_n_u_gain_control_flag);
  PyTuple_SET_ITEM(__pyx_t_30, 0, __pyx_n_u_gain_control_flag);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_30, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1191
 *     ('gain_control_flag', UINT2),
 *     ('clutter_filter_name', '12s'),
 *     ('linear_filter_first_bin', UINT1),             # <<<<<<<<<<<<<<
 *     ('log_filter_first_bin', UINT1),
 *     ('attenuation', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1191, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_31 = PyTuple_New(2); if (unlikely(!__pyx_t_31)) __PYX_ERR(0, 1191, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_31);
  __Pyx_INCREF(__pyx_n_u_linear_filter_first_bin);
  __Pyx_GIVEREF(__pyx_n_u_linear_filter_first_bin);
  PyTuple_SET_ITEM(__pyx_t_31, 0, __pyx_n_u_linear_filter_first_bin);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_31, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1192
 *     ('clutter_filter_name', '12s'),
 *     ('linear_filter_first_bin', UINT1),
 *     ('log_filter_first_bin', UINT1),             # <<<<<<<<<<<<<<
 *     ('attenuation', SINT2),
 *     ('gas_attenuation', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1192, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_32 = PyTuple_New(2); if (unlikely(!__pyx_t_32)) __PYX_ERR(0, 1192, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_32);
  __Pyx_INCREF(__pyx_n_u_log_filter_first_bin);
  __Pyx_GIVEREF(__pyx_n_u_log_filter_first_bin);
  PyTuple_SET_ITEM(__pyx_t_32, 0, __pyx_n_u_log_filter_first_bin);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_32, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1193
 *     ('linear_filter_first_bin', UINT1),
 *     ('log_filter_first_bin', UINT1),
 *     ('attenuation', SINT2),             # <<<<<<<<<<<<<<
 *     ('gas_attenuation', UINT2),
 *     ('cluttermap_flag', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_33 = PyTuple_New(2); if (unlikely(!__pyx_t_33)) __PYX_ERR(0, 1193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_33);
  __Pyx_INCREF(__pyx_n_u_attenuation);
  __Pyx_GIVEREF(__pyx_n_u_attenuation);
  PyTuple_SET_ITEM(__pyx_t_33, 0, __pyx_n_u_attenuation);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_33, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1194
 *     ('log_filter_first_bin', UINT1),
 *     ('attenuation', SINT2),
 *     ('gas_attenuation', UINT2),             # <<<<<<<<<<<<<<
 *     ('cluttermap_flag', UINT2),
 *     ('tranmitter_phase_sequence', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1194, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_34 = PyTuple_New(2); if (unlikely(!__pyx_t_34)) __PYX_ERR(0, 1194, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_34);
  __Pyx_INCREF(__pyx_n_u_gas_attenuation);
  __Pyx_GIVEREF(__pyx_n_u_gas_attenuation);
  PyTuple_SET_ITEM(__pyx_t_34, 0, __pyx_n_u_gas_attenuation);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_34, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1195
 *     ('attenuation', SINT2),
 *     ('gas_attenuation', UINT2),
 *     ('cluttermap_flag', UINT2),             # <<<<<<<<<<<<<<
 *     ('tranmitter_phase_sequence', UINT2),
 *     ('ray_header_mask', UINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_35 = PyTuple_New(2); if (unlikely(!__pyx_t_35)) __PYX_ERR(0, 1195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_35);
  __Pyx_INCREF(__pyx_n_u_cluttermap_flag);
  __Pyx_GIVEREF(__pyx_n_u_cluttermap_flag);
  PyTuple_SET_ITEM(__pyx_t_35, 0, __pyx_n_u_cluttermap_flag);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_35, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1196
 *     ('gas_attenuation', UINT2),
 *     ('cluttermap_flag', UINT2),
 *     ('tranmitter_phase_sequence', UINT2),             # <<<<<<<<<<<<<<
 *     ('ray_header_mask', UINT4),
 *     ('playback_flag', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1196, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_36 = PyTuple_New(2); if (unlikely(!__pyx_t_36)) __PYX_ERR(0, 1196, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_36);
  __Pyx_INCREF(__pyx_n_u_tranmitter_phase_sequence);
  __Pyx_GIVEREF(__pyx_n_u_tranmitter_phase_sequence);
  PyTuple_SET_ITEM(__pyx_t_36, 0, __pyx_n_u_tranmitter_phase_sequence);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_36, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1197
 *     ('cluttermap_flag', UINT2),
 *     ('tranmitter_phase_sequence', UINT2),
 *     ('ray_header_mask', UINT4),             # <<<<<<<<<<<<<<
 *     ('playback_flag', UINT2),
 *     ('spare_1', '2s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1197, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_37 = PyTuple_New(2); if (unlikely(!__pyx_t_37)) __PYX_ERR(0, 1197, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_37);
  __Pyx_INCREF(__pyx_n_u_ray_header_mask);
  __Pyx_GIVEREF(__pyx_n_u_ray_header_mask);
  PyTuple_SET_ITEM(__pyx_t_37, 0, __pyx_n_u_ray_header_mask);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_37, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1198
 *     ('tranmitter_phase_sequence', UINT2),
 *     ('ray_header_mask', UINT4),
 *     ('playback_flag', UINT2),             # <<<<<<<<<<<<<<
 *     ('spare_1', '2s'),
 *     ('custom_ray_header_name', '16s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_38 = PyTuple_New(2); if (unlikely(!__pyx_t_38)) __PYX_ERR(0, 1198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_38);
  __Pyx_INCREF(__pyx_n_u_playback_flag);
  __Pyx_GIVEREF(__pyx_n_u_playback_flag);
  PyTuple_SET_ITEM(__pyx_t_38, 0, __pyx_n_u_playback_flag);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_38, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1177
 * # 320 bytes: task_dsp_info Structure (section 4.2.51, page 61)
 * TASK_DSP_INFO = (
 *     ('major_mode', UINT2),             # <<<<<<<<<<<<<<
 *     ('dsp_type', UINT2),
 *     ('current_data_type_mask', '24s'),      # 24 bytes: dsp_data_mask
 */
  __pyx_t_1 = PyTuple_New(25); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_28);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_28);
  __Pyx_GIVEREF(__pyx_t_27);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_27);
  __Pyx_INCREF(__pyx_tuple__103);
  __Pyx_GIVEREF(__pyx_tuple__103);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_tuple__103);
  __Pyx_INCREF(__pyx_tuple__104);
  __Pyx_GIVEREF(__pyx_tuple__104);
  PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_tuple__104);
  __Pyx_INCREF(__pyx_tuple__105);
  __Pyx_GIVEREF(__pyx_tuple__105);
  PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_tuple__105);
  __Pyx_INCREF(__pyx_tuple__106);
  __Pyx_GIVEREF(__pyx_tuple__106);
  PyTuple_SET_ITEM(__pyx_t_1, 5, __pyx_tuple__106);
  __Pyx_GIVEREF(__pyx_t_26);
  PyTuple_SET_ITEM(__pyx_t_1, 6, __pyx_t_26);
  __Pyx_GIVEREF(__pyx_t_25);
  PyTuple_SET_ITEM(__pyx_t_1, 7, __pyx_t_25);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_1, 8, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_1, 9, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_1, 10, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_29);
  PyTuple_SET_ITEM(__pyx_t_1, 11, __pyx_t_29);
  __Pyx_GIVEREF(__pyx_t_30);
  PyTuple_SET_ITEM(__pyx_t_1, 12, __pyx_t_30);
  __Pyx_INCREF(__pyx_tuple__107);
  __Pyx_GIVEREF(__pyx_tuple__107);
  PyTuple_SET_ITEM(__pyx_t_1, 13, __pyx_tuple__107);
  __Pyx_GIVEREF(__pyx_t_31);
  PyTuple_SET_ITEM(__pyx_t_1, 14, __pyx_t_31);
  __Pyx_GIVEREF(__pyx_t_32);
  PyTuple_SET_ITEM(__pyx_t_1, 15, __pyx_t_32);
  __Pyx_GIVEREF(__pyx_t_33);
  PyTuple_SET_ITEM(__pyx_t_1, 16, __pyx_t_33);
  __Pyx_GIVEREF(__pyx_t_34);
  PyTuple_SET_ITEM(__pyx_t_1, 17, __pyx_t_34);
  __Pyx_GIVEREF(__pyx_t_35);
  PyTuple_SET_ITEM(__pyx_t_1, 18, __pyx_t_35);
  __Pyx_GIVEREF(__pyx_t_36);
  PyTuple_SET_ITEM(__pyx_t_1, 19, __pyx_t_36);
  __Pyx_GIVEREF(__pyx_t_37);
  PyTuple_SET_ITEM(__pyx_t_1, 20, __pyx_t_37);
  __Pyx_GIVEREF(__pyx_t_38);
  PyTuple_SET_ITEM(__pyx_t_1, 21, __pyx_t_38);
  __Pyx_INCREF(__pyx_tuple__108);
  __Pyx_GIVEREF(__pyx_tuple__108);
  PyTuple_SET_ITEM(__pyx_t_1, 22, __pyx_tuple__108);
  __Pyx_INCREF(__pyx_tuple__109);
  __Pyx_GIVEREF(__pyx_tuple__109);
  PyTuple_SET_ITEM(__pyx_t_1, 23, __pyx_tuple__109);
  __Pyx_INCREF(__pyx_tuple__110);
  __Pyx_GIVEREF(__pyx_tuple__110);
  PyTuple_SET_ITEM(__pyx_t_1, 24, __pyx_tuple__110);
  __pyx_t_28 = 0;
  __pyx_t_27 = 0;
  __pyx_t_26 = 0;
  __pyx_t_25 = 0;
  __pyx_t_6 = 0;
  __pyx_t_5 = 0;
  __pyx_t_4 = 0;
  __pyx_t_29 = 0;
  __pyx_t_30 = 0;
  __pyx_t_31 = 0;
  __pyx_t_32 = 0;
  __pyx_t_33 = 0;
  __pyx_t_34 = 0;
  __pyx_t_35 = 0;
  __pyx_t_36 = 0;
  __pyx_t_37 = 0;
  __pyx_t_38 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_TASK_DSP_INFO, __pyx_t_1) < 0) __PYX_ERR(0, 1176, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1206
 * # 24 bytes: dsp_data_mask Structure (section 4.2.7, page 36)
 * DSP_DATA_MASK = (
 *     ('mask_word_0', UINT4),             # <<<<<<<<<<<<<<
 *     ('extended_header_type', UINT4),
 *     ('mask_word_1', UINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_38 = PyTuple_New(2); if (unlikely(!__pyx_t_38)) __PYX_ERR(0, 1206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_38);
  __Pyx_INCREF(__pyx_n_u_mask_word_0);
  __Pyx_GIVEREF(__pyx_n_u_mask_word_0);
  PyTuple_SET_ITEM(__pyx_t_38, 0, __pyx_n_u_mask_word_0);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_38, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1207
 * DSP_DATA_MASK = (
 *     ('mask_word_0', UINT4),
 *     ('extended_header_type', UINT4),             # <<<<<<<<<<<<<<
 *     ('mask_word_1', UINT4),
 *     ('mask_word_2', UINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1207, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_37 = PyTuple_New(2); if (unlikely(!__pyx_t_37)) __PYX_ERR(0, 1207, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_37);
  __Pyx_INCREF(__pyx_n_u_extended_header_type);
  __Pyx_GIVEREF(__pyx_n_u_extended_header_type);
  PyTuple_SET_ITEM(__pyx_t_37, 0, __pyx_n_u_extended_header_type);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_37, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1208
 *     ('mask_word_0', UINT4),
 *     ('extended_header_type', UINT4),
 *     ('mask_word_1', UINT4),             # <<<<<<<<<<<<<<
 *     ('mask_word_2', UINT4),
 *     ('mask_word_3', UINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1208, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_36 = PyTuple_New(2); if (unlikely(!__pyx_t_36)) __PYX_ERR(0, 1208, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_36);
  __Pyx_INCREF(__pyx_n_u_mask_word_1);
  __Pyx_GIVEREF(__pyx_n_u_mask_word_1);
  PyTuple_SET_ITEM(__pyx_t_36, 0, __pyx_n_u_mask_word_1);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_36, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1209
 *     ('extended_header_type', UINT4),
 *     ('mask_word_1', UINT4),
 *     ('mask_word_2', UINT4),             # <<<<<<<<<<<<<<
 *     ('mask_word_3', UINT4),
 *     ('mask_word_4', UINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_35 = PyTuple_New(2); if (unlikely(!__pyx_t_35)) __PYX_ERR(0, 1209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_35);
  __Pyx_INCREF(__pyx_n_u_mask_word_2);
  __Pyx_GIVEREF(__pyx_n_u_mask_word_2);
  PyTuple_SET_ITEM(__pyx_t_35, 0, __pyx_n_u_mask_word_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_35, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1210
 *     ('mask_word_1', UINT4),
 *     ('mask_word_2', UINT4),
 *     ('mask_word_3', UINT4),             # <<<<<<<<<<<<<<
 *     ('mask_word_4', UINT4),
 * )
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_34 = PyTuple_New(2); if (unlikely(!__pyx_t_34)) __PYX_ERR(0, 1210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_34);
  __Pyx_INCREF(__pyx_n_u_mask_word_3);
  __Pyx_GIVEREF(__pyx_n_u_mask_word_3);
  PyTuple_SET_ITEM(__pyx_t_34, 0, __pyx_n_u_mask_word_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_34, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1211
 *     ('mask_word_2', UINT4),
 *     ('mask_word_3', UINT4),
 *     ('mask_word_4', UINT4),             # <<<<<<<<<<<<<<
 * )
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_33 = PyTuple_New(2); if (unlikely(!__pyx_t_33)) __PYX_ERR(0, 1211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_33);
  __Pyx_INCREF(__pyx_n_u_mask_word_4);
  __Pyx_GIVEREF(__pyx_n_u_mask_word_4);
  PyTuple_SET_ITEM(__pyx_t_33, 0, __pyx_n_u_mask_word_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_33, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1206
 * # 24 bytes: dsp_data_mask Structure (section 4.2.7, page 36)
 * DSP_DATA_MASK = (
 *     ('mask_word_0', UINT4),             # <<<<<<<<<<<<<<
 *     ('extended_header_type', UINT4),
 *     ('mask_word_1', UINT4),
 */
  __pyx_t_1 = PyTuple_New(6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_38);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_38);
  __Pyx_GIVEREF(__pyx_t_37);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_37);
  __Pyx_GIVEREF(__pyx_t_36);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_t_36);
  __Pyx_GIVEREF(__pyx_t_35);
  PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_t_35);
  __Pyx_GIVEREF(__pyx_t_34);
  PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_t_34);
  __Pyx_GIVEREF(__pyx_t_33);
  PyTuple_SET_ITEM(__pyx_t_1, 5, __pyx_t_33);
  __pyx_t_38 = 0;
  __pyx_t_37 = 0;
  __pyx_t_36 = 0;
  __pyx_t_35 = 0;
  __pyx_t_34 = 0;
  __pyx_t_33 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DSP_DATA_MASK, __pyx_t_1) < 0) __PYX_ERR(0, 1205, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1216
 * # 32 bytes: task_dsp_mode_batch (section 4.2.52, page 62)
 * TASK_DSP_MODE_BATCH = (
 *     ('low_prf_hz', UINT2),             # <<<<<<<<<<<<<<
 *     ('low_prf_factional', UINT2),
 *     ('low_prf_sample_size', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1216, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_33 = PyTuple_New(2); if (unlikely(!__pyx_t_33)) __PYX_ERR(0, 1216, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_33);
  __Pyx_INCREF(__pyx_n_u_low_prf_hz);
  __Pyx_GIVEREF(__pyx_n_u_low_prf_hz);
  PyTuple_SET_ITEM(__pyx_t_33, 0, __pyx_n_u_low_prf_hz);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_33, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1217
 * TASK_DSP_MODE_BATCH = (
 *     ('low_prf_hz', UINT2),
 *     ('low_prf_factional', UINT2),             # <<<<<<<<<<<<<<
 *     ('low_prf_sample_size', SINT2),
 *     ('low_prf_range_averaging', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1217, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_34 = PyTuple_New(2); if (unlikely(!__pyx_t_34)) __PYX_ERR(0, 1217, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_34);
  __Pyx_INCREF(__pyx_n_u_low_prf_factional);
  __Pyx_GIVEREF(__pyx_n_u_low_prf_factional);
  PyTuple_SET_ITEM(__pyx_t_34, 0, __pyx_n_u_low_prf_factional);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_34, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1218
 *     ('low_prf_hz', UINT2),
 *     ('low_prf_factional', UINT2),
 *     ('low_prf_sample_size', SINT2),             # <<<<<<<<<<<<<<
 *     ('low_prf_range_averaging', SINT2),
 *     ('reflectivity_unfolding_threshold', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1218, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_35 = PyTuple_New(2); if (unlikely(!__pyx_t_35)) __PYX_ERR(0, 1218, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_35);
  __Pyx_INCREF(__pyx_n_u_low_prf_sample_size);
  __Pyx_GIVEREF(__pyx_n_u_low_prf_sample_size);
  PyTuple_SET_ITEM(__pyx_t_35, 0, __pyx_n_u_low_prf_sample_size);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_35, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1219
 *     ('low_prf_factional', UINT2),
 *     ('low_prf_sample_size', SINT2),
 *     ('low_prf_range_averaging', SINT2),             # <<<<<<<<<<<<<<
 *     ('reflectivity_unfolding_threshold', SINT2),
 *     ('velocity_unfolding_threshold', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_36 = PyTuple_New(2); if (unlikely(!__pyx_t_36)) __PYX_ERR(0, 1219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_36);
  __Pyx_INCREF(__pyx_n_u_low_prf_range_averaging);
  __Pyx_GIVEREF(__pyx_n_u_low_prf_range_averaging);
  PyTuple_SET_ITEM(__pyx_t_36, 0, __pyx_n_u_low_prf_range_averaging);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_36, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1220
 *     ('low_prf_sample_size', SINT2),
 *     ('low_prf_range_averaging', SINT2),
 *     ('reflectivity_unfolding_threshold', SINT2),             # <<<<<<<<<<<<<<
 *     ('velocity_unfolding_threshold', SINT2),
 *     ('width_unfolding_threshold', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1220, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_37 = PyTuple_New(2); if (unlikely(!__pyx_t_37)) __PYX_ERR(0, 1220, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_37);
  __Pyx_INCREF(__pyx_n_u_reflectivity_unfolding_threshold);
  __Pyx_GIVEREF(__pyx_n_u_reflectivity_unfolding_threshold);
  PyTuple_SET_ITEM(__pyx_t_37, 0, __pyx_n_u_reflectivity_unfolding_threshold);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_37, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1221
 *     ('low_prf_range_averaging', SINT2),
 *     ('reflectivity_unfolding_threshold', SINT2),
 *     ('velocity_unfolding_threshold', SINT2),             # <<<<<<<<<<<<<<
 *     ('width_unfolding_threshold', SINT2),
 *     ('spare_0', '18s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1221, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_38 = PyTuple_New(2); if (unlikely(!__pyx_t_38)) __PYX_ERR(0, 1221, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_38);
  __Pyx_INCREF(__pyx_n_u_velocity_unfolding_threshold);
  __Pyx_GIVEREF(__pyx_n_u_velocity_unfolding_threshold);
  PyTuple_SET_ITEM(__pyx_t_38, 0, __pyx_n_u_velocity_unfolding_threshold);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_38, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1222
 *     ('reflectivity_unfolding_threshold', SINT2),
 *     ('velocity_unfolding_threshold', SINT2),
 *     ('width_unfolding_threshold', SINT2),             # <<<<<<<<<<<<<<
 *     ('spare_0', '18s'),
 * )
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1222, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_32 = PyTuple_New(2); if (unlikely(!__pyx_t_32)) __PYX_ERR(0, 1222, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_32);
  __Pyx_INCREF(__pyx_n_u_width_unfolding_threshold);
  __Pyx_GIVEREF(__pyx_n_u_width_unfolding_threshold);
  PyTuple_SET_ITEM(__pyx_t_32, 0, __pyx_n_u_width_unfolding_threshold);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_32, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1216
 * # 32 bytes: task_dsp_mode_batch (section 4.2.52, page 62)
 * TASK_DSP_MODE_BATCH = (
 *     ('low_prf_hz', UINT2),             # <<<<<<<<<<<<<<
 *     ('low_prf_factional', UINT2),
 *     ('low_prf_sample_size', SINT2),
 */
  __pyx_t_1 = PyTuple_New(8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1216, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_33);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_33);
  __Pyx_GIVEREF(__pyx_t_34);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_34);
  __Pyx_GIVEREF(__pyx_t_35);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_t_35);
  __Pyx_GIVEREF(__pyx_t_36);
  PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_t_36);
  __Pyx_GIVEREF(__pyx_t_37);
  PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_t_37);
  __Pyx_GIVEREF(__pyx_t_38);
  PyTuple_SET_ITEM(__pyx_t_1, 5, __pyx_t_38);
  __Pyx_GIVEREF(__pyx_t_32);
  PyTuple_SET_ITEM(__pyx_t_1, 6, __pyx_t_32);
  __Pyx_INCREF(__pyx_tuple__111);
  __Pyx_GIVEREF(__pyx_tuple__111);
  PyTuple_SET_ITEM(__pyx_t_1, 7, __pyx_tuple__111);
  __pyx_t_33 = 0;
  __pyx_t_34 = 0;
  __pyx_t_35 = 0;
  __pyx_t_36 = 0;
  __pyx_t_37 = 0;
  __pyx_t_38 = 0;
  __pyx_t_32 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_TASK_DSP_MODE_BATCH, __pyx_t_1) < 0) __PYX_ERR(0, 1215, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1228
 * # 320 bytes: task_calib_info Structure (section 4.2.49, page 59)
 * TASK_CALIB_INFO = (
 *     ('reflectivity_slope', SINT2),             # <<<<<<<<<<<<<<
 *     ('reflectivity_noise_threshold', SINT2),
 *     ('clutter_correction_threshold', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1228, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_32 = PyTuple_New(2); if (unlikely(!__pyx_t_32)) __PYX_ERR(0, 1228, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_32);
  __Pyx_INCREF(__pyx_n_u_reflectivity_slope);
  __Pyx_GIVEREF(__pyx_n_u_reflectivity_slope);
  PyTuple_SET_ITEM(__pyx_t_32, 0, __pyx_n_u_reflectivity_slope);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_32, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1229
 * TASK_CALIB_INFO = (
 *     ('reflectivity_slope', SINT2),
 *     ('reflectivity_noise_threshold', SINT2),             # <<<<<<<<<<<<<<
 *     ('clutter_correction_threshold', SINT2),
 *     ('sqi_threshold', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1229, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_38 = PyTuple_New(2); if (unlikely(!__pyx_t_38)) __PYX_ERR(0, 1229, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_38);
  __Pyx_INCREF(__pyx_n_u_reflectivity_noise_threshold);
  __Pyx_GIVEREF(__pyx_n_u_reflectivity_noise_threshold);
  PyTuple_SET_ITEM(__pyx_t_38, 0, __pyx_n_u_reflectivity_noise_threshold);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_38, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1230
 *     ('reflectivity_slope', SINT2),
 *     ('reflectivity_noise_threshold', SINT2),
 *     ('clutter_correction_threshold', SINT2),             # <<<<<<<<<<<<<<
 *     ('sqi_threshold', SINT2),
 *     ('power_threshold', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_37 = PyTuple_New(2); if (unlikely(!__pyx_t_37)) __PYX_ERR(0, 1230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_37);
  __Pyx_INCREF(__pyx_n_u_clutter_correction_threshold);
  __Pyx_GIVEREF(__pyx_n_u_clutter_correction_threshold);
  PyTuple_SET_ITEM(__pyx_t_37, 0, __pyx_n_u_clutter_correction_threshold);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_37, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1231
 *     ('reflectivity_noise_threshold', SINT2),
 *     ('clutter_correction_threshold', SINT2),
 *     ('sqi_threshold', SINT2),             # <<<<<<<<<<<<<<
 *     ('power_threshold', SINT2),
 *     ('spare_0', '8s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_36 = PyTuple_New(2); if (unlikely(!__pyx_t_36)) __PYX_ERR(0, 1231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_36);
  __Pyx_INCREF(__pyx_n_u_sqi_threshold);
  __Pyx_GIVEREF(__pyx_n_u_sqi_threshold);
  PyTuple_SET_ITEM(__pyx_t_36, 0, __pyx_n_u_sqi_threshold);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_36, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1232
 *     ('clutter_correction_threshold', SINT2),
 *     ('sqi_threshold', SINT2),
 *     ('power_threshold', SINT2),             # <<<<<<<<<<<<<<
 *     ('spare_0', '8s'),
 *     ('reflectivity_calibration', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1232, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_35 = PyTuple_New(2); if (unlikely(!__pyx_t_35)) __PYX_ERR(0, 1232, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_35);
  __Pyx_INCREF(__pyx_n_u_power_threshold);
  __Pyx_GIVEREF(__pyx_n_u_power_threshold);
  PyTuple_SET_ITEM(__pyx_t_35, 0, __pyx_n_u_power_threshold);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_35, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1234
 *     ('power_threshold', SINT2),
 *     ('spare_0', '8s'),
 *     ('reflectivity_calibration', SINT2),             # <<<<<<<<<<<<<<
 *     ('uncorrected_reflectivity_threshold_flags', UINT2),
 *     ('corrected_reflectivity_threshold_flags', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_34 = PyTuple_New(2); if (unlikely(!__pyx_t_34)) __PYX_ERR(0, 1234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_34);
  __Pyx_INCREF(__pyx_n_u_reflectivity_calibration);
  __Pyx_GIVEREF(__pyx_n_u_reflectivity_calibration);
  PyTuple_SET_ITEM(__pyx_t_34, 0, __pyx_n_u_reflectivity_calibration);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_34, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1235
 *     ('spare_0', '8s'),
 *     ('reflectivity_calibration', SINT2),
 *     ('uncorrected_reflectivity_threshold_flags', UINT2),             # <<<<<<<<<<<<<<
 *     ('corrected_reflectivity_threshold_flags', UINT2),
 *     ('velocity_threshold_flags', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1235, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_33 = PyTuple_New(2); if (unlikely(!__pyx_t_33)) __PYX_ERR(0, 1235, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_33);
  __Pyx_INCREF(__pyx_n_u_uncorrected_reflectivity_thresho);
  __Pyx_GIVEREF(__pyx_n_u_uncorrected_reflectivity_thresho);
  PyTuple_SET_ITEM(__pyx_t_33, 0, __pyx_n_u_uncorrected_reflectivity_thresho);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_33, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1236
 *     ('reflectivity_calibration', SINT2),
 *     ('uncorrected_reflectivity_threshold_flags', UINT2),
 *     ('corrected_reflectivity_threshold_flags', UINT2),             # <<<<<<<<<<<<<<
 *     ('velocity_threshold_flags', UINT2),
 *     ('width_threshold_flags', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1236, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_31 = PyTuple_New(2); if (unlikely(!__pyx_t_31)) __PYX_ERR(0, 1236, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_31);
  __Pyx_INCREF(__pyx_n_u_corrected_reflectivity_threshold);
  __Pyx_GIVEREF(__pyx_n_u_corrected_reflectivity_threshold);
  PyTuple_SET_ITEM(__pyx_t_31, 0, __pyx_n_u_corrected_reflectivity_threshold);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_31, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1237
 *     ('uncorrected_reflectivity_threshold_flags', UINT2),
 *     ('corrected_reflectivity_threshold_flags', UINT2),
 *     ('velocity_threshold_flags', UINT2),             # <<<<<<<<<<<<<<
 *     ('width_threshold_flags', UINT2),
 *     ('zdr_threshold_flags', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_30 = PyTuple_New(2); if (unlikely(!__pyx_t_30)) __PYX_ERR(0, 1237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_30);
  __Pyx_INCREF(__pyx_n_u_velocity_threshold_flags);
  __Pyx_GIVEREF(__pyx_n_u_velocity_threshold_flags);
  PyTuple_SET_ITEM(__pyx_t_30, 0, __pyx_n_u_velocity_threshold_flags);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_30, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1238
 *     ('corrected_reflectivity_threshold_flags', UINT2),
 *     ('velocity_threshold_flags', UINT2),
 *     ('width_threshold_flags', UINT2),             # <<<<<<<<<<<<<<
 *     ('zdr_threshold_flags', UINT2),
 *     ('spare_1', '6s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1238, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_29 = PyTuple_New(2); if (unlikely(!__pyx_t_29)) __PYX_ERR(0, 1238, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_29);
  __Pyx_INCREF(__pyx_n_u_width_threshold_flags);
  __Pyx_GIVEREF(__pyx_n_u_width_threshold_flags);
  PyTuple_SET_ITEM(__pyx_t_29, 0, __pyx_n_u_width_threshold_flags);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_29, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1239
 *     ('velocity_threshold_flags', UINT2),
 *     ('width_threshold_flags', UINT2),
 *     ('zdr_threshold_flags', UINT2),             # <<<<<<<<<<<<<<
 *     ('spare_1', '6s'),
 *     ('flags', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1239, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1239, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_n_u_zdr_threshold_flags);
  __Pyx_GIVEREF(__pyx_n_u_zdr_threshold_flags);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_n_u_zdr_threshold_flags);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1241
 *     ('zdr_threshold_flags', UINT2),
 *     ('spare_1', '6s'),
 *     ('flags', UINT2),             # <<<<<<<<<<<<<<
 *     ('spare_2', '2s'),
 *     ('ldr_bias', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_n_u_flags);
  __Pyx_GIVEREF(__pyx_n_u_flags);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_n_u_flags);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1243
 *     ('flags', UINT2),
 *     ('spare_2', '2s'),
 *     ('ldr_bias', SINT2),             # <<<<<<<<<<<<<<
 *     ('zdr_bias', SINT2),
 *     ('nexrad_clutter_threshold', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1243, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1243, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_n_u_ldr_bias);
  __Pyx_GIVEREF(__pyx_n_u_ldr_bias);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_n_u_ldr_bias);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1244
 *     ('spare_2', '2s'),
 *     ('ldr_bias', SINT2),
 *     ('zdr_bias', SINT2),             # <<<<<<<<<<<<<<
 *     ('nexrad_clutter_threshold', SINT2),
 *     ('nexrad_clutter_skip', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_25 = PyTuple_New(2); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 1244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_25);
  __Pyx_INCREF(__pyx_n_u_zdr_bias);
  __Pyx_GIVEREF(__pyx_n_u_zdr_bias);
  PyTuple_SET_ITEM(__pyx_t_25, 0, __pyx_n_u_zdr_bias);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_25, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1245
 *     ('ldr_bias', SINT2),
 *     ('zdr_bias', SINT2),
 *     ('nexrad_clutter_threshold', SINT2),             # <<<<<<<<<<<<<<
 *     ('nexrad_clutter_skip', UINT2),
 *     ('horizontal_i0_calibration', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1245, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_26 = PyTuple_New(2); if (unlikely(!__pyx_t_26)) __PYX_ERR(0, 1245, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_26);
  __Pyx_INCREF(__pyx_n_u_nexrad_clutter_threshold);
  __Pyx_GIVEREF(__pyx_n_u_nexrad_clutter_threshold);
  PyTuple_SET_ITEM(__pyx_t_26, 0, __pyx_n_u_nexrad_clutter_threshold);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_26, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1246
 *     ('zdr_bias', SINT2),
 *     ('nexrad_clutter_threshold', SINT2),
 *     ('nexrad_clutter_skip', UINT2),             # <<<<<<<<<<<<<<
 *     ('horizontal_i0_calibration', SINT2),
 *     ('vertical_i0_calibration', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_27 = PyTuple_New(2); if (unlikely(!__pyx_t_27)) __PYX_ERR(0, 1246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_27);
  __Pyx_INCREF(__pyx_n_u_nexrad_clutter_skip);
  __Pyx_GIVEREF(__pyx_n_u_nexrad_clutter_skip);
  PyTuple_SET_ITEM(__pyx_t_27, 0, __pyx_n_u_nexrad_clutter_skip);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_27, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1247
 *     ('nexrad_clutter_threshold', SINT2),
 *     ('nexrad_clutter_skip', UINT2),
 *     ('horizontal_i0_calibration', SINT2),             # <<<<<<<<<<<<<<
 *     ('vertical_i0_calibration', SINT2),
 *     ('horizontal_noise_calibration', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_28 = PyTuple_New(2); if (unlikely(!__pyx_t_28)) __PYX_ERR(0, 1247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_28);
  __Pyx_INCREF(__pyx_n_u_horizontal_i0_calibration);
  __Pyx_GIVEREF(__pyx_n_u_horizontal_i0_calibration);
  PyTuple_SET_ITEM(__pyx_t_28, 0, __pyx_n_u_horizontal_i0_calibration);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_28, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1248
 *     ('nexrad_clutter_skip', UINT2),
 *     ('horizontal_i0_calibration', SINT2),
 *     ('vertical_i0_calibration', SINT2),             # <<<<<<<<<<<<<<
 *     ('horizontal_noise_calibration', SINT2),
 *     ('vertical_noise_calibration', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_39 = PyTuple_New(2); if (unlikely(!__pyx_t_39)) __PYX_ERR(0, 1248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_39);
  __Pyx_INCREF(__pyx_n_u_vertical_i0_calibration);
  __Pyx_GIVEREF(__pyx_n_u_vertical_i0_calibration);
  PyTuple_SET_ITEM(__pyx_t_39, 0, __pyx_n_u_vertical_i0_calibration);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_39, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1249
 *     ('horizontal_i0_calibration', SINT2),
 *     ('vertical_i0_calibration', SINT2),
 *     ('horizontal_noise_calibration', SINT2),             # <<<<<<<<<<<<<<
 *     ('vertical_noise_calibration', SINT2),
 *     ('horizontal_radar_constant', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_40 = PyTuple_New(2); if (unlikely(!__pyx_t_40)) __PYX_ERR(0, 1249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_40);
  __Pyx_INCREF(__pyx_n_u_horizontal_noise_calibration);
  __Pyx_GIVEREF(__pyx_n_u_horizontal_noise_calibration);
  PyTuple_SET_ITEM(__pyx_t_40, 0, __pyx_n_u_horizontal_noise_calibration);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_40, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1250
 *     ('vertical_i0_calibration', SINT2),
 *     ('horizontal_noise_calibration', SINT2),
 *     ('vertical_noise_calibration', SINT2),             # <<<<<<<<<<<<<<
 *     ('horizontal_radar_constant', SINT2),
 *     ('vertical_radar_constant', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_41 = PyTuple_New(2); if (unlikely(!__pyx_t_41)) __PYX_ERR(0, 1250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_41);
  __Pyx_INCREF(__pyx_n_u_vertical_noise_calibration);
  __Pyx_GIVEREF(__pyx_n_u_vertical_noise_calibration);
  PyTuple_SET_ITEM(__pyx_t_41, 0, __pyx_n_u_vertical_noise_calibration);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_41, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1251
 *     ('horizontal_noise_calibration', SINT2),
 *     ('vertical_noise_calibration', SINT2),
 *     ('horizontal_radar_constant', SINT2),             # <<<<<<<<<<<<<<
 *     ('vertical_radar_constant', SINT2),
 *     ('reciever_bandwidth', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_42 = PyTuple_New(2); if (unlikely(!__pyx_t_42)) __PYX_ERR(0, 1251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_42);
  __Pyx_INCREF(__pyx_n_u_horizontal_radar_constant);
  __Pyx_GIVEREF(__pyx_n_u_horizontal_radar_constant);
  PyTuple_SET_ITEM(__pyx_t_42, 0, __pyx_n_u_horizontal_radar_constant);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_42, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1252
 *     ('vertical_noise_calibration', SINT2),
 *     ('horizontal_radar_constant', SINT2),
 *     ('vertical_radar_constant', SINT2),             # <<<<<<<<<<<<<<
 *     ('reciever_bandwidth', UINT2),
 *     ('flags2', UINT16_T),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1252, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_43 = PyTuple_New(2); if (unlikely(!__pyx_t_43)) __PYX_ERR(0, 1252, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_43);
  __Pyx_INCREF(__pyx_n_u_vertical_radar_constant);
  __Pyx_GIVEREF(__pyx_n_u_vertical_radar_constant);
  PyTuple_SET_ITEM(__pyx_t_43, 0, __pyx_n_u_vertical_radar_constant);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_43, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1253
 *     ('horizontal_radar_constant', SINT2),
 *     ('vertical_radar_constant', SINT2),
 *     ('reciever_bandwidth', UINT2),             # <<<<<<<<<<<<<<
 *     ('flags2', UINT16_T),
 *     ('spare_3', '256s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1253, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_44 = PyTuple_New(2); if (unlikely(!__pyx_t_44)) __PYX_ERR(0, 1253, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_44);
  __Pyx_INCREF(__pyx_n_u_reciever_bandwidth);
  __Pyx_GIVEREF(__pyx_n_u_reciever_bandwidth);
  PyTuple_SET_ITEM(__pyx_t_44, 0, __pyx_n_u_reciever_bandwidth);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_44, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1254
 *     ('vertical_radar_constant', SINT2),
 *     ('reciever_bandwidth', UINT2),
 *     ('flags2', UINT16_T),             # <<<<<<<<<<<<<<
 *     ('spare_3', '256s'),
 * )
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT16_T); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1254, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_45 = PyTuple_New(2); if (unlikely(!__pyx_t_45)) __PYX_ERR(0, 1254, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_45);
  __Pyx_INCREF(__pyx_n_u_flags2);
  __Pyx_GIVEREF(__pyx_n_u_flags2);
  PyTuple_SET_ITEM(__pyx_t_45, 0, __pyx_n_u_flags2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_45, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1228
 * # 320 bytes: task_calib_info Structure (section 4.2.49, page 59)
 * TASK_CALIB_INFO = (
 *     ('reflectivity_slope', SINT2),             # <<<<<<<<<<<<<<
 *     ('reflectivity_noise_threshold', SINT2),
 *     ('clutter_correction_threshold', SINT2),
 */
  __pyx_t_1 = PyTuple_New(28); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1228, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_32);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_32);
  __Pyx_GIVEREF(__pyx_t_38);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_38);
  __Pyx_GIVEREF(__pyx_t_37);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_t_37);
  __Pyx_GIVEREF(__pyx_t_36);
  PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_t_36);
  __Pyx_GIVEREF(__pyx_t_35);
  PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_t_35);
  __Pyx_INCREF(__pyx_tuple__112);
  __Pyx_GIVEREF(__pyx_tuple__112);
  PyTuple_SET_ITEM(__pyx_t_1, 5, __pyx_tuple__112);
  __Pyx_GIVEREF(__pyx_t_34);
  PyTuple_SET_ITEM(__pyx_t_1, 6, __pyx_t_34);
  __Pyx_GIVEREF(__pyx_t_33);
  PyTuple_SET_ITEM(__pyx_t_1, 7, __pyx_t_33);
  __Pyx_GIVEREF(__pyx_t_31);
  PyTuple_SET_ITEM(__pyx_t_1, 8, __pyx_t_31);
  __Pyx_GIVEREF(__pyx_t_30);
  PyTuple_SET_ITEM(__pyx_t_1, 9, __pyx_t_30);
  __Pyx_GIVEREF(__pyx_t_29);
  PyTuple_SET_ITEM(__pyx_t_1, 10, __pyx_t_29);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_1, 11, __pyx_t_4);
  __Pyx_INCREF(__pyx_tuple__113);
  __Pyx_GIVEREF(__pyx_tuple__113);
  PyTuple_SET_ITEM(__pyx_t_1, 12, __pyx_tuple__113);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_1, 13, __pyx_t_5);
  __Pyx_INCREF(__pyx_tuple__89);
  __Pyx_GIVEREF(__pyx_tuple__89);
  PyTuple_SET_ITEM(__pyx_t_1, 14, __pyx_tuple__89);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_1, 15, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_25);
  PyTuple_SET_ITEM(__pyx_t_1, 16, __pyx_t_25);
  __Pyx_GIVEREF(__pyx_t_26);
  PyTuple_SET_ITEM(__pyx_t_1, 17, __pyx_t_26);
  __Pyx_GIVEREF(__pyx_t_27);
  PyTuple_SET_ITEM(__pyx_t_1, 18, __pyx_t_27);
  __Pyx_GIVEREF(__pyx_t_28);
  PyTuple_SET_ITEM(__pyx_t_1, 19, __pyx_t_28);
  __Pyx_GIVEREF(__pyx_t_39);
  PyTuple_SET_ITEM(__pyx_t_1, 20, __pyx_t_39);
  __Pyx_GIVEREF(__pyx_t_40);
  PyTuple_SET_ITEM(__pyx_t_1, 21, __pyx_t_40);
  __Pyx_GIVEREF(__pyx_t_41);
  PyTuple_SET_ITEM(__pyx_t_1, 22, __pyx_t_41);
  __Pyx_GIVEREF(__pyx_t_42);
  PyTuple_SET_ITEM(__pyx_t_1, 23, __pyx_t_42);
  __Pyx_GIVEREF(__pyx_t_43);
  PyTuple_SET_ITEM(__pyx_t_1, 24, __pyx_t_43);
  __Pyx_GIVEREF(__pyx_t_44);
  PyTuple_SET_ITEM(__pyx_t_1, 25, __pyx_t_44);
  __Pyx_GIVEREF(__pyx_t_45);
  PyTuple_SET_ITEM(__pyx_t_1, 26, __pyx_t_45);
  __Pyx_INCREF(__pyx_tuple__114);
  __Pyx_GIVEREF(__pyx_tuple__114);
  PyTuple_SET_ITEM(__pyx_t_1, 27, __pyx_tuple__114);
  __pyx_t_32 = 0;
  __pyx_t_38 = 0;
  __pyx_t_37 = 0;
  __pyx_t_36 = 0;
  __pyx_t_35 = 0;
  __pyx_t_34 = 0;
  __pyx_t_33 = 0;
  __pyx_t_31 = 0;
  __pyx_t_30 = 0;
  __pyx_t_29 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_25 = 0;
  __pyx_t_26 = 0;
  __pyx_t_27 = 0;
  __pyx_t_28 = 0;
  __pyx_t_39 = 0;
  __pyx_t_40 = 0;
  __pyx_t_41 = 0;
  __pyx_t_42 = 0;
  __pyx_t_43 = 0;
  __pyx_t_44 = 0;
  __pyx_t_45 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_TASK_CALIB_INFO, __pyx_t_1) < 0) __PYX_ERR(0, 1227, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1260
 * # 160 bytes: task_range_info Structure (section 4.2.58, page 64)
 * TASK_RANGE_INFO = (
 *     ('first_bin_range', SINT4),             # <<<<<<<<<<<<<<
 *     ('last_bin_range', SINT4),
 *     ('number_input_bins', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_45 = PyTuple_New(2); if (unlikely(!__pyx_t_45)) __PYX_ERR(0, 1260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_45);
  __Pyx_INCREF(__pyx_n_u_first_bin_range);
  __Pyx_GIVEREF(__pyx_n_u_first_bin_range);
  PyTuple_SET_ITEM(__pyx_t_45, 0, __pyx_n_u_first_bin_range);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_45, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1261
 * TASK_RANGE_INFO = (
 *     ('first_bin_range', SINT4),
 *     ('last_bin_range', SINT4),             # <<<<<<<<<<<<<<
 *     ('number_input_bins', SINT2),
 *     ('number_output_bins', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1261, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_44 = PyTuple_New(2); if (unlikely(!__pyx_t_44)) __PYX_ERR(0, 1261, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_44);
  __Pyx_INCREF(__pyx_n_u_last_bin_range);
  __Pyx_GIVEREF(__pyx_n_u_last_bin_range);
  PyTuple_SET_ITEM(__pyx_t_44, 0, __pyx_n_u_last_bin_range);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_44, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1262
 *     ('first_bin_range', SINT4),
 *     ('last_bin_range', SINT4),
 *     ('number_input_bins', SINT2),             # <<<<<<<<<<<<<<
 *     ('number_output_bins', SINT2),
 *     ('step_input_bins', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_43 = PyTuple_New(2); if (unlikely(!__pyx_t_43)) __PYX_ERR(0, 1262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_43);
  __Pyx_INCREF(__pyx_n_u_number_input_bins);
  __Pyx_GIVEREF(__pyx_n_u_number_input_bins);
  PyTuple_SET_ITEM(__pyx_t_43, 0, __pyx_n_u_number_input_bins);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_43, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1263
 *     ('last_bin_range', SINT4),
 *     ('number_input_bins', SINT2),
 *     ('number_output_bins', SINT2),             # <<<<<<<<<<<<<<
 *     ('step_input_bins', SINT4),
 *     ('step_output_bins', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_42 = PyTuple_New(2); if (unlikely(!__pyx_t_42)) __PYX_ERR(0, 1263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_42);
  __Pyx_INCREF(__pyx_n_u_number_output_bins);
  __Pyx_GIVEREF(__pyx_n_u_number_output_bins);
  PyTuple_SET_ITEM(__pyx_t_42, 0, __pyx_n_u_number_output_bins);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_42, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1264
 *     ('number_input_bins', SINT2),
 *     ('number_output_bins', SINT2),
 *     ('step_input_bins', SINT4),             # <<<<<<<<<<<<<<
 *     ('step_output_bins', SINT4),
 *     ('variable_range_bin_flag', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_41 = PyTuple_New(2); if (unlikely(!__pyx_t_41)) __PYX_ERR(0, 1264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_41);
  __Pyx_INCREF(__pyx_n_u_step_input_bins);
  __Pyx_GIVEREF(__pyx_n_u_step_input_bins);
  PyTuple_SET_ITEM(__pyx_t_41, 0, __pyx_n_u_step_input_bins);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_41, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1265
 *     ('number_output_bins', SINT2),
 *     ('step_input_bins', SINT4),
 *     ('step_output_bins', SINT4),             # <<<<<<<<<<<<<<
 *     ('variable_range_bin_flag', UINT2),
 *     ('range_bin_averaging_flag', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_40 = PyTuple_New(2); if (unlikely(!__pyx_t_40)) __PYX_ERR(0, 1265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_40);
  __Pyx_INCREF(__pyx_n_u_step_output_bins);
  __Pyx_GIVEREF(__pyx_n_u_step_output_bins);
  PyTuple_SET_ITEM(__pyx_t_40, 0, __pyx_n_u_step_output_bins);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_40, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1266
 *     ('step_input_bins', SINT4),
 *     ('step_output_bins', SINT4),
 *     ('variable_range_bin_flag', UINT2),             # <<<<<<<<<<<<<<
 *     ('range_bin_averaging_flag', SINT2),
 *     ('spare_0', '136s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1266, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_39 = PyTuple_New(2); if (unlikely(!__pyx_t_39)) __PYX_ERR(0, 1266, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_39);
  __Pyx_INCREF(__pyx_n_u_variable_range_bin_flag);
  __Pyx_GIVEREF(__pyx_n_u_variable_range_bin_flag);
  PyTuple_SET_ITEM(__pyx_t_39, 0, __pyx_n_u_variable_range_bin_flag);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_39, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1267
 *     ('step_output_bins', SINT4),
 *     ('variable_range_bin_flag', UINT2),
 *     ('range_bin_averaging_flag', SINT2),             # <<<<<<<<<<<<<<
 *     ('spare_0', '136s'),
 * )
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_28 = PyTuple_New(2); if (unlikely(!__pyx_t_28)) __PYX_ERR(0, 1267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_28);
  __Pyx_INCREF(__pyx_n_u_range_bin_averaging_flag);
  __Pyx_GIVEREF(__pyx_n_u_range_bin_averaging_flag);
  PyTuple_SET_ITEM(__pyx_t_28, 0, __pyx_n_u_range_bin_averaging_flag);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_28, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1260
 * # 160 bytes: task_range_info Structure (section 4.2.58, page 64)
 * TASK_RANGE_INFO = (
 *     ('first_bin_range', SINT4),             # <<<<<<<<<<<<<<
 *     ('last_bin_range', SINT4),
 *     ('number_input_bins', SINT2),
 */
  __pyx_t_1 = PyTuple_New(9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_45);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_45);
  __Pyx_GIVEREF(__pyx_t_44);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_44);
  __Pyx_GIVEREF(__pyx_t_43);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_t_43);
  __Pyx_GIVEREF(__pyx_t_42);
  PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_t_42);
  __Pyx_GIVEREF(__pyx_t_41);
  PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_t_41);
  __Pyx_GIVEREF(__pyx_t_40);
  PyTuple_SET_ITEM(__pyx_t_1, 5, __pyx_t_40);
  __Pyx_GIVEREF(__pyx_t_39);
  PyTuple_SET_ITEM(__pyx_t_1, 6, __pyx_t_39);
  __Pyx_GIVEREF(__pyx_t_28);
  PyTuple_SET_ITEM(__pyx_t_1, 7, __pyx_t_28);
  __Pyx_INCREF(__pyx_tuple__115);
  __Pyx_GIVEREF(__pyx_tuple__115);
  PyTuple_SET_ITEM(__pyx_t_1, 8, __pyx_tuple__115);
  __pyx_t_45 = 0;
  __pyx_t_44 = 0;
  __pyx_t_43 = 0;
  __pyx_t_42 = 0;
  __pyx_t_41 = 0;
  __pyx_t_40 = 0;
  __pyx_t_39 = 0;
  __pyx_t_28 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_TASK_RANGE_INFO, __pyx_t_1) < 0) __PYX_ERR(0, 1259, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1273
 * # 320 bytes: task_scan_info Structure (section 4.2.60, page 65)
 * TASK_SCAN_INFO = (
 *     ('antenna_scan_mode', UINT2),             # <<<<<<<<<<<<<<
 *     ('angular_resolution_desired', SINT2),
 *     ('spare_0', '2s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1273, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_28 = PyTuple_New(2); if (unlikely(!__pyx_t_28)) __PYX_ERR(0, 1273, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_28);
  __Pyx_INCREF(__pyx_n_u_antenna_scan_mode);
  __Pyx_GIVEREF(__pyx_n_u_antenna_scan_mode);
  PyTuple_SET_ITEM(__pyx_t_28, 0, __pyx_n_u_antenna_scan_mode);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_28, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1274
 * TASK_SCAN_INFO = (
 *     ('antenna_scan_mode', UINT2),
 *     ('angular_resolution_desired', SINT2),             # <<<<<<<<<<<<<<
 *     ('spare_0', '2s'),
 *     ('number_sweeps', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1274, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_39 = PyTuple_New(2); if (unlikely(!__pyx_t_39)) __PYX_ERR(0, 1274, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_39);
  __Pyx_INCREF(__pyx_n_u_angular_resolution_desired);
  __Pyx_GIVEREF(__pyx_n_u_angular_resolution_desired);
  PyTuple_SET_ITEM(__pyx_t_39, 0, __pyx_n_u_angular_resolution_desired);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_39, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1276
 *     ('angular_resolution_desired', SINT2),
 *     ('spare_0', '2s'),
 *     ('number_sweeps', SINT2),             # <<<<<<<<<<<<<<
 *     ('task_scan_type_scan_info', '200s'),   # 200 bytes: task_foo_scan_info
 *     ('spare_1', '112s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_40 = PyTuple_New(2); if (unlikely(!__pyx_t_40)) __PYX_ERR(0, 1276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_40);
  __Pyx_INCREF(__pyx_n_u_number_sweeps);
  __Pyx_GIVEREF(__pyx_n_u_number_sweeps);
  PyTuple_SET_ITEM(__pyx_t_40, 0, __pyx_n_u_number_sweeps);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_40, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1273
 * # 320 bytes: task_scan_info Structure (section 4.2.60, page 65)
 * TASK_SCAN_INFO = (
 *     ('antenna_scan_mode', UINT2),             # <<<<<<<<<<<<<<
 *     ('angular_resolution_desired', SINT2),
 *     ('spare_0', '2s'),
 */
  __pyx_t_1 = PyTuple_New(6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1273, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_28);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_28);
  __Pyx_GIVEREF(__pyx_t_39);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_39);
  __Pyx_INCREF(__pyx_tuple__116);
  __Pyx_GIVEREF(__pyx_tuple__116);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_tuple__116);
  __Pyx_GIVEREF(__pyx_t_40);
  PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_t_40);
  __Pyx_INCREF(__pyx_tuple__117);
  __Pyx_GIVEREF(__pyx_tuple__117);
  PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_tuple__117);
  __Pyx_INCREF(__pyx_tuple__118);
  __Pyx_GIVEREF(__pyx_tuple__118);
  PyTuple_SET_ITEM(__pyx_t_1, 5, __pyx_tuple__118);
  __pyx_t_28 = 0;
  __pyx_t_39 = 0;
  __pyx_t_40 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_TASK_SCAN_INFO, __pyx_t_1) < 0) __PYX_ERR(0, 1272, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1283
 * # 200 bytes: task_rhi_scan_info Structure (section 4.2.59, page 64)
 * TASK_RHI_SCAN_INFO = (
 *     ('lower_elevation_limit', UINT2),             # <<<<<<<<<<<<<<
 *     ('upper_elevation_limit', UINT2),
 *     ('azimuth_list', '80s'),            # UINT2[40]
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1283, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_40 = PyTuple_New(2); if (unlikely(!__pyx_t_40)) __PYX_ERR(0, 1283, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_40);
  __Pyx_INCREF(__pyx_n_u_lower_elevation_limit);
  __Pyx_GIVEREF(__pyx_n_u_lower_elevation_limit);
  PyTuple_SET_ITEM(__pyx_t_40, 0, __pyx_n_u_lower_elevation_limit);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_40, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1284
 * TASK_RHI_SCAN_INFO = (
 *     ('lower_elevation_limit', UINT2),
 *     ('upper_elevation_limit', UINT2),             # <<<<<<<<<<<<<<
 *     ('azimuth_list', '80s'),            # UINT2[40]
 *     ('spare_0', '115s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1284, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_39 = PyTuple_New(2); if (unlikely(!__pyx_t_39)) __PYX_ERR(0, 1284, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_39);
  __Pyx_INCREF(__pyx_n_u_upper_elevation_limit);
  __Pyx_GIVEREF(__pyx_n_u_upper_elevation_limit);
  PyTuple_SET_ITEM(__pyx_t_39, 0, __pyx_n_u_upper_elevation_limit);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_39, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1283
 * # 200 bytes: task_rhi_scan_info Structure (section 4.2.59, page 64)
 * TASK_RHI_SCAN_INFO = (
 *     ('lower_elevation_limit', UINT2),             # <<<<<<<<<<<<<<
 *     ('upper_elevation_limit', UINT2),
 *     ('azimuth_list', '80s'),            # UINT2[40]
 */
  __pyx_t_1 = PyTuple_New(5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1283, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_40);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_40);
  __Pyx_GIVEREF(__pyx_t_39);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_39);
  __Pyx_INCREF(__pyx_tuple__119);
  __Pyx_GIVEREF(__pyx_tuple__119);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_tuple__119);
  __Pyx_INCREF(__pyx_tuple__120);
  __Pyx_GIVEREF(__pyx_tuple__120);
  PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_tuple__120);
  __Pyx_INCREF(__pyx_tuple__121);
  __Pyx_GIVEREF(__pyx_tuple__121);
  PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_tuple__121);
  __pyx_t_40 = 0;
  __pyx_t_39 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_TASK_RHI_SCAN_INFO, __pyx_t_1) < 0) __PYX_ERR(0, 1282, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1292
 * # 200 bytes: task_ppi_scan_info (section 4.2.57, page 64)
 * TASK_PPI_SCAN_INFO = (
 *     ('left_azimuth_limit', BIN2),             # <<<<<<<<<<<<<<
 *     ('right_azimuth_limit', BIN2),
 *     ('elevation_list', '80s'),          # UINT2[40]
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_BIN2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1292, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_39 = PyTuple_New(2); if (unlikely(!__pyx_t_39)) __PYX_ERR(0, 1292, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_39);
  __Pyx_INCREF(__pyx_n_u_left_azimuth_limit);
  __Pyx_GIVEREF(__pyx_n_u_left_azimuth_limit);
  PyTuple_SET_ITEM(__pyx_t_39, 0, __pyx_n_u_left_azimuth_limit);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_39, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1293
 * TASK_PPI_SCAN_INFO = (
 *     ('left_azimuth_limit', BIN2),
 *     ('right_azimuth_limit', BIN2),             # <<<<<<<<<<<<<<
 *     ('elevation_list', '80s'),          # UINT2[40]
 *     ('spare_0', '115s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_BIN2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1293, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_40 = PyTuple_New(2); if (unlikely(!__pyx_t_40)) __PYX_ERR(0, 1293, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_40);
  __Pyx_INCREF(__pyx_n_u_right_azimuth_limit);
  __Pyx_GIVEREF(__pyx_n_u_right_azimuth_limit);
  PyTuple_SET_ITEM(__pyx_t_40, 0, __pyx_n_u_right_azimuth_limit);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_40, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1292
 * # 200 bytes: task_ppi_scan_info (section 4.2.57, page 64)
 * TASK_PPI_SCAN_INFO = (
 *     ('left_azimuth_limit', BIN2),             # <<<<<<<<<<<<<<
 *     ('right_azimuth_limit', BIN2),
 *     ('elevation_list', '80s'),          # UINT2[40]
 */
  __pyx_t_1 = PyTuple_New(5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1292, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_39);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_39);
  __Pyx_GIVEREF(__pyx_t_40);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_40);
  __Pyx_INCREF(__pyx_tuple__122);
  __Pyx_GIVEREF(__pyx_tuple__122);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_tuple__122);
  __Pyx_INCREF(__pyx_tuple__120);
  __Pyx_GIVEREF(__pyx_tuple__120);
  PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_tuple__120);
  __Pyx_INCREF(__pyx_tuple__123);
  __Pyx_GIVEREF(__pyx_tuple__123);
  PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_tuple__123);
  __pyx_t_39 = 0;
  __pyx_t_40 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_TASK_PPI_SCAN_INFO, __pyx_t_1) < 0) __PYX_ERR(0, 1291, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1301
 * # 200 bytes: task_file_scan_info (section 4.2.54, page 63)
 * TASK_FILE_SCAN_INFO = (
 *     ('first_azimuth', UINT2),             # <<<<<<<<<<<<<<
 *     ('first_elevation', UINT2),
 *     ('filename', '12s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1301, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_40 = PyTuple_New(2); if (unlikely(!__pyx_t_40)) __PYX_ERR(0, 1301, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_40);
  __Pyx_INCREF(__pyx_n_u_first_azimuth);
  __Pyx_GIVEREF(__pyx_n_u_first_azimuth);
  PyTuple_SET_ITEM(__pyx_t_40, 0, __pyx_n_u_first_azimuth);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_40, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1302
 * TASK_FILE_SCAN_INFO = (
 *     ('first_azimuth', UINT2),
 *     ('first_elevation', UINT2),             # <<<<<<<<<<<<<<
 *     ('filename', '12s'),
 *     ('spare_0', '184s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1302, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_39 = PyTuple_New(2); if (unlikely(!__pyx_t_39)) __PYX_ERR(0, 1302, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_39);
  __Pyx_INCREF(__pyx_n_u_first_elevation);
  __Pyx_GIVEREF(__pyx_n_u_first_elevation);
  PyTuple_SET_ITEM(__pyx_t_39, 0, __pyx_n_u_first_elevation);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_39, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1301
 * # 200 bytes: task_file_scan_info (section 4.2.54, page 63)
 * TASK_FILE_SCAN_INFO = (
 *     ('first_azimuth', UINT2),             # <<<<<<<<<<<<<<
 *     ('first_elevation', UINT2),
 *     ('filename', '12s'),
 */
  __pyx_t_1 = PyTuple_New(4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1301, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_40);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_40);
  __Pyx_GIVEREF(__pyx_t_39);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_39);
  __Pyx_INCREF(__pyx_tuple__124);
  __Pyx_GIVEREF(__pyx_tuple__124);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_tuple__124);
  __Pyx_INCREF(__pyx_tuple__125);
  __Pyx_GIVEREF(__pyx_tuple__125);
  PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_tuple__125);
  __pyx_t_40 = 0;
  __pyx_t_39 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_TASK_FILE_SCAN_INFO, __pyx_t_1) < 0) __PYX_ERR(0, 1300, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1309
 * # 200 bytes: task_manual_scan_info (section 4.2.55, page 63)
 * TASK_MANUAL_SCAN_INFO = (
 *     ('flags', UINT2),             # <<<<<<<<<<<<<<
 *     ('spare_0', '198s'),
 * )
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1309, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_39 = PyTuple_New(2); if (unlikely(!__pyx_t_39)) __PYX_ERR(0, 1309, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_39);
  __Pyx_INCREF(__pyx_n_u_flags);
  __Pyx_GIVEREF(__pyx_n_u_flags);
  PyTuple_SET_ITEM(__pyx_t_39, 0, __pyx_n_u_flags);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_39, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1310
 * TASK_MANUAL_SCAN_INFO = (
 *     ('flags', UINT2),
 *     ('spare_0', '198s'),             # <<<<<<<<<<<<<<
 * )
 * 
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1309, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_39);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_39);
  __Pyx_INCREF(__pyx_tuple__126);
  __Pyx_GIVEREF(__pyx_tuple__126);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_tuple__126);
  __pyx_t_39 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_TASK_MANUAL_SCAN_INFO, __pyx_t_1) < 0) __PYX_ERR(0, 1308, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1315
 * # 320 bytes: task_misc_info Structure (section 4.2.55, page 63)
 * TASK_MISC_INFO = (
 *     ('wavelength', SINT4),             # <<<<<<<<<<<<<<
 *     ('tr_serial_number', '16s'),
 *     ('transmit_power', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1315, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_39 = PyTuple_New(2); if (unlikely(!__pyx_t_39)) __PYX_ERR(0, 1315, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_39);
  __Pyx_INCREF(__pyx_n_u_wavelength);
  __Pyx_GIVEREF(__pyx_n_u_wavelength);
  PyTuple_SET_ITEM(__pyx_t_39, 0, __pyx_n_u_wavelength);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_39, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1317
 *     ('wavelength', SINT4),
 *     ('tr_serial_number', '16s'),
 *     ('transmit_power', SINT4),             # <<<<<<<<<<<<<<
 *     ('flags', UINT2),
 *     ('polarization_type', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_40 = PyTuple_New(2); if (unlikely(!__pyx_t_40)) __PYX_ERR(0, 1317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_40);
  __Pyx_INCREF(__pyx_n_u_transmit_power);
  __Pyx_GIVEREF(__pyx_n_u_transmit_power);
  PyTuple_SET_ITEM(__pyx_t_40, 0, __pyx_n_u_transmit_power);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_40, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1318
 *     ('tr_serial_number', '16s'),
 *     ('transmit_power', SINT4),
 *     ('flags', UINT2),             # <<<<<<<<<<<<<<
 *     ('polarization_type', UINT2),
 *     ('trucation_height', SINT4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_28 = PyTuple_New(2); if (unlikely(!__pyx_t_28)) __PYX_ERR(0, 1318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_28);
  __Pyx_INCREF(__pyx_n_u_flags);
  __Pyx_GIVEREF(__pyx_n_u_flags);
  PyTuple_SET_ITEM(__pyx_t_28, 0, __pyx_n_u_flags);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_28, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1319
 *     ('transmit_power', SINT4),
 *     ('flags', UINT2),
 *     ('polarization_type', UINT2),             # <<<<<<<<<<<<<<
 *     ('trucation_height', SINT4),
 *     ('spare_0', '18s'),             # 18 bytes
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_41 = PyTuple_New(2); if (unlikely(!__pyx_t_41)) __PYX_ERR(0, 1319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_41);
  __Pyx_INCREF(__pyx_n_u_polarization_type);
  __Pyx_GIVEREF(__pyx_n_u_polarization_type);
  PyTuple_SET_ITEM(__pyx_t_41, 0, __pyx_n_u_polarization_type);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_41, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1320
 *     ('flags', UINT2),
 *     ('polarization_type', UINT2),
 *     ('trucation_height', SINT4),             # <<<<<<<<<<<<<<
 *     ('spare_0', '18s'),             # 18 bytes
 *     ('spare_1', '12s'),             # 12 bytes
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1320, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_42 = PyTuple_New(2); if (unlikely(!__pyx_t_42)) __PYX_ERR(0, 1320, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_42);
  __Pyx_INCREF(__pyx_n_u_trucation_height);
  __Pyx_GIVEREF(__pyx_n_u_trucation_height);
  PyTuple_SET_ITEM(__pyx_t_42, 0, __pyx_n_u_trucation_height);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_42, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1323
 *     ('spare_0', '18s'),             # 18 bytes
 *     ('spare_1', '12s'),             # 12 bytes
 *     ('comment_bytes', SINT2),             # <<<<<<<<<<<<<<
 *     ('horizontal_beamwidth', BIN4),
 *     ('vertical_beamwidth', BIN4),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1323, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_43 = PyTuple_New(2); if (unlikely(!__pyx_t_43)) __PYX_ERR(0, 1323, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_43);
  __Pyx_INCREF(__pyx_n_u_comment_bytes);
  __Pyx_GIVEREF(__pyx_n_u_comment_bytes);
  PyTuple_SET_ITEM(__pyx_t_43, 0, __pyx_n_u_comment_bytes);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_43, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1324
 *     ('spare_1', '12s'),             # 12 bytes
 *     ('comment_bytes', SINT2),
 *     ('horizontal_beamwidth', BIN4),             # <<<<<<<<<<<<<<
 *     ('vertical_beamwidth', BIN4),
 *     ('customer_storage', '40s'),    # 40 bytes, uint4[10]
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_BIN4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1324, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_44 = PyTuple_New(2); if (unlikely(!__pyx_t_44)) __PYX_ERR(0, 1324, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_44);
  __Pyx_INCREF(__pyx_n_u_horizontal_beamwidth);
  __Pyx_GIVEREF(__pyx_n_u_horizontal_beamwidth);
  PyTuple_SET_ITEM(__pyx_t_44, 0, __pyx_n_u_horizontal_beamwidth);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_44, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1325
 *     ('comment_bytes', SINT2),
 *     ('horizontal_beamwidth', BIN4),
 *     ('vertical_beamwidth', BIN4),             # <<<<<<<<<<<<<<
 *     ('customer_storage', '40s'),    # 40 bytes, uint4[10]
 *     ('spare_2', '208s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_BIN4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1325, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_45 = PyTuple_New(2); if (unlikely(!__pyx_t_45)) __PYX_ERR(0, 1325, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_45);
  __Pyx_INCREF(__pyx_n_u_vertical_beamwidth);
  __Pyx_GIVEREF(__pyx_n_u_vertical_beamwidth);
  PyTuple_SET_ITEM(__pyx_t_45, 0, __pyx_n_u_vertical_beamwidth);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_45, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1315
 * # 320 bytes: task_misc_info Structure (section 4.2.55, page 63)
 * TASK_MISC_INFO = (
 *     ('wavelength', SINT4),             # <<<<<<<<<<<<<<
 *     ('tr_serial_number', '16s'),
 *     ('transmit_power', SINT4),
 */
  __pyx_t_1 = PyTuple_New(13); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1315, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_39);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_39);
  __Pyx_INCREF(__pyx_tuple__127);
  __Pyx_GIVEREF(__pyx_tuple__127);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_tuple__127);
  __Pyx_GIVEREF(__pyx_t_40);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_t_40);
  __Pyx_GIVEREF(__pyx_t_28);
  PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_t_28);
  __Pyx_GIVEREF(__pyx_t_41);
  PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_t_41);
  __Pyx_GIVEREF(__pyx_t_42);
  PyTuple_SET_ITEM(__pyx_t_1, 5, __pyx_t_42);
  __Pyx_INCREF(__pyx_tuple__111);
  __Pyx_GIVEREF(__pyx_tuple__111);
  PyTuple_SET_ITEM(__pyx_t_1, 6, __pyx_tuple__111);
  __Pyx_INCREF(__pyx_tuple__128);
  __Pyx_GIVEREF(__pyx_tuple__128);
  PyTuple_SET_ITEM(__pyx_t_1, 7, __pyx_tuple__128);
  __Pyx_GIVEREF(__pyx_t_43);
  PyTuple_SET_ITEM(__pyx_t_1, 8, __pyx_t_43);
  __Pyx_GIVEREF(__pyx_t_44);
  PyTuple_SET_ITEM(__pyx_t_1, 9, __pyx_t_44);
  __Pyx_GIVEREF(__pyx_t_45);
  PyTuple_SET_ITEM(__pyx_t_1, 10, __pyx_t_45);
  __Pyx_INCREF(__pyx_tuple__129);
  __Pyx_GIVEREF(__pyx_tuple__129);
  PyTuple_SET_ITEM(__pyx_t_1, 11, __pyx_tuple__129);
  __Pyx_INCREF(__pyx_tuple__130);
  __Pyx_GIVEREF(__pyx_tuple__130);
  PyTuple_SET_ITEM(__pyx_t_1, 12, __pyx_tuple__130);
  __pyx_t_39 = 0;
  __pyx_t_40 = 0;
  __pyx_t_28 = 0;
  __pyx_t_41 = 0;
  __pyx_t_42 = 0;
  __pyx_t_43 = 0;
  __pyx_t_44 = 0;
  __pyx_t_45 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_TASK_MISC_INFO, __pyx_t_1) < 0) __PYX_ERR(0, 1314, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1332
 * # 320 bytes: task_end_info Structure (section 4.2.53, page 62)
 * TASK_END_INFO = (
 *     ('task_major_number', SINT2),             # <<<<<<<<<<<<<<
 *     ('task_minor_number', SINT2),
 *     ('task_configuration_file_name', '12s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1332, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_45 = PyTuple_New(2); if (unlikely(!__pyx_t_45)) __PYX_ERR(0, 1332, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_45);
  __Pyx_INCREF(__pyx_n_u_task_major_number);
  __Pyx_GIVEREF(__pyx_n_u_task_major_number);
  PyTuple_SET_ITEM(__pyx_t_45, 0, __pyx_n_u_task_major_number);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_45, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1333
 * TASK_END_INFO = (
 *     ('task_major_number', SINT2),
 *     ('task_minor_number', SINT2),             # <<<<<<<<<<<<<<
 *     ('task_configuration_file_name', '12s'),
 *     ('task_description', '80s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1333, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_44 = PyTuple_New(2); if (unlikely(!__pyx_t_44)) __PYX_ERR(0, 1333, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_44);
  __Pyx_INCREF(__pyx_n_u_task_minor_number);
  __Pyx_GIVEREF(__pyx_n_u_task_minor_number);
  PyTuple_SET_ITEM(__pyx_t_44, 0, __pyx_n_u_task_minor_number);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_44, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1336
 *     ('task_configuration_file_name', '12s'),
 *     ('task_description', '80s'),
 *     ('number_tasks', SINT4),             # <<<<<<<<<<<<<<
 *     ('task_state', UINT2),
 *     ('spare_0', '2s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1336, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_43 = PyTuple_New(2); if (unlikely(!__pyx_t_43)) __PYX_ERR(0, 1336, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_43);
  __Pyx_INCREF(__pyx_n_u_number_tasks);
  __Pyx_GIVEREF(__pyx_n_u_number_tasks);
  PyTuple_SET_ITEM(__pyx_t_43, 0, __pyx_n_u_number_tasks);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_43, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1337
 *     ('task_description', '80s'),
 *     ('number_tasks', SINT4),
 *     ('task_state', UINT2),             # <<<<<<<<<<<<<<
 *     ('spare_0', '2s'),
 *     ('task_data_time', '12s'),      # 12 bytes: ymds_time
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1337, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_42 = PyTuple_New(2); if (unlikely(!__pyx_t_42)) __PYX_ERR(0, 1337, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_42);
  __Pyx_INCREF(__pyx_n_u_task_state);
  __Pyx_GIVEREF(__pyx_n_u_task_state);
  PyTuple_SET_ITEM(__pyx_t_42, 0, __pyx_n_u_task_state);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_42, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1332
 * # 320 bytes: task_end_info Structure (section 4.2.53, page 62)
 * TASK_END_INFO = (
 *     ('task_major_number', SINT2),             # <<<<<<<<<<<<<<
 *     ('task_minor_number', SINT2),
 *     ('task_configuration_file_name', '12s'),
 */
  __pyx_t_1 = PyTuple_New(9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1332, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_45);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_45);
  __Pyx_GIVEREF(__pyx_t_44);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_44);
  __Pyx_INCREF(__pyx_tuple__131);
  __Pyx_GIVEREF(__pyx_tuple__131);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_tuple__131);
  __Pyx_INCREF(__pyx_tuple__132);
  __Pyx_GIVEREF(__pyx_tuple__132);
  PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_tuple__132);
  __Pyx_GIVEREF(__pyx_t_43);
  PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_t_43);
  __Pyx_GIVEREF(__pyx_t_42);
  PyTuple_SET_ITEM(__pyx_t_1, 5, __pyx_t_42);
  __Pyx_INCREF(__pyx_tuple__116);
  __Pyx_GIVEREF(__pyx_tuple__116);
  PyTuple_SET_ITEM(__pyx_t_1, 6, __pyx_tuple__116);
  __Pyx_INCREF(__pyx_tuple__133);
  __Pyx_GIVEREF(__pyx_tuple__133);
  PyTuple_SET_ITEM(__pyx_t_1, 7, __pyx_tuple__133);
  __Pyx_INCREF(__pyx_tuple__134);
  __Pyx_GIVEREF(__pyx_tuple__134);
  PyTuple_SET_ITEM(__pyx_t_1, 8, __pyx_tuple__134);
  __pyx_t_45 = 0;
  __pyx_t_44 = 0;
  __pyx_t_43 = 0;
  __pyx_t_42 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_TASK_END_INFO, __pyx_t_1) < 0) __PYX_ERR(0, 1331, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1346
 * # 12 bytes raw_prod_bhdr structure (section 4.2.30, page 50)
 * RAW_PROD_BHDR = (
 *     ('record_number', SINT2),             # <<<<<<<<<<<<<<
 *     ('sweep_number', SINT2),
 *     ('first_ray_offset', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1346, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_42 = PyTuple_New(2); if (unlikely(!__pyx_t_42)) __PYX_ERR(0, 1346, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_42);
  __Pyx_INCREF(__pyx_n_u_record_number);
  __Pyx_GIVEREF(__pyx_n_u_record_number);
  PyTuple_SET_ITEM(__pyx_t_42, 0, __pyx_n_u_record_number);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_42, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1347
 * RAW_PROD_BHDR = (
 *     ('record_number', SINT2),
 *     ('sweep_number', SINT2),             # <<<<<<<<<<<<<<
 *     ('first_ray_offset', SINT2),
 *     ('ray_number', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1347, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_43 = PyTuple_New(2); if (unlikely(!__pyx_t_43)) __PYX_ERR(0, 1347, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_43);
  __Pyx_INCREF(__pyx_n_u_sweep_number);
  __Pyx_GIVEREF(__pyx_n_u_sweep_number);
  PyTuple_SET_ITEM(__pyx_t_43, 0, __pyx_n_u_sweep_number);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_43, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1348
 *     ('record_number', SINT2),
 *     ('sweep_number', SINT2),
 *     ('first_ray_offset', SINT2),             # <<<<<<<<<<<<<<
 *     ('ray_number', SINT2),
 *     ('flags', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1348, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_44 = PyTuple_New(2); if (unlikely(!__pyx_t_44)) __PYX_ERR(0, 1348, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_44);
  __Pyx_INCREF(__pyx_n_u_first_ray_offset);
  __Pyx_GIVEREF(__pyx_n_u_first_ray_offset);
  PyTuple_SET_ITEM(__pyx_t_44, 0, __pyx_n_u_first_ray_offset);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_44, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1349
 *     ('sweep_number', SINT2),
 *     ('first_ray_offset', SINT2),
 *     ('ray_number', SINT2),             # <<<<<<<<<<<<<<
 *     ('flags', UINT2),
 *     ('spare_0', '2s'),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_45 = PyTuple_New(2); if (unlikely(!__pyx_t_45)) __PYX_ERR(0, 1349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_45);
  __Pyx_INCREF(__pyx_n_u_ray_number);
  __Pyx_GIVEREF(__pyx_n_u_ray_number);
  PyTuple_SET_ITEM(__pyx_t_45, 0, __pyx_n_u_ray_number);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_45, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1350
 *     ('first_ray_offset', SINT2),
 *     ('ray_number', SINT2),
 *     ('flags', UINT2),             # <<<<<<<<<<<<<<
 *     ('spare_0', '2s'),
 * )
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1350, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_41 = PyTuple_New(2); if (unlikely(!__pyx_t_41)) __PYX_ERR(0, 1350, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_41);
  __Pyx_INCREF(__pyx_n_u_flags);
  __Pyx_GIVEREF(__pyx_n_u_flags);
  PyTuple_SET_ITEM(__pyx_t_41, 0, __pyx_n_u_flags);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_41, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1346
 * # 12 bytes raw_prod_bhdr structure (section 4.2.30, page 50)
 * RAW_PROD_BHDR = (
 *     ('record_number', SINT2),             # <<<<<<<<<<<<<<
 *     ('sweep_number', SINT2),
 *     ('first_ray_offset', SINT2),
 */
  __pyx_t_1 = PyTuple_New(6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1346, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_42);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_42);
  __Pyx_GIVEREF(__pyx_t_43);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_43);
  __Pyx_GIVEREF(__pyx_t_44);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_t_44);
  __Pyx_GIVEREF(__pyx_t_45);
  PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_t_45);
  __Pyx_GIVEREF(__pyx_t_41);
  PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_t_41);
  __Pyx_INCREF(__pyx_tuple__116);
  __Pyx_GIVEREF(__pyx_tuple__116);
  PyTuple_SET_ITEM(__pyx_t_1, 5, __pyx_tuple__116);
  __pyx_t_42 = 0;
  __pyx_t_43 = 0;
  __pyx_t_44 = 0;
  __pyx_t_45 = 0;
  __pyx_t_41 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_RAW_PROD_BHDR, __pyx_t_1) < 0) __PYX_ERR(0, 1345, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1358
 *     ('structure_header', '12s'),    # 12 bytes: structure_header
 *     ('sweep_start_time', '12s'),    # 12 bytes: ymds_time
 *     ('sweep_number', SINT2),             # <<<<<<<<<<<<<<
 *     ('number_rays_sweep', SINT2),
 *     ('first_ray_index', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1358, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_41 = PyTuple_New(2); if (unlikely(!__pyx_t_41)) __PYX_ERR(0, 1358, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_41);
  __Pyx_INCREF(__pyx_n_u_sweep_number);
  __Pyx_GIVEREF(__pyx_n_u_sweep_number);
  PyTuple_SET_ITEM(__pyx_t_41, 0, __pyx_n_u_sweep_number);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_41, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1359
 *     ('sweep_start_time', '12s'),    # 12 bytes: ymds_time
 *     ('sweep_number', SINT2),
 *     ('number_rays_sweep', SINT2),             # <<<<<<<<<<<<<<
 *     ('first_ray_index', SINT2),
 *     ('number_rays_file_expected', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1359, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_45 = PyTuple_New(2); if (unlikely(!__pyx_t_45)) __PYX_ERR(0, 1359, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_45);
  __Pyx_INCREF(__pyx_n_u_number_rays_sweep);
  __Pyx_GIVEREF(__pyx_n_u_number_rays_sweep);
  PyTuple_SET_ITEM(__pyx_t_45, 0, __pyx_n_u_number_rays_sweep);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_45, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1360
 *     ('sweep_number', SINT2),
 *     ('number_rays_sweep', SINT2),
 *     ('first_ray_index', SINT2),             # <<<<<<<<<<<<<<
 *     ('number_rays_file_expected', SINT2),
 *     ('number_rays_file_actual', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1360, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_44 = PyTuple_New(2); if (unlikely(!__pyx_t_44)) __PYX_ERR(0, 1360, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_44);
  __Pyx_INCREF(__pyx_n_u_first_ray_index);
  __Pyx_GIVEREF(__pyx_n_u_first_ray_index);
  PyTuple_SET_ITEM(__pyx_t_44, 0, __pyx_n_u_first_ray_index);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_44, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1361
 *     ('number_rays_sweep', SINT2),
 *     ('first_ray_index', SINT2),
 *     ('number_rays_file_expected', SINT2),             # <<<<<<<<<<<<<<
 *     ('number_rays_file_actual', SINT2),
 *     ('fixed_angle', BIN2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_43 = PyTuple_New(2); if (unlikely(!__pyx_t_43)) __PYX_ERR(0, 1361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_43);
  __Pyx_INCREF(__pyx_n_u_number_rays_file_expected);
  __Pyx_GIVEREF(__pyx_n_u_number_rays_file_expected);
  PyTuple_SET_ITEM(__pyx_t_43, 0, __pyx_n_u_number_rays_file_expected);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_43, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1362
 *     ('first_ray_index', SINT2),
 *     ('number_rays_file_expected', SINT2),
 *     ('number_rays_file_actual', SINT2),             # <<<<<<<<<<<<<<
 *     ('fixed_angle', BIN2),
 *     ('bit_per_bin', SINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1362, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_42 = PyTuple_New(2); if (unlikely(!__pyx_t_42)) __PYX_ERR(0, 1362, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_42);
  __Pyx_INCREF(__pyx_n_u_number_rays_file_actual);
  __Pyx_GIVEREF(__pyx_n_u_number_rays_file_actual);
  PyTuple_SET_ITEM(__pyx_t_42, 0, __pyx_n_u_number_rays_file_actual);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_42, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1363
 *     ('number_rays_file_expected', SINT2),
 *     ('number_rays_file_actual', SINT2),
 *     ('fixed_angle', BIN2),             # <<<<<<<<<<<<<<
 *     ('bit_per_bin', SINT2),
 *     ('data_type', UINT2),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_BIN2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1363, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_28 = PyTuple_New(2); if (unlikely(!__pyx_t_28)) __PYX_ERR(0, 1363, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_28);
  __Pyx_INCREF(__pyx_n_u_fixed_angle);
  __Pyx_GIVEREF(__pyx_n_u_fixed_angle);
  PyTuple_SET_ITEM(__pyx_t_28, 0, __pyx_n_u_fixed_angle);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_28, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1364
 *     ('number_rays_file_actual', SINT2),
 *     ('fixed_angle', BIN2),
 *     ('bit_per_bin', SINT2),             # <<<<<<<<<<<<<<
 *     ('data_type', UINT2),
 *     ('spare_0', '36s')      # 36 bytes
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1364, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_40 = PyTuple_New(2); if (unlikely(!__pyx_t_40)) __PYX_ERR(0, 1364, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_40);
  __Pyx_INCREF(__pyx_n_u_bit_per_bin);
  __Pyx_GIVEREF(__pyx_n_u_bit_per_bin);
  PyTuple_SET_ITEM(__pyx_t_40, 0, __pyx_n_u_bit_per_bin);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_40, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1365
 *     ('fixed_angle', BIN2),
 *     ('bit_per_bin', SINT2),
 *     ('data_type', UINT2),             # <<<<<<<<<<<<<<
 *     ('spare_0', '36s')      # 36 bytes
 * )
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_UINT2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1365, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_39 = PyTuple_New(2); if (unlikely(!__pyx_t_39)) __PYX_ERR(0, 1365, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_39);
  __Pyx_INCREF(__pyx_n_u_data_type);
  __Pyx_GIVEREF(__pyx_n_u_data_type);
  PyTuple_SET_ITEM(__pyx_t_39, 0, __pyx_n_u_data_type);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_39, 1, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1356
 * # 76 bytes ingest_data_header (section 4.2.15, pages 40)
 * INGEST_DATA_HEADER = (
 *     ('structure_header', '12s'),    # 12 bytes: structure_header             # <<<<<<<<<<<<<<
 *     ('sweep_start_time', '12s'),    # 12 bytes: ymds_time
 *     ('sweep_number', SINT2),
 */
  __pyx_t_1 = PyTuple_New(11); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1356, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_tuple__47);
  __Pyx_GIVEREF(__pyx_tuple__47);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_tuple__47);
  __Pyx_INCREF(__pyx_tuple__135);
  __Pyx_GIVEREF(__pyx_tuple__135);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_tuple__135);
  __Pyx_GIVEREF(__pyx_t_41);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_t_41);
  __Pyx_GIVEREF(__pyx_t_45);
  PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_t_45);
  __Pyx_GIVEREF(__pyx_t_44);
  PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_t_44);
  __Pyx_GIVEREF(__pyx_t_43);
  PyTuple_SET_ITEM(__pyx_t_1, 5, __pyx_t_43);
  __Pyx_GIVEREF(__pyx_t_42);
  PyTuple_SET_ITEM(__pyx_t_1, 6, __pyx_t_42);
  __Pyx_GIVEREF(__pyx_t_28);
  PyTuple_SET_ITEM(__pyx_t_1, 7, __pyx_t_28);
  __Pyx_GIVEREF(__pyx_t_40);
  PyTuple_SET_ITEM(__pyx_t_1, 8, __pyx_t_40);
  __Pyx_GIVEREF(__pyx_t_39);
  PyTuple_SET_ITEM(__pyx_t_1, 9, __pyx_t_39);
  __Pyx_INCREF(__pyx_tuple__136);
  __Pyx_GIVEREF(__pyx_tuple__136);
  PyTuple_SET_ITEM(__pyx_t_1, 10, __pyx_tuple__136);
  __pyx_t_41 = 0;
  __pyx_t_45 = 0;
  __pyx_t_44 = 0;
  __pyx_t_43 = 0;
  __pyx_t_42 = 0;
  __pyx_t_28 = 0;
  __pyx_t_40 = 0;
  __pyx_t_39 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_INGEST_DATA_HEADER, __pyx_t_1) < 0) __PYX_ERR(0, 1355, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_sigmetfile.pyx":1
 * """             # <<<<<<<<<<<<<<
 * A class and supporting functions for reading Sigmet (raw format) files.
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":1014
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_XDECREF(__pyx_t_19);
  __Pyx_XDECREF(__pyx_t_20);
  __Pyx_XDECREF(__pyx_t_21);
  __Pyx_XDECREF(__pyx_t_22);
  __Pyx_XDECREF(__pyx_t_23);
  __Pyx_XDECREF(__pyx_t_24);
  __Pyx_XDECREF(__pyx_t_25);
  __Pyx_XDECREF(__pyx_t_26);
  __Pyx_XDECREF(__pyx_t_27);
  __Pyx_XDECREF(__pyx_t_28);
  __Pyx_XDECREF(__pyx_t_29);
  __Pyx_XDECREF(__pyx_t_30);
  __Pyx_XDECREF(__pyx_t_31);
  __Pyx_XDECREF(__pyx_t_32);
  __Pyx_XDECREF(__pyx_t_33);
  __Pyx_XDECREF(__pyx_t_34);
  __Pyx_XDECREF(__pyx_t_35);
  __Pyx_XDECREF(__pyx_t_36);
  __Pyx_XDECREF(__pyx_t_37);
  __Pyx_XDECREF(__pyx_t_38);
  __Pyx_XDECREF(__pyx_t_39);
  __Pyx_XDECREF(__pyx_t_40);
  __Pyx_XDECREF(__pyx_t_41);
  __Pyx_XDECREF(__pyx_t_42);
  __Pyx_XDECREF(__pyx_t_43);
  __Pyx_XDECREF(__pyx_t_44);
  __Pyx_XDECREF(__pyx_t_45);
  __Pyx_XDECREF(__pyx_t_46);
  __Pyx_XDECREF(__pyx_t_47);
  __Pyx_XDECREF(__pyx_t_48);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init pyart.io._sigmetfile", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init pyart.io._sigmetfile");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* HasAttr */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (__Pyx_PyFastCFunction_Check(func)) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject* index) {
    PyObject *runerr;
    Py_ssize_t key_value;
    PySequenceMethods *m = Py_TYPE(obj)->tp_as_sequence;
    if (unlikely(!(m && m->sq_item))) {
        PyErr_Format(PyExc_TypeError, "'%.200s' object is not subscriptable", Py_TYPE(obj)->tp_name);
        return NULL;
    }
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError, "cannot fit '%.200s' into an index-sized integer", Py_TYPE(index)->tp_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key) {
    PyMappingMethods *m = Py_TYPE(obj)->tp_as_mapping;
    if (likely(m && m->mp_subscript)) {
        return m->mp_subscript(obj, key);
    }
    return __Pyx_PyObject_GetIndex(obj, key);
}
#endif

/* DictGetItem */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            if (unlikely(PyTuple_Check(key))) {
                PyObject* args = PyTuple_Pack(1, key);
                if (likely(args)) {
                    PyErr_SetObject(PyExc_KeyError, args);
                    Py_DECREF(args);
                }
            } else {
                PyErr_SetObject(PyExc_KeyError, key);
            }
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#endif

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* SliceObject */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_slice(obj, cstart, cstop);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object is unsliceable", Py_TYPE(obj)->tp_name);
bad:
    return NULL;
}

/* PyIntBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a + b);
            if (likely((x^a) >= 0 || (x^b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_add(op1, op2);
            }
        }
                x = a + b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla + llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("add", return NULL)
            result = ((double)a) + (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#endif

/* PyObjectGetMethod */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if PY_MAJOR_VERSION >= 3
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type)))
        #endif
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (descr != NULL) {
        *method = descr;
        return 0;
    }
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(name));
#endif
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}

/* PyObjectCallMethod1 */
static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = __Pyx_PyObject_CallOneArg(method, arg);
    Py_DECREF(method);
    return result;
}
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
    PyObject *method = NULL, *result;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_Call2Args(method, obj, arg);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) return NULL;
    return __Pyx__PyObject_CallMethod1(method, arg);
}

/* append */
static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x) {
    if (likely(PyList_CheckExact(L))) {
        if (unlikely(__Pyx_PyList_Append(L, x) < 0)) return -1;
    } else {
        PyObject* retval = __Pyx_PyObject_CallMethod1(L, __pyx_n_s_append, x);
        if (unlikely(!retval))
            return -1;
        Py_DECREF(retval);
    }
    return 0;
}

/* PyFloatBinop */
#if !CYTHON_COMPILING_IN_PYPY
#define __Pyx_PyFloat_TrueDivideCObj_ZeroDivisionError(operand) if (unlikely(zerodivision_check && ((operand) == 0))) {\
    PyErr_SetString(PyExc_ZeroDivisionError, "float division by zero");\
    return NULL;\
}
static PyObject* __Pyx_PyFloat_TrueDivideCObj(PyObject *op1, PyObject *op2, double floatval, int inplace, int zerodivision_check) {
    const double a = floatval;
    double b, result;
    (void)inplace;
    (void)zerodivision_check;
    if (likely(PyFloat_CheckExact(op2))) {
        b = PyFloat_AS_DOUBLE(op2);
        __Pyx_PyFloat_TrueDivideCObj_ZeroDivisionError(b)
    } else
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op2))) {
        b = (double) PyInt_AS_LONG(op2);
        __Pyx_PyFloat_TrueDivideCObj_ZeroDivisionError(b)
    } else
    #endif
    if (likely(PyLong_CheckExact(op2))) {
        #if CYTHON_USE_PYLONG_INTERNALS
        const digit* digits = ((PyLongObject*)op2)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op2);
        switch (size) {
            case  0: __Pyx_PyFloat_TrueDivideCObj_ZeroDivisionError(0) break;
            case -1: b = -(double) digits[0]; break;
            case  1: b = (double) digits[0]; break;
            case -2:
            case 2:
                if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (1 * PyLong_SHIFT < 53))) {
                    b = (double) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53) || (b < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -2)
                            b = -b;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            case -3:
            case 3:
                if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53))) {
                    b = (double) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53) || (b < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -3)
                            b = -b;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            case -4:
            case 4:
                if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53))) {
                    b = (double) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (4 * PyLong_SHIFT < 53) || (b < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -4)
                            b = -b;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            default:
        #else
        {
        #endif
            b = PyLong_AsDouble(op2);
            if (unlikely(b == -1.0 && PyErr_Occurred())) return NULL;
            __Pyx_PyFloat_TrueDivideCObj_ZeroDivisionError(b)
        }
    } else {
        return (inplace ? PyNumber_InPlaceTrueDivide : PyNumber_TrueDivide)(op1, op2);
    }
        __Pyx_PyFloat_TrueDivideCObj_ZeroDivisionError(b)
        PyFPE_START_PROTECT("divide", return NULL)
        result = a / b;
        PyFPE_END_PROTECT(result)
        return PyFloat_FromDouble(result);
}
#endif

/* ExtTypeTest */
  static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* PyIntBinop */
  #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_SubtractObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a - b);
            if (likely((x^a) >= 0 || (x^~b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_subtract(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_subtract(op1, op2);
            }
        }
                x = a - b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla - llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("subtract", return NULL)
            result = ((double)a) - (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceSubtract : PyNumber_Subtract)(op1, op2);
}
#endif

/* PyIntBinop */
  #if !CYTHON_COMPILING_IN_PYPY
#if PY_MAJOR_VERSION < 3 || CYTHON_USE_PYLONG_INTERNALS
#define __Pyx_PyInt_TrueDivideObjC_ZeroDivisionError(operand)\
    if (unlikely(zerodivision_check && ((operand) == 0))) {\
        PyErr_SetString(PyExc_ZeroDivisionError, "integer division by zero");\
        return NULL;\
    }
#endif
static PyObject* __Pyx_PyInt_TrueDivideObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
            __Pyx_PyInt_TrueDivideObjC_ZeroDivisionError(b)
            if (8 * sizeof(long) <= 53 || likely(labs(a) <= ((PY_LONG_LONG)1 << 53))) {
                return PyFloat_FromDouble((double)a / (double)b);
            }
            return PyInt_Type.tp_as_number->nb_true_divide(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT && 1 * PyLong_SHIFT < 53) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT && 1 * PyLong_SHIFT < 53) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT && 2 * PyLong_SHIFT < 53) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT && 2 * PyLong_SHIFT < 53) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT && 3 * PyLong_SHIFT < 53) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT && 3 * PyLong_SHIFT < 53) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_true_divide(op1, op2);
            }
        }
                __Pyx_PyInt_TrueDivideObjC_ZeroDivisionError(b)
                if ((8 * sizeof(long) <= 53 || likely(labs(a) <= ((PY_LONG_LONG)1 << 53)))
                        || __Pyx_sst_abs(size) <= 52 / PyLong_SHIFT) {
                    return PyFloat_FromDouble((double)a / (double)b);
                }
                return PyLong_Type.tp_as_number->nb_true_divide(op1, op2);
            return PyLong_FromLong(x);
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            if (unlikely(zerodivision_check && b == 0)) {
                PyErr_SetString(PyExc_ZeroDivisionError, "float division by zero");
                return NULL;
            }
            PyFPE_START_PROTECT("divide", return NULL)
            result = ((double)a) / (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceTrueDivide : PyNumber_TrueDivide)(op1, op2);
}
#endif

/* PyIntBinop */
  #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddCObj(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op2))) {
        const long a = intval;
        long x;
        long b = PyInt_AS_LONG(op2);
            x = (long)((unsigned long)a + b);
            if (likely((x^a) >= 0 || (x^b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op2))) {
        const long a = intval;
        long b, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG lla = intval;
        PY_LONG_LONG llb, llx;
#endif
        const digit* digits = ((PyLongObject*)op2)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op2);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            b = likely(size) ? digits[0] : 0;
            if (size == -1) b = -b;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        b = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        llb = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        b = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        llb = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        b = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        llb = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        b = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        llb = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        b = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        llb = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        b = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        llb = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_add(op1, op2);
            }
        }
                x = a + b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla + llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op2)) {
        const long a = intval;
        double b = PyFloat_AS_DOUBLE(op2);
            double result;
            PyFPE_START_PROTECT("add", return NULL)
            result = ((double)a) + (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#endif

/* PyIntCompare */
  static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED long inplace) {
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a == b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = Py_SIZE(op1);
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        if (intval == 0) {
            if (size == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
        } else if (intval < 0) {
            if (size >= 0)
                Py_RETURN_FALSE;
            intval = -intval;
            size = -size;
        } else {
            if (size <= 0)
                Py_RETURN_FALSE;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        if (unequal == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
        if ((double)a == (double)b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    return (
        PyObject_RichCompare(op1, op2, Py_EQ));
}

/* IsLittleEndian */
  static CYTHON_INLINE int __Pyx_Is_Little_Endian(void)
{
  union {
    uint32_t u32;
    uint8_t u8[4];
  } S;
  S.u32 = 0x01020304;
  return S.u8[0] == 4;
}

/* BufferFormatCheck */
  static void __Pyx_BufFmt_Init(__Pyx_BufFmt_Context* ctx,
                              __Pyx_BufFmt_StackElem* stack,
                              __Pyx_TypeInfo* type) {
  stack[0].field = &ctx->root;
  stack[0].parent_offset = 0;
  ctx->root.type = type;
  ctx->root.name = "buffer dtype";
  ctx->root.offset = 0;
  ctx->head = stack;
  ctx->head->field = &ctx->root;
  ctx->fmt_offset = 0;
  ctx->head->parent_offset = 0;
  ctx->new_packmode = '@';
  ctx->enc_packmode = '@';
  ctx->new_count = 1;
  ctx->enc_count = 0;
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  ctx->is_valid_array = 0;
  ctx->struct_alignment = 0;
  while (type->typegroup == 'S') {
    ++ctx->head;
    ctx->head->field = type->fields;
    ctx->head->parent_offset = 0;
    type = type->fields->type;
  }
}
static int __Pyx_BufFmt_ParseNumber(const char** ts) {
    int count;
    const char* t = *ts;
    if (*t < '0' || *t > '9') {
      return -1;
    } else {
        count = *t++ - '0';
        while (*t >= '0' && *t <= '9') {
            count *= 10;
            count += *t++ - '0';
        }
    }
    *ts = t;
    return count;
}
static int __Pyx_BufFmt_ExpectNumber(const char **ts) {
    int number = __Pyx_BufFmt_ParseNumber(ts);
    if (number == -1)
        PyErr_Format(PyExc_ValueError,\
                     "Does not understand character buffer dtype format string ('%c')", **ts);
    return number;
}
static void __Pyx_BufFmt_RaiseUnexpectedChar(char ch) {
  PyErr_Format(PyExc_ValueError,
               "Unexpected format string character: '%c'", ch);
}
static const char* __Pyx_BufFmt_DescribeTypeChar(char ch, int is_complex) {
  switch (ch) {
    case '?': return "'bool'";
    case 'c': return "'char'";
    case 'b': return "'signed char'";
    case 'B': return "'unsigned char'";
    case 'h': return "'short'";
    case 'H': return "'unsigned short'";
    case 'i': return "'int'";
    case 'I': return "'unsigned int'";
    case 'l': return "'long'";
    case 'L': return "'unsigned long'";
    case 'q': return "'long long'";
    case 'Q': return "'unsigned long long'";
    case 'f': return (is_complex ? "'complex float'" : "'float'");
    case 'd': return (is_complex ? "'complex double'" : "'double'");
    case 'g': return (is_complex ? "'complex long double'" : "'long double'");
    case 'T': return "a struct";
    case 'O': return "Python object";
    case 'P': return "a pointer";
    case 's': case 'p': return "a string";
    case 0: return "end";
    default: return "unparseable format string";
  }
}
static size_t __Pyx_BufFmt_TypeCharToStandardSize(char ch, int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return 2;
    case 'i': case 'I': case 'l': case 'L': return 4;
    case 'q': case 'Q': return 8;
    case 'f': return (is_complex ? 8 : 4);
    case 'd': return (is_complex ? 16 : 8);
    case 'g': {
      PyErr_SetString(PyExc_ValueError, "Python does not define a standard format string size for long double ('g')..");
      return 0;
    }
    case 'O': case 'P': return sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static size_t __Pyx_BufFmt_TypeCharToNativeSize(char ch, int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(short);
    case 'i': case 'I': return sizeof(int);
    case 'l': case 'L': return sizeof(long);
    #ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(PY_LONG_LONG);
    #endif
    case 'f': return sizeof(float) * (is_complex ? 2 : 1);
    case 'd': return sizeof(double) * (is_complex ? 2 : 1);
    case 'g': return sizeof(long double) * (is_complex ? 2 : 1);
    case 'O': case 'P': return sizeof(void*);
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
typedef struct { char c; short x; } __Pyx_st_short;
typedef struct { char c; int x; } __Pyx_st_int;
typedef struct { char c; long x; } __Pyx_st_long;
typedef struct { char c; float x; } __Pyx_st_float;
typedef struct { char c; double x; } __Pyx_st_double;
typedef struct { char c; long double x; } __Pyx_st_longdouble;
typedef struct { char c; void *x; } __Pyx_st_void_p;
#ifdef HAVE_LONG_LONG
typedef struct { char c; PY_LONG_LONG x; } __Pyx_st_longlong;
#endif
static size_t __Pyx_BufFmt_TypeCharToAlignment(char ch, CYTHON_UNUSED int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_st_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_st_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_st_long) - sizeof(long);
#ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(__Pyx_st_longlong) - sizeof(PY_LONG_LONG);
#endif
    case 'f': return sizeof(__Pyx_st_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_st_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_st_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_st_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
/* These are for computing the padding at the end of the struct to align
   on the first member of the struct. This will probably the same as above,
   but we don't have any guarantees.
 */
typedef struct { short x; char c; } __Pyx_pad_short;
typedef struct { int x; char c; } __Pyx_pad_int;
typedef struct { long x; char c; } __Pyx_pad_long;
typedef struct { float x; char c; } __Pyx_pad_float;
typedef struct { double x; char c; } __Pyx_pad_double;
typedef struct { long double x; char c; } __Pyx_pad_longdouble;
typedef struct { void *x; char c; } __Pyx_pad_void_p;
#ifdef HAVE_LONG_LONG
typedef struct { PY_LONG_LONG x; char c; } __Pyx_pad_longlong;
#endif
static size_t __Pyx_BufFmt_TypeCharToPadding(char ch, CYTHON_UNUSED int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_pad_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_pad_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_pad_long) - sizeof(long);
#ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(__Pyx_pad_longlong) - sizeof(PY_LONG_LONG);
#endif
    case 'f': return sizeof(__Pyx_pad_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_pad_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_pad_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_pad_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static char __Pyx_BufFmt_TypeCharToGroup(char ch, int is_complex) {
  switch (ch) {
    case 'c':
        return 'H';
    case 'b': case 'h': case 'i':
    case 'l': case 'q': case 's': case 'p':
        return 'I';
    case '?': case 'B': case 'H': case 'I': case 'L': case 'Q':
        return 'U';
    case 'f': case 'd': case 'g':
        return (is_complex ? 'C' : 'R');
    case 'O':
        return 'O';
    case 'P':
        return 'P';
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
static void __Pyx_BufFmt_RaiseExpected(__Pyx_BufFmt_Context* ctx) {
  if (ctx->head == NULL || ctx->head->field == &ctx->root) {
    const char* expected;
    const char* quote;
    if (ctx->head == NULL) {
      expected = "end";
      quote = "";
    } else {
      expected = ctx->head->field->type->name;
      quote = "'";
    }
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected %s%s%s but got %s",
                 quote, expected, quote,
                 __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex));
  } else {
    __Pyx_StructField* field = ctx->head->field;
    __Pyx_StructField* parent = (ctx->head - 1)->field;
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected '%s' but got %s in '%s.%s'",
                 field->type->name, __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex),
                 parent->type->name, field->name);
  }
}
static int __Pyx_BufFmt_ProcessTypeChunk(__Pyx_BufFmt_Context* ctx) {
  char group;
  size_t size, offset, arraysize = 1;
  if (ctx->enc_type == 0) return 0;
  if (ctx->head->field->type->arraysize[0]) {
    int i, ndim = 0;
    if (ctx->enc_type == 's' || ctx->enc_type == 'p') {
        ctx->is_valid_array = ctx->head->field->type->ndim == 1;
        ndim = 1;
        if (ctx->enc_count != ctx->head->field->type->arraysize[0]) {
            PyErr_Format(PyExc_ValueError,
                         "Expected a dimension of size %zu, got %zu",
                         ctx->head->field->type->arraysize[0], ctx->enc_count);
            return -1;
        }
    }
    if (!ctx->is_valid_array) {
      PyErr_Format(PyExc_ValueError, "Expected %d dimensions, got %d",
                   ctx->head->field->type->ndim, ndim);
      return -1;
    }
    for (i = 0; i < ctx->head->field->type->ndim; i++) {
      arraysize *= ctx->head->field->type->arraysize[i];
    }
    ctx->is_valid_array = 0;
    ctx->enc_count = 1;
  }
  group = __Pyx_BufFmt_TypeCharToGroup(ctx->enc_type, ctx->is_complex);
  do {
    __Pyx_StructField* field = ctx->head->field;
    __Pyx_TypeInfo* type = field->type;
    if (ctx->enc_packmode == '@' || ctx->enc_packmode == '^') {
      size = __Pyx_BufFmt_TypeCharToNativeSize(ctx->enc_type, ctx->is_complex);
    } else {
      size = __Pyx_BufFmt_TypeCharToStandardSize(ctx->enc_type, ctx->is_complex);
    }
    if (ctx->enc_packmode == '@') {
      size_t align_at = __Pyx_BufFmt_TypeCharToAlignment(ctx->enc_type, ctx->is_complex);
      size_t align_mod_offset;
      if (align_at == 0) return -1;
      align_mod_offset = ctx->fmt_offset % align_at;
      if (align_mod_offset > 0) ctx->fmt_offset += align_at - align_mod_offset;
      if (ctx->struct_alignment == 0)
          ctx->struct_alignment = __Pyx_BufFmt_TypeCharToPadding(ctx->enc_type,
                                                                 ctx->is_complex);
    }
    if (type->size != size || type->typegroup != group) {
      if (type->typegroup == 'C' && type->fields != NULL) {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        ++ctx->head;
        ctx->head->field = type->fields;
        ctx->head->parent_offset = parent_offset;
        continue;
      }
      if ((type->typegroup == 'H' || group == 'H') && type->size == size) {
      } else {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
      }
    }
    offset = ctx->head->parent_offset + field->offset;
    if (ctx->fmt_offset != offset) {
      PyErr_Format(PyExc_ValueError,
                   "Buffer dtype mismatch; next field is at offset %" CYTHON_FORMAT_SSIZE_T "d but %" CYTHON_FORMAT_SSIZE_T "d expected",
                   (Py_ssize_t)ctx->fmt_offset, (Py_ssize_t)offset);
      return -1;
    }
    ctx->fmt_offset += size;
    if (arraysize)
      ctx->fmt_offset += (arraysize - 1) * size;
    --ctx->enc_count;
    while (1) {
      if (field == &ctx->root) {
        ctx->head = NULL;
        if (ctx->enc_count != 0) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
        }
        break;
      }
      ctx->head->field = ++field;
      if (field->type == NULL) {
        --ctx->head;
        field = ctx->head->field;
        continue;
      } else if (field->type->typegroup == 'S') {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        if (field->type->fields->type == NULL) continue;
        field = field->type->fields;
        ++ctx->head;
        ctx->head->field = field;
        ctx->head->parent_offset = parent_offset;
        break;
      } else {
        break;
      }
    }
  } while (ctx->enc_count);
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  return 0;
}
static PyObject *
__pyx_buffmt_parse_array(__Pyx_BufFmt_Context* ctx, const char** tsp)
{
    const char *ts = *tsp;
    int i = 0, number, ndim;
    ++ts;
    if (ctx->new_count != 1) {
        PyErr_SetString(PyExc_ValueError,
                        "Cannot handle repeated arrays in format string");
        return NULL;
    }
    if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
    ndim = ctx->head->field->type->ndim;
    while (*ts && *ts != ')') {
        switch (*ts) {
            case ' ': case '\f': case '\r': case '\n': case '\t': case '\v':  continue;
            default:  break;
        }
        number = __Pyx_BufFmt_ExpectNumber(&ts);
        if (number == -1) return NULL;
        if (i < ndim && (size_t) number != ctx->head->field->type->arraysize[i])
            return PyErr_Format(PyExc_ValueError,
                        "Expected a dimension of size %zu, got %d",
                        ctx->head->field->type->arraysize[i], number);
        if (*ts != ',' && *ts != ')')
            return PyErr_Format(PyExc_ValueError,
                                "Expected a comma in format string, got '%c'", *ts);
        if (*ts == ',') ts++;
        i++;
    }
    if (i != ndim)
        return PyErr_Format(PyExc_ValueError, "Expected %d dimension(s), got %d",
                            ctx->head->field->type->ndim, i);
    if (!*ts) {
        PyErr_SetString(PyExc_ValueError,
                        "Unexpected end of format string, expected ')'");
        return NULL;
    }
    ctx->is_valid_array = 1;
    ctx->new_count = 1;
    *tsp = ++ts;
    return Py_None;
}
static const char* __Pyx_BufFmt_CheckString(__Pyx_BufFmt_Context* ctx, const char* ts) {
  int got_Z = 0;
  while (1) {
    switch(*ts) {
      case 0:
        if (ctx->enc_type != 0 && ctx->head == NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        if (ctx->head != NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        return ts;
      case ' ':
      case '\r':
      case '\n':
        ++ts;
        break;
      case '<':
        if (!__Pyx_Is_Little_Endian()) {
          PyErr_SetString(PyExc_ValueError, "Little-endian buffer not supported on big-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '>':
      case '!':
        if (__Pyx_Is_Little_Endian()) {
          PyErr_SetString(PyExc_ValueError, "Big-endian buffer not supported on little-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '=':
      case '@':
      case '^':
        ctx->new_packmode = *ts++;
        break;
      case 'T':
        {
          const char* ts_after_sub;
          size_t i, struct_count = ctx->new_count;
          size_t struct_alignment = ctx->struct_alignment;
          ctx->new_count = 1;
          ++ts;
          if (*ts != '{') {
            PyErr_SetString(PyExc_ValueError, "Buffer acquisition: Expected '{' after 'T'");
            return NULL;
          }
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          ctx->enc_count = 0;
          ctx->struct_alignment = 0;
          ++ts;
          ts_after_sub = ts;
          for (i = 0; i != struct_count; ++i) {
            ts_after_sub = __Pyx_BufFmt_CheckString(ctx, ts);
            if (!ts_after_sub) return NULL;
          }
          ts = ts_after_sub;
          if (struct_alignment) ctx->struct_alignment = struct_alignment;
        }
        break;
      case '}':
        {
          size_t alignment = ctx->struct_alignment;
          ++ts;
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          if (alignment && ctx->fmt_offset % alignment) {
            ctx->fmt_offset += alignment - (ctx->fmt_offset % alignment);
          }
        }
        return ts;
      case 'x':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->fmt_offset += ctx->new_count;
        ctx->new_count = 1;
        ctx->enc_count = 0;
        ctx->enc_type = 0;
        ctx->enc_packmode = ctx->new_packmode;
        ++ts;
        break;
      case 'Z':
        got_Z = 1;
        ++ts;
        if (*ts != 'f' && *ts != 'd' && *ts != 'g') {
          __Pyx_BufFmt_RaiseUnexpectedChar('Z');
          return NULL;
        }
        CYTHON_FALLTHROUGH;
      case '?': case 'c': case 'b': case 'B': case 'h': case 'H': case 'i': case 'I':
      case 'l': case 'L': case 'q': case 'Q':
      case 'f': case 'd': case 'g':
      case 'O': case 'p':
        if ((ctx->enc_type == *ts) && (got_Z == ctx->is_complex) &&
            (ctx->enc_packmode == ctx->new_packmode) && (!ctx->is_valid_array)) {
          ctx->enc_count += ctx->new_count;
          ctx->new_count = 1;
          got_Z = 0;
          ++ts;
          break;
        }
        CYTHON_FALLTHROUGH;
      case 's':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->enc_count = ctx->new_count;
        ctx->enc_packmode = ctx->new_packmode;
        ctx->enc_type = *ts;
        ctx->is_complex = got_Z;
        ++ts;
        ctx->new_count = 1;
        got_Z = 0;
        break;
      case ':':
        ++ts;
        while(*ts != ':') ++ts;
        ++ts;
        break;
      case '(':
        if (!__pyx_buffmt_parse_array(ctx, &ts)) return NULL;
        break;
      default:
        {
          int number = __Pyx_BufFmt_ExpectNumber(&ts);
          if (number == -1) return NULL;
          ctx->new_count = (size_t)number;
        }
    }
  }
}

/* BufferGetAndValidate */
    static CYTHON_INLINE void __Pyx_SafeReleaseBuffer(Py_buffer* info) {
  if (unlikely(info->buf == NULL)) return;
  if (info->suboffsets == __Pyx_minusones) info->suboffsets = NULL;
  __Pyx_ReleaseBuffer(info);
}
static void __Pyx_ZeroBuffer(Py_buffer* buf) {
  buf->buf = NULL;
  buf->obj = NULL;
  buf->strides = __Pyx_zeros;
  buf->shape = __Pyx_zeros;
  buf->suboffsets = __Pyx_minusones;
}
static int __Pyx__GetBufferAndValidate(
        Py_buffer* buf, PyObject* obj,  __Pyx_TypeInfo* dtype, int flags,
        int nd, int cast, __Pyx_BufFmt_StackElem* stack)
{
  buf->buf = NULL;
  if (unlikely(__Pyx_GetBuffer(obj, buf, flags) == -1)) {
    __Pyx_ZeroBuffer(buf);
    return -1;
  }
  if (unlikely(buf->ndim != nd)) {
    PyErr_Format(PyExc_ValueError,
                 "Buffer has wrong number of dimensions (expected %d, got %d)",
                 nd, buf->ndim);
    goto fail;
  }
  if (!cast) {
    __Pyx_BufFmt_Context ctx;
    __Pyx_BufFmt_Init(&ctx, stack, dtype);
    if (!__Pyx_BufFmt_CheckString(&ctx, buf->format)) goto fail;
  }
  if (unlikely((size_t)buf->itemsize != dtype->size)) {
    PyErr_Format(PyExc_ValueError,
      "Item size of buffer (%" CYTHON_FORMAT_SSIZE_T "d byte%s) does not match size of '%s' (%" CYTHON_FORMAT_SSIZE_T "d byte%s)",
      buf->itemsize, (buf->itemsize > 1) ? "s" : "",
      dtype->name, (Py_ssize_t)dtype->size, (dtype->size > 1) ? "s" : "");
    goto fail;
  }
  if (buf->suboffsets == NULL) buf->suboffsets = __Pyx_minusones;
  return 0;
fail:;
  __Pyx_SafeReleaseBuffer(buf);
  return -1;
}

/* BufferIndexError */
    static void __Pyx_RaiseBufferIndexError(int axis) {
  PyErr_Format(PyExc_IndexError,
     "Out of bounds on buffer access (axis %d)", axis);
}

/* PyErrFetchRestore */
    #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* WriteUnraisableException */
    static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* RaiseException */
    #if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* PyIntBinop */
    #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AndObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
            return PyInt_FromLong(a & b);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_and(op1, op2);
            }
        }
                x = a & b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla & llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    return (inplace ? PyNumber_InPlaceAnd : PyNumber_And)(op1, op2);
}
#endif

/* PyFloatBinop */
    #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyFloat_TrueDivideObjC(PyObject *op1, PyObject *op2, double floatval, int inplace, int zerodivision_check) {
    const double b = floatval;
    double a, result;
    (void)inplace;
    (void)zerodivision_check;
    if (likely(PyFloat_CheckExact(op1))) {
        a = PyFloat_AS_DOUBLE(op1);
        
    } else
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        a = (double) PyInt_AS_LONG(op1);
        
    } else
    #endif
    if (likely(PyLong_CheckExact(op1))) {
        #if CYTHON_USE_PYLONG_INTERNALS
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        switch (size) {
            case  0: a = 0.0; break;
            case -1: a = -(double) digits[0]; break;
            case  1: a = (double) digits[0]; break;
            case -2:
            case 2:
                if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (1 * PyLong_SHIFT < 53))) {
                    a = (double) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53) || (a < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -2)
                            a = -a;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            case -3:
            case 3:
                if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53))) {
                    a = (double) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53) || (a < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -3)
                            a = -a;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            case -4:
            case 4:
                if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53))) {
                    a = (double) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (4 * PyLong_SHIFT < 53) || (a < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -4)
                            a = -a;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            default:
        #else
        {
        #endif
            a = PyLong_AsDouble(op1);
            if (unlikely(a == -1.0 && PyErr_Occurred())) return NULL;
            
        }
    } else {
        return (inplace ? PyNumber_InPlaceTrueDivide : PyNumber_TrueDivide)(op1, op2);
    }
        
        PyFPE_START_PROTECT("divide", return NULL)
        result = a / b;
        PyFPE_END_PROTECT(result)
        return PyFloat_FromDouble(result);
}
#endif

/* PyFloatBinop */
      #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyFloat_SubtractObjC(PyObject *op1, PyObject *op2, double floatval, int inplace, int zerodivision_check) {
    const double b = floatval;
    double a, result;
    (void)inplace;
    (void)zerodivision_check;
    if (likely(PyFloat_CheckExact(op1))) {
        a = PyFloat_AS_DOUBLE(op1);
        
    } else
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        a = (double) PyInt_AS_LONG(op1);
        
    } else
    #endif
    if (likely(PyLong_CheckExact(op1))) {
        #if CYTHON_USE_PYLONG_INTERNALS
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        switch (size) {
            case  0: a = 0.0; break;
            case -1: a = -(double) digits[0]; break;
            case  1: a = (double) digits[0]; break;
            case -2:
            case 2:
                if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (1 * PyLong_SHIFT < 53))) {
                    a = (double) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53) || (a < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -2)
                            a = -a;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            case -3:
            case 3:
                if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53))) {
                    a = (double) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53) || (a < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -3)
                            a = -a;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            case -4:
            case 4:
                if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53))) {
                    a = (double) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (4 * PyLong_SHIFT < 53) || (a < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -4)
                            a = -a;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            default:
        #else
        {
        #endif
            a = PyLong_AsDouble(op1);
            if (unlikely(a == -1.0 && PyErr_Occurred())) return NULL;
            
        }
    } else {
        return (inplace ? PyNumber_InPlaceSubtract : PyNumber_Subtract)(op1, op2);
    }
        
        PyFPE_START_PROTECT("subtract", return NULL)
        result = a - b;
        PyFPE_END_PROTECT(result)
        return PyFloat_FromDouble(result);
}
#endif

/* SliceObject */
        static CYTHON_INLINE int __Pyx_PyObject_SetSlice(PyObject* obj, PyObject* value,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_ass_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_ass_slice(obj, cstart, cstop, value);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_ass_subscript))
#endif
    {
        int result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_ass_subscript(obj, py_slice, value);
#else
        result = value ? PyObject_SetItem(obj, py_slice, value) : PyObject_DelItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object does not support slice %.10s",
        Py_TYPE(obj)->tp_name, value ? "assignment" : "deletion");
bad:
    return -1;
}

/* BytesEquals */
        static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
        static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* PyFloatBinop */
        #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyFloat_SubtractCObj(PyObject *op1, PyObject *op2, double floatval, int inplace, int zerodivision_check) {
    const double a = floatval;
    double b, result;
    (void)inplace;
    (void)zerodivision_check;
    if (likely(PyFloat_CheckExact(op2))) {
        b = PyFloat_AS_DOUBLE(op2);
        
    } else
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op2))) {
        b = (double) PyInt_AS_LONG(op2);
        
    } else
    #endif
    if (likely(PyLong_CheckExact(op2))) {
        #if CYTHON_USE_PYLONG_INTERNALS
        const digit* digits = ((PyLongObject*)op2)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op2);
        switch (size) {
            case  0: b = 0.0; break;
            case -1: b = -(double) digits[0]; break;
            case  1: b = (double) digits[0]; break;
            case -2:
            case 2:
                if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (1 * PyLong_SHIFT < 53))) {
                    b = (double) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53) || (b < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -2)
                            b = -b;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            case -3:
            case 3:
                if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53))) {
                    b = (double) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53) || (b < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -3)
                            b = -b;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            case -4:
            case 4:
                if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53))) {
                    b = (double) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (4 * PyLong_SHIFT < 53) || (b < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -4)
                            b = -b;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            default:
        #else
        {
        #endif
            b = PyLong_AsDouble(op2);
            if (unlikely(b == -1.0 && PyErr_Occurred())) return NULL;
            
        }
    } else {
        return (inplace ? PyNumber_InPlaceSubtract : PyNumber_Subtract)(op1, op2);
    }
        
        PyFPE_START_PROTECT("subtract", return NULL)
        result = a - b;
        PyFPE_END_PROTECT(result)
        return PyFloat_FromDouble(result);
}
#endif

/* PyIntCompare */
          static CYTHON_INLINE PyObject* __Pyx_PyInt_NeObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED long inplace) {
    if (op1 == op2) {
        Py_RETURN_FALSE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a != b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = Py_SIZE(op1);
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        if (intval == 0) {
            if (size != 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
        } else if (intval < 0) {
            if (size >= 0)
                Py_RETURN_TRUE;
            intval = -intval;
            size = -size;
        } else {
            if (size <= 0)
                Py_RETURN_TRUE;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        if (unequal != 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
        if ((double)a != (double)b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    return (
        PyObject_RichCompare(op1, op2, Py_NE));
}

/* GetTopmostException */
          #if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
          #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* PyErrExceptionMatches */
          #if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetException */
          #if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* PyObject_GenericGetAttrNoDict */
          #if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
          #if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
          static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* PyObjectGetAttrStrNoError */
          static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
}

/* SetupReduce */
          static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* TypeImport */
          #ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* Import */
          static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* CLineInTraceback */
          #ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
          static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
          #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_srcfile = NULL;
    py_srcfile = PyString_FromString(filename);
    if (!py_srcfile) goto bad;
    #endif
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        if (!py_funcname) goto bad;
        #endif
    }
    #if PY_MAJOR_VERSION < 3
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    #else
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    #endif
    Py_XDECREF(py_funcname);  // XDECREF since it's only set on Py3 if cline
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_srcfile);
    #endif
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

#if PY_MAJOR_VERSION < 3
static int __Pyx_GetBuffer(PyObject *obj, Py_buffer *view, int flags) {
    if (PyObject_CheckBuffer(obj)) return PyObject_GetBuffer(obj, view, flags);
    PyErr_Format(PyExc_TypeError, "'%.200s' does not have the buffer interface", Py_TYPE(obj)->tp_name);
    return -1;
}
static void __Pyx_ReleaseBuffer(Py_buffer *view) {
    PyObject *obj = view->obj;
    if (!obj) return;
    if (PyObject_CheckBuffer(obj)) {
        PyBuffer_Release(view);
        return;
    }
    if ((0)) {}
    view->obj = NULL;
    Py_DECREF(obj);
}
#endif


          /* CIntFromPyVerify */
          #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* Declarations */
          #if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
          #if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabsf(b.real) >= fabsf(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                float r = b.imag / b.real;
                float s = (float)(1.0) / (b.real + b.imag * r);
                return __pyx_t_float_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            float r = b.real / b.imag;
            float s = (float)(1.0) / (b.imag + b.real * r);
            return __pyx_t_float_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            float denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_float_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_float(a, a);
                    case 3:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, a);
                    case 4:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = powf(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2f(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_float(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

/* Declarations */
          #if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
          #if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabs(b.real) >= fabs(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                double r = b.imag / b.real;
                double s = (double)(1.0) / (b.real + b.imag * r);
                return __pyx_t_double_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            double r = b.real / b.imag;
            double s = (double)(1.0) / (b.imag + b.real * r);
            return __pyx_t_double_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            double denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_double_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_double(a, a);
                    case 3:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, a);
                    case 4:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = pow(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_double(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

/* CIntFromPy */
          static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntToPy */
          static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
          static CYTHON_INLINE PyObject* __Pyx_PyInt_From_short(short value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const short neg_one = (short) -1, const_zero = (short) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(short) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(short) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(short) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(short) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(short) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(short),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
          static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
          static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FastTypeChecks */
          #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
          static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* InitStrings */
          static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
#if PY_MAJOR_VERSION < 3
  } else if (likely(PyInt_CheckExact(o))) {
    return PyInt_AS_LONG(o);
#endif
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyInt_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
