/* Generated by Cython 0.29.28 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [
            "/home/zsherman/anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/core/include/numpy/arrayobject.h",
            "/home/zsherman/anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/core/include/numpy/arrayscalars.h",
            "/home/zsherman/anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/core/include/numpy/ndarrayobject.h",
            "/home/zsherman/anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/core/include/numpy/ndarraytypes.h",
            "/home/zsherman/anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/core/include/numpy/ufuncobject.h"
        ],
        "include_dirs": [
            "/home/zsherman/anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/core/include"
        ],
        "libraries": [
            "m"
        ],
        "name": "pyart.map.ckdtree",
        "sources": [
            "pyart/map/ckdtree.pyx"
        ]
    },
    "module_name": "pyart.map.ckdtree"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_28"
#define CYTHON_HEX_VERSION 0x001D1CF0
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0 || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 0
  #elif !defined(CYTHON_FAST_THREAD_STATE)
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL (PY_VERSION_HEX < 0x030B00A1)
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 0
  #elif !defined(CYTHON_USE_EXC_INFO_STACK)
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_MAJOR_VERSION < 3
    #include "longintrepr.h"
  #endif
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_DefaultClassType PyType_Type
#if PY_VERSION_HEX >= 0x030B00A1
    static CYTHON_INLINE PyCodeObject* __Pyx_PyCode_New(int a, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
        PyObject *kwds=NULL, *argcount=NULL, *posonlyargcount=NULL, *kwonlyargcount=NULL;
        PyObject *nlocals=NULL, *stacksize=NULL, *flags=NULL, *replace=NULL, *call_result=NULL, *empty=NULL;
        const char *fn_cstr=NULL;
        const char *name_cstr=NULL;
        PyCodeObject* co=NULL;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        if (!(kwds=PyDict_New())) goto end;
        if (!(argcount=PyLong_FromLong(a))) goto end;
        if (PyDict_SetItemString(kwds, "co_argcount", argcount) != 0) goto end;
        if (!(posonlyargcount=PyLong_FromLong(0))) goto end;
        if (PyDict_SetItemString(kwds, "co_posonlyargcount", posonlyargcount) != 0) goto end;
        if (!(kwonlyargcount=PyLong_FromLong(k))) goto end;
        if (PyDict_SetItemString(kwds, "co_kwonlyargcount", kwonlyargcount) != 0) goto end;
        if (!(nlocals=PyLong_FromLong(l))) goto end;
        if (PyDict_SetItemString(kwds, "co_nlocals", nlocals) != 0) goto end;
        if (!(stacksize=PyLong_FromLong(s))) goto end;
        if (PyDict_SetItemString(kwds, "co_stacksize", stacksize) != 0) goto end;
        if (!(flags=PyLong_FromLong(f))) goto end;
        if (PyDict_SetItemString(kwds, "co_flags", flags) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_code", code) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_consts", c) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_names", n) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_varnames", v) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_freevars", fv) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_cellvars", cell) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_linetable", lnos) != 0) goto end;
        if (!(fn_cstr=PyUnicode_AsUTF8AndSize(fn, NULL))) goto end;
        if (!(name_cstr=PyUnicode_AsUTF8AndSize(name, NULL))) goto end;
        if (!(co = PyCode_NewEmpty(fn_cstr, name_cstr, fline))) goto end;
        if (!(replace = PyObject_GetAttrString((PyObject*)co, "replace"))) goto cleanup_code_too;
        if (!(empty = PyTuple_New(0))) goto cleanup_code_too; // unfortunately __pyx_empty_tuple isn't available here
        if (!(call_result = PyObject_Call(replace, empty, kwds))) goto cleanup_code_too;
        Py_XDECREF((PyObject*)co);
        co = (PyCodeObject*)call_result;
        call_result = NULL;
        if (0) {
            cleanup_code_too:
            Py_XDECREF((PyObject*)co);
            co = NULL;
        }
        end:
        Py_XDECREF(kwds);
        Py_XDECREF(argcount);
        Py_XDECREF(posonlyargcount);
        Py_XDECREF(kwonlyargcount);
        Py_XDECREF(nlocals);
        Py_XDECREF(stacksize);
        Py_XDECREF(replace);
        Py_XDECREF(call_result);
        Py_XDECREF(empty);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return co;
    }
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if defined(PyUnicode_IS_READY)
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #else
  #define __Pyx_PyUnicode_READY(op)       (0)
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if defined(PyUnicode_IS_READY) && defined(PyUnicode_GET_SIZE)
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
  #endif
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsHash_t
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__pyart__map__ckdtree
#define __PYX_HAVE_API__pyart__map__ckdtree
/* Early includes */
#include <string.h>
#include <stdio.h>
#include "numpy/arrayobject.h"
#include "numpy/ndarrayobject.h"
#include "numpy/ndarraytypes.h"
#include "numpy/arrayscalars.h"
#include "numpy/ufuncobject.h"

    /* NumPy API declarations from "numpy/__init__.pxd" */
    
#include <stdlib.h>
#include "limits.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;

/* Header.proto */
#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif defined(_Complex_I)
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif


static const char *__pyx_f[] = {
  "pyart/map/ckdtree.pyx",
  "stringsource",
  "__init__.pxd",
  "type.pxd",
};
/* BufferFormatStructs.proto */
#define IS_UNSIGNED(type) (((type) -1) > 0)
struct __Pyx_StructField_;
#define __PYX_BUF_FLAGS_PACKED_STRUCT (1 << 0)
typedef struct {
  const char* name;
  struct __Pyx_StructField_* fields;
  size_t size;
  size_t arraysize[8];
  int ndim;
  char typegroup;
  char is_unsigned;
  int flags;
} __Pyx_TypeInfo;
typedef struct __Pyx_StructField_ {
  __Pyx_TypeInfo* type;
  const char* name;
  size_t offset;
} __Pyx_StructField;
typedef struct {
  __Pyx_StructField* field;
  size_t parent_offset;
} __Pyx_BufFmt_StackElem;
typedef struct {
  __Pyx_StructField root;
  __Pyx_BufFmt_StackElem* head;
  size_t fmt_offset;
  size_t new_count, enc_count;
  size_t struct_alignment;
  int is_complex;
  char enc_type;
  char new_packmode;
  char enc_packmode;
  char is_valid_array;
} __Pyx_BufFmt_Context;


/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":690
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 */
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":691
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
 */
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":692
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * #ctypedef npy_int96      int96_t
 */
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":693
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_int96      int96_t
 * #ctypedef npy_int128     int128_t
 */
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":697
 * #ctypedef npy_int128     int128_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 */
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":698
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
 */
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":699
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * #ctypedef npy_uint96     uint96_t
 */
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":700
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_uint96     uint96_t
 * #ctypedef npy_uint128    uint128_t
 */
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":704
 * #ctypedef npy_uint128    uint128_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
 */
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":705
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
 */
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":714
 * # The int types are mapped a bit surprising --
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t
 */
typedef npy_long __pyx_t_5numpy_int_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":715
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   longlong_t
 * 
 */
typedef npy_longlong __pyx_t_5numpy_long_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":716
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_ulong      uint_t
 */
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":718
 * ctypedef npy_longlong   longlong_t
 * 
 * ctypedef npy_ulong      uint_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t
 */
typedef npy_ulong __pyx_t_5numpy_uint_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":719
 * 
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 */
typedef npy_ulonglong __pyx_t_5numpy_ulong_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":720
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
 */
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":722
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
 */
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":723
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
 */
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":725
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
 */
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":726
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
 */
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":727
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cfloat      cfloat_t
 */
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;
/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif
static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);

/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif
static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);


/*--- Type declarations ---*/
struct __pyx_obj_5pyart_3map_7ckdtree_heap;
struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries;
struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle;
struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker;
struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker;
struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":729
 * ctypedef npy_longdouble longdouble_t
 * 
 * ctypedef npy_cfloat      cfloat_t             # <<<<<<<<<<<<<<
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t
 */
typedef npy_cfloat __pyx_t_5numpy_cfloat_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":730
 * 
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t             # <<<<<<<<<<<<<<
 * ctypedef npy_clongdouble clongdouble_t
 * 
 */
typedef npy_cdouble __pyx_t_5numpy_cdouble_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":731
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cdouble     complex_t
 */
typedef npy_clongdouble __pyx_t_5numpy_clongdouble_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":733
 * ctypedef npy_clongdouble clongdouble_t
 * 
 * ctypedef npy_cdouble     complex_t             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 */
typedef npy_cdouble __pyx_t_5numpy_complex_t;
union __pyx_t_5pyart_3map_7ckdtree_heapcontents;
struct __pyx_t_5pyart_3map_7ckdtree_heapitem;
struct __pyx_t_5pyart_3map_7ckdtree_RR_stack_item;
struct __pyx_t_5pyart_3map_7ckdtree_RP_stack_item;
struct __pyx_t_5pyart_3map_7ckdtree_innernode;
struct __pyx_t_5pyart_3map_7ckdtree_leafnode;
struct __pyx_t_5pyart_3map_7ckdtree_nodeinfo;

/* "pyart/map/ckdtree.pyx":124
 * # Priority queue
 * # ==============
 * cdef union heapcontents:    # FIXME: Unions are not always portable, verify this             # <<<<<<<<<<<<<<
 *     np.intp_t intdata     # union is never used in an ABI dependent way.
 *     char* ptrdata
 */
union __pyx_t_5pyart_3map_7ckdtree_heapcontents {
  __pyx_t_5numpy_intp_t intdata;
  char *ptrdata;
};

/* "pyart/map/ckdtree.pyx":128
 *     char* ptrdata
 * 
 * cdef struct heapitem:             # <<<<<<<<<<<<<<
 *     np.float64_t priority
 *     heapcontents contents
 */
struct __pyx_t_5pyart_3map_7ckdtree_heapitem {
  __pyx_t_5numpy_float64_t priority;
  union __pyx_t_5pyart_3map_7ckdtree_heapcontents contents;
};

/* "pyart/map/ckdtree.pyx":448
 * # dist_tracker.pop()
 * 
 * cdef struct RR_stack_item:             # <<<<<<<<<<<<<<
 *     np.intp_t which
 *     np.intp_t split_dim
 */
struct __pyx_t_5pyart_3map_7ckdtree_RR_stack_item {
  __pyx_t_5numpy_intp_t which;
  __pyx_t_5numpy_intp_t split_dim;
  double min_along_dim;
  double max_along_dim;
  __pyx_t_5numpy_float64_t min_distance;
  __pyx_t_5numpy_float64_t max_distance;
};

/* "pyart/map/ckdtree.pyx":631
 * # dist_tracker.pop()
 * 
 * cdef struct RP_stack_item:             # <<<<<<<<<<<<<<
 *     np.intp_t split_dim
 *     double min_along_dim, max_along_dim
 */
struct __pyx_t_5pyart_3map_7ckdtree_RP_stack_item {
  __pyx_t_5numpy_intp_t split_dim;
  double min_along_dim;
  double max_along_dim;
  __pyx_t_5numpy_float64_t min_distance;
  __pyx_t_5numpy_float64_t max_distance;
};

/* "pyart/map/ckdtree.pyx":767
 * # Tree structure
 * # ==============
 * cdef struct innernode:             # <<<<<<<<<<<<<<
 *     np.intp_t split_dim
 *     np.intp_t children
 */
struct __pyx_t_5pyart_3map_7ckdtree_innernode {
  __pyx_t_5numpy_intp_t split_dim;
  __pyx_t_5numpy_intp_t children;
  __pyx_t_5numpy_float64_t split;
  struct __pyx_t_5pyart_3map_7ckdtree_innernode *less;
  struct __pyx_t_5pyart_3map_7ckdtree_innernode *greater;
};

/* "pyart/map/ckdtree.pyx":774
 *     innernode* greater
 * 
 * cdef struct leafnode:             # <<<<<<<<<<<<<<
 *     np.intp_t split_dim
 *     np.intp_t children
 */
struct __pyx_t_5pyart_3map_7ckdtree_leafnode {
  __pyx_t_5numpy_intp_t split_dim;
  __pyx_t_5numpy_intp_t children;
  __pyx_t_5numpy_intp_t start_idx;
  __pyx_t_5numpy_intp_t end_idx;
};

/* "pyart/map/ckdtree.pyx":784
 * # malloc sizeof(nodeinfo)+self.m*sizeof(np.float64_t) bytes.
 * 
 * cdef struct nodeinfo:             # <<<<<<<<<<<<<<
 *     innernode* node
 *     np.float64_t side_distances[0]  # FIXME: Only valid in C99, invalid C++ and C89
 */
struct __pyx_t_5pyart_3map_7ckdtree_nodeinfo {
  struct __pyx_t_5pyart_3map_7ckdtree_innernode *node;
  __pyx_t_5numpy_float64_t side_distances[0];
};

/* "pyart/map/ckdtree.pyx":132
 *     heapcontents contents
 * 
 * cdef class heap(object):             # <<<<<<<<<<<<<<
 *     cdef np.intp_t n
 *     cdef heapitem* heap
 */
struct __pyx_obj_5pyart_3map_7ckdtree_heap {
  PyObject_HEAD
  struct __pyx_vtabstruct_5pyart_3map_7ckdtree_heap *__pyx_vtab;
  __pyx_t_5numpy_intp_t n;
  struct __pyx_t_5pyart_3map_7ckdtree_heapitem *heap;
  __pyx_t_5numpy_intp_t space;
};


/* "pyart/map/ckdtree.pyx":237
 * 
 * # Utility for building a coo matrix incrementally
 * cdef class coo_entries:             # <<<<<<<<<<<<<<
 *     cdef:
 *         np.intp_t n, n_max
 */
struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries {
  PyObject_HEAD
  struct __pyx_vtabstruct_5pyart_3map_7ckdtree_coo_entries *__pyx_vtab;
  __pyx_t_5numpy_intp_t n;
  __pyx_t_5numpy_intp_t n_max;
  PyArrayObject *i;
  PyArrayObject *j;
  PyArrayObject *v;
  __pyx_t_5numpy_intp_t *i_data;
  __pyx_t_5numpy_intp_t *j_data;
  __pyx_t_5numpy_float64_t *v_data;
};


/* "pyart/map/ckdtree.pyx":326
 * # ===================
 * 
 * cdef class Rectangle:             # <<<<<<<<<<<<<<
 *     cdef np.intp_t m
 *     cdef np.float64_t *mins
 */
struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle {
  PyObject_HEAD
  __pyx_t_5numpy_intp_t m;
  __pyx_t_5numpy_float64_t *mins;
  __pyx_t_5numpy_float64_t *maxes;
  PyArrayObject *mins_arr;
  PyArrayObject *maxes_arr;
};


/* "pyart/map/ckdtree.pyx":457
 * cdef np.intp_t GREATER = 2
 * 
 * cdef class RectRectDistanceTracker(object):             # <<<<<<<<<<<<<<
 *     cdef Rectangle rect1, rect2
 *     cdef np.float64_t p, epsfac, upper_bound
 */
struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker {
  PyObject_HEAD
  struct __pyx_vtabstruct_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_vtab;
  struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *rect1;
  struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *rect2;
  __pyx_t_5numpy_float64_t p;
  __pyx_t_5numpy_float64_t epsfac;
  __pyx_t_5numpy_float64_t upper_bound;
  __pyx_t_5numpy_float64_t min_distance;
  __pyx_t_5numpy_float64_t max_distance;
  __pyx_t_5numpy_intp_t stack_size;
  __pyx_t_5numpy_intp_t stack_max_size;
  struct __pyx_t_5pyart_3map_7ckdtree_RR_stack_item *stack;
};


/* "pyart/map/ckdtree.pyx":636
 *     np.float64_t min_distance, max_distance
 * 
 * cdef class PointRectDistanceTracker(object):             # <<<<<<<<<<<<<<
 *     cdef Rectangle rect
 *     cdef np.float64_t *pt
 */
struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker {
  PyObject_HEAD
  struct __pyx_vtabstruct_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_vtab;
  struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *rect;
  __pyx_t_5numpy_float64_t *pt;
  __pyx_t_5numpy_float64_t p;
  __pyx_t_5numpy_float64_t epsfac;
  __pyx_t_5numpy_float64_t upper_bound;
  __pyx_t_5numpy_float64_t min_distance;
  __pyx_t_5numpy_float64_t max_distance;
  __pyx_t_5numpy_intp_t stack_size;
  __pyx_t_5numpy_intp_t stack_max_size;
  struct __pyx_t_5pyart_3map_7ckdtree_RP_stack_item *stack;
};


/* "pyart/map/ckdtree.pyx":791
 * # Main class
 * # ==========
 * cdef class cKDTree:             # <<<<<<<<<<<<<<
 *     """
 *     cKDTree(data, int leafsize=10)
 */
struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree {
  PyObject_HEAD
  struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *__pyx_vtab;
  struct __pyx_t_5pyart_3map_7ckdtree_innernode *tree;
  PyArrayObject *data;
  __pyx_t_5numpy_float64_t *raw_data;
  __pyx_t_5numpy_intp_t n;
  __pyx_t_5numpy_intp_t m;
  __pyx_t_5numpy_intp_t leafsize;
  PyArrayObject *maxes;
  __pyx_t_5numpy_float64_t *raw_maxes;
  PyArrayObject *mins;
  __pyx_t_5numpy_float64_t *raw_mins;
  PyArrayObject *indices;
  __pyx_t_5numpy_intp_t *raw_indices;
};



/* "pyart/map/ckdtree.pyx":132
 *     heapcontents contents
 * 
 * cdef class heap(object):             # <<<<<<<<<<<<<<
 *     cdef np.intp_t n
 *     cdef heapitem* heap
 */

struct __pyx_vtabstruct_5pyart_3map_7ckdtree_heap {
  int (*_resize)(struct __pyx_obj_5pyart_3map_7ckdtree_heap *, __pyx_t_5numpy_intp_t);
  int (*push)(struct __pyx_obj_5pyart_3map_7ckdtree_heap *, struct __pyx_t_5pyart_3map_7ckdtree_heapitem);
  struct __pyx_t_5pyart_3map_7ckdtree_heapitem (*peek)(struct __pyx_obj_5pyart_3map_7ckdtree_heap *);
  int (*remove)(struct __pyx_obj_5pyart_3map_7ckdtree_heap *);
  int (*pop)(struct __pyx_obj_5pyart_3map_7ckdtree_heap *, struct __pyx_t_5pyart_3map_7ckdtree_heapitem *);
};
static struct __pyx_vtabstruct_5pyart_3map_7ckdtree_heap *__pyx_vtabptr_5pyart_3map_7ckdtree_heap;
static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_4heap__resize(struct __pyx_obj_5pyart_3map_7ckdtree_heap *, __pyx_t_5numpy_intp_t);
static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_4heap_push(struct __pyx_obj_5pyart_3map_7ckdtree_heap *, struct __pyx_t_5pyart_3map_7ckdtree_heapitem);


/* "pyart/map/ckdtree.pyx":237
 * 
 * # Utility for building a coo matrix incrementally
 * cdef class coo_entries:             # <<<<<<<<<<<<<<
 *     cdef:
 *         np.intp_t n, n_max
 */

struct __pyx_vtabstruct_5pyart_3map_7ckdtree_coo_entries {
  void (*add)(struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_float64_t);
};
static struct __pyx_vtabstruct_5pyart_3map_7ckdtree_coo_entries *__pyx_vtabptr_5pyart_3map_7ckdtree_coo_entries;


/* "pyart/map/ckdtree.pyx":457
 * cdef np.intp_t GREATER = 2
 * 
 * cdef class RectRectDistanceTracker(object):             # <<<<<<<<<<<<<<
 *     cdef Rectangle rect1, rect2
 *     cdef np.float64_t p, epsfac, upper_bound
 */

struct __pyx_vtabstruct_5pyart_3map_7ckdtree_RectRectDistanceTracker {
  int (*_init_stack)(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *);
  int (*_resize_stack)(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *, __pyx_t_5numpy_intp_t);
  int (*_free_stack)(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *);
  int (*push)(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_float64_t);
  int (*push_less_of)(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *, __pyx_t_5numpy_intp_t, struct __pyx_t_5pyart_3map_7ckdtree_innernode *);
  int (*push_greater_of)(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *, __pyx_t_5numpy_intp_t, struct __pyx_t_5pyart_3map_7ckdtree_innernode *);
  int (*pop)(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *);
};
static struct __pyx_vtabstruct_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_vtabptr_5pyart_3map_7ckdtree_RectRectDistanceTracker;
static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_less_of(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *, __pyx_t_5numpy_intp_t, struct __pyx_t_5pyart_3map_7ckdtree_innernode *);
static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_greater_of(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *, __pyx_t_5numpy_intp_t, struct __pyx_t_5pyart_3map_7ckdtree_innernode *);
static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *);


/* "pyart/map/ckdtree.pyx":636
 *     np.float64_t min_distance, max_distance
 * 
 * cdef class PointRectDistanceTracker(object):             # <<<<<<<<<<<<<<
 *     cdef Rectangle rect
 *     cdef np.float64_t *pt
 */

struct __pyx_vtabstruct_5pyart_3map_7ckdtree_PointRectDistanceTracker {
  int (*_init_stack)(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *);
  int (*_resize_stack)(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *, __pyx_t_5numpy_intp_t);
  int (*_free_stack)(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *);
  PyObject *(*init)(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *, __pyx_t_5numpy_float64_t *, struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *, __pyx_t_5numpy_float64_t, __pyx_t_5numpy_float64_t, __pyx_t_5numpy_float64_t);
  int (*push)(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_float64_t);
  int (*push_less_of)(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *);
  int (*push_greater_of)(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *);
  int (*pop)(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *);
};
static struct __pyx_vtabstruct_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_vtabptr_5pyart_3map_7ckdtree_PointRectDistanceTracker;
static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker_push_less_of(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *);
static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker_push_greater_of(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *);
static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker_pop(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *);


/* "pyart/map/ckdtree.pyx":791
 * # Main class
 * # ==========
 * cdef class cKDTree:             # <<<<<<<<<<<<<<
 *     """
 *     cKDTree(data, int leafsize=10)
 */

struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree {
  struct __pyx_t_5pyart_3map_7ckdtree_innernode *(*__pyx___build)(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_float64_t *, __pyx_t_5numpy_float64_t *);
  PyObject *(*__pyx___free_tree)(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *);
  int (*__pyx___query)(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, __pyx_t_5numpy_float64_t *, __pyx_t_5numpy_intp_t *, __pyx_t_5numpy_float64_t *, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_float64_t, __pyx_t_5numpy_float64_t, __pyx_t_5numpy_float64_t);
  int (*__pyx___query_ball_point_traverse_no_checking)(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, PyObject *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *);
  int (*__pyx___query_ball_point_traverse_checking)(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, PyObject *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *, struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *);
  PyObject *(*__pyx___query_ball_point)(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, __pyx_t_5numpy_float64_t *, __pyx_t_5numpy_float64_t, __pyx_t_5numpy_float64_t, __pyx_t_5numpy_float64_t);
  int (*__pyx___query_ball_tree_traverse_no_checking)(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, PyObject *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *);
  int (*__pyx___query_ball_tree_traverse_checking)(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, PyObject *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *, struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *);
  int (*__pyx___query_pairs_traverse_no_checking)(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, PyObject *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *);
  int (*__pyx___query_pairs_traverse_checking)(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, PyObject *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *, struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *);
  int (*__pyx___count_neighbors_traverse)(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_float64_t *, __pyx_t_5numpy_intp_t *, __pyx_t_5numpy_intp_t *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *, struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *);
  int (*__pyx___sparse_distance_matrix_traverse)(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *, struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *);
};
static struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *__pyx_vtabptr_5pyart_3map_7ckdtree_cKDTree;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
#if CYTHON_FAST_PYCALL
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif // CYTHON_FAST_PYCALL
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* PyObjectFormatAndDecref.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatSimpleAndDecref(PyObject* s, PyObject* f);
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatAndDecref(PyObject* s, PyObject* f);

/* IncludeStringH.proto */
#include <string.h>

/* JoinPyUnicode.proto */
static PyObject* __Pyx_PyUnicode_Join(PyObject* value_tuple, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* KeywordStringCheck.proto */
static int __Pyx_CheckKeywordStrings(PyObject *kwdict, const char* function_name, int kw_allowed);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* IsLittleEndian.proto */
static CYTHON_INLINE int __Pyx_Is_Little_Endian(void);

/* BufferFormatCheck.proto */
static const char* __Pyx_BufFmt_CheckString(__Pyx_BufFmt_Context* ctx, const char* ts);
static void __Pyx_BufFmt_Init(__Pyx_BufFmt_Context* ctx,
                              __Pyx_BufFmt_StackElem* stack,
                              __Pyx_TypeInfo* type);

/* BufferGetAndValidate.proto */
#define __Pyx_GetBufferAndValidate(buf, obj, dtype, flags, nd, cast, stack)\
    ((obj == Py_None || obj == NULL) ?\
    (__Pyx_ZeroBuffer(buf), 0) :\
    __Pyx__GetBufferAndValidate(buf, obj, dtype, flags, nd, cast, stack))
static int  __Pyx__GetBufferAndValidate(Py_buffer* buf, PyObject* obj,
    __Pyx_TypeInfo* dtype, int flags, int nd, int cast, __Pyx_BufFmt_StackElem* stack);
static void __Pyx_ZeroBuffer(Py_buffer* buf);
static CYTHON_INLINE void __Pyx_SafeReleaseBuffer(Py_buffer* info);
static Py_ssize_t __Pyx_minusones[] = { -1, -1, -1, -1, -1, -1, -1, -1 };
static Py_ssize_t __Pyx_zeros[] = { 0, 0, 0, 0, 0, 0, 0, 0 };

/* BufferFallbackError.proto */
static void __Pyx_RaiseBufferFallbackError(void);

#define __Pyx_BufPtrStrided2d(type, buf, i0, s0, i1, s1) (type)((char*)buf + i0 * s0 + i1 * s1)
/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* BufferIndexError.proto */
static void __Pyx_RaiseBufferIndexError(int axis);

#define __Pyx_BufPtrCContig1d(type, buf, i0, s0) ((type)buf + i0)
/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

#define __Pyx_BufPtrStrided1d(type, buf, i0, s0) (type)((char*)buf + i0 * s0)
/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* BufferStructDeclare.proto */
typedef struct {
  Py_ssize_t shape, strides, suboffsets;
} __Pyx_Buf_DimInfo;
typedef struct {
  size_t refcount;
  Py_buffer pybuffer;
} __Pyx_Buffer;
typedef struct {
  __Pyx_Buffer *rcbuffer;
  char *data;
  __Pyx_Buf_DimInfo diminfo[8];
} __Pyx_LocalBuf_ND;

#if PY_MAJOR_VERSION < 3
    static int __Pyx_GetBuffer(PyObject *obj, Py_buffer *view, int flags);
    static void __Pyx_ReleaseBuffer(Py_buffer *view);
#else
    #define __Pyx_GetBuffer PyObject_GetBuffer
    #define __Pyx_ReleaseBuffer PyBuffer_Release
#endif


/* GCCDiagnostics.proto */
#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* RealImag.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if defined(__cplusplus) && CYTHON_CCOMPLEX\
        && (defined(_WIN32) || defined(__clang__) || (defined(__GNUC__) && (__GNUC__ >= 5 || __GNUC__ == 4 && __GNUC_MINOR__ >= 4 )) || __cplusplus >= 201103)
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_float(a, b)   ((a)==(b))
    #define __Pyx_c_sum_float(a, b)  ((a)+(b))
    #define __Pyx_c_diff_float(a, b) ((a)-(b))
    #define __Pyx_c_prod_float(a, b) ((a)*(b))
    #define __Pyx_c_quot_float(a, b) ((a)/(b))
    #define __Pyx_c_neg_float(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_float(z) ((z)==(float)0)
    #define __Pyx_c_conj_float(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (::std::abs(z))
        #define __Pyx_c_pow_float(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_float(z) ((z)==0)
    #define __Pyx_c_conj_float(z)    (conjf(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (cabsf(z))
        #define __Pyx_c_pow_float(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_double(a, b)   ((a)==(b))
    #define __Pyx_c_sum_double(a, b)  ((a)+(b))
    #define __Pyx_c_diff_double(a, b) ((a)-(b))
    #define __Pyx_c_prod_double(a, b) ((a)*(b))
    #define __Pyx_c_quot_double(a, b) ((a)/(b))
    #define __Pyx_c_neg_double(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_double(z) ((z)==(double)0)
    #define __Pyx_c_conj_double(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (::std::abs(z))
        #define __Pyx_c_pow_double(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_double(z) ((z)==0)
    #define __Pyx_c_conj_double(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (cabs(z))
        #define __Pyx_c_pow_double(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE Py_intptr_t __Pyx_PyInt_As_Py_intptr_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_Py_intptr_t(Py_intptr_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_4heap__resize(struct __pyx_obj_5pyart_3map_7ckdtree_heap *__pyx_v_self, __pyx_t_5numpy_intp_t __pyx_v_new_space); /* proto*/
static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_4heap_push(struct __pyx_obj_5pyart_3map_7ckdtree_heap *__pyx_v_self, struct __pyx_t_5pyart_3map_7ckdtree_heapitem __pyx_v_item); /* proto*/
static struct __pyx_t_5pyart_3map_7ckdtree_heapitem __pyx_f_5pyart_3map_7ckdtree_4heap_peek(struct __pyx_obj_5pyart_3map_7ckdtree_heap *__pyx_v_self); /* proto*/
static int __pyx_f_5pyart_3map_7ckdtree_4heap_remove(struct __pyx_obj_5pyart_3map_7ckdtree_heap *__pyx_v_self); /* proto*/
static int __pyx_f_5pyart_3map_7ckdtree_4heap_pop(struct __pyx_obj_5pyart_3map_7ckdtree_heap *__pyx_v_self, struct __pyx_t_5pyart_3map_7ckdtree_heapitem *__pyx_v_it); /* proto*/
static void __pyx_f_5pyart_3map_7ckdtree_11coo_entries_add(struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *__pyx_v_self, __pyx_t_5numpy_intp_t __pyx_v_i, __pyx_t_5numpy_intp_t __pyx_v_j, __pyx_t_5numpy_float64_t __pyx_v_v); /* proto*/
static int __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker__init_stack(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_self); /* proto*/
static int __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker__resize_stack(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_self, __pyx_t_5numpy_intp_t __pyx_v_new_max_size); /* proto*/
static int __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker__free_stack(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_self); /* proto*/
static int __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_self, __pyx_t_5numpy_intp_t __pyx_v_which, __pyx_t_5numpy_intp_t __pyx_v_direction, __pyx_t_5numpy_intp_t __pyx_v_split_dim, __pyx_t_5numpy_float64_t __pyx_v_split_val); /* proto*/
static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_less_of(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_self, __pyx_t_5numpy_intp_t __pyx_v_which, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node); /* proto*/
static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_greater_of(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_self, __pyx_t_5numpy_intp_t __pyx_v_which, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node); /* proto*/
static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_self); /* proto*/
static int __pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker__init_stack(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_v_self); /* proto*/
static int __pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker__resize_stack(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_v_self, __pyx_t_5numpy_intp_t __pyx_v_new_max_size); /* proto*/
static int __pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker__free_stack(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker_init(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_v_self, __pyx_t_5numpy_float64_t *__pyx_v_pt, struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_rect, __pyx_t_5numpy_float64_t __pyx_v_p, __pyx_t_5numpy_float64_t __pyx_v_eps, __pyx_t_5numpy_float64_t __pyx_v_upper_bound); /* proto*/
static int __pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker_push(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_v_self, __pyx_t_5numpy_intp_t __pyx_v_direction, __pyx_t_5numpy_intp_t __pyx_v_split_dim, __pyx_t_5numpy_float64_t __pyx_v_split_val); /* proto*/
static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker_push_less_of(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_v_self, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node); /* proto*/
static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker_push_greater_of(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_v_self, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node); /* proto*/
static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker_pop(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_v_self); /* proto*/
static struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_f_5pyart_3map_7ckdtree_7cKDTree___build(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, __pyx_t_5numpy_intp_t __pyx_v_start_idx, __pyx_t_5numpy_intp_t __pyx_v_end_idx, __pyx_t_5numpy_float64_t *__pyx_v_maxes, __pyx_t_5numpy_float64_t *__pyx_v_mins); /* proto*/
static PyObject *__pyx_f_5pyart_3map_7ckdtree_7cKDTree___free_tree(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node); /* proto*/
static int __pyx_f_5pyart_3map_7ckdtree_7cKDTree___query(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, __pyx_t_5numpy_float64_t *__pyx_v_result_distances, __pyx_t_5numpy_intp_t *__pyx_v_result_indices, __pyx_t_5numpy_float64_t *__pyx_v_x, __pyx_t_5numpy_intp_t __pyx_v_k, __pyx_t_5numpy_float64_t __pyx_v_eps, __pyx_t_5numpy_float64_t __pyx_v_p, __pyx_t_5numpy_float64_t __pyx_v_distance_upper_bound); /* proto*/
static int __pyx_f_5pyart_3map_7ckdtree_7cKDTree___query_ball_point_traverse_no_checking(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, PyObject *__pyx_v_results, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node); /* proto*/
static int __pyx_f_5pyart_3map_7ckdtree_7cKDTree___query_ball_point_traverse_checking(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, PyObject *__pyx_v_results, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node, struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_v_tracker); /* proto*/
static PyObject *__pyx_f_5pyart_3map_7ckdtree_7cKDTree___query_ball_point(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, __pyx_t_5numpy_float64_t *__pyx_v_x, __pyx_t_5numpy_float64_t __pyx_v_r, __pyx_t_5numpy_float64_t __pyx_v_p, __pyx_t_5numpy_float64_t __pyx_v_eps); /* proto*/
static int __pyx_f_5pyart_3map_7ckdtree_7cKDTree___query_ball_tree_traverse_no_checking(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_other, PyObject *__pyx_v_results, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node1, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node2); /* proto*/
static int __pyx_f_5pyart_3map_7ckdtree_7cKDTree___query_ball_tree_traverse_checking(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_other, PyObject *__pyx_v_results, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node1, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node2, struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_tracker); /* proto*/
static int __pyx_f_5pyart_3map_7ckdtree_7cKDTree___query_pairs_traverse_no_checking(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, PyObject *__pyx_v_results, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node1, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node2); /* proto*/
static int __pyx_f_5pyart_3map_7ckdtree_7cKDTree___query_pairs_traverse_checking(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, PyObject *__pyx_v_results, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node1, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node2, struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_tracker); /* proto*/
static int __pyx_f_5pyart_3map_7ckdtree_7cKDTree___count_neighbors_traverse(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_other, __pyx_t_5numpy_intp_t __pyx_v_n_queries, __pyx_t_5numpy_float64_t *__pyx_v_r, __pyx_t_5numpy_intp_t *__pyx_v_results, __pyx_t_5numpy_intp_t *__pyx_v_idx, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node1, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node2, struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_tracker); /* proto*/
static int __pyx_f_5pyart_3map_7ckdtree_7cKDTree___sparse_distance_matrix_traverse(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_other, struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *__pyx_v_results, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node1, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node2, struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_tracker); /* proto*/

/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'numpy' */

/* Module declarations from 'numpy' */
static PyTypeObject *__pyx_ptype_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_5numpy_flatiter = 0;
static PyTypeObject *__pyx_ptype_5numpy_broadcast = 0;
static PyTypeObject *__pyx_ptype_5numpy_ndarray = 0;
static PyTypeObject *__pyx_ptype_5numpy_generic = 0;
static PyTypeObject *__pyx_ptype_5numpy_number = 0;
static PyTypeObject *__pyx_ptype_5numpy_integer = 0;
static PyTypeObject *__pyx_ptype_5numpy_signedinteger = 0;
static PyTypeObject *__pyx_ptype_5numpy_unsignedinteger = 0;
static PyTypeObject *__pyx_ptype_5numpy_inexact = 0;
static PyTypeObject *__pyx_ptype_5numpy_floating = 0;
static PyTypeObject *__pyx_ptype_5numpy_complexfloating = 0;
static PyTypeObject *__pyx_ptype_5numpy_flexible = 0;
static PyTypeObject *__pyx_ptype_5numpy_character = 0;
static PyTypeObject *__pyx_ptype_5numpy_ufunc = 0;

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'cython' */

/* Module declarations from 'pyart.map.ckdtree' */
static PyTypeObject *__pyx_ptype_5pyart_3map_7ckdtree_heap = 0;
static PyTypeObject *__pyx_ptype_5pyart_3map_7ckdtree_coo_entries = 0;
static PyTypeObject *__pyx_ptype_5pyart_3map_7ckdtree_Rectangle = 0;
static PyTypeObject *__pyx_ptype_5pyart_3map_7ckdtree_RectRectDistanceTracker = 0;
static PyTypeObject *__pyx_ptype_5pyart_3map_7ckdtree_PointRectDistanceTracker = 0;
static PyTypeObject *__pyx_ptype_5pyart_3map_7ckdtree_cKDTree = 0;
static __pyx_t_5numpy_float64_t __pyx_v_5pyart_3map_7ckdtree_infinity;
static __pyx_t_5numpy_intp_t __pyx_v_5pyart_3map_7ckdtree_LESS;
static __pyx_t_5numpy_intp_t __pyx_v_5pyart_3map_7ckdtree_GREATER;
static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_set_add_ordered_pair(PyObject *, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_intp_t); /*proto*/
static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_list_append(PyObject *, __pyx_t_5numpy_intp_t); /*proto*/
static CYTHON_INLINE __pyx_t_5numpy_float64_t __pyx_f_5pyart_3map_7ckdtree_dmax(__pyx_t_5numpy_float64_t, __pyx_t_5numpy_float64_t); /*proto*/
static CYTHON_INLINE __pyx_t_5numpy_float64_t __pyx_f_5pyart_3map_7ckdtree_dabs(__pyx_t_5numpy_float64_t); /*proto*/
static CYTHON_INLINE __pyx_t_5numpy_float64_t __pyx_f_5pyart_3map_7ckdtree__distance_p(__pyx_t_5numpy_float64_t *, __pyx_t_5numpy_float64_t *, __pyx_t_5numpy_float64_t, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_float64_t); /*proto*/
static CYTHON_INLINE __pyx_t_5numpy_float64_t __pyx_f_5pyart_3map_7ckdtree_min_dist_point_interval_p(__pyx_t_5numpy_float64_t *, struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_float64_t); /*proto*/
static CYTHON_INLINE __pyx_t_5numpy_float64_t __pyx_f_5pyart_3map_7ckdtree_max_dist_point_interval_p(__pyx_t_5numpy_float64_t *, struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_float64_t); /*proto*/
static CYTHON_INLINE __pyx_t_5numpy_float64_t __pyx_f_5pyart_3map_7ckdtree_min_dist_interval_interval_p(struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *, struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_float64_t); /*proto*/
static CYTHON_INLINE __pyx_t_5numpy_float64_t __pyx_f_5pyart_3map_7ckdtree_max_dist_interval_interval_p(struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *, struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_float64_t); /*proto*/
static CYTHON_INLINE __pyx_t_5numpy_float64_t __pyx_f_5pyart_3map_7ckdtree_min_dist_point_rect_p_inf(__pyx_t_5numpy_float64_t *, struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *); /*proto*/
static CYTHON_INLINE __pyx_t_5numpy_float64_t __pyx_f_5pyart_3map_7ckdtree_max_dist_point_rect_p_inf(__pyx_t_5numpy_float64_t *, struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *); /*proto*/
static CYTHON_INLINE __pyx_t_5numpy_float64_t __pyx_f_5pyart_3map_7ckdtree_min_dist_rect_rect_p_inf(struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *, struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *); /*proto*/
static CYTHON_INLINE __pyx_t_5numpy_float64_t __pyx_f_5pyart_3map_7ckdtree_max_dist_rect_rect_p_inf(struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *, struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *); /*proto*/
static __Pyx_TypeInfo __Pyx_TypeInfo_nn___pyx_t_5numpy_float64_t = { "float64_t", NULL, sizeof(__pyx_t_5numpy_float64_t), { 0 }, 0, 'R', 0, 0 };
static __Pyx_TypeInfo __Pyx_TypeInfo_nn___pyx_t_5numpy_intp_t = { "intp_t", NULL, sizeof(__pyx_t_5numpy_intp_t), { 0 }, 0, IS_UNSIGNED(__pyx_t_5numpy_intp_t) ? 'U' : 'I', IS_UNSIGNED(__pyx_t_5numpy_intp_t), 0 };
#define __Pyx_MODULE_NAME "pyart.map.ckdtree"
extern int __pyx_module_is_main_pyart__map__ckdtree;
int __pyx_module_is_main_pyart__map__ckdtree = 0;

/* Implementation of 'pyart.map.ckdtree' */
static PyObject *__pyx_builtin_MemoryError;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_ImportError;
static const char __pyx_k_C[] = "C";
static const char __pyx_k_d[] = "d";
static const char __pyx_k_k[] = "k";
static const char __pyx_k_p[] = "p";
static const char __pyx_k_r[] = "r";
static const char __pyx_k_x[] = "x";
static const char __pyx_k_np[] = "np";
static const char __pyx_k_all[] = "__all__";
static const char __pyx_k_eps[] = "eps";
static const char __pyx_k_inf[] = "inf";
static const char __pyx_k_amax[] = "amax";
static const char __pyx_k_amin[] = "amin";
static const char __pyx_k_axis[] = "axis";
static const char __pyx_k_data[] = "data";
static const char __pyx_k_fill[] = "fill";
static const char __pyx_k_heap[] = "heap";
static const char __pyx_k_intp[] = "intp";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_prod[] = "prod";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_array[] = "array";
static const char __pyx_k_dtype[] = "dtype";
static const char __pyx_k_empty[] = "empty";
static const char __pyx_k_numpy[] = "numpy";
static const char __pyx_k_order[] = "order";
static const char __pyx_k_other[] = "other";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_rect1[] = "rect1";
static const char __pyx_k_rect2[] = "rect2";
static const char __pyx_k_scipy[] = "scipy";
static const char __pyx_k_shape[] = "shape";
static const char __pyx_k_todok[] = "todok";
static const char __pyx_k_zeros[] = "zeros";
static const char __pyx_k_arange[] = "arange";
static const char __pyx_k_astype[] = "astype";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_object[] = "object";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_resize[] = "resize";
static const char __pyx_k_sparse[] = "sparse";
static const char __pyx_k_asarray[] = "asarray";
static const char __pyx_k_cKDTree[] = "cKDTree";
static const char __pyx_k_float64[] = "float64";
static const char __pyx_k_ndindex[] = "ndindex";
static const char __pyx_k_newaxis[] = "newaxis";
static const char __pyx_k_reshape[] = "reshape";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_leafsize[] = "leafsize";
static const char __pyx_k_mins_arr[] = "mins_arr";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_Rectangle[] = "Rectangle";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_maxes_arr[] = "maxes_arr";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_to_matrix[] = "to_matrix";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_coo_matrix[] = "coo_matrix";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_ImportError[] = "ImportError";
static const char __pyx_k_MemoryError[] = "MemoryError";
static const char __pyx_k_coo_entries[] = "coo_entries";
static const char __pyx_k_upper_bound[] = "upper_bound";
static const char __pyx_k_but_hasshape[] = " but hasshape ";
static const char __pyx_k_initial_size[] = "initial_size";
static const char __pyx_k_max_distance[] = "max_distance";
static const char __pyx_k_scipy_sparse[] = "scipy.sparse";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_Heap_containing[] = "Heap containing ";
static const char __pyx_k_Searching_for_a[] = "Searching for a ";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_ascontiguousarray[] = "ascontiguousarray";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_dimensional_KDTree[] = "-dimensional KDTree";
static const char __pyx_k_distance_upper_bound[] = "distance_upper_bound";
static const char __pyx_k_dimensional_point_in_a[] = "-dimensional point in a ";
static const char __pyx_k_RectRectDistanceTracker[] = "RectRectDistanceTracker";
static const char __pyx_k_PointRectDistanceTracker[] = "PointRectDistanceTracker";
static const char __pyx_k_items_cannot_be_resized_to[] = " items cannot be resized to ";
static const char __pyx_k_leafsize_must_be_at_least_1[] = "leafsize must be at least 1";
static const char __pyx_k_Copyright_c_2001_2002_Enthought[] = "\nCopyright (c) 2001, 2002 Enthought, Inc.\nAll rights reserved.\n\nCopyright (c) 2003-2012 SciPy Developers.\nAll rights reserved.\n\nRedistribution and use in source and binary forms, with or without\nmodification, are permitted provided that the following conditions are met:\n\n  a. Redistributions of source code must retain the above copyright notice,\n     this list of conditions and the following disclaimer.\n  b. Redistributions in binary form must reproduce the above copyright\n     notice, this list of conditions and the following disclaimer in the\n     documentation and/or other materials provided with the distribution.\n  c. Neither the name of Enthought nor the names of the SciPy Developers\n     may be used to endorse or promote products derived from this software\n     without specific prior written permission.\n\n\nTHIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS \"AS IS\"\nAND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE\nIMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE\nARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR\nANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL\nDAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR\nSERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER\nCAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT\nLIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY\nOUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH\nDAMAGE.\n\n";
static const char __pyx_k_numpy_core_multiarray_failed_to[] = "numpy.core.multiarray failed to import";
static const char __pyx_k_query_ball_point_self_x_r_p_eps[] = "query_ball_point(self, x, r, p, eps)\n        \n        Find all points within distance r of point(s) x.\n\n        Parameters\n        ----------\n        x : array_like, shape tuple + (self.m,)\n            The point or points to search for neighbors of.\n        r : positive float\n            The radius of points to return.\n        p : float, optional\n            Which Minkowski p-norm to use.  Should be in the range [1, inf].\n        eps : nonnegative float, optional\n            Approximate search. Branches of the tree are not explored if their\n            nearest points are further than ``r / (1 + eps)``, and branches are\n            added in bulk if their furthest points are nearer than\n            ``r * (1 + eps)``.\n\n        Returns\n        -------\n        results : list or array of lists\n            If `x` is a single point, returns a list of the indices of the\n            neighbors of `x`. If `x` is an array of points, returns an object\n            array of shape tuple containing lists of neighbors.\n\n        Notes\n        -----\n        If you have many points whose neighbors you want to find, you may save\n        substantial amounts of time by putting them in a cKDTree and using\n        query_ball_tree.\n\n        Examples\n        --------\n        >>> from scipy import spatial\n        >>> x, y = np.mgrid[0:4, 0:4]\n        >>> points = zip(x.ravel(), y.ravel())\n        >>> tree = spatial.cKDTree(points)\n        >>> tree.query_ball_point([2, 0], 1)\n        [4, 8, 9, 12]\n\n        ";
static const char __pyx_k_r_must_be_either_a_single_value[] = "r must be either a single value or a one-dimensional array of values";
static const char __pyx_k_Only_p_norms_with_1_p_infinity_p[] = "Only p-norms with 1<=p<=infinity permitted";
static const char __pyx_k_Trees_passed_to_query_ball_trees[] = "Trees passed to query_ball_trees have different dimensionality";
static const char __pyx_k_cKDTree_query_ball_point_line_13[] = "cKDTree.query_ball_point (line 1383)";
static const char __pyx_k_numpy_core_umath_failed_to_impor[] = "numpy.core.umath failed to import";
static const char __pyx_k_rect1_and_rect2_have_different_d[] = "rect1 and rect2 have different dimensions";
static const char __pyx_k_self_heap_cannot_be_converted_to[] = "self.heap cannot be converted to a Python object for pickling";
static const char __pyx_k_self_i_data_self_j_data_self_v_d[] = "self.i_data,self.j_data,self.v_data cannot be converted to a Python object for pickling";
static const char __pyx_k_self_maxes_self_mins_cannot_be_c[] = "self.maxes,self.mins cannot be converted to a Python object for pickling";
static const char __pyx_k_self_pt_self_stack_cannot_be_con[] = "self.pt,self.stack cannot be converted to a Python object for pickling";
static const char __pyx_k_self_raw_data_self_raw_indices_s[] = "self.raw_data,self.raw_indices,self.raw_maxes,self.raw_mins,self.tree cannot be converted to a Python object for pickling";
static const char __pyx_k_self_stack_cannot_be_converted_t[] = "self.stack cannot be converted to a Python object for pickling";
static const char __pyx_k_x_must_consist_of_vectors_of_len[] = "x must consist of vectors of length ";
static PyObject *__pyx_n_u_C;
static PyObject *__pyx_kp_u_Heap_containing;
static PyObject *__pyx_n_s_ImportError;
static PyObject *__pyx_n_s_MemoryError;
static PyObject *__pyx_kp_u_Only_p_norms_with_1_p_infinity_p;
static PyObject *__pyx_n_s_PointRectDistanceTracker;
static PyObject *__pyx_n_s_RectRectDistanceTracker;
static PyObject *__pyx_n_s_Rectangle;
static PyObject *__pyx_kp_u_Searching_for_a;
static PyObject *__pyx_kp_u_Trees_passed_to_query_ball_trees;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_n_s_all;
static PyObject *__pyx_n_s_amax;
static PyObject *__pyx_n_s_amin;
static PyObject *__pyx_n_s_arange;
static PyObject *__pyx_n_s_array;
static PyObject *__pyx_n_s_asarray;
static PyObject *__pyx_n_s_ascontiguousarray;
static PyObject *__pyx_n_s_astype;
static PyObject *__pyx_n_s_axis;
static PyObject *__pyx_kp_u_but_hasshape;
static PyObject *__pyx_n_s_cKDTree;
static PyObject *__pyx_n_u_cKDTree;
static PyObject *__pyx_kp_u_cKDTree_query_ball_point_line_13;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_coo_entries;
static PyObject *__pyx_n_s_coo_matrix;
static PyObject *__pyx_n_u_d;
static PyObject *__pyx_n_s_data;
static PyObject *__pyx_kp_u_dimensional_KDTree;
static PyObject *__pyx_kp_u_dimensional_point_in_a;
static PyObject *__pyx_n_s_distance_upper_bound;
static PyObject *__pyx_n_s_dtype;
static PyObject *__pyx_n_s_empty;
static PyObject *__pyx_n_s_eps;
static PyObject *__pyx_n_s_fill;
static PyObject *__pyx_n_s_float64;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_heap;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_inf;
static PyObject *__pyx_n_s_initial_size;
static PyObject *__pyx_n_s_intp;
static PyObject *__pyx_kp_u_items_cannot_be_resized_to;
static PyObject *__pyx_n_s_k;
static PyObject *__pyx_n_s_leafsize;
static PyObject *__pyx_kp_u_leafsize_must_be_at_least_1;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_max_distance;
static PyObject *__pyx_n_s_maxes_arr;
static PyObject *__pyx_n_s_mins_arr;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_ndindex;
static PyObject *__pyx_n_s_newaxis;
static PyObject *__pyx_n_s_np;
static PyObject *__pyx_n_s_numpy;
static PyObject *__pyx_kp_u_numpy_core_multiarray_failed_to;
static PyObject *__pyx_kp_u_numpy_core_umath_failed_to_impor;
static PyObject *__pyx_n_s_object;
static PyObject *__pyx_n_s_order;
static PyObject *__pyx_n_s_other;
static PyObject *__pyx_n_s_p;
static PyObject *__pyx_n_s_prod;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_kp_u_query_ball_point_self_x_r_p_eps;
static PyObject *__pyx_n_s_r;
static PyObject *__pyx_kp_u_r_must_be_either_a_single_value;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_rect1;
static PyObject *__pyx_kp_u_rect1_and_rect2_have_different_d;
static PyObject *__pyx_n_s_rect2;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_reshape;
static PyObject *__pyx_n_s_resize;
static PyObject *__pyx_n_s_scipy;
static PyObject *__pyx_n_s_scipy_sparse;
static PyObject *__pyx_kp_s_self_heap_cannot_be_converted_to;
static PyObject *__pyx_kp_s_self_i_data_self_j_data_self_v_d;
static PyObject *__pyx_kp_s_self_maxes_self_mins_cannot_be_c;
static PyObject *__pyx_kp_s_self_pt_self_stack_cannot_be_con;
static PyObject *__pyx_kp_s_self_raw_data_self_raw_indices_s;
static PyObject *__pyx_kp_s_self_stack_cannot_be_converted_t;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_shape;
static PyObject *__pyx_n_s_sparse;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_to_matrix;
static PyObject *__pyx_n_s_todok;
static PyObject *__pyx_n_s_upper_bound;
static PyObject *__pyx_n_s_x;
static PyObject *__pyx_kp_u_x_must_consist_of_vectors_of_len;
static PyObject *__pyx_n_s_zeros;
static int __pyx_pf_5pyart_3map_7ckdtree_4heap___init__(struct __pyx_obj_5pyart_3map_7ckdtree_heap *__pyx_v_self, __pyx_t_5numpy_intp_t __pyx_v_initial_size); /* proto */
static void __pyx_pf_5pyart_3map_7ckdtree_4heap_2__dealloc__(struct __pyx_obj_5pyart_3map_7ckdtree_heap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_3map_7ckdtree_4heap_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_3map_7ckdtree_heap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_3map_7ckdtree_4heap_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_3map_7ckdtree_heap *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_5pyart_3map_7ckdtree_11coo_entries___init__(struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_3map_7ckdtree_11coo_entries_2to_matrix(struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *__pyx_v_self, PyObject *__pyx_v_shape); /* proto */
static PyObject *__pyx_pf_5pyart_3map_7ckdtree_11coo_entries_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_3map_7ckdtree_11coo_entries_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_5pyart_3map_7ckdtree_9Rectangle___init__(struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_self, PyObject *__pyx_v_mins_arr, PyObject *__pyx_v_maxes_arr); /* proto */
static PyObject *__pyx_pf_5pyart_3map_7ckdtree_9Rectangle_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_3map_7ckdtree_9Rectangle_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_5pyart_3map_7ckdtree_23RectRectDistanceTracker___init__(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_self, struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_rect1, struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_rect2, __pyx_t_5numpy_float64_t __pyx_v_p, __pyx_t_5numpy_float64_t __pyx_v_eps, __pyx_t_5numpy_float64_t __pyx_v_upper_bound); /* proto */
static void __pyx_pf_5pyart_3map_7ckdtree_23RectRectDistanceTracker_2__dealloc__(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_3map_7ckdtree_23RectRectDistanceTracker_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_3map_7ckdtree_23RectRectDistanceTracker_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static void __pyx_pf_5pyart_3map_7ckdtree_24PointRectDistanceTracker___dealloc__(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_3map_7ckdtree_24PointRectDistanceTracker_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_3map_7ckdtree_24PointRectDistanceTracker_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_5pyart_3map_7ckdtree_7cKDTree___init__(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, PyObject *__pyx_v_data, __pyx_t_5numpy_intp_t __pyx_v_leafsize); /* proto */
static void __pyx_pf_5pyart_3map_7ckdtree_7cKDTree_2__dealloc__(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_4query(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, PyObject *__pyx_v_x, __pyx_t_5numpy_intp_t __pyx_v_k, __pyx_t_5numpy_float64_t __pyx_v_eps, __pyx_t_5numpy_float64_t __pyx_v_p, __pyx_t_5numpy_float64_t __pyx_v_distance_upper_bound); /* proto */
static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_6query_ball_point(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, PyObject *__pyx_v_x, __pyx_t_5numpy_float64_t __pyx_v_r, __pyx_t_5numpy_float64_t __pyx_v_p, __pyx_t_5numpy_float64_t __pyx_v_eps); /* proto */
static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_8query_ball_tree(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_other, __pyx_t_5numpy_float64_t __pyx_v_r, __pyx_t_5numpy_float64_t __pyx_v_p, __pyx_t_5numpy_float64_t __pyx_v_eps); /* proto */
static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_10query_pairs(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, __pyx_t_5numpy_float64_t __pyx_v_r, __pyx_t_5numpy_float64_t __pyx_v_p, __pyx_t_5numpy_float64_t __pyx_v_eps); /* proto */
static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_12count_neighbors(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_other, PyObject *__pyx_v_r, __pyx_t_5numpy_float64_t __pyx_v_p); /* proto */
static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_14sparse_distance_matrix(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_other, __pyx_t_5numpy_float64_t __pyx_v_max_distance, __pyx_t_5numpy_float64_t __pyx_v_p); /* proto */
static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_4data___get__(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_1n___get__(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_1m___get__(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_8leafsize___get__(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_5maxes___get__(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_4mins___get__(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_16__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_18__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_5pyart_3map_7ckdtree_heap(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pyart_3map_7ckdtree_coo_entries(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pyart_3map_7ckdtree_Rectangle(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pyart_3map_7ckdtree_RectRectDistanceTracker(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pyart_3map_7ckdtree_PointRectDistanceTracker(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pyart_3map_7ckdtree_cKDTree(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_float_0_0;
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_12;
static PyObject *__pyx_int_neg_1;
static __pyx_t_5numpy_float64_t __pyx_k__14;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_slice__16;
static PyObject *__pyx_slice__18;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__20;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__22;
static PyObject *__pyx_tuple__23;
static PyObject *__pyx_tuple__24;
/* Late includes */

/* "pyart/map/ckdtree.pyx":77
 * # test is optimized away.
 * 
 * cdef inline int set_add_pair(set results,             # <<<<<<<<<<<<<<
 *                              np.intp_t i,
 *                              np.intp_t j) except -1:
 */

static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_set_add_pair(PyObject *__pyx_v_results, __pyx_t_5numpy_intp_t __pyx_v_i, __pyx_t_5numpy_intp_t __pyx_v_j) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_add_pair", 0);

  /* "pyart/map/ckdtree.pyx":81
 *                              np.intp_t j) except -1:
 * 
 *     if sizeof(long) < sizeof(np.intp_t):             # <<<<<<<<<<<<<<
 *         # Win 64
 *         results.add((int(i), int(j)))
 */
  __pyx_t_1 = (((sizeof(long)) < (sizeof(__pyx_t_5numpy_intp_t))) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":83
 *     if sizeof(long) < sizeof(np.intp_t):
 *         # Win 64
 *         results.add((int(i), int(j)))             # <<<<<<<<<<<<<<
 *     else:
 *         # Other platforms
 */
    if (unlikely(__pyx_v_results == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "add");
      __PYX_ERR(0, 83, __pyx_L1_error)
    }
    __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_i); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 83, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyInt_Type)), __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 83, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_j); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 83, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyInt_Type)), __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 83, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 83, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_5 = PySet_Add(__pyx_v_results, __pyx_t_2); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 83, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyart/map/ckdtree.pyx":81
 *                              np.intp_t j) except -1:
 * 
 *     if sizeof(long) < sizeof(np.intp_t):             # <<<<<<<<<<<<<<
 *         # Win 64
 *         results.add((int(i), int(j)))
 */
    goto __pyx_L3;
  }

  /* "pyart/map/ckdtree.pyx":86
 *     else:
 *         # Other platforms
 *         results.add((i, j))             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  /*else*/ {
    if (unlikely(__pyx_v_results == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "add");
      __PYX_ERR(0, 86, __pyx_L1_error)
    }
    __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_i); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 86, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_j); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 86, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 86, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_t_5 = PySet_Add(__pyx_v_results, __pyx_t_3); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 86, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "pyart/map/ckdtree.pyx":87
 *         # Other platforms
 *         results.add((i, j))
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":77
 * # test is optimized away.
 * 
 * cdef inline int set_add_pair(set results,             # <<<<<<<<<<<<<<
 *                              np.intp_t i,
 *                              np.intp_t j) except -1:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyart.map.ckdtree.set_add_pair", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":90
 * 
 * 
 * cdef inline int set_add_ordered_pair(set results,             # <<<<<<<<<<<<<<
 *                                      np.intp_t i,
 *                                      np.intp_t j) except -1:
 */

static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_set_add_ordered_pair(PyObject *__pyx_v_results, __pyx_t_5numpy_intp_t __pyx_v_i, __pyx_t_5numpy_intp_t __pyx_v_j) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_add_ordered_pair", 0);

  /* "pyart/map/ckdtree.pyx":94
 *                                      np.intp_t j) except -1:
 * 
 *     if sizeof(long) < sizeof(np.intp_t):             # <<<<<<<<<<<<<<
 *         # Win 64
 *         if i < j:
 */
  __pyx_t_1 = (((sizeof(long)) < (sizeof(__pyx_t_5numpy_intp_t))) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":96
 *     if sizeof(long) < sizeof(np.intp_t):
 *         # Win 64
 *         if i < j:             # <<<<<<<<<<<<<<
 *             results.add((int(i), int(j)))
 *         else:
 */
    __pyx_t_1 = ((__pyx_v_i < __pyx_v_j) != 0);
    if (__pyx_t_1) {

      /* "pyart/map/ckdtree.pyx":97
 *         # Win 64
 *         if i < j:
 *             results.add((int(i), int(j)))             # <<<<<<<<<<<<<<
 *         else:
 *             results.add((int(j), int(i)))
 */
      if (unlikely(__pyx_v_results == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "add");
        __PYX_ERR(0, 97, __pyx_L1_error)
      }
      __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_i); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 97, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyInt_Type)), __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 97, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_j); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 97, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyInt_Type)), __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 97, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 97, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
      __pyx_t_3 = 0;
      __pyx_t_4 = 0;
      __pyx_t_5 = PySet_Add(__pyx_v_results, __pyx_t_2); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 97, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "pyart/map/ckdtree.pyx":96
 *     if sizeof(long) < sizeof(np.intp_t):
 *         # Win 64
 *         if i < j:             # <<<<<<<<<<<<<<
 *             results.add((int(i), int(j)))
 *         else:
 */
      goto __pyx_L4;
    }

    /* "pyart/map/ckdtree.pyx":99
 *             results.add((int(i), int(j)))
 *         else:
 *             results.add((int(j), int(i)))             # <<<<<<<<<<<<<<
 *     else:
 *         # Other platforms
 */
    /*else*/ {
      if (unlikely(__pyx_v_results == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "add");
        __PYX_ERR(0, 99, __pyx_L1_error)
      }
      __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_j); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 99, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyInt_Type)), __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 99, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_i); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 99, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyInt_Type)), __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 99, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 99, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
      __pyx_t_4 = 0;
      __pyx_t_3 = 0;
      __pyx_t_5 = PySet_Add(__pyx_v_results, __pyx_t_2); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 99, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __pyx_L4:;

    /* "pyart/map/ckdtree.pyx":94
 *                                      np.intp_t j) except -1:
 * 
 *     if sizeof(long) < sizeof(np.intp_t):             # <<<<<<<<<<<<<<
 *         # Win 64
 *         if i < j:
 */
    goto __pyx_L3;
  }

  /* "pyart/map/ckdtree.pyx":102
 *     else:
 *         # Other platforms
 *         if i < j:             # <<<<<<<<<<<<<<
 *             results.add((i, j))
 *         else:
 */
  /*else*/ {
    __pyx_t_1 = ((__pyx_v_i < __pyx_v_j) != 0);
    if (__pyx_t_1) {

      /* "pyart/map/ckdtree.pyx":103
 *         # Other platforms
 *         if i < j:
 *             results.add((i, j))             # <<<<<<<<<<<<<<
 *         else:
 *             results.add((j, i))
 */
      if (unlikely(__pyx_v_results == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "add");
        __PYX_ERR(0, 103, __pyx_L1_error)
      }
      __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_i); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 103, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_j); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 103, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 103, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_2);
      PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2);
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3);
      __pyx_t_2 = 0;
      __pyx_t_3 = 0;
      __pyx_t_5 = PySet_Add(__pyx_v_results, __pyx_t_4); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 103, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyart/map/ckdtree.pyx":102
 *     else:
 *         # Other platforms
 *         if i < j:             # <<<<<<<<<<<<<<
 *             results.add((i, j))
 *         else:
 */
      goto __pyx_L5;
    }

    /* "pyart/map/ckdtree.pyx":105
 *             results.add((i, j))
 *         else:
 *             results.add((j, i))             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
    /*else*/ {
      if (unlikely(__pyx_v_results == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "add");
        __PYX_ERR(0, 105, __pyx_L1_error)
      }
      __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_j); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 105, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_i); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 105, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 105, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
      __pyx_t_4 = 0;
      __pyx_t_3 = 0;
      __pyx_t_5 = PySet_Add(__pyx_v_results, __pyx_t_2); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 105, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __pyx_L5:;
  }
  __pyx_L3:;

  /* "pyart/map/ckdtree.pyx":106
 *         else:
 *             results.add((j, i))
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef inline int list_append(list results, np.intp_t i) except -1:
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":90
 * 
 * 
 * cdef inline int set_add_ordered_pair(set results,             # <<<<<<<<<<<<<<
 *                                      np.intp_t i,
 *                                      np.intp_t j) except -1:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyart.map.ckdtree.set_add_ordered_pair", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":108
 *     return 0
 * 
 * cdef inline int list_append(list results, np.intp_t i) except -1:             # <<<<<<<<<<<<<<
 *     if sizeof(long) < sizeof(np.intp_t):
 *         # Win 64
 */

static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_list_append(PyObject *__pyx_v_results, __pyx_t_5numpy_intp_t __pyx_v_i) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("list_append", 0);

  /* "pyart/map/ckdtree.pyx":109
 * 
 * cdef inline int list_append(list results, np.intp_t i) except -1:
 *     if sizeof(long) < sizeof(np.intp_t):             # <<<<<<<<<<<<<<
 *         # Win 64
 *         if i <= <np.intp_t>LONG_MAX:  # CHECK COMPARISON DIRECTION
 */
  __pyx_t_1 = (((sizeof(long)) < (sizeof(__pyx_t_5numpy_intp_t))) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":111
 *     if sizeof(long) < sizeof(np.intp_t):
 *         # Win 64
 *         if i <= <np.intp_t>LONG_MAX:  # CHECK COMPARISON DIRECTION             # <<<<<<<<<<<<<<
 *             results.append(int(i))
 *         else:
 */
    __pyx_t_1 = ((__pyx_v_i <= ((__pyx_t_5numpy_intp_t)LONG_MAX)) != 0);
    if (__pyx_t_1) {

      /* "pyart/map/ckdtree.pyx":112
 *         # Win 64
 *         if i <= <np.intp_t>LONG_MAX:  # CHECK COMPARISON DIRECTION
 *             results.append(int(i))             # <<<<<<<<<<<<<<
 *         else:
 *             results.append(i)
 */
      if (unlikely(__pyx_v_results == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
        __PYX_ERR(0, 112, __pyx_L1_error)
      }
      __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_i); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 112, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyInt_Type)), __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 112, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_4 = __Pyx_PyList_Append(__pyx_v_results, __pyx_t_3); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 112, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "pyart/map/ckdtree.pyx":111
 *     if sizeof(long) < sizeof(np.intp_t):
 *         # Win 64
 *         if i <= <np.intp_t>LONG_MAX:  # CHECK COMPARISON DIRECTION             # <<<<<<<<<<<<<<
 *             results.append(int(i))
 *         else:
 */
      goto __pyx_L4;
    }

    /* "pyart/map/ckdtree.pyx":114
 *             results.append(int(i))
 *         else:
 *             results.append(i)             # <<<<<<<<<<<<<<
 *     else:
 *         # Other platforms
 */
    /*else*/ {
      if (unlikely(__pyx_v_results == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
        __PYX_ERR(0, 114, __pyx_L1_error)
      }
      __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_i); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 114, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = __Pyx_PyList_Append(__pyx_v_results, __pyx_t_3); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 114, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __pyx_L4:;

    /* "pyart/map/ckdtree.pyx":109
 * 
 * cdef inline int list_append(list results, np.intp_t i) except -1:
 *     if sizeof(long) < sizeof(np.intp_t):             # <<<<<<<<<<<<<<
 *         # Win 64
 *         if i <= <np.intp_t>LONG_MAX:  # CHECK COMPARISON DIRECTION
 */
    goto __pyx_L3;
  }

  /* "pyart/map/ckdtree.pyx":117
 *     else:
 *         # Other platforms
 *         results.append(i)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  /*else*/ {
    if (unlikely(__pyx_v_results == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
      __PYX_ERR(0, 117, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_i); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 117, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyList_Append(__pyx_v_results, __pyx_t_3); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 117, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "pyart/map/ckdtree.pyx":118
 *         # Other platforms
 *         results.append(i)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":108
 *     return 0
 * 
 * cdef inline int list_append(list results, np.intp_t i) except -1:             # <<<<<<<<<<<<<<
 *     if sizeof(long) < sizeof(np.intp_t):
 *         # Win 64
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pyart.map.ckdtree.list_append", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":137
 *     cdef np.intp_t space
 * 
 *     def __init__(heap self, np.intp_t initial_size):             # <<<<<<<<<<<<<<
 *         cdef void *tmp
 *         self.space = initial_size
 */

/* Python wrapper */
static int __pyx_pw_5pyart_3map_7ckdtree_4heap_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pyart_3map_7ckdtree_4heap_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __pyx_t_5numpy_intp_t __pyx_v_initial_size;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_initial_size,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_initial_size)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 137, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_initial_size = __Pyx_PyInt_As_Py_intptr_t(values[0]); if (unlikely((__pyx_v_initial_size == ((npy_intp)-1)) && PyErr_Occurred())) __PYX_ERR(0, 137, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 137, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.heap.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_4heap___init__(((struct __pyx_obj_5pyart_3map_7ckdtree_heap *)__pyx_v_self), __pyx_v_initial_size);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_3map_7ckdtree_4heap___init__(struct __pyx_obj_5pyart_3map_7ckdtree_heap *__pyx_v_self, __pyx_t_5numpy_intp_t __pyx_v_initial_size) {
  void *__pyx_v_tmp;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pyart/map/ckdtree.pyx":139
 *     def __init__(heap self, np.intp_t initial_size):
 *         cdef void *tmp
 *         self.space = initial_size             # <<<<<<<<<<<<<<
 *         self.heap = <heapitem*> NULL
 *         tmp = stdlib.malloc(sizeof(heapitem)*self.space)
 */
  __pyx_v_self->space = __pyx_v_initial_size;

  /* "pyart/map/ckdtree.pyx":140
 *         cdef void *tmp
 *         self.space = initial_size
 *         self.heap = <heapitem*> NULL             # <<<<<<<<<<<<<<
 *         tmp = stdlib.malloc(sizeof(heapitem)*self.space)
 *         if tmp == NULL:
 */
  __pyx_v_self->heap = ((struct __pyx_t_5pyart_3map_7ckdtree_heapitem *)NULL);

  /* "pyart/map/ckdtree.pyx":141
 *         self.space = initial_size
 *         self.heap = <heapitem*> NULL
 *         tmp = stdlib.malloc(sizeof(heapitem)*self.space)             # <<<<<<<<<<<<<<
 *         if tmp == NULL:
 *             raise MemoryError
 */
  __pyx_v_tmp = malloc(((sizeof(struct __pyx_t_5pyart_3map_7ckdtree_heapitem)) * __pyx_v_self->space));

  /* "pyart/map/ckdtree.pyx":142
 *         self.heap = <heapitem*> NULL
 *         tmp = stdlib.malloc(sizeof(heapitem)*self.space)
 *         if tmp == NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError
 *         self.heap = <heapitem*> tmp
 */
  __pyx_t_1 = ((__pyx_v_tmp == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "pyart/map/ckdtree.pyx":143
 *         tmp = stdlib.malloc(sizeof(heapitem)*self.space)
 *         if tmp == NULL:
 *             raise MemoryError             # <<<<<<<<<<<<<<
 *         self.heap = <heapitem*> tmp
 *         self.n = 0
 */
    PyErr_NoMemory(); __PYX_ERR(0, 143, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":142
 *         self.heap = <heapitem*> NULL
 *         tmp = stdlib.malloc(sizeof(heapitem)*self.space)
 *         if tmp == NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError
 *         self.heap = <heapitem*> tmp
 */
  }

  /* "pyart/map/ckdtree.pyx":144
 *         if tmp == NULL:
 *             raise MemoryError
 *         self.heap = <heapitem*> tmp             # <<<<<<<<<<<<<<
 *         self.n = 0
 * 
 */
  __pyx_v_self->heap = ((struct __pyx_t_5pyart_3map_7ckdtree_heapitem *)__pyx_v_tmp);

  /* "pyart/map/ckdtree.pyx":145
 *             raise MemoryError
 *         self.heap = <heapitem*> tmp
 *         self.n = 0             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(heap self):
 */
  __pyx_v_self->n = 0;

  /* "pyart/map/ckdtree.pyx":137
 *     cdef np.intp_t space
 * 
 *     def __init__(heap self, np.intp_t initial_size):             # <<<<<<<<<<<<<<
 *         cdef void *tmp
 *         self.space = initial_size
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.heap.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":147
 *         self.n = 0
 * 
 *     def __dealloc__(heap self):             # <<<<<<<<<<<<<<
 *         if self.heap != <heapitem*> NULL:
 *             stdlib.free(self.heap)
 */

/* Python wrapper */
static void __pyx_pw_5pyart_3map_7ckdtree_4heap_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5pyart_3map_7ckdtree_4heap_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5pyart_3map_7ckdtree_4heap_2__dealloc__(((struct __pyx_obj_5pyart_3map_7ckdtree_heap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5pyart_3map_7ckdtree_4heap_2__dealloc__(struct __pyx_obj_5pyart_3map_7ckdtree_heap *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "pyart/map/ckdtree.pyx":148
 * 
 *     def __dealloc__(heap self):
 *         if self.heap != <heapitem*> NULL:             # <<<<<<<<<<<<<<
 *             stdlib.free(self.heap)
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->heap != ((struct __pyx_t_5pyart_3map_7ckdtree_heapitem *)NULL)) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":149
 *     def __dealloc__(heap self):
 *         if self.heap != <heapitem*> NULL:
 *             stdlib.free(self.heap)             # <<<<<<<<<<<<<<
 * 
 *     cdef inline int _resize(heap self, np.intp_t new_space) except -1:
 */
    free(__pyx_v_self->heap);

    /* "pyart/map/ckdtree.pyx":148
 * 
 *     def __dealloc__(heap self):
 *         if self.heap != <heapitem*> NULL:             # <<<<<<<<<<<<<<
 *             stdlib.free(self.heap)
 * 
 */
  }

  /* "pyart/map/ckdtree.pyx":147
 *         self.n = 0
 * 
 *     def __dealloc__(heap self):             # <<<<<<<<<<<<<<
 *         if self.heap != <heapitem*> NULL:
 *             stdlib.free(self.heap)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "pyart/map/ckdtree.pyx":151
 *             stdlib.free(self.heap)
 * 
 *     cdef inline int _resize(heap self, np.intp_t new_space) except -1:             # <<<<<<<<<<<<<<
 *         cdef void *tmp
 *         if new_space < self.n:
 */

static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_4heap__resize(struct __pyx_obj_5pyart_3map_7ckdtree_heap *__pyx_v_self, __pyx_t_5numpy_intp_t __pyx_v_new_space) {
  void *__pyx_v_tmp;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  Py_UCS4 __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_resize", 0);

  /* "pyart/map/ckdtree.pyx":153
 *     cdef inline int _resize(heap self, np.intp_t new_space) except -1:
 *         cdef void *tmp
 *         if new_space < self.n:             # <<<<<<<<<<<<<<
 *             raise ValueError("Heap containing %d items cannot be resized to %d" % (int(self.n), int(new_space)))
 *         self.space = new_space
 */
  __pyx_t_1 = ((__pyx_v_new_space < __pyx_v_self->n) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "pyart/map/ckdtree.pyx":154
 *         cdef void *tmp
 *         if new_space < self.n:
 *             raise ValueError("Heap containing %d items cannot be resized to %d" % (int(self.n), int(new_space)))             # <<<<<<<<<<<<<<
 *         self.space = new_space
 *         tmp = stdlib.realloc(<void*>self.heap, new_space*sizeof(heapitem))
 */
    __pyx_t_2 = PyTuple_New(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 154, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = 0;
    __pyx_t_4 = 127;
    __Pyx_INCREF(__pyx_kp_u_Heap_containing);
    __pyx_t_3 += 16;
    __Pyx_GIVEREF(__pyx_kp_u_Heap_containing);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_kp_u_Heap_containing);
    __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->n); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 154, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyInt_Type)), __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 154, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_FormatAndDecref(__Pyx_PyNumber_IntOrLong(__pyx_t_6), __pyx_n_u_d); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 154, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_4 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) > __pyx_t_4) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) : __pyx_t_4;
    __pyx_t_3 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_5);
    __pyx_t_5 = 0;
    __Pyx_INCREF(__pyx_kp_u_items_cannot_be_resized_to);
    __pyx_t_3 += 28;
    __Pyx_GIVEREF(__pyx_kp_u_items_cannot_be_resized_to);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_kp_u_items_cannot_be_resized_to);
    __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_new_space); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 154, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyInt_Type)), __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 154, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_FormatAndDecref(__Pyx_PyNumber_IntOrLong(__pyx_t_6), __pyx_n_u_d); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 154, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_4 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) > __pyx_t_4) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) : __pyx_t_4;
    __pyx_t_3 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyUnicode_Join(__pyx_t_2, 4, __pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 154, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 154, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 154, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":153
 *     cdef inline int _resize(heap self, np.intp_t new_space) except -1:
 *         cdef void *tmp
 *         if new_space < self.n:             # <<<<<<<<<<<<<<
 *             raise ValueError("Heap containing %d items cannot be resized to %d" % (int(self.n), int(new_space)))
 *         self.space = new_space
 */
  }

  /* "pyart/map/ckdtree.pyx":155
 *         if new_space < self.n:
 *             raise ValueError("Heap containing %d items cannot be resized to %d" % (int(self.n), int(new_space)))
 *         self.space = new_space             # <<<<<<<<<<<<<<
 *         tmp = stdlib.realloc(<void*>self.heap, new_space*sizeof(heapitem))
 *         if tmp == NULL:
 */
  __pyx_v_self->space = __pyx_v_new_space;

  /* "pyart/map/ckdtree.pyx":156
 *             raise ValueError("Heap containing %d items cannot be resized to %d" % (int(self.n), int(new_space)))
 *         self.space = new_space
 *         tmp = stdlib.realloc(<void*>self.heap, new_space*sizeof(heapitem))             # <<<<<<<<<<<<<<
 *         if tmp == NULL:
 *             raise MemoryError
 */
  __pyx_v_tmp = realloc(((void *)__pyx_v_self->heap), (__pyx_v_new_space * (sizeof(struct __pyx_t_5pyart_3map_7ckdtree_heapitem))));

  /* "pyart/map/ckdtree.pyx":157
 *         self.space = new_space
 *         tmp = stdlib.realloc(<void*>self.heap, new_space*sizeof(heapitem))
 *         if tmp == NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError
 *         self.heap = <heapitem*> tmp
 */
  __pyx_t_1 = ((__pyx_v_tmp == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "pyart/map/ckdtree.pyx":158
 *         tmp = stdlib.realloc(<void*>self.heap, new_space*sizeof(heapitem))
 *         if tmp == NULL:
 *             raise MemoryError             # <<<<<<<<<<<<<<
 *         self.heap = <heapitem*> tmp
 *         return 0
 */
    PyErr_NoMemory(); __PYX_ERR(0, 158, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":157
 *         self.space = new_space
 *         tmp = stdlib.realloc(<void*>self.heap, new_space*sizeof(heapitem))
 *         if tmp == NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError
 *         self.heap = <heapitem*> tmp
 */
  }

  /* "pyart/map/ckdtree.pyx":159
 *         if tmp == NULL:
 *             raise MemoryError
 *         self.heap = <heapitem*> tmp             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
  __pyx_v_self->heap = ((struct __pyx_t_5pyart_3map_7ckdtree_heapitem *)__pyx_v_tmp);

  /* "pyart/map/ckdtree.pyx":160
 *             raise MemoryError
 *         self.heap = <heapitem*> tmp
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     @cython.cdivision(True)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":151
 *             stdlib.free(self.heap)
 * 
 *     cdef inline int _resize(heap self, np.intp_t new_space) except -1:             # <<<<<<<<<<<<<<
 *         cdef void *tmp
 *         if new_space < self.n:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pyart.map.ckdtree.heap._resize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":163
 * 
 *     @cython.cdivision(True)
 *     cdef inline int push(heap self, heapitem item) except -1:             # <<<<<<<<<<<<<<
 *         cdef np.intp_t i
 *         cdef heapitem t
 */

static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_4heap_push(struct __pyx_obj_5pyart_3map_7ckdtree_heap *__pyx_v_self, struct __pyx_t_5pyart_3map_7ckdtree_heapitem __pyx_v_item) {
  __pyx_t_5numpy_intp_t __pyx_v_i;
  struct __pyx_t_5pyart_3map_7ckdtree_heapitem __pyx_v_t;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);

  /* "pyart/map/ckdtree.pyx":167
 *         cdef heapitem t
 * 
 *         self.n += 1             # <<<<<<<<<<<<<<
 *         if self.n > self.space:
 *             self._resize(2 * self.space + 1)
 */
  __pyx_v_self->n = (__pyx_v_self->n + 1);

  /* "pyart/map/ckdtree.pyx":168
 * 
 *         self.n += 1
 *         if self.n > self.space:             # <<<<<<<<<<<<<<
 *             self._resize(2 * self.space + 1)
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->n > __pyx_v_self->space) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":169
 *         self.n += 1
 *         if self.n > self.space:
 *             self._resize(2 * self.space + 1)             # <<<<<<<<<<<<<<
 * 
 *         i = self.n - 1
 */
    __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_4heap__resize(__pyx_v_self, ((2 * __pyx_v_self->space) + 1)); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 169, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":168
 * 
 *         self.n += 1
 *         if self.n > self.space:             # <<<<<<<<<<<<<<
 *             self._resize(2 * self.space + 1)
 * 
 */
  }

  /* "pyart/map/ckdtree.pyx":171
 *             self._resize(2 * self.space + 1)
 * 
 *         i = self.n - 1             # <<<<<<<<<<<<<<
 *         self.heap[i] = item
 * 
 */
  __pyx_v_i = (__pyx_v_self->n - 1);

  /* "pyart/map/ckdtree.pyx":172
 * 
 *         i = self.n - 1
 *         self.heap[i] = item             # <<<<<<<<<<<<<<
 * 
 *         while i > 0 and self.heap[i].priority < self.heap[(i - 1) // 2].priority:
 */
  (__pyx_v_self->heap[__pyx_v_i]) = __pyx_v_item;

  /* "pyart/map/ckdtree.pyx":174
 *         self.heap[i] = item
 * 
 *         while i > 0 and self.heap[i].priority < self.heap[(i - 1) // 2].priority:             # <<<<<<<<<<<<<<
 *             t = self.heap[(i - 1) // 2]
 *             self.heap[(i - 1) // 2] = self.heap[i]
 */
  while (1) {
    __pyx_t_3 = ((__pyx_v_i > 0) != 0);
    if (__pyx_t_3) {
    } else {
      __pyx_t_1 = __pyx_t_3;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_3 = (((__pyx_v_self->heap[__pyx_v_i]).priority < (__pyx_v_self->heap[((__pyx_v_i - 1) / 2)]).priority) != 0);
    __pyx_t_1 = __pyx_t_3;
    __pyx_L6_bool_binop_done:;
    if (!__pyx_t_1) break;

    /* "pyart/map/ckdtree.pyx":175
 * 
 *         while i > 0 and self.heap[i].priority < self.heap[(i - 1) // 2].priority:
 *             t = self.heap[(i - 1) // 2]             # <<<<<<<<<<<<<<
 *             self.heap[(i - 1) // 2] = self.heap[i]
 *             self.heap[i] = t
 */
    __pyx_v_t = (__pyx_v_self->heap[((__pyx_v_i - 1) / 2)]);

    /* "pyart/map/ckdtree.pyx":176
 *         while i > 0 and self.heap[i].priority < self.heap[(i - 1) // 2].priority:
 *             t = self.heap[(i - 1) // 2]
 *             self.heap[(i - 1) // 2] = self.heap[i]             # <<<<<<<<<<<<<<
 *             self.heap[i] = t
 *             i = (i - 1) // 2
 */
    (__pyx_v_self->heap[((__pyx_v_i - 1) / 2)]) = (__pyx_v_self->heap[__pyx_v_i]);

    /* "pyart/map/ckdtree.pyx":177
 *             t = self.heap[(i - 1) // 2]
 *             self.heap[(i - 1) // 2] = self.heap[i]
 *             self.heap[i] = t             # <<<<<<<<<<<<<<
 *             i = (i - 1) // 2
 *         return 0
 */
    (__pyx_v_self->heap[__pyx_v_i]) = __pyx_v_t;

    /* "pyart/map/ckdtree.pyx":178
 *             self.heap[(i - 1) // 2] = self.heap[i]
 *             self.heap[i] = t
 *             i = (i - 1) // 2             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
    __pyx_v_i = ((__pyx_v_i - 1) / 2);
  }

  /* "pyart/map/ckdtree.pyx":179
 *             self.heap[i] = t
 *             i = (i - 1) // 2
 *         return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":163
 * 
 *     @cython.cdivision(True)
 *     cdef inline int push(heap self, heapitem item) except -1:             # <<<<<<<<<<<<<<
 *         cdef np.intp_t i
 *         cdef heapitem t
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.heap.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":182
 * 
 * 
 *     cdef heapitem peek(heap self):             # <<<<<<<<<<<<<<
 *         return self.heap[0]
 * 
 */

static struct __pyx_t_5pyart_3map_7ckdtree_heapitem __pyx_f_5pyart_3map_7ckdtree_4heap_peek(struct __pyx_obj_5pyart_3map_7ckdtree_heap *__pyx_v_self) {
  struct __pyx_t_5pyart_3map_7ckdtree_heapitem __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("peek", 0);

  /* "pyart/map/ckdtree.pyx":183
 * 
 *     cdef heapitem peek(heap self):
 *         return self.heap[0]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = (__pyx_v_self->heap[0]);
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":182
 * 
 * 
 *     cdef heapitem peek(heap self):             # <<<<<<<<<<<<<<
 *         return self.heap[0]
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":187
 * 
 *     @cython.cdivision(True)
 *     cdef int remove(heap self) except -1:             # <<<<<<<<<<<<<<
 *         cdef heapitem t
 *         cdef np.intp_t i, j, k, l
 */

static int __pyx_f_5pyart_3map_7ckdtree_4heap_remove(struct __pyx_obj_5pyart_3map_7ckdtree_heap *__pyx_v_self) {
  struct __pyx_t_5pyart_3map_7ckdtree_heapitem __pyx_v_t;
  __pyx_t_5numpy_intp_t __pyx_v_i;
  __pyx_t_5numpy_intp_t __pyx_v_j;
  __pyx_t_5numpy_intp_t __pyx_v_k;
  __pyx_t_5numpy_intp_t __pyx_v_l;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("remove", 0);

  /* "pyart/map/ckdtree.pyx":191
 *         cdef np.intp_t i, j, k, l
 * 
 *         self.heap[0] = self.heap[self.n-1]             # <<<<<<<<<<<<<<
 *         self.n -= 1
 *         # No point in freeing up space as the heap empties.
 */
  (__pyx_v_self->heap[0]) = (__pyx_v_self->heap[(__pyx_v_self->n - 1)]);

  /* "pyart/map/ckdtree.pyx":192
 * 
 *         self.heap[0] = self.heap[self.n-1]
 *         self.n -= 1             # <<<<<<<<<<<<<<
 *         # No point in freeing up space as the heap empties.
 *         # The whole heap gets deallocated at the end of any query below
 */
  __pyx_v_self->n = (__pyx_v_self->n - 1);

  /* "pyart/map/ckdtree.pyx":197
 *         #if self.n < self.space//4 and self.space>40: #FIXME: magic number
 *         #    self._resize(self.space // 2 + 1)
 *         i=0             # <<<<<<<<<<<<<<
 *         j=1
 *         k=2
 */
  __pyx_v_i = 0;

  /* "pyart/map/ckdtree.pyx":198
 *         #    self._resize(self.space // 2 + 1)
 *         i=0
 *         j=1             # <<<<<<<<<<<<<<
 *         k=2
 *         while ((j<self.n and
 */
  __pyx_v_j = 1;

  /* "pyart/map/ckdtree.pyx":199
 *         i=0
 *         j=1
 *         k=2             # <<<<<<<<<<<<<<
 *         while ((j<self.n and
 *                     self.heap[i].priority > self.heap[j].priority or
 */
  __pyx_v_k = 2;

  /* "pyart/map/ckdtree.pyx":200
 *         j=1
 *         k=2
 *         while ((j<self.n and             # <<<<<<<<<<<<<<
 *                     self.heap[i].priority > self.heap[j].priority or
 *                 k<self.n and
 */
  while (1) {

    /* "pyart/map/ckdtree.pyx":201
 *         k=2
 *         while ((j<self.n and
 *                     self.heap[i].priority > self.heap[j].priority or             # <<<<<<<<<<<<<<
 *                 k<self.n and
 *                     self.heap[i].priority > self.heap[k].priority)):
 */
    __pyx_t_2 = ((__pyx_v_j < __pyx_v_self->n) != 0);
    if (!__pyx_t_2) {
      goto __pyx_L6_next_or;
    } else {
    }
    __pyx_t_2 = (((__pyx_v_self->heap[__pyx_v_i]).priority > (__pyx_v_self->heap[__pyx_v_j]).priority) != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_L6_next_or:;

    /* "pyart/map/ckdtree.pyx":202
 *         while ((j<self.n and
 *                     self.heap[i].priority > self.heap[j].priority or
 *                 k<self.n and             # <<<<<<<<<<<<<<
 *                     self.heap[i].priority > self.heap[k].priority)):
 *             if k<self.n and self.heap[j].priority>self.heap[k].priority:
 */
    __pyx_t_2 = ((__pyx_v_k < __pyx_v_self->n) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L5_bool_binop_done;
    }

    /* "pyart/map/ckdtree.pyx":203
 *                     self.heap[i].priority > self.heap[j].priority or
 *                 k<self.n and
 *                     self.heap[i].priority > self.heap[k].priority)):             # <<<<<<<<<<<<<<
 *             if k<self.n and self.heap[j].priority>self.heap[k].priority:
 *                 l = k
 */
    __pyx_t_2 = (((__pyx_v_self->heap[__pyx_v_i]).priority > (__pyx_v_self->heap[__pyx_v_k]).priority) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L5_bool_binop_done:;
    if (!__pyx_t_1) break;

    /* "pyart/map/ckdtree.pyx":204
 *                 k<self.n and
 *                     self.heap[i].priority > self.heap[k].priority)):
 *             if k<self.n and self.heap[j].priority>self.heap[k].priority:             # <<<<<<<<<<<<<<
 *                 l = k
 *             else:
 */
    __pyx_t_2 = ((__pyx_v_k < __pyx_v_self->n) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L10_bool_binop_done;
    }
    __pyx_t_2 = (((__pyx_v_self->heap[__pyx_v_j]).priority > (__pyx_v_self->heap[__pyx_v_k]).priority) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L10_bool_binop_done:;
    if (__pyx_t_1) {

      /* "pyart/map/ckdtree.pyx":205
 *                     self.heap[i].priority > self.heap[k].priority)):
 *             if k<self.n and self.heap[j].priority>self.heap[k].priority:
 *                 l = k             # <<<<<<<<<<<<<<
 *             else:
 *                 l = j
 */
      __pyx_v_l = __pyx_v_k;

      /* "pyart/map/ckdtree.pyx":204
 *                 k<self.n and
 *                     self.heap[i].priority > self.heap[k].priority)):
 *             if k<self.n and self.heap[j].priority>self.heap[k].priority:             # <<<<<<<<<<<<<<
 *                 l = k
 *             else:
 */
      goto __pyx_L9;
    }

    /* "pyart/map/ckdtree.pyx":207
 *                 l = k
 *             else:
 *                 l = j             # <<<<<<<<<<<<<<
 *             t = self.heap[l]
 *             self.heap[l] = self.heap[i]
 */
    /*else*/ {
      __pyx_v_l = __pyx_v_j;
    }
    __pyx_L9:;

    /* "pyart/map/ckdtree.pyx":208
 *             else:
 *                 l = j
 *             t = self.heap[l]             # <<<<<<<<<<<<<<
 *             self.heap[l] = self.heap[i]
 *             self.heap[i] = t
 */
    __pyx_v_t = (__pyx_v_self->heap[__pyx_v_l]);

    /* "pyart/map/ckdtree.pyx":209
 *                 l = j
 *             t = self.heap[l]
 *             self.heap[l] = self.heap[i]             # <<<<<<<<<<<<<<
 *             self.heap[i] = t
 *             i = l
 */
    (__pyx_v_self->heap[__pyx_v_l]) = (__pyx_v_self->heap[__pyx_v_i]);

    /* "pyart/map/ckdtree.pyx":210
 *             t = self.heap[l]
 *             self.heap[l] = self.heap[i]
 *             self.heap[i] = t             # <<<<<<<<<<<<<<
 *             i = l
 *             j = 2*i+1
 */
    (__pyx_v_self->heap[__pyx_v_i]) = __pyx_v_t;

    /* "pyart/map/ckdtree.pyx":211
 *             self.heap[l] = self.heap[i]
 *             self.heap[i] = t
 *             i = l             # <<<<<<<<<<<<<<
 *             j = 2*i+1
 *             k = 2*i+2
 */
    __pyx_v_i = __pyx_v_l;

    /* "pyart/map/ckdtree.pyx":212
 *             self.heap[i] = t
 *             i = l
 *             j = 2*i+1             # <<<<<<<<<<<<<<
 *             k = 2*i+2
 *         return 0
 */
    __pyx_v_j = ((2 * __pyx_v_i) + 1);

    /* "pyart/map/ckdtree.pyx":213
 *             i = l
 *             j = 2*i+1
 *             k = 2*i+2             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
    __pyx_v_k = ((2 * __pyx_v_i) + 2);
  }

  /* "pyart/map/ckdtree.pyx":214
 *             j = 2*i+1
 *             k = 2*i+2
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     cdef int pop(heap self, heapitem *it) except -1:
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":187
 * 
 *     @cython.cdivision(True)
 *     cdef int remove(heap self) except -1:             # <<<<<<<<<<<<<<
 *         cdef heapitem t
 *         cdef np.intp_t i, j, k, l
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":216
 *         return 0
 * 
 *     cdef int pop(heap self, heapitem *it) except -1:             # <<<<<<<<<<<<<<
 *         it[0] = self.peek()
 *         self.remove()
 */

static int __pyx_f_5pyart_3map_7ckdtree_4heap_pop(struct __pyx_obj_5pyart_3map_7ckdtree_heap *__pyx_v_self, struct __pyx_t_5pyart_3map_7ckdtree_heapitem *__pyx_v_it) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pop", 0);

  /* "pyart/map/ckdtree.pyx":217
 * 
 *     cdef int pop(heap self, heapitem *it) except -1:
 *         it[0] = self.peek()             # <<<<<<<<<<<<<<
 *         self.remove()
 *         return 0
 */
  (__pyx_v_it[0]) = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_heap *)__pyx_v_self->__pyx_vtab)->peek(__pyx_v_self);

  /* "pyart/map/ckdtree.pyx":218
 *     cdef int pop(heap self, heapitem *it) except -1:
 *         it[0] = self.peek()
 *         self.remove()             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_heap *)__pyx_v_self->__pyx_vtab)->remove(__pyx_v_self); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 218, __pyx_L1_error)

  /* "pyart/map/ckdtree.pyx":219
 *         it[0] = self.peek()
 *         self.remove()
 *         return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":216
 *         return 0
 * 
 *     cdef int pop(heap self, heapitem *it) except -1:             # <<<<<<<<<<<<<<
 *         it[0] = self.peek()
 *         self.remove()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.heap.pop", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.heap cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_4heap_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_4heap_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_4heap_4__reduce_cython__(((struct __pyx_obj_5pyart_3map_7ckdtree_heap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_3map_7ckdtree_4heap_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_3map_7ckdtree_heap *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.heap cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.heap cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.heap cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.map.ckdtree.heap.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.heap cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.heap cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_4heap_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_4heap_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_4heap_6__setstate_cython__(((struct __pyx_obj_5pyart_3map_7ckdtree_heap *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_3map_7ckdtree_4heap_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_3map_7ckdtree_heap *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.heap cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.heap cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.heap cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.heap cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.map.ckdtree.heap.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":224
 * # Utility functions
 * # =================
 * cdef inline np.float64_t dmax(np.float64_t x, np.float64_t y):             # <<<<<<<<<<<<<<
 *     if x>y:
 *         return x
 */

static CYTHON_INLINE __pyx_t_5numpy_float64_t __pyx_f_5pyart_3map_7ckdtree_dmax(__pyx_t_5numpy_float64_t __pyx_v_x, __pyx_t_5numpy_float64_t __pyx_v_y) {
  __pyx_t_5numpy_float64_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("dmax", 0);

  /* "pyart/map/ckdtree.pyx":225
 * # =================
 * cdef inline np.float64_t dmax(np.float64_t x, np.float64_t y):
 *     if x>y:             # <<<<<<<<<<<<<<
 *         return x
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_x > __pyx_v_y) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":226
 * cdef inline np.float64_t dmax(np.float64_t x, np.float64_t y):
 *     if x>y:
 *         return x             # <<<<<<<<<<<<<<
 *     else:
 *         return y
 */
    __pyx_r = __pyx_v_x;
    goto __pyx_L0;

    /* "pyart/map/ckdtree.pyx":225
 * # =================
 * cdef inline np.float64_t dmax(np.float64_t x, np.float64_t y):
 *     if x>y:             # <<<<<<<<<<<<<<
 *         return x
 *     else:
 */
  }

  /* "pyart/map/ckdtree.pyx":228
 *         return x
 *     else:
 *         return y             # <<<<<<<<<<<<<<
 * 
 * cdef inline np.float64_t dabs(np.float64_t x):
 */
  /*else*/ {
    __pyx_r = __pyx_v_y;
    goto __pyx_L0;
  }

  /* "pyart/map/ckdtree.pyx":224
 * # Utility functions
 * # =================
 * cdef inline np.float64_t dmax(np.float64_t x, np.float64_t y):             # <<<<<<<<<<<<<<
 *     if x>y:
 *         return x
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":230
 *         return y
 * 
 * cdef inline np.float64_t dabs(np.float64_t x):             # <<<<<<<<<<<<<<
 *     if x>0:
 *         return x
 */

static CYTHON_INLINE __pyx_t_5numpy_float64_t __pyx_f_5pyart_3map_7ckdtree_dabs(__pyx_t_5numpy_float64_t __pyx_v_x) {
  __pyx_t_5numpy_float64_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("dabs", 0);

  /* "pyart/map/ckdtree.pyx":231
 * 
 * cdef inline np.float64_t dabs(np.float64_t x):
 *     if x>0:             # <<<<<<<<<<<<<<
 *         return x
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_x > 0.0) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":232
 * cdef inline np.float64_t dabs(np.float64_t x):
 *     if x>0:
 *         return x             # <<<<<<<<<<<<<<
 *     else:
 *         return -x
 */
    __pyx_r = __pyx_v_x;
    goto __pyx_L0;

    /* "pyart/map/ckdtree.pyx":231
 * 
 * cdef inline np.float64_t dabs(np.float64_t x):
 *     if x>0:             # <<<<<<<<<<<<<<
 *         return x
 *     else:
 */
  }

  /* "pyart/map/ckdtree.pyx":234
 *         return x
 *     else:
 *         return -x             # <<<<<<<<<<<<<<
 * 
 * # Utility for building a coo matrix incrementally
 */
  /*else*/ {
    __pyx_r = (-__pyx_v_x);
    goto __pyx_L0;
  }

  /* "pyart/map/ckdtree.pyx":230
 *         return y
 * 
 * cdef inline np.float64_t dabs(np.float64_t x):             # <<<<<<<<<<<<<<
 *     if x>0:
 *         return x
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":246
 *         np.float64_t *v_data
 * 
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         self.n = 0
 *         self.n_max = 10
 */

/* Python wrapper */
static int __pyx_pw_5pyart_3map_7ckdtree_11coo_entries_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pyart_3map_7ckdtree_11coo_entries_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 0))) return -1;
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_11coo_entries___init__(((struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_3map_7ckdtree_11coo_entries___init__(struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pyart/map/ckdtree.pyx":247
 * 
 *     def __init__(self):
 *         self.n = 0             # <<<<<<<<<<<<<<
 *         self.n_max = 10
 *         self.i = np.empty(self.n_max, dtype=np.intp)
 */
  __pyx_v_self->n = 0;

  /* "pyart/map/ckdtree.pyx":248
 *     def __init__(self):
 *         self.n = 0
 *         self.n_max = 10             # <<<<<<<<<<<<<<
 *         self.i = np.empty(self.n_max, dtype=np.intp)
 *         self.j = np.empty(self.n_max, dtype=np.intp)
 */
  __pyx_v_self->n_max = 10;

  /* "pyart/map/ckdtree.pyx":249
 *         self.n = 0
 *         self.n_max = 10
 *         self.i = np.empty(self.n_max, dtype=np.intp)             # <<<<<<<<<<<<<<
 *         self.j = np.empty(self.n_max, dtype=np.intp)
 *         self.v = np.empty(self.n_max, dtype=np.float64)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_empty); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->n_max); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_intp); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_5) < 0) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(((__pyx_t_5) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_5, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_5);
  __Pyx_GOTREF(__pyx_v_self->i);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->i));
  __pyx_v_self->i = ((PyArrayObject *)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "pyart/map/ckdtree.pyx":250
 *         self.n_max = 10
 *         self.i = np.empty(self.n_max, dtype=np.intp)
 *         self.j = np.empty(self.n_max, dtype=np.intp)             # <<<<<<<<<<<<<<
 *         self.v = np.empty(self.n_max, dtype=np.float64)
 *         self.i_data = <np.intp_t *>np.PyArray_DATA(self.i)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_empty); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->n_max); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_intp); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_dtype, __pyx_t_4) < 0) __PYX_ERR(0, 250, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_3, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 250, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->j);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->j));
  __pyx_v_self->j = ((PyArrayObject *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "pyart/map/ckdtree.pyx":251
 *         self.i = np.empty(self.n_max, dtype=np.intp)
 *         self.j = np.empty(self.n_max, dtype=np.intp)
 *         self.v = np.empty(self.n_max, dtype=np.float64)             # <<<<<<<<<<<<<<
 *         self.i_data = <np.intp_t *>np.PyArray_DATA(self.i)
 *         self.j_data = <np.intp_t *>np.PyArray_DATA(self.j)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_empty); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->n_max); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_float64); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_2) < 0) __PYX_ERR(0, 251, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 251, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->v);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->v));
  __pyx_v_self->v = ((PyArrayObject *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyart/map/ckdtree.pyx":252
 *         self.j = np.empty(self.n_max, dtype=np.intp)
 *         self.v = np.empty(self.n_max, dtype=np.float64)
 *         self.i_data = <np.intp_t *>np.PyArray_DATA(self.i)             # <<<<<<<<<<<<<<
 *         self.j_data = <np.intp_t *>np.PyArray_DATA(self.j)
 *         self.v_data = <np.float64_t*>np.PyArray_DATA(self.v)
 */
  __pyx_t_2 = ((PyObject *)__pyx_v_self->i);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_v_self->i_data = ((__pyx_t_5numpy_intp_t *)PyArray_DATA(((PyArrayObject *)__pyx_t_2)));
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyart/map/ckdtree.pyx":253
 *         self.v = np.empty(self.n_max, dtype=np.float64)
 *         self.i_data = <np.intp_t *>np.PyArray_DATA(self.i)
 *         self.j_data = <np.intp_t *>np.PyArray_DATA(self.j)             # <<<<<<<<<<<<<<
 *         self.v_data = <np.float64_t*>np.PyArray_DATA(self.v)
 * 
 */
  __pyx_t_2 = ((PyObject *)__pyx_v_self->j);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_v_self->j_data = ((__pyx_t_5numpy_intp_t *)PyArray_DATA(((PyArrayObject *)__pyx_t_2)));
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyart/map/ckdtree.pyx":254
 *         self.i_data = <np.intp_t *>np.PyArray_DATA(self.i)
 *         self.j_data = <np.intp_t *>np.PyArray_DATA(self.j)
 *         self.v_data = <np.float64_t*>np.PyArray_DATA(self.v)             # <<<<<<<<<<<<<<
 * 
 *     cdef void add(coo_entries self, np.intp_t i, np.intp_t j, np.float64_t v):
 */
  __pyx_t_2 = ((PyObject *)__pyx_v_self->v);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_v_self->v_data = ((__pyx_t_5numpy_float64_t *)PyArray_DATA(((PyArrayObject *)__pyx_t_2)));
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyart/map/ckdtree.pyx":246
 *         np.float64_t *v_data
 * 
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         self.n = 0
 *         self.n_max = 10
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyart.map.ckdtree.coo_entries.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":256
 *         self.v_data = <np.float64_t*>np.PyArray_DATA(self.v)
 * 
 *     cdef void add(coo_entries self, np.intp_t i, np.intp_t j, np.float64_t v):             # <<<<<<<<<<<<<<
 *         cdef np.intp_t k
 *         if self.n == self.n_max:
 */

static void __pyx_f_5pyart_3map_7ckdtree_11coo_entries_add(struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *__pyx_v_self, __pyx_t_5numpy_intp_t __pyx_v_i, __pyx_t_5numpy_intp_t __pyx_v_j, __pyx_t_5numpy_float64_t __pyx_v_v) {
  __pyx_t_5numpy_intp_t __pyx_v_k;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __pyx_t_5numpy_intp_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("add", 0);

  /* "pyart/map/ckdtree.pyx":258
 *     cdef void add(coo_entries self, np.intp_t i, np.intp_t j, np.float64_t v):
 *         cdef np.intp_t k
 *         if self.n == self.n_max:             # <<<<<<<<<<<<<<
 *             self.n_max *= 2
 *             self.i.resize(self.n_max)
 */
  __pyx_t_1 = ((__pyx_v_self->n == __pyx_v_self->n_max) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":259
 *         cdef np.intp_t k
 *         if self.n == self.n_max:
 *             self.n_max *= 2             # <<<<<<<<<<<<<<
 *             self.i.resize(self.n_max)
 *             self.j.resize(self.n_max)
 */
    __pyx_v_self->n_max = (__pyx_v_self->n_max * 2);

    /* "pyart/map/ckdtree.pyx":260
 *         if self.n == self.n_max:
 *             self.n_max *= 2
 *             self.i.resize(self.n_max)             # <<<<<<<<<<<<<<
 *             self.j.resize(self.n_max)
 *             self.v.resize(self.n_max)
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->i), __pyx_n_s_resize); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 260, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->n_max); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 260, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 260, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyart/map/ckdtree.pyx":261
 *             self.n_max *= 2
 *             self.i.resize(self.n_max)
 *             self.j.resize(self.n_max)             # <<<<<<<<<<<<<<
 *             self.v.resize(self.n_max)
 *             self.i_data = <np.intp_t *>np.PyArray_DATA(self.i)
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->j), __pyx_n_s_resize); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 261, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->n_max); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 261, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 261, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyart/map/ckdtree.pyx":262
 *             self.i.resize(self.n_max)
 *             self.j.resize(self.n_max)
 *             self.v.resize(self.n_max)             # <<<<<<<<<<<<<<
 *             self.i_data = <np.intp_t *>np.PyArray_DATA(self.i)
 *             self.j_data = <np.intp_t *>np.PyArray_DATA(self.j)
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->v), __pyx_n_s_resize); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 262, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->n_max); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 262, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 262, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyart/map/ckdtree.pyx":263
 *             self.j.resize(self.n_max)
 *             self.v.resize(self.n_max)
 *             self.i_data = <np.intp_t *>np.PyArray_DATA(self.i)             # <<<<<<<<<<<<<<
 *             self.j_data = <np.intp_t *>np.PyArray_DATA(self.j)
 *             self.v_data = <np.float64_t*>np.PyArray_DATA(self.v)
 */
    __pyx_t_2 = ((PyObject *)__pyx_v_self->i);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_self->i_data = ((__pyx_t_5numpy_intp_t *)PyArray_DATA(((PyArrayObject *)__pyx_t_2)));
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyart/map/ckdtree.pyx":264
 *             self.v.resize(self.n_max)
 *             self.i_data = <np.intp_t *>np.PyArray_DATA(self.i)
 *             self.j_data = <np.intp_t *>np.PyArray_DATA(self.j)             # <<<<<<<<<<<<<<
 *             self.v_data = <np.float64_t*>np.PyArray_DATA(self.v)
 *         k = self.n
 */
    __pyx_t_2 = ((PyObject *)__pyx_v_self->j);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_self->j_data = ((__pyx_t_5numpy_intp_t *)PyArray_DATA(((PyArrayObject *)__pyx_t_2)));
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyart/map/ckdtree.pyx":265
 *             self.i_data = <np.intp_t *>np.PyArray_DATA(self.i)
 *             self.j_data = <np.intp_t *>np.PyArray_DATA(self.j)
 *             self.v_data = <np.float64_t*>np.PyArray_DATA(self.v)             # <<<<<<<<<<<<<<
 *         k = self.n
 *         self.i_data[k] = i
 */
    __pyx_t_2 = ((PyObject *)__pyx_v_self->v);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_self->v_data = ((__pyx_t_5numpy_float64_t *)PyArray_DATA(((PyArrayObject *)__pyx_t_2)));
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyart/map/ckdtree.pyx":258
 *     cdef void add(coo_entries self, np.intp_t i, np.intp_t j, np.float64_t v):
 *         cdef np.intp_t k
 *         if self.n == self.n_max:             # <<<<<<<<<<<<<<
 *             self.n_max *= 2
 *             self.i.resize(self.n_max)
 */
  }

  /* "pyart/map/ckdtree.pyx":266
 *             self.j_data = <np.intp_t *>np.PyArray_DATA(self.j)
 *             self.v_data = <np.float64_t*>np.PyArray_DATA(self.v)
 *         k = self.n             # <<<<<<<<<<<<<<
 *         self.i_data[k] = i
 *         self.j_data[k] = j
 */
  __pyx_t_6 = __pyx_v_self->n;
  __pyx_v_k = __pyx_t_6;

  /* "pyart/map/ckdtree.pyx":267
 *             self.v_data = <np.float64_t*>np.PyArray_DATA(self.v)
 *         k = self.n
 *         self.i_data[k] = i             # <<<<<<<<<<<<<<
 *         self.j_data[k] = j
 *         self.v_data[k] = v
 */
  (__pyx_v_self->i_data[__pyx_v_k]) = __pyx_v_i;

  /* "pyart/map/ckdtree.pyx":268
 *         k = self.n
 *         self.i_data[k] = i
 *         self.j_data[k] = j             # <<<<<<<<<<<<<<
 *         self.v_data[k] = v
 *         self.n += 1
 */
  (__pyx_v_self->j_data[__pyx_v_k]) = __pyx_v_j;

  /* "pyart/map/ckdtree.pyx":269
 *         self.i_data[k] = i
 *         self.j_data[k] = j
 *         self.v_data[k] = v             # <<<<<<<<<<<<<<
 *         self.n += 1
 * 
 */
  (__pyx_v_self->v_data[__pyx_v_k]) = __pyx_v_v;

  /* "pyart/map/ckdtree.pyx":270
 *         self.j_data[k] = j
 *         self.v_data[k] = v
 *         self.n += 1             # <<<<<<<<<<<<<<
 * 
 *     def to_matrix(coo_entries self, shape=None):
 */
  __pyx_v_self->n = (__pyx_v_self->n + 1);

  /* "pyart/map/ckdtree.pyx":256
 *         self.v_data = <np.float64_t*>np.PyArray_DATA(self.v)
 * 
 *     cdef void add(coo_entries self, np.intp_t i, np.intp_t j, np.float64_t v):             # <<<<<<<<<<<<<<
 *         cdef np.intp_t k
 *         if self.n == self.n_max:
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_WriteUnraisable("pyart.map.ckdtree.coo_entries.add", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pyart/map/ckdtree.pyx":272
 *         self.n += 1
 * 
 *     def to_matrix(coo_entries self, shape=None):             # <<<<<<<<<<<<<<
 *         # Shrink arrays to size
 *         self.i.resize(self.n)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_11coo_entries_3to_matrix(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_11coo_entries_3to_matrix(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_shape = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("to_matrix (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_shape,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_shape);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "to_matrix") < 0)) __PYX_ERR(0, 272, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_shape = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("to_matrix", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 272, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.coo_entries.to_matrix", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_11coo_entries_2to_matrix(((struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *)__pyx_v_self), __pyx_v_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_3map_7ckdtree_11coo_entries_2to_matrix(struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *__pyx_v_self, PyObject *__pyx_v_shape) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __pyx_t_5numpy_intp_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("to_matrix", 0);

  /* "pyart/map/ckdtree.pyx":274
 *     def to_matrix(coo_entries self, shape=None):
 *         # Shrink arrays to size
 *         self.i.resize(self.n)             # <<<<<<<<<<<<<<
 *         self.j.resize(self.n)
 *         self.v.resize(self.n)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->i), __pyx_n_s_resize); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 274, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->n); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 274, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 274, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/map/ckdtree.pyx":275
 *         # Shrink arrays to size
 *         self.i.resize(self.n)
 *         self.j.resize(self.n)             # <<<<<<<<<<<<<<
 *         self.v.resize(self.n)
 *         self.i_data = <np.intp_t *>np.PyArray_DATA(self.i)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->j), __pyx_n_s_resize); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->n); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/map/ckdtree.pyx":276
 *         self.i.resize(self.n)
 *         self.j.resize(self.n)
 *         self.v.resize(self.n)             # <<<<<<<<<<<<<<
 *         self.i_data = <np.intp_t *>np.PyArray_DATA(self.i)
 *         self.j_data = <np.intp_t *>np.PyArray_DATA(self.j)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->v), __pyx_n_s_resize); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->n); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/map/ckdtree.pyx":277
 *         self.j.resize(self.n)
 *         self.v.resize(self.n)
 *         self.i_data = <np.intp_t *>np.PyArray_DATA(self.i)             # <<<<<<<<<<<<<<
 *         self.j_data = <np.intp_t *>np.PyArray_DATA(self.j)
 *         self.v_data = <np.float64_t*>np.PyArray_DATA(self.v)
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_self->i);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_self->i_data = ((__pyx_t_5numpy_intp_t *)PyArray_DATA(((PyArrayObject *)__pyx_t_1)));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/map/ckdtree.pyx":278
 *         self.v.resize(self.n)
 *         self.i_data = <np.intp_t *>np.PyArray_DATA(self.i)
 *         self.j_data = <np.intp_t *>np.PyArray_DATA(self.j)             # <<<<<<<<<<<<<<
 *         self.v_data = <np.float64_t*>np.PyArray_DATA(self.v)
 *         self.n_max = self.n
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_self->j);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_self->j_data = ((__pyx_t_5numpy_intp_t *)PyArray_DATA(((PyArrayObject *)__pyx_t_1)));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/map/ckdtree.pyx":279
 *         self.i_data = <np.intp_t *>np.PyArray_DATA(self.i)
 *         self.j_data = <np.intp_t *>np.PyArray_DATA(self.j)
 *         self.v_data = <np.float64_t*>np.PyArray_DATA(self.v)             # <<<<<<<<<<<<<<
 *         self.n_max = self.n
 *         return scipy.sparse.coo_matrix((self.v, (self.i, self.j)),
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_self->v);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_self->v_data = ((__pyx_t_5numpy_float64_t *)PyArray_DATA(((PyArrayObject *)__pyx_t_1)));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/map/ckdtree.pyx":280
 *         self.j_data = <np.intp_t *>np.PyArray_DATA(self.j)
 *         self.v_data = <np.float64_t*>np.PyArray_DATA(self.v)
 *         self.n_max = self.n             # <<<<<<<<<<<<<<
 *         return scipy.sparse.coo_matrix((self.v, (self.i, self.j)),
 *                                        shape=shape)
 */
  __pyx_t_5 = __pyx_v_self->n;
  __pyx_v_self->n_max = __pyx_t_5;

  /* "pyart/map/ckdtree.pyx":281
 *         self.v_data = <np.float64_t*>np.PyArray_DATA(self.v)
 *         self.n_max = self.n
 *         return scipy.sparse.coo_matrix((self.v, (self.i, self.j)),             # <<<<<<<<<<<<<<
 *                                        shape=shape)
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_scipy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 281, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_sparse); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 281, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_coo_matrix); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 281, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 281, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->i));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->i));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_self->i));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->j));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->j));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self->j));
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 281, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->v));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->v));
  PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_v_self->v));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 281, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
  __pyx_t_3 = 0;

  /* "pyart/map/ckdtree.pyx":282
 *         self.n_max = self.n
 *         return scipy.sparse.coo_matrix((self.v, (self.i, self.j)),
 *                                        shape=shape)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 282, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_v_shape) < 0) __PYX_ERR(0, 282, __pyx_L1_error)

  /* "pyart/map/ckdtree.pyx":281
 *         self.v_data = <np.float64_t*>np.PyArray_DATA(self.v)
 *         self.n_max = self.n
 *         return scipy.sparse.coo_matrix((self.v, (self.i, self.j)),             # <<<<<<<<<<<<<<
 *                                        shape=shape)
 * 
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 281, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":272
 *         self.n += 1
 * 
 *     def to_matrix(coo_entries self, shape=None):             # <<<<<<<<<<<<<<
 *         # Shrink arrays to size
 *         self.i.resize(self.n)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyart.map.ckdtree.coo_entries.to_matrix", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.i_data,self.j_data,self.v_data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_11coo_entries_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_11coo_entries_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_11coo_entries_4__reduce_cython__(((struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_3map_7ckdtree_11coo_entries_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.i_data,self.j_data,self.v_data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.i_data,self.j_data,self.v_data cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.i_data,self.j_data,self.v_data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.map.ckdtree.coo_entries.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.i_data,self.j_data,self.v_data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.i_data,self.j_data,self.v_data cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_11coo_entries_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_11coo_entries_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_11coo_entries_6__setstate_cython__(((struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_3map_7ckdtree_11coo_entries_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.i_data,self.j_data,self.v_data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.i_data,self.j_data,self.v_data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.i_data,self.j_data,self.v_data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.i_data,self.j_data,self.v_data cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.map.ckdtree.coo_entries.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":287
 * # Measuring distances
 * # ===================
 * cdef inline np.float64_t _distance_p(np.float64_t *x, np.float64_t *y,             # <<<<<<<<<<<<<<
 *                                      np.float64_t p, np.intp_t k,
 *                                      np.float64_t upperbound):
 */

static CYTHON_INLINE __pyx_t_5numpy_float64_t __pyx_f_5pyart_3map_7ckdtree__distance_p(__pyx_t_5numpy_float64_t *__pyx_v_x, __pyx_t_5numpy_float64_t *__pyx_v_y, __pyx_t_5numpy_float64_t __pyx_v_p, __pyx_t_5numpy_intp_t __pyx_v_k, __pyx_t_5numpy_float64_t __pyx_v_upperbound) {
  __pyx_t_5numpy_intp_t __pyx_v_i;
  __pyx_t_5numpy_float64_t __pyx_v_r;
  __pyx_t_5numpy_float64_t __pyx_v_z;
  __pyx_t_5numpy_float64_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __pyx_t_5numpy_intp_t __pyx_t_2;
  __pyx_t_5numpy_intp_t __pyx_t_3;
  __pyx_t_5numpy_intp_t __pyx_t_4;
  __Pyx_RefNannySetupContext("_distance_p", 0);

  /* "pyart/map/ckdtree.pyx":298
 *     cdef np.intp_t i
 *     cdef np.float64_t r, z
 *     r = 0             # <<<<<<<<<<<<<<
 *     if p==2:
 *         for i in range(k):
 */
  __pyx_v_r = 0.0;

  /* "pyart/map/ckdtree.pyx":299
 *     cdef np.float64_t r, z
 *     r = 0
 *     if p==2:             # <<<<<<<<<<<<<<
 *         for i in range(k):
 *             z = x[i] - y[i]
 */
  __pyx_t_1 = ((__pyx_v_p == 2.0) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":300
 *     r = 0
 *     if p==2:
 *         for i in range(k):             # <<<<<<<<<<<<<<
 *             z = x[i] - y[i]
 *             r += z*z
 */
    __pyx_t_2 = __pyx_v_k;
    __pyx_t_3 = __pyx_t_2;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_i = __pyx_t_4;

      /* "pyart/map/ckdtree.pyx":301
 *     if p==2:
 *         for i in range(k):
 *             z = x[i] - y[i]             # <<<<<<<<<<<<<<
 *             r += z*z
 *             if r>upperbound:
 */
      __pyx_v_z = ((__pyx_v_x[__pyx_v_i]) - (__pyx_v_y[__pyx_v_i]));

      /* "pyart/map/ckdtree.pyx":302
 *         for i in range(k):
 *             z = x[i] - y[i]
 *             r += z*z             # <<<<<<<<<<<<<<
 *             if r>upperbound:
 *                 return r
 */
      __pyx_v_r = (__pyx_v_r + (__pyx_v_z * __pyx_v_z));

      /* "pyart/map/ckdtree.pyx":303
 *             z = x[i] - y[i]
 *             r += z*z
 *             if r>upperbound:             # <<<<<<<<<<<<<<
 *                 return r
 *     elif p==infinity:
 */
      __pyx_t_1 = ((__pyx_v_r > __pyx_v_upperbound) != 0);
      if (__pyx_t_1) {

        /* "pyart/map/ckdtree.pyx":304
 *             r += z*z
 *             if r>upperbound:
 *                 return r             # <<<<<<<<<<<<<<
 *     elif p==infinity:
 *         for i in range(k):
 */
        __pyx_r = __pyx_v_r;
        goto __pyx_L0;

        /* "pyart/map/ckdtree.pyx":303
 *             z = x[i] - y[i]
 *             r += z*z
 *             if r>upperbound:             # <<<<<<<<<<<<<<
 *                 return r
 *     elif p==infinity:
 */
      }
    }

    /* "pyart/map/ckdtree.pyx":299
 *     cdef np.float64_t r, z
 *     r = 0
 *     if p==2:             # <<<<<<<<<<<<<<
 *         for i in range(k):
 *             z = x[i] - y[i]
 */
    goto __pyx_L3;
  }

  /* "pyart/map/ckdtree.pyx":305
 *             if r>upperbound:
 *                 return r
 *     elif p==infinity:             # <<<<<<<<<<<<<<
 *         for i in range(k):
 *             r = dmax(r,dabs(x[i]-y[i]))
 */
  __pyx_t_1 = ((__pyx_v_p == __pyx_v_5pyart_3map_7ckdtree_infinity) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":306
 *                 return r
 *     elif p==infinity:
 *         for i in range(k):             # <<<<<<<<<<<<<<
 *             r = dmax(r,dabs(x[i]-y[i]))
 *             if r>upperbound:
 */
    __pyx_t_2 = __pyx_v_k;
    __pyx_t_3 = __pyx_t_2;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_i = __pyx_t_4;

      /* "pyart/map/ckdtree.pyx":307
 *     elif p==infinity:
 *         for i in range(k):
 *             r = dmax(r,dabs(x[i]-y[i]))             # <<<<<<<<<<<<<<
 *             if r>upperbound:
 *                 return r
 */
      __pyx_v_r = __pyx_f_5pyart_3map_7ckdtree_dmax(__pyx_v_r, __pyx_f_5pyart_3map_7ckdtree_dabs(((__pyx_v_x[__pyx_v_i]) - (__pyx_v_y[__pyx_v_i]))));

      /* "pyart/map/ckdtree.pyx":308
 *         for i in range(k):
 *             r = dmax(r,dabs(x[i]-y[i]))
 *             if r>upperbound:             # <<<<<<<<<<<<<<
 *                 return r
 *     elif p==1:
 */
      __pyx_t_1 = ((__pyx_v_r > __pyx_v_upperbound) != 0);
      if (__pyx_t_1) {

        /* "pyart/map/ckdtree.pyx":309
 *             r = dmax(r,dabs(x[i]-y[i]))
 *             if r>upperbound:
 *                 return r             # <<<<<<<<<<<<<<
 *     elif p==1:
 *         for i in range(k):
 */
        __pyx_r = __pyx_v_r;
        goto __pyx_L0;

        /* "pyart/map/ckdtree.pyx":308
 *         for i in range(k):
 *             r = dmax(r,dabs(x[i]-y[i]))
 *             if r>upperbound:             # <<<<<<<<<<<<<<
 *                 return r
 *     elif p==1:
 */
      }
    }

    /* "pyart/map/ckdtree.pyx":305
 *             if r>upperbound:
 *                 return r
 *     elif p==infinity:             # <<<<<<<<<<<<<<
 *         for i in range(k):
 *             r = dmax(r,dabs(x[i]-y[i]))
 */
    goto __pyx_L3;
  }

  /* "pyart/map/ckdtree.pyx":310
 *             if r>upperbound:
 *                 return r
 *     elif p==1:             # <<<<<<<<<<<<<<
 *         for i in range(k):
 *             r += dabs(x[i]-y[i])
 */
  __pyx_t_1 = ((__pyx_v_p == 1.0) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":311
 *                 return r
 *     elif p==1:
 *         for i in range(k):             # <<<<<<<<<<<<<<
 *             r += dabs(x[i]-y[i])
 *             if r>upperbound:
 */
    __pyx_t_2 = __pyx_v_k;
    __pyx_t_3 = __pyx_t_2;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_i = __pyx_t_4;

      /* "pyart/map/ckdtree.pyx":312
 *     elif p==1:
 *         for i in range(k):
 *             r += dabs(x[i]-y[i])             # <<<<<<<<<<<<<<
 *             if r>upperbound:
 *                 return r
 */
      __pyx_v_r = (__pyx_v_r + __pyx_f_5pyart_3map_7ckdtree_dabs(((__pyx_v_x[__pyx_v_i]) - (__pyx_v_y[__pyx_v_i]))));

      /* "pyart/map/ckdtree.pyx":313
 *         for i in range(k):
 *             r += dabs(x[i]-y[i])
 *             if r>upperbound:             # <<<<<<<<<<<<<<
 *                 return r
 *     else:
 */
      __pyx_t_1 = ((__pyx_v_r > __pyx_v_upperbound) != 0);
      if (__pyx_t_1) {

        /* "pyart/map/ckdtree.pyx":314
 *             r += dabs(x[i]-y[i])
 *             if r>upperbound:
 *                 return r             # <<<<<<<<<<<<<<
 *     else:
 *         for i in range(k):
 */
        __pyx_r = __pyx_v_r;
        goto __pyx_L0;

        /* "pyart/map/ckdtree.pyx":313
 *         for i in range(k):
 *             r += dabs(x[i]-y[i])
 *             if r>upperbound:             # <<<<<<<<<<<<<<
 *                 return r
 *     else:
 */
      }
    }

    /* "pyart/map/ckdtree.pyx":310
 *             if r>upperbound:
 *                 return r
 *     elif p==1:             # <<<<<<<<<<<<<<
 *         for i in range(k):
 *             r += dabs(x[i]-y[i])
 */
    goto __pyx_L3;
  }

  /* "pyart/map/ckdtree.pyx":316
 *                 return r
 *     else:
 *         for i in range(k):             # <<<<<<<<<<<<<<
 *             r += dabs(x[i]-y[i])**p
 *             if r>upperbound:
 */
  /*else*/ {
    __pyx_t_2 = __pyx_v_k;
    __pyx_t_3 = __pyx_t_2;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_i = __pyx_t_4;

      /* "pyart/map/ckdtree.pyx":317
 *     else:
 *         for i in range(k):
 *             r += dabs(x[i]-y[i])**p             # <<<<<<<<<<<<<<
 *             if r>upperbound:
 *                 return r
 */
      __pyx_v_r = (__pyx_v_r + pow(__pyx_f_5pyart_3map_7ckdtree_dabs(((__pyx_v_x[__pyx_v_i]) - (__pyx_v_y[__pyx_v_i]))), __pyx_v_p));

      /* "pyart/map/ckdtree.pyx":318
 *         for i in range(k):
 *             r += dabs(x[i]-y[i])**p
 *             if r>upperbound:             # <<<<<<<<<<<<<<
 *                 return r
 *     return r
 */
      __pyx_t_1 = ((__pyx_v_r > __pyx_v_upperbound) != 0);
      if (__pyx_t_1) {

        /* "pyart/map/ckdtree.pyx":319
 *             r += dabs(x[i]-y[i])**p
 *             if r>upperbound:
 *                 return r             # <<<<<<<<<<<<<<
 *     return r
 * 
 */
        __pyx_r = __pyx_v_r;
        goto __pyx_L0;

        /* "pyart/map/ckdtree.pyx":318
 *         for i in range(k):
 *             r += dabs(x[i]-y[i])**p
 *             if r>upperbound:             # <<<<<<<<<<<<<<
 *                 return r
 *     return r
 */
      }
    }
  }
  __pyx_L3:;

  /* "pyart/map/ckdtree.pyx":320
 *             if r>upperbound:
 *                 return r
 *     return r             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_r;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":287
 * # Measuring distances
 * # ===================
 * cdef inline np.float64_t _distance_p(np.float64_t *x, np.float64_t *y,             # <<<<<<<<<<<<<<
 *                                      np.float64_t p, np.intp_t k,
 *                                      np.float64_t upperbound):
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":332
 *     cdef np.ndarray mins_arr, maxes_arr
 * 
 *     def __init__(self, mins_arr, maxes_arr):             # <<<<<<<<<<<<<<
 *         # Copy array data
 *         self.mins_arr = np.array(mins_arr, dtype=np.float64, order='C')
 */

/* Python wrapper */
static int __pyx_pw_5pyart_3map_7ckdtree_9Rectangle_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pyart_3map_7ckdtree_9Rectangle_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_mins_arr = 0;
  PyObject *__pyx_v_maxes_arr = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_mins_arr,&__pyx_n_s_maxes_arr,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_mins_arr)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_maxes_arr)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 332, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 332, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_mins_arr = values[0];
    __pyx_v_maxes_arr = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 332, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.Rectangle.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_9Rectangle___init__(((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)__pyx_v_self), __pyx_v_mins_arr, __pyx_v_maxes_arr);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_3map_7ckdtree_9Rectangle___init__(struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_self, PyObject *__pyx_v_mins_arr, PyObject *__pyx_v_maxes_arr) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pyart/map/ckdtree.pyx":334
 *     def __init__(self, mins_arr, maxes_arr):
 *         # Copy array data
 *         self.mins_arr = np.array(mins_arr, dtype=np.float64, order='C')             # <<<<<<<<<<<<<<
 *         self.maxes_arr = np.array(maxes_arr, dtype=np.float64, order='C')
 *         self.mins = <np.float64_t*>np.PyArray_DATA(self.mins_arr)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 334, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_array); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 334, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 334, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_mins_arr);
  __Pyx_GIVEREF(__pyx_v_mins_arr);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_mins_arr);
  __pyx_t_3 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 334, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 334, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_float64); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 334, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_5) < 0) __PYX_ERR(0, 334, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_order, __pyx_n_u_C) < 0) __PYX_ERR(0, 334, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 334, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (!(likely(((__pyx_t_5) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_5, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 334, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_5);
  __Pyx_GOTREF(__pyx_v_self->mins_arr);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->mins_arr));
  __pyx_v_self->mins_arr = ((PyArrayObject *)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "pyart/map/ckdtree.pyx":335
 *         # Copy array data
 *         self.mins_arr = np.array(mins_arr, dtype=np.float64, order='C')
 *         self.maxes_arr = np.array(maxes_arr, dtype=np.float64, order='C')             # <<<<<<<<<<<<<<
 *         self.mins = <np.float64_t*>np.PyArray_DATA(self.mins_arr)
 *         self.maxes = <np.float64_t*>np.PyArray_DATA(self.maxes_arr)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_array); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_v_maxes_arr);
  __Pyx_GIVEREF(__pyx_v_maxes_arr);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_maxes_arr);
  __pyx_t_1 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_float64); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_order, __pyx_n_u_C) < 0) __PYX_ERR(0, 335, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->maxes_arr);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->maxes_arr));
  __pyx_v_self->maxes_arr = ((PyArrayObject *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "pyart/map/ckdtree.pyx":336
 *         self.mins_arr = np.array(mins_arr, dtype=np.float64, order='C')
 *         self.maxes_arr = np.array(maxes_arr, dtype=np.float64, order='C')
 *         self.mins = <np.float64_t*>np.PyArray_DATA(self.mins_arr)             # <<<<<<<<<<<<<<
 *         self.maxes = <np.float64_t*>np.PyArray_DATA(self.maxes_arr)
 *         self.m = self.mins_arr.shape[0]
 */
  __pyx_t_4 = ((PyObject *)__pyx_v_self->mins_arr);
  __Pyx_INCREF(__pyx_t_4);
  __pyx_v_self->mins = ((__pyx_t_5numpy_float64_t *)PyArray_DATA(((PyArrayObject *)__pyx_t_4)));
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyart/map/ckdtree.pyx":337
 *         self.maxes_arr = np.array(maxes_arr, dtype=np.float64, order='C')
 *         self.mins = <np.float64_t*>np.PyArray_DATA(self.mins_arr)
 *         self.maxes = <np.float64_t*>np.PyArray_DATA(self.maxes_arr)             # <<<<<<<<<<<<<<
 *         self.m = self.mins_arr.shape[0]
 * 
 */
  __pyx_t_4 = ((PyObject *)__pyx_v_self->maxes_arr);
  __Pyx_INCREF(__pyx_t_4);
  __pyx_v_self->maxes = ((__pyx_t_5numpy_float64_t *)PyArray_DATA(((PyArrayObject *)__pyx_t_4)));
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyart/map/ckdtree.pyx":338
 *         self.mins = <np.float64_t*>np.PyArray_DATA(self.mins_arr)
 *         self.maxes = <np.float64_t*>np.PyArray_DATA(self.maxes_arr)
 *         self.m = self.mins_arr.shape[0]             # <<<<<<<<<<<<<<
 * 
 * # 1-d pieces
 */
  __pyx_v_self->m = (__pyx_v_self->mins_arr->dimensions[0]);

  /* "pyart/map/ckdtree.pyx":332
 *     cdef np.ndarray mins_arr, maxes_arr
 * 
 *     def __init__(self, mins_arr, maxes_arr):             # <<<<<<<<<<<<<<
 *         # Copy array data
 *         self.mins_arr = np.array(mins_arr, dtype=np.float64, order='C')
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyart.map.ckdtree.Rectangle.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.maxes,self.mins cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_9Rectangle_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_9Rectangle_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_9Rectangle_2__reduce_cython__(((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_3map_7ckdtree_9Rectangle_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.maxes,self.mins cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.maxes,self.mins cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.maxes,self.mins cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.map.ckdtree.Rectangle.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.maxes,self.mins cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.maxes,self.mins cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_9Rectangle_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_9Rectangle_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_9Rectangle_4__setstate_cython__(((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_3map_7ckdtree_9Rectangle_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.maxes,self.mins cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.maxes,self.mins cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.maxes,self.mins cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.maxes,self.mins cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.map.ckdtree.Rectangle.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":342
 * # 1-d pieces
 * # These should only be used if p != infinity
 * cdef inline np.float64_t min_dist_point_interval_p(np.float64_t* x,             # <<<<<<<<<<<<<<
 *                                                    Rectangle rect,
 *                                                    np.intp_t k,
 */

static CYTHON_INLINE __pyx_t_5numpy_float64_t __pyx_f_5pyart_3map_7ckdtree_min_dist_point_interval_p(__pyx_t_5numpy_float64_t *__pyx_v_x, struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_rect, __pyx_t_5numpy_intp_t __pyx_v_k, __pyx_t_5numpy_float64_t __pyx_v_p) {
  __pyx_t_5numpy_float64_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("min_dist_point_interval_p", 0);

  /* "pyart/map/ckdtree.pyx":349
 *     a point in the hyperrectangle.
 *     """
 *     return dmax(0, dmax(rect.mins[k] - x[k], x[k] - rect.maxes[k])) ** p             # <<<<<<<<<<<<<<
 * 
 * cdef inline np.float64_t max_dist_point_interval_p(np.float64_t* x,
 */
  __pyx_r = pow(__pyx_f_5pyart_3map_7ckdtree_dmax(0.0, __pyx_f_5pyart_3map_7ckdtree_dmax(((__pyx_v_rect->mins[__pyx_v_k]) - (__pyx_v_x[__pyx_v_k])), ((__pyx_v_x[__pyx_v_k]) - (__pyx_v_rect->maxes[__pyx_v_k])))), __pyx_v_p);
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":342
 * # 1-d pieces
 * # These should only be used if p != infinity
 * cdef inline np.float64_t min_dist_point_interval_p(np.float64_t* x,             # <<<<<<<<<<<<<<
 *                                                    Rectangle rect,
 *                                                    np.intp_t k,
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":351
 *     return dmax(0, dmax(rect.mins[k] - x[k], x[k] - rect.maxes[k])) ** p
 * 
 * cdef inline np.float64_t max_dist_point_interval_p(np.float64_t* x,             # <<<<<<<<<<<<<<
 *                                                    Rectangle rect,
 *                                                    np.intp_t k,
 */

static CYTHON_INLINE __pyx_t_5numpy_float64_t __pyx_f_5pyart_3map_7ckdtree_max_dist_point_interval_p(__pyx_t_5numpy_float64_t *__pyx_v_x, struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_rect, __pyx_t_5numpy_intp_t __pyx_v_k, __pyx_t_5numpy_float64_t __pyx_v_p) {
  __pyx_t_5numpy_float64_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("max_dist_point_interval_p", 0);

  /* "pyart/map/ckdtree.pyx":358
 *     a point in the hyperrectangle.
 *     """
 *     return dmax(rect.maxes[k] - x[k], x[k] - rect.mins[k]) ** p             # <<<<<<<<<<<<<<
 * 
 * cdef inline np.float64_t min_dist_interval_interval_p(Rectangle rect1,
 */
  __pyx_r = pow(__pyx_f_5pyart_3map_7ckdtree_dmax(((__pyx_v_rect->maxes[__pyx_v_k]) - (__pyx_v_x[__pyx_v_k])), ((__pyx_v_x[__pyx_v_k]) - (__pyx_v_rect->mins[__pyx_v_k]))), __pyx_v_p);
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":351
 *     return dmax(0, dmax(rect.mins[k] - x[k], x[k] - rect.maxes[k])) ** p
 * 
 * cdef inline np.float64_t max_dist_point_interval_p(np.float64_t* x,             # <<<<<<<<<<<<<<
 *                                                    Rectangle rect,
 *                                                    np.intp_t k,
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":360
 *     return dmax(rect.maxes[k] - x[k], x[k] - rect.mins[k]) ** p
 * 
 * cdef inline np.float64_t min_dist_interval_interval_p(Rectangle rect1,             # <<<<<<<<<<<<<<
 *                                                       Rectangle rect2,
 *                                                       np.intp_t k,
 */

static CYTHON_INLINE __pyx_t_5numpy_float64_t __pyx_f_5pyart_3map_7ckdtree_min_dist_interval_interval_p(struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_rect1, struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_rect2, __pyx_t_5numpy_intp_t __pyx_v_k, __pyx_t_5numpy_float64_t __pyx_v_p) {
  __pyx_t_5numpy_float64_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("min_dist_interval_interval_p", 0);

  /* "pyart/map/ckdtree.pyx":368
 *     """
 *     return dmax(0, dmax(rect1.mins[k] - rect2.maxes[k],
 *                         rect2.mins[k] - rect1.maxes[k])) ** p             # <<<<<<<<<<<<<<
 * 
 * cdef inline np.float64_t max_dist_interval_interval_p(Rectangle rect1,
 */
  __pyx_r = pow(__pyx_f_5pyart_3map_7ckdtree_dmax(0.0, __pyx_f_5pyart_3map_7ckdtree_dmax(((__pyx_v_rect1->mins[__pyx_v_k]) - (__pyx_v_rect2->maxes[__pyx_v_k])), ((__pyx_v_rect2->mins[__pyx_v_k]) - (__pyx_v_rect1->maxes[__pyx_v_k])))), __pyx_v_p);
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":360
 *     return dmax(rect.maxes[k] - x[k], x[k] - rect.mins[k]) ** p
 * 
 * cdef inline np.float64_t min_dist_interval_interval_p(Rectangle rect1,             # <<<<<<<<<<<<<<
 *                                                       Rectangle rect2,
 *                                                       np.intp_t k,
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":370
 *                         rect2.mins[k] - rect1.maxes[k])) ** p
 * 
 * cdef inline np.float64_t max_dist_interval_interval_p(Rectangle rect1,             # <<<<<<<<<<<<<<
 *                                                       Rectangle rect2,
 *                                                       np.intp_t k,
 */

static CYTHON_INLINE __pyx_t_5numpy_float64_t __pyx_f_5pyart_3map_7ckdtree_max_dist_interval_interval_p(struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_rect1, struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_rect2, __pyx_t_5numpy_intp_t __pyx_v_k, __pyx_t_5numpy_float64_t __pyx_v_p) {
  __pyx_t_5numpy_float64_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("max_dist_interval_interval_p", 0);

  /* "pyart/map/ckdtree.pyx":377
 *     two hyperrectangles.
 *     """
 *     return dmax(rect1.maxes[k] - rect2.mins[k], rect2.maxes[k] - rect1.mins[k]) ** p             # <<<<<<<<<<<<<<
 * 
 * # Interval arithmetic in m-D
 */
  __pyx_r = pow(__pyx_f_5pyart_3map_7ckdtree_dmax(((__pyx_v_rect1->maxes[__pyx_v_k]) - (__pyx_v_rect2->mins[__pyx_v_k])), ((__pyx_v_rect2->maxes[__pyx_v_k]) - (__pyx_v_rect1->mins[__pyx_v_k]))), __pyx_v_p);
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":370
 *                         rect2.mins[k] - rect1.maxes[k])) ** p
 * 
 * cdef inline np.float64_t max_dist_interval_interval_p(Rectangle rect1,             # <<<<<<<<<<<<<<
 *                                                       Rectangle rect2,
 *                                                       np.intp_t k,
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":383
 * 
 * # These should be used only for p == infinity
 * cdef inline np.float64_t min_dist_point_rect_p_inf(np.float64_t* x,             # <<<<<<<<<<<<<<
 *                                                    Rectangle rect):
 *     """Compute the minimum distance between x and the given hyperrectangle."""
 */

static CYTHON_INLINE __pyx_t_5numpy_float64_t __pyx_f_5pyart_3map_7ckdtree_min_dist_point_rect_p_inf(__pyx_t_5numpy_float64_t *__pyx_v_x, struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_rect) {
  __pyx_t_5numpy_intp_t __pyx_v_i;
  __pyx_t_5numpy_float64_t __pyx_v_min_dist;
  __pyx_t_5numpy_float64_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __pyx_t_5numpy_intp_t __pyx_t_1;
  __pyx_t_5numpy_intp_t __pyx_t_2;
  __pyx_t_5numpy_intp_t __pyx_t_3;
  __Pyx_RefNannySetupContext("min_dist_point_rect_p_inf", 0);

  /* "pyart/map/ckdtree.pyx":387
 *     """Compute the minimum distance between x and the given hyperrectangle."""
 *     cdef np.intp_t i
 *     cdef np.float64_t min_dist = 0.             # <<<<<<<<<<<<<<
 *     for i in range(rect.m):
 *         min_dist = dmax(min_dist, dmax(rect.mins[i]-x[i], x[i]-rect.maxes[i]))
 */
  __pyx_v_min_dist = 0.;

  /* "pyart/map/ckdtree.pyx":388
 *     cdef np.intp_t i
 *     cdef np.float64_t min_dist = 0.
 *     for i in range(rect.m):             # <<<<<<<<<<<<<<
 *         min_dist = dmax(min_dist, dmax(rect.mins[i]-x[i], x[i]-rect.maxes[i]))
 *     return min_dist
 */
  __pyx_t_1 = __pyx_v_rect->m;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "pyart/map/ckdtree.pyx":389
 *     cdef np.float64_t min_dist = 0.
 *     for i in range(rect.m):
 *         min_dist = dmax(min_dist, dmax(rect.mins[i]-x[i], x[i]-rect.maxes[i]))             # <<<<<<<<<<<<<<
 *     return min_dist
 * 
 */
    __pyx_v_min_dist = __pyx_f_5pyart_3map_7ckdtree_dmax(__pyx_v_min_dist, __pyx_f_5pyart_3map_7ckdtree_dmax(((__pyx_v_rect->mins[__pyx_v_i]) - (__pyx_v_x[__pyx_v_i])), ((__pyx_v_x[__pyx_v_i]) - (__pyx_v_rect->maxes[__pyx_v_i]))));
  }

  /* "pyart/map/ckdtree.pyx":390
 *     for i in range(rect.m):
 *         min_dist = dmax(min_dist, dmax(rect.mins[i]-x[i], x[i]-rect.maxes[i]))
 *     return min_dist             # <<<<<<<<<<<<<<
 * 
 * cdef inline np.float64_t max_dist_point_rect_p_inf(np.float64_t* x,
 */
  __pyx_r = __pyx_v_min_dist;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":383
 * 
 * # These should be used only for p == infinity
 * cdef inline np.float64_t min_dist_point_rect_p_inf(np.float64_t* x,             # <<<<<<<<<<<<<<
 *                                                    Rectangle rect):
 *     """Compute the minimum distance between x and the given hyperrectangle."""
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":392
 *     return min_dist
 * 
 * cdef inline np.float64_t max_dist_point_rect_p_inf(np.float64_t* x,             # <<<<<<<<<<<<<<
 *                                                    Rectangle rect):
 *     """Compute the maximum distance between x and the given hyperrectangle."""
 */

static CYTHON_INLINE __pyx_t_5numpy_float64_t __pyx_f_5pyart_3map_7ckdtree_max_dist_point_rect_p_inf(__pyx_t_5numpy_float64_t *__pyx_v_x, struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_rect) {
  __pyx_t_5numpy_intp_t __pyx_v_i;
  __pyx_t_5numpy_float64_t __pyx_v_max_dist;
  __pyx_t_5numpy_float64_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __pyx_t_5numpy_intp_t __pyx_t_1;
  __pyx_t_5numpy_intp_t __pyx_t_2;
  __pyx_t_5numpy_intp_t __pyx_t_3;
  __Pyx_RefNannySetupContext("max_dist_point_rect_p_inf", 0);

  /* "pyart/map/ckdtree.pyx":396
 *     """Compute the maximum distance between x and the given hyperrectangle."""
 *     cdef np.intp_t i
 *     cdef np.float64_t max_dist = 0.             # <<<<<<<<<<<<<<
 *     for i in range(rect.m):
 *         max_dist = dmax(max_dist, dmax(rect.maxes[i]-x[i], x[i]-rect.mins[i]))
 */
  __pyx_v_max_dist = 0.;

  /* "pyart/map/ckdtree.pyx":397
 *     cdef np.intp_t i
 *     cdef np.float64_t max_dist = 0.
 *     for i in range(rect.m):             # <<<<<<<<<<<<<<
 *         max_dist = dmax(max_dist, dmax(rect.maxes[i]-x[i], x[i]-rect.mins[i]))
 *     return max_dist
 */
  __pyx_t_1 = __pyx_v_rect->m;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "pyart/map/ckdtree.pyx":398
 *     cdef np.float64_t max_dist = 0.
 *     for i in range(rect.m):
 *         max_dist = dmax(max_dist, dmax(rect.maxes[i]-x[i], x[i]-rect.mins[i]))             # <<<<<<<<<<<<<<
 *     return max_dist
 * 
 */
    __pyx_v_max_dist = __pyx_f_5pyart_3map_7ckdtree_dmax(__pyx_v_max_dist, __pyx_f_5pyart_3map_7ckdtree_dmax(((__pyx_v_rect->maxes[__pyx_v_i]) - (__pyx_v_x[__pyx_v_i])), ((__pyx_v_x[__pyx_v_i]) - (__pyx_v_rect->mins[__pyx_v_i]))));
  }

  /* "pyart/map/ckdtree.pyx":399
 *     for i in range(rect.m):
 *         max_dist = dmax(max_dist, dmax(rect.maxes[i]-x[i], x[i]-rect.mins[i]))
 *     return max_dist             # <<<<<<<<<<<<<<
 * 
 * cdef inline np.float64_t min_dist_rect_rect_p_inf(Rectangle rect1,
 */
  __pyx_r = __pyx_v_max_dist;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":392
 *     return min_dist
 * 
 * cdef inline np.float64_t max_dist_point_rect_p_inf(np.float64_t* x,             # <<<<<<<<<<<<<<
 *                                                    Rectangle rect):
 *     """Compute the maximum distance between x and the given hyperrectangle."""
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":401
 *     return max_dist
 * 
 * cdef inline np.float64_t min_dist_rect_rect_p_inf(Rectangle rect1,             # <<<<<<<<<<<<<<
 *                                                   Rectangle rect2):
 *     """Compute the minimum distance between points in two hyperrectangles."""
 */

static CYTHON_INLINE __pyx_t_5numpy_float64_t __pyx_f_5pyart_3map_7ckdtree_min_dist_rect_rect_p_inf(struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_rect1, struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_rect2) {
  __pyx_t_5numpy_intp_t __pyx_v_i;
  __pyx_t_5numpy_float64_t __pyx_v_min_dist;
  __pyx_t_5numpy_float64_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __pyx_t_5numpy_intp_t __pyx_t_1;
  __pyx_t_5numpy_intp_t __pyx_t_2;
  __pyx_t_5numpy_intp_t __pyx_t_3;
  __Pyx_RefNannySetupContext("min_dist_rect_rect_p_inf", 0);

  /* "pyart/map/ckdtree.pyx":405
 *     """Compute the minimum distance between points in two hyperrectangles."""
 *     cdef np.intp_t i
 *     cdef np.float64_t min_dist = 0.             # <<<<<<<<<<<<<<
 *     for i in range(rect1.m):
 *         min_dist = dmax(min_dist, dmax(rect1.mins[i] - rect2.maxes[i],
 */
  __pyx_v_min_dist = 0.;

  /* "pyart/map/ckdtree.pyx":406
 *     cdef np.intp_t i
 *     cdef np.float64_t min_dist = 0.
 *     for i in range(rect1.m):             # <<<<<<<<<<<<<<
 *         min_dist = dmax(min_dist, dmax(rect1.mins[i] - rect2.maxes[i],
 *                                        rect2.mins[i] - rect1.maxes[i]))
 */
  __pyx_t_1 = __pyx_v_rect1->m;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "pyart/map/ckdtree.pyx":407
 *     cdef np.float64_t min_dist = 0.
 *     for i in range(rect1.m):
 *         min_dist = dmax(min_dist, dmax(rect1.mins[i] - rect2.maxes[i],             # <<<<<<<<<<<<<<
 *                                        rect2.mins[i] - rect1.maxes[i]))
 *     return min_dist
 */
    __pyx_v_min_dist = __pyx_f_5pyart_3map_7ckdtree_dmax(__pyx_v_min_dist, __pyx_f_5pyart_3map_7ckdtree_dmax(((__pyx_v_rect1->mins[__pyx_v_i]) - (__pyx_v_rect2->maxes[__pyx_v_i])), ((__pyx_v_rect2->mins[__pyx_v_i]) - (__pyx_v_rect1->maxes[__pyx_v_i]))));
  }

  /* "pyart/map/ckdtree.pyx":409
 *         min_dist = dmax(min_dist, dmax(rect1.mins[i] - rect2.maxes[i],
 *                                        rect2.mins[i] - rect1.maxes[i]))
 *     return min_dist             # <<<<<<<<<<<<<<
 * 
 * cdef inline np.float64_t max_dist_rect_rect_p_inf(Rectangle rect1,
 */
  __pyx_r = __pyx_v_min_dist;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":401
 *     return max_dist
 * 
 * cdef inline np.float64_t min_dist_rect_rect_p_inf(Rectangle rect1,             # <<<<<<<<<<<<<<
 *                                                   Rectangle rect2):
 *     """Compute the minimum distance between points in two hyperrectangles."""
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":411
 *     return min_dist
 * 
 * cdef inline np.float64_t max_dist_rect_rect_p_inf(Rectangle rect1,             # <<<<<<<<<<<<<<
 *                                                   Rectangle rect2):
 *     """Compute the maximum distance between points in two hyperrectangles."""
 */

static CYTHON_INLINE __pyx_t_5numpy_float64_t __pyx_f_5pyart_3map_7ckdtree_max_dist_rect_rect_p_inf(struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_rect1, struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_rect2) {
  __pyx_t_5numpy_intp_t __pyx_v_i;
  __pyx_t_5numpy_float64_t __pyx_v_max_dist;
  __pyx_t_5numpy_float64_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __pyx_t_5numpy_intp_t __pyx_t_1;
  __pyx_t_5numpy_intp_t __pyx_t_2;
  __pyx_t_5numpy_intp_t __pyx_t_3;
  __Pyx_RefNannySetupContext("max_dist_rect_rect_p_inf", 0);

  /* "pyart/map/ckdtree.pyx":415
 *     """Compute the maximum distance between points in two hyperrectangles."""
 *     cdef np.intp_t i
 *     cdef np.float64_t max_dist = 0.             # <<<<<<<<<<<<<<
 *     for i in range(rect1.m):
 *         max_dist = dmax(max_dist, dmax(rect1.maxes[i] - rect2.mins[i],
 */
  __pyx_v_max_dist = 0.;

  /* "pyart/map/ckdtree.pyx":416
 *     cdef np.intp_t i
 *     cdef np.float64_t max_dist = 0.
 *     for i in range(rect1.m):             # <<<<<<<<<<<<<<
 *         max_dist = dmax(max_dist, dmax(rect1.maxes[i] - rect2.mins[i],
 *                                        rect2.maxes[i] - rect1.mins[i]))
 */
  __pyx_t_1 = __pyx_v_rect1->m;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "pyart/map/ckdtree.pyx":417
 *     cdef np.float64_t max_dist = 0.
 *     for i in range(rect1.m):
 *         max_dist = dmax(max_dist, dmax(rect1.maxes[i] - rect2.mins[i],             # <<<<<<<<<<<<<<
 *                                        rect2.maxes[i] - rect1.mins[i]))
 *     return max_dist
 */
    __pyx_v_max_dist = __pyx_f_5pyart_3map_7ckdtree_dmax(__pyx_v_max_dist, __pyx_f_5pyart_3map_7ckdtree_dmax(((__pyx_v_rect1->maxes[__pyx_v_i]) - (__pyx_v_rect2->mins[__pyx_v_i])), ((__pyx_v_rect2->maxes[__pyx_v_i]) - (__pyx_v_rect1->mins[__pyx_v_i]))));
  }

  /* "pyart/map/ckdtree.pyx":419
 *         max_dist = dmax(max_dist, dmax(rect1.maxes[i] - rect2.mins[i],
 *                                        rect2.maxes[i] - rect1.mins[i]))
 *     return max_dist             # <<<<<<<<<<<<<<
 * 
 * # Rectangle-to-rectangle distance tracker
 */
  __pyx_r = __pyx_v_max_dist;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":411
 *     return min_dist
 * 
 * cdef inline np.float64_t max_dist_rect_rect_p_inf(Rectangle rect1,             # <<<<<<<<<<<<<<
 *                                                   Rectangle rect2):
 *     """Compute the maximum distance between points in two hyperrectangles."""
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":466
 * 
 *     # Stack handling
 *     cdef int _init_stack(self) except -1:             # <<<<<<<<<<<<<<
 *         cdef void *tmp
 *         self.stack_max_size = 10
 */

static int __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker__init_stack(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_self) {
  void *__pyx_v_tmp;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_init_stack", 0);

  /* "pyart/map/ckdtree.pyx":468
 *     cdef int _init_stack(self) except -1:
 *         cdef void *tmp
 *         self.stack_max_size = 10             # <<<<<<<<<<<<<<
 *         tmp = stdlib.malloc(sizeof(RR_stack_item) *
 *                             self.stack_max_size)
 */
  __pyx_v_self->stack_max_size = 10;

  /* "pyart/map/ckdtree.pyx":469
 *         cdef void *tmp
 *         self.stack_max_size = 10
 *         tmp = stdlib.malloc(sizeof(RR_stack_item) *             # <<<<<<<<<<<<<<
 *                             self.stack_max_size)
 *         if tmp == NULL:
 */
  __pyx_v_tmp = malloc(((sizeof(struct __pyx_t_5pyart_3map_7ckdtree_RR_stack_item)) * __pyx_v_self->stack_max_size));

  /* "pyart/map/ckdtree.pyx":471
 *         tmp = stdlib.malloc(sizeof(RR_stack_item) *
 *                             self.stack_max_size)
 *         if tmp == NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError
 *         self.stack = <RR_stack_item*> tmp
 */
  __pyx_t_1 = ((__pyx_v_tmp == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "pyart/map/ckdtree.pyx":472
 *                             self.stack_max_size)
 *         if tmp == NULL:
 *             raise MemoryError             # <<<<<<<<<<<<<<
 *         self.stack = <RR_stack_item*> tmp
 *         self.stack_size = 0
 */
    PyErr_NoMemory(); __PYX_ERR(0, 472, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":471
 *         tmp = stdlib.malloc(sizeof(RR_stack_item) *
 *                             self.stack_max_size)
 *         if tmp == NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError
 *         self.stack = <RR_stack_item*> tmp
 */
  }

  /* "pyart/map/ckdtree.pyx":473
 *         if tmp == NULL:
 *             raise MemoryError
 *         self.stack = <RR_stack_item*> tmp             # <<<<<<<<<<<<<<
 *         self.stack_size = 0
 *         return 0
 */
  __pyx_v_self->stack = ((struct __pyx_t_5pyart_3map_7ckdtree_RR_stack_item *)__pyx_v_tmp);

  /* "pyart/map/ckdtree.pyx":474
 *             raise MemoryError
 *         self.stack = <RR_stack_item*> tmp
 *         self.stack_size = 0             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
  __pyx_v_self->stack_size = 0;

  /* "pyart/map/ckdtree.pyx":475
 *         self.stack = <RR_stack_item*> tmp
 *         self.stack_size = 0
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     cdef int _resize_stack(self, np.intp_t new_max_size) except -1:
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":466
 * 
 *     # Stack handling
 *     cdef int _init_stack(self) except -1:             # <<<<<<<<<<<<<<
 *         cdef void *tmp
 *         self.stack_max_size = 10
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.RectRectDistanceTracker._init_stack", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":477
 *         return 0
 * 
 *     cdef int _resize_stack(self, np.intp_t new_max_size) except -1:             # <<<<<<<<<<<<<<
 *         cdef void *tmp
 *         self.stack_max_size = new_max_size
 */

static int __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker__resize_stack(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_self, __pyx_t_5numpy_intp_t __pyx_v_new_max_size) {
  void *__pyx_v_tmp;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_resize_stack", 0);

  /* "pyart/map/ckdtree.pyx":479
 *     cdef int _resize_stack(self, np.intp_t new_max_size) except -1:
 *         cdef void *tmp
 *         self.stack_max_size = new_max_size             # <<<<<<<<<<<<<<
 *         tmp = stdlib.realloc(<RR_stack_item*> self.stack,
 *                              new_max_size * sizeof(RR_stack_item))
 */
  __pyx_v_self->stack_max_size = __pyx_v_new_max_size;

  /* "pyart/map/ckdtree.pyx":480
 *         cdef void *tmp
 *         self.stack_max_size = new_max_size
 *         tmp = stdlib.realloc(<RR_stack_item*> self.stack,             # <<<<<<<<<<<<<<
 *                              new_max_size * sizeof(RR_stack_item))
 *         if tmp == NULL:
 */
  __pyx_v_tmp = realloc(((struct __pyx_t_5pyart_3map_7ckdtree_RR_stack_item *)__pyx_v_self->stack), (__pyx_v_new_max_size * (sizeof(struct __pyx_t_5pyart_3map_7ckdtree_RR_stack_item))));

  /* "pyart/map/ckdtree.pyx":482
 *         tmp = stdlib.realloc(<RR_stack_item*> self.stack,
 *                              new_max_size * sizeof(RR_stack_item))
 *         if tmp == NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError
 *         self.stack = <RR_stack_item*> tmp
 */
  __pyx_t_1 = ((__pyx_v_tmp == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "pyart/map/ckdtree.pyx":483
 *                              new_max_size * sizeof(RR_stack_item))
 *         if tmp == NULL:
 *             raise MemoryError             # <<<<<<<<<<<<<<
 *         self.stack = <RR_stack_item*> tmp
 *         return 0
 */
    PyErr_NoMemory(); __PYX_ERR(0, 483, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":482
 *         tmp = stdlib.realloc(<RR_stack_item*> self.stack,
 *                              new_max_size * sizeof(RR_stack_item))
 *         if tmp == NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError
 *         self.stack = <RR_stack_item*> tmp
 */
  }

  /* "pyart/map/ckdtree.pyx":484
 *         if tmp == NULL:
 *             raise MemoryError
 *         self.stack = <RR_stack_item*> tmp             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
  __pyx_v_self->stack = ((struct __pyx_t_5pyart_3map_7ckdtree_RR_stack_item *)__pyx_v_tmp);

  /* "pyart/map/ckdtree.pyx":485
 *             raise MemoryError
 *         self.stack = <RR_stack_item*> tmp
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     cdef int _free_stack(self) except -1:
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":477
 *         return 0
 * 
 *     cdef int _resize_stack(self, np.intp_t new_max_size) except -1:             # <<<<<<<<<<<<<<
 *         cdef void *tmp
 *         self.stack_max_size = new_max_size
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.RectRectDistanceTracker._resize_stack", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":487
 *         return 0
 * 
 *     cdef int _free_stack(self) except -1:             # <<<<<<<<<<<<<<
 *         if self.stack != <RR_stack_item*> NULL:
 *             stdlib.free(self.stack)
 */

static int __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker__free_stack(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("_free_stack", 0);

  /* "pyart/map/ckdtree.pyx":488
 * 
 *     cdef int _free_stack(self) except -1:
 *         if self.stack != <RR_stack_item*> NULL:             # <<<<<<<<<<<<<<
 *             stdlib.free(self.stack)
 *         return 0
 */
  __pyx_t_1 = ((__pyx_v_self->stack != ((struct __pyx_t_5pyart_3map_7ckdtree_RR_stack_item *)NULL)) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":489
 *     cdef int _free_stack(self) except -1:
 *         if self.stack != <RR_stack_item*> NULL:
 *             stdlib.free(self.stack)             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
    free(__pyx_v_self->stack);

    /* "pyart/map/ckdtree.pyx":488
 * 
 *     cdef int _free_stack(self) except -1:
 *         if self.stack != <RR_stack_item*> NULL:             # <<<<<<<<<<<<<<
 *             stdlib.free(self.stack)
 *         return 0
 */
  }

  /* "pyart/map/ckdtree.pyx":490
 *         if self.stack != <RR_stack_item*> NULL:
 *             stdlib.free(self.stack)
 *         return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":487
 *         return 0
 * 
 *     cdef int _free_stack(self) except -1:             # <<<<<<<<<<<<<<
 *         if self.stack != <RR_stack_item*> NULL:
 *             stdlib.free(self.stack)
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":493
 * 
 * 
 *     def __init__(self, Rectangle rect1, Rectangle rect2,             # <<<<<<<<<<<<<<
 *                  np.float64_t p, np.float64_t eps, np.float64_t upper_bound):
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_3map_7ckdtree_23RectRectDistanceTracker_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pyart_3map_7ckdtree_23RectRectDistanceTracker_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_rect1 = 0;
  struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_rect2 = 0;
  __pyx_t_5numpy_float64_t __pyx_v_p;
  __pyx_t_5numpy_float64_t __pyx_v_eps;
  __pyx_t_5numpy_float64_t __pyx_v_upper_bound;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_rect1,&__pyx_n_s_rect2,&__pyx_n_s_p,&__pyx_n_s_eps,&__pyx_n_s_upper_bound,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_rect1)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_rect2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 5, 5, 1); __PYX_ERR(0, 493, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_p)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 5, 5, 2); __PYX_ERR(0, 493, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_eps)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 5, 5, 3); __PYX_ERR(0, 493, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_upper_bound)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 5, 5, 4); __PYX_ERR(0, 493, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 493, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 5) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
    }
    __pyx_v_rect1 = ((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)values[0]);
    __pyx_v_rect2 = ((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)values[1]);
    __pyx_v_p = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_p == ((npy_float64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 494, __pyx_L3_error)
    __pyx_v_eps = __pyx_PyFloat_AsDouble(values[3]); if (unlikely((__pyx_v_eps == ((npy_float64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 494, __pyx_L3_error)
    __pyx_v_upper_bound = __pyx_PyFloat_AsDouble(values[4]); if (unlikely((__pyx_v_upper_bound == ((npy_float64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 494, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 5, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 493, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.RectRectDistanceTracker.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_rect1), __pyx_ptype_5pyart_3map_7ckdtree_Rectangle, 1, "rect1", 0))) __PYX_ERR(0, 493, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_rect2), __pyx_ptype_5pyart_3map_7ckdtree_Rectangle, 1, "rect2", 0))) __PYX_ERR(0, 493, __pyx_L1_error)
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_23RectRectDistanceTracker___init__(((struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *)__pyx_v_self), __pyx_v_rect1, __pyx_v_rect2, __pyx_v_p, __pyx_v_eps, __pyx_v_upper_bound);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_3map_7ckdtree_23RectRectDistanceTracker___init__(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_self, struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_rect1, struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_rect2, __pyx_t_5numpy_float64_t __pyx_v_p, __pyx_t_5numpy_float64_t __pyx_v_eps, __pyx_t_5numpy_float64_t __pyx_v_upper_bound) {
  PyObject *__pyx_v_i = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  __pyx_t_5numpy_float64_t __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  Py_ssize_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  __pyx_t_5numpy_intp_t __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pyart/map/ckdtree.pyx":496
 *                  np.float64_t p, np.float64_t eps, np.float64_t upper_bound):
 * 
 *         if rect1.m != rect2.m:             # <<<<<<<<<<<<<<
 *             raise ValueError("rect1 and rect2 have different dimensions")
 * 
 */
  __pyx_t_1 = ((__pyx_v_rect1->m != __pyx_v_rect2->m) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "pyart/map/ckdtree.pyx":497
 * 
 *         if rect1.m != rect2.m:
 *             raise ValueError("rect1 and rect2 have different dimensions")             # <<<<<<<<<<<<<<
 * 
 *         self.rect1 = rect1
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 497, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 497, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":496
 *                  np.float64_t p, np.float64_t eps, np.float64_t upper_bound):
 * 
 *         if rect1.m != rect2.m:             # <<<<<<<<<<<<<<
 *             raise ValueError("rect1 and rect2 have different dimensions")
 * 
 */
  }

  /* "pyart/map/ckdtree.pyx":499
 *             raise ValueError("rect1 and rect2 have different dimensions")
 * 
 *         self.rect1 = rect1             # <<<<<<<<<<<<<<
 *         self.rect2 = rect2
 *         self.p = p
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_rect1));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_rect1));
  __Pyx_GOTREF(__pyx_v_self->rect1);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->rect1));
  __pyx_v_self->rect1 = __pyx_v_rect1;

  /* "pyart/map/ckdtree.pyx":500
 * 
 *         self.rect1 = rect1
 *         self.rect2 = rect2             # <<<<<<<<<<<<<<
 *         self.p = p
 * 
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_rect2));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_rect2));
  __Pyx_GOTREF(__pyx_v_self->rect2);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->rect2));
  __pyx_v_self->rect2 = __pyx_v_rect2;

  /* "pyart/map/ckdtree.pyx":501
 *         self.rect1 = rect1
 *         self.rect2 = rect2
 *         self.p = p             # <<<<<<<<<<<<<<
 * 
 *         # internally we represent all distances as distance ** p
 */
  __pyx_v_self->p = __pyx_v_p;

  /* "pyart/map/ckdtree.pyx":504
 * 
 *         # internally we represent all distances as distance ** p
 *         if p != infinity and upper_bound != infinity:             # <<<<<<<<<<<<<<
 *             self.upper_bound = upper_bound ** p
 *         else:
 */
  __pyx_t_3 = ((__pyx_v_p != __pyx_v_5pyart_3map_7ckdtree_infinity) != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_3 = ((__pyx_v_upper_bound != __pyx_v_5pyart_3map_7ckdtree_infinity) != 0);
  __pyx_t_1 = __pyx_t_3;
  __pyx_L5_bool_binop_done:;
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":505
 *         # internally we represent all distances as distance ** p
 *         if p != infinity and upper_bound != infinity:
 *             self.upper_bound = upper_bound ** p             # <<<<<<<<<<<<<<
 *         else:
 *             self.upper_bound = upper_bound
 */
    __pyx_v_self->upper_bound = pow(__pyx_v_upper_bound, __pyx_v_p);

    /* "pyart/map/ckdtree.pyx":504
 * 
 *         # internally we represent all distances as distance ** p
 *         if p != infinity and upper_bound != infinity:             # <<<<<<<<<<<<<<
 *             self.upper_bound = upper_bound ** p
 *         else:
 */
    goto __pyx_L4;
  }

  /* "pyart/map/ckdtree.pyx":507
 *             self.upper_bound = upper_bound ** p
 *         else:
 *             self.upper_bound = upper_bound             # <<<<<<<<<<<<<<
 * 
 *         # fiddle approximation factor
 */
  /*else*/ {
    __pyx_v_self->upper_bound = __pyx_v_upper_bound;
  }
  __pyx_L4:;

  /* "pyart/map/ckdtree.pyx":510
 * 
 *         # fiddle approximation factor
 *         if eps == 0:             # <<<<<<<<<<<<<<
 *             self.epsfac = 1
 *         elif p == infinity:
 */
  __pyx_t_1 = ((__pyx_v_eps == 0.0) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":511
 *         # fiddle approximation factor
 *         if eps == 0:
 *             self.epsfac = 1             # <<<<<<<<<<<<<<
 *         elif p == infinity:
 *             self.epsfac = 1 / (1 + eps)
 */
    __pyx_v_self->epsfac = 1.0;

    /* "pyart/map/ckdtree.pyx":510
 * 
 *         # fiddle approximation factor
 *         if eps == 0:             # <<<<<<<<<<<<<<
 *             self.epsfac = 1
 *         elif p == infinity:
 */
    goto __pyx_L7;
  }

  /* "pyart/map/ckdtree.pyx":512
 *         if eps == 0:
 *             self.epsfac = 1
 *         elif p == infinity:             # <<<<<<<<<<<<<<
 *             self.epsfac = 1 / (1 + eps)
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_p == __pyx_v_5pyart_3map_7ckdtree_infinity) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":513
 *             self.epsfac = 1
 *         elif p == infinity:
 *             self.epsfac = 1 / (1 + eps)             # <<<<<<<<<<<<<<
 *         else:
 *             self.epsfac = 1 / (1 + eps) ** p
 */
    __pyx_t_4 = (1.0 + __pyx_v_eps);
    if (unlikely(__pyx_t_4 == 0)) {
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      __PYX_ERR(0, 513, __pyx_L1_error)
    }
    __pyx_v_self->epsfac = (1.0 / __pyx_t_4);

    /* "pyart/map/ckdtree.pyx":512
 *         if eps == 0:
 *             self.epsfac = 1
 *         elif p == infinity:             # <<<<<<<<<<<<<<
 *             self.epsfac = 1 / (1 + eps)
 *         else:
 */
    goto __pyx_L7;
  }

  /* "pyart/map/ckdtree.pyx":515
 *             self.epsfac = 1 / (1 + eps)
 *         else:
 *             self.epsfac = 1 / (1 + eps) ** p             # <<<<<<<<<<<<<<
 * 
 *         self._init_stack()
 */
  /*else*/ {
    __pyx_t_4 = pow((1.0 + __pyx_v_eps), __pyx_v_p);
    if (unlikely(__pyx_t_4 == 0)) {
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      __PYX_ERR(0, 515, __pyx_L1_error)
    }
    __pyx_v_self->epsfac = (1.0 / __pyx_t_4);
  }
  __pyx_L7:;

  /* "pyart/map/ckdtree.pyx":517
 *             self.epsfac = 1 / (1 + eps) ** p
 * 
 *         self._init_stack()             # <<<<<<<<<<<<<<
 * 
 *         # Compute initial min and max distances
 */
  __pyx_t_5 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_RectRectDistanceTracker *)__pyx_v_self->__pyx_vtab)->_init_stack(__pyx_v_self); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 517, __pyx_L1_error)

  /* "pyart/map/ckdtree.pyx":520
 * 
 *         # Compute initial min and max distances
 *         if self.p == infinity:             # <<<<<<<<<<<<<<
 *             self.min_distance = min_dist_rect_rect_p_inf(rect1, rect2)
 *             self.max_distance = max_dist_rect_rect_p_inf(rect1, rect2)
 */
  __pyx_t_1 = ((__pyx_v_self->p == __pyx_v_5pyart_3map_7ckdtree_infinity) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":521
 *         # Compute initial min and max distances
 *         if self.p == infinity:
 *             self.min_distance = min_dist_rect_rect_p_inf(rect1, rect2)             # <<<<<<<<<<<<<<
 *             self.max_distance = max_dist_rect_rect_p_inf(rect1, rect2)
 *         else:
 */
    __pyx_v_self->min_distance = __pyx_f_5pyart_3map_7ckdtree_min_dist_rect_rect_p_inf(__pyx_v_rect1, __pyx_v_rect2);

    /* "pyart/map/ckdtree.pyx":522
 *         if self.p == infinity:
 *             self.min_distance = min_dist_rect_rect_p_inf(rect1, rect2)
 *             self.max_distance = max_dist_rect_rect_p_inf(rect1, rect2)             # <<<<<<<<<<<<<<
 *         else:
 *             self.min_distance = 0.
 */
    __pyx_v_self->max_distance = __pyx_f_5pyart_3map_7ckdtree_max_dist_rect_rect_p_inf(__pyx_v_rect1, __pyx_v_rect2);

    /* "pyart/map/ckdtree.pyx":520
 * 
 *         # Compute initial min and max distances
 *         if self.p == infinity:             # <<<<<<<<<<<<<<
 *             self.min_distance = min_dist_rect_rect_p_inf(rect1, rect2)
 *             self.max_distance = max_dist_rect_rect_p_inf(rect1, rect2)
 */
    goto __pyx_L8;
  }

  /* "pyart/map/ckdtree.pyx":524
 *             self.max_distance = max_dist_rect_rect_p_inf(rect1, rect2)
 *         else:
 *             self.min_distance = 0.             # <<<<<<<<<<<<<<
 *             self.max_distance = 0.
 *             for i in range(rect1.m):
 */
  /*else*/ {
    __pyx_v_self->min_distance = 0.;

    /* "pyart/map/ckdtree.pyx":525
 *         else:
 *             self.min_distance = 0.
 *             self.max_distance = 0.             # <<<<<<<<<<<<<<
 *             for i in range(rect1.m):
 *                 self.min_distance += min_dist_interval_interval_p(rect1, rect2, i, p)
 */
    __pyx_v_self->max_distance = 0.;

    /* "pyart/map/ckdtree.pyx":526
 *             self.min_distance = 0.
 *             self.max_distance = 0.
 *             for i in range(rect1.m):             # <<<<<<<<<<<<<<
 *                 self.min_distance += min_dist_interval_interval_p(rect1, rect2, i, p)
 *                 self.max_distance += max_dist_interval_interval_p(rect1, rect2, i, p)
 */
    __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_rect1->m); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 526, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_t_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 526, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (likely(PyList_CheckExact(__pyx_t_6)) || PyTuple_CheckExact(__pyx_t_6)) {
      __pyx_t_2 = __pyx_t_6; __Pyx_INCREF(__pyx_t_2); __pyx_t_7 = 0;
      __pyx_t_8 = NULL;
    } else {
      __pyx_t_7 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 526, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_8 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 526, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    for (;;) {
      if (likely(!__pyx_t_8)) {
        if (likely(PyList_CheckExact(__pyx_t_2))) {
          if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_6 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_7); __Pyx_INCREF(__pyx_t_6); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 526, __pyx_L1_error)
          #else
          __pyx_t_6 = PySequence_ITEM(__pyx_t_2, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 526, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          #endif
        } else {
          if (__pyx_t_7 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_6 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_7); __Pyx_INCREF(__pyx_t_6); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 526, __pyx_L1_error)
          #else
          __pyx_t_6 = PySequence_ITEM(__pyx_t_2, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 526, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          #endif
        }
      } else {
        __pyx_t_6 = __pyx_t_8(__pyx_t_2);
        if (unlikely(!__pyx_t_6)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 526, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_6);
      }
      __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "pyart/map/ckdtree.pyx":527
 *             self.max_distance = 0.
 *             for i in range(rect1.m):
 *                 self.min_distance += min_dist_interval_interval_p(rect1, rect2, i, p)             # <<<<<<<<<<<<<<
 *                 self.max_distance += max_dist_interval_interval_p(rect1, rect2, i, p)
 * 
 */
      __pyx_t_9 = __Pyx_PyInt_As_Py_intptr_t(__pyx_v_i); if (unlikely((__pyx_t_9 == ((npy_intp)-1)) && PyErr_Occurred())) __PYX_ERR(0, 527, __pyx_L1_error)
      __pyx_v_self->min_distance = (__pyx_v_self->min_distance + __pyx_f_5pyart_3map_7ckdtree_min_dist_interval_interval_p(__pyx_v_rect1, __pyx_v_rect2, __pyx_t_9, __pyx_v_p));

      /* "pyart/map/ckdtree.pyx":528
 *             for i in range(rect1.m):
 *                 self.min_distance += min_dist_interval_interval_p(rect1, rect2, i, p)
 *                 self.max_distance += max_dist_interval_interval_p(rect1, rect2, i, p)             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
      __pyx_t_9 = __Pyx_PyInt_As_Py_intptr_t(__pyx_v_i); if (unlikely((__pyx_t_9 == ((npy_intp)-1)) && PyErr_Occurred())) __PYX_ERR(0, 528, __pyx_L1_error)
      __pyx_v_self->max_distance = (__pyx_v_self->max_distance + __pyx_f_5pyart_3map_7ckdtree_max_dist_interval_interval_p(__pyx_v_rect1, __pyx_v_rect2, __pyx_t_9, __pyx_v_p));

      /* "pyart/map/ckdtree.pyx":526
 *             self.min_distance = 0.
 *             self.max_distance = 0.
 *             for i in range(rect1.m):             # <<<<<<<<<<<<<<
 *                 self.min_distance += min_dist_interval_interval_p(rect1, rect2, i, p)
 *                 self.max_distance += max_dist_interval_interval_p(rect1, rect2, i, p)
 */
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_L8:;

  /* "pyart/map/ckdtree.pyx":493
 * 
 * 
 *     def __init__(self, Rectangle rect1, Rectangle rect2,             # <<<<<<<<<<<<<<
 *                  np.float64_t p, np.float64_t eps, np.float64_t upper_bound):
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pyart.map.ckdtree.RectRectDistanceTracker.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":530
 *                 self.max_distance += max_dist_interval_interval_p(rect1, rect2, i, p)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         self._free_stack()
 * 
 */

/* Python wrapper */
static void __pyx_pw_5pyart_3map_7ckdtree_23RectRectDistanceTracker_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5pyart_3map_7ckdtree_23RectRectDistanceTracker_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5pyart_3map_7ckdtree_23RectRectDistanceTracker_2__dealloc__(((struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5pyart_3map_7ckdtree_23RectRectDistanceTracker_2__dealloc__(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "pyart/map/ckdtree.pyx":531
 * 
 *     def __dealloc__(self):
 *         self._free_stack()             # <<<<<<<<<<<<<<
 * 
 *     cdef int push(self, np.intp_t which, np.intp_t direction,
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_RectRectDistanceTracker *)__pyx_v_self->__pyx_vtab)->_free_stack(__pyx_v_self); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 531, __pyx_L1_error)

  /* "pyart/map/ckdtree.pyx":530
 *                 self.max_distance += max_dist_interval_interval_p(rect1, rect2, i, p)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         self._free_stack()
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pyart.map.ckdtree.RectRectDistanceTracker.__dealloc__", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pyart/map/ckdtree.pyx":533
 *         self._free_stack()
 * 
 *     cdef int push(self, np.intp_t which, np.intp_t direction,             # <<<<<<<<<<<<<<
 *                   np.intp_t split_dim,
 *                   np.float64_t split_val) except -1:
 */

static int __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_self, __pyx_t_5numpy_intp_t __pyx_v_which, __pyx_t_5numpy_intp_t __pyx_v_direction, __pyx_t_5numpy_intp_t __pyx_v_split_dim, __pyx_t_5numpy_float64_t __pyx_v_split_val) {
  struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_rect = 0;
  struct __pyx_t_5pyart_3map_7ckdtree_RR_stack_item *__pyx_v_item;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  __pyx_t_5numpy_float64_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);

  /* "pyart/map/ckdtree.pyx":538
 * 
 *         cdef Rectangle rect
 *         if which == 1:             # <<<<<<<<<<<<<<
 *             rect = self.rect1
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_which == 1) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":539
 *         cdef Rectangle rect
 *         if which == 1:
 *             rect = self.rect1             # <<<<<<<<<<<<<<
 *         else:
 *             rect = self.rect2
 */
    __pyx_t_2 = ((PyObject *)__pyx_v_self->rect1);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_rect = ((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "pyart/map/ckdtree.pyx":538
 * 
 *         cdef Rectangle rect
 *         if which == 1:             # <<<<<<<<<<<<<<
 *             rect = self.rect1
 *         else:
 */
    goto __pyx_L3;
  }

  /* "pyart/map/ckdtree.pyx":541
 *             rect = self.rect1
 *         else:
 *             rect = self.rect2             # <<<<<<<<<<<<<<
 * 
 *         # Push onto stack
 */
  /*else*/ {
    __pyx_t_2 = ((PyObject *)__pyx_v_self->rect2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_rect = ((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)__pyx_t_2);
    __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "pyart/map/ckdtree.pyx":544
 * 
 *         # Push onto stack
 *         if self.stack_size == self.stack_max_size:             # <<<<<<<<<<<<<<
 *             self._resize_stack(self.stack_max_size * 2)
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->stack_size == __pyx_v_self->stack_max_size) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":545
 *         # Push onto stack
 *         if self.stack_size == self.stack_max_size:
 *             self._resize_stack(self.stack_max_size * 2)             # <<<<<<<<<<<<<<
 * 
 *         cdef RR_stack_item *item = &self.stack[self.stack_size]
 */
    __pyx_t_3 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_RectRectDistanceTracker *)__pyx_v_self->__pyx_vtab)->_resize_stack(__pyx_v_self, (__pyx_v_self->stack_max_size * 2)); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 545, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":544
 * 
 *         # Push onto stack
 *         if self.stack_size == self.stack_max_size:             # <<<<<<<<<<<<<<
 *             self._resize_stack(self.stack_max_size * 2)
 * 
 */
  }

  /* "pyart/map/ckdtree.pyx":547
 *             self._resize_stack(self.stack_max_size * 2)
 * 
 *         cdef RR_stack_item *item = &self.stack[self.stack_size]             # <<<<<<<<<<<<<<
 *         self.stack_size += 1
 *         item.which = which
 */
  __pyx_v_item = (&(__pyx_v_self->stack[__pyx_v_self->stack_size]));

  /* "pyart/map/ckdtree.pyx":548
 * 
 *         cdef RR_stack_item *item = &self.stack[self.stack_size]
 *         self.stack_size += 1             # <<<<<<<<<<<<<<
 *         item.which = which
 *         item.split_dim = split_dim
 */
  __pyx_v_self->stack_size = (__pyx_v_self->stack_size + 1);

  /* "pyart/map/ckdtree.pyx":549
 *         cdef RR_stack_item *item = &self.stack[self.stack_size]
 *         self.stack_size += 1
 *         item.which = which             # <<<<<<<<<<<<<<
 *         item.split_dim = split_dim
 *         item.min_distance = self.min_distance
 */
  __pyx_v_item->which = __pyx_v_which;

  /* "pyart/map/ckdtree.pyx":550
 *         self.stack_size += 1
 *         item.which = which
 *         item.split_dim = split_dim             # <<<<<<<<<<<<<<
 *         item.min_distance = self.min_distance
 *         item.max_distance = self.max_distance
 */
  __pyx_v_item->split_dim = __pyx_v_split_dim;

  /* "pyart/map/ckdtree.pyx":551
 *         item.which = which
 *         item.split_dim = split_dim
 *         item.min_distance = self.min_distance             # <<<<<<<<<<<<<<
 *         item.max_distance = self.max_distance
 *         item.min_along_dim = rect.mins[split_dim]
 */
  __pyx_t_4 = __pyx_v_self->min_distance;
  __pyx_v_item->min_distance = __pyx_t_4;

  /* "pyart/map/ckdtree.pyx":552
 *         item.split_dim = split_dim
 *         item.min_distance = self.min_distance
 *         item.max_distance = self.max_distance             # <<<<<<<<<<<<<<
 *         item.min_along_dim = rect.mins[split_dim]
 *         item.max_along_dim = rect.maxes[split_dim]
 */
  __pyx_t_4 = __pyx_v_self->max_distance;
  __pyx_v_item->max_distance = __pyx_t_4;

  /* "pyart/map/ckdtree.pyx":553
 *         item.min_distance = self.min_distance
 *         item.max_distance = self.max_distance
 *         item.min_along_dim = rect.mins[split_dim]             # <<<<<<<<<<<<<<
 *         item.max_along_dim = rect.maxes[split_dim]
 * 
 */
  __pyx_v_item->min_along_dim = (__pyx_v_rect->mins[__pyx_v_split_dim]);

  /* "pyart/map/ckdtree.pyx":554
 *         item.max_distance = self.max_distance
 *         item.min_along_dim = rect.mins[split_dim]
 *         item.max_along_dim = rect.maxes[split_dim]             # <<<<<<<<<<<<<<
 * 
 *         # Update min/max distances
 */
  __pyx_v_item->max_along_dim = (__pyx_v_rect->maxes[__pyx_v_split_dim]);

  /* "pyart/map/ckdtree.pyx":557
 * 
 *         # Update min/max distances
 *         if self.p != infinity:             # <<<<<<<<<<<<<<
 *             self.min_distance -= min_dist_interval_interval_p(self.rect1, self.rect2, split_dim, self.p)
 *             self.max_distance -= max_dist_interval_interval_p(self.rect1, self.rect2, split_dim, self.p)
 */
  __pyx_t_1 = ((__pyx_v_self->p != __pyx_v_5pyart_3map_7ckdtree_infinity) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":558
 *         # Update min/max distances
 *         if self.p != infinity:
 *             self.min_distance -= min_dist_interval_interval_p(self.rect1, self.rect2, split_dim, self.p)             # <<<<<<<<<<<<<<
 *             self.max_distance -= max_dist_interval_interval_p(self.rect1, self.rect2, split_dim, self.p)
 * 
 */
    __pyx_t_2 = ((PyObject *)__pyx_v_self->rect1);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_5 = ((PyObject *)__pyx_v_self->rect2);
    __Pyx_INCREF(__pyx_t_5);
    __pyx_v_self->min_distance = (__pyx_v_self->min_distance - __pyx_f_5pyart_3map_7ckdtree_min_dist_interval_interval_p(((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)__pyx_t_2), ((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)__pyx_t_5), __pyx_v_split_dim, __pyx_v_self->p));
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "pyart/map/ckdtree.pyx":559
 *         if self.p != infinity:
 *             self.min_distance -= min_dist_interval_interval_p(self.rect1, self.rect2, split_dim, self.p)
 *             self.max_distance -= max_dist_interval_interval_p(self.rect1, self.rect2, split_dim, self.p)             # <<<<<<<<<<<<<<
 * 
 *         if direction == LESS:
 */
    __pyx_t_5 = ((PyObject *)__pyx_v_self->rect1);
    __Pyx_INCREF(__pyx_t_5);
    __pyx_t_2 = ((PyObject *)__pyx_v_self->rect2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_self->max_distance = (__pyx_v_self->max_distance - __pyx_f_5pyart_3map_7ckdtree_max_dist_interval_interval_p(((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)__pyx_t_5), ((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)__pyx_t_2), __pyx_v_split_dim, __pyx_v_self->p));
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyart/map/ckdtree.pyx":557
 * 
 *         # Update min/max distances
 *         if self.p != infinity:             # <<<<<<<<<<<<<<
 *             self.min_distance -= min_dist_interval_interval_p(self.rect1, self.rect2, split_dim, self.p)
 *             self.max_distance -= max_dist_interval_interval_p(self.rect1, self.rect2, split_dim, self.p)
 */
  }

  /* "pyart/map/ckdtree.pyx":561
 *             self.max_distance -= max_dist_interval_interval_p(self.rect1, self.rect2, split_dim, self.p)
 * 
 *         if direction == LESS:             # <<<<<<<<<<<<<<
 *             rect.maxes[split_dim] = split_val
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_direction == __pyx_v_5pyart_3map_7ckdtree_LESS) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":562
 * 
 *         if direction == LESS:
 *             rect.maxes[split_dim] = split_val             # <<<<<<<<<<<<<<
 *         else:
 *             rect.mins[split_dim] = split_val
 */
    (__pyx_v_rect->maxes[__pyx_v_split_dim]) = __pyx_v_split_val;

    /* "pyart/map/ckdtree.pyx":561
 *             self.max_distance -= max_dist_interval_interval_p(self.rect1, self.rect2, split_dim, self.p)
 * 
 *         if direction == LESS:             # <<<<<<<<<<<<<<
 *             rect.maxes[split_dim] = split_val
 *         else:
 */
    goto __pyx_L6;
  }

  /* "pyart/map/ckdtree.pyx":564
 *             rect.maxes[split_dim] = split_val
 *         else:
 *             rect.mins[split_dim] = split_val             # <<<<<<<<<<<<<<
 * 
 *         if self.p != infinity:
 */
  /*else*/ {
    (__pyx_v_rect->mins[__pyx_v_split_dim]) = __pyx_v_split_val;
  }
  __pyx_L6:;

  /* "pyart/map/ckdtree.pyx":566
 *             rect.mins[split_dim] = split_val
 * 
 *         if self.p != infinity:             # <<<<<<<<<<<<<<
 *             self.min_distance += min_dist_interval_interval_p(self.rect1, self.rect2, split_dim, self.p)
 *             self.max_distance += max_dist_interval_interval_p(self.rect1, self.rect2, split_dim, self.p)
 */
  __pyx_t_1 = ((__pyx_v_self->p != __pyx_v_5pyart_3map_7ckdtree_infinity) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":567
 * 
 *         if self.p != infinity:
 *             self.min_distance += min_dist_interval_interval_p(self.rect1, self.rect2, split_dim, self.p)             # <<<<<<<<<<<<<<
 *             self.max_distance += max_dist_interval_interval_p(self.rect1, self.rect2, split_dim, self.p)
 *         else:
 */
    __pyx_t_2 = ((PyObject *)__pyx_v_self->rect1);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_5 = ((PyObject *)__pyx_v_self->rect2);
    __Pyx_INCREF(__pyx_t_5);
    __pyx_v_self->min_distance = (__pyx_v_self->min_distance + __pyx_f_5pyart_3map_7ckdtree_min_dist_interval_interval_p(((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)__pyx_t_2), ((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)__pyx_t_5), __pyx_v_split_dim, __pyx_v_self->p));
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "pyart/map/ckdtree.pyx":568
 *         if self.p != infinity:
 *             self.min_distance += min_dist_interval_interval_p(self.rect1, self.rect2, split_dim, self.p)
 *             self.max_distance += max_dist_interval_interval_p(self.rect1, self.rect2, split_dim, self.p)             # <<<<<<<<<<<<<<
 *         else:
 *             self.min_distance = min_dist_rect_rect_p_inf(self.rect1, self.rect2)
 */
    __pyx_t_5 = ((PyObject *)__pyx_v_self->rect1);
    __Pyx_INCREF(__pyx_t_5);
    __pyx_t_2 = ((PyObject *)__pyx_v_self->rect2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_self->max_distance = (__pyx_v_self->max_distance + __pyx_f_5pyart_3map_7ckdtree_max_dist_interval_interval_p(((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)__pyx_t_5), ((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)__pyx_t_2), __pyx_v_split_dim, __pyx_v_self->p));
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyart/map/ckdtree.pyx":566
 *             rect.mins[split_dim] = split_val
 * 
 *         if self.p != infinity:             # <<<<<<<<<<<<<<
 *             self.min_distance += min_dist_interval_interval_p(self.rect1, self.rect2, split_dim, self.p)
 *             self.max_distance += max_dist_interval_interval_p(self.rect1, self.rect2, split_dim, self.p)
 */
    goto __pyx_L7;
  }

  /* "pyart/map/ckdtree.pyx":570
 *             self.max_distance += max_dist_interval_interval_p(self.rect1, self.rect2, split_dim, self.p)
 *         else:
 *             self.min_distance = min_dist_rect_rect_p_inf(self.rect1, self.rect2)             # <<<<<<<<<<<<<<
 *             self.max_distance = max_dist_rect_rect_p_inf(self.rect1, self.rect2)
 * 
 */
  /*else*/ {
    __pyx_t_2 = ((PyObject *)__pyx_v_self->rect1);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_5 = ((PyObject *)__pyx_v_self->rect2);
    __Pyx_INCREF(__pyx_t_5);
    __pyx_v_self->min_distance = __pyx_f_5pyart_3map_7ckdtree_min_dist_rect_rect_p_inf(((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)__pyx_t_2), ((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)__pyx_t_5));
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "pyart/map/ckdtree.pyx":571
 *         else:
 *             self.min_distance = min_dist_rect_rect_p_inf(self.rect1, self.rect2)
 *             self.max_distance = max_dist_rect_rect_p_inf(self.rect1, self.rect2)             # <<<<<<<<<<<<<<
 * 
 *         return 0
 */
    __pyx_t_5 = ((PyObject *)__pyx_v_self->rect1);
    __Pyx_INCREF(__pyx_t_5);
    __pyx_t_2 = ((PyObject *)__pyx_v_self->rect2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_self->max_distance = __pyx_f_5pyart_3map_7ckdtree_max_dist_rect_rect_p_inf(((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)__pyx_t_5), ((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)__pyx_t_2));
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_L7:;

  /* "pyart/map/ckdtree.pyx":573
 *             self.max_distance = max_dist_rect_rect_p_inf(self.rect1, self.rect2)
 * 
 *         return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":533
 *         self._free_stack()
 * 
 *     cdef int push(self, np.intp_t which, np.intp_t direction,             # <<<<<<<<<<<<<<
 *                   np.intp_t split_dim,
 *                   np.float64_t split_val) except -1:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyart.map.ckdtree.RectRectDistanceTracker.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_rect);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":576
 * 
 * 
 *     cdef inline int push_less_of(self, np.intp_t which,             # <<<<<<<<<<<<<<
 *                                  innernode *node) except -1:
 *         return self.push(which, LESS, node.split_dim, node.split)
 */

static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_less_of(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_self, __pyx_t_5numpy_intp_t __pyx_v_which, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push_less_of", 0);

  /* "pyart/map/ckdtree.pyx":578
 *     cdef inline int push_less_of(self, np.intp_t which,
 *                                  innernode *node) except -1:
 *         return self.push(which, LESS, node.split_dim, node.split)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_RectRectDistanceTracker *)__pyx_v_self->__pyx_vtab)->push(__pyx_v_self, __pyx_v_which, __pyx_v_5pyart_3map_7ckdtree_LESS, __pyx_v_node->split_dim, __pyx_v_node->split); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 578, __pyx_L1_error)
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":576
 * 
 * 
 *     cdef inline int push_less_of(self, np.intp_t which,             # <<<<<<<<<<<<<<
 *                                  innernode *node) except -1:
 *         return self.push(which, LESS, node.split_dim, node.split)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.RectRectDistanceTracker.push_less_of", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":581
 * 
 * 
 *     cdef inline int push_greater_of(self, np.intp_t which,             # <<<<<<<<<<<<<<
 *                                     innernode *node) except -1:
 *         return self.push(which, GREATER, node.split_dim, node.split)
 */

static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_greater_of(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_self, __pyx_t_5numpy_intp_t __pyx_v_which, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push_greater_of", 0);

  /* "pyart/map/ckdtree.pyx":583
 *     cdef inline int push_greater_of(self, np.intp_t which,
 *                                     innernode *node) except -1:
 *         return self.push(which, GREATER, node.split_dim, node.split)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_RectRectDistanceTracker *)__pyx_v_self->__pyx_vtab)->push(__pyx_v_self, __pyx_v_which, __pyx_v_5pyart_3map_7ckdtree_GREATER, __pyx_v_node->split_dim, __pyx_v_node->split); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 583, __pyx_L1_error)
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":581
 * 
 * 
 *     cdef inline int push_greater_of(self, np.intp_t which,             # <<<<<<<<<<<<<<
 *                                     innernode *node) except -1:
 *         return self.push(which, GREATER, node.split_dim, node.split)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.RectRectDistanceTracker.push_greater_of", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":586
 * 
 * 
 *     cdef inline int pop(self) except -1:             # <<<<<<<<<<<<<<
 *         # Pop from stack
 *         self.stack_size -= 1
 */

static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_self) {
  struct __pyx_t_5pyart_3map_7ckdtree_RR_stack_item *__pyx_v_item;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __pyx_t_5numpy_float64_t __pyx_t_1;
  int __pyx_t_2;
  double __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pop", 0);

  /* "pyart/map/ckdtree.pyx":588
 *     cdef inline int pop(self) except -1:
 *         # Pop from stack
 *         self.stack_size -= 1             # <<<<<<<<<<<<<<
 *         assert self.stack_size >= 0
 * 
 */
  __pyx_v_self->stack_size = (__pyx_v_self->stack_size - 1);

  /* "pyart/map/ckdtree.pyx":589
 *         # Pop from stack
 *         self.stack_size -= 1
 *         assert self.stack_size >= 0             # <<<<<<<<<<<<<<
 * 
 *         cdef RR_stack_item* item = &self.stack[self.stack_size]
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_self->stack_size >= 0) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 589, __pyx_L1_error)
    }
  }
  #endif

  /* "pyart/map/ckdtree.pyx":591
 *         assert self.stack_size >= 0
 * 
 *         cdef RR_stack_item* item = &self.stack[self.stack_size]             # <<<<<<<<<<<<<<
 *         self.min_distance = item.min_distance
 *         self.max_distance = item.max_distance
 */
  __pyx_v_item = (&(__pyx_v_self->stack[__pyx_v_self->stack_size]));

  /* "pyart/map/ckdtree.pyx":592
 * 
 *         cdef RR_stack_item* item = &self.stack[self.stack_size]
 *         self.min_distance = item.min_distance             # <<<<<<<<<<<<<<
 *         self.max_distance = item.max_distance
 * 
 */
  __pyx_t_1 = __pyx_v_item->min_distance;
  __pyx_v_self->min_distance = __pyx_t_1;

  /* "pyart/map/ckdtree.pyx":593
 *         cdef RR_stack_item* item = &self.stack[self.stack_size]
 *         self.min_distance = item.min_distance
 *         self.max_distance = item.max_distance             # <<<<<<<<<<<<<<
 * 
 *         if item.which == 1:
 */
  __pyx_t_1 = __pyx_v_item->max_distance;
  __pyx_v_self->max_distance = __pyx_t_1;

  /* "pyart/map/ckdtree.pyx":595
 *         self.max_distance = item.max_distance
 * 
 *         if item.which == 1:             # <<<<<<<<<<<<<<
 *             self.rect1.mins[item.split_dim] = item.min_along_dim
 *             self.rect1.maxes[item.split_dim] = item.max_along_dim
 */
  __pyx_t_2 = ((__pyx_v_item->which == 1) != 0);
  if (__pyx_t_2) {

    /* "pyart/map/ckdtree.pyx":596
 * 
 *         if item.which == 1:
 *             self.rect1.mins[item.split_dim] = item.min_along_dim             # <<<<<<<<<<<<<<
 *             self.rect1.maxes[item.split_dim] = item.max_along_dim
 *         else:
 */
    __pyx_t_3 = __pyx_v_item->min_along_dim;
    (__pyx_v_self->rect1->mins[__pyx_v_item->split_dim]) = __pyx_t_3;

    /* "pyart/map/ckdtree.pyx":597
 *         if item.which == 1:
 *             self.rect1.mins[item.split_dim] = item.min_along_dim
 *             self.rect1.maxes[item.split_dim] = item.max_along_dim             # <<<<<<<<<<<<<<
 *         else:
 *             self.rect2.mins[item.split_dim] = item.min_along_dim
 */
    __pyx_t_3 = __pyx_v_item->max_along_dim;
    (__pyx_v_self->rect1->maxes[__pyx_v_item->split_dim]) = __pyx_t_3;

    /* "pyart/map/ckdtree.pyx":595
 *         self.max_distance = item.max_distance
 * 
 *         if item.which == 1:             # <<<<<<<<<<<<<<
 *             self.rect1.mins[item.split_dim] = item.min_along_dim
 *             self.rect1.maxes[item.split_dim] = item.max_along_dim
 */
    goto __pyx_L3;
  }

  /* "pyart/map/ckdtree.pyx":599
 *             self.rect1.maxes[item.split_dim] = item.max_along_dim
 *         else:
 *             self.rect2.mins[item.split_dim] = item.min_along_dim             # <<<<<<<<<<<<<<
 *             self.rect2.maxes[item.split_dim] = item.max_along_dim
 * 
 */
  /*else*/ {
    __pyx_t_3 = __pyx_v_item->min_along_dim;
    (__pyx_v_self->rect2->mins[__pyx_v_item->split_dim]) = __pyx_t_3;

    /* "pyart/map/ckdtree.pyx":600
 *         else:
 *             self.rect2.mins[item.split_dim] = item.min_along_dim
 *             self.rect2.maxes[item.split_dim] = item.max_along_dim             # <<<<<<<<<<<<<<
 * 
 *         return 0
 */
    __pyx_t_3 = __pyx_v_item->max_along_dim;
    (__pyx_v_self->rect2->maxes[__pyx_v_item->split_dim]) = __pyx_t_3;
  }
  __pyx_L3:;

  /* "pyart/map/ckdtree.pyx":602
 *             self.rect2.maxes[item.split_dim] = item.max_along_dim
 * 
 *         return 0             # <<<<<<<<<<<<<<
 * 
 * # Point-to-rectangle distance tracker
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":586
 * 
 * 
 *     cdef inline int pop(self) except -1:             # <<<<<<<<<<<<<<
 *         # Pop from stack
 *         self.stack_size -= 1
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.RectRectDistanceTracker.pop", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.stack cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_23RectRectDistanceTracker_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_23RectRectDistanceTracker_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_23RectRectDistanceTracker_4__reduce_cython__(((struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_3map_7ckdtree_23RectRectDistanceTracker_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.stack cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.stack cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.stack cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.map.ckdtree.RectRectDistanceTracker.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.stack cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.stack cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_23RectRectDistanceTracker_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_23RectRectDistanceTracker_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_23RectRectDistanceTracker_6__setstate_cython__(((struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_3map_7ckdtree_23RectRectDistanceTracker_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.stack cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.stack cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.stack cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.stack cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.map.ckdtree.RectRectDistanceTracker.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":646
 * 
 *     # Stack handling
 *     cdef int _init_stack(self) except -1:             # <<<<<<<<<<<<<<
 *         cdef void *tmp
 *         self.stack_max_size = 10
 */

static int __pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker__init_stack(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_v_self) {
  void *__pyx_v_tmp;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_init_stack", 0);

  /* "pyart/map/ckdtree.pyx":648
 *     cdef int _init_stack(self) except -1:
 *         cdef void *tmp
 *         self.stack_max_size = 10             # <<<<<<<<<<<<<<
 *         tmp = stdlib.malloc(sizeof(RP_stack_item) *
 *                             self.stack_max_size)
 */
  __pyx_v_self->stack_max_size = 10;

  /* "pyart/map/ckdtree.pyx":649
 *         cdef void *tmp
 *         self.stack_max_size = 10
 *         tmp = stdlib.malloc(sizeof(RP_stack_item) *             # <<<<<<<<<<<<<<
 *                             self.stack_max_size)
 *         if tmp == NULL:
 */
  __pyx_v_tmp = malloc(((sizeof(struct __pyx_t_5pyart_3map_7ckdtree_RP_stack_item)) * __pyx_v_self->stack_max_size));

  /* "pyart/map/ckdtree.pyx":651
 *         tmp = stdlib.malloc(sizeof(RP_stack_item) *
 *                             self.stack_max_size)
 *         if tmp == NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError
 *         self.stack = <RP_stack_item*> tmp
 */
  __pyx_t_1 = ((__pyx_v_tmp == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "pyart/map/ckdtree.pyx":652
 *                             self.stack_max_size)
 *         if tmp == NULL:
 *             raise MemoryError             # <<<<<<<<<<<<<<
 *         self.stack = <RP_stack_item*> tmp
 *         self.stack_size = 0
 */
    PyErr_NoMemory(); __PYX_ERR(0, 652, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":651
 *         tmp = stdlib.malloc(sizeof(RP_stack_item) *
 *                             self.stack_max_size)
 *         if tmp == NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError
 *         self.stack = <RP_stack_item*> tmp
 */
  }

  /* "pyart/map/ckdtree.pyx":653
 *         if tmp == NULL:
 *             raise MemoryError
 *         self.stack = <RP_stack_item*> tmp             # <<<<<<<<<<<<<<
 *         self.stack_size = 0
 *         return 0
 */
  __pyx_v_self->stack = ((struct __pyx_t_5pyart_3map_7ckdtree_RP_stack_item *)__pyx_v_tmp);

  /* "pyart/map/ckdtree.pyx":654
 *             raise MemoryError
 *         self.stack = <RP_stack_item*> tmp
 *         self.stack_size = 0             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
  __pyx_v_self->stack_size = 0;

  /* "pyart/map/ckdtree.pyx":655
 *         self.stack = <RP_stack_item*> tmp
 *         self.stack_size = 0
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     cdef int _resize_stack(self, np.intp_t new_max_size) except -1:
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":646
 * 
 *     # Stack handling
 *     cdef int _init_stack(self) except -1:             # <<<<<<<<<<<<<<
 *         cdef void *tmp
 *         self.stack_max_size = 10
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.PointRectDistanceTracker._init_stack", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":657
 *         return 0
 * 
 *     cdef int _resize_stack(self, np.intp_t new_max_size) except -1:             # <<<<<<<<<<<<<<
 *         cdef void *tmp
 *         self.stack_max_size = new_max_size
 */

static int __pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker__resize_stack(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_v_self, __pyx_t_5numpy_intp_t __pyx_v_new_max_size) {
  void *__pyx_v_tmp;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_resize_stack", 0);

  /* "pyart/map/ckdtree.pyx":659
 *     cdef int _resize_stack(self, np.intp_t new_max_size) except -1:
 *         cdef void *tmp
 *         self.stack_max_size = new_max_size             # <<<<<<<<<<<<<<
 *         tmp = stdlib.realloc(<RP_stack_item*> self.stack,
 *                               new_max_size * sizeof(RP_stack_item))
 */
  __pyx_v_self->stack_max_size = __pyx_v_new_max_size;

  /* "pyart/map/ckdtree.pyx":660
 *         cdef void *tmp
 *         self.stack_max_size = new_max_size
 *         tmp = stdlib.realloc(<RP_stack_item*> self.stack,             # <<<<<<<<<<<<<<
 *                               new_max_size * sizeof(RP_stack_item))
 *         if tmp == NULL:
 */
  __pyx_v_tmp = realloc(((struct __pyx_t_5pyart_3map_7ckdtree_RP_stack_item *)__pyx_v_self->stack), (__pyx_v_new_max_size * (sizeof(struct __pyx_t_5pyart_3map_7ckdtree_RP_stack_item))));

  /* "pyart/map/ckdtree.pyx":662
 *         tmp = stdlib.realloc(<RP_stack_item*> self.stack,
 *                               new_max_size * sizeof(RP_stack_item))
 *         if tmp == NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError
 *         self.stack = <RP_stack_item*> tmp
 */
  __pyx_t_1 = ((__pyx_v_tmp == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "pyart/map/ckdtree.pyx":663
 *                               new_max_size * sizeof(RP_stack_item))
 *         if tmp == NULL:
 *             raise MemoryError             # <<<<<<<<<<<<<<
 *         self.stack = <RP_stack_item*> tmp
 *         return 0
 */
    PyErr_NoMemory(); __PYX_ERR(0, 663, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":662
 *         tmp = stdlib.realloc(<RP_stack_item*> self.stack,
 *                               new_max_size * sizeof(RP_stack_item))
 *         if tmp == NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError
 *         self.stack = <RP_stack_item*> tmp
 */
  }

  /* "pyart/map/ckdtree.pyx":664
 *         if tmp == NULL:
 *             raise MemoryError
 *         self.stack = <RP_stack_item*> tmp             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
  __pyx_v_self->stack = ((struct __pyx_t_5pyart_3map_7ckdtree_RP_stack_item *)__pyx_v_tmp);

  /* "pyart/map/ckdtree.pyx":665
 *             raise MemoryError
 *         self.stack = <RP_stack_item*> tmp
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     cdef int _free_stack(self) except -1:
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":657
 *         return 0
 * 
 *     cdef int _resize_stack(self, np.intp_t new_max_size) except -1:             # <<<<<<<<<<<<<<
 *         cdef void *tmp
 *         self.stack_max_size = new_max_size
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.PointRectDistanceTracker._resize_stack", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":667
 *         return 0
 * 
 *     cdef int _free_stack(self) except -1:             # <<<<<<<<<<<<<<
 *         if self.stack != <RP_stack_item*> NULL:
 *             stdlib.free(self.stack)
 */

static int __pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker__free_stack(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("_free_stack", 0);

  /* "pyart/map/ckdtree.pyx":668
 * 
 *     cdef int _free_stack(self) except -1:
 *         if self.stack != <RP_stack_item*> NULL:             # <<<<<<<<<<<<<<
 *             stdlib.free(self.stack)
 *         return 0
 */
  __pyx_t_1 = ((__pyx_v_self->stack != ((struct __pyx_t_5pyart_3map_7ckdtree_RP_stack_item *)NULL)) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":669
 *     cdef int _free_stack(self) except -1:
 *         if self.stack != <RP_stack_item*> NULL:
 *             stdlib.free(self.stack)             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
    free(__pyx_v_self->stack);

    /* "pyart/map/ckdtree.pyx":668
 * 
 *     cdef int _free_stack(self) except -1:
 *         if self.stack != <RP_stack_item*> NULL:             # <<<<<<<<<<<<<<
 *             stdlib.free(self.stack)
 *         return 0
 */
  }

  /* "pyart/map/ckdtree.pyx":670
 *         if self.stack != <RP_stack_item*> NULL:
 *             stdlib.free(self.stack)
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     cdef init(self, np.float64_t *pt, Rectangle rect,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":667
 *         return 0
 * 
 *     cdef int _free_stack(self) except -1:             # <<<<<<<<<<<<<<
 *         if self.stack != <RP_stack_item*> NULL:
 *             stdlib.free(self.stack)
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":672
 *         return 0
 * 
 *     cdef init(self, np.float64_t *pt, Rectangle rect,             # <<<<<<<<<<<<<<
 *               np.float64_t p, np.float64_t eps, np.float64_t upper_bound):
 * 
 */

static PyObject *__pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker_init(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_v_self, __pyx_t_5numpy_float64_t *__pyx_v_pt, struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *__pyx_v_rect, __pyx_t_5numpy_float64_t __pyx_v_p, __pyx_t_5numpy_float64_t __pyx_v_eps, __pyx_t_5numpy_float64_t __pyx_v_upper_bound) {
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __pyx_t_5numpy_float64_t __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  Py_ssize_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  __pyx_t_5numpy_intp_t __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("init", 0);

  /* "pyart/map/ckdtree.pyx":675
 *               np.float64_t p, np.float64_t eps, np.float64_t upper_bound):
 * 
 *         self.pt = pt             # <<<<<<<<<<<<<<
 *         self.rect = rect
 *         self.p = p
 */
  __pyx_v_self->pt = __pyx_v_pt;

  /* "pyart/map/ckdtree.pyx":676
 * 
 *         self.pt = pt
 *         self.rect = rect             # <<<<<<<<<<<<<<
 *         self.p = p
 * 
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_rect));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_rect));
  __Pyx_GOTREF(__pyx_v_self->rect);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->rect));
  __pyx_v_self->rect = __pyx_v_rect;

  /* "pyart/map/ckdtree.pyx":677
 *         self.pt = pt
 *         self.rect = rect
 *         self.p = p             # <<<<<<<<<<<<<<
 * 
 *         # internally we represent all distances as distance ** p
 */
  __pyx_v_self->p = __pyx_v_p;

  /* "pyart/map/ckdtree.pyx":680
 * 
 *         # internally we represent all distances as distance ** p
 *         if p != infinity and upper_bound != infinity:             # <<<<<<<<<<<<<<
 *             self.upper_bound = upper_bound ** p
 *         else:
 */
  __pyx_t_2 = ((__pyx_v_p != __pyx_v_5pyart_3map_7ckdtree_infinity) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_upper_bound != __pyx_v_5pyart_3map_7ckdtree_infinity) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":681
 *         # internally we represent all distances as distance ** p
 *         if p != infinity and upper_bound != infinity:
 *             self.upper_bound = upper_bound ** p             # <<<<<<<<<<<<<<
 *         else:
 *             self.upper_bound = upper_bound
 */
    __pyx_v_self->upper_bound = pow(__pyx_v_upper_bound, __pyx_v_p);

    /* "pyart/map/ckdtree.pyx":680
 * 
 *         # internally we represent all distances as distance ** p
 *         if p != infinity and upper_bound != infinity:             # <<<<<<<<<<<<<<
 *             self.upper_bound = upper_bound ** p
 *         else:
 */
    goto __pyx_L3;
  }

  /* "pyart/map/ckdtree.pyx":683
 *             self.upper_bound = upper_bound ** p
 *         else:
 *             self.upper_bound = upper_bound             # <<<<<<<<<<<<<<
 * 
 *         # fiddle approximation factor
 */
  /*else*/ {
    __pyx_v_self->upper_bound = __pyx_v_upper_bound;
  }
  __pyx_L3:;

  /* "pyart/map/ckdtree.pyx":686
 * 
 *         # fiddle approximation factor
 *         if eps == 0:             # <<<<<<<<<<<<<<
 *             self.epsfac = 1
 *         elif p == infinity:
 */
  __pyx_t_1 = ((__pyx_v_eps == 0.0) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":687
 *         # fiddle approximation factor
 *         if eps == 0:
 *             self.epsfac = 1             # <<<<<<<<<<<<<<
 *         elif p == infinity:
 *             self.epsfac = 1 / (1 + eps)
 */
    __pyx_v_self->epsfac = 1.0;

    /* "pyart/map/ckdtree.pyx":686
 * 
 *         # fiddle approximation factor
 *         if eps == 0:             # <<<<<<<<<<<<<<
 *             self.epsfac = 1
 *         elif p == infinity:
 */
    goto __pyx_L6;
  }

  /* "pyart/map/ckdtree.pyx":688
 *         if eps == 0:
 *             self.epsfac = 1
 *         elif p == infinity:             # <<<<<<<<<<<<<<
 *             self.epsfac = 1 / (1 + eps)
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_p == __pyx_v_5pyart_3map_7ckdtree_infinity) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":689
 *             self.epsfac = 1
 *         elif p == infinity:
 *             self.epsfac = 1 / (1 + eps)             # <<<<<<<<<<<<<<
 *         else:
 *             self.epsfac = 1 / (1 + eps) ** p
 */
    __pyx_t_3 = (1.0 + __pyx_v_eps);
    if (unlikely(__pyx_t_3 == 0)) {
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      __PYX_ERR(0, 689, __pyx_L1_error)
    }
    __pyx_v_self->epsfac = (1.0 / __pyx_t_3);

    /* "pyart/map/ckdtree.pyx":688
 *         if eps == 0:
 *             self.epsfac = 1
 *         elif p == infinity:             # <<<<<<<<<<<<<<
 *             self.epsfac = 1 / (1 + eps)
 *         else:
 */
    goto __pyx_L6;
  }

  /* "pyart/map/ckdtree.pyx":691
 *             self.epsfac = 1 / (1 + eps)
 *         else:
 *             self.epsfac = 1 / (1 + eps) ** p             # <<<<<<<<<<<<<<
 * 
 *         self._init_stack()
 */
  /*else*/ {
    __pyx_t_3 = pow((1.0 + __pyx_v_eps), __pyx_v_p);
    if (unlikely(__pyx_t_3 == 0)) {
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      __PYX_ERR(0, 691, __pyx_L1_error)
    }
    __pyx_v_self->epsfac = (1.0 / __pyx_t_3);
  }
  __pyx_L6:;

  /* "pyart/map/ckdtree.pyx":693
 *             self.epsfac = 1 / (1 + eps) ** p
 * 
 *         self._init_stack()             # <<<<<<<<<<<<<<
 * 
 *         # Compute initial min and max distances
 */
  __pyx_t_4 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_PointRectDistanceTracker *)__pyx_v_self->__pyx_vtab)->_init_stack(__pyx_v_self); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 693, __pyx_L1_error)

  /* "pyart/map/ckdtree.pyx":696
 * 
 *         # Compute initial min and max distances
 *         if self.p == infinity:             # <<<<<<<<<<<<<<
 *             self.min_distance = min_dist_point_rect_p_inf(pt, rect)
 *             self.max_distance = max_dist_point_rect_p_inf(pt, rect)
 */
  __pyx_t_1 = ((__pyx_v_self->p == __pyx_v_5pyart_3map_7ckdtree_infinity) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":697
 *         # Compute initial min and max distances
 *         if self.p == infinity:
 *             self.min_distance = min_dist_point_rect_p_inf(pt, rect)             # <<<<<<<<<<<<<<
 *             self.max_distance = max_dist_point_rect_p_inf(pt, rect)
 *         else:
 */
    __pyx_v_self->min_distance = __pyx_f_5pyart_3map_7ckdtree_min_dist_point_rect_p_inf(__pyx_v_pt, __pyx_v_rect);

    /* "pyart/map/ckdtree.pyx":698
 *         if self.p == infinity:
 *             self.min_distance = min_dist_point_rect_p_inf(pt, rect)
 *             self.max_distance = max_dist_point_rect_p_inf(pt, rect)             # <<<<<<<<<<<<<<
 *         else:
 *             self.min_distance = 0.
 */
    __pyx_v_self->max_distance = __pyx_f_5pyart_3map_7ckdtree_max_dist_point_rect_p_inf(__pyx_v_pt, __pyx_v_rect);

    /* "pyart/map/ckdtree.pyx":696
 * 
 *         # Compute initial min and max distances
 *         if self.p == infinity:             # <<<<<<<<<<<<<<
 *             self.min_distance = min_dist_point_rect_p_inf(pt, rect)
 *             self.max_distance = max_dist_point_rect_p_inf(pt, rect)
 */
    goto __pyx_L7;
  }

  /* "pyart/map/ckdtree.pyx":700
 *             self.max_distance = max_dist_point_rect_p_inf(pt, rect)
 *         else:
 *             self.min_distance = 0.             # <<<<<<<<<<<<<<
 *             self.max_distance = 0.
 *             for i in range(rect.m):
 */
  /*else*/ {
    __pyx_v_self->min_distance = 0.;

    /* "pyart/map/ckdtree.pyx":701
 *         else:
 *             self.min_distance = 0.
 *             self.max_distance = 0.             # <<<<<<<<<<<<<<
 *             for i in range(rect.m):
 *                 self.min_distance += min_dist_point_interval_p(pt, rect, i, p)
 */
    __pyx_v_self->max_distance = 0.;

    /* "pyart/map/ckdtree.pyx":702
 *             self.min_distance = 0.
 *             self.max_distance = 0.
 *             for i in range(rect.m):             # <<<<<<<<<<<<<<
 *                 self.min_distance += min_dist_point_interval_p(pt, rect, i, p)
 *                 self.max_distance += max_dist_point_interval_p(pt, rect, i, p)
 */
    __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_rect->m); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 702, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 702, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (likely(PyList_CheckExact(__pyx_t_6)) || PyTuple_CheckExact(__pyx_t_6)) {
      __pyx_t_5 = __pyx_t_6; __Pyx_INCREF(__pyx_t_5); __pyx_t_7 = 0;
      __pyx_t_8 = NULL;
    } else {
      __pyx_t_7 = -1; __pyx_t_5 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 702, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_8 = Py_TYPE(__pyx_t_5)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 702, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    for (;;) {
      if (likely(!__pyx_t_8)) {
        if (likely(PyList_CheckExact(__pyx_t_5))) {
          if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_5)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_6 = PyList_GET_ITEM(__pyx_t_5, __pyx_t_7); __Pyx_INCREF(__pyx_t_6); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 702, __pyx_L1_error)
          #else
          __pyx_t_6 = PySequence_ITEM(__pyx_t_5, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 702, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          #endif
        } else {
          if (__pyx_t_7 >= PyTuple_GET_SIZE(__pyx_t_5)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_6 = PyTuple_GET_ITEM(__pyx_t_5, __pyx_t_7); __Pyx_INCREF(__pyx_t_6); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 702, __pyx_L1_error)
          #else
          __pyx_t_6 = PySequence_ITEM(__pyx_t_5, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 702, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          #endif
        }
      } else {
        __pyx_t_6 = __pyx_t_8(__pyx_t_5);
        if (unlikely(!__pyx_t_6)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 702, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_6);
      }
      __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "pyart/map/ckdtree.pyx":703
 *             self.max_distance = 0.
 *             for i in range(rect.m):
 *                 self.min_distance += min_dist_point_interval_p(pt, rect, i, p)             # <<<<<<<<<<<<<<
 *                 self.max_distance += max_dist_point_interval_p(pt, rect, i, p)
 * 
 */
      __pyx_t_9 = __Pyx_PyInt_As_Py_intptr_t(__pyx_v_i); if (unlikely((__pyx_t_9 == ((npy_intp)-1)) && PyErr_Occurred())) __PYX_ERR(0, 703, __pyx_L1_error)
      __pyx_v_self->min_distance = (__pyx_v_self->min_distance + __pyx_f_5pyart_3map_7ckdtree_min_dist_point_interval_p(__pyx_v_pt, __pyx_v_rect, __pyx_t_9, __pyx_v_p));

      /* "pyart/map/ckdtree.pyx":704
 *             for i in range(rect.m):
 *                 self.min_distance += min_dist_point_interval_p(pt, rect, i, p)
 *                 self.max_distance += max_dist_point_interval_p(pt, rect, i, p)             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
      __pyx_t_9 = __Pyx_PyInt_As_Py_intptr_t(__pyx_v_i); if (unlikely((__pyx_t_9 == ((npy_intp)-1)) && PyErr_Occurred())) __PYX_ERR(0, 704, __pyx_L1_error)
      __pyx_v_self->max_distance = (__pyx_v_self->max_distance + __pyx_f_5pyart_3map_7ckdtree_max_dist_point_interval_p(__pyx_v_pt, __pyx_v_rect, __pyx_t_9, __pyx_v_p));

      /* "pyart/map/ckdtree.pyx":702
 *             self.min_distance = 0.
 *             self.max_distance = 0.
 *             for i in range(rect.m):             # <<<<<<<<<<<<<<
 *                 self.min_distance += min_dist_point_interval_p(pt, rect, i, p)
 *                 self.max_distance += max_dist_point_interval_p(pt, rect, i, p)
 */
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __pyx_L7:;

  /* "pyart/map/ckdtree.pyx":672
 *         return 0
 * 
 *     cdef init(self, np.float64_t *pt, Rectangle rect,             # <<<<<<<<<<<<<<
 *               np.float64_t p, np.float64_t eps, np.float64_t upper_bound):
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pyart.map.ckdtree.PointRectDistanceTracker.init", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":706
 *                 self.max_distance += max_dist_point_interval_p(pt, rect, i, p)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         self._free_stack()
 * 
 */

/* Python wrapper */
static void __pyx_pw_5pyart_3map_7ckdtree_24PointRectDistanceTracker_1__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5pyart_3map_7ckdtree_24PointRectDistanceTracker_1__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5pyart_3map_7ckdtree_24PointRectDistanceTracker___dealloc__(((struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5pyart_3map_7ckdtree_24PointRectDistanceTracker___dealloc__(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "pyart/map/ckdtree.pyx":707
 * 
 *     def __dealloc__(self):
 *         self._free_stack()             # <<<<<<<<<<<<<<
 * 
 *     cdef int push(self, np.intp_t direction,
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_PointRectDistanceTracker *)__pyx_v_self->__pyx_vtab)->_free_stack(__pyx_v_self); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 707, __pyx_L1_error)

  /* "pyart/map/ckdtree.pyx":706
 *                 self.max_distance += max_dist_point_interval_p(pt, rect, i, p)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         self._free_stack()
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pyart.map.ckdtree.PointRectDistanceTracker.__dealloc__", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pyart/map/ckdtree.pyx":709
 *         self._free_stack()
 * 
 *     cdef int push(self, np.intp_t direction,             # <<<<<<<<<<<<<<
 *                   np.intp_t split_dim,
 *                   np.float64_t split_val) except -1:
 */

static int __pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker_push(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_v_self, __pyx_t_5numpy_intp_t __pyx_v_direction, __pyx_t_5numpy_intp_t __pyx_v_split_dim, __pyx_t_5numpy_float64_t __pyx_v_split_val) {
  struct __pyx_t_5pyart_3map_7ckdtree_RP_stack_item *__pyx_v_item;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __pyx_t_5numpy_float64_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);

  /* "pyart/map/ckdtree.pyx":714
 * 
 *         # Push onto stack
 *         if self.stack_size == self.stack_max_size:             # <<<<<<<<<<<<<<
 *             self._resize_stack(self.stack_max_size * 2)
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->stack_size == __pyx_v_self->stack_max_size) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":715
 *         # Push onto stack
 *         if self.stack_size == self.stack_max_size:
 *             self._resize_stack(self.stack_max_size * 2)             # <<<<<<<<<<<<<<
 * 
 *         cdef RP_stack_item *item = &self.stack[self.stack_size]
 */
    __pyx_t_2 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_PointRectDistanceTracker *)__pyx_v_self->__pyx_vtab)->_resize_stack(__pyx_v_self, (__pyx_v_self->stack_max_size * 2)); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 715, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":714
 * 
 *         # Push onto stack
 *         if self.stack_size == self.stack_max_size:             # <<<<<<<<<<<<<<
 *             self._resize_stack(self.stack_max_size * 2)
 * 
 */
  }

  /* "pyart/map/ckdtree.pyx":717
 *             self._resize_stack(self.stack_max_size * 2)
 * 
 *         cdef RP_stack_item *item = &self.stack[self.stack_size]             # <<<<<<<<<<<<<<
 *         self.stack_size += 1
 * 
 */
  __pyx_v_item = (&(__pyx_v_self->stack[__pyx_v_self->stack_size]));

  /* "pyart/map/ckdtree.pyx":718
 * 
 *         cdef RP_stack_item *item = &self.stack[self.stack_size]
 *         self.stack_size += 1             # <<<<<<<<<<<<<<
 * 
 *         item.split_dim = split_dim
 */
  __pyx_v_self->stack_size = (__pyx_v_self->stack_size + 1);

  /* "pyart/map/ckdtree.pyx":720
 *         self.stack_size += 1
 * 
 *         item.split_dim = split_dim             # <<<<<<<<<<<<<<
 *         item.min_distance = self.min_distance
 *         item.max_distance = self.max_distance
 */
  __pyx_v_item->split_dim = __pyx_v_split_dim;

  /* "pyart/map/ckdtree.pyx":721
 * 
 *         item.split_dim = split_dim
 *         item.min_distance = self.min_distance             # <<<<<<<<<<<<<<
 *         item.max_distance = self.max_distance
 *         item.min_along_dim = self.rect.mins[split_dim]
 */
  __pyx_t_3 = __pyx_v_self->min_distance;
  __pyx_v_item->min_distance = __pyx_t_3;

  /* "pyart/map/ckdtree.pyx":722
 *         item.split_dim = split_dim
 *         item.min_distance = self.min_distance
 *         item.max_distance = self.max_distance             # <<<<<<<<<<<<<<
 *         item.min_along_dim = self.rect.mins[split_dim]
 *         item.max_along_dim = self.rect.maxes[split_dim]
 */
  __pyx_t_3 = __pyx_v_self->max_distance;
  __pyx_v_item->max_distance = __pyx_t_3;

  /* "pyart/map/ckdtree.pyx":723
 *         item.min_distance = self.min_distance
 *         item.max_distance = self.max_distance
 *         item.min_along_dim = self.rect.mins[split_dim]             # <<<<<<<<<<<<<<
 *         item.max_along_dim = self.rect.maxes[split_dim]
 * 
 */
  __pyx_v_item->min_along_dim = (__pyx_v_self->rect->mins[__pyx_v_split_dim]);

  /* "pyart/map/ckdtree.pyx":724
 *         item.max_distance = self.max_distance
 *         item.min_along_dim = self.rect.mins[split_dim]
 *         item.max_along_dim = self.rect.maxes[split_dim]             # <<<<<<<<<<<<<<
 * 
 *         if self.p != infinity:
 */
  __pyx_v_item->max_along_dim = (__pyx_v_self->rect->maxes[__pyx_v_split_dim]);

  /* "pyart/map/ckdtree.pyx":726
 *         item.max_along_dim = self.rect.maxes[split_dim]
 * 
 *         if self.p != infinity:             # <<<<<<<<<<<<<<
 *             self.min_distance -= min_dist_point_interval_p(self.pt, self.rect, split_dim, self.p)
 *             self.max_distance -= max_dist_point_interval_p(self.pt, self.rect, split_dim, self.p)
 */
  __pyx_t_1 = ((__pyx_v_self->p != __pyx_v_5pyart_3map_7ckdtree_infinity) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":727
 * 
 *         if self.p != infinity:
 *             self.min_distance -= min_dist_point_interval_p(self.pt, self.rect, split_dim, self.p)             # <<<<<<<<<<<<<<
 *             self.max_distance -= max_dist_point_interval_p(self.pt, self.rect, split_dim, self.p)
 * 
 */
    __pyx_t_4 = ((PyObject *)__pyx_v_self->rect);
    __Pyx_INCREF(__pyx_t_4);
    __pyx_v_self->min_distance = (__pyx_v_self->min_distance - __pyx_f_5pyart_3map_7ckdtree_min_dist_point_interval_p(__pyx_v_self->pt, ((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)__pyx_t_4), __pyx_v_split_dim, __pyx_v_self->p));
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "pyart/map/ckdtree.pyx":728
 *         if self.p != infinity:
 *             self.min_distance -= min_dist_point_interval_p(self.pt, self.rect, split_dim, self.p)
 *             self.max_distance -= max_dist_point_interval_p(self.pt, self.rect, split_dim, self.p)             # <<<<<<<<<<<<<<
 * 
 *         if direction == LESS:
 */
    __pyx_t_4 = ((PyObject *)__pyx_v_self->rect);
    __Pyx_INCREF(__pyx_t_4);
    __pyx_v_self->max_distance = (__pyx_v_self->max_distance - __pyx_f_5pyart_3map_7ckdtree_max_dist_point_interval_p(__pyx_v_self->pt, ((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)__pyx_t_4), __pyx_v_split_dim, __pyx_v_self->p));
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "pyart/map/ckdtree.pyx":726
 *         item.max_along_dim = self.rect.maxes[split_dim]
 * 
 *         if self.p != infinity:             # <<<<<<<<<<<<<<
 *             self.min_distance -= min_dist_point_interval_p(self.pt, self.rect, split_dim, self.p)
 *             self.max_distance -= max_dist_point_interval_p(self.pt, self.rect, split_dim, self.p)
 */
  }

  /* "pyart/map/ckdtree.pyx":730
 *             self.max_distance -= max_dist_point_interval_p(self.pt, self.rect, split_dim, self.p)
 * 
 *         if direction == LESS:             # <<<<<<<<<<<<<<
 *             self.rect.maxes[split_dim] = split_val
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_direction == __pyx_v_5pyart_3map_7ckdtree_LESS) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":731
 * 
 *         if direction == LESS:
 *             self.rect.maxes[split_dim] = split_val             # <<<<<<<<<<<<<<
 *         else:
 *             self.rect.mins[split_dim] = split_val
 */
    (__pyx_v_self->rect->maxes[__pyx_v_split_dim]) = __pyx_v_split_val;

    /* "pyart/map/ckdtree.pyx":730
 *             self.max_distance -= max_dist_point_interval_p(self.pt, self.rect, split_dim, self.p)
 * 
 *         if direction == LESS:             # <<<<<<<<<<<<<<
 *             self.rect.maxes[split_dim] = split_val
 *         else:
 */
    goto __pyx_L5;
  }

  /* "pyart/map/ckdtree.pyx":733
 *             self.rect.maxes[split_dim] = split_val
 *         else:
 *             self.rect.mins[split_dim] = split_val             # <<<<<<<<<<<<<<
 * 
 *         if self.p != infinity:
 */
  /*else*/ {
    (__pyx_v_self->rect->mins[__pyx_v_split_dim]) = __pyx_v_split_val;
  }
  __pyx_L5:;

  /* "pyart/map/ckdtree.pyx":735
 *             self.rect.mins[split_dim] = split_val
 * 
 *         if self.p != infinity:             # <<<<<<<<<<<<<<
 *             self.min_distance += min_dist_point_interval_p(self.pt, self.rect, split_dim, self.p)
 *             self.max_distance += max_dist_point_interval_p(self.pt, self.rect, split_dim, self.p)
 */
  __pyx_t_1 = ((__pyx_v_self->p != __pyx_v_5pyart_3map_7ckdtree_infinity) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":736
 * 
 *         if self.p != infinity:
 *             self.min_distance += min_dist_point_interval_p(self.pt, self.rect, split_dim, self.p)             # <<<<<<<<<<<<<<
 *             self.max_distance += max_dist_point_interval_p(self.pt, self.rect, split_dim, self.p)
 *         else:
 */
    __pyx_t_4 = ((PyObject *)__pyx_v_self->rect);
    __Pyx_INCREF(__pyx_t_4);
    __pyx_v_self->min_distance = (__pyx_v_self->min_distance + __pyx_f_5pyart_3map_7ckdtree_min_dist_point_interval_p(__pyx_v_self->pt, ((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)__pyx_t_4), __pyx_v_split_dim, __pyx_v_self->p));
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "pyart/map/ckdtree.pyx":737
 *         if self.p != infinity:
 *             self.min_distance += min_dist_point_interval_p(self.pt, self.rect, split_dim, self.p)
 *             self.max_distance += max_dist_point_interval_p(self.pt, self.rect, split_dim, self.p)             # <<<<<<<<<<<<<<
 *         else:
 *             self.min_distance = min_dist_point_rect_p_inf(self.pt, self.rect)
 */
    __pyx_t_4 = ((PyObject *)__pyx_v_self->rect);
    __Pyx_INCREF(__pyx_t_4);
    __pyx_v_self->max_distance = (__pyx_v_self->max_distance + __pyx_f_5pyart_3map_7ckdtree_max_dist_point_interval_p(__pyx_v_self->pt, ((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)__pyx_t_4), __pyx_v_split_dim, __pyx_v_self->p));
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "pyart/map/ckdtree.pyx":735
 *             self.rect.mins[split_dim] = split_val
 * 
 *         if self.p != infinity:             # <<<<<<<<<<<<<<
 *             self.min_distance += min_dist_point_interval_p(self.pt, self.rect, split_dim, self.p)
 *             self.max_distance += max_dist_point_interval_p(self.pt, self.rect, split_dim, self.p)
 */
    goto __pyx_L6;
  }

  /* "pyart/map/ckdtree.pyx":739
 *             self.max_distance += max_dist_point_interval_p(self.pt, self.rect, split_dim, self.p)
 *         else:
 *             self.min_distance = min_dist_point_rect_p_inf(self.pt, self.rect)             # <<<<<<<<<<<<<<
 *             self.max_distance = max_dist_point_rect_p_inf(self.pt, self.rect)
 * 
 */
  /*else*/ {
    __pyx_t_4 = ((PyObject *)__pyx_v_self->rect);
    __Pyx_INCREF(__pyx_t_4);
    __pyx_v_self->min_distance = __pyx_f_5pyart_3map_7ckdtree_min_dist_point_rect_p_inf(__pyx_v_self->pt, ((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)__pyx_t_4));
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "pyart/map/ckdtree.pyx":740
 *         else:
 *             self.min_distance = min_dist_point_rect_p_inf(self.pt, self.rect)
 *             self.max_distance = max_dist_point_rect_p_inf(self.pt, self.rect)             # <<<<<<<<<<<<<<
 * 
 *         return 0
 */
    __pyx_t_4 = ((PyObject *)__pyx_v_self->rect);
    __Pyx_INCREF(__pyx_t_4);
    __pyx_v_self->max_distance = __pyx_f_5pyart_3map_7ckdtree_max_dist_point_rect_p_inf(__pyx_v_self->pt, ((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)__pyx_t_4));
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __pyx_L6:;

  /* "pyart/map/ckdtree.pyx":742
 *             self.max_distance = max_dist_point_rect_p_inf(self.pt, self.rect)
 * 
 *         return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":709
 *         self._free_stack()
 * 
 *     cdef int push(self, np.intp_t direction,             # <<<<<<<<<<<<<<
 *                   np.intp_t split_dim,
 *                   np.float64_t split_val) except -1:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyart.map.ckdtree.PointRectDistanceTracker.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":745
 * 
 * 
 *     cdef inline int push_less_of(self, innernode* node) except -1:             # <<<<<<<<<<<<<<
 *         return self.push(LESS, node.split_dim, node.split)
 * 
 */

static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker_push_less_of(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_v_self, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push_less_of", 0);

  /* "pyart/map/ckdtree.pyx":746
 * 
 *     cdef inline int push_less_of(self, innernode* node) except -1:
 *         return self.push(LESS, node.split_dim, node.split)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_PointRectDistanceTracker *)__pyx_v_self->__pyx_vtab)->push(__pyx_v_self, __pyx_v_5pyart_3map_7ckdtree_LESS, __pyx_v_node->split_dim, __pyx_v_node->split); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 746, __pyx_L1_error)
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":745
 * 
 * 
 *     cdef inline int push_less_of(self, innernode* node) except -1:             # <<<<<<<<<<<<<<
 *         return self.push(LESS, node.split_dim, node.split)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.PointRectDistanceTracker.push_less_of", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":749
 * 
 * 
 *     cdef inline int push_greater_of(self, innernode* node) except -1:             # <<<<<<<<<<<<<<
 *         return self.push(GREATER, node.split_dim, node.split)
 * 
 */

static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker_push_greater_of(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_v_self, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push_greater_of", 0);

  /* "pyart/map/ckdtree.pyx":750
 * 
 *     cdef inline int push_greater_of(self, innernode* node) except -1:
 *         return self.push(GREATER, node.split_dim, node.split)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_PointRectDistanceTracker *)__pyx_v_self->__pyx_vtab)->push(__pyx_v_self, __pyx_v_5pyart_3map_7ckdtree_GREATER, __pyx_v_node->split_dim, __pyx_v_node->split); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 750, __pyx_L1_error)
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":749
 * 
 * 
 *     cdef inline int push_greater_of(self, innernode* node) except -1:             # <<<<<<<<<<<<<<
 *         return self.push(GREATER, node.split_dim, node.split)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.PointRectDistanceTracker.push_greater_of", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":753
 * 
 * 
 *     cdef inline int pop(self) except -1:             # <<<<<<<<<<<<<<
 *         self.stack_size -= 1
 *         assert self.stack_size >= 0
 */

static CYTHON_INLINE int __pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker_pop(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_v_self) {
  struct __pyx_t_5pyart_3map_7ckdtree_RP_stack_item *__pyx_v_item;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __pyx_t_5numpy_float64_t __pyx_t_1;
  double __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pop", 0);

  /* "pyart/map/ckdtree.pyx":754
 * 
 *     cdef inline int pop(self) except -1:
 *         self.stack_size -= 1             # <<<<<<<<<<<<<<
 *         assert self.stack_size >= 0
 * 
 */
  __pyx_v_self->stack_size = (__pyx_v_self->stack_size - 1);

  /* "pyart/map/ckdtree.pyx":755
 *     cdef inline int pop(self) except -1:
 *         self.stack_size -= 1
 *         assert self.stack_size >= 0             # <<<<<<<<<<<<<<
 * 
 *         cdef RP_stack_item* item = &self.stack[self.stack_size]
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_self->stack_size >= 0) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 755, __pyx_L1_error)
    }
  }
  #endif

  /* "pyart/map/ckdtree.pyx":757
 *         assert self.stack_size >= 0
 * 
 *         cdef RP_stack_item* item = &self.stack[self.stack_size]             # <<<<<<<<<<<<<<
 *         self.min_distance = item.min_distance
 *         self.max_distance = item.max_distance
 */
  __pyx_v_item = (&(__pyx_v_self->stack[__pyx_v_self->stack_size]));

  /* "pyart/map/ckdtree.pyx":758
 * 
 *         cdef RP_stack_item* item = &self.stack[self.stack_size]
 *         self.min_distance = item.min_distance             # <<<<<<<<<<<<<<
 *         self.max_distance = item.max_distance
 *         self.rect.mins[item.split_dim] = item.min_along_dim
 */
  __pyx_t_1 = __pyx_v_item->min_distance;
  __pyx_v_self->min_distance = __pyx_t_1;

  /* "pyart/map/ckdtree.pyx":759
 *         cdef RP_stack_item* item = &self.stack[self.stack_size]
 *         self.min_distance = item.min_distance
 *         self.max_distance = item.max_distance             # <<<<<<<<<<<<<<
 *         self.rect.mins[item.split_dim] = item.min_along_dim
 *         self.rect.maxes[item.split_dim] = item.max_along_dim
 */
  __pyx_t_1 = __pyx_v_item->max_distance;
  __pyx_v_self->max_distance = __pyx_t_1;

  /* "pyart/map/ckdtree.pyx":760
 *         self.min_distance = item.min_distance
 *         self.max_distance = item.max_distance
 *         self.rect.mins[item.split_dim] = item.min_along_dim             # <<<<<<<<<<<<<<
 *         self.rect.maxes[item.split_dim] = item.max_along_dim
 * 
 */
  __pyx_t_2 = __pyx_v_item->min_along_dim;
  (__pyx_v_self->rect->mins[__pyx_v_item->split_dim]) = __pyx_t_2;

  /* "pyart/map/ckdtree.pyx":761
 *         self.max_distance = item.max_distance
 *         self.rect.mins[item.split_dim] = item.min_along_dim
 *         self.rect.maxes[item.split_dim] = item.max_along_dim             # <<<<<<<<<<<<<<
 * 
 *         return 0
 */
  __pyx_t_2 = __pyx_v_item->max_along_dim;
  (__pyx_v_self->rect->maxes[__pyx_v_item->split_dim]) = __pyx_t_2;

  /* "pyart/map/ckdtree.pyx":763
 *         self.rect.maxes[item.split_dim] = item.max_along_dim
 * 
 *         return 0             # <<<<<<<<<<<<<<
 * 
 * # Tree structure
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":753
 * 
 * 
 *     cdef inline int pop(self) except -1:             # <<<<<<<<<<<<<<
 *         self.stack_size -= 1
 *         assert self.stack_size >= 0
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.PointRectDistanceTracker.pop", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.pt,self.stack cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_24PointRectDistanceTracker_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_24PointRectDistanceTracker_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_24PointRectDistanceTracker_2__reduce_cython__(((struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_3map_7ckdtree_24PointRectDistanceTracker_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.pt,self.stack cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.pt,self.stack cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.pt,self.stack cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.map.ckdtree.PointRectDistanceTracker.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.pt,self.stack cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.pt,self.stack cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_24PointRectDistanceTracker_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_24PointRectDistanceTracker_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_24PointRectDistanceTracker_4__setstate_cython__(((struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_3map_7ckdtree_24PointRectDistanceTracker_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.pt,self.stack cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.pt,self.stack cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__11, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.pt,self.stack cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.pt,self.stack cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.map.ckdtree.PointRectDistanceTracker.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":845
 *     cdef np.intp_t* raw_indices
 * 
 *     def __init__(cKDTree self, data, np.intp_t leafsize=10):             # <<<<<<<<<<<<<<
 *         self.data = np.ascontiguousarray(data,dtype=np.float64)
 *         self.n, self.m = np.shape(self.data)
 */

/* Python wrapper */
static int __pyx_pw_5pyart_3map_7ckdtree_7cKDTree_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pyart_3map_7ckdtree_7cKDTree_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  __pyx_t_5numpy_intp_t __pyx_v_leafsize;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_leafsize,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_leafsize);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 845, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_data = values[0];
    if (values[1]) {
      __pyx_v_leafsize = __Pyx_PyInt_As_Py_intptr_t(values[1]); if (unlikely((__pyx_v_leafsize == ((npy_intp)-1)) && PyErr_Occurred())) __PYX_ERR(0, 845, __pyx_L3_error)
    } else {
      __pyx_v_leafsize = ((__pyx_t_5numpy_intp_t)10);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 845, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_7cKDTree___init__(((struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self), __pyx_v_data, __pyx_v_leafsize);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_3map_7ckdtree_7cKDTree___init__(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, PyObject *__pyx_v_data, __pyx_t_5numpy_intp_t __pyx_v_leafsize) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *(*__pyx_t_6)(PyObject *);
  __pyx_t_5numpy_intp_t __pyx_t_7;
  __pyx_t_5numpy_intp_t __pyx_t_8;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pyart/map/ckdtree.pyx":846
 * 
 *     def __init__(cKDTree self, data, np.intp_t leafsize=10):
 *         self.data = np.ascontiguousarray(data,dtype=np.float64)             # <<<<<<<<<<<<<<
 *         self.n, self.m = np.shape(self.data)
 *         self.leafsize = leafsize
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 846, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_ascontiguousarray); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 846, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 846, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_data);
  __Pyx_GIVEREF(__pyx_v_data);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_data);
  __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 846, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 846, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_float64); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 846, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_5) < 0) __PYX_ERR(0, 846, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 846, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (!(likely(((__pyx_t_5) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_5, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 846, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_5);
  __Pyx_GOTREF(__pyx_v_self->data);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->data));
  __pyx_v_self->data = ((PyArrayObject *)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "pyart/map/ckdtree.pyx":847
 *     def __init__(cKDTree self, data, np.intp_t leafsize=10):
 *         self.data = np.ascontiguousarray(data,dtype=np.float64)
 *         self.n, self.m = np.shape(self.data)             # <<<<<<<<<<<<<<
 *         self.leafsize = leafsize
 *         if self.leafsize<1:
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 847, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 847, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_5 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_3, ((PyObject *)__pyx_v_self->data)) : __Pyx_PyObject_CallOneArg(__pyx_t_1, ((PyObject *)__pyx_v_self->data));
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 847, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if ((likely(PyTuple_CheckExact(__pyx_t_5))) || (PyList_CheckExact(__pyx_t_5))) {
    PyObject* sequence = __pyx_t_5;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 847, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_1 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_3);
    #else
    __pyx_t_1 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 847, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 847, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_2 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 847, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_6 = Py_TYPE(__pyx_t_2)->tp_iternext;
    index = 0; __pyx_t_1 = __pyx_t_6(__pyx_t_2); if (unlikely(!__pyx_t_1)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_1);
    index = 1; __pyx_t_3 = __pyx_t_6(__pyx_t_2); if (unlikely(!__pyx_t_3)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_2), 2) < 0) __PYX_ERR(0, 847, __pyx_L1_error)
    __pyx_t_6 = NULL;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_6 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 847, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }
  __pyx_t_7 = __Pyx_PyInt_As_Py_intptr_t(__pyx_t_1); if (unlikely((__pyx_t_7 == ((npy_intp)-1)) && PyErr_Occurred())) __PYX_ERR(0, 847, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = __Pyx_PyInt_As_Py_intptr_t(__pyx_t_3); if (unlikely((__pyx_t_8 == ((npy_intp)-1)) && PyErr_Occurred())) __PYX_ERR(0, 847, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_self->n = __pyx_t_7;
  __pyx_v_self->m = __pyx_t_8;

  /* "pyart/map/ckdtree.pyx":848
 *         self.data = np.ascontiguousarray(data,dtype=np.float64)
 *         self.n, self.m = np.shape(self.data)
 *         self.leafsize = leafsize             # <<<<<<<<<<<<<<
 *         if self.leafsize<1:
 *             raise ValueError("leafsize must be at least 1")
 */
  __pyx_v_self->leafsize = __pyx_v_leafsize;

  /* "pyart/map/ckdtree.pyx":849
 *         self.n, self.m = np.shape(self.data)
 *         self.leafsize = leafsize
 *         if self.leafsize<1:             # <<<<<<<<<<<<<<
 *             raise ValueError("leafsize must be at least 1")
 *         self.maxes = np.ascontiguousarray(np.amax(self.data,axis=0), dtype=np.float64)
 */
  __pyx_t_9 = ((__pyx_v_self->leafsize < 1) != 0);
  if (unlikely(__pyx_t_9)) {

    /* "pyart/map/ckdtree.pyx":850
 *         self.leafsize = leafsize
 *         if self.leafsize<1:
 *             raise ValueError("leafsize must be at least 1")             # <<<<<<<<<<<<<<
 *         self.maxes = np.ascontiguousarray(np.amax(self.data,axis=0), dtype=np.float64)
 *         self.mins = np.ascontiguousarray(np.amin(self.data,axis=0), dtype=np.float64)
 */
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 850, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(0, 850, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":849
 *         self.n, self.m = np.shape(self.data)
 *         self.leafsize = leafsize
 *         if self.leafsize<1:             # <<<<<<<<<<<<<<
 *             raise ValueError("leafsize must be at least 1")
 *         self.maxes = np.ascontiguousarray(np.amax(self.data,axis=0), dtype=np.float64)
 */
  }

  /* "pyart/map/ckdtree.pyx":851
 *         if self.leafsize<1:
 *             raise ValueError("leafsize must be at least 1")
 *         self.maxes = np.ascontiguousarray(np.amax(self.data,axis=0), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         self.mins = np.ascontiguousarray(np.amin(self.data,axis=0), dtype=np.float64)
 *         self.indices = np.ascontiguousarray(np.arange(self.n,dtype=np.intp))
 */
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 851, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_ascontiguousarray); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 851, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 851, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_amax); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 851, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 851, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->data));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->data));
  PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)__pyx_v_self->data));
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 851, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_axis, __pyx_int_0) < 0) __PYX_ERR(0, 851, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_5, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 851, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 851, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 851, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 851, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_float64); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 851, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_1) < 0) __PYX_ERR(0, 851, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 851, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 851, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->maxes);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->maxes));
  __pyx_v_self->maxes = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/map/ckdtree.pyx":852
 *             raise ValueError("leafsize must be at least 1")
 *         self.maxes = np.ascontiguousarray(np.amax(self.data,axis=0), dtype=np.float64)
 *         self.mins = np.ascontiguousarray(np.amin(self.data,axis=0), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         self.indices = np.ascontiguousarray(np.arange(self.n,dtype=np.intp))
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 852, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_ascontiguousarray); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 852, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 852, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_amin); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 852, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 852, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->data));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->data));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_self->data));
  __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 852, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_axis, __pyx_int_0) < 0) __PYX_ERR(0, 852, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 852, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 852, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 852, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 852, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_float64); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 852, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_dtype, __pyx_t_2) < 0) __PYX_ERR(0, 852, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_3, __pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 852, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 852, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->mins);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->mins));
  __pyx_v_self->mins = ((PyArrayObject *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyart/map/ckdtree.pyx":853
 *         self.maxes = np.ascontiguousarray(np.amax(self.data,axis=0), dtype=np.float64)
 *         self.mins = np.ascontiguousarray(np.amin(self.data,axis=0), dtype=np.float64)
 *         self.indices = np.ascontiguousarray(np.arange(self.n,dtype=np.intp))             # <<<<<<<<<<<<<<
 * 
 *         self.raw_data = <np.float64_t*>np.PyArray_DATA(self.data)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 853, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_ascontiguousarray); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 853, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 853, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_arange); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 853, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->n); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 853, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 853, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 853, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_np); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 853, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_intp); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 853, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_dtype, __pyx_t_11) < 0) __PYX_ERR(0, 853, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_1, __pyx_t_5); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 853, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_11) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_11);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 853, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 853, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->indices);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->indices));
  __pyx_v_self->indices = ((PyArrayObject *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyart/map/ckdtree.pyx":855
 *         self.indices = np.ascontiguousarray(np.arange(self.n,dtype=np.intp))
 * 
 *         self.raw_data = <np.float64_t*>np.PyArray_DATA(self.data)             # <<<<<<<<<<<<<<
 *         self.raw_maxes = <np.float64_t*>np.PyArray_DATA(self.maxes)
 *         self.raw_mins = <np.float64_t*>np.PyArray_DATA(self.mins)
 */
  __pyx_t_2 = ((PyObject *)__pyx_v_self->data);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_v_self->raw_data = ((__pyx_t_5numpy_float64_t *)PyArray_DATA(((PyArrayObject *)__pyx_t_2)));
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyart/map/ckdtree.pyx":856
 * 
 *         self.raw_data = <np.float64_t*>np.PyArray_DATA(self.data)
 *         self.raw_maxes = <np.float64_t*>np.PyArray_DATA(self.maxes)             # <<<<<<<<<<<<<<
 *         self.raw_mins = <np.float64_t*>np.PyArray_DATA(self.mins)
 *         self.raw_indices = <np.intp_t*>np.PyArray_DATA(self.indices)
 */
  __pyx_t_2 = ((PyObject *)__pyx_v_self->maxes);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_v_self->raw_maxes = ((__pyx_t_5numpy_float64_t *)PyArray_DATA(((PyArrayObject *)__pyx_t_2)));
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyart/map/ckdtree.pyx":857
 *         self.raw_data = <np.float64_t*>np.PyArray_DATA(self.data)
 *         self.raw_maxes = <np.float64_t*>np.PyArray_DATA(self.maxes)
 *         self.raw_mins = <np.float64_t*>np.PyArray_DATA(self.mins)             # <<<<<<<<<<<<<<
 *         self.raw_indices = <np.intp_t*>np.PyArray_DATA(self.indices)
 * 
 */
  __pyx_t_2 = ((PyObject *)__pyx_v_self->mins);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_v_self->raw_mins = ((__pyx_t_5numpy_float64_t *)PyArray_DATA(((PyArrayObject *)__pyx_t_2)));
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyart/map/ckdtree.pyx":858
 *         self.raw_maxes = <np.float64_t*>np.PyArray_DATA(self.maxes)
 *         self.raw_mins = <np.float64_t*>np.PyArray_DATA(self.mins)
 *         self.raw_indices = <np.intp_t*>np.PyArray_DATA(self.indices)             # <<<<<<<<<<<<<<
 * 
 *         self.tree = self.__build(0, self.n, self.raw_maxes, self.raw_mins)
 */
  __pyx_t_2 = ((PyObject *)__pyx_v_self->indices);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_v_self->raw_indices = ((__pyx_t_5numpy_intp_t *)PyArray_DATA(((PyArrayObject *)__pyx_t_2)));
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyart/map/ckdtree.pyx":860
 *         self.raw_indices = <np.intp_t*>np.PyArray_DATA(self.indices)
 * 
 *         self.tree = self.__build(0, self.n, self.raw_maxes, self.raw_mins)             # <<<<<<<<<<<<<<
 * 
 *     cdef innernode* __build(cKDTree self, np.intp_t start_idx, np.intp_t end_idx,
 */
  __pyx_t_12 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___build(__pyx_v_self, 0, __pyx_v_self->n, __pyx_v_self->raw_maxes, __pyx_v_self->raw_mins); if (unlikely(__pyx_t_12 == ((struct __pyx_t_5pyart_3map_7ckdtree_innernode *)((struct __pyx_t_5pyart_3map_7ckdtree_innernode *)NULL)) && PyErr_Occurred())) __PYX_ERR(0, 860, __pyx_L1_error)
  __pyx_v_self->tree = __pyx_t_12;

  /* "pyart/map/ckdtree.pyx":845
 *     cdef np.intp_t* raw_indices
 * 
 *     def __init__(cKDTree self, data, np.intp_t leafsize=10):             # <<<<<<<<<<<<<<
 *         self.data = np.ascontiguousarray(data,dtype=np.float64)
 *         self.n, self.m = np.shape(self.data)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":862
 *         self.tree = self.__build(0, self.n, self.raw_maxes, self.raw_mins)
 * 
 *     cdef innernode* __build(cKDTree self, np.intp_t start_idx, np.intp_t end_idx,             # <<<<<<<<<<<<<<
 *                             np.float64_t* maxes, np.float64_t* mins) except? <innernode*> NULL:
 *         cdef leafnode* n
 */

static struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_f_5pyart_3map_7ckdtree_7cKDTree___build(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, __pyx_t_5numpy_intp_t __pyx_v_start_idx, __pyx_t_5numpy_intp_t __pyx_v_end_idx, __pyx_t_5numpy_float64_t *__pyx_v_maxes, __pyx_t_5numpy_float64_t *__pyx_v_mins) {
  struct __pyx_t_5pyart_3map_7ckdtree_leafnode *__pyx_v_n;
  struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_ni;
  __pyx_t_5numpy_intp_t __pyx_v_i;
  __pyx_t_5numpy_intp_t __pyx_v_j;
  __pyx_t_5numpy_intp_t __pyx_v_t;
  __pyx_t_5numpy_intp_t __pyx_v_p;
  __pyx_t_5numpy_intp_t __pyx_v_q;
  __pyx_t_5numpy_intp_t __pyx_v_d;
  __pyx_t_5numpy_float64_t __pyx_v_size;
  __pyx_t_5numpy_float64_t __pyx_v_split;
  __pyx_t_5numpy_float64_t __pyx_v_minval;
  __pyx_t_5numpy_float64_t __pyx_v_maxval;
  __pyx_t_5numpy_float64_t *__pyx_v_mids;
  struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __pyx_t_5numpy_intp_t __pyx_t_2;
  __pyx_t_5numpy_intp_t __pyx_t_3;
  __pyx_t_5numpy_intp_t __pyx_t_4;
  long __pyx_t_5;
  long __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__build", 0);

  /* "pyart/map/ckdtree.pyx":869
 *         cdef np.float64_t size, split, minval, maxval
 *         cdef np.float64_t*mids
 *         if end_idx-start_idx<=self.leafsize:             # <<<<<<<<<<<<<<
 *             n = <leafnode*>stdlib.malloc(sizeof(leafnode))
 *             if n == <leafnode*> NULL:
 */
  __pyx_t_1 = (((__pyx_v_end_idx - __pyx_v_start_idx) <= __pyx_v_self->leafsize) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":870
 *         cdef np.float64_t*mids
 *         if end_idx-start_idx<=self.leafsize:
 *             n = <leafnode*>stdlib.malloc(sizeof(leafnode))             # <<<<<<<<<<<<<<
 *             if n == <leafnode*> NULL:
 *                 raise MemoryError
 */
    __pyx_v_n = ((struct __pyx_t_5pyart_3map_7ckdtree_leafnode *)malloc((sizeof(struct __pyx_t_5pyart_3map_7ckdtree_leafnode))));

    /* "pyart/map/ckdtree.pyx":871
 *         if end_idx-start_idx<=self.leafsize:
 *             n = <leafnode*>stdlib.malloc(sizeof(leafnode))
 *             if n == <leafnode*> NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError
 *             n.split_dim = -1
 */
    __pyx_t_1 = ((__pyx_v_n == ((struct __pyx_t_5pyart_3map_7ckdtree_leafnode *)NULL)) != 0);
    if (unlikely(__pyx_t_1)) {

      /* "pyart/map/ckdtree.pyx":872
 *             n = <leafnode*>stdlib.malloc(sizeof(leafnode))
 *             if n == <leafnode*> NULL:
 *                 raise MemoryError             # <<<<<<<<<<<<<<
 *             n.split_dim = -1
 *             n.children = end_idx - start_idx
 */
      PyErr_NoMemory(); __PYX_ERR(0, 872, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":871
 *         if end_idx-start_idx<=self.leafsize:
 *             n = <leafnode*>stdlib.malloc(sizeof(leafnode))
 *             if n == <leafnode*> NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError
 *             n.split_dim = -1
 */
    }

    /* "pyart/map/ckdtree.pyx":873
 *             if n == <leafnode*> NULL:
 *                 raise MemoryError
 *             n.split_dim = -1             # <<<<<<<<<<<<<<
 *             n.children = end_idx - start_idx
 *             n.start_idx = start_idx
 */
    __pyx_v_n->split_dim = -1;

    /* "pyart/map/ckdtree.pyx":874
 *                 raise MemoryError
 *             n.split_dim = -1
 *             n.children = end_idx - start_idx             # <<<<<<<<<<<<<<
 *             n.start_idx = start_idx
 *             n.end_idx = end_idx
 */
    __pyx_v_n->children = (__pyx_v_end_idx - __pyx_v_start_idx);

    /* "pyart/map/ckdtree.pyx":875
 *             n.split_dim = -1
 *             n.children = end_idx - start_idx
 *             n.start_idx = start_idx             # <<<<<<<<<<<<<<
 *             n.end_idx = end_idx
 *             return <innernode*>n
 */
    __pyx_v_n->start_idx = __pyx_v_start_idx;

    /* "pyart/map/ckdtree.pyx":876
 *             n.children = end_idx - start_idx
 *             n.start_idx = start_idx
 *             n.end_idx = end_idx             # <<<<<<<<<<<<<<
 *             return <innernode*>n
 *         else:
 */
    __pyx_v_n->end_idx = __pyx_v_end_idx;

    /* "pyart/map/ckdtree.pyx":877
 *             n.start_idx = start_idx
 *             n.end_idx = end_idx
 *             return <innernode*>n             # <<<<<<<<<<<<<<
 *         else:
 *             d = 0
 */
    __pyx_r = ((struct __pyx_t_5pyart_3map_7ckdtree_innernode *)__pyx_v_n);
    goto __pyx_L0;

    /* "pyart/map/ckdtree.pyx":869
 *         cdef np.float64_t size, split, minval, maxval
 *         cdef np.float64_t*mids
 *         if end_idx-start_idx<=self.leafsize:             # <<<<<<<<<<<<<<
 *             n = <leafnode*>stdlib.malloc(sizeof(leafnode))
 *             if n == <leafnode*> NULL:
 */
  }

  /* "pyart/map/ckdtree.pyx":879
 *             return <innernode*>n
 *         else:
 *             d = 0             # <<<<<<<<<<<<<<
 *             size = 0
 *             for i in range(self.m):
 */
  /*else*/ {
    __pyx_v_d = 0;

    /* "pyart/map/ckdtree.pyx":880
 *         else:
 *             d = 0
 *             size = 0             # <<<<<<<<<<<<<<
 *             for i in range(self.m):
 *                 if maxes[i]-mins[i] > size:
 */
    __pyx_v_size = 0.0;

    /* "pyart/map/ckdtree.pyx":881
 *             d = 0
 *             size = 0
 *             for i in range(self.m):             # <<<<<<<<<<<<<<
 *                 if maxes[i]-mins[i] > size:
 *                     d = i
 */
    __pyx_t_2 = __pyx_v_self->m;
    __pyx_t_3 = __pyx_t_2;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_i = __pyx_t_4;

      /* "pyart/map/ckdtree.pyx":882
 *             size = 0
 *             for i in range(self.m):
 *                 if maxes[i]-mins[i] > size:             # <<<<<<<<<<<<<<
 *                     d = i
 *                     size =  maxes[i]-mins[i]
 */
      __pyx_t_1 = ((((__pyx_v_maxes[__pyx_v_i]) - (__pyx_v_mins[__pyx_v_i])) > __pyx_v_size) != 0);
      if (__pyx_t_1) {

        /* "pyart/map/ckdtree.pyx":883
 *             for i in range(self.m):
 *                 if maxes[i]-mins[i] > size:
 *                     d = i             # <<<<<<<<<<<<<<
 *                     size =  maxes[i]-mins[i]
 *             maxval = maxes[d]
 */
        __pyx_v_d = __pyx_v_i;

        /* "pyart/map/ckdtree.pyx":884
 *                 if maxes[i]-mins[i] > size:
 *                     d = i
 *                     size =  maxes[i]-mins[i]             # <<<<<<<<<<<<<<
 *             maxval = maxes[d]
 *             minval = mins[d]
 */
        __pyx_v_size = ((__pyx_v_maxes[__pyx_v_i]) - (__pyx_v_mins[__pyx_v_i]));

        /* "pyart/map/ckdtree.pyx":882
 *             size = 0
 *             for i in range(self.m):
 *                 if maxes[i]-mins[i] > size:             # <<<<<<<<<<<<<<
 *                     d = i
 *                     size =  maxes[i]-mins[i]
 */
      }
    }

    /* "pyart/map/ckdtree.pyx":885
 *                     d = i
 *                     size =  maxes[i]-mins[i]
 *             maxval = maxes[d]             # <<<<<<<<<<<<<<
 *             minval = mins[d]
 *             if maxval==minval:
 */
    __pyx_v_maxval = (__pyx_v_maxes[__pyx_v_d]);

    /* "pyart/map/ckdtree.pyx":886
 *                     size =  maxes[i]-mins[i]
 *             maxval = maxes[d]
 *             minval = mins[d]             # <<<<<<<<<<<<<<
 *             if maxval==minval:
 *                 # all points are identical; warn user?
 */
    __pyx_v_minval = (__pyx_v_mins[__pyx_v_d]);

    /* "pyart/map/ckdtree.pyx":887
 *             maxval = maxes[d]
 *             minval = mins[d]
 *             if maxval==minval:             # <<<<<<<<<<<<<<
 *                 # all points are identical; warn user?
 *                 n = <leafnode*>stdlib.malloc(sizeof(leafnode))
 */
    __pyx_t_1 = ((__pyx_v_maxval == __pyx_v_minval) != 0);
    if (__pyx_t_1) {

      /* "pyart/map/ckdtree.pyx":889
 *             if maxval==minval:
 *                 # all points are identical; warn user?
 *                 n = <leafnode*>stdlib.malloc(sizeof(leafnode))             # <<<<<<<<<<<<<<
 *                 if n == <leafnode*> NULL:
 *                     raise MemoryError
 */
      __pyx_v_n = ((struct __pyx_t_5pyart_3map_7ckdtree_leafnode *)malloc((sizeof(struct __pyx_t_5pyart_3map_7ckdtree_leafnode))));

      /* "pyart/map/ckdtree.pyx":890
 *                 # all points are identical; warn user?
 *                 n = <leafnode*>stdlib.malloc(sizeof(leafnode))
 *                 if n == <leafnode*> NULL:             # <<<<<<<<<<<<<<
 *                     raise MemoryError
 *                 n.split_dim = -1
 */
      __pyx_t_1 = ((__pyx_v_n == ((struct __pyx_t_5pyart_3map_7ckdtree_leafnode *)NULL)) != 0);
      if (unlikely(__pyx_t_1)) {

        /* "pyart/map/ckdtree.pyx":891
 *                 n = <leafnode*>stdlib.malloc(sizeof(leafnode))
 *                 if n == <leafnode*> NULL:
 *                     raise MemoryError             # <<<<<<<<<<<<<<
 *                 n.split_dim = -1
 *                 n.children = end_idx - start_idx
 */
        PyErr_NoMemory(); __PYX_ERR(0, 891, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":890
 *                 # all points are identical; warn user?
 *                 n = <leafnode*>stdlib.malloc(sizeof(leafnode))
 *                 if n == <leafnode*> NULL:             # <<<<<<<<<<<<<<
 *                     raise MemoryError
 *                 n.split_dim = -1
 */
      }

      /* "pyart/map/ckdtree.pyx":892
 *                 if n == <leafnode*> NULL:
 *                     raise MemoryError
 *                 n.split_dim = -1             # <<<<<<<<<<<<<<
 *                 n.children = end_idx - start_idx
 *                 n.start_idx = start_idx
 */
      __pyx_v_n->split_dim = -1;

      /* "pyart/map/ckdtree.pyx":893
 *                     raise MemoryError
 *                 n.split_dim = -1
 *                 n.children = end_idx - start_idx             # <<<<<<<<<<<<<<
 *                 n.start_idx = start_idx
 *                 n.end_idx = end_idx
 */
      __pyx_v_n->children = (__pyx_v_end_idx - __pyx_v_start_idx);

      /* "pyart/map/ckdtree.pyx":894
 *                 n.split_dim = -1
 *                 n.children = end_idx - start_idx
 *                 n.start_idx = start_idx             # <<<<<<<<<<<<<<
 *                 n.end_idx = end_idx
 *                 return <innernode*>n
 */
      __pyx_v_n->start_idx = __pyx_v_start_idx;

      /* "pyart/map/ckdtree.pyx":895
 *                 n.children = end_idx - start_idx
 *                 n.start_idx = start_idx
 *                 n.end_idx = end_idx             # <<<<<<<<<<<<<<
 *                 return <innernode*>n
 * 
 */
      __pyx_v_n->end_idx = __pyx_v_end_idx;

      /* "pyart/map/ckdtree.pyx":896
 *                 n.start_idx = start_idx
 *                 n.end_idx = end_idx
 *                 return <innernode*>n             # <<<<<<<<<<<<<<
 * 
 *             split = (maxval+minval)/2
 */
      __pyx_r = ((struct __pyx_t_5pyart_3map_7ckdtree_innernode *)__pyx_v_n);
      goto __pyx_L0;

      /* "pyart/map/ckdtree.pyx":887
 *             maxval = maxes[d]
 *             minval = mins[d]
 *             if maxval==minval:             # <<<<<<<<<<<<<<
 *                 # all points are identical; warn user?
 *                 n = <leafnode*>stdlib.malloc(sizeof(leafnode))
 */
    }

    /* "pyart/map/ckdtree.pyx":898
 *                 return <innernode*>n
 * 
 *             split = (maxval+minval)/2             # <<<<<<<<<<<<<<
 * 
 *             p = start_idx
 */
    __pyx_v_split = ((__pyx_v_maxval + __pyx_v_minval) / 2.0);

    /* "pyart/map/ckdtree.pyx":900
 *             split = (maxval+minval)/2
 * 
 *             p = start_idx             # <<<<<<<<<<<<<<
 *             q = end_idx-1
 *             while p<=q:
 */
    __pyx_v_p = __pyx_v_start_idx;

    /* "pyart/map/ckdtree.pyx":901
 * 
 *             p = start_idx
 *             q = end_idx-1             # <<<<<<<<<<<<<<
 *             while p<=q:
 *                 if self.raw_data[self.raw_indices[p]*self.m+d]<split:
 */
    __pyx_v_q = (__pyx_v_end_idx - 1);

    /* "pyart/map/ckdtree.pyx":902
 *             p = start_idx
 *             q = end_idx-1
 *             while p<=q:             # <<<<<<<<<<<<<<
 *                 if self.raw_data[self.raw_indices[p]*self.m+d]<split:
 *                     p+=1
 */
    while (1) {
      __pyx_t_1 = ((__pyx_v_p <= __pyx_v_q) != 0);
      if (!__pyx_t_1) break;

      /* "pyart/map/ckdtree.pyx":903
 *             q = end_idx-1
 *             while p<=q:
 *                 if self.raw_data[self.raw_indices[p]*self.m+d]<split:             # <<<<<<<<<<<<<<
 *                     p+=1
 *                 elif self.raw_data[self.raw_indices[q]*self.m+d]>=split:
 */
      __pyx_t_1 = (((__pyx_v_self->raw_data[(((__pyx_v_self->raw_indices[__pyx_v_p]) * __pyx_v_self->m) + __pyx_v_d)]) < __pyx_v_split) != 0);
      if (__pyx_t_1) {

        /* "pyart/map/ckdtree.pyx":904
 *             while p<=q:
 *                 if self.raw_data[self.raw_indices[p]*self.m+d]<split:
 *                     p+=1             # <<<<<<<<<<<<<<
 *                 elif self.raw_data[self.raw_indices[q]*self.m+d]>=split:
 *                     q-=1
 */
        __pyx_v_p = (__pyx_v_p + 1);

        /* "pyart/map/ckdtree.pyx":903
 *             q = end_idx-1
 *             while p<=q:
 *                 if self.raw_data[self.raw_indices[p]*self.m+d]<split:             # <<<<<<<<<<<<<<
 *                     p+=1
 *                 elif self.raw_data[self.raw_indices[q]*self.m+d]>=split:
 */
        goto __pyx_L12;
      }

      /* "pyart/map/ckdtree.pyx":905
 *                 if self.raw_data[self.raw_indices[p]*self.m+d]<split:
 *                     p+=1
 *                 elif self.raw_data[self.raw_indices[q]*self.m+d]>=split:             # <<<<<<<<<<<<<<
 *                     q-=1
 *                 else:
 */
      __pyx_t_1 = (((__pyx_v_self->raw_data[(((__pyx_v_self->raw_indices[__pyx_v_q]) * __pyx_v_self->m) + __pyx_v_d)]) >= __pyx_v_split) != 0);
      if (__pyx_t_1) {

        /* "pyart/map/ckdtree.pyx":906
 *                     p+=1
 *                 elif self.raw_data[self.raw_indices[q]*self.m+d]>=split:
 *                     q-=1             # <<<<<<<<<<<<<<
 *                 else:
 *                     t = self.raw_indices[p]
 */
        __pyx_v_q = (__pyx_v_q - 1);

        /* "pyart/map/ckdtree.pyx":905
 *                 if self.raw_data[self.raw_indices[p]*self.m+d]<split:
 *                     p+=1
 *                 elif self.raw_data[self.raw_indices[q]*self.m+d]>=split:             # <<<<<<<<<<<<<<
 *                     q-=1
 *                 else:
 */
        goto __pyx_L12;
      }

      /* "pyart/map/ckdtree.pyx":908
 *                     q-=1
 *                 else:
 *                     t = self.raw_indices[p]             # <<<<<<<<<<<<<<
 *                     self.raw_indices[p] = self.raw_indices[q]
 *                     self.raw_indices[q] = t
 */
      /*else*/ {
        __pyx_v_t = (__pyx_v_self->raw_indices[__pyx_v_p]);

        /* "pyart/map/ckdtree.pyx":909
 *                 else:
 *                     t = self.raw_indices[p]
 *                     self.raw_indices[p] = self.raw_indices[q]             # <<<<<<<<<<<<<<
 *                     self.raw_indices[q] = t
 *                     p+=1
 */
        (__pyx_v_self->raw_indices[__pyx_v_p]) = (__pyx_v_self->raw_indices[__pyx_v_q]);

        /* "pyart/map/ckdtree.pyx":910
 *                     t = self.raw_indices[p]
 *                     self.raw_indices[p] = self.raw_indices[q]
 *                     self.raw_indices[q] = t             # <<<<<<<<<<<<<<
 *                     p+=1
 *                     q-=1
 */
        (__pyx_v_self->raw_indices[__pyx_v_q]) = __pyx_v_t;

        /* "pyart/map/ckdtree.pyx":911
 *                     self.raw_indices[p] = self.raw_indices[q]
 *                     self.raw_indices[q] = t
 *                     p+=1             # <<<<<<<<<<<<<<
 *                     q-=1
 * 
 */
        __pyx_v_p = (__pyx_v_p + 1);

        /* "pyart/map/ckdtree.pyx":912
 *                     self.raw_indices[q] = t
 *                     p+=1
 *                     q-=1             # <<<<<<<<<<<<<<
 * 
 *             # slide midpoint if necessary
 */
        __pyx_v_q = (__pyx_v_q - 1);
      }
      __pyx_L12:;
    }

    /* "pyart/map/ckdtree.pyx":915
 * 
 *             # slide midpoint if necessary
 *             if p==start_idx:             # <<<<<<<<<<<<<<
 *                 # no points less than split
 *                 j = start_idx
 */
    __pyx_t_1 = ((__pyx_v_p == __pyx_v_start_idx) != 0);
    if (__pyx_t_1) {

      /* "pyart/map/ckdtree.pyx":917
 *             if p==start_idx:
 *                 # no points less than split
 *                 j = start_idx             # <<<<<<<<<<<<<<
 *                 split = self.raw_data[self.raw_indices[j]*self.m+d]
 *                 for i in range(start_idx+1, end_idx):
 */
      __pyx_v_j = __pyx_v_start_idx;

      /* "pyart/map/ckdtree.pyx":918
 *                 # no points less than split
 *                 j = start_idx
 *                 split = self.raw_data[self.raw_indices[j]*self.m+d]             # <<<<<<<<<<<<<<
 *                 for i in range(start_idx+1, end_idx):
 *                     if self.raw_data[self.raw_indices[i]*self.m+d]<split:
 */
      __pyx_v_split = (__pyx_v_self->raw_data[(((__pyx_v_self->raw_indices[__pyx_v_j]) * __pyx_v_self->m) + __pyx_v_d)]);

      /* "pyart/map/ckdtree.pyx":919
 *                 j = start_idx
 *                 split = self.raw_data[self.raw_indices[j]*self.m+d]
 *                 for i in range(start_idx+1, end_idx):             # <<<<<<<<<<<<<<
 *                     if self.raw_data[self.raw_indices[i]*self.m+d]<split:
 *                         j = i
 */
      __pyx_t_2 = __pyx_v_end_idx;
      __pyx_t_3 = __pyx_t_2;
      for (__pyx_t_4 = (__pyx_v_start_idx + 1); __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
        __pyx_v_i = __pyx_t_4;

        /* "pyart/map/ckdtree.pyx":920
 *                 split = self.raw_data[self.raw_indices[j]*self.m+d]
 *                 for i in range(start_idx+1, end_idx):
 *                     if self.raw_data[self.raw_indices[i]*self.m+d]<split:             # <<<<<<<<<<<<<<
 *                         j = i
 *                         split = self.raw_data[self.raw_indices[j]*self.m+d]
 */
        __pyx_t_1 = (((__pyx_v_self->raw_data[(((__pyx_v_self->raw_indices[__pyx_v_i]) * __pyx_v_self->m) + __pyx_v_d)]) < __pyx_v_split) != 0);
        if (__pyx_t_1) {

          /* "pyart/map/ckdtree.pyx":921
 *                 for i in range(start_idx+1, end_idx):
 *                     if self.raw_data[self.raw_indices[i]*self.m+d]<split:
 *                         j = i             # <<<<<<<<<<<<<<
 *                         split = self.raw_data[self.raw_indices[j]*self.m+d]
 *                 t = self.raw_indices[start_idx]
 */
          __pyx_v_j = __pyx_v_i;

          /* "pyart/map/ckdtree.pyx":922
 *                     if self.raw_data[self.raw_indices[i]*self.m+d]<split:
 *                         j = i
 *                         split = self.raw_data[self.raw_indices[j]*self.m+d]             # <<<<<<<<<<<<<<
 *                 t = self.raw_indices[start_idx]
 *                 self.raw_indices[start_idx] = self.raw_indices[j]
 */
          __pyx_v_split = (__pyx_v_self->raw_data[(((__pyx_v_self->raw_indices[__pyx_v_j]) * __pyx_v_self->m) + __pyx_v_d)]);

          /* "pyart/map/ckdtree.pyx":920
 *                 split = self.raw_data[self.raw_indices[j]*self.m+d]
 *                 for i in range(start_idx+1, end_idx):
 *                     if self.raw_data[self.raw_indices[i]*self.m+d]<split:             # <<<<<<<<<<<<<<
 *                         j = i
 *                         split = self.raw_data[self.raw_indices[j]*self.m+d]
 */
        }
      }

      /* "pyart/map/ckdtree.pyx":923
 *                         j = i
 *                         split = self.raw_data[self.raw_indices[j]*self.m+d]
 *                 t = self.raw_indices[start_idx]             # <<<<<<<<<<<<<<
 *                 self.raw_indices[start_idx] = self.raw_indices[j]
 *                 self.raw_indices[j] = t
 */
      __pyx_v_t = (__pyx_v_self->raw_indices[__pyx_v_start_idx]);

      /* "pyart/map/ckdtree.pyx":924
 *                         split = self.raw_data[self.raw_indices[j]*self.m+d]
 *                 t = self.raw_indices[start_idx]
 *                 self.raw_indices[start_idx] = self.raw_indices[j]             # <<<<<<<<<<<<<<
 *                 self.raw_indices[j] = t
 *                 p = start_idx+1
 */
      (__pyx_v_self->raw_indices[__pyx_v_start_idx]) = (__pyx_v_self->raw_indices[__pyx_v_j]);

      /* "pyart/map/ckdtree.pyx":925
 *                 t = self.raw_indices[start_idx]
 *                 self.raw_indices[start_idx] = self.raw_indices[j]
 *                 self.raw_indices[j] = t             # <<<<<<<<<<<<<<
 *                 p = start_idx+1
 *                 q = start_idx
 */
      (__pyx_v_self->raw_indices[__pyx_v_j]) = __pyx_v_t;

      /* "pyart/map/ckdtree.pyx":926
 *                 self.raw_indices[start_idx] = self.raw_indices[j]
 *                 self.raw_indices[j] = t
 *                 p = start_idx+1             # <<<<<<<<<<<<<<
 *                 q = start_idx
 *             elif p==end_idx:
 */
      __pyx_v_p = (__pyx_v_start_idx + 1);

      /* "pyart/map/ckdtree.pyx":927
 *                 self.raw_indices[j] = t
 *                 p = start_idx+1
 *                 q = start_idx             # <<<<<<<<<<<<<<
 *             elif p==end_idx:
 *                 # no points greater than split
 */
      __pyx_v_q = __pyx_v_start_idx;

      /* "pyart/map/ckdtree.pyx":915
 * 
 *             # slide midpoint if necessary
 *             if p==start_idx:             # <<<<<<<<<<<<<<
 *                 # no points less than split
 *                 j = start_idx
 */
      goto __pyx_L13;
    }

    /* "pyart/map/ckdtree.pyx":928
 *                 p = start_idx+1
 *                 q = start_idx
 *             elif p==end_idx:             # <<<<<<<<<<<<<<
 *                 # no points greater than split
 *                 j = end_idx-1
 */
    __pyx_t_1 = ((__pyx_v_p == __pyx_v_end_idx) != 0);
    if (__pyx_t_1) {

      /* "pyart/map/ckdtree.pyx":930
 *             elif p==end_idx:
 *                 # no points greater than split
 *                 j = end_idx-1             # <<<<<<<<<<<<<<
 *                 split = self.raw_data[self.raw_indices[j]*self.m+d]
 *                 for i in range(start_idx, end_idx-1):
 */
      __pyx_v_j = (__pyx_v_end_idx - 1);

      /* "pyart/map/ckdtree.pyx":931
 *                 # no points greater than split
 *                 j = end_idx-1
 *                 split = self.raw_data[self.raw_indices[j]*self.m+d]             # <<<<<<<<<<<<<<
 *                 for i in range(start_idx, end_idx-1):
 *                     if self.raw_data[self.raw_indices[i]*self.m+d]>split:
 */
      __pyx_v_split = (__pyx_v_self->raw_data[(((__pyx_v_self->raw_indices[__pyx_v_j]) * __pyx_v_self->m) + __pyx_v_d)]);

      /* "pyart/map/ckdtree.pyx":932
 *                 j = end_idx-1
 *                 split = self.raw_data[self.raw_indices[j]*self.m+d]
 *                 for i in range(start_idx, end_idx-1):             # <<<<<<<<<<<<<<
 *                     if self.raw_data[self.raw_indices[i]*self.m+d]>split:
 *                         j = i
 */
      __pyx_t_5 = (__pyx_v_end_idx - 1);
      __pyx_t_6 = __pyx_t_5;
      for (__pyx_t_2 = __pyx_v_start_idx; __pyx_t_2 < __pyx_t_6; __pyx_t_2+=1) {
        __pyx_v_i = __pyx_t_2;

        /* "pyart/map/ckdtree.pyx":933
 *                 split = self.raw_data[self.raw_indices[j]*self.m+d]
 *                 for i in range(start_idx, end_idx-1):
 *                     if self.raw_data[self.raw_indices[i]*self.m+d]>split:             # <<<<<<<<<<<<<<
 *                         j = i
 *                         split = self.raw_data[self.raw_indices[j]*self.m+d]
 */
        __pyx_t_1 = (((__pyx_v_self->raw_data[(((__pyx_v_self->raw_indices[__pyx_v_i]) * __pyx_v_self->m) + __pyx_v_d)]) > __pyx_v_split) != 0);
        if (__pyx_t_1) {

          /* "pyart/map/ckdtree.pyx":934
 *                 for i in range(start_idx, end_idx-1):
 *                     if self.raw_data[self.raw_indices[i]*self.m+d]>split:
 *                         j = i             # <<<<<<<<<<<<<<
 *                         split = self.raw_data[self.raw_indices[j]*self.m+d]
 *                 t = self.raw_indices[end_idx-1]
 */
          __pyx_v_j = __pyx_v_i;

          /* "pyart/map/ckdtree.pyx":935
 *                     if self.raw_data[self.raw_indices[i]*self.m+d]>split:
 *                         j = i
 *                         split = self.raw_data[self.raw_indices[j]*self.m+d]             # <<<<<<<<<<<<<<
 *                 t = self.raw_indices[end_idx-1]
 *                 self.raw_indices[end_idx-1] = self.raw_indices[j]
 */
          __pyx_v_split = (__pyx_v_self->raw_data[(((__pyx_v_self->raw_indices[__pyx_v_j]) * __pyx_v_self->m) + __pyx_v_d)]);

          /* "pyart/map/ckdtree.pyx":933
 *                 split = self.raw_data[self.raw_indices[j]*self.m+d]
 *                 for i in range(start_idx, end_idx-1):
 *                     if self.raw_data[self.raw_indices[i]*self.m+d]>split:             # <<<<<<<<<<<<<<
 *                         j = i
 *                         split = self.raw_data[self.raw_indices[j]*self.m+d]
 */
        }
      }

      /* "pyart/map/ckdtree.pyx":936
 *                         j = i
 *                         split = self.raw_data[self.raw_indices[j]*self.m+d]
 *                 t = self.raw_indices[end_idx-1]             # <<<<<<<<<<<<<<
 *                 self.raw_indices[end_idx-1] = self.raw_indices[j]
 *                 self.raw_indices[j] = t
 */
      __pyx_v_t = (__pyx_v_self->raw_indices[(__pyx_v_end_idx - 1)]);

      /* "pyart/map/ckdtree.pyx":937
 *                         split = self.raw_data[self.raw_indices[j]*self.m+d]
 *                 t = self.raw_indices[end_idx-1]
 *                 self.raw_indices[end_idx-1] = self.raw_indices[j]             # <<<<<<<<<<<<<<
 *                 self.raw_indices[j] = t
 *                 p = end_idx-1
 */
      (__pyx_v_self->raw_indices[(__pyx_v_end_idx - 1)]) = (__pyx_v_self->raw_indices[__pyx_v_j]);

      /* "pyart/map/ckdtree.pyx":938
 *                 t = self.raw_indices[end_idx-1]
 *                 self.raw_indices[end_idx-1] = self.raw_indices[j]
 *                 self.raw_indices[j] = t             # <<<<<<<<<<<<<<
 *                 p = end_idx-1
 *                 q = end_idx-2
 */
      (__pyx_v_self->raw_indices[__pyx_v_j]) = __pyx_v_t;

      /* "pyart/map/ckdtree.pyx":939
 *                 self.raw_indices[end_idx-1] = self.raw_indices[j]
 *                 self.raw_indices[j] = t
 *                 p = end_idx-1             # <<<<<<<<<<<<<<
 *                 q = end_idx-2
 * 
 */
      __pyx_v_p = (__pyx_v_end_idx - 1);

      /* "pyart/map/ckdtree.pyx":940
 *                 self.raw_indices[j] = t
 *                 p = end_idx-1
 *                 q = end_idx-2             # <<<<<<<<<<<<<<
 * 
 *             # construct new node representation
 */
      __pyx_v_q = (__pyx_v_end_idx - 2);

      /* "pyart/map/ckdtree.pyx":928
 *                 p = start_idx+1
 *                 q = start_idx
 *             elif p==end_idx:             # <<<<<<<<<<<<<<
 *                 # no points greater than split
 *                 j = end_idx-1
 */
    }
    __pyx_L13:;

    /* "pyart/map/ckdtree.pyx":943
 * 
 *             # construct new node representation
 *             ni = <innernode*>stdlib.malloc(sizeof(innernode))             # <<<<<<<<<<<<<<
 *             if ni ==  <innernode*> NULL:
 *                 raise MemoryError
 */
    __pyx_v_ni = ((struct __pyx_t_5pyart_3map_7ckdtree_innernode *)malloc((sizeof(struct __pyx_t_5pyart_3map_7ckdtree_innernode))));

    /* "pyart/map/ckdtree.pyx":944
 *             # construct new node representation
 *             ni = <innernode*>stdlib.malloc(sizeof(innernode))
 *             if ni ==  <innernode*> NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError
 * 
 */
    __pyx_t_1 = ((__pyx_v_ni == ((struct __pyx_t_5pyart_3map_7ckdtree_innernode *)NULL)) != 0);
    if (unlikely(__pyx_t_1)) {

      /* "pyart/map/ckdtree.pyx":945
 *             ni = <innernode*>stdlib.malloc(sizeof(innernode))
 *             if ni ==  <innernode*> NULL:
 *                 raise MemoryError             # <<<<<<<<<<<<<<
 * 
 *             try:
 */
      PyErr_NoMemory(); __PYX_ERR(0, 945, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":944
 *             # construct new node representation
 *             ni = <innernode*>stdlib.malloc(sizeof(innernode))
 *             if ni ==  <innernode*> NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError
 * 
 */
    }

    /* "pyart/map/ckdtree.pyx":947
 *                 raise MemoryError
 * 
 *             try:             # <<<<<<<<<<<<<<
 *                 mids = <np.float64_t*>stdlib.malloc(sizeof(np.float64_t)*self.m)
 *                 if mids == <np.float64_t*> NULL:
 */
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_8);
      __Pyx_XGOTREF(__pyx_t_9);
      /*try:*/ {

        /* "pyart/map/ckdtree.pyx":948
 * 
 *             try:
 *                 mids = <np.float64_t*>stdlib.malloc(sizeof(np.float64_t)*self.m)             # <<<<<<<<<<<<<<
 *                 if mids == <np.float64_t*> NULL:
 *                     raise MemoryError
 */
        __pyx_v_mids = ((__pyx_t_5numpy_float64_t *)malloc(((sizeof(__pyx_t_5numpy_float64_t)) * __pyx_v_self->m)));

        /* "pyart/map/ckdtree.pyx":949
 *             try:
 *                 mids = <np.float64_t*>stdlib.malloc(sizeof(np.float64_t)*self.m)
 *                 if mids == <np.float64_t*> NULL:             # <<<<<<<<<<<<<<
 *                     raise MemoryError
 * 
 */
        __pyx_t_1 = ((__pyx_v_mids == ((__pyx_t_5numpy_float64_t *)NULL)) != 0);
        if (unlikely(__pyx_t_1)) {

          /* "pyart/map/ckdtree.pyx":950
 *                 mids = <np.float64_t*>stdlib.malloc(sizeof(np.float64_t)*self.m)
 *                 if mids == <np.float64_t*> NULL:
 *                     raise MemoryError             # <<<<<<<<<<<<<<
 * 
 *                 for i in range(self.m):
 */
          PyErr_NoMemory(); __PYX_ERR(0, 950, __pyx_L21_error)

          /* "pyart/map/ckdtree.pyx":949
 *             try:
 *                 mids = <np.float64_t*>stdlib.malloc(sizeof(np.float64_t)*self.m)
 *                 if mids == <np.float64_t*> NULL:             # <<<<<<<<<<<<<<
 *                     raise MemoryError
 * 
 */
        }

        /* "pyart/map/ckdtree.pyx":952
 *                     raise MemoryError
 * 
 *                 for i in range(self.m):             # <<<<<<<<<<<<<<
 *                     mids[i] = maxes[i]
 *                 mids[d] = split
 */
        __pyx_t_2 = __pyx_v_self->m;
        __pyx_t_3 = __pyx_t_2;
        for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
          __pyx_v_i = __pyx_t_4;

          /* "pyart/map/ckdtree.pyx":953
 * 
 *                 for i in range(self.m):
 *                     mids[i] = maxes[i]             # <<<<<<<<<<<<<<
 *                 mids[d] = split
 *                 ni.less = self.__build(start_idx,p,mids,mins)
 */
          (__pyx_v_mids[__pyx_v_i]) = (__pyx_v_maxes[__pyx_v_i]);
        }

        /* "pyart/map/ckdtree.pyx":954
 *                 for i in range(self.m):
 *                     mids[i] = maxes[i]
 *                 mids[d] = split             # <<<<<<<<<<<<<<
 *                 ni.less = self.__build(start_idx,p,mids,mins)
 * 
 */
        (__pyx_v_mids[__pyx_v_d]) = __pyx_v_split;

        /* "pyart/map/ckdtree.pyx":955
 *                     mids[i] = maxes[i]
 *                 mids[d] = split
 *                 ni.less = self.__build(start_idx,p,mids,mins)             # <<<<<<<<<<<<<<
 * 
 *                 for i in range(self.m):
 */
        __pyx_t_10 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___build(__pyx_v_self, __pyx_v_start_idx, __pyx_v_p, __pyx_v_mids, __pyx_v_mins); if (unlikely(__pyx_t_10 == ((struct __pyx_t_5pyart_3map_7ckdtree_innernode *)((struct __pyx_t_5pyart_3map_7ckdtree_innernode *)NULL)) && PyErr_Occurred())) __PYX_ERR(0, 955, __pyx_L21_error)
        __pyx_v_ni->less = __pyx_t_10;

        /* "pyart/map/ckdtree.pyx":957
 *                 ni.less = self.__build(start_idx,p,mids,mins)
 * 
 *                 for i in range(self.m):             # <<<<<<<<<<<<<<
 *                     mids[i] = mins[i]
 *                 mids[d] = split
 */
        __pyx_t_2 = __pyx_v_self->m;
        __pyx_t_3 = __pyx_t_2;
        for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
          __pyx_v_i = __pyx_t_4;

          /* "pyart/map/ckdtree.pyx":958
 * 
 *                 for i in range(self.m):
 *                     mids[i] = mins[i]             # <<<<<<<<<<<<<<
 *                 mids[d] = split
 *                 ni.greater = self.__build(p,end_idx,maxes,mids)
 */
          (__pyx_v_mids[__pyx_v_i]) = (__pyx_v_mins[__pyx_v_i]);
        }

        /* "pyart/map/ckdtree.pyx":959
 *                 for i in range(self.m):
 *                     mids[i] = mins[i]
 *                 mids[d] = split             # <<<<<<<<<<<<<<
 *                 ni.greater = self.__build(p,end_idx,maxes,mids)
 * 
 */
        (__pyx_v_mids[__pyx_v_d]) = __pyx_v_split;

        /* "pyart/map/ckdtree.pyx":960
 *                     mids[i] = mins[i]
 *                 mids[d] = split
 *                 ni.greater = self.__build(p,end_idx,maxes,mids)             # <<<<<<<<<<<<<<
 * 
 *                 ni.children = ni.less.children + ni.greater.children
 */
        __pyx_t_10 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___build(__pyx_v_self, __pyx_v_p, __pyx_v_end_idx, __pyx_v_maxes, __pyx_v_mids); if (unlikely(__pyx_t_10 == ((struct __pyx_t_5pyart_3map_7ckdtree_innernode *)((struct __pyx_t_5pyart_3map_7ckdtree_innernode *)NULL)) && PyErr_Occurred())) __PYX_ERR(0, 960, __pyx_L21_error)
        __pyx_v_ni->greater = __pyx_t_10;

        /* "pyart/map/ckdtree.pyx":962
 *                 ni.greater = self.__build(p,end_idx,maxes,mids)
 * 
 *                 ni.children = ni.less.children + ni.greater.children             # <<<<<<<<<<<<<<
 * 
 *             except:
 */
        __pyx_v_ni->children = (__pyx_v_ni->less->children + __pyx_v_ni->greater->children);

        /* "pyart/map/ckdtree.pyx":947
 *                 raise MemoryError
 * 
 *             try:             # <<<<<<<<<<<<<<
 *                 mids = <np.float64_t*>stdlib.malloc(sizeof(np.float64_t)*self.m)
 *                 if mids == <np.float64_t*> NULL:
 */
      }

      /* "pyart/map/ckdtree.pyx":972
 *                 raise
 *             else:
 *                 if mids != <np.float64_t*> NULL:             # <<<<<<<<<<<<<<
 *                     stdlib.free(mids)
 * 
 */
      /*else:*/ {
        __pyx_t_1 = ((__pyx_v_mids != ((__pyx_t_5numpy_float64_t *)NULL)) != 0);
        if (__pyx_t_1) {

          /* "pyart/map/ckdtree.pyx":973
 *             else:
 *                 if mids != <np.float64_t*> NULL:
 *                     stdlib.free(mids)             # <<<<<<<<<<<<<<
 * 
 *             ni.split_dim = d
 */
          free(__pyx_v_mids);

          /* "pyart/map/ckdtree.pyx":972
 *                 raise
 *             else:
 *                 if mids != <np.float64_t*> NULL:             # <<<<<<<<<<<<<<
 *                     stdlib.free(mids)
 * 
 */
        }
      }
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L26_try_end;
      __pyx_L21_error:;

      /* "pyart/map/ckdtree.pyx":964
 *                 ni.children = ni.less.children + ni.greater.children
 * 
 *             except:             # <<<<<<<<<<<<<<
 *                 # free ni if it cannot be returned
 *                 if ni !=  <innernode*> NULL:
 */
      /*except:*/ {
        __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.__build", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_11, &__pyx_t_12, &__pyx_t_13) < 0) __PYX_ERR(0, 964, __pyx_L23_except_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_GOTREF(__pyx_t_13);

        /* "pyart/map/ckdtree.pyx":966
 *             except:
 *                 # free ni if it cannot be returned
 *                 if ni !=  <innernode*> NULL:             # <<<<<<<<<<<<<<
 *                     stdlib.free(mids)
 *                 if mids != <np.float64_t*> NULL:
 */
        __pyx_t_1 = ((__pyx_v_ni != ((struct __pyx_t_5pyart_3map_7ckdtree_innernode *)NULL)) != 0);
        if (__pyx_t_1) {

          /* "pyart/map/ckdtree.pyx":967
 *                 # free ni if it cannot be returned
 *                 if ni !=  <innernode*> NULL:
 *                     stdlib.free(mids)             # <<<<<<<<<<<<<<
 *                 if mids != <np.float64_t*> NULL:
 *                     stdlib.free(mids)
 */
          free(__pyx_v_mids);

          /* "pyart/map/ckdtree.pyx":966
 *             except:
 *                 # free ni if it cannot be returned
 *                 if ni !=  <innernode*> NULL:             # <<<<<<<<<<<<<<
 *                     stdlib.free(mids)
 *                 if mids != <np.float64_t*> NULL:
 */
        }

        /* "pyart/map/ckdtree.pyx":968
 *                 if ni !=  <innernode*> NULL:
 *                     stdlib.free(mids)
 *                 if mids != <np.float64_t*> NULL:             # <<<<<<<<<<<<<<
 *                     stdlib.free(mids)
 *                 raise
 */
        __pyx_t_1 = ((__pyx_v_mids != ((__pyx_t_5numpy_float64_t *)NULL)) != 0);
        if (__pyx_t_1) {

          /* "pyart/map/ckdtree.pyx":969
 *                     stdlib.free(mids)
 *                 if mids != <np.float64_t*> NULL:
 *                     stdlib.free(mids)             # <<<<<<<<<<<<<<
 *                 raise
 *             else:
 */
          free(__pyx_v_mids);

          /* "pyart/map/ckdtree.pyx":968
 *                 if ni !=  <innernode*> NULL:
 *                     stdlib.free(mids)
 *                 if mids != <np.float64_t*> NULL:             # <<<<<<<<<<<<<<
 *                     stdlib.free(mids)
 *                 raise
 */
        }

        /* "pyart/map/ckdtree.pyx":970
 *                 if mids != <np.float64_t*> NULL:
 *                     stdlib.free(mids)
 *                 raise             # <<<<<<<<<<<<<<
 *             else:
 *                 if mids != <np.float64_t*> NULL:
 */
        __Pyx_GIVEREF(__pyx_t_11);
        __Pyx_GIVEREF(__pyx_t_12);
        __Pyx_XGIVEREF(__pyx_t_13);
        __Pyx_ErrRestoreWithState(__pyx_t_11, __pyx_t_12, __pyx_t_13);
        __pyx_t_11 = 0; __pyx_t_12 = 0; __pyx_t_13 = 0; 
        __PYX_ERR(0, 970, __pyx_L23_except_error)
      }
      __pyx_L23_except_error:;

      /* "pyart/map/ckdtree.pyx":947
 *                 raise MemoryError
 * 
 *             try:             # <<<<<<<<<<<<<<
 *                 mids = <np.float64_t*>stdlib.malloc(sizeof(np.float64_t)*self.m)
 *                 if mids == <np.float64_t*> NULL:
 */
      __Pyx_XGIVEREF(__pyx_t_7);
      __Pyx_XGIVEREF(__pyx_t_8);
      __Pyx_XGIVEREF(__pyx_t_9);
      __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
      goto __pyx_L1_error;
      __pyx_L26_try_end:;
    }

    /* "pyart/map/ckdtree.pyx":975
 *                     stdlib.free(mids)
 * 
 *             ni.split_dim = d             # <<<<<<<<<<<<<<
 *             ni.split = split
 *             return ni
 */
    __pyx_v_ni->split_dim = __pyx_v_d;

    /* "pyart/map/ckdtree.pyx":976
 * 
 *             ni.split_dim = d
 *             ni.split = split             # <<<<<<<<<<<<<<
 *             return ni
 * 
 */
    __pyx_v_ni->split = __pyx_v_split;

    /* "pyart/map/ckdtree.pyx":977
 *             ni.split_dim = d
 *             ni.split = split
 *             return ni             # <<<<<<<<<<<<<<
 * 
 *     cdef __free_tree(cKDTree self, innernode* node):
 */
    __pyx_r = __pyx_v_ni;
    goto __pyx_L0;
  }

  /* "pyart/map/ckdtree.pyx":862
 *         self.tree = self.__build(0, self.n, self.raw_maxes, self.raw_mins)
 * 
 *     cdef innernode* __build(cKDTree self, np.intp_t start_idx, np.intp_t end_idx,             # <<<<<<<<<<<<<<
 *                             np.float64_t* maxes, np.float64_t* mins) except? <innernode*> NULL:
 *         cdef leafnode* n
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.__build", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = ((struct __pyx_t_5pyart_3map_7ckdtree_innernode *)NULL);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":979
 *             return ni
 * 
 *     cdef __free_tree(cKDTree self, innernode* node):             # <<<<<<<<<<<<<<
 *         if node.split_dim!=-1:
 *             self.__free_tree(node.less)
 */

static PyObject *__pyx_f_5pyart_3map_7ckdtree_7cKDTree___free_tree(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__free_tree", 0);

  /* "pyart/map/ckdtree.pyx":980
 * 
 *     cdef __free_tree(cKDTree self, innernode* node):
 *         if node.split_dim!=-1:             # <<<<<<<<<<<<<<
 *             self.__free_tree(node.less)
 *             self.__free_tree(node.greater)
 */
  __pyx_t_1 = ((__pyx_v_node->split_dim != -1L) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":981
 *     cdef __free_tree(cKDTree self, innernode* node):
 *         if node.split_dim!=-1:
 *             self.__free_tree(node.less)             # <<<<<<<<<<<<<<
 *             self.__free_tree(node.greater)
 *         stdlib.free(node)
 */
    __pyx_t_2 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___free_tree(__pyx_v_self, __pyx_v_node->less); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 981, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyart/map/ckdtree.pyx":982
 *         if node.split_dim!=-1:
 *             self.__free_tree(node.less)
 *             self.__free_tree(node.greater)             # <<<<<<<<<<<<<<
 *         stdlib.free(node)
 * 
 */
    __pyx_t_2 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___free_tree(__pyx_v_self, __pyx_v_node->greater); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 982, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyart/map/ckdtree.pyx":980
 * 
 *     cdef __free_tree(cKDTree self, innernode* node):
 *         if node.split_dim!=-1:             # <<<<<<<<<<<<<<
 *             self.__free_tree(node.less)
 *             self.__free_tree(node.greater)
 */
  }

  /* "pyart/map/ckdtree.pyx":983
 *             self.__free_tree(node.less)
 *             self.__free_tree(node.greater)
 *         stdlib.free(node)             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(cKDTree self):
 */
  free(__pyx_v_node);

  /* "pyart/map/ckdtree.pyx":979
 *             return ni
 * 
 *     cdef __free_tree(cKDTree self, innernode* node):             # <<<<<<<<<<<<<<
 *         if node.split_dim!=-1:
 *             self.__free_tree(node.less)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.__free_tree", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":985
 *         stdlib.free(node)
 * 
 *     def __dealloc__(cKDTree self):             # <<<<<<<<<<<<<<
 *         if <np.intp_t>(self.tree) == 0:
 *             # should happen only if __init__ was never called
 */

/* Python wrapper */
static void __pyx_pw_5pyart_3map_7ckdtree_7cKDTree_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5pyart_3map_7ckdtree_7cKDTree_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5pyart_3map_7ckdtree_7cKDTree_2__dealloc__(((struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5pyart_3map_7ckdtree_7cKDTree_2__dealloc__(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "pyart/map/ckdtree.pyx":986
 * 
 *     def __dealloc__(cKDTree self):
 *         if <np.intp_t>(self.tree) == 0:             # <<<<<<<<<<<<<<
 *             # should happen only if __init__ was never called
 *             return
 */
  __pyx_t_1 = ((((__pyx_t_5numpy_intp_t)__pyx_v_self->tree) == 0) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":988
 *         if <np.intp_t>(self.tree) == 0:
 *             # should happen only if __init__ was never called
 *             return             # <<<<<<<<<<<<<<
 *         self.__free_tree(self.tree)
 * 
 */
    goto __pyx_L0;

    /* "pyart/map/ckdtree.pyx":986
 * 
 *     def __dealloc__(cKDTree self):
 *         if <np.intp_t>(self.tree) == 0:             # <<<<<<<<<<<<<<
 *             # should happen only if __init__ was never called
 *             return
 */
  }

  /* "pyart/map/ckdtree.pyx":989
 *             # should happen only if __init__ was never called
 *             return
 *         self.__free_tree(self.tree)             # <<<<<<<<<<<<<<
 * 
 *     # -----
 */
  __pyx_t_2 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___free_tree(__pyx_v_self, __pyx_v_self->tree); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 989, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyart/map/ckdtree.pyx":985
 *         stdlib.free(node)
 * 
 *     def __dealloc__(cKDTree self):             # <<<<<<<<<<<<<<
 *         if <np.intp_t>(self.tree) == 0:
 *             # should happen only if __init__ was never called
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_WriteUnraisable("pyart.map.ckdtree.cKDTree.__dealloc__", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pyart/map/ckdtree.pyx":995
 *     # -----
 * 
 *     cdef int __query(cKDTree self,             # <<<<<<<<<<<<<<
 *             np.float64_t*result_distances,
 *             np.intp_t*result_indices,
 */

static int __pyx_f_5pyart_3map_7ckdtree_7cKDTree___query(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, __pyx_t_5numpy_float64_t *__pyx_v_result_distances, __pyx_t_5numpy_intp_t *__pyx_v_result_indices, __pyx_t_5numpy_float64_t *__pyx_v_x, __pyx_t_5numpy_intp_t __pyx_v_k, __pyx_t_5numpy_float64_t __pyx_v_eps, __pyx_t_5numpy_float64_t __pyx_v_p, __pyx_t_5numpy_float64_t __pyx_v_distance_upper_bound) {
  struct __pyx_obj_5pyart_3map_7ckdtree_heap *__pyx_v_q = 0;
  struct __pyx_obj_5pyart_3map_7ckdtree_heap *__pyx_v_neighbors = 0;
  __pyx_t_5numpy_intp_t __pyx_v_i;
  __pyx_t_5numpy_float64_t __pyx_v_t;
  struct __pyx_t_5pyart_3map_7ckdtree_nodeinfo *__pyx_v_inf;
  struct __pyx_t_5pyart_3map_7ckdtree_nodeinfo *__pyx_v_inf2;
  __pyx_t_5numpy_float64_t __pyx_v_d;
  __pyx_t_5numpy_float64_t __pyx_v_epsfac;
  __pyx_t_5numpy_float64_t __pyx_v_min_distance;
  __pyx_t_5numpy_float64_t __pyx_v_far_min_distance;
  struct __pyx_t_5pyart_3map_7ckdtree_heapitem __pyx_v_it;
  struct __pyx_t_5pyart_3map_7ckdtree_heapitem __pyx_v_it2;
  struct __pyx_t_5pyart_3map_7ckdtree_heapitem __pyx_v_neighbor;
  struct __pyx_t_5pyart_3map_7ckdtree_leafnode *__pyx_v_node;
  struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_inode;
  struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_near;
  struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_far;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_t_4;
  __pyx_t_5numpy_intp_t __pyx_t_5;
  __pyx_t_5numpy_intp_t __pyx_t_6;
  __pyx_t_5numpy_intp_t __pyx_t_7;
  int __pyx_t_8;
  __pyx_t_5numpy_float64_t __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  char const *__pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__query", 0);

  /* "pyart/map/ckdtree.pyx":1028
 *         #  distances between the nearest side of the cell and the target
 *         #  the head node of the cell
 *         q = heap(12)             # <<<<<<<<<<<<<<
 * 
 *         # priority queue for the nearest neighbors
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pyart_3map_7ckdtree_heap), __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1028, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_q = ((struct __pyx_obj_5pyart_3map_7ckdtree_heap *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/map/ckdtree.pyx":1033
 *         # furthest known neighbor first
 *         # entries are (-distance**p, i)
 *         neighbors = heap(k)             # <<<<<<<<<<<<<<
 * 
 *         inf = inf2 = <nodeinfo*> NULL
 */
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_k); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1033, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_5pyart_3map_7ckdtree_heap), __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1033, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_neighbors = ((struct __pyx_obj_5pyart_3map_7ckdtree_heap *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyart/map/ckdtree.pyx":1035
 *         neighbors = heap(k)
 * 
 *         inf = inf2 = <nodeinfo*> NULL             # <<<<<<<<<<<<<<
 * 
 *         try:
 */
  __pyx_v_inf = ((struct __pyx_t_5pyart_3map_7ckdtree_nodeinfo *)NULL);
  __pyx_v_inf2 = ((struct __pyx_t_5pyart_3map_7ckdtree_nodeinfo *)NULL);

  /* "pyart/map/ckdtree.pyx":1037
 *         inf = inf2 = <nodeinfo*> NULL
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             # set up first nodeinfo
 *             inf = <nodeinfo*>stdlib.malloc(sizeof(nodeinfo)+self.m*sizeof(np.float64_t))
 */
  /*try:*/ {

    /* "pyart/map/ckdtree.pyx":1039
 *         try:
 *             # set up first nodeinfo
 *             inf = <nodeinfo*>stdlib.malloc(sizeof(nodeinfo)+self.m*sizeof(np.float64_t))             # <<<<<<<<<<<<<<
 *             if inf == <nodeinfo*> NULL:
 *                 raise MemoryError
 */
    __pyx_v_inf = ((struct __pyx_t_5pyart_3map_7ckdtree_nodeinfo *)malloc(((sizeof(struct __pyx_t_5pyart_3map_7ckdtree_nodeinfo)) + (__pyx_v_self->m * (sizeof(__pyx_t_5numpy_float64_t))))));

    /* "pyart/map/ckdtree.pyx":1040
 *             # set up first nodeinfo
 *             inf = <nodeinfo*>stdlib.malloc(sizeof(nodeinfo)+self.m*sizeof(np.float64_t))
 *             if inf == <nodeinfo*> NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError
 *             inf.node = self.tree
 */
    __pyx_t_3 = ((__pyx_v_inf == ((struct __pyx_t_5pyart_3map_7ckdtree_nodeinfo *)NULL)) != 0);
    if (unlikely(__pyx_t_3)) {

      /* "pyart/map/ckdtree.pyx":1041
 *             inf = <nodeinfo*>stdlib.malloc(sizeof(nodeinfo)+self.m*sizeof(np.float64_t))
 *             if inf == <nodeinfo*> NULL:
 *                 raise MemoryError             # <<<<<<<<<<<<<<
 *             inf.node = self.tree
 *             for i in range(self.m):
 */
      PyErr_NoMemory(); __PYX_ERR(0, 1041, __pyx_L4_error)

      /* "pyart/map/ckdtree.pyx":1040
 *             # set up first nodeinfo
 *             inf = <nodeinfo*>stdlib.malloc(sizeof(nodeinfo)+self.m*sizeof(np.float64_t))
 *             if inf == <nodeinfo*> NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError
 *             inf.node = self.tree
 */
    }

    /* "pyart/map/ckdtree.pyx":1042
 *             if inf == <nodeinfo*> NULL:
 *                 raise MemoryError
 *             inf.node = self.tree             # <<<<<<<<<<<<<<
 *             for i in range(self.m):
 *                 inf.side_distances[i] = 0
 */
    __pyx_t_4 = __pyx_v_self->tree;
    __pyx_v_inf->node = __pyx_t_4;

    /* "pyart/map/ckdtree.pyx":1043
 *                 raise MemoryError
 *             inf.node = self.tree
 *             for i in range(self.m):             # <<<<<<<<<<<<<<
 *                 inf.side_distances[i] = 0
 *                 t = x[i]-self.raw_maxes[i]
 */
    __pyx_t_5 = __pyx_v_self->m;
    __pyx_t_6 = __pyx_t_5;
    for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
      __pyx_v_i = __pyx_t_7;

      /* "pyart/map/ckdtree.pyx":1044
 *             inf.node = self.tree
 *             for i in range(self.m):
 *                 inf.side_distances[i] = 0             # <<<<<<<<<<<<<<
 *                 t = x[i]-self.raw_maxes[i]
 *                 if t>inf.side_distances[i]:
 */
      (__pyx_v_inf->side_distances[__pyx_v_i]) = 0.0;

      /* "pyart/map/ckdtree.pyx":1045
 *             for i in range(self.m):
 *                 inf.side_distances[i] = 0
 *                 t = x[i]-self.raw_maxes[i]             # <<<<<<<<<<<<<<
 *                 if t>inf.side_distances[i]:
 *                     inf.side_distances[i] = t
 */
      __pyx_v_t = ((__pyx_v_x[__pyx_v_i]) - (__pyx_v_self->raw_maxes[__pyx_v_i]));

      /* "pyart/map/ckdtree.pyx":1046
 *                 inf.side_distances[i] = 0
 *                 t = x[i]-self.raw_maxes[i]
 *                 if t>inf.side_distances[i]:             # <<<<<<<<<<<<<<
 *                     inf.side_distances[i] = t
 *                 else:
 */
      __pyx_t_3 = ((__pyx_v_t > (__pyx_v_inf->side_distances[__pyx_v_i])) != 0);
      if (__pyx_t_3) {

        /* "pyart/map/ckdtree.pyx":1047
 *                 t = x[i]-self.raw_maxes[i]
 *                 if t>inf.side_distances[i]:
 *                     inf.side_distances[i] = t             # <<<<<<<<<<<<<<
 *                 else:
 *                     t = self.raw_mins[i]-x[i]
 */
        (__pyx_v_inf->side_distances[__pyx_v_i]) = __pyx_v_t;

        /* "pyart/map/ckdtree.pyx":1046
 *                 inf.side_distances[i] = 0
 *                 t = x[i]-self.raw_maxes[i]
 *                 if t>inf.side_distances[i]:             # <<<<<<<<<<<<<<
 *                     inf.side_distances[i] = t
 *                 else:
 */
        goto __pyx_L9;
      }

      /* "pyart/map/ckdtree.pyx":1049
 *                     inf.side_distances[i] = t
 *                 else:
 *                     t = self.raw_mins[i]-x[i]             # <<<<<<<<<<<<<<
 *                     if t>inf.side_distances[i]:
 *                         inf.side_distances[i] = t
 */
      /*else*/ {
        __pyx_v_t = ((__pyx_v_self->raw_mins[__pyx_v_i]) - (__pyx_v_x[__pyx_v_i]));

        /* "pyart/map/ckdtree.pyx":1050
 *                 else:
 *                     t = self.raw_mins[i]-x[i]
 *                     if t>inf.side_distances[i]:             # <<<<<<<<<<<<<<
 *                         inf.side_distances[i] = t
 *                 if p!=1 and p!=infinity:
 */
        __pyx_t_3 = ((__pyx_v_t > (__pyx_v_inf->side_distances[__pyx_v_i])) != 0);
        if (__pyx_t_3) {

          /* "pyart/map/ckdtree.pyx":1051
 *                     t = self.raw_mins[i]-x[i]
 *                     if t>inf.side_distances[i]:
 *                         inf.side_distances[i] = t             # <<<<<<<<<<<<<<
 *                 if p!=1 and p!=infinity:
 *                     inf.side_distances[i]=inf.side_distances[i]**p
 */
          (__pyx_v_inf->side_distances[__pyx_v_i]) = __pyx_v_t;

          /* "pyart/map/ckdtree.pyx":1050
 *                 else:
 *                     t = self.raw_mins[i]-x[i]
 *                     if t>inf.side_distances[i]:             # <<<<<<<<<<<<<<
 *                         inf.side_distances[i] = t
 *                 if p!=1 and p!=infinity:
 */
        }
      }
      __pyx_L9:;

      /* "pyart/map/ckdtree.pyx":1052
 *                     if t>inf.side_distances[i]:
 *                         inf.side_distances[i] = t
 *                 if p!=1 and p!=infinity:             # <<<<<<<<<<<<<<
 *                     inf.side_distances[i]=inf.side_distances[i]**p
 * 
 */
      __pyx_t_8 = ((__pyx_v_p != 1.0) != 0);
      if (__pyx_t_8) {
      } else {
        __pyx_t_3 = __pyx_t_8;
        goto __pyx_L12_bool_binop_done;
      }
      __pyx_t_8 = ((__pyx_v_p != __pyx_v_5pyart_3map_7ckdtree_infinity) != 0);
      __pyx_t_3 = __pyx_t_8;
      __pyx_L12_bool_binop_done:;
      if (__pyx_t_3) {

        /* "pyart/map/ckdtree.pyx":1053
 *                         inf.side_distances[i] = t
 *                 if p!=1 and p!=infinity:
 *                     inf.side_distances[i]=inf.side_distances[i]**p             # <<<<<<<<<<<<<<
 * 
 *             # compute first distance
 */
        (__pyx_v_inf->side_distances[__pyx_v_i]) = pow((__pyx_v_inf->side_distances[__pyx_v_i]), __pyx_v_p);

        /* "pyart/map/ckdtree.pyx":1052
 *                     if t>inf.side_distances[i]:
 *                         inf.side_distances[i] = t
 *                 if p!=1 and p!=infinity:             # <<<<<<<<<<<<<<
 *                     inf.side_distances[i]=inf.side_distances[i]**p
 * 
 */
      }
    }

    /* "pyart/map/ckdtree.pyx":1056
 * 
 *             # compute first distance
 *             min_distance = 0.             # <<<<<<<<<<<<<<
 *             for i in range(self.m):
 *                 if p==infinity:
 */
    __pyx_v_min_distance = 0.;

    /* "pyart/map/ckdtree.pyx":1057
 *             # compute first distance
 *             min_distance = 0.
 *             for i in range(self.m):             # <<<<<<<<<<<<<<
 *                 if p==infinity:
 *                     min_distance = dmax(min_distance,inf.side_distances[i])
 */
    __pyx_t_5 = __pyx_v_self->m;
    __pyx_t_6 = __pyx_t_5;
    for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
      __pyx_v_i = __pyx_t_7;

      /* "pyart/map/ckdtree.pyx":1058
 *             min_distance = 0.
 *             for i in range(self.m):
 *                 if p==infinity:             # <<<<<<<<<<<<<<
 *                     min_distance = dmax(min_distance,inf.side_distances[i])
 *                 else:
 */
      __pyx_t_3 = ((__pyx_v_p == __pyx_v_5pyart_3map_7ckdtree_infinity) != 0);
      if (__pyx_t_3) {

        /* "pyart/map/ckdtree.pyx":1059
 *             for i in range(self.m):
 *                 if p==infinity:
 *                     min_distance = dmax(min_distance,inf.side_distances[i])             # <<<<<<<<<<<<<<
 *                 else:
 *                     min_distance += inf.side_distances[i]
 */
        __pyx_v_min_distance = __pyx_f_5pyart_3map_7ckdtree_dmax(__pyx_v_min_distance, (__pyx_v_inf->side_distances[__pyx_v_i]));

        /* "pyart/map/ckdtree.pyx":1058
 *             min_distance = 0.
 *             for i in range(self.m):
 *                 if p==infinity:             # <<<<<<<<<<<<<<
 *                     min_distance = dmax(min_distance,inf.side_distances[i])
 *                 else:
 */
        goto __pyx_L16;
      }

      /* "pyart/map/ckdtree.pyx":1061
 *                     min_distance = dmax(min_distance,inf.side_distances[i])
 *                 else:
 *                     min_distance += inf.side_distances[i]             # <<<<<<<<<<<<<<
 * 
 *             # fiddle approximation factor
 */
      /*else*/ {
        __pyx_v_min_distance = (__pyx_v_min_distance + (__pyx_v_inf->side_distances[__pyx_v_i]));
      }
      __pyx_L16:;
    }

    /* "pyart/map/ckdtree.pyx":1064
 * 
 *             # fiddle approximation factor
 *             if eps==0:             # <<<<<<<<<<<<<<
 *                 epsfac=1
 *             elif p==infinity:
 */
    __pyx_t_3 = ((__pyx_v_eps == 0.0) != 0);
    if (__pyx_t_3) {

      /* "pyart/map/ckdtree.pyx":1065
 *             # fiddle approximation factor
 *             if eps==0:
 *                 epsfac=1             # <<<<<<<<<<<<<<
 *             elif p==infinity:
 *                 epsfac = 1/(1+eps)
 */
      __pyx_v_epsfac = 1.0;

      /* "pyart/map/ckdtree.pyx":1064
 * 
 *             # fiddle approximation factor
 *             if eps==0:             # <<<<<<<<<<<<<<
 *                 epsfac=1
 *             elif p==infinity:
 */
      goto __pyx_L17;
    }

    /* "pyart/map/ckdtree.pyx":1066
 *             if eps==0:
 *                 epsfac=1
 *             elif p==infinity:             # <<<<<<<<<<<<<<
 *                 epsfac = 1/(1+eps)
 *             else:
 */
    __pyx_t_3 = ((__pyx_v_p == __pyx_v_5pyart_3map_7ckdtree_infinity) != 0);
    if (__pyx_t_3) {

      /* "pyart/map/ckdtree.pyx":1067
 *                 epsfac=1
 *             elif p==infinity:
 *                 epsfac = 1/(1+eps)             # <<<<<<<<<<<<<<
 *             else:
 *                 epsfac = 1/(1+eps)**p
 */
      __pyx_t_9 = (1.0 + __pyx_v_eps);
      if (unlikely(__pyx_t_9 == 0)) {
        PyErr_SetString(PyExc_ZeroDivisionError, "float division");
        __PYX_ERR(0, 1067, __pyx_L4_error)
      }
      __pyx_v_epsfac = (1.0 / __pyx_t_9);

      /* "pyart/map/ckdtree.pyx":1066
 *             if eps==0:
 *                 epsfac=1
 *             elif p==infinity:             # <<<<<<<<<<<<<<
 *                 epsfac = 1/(1+eps)
 *             else:
 */
      goto __pyx_L17;
    }

    /* "pyart/map/ckdtree.pyx":1069
 *                 epsfac = 1/(1+eps)
 *             else:
 *                 epsfac = 1/(1+eps)**p             # <<<<<<<<<<<<<<
 * 
 *             # internally we represent all distances as distance**p
 */
    /*else*/ {
      __pyx_t_9 = pow((1.0 + __pyx_v_eps), __pyx_v_p);
      if (unlikely(__pyx_t_9 == 0)) {
        PyErr_SetString(PyExc_ZeroDivisionError, "float division");
        __PYX_ERR(0, 1069, __pyx_L4_error)
      }
      __pyx_v_epsfac = (1.0 / __pyx_t_9);
    }
    __pyx_L17:;

    /* "pyart/map/ckdtree.pyx":1072
 * 
 *             # internally we represent all distances as distance**p
 *             if p!=infinity and distance_upper_bound!=infinity:             # <<<<<<<<<<<<<<
 *                 distance_upper_bound = distance_upper_bound**p
 * 
 */
    __pyx_t_8 = ((__pyx_v_p != __pyx_v_5pyart_3map_7ckdtree_infinity) != 0);
    if (__pyx_t_8) {
    } else {
      __pyx_t_3 = __pyx_t_8;
      goto __pyx_L19_bool_binop_done;
    }
    __pyx_t_8 = ((__pyx_v_distance_upper_bound != __pyx_v_5pyart_3map_7ckdtree_infinity) != 0);
    __pyx_t_3 = __pyx_t_8;
    __pyx_L19_bool_binop_done:;
    if (__pyx_t_3) {

      /* "pyart/map/ckdtree.pyx":1073
 *             # internally we represent all distances as distance**p
 *             if p!=infinity and distance_upper_bound!=infinity:
 *                 distance_upper_bound = distance_upper_bound**p             # <<<<<<<<<<<<<<
 * 
 *             while True:
 */
      __pyx_v_distance_upper_bound = pow(__pyx_v_distance_upper_bound, __pyx_v_p);

      /* "pyart/map/ckdtree.pyx":1072
 * 
 *             # internally we represent all distances as distance**p
 *             if p!=infinity and distance_upper_bound!=infinity:             # <<<<<<<<<<<<<<
 *                 distance_upper_bound = distance_upper_bound**p
 * 
 */
    }

    /* "pyart/map/ckdtree.pyx":1075
 *                 distance_upper_bound = distance_upper_bound**p
 * 
 *             while True:             # <<<<<<<<<<<<<<
 *                 if inf.node.split_dim==-1:
 *                     node = <leafnode*>inf.node
 */
    while (1) {

      /* "pyart/map/ckdtree.pyx":1076
 * 
 *             while True:
 *                 if inf.node.split_dim==-1:             # <<<<<<<<<<<<<<
 *                     node = <leafnode*>inf.node
 * 
 */
      __pyx_t_3 = ((__pyx_v_inf->node->split_dim == -1L) != 0);
      if (__pyx_t_3) {

        /* "pyart/map/ckdtree.pyx":1077
 *             while True:
 *                 if inf.node.split_dim==-1:
 *                     node = <leafnode*>inf.node             # <<<<<<<<<<<<<<
 * 
 *                     # brute-force
 */
        __pyx_v_node = ((struct __pyx_t_5pyart_3map_7ckdtree_leafnode *)__pyx_v_inf->node);

        /* "pyart/map/ckdtree.pyx":1080
 * 
 *                     # brute-force
 *                     for i in range(node.start_idx,node.end_idx):             # <<<<<<<<<<<<<<
 *                         d = _distance_p(
 *                                 self.raw_data+self.raw_indices[i]*self.m,
 */
        __pyx_t_5 = __pyx_v_node->end_idx;
        __pyx_t_6 = __pyx_t_5;
        for (__pyx_t_7 = __pyx_v_node->start_idx; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
          __pyx_v_i = __pyx_t_7;

          /* "pyart/map/ckdtree.pyx":1081
 *                     # brute-force
 *                     for i in range(node.start_idx,node.end_idx):
 *                         d = _distance_p(             # <<<<<<<<<<<<<<
 *                                 self.raw_data+self.raw_indices[i]*self.m,
 *                                 x,p,self.m,distance_upper_bound)
 */
          __pyx_v_d = __pyx_f_5pyart_3map_7ckdtree__distance_p((__pyx_v_self->raw_data + ((__pyx_v_self->raw_indices[__pyx_v_i]) * __pyx_v_self->m)), __pyx_v_x, __pyx_v_p, __pyx_v_self->m, __pyx_v_distance_upper_bound);

          /* "pyart/map/ckdtree.pyx":1085
 *                                 x,p,self.m,distance_upper_bound)
 * 
 *                         if d<distance_upper_bound:             # <<<<<<<<<<<<<<
 *                             # replace furthest neighbor
 *                             if neighbors.n==k:
 */
          __pyx_t_3 = ((__pyx_v_d < __pyx_v_distance_upper_bound) != 0);
          if (__pyx_t_3) {

            /* "pyart/map/ckdtree.pyx":1087
 *                         if d<distance_upper_bound:
 *                             # replace furthest neighbor
 *                             if neighbors.n==k:             # <<<<<<<<<<<<<<
 *                                 neighbors.remove()
 *                             neighbor.priority = -d
 */
            __pyx_t_3 = ((__pyx_v_neighbors->n == __pyx_v_k) != 0);
            if (__pyx_t_3) {

              /* "pyart/map/ckdtree.pyx":1088
 *                             # replace furthest neighbor
 *                             if neighbors.n==k:
 *                                 neighbors.remove()             # <<<<<<<<<<<<<<
 *                             neighbor.priority = -d
 *                             neighbor.contents.intdata = self.raw_indices[i]
 */
              __pyx_t_10 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_heap *)__pyx_v_neighbors->__pyx_vtab)->remove(__pyx_v_neighbors); if (unlikely(__pyx_t_10 == ((int)-1))) __PYX_ERR(0, 1088, __pyx_L4_error)

              /* "pyart/map/ckdtree.pyx":1087
 *                         if d<distance_upper_bound:
 *                             # replace furthest neighbor
 *                             if neighbors.n==k:             # <<<<<<<<<<<<<<
 *                                 neighbors.remove()
 *                             neighbor.priority = -d
 */
            }

            /* "pyart/map/ckdtree.pyx":1089
 *                             if neighbors.n==k:
 *                                 neighbors.remove()
 *                             neighbor.priority = -d             # <<<<<<<<<<<<<<
 *                             neighbor.contents.intdata = self.raw_indices[i]
 *                             neighbors.push(neighbor)
 */
            __pyx_v_neighbor.priority = (-__pyx_v_d);

            /* "pyart/map/ckdtree.pyx":1090
 *                                 neighbors.remove()
 *                             neighbor.priority = -d
 *                             neighbor.contents.intdata = self.raw_indices[i]             # <<<<<<<<<<<<<<
 *                             neighbors.push(neighbor)
 * 
 */
            __pyx_v_neighbor.contents.intdata = (__pyx_v_self->raw_indices[__pyx_v_i]);

            /* "pyart/map/ckdtree.pyx":1091
 *                             neighbor.priority = -d
 *                             neighbor.contents.intdata = self.raw_indices[i]
 *                             neighbors.push(neighbor)             # <<<<<<<<<<<<<<
 * 
 *                             # adjust upper bound for efficiency
 */
            __pyx_t_10 = __pyx_f_5pyart_3map_7ckdtree_4heap_push(__pyx_v_neighbors, __pyx_v_neighbor); if (unlikely(__pyx_t_10 == ((int)-1))) __PYX_ERR(0, 1091, __pyx_L4_error)

            /* "pyart/map/ckdtree.pyx":1094
 * 
 *                             # adjust upper bound for efficiency
 *                             if neighbors.n==k:             # <<<<<<<<<<<<<<
 *                                 distance_upper_bound = -neighbors.peek().priority
 * 
 */
            __pyx_t_3 = ((__pyx_v_neighbors->n == __pyx_v_k) != 0);
            if (__pyx_t_3) {

              /* "pyart/map/ckdtree.pyx":1095
 *                             # adjust upper bound for efficiency
 *                             if neighbors.n==k:
 *                                 distance_upper_bound = -neighbors.peek().priority             # <<<<<<<<<<<<<<
 * 
 *                     # done with this node, get another
 */
              __pyx_v_distance_upper_bound = (-((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_heap *)__pyx_v_neighbors->__pyx_vtab)->peek(__pyx_v_neighbors).priority);

              /* "pyart/map/ckdtree.pyx":1094
 * 
 *                             # adjust upper bound for efficiency
 *                             if neighbors.n==k:             # <<<<<<<<<<<<<<
 *                                 distance_upper_bound = -neighbors.peek().priority
 * 
 */
            }

            /* "pyart/map/ckdtree.pyx":1085
 *                                 x,p,self.m,distance_upper_bound)
 * 
 *                         if d<distance_upper_bound:             # <<<<<<<<<<<<<<
 *                             # replace furthest neighbor
 *                             if neighbors.n==k:
 */
          }
        }

        /* "pyart/map/ckdtree.pyx":1098
 * 
 *                     # done with this node, get another
 *                     stdlib.free(inf)             # <<<<<<<<<<<<<<
 *                     inf = <nodeinfo*> NULL
 * 
 */
        free(__pyx_v_inf);

        /* "pyart/map/ckdtree.pyx":1099
 *                     # done with this node, get another
 *                     stdlib.free(inf)
 *                     inf = <nodeinfo*> NULL             # <<<<<<<<<<<<<<
 * 
 *                     if q.n==0:
 */
        __pyx_v_inf = ((struct __pyx_t_5pyart_3map_7ckdtree_nodeinfo *)NULL);

        /* "pyart/map/ckdtree.pyx":1101
 *                     inf = <nodeinfo*> NULL
 * 
 *                     if q.n==0:             # <<<<<<<<<<<<<<
 *                         # no more nodes to visit
 *                         break
 */
        __pyx_t_3 = ((__pyx_v_q->n == 0) != 0);
        if (__pyx_t_3) {

          /* "pyart/map/ckdtree.pyx":1103
 *                     if q.n==0:
 *                         # no more nodes to visit
 *                         break             # <<<<<<<<<<<<<<
 *                     else:
 *                         q.pop(&it)
 */
          goto __pyx_L22_break;

          /* "pyart/map/ckdtree.pyx":1101
 *                     inf = <nodeinfo*> NULL
 * 
 *                     if q.n==0:             # <<<<<<<<<<<<<<
 *                         # no more nodes to visit
 *                         break
 */
        }

        /* "pyart/map/ckdtree.pyx":1105
 *                         break
 *                     else:
 *                         q.pop(&it)             # <<<<<<<<<<<<<<
 *                         inf = <nodeinfo*>it.contents.ptrdata
 *                         min_distance = it.priority
 */
        /*else*/ {
          __pyx_t_10 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_heap *)__pyx_v_q->__pyx_vtab)->pop(__pyx_v_q, (&__pyx_v_it)); if (unlikely(__pyx_t_10 == ((int)-1))) __PYX_ERR(0, 1105, __pyx_L4_error)

          /* "pyart/map/ckdtree.pyx":1106
 *                     else:
 *                         q.pop(&it)
 *                         inf = <nodeinfo*>it.contents.ptrdata             # <<<<<<<<<<<<<<
 *                         min_distance = it.priority
 *                 else:
 */
          __pyx_v_inf = ((struct __pyx_t_5pyart_3map_7ckdtree_nodeinfo *)__pyx_v_it.contents.ptrdata);

          /* "pyart/map/ckdtree.pyx":1107
 *                         q.pop(&it)
 *                         inf = <nodeinfo*>it.contents.ptrdata
 *                         min_distance = it.priority             # <<<<<<<<<<<<<<
 *                 else:
 *                     inode = <innernode*>inf.node
 */
          __pyx_t_9 = __pyx_v_it.priority;
          __pyx_v_min_distance = __pyx_t_9;
        }

        /* "pyart/map/ckdtree.pyx":1076
 * 
 *             while True:
 *                 if inf.node.split_dim==-1:             # <<<<<<<<<<<<<<
 *                     node = <leafnode*>inf.node
 * 
 */
        goto __pyx_L23;
      }

      /* "pyart/map/ckdtree.pyx":1109
 *                         min_distance = it.priority
 *                 else:
 *                     inode = <innernode*>inf.node             # <<<<<<<<<<<<<<
 * 
 *                     # we don't push cells that are too far onto the queue at all,
 */
      /*else*/ {
        __pyx_v_inode = ((struct __pyx_t_5pyart_3map_7ckdtree_innernode *)__pyx_v_inf->node);

        /* "pyart/map/ckdtree.pyx":1114
 *                     # but since the distance_upper_bound decreases, we might get
 *                     # here even if the cell's too far
 *                     if min_distance>distance_upper_bound*epsfac:             # <<<<<<<<<<<<<<
 * 
 *                         # since this is the nearest cell, we're done, bail out
 */
        __pyx_t_3 = ((__pyx_v_min_distance > (__pyx_v_distance_upper_bound * __pyx_v_epsfac)) != 0);
        if (__pyx_t_3) {

          /* "pyart/map/ckdtree.pyx":1117
 * 
 *                         # since this is the nearest cell, we're done, bail out
 *                         stdlib.free(inf)             # <<<<<<<<<<<<<<
 *                         inf = <nodeinfo*> NULL
 * 
 */
          free(__pyx_v_inf);

          /* "pyart/map/ckdtree.pyx":1118
 *                         # since this is the nearest cell, we're done, bail out
 *                         stdlib.free(inf)
 *                         inf = <nodeinfo*> NULL             # <<<<<<<<<<<<<<
 * 
 *                         # free all the nodes still on the heap
 */
          __pyx_v_inf = ((struct __pyx_t_5pyart_3map_7ckdtree_nodeinfo *)NULL);

          /* "pyart/map/ckdtree.pyx":1121
 * 
 *                         # free all the nodes still on the heap
 *                         for i in range(q.n):             # <<<<<<<<<<<<<<
 *                             stdlib.free(q.heap[i].contents.ptrdata)
 *                             q.heap[i].contents.ptrdata = <char*> NULL
 */
          __pyx_t_5 = __pyx_v_q->n;
          __pyx_t_6 = __pyx_t_5;
          for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
            __pyx_v_i = __pyx_t_7;

            /* "pyart/map/ckdtree.pyx":1122
 *                         # free all the nodes still on the heap
 *                         for i in range(q.n):
 *                             stdlib.free(q.heap[i].contents.ptrdata)             # <<<<<<<<<<<<<<
 *                             q.heap[i].contents.ptrdata = <char*> NULL
 *                         break
 */
            free((__pyx_v_q->heap[__pyx_v_i]).contents.ptrdata);

            /* "pyart/map/ckdtree.pyx":1123
 *                         for i in range(q.n):
 *                             stdlib.free(q.heap[i].contents.ptrdata)
 *                             q.heap[i].contents.ptrdata = <char*> NULL             # <<<<<<<<<<<<<<
 *                         break
 * 
 */
            (__pyx_v_q->heap[__pyx_v_i]).contents.ptrdata = ((char *)NULL);
          }

          /* "pyart/map/ckdtree.pyx":1124
 *                             stdlib.free(q.heap[i].contents.ptrdata)
 *                             q.heap[i].contents.ptrdata = <char*> NULL
 *                         break             # <<<<<<<<<<<<<<
 * 
 *                     # set up children for searching
 */
          goto __pyx_L22_break;

          /* "pyart/map/ckdtree.pyx":1114
 *                     # but since the distance_upper_bound decreases, we might get
 *                     # here even if the cell's too far
 *                     if min_distance>distance_upper_bound*epsfac:             # <<<<<<<<<<<<<<
 * 
 *                         # since this is the nearest cell, we're done, bail out
 */
        }

        /* "pyart/map/ckdtree.pyx":1127
 * 
 *                     # set up children for searching
 *                     if x[inode.split_dim]<inode.split:             # <<<<<<<<<<<<<<
 *                         near = inode.less
 *                         far = inode.greater
 */
        __pyx_t_3 = (((__pyx_v_x[__pyx_v_inode->split_dim]) < __pyx_v_inode->split) != 0);
        if (__pyx_t_3) {

          /* "pyart/map/ckdtree.pyx":1128
 *                     # set up children for searching
 *                     if x[inode.split_dim]<inode.split:
 *                         near = inode.less             # <<<<<<<<<<<<<<
 *                         far = inode.greater
 *                     else:
 */
          __pyx_t_4 = __pyx_v_inode->less;
          __pyx_v_near = __pyx_t_4;

          /* "pyart/map/ckdtree.pyx":1129
 *                     if x[inode.split_dim]<inode.split:
 *                         near = inode.less
 *                         far = inode.greater             # <<<<<<<<<<<<<<
 *                     else:
 *                         near = inode.greater
 */
          __pyx_t_4 = __pyx_v_inode->greater;
          __pyx_v_far = __pyx_t_4;

          /* "pyart/map/ckdtree.pyx":1127
 * 
 *                     # set up children for searching
 *                     if x[inode.split_dim]<inode.split:             # <<<<<<<<<<<<<<
 *                         near = inode.less
 *                         far = inode.greater
 */
          goto __pyx_L33;
        }

        /* "pyart/map/ckdtree.pyx":1131
 *                         far = inode.greater
 *                     else:
 *                         near = inode.greater             # <<<<<<<<<<<<<<
 *                         far = inode.less
 * 
 */
        /*else*/ {
          __pyx_t_4 = __pyx_v_inode->greater;
          __pyx_v_near = __pyx_t_4;

          /* "pyart/map/ckdtree.pyx":1132
 *                     else:
 *                         near = inode.greater
 *                         far = inode.less             # <<<<<<<<<<<<<<
 * 
 *                     # near child is at the same distance as the current node
 */
          __pyx_t_4 = __pyx_v_inode->less;
          __pyx_v_far = __pyx_t_4;
        }
        __pyx_L33:;

        /* "pyart/map/ckdtree.pyx":1137
 *                     # we're going here next, so no point pushing it on the queue
 *                     # no need to recompute the distance or the side_distances
 *                     inf.node = near             # <<<<<<<<<<<<<<
 * 
 *                     # far child is further by an amount depending only
 */
        __pyx_v_inf->node = __pyx_v_near;

        /* "pyart/map/ckdtree.pyx":1142
 *                     # on the split value; compute its distance and side_distances
 *                     # and push it on the queue if it's near enough
 *                     inf2 = <nodeinfo*>stdlib.malloc(sizeof(nodeinfo)+self.m*sizeof(np.float64_t))             # <<<<<<<<<<<<<<
 *                     if inf2 == <nodeinfo*> NULL:
 *                         raise MemoryError
 */
        __pyx_v_inf2 = ((struct __pyx_t_5pyart_3map_7ckdtree_nodeinfo *)malloc(((sizeof(struct __pyx_t_5pyart_3map_7ckdtree_nodeinfo)) + (__pyx_v_self->m * (sizeof(__pyx_t_5numpy_float64_t))))));

        /* "pyart/map/ckdtree.pyx":1143
 *                     # and push it on the queue if it's near enough
 *                     inf2 = <nodeinfo*>stdlib.malloc(sizeof(nodeinfo)+self.m*sizeof(np.float64_t))
 *                     if inf2 == <nodeinfo*> NULL:             # <<<<<<<<<<<<<<
 *                         raise MemoryError
 * 
 */
        __pyx_t_3 = ((__pyx_v_inf2 == ((struct __pyx_t_5pyart_3map_7ckdtree_nodeinfo *)NULL)) != 0);
        if (unlikely(__pyx_t_3)) {

          /* "pyart/map/ckdtree.pyx":1144
 *                     inf2 = <nodeinfo*>stdlib.malloc(sizeof(nodeinfo)+self.m*sizeof(np.float64_t))
 *                     if inf2 == <nodeinfo*> NULL:
 *                         raise MemoryError             # <<<<<<<<<<<<<<
 * 
 *                     it2.contents.ptrdata = <char*> inf2
 */
          PyErr_NoMemory(); __PYX_ERR(0, 1144, __pyx_L4_error)

          /* "pyart/map/ckdtree.pyx":1143
 *                     # and push it on the queue if it's near enough
 *                     inf2 = <nodeinfo*>stdlib.malloc(sizeof(nodeinfo)+self.m*sizeof(np.float64_t))
 *                     if inf2 == <nodeinfo*> NULL:             # <<<<<<<<<<<<<<
 *                         raise MemoryError
 * 
 */
        }

        /* "pyart/map/ckdtree.pyx":1146
 *                         raise MemoryError
 * 
 *                     it2.contents.ptrdata = <char*> inf2             # <<<<<<<<<<<<<<
 *                     inf2.node = far
 *                     # most side distances unchanged
 */
        __pyx_v_it2.contents.ptrdata = ((char *)__pyx_v_inf2);

        /* "pyart/map/ckdtree.pyx":1147
 * 
 *                     it2.contents.ptrdata = <char*> inf2
 *                     inf2.node = far             # <<<<<<<<<<<<<<
 *                     # most side distances unchanged
 *                     for i in range(self.m):
 */
        __pyx_v_inf2->node = __pyx_v_far;

        /* "pyart/map/ckdtree.pyx":1149
 *                     inf2.node = far
 *                     # most side distances unchanged
 *                     for i in range(self.m):             # <<<<<<<<<<<<<<
 *                         inf2.side_distances[i] = inf.side_distances[i]
 * 
 */
        __pyx_t_5 = __pyx_v_self->m;
        __pyx_t_6 = __pyx_t_5;
        for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
          __pyx_v_i = __pyx_t_7;

          /* "pyart/map/ckdtree.pyx":1150
 *                     # most side distances unchanged
 *                     for i in range(self.m):
 *                         inf2.side_distances[i] = inf.side_distances[i]             # <<<<<<<<<<<<<<
 * 
 *                     # one side distance changes
 */
          (__pyx_v_inf2->side_distances[__pyx_v_i]) = (__pyx_v_inf->side_distances[__pyx_v_i]);
        }

        /* "pyart/map/ckdtree.pyx":1154
 *                     # one side distance changes
 *                     # we can adjust the minimum distance without recomputing
 *                     if p == infinity:             # <<<<<<<<<<<<<<
 *                         # we never use side_distances in the l_infinity case
 *                         # inf2.side_distances[inode.split_dim] = dabs(inode.split-x[inode.split_dim])
 */
        __pyx_t_3 = ((__pyx_v_p == __pyx_v_5pyart_3map_7ckdtree_infinity) != 0);
        if (__pyx_t_3) {

          /* "pyart/map/ckdtree.pyx":1157
 *                         # we never use side_distances in the l_infinity case
 *                         # inf2.side_distances[inode.split_dim] = dabs(inode.split-x[inode.split_dim])
 *                         far_min_distance = dmax(min_distance, dabs(inode.split-x[inode.split_dim]))             # <<<<<<<<<<<<<<
 *                     elif p == 1:
 *                         inf2.side_distances[inode.split_dim] = dabs(inode.split-x[inode.split_dim])
 */
          __pyx_v_far_min_distance = __pyx_f_5pyart_3map_7ckdtree_dmax(__pyx_v_min_distance, __pyx_f_5pyart_3map_7ckdtree_dabs((__pyx_v_inode->split - (__pyx_v_x[__pyx_v_inode->split_dim]))));

          /* "pyart/map/ckdtree.pyx":1154
 *                     # one side distance changes
 *                     # we can adjust the minimum distance without recomputing
 *                     if p == infinity:             # <<<<<<<<<<<<<<
 *                         # we never use side_distances in the l_infinity case
 *                         # inf2.side_distances[inode.split_dim] = dabs(inode.split-x[inode.split_dim])
 */
          goto __pyx_L37;
        }

        /* "pyart/map/ckdtree.pyx":1158
 *                         # inf2.side_distances[inode.split_dim] = dabs(inode.split-x[inode.split_dim])
 *                         far_min_distance = dmax(min_distance, dabs(inode.split-x[inode.split_dim]))
 *                     elif p == 1:             # <<<<<<<<<<<<<<
 *                         inf2.side_distances[inode.split_dim] = dabs(inode.split-x[inode.split_dim])
 *                         far_min_distance = min_distance - \
 */
        __pyx_t_3 = ((__pyx_v_p == 1.0) != 0);
        if (__pyx_t_3) {

          /* "pyart/map/ckdtree.pyx":1159
 *                         far_min_distance = dmax(min_distance, dabs(inode.split-x[inode.split_dim]))
 *                     elif p == 1:
 *                         inf2.side_distances[inode.split_dim] = dabs(inode.split-x[inode.split_dim])             # <<<<<<<<<<<<<<
 *                         far_min_distance = min_distance - \
 *                             inf.side_distances[inode.split_dim] + \
 */
          (__pyx_v_inf2->side_distances[__pyx_v_inode->split_dim]) = __pyx_f_5pyart_3map_7ckdtree_dabs((__pyx_v_inode->split - (__pyx_v_x[__pyx_v_inode->split_dim])));

          /* "pyart/map/ckdtree.pyx":1161
 *                         inf2.side_distances[inode.split_dim] = dabs(inode.split-x[inode.split_dim])
 *                         far_min_distance = min_distance - \
 *                             inf.side_distances[inode.split_dim] + \             # <<<<<<<<<<<<<<
 *                             inf2.side_distances[inode.split_dim]
 *                     else:
 */
          __pyx_v_far_min_distance = ((__pyx_v_min_distance - (__pyx_v_inf->side_distances[__pyx_v_inode->split_dim])) + (__pyx_v_inf2->side_distances[__pyx_v_inode->split_dim]));

          /* "pyart/map/ckdtree.pyx":1158
 *                         # inf2.side_distances[inode.split_dim] = dabs(inode.split-x[inode.split_dim])
 *                         far_min_distance = dmax(min_distance, dabs(inode.split-x[inode.split_dim]))
 *                     elif p == 1:             # <<<<<<<<<<<<<<
 *                         inf2.side_distances[inode.split_dim] = dabs(inode.split-x[inode.split_dim])
 *                         far_min_distance = min_distance - \
 */
          goto __pyx_L37;
        }

        /* "pyart/map/ckdtree.pyx":1164
 *                             inf2.side_distances[inode.split_dim]
 *                     else:
 *                         inf2.side_distances[inode.split_dim] = dabs(inode.split -             # <<<<<<<<<<<<<<
 *                                                                     x[inode.split_dim])**p
 *                         far_min_distance = min_distance - \
 */
        /*else*/ {

          /* "pyart/map/ckdtree.pyx":1165
 *                     else:
 *                         inf2.side_distances[inode.split_dim] = dabs(inode.split -
 *                                                                     x[inode.split_dim])**p             # <<<<<<<<<<<<<<
 *                         far_min_distance = min_distance - \
 *                             inf.side_distances[inode.split_dim] + \
 */
          (__pyx_v_inf2->side_distances[__pyx_v_inode->split_dim]) = pow(__pyx_f_5pyart_3map_7ckdtree_dabs((__pyx_v_inode->split - (__pyx_v_x[__pyx_v_inode->split_dim]))), __pyx_v_p);

          /* "pyart/map/ckdtree.pyx":1167
 *                                                                     x[inode.split_dim])**p
 *                         far_min_distance = min_distance - \
 *                             inf.side_distances[inode.split_dim] + \             # <<<<<<<<<<<<<<
 *                             inf2.side_distances[inode.split_dim]
 * 
 */
          __pyx_v_far_min_distance = ((__pyx_v_min_distance - (__pyx_v_inf->side_distances[__pyx_v_inode->split_dim])) + (__pyx_v_inf2->side_distances[__pyx_v_inode->split_dim]));
        }
        __pyx_L37:;

        /* "pyart/map/ckdtree.pyx":1170
 *                             inf2.side_distances[inode.split_dim]
 * 
 *                     it2.priority = far_min_distance             # <<<<<<<<<<<<<<
 * 
 * 
 */
        __pyx_v_it2.priority = __pyx_v_far_min_distance;

        /* "pyart/map/ckdtree.pyx":1174
 * 
 *                     # far child might be too far, if so, don't bother pushing it
 *                     if far_min_distance<=distance_upper_bound*epsfac:             # <<<<<<<<<<<<<<
 *                         q.push(it2)
 *                     else:
 */
        __pyx_t_3 = ((__pyx_v_far_min_distance <= (__pyx_v_distance_upper_bound * __pyx_v_epsfac)) != 0);
        if (__pyx_t_3) {

          /* "pyart/map/ckdtree.pyx":1175
 *                     # far child might be too far, if so, don't bother pushing it
 *                     if far_min_distance<=distance_upper_bound*epsfac:
 *                         q.push(it2)             # <<<<<<<<<<<<<<
 *                     else:
 *                         stdlib.free(inf2)
 */
          __pyx_t_10 = __pyx_f_5pyart_3map_7ckdtree_4heap_push(__pyx_v_q, __pyx_v_it2); if (unlikely(__pyx_t_10 == ((int)-1))) __PYX_ERR(0, 1175, __pyx_L4_error)

          /* "pyart/map/ckdtree.pyx":1174
 * 
 *                     # far child might be too far, if so, don't bother pushing it
 *                     if far_min_distance<=distance_upper_bound*epsfac:             # <<<<<<<<<<<<<<
 *                         q.push(it2)
 *                     else:
 */
          goto __pyx_L38;
        }

        /* "pyart/map/ckdtree.pyx":1177
 *                         q.push(it2)
 *                     else:
 *                         stdlib.free(inf2)             # <<<<<<<<<<<<<<
 *                         inf2 = <nodeinfo*> NULL
 *                         # just in case
 */
        /*else*/ {
          free(__pyx_v_inf2);

          /* "pyart/map/ckdtree.pyx":1178
 *                     else:
 *                         stdlib.free(inf2)
 *                         inf2 = <nodeinfo*> NULL             # <<<<<<<<<<<<<<
 *                         # just in case
 *                         it2.contents.ptrdata = <char*> NULL
 */
          __pyx_v_inf2 = ((struct __pyx_t_5pyart_3map_7ckdtree_nodeinfo *)NULL);

          /* "pyart/map/ckdtree.pyx":1180
 *                         inf2 = <nodeinfo*> NULL
 *                         # just in case
 *                         it2.contents.ptrdata = <char*> NULL             # <<<<<<<<<<<<<<
 * 
 *             # fill output arrays with sorted neighbors
 */
          __pyx_v_it2.contents.ptrdata = ((char *)NULL);
        }
        __pyx_L38:;
      }
      __pyx_L23:;
    }
    __pyx_L22_break:;

    /* "pyart/map/ckdtree.pyx":1183
 * 
 *             # fill output arrays with sorted neighbors
 *             for i in range(neighbors.n-1,-1,-1):             # <<<<<<<<<<<<<<
 *                 neighbors.pop(&neighbor)
 *                 result_indices[i] = neighbor.contents.intdata
 */
    for (__pyx_t_5 = (__pyx_v_neighbors->n - 1); __pyx_t_5 > -1; __pyx_t_5-=1) {
      __pyx_v_i = __pyx_t_5;

      /* "pyart/map/ckdtree.pyx":1184
 *             # fill output arrays with sorted neighbors
 *             for i in range(neighbors.n-1,-1,-1):
 *                 neighbors.pop(&neighbor)             # <<<<<<<<<<<<<<
 *                 result_indices[i] = neighbor.contents.intdata
 *                 if p==1 or p==infinity:
 */
      __pyx_t_10 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_heap *)__pyx_v_neighbors->__pyx_vtab)->pop(__pyx_v_neighbors, (&__pyx_v_neighbor)); if (unlikely(__pyx_t_10 == ((int)-1))) __PYX_ERR(0, 1184, __pyx_L4_error)

      /* "pyart/map/ckdtree.pyx":1185
 *             for i in range(neighbors.n-1,-1,-1):
 *                 neighbors.pop(&neighbor)
 *                 result_indices[i] = neighbor.contents.intdata             # <<<<<<<<<<<<<<
 *                 if p==1 or p==infinity:
 *                     result_distances[i] = -neighbor.priority
 */
      __pyx_t_6 = __pyx_v_neighbor.contents.intdata;
      (__pyx_v_result_indices[__pyx_v_i]) = __pyx_t_6;

      /* "pyart/map/ckdtree.pyx":1186
 *                 neighbors.pop(&neighbor)
 *                 result_indices[i] = neighbor.contents.intdata
 *                 if p==1 or p==infinity:             # <<<<<<<<<<<<<<
 *                     result_distances[i] = -neighbor.priority
 *                 else:
 */
      __pyx_t_8 = ((__pyx_v_p == 1.0) != 0);
      if (!__pyx_t_8) {
      } else {
        __pyx_t_3 = __pyx_t_8;
        goto __pyx_L42_bool_binop_done;
      }
      __pyx_t_8 = ((__pyx_v_p == __pyx_v_5pyart_3map_7ckdtree_infinity) != 0);
      __pyx_t_3 = __pyx_t_8;
      __pyx_L42_bool_binop_done:;
      if (__pyx_t_3) {

        /* "pyart/map/ckdtree.pyx":1187
 *                 result_indices[i] = neighbor.contents.intdata
 *                 if p==1 or p==infinity:
 *                     result_distances[i] = -neighbor.priority             # <<<<<<<<<<<<<<
 *                 else:
 *                     result_distances[i] = (-neighbor.priority)**(1./p)
 */
        (__pyx_v_result_distances[__pyx_v_i]) = (-__pyx_v_neighbor.priority);

        /* "pyart/map/ckdtree.pyx":1186
 *                 neighbors.pop(&neighbor)
 *                 result_indices[i] = neighbor.contents.intdata
 *                 if p==1 or p==infinity:             # <<<<<<<<<<<<<<
 *                     result_distances[i] = -neighbor.priority
 *                 else:
 */
        goto __pyx_L41;
      }

      /* "pyart/map/ckdtree.pyx":1189
 *                     result_distances[i] = -neighbor.priority
 *                 else:
 *                     result_distances[i] = (-neighbor.priority)**(1./p)             # <<<<<<<<<<<<<<
 * 
 *             inf = inf2 = <nodeinfo*> NULL
 */
      /*else*/ {
        if (unlikely(__pyx_v_p == 0)) {
          PyErr_SetString(PyExc_ZeroDivisionError, "float division");
          __PYX_ERR(0, 1189, __pyx_L4_error)
        }
        (__pyx_v_result_distances[__pyx_v_i]) = pow((-__pyx_v_neighbor.priority), (((__pyx_t_5numpy_float64_t)1.) / __pyx_v_p));
      }
      __pyx_L41:;
    }

    /* "pyart/map/ckdtree.pyx":1191
 *                     result_distances[i] = (-neighbor.priority)**(1./p)
 * 
 *             inf = inf2 = <nodeinfo*> NULL             # <<<<<<<<<<<<<<
 * 
 *         finally:
 */
    __pyx_v_inf = ((struct __pyx_t_5pyart_3map_7ckdtree_nodeinfo *)NULL);
    __pyx_v_inf2 = ((struct __pyx_t_5pyart_3map_7ckdtree_nodeinfo *)NULL);
  }

  /* "pyart/map/ckdtree.pyx":1194
 * 
 *         finally:
 *             if inf2 != <nodeinfo*> NULL:             # <<<<<<<<<<<<<<
 *                 stdlib.free(inf2)
 * 
 */
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_3 = ((__pyx_v_inf2 != ((struct __pyx_t_5pyart_3map_7ckdtree_nodeinfo *)NULL)) != 0);
      if (__pyx_t_3) {

        /* "pyart/map/ckdtree.pyx":1195
 *         finally:
 *             if inf2 != <nodeinfo*> NULL:
 *                 stdlib.free(inf2)             # <<<<<<<<<<<<<<
 * 
 *             if inf != <nodeinfo*> NULL:
 */
        free(__pyx_v_inf2);

        /* "pyart/map/ckdtree.pyx":1194
 * 
 *         finally:
 *             if inf2 != <nodeinfo*> NULL:             # <<<<<<<<<<<<<<
 *                 stdlib.free(inf2)
 * 
 */
      }

      /* "pyart/map/ckdtree.pyx":1197
 *                 stdlib.free(inf2)
 * 
 *             if inf != <nodeinfo*> NULL:             # <<<<<<<<<<<<<<
 *                 stdlib.free(inf)
 * 
 */
      __pyx_t_3 = ((__pyx_v_inf != ((struct __pyx_t_5pyart_3map_7ckdtree_nodeinfo *)NULL)) != 0);
      if (__pyx_t_3) {

        /* "pyart/map/ckdtree.pyx":1198
 * 
 *             if inf != <nodeinfo*> NULL:
 *                 stdlib.free(inf)             # <<<<<<<<<<<<<<
 * 
 *         return 0
 */
        free(__pyx_v_inf);

        /* "pyart/map/ckdtree.pyx":1197
 *                 stdlib.free(inf2)
 * 
 *             if inf != <nodeinfo*> NULL:             # <<<<<<<<<<<<<<
 *                 stdlib.free(inf)
 * 
 */
      }
      goto __pyx_L5;
    }
    __pyx_L4_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18);
      if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_13, &__pyx_t_14, &__pyx_t_15) < 0)) __Pyx_ErrFetch(&__pyx_t_13, &__pyx_t_14, &__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_16);
      __Pyx_XGOTREF(__pyx_t_17);
      __Pyx_XGOTREF(__pyx_t_18);
      __pyx_t_10 = __pyx_lineno; __pyx_t_11 = __pyx_clineno; __pyx_t_12 = __pyx_filename;
      {

        /* "pyart/map/ckdtree.pyx":1194
 * 
 *         finally:
 *             if inf2 != <nodeinfo*> NULL:             # <<<<<<<<<<<<<<
 *                 stdlib.free(inf2)
 * 
 */
        __pyx_t_3 = ((__pyx_v_inf2 != ((struct __pyx_t_5pyart_3map_7ckdtree_nodeinfo *)NULL)) != 0);
        if (__pyx_t_3) {

          /* "pyart/map/ckdtree.pyx":1195
 *         finally:
 *             if inf2 != <nodeinfo*> NULL:
 *                 stdlib.free(inf2)             # <<<<<<<<<<<<<<
 * 
 *             if inf != <nodeinfo*> NULL:
 */
          free(__pyx_v_inf2);

          /* "pyart/map/ckdtree.pyx":1194
 * 
 *         finally:
 *             if inf2 != <nodeinfo*> NULL:             # <<<<<<<<<<<<<<
 *                 stdlib.free(inf2)
 * 
 */
        }

        /* "pyart/map/ckdtree.pyx":1197
 *                 stdlib.free(inf2)
 * 
 *             if inf != <nodeinfo*> NULL:             # <<<<<<<<<<<<<<
 *                 stdlib.free(inf)
 * 
 */
        __pyx_t_3 = ((__pyx_v_inf != ((struct __pyx_t_5pyart_3map_7ckdtree_nodeinfo *)NULL)) != 0);
        if (__pyx_t_3) {

          /* "pyart/map/ckdtree.pyx":1198
 * 
 *             if inf != <nodeinfo*> NULL:
 *                 stdlib.free(inf)             # <<<<<<<<<<<<<<
 * 
 *         return 0
 */
          free(__pyx_v_inf);

          /* "pyart/map/ckdtree.pyx":1197
 *                 stdlib.free(inf2)
 * 
 *             if inf != <nodeinfo*> NULL:             # <<<<<<<<<<<<<<
 *                 stdlib.free(inf)
 * 
 */
        }
      }
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_16);
        __Pyx_XGIVEREF(__pyx_t_17);
        __Pyx_XGIVEREF(__pyx_t_18);
        __Pyx_ExceptionReset(__pyx_t_16, __pyx_t_17, __pyx_t_18);
      }
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_ErrRestore(__pyx_t_13, __pyx_t_14, __pyx_t_15);
      __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0;
      __pyx_lineno = __pyx_t_10; __pyx_clineno = __pyx_t_11; __pyx_filename = __pyx_t_12;
      goto __pyx_L1_error;
    }
    __pyx_L5:;
  }

  /* "pyart/map/ckdtree.pyx":1200
 *                 stdlib.free(inf)
 * 
 *         return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":995
 *     # -----
 * 
 *     cdef int __query(cKDTree self,             # <<<<<<<<<<<<<<
 *             np.float64_t*result_distances,
 *             np.intp_t*result_indices,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.__query", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_q);
  __Pyx_XDECREF((PyObject *)__pyx_v_neighbors);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":1205
 *     @cython.boundscheck(False)
 *     @cython.wraparound(False)
 *     def query(cKDTree self, object x, np.intp_t k=1, np.float64_t eps=0,             # <<<<<<<<<<<<<<
 *               np.float64_t p=2, np.float64_t distance_upper_bound=infinity):
 *         """query(self, x, k=1, eps=0, p=2, distance_upper_bound=np.inf)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_5query(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5pyart_3map_7ckdtree_7cKDTree_4query[] = "query(self, x, k=1, eps=0, p=2, distance_upper_bound=np.inf)\n        \n        Query the kd-tree for nearest neighbors\n\n        Parameters\n        ----------\n        x : array_like, last dimension self.m\n            An array of points to query.\n        k : integer\n            The number of nearest neighbors to return.\n        eps : non-negative float\n            Return approximate nearest neighbors; the kth returned value \n            is guaranteed to be no further than (1+eps) times the \n            distance to the real k-th nearest neighbor.\n        p : float, 1<=p<=infinity\n            Which Minkowski p-norm to use. \n            1 is the sum-of-absolute-values \"Manhattan\" distance\n            2 is the usual Euclidean distance\n            infinity is the maximum-coordinate-difference distance\n        distance_upper_bound : nonnegative float\n            Return only neighbors within this distance.  This is used to prune\n            tree searches, so if you are doing a series of nearest-neighbor\n            queries, it may help to supply the distance to the nearest neighbor\n            of the most recent point.\n\n        Returns\n        -------\n        d : array of floats\n            The distances to the nearest neighbors. \n            If x has shape tuple+(self.m,), then d has shape tuple+(k,).\n            Missing neighbors are indicated with infinite distances.\n        i : ndarray of ints\n            The locations of the neighbors in self.data.\n            If `x` has shape tuple+(self.m,), then `i` has shape tuple+(k,).\n            Missing neighbors are indicated with self.n.\n\n        ";
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_5query(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_x = 0;
  __pyx_t_5numpy_intp_t __pyx_v_k;
  __pyx_t_5numpy_float64_t __pyx_v_eps;
  __pyx_t_5numpy_float64_t __pyx_v_p;
  __pyx_t_5numpy_float64_t __pyx_v_distance_upper_bound;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("query (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_k,&__pyx_n_s_eps,&__pyx_n_s_p,&__pyx_n_s_distance_upper_bound,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_k);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_eps);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_p);
          if (value) { values[3] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_distance_upper_bound);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "query") < 0)) __PYX_ERR(0, 1205, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_x = values[0];
    if (values[1]) {
      __pyx_v_k = __Pyx_PyInt_As_Py_intptr_t(values[1]); if (unlikely((__pyx_v_k == ((npy_intp)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1205, __pyx_L3_error)
    } else {
      __pyx_v_k = ((__pyx_t_5numpy_intp_t)1);
    }
    if (values[2]) {
      __pyx_v_eps = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_eps == ((npy_float64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1205, __pyx_L3_error)
    } else {
      __pyx_v_eps = ((__pyx_t_5numpy_float64_t)0.0);
    }
    if (values[3]) {
      __pyx_v_p = __pyx_PyFloat_AsDouble(values[3]); if (unlikely((__pyx_v_p == ((npy_float64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1206, __pyx_L3_error)
    } else {
      __pyx_v_p = ((__pyx_t_5numpy_float64_t)2.0);
    }
    if (values[4]) {
      __pyx_v_distance_upper_bound = __pyx_PyFloat_AsDouble(values[4]); if (unlikely((__pyx_v_distance_upper_bound == ((npy_float64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1206, __pyx_L3_error)
    } else {
      __pyx_v_distance_upper_bound = __pyx_k__14;
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("query", 0, 1, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1205, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.query", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_7cKDTree_4query(((struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self), __pyx_v_x, __pyx_v_k, __pyx_v_eps, __pyx_v_p, __pyx_v_distance_upper_bound);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_4query(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, PyObject *__pyx_v_x, __pyx_t_5numpy_intp_t __pyx_v_k, __pyx_t_5numpy_float64_t __pyx_v_eps, __pyx_t_5numpy_float64_t __pyx_v_p, __pyx_t_5numpy_float64_t __pyx_v_distance_upper_bound) {
  PyArrayObject *__pyx_v_ii = 0;
  PyArrayObject *__pyx_v_dd = 0;
  PyArrayObject *__pyx_v_xx = 0;
  __pyx_t_5numpy_intp_t __pyx_v_c;
  __pyx_t_5numpy_intp_t __pyx_v_n;
  __pyx_t_5numpy_intp_t __pyx_v_i;
  __pyx_t_5numpy_intp_t __pyx_v_j;
  PyObject *__pyx_v_sh = NULL;
  int __pyx_v_single;
  PyObject *__pyx_v_retshape = NULL;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_dd;
  __Pyx_Buffer __pyx_pybuffer_dd;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_ii;
  __Pyx_Buffer __pyx_pybuffer_ii;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_xx;
  __Pyx_Buffer __pyx_pybuffer_xx;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  Py_UCS4 __pyx_t_8;
  __pyx_t_5numpy_intp_t __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  PyArrayObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyArrayObject *__pyx_t_16 = NULL;
  PyArrayObject *__pyx_t_17 = NULL;
  __pyx_t_5numpy_intp_t __pyx_t_18;
  __pyx_t_5numpy_intp_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  __pyx_t_5numpy_intp_t __pyx_t_26;
  __pyx_t_5numpy_intp_t __pyx_t_27;
  __pyx_t_5numpy_intp_t __pyx_t_28;
  PyObject *__pyx_t_29 = NULL;
  PyObject *__pyx_t_30 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("query", 0);
  __Pyx_INCREF(__pyx_v_x);
  __pyx_pybuffer_ii.pybuffer.buf = NULL;
  __pyx_pybuffer_ii.refcount = 0;
  __pyx_pybuffernd_ii.data = NULL;
  __pyx_pybuffernd_ii.rcbuffer = &__pyx_pybuffer_ii;
  __pyx_pybuffer_dd.pybuffer.buf = NULL;
  __pyx_pybuffer_dd.refcount = 0;
  __pyx_pybuffernd_dd.data = NULL;
  __pyx_pybuffernd_dd.rcbuffer = &__pyx_pybuffer_dd;
  __pyx_pybuffer_xx.pybuffer.buf = NULL;
  __pyx_pybuffer_xx.refcount = 0;
  __pyx_pybuffernd_xx.data = NULL;
  __pyx_pybuffernd_xx.rcbuffer = &__pyx_pybuffer_xx;

  /* "pyart/map/ckdtree.pyx":1248
 *         cdef np.ndarray[np.float64_t, ndim=2] xx
 *         cdef np.intp_t c, n, i, j
 *         x = np.asarray(x).astype(np.float64)             # <<<<<<<<<<<<<<
 *         sh = np.shape(x)
 *         if sh[len(sh)-1] != self.m:
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_asarray); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_3, __pyx_v_x) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_x);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_astype); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_float64); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF_SET(__pyx_v_x, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/map/ckdtree.pyx":1249
 *         cdef np.intp_t c, n, i, j
 *         x = np.asarray(x).astype(np.float64)
 *         sh = np.shape(x)             # <<<<<<<<<<<<<<
 *         if sh[len(sh)-1] != self.m:
 *             raise ValueError("x must consist of vectors of length %d but has"
 */
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_shape); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_x) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_x);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_sh = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyart/map/ckdtree.pyx":1250
 *         x = np.asarray(x).astype(np.float64)
 *         sh = np.shape(x)
 *         if sh[len(sh)-1] != self.m:             # <<<<<<<<<<<<<<
 *             raise ValueError("x must consist of vectors of length %d but has"
 *                              "shape %s" % (int(self.m), np.shape(x)))
 */
  __pyx_t_5 = PyObject_Length(__pyx_v_sh); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1250, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 - 1);
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_sh, __pyx_t_6, Py_ssize_t, 1, PyInt_FromSsize_t, 0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->m); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyObject_RichCompare(__pyx_t_1, __pyx_t_3, Py_NE); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1250, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 1250, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(__pyx_t_7)) {

    /* "pyart/map/ckdtree.pyx":1251
 *         sh = np.shape(x)
 *         if sh[len(sh)-1] != self.m:
 *             raise ValueError("x must consist of vectors of length %d but has"             # <<<<<<<<<<<<<<
 *                              "shape %s" % (int(self.m), np.shape(x)))
 *         if p < 1:
 */
    __pyx_t_4 = PyTuple_New(4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1251, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = 0;
    __pyx_t_8 = 127;
    __Pyx_INCREF(__pyx_kp_u_x_must_consist_of_vectors_of_len);
    __pyx_t_6 += 36;
    __Pyx_GIVEREF(__pyx_kp_u_x_must_consist_of_vectors_of_len);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_kp_u_x_must_consist_of_vectors_of_len);

    /* "pyart/map/ckdtree.pyx":1252
 *         if sh[len(sh)-1] != self.m:
 *             raise ValueError("x must consist of vectors of length %d but has"
 *                              "shape %s" % (int(self.m), np.shape(x)))             # <<<<<<<<<<<<<<
 *         if p < 1:
 *             raise ValueError("Only p-norms with 1<=p<=infinity permitted")
 */
    __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->m); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1252, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyInt_Type)), __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1252, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_FormatAndDecref(__Pyx_PyNumber_IntOrLong(__pyx_t_1), __pyx_n_u_d); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1252, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_8 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) > __pyx_t_8) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) : __pyx_t_8;
    __pyx_t_6 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3);
    __pyx_t_3 = 0;
    __Pyx_INCREF(__pyx_kp_u_but_hasshape);
    __pyx_t_6 += 14;
    __Pyx_GIVEREF(__pyx_kp_u_but_hasshape);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_kp_u_but_hasshape);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1252, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_shape); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1252, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_1, __pyx_v_x) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_x);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1252, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Unicode(__pyx_t_3), __pyx_empty_unicode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1252, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_8 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_2) > __pyx_t_8) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_2) : __pyx_t_8;
    __pyx_t_6 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "pyart/map/ckdtree.pyx":1251
 *         sh = np.shape(x)
 *         if sh[len(sh)-1] != self.m:
 *             raise ValueError("x must consist of vectors of length %d but has"             # <<<<<<<<<<<<<<
 *                              "shape %s" % (int(self.m), np.shape(x)))
 *         if p < 1:
 */
    __pyx_t_2 = __Pyx_PyUnicode_Join(__pyx_t_4, 4, __pyx_t_6, __pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1251, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1251, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 1251, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":1250
 *         x = np.asarray(x).astype(np.float64)
 *         sh = np.shape(x)
 *         if sh[len(sh)-1] != self.m:             # <<<<<<<<<<<<<<
 *             raise ValueError("x must consist of vectors of length %d but has"
 *                              "shape %s" % (int(self.m), np.shape(x)))
 */
  }

  /* "pyart/map/ckdtree.pyx":1253
 *             raise ValueError("x must consist of vectors of length %d but has"
 *                              "shape %s" % (int(self.m), np.shape(x)))
 *         if p < 1:             # <<<<<<<<<<<<<<
 *             raise ValueError("Only p-norms with 1<=p<=infinity permitted")
 *         if len(x.shape)==1:
 */
  __pyx_t_7 = ((__pyx_v_p < 1.0) != 0);
  if (unlikely(__pyx_t_7)) {

    /* "pyart/map/ckdtree.pyx":1254
 *                              "shape %s" % (int(self.m), np.shape(x)))
 *         if p < 1:
 *             raise ValueError("Only p-norms with 1<=p<=infinity permitted")             # <<<<<<<<<<<<<<
 *         if len(x.shape)==1:
 *             single = True
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__15, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1254, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 1254, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":1253
 *             raise ValueError("x must consist of vectors of length %d but has"
 *                              "shape %s" % (int(self.m), np.shape(x)))
 *         if p < 1:             # <<<<<<<<<<<<<<
 *             raise ValueError("Only p-norms with 1<=p<=infinity permitted")
 *         if len(x.shape)==1:
 */
  }

  /* "pyart/map/ckdtree.pyx":1255
 *         if p < 1:
 *             raise ValueError("Only p-norms with 1<=p<=infinity permitted")
 *         if len(x.shape)==1:             # <<<<<<<<<<<<<<
 *             single = True
 *             x = x[np.newaxis,:]
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_x, __pyx_n_s_shape); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = PyObject_Length(__pyx_t_4); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1255, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_7 = ((__pyx_t_6 == 1) != 0);
  if (__pyx_t_7) {

    /* "pyart/map/ckdtree.pyx":1256
 *             raise ValueError("Only p-norms with 1<=p<=infinity permitted")
 *         if len(x.shape)==1:
 *             single = True             # <<<<<<<<<<<<<<
 *             x = x[np.newaxis,:]
 *         else:
 */
    __pyx_v_single = 1;

    /* "pyart/map/ckdtree.pyx":1257
 *         if len(x.shape)==1:
 *             single = True
 *             x = x[np.newaxis,:]             # <<<<<<<<<<<<<<
 *         else:
 *             single = False
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1257, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_newaxis); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1257, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1257, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2);
    __Pyx_INCREF(__pyx_slice__16);
    __Pyx_GIVEREF(__pyx_slice__16);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_slice__16);
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_x, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1257, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_x, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "pyart/map/ckdtree.pyx":1255
 *         if p < 1:
 *             raise ValueError("Only p-norms with 1<=p<=infinity permitted")
 *         if len(x.shape)==1:             # <<<<<<<<<<<<<<
 *             single = True
 *             x = x[np.newaxis,:]
 */
    goto __pyx_L5;
  }

  /* "pyart/map/ckdtree.pyx":1259
 *             x = x[np.newaxis,:]
 *         else:
 *             single = False             # <<<<<<<<<<<<<<
 *         sh = np.shape(x)
 *         retshape = sh[len(sh) - 1]
 */
  /*else*/ {
    __pyx_v_single = 0;
  }
  __pyx_L5:;

  /* "pyart/map/ckdtree.pyx":1260
 *         else:
 *             single = False
 *         sh = np.shape(x)             # <<<<<<<<<<<<<<
 *         retshape = sh[len(sh) - 1]
 *         n = <np.intp_t> np.prod(retshape)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_shape); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_x) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_x);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF_SET(__pyx_v_sh, __pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyart/map/ckdtree.pyx":1261
 *             single = False
 *         sh = np.shape(x)
 *         retshape = sh[len(sh) - 1]             # <<<<<<<<<<<<<<
 *         n = <np.intp_t> np.prod(retshape)
 *         xx = np.reshape(x,(n,self.m))
 */
  __pyx_t_6 = PyObject_Length(__pyx_v_sh); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1261, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_6 - 1);
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_sh, __pyx_t_5, Py_ssize_t, 1, PyInt_FromSsize_t, 0, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1261, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_retshape = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pyart/map/ckdtree.pyx":1262
 *         sh = np.shape(x)
 *         retshape = sh[len(sh) - 1]
 *         n = <np.intp_t> np.prod(retshape)             # <<<<<<<<<<<<<<
 *         xx = np.reshape(x,(n,self.m))
 *         xx = np.ascontiguousarray(xx,dtype=np.float64)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_prod); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_3, __pyx_v_retshape) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_retshape);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_9 = __Pyx_PyInt_As_Py_intptr_t(__pyx_t_2); if (unlikely((__pyx_t_9 == ((npy_intp)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1262, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_n = ((__pyx_t_5numpy_intp_t)__pyx_t_9);

  /* "pyart/map/ckdtree.pyx":1263
 *         retshape = sh[len(sh) - 1]
 *         n = <np.intp_t> np.prod(retshape)
 *         xx = np.reshape(x,(n,self.m))             # <<<<<<<<<<<<<<
 *         xx = np.ascontiguousarray(xx,dtype=np.float64)
 *         dd = np.empty((n,k),dtype=np.float64)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_reshape); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->m); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_10 = PyTuple_New(2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_1);
  __pyx_t_4 = 0;
  __pyx_t_1 = 0;
  __pyx_t_1 = NULL;
  __pyx_t_11 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_11 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_x, __pyx_t_10};
    __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1263, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_x, __pyx_t_10};
    __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1263, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  } else
  #endif
  {
    __pyx_t_4 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1263, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__pyx_t_1) {
      __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1); __pyx_t_1 = NULL;
    }
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_11, __pyx_v_x);
    __Pyx_GIVEREF(__pyx_t_10);
    PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_11, __pyx_t_10);
    __pyx_t_10 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1263, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1263, __pyx_L1_error)
  __pyx_t_12 = ((PyArrayObject *)__pyx_t_2);
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_xx.rcbuffer->pybuffer);
    __pyx_t_11 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_xx.rcbuffer->pybuffer, (PyObject*)__pyx_t_12, &__Pyx_TypeInfo_nn___pyx_t_5numpy_float64_t, PyBUF_FORMAT| PyBUF_STRIDES, 2, 0, __pyx_stack);
    if (unlikely(__pyx_t_11 < 0)) {
      PyErr_Fetch(&__pyx_t_13, &__pyx_t_14, &__pyx_t_15);
      if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_xx.rcbuffer->pybuffer, (PyObject*)__pyx_v_xx, &__Pyx_TypeInfo_nn___pyx_t_5numpy_float64_t, PyBUF_FORMAT| PyBUF_STRIDES, 2, 0, __pyx_stack) == -1)) {
        Py_XDECREF(__pyx_t_13); Py_XDECREF(__pyx_t_14); Py_XDECREF(__pyx_t_15);
        __Pyx_RaiseBufferFallbackError();
      } else {
        PyErr_Restore(__pyx_t_13, __pyx_t_14, __pyx_t_15);
      }
      __pyx_t_13 = __pyx_t_14 = __pyx_t_15 = 0;
    }
    __pyx_pybuffernd_xx.diminfo[0].strides = __pyx_pybuffernd_xx.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_xx.diminfo[0].shape = __pyx_pybuffernd_xx.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_xx.diminfo[1].strides = __pyx_pybuffernd_xx.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_xx.diminfo[1].shape = __pyx_pybuffernd_xx.rcbuffer->pybuffer.shape[1];
    if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 1263, __pyx_L1_error)
  }
  __pyx_t_12 = 0;
  __pyx_v_xx = ((PyArrayObject *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyart/map/ckdtree.pyx":1264
 *         n = <np.intp_t> np.prod(retshape)
 *         xx = np.reshape(x,(n,self.m))
 *         xx = np.ascontiguousarray(xx,dtype=np.float64)             # <<<<<<<<<<<<<<
 *         dd = np.empty((n,k),dtype=np.float64)
 *         dd.fill(infinity)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_ascontiguousarray); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_xx));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_xx));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_xx));
  __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_np); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_float64); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_1) < 0) __PYX_ERR(0, 1264, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1264, __pyx_L1_error)
  __pyx_t_12 = ((PyArrayObject *)__pyx_t_1);
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_xx.rcbuffer->pybuffer);
    __pyx_t_11 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_xx.rcbuffer->pybuffer, (PyObject*)__pyx_t_12, &__Pyx_TypeInfo_nn___pyx_t_5numpy_float64_t, PyBUF_FORMAT| PyBUF_STRIDES, 2, 0, __pyx_stack);
    if (unlikely(__pyx_t_11 < 0)) {
      PyErr_Fetch(&__pyx_t_15, &__pyx_t_14, &__pyx_t_13);
      if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_xx.rcbuffer->pybuffer, (PyObject*)__pyx_v_xx, &__Pyx_TypeInfo_nn___pyx_t_5numpy_float64_t, PyBUF_FORMAT| PyBUF_STRIDES, 2, 0, __pyx_stack) == -1)) {
        Py_XDECREF(__pyx_t_15); Py_XDECREF(__pyx_t_14); Py_XDECREF(__pyx_t_13);
        __Pyx_RaiseBufferFallbackError();
      } else {
        PyErr_Restore(__pyx_t_15, __pyx_t_14, __pyx_t_13);
      }
      __pyx_t_15 = __pyx_t_14 = __pyx_t_13 = 0;
    }
    __pyx_pybuffernd_xx.diminfo[0].strides = __pyx_pybuffernd_xx.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_xx.diminfo[0].shape = __pyx_pybuffernd_xx.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_xx.diminfo[1].strides = __pyx_pybuffernd_xx.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_xx.diminfo[1].shape = __pyx_pybuffernd_xx.rcbuffer->pybuffer.shape[1];
    if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 1264, __pyx_L1_error)
  }
  __pyx_t_12 = 0;
  __Pyx_DECREF_SET(__pyx_v_xx, ((PyArrayObject *)__pyx_t_1));
  __pyx_t_1 = 0;

  /* "pyart/map/ckdtree.pyx":1265
 *         xx = np.reshape(x,(n,self.m))
 *         xx = np.ascontiguousarray(xx,dtype=np.float64)
 *         dd = np.empty((n,k),dtype=np.float64)             # <<<<<<<<<<<<<<
 *         dd.fill(infinity)
 *         ii = np.empty((n,k),dtype=np.intp)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_empty); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_k); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_float64); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_10) < 0) __PYX_ERR(0, 1265, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (!(likely(((__pyx_t_10) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_10, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1265, __pyx_L1_error)
  __pyx_t_16 = ((PyArrayObject *)__pyx_t_10);
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_dd.rcbuffer->pybuffer);
    __pyx_t_11 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_dd.rcbuffer->pybuffer, (PyObject*)__pyx_t_16, &__Pyx_TypeInfo_nn___pyx_t_5numpy_float64_t, PyBUF_FORMAT| PyBUF_STRIDES, 2, 0, __pyx_stack);
    if (unlikely(__pyx_t_11 < 0)) {
      PyErr_Fetch(&__pyx_t_13, &__pyx_t_14, &__pyx_t_15);
      if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_dd.rcbuffer->pybuffer, (PyObject*)__pyx_v_dd, &__Pyx_TypeInfo_nn___pyx_t_5numpy_float64_t, PyBUF_FORMAT| PyBUF_STRIDES, 2, 0, __pyx_stack) == -1)) {
        Py_XDECREF(__pyx_t_13); Py_XDECREF(__pyx_t_14); Py_XDECREF(__pyx_t_15);
        __Pyx_RaiseBufferFallbackError();
      } else {
        PyErr_Restore(__pyx_t_13, __pyx_t_14, __pyx_t_15);
      }
      __pyx_t_13 = __pyx_t_14 = __pyx_t_15 = 0;
    }
    __pyx_pybuffernd_dd.diminfo[0].strides = __pyx_pybuffernd_dd.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_dd.diminfo[0].shape = __pyx_pybuffernd_dd.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_dd.diminfo[1].strides = __pyx_pybuffernd_dd.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_dd.diminfo[1].shape = __pyx_pybuffernd_dd.rcbuffer->pybuffer.shape[1];
    if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 1265, __pyx_L1_error)
  }
  __pyx_t_16 = 0;
  __pyx_v_dd = ((PyArrayObject *)__pyx_t_10);
  __pyx_t_10 = 0;

  /* "pyart/map/ckdtree.pyx":1266
 *         xx = np.ascontiguousarray(xx,dtype=np.float64)
 *         dd = np.empty((n,k),dtype=np.float64)
 *         dd.fill(infinity)             # <<<<<<<<<<<<<<
 *         ii = np.empty((n,k),dtype=np.intp)
 *         ii.fill(self.n)
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_dd), __pyx_n_s_fill); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1266, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_5pyart_3map_7ckdtree_infinity); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1266, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_10 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1266, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "pyart/map/ckdtree.pyx":1267
 *         dd = np.empty((n,k),dtype=np.float64)
 *         dd.fill(infinity)
 *         ii = np.empty((n,k),dtype=np.intp)             # <<<<<<<<<<<<<<
 *         ii.fill(self.n)
 *         for c in range(n):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_np); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_k); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_10);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);
  __pyx_t_10 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_np); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_intp); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_1) < 0) __PYX_ERR(0, 1267, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1267, __pyx_L1_error)
  __pyx_t_17 = ((PyArrayObject *)__pyx_t_1);
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ii.rcbuffer->pybuffer);
    __pyx_t_11 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_ii.rcbuffer->pybuffer, (PyObject*)__pyx_t_17, &__Pyx_TypeInfo_nn___pyx_t_5numpy_intp_t, PyBUF_FORMAT| PyBUF_STRIDES, 2, 0, __pyx_stack);
    if (unlikely(__pyx_t_11 < 0)) {
      PyErr_Fetch(&__pyx_t_15, &__pyx_t_14, &__pyx_t_13);
      if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_ii.rcbuffer->pybuffer, (PyObject*)__pyx_v_ii, &__Pyx_TypeInfo_nn___pyx_t_5numpy_intp_t, PyBUF_FORMAT| PyBUF_STRIDES, 2, 0, __pyx_stack) == -1)) {
        Py_XDECREF(__pyx_t_15); Py_XDECREF(__pyx_t_14); Py_XDECREF(__pyx_t_13);
        __Pyx_RaiseBufferFallbackError();
      } else {
        PyErr_Restore(__pyx_t_15, __pyx_t_14, __pyx_t_13);
      }
      __pyx_t_15 = __pyx_t_14 = __pyx_t_13 = 0;
    }
    __pyx_pybuffernd_ii.diminfo[0].strides = __pyx_pybuffernd_ii.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_ii.diminfo[0].shape = __pyx_pybuffernd_ii.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_ii.diminfo[1].strides = __pyx_pybuffernd_ii.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_ii.diminfo[1].shape = __pyx_pybuffernd_ii.rcbuffer->pybuffer.shape[1];
    if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 1267, __pyx_L1_error)
  }
  __pyx_t_17 = 0;
  __pyx_v_ii = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/map/ckdtree.pyx":1268
 *         dd.fill(infinity)
 *         ii = np.empty((n,k),dtype=np.intp)
 *         ii.fill(self.n)             # <<<<<<<<<<<<<<
 *         for c in range(n):
 *             self.__query(&dd[c, 0], &ii[c, 0], &xx[c, 0],
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_ii), __pyx_n_s_fill); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1268, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->n); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1268, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1268, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/map/ckdtree.pyx":1269
 *         ii = np.empty((n,k),dtype=np.intp)
 *         ii.fill(self.n)
 *         for c in range(n):             # <<<<<<<<<<<<<<
 *             self.__query(&dd[c, 0], &ii[c, 0], &xx[c, 0],
 *                          k, eps, p, distance_upper_bound)
 */
  __pyx_t_9 = __pyx_v_n;
  __pyx_t_18 = __pyx_t_9;
  for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
    __pyx_v_c = __pyx_t_19;

    /* "pyart/map/ckdtree.pyx":1270
 *         ii.fill(self.n)
 *         for c in range(n):
 *             self.__query(&dd[c, 0], &ii[c, 0], &xx[c, 0],             # <<<<<<<<<<<<<<
 *                          k, eps, p, distance_upper_bound)
 * 
 */
    __pyx_t_20 = __pyx_v_c;
    __pyx_t_21 = 0;
    __pyx_t_22 = __pyx_v_c;
    __pyx_t_23 = 0;
    __pyx_t_24 = __pyx_v_c;
    __pyx_t_25 = 0;

    /* "pyart/map/ckdtree.pyx":1271
 *         for c in range(n):
 *             self.__query(&dd[c, 0], &ii[c, 0], &xx[c, 0],
 *                          k, eps, p, distance_upper_bound)             # <<<<<<<<<<<<<<
 * 
 *         if single:
 */
    __pyx_t_11 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query(__pyx_v_self, (&(*__Pyx_BufPtrStrided2d(__pyx_t_5numpy_float64_t *, __pyx_pybuffernd_dd.rcbuffer->pybuffer.buf, __pyx_t_20, __pyx_pybuffernd_dd.diminfo[0].strides, __pyx_t_21, __pyx_pybuffernd_dd.diminfo[1].strides))), (&(*__Pyx_BufPtrStrided2d(__pyx_t_5numpy_intp_t *, __pyx_pybuffernd_ii.rcbuffer->pybuffer.buf, __pyx_t_22, __pyx_pybuffernd_ii.diminfo[0].strides, __pyx_t_23, __pyx_pybuffernd_ii.diminfo[1].strides))), (&(*__Pyx_BufPtrStrided2d(__pyx_t_5numpy_float64_t *, __pyx_pybuffernd_xx.rcbuffer->pybuffer.buf, __pyx_t_24, __pyx_pybuffernd_xx.diminfo[0].strides, __pyx_t_25, __pyx_pybuffernd_xx.diminfo[1].strides))), __pyx_v_k, __pyx_v_eps, __pyx_v_p, __pyx_v_distance_upper_bound); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(0, 1270, __pyx_L1_error)
  }

  /* "pyart/map/ckdtree.pyx":1273
 *                          k, eps, p, distance_upper_bound)
 * 
 *         if single:             # <<<<<<<<<<<<<<
 *             if k==1:
 *                 if sizeof(long) < sizeof(np.intp_t):
 */
  __pyx_t_7 = (__pyx_v_single != 0);
  if (__pyx_t_7) {

    /* "pyart/map/ckdtree.pyx":1274
 * 
 *         if single:
 *             if k==1:             # <<<<<<<<<<<<<<
 *                 if sizeof(long) < sizeof(np.intp_t):
 *                     # ... e.g. Windows 64
 */
    __pyx_t_7 = ((__pyx_v_k == 1) != 0);
    if (__pyx_t_7) {

      /* "pyart/map/ckdtree.pyx":1275
 *         if single:
 *             if k==1:
 *                 if sizeof(long) < sizeof(np.intp_t):             # <<<<<<<<<<<<<<
 *                     # ... e.g. Windows 64
 *                     if ii[0,0] <= <np.intp_t>LONG_MAX:
 */
      __pyx_t_7 = (((sizeof(long)) < (sizeof(__pyx_t_5numpy_intp_t))) != 0);
      if (__pyx_t_7) {

        /* "pyart/map/ckdtree.pyx":1277
 *                 if sizeof(long) < sizeof(np.intp_t):
 *                     # ... e.g. Windows 64
 *                     if ii[0,0] <= <np.intp_t>LONG_MAX:             # <<<<<<<<<<<<<<
 *                         return dd[0,0], int(ii[0,0])
 *                     else:
 */
        __pyx_t_25 = 0;
        __pyx_t_24 = 0;
        __pyx_t_7 = (((*__Pyx_BufPtrStrided2d(__pyx_t_5numpy_intp_t *, __pyx_pybuffernd_ii.rcbuffer->pybuffer.buf, __pyx_t_25, __pyx_pybuffernd_ii.diminfo[0].strides, __pyx_t_24, __pyx_pybuffernd_ii.diminfo[1].strides)) <= ((__pyx_t_5numpy_intp_t)LONG_MAX)) != 0);
        if (__pyx_t_7) {

          /* "pyart/map/ckdtree.pyx":1278
 *                     # ... e.g. Windows 64
 *                     if ii[0,0] <= <np.intp_t>LONG_MAX:
 *                         return dd[0,0], int(ii[0,0])             # <<<<<<<<<<<<<<
 *                     else:
 *                         return dd[0,0], ii[0,0]
 */
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_24 = 0;
          __pyx_t_25 = 0;
          __pyx_t_1 = PyFloat_FromDouble((*__Pyx_BufPtrStrided2d(__pyx_t_5numpy_float64_t *, __pyx_pybuffernd_dd.rcbuffer->pybuffer.buf, __pyx_t_24, __pyx_pybuffernd_dd.diminfo[0].strides, __pyx_t_25, __pyx_pybuffernd_dd.diminfo[1].strides))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1278, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_25 = 0;
          __pyx_t_24 = 0;
          __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t((*__Pyx_BufPtrStrided2d(__pyx_t_5numpy_intp_t *, __pyx_pybuffernd_ii.rcbuffer->pybuffer.buf, __pyx_t_25, __pyx_pybuffernd_ii.diminfo[0].strides, __pyx_t_24, __pyx_pybuffernd_ii.diminfo[1].strides))); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1278, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyInt_Type)), __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1278, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1278, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_GIVEREF(__pyx_t_1);
          PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
          __Pyx_GIVEREF(__pyx_t_2);
          PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);
          __pyx_t_1 = 0;
          __pyx_t_2 = 0;
          __pyx_r = __pyx_t_4;
          __pyx_t_4 = 0;
          goto __pyx_L0;

          /* "pyart/map/ckdtree.pyx":1277
 *                 if sizeof(long) < sizeof(np.intp_t):
 *                     # ... e.g. Windows 64
 *                     if ii[0,0] <= <np.intp_t>LONG_MAX:             # <<<<<<<<<<<<<<
 *                         return dd[0,0], int(ii[0,0])
 *                     else:
 */
        }

        /* "pyart/map/ckdtree.pyx":1280
 *                         return dd[0,0], int(ii[0,0])
 *                     else:
 *                         return dd[0,0], ii[0,0]             # <<<<<<<<<<<<<<
 *                 else:
 *                     # ... most other platforms
 */
        /*else*/ {
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_24 = 0;
          __pyx_t_25 = 0;
          __pyx_t_4 = PyFloat_FromDouble((*__Pyx_BufPtrStrided2d(__pyx_t_5numpy_float64_t *, __pyx_pybuffernd_dd.rcbuffer->pybuffer.buf, __pyx_t_24, __pyx_pybuffernd_dd.diminfo[0].strides, __pyx_t_25, __pyx_pybuffernd_dd.diminfo[1].strides))); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1280, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_25 = 0;
          __pyx_t_24 = 0;
          __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t((*__Pyx_BufPtrStrided2d(__pyx_t_5numpy_intp_t *, __pyx_pybuffernd_ii.rcbuffer->pybuffer.buf, __pyx_t_25, __pyx_pybuffernd_ii.diminfo[0].strides, __pyx_t_24, __pyx_pybuffernd_ii.diminfo[1].strides))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1280, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1280, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_GIVEREF(__pyx_t_4);
          PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_4);
          __Pyx_GIVEREF(__pyx_t_2);
          PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_2);
          __pyx_t_4 = 0;
          __pyx_t_2 = 0;
          __pyx_r = __pyx_t_1;
          __pyx_t_1 = 0;
          goto __pyx_L0;
        }

        /* "pyart/map/ckdtree.pyx":1275
 *         if single:
 *             if k==1:
 *                 if sizeof(long) < sizeof(np.intp_t):             # <<<<<<<<<<<<<<
 *                     # ... e.g. Windows 64
 *                     if ii[0,0] <= <np.intp_t>LONG_MAX:
 */
      }

      /* "pyart/map/ckdtree.pyx":1283
 *                 else:
 *                     # ... most other platforms
 *                     return dd[0,0], ii[0,0]             # <<<<<<<<<<<<<<
 *             else:
 *                 return dd[0], ii[0]
 */
      /*else*/ {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_24 = 0;
        __pyx_t_25 = 0;
        __pyx_t_1 = PyFloat_FromDouble((*__Pyx_BufPtrStrided2d(__pyx_t_5numpy_float64_t *, __pyx_pybuffernd_dd.rcbuffer->pybuffer.buf, __pyx_t_24, __pyx_pybuffernd_dd.diminfo[0].strides, __pyx_t_25, __pyx_pybuffernd_dd.diminfo[1].strides))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1283, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_25 = 0;
        __pyx_t_24 = 0;
        __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t((*__Pyx_BufPtrStrided2d(__pyx_t_5numpy_intp_t *, __pyx_pybuffernd_ii.rcbuffer->pybuffer.buf, __pyx_t_25, __pyx_pybuffernd_ii.diminfo[0].strides, __pyx_t_24, __pyx_pybuffernd_ii.diminfo[1].strides))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1283, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1283, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_1);
        PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
        __Pyx_GIVEREF(__pyx_t_2);
        PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);
        __pyx_t_1 = 0;
        __pyx_t_2 = 0;
        __pyx_r = __pyx_t_4;
        __pyx_t_4 = 0;
        goto __pyx_L0;
      }

      /* "pyart/map/ckdtree.pyx":1274
 * 
 *         if single:
 *             if k==1:             # <<<<<<<<<<<<<<
 *                 if sizeof(long) < sizeof(np.intp_t):
 *                     # ... e.g. Windows 64
 */
    }

    /* "pyart/map/ckdtree.pyx":1285
 *                     return dd[0,0], ii[0,0]
 *             else:
 *                 return dd[0], ii[0]             # <<<<<<<<<<<<<<
 *         else:
 *             if sizeof(long) < sizeof(np.intp_t):
 */
    /*else*/ {
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_4 = __Pyx_GetItemInt(((PyObject *)__pyx_v_dd), 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1285, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_GetItemInt(((PyObject *)__pyx_v_ii), 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1285, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1285, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_2);
      PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_2);
      __pyx_t_4 = 0;
      __pyx_t_2 = 0;
      __pyx_r = __pyx_t_1;
      __pyx_t_1 = 0;
      goto __pyx_L0;
    }

    /* "pyart/map/ckdtree.pyx":1273
 *                          k, eps, p, distance_upper_bound)
 * 
 *         if single:             # <<<<<<<<<<<<<<
 *             if k==1:
 *                 if sizeof(long) < sizeof(np.intp_t):
 */
  }

  /* "pyart/map/ckdtree.pyx":1287
 *                 return dd[0], ii[0]
 *         else:
 *             if sizeof(long) < sizeof(np.intp_t):             # <<<<<<<<<<<<<<
 *                 # ... e.g. Windows 64
 *                 for i in range(n):
 */
  /*else*/ {
    __pyx_t_7 = (((sizeof(long)) < (sizeof(__pyx_t_5numpy_intp_t))) != 0);
    if (__pyx_t_7) {

      /* "pyart/map/ckdtree.pyx":1289
 *             if sizeof(long) < sizeof(np.intp_t):
 *                 # ... e.g. Windows 64
 *                 for i in range(n):             # <<<<<<<<<<<<<<
 *                     for j in range(k):
 *                         if ii[i,j] > <np.intp_t>LONG_MAX:
 */
      __pyx_t_9 = __pyx_v_n;
      __pyx_t_18 = __pyx_t_9;
      for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
        __pyx_v_i = __pyx_t_19;

        /* "pyart/map/ckdtree.pyx":1290
 *                 # ... e.g. Windows 64
 *                 for i in range(n):
 *                     for j in range(k):             # <<<<<<<<<<<<<<
 *                         if ii[i,j] > <np.intp_t>LONG_MAX:
 *                             # C long overlow, return array of dtype=np.int_p
 */
        __pyx_t_26 = __pyx_v_k;
        __pyx_t_27 = __pyx_t_26;
        for (__pyx_t_28 = 0; __pyx_t_28 < __pyx_t_27; __pyx_t_28+=1) {
          __pyx_v_j = __pyx_t_28;

          /* "pyart/map/ckdtree.pyx":1291
 *                 for i in range(n):
 *                     for j in range(k):
 *                         if ii[i,j] > <np.intp_t>LONG_MAX:             # <<<<<<<<<<<<<<
 *                             # C long overlow, return array of dtype=np.int_p
 *                             if k==1:
 */
          __pyx_t_24 = __pyx_v_i;
          __pyx_t_25 = __pyx_v_j;
          __pyx_t_7 = (((*__Pyx_BufPtrStrided2d(__pyx_t_5numpy_intp_t *, __pyx_pybuffernd_ii.rcbuffer->pybuffer.buf, __pyx_t_24, __pyx_pybuffernd_ii.diminfo[0].strides, __pyx_t_25, __pyx_pybuffernd_ii.diminfo[1].strides)) > ((__pyx_t_5numpy_intp_t)LONG_MAX)) != 0);
          if (__pyx_t_7) {

            /* "pyart/map/ckdtree.pyx":1293
 *                         if ii[i,j] > <np.intp_t>LONG_MAX:
 *                             # C long overlow, return array of dtype=np.int_p
 *                             if k==1:             # <<<<<<<<<<<<<<
 *                                 return np.reshape(dd[...,0],retshape), np.reshape(ii[...,0],retshape)
 *                             else:
 */
            __pyx_t_7 = ((__pyx_v_k == 1) != 0);
            if (__pyx_t_7) {

              /* "pyart/map/ckdtree.pyx":1294
 *                             # C long overlow, return array of dtype=np.int_p
 *                             if k==1:
 *                                 return np.reshape(dd[...,0],retshape), np.reshape(ii[...,0],retshape)             # <<<<<<<<<<<<<<
 *                             else:
 *                                 return np.reshape(dd,retshape+(k,)), np.reshape(ii,retshape+(k,))
 */
              __Pyx_XDECREF(__pyx_r);
              __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1294, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_2);
              __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_reshape); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1294, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_4);
              __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
              __pyx_t_2 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_dd), __pyx_tuple__17); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1294, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_2);
              __pyx_t_3 = NULL;
              __pyx_t_11 = 0;
              if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
                __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
                if (likely(__pyx_t_3)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
                  __Pyx_INCREF(__pyx_t_3);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_4, function);
                  __pyx_t_11 = 1;
                }
              }
              #if CYTHON_FAST_PYCALL
              if (PyFunction_Check(__pyx_t_4)) {
                PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_t_2, __pyx_v_retshape};
                __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1294, __pyx_L1_error)
                __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
                __Pyx_GOTREF(__pyx_t_1);
                __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
              } else
              #endif
              #if CYTHON_FAST_PYCCALL
              if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
                PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_t_2, __pyx_v_retshape};
                __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1294, __pyx_L1_error)
                __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
                __Pyx_GOTREF(__pyx_t_1);
                __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
              } else
              #endif
              {
                __pyx_t_10 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1294, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_10);
                if (__pyx_t_3) {
                  __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_3); __pyx_t_3 = NULL;
                }
                __Pyx_GIVEREF(__pyx_t_2);
                PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_11, __pyx_t_2);
                __Pyx_INCREF(__pyx_v_retshape);
                __Pyx_GIVEREF(__pyx_v_retshape);
                PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_11, __pyx_v_retshape);
                __pyx_t_2 = 0;
                __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_10, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1294, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_1);
                __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              }
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
              __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_np); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1294, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_10);
              __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_reshape); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1294, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_2);
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              __pyx_t_10 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_ii), __pyx_tuple__17); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1294, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_10);
              __pyx_t_3 = NULL;
              __pyx_t_11 = 0;
              if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
                __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
                if (likely(__pyx_t_3)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
                  __Pyx_INCREF(__pyx_t_3);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_2, function);
                  __pyx_t_11 = 1;
                }
              }
              #if CYTHON_FAST_PYCALL
              if (PyFunction_Check(__pyx_t_2)) {
                PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_t_10, __pyx_v_retshape};
                __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1294, __pyx_L1_error)
                __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
                __Pyx_GOTREF(__pyx_t_4);
                __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              } else
              #endif
              #if CYTHON_FAST_PYCCALL
              if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
                PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_t_10, __pyx_v_retshape};
                __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1294, __pyx_L1_error)
                __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
                __Pyx_GOTREF(__pyx_t_4);
                __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              } else
              #endif
              {
                __pyx_t_29 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_29)) __PYX_ERR(0, 1294, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_29);
                if (__pyx_t_3) {
                  __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_29, 0, __pyx_t_3); __pyx_t_3 = NULL;
                }
                __Pyx_GIVEREF(__pyx_t_10);
                PyTuple_SET_ITEM(__pyx_t_29, 0+__pyx_t_11, __pyx_t_10);
                __Pyx_INCREF(__pyx_v_retshape);
                __Pyx_GIVEREF(__pyx_v_retshape);
                PyTuple_SET_ITEM(__pyx_t_29, 1+__pyx_t_11, __pyx_v_retshape);
                __pyx_t_10 = 0;
                __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_29, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1294, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_4);
                __Pyx_DECREF(__pyx_t_29); __pyx_t_29 = 0;
              }
              __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
              __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1294, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_2);
              __Pyx_GIVEREF(__pyx_t_1);
              PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
              __Pyx_GIVEREF(__pyx_t_4);
              PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
              __pyx_t_1 = 0;
              __pyx_t_4 = 0;
              __pyx_r = __pyx_t_2;
              __pyx_t_2 = 0;
              goto __pyx_L0;

              /* "pyart/map/ckdtree.pyx":1293
 *                         if ii[i,j] > <np.intp_t>LONG_MAX:
 *                             # C long overlow, return array of dtype=np.int_p
 *                             if k==1:             # <<<<<<<<<<<<<<
 *                                 return np.reshape(dd[...,0],retshape), np.reshape(ii[...,0],retshape)
 *                             else:
 */
            }

            /* "pyart/map/ckdtree.pyx":1296
 *                                 return np.reshape(dd[...,0],retshape), np.reshape(ii[...,0],retshape)
 *                             else:
 *                                 return np.reshape(dd,retshape+(k,)), np.reshape(ii,retshape+(k,))             # <<<<<<<<<<<<<<
 * 
 *                 # no C long overlow, return array of dtype=int
 */
            /*else*/ {
              __Pyx_XDECREF(__pyx_r);
              __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1296, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_4);
              __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_reshape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1296, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
              __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_k); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1296, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_4);
              __pyx_t_29 = PyTuple_New(1); if (unlikely(!__pyx_t_29)) __PYX_ERR(0, 1296, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_29);
              __Pyx_GIVEREF(__pyx_t_4);
              PyTuple_SET_ITEM(__pyx_t_29, 0, __pyx_t_4);
              __pyx_t_4 = 0;
              __pyx_t_4 = PyNumber_Add(__pyx_v_retshape, __pyx_t_29); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1296, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_4);
              __Pyx_DECREF(__pyx_t_29); __pyx_t_29 = 0;
              __pyx_t_29 = NULL;
              __pyx_t_11 = 0;
              if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
                __pyx_t_29 = PyMethod_GET_SELF(__pyx_t_1);
                if (likely(__pyx_t_29)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
                  __Pyx_INCREF(__pyx_t_29);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_1, function);
                  __pyx_t_11 = 1;
                }
              }
              #if CYTHON_FAST_PYCALL
              if (PyFunction_Check(__pyx_t_1)) {
                PyObject *__pyx_temp[3] = {__pyx_t_29, ((PyObject *)__pyx_v_dd), __pyx_t_4};
                __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1296, __pyx_L1_error)
                __Pyx_XDECREF(__pyx_t_29); __pyx_t_29 = 0;
                __Pyx_GOTREF(__pyx_t_2);
                __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
              } else
              #endif
              #if CYTHON_FAST_PYCCALL
              if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
                PyObject *__pyx_temp[3] = {__pyx_t_29, ((PyObject *)__pyx_v_dd), __pyx_t_4};
                __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1296, __pyx_L1_error)
                __Pyx_XDECREF(__pyx_t_29); __pyx_t_29 = 0;
                __Pyx_GOTREF(__pyx_t_2);
                __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
              } else
              #endif
              {
                __pyx_t_10 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1296, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_10);
                if (__pyx_t_29) {
                  __Pyx_GIVEREF(__pyx_t_29); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_29); __pyx_t_29 = NULL;
                }
                __Pyx_INCREF(((PyObject *)__pyx_v_dd));
                __Pyx_GIVEREF(((PyObject *)__pyx_v_dd));
                PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_11, ((PyObject *)__pyx_v_dd));
                __Pyx_GIVEREF(__pyx_t_4);
                PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_11, __pyx_t_4);
                __pyx_t_4 = 0;
                __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_10, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1296, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_2);
                __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              }
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
              __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_np); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1296, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_10);
              __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_reshape); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1296, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_4);
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              __pyx_t_10 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_k); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1296, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_10);
              __pyx_t_29 = PyTuple_New(1); if (unlikely(!__pyx_t_29)) __PYX_ERR(0, 1296, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_29);
              __Pyx_GIVEREF(__pyx_t_10);
              PyTuple_SET_ITEM(__pyx_t_29, 0, __pyx_t_10);
              __pyx_t_10 = 0;
              __pyx_t_10 = PyNumber_Add(__pyx_v_retshape, __pyx_t_29); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1296, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_10);
              __Pyx_DECREF(__pyx_t_29); __pyx_t_29 = 0;
              __pyx_t_29 = NULL;
              __pyx_t_11 = 0;
              if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
                __pyx_t_29 = PyMethod_GET_SELF(__pyx_t_4);
                if (likely(__pyx_t_29)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
                  __Pyx_INCREF(__pyx_t_29);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_4, function);
                  __pyx_t_11 = 1;
                }
              }
              #if CYTHON_FAST_PYCALL
              if (PyFunction_Check(__pyx_t_4)) {
                PyObject *__pyx_temp[3] = {__pyx_t_29, ((PyObject *)__pyx_v_ii), __pyx_t_10};
                __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1296, __pyx_L1_error)
                __Pyx_XDECREF(__pyx_t_29); __pyx_t_29 = 0;
                __Pyx_GOTREF(__pyx_t_1);
                __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              } else
              #endif
              #if CYTHON_FAST_PYCCALL
              if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
                PyObject *__pyx_temp[3] = {__pyx_t_29, ((PyObject *)__pyx_v_ii), __pyx_t_10};
                __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1296, __pyx_L1_error)
                __Pyx_XDECREF(__pyx_t_29); __pyx_t_29 = 0;
                __Pyx_GOTREF(__pyx_t_1);
                __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              } else
              #endif
              {
                __pyx_t_3 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1296, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_3);
                if (__pyx_t_29) {
                  __Pyx_GIVEREF(__pyx_t_29); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_29); __pyx_t_29 = NULL;
                }
                __Pyx_INCREF(((PyObject *)__pyx_v_ii));
                __Pyx_GIVEREF(((PyObject *)__pyx_v_ii));
                PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_11, ((PyObject *)__pyx_v_ii));
                __Pyx_GIVEREF(__pyx_t_10);
                PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_11, __pyx_t_10);
                __pyx_t_10 = 0;
                __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1296, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_1);
                __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
              }
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
              __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1296, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_4);
              __Pyx_GIVEREF(__pyx_t_2);
              PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2);
              __Pyx_GIVEREF(__pyx_t_1);
              PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
              __pyx_t_2 = 0;
              __pyx_t_1 = 0;
              __pyx_r = __pyx_t_4;
              __pyx_t_4 = 0;
              goto __pyx_L0;
            }

            /* "pyart/map/ckdtree.pyx":1291
 *                 for i in range(n):
 *                     for j in range(k):
 *                         if ii[i,j] > <np.intp_t>LONG_MAX:             # <<<<<<<<<<<<<<
 *                             # C long overlow, return array of dtype=np.int_p
 *                             if k==1:
 */
          }
        }
      }

      /* "pyart/map/ckdtree.pyx":1299
 * 
 *                 # no C long overlow, return array of dtype=int
 *                 if k==1:             # <<<<<<<<<<<<<<
 *                     return np.reshape(dd[...,0],retshape), np.reshape(ii[...,0],retshape).astype(int)
 *                 else:
 */
      __pyx_t_7 = ((__pyx_v_k == 1) != 0);
      if (__pyx_t_7) {

        /* "pyart/map/ckdtree.pyx":1300
 *                 # no C long overlow, return array of dtype=int
 *                 if k==1:
 *                     return np.reshape(dd[...,0],retshape), np.reshape(ii[...,0],retshape).astype(int)             # <<<<<<<<<<<<<<
 *                 else:
 *                     return np.reshape(dd,retshape+(k,)), np.reshape(ii,retshape+(k,)).astype(int)
 */
        __Pyx_XDECREF(__pyx_r);
        __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1300, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_reshape); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1300, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_dd), __pyx_tuple__17); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1300, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_3 = NULL;
        __pyx_t_11 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
          if (likely(__pyx_t_3)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
            __Pyx_INCREF(__pyx_t_3);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_2, function);
            __pyx_t_11 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_2)) {
          PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_t_1, __pyx_v_retshape};
          __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1300, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
          PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_t_1, __pyx_v_retshape};
          __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1300, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        } else
        #endif
        {
          __pyx_t_10 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1300, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          if (__pyx_t_3) {
            __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_3); __pyx_t_3 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_1);
          PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_11, __pyx_t_1);
          __Pyx_INCREF(__pyx_v_retshape);
          __Pyx_GIVEREF(__pyx_v_retshape);
          PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_11, __pyx_v_retshape);
          __pyx_t_1 = 0;
          __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_10, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1300, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        }
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1300, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_reshape); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1300, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_ii), __pyx_tuple__17); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1300, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_29 = NULL;
        __pyx_t_11 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_29 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_29)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_29);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_11 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_29, __pyx_t_1, __pyx_v_retshape};
          __pyx_t_10 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1300, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_29); __pyx_t_29 = 0;
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_29, __pyx_t_1, __pyx_v_retshape};
          __pyx_t_10 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1300, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_29); __pyx_t_29 = 0;
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        } else
        #endif
        {
          __pyx_t_30 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_30)) __PYX_ERR(0, 1300, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_30);
          if (__pyx_t_29) {
            __Pyx_GIVEREF(__pyx_t_29); PyTuple_SET_ITEM(__pyx_t_30, 0, __pyx_t_29); __pyx_t_29 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_1);
          PyTuple_SET_ITEM(__pyx_t_30, 0+__pyx_t_11, __pyx_t_1);
          __Pyx_INCREF(__pyx_v_retshape);
          __Pyx_GIVEREF(__pyx_v_retshape);
          PyTuple_SET_ITEM(__pyx_t_30, 1+__pyx_t_11, __pyx_v_retshape);
          __pyx_t_1 = 0;
          __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_30, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1300, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_30); __pyx_t_30 = 0;
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_astype); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1300, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_t_10 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_10)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_10);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_10, ((PyObject *)(&PyInt_Type))) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)(&PyInt_Type)));
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1300, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1300, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_2);
        PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
        __pyx_t_4 = 0;
        __pyx_t_2 = 0;
        __pyx_r = __pyx_t_3;
        __pyx_t_3 = 0;
        goto __pyx_L0;

        /* "pyart/map/ckdtree.pyx":1299
 * 
 *                 # no C long overlow, return array of dtype=int
 *                 if k==1:             # <<<<<<<<<<<<<<
 *                     return np.reshape(dd[...,0],retshape), np.reshape(ii[...,0],retshape).astype(int)
 *                 else:
 */
      }

      /* "pyart/map/ckdtree.pyx":1302
 *                     return np.reshape(dd[...,0],retshape), np.reshape(ii[...,0],retshape).astype(int)
 *                 else:
 *                     return np.reshape(dd,retshape+(k,)), np.reshape(ii,retshape+(k,)).astype(int)             # <<<<<<<<<<<<<<
 * 
 *             else:
 */
      /*else*/ {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1302, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_reshape); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1302, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_k); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1302, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_10 = PyTuple_New(1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1302, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_GIVEREF(__pyx_t_2);
        PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_2);
        __pyx_t_2 = 0;
        __pyx_t_2 = PyNumber_Add(__pyx_v_retshape, __pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1302, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_t_10 = NULL;
        __pyx_t_11 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_10)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_10);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
            __pyx_t_11 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[3] = {__pyx_t_10, ((PyObject *)__pyx_v_dd), __pyx_t_2};
          __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1302, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[3] = {__pyx_t_10, ((PyObject *)__pyx_v_dd), __pyx_t_2};
          __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1302, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        } else
        #endif
        {
          __pyx_t_30 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_30)) __PYX_ERR(0, 1302, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_30);
          if (__pyx_t_10) {
            __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_30, 0, __pyx_t_10); __pyx_t_10 = NULL;
          }
          __Pyx_INCREF(((PyObject *)__pyx_v_dd));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_dd));
          PyTuple_SET_ITEM(__pyx_t_30, 0+__pyx_t_11, ((PyObject *)__pyx_v_dd));
          __Pyx_GIVEREF(__pyx_t_2);
          PyTuple_SET_ITEM(__pyx_t_30, 1+__pyx_t_11, __pyx_t_2);
          __pyx_t_2 = 0;
          __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_30, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1302, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_30); __pyx_t_30 = 0;
        }
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1302, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_reshape); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1302, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_k); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1302, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1302, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_GIVEREF(__pyx_t_2);
        PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
        __pyx_t_2 = 0;
        __pyx_t_2 = PyNumber_Add(__pyx_v_retshape, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1302, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_1 = NULL;
        __pyx_t_11 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
          __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_10);
          if (likely(__pyx_t_1)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
            __Pyx_INCREF(__pyx_t_1);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_10, function);
            __pyx_t_11 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_10)) {
          PyObject *__pyx_temp[3] = {__pyx_t_1, ((PyObject *)__pyx_v_ii), __pyx_t_2};
          __pyx_t_30 = __Pyx_PyFunction_FastCall(__pyx_t_10, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_30)) __PYX_ERR(0, 1302, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_GOTREF(__pyx_t_30);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_10)) {
          PyObject *__pyx_temp[3] = {__pyx_t_1, ((PyObject *)__pyx_v_ii), __pyx_t_2};
          __pyx_t_30 = __Pyx_PyCFunction_FastCall(__pyx_t_10, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_30)) __PYX_ERR(0, 1302, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_GOTREF(__pyx_t_30);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        } else
        #endif
        {
          __pyx_t_29 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_29)) __PYX_ERR(0, 1302, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_29);
          if (__pyx_t_1) {
            __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_29, 0, __pyx_t_1); __pyx_t_1 = NULL;
          }
          __Pyx_INCREF(((PyObject *)__pyx_v_ii));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_ii));
          PyTuple_SET_ITEM(__pyx_t_29, 0+__pyx_t_11, ((PyObject *)__pyx_v_ii));
          __Pyx_GIVEREF(__pyx_t_2);
          PyTuple_SET_ITEM(__pyx_t_29, 1+__pyx_t_11, __pyx_t_2);
          __pyx_t_2 = 0;
          __pyx_t_30 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_t_29, NULL); if (unlikely(!__pyx_t_30)) __PYX_ERR(0, 1302, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_30);
          __Pyx_DECREF(__pyx_t_29); __pyx_t_29 = 0;
        }
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_30, __pyx_n_s_astype); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1302, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_30); __pyx_t_30 = 0;
        __pyx_t_30 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
          __pyx_t_30 = PyMethod_GET_SELF(__pyx_t_10);
          if (likely(__pyx_t_30)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
            __Pyx_INCREF(__pyx_t_30);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_10, function);
          }
        }
        __pyx_t_4 = (__pyx_t_30) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_30, ((PyObject *)(&PyInt_Type))) : __Pyx_PyObject_CallOneArg(__pyx_t_10, ((PyObject *)(&PyInt_Type)));
        __Pyx_XDECREF(__pyx_t_30); __pyx_t_30 = 0;
        if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1302, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_t_10 = PyTuple_New(2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1302, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_4);
        __pyx_t_3 = 0;
        __pyx_t_4 = 0;
        __pyx_r = __pyx_t_10;
        __pyx_t_10 = 0;
        goto __pyx_L0;
      }

      /* "pyart/map/ckdtree.pyx":1287
 *                 return dd[0], ii[0]
 *         else:
 *             if sizeof(long) < sizeof(np.intp_t):             # <<<<<<<<<<<<<<
 *                 # ... e.g. Windows 64
 *                 for i in range(n):
 */
    }

    /* "pyart/map/ckdtree.pyx":1306
 *             else:
 *                 # ... most other platforms
 *                 if k==1:             # <<<<<<<<<<<<<<
 *                     return np.reshape(dd[...,0],retshape), np.reshape(ii[...,0],retshape)
 *                 else:
 */
    /*else*/ {
      __pyx_t_7 = ((__pyx_v_k == 1) != 0);
      if (__pyx_t_7) {

        /* "pyart/map/ckdtree.pyx":1307
 *                 # ... most other platforms
 *                 if k==1:
 *                     return np.reshape(dd[...,0],retshape), np.reshape(ii[...,0],retshape)             # <<<<<<<<<<<<<<
 *                 else:
 *                     return np.reshape(dd,retshape+(k,)), np.reshape(ii,retshape+(k,))
 */
        __Pyx_XDECREF(__pyx_r);
        __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1307, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_reshape); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1307, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_4 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_dd), __pyx_tuple__17); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1307, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_30 = NULL;
        __pyx_t_11 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_30 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_30)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_30);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_11 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_30, __pyx_t_4, __pyx_v_retshape};
          __pyx_t_10 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1307, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_30); __pyx_t_30 = 0;
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_30, __pyx_t_4, __pyx_v_retshape};
          __pyx_t_10 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1307, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_30); __pyx_t_30 = 0;
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        {
          __pyx_t_29 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_29)) __PYX_ERR(0, 1307, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_29);
          if (__pyx_t_30) {
            __Pyx_GIVEREF(__pyx_t_30); PyTuple_SET_ITEM(__pyx_t_29, 0, __pyx_t_30); __pyx_t_30 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_4);
          PyTuple_SET_ITEM(__pyx_t_29, 0+__pyx_t_11, __pyx_t_4);
          __Pyx_INCREF(__pyx_v_retshape);
          __Pyx_GIVEREF(__pyx_v_retshape);
          PyTuple_SET_ITEM(__pyx_t_29, 1+__pyx_t_11, __pyx_v_retshape);
          __pyx_t_4 = 0;
          __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_29, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1307, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_29); __pyx_t_29 = 0;
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GetModuleGlobalName(__pyx_t_29, __pyx_n_s_np); if (unlikely(!__pyx_t_29)) __PYX_ERR(0, 1307, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_29);
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_29, __pyx_n_s_reshape); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1307, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_29); __pyx_t_29 = 0;
        __pyx_t_29 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_ii), __pyx_tuple__17); if (unlikely(!__pyx_t_29)) __PYX_ERR(0, 1307, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_29);
        __pyx_t_30 = NULL;
        __pyx_t_11 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_30 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_30)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_30);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
            __pyx_t_11 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[3] = {__pyx_t_30, __pyx_t_29, __pyx_v_retshape};
          __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1307, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_30); __pyx_t_30 = 0;
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_29); __pyx_t_29 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[3] = {__pyx_t_30, __pyx_t_29, __pyx_v_retshape};
          __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1307, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_30); __pyx_t_30 = 0;
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_29); __pyx_t_29 = 0;
        } else
        #endif
        {
          __pyx_t_2 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1307, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          if (__pyx_t_30) {
            __Pyx_GIVEREF(__pyx_t_30); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_30); __pyx_t_30 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_29);
          PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_11, __pyx_t_29);
          __Pyx_INCREF(__pyx_v_retshape);
          __Pyx_GIVEREF(__pyx_v_retshape);
          PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_11, __pyx_v_retshape);
          __pyx_t_29 = 0;
          __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1307, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        }
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1307, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_10);
        PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_10);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3);
        __pyx_t_10 = 0;
        __pyx_t_3 = 0;
        __pyx_r = __pyx_t_4;
        __pyx_t_4 = 0;
        goto __pyx_L0;

        /* "pyart/map/ckdtree.pyx":1306
 *             else:
 *                 # ... most other platforms
 *                 if k==1:             # <<<<<<<<<<<<<<
 *                     return np.reshape(dd[...,0],retshape), np.reshape(ii[...,0],retshape)
 *                 else:
 */
      }

      /* "pyart/map/ckdtree.pyx":1309
 *                     return np.reshape(dd[...,0],retshape), np.reshape(ii[...,0],retshape)
 *                 else:
 *                     return np.reshape(dd,retshape+(k,)), np.reshape(ii,retshape+(k,))             # <<<<<<<<<<<<<<
 * 
 *     # ----------------
 */
      /*else*/ {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1309, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_reshape); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1309, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_k); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1309, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1309, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_3 = PyNumber_Add(__pyx_v_retshape, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1309, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_2 = NULL;
        __pyx_t_11 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
          __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_10);
          if (likely(__pyx_t_2)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
            __Pyx_INCREF(__pyx_t_2);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_10, function);
            __pyx_t_11 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_10)) {
          PyObject *__pyx_temp[3] = {__pyx_t_2, ((PyObject *)__pyx_v_dd), __pyx_t_3};
          __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_10, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1309, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_10)) {
          PyObject *__pyx_temp[3] = {__pyx_t_2, ((PyObject *)__pyx_v_dd), __pyx_t_3};
          __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_10, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1309, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        {
          __pyx_t_29 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_29)) __PYX_ERR(0, 1309, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_29);
          if (__pyx_t_2) {
            __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_29, 0, __pyx_t_2); __pyx_t_2 = NULL;
          }
          __Pyx_INCREF(((PyObject *)__pyx_v_dd));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_dd));
          PyTuple_SET_ITEM(__pyx_t_29, 0+__pyx_t_11, ((PyObject *)__pyx_v_dd));
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_29, 1+__pyx_t_11, __pyx_t_3);
          __pyx_t_3 = 0;
          __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_t_29, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1309, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_29); __pyx_t_29 = 0;
        }
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_GetModuleGlobalName(__pyx_t_29, __pyx_n_s_np); if (unlikely(!__pyx_t_29)) __PYX_ERR(0, 1309, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_29);
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_29, __pyx_n_s_reshape); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1309, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_29); __pyx_t_29 = 0;
        __pyx_t_29 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_k); if (unlikely(!__pyx_t_29)) __PYX_ERR(0, 1309, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_29);
        __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1309, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_GIVEREF(__pyx_t_29);
        PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_29);
        __pyx_t_29 = 0;
        __pyx_t_29 = PyNumber_Add(__pyx_v_retshape, __pyx_t_2); if (unlikely(!__pyx_t_29)) __PYX_ERR(0, 1309, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_29);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_2 = NULL;
        __pyx_t_11 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_2)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_2);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_11 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_2, ((PyObject *)__pyx_v_ii), __pyx_t_29};
          __pyx_t_10 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1309, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_29); __pyx_t_29 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_2, ((PyObject *)__pyx_v_ii), __pyx_t_29};
          __pyx_t_10 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1309, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_29); __pyx_t_29 = 0;
        } else
        #endif
        {
          __pyx_t_30 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_30)) __PYX_ERR(0, 1309, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_30);
          if (__pyx_t_2) {
            __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_30, 0, __pyx_t_2); __pyx_t_2 = NULL;
          }
          __Pyx_INCREF(((PyObject *)__pyx_v_ii));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_ii));
          PyTuple_SET_ITEM(__pyx_t_30, 0+__pyx_t_11, ((PyObject *)__pyx_v_ii));
          __Pyx_GIVEREF(__pyx_t_29);
          PyTuple_SET_ITEM(__pyx_t_30, 1+__pyx_t_11, __pyx_t_29);
          __pyx_t_29 = 0;
          __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_30, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1309, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_30); __pyx_t_30 = 0;
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1309, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_10);
        PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_10);
        __pyx_t_4 = 0;
        __pyx_t_10 = 0;
        __pyx_r = __pyx_t_3;
        __pyx_t_3 = 0;
        goto __pyx_L0;
      }
    }
  }

  /* "pyart/map/ckdtree.pyx":1205
 *     @cython.boundscheck(False)
 *     @cython.wraparound(False)
 *     def query(cKDTree self, object x, np.intp_t k=1, np.float64_t eps=0,             # <<<<<<<<<<<<<<
 *               np.float64_t p=2, np.float64_t distance_upper_bound=infinity):
 *         """query(self, x, k=1, eps=0, p=2, distance_upper_bound=np.inf)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_29);
  __Pyx_XDECREF(__pyx_t_30);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_dd.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ii.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_xx.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.query", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_dd.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ii.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_xx.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_ii);
  __Pyx_XDECREF((PyObject *)__pyx_v_dd);
  __Pyx_XDECREF((PyObject *)__pyx_v_xx);
  __Pyx_XDECREF(__pyx_v_sh);
  __Pyx_XDECREF(__pyx_v_retshape);
  __Pyx_XDECREF(__pyx_v_x);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":1314
 *     # query_ball_point
 *     # ----------------
 *     cdef int __query_ball_point_traverse_no_checking(cKDTree self,             # <<<<<<<<<<<<<<
 *                                                      list results,
 *                                                      innernode* node) except -1:
 */

static int __pyx_f_5pyart_3map_7ckdtree_7cKDTree___query_ball_point_traverse_no_checking(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, PyObject *__pyx_v_results, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node) {
  struct __pyx_t_5pyart_3map_7ckdtree_leafnode *__pyx_v_lnode;
  __pyx_t_5numpy_intp_t __pyx_v_i;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __pyx_t_5numpy_intp_t __pyx_t_2;
  __pyx_t_5numpy_intp_t __pyx_t_3;
  __pyx_t_5numpy_intp_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__query_ball_point_traverse_no_checking", 0);

  /* "pyart/map/ckdtree.pyx":1320
 *         cdef np.intp_t i
 * 
 *         if node.split_dim == -1:  # leaf node             # <<<<<<<<<<<<<<
 *             lnode = <leafnode*> node
 *             for i in range(lnode.start_idx, lnode.end_idx):
 */
  __pyx_t_1 = ((__pyx_v_node->split_dim == -1L) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":1321
 * 
 *         if node.split_dim == -1:  # leaf node
 *             lnode = <leafnode*> node             # <<<<<<<<<<<<<<
 *             for i in range(lnode.start_idx, lnode.end_idx):
 *                 list_append(results, self.raw_indices[i])
 */
    __pyx_v_lnode = ((struct __pyx_t_5pyart_3map_7ckdtree_leafnode *)__pyx_v_node);

    /* "pyart/map/ckdtree.pyx":1322
 *         if node.split_dim == -1:  # leaf node
 *             lnode = <leafnode*> node
 *             for i in range(lnode.start_idx, lnode.end_idx):             # <<<<<<<<<<<<<<
 *                 list_append(results, self.raw_indices[i])
 *         else:
 */
    __pyx_t_2 = __pyx_v_lnode->end_idx;
    __pyx_t_3 = __pyx_t_2;
    for (__pyx_t_4 = __pyx_v_lnode->start_idx; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_i = __pyx_t_4;

      /* "pyart/map/ckdtree.pyx":1323
 *             lnode = <leafnode*> node
 *             for i in range(lnode.start_idx, lnode.end_idx):
 *                 list_append(results, self.raw_indices[i])             # <<<<<<<<<<<<<<
 *         else:
 *             self.__query_ball_point_traverse_no_checking(results, node.less)
 */
      __pyx_t_5 = __pyx_f_5pyart_3map_7ckdtree_list_append(__pyx_v_results, (__pyx_v_self->raw_indices[__pyx_v_i])); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 1323, __pyx_L1_error)
    }

    /* "pyart/map/ckdtree.pyx":1320
 *         cdef np.intp_t i
 * 
 *         if node.split_dim == -1:  # leaf node             # <<<<<<<<<<<<<<
 *             lnode = <leafnode*> node
 *             for i in range(lnode.start_idx, lnode.end_idx):
 */
    goto __pyx_L3;
  }

  /* "pyart/map/ckdtree.pyx":1325
 *                 list_append(results, self.raw_indices[i])
 *         else:
 *             self.__query_ball_point_traverse_no_checking(results, node.less)             # <<<<<<<<<<<<<<
 *             self.__query_ball_point_traverse_no_checking(results, node.greater)
 * 
 */
  /*else*/ {
    __pyx_t_5 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_ball_point_traverse_no_checking(__pyx_v_self, __pyx_v_results, __pyx_v_node->less); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 1325, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":1326
 *         else:
 *             self.__query_ball_point_traverse_no_checking(results, node.less)
 *             self.__query_ball_point_traverse_no_checking(results, node.greater)             # <<<<<<<<<<<<<<
 * 
 *         return 0
 */
    __pyx_t_5 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_ball_point_traverse_no_checking(__pyx_v_self, __pyx_v_results, __pyx_v_node->greater); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 1326, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "pyart/map/ckdtree.pyx":1328
 *             self.__query_ball_point_traverse_no_checking(results, node.greater)
 * 
 *         return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":1314
 *     # query_ball_point
 *     # ----------------
 *     cdef int __query_ball_point_traverse_no_checking(cKDTree self,             # <<<<<<<<<<<<<<
 *                                                      list results,
 *                                                      innernode* node) except -1:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.__query_ball_point_traverse_no_checking", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":1332
 * 
 *     @cython.cdivision(True)
 *     cdef int __query_ball_point_traverse_checking(cKDTree self,             # <<<<<<<<<<<<<<
 *                                                   list results,
 *                                                   innernode* node,
 */

static int __pyx_f_5pyart_3map_7ckdtree_7cKDTree___query_ball_point_traverse_checking(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, PyObject *__pyx_v_results, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node, struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_v_tracker) {
  struct __pyx_t_5pyart_3map_7ckdtree_leafnode *__pyx_v_lnode;
  __pyx_t_5numpy_float64_t __pyx_v_d;
  __pyx_t_5numpy_intp_t __pyx_v_i;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __pyx_t_5numpy_intp_t __pyx_t_3;
  __pyx_t_5numpy_intp_t __pyx_t_4;
  __pyx_t_5numpy_intp_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__query_ball_point_traverse_checking", 0);

  /* "pyart/map/ckdtree.pyx":1340
 *         cdef np.intp_t i
 * 
 *         if tracker.min_distance > tracker.upper_bound * tracker.epsfac:             # <<<<<<<<<<<<<<
 *             return 0
 *         elif tracker.max_distance < tracker.upper_bound / tracker.epsfac:
 */
  __pyx_t_1 = ((__pyx_v_tracker->min_distance > (__pyx_v_tracker->upper_bound * __pyx_v_tracker->epsfac)) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":1341
 * 
 *         if tracker.min_distance > tracker.upper_bound * tracker.epsfac:
 *             return 0             # <<<<<<<<<<<<<<
 *         elif tracker.max_distance < tracker.upper_bound / tracker.epsfac:
 *             self.__query_ball_point_traverse_no_checking(results, node)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "pyart/map/ckdtree.pyx":1340
 *         cdef np.intp_t i
 * 
 *         if tracker.min_distance > tracker.upper_bound * tracker.epsfac:             # <<<<<<<<<<<<<<
 *             return 0
 *         elif tracker.max_distance < tracker.upper_bound / tracker.epsfac:
 */
  }

  /* "pyart/map/ckdtree.pyx":1342
 *         if tracker.min_distance > tracker.upper_bound * tracker.epsfac:
 *             return 0
 *         elif tracker.max_distance < tracker.upper_bound / tracker.epsfac:             # <<<<<<<<<<<<<<
 *             self.__query_ball_point_traverse_no_checking(results, node)
 *         elif node.split_dim == -1:  # leaf node
 */
  __pyx_t_1 = ((__pyx_v_tracker->max_distance < (__pyx_v_tracker->upper_bound / __pyx_v_tracker->epsfac)) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":1343
 *             return 0
 *         elif tracker.max_distance < tracker.upper_bound / tracker.epsfac:
 *             self.__query_ball_point_traverse_no_checking(results, node)             # <<<<<<<<<<<<<<
 *         elif node.split_dim == -1:  # leaf node
 *             lnode = <leafnode*>node
 */
    __pyx_t_2 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_ball_point_traverse_no_checking(__pyx_v_self, __pyx_v_results, __pyx_v_node); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1343, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":1342
 *         if tracker.min_distance > tracker.upper_bound * tracker.epsfac:
 *             return 0
 *         elif tracker.max_distance < tracker.upper_bound / tracker.epsfac:             # <<<<<<<<<<<<<<
 *             self.__query_ball_point_traverse_no_checking(results, node)
 *         elif node.split_dim == -1:  # leaf node
 */
    goto __pyx_L3;
  }

  /* "pyart/map/ckdtree.pyx":1344
 *         elif tracker.max_distance < tracker.upper_bound / tracker.epsfac:
 *             self.__query_ball_point_traverse_no_checking(results, node)
 *         elif node.split_dim == -1:  # leaf node             # <<<<<<<<<<<<<<
 *             lnode = <leafnode*>node
 *             # brute-force
 */
  __pyx_t_1 = ((__pyx_v_node->split_dim == -1L) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":1345
 *             self.__query_ball_point_traverse_no_checking(results, node)
 *         elif node.split_dim == -1:  # leaf node
 *             lnode = <leafnode*>node             # <<<<<<<<<<<<<<
 *             # brute-force
 *             for i in range(lnode.start_idx, lnode.end_idx):
 */
    __pyx_v_lnode = ((struct __pyx_t_5pyart_3map_7ckdtree_leafnode *)__pyx_v_node);

    /* "pyart/map/ckdtree.pyx":1347
 *             lnode = <leafnode*>node
 *             # brute-force
 *             for i in range(lnode.start_idx, lnode.end_idx):             # <<<<<<<<<<<<<<
 *                 d = _distance_p(
 *                     self.raw_data + self.raw_indices[i] * self.m,
 */
    __pyx_t_3 = __pyx_v_lnode->end_idx;
    __pyx_t_4 = __pyx_t_3;
    for (__pyx_t_5 = __pyx_v_lnode->start_idx; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
      __pyx_v_i = __pyx_t_5;

      /* "pyart/map/ckdtree.pyx":1348
 *             # brute-force
 *             for i in range(lnode.start_idx, lnode.end_idx):
 *                 d = _distance_p(             # <<<<<<<<<<<<<<
 *                     self.raw_data + self.raw_indices[i] * self.m,
 *                     tracker.pt, tracker.p, self.m, tracker.upper_bound)
 */
      __pyx_v_d = __pyx_f_5pyart_3map_7ckdtree__distance_p((__pyx_v_self->raw_data + ((__pyx_v_self->raw_indices[__pyx_v_i]) * __pyx_v_self->m)), __pyx_v_tracker->pt, __pyx_v_tracker->p, __pyx_v_self->m, __pyx_v_tracker->upper_bound);

      /* "pyart/map/ckdtree.pyx":1351
 *                     self.raw_data + self.raw_indices[i] * self.m,
 *                     tracker.pt, tracker.p, self.m, tracker.upper_bound)
 *                 if d <= tracker.upper_bound:             # <<<<<<<<<<<<<<
 *                     list_append(results, self.raw_indices[i])
 *         else:
 */
      __pyx_t_1 = ((__pyx_v_d <= __pyx_v_tracker->upper_bound) != 0);
      if (__pyx_t_1) {

        /* "pyart/map/ckdtree.pyx":1352
 *                     tracker.pt, tracker.p, self.m, tracker.upper_bound)
 *                 if d <= tracker.upper_bound:
 *                     list_append(results, self.raw_indices[i])             # <<<<<<<<<<<<<<
 *         else:
 *             tracker.push_less_of(node)
 */
        __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_list_append(__pyx_v_results, (__pyx_v_self->raw_indices[__pyx_v_i])); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1352, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1351
 *                     self.raw_data + self.raw_indices[i] * self.m,
 *                     tracker.pt, tracker.p, self.m, tracker.upper_bound)
 *                 if d <= tracker.upper_bound:             # <<<<<<<<<<<<<<
 *                     list_append(results, self.raw_indices[i])
 *         else:
 */
      }
    }

    /* "pyart/map/ckdtree.pyx":1344
 *         elif tracker.max_distance < tracker.upper_bound / tracker.epsfac:
 *             self.__query_ball_point_traverse_no_checking(results, node)
 *         elif node.split_dim == -1:  # leaf node             # <<<<<<<<<<<<<<
 *             lnode = <leafnode*>node
 *             # brute-force
 */
    goto __pyx_L3;
  }

  /* "pyart/map/ckdtree.pyx":1354
 *                     list_append(results, self.raw_indices[i])
 *         else:
 *             tracker.push_less_of(node)             # <<<<<<<<<<<<<<
 *             self.__query_ball_point_traverse_checking(
 *                 results, node.less, tracker)
 */
  /*else*/ {
    __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker_push_less_of(__pyx_v_tracker, __pyx_v_node); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1354, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":1355
 *         else:
 *             tracker.push_less_of(node)
 *             self.__query_ball_point_traverse_checking(             # <<<<<<<<<<<<<<
 *                 results, node.less, tracker)
 *             tracker.pop()
 */
    __pyx_t_2 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_ball_point_traverse_checking(__pyx_v_self, __pyx_v_results, __pyx_v_node->less, __pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1355, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":1357
 *             self.__query_ball_point_traverse_checking(
 *                 results, node.less, tracker)
 *             tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *             tracker.push_greater_of(node)
 */
    __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1357, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":1359
 *             tracker.pop()
 * 
 *             tracker.push_greater_of(node)             # <<<<<<<<<<<<<<
 *             self.__query_ball_point_traverse_checking(
 *                 results, node.greater, tracker)
 */
    __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker_push_greater_of(__pyx_v_tracker, __pyx_v_node); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1359, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":1360
 * 
 *             tracker.push_greater_of(node)
 *             self.__query_ball_point_traverse_checking(             # <<<<<<<<<<<<<<
 *                 results, node.greater, tracker)
 *             tracker.pop()
 */
    __pyx_t_2 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_ball_point_traverse_checking(__pyx_v_self, __pyx_v_results, __pyx_v_node->greater, __pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1360, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":1362
 *             self.__query_ball_point_traverse_checking(
 *                 results, node.greater, tracker)
 *             tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *         return 0
 */
    __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1362, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "pyart/map/ckdtree.pyx":1364
 *             tracker.pop()
 * 
 *         return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":1332
 * 
 *     @cython.cdivision(True)
 *     cdef int __query_ball_point_traverse_checking(cKDTree self,             # <<<<<<<<<<<<<<
 *                                                   list results,
 *                                                   innernode* node,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.__query_ball_point_traverse_checking", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":1367
 * 
 * 
 *     cdef list __query_ball_point(cKDTree self,             # <<<<<<<<<<<<<<
 *                                  np.float64_t* x,
 *                                  np.float64_t r,
 */

static PyObject *__pyx_f_5pyart_3map_7ckdtree_7cKDTree___query_ball_point(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, __pyx_t_5numpy_float64_t *__pyx_v_x, __pyx_t_5numpy_float64_t __pyx_v_r, __pyx_t_5numpy_float64_t __pyx_v_p, __pyx_t_5numpy_float64_t __pyx_v_eps) {
  struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *__pyx_v_tracker = NULL;
  PyObject *__pyx_v_results = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__query_ball_point", 0);

  /* "pyart/map/ckdtree.pyx":1373
 *                                  np.float64_t eps):
 * 
 *         tracker = PointRectDistanceTracker()             # <<<<<<<<<<<<<<
 *         tracker.init(x, Rectangle(self.mins, self.maxes),
 *                      p, eps, r)
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_5pyart_3map_7ckdtree_PointRectDistanceTracker)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1373, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_tracker = ((struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/map/ckdtree.pyx":1374
 * 
 *         tracker = PointRectDistanceTracker()
 *         tracker.init(x, Rectangle(self.mins, self.maxes),             # <<<<<<<<<<<<<<
 *                      p, eps, r)
 * 
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1374, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->mins));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->mins));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_self->mins));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->maxes));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->maxes));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self->maxes));
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pyart_3map_7ckdtree_Rectangle), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1374, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/map/ckdtree.pyx":1375
 *         tracker = PointRectDistanceTracker()
 *         tracker.init(x, Rectangle(self.mins, self.maxes),
 *                      p, eps, r)             # <<<<<<<<<<<<<<
 * 
 *         results = []
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_PointRectDistanceTracker *)__pyx_v_tracker->__pyx_vtab)->init(__pyx_v_tracker, __pyx_v_x, ((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)__pyx_t_2), __pyx_v_p, __pyx_v_eps, __pyx_v_r); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1374, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/map/ckdtree.pyx":1377
 *                      p, eps, r)
 * 
 *         results = []             # <<<<<<<<<<<<<<
 *         self.__query_ball_point_traverse_checking(
 *             results, self.tree, tracker)
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1377, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_results = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/map/ckdtree.pyx":1378
 * 
 *         results = []
 *         self.__query_ball_point_traverse_checking(             # <<<<<<<<<<<<<<
 *             results, self.tree, tracker)
 *         return results
 */
  __pyx_t_3 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_ball_point_traverse_checking(__pyx_v_self, __pyx_v_results, __pyx_v_self->tree, __pyx_v_tracker); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 1378, __pyx_L1_error)

  /* "pyart/map/ckdtree.pyx":1380
 *         self.__query_ball_point_traverse_checking(
 *             results, self.tree, tracker)
 *         return results             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_results);
  __pyx_r = __pyx_v_results;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":1367
 * 
 * 
 *     cdef list __query_ball_point(cKDTree self,             # <<<<<<<<<<<<<<
 *                                  np.float64_t* x,
 *                                  np.float64_t r,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.__query_ball_point", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_tracker);
  __Pyx_XDECREF(__pyx_v_results);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":1383
 * 
 * 
 *     def query_ball_point(cKDTree self, object x, np.float64_t r,             # <<<<<<<<<<<<<<
 *                          np.float64_t p=2., np.float64_t eps=0):
 *         """query_ball_point(self, x, r, p, eps)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_7query_ball_point(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5pyart_3map_7ckdtree_7cKDTree_6query_ball_point[] = "query_ball_point(self, x, r, p, eps)\n        \n        Find all points within distance r of point(s) x.\n\n        Parameters\n        ----------\n        x : array_like, shape tuple + (self.m,)\n            The point or points to search for neighbors of.\n        r : positive float\n            The radius of points to return.\n        p : float, optional\n            Which Minkowski p-norm to use.  Should be in the range [1, inf].\n        eps : nonnegative float, optional\n            Approximate search. Branches of the tree are not explored if their\n            nearest points are further than ``r / (1 + eps)``, and branches are\n            added in bulk if their furthest points are nearer than\n            ``r * (1 + eps)``.\n\n        Returns\n        -------\n        results : list or array of lists\n            If `x` is a single point, returns a list of the indices of the\n            neighbors of `x`. If `x` is an array of points, returns an object\n            array of shape tuple containing lists of neighbors.\n\n        Notes\n        -----\n        If you have many points whose neighbors you want to find, you may save\n        substantial amounts of time by putting them in a cKDTree and using\n        query_ball_tree.\n\n        Examples\n        --------\n        >>> from scipy import spatial\n        >>> x, y = np.mgrid[0:4, 0:4]\n        >>> points = zip(x.ravel(), y.ravel())\n        >>> tree = spatial.cKDTree(points)\n        >>> tree.query_ball_point([2, 0], 1)\n        [4, 8, 9, 12]\n\n        ";
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_7query_ball_point(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_x = 0;
  __pyx_t_5numpy_float64_t __pyx_v_r;
  __pyx_t_5numpy_float64_t __pyx_v_p;
  __pyx_t_5numpy_float64_t __pyx_v_eps;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("query_ball_point (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_r,&__pyx_n_s_p,&__pyx_n_s_eps,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_r)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("query_ball_point", 0, 2, 4, 1); __PYX_ERR(0, 1383, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_p);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_eps);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "query_ball_point") < 0)) __PYX_ERR(0, 1383, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_x = values[0];
    __pyx_v_r = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_r == ((npy_float64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1383, __pyx_L3_error)
    if (values[2]) {
      __pyx_v_p = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_p == ((npy_float64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1384, __pyx_L3_error)
    } else {
      __pyx_v_p = ((__pyx_t_5numpy_float64_t)2.);
    }
    if (values[3]) {
      __pyx_v_eps = __pyx_PyFloat_AsDouble(values[3]); if (unlikely((__pyx_v_eps == ((npy_float64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1384, __pyx_L3_error)
    } else {
      __pyx_v_eps = ((__pyx_t_5numpy_float64_t)0.0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("query_ball_point", 0, 2, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1383, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.query_ball_point", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_7cKDTree_6query_ball_point(((struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self), __pyx_v_x, __pyx_v_r, __pyx_v_p, __pyx_v_eps);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_6query_ball_point(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, PyObject *__pyx_v_x, __pyx_t_5numpy_float64_t __pyx_v_r, __pyx_t_5numpy_float64_t __pyx_v_p, __pyx_t_5numpy_float64_t __pyx_v_eps) {
  PyArrayObject *__pyx_v_xx = 0;
  PyObject *__pyx_v_retshape = NULL;
  PyObject *__pyx_v_result = NULL;
  PyObject *__pyx_v_c = NULL;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_xx;
  __Pyx_Buffer __pyx_pybuffer_xx;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  Py_UCS4 __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyArrayObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("query_ball_point", 0);
  __Pyx_INCREF(__pyx_v_x);
  __pyx_pybuffer_xx.pybuffer.buf = NULL;
  __pyx_pybuffer_xx.refcount = 0;
  __pyx_pybuffernd_xx.data = NULL;
  __pyx_pybuffernd_xx.rcbuffer = &__pyx_pybuffer_xx;

  /* "pyart/map/ckdtree.pyx":1428
 *         cdef np.ndarray[np.float64_t, ndim=1, mode="c"] xx
 * 
 *         x = np.asarray(x).astype(np.float64)             # <<<<<<<<<<<<<<
 *         if x.shape[-1] != self.m:
 *             raise ValueError("Searching for a %d-dimensional point in a " \
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1428, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_asarray); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1428, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_3, __pyx_v_x) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_x);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1428, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_astype); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1428, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1428, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_float64); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1428, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1428, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF_SET(__pyx_v_x, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/map/ckdtree.pyx":1429
 * 
 *         x = np.asarray(x).astype(np.float64)
 *         if x.shape[-1] != self.m:             # <<<<<<<<<<<<<<
 *             raise ValueError("Searching for a %d-dimensional point in a " \
 *                              "%d-dimensional KDTree" % (int(x.shape[-1]), int(self.m)))
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_x, __pyx_n_s_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1429, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_GetItemInt(__pyx_t_1, -1L, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1429, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->m); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1429, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_4, __pyx_t_1, Py_NE); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1429, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 1429, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(__pyx_t_5)) {

    /* "pyart/map/ckdtree.pyx":1430
 *         x = np.asarray(x).astype(np.float64)
 *         if x.shape[-1] != self.m:
 *             raise ValueError("Searching for a %d-dimensional point in a " \             # <<<<<<<<<<<<<<
 *                              "%d-dimensional KDTree" % (int(x.shape[-1]), int(self.m)))
 *         if len(x.shape) == 1:
 */
    __pyx_t_3 = PyTuple_New(5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1430, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = 0;
    __pyx_t_7 = 127;
    __Pyx_INCREF(__pyx_kp_u_Searching_for_a);
    __pyx_t_6 += 16;
    __Pyx_GIVEREF(__pyx_kp_u_Searching_for_a);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_kp_u_Searching_for_a);

    /* "pyart/map/ckdtree.pyx":1431
 *         if x.shape[-1] != self.m:
 *             raise ValueError("Searching for a %d-dimensional point in a " \
 *                              "%d-dimensional KDTree" % (int(x.shape[-1]), int(self.m)))             # <<<<<<<<<<<<<<
 *         if len(x.shape) == 1:
 *             xx = np.ascontiguousarray(x, dtype=np.float64)
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_x, __pyx_n_s_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1431, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_GetItemInt(__pyx_t_1, -1L, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1431, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyNumber_Int(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1431, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_FormatAndDecref(__Pyx_PyNumber_IntOrLong(__pyx_t_1), __pyx_n_u_d); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1431, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) > __pyx_t_7) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) : __pyx_t_7;
    __pyx_t_6 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_4 = 0;
    __Pyx_INCREF(__pyx_kp_u_dimensional_point_in_a);
    __pyx_t_6 += 24;
    __Pyx_GIVEREF(__pyx_kp_u_dimensional_point_in_a);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_kp_u_dimensional_point_in_a);
    __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->m); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1431, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyInt_Type)), __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1431, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_FormatAndDecref(__Pyx_PyNumber_IntOrLong(__pyx_t_1), __pyx_n_u_d); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1431, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) > __pyx_t_7) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) : __pyx_t_7;
    __pyx_t_6 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 3, __pyx_t_4);
    __pyx_t_4 = 0;
    __Pyx_INCREF(__pyx_kp_u_dimensional_KDTree);
    __pyx_t_6 += 19;
    __Pyx_GIVEREF(__pyx_kp_u_dimensional_KDTree);
    PyTuple_SET_ITEM(__pyx_t_3, 4, __pyx_kp_u_dimensional_KDTree);

    /* "pyart/map/ckdtree.pyx":1430
 *         x = np.asarray(x).astype(np.float64)
 *         if x.shape[-1] != self.m:
 *             raise ValueError("Searching for a %d-dimensional point in a " \             # <<<<<<<<<<<<<<
 *                              "%d-dimensional KDTree" % (int(x.shape[-1]), int(self.m)))
 *         if len(x.shape) == 1:
 */
    __pyx_t_4 = __Pyx_PyUnicode_Join(__pyx_t_3, 5, __pyx_t_6, __pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1430, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1430, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 1430, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":1429
 * 
 *         x = np.asarray(x).astype(np.float64)
 *         if x.shape[-1] != self.m:             # <<<<<<<<<<<<<<
 *             raise ValueError("Searching for a %d-dimensional point in a " \
 *                              "%d-dimensional KDTree" % (int(x.shape[-1]), int(self.m)))
 */
  }

  /* "pyart/map/ckdtree.pyx":1432
 *             raise ValueError("Searching for a %d-dimensional point in a " \
 *                              "%d-dimensional KDTree" % (int(x.shape[-1]), int(self.m)))
 *         if len(x.shape) == 1:             # <<<<<<<<<<<<<<
 *             xx = np.ascontiguousarray(x, dtype=np.float64)
 *             return self.__query_ball_point(&xx[0], r, p, eps)
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_x, __pyx_n_s_shape); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1432, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = PyObject_Length(__pyx_t_3); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1432, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = ((__pyx_t_6 == 1) != 0);
  if (__pyx_t_5) {

    /* "pyart/map/ckdtree.pyx":1433
 *                              "%d-dimensional KDTree" % (int(x.shape[-1]), int(self.m)))
 *         if len(x.shape) == 1:
 *             xx = np.ascontiguousarray(x, dtype=np.float64)             # <<<<<<<<<<<<<<
 *             return self.__query_ball_point(&xx[0], r, p, eps)
 *         else:
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1433, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_ascontiguousarray); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1433, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1433, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_x);
    __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1433, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1433, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_float64); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1433, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_8) < 0) __PYX_ERR(0, 1433, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1433, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (!(likely(((__pyx_t_8) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_8, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1433, __pyx_L1_error)
    __pyx_t_9 = ((PyArrayObject *)__pyx_t_8);
    {
      __Pyx_BufFmt_StackElem __pyx_stack[1];
      __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_xx.rcbuffer->pybuffer);
      __pyx_t_10 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_xx.rcbuffer->pybuffer, (PyObject*)__pyx_t_9, &__Pyx_TypeInfo_nn___pyx_t_5numpy_float64_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack);
      if (unlikely(__pyx_t_10 < 0)) {
        PyErr_Fetch(&__pyx_t_11, &__pyx_t_12, &__pyx_t_13);
        if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_xx.rcbuffer->pybuffer, (PyObject*)__pyx_v_xx, &__Pyx_TypeInfo_nn___pyx_t_5numpy_float64_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {
          Py_XDECREF(__pyx_t_11); Py_XDECREF(__pyx_t_12); Py_XDECREF(__pyx_t_13);
          __Pyx_RaiseBufferFallbackError();
        } else {
          PyErr_Restore(__pyx_t_11, __pyx_t_12, __pyx_t_13);
        }
        __pyx_t_11 = __pyx_t_12 = __pyx_t_13 = 0;
      }
      __pyx_pybuffernd_xx.diminfo[0].strides = __pyx_pybuffernd_xx.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_xx.diminfo[0].shape = __pyx_pybuffernd_xx.rcbuffer->pybuffer.shape[0];
      if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 1433, __pyx_L1_error)
    }
    __pyx_t_9 = 0;
    __pyx_v_xx = ((PyArrayObject *)__pyx_t_8);
    __pyx_t_8 = 0;

    /* "pyart/map/ckdtree.pyx":1434
 *         if len(x.shape) == 1:
 *             xx = np.ascontiguousarray(x, dtype=np.float64)
 *             return self.__query_ball_point(&xx[0], r, p, eps)             # <<<<<<<<<<<<<<
 *         else:
 *             retshape = x.shape[:-1]
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_14 = 0;
    __pyx_t_10 = -1;
    if (__pyx_t_14 < 0) {
      __pyx_t_14 += __pyx_pybuffernd_xx.diminfo[0].shape;
      if (unlikely(__pyx_t_14 < 0)) __pyx_t_10 = 0;
    } else if (unlikely(__pyx_t_14 >= __pyx_pybuffernd_xx.diminfo[0].shape)) __pyx_t_10 = 0;
    if (unlikely(__pyx_t_10 != -1)) {
      __Pyx_RaiseBufferIndexError(__pyx_t_10);
      __PYX_ERR(0, 1434, __pyx_L1_error)
    }
    __pyx_t_8 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_ball_point(__pyx_v_self, (&(*__Pyx_BufPtrCContig1d(__pyx_t_5numpy_float64_t *, __pyx_pybuffernd_xx.rcbuffer->pybuffer.buf, __pyx_t_14, __pyx_pybuffernd_xx.diminfo[0].strides))), __pyx_v_r, __pyx_v_p, __pyx_v_eps); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1434, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;

    /* "pyart/map/ckdtree.pyx":1432
 *             raise ValueError("Searching for a %d-dimensional point in a " \
 *                              "%d-dimensional KDTree" % (int(x.shape[-1]), int(self.m)))
 *         if len(x.shape) == 1:             # <<<<<<<<<<<<<<
 *             xx = np.ascontiguousarray(x, dtype=np.float64)
 *             return self.__query_ball_point(&xx[0], r, p, eps)
 */
  }

  /* "pyart/map/ckdtree.pyx":1436
 *             return self.__query_ball_point(&xx[0], r, p, eps)
 *         else:
 *             retshape = x.shape[:-1]             # <<<<<<<<<<<<<<
 *             result = np.empty(retshape, dtype=np.object)
 *             for c in np.ndindex(retshape):
 */
  /*else*/ {
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_x, __pyx_n_s_shape); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1436, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_1 = __Pyx_PyObject_GetSlice(__pyx_t_8, 0, -1L, NULL, NULL, &__pyx_slice__18, 0, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1436, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_v_retshape = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "pyart/map/ckdtree.pyx":1437
 *         else:
 *             retshape = x.shape[:-1]
 *             result = np.empty(retshape, dtype=np.object)             # <<<<<<<<<<<<<<
 *             for c in np.ndindex(retshape):
 *                 xx = np.ascontiguousarray(x[c], dtype=np.float64)
 */
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1437, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_empty); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1437, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1437, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v_retshape);
    __Pyx_GIVEREF(__pyx_v_retshape);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_retshape);
    __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1437, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1437, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_object); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1437, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_2) < 0) __PYX_ERR(0, 1437, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1437, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_result = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "pyart/map/ckdtree.pyx":1438
 *             retshape = x.shape[:-1]
 *             result = np.empty(retshape, dtype=np.object)
 *             for c in np.ndindex(retshape):             # <<<<<<<<<<<<<<
 *                 xx = np.ascontiguousarray(x[c], dtype=np.float64)
 *                 result[c] = self.__query_ball_point(&xx[0], r, p, eps)
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1438, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_ndindex); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1438, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_3, __pyx_v_retshape) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v_retshape);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1438, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
      __pyx_t_1 = __pyx_t_2; __Pyx_INCREF(__pyx_t_1); __pyx_t_6 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_6 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1438, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_15 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1438, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_2); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 1438, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1438, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        } else {
          if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_2); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 1438, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1438, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        }
      } else {
        __pyx_t_2 = __pyx_t_15(__pyx_t_1);
        if (unlikely(!__pyx_t_2)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 1438, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_XDECREF_SET(__pyx_v_c, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "pyart/map/ckdtree.pyx":1439
 *             result = np.empty(retshape, dtype=np.object)
 *             for c in np.ndindex(retshape):
 *                 xx = np.ascontiguousarray(x[c], dtype=np.float64)             # <<<<<<<<<<<<<<
 *                 result[c] = self.__query_ball_point(&xx[0], r, p, eps)
 *             return result
 */
      __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1439, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_ascontiguousarray); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1439, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_x, __pyx_v_c); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1439, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_8 = PyTuple_New(1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1439, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_2);
      PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_2);
      __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1439, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1439, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_16 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_float64); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1439, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_16);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_16) < 0) __PYX_ERR(0, 1439, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
      __pyx_t_16 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_8, __pyx_t_2); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1439, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_16);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (!(likely(((__pyx_t_16) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_16, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1439, __pyx_L1_error)
      __pyx_t_9 = ((PyArrayObject *)__pyx_t_16);
      {
        __Pyx_BufFmt_StackElem __pyx_stack[1];
        __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_xx.rcbuffer->pybuffer);
        __pyx_t_10 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_xx.rcbuffer->pybuffer, (PyObject*)__pyx_t_9, &__Pyx_TypeInfo_nn___pyx_t_5numpy_float64_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack);
        if (unlikely(__pyx_t_10 < 0)) {
          PyErr_Fetch(&__pyx_t_13, &__pyx_t_12, &__pyx_t_11);
          if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_xx.rcbuffer->pybuffer, (PyObject*)__pyx_v_xx, &__Pyx_TypeInfo_nn___pyx_t_5numpy_float64_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {
            Py_XDECREF(__pyx_t_13); Py_XDECREF(__pyx_t_12); Py_XDECREF(__pyx_t_11);
            __Pyx_RaiseBufferFallbackError();
          } else {
            PyErr_Restore(__pyx_t_13, __pyx_t_12, __pyx_t_11);
          }
          __pyx_t_13 = __pyx_t_12 = __pyx_t_11 = 0;
        }
        __pyx_pybuffernd_xx.diminfo[0].strides = __pyx_pybuffernd_xx.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_xx.diminfo[0].shape = __pyx_pybuffernd_xx.rcbuffer->pybuffer.shape[0];
        if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 1439, __pyx_L1_error)
      }
      __pyx_t_9 = 0;
      __Pyx_XDECREF_SET(__pyx_v_xx, ((PyArrayObject *)__pyx_t_16));
      __pyx_t_16 = 0;

      /* "pyart/map/ckdtree.pyx":1440
 *             for c in np.ndindex(retshape):
 *                 xx = np.ascontiguousarray(x[c], dtype=np.float64)
 *                 result[c] = self.__query_ball_point(&xx[0], r, p, eps)             # <<<<<<<<<<<<<<
 *             return result
 * 
 */
      __pyx_t_14 = 0;
      __pyx_t_10 = -1;
      if (__pyx_t_14 < 0) {
        __pyx_t_14 += __pyx_pybuffernd_xx.diminfo[0].shape;
        if (unlikely(__pyx_t_14 < 0)) __pyx_t_10 = 0;
      } else if (unlikely(__pyx_t_14 >= __pyx_pybuffernd_xx.diminfo[0].shape)) __pyx_t_10 = 0;
      if (unlikely(__pyx_t_10 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_10);
        __PYX_ERR(0, 1440, __pyx_L1_error)
      }
      __pyx_t_16 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_ball_point(__pyx_v_self, (&(*__Pyx_BufPtrCContig1d(__pyx_t_5numpy_float64_t *, __pyx_pybuffernd_xx.rcbuffer->pybuffer.buf, __pyx_t_14, __pyx_pybuffernd_xx.diminfo[0].strides))), __pyx_v_r, __pyx_v_p, __pyx_v_eps); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1440, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_16);
      if (unlikely(PyObject_SetItem(__pyx_v_result, __pyx_v_c, __pyx_t_16) < 0)) __PYX_ERR(0, 1440, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;

      /* "pyart/map/ckdtree.pyx":1438
 *             retshape = x.shape[:-1]
 *             result = np.empty(retshape, dtype=np.object)
 *             for c in np.ndindex(retshape):             # <<<<<<<<<<<<<<
 *                 xx = np.ascontiguousarray(x[c], dtype=np.float64)
 *                 result[c] = self.__query_ball_point(&xx[0], r, p, eps)
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyart/map/ckdtree.pyx":1441
 *                 xx = np.ascontiguousarray(x[c], dtype=np.float64)
 *                 result[c] = self.__query_ball_point(&xx[0], r, p, eps)
 *             return result             # <<<<<<<<<<<<<<
 * 
 *     # ---------------
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_result);
    __pyx_r = __pyx_v_result;
    goto __pyx_L0;
  }

  /* "pyart/map/ckdtree.pyx":1383
 * 
 * 
 *     def query_ball_point(cKDTree self, object x, np.float64_t r,             # <<<<<<<<<<<<<<
 *                          np.float64_t p=2., np.float64_t eps=0):
 *         """query_ball_point(self, x, r, p, eps)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_16);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_xx.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.query_ball_point", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_xx.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_xx);
  __Pyx_XDECREF(__pyx_v_retshape);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_c);
  __Pyx_XDECREF(__pyx_v_x);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":1446
 *     # query_ball_tree
 *     # ---------------
 *     cdef int __query_ball_tree_traverse_no_checking(cKDTree self,             # <<<<<<<<<<<<<<
 *                                                     cKDTree other,
 *                                                     list results,
 */

static int __pyx_f_5pyart_3map_7ckdtree_7cKDTree___query_ball_tree_traverse_no_checking(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_other, PyObject *__pyx_v_results, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node1, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node2) {
  struct __pyx_t_5pyart_3map_7ckdtree_leafnode *__pyx_v_lnode1;
  struct __pyx_t_5pyart_3map_7ckdtree_leafnode *__pyx_v_lnode2;
  PyObject *__pyx_v_results_i = 0;
  __pyx_t_5numpy_intp_t __pyx_v_i;
  __pyx_t_5numpy_intp_t __pyx_v_j;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __pyx_t_5numpy_intp_t __pyx_t_2;
  __pyx_t_5numpy_intp_t __pyx_t_3;
  __pyx_t_5numpy_intp_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  __pyx_t_5numpy_intp_t __pyx_t_6;
  __pyx_t_5numpy_intp_t __pyx_t_7;
  __pyx_t_5numpy_intp_t __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__query_ball_tree_traverse_no_checking", 0);

  /* "pyart/map/ckdtree.pyx":1456
 *         cdef np.intp_t i, j
 * 
 *         if node1.split_dim == -1:  # leaf node             # <<<<<<<<<<<<<<
 *             lnode1 = <leafnode*>node1
 * 
 */
  __pyx_t_1 = ((__pyx_v_node1->split_dim == -1L) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":1457
 * 
 *         if node1.split_dim == -1:  # leaf node
 *             lnode1 = <leafnode*>node1             # <<<<<<<<<<<<<<
 * 
 *             if node2.split_dim == -1:  # leaf node
 */
    __pyx_v_lnode1 = ((struct __pyx_t_5pyart_3map_7ckdtree_leafnode *)__pyx_v_node1);

    /* "pyart/map/ckdtree.pyx":1459
 *             lnode1 = <leafnode*>node1
 * 
 *             if node2.split_dim == -1:  # leaf node             # <<<<<<<<<<<<<<
 *                 lnode2 = <leafnode*>node2
 * 
 */
    __pyx_t_1 = ((__pyx_v_node2->split_dim == -1L) != 0);
    if (__pyx_t_1) {

      /* "pyart/map/ckdtree.pyx":1460
 * 
 *             if node2.split_dim == -1:  # leaf node
 *                 lnode2 = <leafnode*>node2             # <<<<<<<<<<<<<<
 * 
 *                 for i in range(lnode1.start_idx, lnode1.end_idx):
 */
      __pyx_v_lnode2 = ((struct __pyx_t_5pyart_3map_7ckdtree_leafnode *)__pyx_v_node2);

      /* "pyart/map/ckdtree.pyx":1462
 *                 lnode2 = <leafnode*>node2
 * 
 *                 for i in range(lnode1.start_idx, lnode1.end_idx):             # <<<<<<<<<<<<<<
 *                     results_i = results[self.raw_indices[i]]
 *                     for j in range(lnode2.start_idx, lnode2.end_idx):
 */
      __pyx_t_2 = __pyx_v_lnode1->end_idx;
      __pyx_t_3 = __pyx_t_2;
      for (__pyx_t_4 = __pyx_v_lnode1->start_idx; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
        __pyx_v_i = __pyx_t_4;

        /* "pyart/map/ckdtree.pyx":1463
 * 
 *                 for i in range(lnode1.start_idx, lnode1.end_idx):
 *                     results_i = results[self.raw_indices[i]]             # <<<<<<<<<<<<<<
 *                     for j in range(lnode2.start_idx, lnode2.end_idx):
 *                         list_append(results_i, other.raw_indices[j])
 */
        if (unlikely(__pyx_v_results == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 1463, __pyx_L1_error)
        }
        __pyx_t_5 = __Pyx_GetItemInt_List(__pyx_v_results, (__pyx_v_self->raw_indices[__pyx_v_i]), __pyx_t_5numpy_intp_t, 1, __Pyx_PyInt_From_Py_intptr_t, 1, 1, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1463, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        if (!(likely(PyList_CheckExact(__pyx_t_5))||((__pyx_t_5) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_5)->tp_name), 0))) __PYX_ERR(0, 1463, __pyx_L1_error)
        __Pyx_XDECREF_SET(__pyx_v_results_i, ((PyObject*)__pyx_t_5));
        __pyx_t_5 = 0;

        /* "pyart/map/ckdtree.pyx":1464
 *                 for i in range(lnode1.start_idx, lnode1.end_idx):
 *                     results_i = results[self.raw_indices[i]]
 *                     for j in range(lnode2.start_idx, lnode2.end_idx):             # <<<<<<<<<<<<<<
 *                         list_append(results_i, other.raw_indices[j])
 *             else:
 */
        __pyx_t_6 = __pyx_v_lnode2->end_idx;
        __pyx_t_7 = __pyx_t_6;
        for (__pyx_t_8 = __pyx_v_lnode2->start_idx; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
          __pyx_v_j = __pyx_t_8;

          /* "pyart/map/ckdtree.pyx":1465
 *                     results_i = results[self.raw_indices[i]]
 *                     for j in range(lnode2.start_idx, lnode2.end_idx):
 *                         list_append(results_i, other.raw_indices[j])             # <<<<<<<<<<<<<<
 *             else:
 * 
 */
          __pyx_t_9 = __pyx_f_5pyart_3map_7ckdtree_list_append(__pyx_v_results_i, (__pyx_v_other->raw_indices[__pyx_v_j])); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 1465, __pyx_L1_error)
        }
      }

      /* "pyart/map/ckdtree.pyx":1459
 *             lnode1 = <leafnode*>node1
 * 
 *             if node2.split_dim == -1:  # leaf node             # <<<<<<<<<<<<<<
 *                 lnode2 = <leafnode*>node2
 * 
 */
      goto __pyx_L4;
    }

    /* "pyart/map/ckdtree.pyx":1468
 *             else:
 * 
 *                 self.__query_ball_tree_traverse_no_checking(other, results, node1, node2.less)             # <<<<<<<<<<<<<<
 *                 self.__query_ball_tree_traverse_no_checking(other, results, node1, node2.greater)
 *         else:
 */
    /*else*/ {
      __pyx_t_9 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_ball_tree_traverse_no_checking(__pyx_v_self, __pyx_v_other, __pyx_v_results, __pyx_v_node1, __pyx_v_node2->less); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 1468, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1469
 * 
 *                 self.__query_ball_tree_traverse_no_checking(other, results, node1, node2.less)
 *                 self.__query_ball_tree_traverse_no_checking(other, results, node1, node2.greater)             # <<<<<<<<<<<<<<
 *         else:
 * 
 */
      __pyx_t_9 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_ball_tree_traverse_no_checking(__pyx_v_self, __pyx_v_other, __pyx_v_results, __pyx_v_node1, __pyx_v_node2->greater); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 1469, __pyx_L1_error)
    }
    __pyx_L4:;

    /* "pyart/map/ckdtree.pyx":1456
 *         cdef np.intp_t i, j
 * 
 *         if node1.split_dim == -1:  # leaf node             # <<<<<<<<<<<<<<
 *             lnode1 = <leafnode*>node1
 * 
 */
    goto __pyx_L3;
  }

  /* "pyart/map/ckdtree.pyx":1472
 *         else:
 * 
 *             self.__query_ball_tree_traverse_no_checking(other, results, node1.less, node2)             # <<<<<<<<<<<<<<
 *             self.__query_ball_tree_traverse_no_checking(other, results, node1.greater, node2)
 * 
 */
  /*else*/ {
    __pyx_t_9 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_ball_tree_traverse_no_checking(__pyx_v_self, __pyx_v_other, __pyx_v_results, __pyx_v_node1->less, __pyx_v_node2); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 1472, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":1473
 * 
 *             self.__query_ball_tree_traverse_no_checking(other, results, node1.less, node2)
 *             self.__query_ball_tree_traverse_no_checking(other, results, node1.greater, node2)             # <<<<<<<<<<<<<<
 * 
 *         return 0
 */
    __pyx_t_9 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_ball_tree_traverse_no_checking(__pyx_v_self, __pyx_v_other, __pyx_v_results, __pyx_v_node1->greater, __pyx_v_node2); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 1473, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "pyart/map/ckdtree.pyx":1475
 *             self.__query_ball_tree_traverse_no_checking(other, results, node1.greater, node2)
 * 
 *         return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":1446
 *     # query_ball_tree
 *     # ---------------
 *     cdef int __query_ball_tree_traverse_no_checking(cKDTree self,             # <<<<<<<<<<<<<<
 *                                                     cKDTree other,
 *                                                     list results,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.__query_ball_tree_traverse_no_checking", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_results_i);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":1479
 * 
 *     @cython.cdivision(True)
 *     cdef int __query_ball_tree_traverse_checking(cKDTree self,             # <<<<<<<<<<<<<<
 *                                                  cKDTree other,
 *                                                  list results,
 */

static int __pyx_f_5pyart_3map_7ckdtree_7cKDTree___query_ball_tree_traverse_checking(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_other, PyObject *__pyx_v_results, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node1, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node2, struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_tracker) {
  struct __pyx_t_5pyart_3map_7ckdtree_leafnode *__pyx_v_lnode1;
  struct __pyx_t_5pyart_3map_7ckdtree_leafnode *__pyx_v_lnode2;
  PyObject *__pyx_v_results_i = 0;
  __pyx_t_5numpy_float64_t __pyx_v_d;
  __pyx_t_5numpy_intp_t __pyx_v_i;
  __pyx_t_5numpy_intp_t __pyx_v_j;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __pyx_t_5numpy_intp_t __pyx_t_3;
  __pyx_t_5numpy_intp_t __pyx_t_4;
  __pyx_t_5numpy_intp_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  __pyx_t_5numpy_intp_t __pyx_t_7;
  __pyx_t_5numpy_intp_t __pyx_t_8;
  __pyx_t_5numpy_intp_t __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__query_ball_tree_traverse_checking", 0);

  /* "pyart/map/ckdtree.pyx":1491
 *         cdef np.intp_t i, j
 * 
 *         if tracker.min_distance > tracker.upper_bound * tracker.epsfac:             # <<<<<<<<<<<<<<
 *             return 0
 *         elif tracker.max_distance < tracker.upper_bound / tracker.epsfac:
 */
  __pyx_t_1 = ((__pyx_v_tracker->min_distance > (__pyx_v_tracker->upper_bound * __pyx_v_tracker->epsfac)) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":1492
 * 
 *         if tracker.min_distance > tracker.upper_bound * tracker.epsfac:
 *             return 0             # <<<<<<<<<<<<<<
 *         elif tracker.max_distance < tracker.upper_bound / tracker.epsfac:
 *             self.__query_ball_tree_traverse_no_checking(other, results, node1, node2)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "pyart/map/ckdtree.pyx":1491
 *         cdef np.intp_t i, j
 * 
 *         if tracker.min_distance > tracker.upper_bound * tracker.epsfac:             # <<<<<<<<<<<<<<
 *             return 0
 *         elif tracker.max_distance < tracker.upper_bound / tracker.epsfac:
 */
  }

  /* "pyart/map/ckdtree.pyx":1493
 *         if tracker.min_distance > tracker.upper_bound * tracker.epsfac:
 *             return 0
 *         elif tracker.max_distance < tracker.upper_bound / tracker.epsfac:             # <<<<<<<<<<<<<<
 *             self.__query_ball_tree_traverse_no_checking(other, results, node1, node2)
 *         elif node1.split_dim == -1:  # 1 is leaf node
 */
  __pyx_t_1 = ((__pyx_v_tracker->max_distance < (__pyx_v_tracker->upper_bound / __pyx_v_tracker->epsfac)) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":1494
 *             return 0
 *         elif tracker.max_distance < tracker.upper_bound / tracker.epsfac:
 *             self.__query_ball_tree_traverse_no_checking(other, results, node1, node2)             # <<<<<<<<<<<<<<
 *         elif node1.split_dim == -1:  # 1 is leaf node
 *             lnode1 = <leafnode*>node1
 */
    __pyx_t_2 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_ball_tree_traverse_no_checking(__pyx_v_self, __pyx_v_other, __pyx_v_results, __pyx_v_node1, __pyx_v_node2); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1494, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":1493
 *         if tracker.min_distance > tracker.upper_bound * tracker.epsfac:
 *             return 0
 *         elif tracker.max_distance < tracker.upper_bound / tracker.epsfac:             # <<<<<<<<<<<<<<
 *             self.__query_ball_tree_traverse_no_checking(other, results, node1, node2)
 *         elif node1.split_dim == -1:  # 1 is leaf node
 */
    goto __pyx_L3;
  }

  /* "pyart/map/ckdtree.pyx":1495
 *         elif tracker.max_distance < tracker.upper_bound / tracker.epsfac:
 *             self.__query_ball_tree_traverse_no_checking(other, results, node1, node2)
 *         elif node1.split_dim == -1:  # 1 is leaf node             # <<<<<<<<<<<<<<
 *             lnode1 = <leafnode*>node1
 * 
 */
  __pyx_t_1 = ((__pyx_v_node1->split_dim == -1L) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":1496
 *             self.__query_ball_tree_traverse_no_checking(other, results, node1, node2)
 *         elif node1.split_dim == -1:  # 1 is leaf node
 *             lnode1 = <leafnode*>node1             # <<<<<<<<<<<<<<
 * 
 *             if node2.split_dim == -1:  # 1 & 2 are leaves
 */
    __pyx_v_lnode1 = ((struct __pyx_t_5pyart_3map_7ckdtree_leafnode *)__pyx_v_node1);

    /* "pyart/map/ckdtree.pyx":1498
 *             lnode1 = <leafnode*>node1
 * 
 *             if node2.split_dim == -1:  # 1 & 2 are leaves             # <<<<<<<<<<<<<<
 *                 lnode2 = <leafnode*>node2
 * 
 */
    __pyx_t_1 = ((__pyx_v_node2->split_dim == -1L) != 0);
    if (__pyx_t_1) {

      /* "pyart/map/ckdtree.pyx":1499
 * 
 *             if node2.split_dim == -1:  # 1 & 2 are leaves
 *                 lnode2 = <leafnode*>node2             # <<<<<<<<<<<<<<
 * 
 *                 # brute-force
 */
      __pyx_v_lnode2 = ((struct __pyx_t_5pyart_3map_7ckdtree_leafnode *)__pyx_v_node2);

      /* "pyart/map/ckdtree.pyx":1502
 * 
 *                 # brute-force
 *                 for i in range(lnode1.start_idx, lnode1.end_idx):             # <<<<<<<<<<<<<<
 *                     results_i = results[self.raw_indices[i]]
 *                     for j in range(lnode2.start_idx, lnode2.end_idx):
 */
      __pyx_t_3 = __pyx_v_lnode1->end_idx;
      __pyx_t_4 = __pyx_t_3;
      for (__pyx_t_5 = __pyx_v_lnode1->start_idx; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
        __pyx_v_i = __pyx_t_5;

        /* "pyart/map/ckdtree.pyx":1503
 *                 # brute-force
 *                 for i in range(lnode1.start_idx, lnode1.end_idx):
 *                     results_i = results[self.raw_indices[i]]             # <<<<<<<<<<<<<<
 *                     for j in range(lnode2.start_idx, lnode2.end_idx):
 *                         d = _distance_p(
 */
        if (unlikely(__pyx_v_results == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 1503, __pyx_L1_error)
        }
        __pyx_t_6 = __Pyx_GetItemInt_List(__pyx_v_results, (__pyx_v_self->raw_indices[__pyx_v_i]), __pyx_t_5numpy_intp_t, 1, __Pyx_PyInt_From_Py_intptr_t, 1, 1, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1503, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (!(likely(PyList_CheckExact(__pyx_t_6))||((__pyx_t_6) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_6)->tp_name), 0))) __PYX_ERR(0, 1503, __pyx_L1_error)
        __Pyx_XDECREF_SET(__pyx_v_results_i, ((PyObject*)__pyx_t_6));
        __pyx_t_6 = 0;

        /* "pyart/map/ckdtree.pyx":1504
 *                 for i in range(lnode1.start_idx, lnode1.end_idx):
 *                     results_i = results[self.raw_indices[i]]
 *                     for j in range(lnode2.start_idx, lnode2.end_idx):             # <<<<<<<<<<<<<<
 *                         d = _distance_p(
 *                             self.raw_data + self.raw_indices[i] * self.m,
 */
        __pyx_t_7 = __pyx_v_lnode2->end_idx;
        __pyx_t_8 = __pyx_t_7;
        for (__pyx_t_9 = __pyx_v_lnode2->start_idx; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_j = __pyx_t_9;

          /* "pyart/map/ckdtree.pyx":1505
 *                     results_i = results[self.raw_indices[i]]
 *                     for j in range(lnode2.start_idx, lnode2.end_idx):
 *                         d = _distance_p(             # <<<<<<<<<<<<<<
 *                             self.raw_data + self.raw_indices[i] * self.m,
 *                             other.raw_data + other.raw_indices[j] * other.m,
 */
          __pyx_v_d = __pyx_f_5pyart_3map_7ckdtree__distance_p((__pyx_v_self->raw_data + ((__pyx_v_self->raw_indices[__pyx_v_i]) * __pyx_v_self->m)), (__pyx_v_other->raw_data + ((__pyx_v_other->raw_indices[__pyx_v_j]) * __pyx_v_other->m)), __pyx_v_tracker->p, __pyx_v_self->m, __pyx_v_tracker->upper_bound);

          /* "pyart/map/ckdtree.pyx":1509
 *                             other.raw_data + other.raw_indices[j] * other.m,
 *                             tracker.p, self.m, tracker.upper_bound)
 *                         if d <= tracker.upper_bound:             # <<<<<<<<<<<<<<
 *                             list_append(results_i, other.raw_indices[j])
 * 
 */
          __pyx_t_1 = ((__pyx_v_d <= __pyx_v_tracker->upper_bound) != 0);
          if (__pyx_t_1) {

            /* "pyart/map/ckdtree.pyx":1510
 *                             tracker.p, self.m, tracker.upper_bound)
 *                         if d <= tracker.upper_bound:
 *                             list_append(results_i, other.raw_indices[j])             # <<<<<<<<<<<<<<
 * 
 *             else:  # 1 is a leaf node, 2 is inner node
 */
            __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_list_append(__pyx_v_results_i, (__pyx_v_other->raw_indices[__pyx_v_j])); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1510, __pyx_L1_error)

            /* "pyart/map/ckdtree.pyx":1509
 *                             other.raw_data + other.raw_indices[j] * other.m,
 *                             tracker.p, self.m, tracker.upper_bound)
 *                         if d <= tracker.upper_bound:             # <<<<<<<<<<<<<<
 *                             list_append(results_i, other.raw_indices[j])
 * 
 */
          }
        }
      }

      /* "pyart/map/ckdtree.pyx":1498
 *             lnode1 = <leafnode*>node1
 * 
 *             if node2.split_dim == -1:  # 1 & 2 are leaves             # <<<<<<<<<<<<<<
 *                 lnode2 = <leafnode*>node2
 * 
 */
      goto __pyx_L4;
    }

    /* "pyart/map/ckdtree.pyx":1514
 *             else:  # 1 is a leaf node, 2 is inner node
 * 
 *                 tracker.push_less_of(2, node2)             # <<<<<<<<<<<<<<
 *                 self.__query_ball_tree_traverse_checking(
 *                     other, results, node1, node2.less, tracker)
 */
    /*else*/ {
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_less_of(__pyx_v_tracker, 2, __pyx_v_node2); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1514, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1515
 * 
 *                 tracker.push_less_of(2, node2)
 *                 self.__query_ball_tree_traverse_checking(             # <<<<<<<<<<<<<<
 *                     other, results, node1, node2.less, tracker)
 *                 tracker.pop()
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_ball_tree_traverse_checking(__pyx_v_self, __pyx_v_other, __pyx_v_results, __pyx_v_node1, __pyx_v_node2->less, __pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1515, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1517
 *                 self.__query_ball_tree_traverse_checking(
 *                     other, results, node1, node2.less, tracker)
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *                 tracker.push_greater_of(2, node2)
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1517, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1519
 *                 tracker.pop()
 * 
 *                 tracker.push_greater_of(2, node2)             # <<<<<<<<<<<<<<
 *                 self.__query_ball_tree_traverse_checking(
 *                     other, results, node1, node2.greater, tracker)
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_greater_of(__pyx_v_tracker, 2, __pyx_v_node2); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1519, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1520
 * 
 *                 tracker.push_greater_of(2, node2)
 *                 self.__query_ball_tree_traverse_checking(             # <<<<<<<<<<<<<<
 *                     other, results, node1, node2.greater, tracker)
 *                 tracker.pop()
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_ball_tree_traverse_checking(__pyx_v_self, __pyx_v_other, __pyx_v_results, __pyx_v_node1, __pyx_v_node2->greater, __pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1520, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1522
 *                 self.__query_ball_tree_traverse_checking(
 *                     other, results, node1, node2.greater, tracker)
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1522, __pyx_L1_error)
    }
    __pyx_L4:;

    /* "pyart/map/ckdtree.pyx":1495
 *         elif tracker.max_distance < tracker.upper_bound / tracker.epsfac:
 *             self.__query_ball_tree_traverse_no_checking(other, results, node1, node2)
 *         elif node1.split_dim == -1:  # 1 is leaf node             # <<<<<<<<<<<<<<
 *             lnode1 = <leafnode*>node1
 * 
 */
    goto __pyx_L3;
  }

  /* "pyart/map/ckdtree.pyx":1526
 * 
 *         else:  # 1 is an inner node
 *             if node2.split_dim == -1:  # 1 is an inner node, 2 is a leaf node             # <<<<<<<<<<<<<<
 *                 tracker.push_less_of(1, node1)
 *                 self.__query_ball_tree_traverse_checking(
 */
  /*else*/ {
    __pyx_t_1 = ((__pyx_v_node2->split_dim == -1L) != 0);
    if (__pyx_t_1) {

      /* "pyart/map/ckdtree.pyx":1527
 *         else:  # 1 is an inner node
 *             if node2.split_dim == -1:  # 1 is an inner node, 2 is a leaf node
 *                 tracker.push_less_of(1, node1)             # <<<<<<<<<<<<<<
 *                 self.__query_ball_tree_traverse_checking(
 *                     other, results, node1.less, node2, tracker)
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_less_of(__pyx_v_tracker, 1, __pyx_v_node1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1527, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1528
 *             if node2.split_dim == -1:  # 1 is an inner node, 2 is a leaf node
 *                 tracker.push_less_of(1, node1)
 *                 self.__query_ball_tree_traverse_checking(             # <<<<<<<<<<<<<<
 *                     other, results, node1.less, node2, tracker)
 *                 tracker.pop()
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_ball_tree_traverse_checking(__pyx_v_self, __pyx_v_other, __pyx_v_results, __pyx_v_node1->less, __pyx_v_node2, __pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1528, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1530
 *                 self.__query_ball_tree_traverse_checking(
 *                     other, results, node1.less, node2, tracker)
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *                 tracker.push_greater_of(1, node1)
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1530, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1532
 *                 tracker.pop()
 * 
 *                 tracker.push_greater_of(1, node1)             # <<<<<<<<<<<<<<
 *                 self.__query_ball_tree_traverse_checking(
 *                     other, results, node1.greater, node2, tracker)
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_greater_of(__pyx_v_tracker, 1, __pyx_v_node1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1532, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1533
 * 
 *                 tracker.push_greater_of(1, node1)
 *                 self.__query_ball_tree_traverse_checking(             # <<<<<<<<<<<<<<
 *                     other, results, node1.greater, node2, tracker)
 *                 tracker.pop()
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_ball_tree_traverse_checking(__pyx_v_self, __pyx_v_other, __pyx_v_results, __pyx_v_node1->greater, __pyx_v_node2, __pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1533, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1535
 *                 self.__query_ball_tree_traverse_checking(
 *                     other, results, node1.greater, node2, tracker)
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *             else: # 1 & 2 are inner nodes
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1535, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1526
 * 
 *         else:  # 1 is an inner node
 *             if node2.split_dim == -1:  # 1 is an inner node, 2 is a leaf node             # <<<<<<<<<<<<<<
 *                 tracker.push_less_of(1, node1)
 *                 self.__query_ball_tree_traverse_checking(
 */
      goto __pyx_L10;
    }

    /* "pyart/map/ckdtree.pyx":1539
 *             else: # 1 & 2 are inner nodes
 * 
 *                 tracker.push_less_of(1, node1)             # <<<<<<<<<<<<<<
 *                 tracker.push_less_of(2, node2)
 *                 self.__query_ball_tree_traverse_checking(
 */
    /*else*/ {
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_less_of(__pyx_v_tracker, 1, __pyx_v_node1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1539, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1540
 * 
 *                 tracker.push_less_of(1, node1)
 *                 tracker.push_less_of(2, node2)             # <<<<<<<<<<<<<<
 *                 self.__query_ball_tree_traverse_checking(
 *                     other, results, node1.less, node2.less, tracker)
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_less_of(__pyx_v_tracker, 2, __pyx_v_node2); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1540, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1541
 *                 tracker.push_less_of(1, node1)
 *                 tracker.push_less_of(2, node2)
 *                 self.__query_ball_tree_traverse_checking(             # <<<<<<<<<<<<<<
 *                     other, results, node1.less, node2.less, tracker)
 *                 tracker.pop()
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_ball_tree_traverse_checking(__pyx_v_self, __pyx_v_other, __pyx_v_results, __pyx_v_node1->less, __pyx_v_node2->less, __pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1541, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1543
 *                 self.__query_ball_tree_traverse_checking(
 *                     other, results, node1.less, node2.less, tracker)
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *                 tracker.push_greater_of(2, node2)
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1543, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1545
 *                 tracker.pop()
 * 
 *                 tracker.push_greater_of(2, node2)             # <<<<<<<<<<<<<<
 *                 self.__query_ball_tree_traverse_checking(
 *                     other, results, node1.less, node2.greater, tracker)
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_greater_of(__pyx_v_tracker, 2, __pyx_v_node2); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1545, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1546
 * 
 *                 tracker.push_greater_of(2, node2)
 *                 self.__query_ball_tree_traverse_checking(             # <<<<<<<<<<<<<<
 *                     other, results, node1.less, node2.greater, tracker)
 *                 tracker.pop()
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_ball_tree_traverse_checking(__pyx_v_self, __pyx_v_other, __pyx_v_results, __pyx_v_node1->less, __pyx_v_node2->greater, __pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1546, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1548
 *                 self.__query_ball_tree_traverse_checking(
 *                     other, results, node1.less, node2.greater, tracker)
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 *                 tracker.pop()
 * 
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1548, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1549
 *                     other, results, node1.less, node2.greater, tracker)
 *                 tracker.pop()
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1549, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1552
 * 
 * 
 *                 tracker.push_greater_of(1, node1)             # <<<<<<<<<<<<<<
 *                 tracker.push_less_of(2, node2)
 *                 self.__query_ball_tree_traverse_checking(
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_greater_of(__pyx_v_tracker, 1, __pyx_v_node1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1552, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1553
 * 
 *                 tracker.push_greater_of(1, node1)
 *                 tracker.push_less_of(2, node2)             # <<<<<<<<<<<<<<
 *                 self.__query_ball_tree_traverse_checking(
 *                     other, results, node1.greater, node2.less, tracker)
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_less_of(__pyx_v_tracker, 2, __pyx_v_node2); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1553, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1554
 *                 tracker.push_greater_of(1, node1)
 *                 tracker.push_less_of(2, node2)
 *                 self.__query_ball_tree_traverse_checking(             # <<<<<<<<<<<<<<
 *                     other, results, node1.greater, node2.less, tracker)
 *                 tracker.pop()
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_ball_tree_traverse_checking(__pyx_v_self, __pyx_v_other, __pyx_v_results, __pyx_v_node1->greater, __pyx_v_node2->less, __pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1554, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1556
 *                 self.__query_ball_tree_traverse_checking(
 *                     other, results, node1.greater, node2.less, tracker)
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *                 tracker.push_greater_of(2, node2)
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1556, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1558
 *                 tracker.pop()
 * 
 *                 tracker.push_greater_of(2, node2)             # <<<<<<<<<<<<<<
 *                 self.__query_ball_tree_traverse_checking(
 *                     other, results, node1.greater, node2.greater, tracker)
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_greater_of(__pyx_v_tracker, 2, __pyx_v_node2); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1558, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1559
 * 
 *                 tracker.push_greater_of(2, node2)
 *                 self.__query_ball_tree_traverse_checking(             # <<<<<<<<<<<<<<
 *                     other, results, node1.greater, node2.greater, tracker)
 *                 tracker.pop()
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_ball_tree_traverse_checking(__pyx_v_self, __pyx_v_other, __pyx_v_results, __pyx_v_node1->greater, __pyx_v_node2->greater, __pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1559, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1561
 *                 self.__query_ball_tree_traverse_checking(
 *                     other, results, node1.greater, node2.greater, tracker)
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 *                 tracker.pop()
 * 
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1561, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1562
 *                     other, results, node1.greater, node2.greater, tracker)
 *                 tracker.pop()
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *         return 0
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1562, __pyx_L1_error)
    }
    __pyx_L10:;
  }
  __pyx_L3:;

  /* "pyart/map/ckdtree.pyx":1564
 *                 tracker.pop()
 * 
 *         return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":1479
 * 
 *     @cython.cdivision(True)
 *     cdef int __query_ball_tree_traverse_checking(cKDTree self,             # <<<<<<<<<<<<<<
 *                                                  cKDTree other,
 *                                                  list results,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.__query_ball_tree_traverse_checking", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_results_i);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":1567
 * 
 * 
 *     def query_ball_tree(cKDTree self, cKDTree other,             # <<<<<<<<<<<<<<
 *                         np.float64_t r, np.float64_t p=2., np.float64_t eps=0):
 *         """query_ball_tree(self, other, r, p, eps)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_9query_ball_tree(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5pyart_3map_7ckdtree_7cKDTree_8query_ball_tree[] = "query_ball_tree(self, other, r, p, eps)\n\n        Find all pairs of points whose distance is at most r\n\n        Parameters\n        ----------\n        other : KDTree instance\n            The tree containing points to search against.\n        r : float\n            The maximum distance, has to be positive.\n        p : float, optional\n            Which Minkowski norm to use.  `p` has to meet the condition\n            ``1 <= p <= infinity``.\n        eps : float, optional\n            Approximate search.  Branches of the tree are not explored\n            if their nearest points are further than ``r/(1+eps)``, and\n            branches are added in bulk if their furthest points are nearer\n            than ``r * (1+eps)``.  `eps` has to be non-negative.\n\n        Returns\n        -------\n        results : list of lists\n            For each element ``self.data[i]`` of this tree, ``results[i]`` is a\n            list of the indices of its neighbors in ``other.data``.\n\n        ";
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_9query_ball_tree(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_other = 0;
  __pyx_t_5numpy_float64_t __pyx_v_r;
  __pyx_t_5numpy_float64_t __pyx_v_p;
  __pyx_t_5numpy_float64_t __pyx_v_eps;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("query_ball_tree (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_other,&__pyx_n_s_r,&__pyx_n_s_p,&__pyx_n_s_eps,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_other)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_r)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("query_ball_tree", 0, 2, 4, 1); __PYX_ERR(0, 1567, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_p);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_eps);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "query_ball_tree") < 0)) __PYX_ERR(0, 1567, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_other = ((struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *)values[0]);
    __pyx_v_r = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_r == ((npy_float64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1568, __pyx_L3_error)
    if (values[2]) {
      __pyx_v_p = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_p == ((npy_float64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1568, __pyx_L3_error)
    } else {
      __pyx_v_p = ((__pyx_t_5numpy_float64_t)2.);
    }
    if (values[3]) {
      __pyx_v_eps = __pyx_PyFloat_AsDouble(values[3]); if (unlikely((__pyx_v_eps == ((npy_float64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1568, __pyx_L3_error)
    } else {
      __pyx_v_eps = ((__pyx_t_5numpy_float64_t)0.0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("query_ball_tree", 0, 2, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1567, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.query_ball_tree", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_5pyart_3map_7ckdtree_cKDTree, 1, "other", 0))) __PYX_ERR(0, 1567, __pyx_L1_error)
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_7cKDTree_8query_ball_tree(((struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self), __pyx_v_other, __pyx_v_r, __pyx_v_p, __pyx_v_eps);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_8query_ball_tree(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_other, __pyx_t_5numpy_float64_t __pyx_v_r, __pyx_t_5numpy_float64_t __pyx_v_p, __pyx_t_5numpy_float64_t __pyx_v_eps) {
  struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_tracker = NULL;
  PyObject *__pyx_v_results = NULL;
  CYTHON_UNUSED __pyx_t_5numpy_intp_t __pyx_7genexpr__pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  __pyx_t_5numpy_intp_t __pyx_t_8;
  __pyx_t_5numpy_intp_t __pyx_t_9;
  __pyx_t_5numpy_intp_t __pyx_t_10;
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("query_ball_tree", 0);

  /* "pyart/map/ckdtree.pyx":1597
 * 
 *         # Make sure trees are compatible
 *         if self.m != other.m:             # <<<<<<<<<<<<<<
 *             raise ValueError("Trees passed to query_ball_trees have different dimensionality")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->m != __pyx_v_other->m) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "pyart/map/ckdtree.pyx":1598
 *         # Make sure trees are compatible
 *         if self.m != other.m:
 *             raise ValueError("Trees passed to query_ball_trees have different dimensionality")             # <<<<<<<<<<<<<<
 * 
 *         # Track node-to-node min/max distances
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__19, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1598, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1598, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":1597
 * 
 *         # Make sure trees are compatible
 *         if self.m != other.m:             # <<<<<<<<<<<<<<
 *             raise ValueError("Trees passed to query_ball_trees have different dimensionality")
 * 
 */
  }

  /* "pyart/map/ckdtree.pyx":1602
 *         # Track node-to-node min/max distances
 *         tracker = RectRectDistanceTracker(
 *             Rectangle(self.mins, self.maxes),             # <<<<<<<<<<<<<<
 *             Rectangle(other.mins, other.maxes),
 *             p, eps, r)
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1602, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->mins));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->mins));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_self->mins));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->maxes));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->maxes));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self->maxes));
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pyart_3map_7ckdtree_Rectangle), __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1602, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyart/map/ckdtree.pyx":1603
 *         tracker = RectRectDistanceTracker(
 *             Rectangle(self.mins, self.maxes),
 *             Rectangle(other.mins, other.maxes),             # <<<<<<<<<<<<<<
 *             p, eps, r)
 * 
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1603, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_other->mins));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_other->mins));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_other->mins));
  __Pyx_INCREF(((PyObject *)__pyx_v_other->maxes));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_other->maxes));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_other->maxes));
  __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pyart_3map_7ckdtree_Rectangle), __pyx_t_2, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1603, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyart/map/ckdtree.pyx":1604
 *             Rectangle(self.mins, self.maxes),
 *             Rectangle(other.mins, other.maxes),
 *             p, eps, r)             # <<<<<<<<<<<<<<
 * 
 *         results = [[] for i in range(self.n)]
 */
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_p); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1604, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = PyFloat_FromDouble(__pyx_v_eps); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1604, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyFloat_FromDouble(__pyx_v_r); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1604, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);

  /* "pyart/map/ckdtree.pyx":1601
 * 
 *         # Track node-to-node min/max distances
 *         tracker = RectRectDistanceTracker(             # <<<<<<<<<<<<<<
 *             Rectangle(self.mins, self.maxes),
 *             Rectangle(other.mins, other.maxes),
 */
  __pyx_t_7 = PyTuple_New(5); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1601, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_7, 2, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_7, 3, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_7, 4, __pyx_t_6);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_2 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pyart_3map_7ckdtree_RectRectDistanceTracker), __pyx_t_7, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1601, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_tracker = ((struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "pyart/map/ckdtree.pyx":1606
 *             p, eps, r)
 * 
 *         results = [[] for i in range(self.n)]             # <<<<<<<<<<<<<<
 *         self.__query_ball_tree_traverse_checking(
 *             other, results, self.tree, other.tree, tracker)
 */
  { /* enter inner scope */
    __pyx_t_6 = PyList_New(0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1606, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = __pyx_v_self->n;
    __pyx_t_9 = __pyx_t_8;
    for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
      __pyx_7genexpr__pyx_v_i = __pyx_t_10;
      __pyx_t_7 = PyList_New(0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1606, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_6, (PyObject*)__pyx_t_7))) __PYX_ERR(0, 1606, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
  } /* exit inner scope */
  __pyx_v_results = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "pyart/map/ckdtree.pyx":1607
 * 
 *         results = [[] for i in range(self.n)]
 *         self.__query_ball_tree_traverse_checking(             # <<<<<<<<<<<<<<
 *             other, results, self.tree, other.tree, tracker)
 * 
 */
  __pyx_t_11 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_ball_tree_traverse_checking(__pyx_v_self, __pyx_v_other, __pyx_v_results, __pyx_v_self->tree, __pyx_v_other->tree, __pyx_v_tracker); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(0, 1607, __pyx_L1_error)

  /* "pyart/map/ckdtree.pyx":1610
 *             other, results, self.tree, other.tree, tracker)
 * 
 *         return results             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_results);
  __pyx_r = __pyx_v_results;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":1567
 * 
 * 
 *     def query_ball_tree(cKDTree self, cKDTree other,             # <<<<<<<<<<<<<<
 *                         np.float64_t r, np.float64_t p=2., np.float64_t eps=0):
 *         """query_ball_tree(self, other, r, p, eps)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.query_ball_tree", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_tracker);
  __Pyx_XDECREF(__pyx_v_results);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":1616
 *     # query_pairs
 *     # -----------
 *     cdef int __query_pairs_traverse_no_checking(cKDTree self,             # <<<<<<<<<<<<<<
 *                                                 set results,
 *                                                 innernode* node1,
 */

static int __pyx_f_5pyart_3map_7ckdtree_7cKDTree___query_pairs_traverse_no_checking(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, PyObject *__pyx_v_results, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node1, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node2) {
  struct __pyx_t_5pyart_3map_7ckdtree_leafnode *__pyx_v_lnode1;
  struct __pyx_t_5pyart_3map_7ckdtree_leafnode *__pyx_v_lnode2;
  __pyx_t_5numpy_intp_t __pyx_v_i;
  __pyx_t_5numpy_intp_t __pyx_v_j;
  __pyx_t_5numpy_intp_t __pyx_v_min_j;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __pyx_t_5numpy_intp_t __pyx_t_2;
  __pyx_t_5numpy_intp_t __pyx_t_3;
  __pyx_t_5numpy_intp_t __pyx_t_4;
  __pyx_t_5numpy_intp_t __pyx_t_5;
  __pyx_t_5numpy_intp_t __pyx_t_6;
  __pyx_t_5numpy_intp_t __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__query_pairs_traverse_no_checking", 0);

  /* "pyart/map/ckdtree.pyx":1625
 *         cdef np.intp_t i, j, min_j
 * 
 *         if node1.split_dim == -1:  # leaf node             # <<<<<<<<<<<<<<
 *             lnode1 = <leafnode*>node1
 * 
 */
  __pyx_t_1 = ((__pyx_v_node1->split_dim == -1L) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":1626
 * 
 *         if node1.split_dim == -1:  # leaf node
 *             lnode1 = <leafnode*>node1             # <<<<<<<<<<<<<<
 * 
 *             if node2.split_dim == -1:  # leaf node
 */
    __pyx_v_lnode1 = ((struct __pyx_t_5pyart_3map_7ckdtree_leafnode *)__pyx_v_node1);

    /* "pyart/map/ckdtree.pyx":1628
 *             lnode1 = <leafnode*>node1
 * 
 *             if node2.split_dim == -1:  # leaf node             # <<<<<<<<<<<<<<
 *                 lnode2 = <leafnode*>node2
 * 
 */
    __pyx_t_1 = ((__pyx_v_node2->split_dim == -1L) != 0);
    if (__pyx_t_1) {

      /* "pyart/map/ckdtree.pyx":1629
 * 
 *             if node2.split_dim == -1:  # leaf node
 *                 lnode2 = <leafnode*>node2             # <<<<<<<<<<<<<<
 * 
 *                 for i in range(lnode1.start_idx, lnode1.end_idx):
 */
      __pyx_v_lnode2 = ((struct __pyx_t_5pyart_3map_7ckdtree_leafnode *)__pyx_v_node2);

      /* "pyart/map/ckdtree.pyx":1631
 *                 lnode2 = <leafnode*>node2
 * 
 *                 for i in range(lnode1.start_idx, lnode1.end_idx):             # <<<<<<<<<<<<<<
 *                     # Special care here to avoid duplicate pairs
 *                     if node1 == node2:
 */
      __pyx_t_2 = __pyx_v_lnode1->end_idx;
      __pyx_t_3 = __pyx_t_2;
      for (__pyx_t_4 = __pyx_v_lnode1->start_idx; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
        __pyx_v_i = __pyx_t_4;

        /* "pyart/map/ckdtree.pyx":1633
 *                 for i in range(lnode1.start_idx, lnode1.end_idx):
 *                     # Special care here to avoid duplicate pairs
 *                     if node1 == node2:             # <<<<<<<<<<<<<<
 *                         min_j = i + 1
 *                     else:
 */
        __pyx_t_1 = ((__pyx_v_node1 == __pyx_v_node2) != 0);
        if (__pyx_t_1) {

          /* "pyart/map/ckdtree.pyx":1634
 *                     # Special care here to avoid duplicate pairs
 *                     if node1 == node2:
 *                         min_j = i + 1             # <<<<<<<<<<<<<<
 *                     else:
 *                         min_j = lnode2.start_idx
 */
          __pyx_v_min_j = (__pyx_v_i + 1);

          /* "pyart/map/ckdtree.pyx":1633
 *                 for i in range(lnode1.start_idx, lnode1.end_idx):
 *                     # Special care here to avoid duplicate pairs
 *                     if node1 == node2:             # <<<<<<<<<<<<<<
 *                         min_j = i + 1
 *                     else:
 */
          goto __pyx_L7;
        }

        /* "pyart/map/ckdtree.pyx":1636
 *                         min_j = i + 1
 *                     else:
 *                         min_j = lnode2.start_idx             # <<<<<<<<<<<<<<
 * 
 *                     for j in range(min_j, lnode2.end_idx):
 */
        /*else*/ {
          __pyx_t_5 = __pyx_v_lnode2->start_idx;
          __pyx_v_min_j = __pyx_t_5;
        }
        __pyx_L7:;

        /* "pyart/map/ckdtree.pyx":1638
 *                         min_j = lnode2.start_idx
 * 
 *                     for j in range(min_j, lnode2.end_idx):             # <<<<<<<<<<<<<<
 *                         set_add_ordered_pair(results,
 *                                              self.raw_indices[i],
 */
        __pyx_t_5 = __pyx_v_lnode2->end_idx;
        __pyx_t_6 = __pyx_t_5;
        for (__pyx_t_7 = __pyx_v_min_j; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
          __pyx_v_j = __pyx_t_7;

          /* "pyart/map/ckdtree.pyx":1639
 * 
 *                     for j in range(min_j, lnode2.end_idx):
 *                         set_add_ordered_pair(results,             # <<<<<<<<<<<<<<
 *                                              self.raw_indices[i],
 *                                              self.raw_indices[j])
 */
          __pyx_t_8 = __pyx_f_5pyart_3map_7ckdtree_set_add_ordered_pair(__pyx_v_results, (__pyx_v_self->raw_indices[__pyx_v_i]), (__pyx_v_self->raw_indices[__pyx_v_j])); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 1639, __pyx_L1_error)
        }
      }

      /* "pyart/map/ckdtree.pyx":1628
 *             lnode1 = <leafnode*>node1
 * 
 *             if node2.split_dim == -1:  # leaf node             # <<<<<<<<<<<<<<
 *                 lnode2 = <leafnode*>node2
 * 
 */
      goto __pyx_L4;
    }

    /* "pyart/map/ckdtree.pyx":1644
 * 
 *             else:
 *                 self.__query_pairs_traverse_no_checking(results, node1, node2.less)             # <<<<<<<<<<<<<<
 *                 self.__query_pairs_traverse_no_checking(results, node1, node2.greater)
 *         else:
 */
    /*else*/ {
      __pyx_t_8 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_pairs_traverse_no_checking(__pyx_v_self, __pyx_v_results, __pyx_v_node1, __pyx_v_node2->less); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 1644, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1645
 *             else:
 *                 self.__query_pairs_traverse_no_checking(results, node1, node2.less)
 *                 self.__query_pairs_traverse_no_checking(results, node1, node2.greater)             # <<<<<<<<<<<<<<
 *         else:
 *             if node1 == node2:
 */
      __pyx_t_8 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_pairs_traverse_no_checking(__pyx_v_self, __pyx_v_results, __pyx_v_node1, __pyx_v_node2->greater); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 1645, __pyx_L1_error)
    }
    __pyx_L4:;

    /* "pyart/map/ckdtree.pyx":1625
 *         cdef np.intp_t i, j, min_j
 * 
 *         if node1.split_dim == -1:  # leaf node             # <<<<<<<<<<<<<<
 *             lnode1 = <leafnode*>node1
 * 
 */
    goto __pyx_L3;
  }

  /* "pyart/map/ckdtree.pyx":1647
 *                 self.__query_pairs_traverse_no_checking(results, node1, node2.greater)
 *         else:
 *             if node1 == node2:             # <<<<<<<<<<<<<<
 *                 # Avoid traversing (node1.less, node2.greater) and
 *                 # (node1.greater, node2.less) (it's the same node pair twice
 */
  /*else*/ {
    __pyx_t_1 = ((__pyx_v_node1 == __pyx_v_node2) != 0);
    if (__pyx_t_1) {

      /* "pyart/map/ckdtree.pyx":1652
 *                 # over, which is the source of the complication in the
 *                 # original KDTree.query_pairs)
 *                 self.__query_pairs_traverse_no_checking(results, node1.less, node2.less)             # <<<<<<<<<<<<<<
 *                 self.__query_pairs_traverse_no_checking(results, node1.less, node2.greater)
 *                 self.__query_pairs_traverse_no_checking(results, node1.greater, node2.greater)
 */
      __pyx_t_8 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_pairs_traverse_no_checking(__pyx_v_self, __pyx_v_results, __pyx_v_node1->less, __pyx_v_node2->less); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 1652, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1653
 *                 # original KDTree.query_pairs)
 *                 self.__query_pairs_traverse_no_checking(results, node1.less, node2.less)
 *                 self.__query_pairs_traverse_no_checking(results, node1.less, node2.greater)             # <<<<<<<<<<<<<<
 *                 self.__query_pairs_traverse_no_checking(results, node1.greater, node2.greater)
 *             else:
 */
      __pyx_t_8 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_pairs_traverse_no_checking(__pyx_v_self, __pyx_v_results, __pyx_v_node1->less, __pyx_v_node2->greater); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 1653, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1654
 *                 self.__query_pairs_traverse_no_checking(results, node1.less, node2.less)
 *                 self.__query_pairs_traverse_no_checking(results, node1.less, node2.greater)
 *                 self.__query_pairs_traverse_no_checking(results, node1.greater, node2.greater)             # <<<<<<<<<<<<<<
 *             else:
 *                 self.__query_pairs_traverse_no_checking(results, node1.less, node2)
 */
      __pyx_t_8 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_pairs_traverse_no_checking(__pyx_v_self, __pyx_v_results, __pyx_v_node1->greater, __pyx_v_node2->greater); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 1654, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1647
 *                 self.__query_pairs_traverse_no_checking(results, node1, node2.greater)
 *         else:
 *             if node1 == node2:             # <<<<<<<<<<<<<<
 *                 # Avoid traversing (node1.less, node2.greater) and
 *                 # (node1.greater, node2.less) (it's the same node pair twice
 */
      goto __pyx_L10;
    }

    /* "pyart/map/ckdtree.pyx":1656
 *                 self.__query_pairs_traverse_no_checking(results, node1.greater, node2.greater)
 *             else:
 *                 self.__query_pairs_traverse_no_checking(results, node1.less, node2)             # <<<<<<<<<<<<<<
 *                 self.__query_pairs_traverse_no_checking(results, node1.greater, node2)
 * 
 */
    /*else*/ {
      __pyx_t_8 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_pairs_traverse_no_checking(__pyx_v_self, __pyx_v_results, __pyx_v_node1->less, __pyx_v_node2); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 1656, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1657
 *             else:
 *                 self.__query_pairs_traverse_no_checking(results, node1.less, node2)
 *                 self.__query_pairs_traverse_no_checking(results, node1.greater, node2)             # <<<<<<<<<<<<<<
 * 
 *         return 0
 */
      __pyx_t_8 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_pairs_traverse_no_checking(__pyx_v_self, __pyx_v_results, __pyx_v_node1->greater, __pyx_v_node2); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 1657, __pyx_L1_error)
    }
    __pyx_L10:;
  }
  __pyx_L3:;

  /* "pyart/map/ckdtree.pyx":1659
 *                 self.__query_pairs_traverse_no_checking(results, node1.greater, node2)
 * 
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     @cython.cdivision(True)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":1616
 *     # query_pairs
 *     # -----------
 *     cdef int __query_pairs_traverse_no_checking(cKDTree self,             # <<<<<<<<<<<<<<
 *                                                 set results,
 *                                                 innernode* node1,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.__query_pairs_traverse_no_checking", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":1662
 * 
 *     @cython.cdivision(True)
 *     cdef int __query_pairs_traverse_checking(cKDTree self,             # <<<<<<<<<<<<<<
 *                                              set results,
 *                                              innernode* node1,
 */

static int __pyx_f_5pyart_3map_7ckdtree_7cKDTree___query_pairs_traverse_checking(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, PyObject *__pyx_v_results, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node1, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node2, struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_tracker) {
  struct __pyx_t_5pyart_3map_7ckdtree_leafnode *__pyx_v_lnode1;
  struct __pyx_t_5pyart_3map_7ckdtree_leafnode *__pyx_v_lnode2;
  __pyx_t_5numpy_float64_t __pyx_v_d;
  __pyx_t_5numpy_intp_t __pyx_v_i;
  __pyx_t_5numpy_intp_t __pyx_v_j;
  __pyx_t_5numpy_intp_t __pyx_v_min_j;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __pyx_t_5numpy_intp_t __pyx_t_3;
  __pyx_t_5numpy_intp_t __pyx_t_4;
  __pyx_t_5numpy_intp_t __pyx_t_5;
  __pyx_t_5numpy_intp_t __pyx_t_6;
  __pyx_t_5numpy_intp_t __pyx_t_7;
  __pyx_t_5numpy_intp_t __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__query_pairs_traverse_checking", 0);

  /* "pyart/map/ckdtree.pyx":1673
 *         cdef np.intp_t i, j, min_j
 * 
 *         if tracker.min_distance > tracker.upper_bound * tracker.epsfac:             # <<<<<<<<<<<<<<
 *             return 0
 *         elif tracker.max_distance < tracker.upper_bound / tracker.epsfac:
 */
  __pyx_t_1 = ((__pyx_v_tracker->min_distance > (__pyx_v_tracker->upper_bound * __pyx_v_tracker->epsfac)) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":1674
 * 
 *         if tracker.min_distance > tracker.upper_bound * tracker.epsfac:
 *             return 0             # <<<<<<<<<<<<<<
 *         elif tracker.max_distance < tracker.upper_bound / tracker.epsfac:
 *             self.__query_pairs_traverse_no_checking(results, node1, node2)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "pyart/map/ckdtree.pyx":1673
 *         cdef np.intp_t i, j, min_j
 * 
 *         if tracker.min_distance > tracker.upper_bound * tracker.epsfac:             # <<<<<<<<<<<<<<
 *             return 0
 *         elif tracker.max_distance < tracker.upper_bound / tracker.epsfac:
 */
  }

  /* "pyart/map/ckdtree.pyx":1675
 *         if tracker.min_distance > tracker.upper_bound * tracker.epsfac:
 *             return 0
 *         elif tracker.max_distance < tracker.upper_bound / tracker.epsfac:             # <<<<<<<<<<<<<<
 *             self.__query_pairs_traverse_no_checking(results, node1, node2)
 *         elif node1.split_dim == -1:  # 1 is leaf node
 */
  __pyx_t_1 = ((__pyx_v_tracker->max_distance < (__pyx_v_tracker->upper_bound / __pyx_v_tracker->epsfac)) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":1676
 *             return 0
 *         elif tracker.max_distance < tracker.upper_bound / tracker.epsfac:
 *             self.__query_pairs_traverse_no_checking(results, node1, node2)             # <<<<<<<<<<<<<<
 *         elif node1.split_dim == -1:  # 1 is leaf node
 *             lnode1 = <leafnode*>node1
 */
    __pyx_t_2 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_pairs_traverse_no_checking(__pyx_v_self, __pyx_v_results, __pyx_v_node1, __pyx_v_node2); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1676, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":1675
 *         if tracker.min_distance > tracker.upper_bound * tracker.epsfac:
 *             return 0
 *         elif tracker.max_distance < tracker.upper_bound / tracker.epsfac:             # <<<<<<<<<<<<<<
 *             self.__query_pairs_traverse_no_checking(results, node1, node2)
 *         elif node1.split_dim == -1:  # 1 is leaf node
 */
    goto __pyx_L3;
  }

  /* "pyart/map/ckdtree.pyx":1677
 *         elif tracker.max_distance < tracker.upper_bound / tracker.epsfac:
 *             self.__query_pairs_traverse_no_checking(results, node1, node2)
 *         elif node1.split_dim == -1:  # 1 is leaf node             # <<<<<<<<<<<<<<
 *             lnode1 = <leafnode*>node1
 * 
 */
  __pyx_t_1 = ((__pyx_v_node1->split_dim == -1L) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":1678
 *             self.__query_pairs_traverse_no_checking(results, node1, node2)
 *         elif node1.split_dim == -1:  # 1 is leaf node
 *             lnode1 = <leafnode*>node1             # <<<<<<<<<<<<<<
 * 
 *             if node2.split_dim == -1:  # 1 & 2 are leaves
 */
    __pyx_v_lnode1 = ((struct __pyx_t_5pyart_3map_7ckdtree_leafnode *)__pyx_v_node1);

    /* "pyart/map/ckdtree.pyx":1680
 *             lnode1 = <leafnode*>node1
 * 
 *             if node2.split_dim == -1:  # 1 & 2 are leaves             # <<<<<<<<<<<<<<
 *                 lnode2 = <leafnode*>node2
 * 
 */
    __pyx_t_1 = ((__pyx_v_node2->split_dim == -1L) != 0);
    if (__pyx_t_1) {

      /* "pyart/map/ckdtree.pyx":1681
 * 
 *             if node2.split_dim == -1:  # 1 & 2 are leaves
 *                 lnode2 = <leafnode*>node2             # <<<<<<<<<<<<<<
 * 
 *                 # brute-force
 */
      __pyx_v_lnode2 = ((struct __pyx_t_5pyart_3map_7ckdtree_leafnode *)__pyx_v_node2);

      /* "pyart/map/ckdtree.pyx":1684
 * 
 *                 # brute-force
 *                 for i in range(lnode1.start_idx, lnode1.end_idx):             # <<<<<<<<<<<<<<
 * 
 *                     # Special care here to avoid duplicate pairs
 */
      __pyx_t_3 = __pyx_v_lnode1->end_idx;
      __pyx_t_4 = __pyx_t_3;
      for (__pyx_t_5 = __pyx_v_lnode1->start_idx; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
        __pyx_v_i = __pyx_t_5;

        /* "pyart/map/ckdtree.pyx":1687
 * 
 *                     # Special care here to avoid duplicate pairs
 *                     if node1 == node2:             # <<<<<<<<<<<<<<
 *                         min_j = i + 1
 *                     else:
 */
        __pyx_t_1 = ((__pyx_v_node1 == __pyx_v_node2) != 0);
        if (__pyx_t_1) {

          /* "pyart/map/ckdtree.pyx":1688
 *                     # Special care here to avoid duplicate pairs
 *                     if node1 == node2:
 *                         min_j = i + 1             # <<<<<<<<<<<<<<
 *                     else:
 *                         min_j = lnode2.start_idx
 */
          __pyx_v_min_j = (__pyx_v_i + 1);

          /* "pyart/map/ckdtree.pyx":1687
 * 
 *                     # Special care here to avoid duplicate pairs
 *                     if node1 == node2:             # <<<<<<<<<<<<<<
 *                         min_j = i + 1
 *                     else:
 */
          goto __pyx_L7;
        }

        /* "pyart/map/ckdtree.pyx":1690
 *                         min_j = i + 1
 *                     else:
 *                         min_j = lnode2.start_idx             # <<<<<<<<<<<<<<
 * 
 *                     for j in range(min_j, lnode2.end_idx):
 */
        /*else*/ {
          __pyx_t_6 = __pyx_v_lnode2->start_idx;
          __pyx_v_min_j = __pyx_t_6;
        }
        __pyx_L7:;

        /* "pyart/map/ckdtree.pyx":1692
 *                         min_j = lnode2.start_idx
 * 
 *                     for j in range(min_j, lnode2.end_idx):             # <<<<<<<<<<<<<<
 *                         d = _distance_p(
 *                             self.raw_data + self.raw_indices[i] * self.m,
 */
        __pyx_t_6 = __pyx_v_lnode2->end_idx;
        __pyx_t_7 = __pyx_t_6;
        for (__pyx_t_8 = __pyx_v_min_j; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
          __pyx_v_j = __pyx_t_8;

          /* "pyart/map/ckdtree.pyx":1693
 * 
 *                     for j in range(min_j, lnode2.end_idx):
 *                         d = _distance_p(             # <<<<<<<<<<<<<<
 *                             self.raw_data + self.raw_indices[i] * self.m,
 *                             self.raw_data + self.raw_indices[j] * self.m,
 */
          __pyx_v_d = __pyx_f_5pyart_3map_7ckdtree__distance_p((__pyx_v_self->raw_data + ((__pyx_v_self->raw_indices[__pyx_v_i]) * __pyx_v_self->m)), (__pyx_v_self->raw_data + ((__pyx_v_self->raw_indices[__pyx_v_j]) * __pyx_v_self->m)), __pyx_v_tracker->p, __pyx_v_self->m, __pyx_v_tracker->upper_bound);

          /* "pyart/map/ckdtree.pyx":1697
 *                             self.raw_data + self.raw_indices[j] * self.m,
 *                             tracker.p, self.m, tracker.upper_bound)
 *                         if d <= tracker.upper_bound:             # <<<<<<<<<<<<<<
 *                             set_add_ordered_pair(results,
 *                                                  self.raw_indices[i],
 */
          __pyx_t_1 = ((__pyx_v_d <= __pyx_v_tracker->upper_bound) != 0);
          if (__pyx_t_1) {

            /* "pyart/map/ckdtree.pyx":1698
 *                             tracker.p, self.m, tracker.upper_bound)
 *                         if d <= tracker.upper_bound:
 *                             set_add_ordered_pair(results,             # <<<<<<<<<<<<<<
 *                                                  self.raw_indices[i],
 *                                                  self.raw_indices[j])
 */
            __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_set_add_ordered_pair(__pyx_v_results, (__pyx_v_self->raw_indices[__pyx_v_i]), (__pyx_v_self->raw_indices[__pyx_v_j])); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1698, __pyx_L1_error)

            /* "pyart/map/ckdtree.pyx":1697
 *                             self.raw_data + self.raw_indices[j] * self.m,
 *                             tracker.p, self.m, tracker.upper_bound)
 *                         if d <= tracker.upper_bound:             # <<<<<<<<<<<<<<
 *                             set_add_ordered_pair(results,
 *                                                  self.raw_indices[i],
 */
          }
        }
      }

      /* "pyart/map/ckdtree.pyx":1680
 *             lnode1 = <leafnode*>node1
 * 
 *             if node2.split_dim == -1:  # 1 & 2 are leaves             # <<<<<<<<<<<<<<
 *                 lnode2 = <leafnode*>node2
 * 
 */
      goto __pyx_L4;
    }

    /* "pyart/map/ckdtree.pyx":1703
 * 
 *             else:  # 1 is a leaf node, 2 is inner node
 *                 tracker.push_less_of(2, node2)             # <<<<<<<<<<<<<<
 *                 self.__query_pairs_traverse_checking(
 *                     results, node1, node2.less, tracker)
 */
    /*else*/ {
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_less_of(__pyx_v_tracker, 2, __pyx_v_node2); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1703, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1704
 *             else:  # 1 is a leaf node, 2 is inner node
 *                 tracker.push_less_of(2, node2)
 *                 self.__query_pairs_traverse_checking(             # <<<<<<<<<<<<<<
 *                     results, node1, node2.less, tracker)
 *                 tracker.pop()
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_pairs_traverse_checking(__pyx_v_self, __pyx_v_results, __pyx_v_node1, __pyx_v_node2->less, __pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1704, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1706
 *                 self.__query_pairs_traverse_checking(
 *                     results, node1, node2.less, tracker)
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *                 tracker.push_greater_of(2, node2)
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1706, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1708
 *                 tracker.pop()
 * 
 *                 tracker.push_greater_of(2, node2)             # <<<<<<<<<<<<<<
 *                 self.__query_pairs_traverse_checking(
 *                     results, node1, node2.greater, tracker)
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_greater_of(__pyx_v_tracker, 2, __pyx_v_node2); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1708, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1709
 * 
 *                 tracker.push_greater_of(2, node2)
 *                 self.__query_pairs_traverse_checking(             # <<<<<<<<<<<<<<
 *                     results, node1, node2.greater, tracker)
 *                 tracker.pop()
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_pairs_traverse_checking(__pyx_v_self, __pyx_v_results, __pyx_v_node1, __pyx_v_node2->greater, __pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1709, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1711
 *                 self.__query_pairs_traverse_checking(
 *                     results, node1, node2.greater, tracker)
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *         else:  # 1 is an inner node
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1711, __pyx_L1_error)
    }
    __pyx_L4:;

    /* "pyart/map/ckdtree.pyx":1677
 *         elif tracker.max_distance < tracker.upper_bound / tracker.epsfac:
 *             self.__query_pairs_traverse_no_checking(results, node1, node2)
 *         elif node1.split_dim == -1:  # 1 is leaf node             # <<<<<<<<<<<<<<
 *             lnode1 = <leafnode*>node1
 * 
 */
    goto __pyx_L3;
  }

  /* "pyart/map/ckdtree.pyx":1714
 * 
 *         else:  # 1 is an inner node
 *             if node2.split_dim == -1:  # 1 is an inner node, 2 is a leaf node             # <<<<<<<<<<<<<<
 *                 tracker.push_less_of(1, node1)
 *                 self.__query_pairs_traverse_checking(
 */
  /*else*/ {
    __pyx_t_1 = ((__pyx_v_node2->split_dim == -1L) != 0);
    if (__pyx_t_1) {

      /* "pyart/map/ckdtree.pyx":1715
 *         else:  # 1 is an inner node
 *             if node2.split_dim == -1:  # 1 is an inner node, 2 is a leaf node
 *                 tracker.push_less_of(1, node1)             # <<<<<<<<<<<<<<
 *                 self.__query_pairs_traverse_checking(
 *                     results, node1.less, node2, tracker)
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_less_of(__pyx_v_tracker, 1, __pyx_v_node1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1715, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1716
 *             if node2.split_dim == -1:  # 1 is an inner node, 2 is a leaf node
 *                 tracker.push_less_of(1, node1)
 *                 self.__query_pairs_traverse_checking(             # <<<<<<<<<<<<<<
 *                     results, node1.less, node2, tracker)
 *                 tracker.pop()
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_pairs_traverse_checking(__pyx_v_self, __pyx_v_results, __pyx_v_node1->less, __pyx_v_node2, __pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1716, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1718
 *                 self.__query_pairs_traverse_checking(
 *                     results, node1.less, node2, tracker)
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *                 tracker.push_greater_of(1, node1)
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1718, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1720
 *                 tracker.pop()
 * 
 *                 tracker.push_greater_of(1, node1)             # <<<<<<<<<<<<<<
 *                 self.__query_pairs_traverse_checking(
 *                     results, node1.greater, node2, tracker)
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_greater_of(__pyx_v_tracker, 1, __pyx_v_node1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1720, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1721
 * 
 *                 tracker.push_greater_of(1, node1)
 *                 self.__query_pairs_traverse_checking(             # <<<<<<<<<<<<<<
 *                     results, node1.greater, node2, tracker)
 *                 tracker.pop()
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_pairs_traverse_checking(__pyx_v_self, __pyx_v_results, __pyx_v_node1->greater, __pyx_v_node2, __pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1721, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1723
 *                 self.__query_pairs_traverse_checking(
 *                     results, node1.greater, node2, tracker)
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *             else: # 1 and 2 are inner nodes
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1723, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1714
 * 
 *         else:  # 1 is an inner node
 *             if node2.split_dim == -1:  # 1 is an inner node, 2 is a leaf node             # <<<<<<<<<<<<<<
 *                 tracker.push_less_of(1, node1)
 *                 self.__query_pairs_traverse_checking(
 */
      goto __pyx_L11;
    }

    /* "pyart/map/ckdtree.pyx":1726
 * 
 *             else: # 1 and 2 are inner nodes
 *                 tracker.push_less_of(1, node1)             # <<<<<<<<<<<<<<
 *                 tracker.push_less_of(2, node2)
 *                 self.__query_pairs_traverse_checking(
 */
    /*else*/ {
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_less_of(__pyx_v_tracker, 1, __pyx_v_node1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1726, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1727
 *             else: # 1 and 2 are inner nodes
 *                 tracker.push_less_of(1, node1)
 *                 tracker.push_less_of(2, node2)             # <<<<<<<<<<<<<<
 *                 self.__query_pairs_traverse_checking(
 *                     results, node1.less, node2.less, tracker)
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_less_of(__pyx_v_tracker, 2, __pyx_v_node2); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1727, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1728
 *                 tracker.push_less_of(1, node1)
 *                 tracker.push_less_of(2, node2)
 *                 self.__query_pairs_traverse_checking(             # <<<<<<<<<<<<<<
 *                     results, node1.less, node2.less, tracker)
 *                 tracker.pop()
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_pairs_traverse_checking(__pyx_v_self, __pyx_v_results, __pyx_v_node1->less, __pyx_v_node2->less, __pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1728, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1730
 *                 self.__query_pairs_traverse_checking(
 *                     results, node1.less, node2.less, tracker)
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *                 tracker.push_greater_of(2, node2)
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1730, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1732
 *                 tracker.pop()
 * 
 *                 tracker.push_greater_of(2, node2)             # <<<<<<<<<<<<<<
 *                 self.__query_pairs_traverse_checking(
 *                     results, node1.less, node2.greater, tracker)
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_greater_of(__pyx_v_tracker, 2, __pyx_v_node2); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1732, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1733
 * 
 *                 tracker.push_greater_of(2, node2)
 *                 self.__query_pairs_traverse_checking(             # <<<<<<<<<<<<<<
 *                     results, node1.less, node2.greater, tracker)
 *                 tracker.pop()
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_pairs_traverse_checking(__pyx_v_self, __pyx_v_results, __pyx_v_node1->less, __pyx_v_node2->greater, __pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1733, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1735
 *                 self.__query_pairs_traverse_checking(
 *                     results, node1.less, node2.greater, tracker)
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 *                 tracker.pop()
 * 
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1735, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1736
 *                     results, node1.less, node2.greater, tracker)
 *                 tracker.pop()
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *                 tracker.push_greater_of(1, node1)
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1736, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1738
 *                 tracker.pop()
 * 
 *                 tracker.push_greater_of(1, node1)             # <<<<<<<<<<<<<<
 *                 if node1 != node2:
 *                     # Avoid traversing (node1.less, node2.greater) and
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_greater_of(__pyx_v_tracker, 1, __pyx_v_node1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1738, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1739
 * 
 *                 tracker.push_greater_of(1, node1)
 *                 if node1 != node2:             # <<<<<<<<<<<<<<
 *                     # Avoid traversing (node1.less, node2.greater) and
 *                     # (node1.greater, node2.less) (it's the same node pair
 */
      __pyx_t_1 = ((__pyx_v_node1 != __pyx_v_node2) != 0);
      if (__pyx_t_1) {

        /* "pyart/map/ckdtree.pyx":1744
 *                     # twice over, which is the source of the complication in
 *                     # the original KDTree.query_pairs)
 *                     tracker.push_less_of(2, node2)             # <<<<<<<<<<<<<<
 *                     self.__query_pairs_traverse_checking(
 *                         results, node1.greater, node2.less, tracker)
 */
        __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_less_of(__pyx_v_tracker, 2, __pyx_v_node2); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1744, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1745
 *                     # the original KDTree.query_pairs)
 *                     tracker.push_less_of(2, node2)
 *                     self.__query_pairs_traverse_checking(             # <<<<<<<<<<<<<<
 *                         results, node1.greater, node2.less, tracker)
 *                     tracker.pop()
 */
        __pyx_t_2 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_pairs_traverse_checking(__pyx_v_self, __pyx_v_results, __pyx_v_node1->greater, __pyx_v_node2->less, __pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1745, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1747
 *                     self.__query_pairs_traverse_checking(
 *                         results, node1.greater, node2.less, tracker)
 *                     tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *                 tracker.push_greater_of(2, node2)
 */
        __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1747, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1739
 * 
 *                 tracker.push_greater_of(1, node1)
 *                 if node1 != node2:             # <<<<<<<<<<<<<<
 *                     # Avoid traversing (node1.less, node2.greater) and
 *                     # (node1.greater, node2.less) (it's the same node pair
 */
      }

      /* "pyart/map/ckdtree.pyx":1749
 *                     tracker.pop()
 * 
 *                 tracker.push_greater_of(2, node2)             # <<<<<<<<<<<<<<
 *                 self.__query_pairs_traverse_checking(
 *                     results, node1.greater, node2.greater, tracker)
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_greater_of(__pyx_v_tracker, 2, __pyx_v_node2); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1749, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1750
 * 
 *                 tracker.push_greater_of(2, node2)
 *                 self.__query_pairs_traverse_checking(             # <<<<<<<<<<<<<<
 *                     results, node1.greater, node2.greater, tracker)
 *                 tracker.pop()
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_pairs_traverse_checking(__pyx_v_self, __pyx_v_results, __pyx_v_node1->greater, __pyx_v_node2->greater, __pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1750, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1752
 *                 self.__query_pairs_traverse_checking(
 *                     results, node1.greater, node2.greater, tracker)
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 *                 tracker.pop()
 * 
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1752, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":1753
 *                     results, node1.greater, node2.greater, tracker)
 *                 tracker.pop()
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *         return 0
 */
      __pyx_t_2 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1753, __pyx_L1_error)
    }
    __pyx_L11:;
  }
  __pyx_L3:;

  /* "pyart/map/ckdtree.pyx":1755
 *                 tracker.pop()
 * 
 *         return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":1662
 * 
 *     @cython.cdivision(True)
 *     cdef int __query_pairs_traverse_checking(cKDTree self,             # <<<<<<<<<<<<<<
 *                                              set results,
 *                                              innernode* node1,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.__query_pairs_traverse_checking", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":1758
 * 
 * 
 *     def query_pairs(cKDTree self, np.float64_t r, np.float64_t p=2.,             # <<<<<<<<<<<<<<
 *                     np.float64_t eps=0):
 *         """query_pairs(self, r, p, eps)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_11query_pairs(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5pyart_3map_7ckdtree_7cKDTree_10query_pairs[] = "query_pairs(self, r, p, eps)\n\n        Find all pairs of points whose distance is at most r.\n\n        Parameters\n        ----------\n        r : positive float\n            The maximum distance.\n        p : float, optional\n            Which Minkowski norm to use.  `p` has to meet the condition\n            ``1 <= p <= infinity``.\n        eps : float, optional\n            Approximate search.  Branches of the tree are not explored\n            if their nearest points are further than ``r/(1+eps)``, and\n            branches are added in bulk if their furthest points are nearer\n            than ``r * (1+eps)``.  `eps` has to be non-negative.\n\n        Returns\n        -------\n        results : set\n            Set of pairs ``(i,j)``, with ``i < j`, for which the corresponding\n            positions are close.\n\n        ";
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_11query_pairs(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __pyx_t_5numpy_float64_t __pyx_v_r;
  __pyx_t_5numpy_float64_t __pyx_v_p;
  __pyx_t_5numpy_float64_t __pyx_v_eps;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("query_pairs (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_r,&__pyx_n_s_p,&__pyx_n_s_eps,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_r)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_p);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_eps);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "query_pairs") < 0)) __PYX_ERR(0, 1758, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_r = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_r == ((npy_float64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1758, __pyx_L3_error)
    if (values[1]) {
      __pyx_v_p = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_p == ((npy_float64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1758, __pyx_L3_error)
    } else {
      __pyx_v_p = ((__pyx_t_5numpy_float64_t)2.);
    }
    if (values[2]) {
      __pyx_v_eps = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_eps == ((npy_float64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1759, __pyx_L3_error)
    } else {
      __pyx_v_eps = ((__pyx_t_5numpy_float64_t)0.0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("query_pairs", 0, 1, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1758, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.query_pairs", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_7cKDTree_10query_pairs(((struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self), __pyx_v_r, __pyx_v_p, __pyx_v_eps);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_10query_pairs(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, __pyx_t_5numpy_float64_t __pyx_v_r, __pyx_t_5numpy_float64_t __pyx_v_p, __pyx_t_5numpy_float64_t __pyx_v_eps) {
  struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_tracker = NULL;
  PyObject *__pyx_v_results = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("query_pairs", 0);

  /* "pyart/map/ckdtree.pyx":1786
 * 
 *         tracker = RectRectDistanceTracker(
 *             Rectangle(self.mins, self.maxes),             # <<<<<<<<<<<<<<
 *             Rectangle(self.mins, self.maxes),
 *             p, eps, r)
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1786, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->mins));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->mins));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_self->mins));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->maxes));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->maxes));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self->maxes));
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pyart_3map_7ckdtree_Rectangle), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1786, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/map/ckdtree.pyx":1787
 *         tracker = RectRectDistanceTracker(
 *             Rectangle(self.mins, self.maxes),
 *             Rectangle(self.mins, self.maxes),             # <<<<<<<<<<<<<<
 *             p, eps, r)
 * 
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1787, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->mins));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->mins));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_self->mins));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->maxes));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->maxes));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self->maxes));
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pyart_3map_7ckdtree_Rectangle), __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1787, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/map/ckdtree.pyx":1788
 *             Rectangle(self.mins, self.maxes),
 *             Rectangle(self.mins, self.maxes),
 *             p, eps, r)             # <<<<<<<<<<<<<<
 * 
 *         results = set()
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_p); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1788, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_eps); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1788, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyFloat_FromDouble(__pyx_v_r); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1788, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);

  /* "pyart/map/ckdtree.pyx":1785
 *         """
 * 
 *         tracker = RectRectDistanceTracker(             # <<<<<<<<<<<<<<
 *             Rectangle(self.mins, self.maxes),
 *             Rectangle(self.mins, self.maxes),
 */
  __pyx_t_6 = PyTuple_New(5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1785, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_t_5);
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_1 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pyart_3map_7ckdtree_RectRectDistanceTracker), __pyx_t_6, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1785, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_v_tracker = ((struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "pyart/map/ckdtree.pyx":1790
 *             p, eps, r)
 * 
 *         results = set()             # <<<<<<<<<<<<<<
 *         self.__query_pairs_traverse_checking(
 *             results, self.tree, self.tree, tracker)
 */
  __pyx_t_5 = PySet_New(0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1790, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_v_results = ((PyObject*)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "pyart/map/ckdtree.pyx":1791
 * 
 *         results = set()
 *         self.__query_pairs_traverse_checking(             # <<<<<<<<<<<<<<
 *             results, self.tree, self.tree, tracker)
 * 
 */
  __pyx_t_7 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___query_pairs_traverse_checking(__pyx_v_self, __pyx_v_results, __pyx_v_self->tree, __pyx_v_self->tree, __pyx_v_tracker); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1791, __pyx_L1_error)

  /* "pyart/map/ckdtree.pyx":1794
 *             results, self.tree, self.tree, tracker)
 * 
 *         return results             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_results);
  __pyx_r = __pyx_v_results;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":1758
 * 
 * 
 *     def query_pairs(cKDTree self, np.float64_t r, np.float64_t p=2.,             # <<<<<<<<<<<<<<
 *                     np.float64_t eps=0):
 *         """query_pairs(self, r, p, eps)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.query_pairs", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_tracker);
  __Pyx_XDECREF(__pyx_v_results);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":1800
 *     # count_neighbors
 *     # ---------------
 *     cdef int __count_neighbors_traverse(cKDTree self,             # <<<<<<<<<<<<<<
 *                                         cKDTree other,
 *                                         np.intp_t n_queries,
 */

static int __pyx_f_5pyart_3map_7ckdtree_7cKDTree___count_neighbors_traverse(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_other, __pyx_t_5numpy_intp_t __pyx_v_n_queries, __pyx_t_5numpy_float64_t *__pyx_v_r, __pyx_t_5numpy_intp_t *__pyx_v_results, __pyx_t_5numpy_intp_t *__pyx_v_idx, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node1, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node2, struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_tracker) {
  struct __pyx_t_5pyart_3map_7ckdtree_leafnode *__pyx_v_lnode1;
  struct __pyx_t_5pyart_3map_7ckdtree_leafnode *__pyx_v_lnode2;
  __pyx_t_5numpy_float64_t __pyx_v_d;
  __pyx_t_5numpy_intp_t *__pyx_v_old_idx;
  __pyx_t_5numpy_intp_t __pyx_v_old_n_queries;
  __pyx_t_5numpy_intp_t __pyx_v_l;
  __pyx_t_5numpy_intp_t __pyx_v_i;
  __pyx_t_5numpy_intp_t __pyx_v_j;
  PyArrayObject *__pyx_v_inner_idx = 0;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_inner_idx;
  __Pyx_Buffer __pyx_pybuffer_inner_idx;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyArrayObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  Py_ssize_t __pyx_t_11;
  __pyx_t_5numpy_intp_t __pyx_t_12;
  __pyx_t_5numpy_intp_t __pyx_t_13;
  __pyx_t_5numpy_intp_t __pyx_t_14;
  int __pyx_t_15;
  __pyx_t_5numpy_intp_t __pyx_t_16;
  __pyx_t_5numpy_intp_t __pyx_t_17;
  __pyx_t_5numpy_intp_t __pyx_t_18;
  __pyx_t_5numpy_intp_t __pyx_t_19;
  __pyx_t_5numpy_intp_t __pyx_t_20;
  __pyx_t_5numpy_intp_t __pyx_t_21;
  __pyx_t_5numpy_intp_t __pyx_t_22;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__count_neighbors_traverse", 0);
  __pyx_pybuffer_inner_idx.pybuffer.buf = NULL;
  __pyx_pybuffer_inner_idx.refcount = 0;
  __pyx_pybuffernd_inner_idx.data = NULL;
  __pyx_pybuffernd_inner_idx.rcbuffer = &__pyx_pybuffer_inner_idx;

  /* "pyart/map/ckdtree.pyx":1817
 *         # Speed through pairs of nodes all of whose children are close
 *         # and see if any work remains to be done
 *         old_idx = idx             # <<<<<<<<<<<<<<
 *         cdef np.ndarray[np.intp_t, ndim=1] inner_idx
 *         inner_idx = np.empty((n_queries,), dtype=np.intp)
 */
  __pyx_v_old_idx = __pyx_v_idx;

  /* "pyart/map/ckdtree.pyx":1819
 *         old_idx = idx
 *         cdef np.ndarray[np.intp_t, ndim=1] inner_idx
 *         inner_idx = np.empty((n_queries,), dtype=np.intp)             # <<<<<<<<<<<<<<
 *         idx = &inner_idx[0]
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1819, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_empty); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1819, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n_queries); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1819, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1819, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1819, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1819, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1819, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_intp); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1819, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_5) < 0) __PYX_ERR(0, 1819, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1819, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (!(likely(((__pyx_t_5) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_5, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1819, __pyx_L1_error)
  __pyx_t_6 = ((PyArrayObject *)__pyx_t_5);
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_inner_idx.rcbuffer->pybuffer);
    __pyx_t_7 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_inner_idx.rcbuffer->pybuffer, (PyObject*)__pyx_t_6, &__Pyx_TypeInfo_nn___pyx_t_5numpy_intp_t, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack);
    if (unlikely(__pyx_t_7 < 0)) {
      PyErr_Fetch(&__pyx_t_8, &__pyx_t_9, &__pyx_t_10);
      if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_inner_idx.rcbuffer->pybuffer, (PyObject*)__pyx_v_inner_idx, &__Pyx_TypeInfo_nn___pyx_t_5numpy_intp_t, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack) == -1)) {
        Py_XDECREF(__pyx_t_8); Py_XDECREF(__pyx_t_9); Py_XDECREF(__pyx_t_10);
        __Pyx_RaiseBufferFallbackError();
      } else {
        PyErr_Restore(__pyx_t_8, __pyx_t_9, __pyx_t_10);
      }
      __pyx_t_8 = __pyx_t_9 = __pyx_t_10 = 0;
    }
    __pyx_pybuffernd_inner_idx.diminfo[0].strides = __pyx_pybuffernd_inner_idx.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_inner_idx.diminfo[0].shape = __pyx_pybuffernd_inner_idx.rcbuffer->pybuffer.shape[0];
    if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 1819, __pyx_L1_error)
  }
  __pyx_t_6 = 0;
  __pyx_v_inner_idx = ((PyArrayObject *)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "pyart/map/ckdtree.pyx":1820
 *         cdef np.ndarray[np.intp_t, ndim=1] inner_idx
 *         inner_idx = np.empty((n_queries,), dtype=np.intp)
 *         idx = &inner_idx[0]             # <<<<<<<<<<<<<<
 * 
 *         old_n_queries = n_queries
 */
  __pyx_t_11 = 0;
  __pyx_t_7 = -1;
  if (__pyx_t_11 < 0) {
    __pyx_t_11 += __pyx_pybuffernd_inner_idx.diminfo[0].shape;
    if (unlikely(__pyx_t_11 < 0)) __pyx_t_7 = 0;
  } else if (unlikely(__pyx_t_11 >= __pyx_pybuffernd_inner_idx.diminfo[0].shape)) __pyx_t_7 = 0;
  if (unlikely(__pyx_t_7 != -1)) {
    __Pyx_RaiseBufferIndexError(__pyx_t_7);
    __PYX_ERR(0, 1820, __pyx_L1_error)
  }
  __pyx_v_idx = (&(*__Pyx_BufPtrStrided1d(__pyx_t_5numpy_intp_t *, __pyx_pybuffernd_inner_idx.rcbuffer->pybuffer.buf, __pyx_t_11, __pyx_pybuffernd_inner_idx.diminfo[0].strides)));

  /* "pyart/map/ckdtree.pyx":1822
 *         idx = &inner_idx[0]
 * 
 *         old_n_queries = n_queries             # <<<<<<<<<<<<<<
 *         n_queries = 0
 *         for i in range(old_n_queries):
 */
  __pyx_v_old_n_queries = __pyx_v_n_queries;

  /* "pyart/map/ckdtree.pyx":1823
 * 
 *         old_n_queries = n_queries
 *         n_queries = 0             # <<<<<<<<<<<<<<
 *         for i in range(old_n_queries):
 *             if tracker.max_distance < r[old_idx[i]]:
 */
  __pyx_v_n_queries = 0;

  /* "pyart/map/ckdtree.pyx":1824
 *         old_n_queries = n_queries
 *         n_queries = 0
 *         for i in range(old_n_queries):             # <<<<<<<<<<<<<<
 *             if tracker.max_distance < r[old_idx[i]]:
 *                 results[old_idx[i]] += node1.children * node2.children
 */
  __pyx_t_12 = __pyx_v_old_n_queries;
  __pyx_t_13 = __pyx_t_12;
  for (__pyx_t_14 = 0; __pyx_t_14 < __pyx_t_13; __pyx_t_14+=1) {
    __pyx_v_i = __pyx_t_14;

    /* "pyart/map/ckdtree.pyx":1825
 *         n_queries = 0
 *         for i in range(old_n_queries):
 *             if tracker.max_distance < r[old_idx[i]]:             # <<<<<<<<<<<<<<
 *                 results[old_idx[i]] += node1.children * node2.children
 *             elif tracker.min_distance <= r[old_idx[i]]:
 */
    __pyx_t_15 = ((__pyx_v_tracker->max_distance < (__pyx_v_r[(__pyx_v_old_idx[__pyx_v_i])])) != 0);
    if (__pyx_t_15) {

      /* "pyart/map/ckdtree.pyx":1826
 *         for i in range(old_n_queries):
 *             if tracker.max_distance < r[old_idx[i]]:
 *                 results[old_idx[i]] += node1.children * node2.children             # <<<<<<<<<<<<<<
 *             elif tracker.min_distance <= r[old_idx[i]]:
 *                 idx[n_queries] = old_idx[i]
 */
      __pyx_t_16 = (__pyx_v_old_idx[__pyx_v_i]);
      (__pyx_v_results[__pyx_t_16]) = ((__pyx_v_results[__pyx_t_16]) + (__pyx_v_node1->children * __pyx_v_node2->children));

      /* "pyart/map/ckdtree.pyx":1825
 *         n_queries = 0
 *         for i in range(old_n_queries):
 *             if tracker.max_distance < r[old_idx[i]]:             # <<<<<<<<<<<<<<
 *                 results[old_idx[i]] += node1.children * node2.children
 *             elif tracker.min_distance <= r[old_idx[i]]:
 */
      goto __pyx_L5;
    }

    /* "pyart/map/ckdtree.pyx":1827
 *             if tracker.max_distance < r[old_idx[i]]:
 *                 results[old_idx[i]] += node1.children * node2.children
 *             elif tracker.min_distance <= r[old_idx[i]]:             # <<<<<<<<<<<<<<
 *                 idx[n_queries] = old_idx[i]
 *                 n_queries += 1
 */
    __pyx_t_15 = ((__pyx_v_tracker->min_distance <= (__pyx_v_r[(__pyx_v_old_idx[__pyx_v_i])])) != 0);
    if (__pyx_t_15) {

      /* "pyart/map/ckdtree.pyx":1828
 *                 results[old_idx[i]] += node1.children * node2.children
 *             elif tracker.min_distance <= r[old_idx[i]]:
 *                 idx[n_queries] = old_idx[i]             # <<<<<<<<<<<<<<
 *                 n_queries += 1
 * 
 */
      (__pyx_v_idx[__pyx_v_n_queries]) = (__pyx_v_old_idx[__pyx_v_i]);

      /* "pyart/map/ckdtree.pyx":1829
 *             elif tracker.min_distance <= r[old_idx[i]]:
 *                 idx[n_queries] = old_idx[i]
 *                 n_queries += 1             # <<<<<<<<<<<<<<
 * 
 *         if n_queries > 0:
 */
      __pyx_v_n_queries = (__pyx_v_n_queries + 1);

      /* "pyart/map/ckdtree.pyx":1827
 *             if tracker.max_distance < r[old_idx[i]]:
 *                 results[old_idx[i]] += node1.children * node2.children
 *             elif tracker.min_distance <= r[old_idx[i]]:             # <<<<<<<<<<<<<<
 *                 idx[n_queries] = old_idx[i]
 *                 n_queries += 1
 */
    }
    __pyx_L5:;
  }

  /* "pyart/map/ckdtree.pyx":1831
 *                 n_queries += 1
 * 
 *         if n_queries > 0:             # <<<<<<<<<<<<<<
 *             # OK, need to probe a bit deeper
 *             if node1.split_dim == -1:  # 1 is leaf node
 */
  __pyx_t_15 = ((__pyx_v_n_queries > 0) != 0);
  if (__pyx_t_15) {

    /* "pyart/map/ckdtree.pyx":1833
 *         if n_queries > 0:
 *             # OK, need to probe a bit deeper
 *             if node1.split_dim == -1:  # 1 is leaf node             # <<<<<<<<<<<<<<
 *                 lnode1 = <leafnode*>node1
 *                 if node2.split_dim == -1:  # 1 & 2 are leaves
 */
    __pyx_t_15 = ((__pyx_v_node1->split_dim == -1L) != 0);
    if (__pyx_t_15) {

      /* "pyart/map/ckdtree.pyx":1834
 *             # OK, need to probe a bit deeper
 *             if node1.split_dim == -1:  # 1 is leaf node
 *                 lnode1 = <leafnode*>node1             # <<<<<<<<<<<<<<
 *                 if node2.split_dim == -1:  # 1 & 2 are leaves
 *                     lnode2 = <leafnode*>node2
 */
      __pyx_v_lnode1 = ((struct __pyx_t_5pyart_3map_7ckdtree_leafnode *)__pyx_v_node1);

      /* "pyart/map/ckdtree.pyx":1835
 *             if node1.split_dim == -1:  # 1 is leaf node
 *                 lnode1 = <leafnode*>node1
 *                 if node2.split_dim == -1:  # 1 & 2 are leaves             # <<<<<<<<<<<<<<
 *                     lnode2 = <leafnode*>node2
 * 
 */
      __pyx_t_15 = ((__pyx_v_node2->split_dim == -1L) != 0);
      if (__pyx_t_15) {

        /* "pyart/map/ckdtree.pyx":1836
 *                 lnode1 = <leafnode*>node1
 *                 if node2.split_dim == -1:  # 1 & 2 are leaves
 *                     lnode2 = <leafnode*>node2             # <<<<<<<<<<<<<<
 * 
 *                     # brute-force
 */
        __pyx_v_lnode2 = ((struct __pyx_t_5pyart_3map_7ckdtree_leafnode *)__pyx_v_node2);

        /* "pyart/map/ckdtree.pyx":1839
 * 
 *                     # brute-force
 *                     for i in range(lnode1.start_idx, lnode1.end_idx):             # <<<<<<<<<<<<<<
 *                         for j in range(lnode2.start_idx, lnode2.end_idx):
 *                             d = _distance_p(
 */
        __pyx_t_12 = __pyx_v_lnode1->end_idx;
        __pyx_t_13 = __pyx_t_12;
        for (__pyx_t_14 = __pyx_v_lnode1->start_idx; __pyx_t_14 < __pyx_t_13; __pyx_t_14+=1) {
          __pyx_v_i = __pyx_t_14;

          /* "pyart/map/ckdtree.pyx":1840
 *                     # brute-force
 *                     for i in range(lnode1.start_idx, lnode1.end_idx):
 *                         for j in range(lnode2.start_idx, lnode2.end_idx):             # <<<<<<<<<<<<<<
 *                             d = _distance_p(
 *                                 self.raw_data + self.raw_indices[i] * self.m,
 */
          __pyx_t_16 = __pyx_v_lnode2->end_idx;
          __pyx_t_17 = __pyx_t_16;
          for (__pyx_t_18 = __pyx_v_lnode2->start_idx; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_j = __pyx_t_18;

            /* "pyart/map/ckdtree.pyx":1841
 *                     for i in range(lnode1.start_idx, lnode1.end_idx):
 *                         for j in range(lnode2.start_idx, lnode2.end_idx):
 *                             d = _distance_p(             # <<<<<<<<<<<<<<
 *                                 self.raw_data + self.raw_indices[i] * self.m,
 *                                 other.raw_data + other.raw_indices[j] * other.m,
 */
            __pyx_v_d = __pyx_f_5pyart_3map_7ckdtree__distance_p((__pyx_v_self->raw_data + ((__pyx_v_self->raw_indices[__pyx_v_i]) * __pyx_v_self->m)), (__pyx_v_other->raw_data + ((__pyx_v_other->raw_indices[__pyx_v_j]) * __pyx_v_other->m)), __pyx_v_tracker->p, __pyx_v_self->m, __pyx_v_tracker->max_distance);

            /* "pyart/map/ckdtree.pyx":1848
 *                             # than to generate a distance array, sort it, then
 *                             # search for all r's via binary search
 *                             for l in range(n_queries):             # <<<<<<<<<<<<<<
 *                                 if d <= r[idx[l]]:
 *                                     results[idx[l]] += 1
 */
            __pyx_t_19 = __pyx_v_n_queries;
            __pyx_t_20 = __pyx_t_19;
            for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
              __pyx_v_l = __pyx_t_21;

              /* "pyart/map/ckdtree.pyx":1849
 *                             # search for all r's via binary search
 *                             for l in range(n_queries):
 *                                 if d <= r[idx[l]]:             # <<<<<<<<<<<<<<
 *                                     results[idx[l]] += 1
 * 
 */
              __pyx_t_15 = ((__pyx_v_d <= (__pyx_v_r[(__pyx_v_idx[__pyx_v_l])])) != 0);
              if (__pyx_t_15) {

                /* "pyart/map/ckdtree.pyx":1850
 *                             for l in range(n_queries):
 *                                 if d <= r[idx[l]]:
 *                                     results[idx[l]] += 1             # <<<<<<<<<<<<<<
 * 
 *                 else:  # 1 is a leaf node, 2 is inner node
 */
                __pyx_t_22 = (__pyx_v_idx[__pyx_v_l]);
                (__pyx_v_results[__pyx_t_22]) = ((__pyx_v_results[__pyx_t_22]) + 1);

                /* "pyart/map/ckdtree.pyx":1849
 *                             # search for all r's via binary search
 *                             for l in range(n_queries):
 *                                 if d <= r[idx[l]]:             # <<<<<<<<<<<<<<
 *                                     results[idx[l]] += 1
 * 
 */
              }
            }
          }
        }

        /* "pyart/map/ckdtree.pyx":1835
 *             if node1.split_dim == -1:  # 1 is leaf node
 *                 lnode1 = <leafnode*>node1
 *                 if node2.split_dim == -1:  # 1 & 2 are leaves             # <<<<<<<<<<<<<<
 *                     lnode2 = <leafnode*>node2
 * 
 */
        goto __pyx_L8;
      }

      /* "pyart/map/ckdtree.pyx":1853
 * 
 *                 else:  # 1 is a leaf node, 2 is inner node
 *                     tracker.push_less_of(2, node2)             # <<<<<<<<<<<<<<
 *                     self.__count_neighbors_traverse(
 *                         other, n_queries, r, results, idx,
 */
      /*else*/ {
        __pyx_t_7 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_less_of(__pyx_v_tracker, 2, __pyx_v_node2); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1853, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1854
 *                 else:  # 1 is a leaf node, 2 is inner node
 *                     tracker.push_less_of(2, node2)
 *                     self.__count_neighbors_traverse(             # <<<<<<<<<<<<<<
 *                         other, n_queries, r, results, idx,
 *                         node1, node2.less, tracker)
 */
        __pyx_t_7 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___count_neighbors_traverse(__pyx_v_self, __pyx_v_other, __pyx_v_n_queries, __pyx_v_r, __pyx_v_results, __pyx_v_idx, __pyx_v_node1, __pyx_v_node2->less, __pyx_v_tracker); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1854, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1857
 *                         other, n_queries, r, results, idx,
 *                         node1, node2.less, tracker)
 *                     tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *                     tracker.push_greater_of(2, node2)
 */
        __pyx_t_7 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1857, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1859
 *                     tracker.pop()
 * 
 *                     tracker.push_greater_of(2, node2)             # <<<<<<<<<<<<<<
 *                     self.__count_neighbors_traverse(
 *                         other, n_queries, r, results, idx,
 */
        __pyx_t_7 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_greater_of(__pyx_v_tracker, 2, __pyx_v_node2); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1859, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1860
 * 
 *                     tracker.push_greater_of(2, node2)
 *                     self.__count_neighbors_traverse(             # <<<<<<<<<<<<<<
 *                         other, n_queries, r, results, idx,
 *                         node1, node2.greater, tracker)
 */
        __pyx_t_7 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___count_neighbors_traverse(__pyx_v_self, __pyx_v_other, __pyx_v_n_queries, __pyx_v_r, __pyx_v_results, __pyx_v_idx, __pyx_v_node1, __pyx_v_node2->greater, __pyx_v_tracker); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1860, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1863
 *                         other, n_queries, r, results, idx,
 *                         node1, node2.greater, tracker)
 *                     tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *             else:  # 1 is an inner node
 */
        __pyx_t_7 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1863, __pyx_L1_error)
      }
      __pyx_L8:;

      /* "pyart/map/ckdtree.pyx":1833
 *         if n_queries > 0:
 *             # OK, need to probe a bit deeper
 *             if node1.split_dim == -1:  # 1 is leaf node             # <<<<<<<<<<<<<<
 *                 lnode1 = <leafnode*>node1
 *                 if node2.split_dim == -1:  # 1 & 2 are leaves
 */
      goto __pyx_L7;
    }

    /* "pyart/map/ckdtree.pyx":1866
 * 
 *             else:  # 1 is an inner node
 *                 if node2.split_dim == -1:  # 1 is an inner node, 2 is a leaf node             # <<<<<<<<<<<<<<
 *                     tracker.push_less_of(1, node1)
 *                     self.__count_neighbors_traverse(
 */
    /*else*/ {
      __pyx_t_15 = ((__pyx_v_node2->split_dim == -1L) != 0);
      if (__pyx_t_15) {

        /* "pyart/map/ckdtree.pyx":1867
 *             else:  # 1 is an inner node
 *                 if node2.split_dim == -1:  # 1 is an inner node, 2 is a leaf node
 *                     tracker.push_less_of(1, node1)             # <<<<<<<<<<<<<<
 *                     self.__count_neighbors_traverse(
 *                         other, n_queries, r, results, idx,
 */
        __pyx_t_7 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_less_of(__pyx_v_tracker, 1, __pyx_v_node1); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1867, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1868
 *                 if node2.split_dim == -1:  # 1 is an inner node, 2 is a leaf node
 *                     tracker.push_less_of(1, node1)
 *                     self.__count_neighbors_traverse(             # <<<<<<<<<<<<<<
 *                         other, n_queries, r, results, idx,
 *                         node1.less, node2, tracker)
 */
        __pyx_t_7 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___count_neighbors_traverse(__pyx_v_self, __pyx_v_other, __pyx_v_n_queries, __pyx_v_r, __pyx_v_results, __pyx_v_idx, __pyx_v_node1->less, __pyx_v_node2, __pyx_v_tracker); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1868, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1871
 *                         other, n_queries, r, results, idx,
 *                         node1.less, node2, tracker)
 *                     tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *                     tracker.push_greater_of(1, node1)
 */
        __pyx_t_7 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1871, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1873
 *                     tracker.pop()
 * 
 *                     tracker.push_greater_of(1, node1)             # <<<<<<<<<<<<<<
 *                     self.__count_neighbors_traverse(
 *                         other, n_queries, r, results, idx,
 */
        __pyx_t_7 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_greater_of(__pyx_v_tracker, 1, __pyx_v_node1); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1873, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1874
 * 
 *                     tracker.push_greater_of(1, node1)
 *                     self.__count_neighbors_traverse(             # <<<<<<<<<<<<<<
 *                         other, n_queries, r, results, idx,
 *                         node1.greater, node2, tracker)
 */
        __pyx_t_7 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___count_neighbors_traverse(__pyx_v_self, __pyx_v_other, __pyx_v_n_queries, __pyx_v_r, __pyx_v_results, __pyx_v_idx, __pyx_v_node1->greater, __pyx_v_node2, __pyx_v_tracker); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1874, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1877
 *                         other, n_queries, r, results, idx,
 *                         node1.greater, node2, tracker)
 *                     tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *                 else: # 1 and 2 are inner nodes
 */
        __pyx_t_7 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1877, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1866
 * 
 *             else:  # 1 is an inner node
 *                 if node2.split_dim == -1:  # 1 is an inner node, 2 is a leaf node             # <<<<<<<<<<<<<<
 *                     tracker.push_less_of(1, node1)
 *                     self.__count_neighbors_traverse(
 */
        goto __pyx_L16;
      }

      /* "pyart/map/ckdtree.pyx":1880
 * 
 *                 else: # 1 and 2 are inner nodes
 *                     tracker.push_less_of(1, node1)             # <<<<<<<<<<<<<<
 *                     tracker.push_less_of(2, node2)
 *                     self.__count_neighbors_traverse(
 */
      /*else*/ {
        __pyx_t_7 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_less_of(__pyx_v_tracker, 1, __pyx_v_node1); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1880, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1881
 *                 else: # 1 and 2 are inner nodes
 *                     tracker.push_less_of(1, node1)
 *                     tracker.push_less_of(2, node2)             # <<<<<<<<<<<<<<
 *                     self.__count_neighbors_traverse(
 *                         other, n_queries, r, results, idx,
 */
        __pyx_t_7 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_less_of(__pyx_v_tracker, 2, __pyx_v_node2); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1881, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1882
 *                     tracker.push_less_of(1, node1)
 *                     tracker.push_less_of(2, node2)
 *                     self.__count_neighbors_traverse(             # <<<<<<<<<<<<<<
 *                         other, n_queries, r, results, idx,
 *                         node1.less, node2.less, tracker)
 */
        __pyx_t_7 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___count_neighbors_traverse(__pyx_v_self, __pyx_v_other, __pyx_v_n_queries, __pyx_v_r, __pyx_v_results, __pyx_v_idx, __pyx_v_node1->less, __pyx_v_node2->less, __pyx_v_tracker); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1882, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1885
 *                         other, n_queries, r, results, idx,
 *                         node1.less, node2.less, tracker)
 *                     tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *                     tracker.push_greater_of(2, node2)
 */
        __pyx_t_7 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1885, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1887
 *                     tracker.pop()
 * 
 *                     tracker.push_greater_of(2, node2)             # <<<<<<<<<<<<<<
 *                     self.__count_neighbors_traverse(
 *                         other, n_queries, r, results, idx,
 */
        __pyx_t_7 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_greater_of(__pyx_v_tracker, 2, __pyx_v_node2); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1887, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1888
 * 
 *                     tracker.push_greater_of(2, node2)
 *                     self.__count_neighbors_traverse(             # <<<<<<<<<<<<<<
 *                         other, n_queries, r, results, idx,
 *                         node1.less, node2.greater, tracker)
 */
        __pyx_t_7 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___count_neighbors_traverse(__pyx_v_self, __pyx_v_other, __pyx_v_n_queries, __pyx_v_r, __pyx_v_results, __pyx_v_idx, __pyx_v_node1->less, __pyx_v_node2->greater, __pyx_v_tracker); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1888, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1891
 *                         other, n_queries, r, results, idx,
 *                         node1.less, node2.greater, tracker)
 *                     tracker.pop()             # <<<<<<<<<<<<<<
 *                     tracker.pop()
 * 
 */
        __pyx_t_7 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1891, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1892
 *                         node1.less, node2.greater, tracker)
 *                     tracker.pop()
 *                     tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *                     tracker.push_greater_of(1, node1)
 */
        __pyx_t_7 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1892, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1894
 *                     tracker.pop()
 * 
 *                     tracker.push_greater_of(1, node1)             # <<<<<<<<<<<<<<
 *                     tracker.push_less_of(2, node2)
 *                     self.__count_neighbors_traverse(
 */
        __pyx_t_7 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_greater_of(__pyx_v_tracker, 1, __pyx_v_node1); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1894, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1895
 * 
 *                     tracker.push_greater_of(1, node1)
 *                     tracker.push_less_of(2, node2)             # <<<<<<<<<<<<<<
 *                     self.__count_neighbors_traverse(
 *                         other, n_queries, r, results, idx,
 */
        __pyx_t_7 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_less_of(__pyx_v_tracker, 2, __pyx_v_node2); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1895, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1896
 *                     tracker.push_greater_of(1, node1)
 *                     tracker.push_less_of(2, node2)
 *                     self.__count_neighbors_traverse(             # <<<<<<<<<<<<<<
 *                         other, n_queries, r, results, idx,
 *                         node1.greater, node2.less, tracker)
 */
        __pyx_t_7 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___count_neighbors_traverse(__pyx_v_self, __pyx_v_other, __pyx_v_n_queries, __pyx_v_r, __pyx_v_results, __pyx_v_idx, __pyx_v_node1->greater, __pyx_v_node2->less, __pyx_v_tracker); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1896, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1899
 *                         other, n_queries, r, results, idx,
 *                         node1.greater, node2.less, tracker)
 *                     tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *                     tracker.push_greater_of(2, node2)
 */
        __pyx_t_7 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1899, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1901
 *                     tracker.pop()
 * 
 *                     tracker.push_greater_of(2, node2)             # <<<<<<<<<<<<<<
 *                     self.__count_neighbors_traverse(
 *                         other, n_queries, r, results, idx,
 */
        __pyx_t_7 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_greater_of(__pyx_v_tracker, 2, __pyx_v_node2); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1901, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1902
 * 
 *                     tracker.push_greater_of(2, node2)
 *                     self.__count_neighbors_traverse(             # <<<<<<<<<<<<<<
 *                         other, n_queries, r, results, idx,
 *                         node1.greater, node2.greater, tracker)
 */
        __pyx_t_7 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___count_neighbors_traverse(__pyx_v_self, __pyx_v_other, __pyx_v_n_queries, __pyx_v_r, __pyx_v_results, __pyx_v_idx, __pyx_v_node1->greater, __pyx_v_node2->greater, __pyx_v_tracker); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1902, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1905
 *                         other, n_queries, r, results, idx,
 *                         node1.greater, node2.greater, tracker)
 *                     tracker.pop()             # <<<<<<<<<<<<<<
 *                     tracker.pop()
 * 
 */
        __pyx_t_7 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1905, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":1906
 *                         node1.greater, node2.greater, tracker)
 *                     tracker.pop()
 *                     tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *         return 0
 */
        __pyx_t_7 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1906, __pyx_L1_error)
      }
      __pyx_L16:;
    }
    __pyx_L7:;

    /* "pyart/map/ckdtree.pyx":1831
 *                 n_queries += 1
 * 
 *         if n_queries > 0:             # <<<<<<<<<<<<<<
 *             # OK, need to probe a bit deeper
 *             if node1.split_dim == -1:  # 1 is leaf node
 */
  }

  /* "pyart/map/ckdtree.pyx":1908
 *                     tracker.pop()
 * 
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     @cython.boundscheck(False)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":1800
 *     # count_neighbors
 *     # ---------------
 *     cdef int __count_neighbors_traverse(cKDTree self,             # <<<<<<<<<<<<<<
 *                                         cKDTree other,
 *                                         np.intp_t n_queries,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_inner_idx.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.__count_neighbors_traverse", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_inner_idx.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_inner_idx);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":1912
 *     @cython.boundscheck(False)
 *     @cython.wraparound(False)
 *     def count_neighbors(cKDTree self, cKDTree other, object r, np.float64_t p=2.):             # <<<<<<<<<<<<<<
 *         """count_neighbors(self, other, r, p)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_13count_neighbors(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5pyart_3map_7ckdtree_7cKDTree_12count_neighbors[] = "count_neighbors(self, other, r, p)\n\n        Count how many nearby pairs can be formed.\n\n        Count the number of pairs (x1,x2) can be formed, with x1 drawn\n        from self and x2 drawn from `other`, and where\n        ``distance(x1, x2, p) <= r``.\n        This is the \"two-point correlation\" described in Gray and Moore 2000,\n        \"N-body problems in statistical learning\", and the code here is based\n        on their algorithm.\n\n        Parameters\n        ----------\n        other : KDTree instance\n            The other tree to draw points from.\n        r : float or one-dimensional array of floats\n            The radius to produce a count for. Multiple radii are searched with\n            a single tree traversal.\n        p : float, 1<=p<=infinity\n            Which Minkowski p-norm to use\n\n        Returns\n        -------\n        result : int or 1-D array of ints\n            The number of pairs. Note that this is internally stored in a numpy int,\n            and so may overflow if very large (2e9).\n\n        ";
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_13count_neighbors(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_other = 0;
  PyObject *__pyx_v_r = 0;
  __pyx_t_5numpy_float64_t __pyx_v_p;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("count_neighbors (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_other,&__pyx_n_s_r,&__pyx_n_s_p,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_other)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_r)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("count_neighbors", 0, 2, 3, 1); __PYX_ERR(0, 1912, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_p);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "count_neighbors") < 0)) __PYX_ERR(0, 1912, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_other = ((struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *)values[0]);
    __pyx_v_r = values[1];
    if (values[2]) {
      __pyx_v_p = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_p == ((npy_float64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1912, __pyx_L3_error)
    } else {
      __pyx_v_p = ((__pyx_t_5numpy_float64_t)2.);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("count_neighbors", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1912, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.count_neighbors", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_5pyart_3map_7ckdtree_cKDTree, 1, "other", 0))) __PYX_ERR(0, 1912, __pyx_L1_error)
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_7cKDTree_12count_neighbors(((struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self), __pyx_v_other, __pyx_v_r, __pyx_v_p);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_12count_neighbors(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_other, PyObject *__pyx_v_r, __pyx_t_5numpy_float64_t __pyx_v_p) {
  __pyx_t_5numpy_intp_t __pyx_v_n_queries;
  __pyx_t_5numpy_intp_t __pyx_v_i;
  PyArrayObject *__pyx_v_real_r = 0;
  PyArrayObject *__pyx_v_results = 0;
  PyArrayObject *__pyx_v_idx = 0;
  struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_tracker = NULL;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_idx;
  __Pyx_Buffer __pyx_pybuffer_idx;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_real_r;
  __Pyx_Buffer __pyx_pybuffer_real_r;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_results;
  __Pyx_Buffer __pyx_pybuffer_results;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyArrayObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  Py_ssize_t __pyx_t_12;
  __pyx_t_5numpy_intp_t __pyx_t_13;
  __pyx_t_5numpy_intp_t __pyx_t_14;
  __pyx_t_5numpy_intp_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  PyArrayObject *__pyx_t_18 = NULL;
  Py_ssize_t __pyx_t_19;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("count_neighbors", 0);
  __pyx_pybuffer_real_r.pybuffer.buf = NULL;
  __pyx_pybuffer_real_r.refcount = 0;
  __pyx_pybuffernd_real_r.data = NULL;
  __pyx_pybuffernd_real_r.rcbuffer = &__pyx_pybuffer_real_r;
  __pyx_pybuffer_results.pybuffer.buf = NULL;
  __pyx_pybuffer_results.refcount = 0;
  __pyx_pybuffernd_results.data = NULL;
  __pyx_pybuffernd_results.rcbuffer = &__pyx_pybuffer_results;
  __pyx_pybuffer_idx.pybuffer.buf = NULL;
  __pyx_pybuffer_idx.refcount = 0;
  __pyx_pybuffernd_idx.data = NULL;
  __pyx_pybuffernd_idx.rcbuffer = &__pyx_pybuffer_idx;

  /* "pyart/map/ckdtree.pyx":1946
 * 
 *         # Make sure trees are compatible
 *         if self.m != other.m:             # <<<<<<<<<<<<<<
 *             raise ValueError("Trees passed to query_ball_trees have different dimensionality")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->m != __pyx_v_other->m) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "pyart/map/ckdtree.pyx":1947
 *         # Make sure trees are compatible
 *         if self.m != other.m:
 *             raise ValueError("Trees passed to query_ball_trees have different dimensionality")             # <<<<<<<<<<<<<<
 * 
 *         # Make a copy of r array to ensure it's contiguous and to modify it
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__19, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1947, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1947, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":1946
 * 
 *         # Make sure trees are compatible
 *         if self.m != other.m:             # <<<<<<<<<<<<<<
 *             raise ValueError("Trees passed to query_ball_trees have different dimensionality")
 * 
 */
  }

  /* "pyart/map/ckdtree.pyx":1951
 *         # Make a copy of r array to ensure it's contiguous and to modify it
 *         # below
 *         if np.shape(r) == ():             # <<<<<<<<<<<<<<
 *             real_r = np.array([r], dtype=np.float64)
 *             n_queries = 1
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1951, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_shape); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1951, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_3, __pyx_v_r) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_r);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1951, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyObject_RichCompare(__pyx_t_2, __pyx_empty_tuple, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1951, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 1951, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":1952
 *         # below
 *         if np.shape(r) == ():
 *             real_r = np.array([r], dtype=np.float64)             # <<<<<<<<<<<<<<
 *             n_queries = 1
 *         elif len(np.shape(r))==1:
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1952, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_array); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1952, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyList_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1952, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v_r);
    __Pyx_GIVEREF(__pyx_v_r);
    PyList_SET_ITEM(__pyx_t_4, 0, __pyx_v_r);
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1952, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1952, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1952, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_float64); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1952, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_6) < 0) __PYX_ERR(0, 1952, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1952, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1952, __pyx_L1_error)
    __pyx_t_7 = ((PyArrayObject *)__pyx_t_6);
    {
      __Pyx_BufFmt_StackElem __pyx_stack[1];
      __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_real_r.rcbuffer->pybuffer);
      __pyx_t_8 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_real_r.rcbuffer->pybuffer, (PyObject*)__pyx_t_7, &__Pyx_TypeInfo_nn___pyx_t_5numpy_float64_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS| PyBUF_WRITABLE, 1, 0, __pyx_stack);
      if (unlikely(__pyx_t_8 < 0)) {
        PyErr_Fetch(&__pyx_t_9, &__pyx_t_10, &__pyx_t_11);
        if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_real_r.rcbuffer->pybuffer, (PyObject*)__pyx_v_real_r, &__Pyx_TypeInfo_nn___pyx_t_5numpy_float64_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS| PyBUF_WRITABLE, 1, 0, __pyx_stack) == -1)) {
          Py_XDECREF(__pyx_t_9); Py_XDECREF(__pyx_t_10); Py_XDECREF(__pyx_t_11);
          __Pyx_RaiseBufferFallbackError();
        } else {
          PyErr_Restore(__pyx_t_9, __pyx_t_10, __pyx_t_11);
        }
        __pyx_t_9 = __pyx_t_10 = __pyx_t_11 = 0;
      }
      __pyx_pybuffernd_real_r.diminfo[0].strides = __pyx_pybuffernd_real_r.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_real_r.diminfo[0].shape = __pyx_pybuffernd_real_r.rcbuffer->pybuffer.shape[0];
      if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 1952, __pyx_L1_error)
    }
    __pyx_t_7 = 0;
    __pyx_v_real_r = ((PyArrayObject *)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "pyart/map/ckdtree.pyx":1953
 *         if np.shape(r) == ():
 *             real_r = np.array([r], dtype=np.float64)
 *             n_queries = 1             # <<<<<<<<<<<<<<
 *         elif len(np.shape(r))==1:
 *             real_r = np.array(r, dtype=np.float64)
 */
    __pyx_v_n_queries = 1;

    /* "pyart/map/ckdtree.pyx":1951
 *         # Make a copy of r array to ensure it's contiguous and to modify it
 *         # below
 *         if np.shape(r) == ():             # <<<<<<<<<<<<<<
 *             real_r = np.array([r], dtype=np.float64)
 *             n_queries = 1
 */
    goto __pyx_L4;
  }

  /* "pyart/map/ckdtree.pyx":1954
 *             real_r = np.array([r], dtype=np.float64)
 *             n_queries = 1
 *         elif len(np.shape(r))==1:             # <<<<<<<<<<<<<<
 *             real_r = np.array(r, dtype=np.float64)
 *             n_queries = r.shape[0]
 */
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1954, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_shape); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1954, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_6 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_r) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_r);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1954, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_12 = PyObject_Length(__pyx_t_6); if (unlikely(__pyx_t_12 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1954, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_1 = ((__pyx_t_12 == 1) != 0);
  if (likely(__pyx_t_1)) {

    /* "pyart/map/ckdtree.pyx":1955
 *             n_queries = 1
 *         elif len(np.shape(r))==1:
 *             real_r = np.array(r, dtype=np.float64)             # <<<<<<<<<<<<<<
 *             n_queries = r.shape[0]
 *         else:
 */
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_np); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1955, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_array); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1955, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1955, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v_r);
    __Pyx_GIVEREF(__pyx_v_r);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_r);
    __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1955, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1955, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_float64); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1955, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_5) < 0) __PYX_ERR(0, 1955, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1955, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (!(likely(((__pyx_t_5) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_5, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1955, __pyx_L1_error)
    __pyx_t_7 = ((PyArrayObject *)__pyx_t_5);
    {
      __Pyx_BufFmt_StackElem __pyx_stack[1];
      __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_real_r.rcbuffer->pybuffer);
      __pyx_t_8 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_real_r.rcbuffer->pybuffer, (PyObject*)__pyx_t_7, &__Pyx_TypeInfo_nn___pyx_t_5numpy_float64_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS| PyBUF_WRITABLE, 1, 0, __pyx_stack);
      if (unlikely(__pyx_t_8 < 0)) {
        PyErr_Fetch(&__pyx_t_11, &__pyx_t_10, &__pyx_t_9);
        if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_real_r.rcbuffer->pybuffer, (PyObject*)__pyx_v_real_r, &__Pyx_TypeInfo_nn___pyx_t_5numpy_float64_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS| PyBUF_WRITABLE, 1, 0, __pyx_stack) == -1)) {
          Py_XDECREF(__pyx_t_11); Py_XDECREF(__pyx_t_10); Py_XDECREF(__pyx_t_9);
          __Pyx_RaiseBufferFallbackError();
        } else {
          PyErr_Restore(__pyx_t_11, __pyx_t_10, __pyx_t_9);
        }
        __pyx_t_11 = __pyx_t_10 = __pyx_t_9 = 0;
      }
      __pyx_pybuffernd_real_r.diminfo[0].strides = __pyx_pybuffernd_real_r.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_real_r.diminfo[0].shape = __pyx_pybuffernd_real_r.rcbuffer->pybuffer.shape[0];
      if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 1955, __pyx_L1_error)
    }
    __pyx_t_7 = 0;
    __pyx_v_real_r = ((PyArrayObject *)__pyx_t_5);
    __pyx_t_5 = 0;

    /* "pyart/map/ckdtree.pyx":1956
 *         elif len(np.shape(r))==1:
 *             real_r = np.array(r, dtype=np.float64)
 *             n_queries = r.shape[0]             # <<<<<<<<<<<<<<
 *         else:
 *             raise ValueError("r must be either a single value or a one-dimensional array of values")
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_r, __pyx_n_s_shape); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1956, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = __Pyx_GetItemInt(__pyx_t_5, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1956, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_13 = __Pyx_PyInt_As_Py_intptr_t(__pyx_t_4); if (unlikely((__pyx_t_13 == ((npy_intp)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1956, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_n_queries = __pyx_t_13;

    /* "pyart/map/ckdtree.pyx":1954
 *             real_r = np.array([r], dtype=np.float64)
 *             n_queries = 1
 *         elif len(np.shape(r))==1:             # <<<<<<<<<<<<<<
 *             real_r = np.array(r, dtype=np.float64)
 *             n_queries = r.shape[0]
 */
    goto __pyx_L4;
  }

  /* "pyart/map/ckdtree.pyx":1958
 *             n_queries = r.shape[0]
 *         else:
 *             raise ValueError("r must be either a single value or a one-dimensional array of values")             # <<<<<<<<<<<<<<
 * 
 *         # Internally, we represent all distances as distance ** p
 */
  /*else*/ {
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__20, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1958, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 1958, __pyx_L1_error)
  }
  __pyx_L4:;

  /* "pyart/map/ckdtree.pyx":1961
 * 
 *         # Internally, we represent all distances as distance ** p
 *         if p != infinity:             # <<<<<<<<<<<<<<
 *             for i in range(n_queries):
 *                 if real_r[i] != infinity:
 */
  __pyx_t_1 = ((__pyx_v_p != __pyx_v_5pyart_3map_7ckdtree_infinity) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":1962
 *         # Internally, we represent all distances as distance ** p
 *         if p != infinity:
 *             for i in range(n_queries):             # <<<<<<<<<<<<<<
 *                 if real_r[i] != infinity:
 *                     real_r[i] = real_r[i] ** p
 */
    __pyx_t_13 = __pyx_v_n_queries;
    __pyx_t_14 = __pyx_t_13;
    for (__pyx_t_15 = 0; __pyx_t_15 < __pyx_t_14; __pyx_t_15+=1) {
      __pyx_v_i = __pyx_t_15;

      /* "pyart/map/ckdtree.pyx":1963
 *         if p != infinity:
 *             for i in range(n_queries):
 *                 if real_r[i] != infinity:             # <<<<<<<<<<<<<<
 *                     real_r[i] = real_r[i] ** p
 * 
 */
      __pyx_t_16 = __pyx_v_i;
      __pyx_t_1 = (((*__Pyx_BufPtrCContig1d(__pyx_t_5numpy_float64_t *, __pyx_pybuffernd_real_r.rcbuffer->pybuffer.buf, __pyx_t_16, __pyx_pybuffernd_real_r.diminfo[0].strides)) != __pyx_v_5pyart_3map_7ckdtree_infinity) != 0);
      if (__pyx_t_1) {

        /* "pyart/map/ckdtree.pyx":1964
 *             for i in range(n_queries):
 *                 if real_r[i] != infinity:
 *                     real_r[i] = real_r[i] ** p             # <<<<<<<<<<<<<<
 * 
 *         # Track node-to-node min/max distances
 */
        __pyx_t_16 = __pyx_v_i;
        __pyx_t_17 = __pyx_v_i;
        *__Pyx_BufPtrCContig1d(__pyx_t_5numpy_float64_t *, __pyx_pybuffernd_real_r.rcbuffer->pybuffer.buf, __pyx_t_17, __pyx_pybuffernd_real_r.diminfo[0].strides) = pow((*__Pyx_BufPtrCContig1d(__pyx_t_5numpy_float64_t *, __pyx_pybuffernd_real_r.rcbuffer->pybuffer.buf, __pyx_t_16, __pyx_pybuffernd_real_r.diminfo[0].strides)), __pyx_v_p);

        /* "pyart/map/ckdtree.pyx":1963
 *         if p != infinity:
 *             for i in range(n_queries):
 *                 if real_r[i] != infinity:             # <<<<<<<<<<<<<<
 *                     real_r[i] = real_r[i] ** p
 * 
 */
      }
    }

    /* "pyart/map/ckdtree.pyx":1961
 * 
 *         # Internally, we represent all distances as distance ** p
 *         if p != infinity:             # <<<<<<<<<<<<<<
 *             for i in range(n_queries):
 *                 if real_r[i] != infinity:
 */
  }

  /* "pyart/map/ckdtree.pyx":1968
 *         # Track node-to-node min/max distances
 *         tracker = RectRectDistanceTracker(
 *             Rectangle(self.mins, self.maxes),             # <<<<<<<<<<<<<<
 *             Rectangle(other.mins, other.maxes),
 *             p, 0.0, 0.0)
 */
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1968, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->mins));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->mins));
  PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)__pyx_v_self->mins));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->maxes));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->maxes));
  PyTuple_SET_ITEM(__pyx_t_4, 1, ((PyObject *)__pyx_v_self->maxes));
  __pyx_t_5 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pyart_3map_7ckdtree_Rectangle), __pyx_t_4, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1968, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyart/map/ckdtree.pyx":1969
 *         tracker = RectRectDistanceTracker(
 *             Rectangle(self.mins, self.maxes),
 *             Rectangle(other.mins, other.maxes),             # <<<<<<<<<<<<<<
 *             p, 0.0, 0.0)
 * 
 */
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1969, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(((PyObject *)__pyx_v_other->mins));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_other->mins));
  PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)__pyx_v_other->mins));
  __Pyx_INCREF(((PyObject *)__pyx_v_other->maxes));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_other->maxes));
  PyTuple_SET_ITEM(__pyx_t_4, 1, ((PyObject *)__pyx_v_other->maxes));
  __pyx_t_6 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pyart_3map_7ckdtree_Rectangle), __pyx_t_4, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1969, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyart/map/ckdtree.pyx":1970
 *             Rectangle(self.mins, self.maxes),
 *             Rectangle(other.mins, other.maxes),
 *             p, 0.0, 0.0)             # <<<<<<<<<<<<<<
 * 
 *         # Go!
 */
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_p); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1970, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "pyart/map/ckdtree.pyx":1967
 * 
 *         # Track node-to-node min/max distances
 *         tracker = RectRectDistanceTracker(             # <<<<<<<<<<<<<<
 *             Rectangle(self.mins, self.maxes),
 *             Rectangle(other.mins, other.maxes),
 */
  __pyx_t_3 = PyTuple_New(5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1967, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_4);
  __Pyx_INCREF(__pyx_float_0_0);
  __Pyx_GIVEREF(__pyx_float_0_0);
  PyTuple_SET_ITEM(__pyx_t_3, 3, __pyx_float_0_0);
  __Pyx_INCREF(__pyx_float_0_0);
  __Pyx_GIVEREF(__pyx_float_0_0);
  PyTuple_SET_ITEM(__pyx_t_3, 4, __pyx_float_0_0);
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pyart_3map_7ckdtree_RectRectDistanceTracker), __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1967, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_tracker = ((struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "pyart/map/ckdtree.pyx":1973
 * 
 *         # Go!
 *         results = np.zeros((n_queries,), dtype=np.intp)             # <<<<<<<<<<<<<<
 *         idx = np.arange(n_queries, dtype=np.intp)
 *         self.__count_neighbors_traverse(other, n_queries,
 */
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1973, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1973, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n_queries); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1973, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1973, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1973, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6);
  __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1973, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1973, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_intp); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1973, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_dtype, __pyx_t_2) < 0) __PYX_ERR(0, 1973, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1973, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1973, __pyx_L1_error)
  __pyx_t_18 = ((PyArrayObject *)__pyx_t_2);
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_results.rcbuffer->pybuffer);
    __pyx_t_8 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_results.rcbuffer->pybuffer, (PyObject*)__pyx_t_18, &__Pyx_TypeInfo_nn___pyx_t_5numpy_intp_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack);
    if (unlikely(__pyx_t_8 < 0)) {
      PyErr_Fetch(&__pyx_t_9, &__pyx_t_10, &__pyx_t_11);
      if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_results.rcbuffer->pybuffer, (PyObject*)__pyx_v_results, &__Pyx_TypeInfo_nn___pyx_t_5numpy_intp_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {
        Py_XDECREF(__pyx_t_9); Py_XDECREF(__pyx_t_10); Py_XDECREF(__pyx_t_11);
        __Pyx_RaiseBufferFallbackError();
      } else {
        PyErr_Restore(__pyx_t_9, __pyx_t_10, __pyx_t_11);
      }
      __pyx_t_9 = __pyx_t_10 = __pyx_t_11 = 0;
    }
    __pyx_pybuffernd_results.diminfo[0].strides = __pyx_pybuffernd_results.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_results.diminfo[0].shape = __pyx_pybuffernd_results.rcbuffer->pybuffer.shape[0];
    if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 1973, __pyx_L1_error)
  }
  __pyx_t_18 = 0;
  __pyx_v_results = ((PyArrayObject *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyart/map/ckdtree.pyx":1974
 *         # Go!
 *         results = np.zeros((n_queries,), dtype=np.intp)
 *         idx = np.arange(n_queries, dtype=np.intp)             # <<<<<<<<<<<<<<
 *         self.__count_neighbors_traverse(other, n_queries,
 *                                         &real_r[0], &results[0], &idx[0],
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1974, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_arange); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1974, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n_queries); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1974, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1974, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1974, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1974, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_intp); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1974, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_5) < 0) __PYX_ERR(0, 1974, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_4, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1974, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(((__pyx_t_5) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_5, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1974, __pyx_L1_error)
  __pyx_t_18 = ((PyArrayObject *)__pyx_t_5);
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_idx.rcbuffer->pybuffer);
    __pyx_t_8 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_idx.rcbuffer->pybuffer, (PyObject*)__pyx_t_18, &__Pyx_TypeInfo_nn___pyx_t_5numpy_intp_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack);
    if (unlikely(__pyx_t_8 < 0)) {
      PyErr_Fetch(&__pyx_t_11, &__pyx_t_10, &__pyx_t_9);
      if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_idx.rcbuffer->pybuffer, (PyObject*)__pyx_v_idx, &__Pyx_TypeInfo_nn___pyx_t_5numpy_intp_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {
        Py_XDECREF(__pyx_t_11); Py_XDECREF(__pyx_t_10); Py_XDECREF(__pyx_t_9);
        __Pyx_RaiseBufferFallbackError();
      } else {
        PyErr_Restore(__pyx_t_11, __pyx_t_10, __pyx_t_9);
      }
      __pyx_t_11 = __pyx_t_10 = __pyx_t_9 = 0;
    }
    __pyx_pybuffernd_idx.diminfo[0].strides = __pyx_pybuffernd_idx.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_idx.diminfo[0].shape = __pyx_pybuffernd_idx.rcbuffer->pybuffer.shape[0];
    if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 1974, __pyx_L1_error)
  }
  __pyx_t_18 = 0;
  __pyx_v_idx = ((PyArrayObject *)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "pyart/map/ckdtree.pyx":1976
 *         idx = np.arange(n_queries, dtype=np.intp)
 *         self.__count_neighbors_traverse(other, n_queries,
 *                                         &real_r[0], &results[0], &idx[0],             # <<<<<<<<<<<<<<
 *                                         self.tree, other.tree,
 *                                         tracker)
 */
  __pyx_t_16 = 0;
  __pyx_t_17 = 0;
  __pyx_t_19 = 0;

  /* "pyart/map/ckdtree.pyx":1975
 *         results = np.zeros((n_queries,), dtype=np.intp)
 *         idx = np.arange(n_queries, dtype=np.intp)
 *         self.__count_neighbors_traverse(other, n_queries,             # <<<<<<<<<<<<<<
 *                                         &real_r[0], &results[0], &idx[0],
 *                                         self.tree, other.tree,
 */
  __pyx_t_8 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___count_neighbors_traverse(__pyx_v_self, __pyx_v_other, __pyx_v_n_queries, (&(*__Pyx_BufPtrCContig1d(__pyx_t_5numpy_float64_t *, __pyx_pybuffernd_real_r.rcbuffer->pybuffer.buf, __pyx_t_16, __pyx_pybuffernd_real_r.diminfo[0].strides))), (&(*__Pyx_BufPtrCContig1d(__pyx_t_5numpy_intp_t *, __pyx_pybuffernd_results.rcbuffer->pybuffer.buf, __pyx_t_17, __pyx_pybuffernd_results.diminfo[0].strides))), (&(*__Pyx_BufPtrCContig1d(__pyx_t_5numpy_intp_t *, __pyx_pybuffernd_idx.rcbuffer->pybuffer.buf, __pyx_t_19, __pyx_pybuffernd_idx.diminfo[0].strides))), __pyx_v_self->tree, __pyx_v_other->tree, __pyx_v_tracker); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 1975, __pyx_L1_error)

  /* "pyart/map/ckdtree.pyx":1980
 *                                         tracker)
 * 
 *         if np.shape(r) == ():             # <<<<<<<<<<<<<<
 *             if results[0] <= <np.intp_t> LONG_MAX:
 *                 return int(results[0])
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1980, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_shape); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1980, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_5 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_2, __pyx_v_r) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_r);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1980, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyObject_RichCompare(__pyx_t_5, __pyx_empty_tuple, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1980, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 1980, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":1981
 * 
 *         if np.shape(r) == ():
 *             if results[0] <= <np.intp_t> LONG_MAX:             # <<<<<<<<<<<<<<
 *                 return int(results[0])
 *             else:
 */
    __pyx_t_19 = 0;
    __pyx_t_1 = (((*__Pyx_BufPtrCContig1d(__pyx_t_5numpy_intp_t *, __pyx_pybuffernd_results.rcbuffer->pybuffer.buf, __pyx_t_19, __pyx_pybuffernd_results.diminfo[0].strides)) <= ((__pyx_t_5numpy_intp_t)LONG_MAX)) != 0);
    if (__pyx_t_1) {

      /* "pyart/map/ckdtree.pyx":1982
 *         if np.shape(r) == ():
 *             if results[0] <= <np.intp_t> LONG_MAX:
 *                 return int(results[0])             # <<<<<<<<<<<<<<
 *             else:
 *                 return results[0]
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_19 = 0;
      __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t((*__Pyx_BufPtrCContig1d(__pyx_t_5numpy_intp_t *, __pyx_pybuffernd_results.rcbuffer->pybuffer.buf, __pyx_t_19, __pyx_pybuffernd_results.diminfo[0].strides))); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1982, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyInt_Type)), __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1982, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_r = __pyx_t_5;
      __pyx_t_5 = 0;
      goto __pyx_L0;

      /* "pyart/map/ckdtree.pyx":1981
 * 
 *         if np.shape(r) == ():
 *             if results[0] <= <np.intp_t> LONG_MAX:             # <<<<<<<<<<<<<<
 *                 return int(results[0])
 *             else:
 */
    }

    /* "pyart/map/ckdtree.pyx":1984
 *                 return int(results[0])
 *             else:
 *                 return results[0]             # <<<<<<<<<<<<<<
 *         elif len(np.shape(r))==1:
 *             return results
 */
    /*else*/ {
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_19 = 0;
      __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t((*__Pyx_BufPtrCContig1d(__pyx_t_5numpy_intp_t *, __pyx_pybuffernd_results.rcbuffer->pybuffer.buf, __pyx_t_19, __pyx_pybuffernd_results.diminfo[0].strides))); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1984, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_r = __pyx_t_5;
      __pyx_t_5 = 0;
      goto __pyx_L0;
    }

    /* "pyart/map/ckdtree.pyx":1980
 *                                         tracker)
 * 
 *         if np.shape(r) == ():             # <<<<<<<<<<<<<<
 *             if results[0] <= <np.intp_t> LONG_MAX:
 *                 return int(results[0])
 */
  }

  /* "pyart/map/ckdtree.pyx":1985
 *             else:
 *                 return results[0]
 *         elif len(np.shape(r))==1:             # <<<<<<<<<<<<<<
 *             return results
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1985, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_shape); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1985, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_5 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v_r) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_r);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1985, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_12 = PyObject_Length(__pyx_t_5); if (unlikely(__pyx_t_12 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1985, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_1 = ((__pyx_t_12 == 1) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":1986
 *                 return results[0]
 *         elif len(np.shape(r))==1:
 *             return results             # <<<<<<<<<<<<<<
 * 
 *     # ----------------------
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_results));
    __pyx_r = ((PyObject *)__pyx_v_results);
    goto __pyx_L0;

    /* "pyart/map/ckdtree.pyx":1985
 *             else:
 *                 return results[0]
 *         elif len(np.shape(r))==1:             # <<<<<<<<<<<<<<
 *             return results
 * 
 */
  }

  /* "pyart/map/ckdtree.pyx":1912
 *     @cython.boundscheck(False)
 *     @cython.wraparound(False)
 *     def count_neighbors(cKDTree self, cKDTree other, object r, np.float64_t p=2.):             # <<<<<<<<<<<<<<
 *         """count_neighbors(self, other, r, p)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_idx.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_real_r.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_results.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.count_neighbors", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_idx.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_real_r.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_results.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_real_r);
  __Pyx_XDECREF((PyObject *)__pyx_v_results);
  __Pyx_XDECREF((PyObject *)__pyx_v_idx);
  __Pyx_XDECREF((PyObject *)__pyx_v_tracker);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":1991
 *     # sparse_distance_matrix
 *     # ----------------------
 *     cdef int __sparse_distance_matrix_traverse(cKDTree self, cKDTree other,             # <<<<<<<<<<<<<<
 *                                                coo_entries results,
 *                                                innernode* node1, innernode* node2,
 */

static int __pyx_f_5pyart_3map_7ckdtree_7cKDTree___sparse_distance_matrix_traverse(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_other, struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *__pyx_v_results, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node1, struct __pyx_t_5pyart_3map_7ckdtree_innernode *__pyx_v_node2, struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_tracker) {
  struct __pyx_t_5pyart_3map_7ckdtree_leafnode *__pyx_v_lnode1;
  struct __pyx_t_5pyart_3map_7ckdtree_leafnode *__pyx_v_lnode2;
  __pyx_t_5numpy_float64_t __pyx_v_d;
  __pyx_t_5numpy_intp_t __pyx_v_i;
  __pyx_t_5numpy_intp_t __pyx_v_j;
  __pyx_t_5numpy_intp_t __pyx_v_min_j;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __pyx_t_5numpy_intp_t __pyx_t_2;
  __pyx_t_5numpy_intp_t __pyx_t_3;
  __pyx_t_5numpy_intp_t __pyx_t_4;
  __pyx_t_5numpy_intp_t __pyx_t_5;
  __pyx_t_5numpy_intp_t __pyx_t_6;
  __pyx_t_5numpy_intp_t __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__sparse_distance_matrix_traverse", 0);

  /* "pyart/map/ckdtree.pyx":2001
 *         cdef np.intp_t i, j, min_j
 * 
 *         if tracker.min_distance > tracker.upper_bound:             # <<<<<<<<<<<<<<
 *             return 0
 *         elif node1.split_dim == -1:  # 1 is leaf node
 */
  __pyx_t_1 = ((__pyx_v_tracker->min_distance > __pyx_v_tracker->upper_bound) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":2002
 * 
 *         if tracker.min_distance > tracker.upper_bound:
 *             return 0             # <<<<<<<<<<<<<<
 *         elif node1.split_dim == -1:  # 1 is leaf node
 *             lnode1 = <leafnode*>node1
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "pyart/map/ckdtree.pyx":2001
 *         cdef np.intp_t i, j, min_j
 * 
 *         if tracker.min_distance > tracker.upper_bound:             # <<<<<<<<<<<<<<
 *             return 0
 *         elif node1.split_dim == -1:  # 1 is leaf node
 */
  }

  /* "pyart/map/ckdtree.pyx":2003
 *         if tracker.min_distance > tracker.upper_bound:
 *             return 0
 *         elif node1.split_dim == -1:  # 1 is leaf node             # <<<<<<<<<<<<<<
 *             lnode1 = <leafnode*>node1
 * 
 */
  __pyx_t_1 = ((__pyx_v_node1->split_dim == -1L) != 0);
  if (__pyx_t_1) {

    /* "pyart/map/ckdtree.pyx":2004
 *             return 0
 *         elif node1.split_dim == -1:  # 1 is leaf node
 *             lnode1 = <leafnode*>node1             # <<<<<<<<<<<<<<
 * 
 *             if node2.split_dim == -1:  # 1 & 2 are leaves
 */
    __pyx_v_lnode1 = ((struct __pyx_t_5pyart_3map_7ckdtree_leafnode *)__pyx_v_node1);

    /* "pyart/map/ckdtree.pyx":2006
 *             lnode1 = <leafnode*>node1
 * 
 *             if node2.split_dim == -1:  # 1 & 2 are leaves             # <<<<<<<<<<<<<<
 *                 lnode2 = <leafnode*>node2
 * 
 */
    __pyx_t_1 = ((__pyx_v_node2->split_dim == -1L) != 0);
    if (__pyx_t_1) {

      /* "pyart/map/ckdtree.pyx":2007
 * 
 *             if node2.split_dim == -1:  # 1 & 2 are leaves
 *                 lnode2 = <leafnode*>node2             # <<<<<<<<<<<<<<
 * 
 *                 # brute-force
 */
      __pyx_v_lnode2 = ((struct __pyx_t_5pyart_3map_7ckdtree_leafnode *)__pyx_v_node2);

      /* "pyart/map/ckdtree.pyx":2010
 * 
 *                 # brute-force
 *                 for i in range(lnode1.start_idx, lnode1.end_idx):             # <<<<<<<<<<<<<<
 *                     # Special care here to avoid duplicate pairs
 *                     if node1 == node2:
 */
      __pyx_t_2 = __pyx_v_lnode1->end_idx;
      __pyx_t_3 = __pyx_t_2;
      for (__pyx_t_4 = __pyx_v_lnode1->start_idx; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
        __pyx_v_i = __pyx_t_4;

        /* "pyart/map/ckdtree.pyx":2012
 *                 for i in range(lnode1.start_idx, lnode1.end_idx):
 *                     # Special care here to avoid duplicate pairs
 *                     if node1 == node2:             # <<<<<<<<<<<<<<
 *                         min_j = i+1
 *                     else:
 */
        __pyx_t_1 = ((__pyx_v_node1 == __pyx_v_node2) != 0);
        if (__pyx_t_1) {

          /* "pyart/map/ckdtree.pyx":2013
 *                     # Special care here to avoid duplicate pairs
 *                     if node1 == node2:
 *                         min_j = i+1             # <<<<<<<<<<<<<<
 *                     else:
 *                         min_j = lnode2.end_idx
 */
          __pyx_v_min_j = (__pyx_v_i + 1);

          /* "pyart/map/ckdtree.pyx":2012
 *                 for i in range(lnode1.start_idx, lnode1.end_idx):
 *                     # Special care here to avoid duplicate pairs
 *                     if node1 == node2:             # <<<<<<<<<<<<<<
 *                         min_j = i+1
 *                     else:
 */
          goto __pyx_L7;
        }

        /* "pyart/map/ckdtree.pyx":2015
 *                         min_j = i+1
 *                     else:
 *                         min_j = lnode2.end_idx             # <<<<<<<<<<<<<<
 * 
 *                     for j in range(min_j, lnode2.end_idx):
 */
        /*else*/ {
          __pyx_t_5 = __pyx_v_lnode2->end_idx;
          __pyx_v_min_j = __pyx_t_5;
        }
        __pyx_L7:;

        /* "pyart/map/ckdtree.pyx":2017
 *                         min_j = lnode2.end_idx
 * 
 *                     for j in range(min_j, lnode2.end_idx):             # <<<<<<<<<<<<<<
 *                         d = _distance_p(
 *                             self.raw_data + self.raw_indices[i] * self.m,
 */
        __pyx_t_5 = __pyx_v_lnode2->end_idx;
        __pyx_t_6 = __pyx_t_5;
        for (__pyx_t_7 = __pyx_v_min_j; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
          __pyx_v_j = __pyx_t_7;

          /* "pyart/map/ckdtree.pyx":2018
 * 
 *                     for j in range(min_j, lnode2.end_idx):
 *                         d = _distance_p(             # <<<<<<<<<<<<<<
 *                             self.raw_data + self.raw_indices[i] * self.m,
 *                             other.raw_data + other.raw_indices[j] * self.m,
 */
          __pyx_v_d = __pyx_f_5pyart_3map_7ckdtree__distance_p((__pyx_v_self->raw_data + ((__pyx_v_self->raw_indices[__pyx_v_i]) * __pyx_v_self->m)), (__pyx_v_other->raw_data + ((__pyx_v_other->raw_indices[__pyx_v_j]) * __pyx_v_self->m)), __pyx_v_tracker->p, __pyx_v_self->m, __pyx_v_tracker->upper_bound);

          /* "pyart/map/ckdtree.pyx":2022
 *                             other.raw_data + other.raw_indices[j] * self.m,
 *                             tracker.p, self.m, tracker.upper_bound)
 *                         if d <= tracker.upper_bound:             # <<<<<<<<<<<<<<
 *                             results.add(self.raw_indices[i],
 *                                         self.raw_indices[j], d)
 */
          __pyx_t_1 = ((__pyx_v_d <= __pyx_v_tracker->upper_bound) != 0);
          if (__pyx_t_1) {

            /* "pyart/map/ckdtree.pyx":2023
 *                             tracker.p, self.m, tracker.upper_bound)
 *                         if d <= tracker.upper_bound:
 *                             results.add(self.raw_indices[i],             # <<<<<<<<<<<<<<
 *                                         self.raw_indices[j], d)
 *                             if node1 == node2:
 */
            ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_coo_entries *)__pyx_v_results->__pyx_vtab)->add(__pyx_v_results, (__pyx_v_self->raw_indices[__pyx_v_i]), (__pyx_v_self->raw_indices[__pyx_v_j]), __pyx_v_d);

            /* "pyart/map/ckdtree.pyx":2025
 *                             results.add(self.raw_indices[i],
 *                                         self.raw_indices[j], d)
 *                             if node1 == node2:             # <<<<<<<<<<<<<<
 *                                 results.add(self.raw_indices[j],
 *                                             self.raw_indices[i], d)
 */
            __pyx_t_1 = ((__pyx_v_node1 == __pyx_v_node2) != 0);
            if (__pyx_t_1) {

              /* "pyart/map/ckdtree.pyx":2026
 *                                         self.raw_indices[j], d)
 *                             if node1 == node2:
 *                                 results.add(self.raw_indices[j],             # <<<<<<<<<<<<<<
 *                                             self.raw_indices[i], d)
 * 
 */
              ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_coo_entries *)__pyx_v_results->__pyx_vtab)->add(__pyx_v_results, (__pyx_v_self->raw_indices[__pyx_v_j]), (__pyx_v_self->raw_indices[__pyx_v_i]), __pyx_v_d);

              /* "pyart/map/ckdtree.pyx":2025
 *                             results.add(self.raw_indices[i],
 *                                         self.raw_indices[j], d)
 *                             if node1 == node2:             # <<<<<<<<<<<<<<
 *                                 results.add(self.raw_indices[j],
 *                                             self.raw_indices[i], d)
 */
            }

            /* "pyart/map/ckdtree.pyx":2022
 *                             other.raw_data + other.raw_indices[j] * self.m,
 *                             tracker.p, self.m, tracker.upper_bound)
 *                         if d <= tracker.upper_bound:             # <<<<<<<<<<<<<<
 *                             results.add(self.raw_indices[i],
 *                                         self.raw_indices[j], d)
 */
          }
        }
      }

      /* "pyart/map/ckdtree.pyx":2006
 *             lnode1 = <leafnode*>node1
 * 
 *             if node2.split_dim == -1:  # 1 & 2 are leaves             # <<<<<<<<<<<<<<
 *                 lnode2 = <leafnode*>node2
 * 
 */
      goto __pyx_L4;
    }

    /* "pyart/map/ckdtree.pyx":2030
 * 
 *             else:  # 1 is a leaf node, 2 is inner node
 *                 tracker.push_less_of(2, node2)             # <<<<<<<<<<<<<<
 *                 self.__sparse_distance_matrix_traverse(
 *                     other, results, node1, node2.less, tracker)
 */
    /*else*/ {
      __pyx_t_8 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_less_of(__pyx_v_tracker, 2, __pyx_v_node2); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2030, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":2031
 *             else:  # 1 is a leaf node, 2 is inner node
 *                 tracker.push_less_of(2, node2)
 *                 self.__sparse_distance_matrix_traverse(             # <<<<<<<<<<<<<<
 *                     other, results, node1, node2.less, tracker)
 *                 tracker.pop()
 */
      __pyx_t_8 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___sparse_distance_matrix_traverse(__pyx_v_self, __pyx_v_other, __pyx_v_results, __pyx_v_node1, __pyx_v_node2->less, __pyx_v_tracker); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2031, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":2033
 *                 self.__sparse_distance_matrix_traverse(
 *                     other, results, node1, node2.less, tracker)
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *                 tracker.push_greater_of(2, node2)
 */
      __pyx_t_8 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2033, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":2035
 *                 tracker.pop()
 * 
 *                 tracker.push_greater_of(2, node2)             # <<<<<<<<<<<<<<
 *                 self.__sparse_distance_matrix_traverse(
 *                     other, results, node1, node2.greater, tracker)
 */
      __pyx_t_8 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_greater_of(__pyx_v_tracker, 2, __pyx_v_node2); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2035, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":2036
 * 
 *                 tracker.push_greater_of(2, node2)
 *                 self.__sparse_distance_matrix_traverse(             # <<<<<<<<<<<<<<
 *                     other, results, node1, node2.greater, tracker)
 *                 tracker.pop()
 */
      __pyx_t_8 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___sparse_distance_matrix_traverse(__pyx_v_self, __pyx_v_other, __pyx_v_results, __pyx_v_node1, __pyx_v_node2->greater, __pyx_v_tracker); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2036, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":2038
 *                 self.__sparse_distance_matrix_traverse(
 *                     other, results, node1, node2.greater, tracker)
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *         else:  # 1 is an inner node
 */
      __pyx_t_8 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2038, __pyx_L1_error)
    }
    __pyx_L4:;

    /* "pyart/map/ckdtree.pyx":2003
 *         if tracker.min_distance > tracker.upper_bound:
 *             return 0
 *         elif node1.split_dim == -1:  # 1 is leaf node             # <<<<<<<<<<<<<<
 *             lnode1 = <leafnode*>node1
 * 
 */
    goto __pyx_L3;
  }

  /* "pyart/map/ckdtree.pyx":2041
 * 
 *         else:  # 1 is an inner node
 *             if node2.split_dim == -1:  # 1 is an inner node, 2 is a leaf node             # <<<<<<<<<<<<<<
 *                 tracker.push_less_of(1, node1)
 *                 self.__sparse_distance_matrix_traverse(
 */
  /*else*/ {
    __pyx_t_1 = ((__pyx_v_node2->split_dim == -1L) != 0);
    if (__pyx_t_1) {

      /* "pyart/map/ckdtree.pyx":2042
 *         else:  # 1 is an inner node
 *             if node2.split_dim == -1:  # 1 is an inner node, 2 is a leaf node
 *                 tracker.push_less_of(1, node1)             # <<<<<<<<<<<<<<
 *                 self.__sparse_distance_matrix_traverse(
 *                     other, results, node1.less, node2, tracker)
 */
      __pyx_t_8 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_less_of(__pyx_v_tracker, 1, __pyx_v_node1); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2042, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":2043
 *             if node2.split_dim == -1:  # 1 is an inner node, 2 is a leaf node
 *                 tracker.push_less_of(1, node1)
 *                 self.__sparse_distance_matrix_traverse(             # <<<<<<<<<<<<<<
 *                     other, results, node1.less, node2, tracker)
 *                 tracker.pop()
 */
      __pyx_t_8 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___sparse_distance_matrix_traverse(__pyx_v_self, __pyx_v_other, __pyx_v_results, __pyx_v_node1->less, __pyx_v_node2, __pyx_v_tracker); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2043, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":2045
 *                 self.__sparse_distance_matrix_traverse(
 *                     other, results, node1.less, node2, tracker)
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *                 tracker.push_greater_of(1, node1)
 */
      __pyx_t_8 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2045, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":2047
 *                 tracker.pop()
 * 
 *                 tracker.push_greater_of(1, node1)             # <<<<<<<<<<<<<<
 *                 self.__sparse_distance_matrix_traverse(
 *                     other, results, node1.greater, node2, tracker)
 */
      __pyx_t_8 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_greater_of(__pyx_v_tracker, 1, __pyx_v_node1); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2047, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":2048
 * 
 *                 tracker.push_greater_of(1, node1)
 *                 self.__sparse_distance_matrix_traverse(             # <<<<<<<<<<<<<<
 *                     other, results, node1.greater, node2, tracker)
 *                 tracker.pop()
 */
      __pyx_t_8 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___sparse_distance_matrix_traverse(__pyx_v_self, __pyx_v_other, __pyx_v_results, __pyx_v_node1->greater, __pyx_v_node2, __pyx_v_tracker); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2048, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":2050
 *                 self.__sparse_distance_matrix_traverse(
 *                     other, results, node1.greater, node2, tracker)
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *             else: # 1 and 2 are inner nodes
 */
      __pyx_t_8 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2050, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":2041
 * 
 *         else:  # 1 is an inner node
 *             if node2.split_dim == -1:  # 1 is an inner node, 2 is a leaf node             # <<<<<<<<<<<<<<
 *                 tracker.push_less_of(1, node1)
 *                 self.__sparse_distance_matrix_traverse(
 */
      goto __pyx_L12;
    }

    /* "pyart/map/ckdtree.pyx":2053
 * 
 *             else: # 1 and 2 are inner nodes
 *                 tracker.push_less_of(1, node1)             # <<<<<<<<<<<<<<
 *                 tracker.push_less_of(2, node2)
 *                 self.__sparse_distance_matrix_traverse(
 */
    /*else*/ {
      __pyx_t_8 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_less_of(__pyx_v_tracker, 1, __pyx_v_node1); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2053, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":2054
 *             else: # 1 and 2 are inner nodes
 *                 tracker.push_less_of(1, node1)
 *                 tracker.push_less_of(2, node2)             # <<<<<<<<<<<<<<
 *                 self.__sparse_distance_matrix_traverse(
 *                     other, results, node1.less, node2.less, tracker)
 */
      __pyx_t_8 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_less_of(__pyx_v_tracker, 2, __pyx_v_node2); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2054, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":2055
 *                 tracker.push_less_of(1, node1)
 *                 tracker.push_less_of(2, node2)
 *                 self.__sparse_distance_matrix_traverse(             # <<<<<<<<<<<<<<
 *                     other, results, node1.less, node2.less, tracker)
 *                 tracker.pop()
 */
      __pyx_t_8 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___sparse_distance_matrix_traverse(__pyx_v_self, __pyx_v_other, __pyx_v_results, __pyx_v_node1->less, __pyx_v_node2->less, __pyx_v_tracker); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2055, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":2057
 *                 self.__sparse_distance_matrix_traverse(
 *                     other, results, node1.less, node2.less, tracker)
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *                 tracker.push_greater_of(2, node2)
 */
      __pyx_t_8 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2057, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":2059
 *                 tracker.pop()
 * 
 *                 tracker.push_greater_of(2, node2)             # <<<<<<<<<<<<<<
 *                 self.__sparse_distance_matrix_traverse(
 *                     other, results, node1.less, node2.greater, tracker)
 */
      __pyx_t_8 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_greater_of(__pyx_v_tracker, 2, __pyx_v_node2); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2059, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":2060
 * 
 *                 tracker.push_greater_of(2, node2)
 *                 self.__sparse_distance_matrix_traverse(             # <<<<<<<<<<<<<<
 *                     other, results, node1.less, node2.greater, tracker)
 *                 tracker.pop()
 */
      __pyx_t_8 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___sparse_distance_matrix_traverse(__pyx_v_self, __pyx_v_other, __pyx_v_results, __pyx_v_node1->less, __pyx_v_node2->greater, __pyx_v_tracker); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2060, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":2062
 *                 self.__sparse_distance_matrix_traverse(
 *                     other, results, node1.less, node2.greater, tracker)
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 *                 tracker.pop()
 * 
 */
      __pyx_t_8 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2062, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":2063
 *                     other, results, node1.less, node2.greater, tracker)
 *                 tracker.pop()
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *                 tracker.push_greater_of(1, node1)
 */
      __pyx_t_8 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2063, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":2065
 *                 tracker.pop()
 * 
 *                 tracker.push_greater_of(1, node1)             # <<<<<<<<<<<<<<
 *                 if node1 != node2:
 *                     # Avoid traversing (node1.less, node2.greater) and
 */
      __pyx_t_8 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_greater_of(__pyx_v_tracker, 1, __pyx_v_node1); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2065, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":2066
 * 
 *                 tracker.push_greater_of(1, node1)
 *                 if node1 != node2:             # <<<<<<<<<<<<<<
 *                     # Avoid traversing (node1.less, node2.greater) and
 *                     # (node1.greater, node2.less) (it's the same node pair
 */
      __pyx_t_1 = ((__pyx_v_node1 != __pyx_v_node2) != 0);
      if (__pyx_t_1) {

        /* "pyart/map/ckdtree.pyx":2071
 *                     # twice over, which is the source of the complication in
 *                     # the original KDTree.sparse_distance_matrix)
 *                     tracker.push_less_of(2, node2)             # <<<<<<<<<<<<<<
 *                     self.__sparse_distance_matrix_traverse(
 *                         other, results, node1.greater, node2.less, tracker)
 */
        __pyx_t_8 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_less_of(__pyx_v_tracker, 2, __pyx_v_node2); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2071, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":2072
 *                     # the original KDTree.sparse_distance_matrix)
 *                     tracker.push_less_of(2, node2)
 *                     self.__sparse_distance_matrix_traverse(             # <<<<<<<<<<<<<<
 *                         other, results, node1.greater, node2.less, tracker)
 *                     tracker.pop()
 */
        __pyx_t_8 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___sparse_distance_matrix_traverse(__pyx_v_self, __pyx_v_other, __pyx_v_results, __pyx_v_node1->greater, __pyx_v_node2->less, __pyx_v_tracker); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2072, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":2074
 *                     self.__sparse_distance_matrix_traverse(
 *                         other, results, node1.greater, node2.less, tracker)
 *                     tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *                 tracker.push_greater_of(2, node2)
 */
        __pyx_t_8 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2074, __pyx_L1_error)

        /* "pyart/map/ckdtree.pyx":2066
 * 
 *                 tracker.push_greater_of(1, node1)
 *                 if node1 != node2:             # <<<<<<<<<<<<<<
 *                     # Avoid traversing (node1.less, node2.greater) and
 *                     # (node1.greater, node2.less) (it's the same node pair
 */
      }

      /* "pyart/map/ckdtree.pyx":2076
 *                     tracker.pop()
 * 
 *                 tracker.push_greater_of(2, node2)             # <<<<<<<<<<<<<<
 *                 self.__sparse_distance_matrix_traverse(
 *                     other, results, node1.greater, node2.greater, tracker)
 */
      __pyx_t_8 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_greater_of(__pyx_v_tracker, 2, __pyx_v_node2); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2076, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":2077
 * 
 *                 tracker.push_greater_of(2, node2)
 *                 self.__sparse_distance_matrix_traverse(             # <<<<<<<<<<<<<<
 *                     other, results, node1.greater, node2.greater, tracker)
 *                 tracker.pop()
 */
      __pyx_t_8 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___sparse_distance_matrix_traverse(__pyx_v_self, __pyx_v_other, __pyx_v_results, __pyx_v_node1->greater, __pyx_v_node2->greater, __pyx_v_tracker); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2077, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":2079
 *                 self.__sparse_distance_matrix_traverse(
 *                     other, results, node1.greater, node2.greater, tracker)
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 *                 tracker.pop()
 * 
 */
      __pyx_t_8 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2079, __pyx_L1_error)

      /* "pyart/map/ckdtree.pyx":2080
 *                     other, results, node1.greater, node2.greater, tracker)
 *                 tracker.pop()
 *                 tracker.pop()             # <<<<<<<<<<<<<<
 * 
 *         return 0
 */
      __pyx_t_8 = __pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop(__pyx_v_tracker); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 2080, __pyx_L1_error)
    }
    __pyx_L12:;
  }
  __pyx_L3:;

  /* "pyart/map/ckdtree.pyx":2082
 *                 tracker.pop()
 * 
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     def sparse_distance_matrix(cKDTree self, cKDTree other,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":1991
 *     # sparse_distance_matrix
 *     # ----------------------
 *     cdef int __sparse_distance_matrix_traverse(cKDTree self, cKDTree other,             # <<<<<<<<<<<<<<
 *                                                coo_entries results,
 *                                                innernode* node1, innernode* node2,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.__sparse_distance_matrix_traverse", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":2084
 *         return 0
 * 
 *     def sparse_distance_matrix(cKDTree self, cKDTree other,             # <<<<<<<<<<<<<<
 *                                np.float64_t max_distance,
 *                                np.float64_t p=2.):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_15sparse_distance_matrix(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5pyart_3map_7ckdtree_7cKDTree_14sparse_distance_matrix[] = "sparse_distance_matrix(self, max_distance, p)\n\n        Compute a sparse distance matrix\n\n        Computes a distance matrix between two KDTrees, leaving as zero\n        any distance greater than max_distance.\n\n        Parameters\n        ----------\n        other : cKDTree\n\n        max_distance : positive float\n\n        Returns\n        -------\n        result : dok_matrix\n            Sparse matrix representing the results in \"dictionary of keys\" format.\n            FIXME: Internally, built as a COO matrix, it would be more\n            efficient to return this COO matrix.\n\n        ";
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_15sparse_distance_matrix(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_other = 0;
  __pyx_t_5numpy_float64_t __pyx_v_max_distance;
  __pyx_t_5numpy_float64_t __pyx_v_p;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("sparse_distance_matrix (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_other,&__pyx_n_s_max_distance,&__pyx_n_s_p,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_other)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_max_distance)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("sparse_distance_matrix", 0, 2, 3, 1); __PYX_ERR(0, 2084, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_p);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "sparse_distance_matrix") < 0)) __PYX_ERR(0, 2084, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_other = ((struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *)values[0]);
    __pyx_v_max_distance = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_max_distance == ((npy_float64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2085, __pyx_L3_error)
    if (values[2]) {
      __pyx_v_p = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_p == ((npy_float64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2086, __pyx_L3_error)
    } else {
      __pyx_v_p = ((__pyx_t_5numpy_float64_t)2.);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("sparse_distance_matrix", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 2084, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.sparse_distance_matrix", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_5pyart_3map_7ckdtree_cKDTree, 1, "other", 0))) __PYX_ERR(0, 2084, __pyx_L1_error)
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_7cKDTree_14sparse_distance_matrix(((struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self), __pyx_v_other, __pyx_v_max_distance, __pyx_v_p);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_14sparse_distance_matrix(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_other, __pyx_t_5numpy_float64_t __pyx_v_max_distance, __pyx_t_5numpy_float64_t __pyx_v_p) {
  struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *__pyx_v_tracker = NULL;
  struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *__pyx_v_results = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("sparse_distance_matrix", 0);

  /* "pyart/map/ckdtree.pyx":2110
 * 
 *         # Make sure trees are compatible
 *         if self.m != other.m:             # <<<<<<<<<<<<<<
 *             raise ValueError("Trees passed to query_ball_trees have different dimensionality")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->m != __pyx_v_other->m) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "pyart/map/ckdtree.pyx":2111
 *         # Make sure trees are compatible
 *         if self.m != other.m:
 *             raise ValueError("Trees passed to query_ball_trees have different dimensionality")             # <<<<<<<<<<<<<<
 * 
 *         # Calculate mins and maxes to outer box
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__19, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2111, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 2111, __pyx_L1_error)

    /* "pyart/map/ckdtree.pyx":2110
 * 
 *         # Make sure trees are compatible
 *         if self.m != other.m:             # <<<<<<<<<<<<<<
 *             raise ValueError("Trees passed to query_ball_trees have different dimensionality")
 * 
 */
  }

  /* "pyart/map/ckdtree.pyx":2115
 *         # Calculate mins and maxes to outer box
 *         tracker = RectRectDistanceTracker(
 *             Rectangle(self.mins, self.maxes),             # <<<<<<<<<<<<<<
 *             Rectangle(other.mins, other.maxes),
 *             p, 0, max_distance)
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->mins));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->mins));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_self->mins));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->maxes));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->maxes));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self->maxes));
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pyart_3map_7ckdtree_Rectangle), __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyart/map/ckdtree.pyx":2116
 *         tracker = RectRectDistanceTracker(
 *             Rectangle(self.mins, self.maxes),
 *             Rectangle(other.mins, other.maxes),             # <<<<<<<<<<<<<<
 *             p, 0, max_distance)
 * 
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_other->mins));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_other->mins));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_other->mins));
  __Pyx_INCREF(((PyObject *)__pyx_v_other->maxes));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_other->maxes));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_other->maxes));
  __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pyart_3map_7ckdtree_Rectangle), __pyx_t_2, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyart/map/ckdtree.pyx":2117
 *             Rectangle(self.mins, self.maxes),
 *             Rectangle(other.mins, other.maxes),
 *             p, 0, max_distance)             # <<<<<<<<<<<<<<
 * 
 *         results = coo_entries()
 */
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_p); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = PyFloat_FromDouble(__pyx_v_max_distance); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);

  /* "pyart/map/ckdtree.pyx":2114
 * 
 *         # Calculate mins and maxes to outer box
 *         tracker = RectRectDistanceTracker(             # <<<<<<<<<<<<<<
 *             Rectangle(self.mins, self.maxes),
 *             Rectangle(other.mins, other.maxes),
 */
  __pyx_t_6 = PyTuple_New(5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2114, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_2);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_int_0);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_t_5);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_2 = 0;
  __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pyart_3map_7ckdtree_RectRectDistanceTracker), __pyx_t_6, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2114, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_v_tracker = ((struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "pyart/map/ckdtree.pyx":2119
 *             p, 0, max_distance)
 * 
 *         results = coo_entries()             # <<<<<<<<<<<<<<
 *         self.__sparse_distance_matrix_traverse(other, results,
 *                                                self.tree, other.tree,
 */
  __pyx_t_5 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_5pyart_3map_7ckdtree_coo_entries)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2119, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_v_results = ((struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "pyart/map/ckdtree.pyx":2120
 * 
 *         results = coo_entries()
 *         self.__sparse_distance_matrix_traverse(other, results,             # <<<<<<<<<<<<<<
 *                                                self.tree, other.tree,
 *                                                tracker)
 */
  __pyx_t_7 = ((struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self->__pyx_vtab)->__pyx___sparse_distance_matrix_traverse(__pyx_v_self, __pyx_v_other, __pyx_v_results, __pyx_v_self->tree, __pyx_v_other->tree, __pyx_v_tracker); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 2120, __pyx_L1_error)

  /* "pyart/map/ckdtree.pyx":2124
 *                                                tracker)
 * 
 *         return results.to_matrix(shape=(self.n, other.n)).todok()             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_results), __pyx_n_s_to_matrix); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->n); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_other->n); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_3);
  __pyx_t_4 = 0;
  __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_t_8) < 0) __PYX_ERR(0, 2124, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_todok); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_5 = (__pyx_t_8) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_8) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "pyart/map/ckdtree.pyx":2084
 *         return 0
 * 
 *     def sparse_distance_matrix(cKDTree self, cKDTree other,             # <<<<<<<<<<<<<<
 *                                np.float64_t max_distance,
 *                                np.float64_t p=2.):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.sparse_distance_matrix", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_tracker);
  __Pyx_XDECREF((PyObject *)__pyx_v_results);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":834
 * 
 *     cdef innernode* tree
 *     cdef readonly np.ndarray data             # <<<<<<<<<<<<<<
 *     cdef np.float64_t* raw_data
 *     cdef readonly np.intp_t n, m
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_4data_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_4data_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_7cKDTree_4data___get__(((struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_4data___get__(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->data));
  __pyx_r = ((PyObject *)__pyx_v_self->data);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":836
 *     cdef readonly np.ndarray data
 *     cdef np.float64_t* raw_data
 *     cdef readonly np.intp_t n, m             # <<<<<<<<<<<<<<
 *     cdef readonly np.intp_t leafsize
 *     cdef readonly np.ndarray maxes
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_1n_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_1n_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_7cKDTree_1n___get__(((struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_1n___get__(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->n); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 836, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.n.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_1m_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_1m_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_7cKDTree_1m___get__(((struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_1m___get__(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->m); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 836, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.m.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":837
 *     cdef np.float64_t* raw_data
 *     cdef readonly np.intp_t n, m
 *     cdef readonly np.intp_t leafsize             # <<<<<<<<<<<<<<
 *     cdef readonly np.ndarray maxes
 *     cdef np.float64_t* raw_maxes
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_8leafsize_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_8leafsize_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_7cKDTree_8leafsize___get__(((struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_8leafsize___get__(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->leafsize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 837, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.leafsize.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":838
 *     cdef readonly np.intp_t n, m
 *     cdef readonly np.intp_t leafsize
 *     cdef readonly np.ndarray maxes             # <<<<<<<<<<<<<<
 *     cdef np.float64_t* raw_maxes
 *     cdef readonly np.ndarray mins
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_5maxes_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_5maxes_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_7cKDTree_5maxes___get__(((struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_5maxes___get__(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->maxes));
  __pyx_r = ((PyObject *)__pyx_v_self->maxes);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/map/ckdtree.pyx":840
 *     cdef readonly np.ndarray maxes
 *     cdef np.float64_t* raw_maxes
 *     cdef readonly np.ndarray mins             # <<<<<<<<<<<<<<
 *     cdef np.float64_t* raw_mins
 *     cdef np.ndarray indices
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_4mins_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_4mins_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_7cKDTree_4mins___get__(((struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_4mins___get__(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->mins));
  __pyx_r = ((PyObject *)__pyx_v_self->mins);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.raw_data,self.raw_indices,self.raw_maxes,self.raw_mins,self.tree cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_17__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_17__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_7cKDTree_16__reduce_cython__(((struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_16__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.raw_data,self.raw_indices,self.raw_maxes,self.raw_mins,self.tree cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.raw_data,self.raw_indices,self.raw_maxes,self.raw_mins,self.tree cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__21, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.raw_data,self.raw_indices,self.raw_maxes,self.raw_mins,self.tree cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.raw_data,self.raw_indices,self.raw_maxes,self.raw_mins,self.tree cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.raw_data,self.raw_indices,self.raw_maxes,self.raw_mins,self.tree cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_19__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_19__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_3map_7ckdtree_7cKDTree_18__setstate_cython__(((struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_3map_7ckdtree_7cKDTree_18__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.raw_data,self.raw_indices,self.raw_maxes,self.raw_mins,self.tree cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.raw_data,self.raw_indices,self.raw_maxes,self.raw_mins,self.tree cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__22, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.raw_data,self.raw_indices,self.raw_maxes,self.raw_mins,self.tree cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.raw_data,self.raw_indices,self.raw_maxes,self.raw_mins,self.tree cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.map.ckdtree.cKDTree.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":735
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":736
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 736, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":735
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":738
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":739
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 739, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":738
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":741
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":742
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 742, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":741
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":744
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":745
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 745, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":744
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":747
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":748
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 748, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":747
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":750
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyDataType_SHAPE(PyArray_Descr *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("PyDataType_SHAPE", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":751
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  __pyx_t_1 = (PyDataType_HASSUBARRAY(__pyx_v_d) != 0);
  if (__pyx_t_1) {

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":752
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape             # <<<<<<<<<<<<<<
 *     else:
 *         return ()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject*)__pyx_v_d->subarray->shape));
    __pyx_r = ((PyObject*)__pyx_v_d->subarray->shape);
    goto __pyx_L0;

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":751
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  }

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":754
 *         return <tuple>d.subarray.shape
 *     else:
 *         return ()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_empty_tuple);
    __pyx_r = __pyx_empty_tuple;
    goto __pyx_L0;
  }

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":750
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":929
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_array_base", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":930
 * 
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!             # <<<<<<<<<<<<<<
 *     PyArray_SetBaseObject(arr, base)
 * 
 */
  Py_INCREF(__pyx_v_base);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":931
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
 */
  (void)(PyArray_SetBaseObject(__pyx_v_arr, __pyx_v_base));

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":929
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":933
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_v_base;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_array_base", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":934
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)             # <<<<<<<<<<<<<<
 *     if base is NULL:
 *         return None
 */
  __pyx_v_base = PyArray_BASE(__pyx_v_arr);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":935
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  __pyx_t_1 = ((__pyx_v_base == NULL) != 0);
  if (__pyx_t_1) {

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":936
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     return <object>base
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":935
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  }

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":937
 *     if base is NULL:
 *         return None
 *     return <object>base             # <<<<<<<<<<<<<<
 * 
 * # Versions of the import_* functions which are more suitable for
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_base));
  __pyx_r = ((PyObject *)__pyx_v_base);
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":933
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":941
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_array(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_array", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":942
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":943
 * cdef inline int import_array() except -1:
 *     try:
 *         __pyx_import_array()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")
 */
      __pyx_t_4 = _import_array(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 943, __pyx_L3_error)

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":942
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":944
 *     try:
 *         __pyx_import_array()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 944, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":945
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__23, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 945, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 945, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":942
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":941
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":947
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_umath(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_umath", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":948
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":949
 * cdef inline int import_umath() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 949, __pyx_L3_error)

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":948
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":950
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 950, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":951
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__24, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 951, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 951, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":948
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":947
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":953
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_ufunc(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_ufunc", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":954
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":955
 * cdef inline int import_ufunc() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 955, __pyx_L3_error)

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":954
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":956
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 956, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":957
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef extern from *:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__24, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 957, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 957, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":954
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":953
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":967
 * 
 * 
 * cdef inline bint is_timedelta64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.timedelta64)`
 */

static CYTHON_INLINE int __pyx_f_5numpy_is_timedelta64_object(PyObject *__pyx_v_obj) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_timedelta64_object", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":979
 *     bool
 *     """
 *     return PyObject_TypeCheck(obj, &PyTimedeltaArrType_Type)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = PyObject_TypeCheck(__pyx_v_obj, (&PyTimedeltaArrType_Type));
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":967
 * 
 * 
 * cdef inline bint is_timedelta64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.timedelta64)`
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":982
 * 
 * 
 * cdef inline bint is_datetime64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.datetime64)`
 */

static CYTHON_INLINE int __pyx_f_5numpy_is_datetime64_object(PyObject *__pyx_v_obj) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_datetime64_object", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":994
 *     bool
 *     """
 *     return PyObject_TypeCheck(obj, &PyDatetimeArrType_Type)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = PyObject_TypeCheck(__pyx_v_obj, (&PyDatetimeArrType_Type));
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":982
 * 
 * 
 * cdef inline bint is_datetime64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.datetime64)`
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":997
 * 
 * 
 * cdef inline npy_datetime get_datetime64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy datetime64 object
 */

static CYTHON_INLINE npy_datetime __pyx_f_5numpy_get_datetime64_value(PyObject *__pyx_v_obj) {
  npy_datetime __pyx_r;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":1004
 *     also needed.  That can be found using `get_datetime64_unit`.
 *     """
 *     return (<PyDatetimeScalarObject*>obj).obval             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((PyDatetimeScalarObject *)__pyx_v_obj)->obval;
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":997
 * 
 * 
 * cdef inline npy_datetime get_datetime64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy datetime64 object
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":1007
 * 
 * 
 * cdef inline npy_timedelta get_timedelta64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy timedelta64 object
 */

static CYTHON_INLINE npy_timedelta __pyx_f_5numpy_get_timedelta64_value(PyObject *__pyx_v_obj) {
  npy_timedelta __pyx_r;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":1011
 *     returns the int64 value underlying scalar numpy timedelta64 object
 *     """
 *     return (<PyTimedeltaScalarObject*>obj).obval             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((PyTimedeltaScalarObject *)__pyx_v_obj)->obval;
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":1007
 * 
 * 
 * cdef inline npy_timedelta get_timedelta64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy timedelta64 object
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":1014
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
 */

static CYTHON_INLINE NPY_DATETIMEUNIT __pyx_f_5numpy_get_datetime64_unit(PyObject *__pyx_v_obj) {
  NPY_DATETIMEUNIT __pyx_r;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":1018
 *     returns the unit part of the dtype for a numpy datetime64 object.
 *     """
 *     return <NPY_DATETIMEUNIT>(<PyDatetimeScalarObject*>obj).obmeta.base             # <<<<<<<<<<<<<<
 */
  __pyx_r = ((NPY_DATETIMEUNIT)((PyDatetimeScalarObject *)__pyx_v_obj)->obmeta.base);
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":1014
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}
static struct __pyx_vtabstruct_5pyart_3map_7ckdtree_heap __pyx_vtable_5pyart_3map_7ckdtree_heap;

static PyObject *__pyx_tp_new_5pyart_3map_7ckdtree_heap(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5pyart_3map_7ckdtree_heap *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pyart_3map_7ckdtree_heap *)o);
  p->__pyx_vtab = __pyx_vtabptr_5pyart_3map_7ckdtree_heap;
  return o;
}

static void __pyx_tp_dealloc_5pyart_3map_7ckdtree_heap(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_5pyart_3map_7ckdtree_4heap_3__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_5pyart_3map_7ckdtree_heap[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5pyart_3map_7ckdtree_4heap_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5pyart_3map_7ckdtree_4heap_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pyart_3map_7ckdtree_heap = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyart.map.ckdtree.heap", /*tp_name*/
  sizeof(struct __pyx_obj_5pyart_3map_7ckdtree_heap), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pyart_3map_7ckdtree_heap, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pyart_3map_7ckdtree_heap, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5pyart_3map_7ckdtree_4heap_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pyart_3map_7ckdtree_heap, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_5pyart_3map_7ckdtree_coo_entries __pyx_vtable_5pyart_3map_7ckdtree_coo_entries;

static PyObject *__pyx_tp_new_5pyart_3map_7ckdtree_coo_entries(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *)o);
  p->__pyx_vtab = __pyx_vtabptr_5pyart_3map_7ckdtree_coo_entries;
  p->i = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  p->j = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  p->v = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5pyart_3map_7ckdtree_coo_entries(PyObject *o) {
  struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *p = (struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->i);
  Py_CLEAR(p->j);
  Py_CLEAR(p->v);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_5pyart_3map_7ckdtree_coo_entries(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *p = (struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *)o;
  if (p->i) {
    e = (*v)(((PyObject *)p->i), a); if (e) return e;
  }
  if (p->j) {
    e = (*v)(((PyObject *)p->j), a); if (e) return e;
  }
  if (p->v) {
    e = (*v)(((PyObject *)p->v), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5pyart_3map_7ckdtree_coo_entries(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *p = (struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *)o;
  tmp = ((PyObject*)p->i);
  p->i = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->j);
  p->j = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->v);
  p->v = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_5pyart_3map_7ckdtree_coo_entries[] = {
  {"to_matrix", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5pyart_3map_7ckdtree_11coo_entries_3to_matrix, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5pyart_3map_7ckdtree_11coo_entries_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5pyart_3map_7ckdtree_11coo_entries_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pyart_3map_7ckdtree_coo_entries = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyart.map.ckdtree.coo_entries", /*tp_name*/
  sizeof(struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pyart_3map_7ckdtree_coo_entries, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5pyart_3map_7ckdtree_coo_entries, /*tp_traverse*/
  __pyx_tp_clear_5pyart_3map_7ckdtree_coo_entries, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pyart_3map_7ckdtree_coo_entries, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5pyart_3map_7ckdtree_11coo_entries_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pyart_3map_7ckdtree_coo_entries, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};

static PyObject *__pyx_tp_new_5pyart_3map_7ckdtree_Rectangle(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)o);
  p->mins_arr = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  p->maxes_arr = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5pyart_3map_7ckdtree_Rectangle(PyObject *o) {
  struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *p = (struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->mins_arr);
  Py_CLEAR(p->maxes_arr);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_5pyart_3map_7ckdtree_Rectangle(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *p = (struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)o;
  if (p->mins_arr) {
    e = (*v)(((PyObject *)p->mins_arr), a); if (e) return e;
  }
  if (p->maxes_arr) {
    e = (*v)(((PyObject *)p->maxes_arr), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5pyart_3map_7ckdtree_Rectangle(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *p = (struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)o;
  tmp = ((PyObject*)p->mins_arr);
  p->mins_arr = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->maxes_arr);
  p->maxes_arr = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_5pyart_3map_7ckdtree_Rectangle[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5pyart_3map_7ckdtree_9Rectangle_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5pyart_3map_7ckdtree_9Rectangle_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pyart_3map_7ckdtree_Rectangle = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyart.map.ckdtree.Rectangle", /*tp_name*/
  sizeof(struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pyart_3map_7ckdtree_Rectangle, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5pyart_3map_7ckdtree_Rectangle, /*tp_traverse*/
  __pyx_tp_clear_5pyart_3map_7ckdtree_Rectangle, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pyart_3map_7ckdtree_Rectangle, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5pyart_3map_7ckdtree_9Rectangle_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pyart_3map_7ckdtree_Rectangle, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_5pyart_3map_7ckdtree_RectRectDistanceTracker __pyx_vtable_5pyart_3map_7ckdtree_RectRectDistanceTracker;

static PyObject *__pyx_tp_new_5pyart_3map_7ckdtree_RectRectDistanceTracker(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *)o);
  p->__pyx_vtab = __pyx_vtabptr_5pyart_3map_7ckdtree_RectRectDistanceTracker;
  p->rect1 = ((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)Py_None); Py_INCREF(Py_None);
  p->rect2 = ((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5pyart_3map_7ckdtree_RectRectDistanceTracker(PyObject *o) {
  struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *p = (struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_5pyart_3map_7ckdtree_23RectRectDistanceTracker_3__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->rect1);
  Py_CLEAR(p->rect2);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_5pyart_3map_7ckdtree_RectRectDistanceTracker(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *p = (struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *)o;
  if (p->rect1) {
    e = (*v)(((PyObject *)p->rect1), a); if (e) return e;
  }
  if (p->rect2) {
    e = (*v)(((PyObject *)p->rect2), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5pyart_3map_7ckdtree_RectRectDistanceTracker(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *p = (struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *)o;
  tmp = ((PyObject*)p->rect1);
  p->rect1 = ((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->rect2);
  p->rect2 = ((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_5pyart_3map_7ckdtree_RectRectDistanceTracker[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5pyart_3map_7ckdtree_23RectRectDistanceTracker_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5pyart_3map_7ckdtree_23RectRectDistanceTracker_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pyart_3map_7ckdtree_RectRectDistanceTracker = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyart.map.ckdtree.RectRectDistanceTracker", /*tp_name*/
  sizeof(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pyart_3map_7ckdtree_RectRectDistanceTracker, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5pyart_3map_7ckdtree_RectRectDistanceTracker, /*tp_traverse*/
  __pyx_tp_clear_5pyart_3map_7ckdtree_RectRectDistanceTracker, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pyart_3map_7ckdtree_RectRectDistanceTracker, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5pyart_3map_7ckdtree_23RectRectDistanceTracker_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pyart_3map_7ckdtree_RectRectDistanceTracker, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_5pyart_3map_7ckdtree_PointRectDistanceTracker __pyx_vtable_5pyart_3map_7ckdtree_PointRectDistanceTracker;

static PyObject *__pyx_tp_new_5pyart_3map_7ckdtree_PointRectDistanceTracker(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *)o);
  p->__pyx_vtab = __pyx_vtabptr_5pyart_3map_7ckdtree_PointRectDistanceTracker;
  p->rect = ((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5pyart_3map_7ckdtree_PointRectDistanceTracker(PyObject *o) {
  struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *p = (struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_5pyart_3map_7ckdtree_24PointRectDistanceTracker_1__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->rect);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_5pyart_3map_7ckdtree_PointRectDistanceTracker(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *p = (struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *)o;
  if (p->rect) {
    e = (*v)(((PyObject *)p->rect), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5pyart_3map_7ckdtree_PointRectDistanceTracker(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *p = (struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *)o;
  tmp = ((PyObject*)p->rect);
  p->rect = ((struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_5pyart_3map_7ckdtree_PointRectDistanceTracker[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5pyart_3map_7ckdtree_24PointRectDistanceTracker_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5pyart_3map_7ckdtree_24PointRectDistanceTracker_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pyart_3map_7ckdtree_PointRectDistanceTracker = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyart.map.ckdtree.PointRectDistanceTracker", /*tp_name*/
  sizeof(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pyart_3map_7ckdtree_PointRectDistanceTracker, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5pyart_3map_7ckdtree_PointRectDistanceTracker, /*tp_traverse*/
  __pyx_tp_clear_5pyart_3map_7ckdtree_PointRectDistanceTracker, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pyart_3map_7ckdtree_PointRectDistanceTracker, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pyart_3map_7ckdtree_PointRectDistanceTracker, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_5pyart_3map_7ckdtree_cKDTree __pyx_vtable_5pyart_3map_7ckdtree_cKDTree;

static PyObject *__pyx_tp_new_5pyart_3map_7ckdtree_cKDTree(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *)o);
  p->__pyx_vtab = __pyx_vtabptr_5pyart_3map_7ckdtree_cKDTree;
  p->data = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  p->maxes = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  p->mins = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  p->indices = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5pyart_3map_7ckdtree_cKDTree(PyObject *o) {
  struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *p = (struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_5pyart_3map_7ckdtree_7cKDTree_3__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->data);
  Py_CLEAR(p->maxes);
  Py_CLEAR(p->mins);
  Py_CLEAR(p->indices);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_5pyart_3map_7ckdtree_cKDTree(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *p = (struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *)o;
  if (p->data) {
    e = (*v)(((PyObject *)p->data), a); if (e) return e;
  }
  if (p->maxes) {
    e = (*v)(((PyObject *)p->maxes), a); if (e) return e;
  }
  if (p->mins) {
    e = (*v)(((PyObject *)p->mins), a); if (e) return e;
  }
  if (p->indices) {
    e = (*v)(((PyObject *)p->indices), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5pyart_3map_7ckdtree_cKDTree(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *p = (struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *)o;
  tmp = ((PyObject*)p->data);
  p->data = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->maxes);
  p->maxes = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->mins);
  p->mins = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->indices);
  p->indices = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_5pyart_3map_7ckdtree_7cKDTree_data(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_3map_7ckdtree_7cKDTree_4data_1__get__(o);
}

static PyObject *__pyx_getprop_5pyart_3map_7ckdtree_7cKDTree_n(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_3map_7ckdtree_7cKDTree_1n_1__get__(o);
}

static PyObject *__pyx_getprop_5pyart_3map_7ckdtree_7cKDTree_m(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_3map_7ckdtree_7cKDTree_1m_1__get__(o);
}

static PyObject *__pyx_getprop_5pyart_3map_7ckdtree_7cKDTree_leafsize(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_3map_7ckdtree_7cKDTree_8leafsize_1__get__(o);
}

static PyObject *__pyx_getprop_5pyart_3map_7ckdtree_7cKDTree_maxes(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_3map_7ckdtree_7cKDTree_5maxes_1__get__(o);
}

static PyObject *__pyx_getprop_5pyart_3map_7ckdtree_7cKDTree_mins(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_3map_7ckdtree_7cKDTree_4mins_1__get__(o);
}

static PyMethodDef __pyx_methods_5pyart_3map_7ckdtree_cKDTree[] = {
  {"query", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_5query, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5pyart_3map_7ckdtree_7cKDTree_4query},
  {"query_ball_point", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_7query_ball_point, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5pyart_3map_7ckdtree_7cKDTree_6query_ball_point},
  {"query_ball_tree", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_9query_ball_tree, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5pyart_3map_7ckdtree_7cKDTree_8query_ball_tree},
  {"query_pairs", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_11query_pairs, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5pyart_3map_7ckdtree_7cKDTree_10query_pairs},
  {"count_neighbors", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_13count_neighbors, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5pyart_3map_7ckdtree_7cKDTree_12count_neighbors},
  {"sparse_distance_matrix", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_15sparse_distance_matrix, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5pyart_3map_7ckdtree_7cKDTree_14sparse_distance_matrix},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_17__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5pyart_3map_7ckdtree_7cKDTree_19__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5pyart_3map_7ckdtree_cKDTree[] = {
  {(char *)"data", __pyx_getprop_5pyart_3map_7ckdtree_7cKDTree_data, 0, (char *)0, 0},
  {(char *)"n", __pyx_getprop_5pyart_3map_7ckdtree_7cKDTree_n, 0, (char *)0, 0},
  {(char *)"m", __pyx_getprop_5pyart_3map_7ckdtree_7cKDTree_m, 0, (char *)0, 0},
  {(char *)"leafsize", __pyx_getprop_5pyart_3map_7ckdtree_7cKDTree_leafsize, 0, (char *)0, 0},
  {(char *)"maxes", __pyx_getprop_5pyart_3map_7ckdtree_7cKDTree_maxes, 0, (char *)0, 0},
  {(char *)"mins", __pyx_getprop_5pyart_3map_7ckdtree_7cKDTree_mins, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pyart_3map_7ckdtree_cKDTree = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyart.map.ckdtree.cKDTree", /*tp_name*/
  sizeof(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pyart_3map_7ckdtree_cKDTree, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    cKDTree(data, int leafsize=10)\n\n    kd-tree for quick nearest-neighbor lookup\n\n    This class provides an index into a set of k-dimensional points\n    which can be used to rapidly look up the nearest neighbors of any\n    point. \n\n    The algorithm used is described in Maneewongvatana and Mount 1999. \n    The general idea is that the kd-tree is a binary trie, each of whose\n    nodes represents an axis-aligned hyperrectangle. Each node specifies\n    an axis and splits the set of points based on whether their coordinate\n    along that axis is greater than or less than a particular value. \n\n    During construction, the axis and splitting point are chosen by the \n    \"sliding midpoint\" rule, which ensures that the cells do not all\n    become long and thin. \n\n    The tree can be queried for the r closest neighbors of any given point \n    (optionally returning only those within some maximum distance of the \n    point). It can also be queried, with a substantial gain in efficiency, \n    for the r approximate closest neighbors.\n\n    For large dimensions (20 is already large) do not expect this to run \n    significantly faster than brute force. High-dimensional nearest-neighbor\n    queries are a substantial open problem in computer science.\n\n    Parameters\n    ----------\n    data : array-like, shape (n,m)\n        The n data points of dimension mto be indexed. This array is \n        not copied unless this is necessary to produce a contiguous \n        array of doubles, and so modifying this data will result in \n        bogus results.\n    leafsize : positive integer\n        The number of points at which the algorithm switches over to\n        brute-force.\n\n    ", /*tp_doc*/
  __pyx_tp_traverse_5pyart_3map_7ckdtree_cKDTree, /*tp_traverse*/
  __pyx_tp_clear_5pyart_3map_7ckdtree_cKDTree, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pyart_3map_7ckdtree_cKDTree, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5pyart_3map_7ckdtree_cKDTree, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5pyart_3map_7ckdtree_7cKDTree_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pyart_3map_7ckdtree_cKDTree, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_ckdtree(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_ckdtree},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "ckdtree",
    __pyx_k_Copyright_c_2001_2002_Enthought, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_u_C, __pyx_k_C, sizeof(__pyx_k_C), 0, 1, 0, 1},
  {&__pyx_kp_u_Heap_containing, __pyx_k_Heap_containing, sizeof(__pyx_k_Heap_containing), 0, 1, 0, 0},
  {&__pyx_n_s_ImportError, __pyx_k_ImportError, sizeof(__pyx_k_ImportError), 0, 0, 1, 1},
  {&__pyx_n_s_MemoryError, __pyx_k_MemoryError, sizeof(__pyx_k_MemoryError), 0, 0, 1, 1},
  {&__pyx_kp_u_Only_p_norms_with_1_p_infinity_p, __pyx_k_Only_p_norms_with_1_p_infinity_p, sizeof(__pyx_k_Only_p_norms_with_1_p_infinity_p), 0, 1, 0, 0},
  {&__pyx_n_s_PointRectDistanceTracker, __pyx_k_PointRectDistanceTracker, sizeof(__pyx_k_PointRectDistanceTracker), 0, 0, 1, 1},
  {&__pyx_n_s_RectRectDistanceTracker, __pyx_k_RectRectDistanceTracker, sizeof(__pyx_k_RectRectDistanceTracker), 0, 0, 1, 1},
  {&__pyx_n_s_Rectangle, __pyx_k_Rectangle, sizeof(__pyx_k_Rectangle), 0, 0, 1, 1},
  {&__pyx_kp_u_Searching_for_a, __pyx_k_Searching_for_a, sizeof(__pyx_k_Searching_for_a), 0, 1, 0, 0},
  {&__pyx_kp_u_Trees_passed_to_query_ball_trees, __pyx_k_Trees_passed_to_query_ball_trees, sizeof(__pyx_k_Trees_passed_to_query_ball_trees), 0, 1, 0, 0},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_n_s_all, __pyx_k_all, sizeof(__pyx_k_all), 0, 0, 1, 1},
  {&__pyx_n_s_amax, __pyx_k_amax, sizeof(__pyx_k_amax), 0, 0, 1, 1},
  {&__pyx_n_s_amin, __pyx_k_amin, sizeof(__pyx_k_amin), 0, 0, 1, 1},
  {&__pyx_n_s_arange, __pyx_k_arange, sizeof(__pyx_k_arange), 0, 0, 1, 1},
  {&__pyx_n_s_array, __pyx_k_array, sizeof(__pyx_k_array), 0, 0, 1, 1},
  {&__pyx_n_s_asarray, __pyx_k_asarray, sizeof(__pyx_k_asarray), 0, 0, 1, 1},
  {&__pyx_n_s_ascontiguousarray, __pyx_k_ascontiguousarray, sizeof(__pyx_k_ascontiguousarray), 0, 0, 1, 1},
  {&__pyx_n_s_astype, __pyx_k_astype, sizeof(__pyx_k_astype), 0, 0, 1, 1},
  {&__pyx_n_s_axis, __pyx_k_axis, sizeof(__pyx_k_axis), 0, 0, 1, 1},
  {&__pyx_kp_u_but_hasshape, __pyx_k_but_hasshape, sizeof(__pyx_k_but_hasshape), 0, 1, 0, 0},
  {&__pyx_n_s_cKDTree, __pyx_k_cKDTree, sizeof(__pyx_k_cKDTree), 0, 0, 1, 1},
  {&__pyx_n_u_cKDTree, __pyx_k_cKDTree, sizeof(__pyx_k_cKDTree), 0, 1, 0, 1},
  {&__pyx_kp_u_cKDTree_query_ball_point_line_13, __pyx_k_cKDTree_query_ball_point_line_13, sizeof(__pyx_k_cKDTree_query_ball_point_line_13), 0, 1, 0, 0},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_coo_entries, __pyx_k_coo_entries, sizeof(__pyx_k_coo_entries), 0, 0, 1, 1},
  {&__pyx_n_s_coo_matrix, __pyx_k_coo_matrix, sizeof(__pyx_k_coo_matrix), 0, 0, 1, 1},
  {&__pyx_n_u_d, __pyx_k_d, sizeof(__pyx_k_d), 0, 1, 0, 1},
  {&__pyx_n_s_data, __pyx_k_data, sizeof(__pyx_k_data), 0, 0, 1, 1},
  {&__pyx_kp_u_dimensional_KDTree, __pyx_k_dimensional_KDTree, sizeof(__pyx_k_dimensional_KDTree), 0, 1, 0, 0},
  {&__pyx_kp_u_dimensional_point_in_a, __pyx_k_dimensional_point_in_a, sizeof(__pyx_k_dimensional_point_in_a), 0, 1, 0, 0},
  {&__pyx_n_s_distance_upper_bound, __pyx_k_distance_upper_bound, sizeof(__pyx_k_distance_upper_bound), 0, 0, 1, 1},
  {&__pyx_n_s_dtype, __pyx_k_dtype, sizeof(__pyx_k_dtype), 0, 0, 1, 1},
  {&__pyx_n_s_empty, __pyx_k_empty, sizeof(__pyx_k_empty), 0, 0, 1, 1},
  {&__pyx_n_s_eps, __pyx_k_eps, sizeof(__pyx_k_eps), 0, 0, 1, 1},
  {&__pyx_n_s_fill, __pyx_k_fill, sizeof(__pyx_k_fill), 0, 0, 1, 1},
  {&__pyx_n_s_float64, __pyx_k_float64, sizeof(__pyx_k_float64), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_heap, __pyx_k_heap, sizeof(__pyx_k_heap), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_inf, __pyx_k_inf, sizeof(__pyx_k_inf), 0, 0, 1, 1},
  {&__pyx_n_s_initial_size, __pyx_k_initial_size, sizeof(__pyx_k_initial_size), 0, 0, 1, 1},
  {&__pyx_n_s_intp, __pyx_k_intp, sizeof(__pyx_k_intp), 0, 0, 1, 1},
  {&__pyx_kp_u_items_cannot_be_resized_to, __pyx_k_items_cannot_be_resized_to, sizeof(__pyx_k_items_cannot_be_resized_to), 0, 1, 0, 0},
  {&__pyx_n_s_k, __pyx_k_k, sizeof(__pyx_k_k), 0, 0, 1, 1},
  {&__pyx_n_s_leafsize, __pyx_k_leafsize, sizeof(__pyx_k_leafsize), 0, 0, 1, 1},
  {&__pyx_kp_u_leafsize_must_be_at_least_1, __pyx_k_leafsize_must_be_at_least_1, sizeof(__pyx_k_leafsize_must_be_at_least_1), 0, 1, 0, 0},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_max_distance, __pyx_k_max_distance, sizeof(__pyx_k_max_distance), 0, 0, 1, 1},
  {&__pyx_n_s_maxes_arr, __pyx_k_maxes_arr, sizeof(__pyx_k_maxes_arr), 0, 0, 1, 1},
  {&__pyx_n_s_mins_arr, __pyx_k_mins_arr, sizeof(__pyx_k_mins_arr), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_ndindex, __pyx_k_ndindex, sizeof(__pyx_k_ndindex), 0, 0, 1, 1},
  {&__pyx_n_s_newaxis, __pyx_k_newaxis, sizeof(__pyx_k_newaxis), 0, 0, 1, 1},
  {&__pyx_n_s_np, __pyx_k_np, sizeof(__pyx_k_np), 0, 0, 1, 1},
  {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
  {&__pyx_kp_u_numpy_core_multiarray_failed_to, __pyx_k_numpy_core_multiarray_failed_to, sizeof(__pyx_k_numpy_core_multiarray_failed_to), 0, 1, 0, 0},
  {&__pyx_kp_u_numpy_core_umath_failed_to_impor, __pyx_k_numpy_core_umath_failed_to_impor, sizeof(__pyx_k_numpy_core_umath_failed_to_impor), 0, 1, 0, 0},
  {&__pyx_n_s_object, __pyx_k_object, sizeof(__pyx_k_object), 0, 0, 1, 1},
  {&__pyx_n_s_order, __pyx_k_order, sizeof(__pyx_k_order), 0, 0, 1, 1},
  {&__pyx_n_s_other, __pyx_k_other, sizeof(__pyx_k_other), 0, 0, 1, 1},
  {&__pyx_n_s_p, __pyx_k_p, sizeof(__pyx_k_p), 0, 0, 1, 1},
  {&__pyx_n_s_prod, __pyx_k_prod, sizeof(__pyx_k_prod), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_kp_u_query_ball_point_self_x_r_p_eps, __pyx_k_query_ball_point_self_x_r_p_eps, sizeof(__pyx_k_query_ball_point_self_x_r_p_eps), 0, 1, 0, 0},
  {&__pyx_n_s_r, __pyx_k_r, sizeof(__pyx_k_r), 0, 0, 1, 1},
  {&__pyx_kp_u_r_must_be_either_a_single_value, __pyx_k_r_must_be_either_a_single_value, sizeof(__pyx_k_r_must_be_either_a_single_value), 0, 1, 0, 0},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_rect1, __pyx_k_rect1, sizeof(__pyx_k_rect1), 0, 0, 1, 1},
  {&__pyx_kp_u_rect1_and_rect2_have_different_d, __pyx_k_rect1_and_rect2_have_different_d, sizeof(__pyx_k_rect1_and_rect2_have_different_d), 0, 1, 0, 0},
  {&__pyx_n_s_rect2, __pyx_k_rect2, sizeof(__pyx_k_rect2), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_reshape, __pyx_k_reshape, sizeof(__pyx_k_reshape), 0, 0, 1, 1},
  {&__pyx_n_s_resize, __pyx_k_resize, sizeof(__pyx_k_resize), 0, 0, 1, 1},
  {&__pyx_n_s_scipy, __pyx_k_scipy, sizeof(__pyx_k_scipy), 0, 0, 1, 1},
  {&__pyx_n_s_scipy_sparse, __pyx_k_scipy_sparse, sizeof(__pyx_k_scipy_sparse), 0, 0, 1, 1},
  {&__pyx_kp_s_self_heap_cannot_be_converted_to, __pyx_k_self_heap_cannot_be_converted_to, sizeof(__pyx_k_self_heap_cannot_be_converted_to), 0, 0, 1, 0},
  {&__pyx_kp_s_self_i_data_self_j_data_self_v_d, __pyx_k_self_i_data_self_j_data_self_v_d, sizeof(__pyx_k_self_i_data_self_j_data_self_v_d), 0, 0, 1, 0},
  {&__pyx_kp_s_self_maxes_self_mins_cannot_be_c, __pyx_k_self_maxes_self_mins_cannot_be_c, sizeof(__pyx_k_self_maxes_self_mins_cannot_be_c), 0, 0, 1, 0},
  {&__pyx_kp_s_self_pt_self_stack_cannot_be_con, __pyx_k_self_pt_self_stack_cannot_be_con, sizeof(__pyx_k_self_pt_self_stack_cannot_be_con), 0, 0, 1, 0},
  {&__pyx_kp_s_self_raw_data_self_raw_indices_s, __pyx_k_self_raw_data_self_raw_indices_s, sizeof(__pyx_k_self_raw_data_self_raw_indices_s), 0, 0, 1, 0},
  {&__pyx_kp_s_self_stack_cannot_be_converted_t, __pyx_k_self_stack_cannot_be_converted_t, sizeof(__pyx_k_self_stack_cannot_be_converted_t), 0, 0, 1, 0},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_shape, __pyx_k_shape, sizeof(__pyx_k_shape), 0, 0, 1, 1},
  {&__pyx_n_s_sparse, __pyx_k_sparse, sizeof(__pyx_k_sparse), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_to_matrix, __pyx_k_to_matrix, sizeof(__pyx_k_to_matrix), 0, 0, 1, 1},
  {&__pyx_n_s_todok, __pyx_k_todok, sizeof(__pyx_k_todok), 0, 0, 1, 1},
  {&__pyx_n_s_upper_bound, __pyx_k_upper_bound, sizeof(__pyx_k_upper_bound), 0, 0, 1, 1},
  {&__pyx_n_s_x, __pyx_k_x, sizeof(__pyx_k_x), 0, 0, 1, 1},
  {&__pyx_kp_u_x_must_consist_of_vectors_of_len, __pyx_k_x_must_consist_of_vectors_of_len, sizeof(__pyx_k_x_must_consist_of_vectors_of_len), 0, 1, 0, 0},
  {&__pyx_n_s_zeros, __pyx_k_zeros, sizeof(__pyx_k_zeros), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_MemoryError = __Pyx_GetBuiltinName(__pyx_n_s_MemoryError); if (!__pyx_builtin_MemoryError) __PYX_ERR(0, 143, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 154, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(1, 2, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 300, __pyx_L1_error)
  __pyx_builtin_ImportError = __Pyx_GetBuiltinName(__pyx_n_s_ImportError); if (!__pyx_builtin_ImportError) __PYX_ERR(2, 945, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.heap cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.heap cannot be converted to a Python object for pickling")
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_s_self_heap_cannot_be_converted_to); if (unlikely(!__pyx_tuple_)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "(tree fragment)":4
 *     raise TypeError("self.heap cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.heap cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_s_self_heap_cannot_be_converted_to); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.i_data,self.j_data,self.v_data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.i_data,self.j_data,self.v_data cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_s_self_i_data_self_j_data_self_v_d); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "(tree fragment)":4
 *     raise TypeError("self.i_data,self.j_data,self.v_data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.i_data,self.j_data,self.v_data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_s_self_i_data_self_j_data_self_v_d); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.maxes,self.mins cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.maxes,self.mins cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_s_self_maxes_self_mins_cannot_be_c); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "(tree fragment)":4
 *     raise TypeError("self.maxes,self.mins cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.maxes,self.mins cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_s_self_maxes_self_mins_cannot_be_c); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "pyart/map/ckdtree.pyx":497
 * 
 *         if rect1.m != rect2.m:
 *             raise ValueError("rect1 and rect2 have different dimensions")             # <<<<<<<<<<<<<<
 * 
 *         self.rect1 = rect1
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_kp_u_rect1_and_rect2_have_different_d); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(0, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.stack cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.stack cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_kp_s_self_stack_cannot_be_converted_t); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "(tree fragment)":4
 *     raise TypeError("self.stack cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.stack cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_kp_s_self_stack_cannot_be_converted_t); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.pt,self.stack cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.pt,self.stack cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_kp_s_self_pt_self_stack_cannot_be_con); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "(tree fragment)":4
 *     raise TypeError("self.pt,self.stack cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.pt,self.stack cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_kp_s_self_pt_self_stack_cannot_be_con); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "pyart/map/ckdtree.pyx":850
 *         self.leafsize = leafsize
 *         if self.leafsize<1:
 *             raise ValueError("leafsize must be at least 1")             # <<<<<<<<<<<<<<
 *         self.maxes = np.ascontiguousarray(np.amax(self.data,axis=0), dtype=np.float64)
 *         self.mins = np.ascontiguousarray(np.amin(self.data,axis=0), dtype=np.float64)
 */
  __pyx_tuple__12 = PyTuple_Pack(1, __pyx_kp_u_leafsize_must_be_at_least_1); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(0, 850, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "pyart/map/ckdtree.pyx":1028
 *         #  distances between the nearest side of the cell and the target
 *         #  the head node of the cell
 *         q = heap(12)             # <<<<<<<<<<<<<<
 * 
 *         # priority queue for the nearest neighbors
 */
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_int_12); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(0, 1028, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "pyart/map/ckdtree.pyx":1254
 *                              "shape %s" % (int(self.m), np.shape(x)))
 *         if p < 1:
 *             raise ValueError("Only p-norms with 1<=p<=infinity permitted")             # <<<<<<<<<<<<<<
 *         if len(x.shape)==1:
 *             single = True
 */
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_kp_u_Only_p_norms_with_1_p_infinity_p); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(0, 1254, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);

  /* "pyart/map/ckdtree.pyx":1257
 *         if len(x.shape)==1:
 *             single = True
 *             x = x[np.newaxis,:]             # <<<<<<<<<<<<<<
 *         else:
 *             single = False
 */
  __pyx_slice__16 = PySlice_New(Py_None, Py_None, Py_None); if (unlikely(!__pyx_slice__16)) __PYX_ERR(0, 1257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__16);
  __Pyx_GIVEREF(__pyx_slice__16);

  /* "pyart/map/ckdtree.pyx":1294
 *                             # C long overlow, return array of dtype=np.int_p
 *                             if k==1:
 *                                 return np.reshape(dd[...,0],retshape), np.reshape(ii[...,0],retshape)             # <<<<<<<<<<<<<<
 *                             else:
 *                                 return np.reshape(dd,retshape+(k,)), np.reshape(ii,retshape+(k,))
 */
  __pyx_tuple__17 = PyTuple_Pack(2, Py_Ellipsis, __pyx_int_0); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(0, 1294, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);

  /* "pyart/map/ckdtree.pyx":1436
 *             return self.__query_ball_point(&xx[0], r, p, eps)
 *         else:
 *             retshape = x.shape[:-1]             # <<<<<<<<<<<<<<
 *             result = np.empty(retshape, dtype=np.object)
 *             for c in np.ndindex(retshape):
 */
  __pyx_slice__18 = PySlice_New(Py_None, __pyx_int_neg_1, Py_None); if (unlikely(!__pyx_slice__18)) __PYX_ERR(0, 1436, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__18);
  __Pyx_GIVEREF(__pyx_slice__18);

  /* "pyart/map/ckdtree.pyx":1598
 *         # Make sure trees are compatible
 *         if self.m != other.m:
 *             raise ValueError("Trees passed to query_ball_trees have different dimensionality")             # <<<<<<<<<<<<<<
 * 
 *         # Track node-to-node min/max distances
 */
  __pyx_tuple__19 = PyTuple_Pack(1, __pyx_kp_u_Trees_passed_to_query_ball_trees); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(0, 1598, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);

  /* "pyart/map/ckdtree.pyx":1958
 *             n_queries = r.shape[0]
 *         else:
 *             raise ValueError("r must be either a single value or a one-dimensional array of values")             # <<<<<<<<<<<<<<
 * 
 *         # Internally, we represent all distances as distance ** p
 */
  __pyx_tuple__20 = PyTuple_Pack(1, __pyx_kp_u_r_must_be_either_a_single_value); if (unlikely(!__pyx_tuple__20)) __PYX_ERR(0, 1958, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__20);
  __Pyx_GIVEREF(__pyx_tuple__20);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.raw_data,self.raw_indices,self.raw_maxes,self.raw_mins,self.tree cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.raw_data,self.raw_indices,self.raw_maxes,self.raw_mins,self.tree cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__21 = PyTuple_Pack(1, __pyx_kp_s_self_raw_data_self_raw_indices_s); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);

  /* "(tree fragment)":4
 *     raise TypeError("self.raw_data,self.raw_indices,self.raw_maxes,self.raw_mins,self.tree cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.raw_data,self.raw_indices,self.raw_maxes,self.raw_mins,self.tree cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__22 = PyTuple_Pack(1, __pyx_kp_s_self_raw_data_self_raw_indices_s); if (unlikely(!__pyx_tuple__22)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__22);
  __Pyx_GIVEREF(__pyx_tuple__22);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":945
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
  __pyx_tuple__23 = PyTuple_Pack(1, __pyx_kp_u_numpy_core_multiarray_failed_to); if (unlikely(!__pyx_tuple__23)) __PYX_ERR(2, 945, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__23);
  __Pyx_GIVEREF(__pyx_tuple__23);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":951
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
  __pyx_tuple__24 = PyTuple_Pack(1, __pyx_kp_u_numpy_core_umath_failed_to_impor); if (unlikely(!__pyx_tuple__24)) __PYX_ERR(2, 951, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__24);
  __Pyx_GIVEREF(__pyx_tuple__24);
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_float_0_0 = PyFloat_FromDouble(0.0); if (unlikely(!__pyx_float_0_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_12 = PyInt_FromLong(12); if (unlikely(!__pyx_int_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_1 = PyInt_FromLong(-1); if (unlikely(!__pyx_int_neg_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_5pyart_3map_7ckdtree_heap = &__pyx_vtable_5pyart_3map_7ckdtree_heap;
  __pyx_vtable_5pyart_3map_7ckdtree_heap._resize = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_heap *, __pyx_t_5numpy_intp_t))__pyx_f_5pyart_3map_7ckdtree_4heap__resize;
  __pyx_vtable_5pyart_3map_7ckdtree_heap.push = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_heap *, struct __pyx_t_5pyart_3map_7ckdtree_heapitem))__pyx_f_5pyart_3map_7ckdtree_4heap_push;
  __pyx_vtable_5pyart_3map_7ckdtree_heap.peek = (struct __pyx_t_5pyart_3map_7ckdtree_heapitem (*)(struct __pyx_obj_5pyart_3map_7ckdtree_heap *))__pyx_f_5pyart_3map_7ckdtree_4heap_peek;
  __pyx_vtable_5pyart_3map_7ckdtree_heap.remove = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_heap *))__pyx_f_5pyart_3map_7ckdtree_4heap_remove;
  __pyx_vtable_5pyart_3map_7ckdtree_heap.pop = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_heap *, struct __pyx_t_5pyart_3map_7ckdtree_heapitem *))__pyx_f_5pyart_3map_7ckdtree_4heap_pop;
  if (PyType_Ready(&__pyx_type_5pyart_3map_7ckdtree_heap) < 0) __PYX_ERR(0, 132, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_5pyart_3map_7ckdtree_heap.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_5pyart_3map_7ckdtree_heap.tp_dictoffset && __pyx_type_5pyart_3map_7ckdtree_heap.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_5pyart_3map_7ckdtree_heap.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_5pyart_3map_7ckdtree_heap.tp_dict, __pyx_vtabptr_5pyart_3map_7ckdtree_heap) < 0) __PYX_ERR(0, 132, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_heap, (PyObject *)&__pyx_type_5pyart_3map_7ckdtree_heap) < 0) __PYX_ERR(0, 132, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5pyart_3map_7ckdtree_heap) < 0) __PYX_ERR(0, 132, __pyx_L1_error)
  __pyx_ptype_5pyart_3map_7ckdtree_heap = &__pyx_type_5pyart_3map_7ckdtree_heap;
  __pyx_vtabptr_5pyart_3map_7ckdtree_coo_entries = &__pyx_vtable_5pyart_3map_7ckdtree_coo_entries;
  __pyx_vtable_5pyart_3map_7ckdtree_coo_entries.add = (void (*)(struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_float64_t))__pyx_f_5pyart_3map_7ckdtree_11coo_entries_add;
  if (PyType_Ready(&__pyx_type_5pyart_3map_7ckdtree_coo_entries) < 0) __PYX_ERR(0, 237, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_5pyart_3map_7ckdtree_coo_entries.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_5pyart_3map_7ckdtree_coo_entries.tp_dictoffset && __pyx_type_5pyart_3map_7ckdtree_coo_entries.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_5pyart_3map_7ckdtree_coo_entries.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_5pyart_3map_7ckdtree_coo_entries.tp_dict, __pyx_vtabptr_5pyart_3map_7ckdtree_coo_entries) < 0) __PYX_ERR(0, 237, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_coo_entries, (PyObject *)&__pyx_type_5pyart_3map_7ckdtree_coo_entries) < 0) __PYX_ERR(0, 237, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5pyart_3map_7ckdtree_coo_entries) < 0) __PYX_ERR(0, 237, __pyx_L1_error)
  __pyx_ptype_5pyart_3map_7ckdtree_coo_entries = &__pyx_type_5pyart_3map_7ckdtree_coo_entries;
  if (PyType_Ready(&__pyx_type_5pyart_3map_7ckdtree_Rectangle) < 0) __PYX_ERR(0, 326, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_5pyart_3map_7ckdtree_Rectangle.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_5pyart_3map_7ckdtree_Rectangle.tp_dictoffset && __pyx_type_5pyart_3map_7ckdtree_Rectangle.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_5pyart_3map_7ckdtree_Rectangle.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_Rectangle, (PyObject *)&__pyx_type_5pyart_3map_7ckdtree_Rectangle) < 0) __PYX_ERR(0, 326, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5pyart_3map_7ckdtree_Rectangle) < 0) __PYX_ERR(0, 326, __pyx_L1_error)
  __pyx_ptype_5pyart_3map_7ckdtree_Rectangle = &__pyx_type_5pyart_3map_7ckdtree_Rectangle;
  __pyx_vtabptr_5pyart_3map_7ckdtree_RectRectDistanceTracker = &__pyx_vtable_5pyart_3map_7ckdtree_RectRectDistanceTracker;
  __pyx_vtable_5pyart_3map_7ckdtree_RectRectDistanceTracker._init_stack = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *))__pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker__init_stack;
  __pyx_vtable_5pyart_3map_7ckdtree_RectRectDistanceTracker._resize_stack = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *, __pyx_t_5numpy_intp_t))__pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker__resize_stack;
  __pyx_vtable_5pyart_3map_7ckdtree_RectRectDistanceTracker._free_stack = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *))__pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker__free_stack;
  __pyx_vtable_5pyart_3map_7ckdtree_RectRectDistanceTracker.push = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_float64_t))__pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push;
  __pyx_vtable_5pyart_3map_7ckdtree_RectRectDistanceTracker.push_less_of = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *, __pyx_t_5numpy_intp_t, struct __pyx_t_5pyart_3map_7ckdtree_innernode *))__pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_less_of;
  __pyx_vtable_5pyart_3map_7ckdtree_RectRectDistanceTracker.push_greater_of = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *, __pyx_t_5numpy_intp_t, struct __pyx_t_5pyart_3map_7ckdtree_innernode *))__pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_push_greater_of;
  __pyx_vtable_5pyart_3map_7ckdtree_RectRectDistanceTracker.pop = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *))__pyx_f_5pyart_3map_7ckdtree_23RectRectDistanceTracker_pop;
  if (PyType_Ready(&__pyx_type_5pyart_3map_7ckdtree_RectRectDistanceTracker) < 0) __PYX_ERR(0, 457, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_5pyart_3map_7ckdtree_RectRectDistanceTracker.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_5pyart_3map_7ckdtree_RectRectDistanceTracker.tp_dictoffset && __pyx_type_5pyart_3map_7ckdtree_RectRectDistanceTracker.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_5pyart_3map_7ckdtree_RectRectDistanceTracker.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_5pyart_3map_7ckdtree_RectRectDistanceTracker.tp_dict, __pyx_vtabptr_5pyart_3map_7ckdtree_RectRectDistanceTracker) < 0) __PYX_ERR(0, 457, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_RectRectDistanceTracker, (PyObject *)&__pyx_type_5pyart_3map_7ckdtree_RectRectDistanceTracker) < 0) __PYX_ERR(0, 457, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5pyart_3map_7ckdtree_RectRectDistanceTracker) < 0) __PYX_ERR(0, 457, __pyx_L1_error)
  __pyx_ptype_5pyart_3map_7ckdtree_RectRectDistanceTracker = &__pyx_type_5pyart_3map_7ckdtree_RectRectDistanceTracker;
  __pyx_vtabptr_5pyart_3map_7ckdtree_PointRectDistanceTracker = &__pyx_vtable_5pyart_3map_7ckdtree_PointRectDistanceTracker;
  __pyx_vtable_5pyart_3map_7ckdtree_PointRectDistanceTracker._init_stack = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *))__pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker__init_stack;
  __pyx_vtable_5pyart_3map_7ckdtree_PointRectDistanceTracker._resize_stack = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *, __pyx_t_5numpy_intp_t))__pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker__resize_stack;
  __pyx_vtable_5pyart_3map_7ckdtree_PointRectDistanceTracker._free_stack = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *))__pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker__free_stack;
  __pyx_vtable_5pyart_3map_7ckdtree_PointRectDistanceTracker.init = (PyObject *(*)(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *, __pyx_t_5numpy_float64_t *, struct __pyx_obj_5pyart_3map_7ckdtree_Rectangle *, __pyx_t_5numpy_float64_t, __pyx_t_5numpy_float64_t, __pyx_t_5numpy_float64_t))__pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker_init;
  __pyx_vtable_5pyart_3map_7ckdtree_PointRectDistanceTracker.push = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_float64_t))__pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker_push;
  __pyx_vtable_5pyart_3map_7ckdtree_PointRectDistanceTracker.push_less_of = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *))__pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker_push_less_of;
  __pyx_vtable_5pyart_3map_7ckdtree_PointRectDistanceTracker.push_greater_of = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *))__pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker_push_greater_of;
  __pyx_vtable_5pyart_3map_7ckdtree_PointRectDistanceTracker.pop = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *))__pyx_f_5pyart_3map_7ckdtree_24PointRectDistanceTracker_pop;
  if (PyType_Ready(&__pyx_type_5pyart_3map_7ckdtree_PointRectDistanceTracker) < 0) __PYX_ERR(0, 636, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_5pyart_3map_7ckdtree_PointRectDistanceTracker.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_5pyart_3map_7ckdtree_PointRectDistanceTracker.tp_dictoffset && __pyx_type_5pyart_3map_7ckdtree_PointRectDistanceTracker.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_5pyart_3map_7ckdtree_PointRectDistanceTracker.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_5pyart_3map_7ckdtree_PointRectDistanceTracker.tp_dict, __pyx_vtabptr_5pyart_3map_7ckdtree_PointRectDistanceTracker) < 0) __PYX_ERR(0, 636, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_PointRectDistanceTracker, (PyObject *)&__pyx_type_5pyart_3map_7ckdtree_PointRectDistanceTracker) < 0) __PYX_ERR(0, 636, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5pyart_3map_7ckdtree_PointRectDistanceTracker) < 0) __PYX_ERR(0, 636, __pyx_L1_error)
  __pyx_ptype_5pyart_3map_7ckdtree_PointRectDistanceTracker = &__pyx_type_5pyart_3map_7ckdtree_PointRectDistanceTracker;
  __pyx_vtabptr_5pyart_3map_7ckdtree_cKDTree = &__pyx_vtable_5pyart_3map_7ckdtree_cKDTree;
  __pyx_vtable_5pyart_3map_7ckdtree_cKDTree.__pyx___build = (struct __pyx_t_5pyart_3map_7ckdtree_innernode *(*)(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_float64_t *, __pyx_t_5numpy_float64_t *))__pyx_f_5pyart_3map_7ckdtree_7cKDTree___build;
  __pyx_vtable_5pyart_3map_7ckdtree_cKDTree.__pyx___free_tree = (PyObject *(*)(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *))__pyx_f_5pyart_3map_7ckdtree_7cKDTree___free_tree;
  __pyx_vtable_5pyart_3map_7ckdtree_cKDTree.__pyx___query = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, __pyx_t_5numpy_float64_t *, __pyx_t_5numpy_intp_t *, __pyx_t_5numpy_float64_t *, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_float64_t, __pyx_t_5numpy_float64_t, __pyx_t_5numpy_float64_t))__pyx_f_5pyart_3map_7ckdtree_7cKDTree___query;
  __pyx_vtable_5pyart_3map_7ckdtree_cKDTree.__pyx___query_ball_point_traverse_no_checking = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, PyObject *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *))__pyx_f_5pyart_3map_7ckdtree_7cKDTree___query_ball_point_traverse_no_checking;
  __pyx_vtable_5pyart_3map_7ckdtree_cKDTree.__pyx___query_ball_point_traverse_checking = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, PyObject *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *, struct __pyx_obj_5pyart_3map_7ckdtree_PointRectDistanceTracker *))__pyx_f_5pyart_3map_7ckdtree_7cKDTree___query_ball_point_traverse_checking;
  __pyx_vtable_5pyart_3map_7ckdtree_cKDTree.__pyx___query_ball_point = (PyObject *(*)(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, __pyx_t_5numpy_float64_t *, __pyx_t_5numpy_float64_t, __pyx_t_5numpy_float64_t, __pyx_t_5numpy_float64_t))__pyx_f_5pyart_3map_7ckdtree_7cKDTree___query_ball_point;
  __pyx_vtable_5pyart_3map_7ckdtree_cKDTree.__pyx___query_ball_tree_traverse_no_checking = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, PyObject *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *))__pyx_f_5pyart_3map_7ckdtree_7cKDTree___query_ball_tree_traverse_no_checking;
  __pyx_vtable_5pyart_3map_7ckdtree_cKDTree.__pyx___query_ball_tree_traverse_checking = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, PyObject *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *, struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *))__pyx_f_5pyart_3map_7ckdtree_7cKDTree___query_ball_tree_traverse_checking;
  __pyx_vtable_5pyart_3map_7ckdtree_cKDTree.__pyx___query_pairs_traverse_no_checking = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, PyObject *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *))__pyx_f_5pyart_3map_7ckdtree_7cKDTree___query_pairs_traverse_no_checking;
  __pyx_vtable_5pyart_3map_7ckdtree_cKDTree.__pyx___query_pairs_traverse_checking = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, PyObject *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *, struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *))__pyx_f_5pyart_3map_7ckdtree_7cKDTree___query_pairs_traverse_checking;
  __pyx_vtable_5pyart_3map_7ckdtree_cKDTree.__pyx___count_neighbors_traverse = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, __pyx_t_5numpy_intp_t, __pyx_t_5numpy_float64_t *, __pyx_t_5numpy_intp_t *, __pyx_t_5numpy_intp_t *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *, struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *))__pyx_f_5pyart_3map_7ckdtree_7cKDTree___count_neighbors_traverse;
  __pyx_vtable_5pyart_3map_7ckdtree_cKDTree.__pyx___sparse_distance_matrix_traverse = (int (*)(struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, struct __pyx_obj_5pyart_3map_7ckdtree_cKDTree *, struct __pyx_obj_5pyart_3map_7ckdtree_coo_entries *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *, struct __pyx_t_5pyart_3map_7ckdtree_innernode *, struct __pyx_obj_5pyart_3map_7ckdtree_RectRectDistanceTracker *))__pyx_f_5pyart_3map_7ckdtree_7cKDTree___sparse_distance_matrix_traverse;
  if (PyType_Ready(&__pyx_type_5pyart_3map_7ckdtree_cKDTree) < 0) __PYX_ERR(0, 791, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_5pyart_3map_7ckdtree_cKDTree.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_5pyart_3map_7ckdtree_cKDTree.tp_dictoffset && __pyx_type_5pyart_3map_7ckdtree_cKDTree.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_5pyart_3map_7ckdtree_cKDTree.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_5pyart_3map_7ckdtree_cKDTree.tp_dict, __pyx_vtabptr_5pyart_3map_7ckdtree_cKDTree) < 0) __PYX_ERR(0, 791, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_cKDTree, (PyObject *)&__pyx_type_5pyart_3map_7ckdtree_cKDTree) < 0) __PYX_ERR(0, 791, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5pyart_3map_7ckdtree_cKDTree) < 0) __PYX_ERR(0, 791, __pyx_L1_error)
  __pyx_ptype_5pyart_3map_7ckdtree_cKDTree = &__pyx_type_5pyart_3map_7ckdtree_cKDTree;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("numpy"); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_5numpy_dtype = __Pyx_ImportType(__pyx_t_1, "numpy", "dtype", sizeof(PyArray_Descr), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_dtype) __PYX_ERR(2, 200, __pyx_L1_error)
  __pyx_ptype_5numpy_flatiter = __Pyx_ImportType(__pyx_t_1, "numpy", "flatiter", sizeof(PyArrayIterObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_flatiter) __PYX_ERR(2, 223, __pyx_L1_error)
  __pyx_ptype_5numpy_broadcast = __Pyx_ImportType(__pyx_t_1, "numpy", "broadcast", sizeof(PyArrayMultiIterObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_broadcast) __PYX_ERR(2, 227, __pyx_L1_error)
  __pyx_ptype_5numpy_ndarray = __Pyx_ImportType(__pyx_t_1, "numpy", "ndarray", sizeof(PyArrayObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_ndarray) __PYX_ERR(2, 239, __pyx_L1_error)
  __pyx_ptype_5numpy_generic = __Pyx_ImportType(__pyx_t_1, "numpy", "generic", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_generic) __PYX_ERR(2, 771, __pyx_L1_error)
  __pyx_ptype_5numpy_number = __Pyx_ImportType(__pyx_t_1, "numpy", "number", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_number) __PYX_ERR(2, 773, __pyx_L1_error)
  __pyx_ptype_5numpy_integer = __Pyx_ImportType(__pyx_t_1, "numpy", "integer", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_integer) __PYX_ERR(2, 775, __pyx_L1_error)
  __pyx_ptype_5numpy_signedinteger = __Pyx_ImportType(__pyx_t_1, "numpy", "signedinteger", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_signedinteger) __PYX_ERR(2, 777, __pyx_L1_error)
  __pyx_ptype_5numpy_unsignedinteger = __Pyx_ImportType(__pyx_t_1, "numpy", "unsignedinteger", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_unsignedinteger) __PYX_ERR(2, 779, __pyx_L1_error)
  __pyx_ptype_5numpy_inexact = __Pyx_ImportType(__pyx_t_1, "numpy", "inexact", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_inexact) __PYX_ERR(2, 781, __pyx_L1_error)
  __pyx_ptype_5numpy_floating = __Pyx_ImportType(__pyx_t_1, "numpy", "floating", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_floating) __PYX_ERR(2, 783, __pyx_L1_error)
  __pyx_ptype_5numpy_complexfloating = __Pyx_ImportType(__pyx_t_1, "numpy", "complexfloating", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_complexfloating) __PYX_ERR(2, 785, __pyx_L1_error)
  __pyx_ptype_5numpy_flexible = __Pyx_ImportType(__pyx_t_1, "numpy", "flexible", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_flexible) __PYX_ERR(2, 787, __pyx_L1_error)
  __pyx_ptype_5numpy_character = __Pyx_ImportType(__pyx_t_1, "numpy", "character", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_character) __PYX_ERR(2, 789, __pyx_L1_error)
  __pyx_ptype_5numpy_ufunc = __Pyx_ImportType(__pyx_t_1, "numpy", "ufunc", sizeof(PyUFuncObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_ufunc) __PYX_ERR(2, 827, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initckdtree(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initckdtree(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_ckdtree(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_ckdtree(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_ckdtree(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __pyx_t_5numpy_float64_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'ckdtree' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_ckdtree(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("ckdtree", __pyx_methods, __pyx_k_Copyright_c_2001_2002_Enthought, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_pyart__map__ckdtree) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "pyart.map.ckdtree")) {
      if (unlikely(PyDict_SetItemString(modules, "pyart.map.ckdtree", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely(__Pyx_modinit_type_init_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_type_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "pyart/map/ckdtree.pyx":40
 * # Additional contributions by Patrick Varilly and Sturla Molden
 * # Released under the scipy license
 * import numpy as np             # <<<<<<<<<<<<<<
 * import scipy.sparse
 * 
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_np, __pyx_t_1) < 0) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/map/ckdtree.pyx":41
 * # Released under the scipy license
 * import numpy as np
 * import scipy.sparse             # <<<<<<<<<<<<<<
 * 
 * cimport numpy as np
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_scipy_sparse, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_scipy, __pyx_t_1) < 0) __PYX_ERR(0, 41, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/map/ckdtree.pyx":49
 * cdef extern from "limits.h":
 *     long LONG_MAX
 * cdef np.float64_t infinity = np.inf             # <<<<<<<<<<<<<<
 * 
 * __all__ = ['cKDTree']
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_inf); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_3 == ((npy_float64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 49, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_5pyart_3map_7ckdtree_infinity = __pyx_t_3;

  /* "pyart/map/ckdtree.pyx":51
 * cdef np.float64_t infinity = np.inf
 * 
 * __all__ = ['cKDTree']             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_cKDTree);
  __Pyx_GIVEREF(__pyx_n_u_cKDTree);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_cKDTree);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_all, __pyx_t_2) < 0) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyart/map/ckdtree.pyx":454
 *     np.float64_t min_distance, max_distance
 * 
 * cdef np.intp_t LESS = 1             # <<<<<<<<<<<<<<
 * cdef np.intp_t GREATER = 2
 * 
 */
  __pyx_v_5pyart_3map_7ckdtree_LESS = 1;

  /* "pyart/map/ckdtree.pyx":455
 * 
 * cdef np.intp_t LESS = 1
 * cdef np.intp_t GREATER = 2             # <<<<<<<<<<<<<<
 * 
 * cdef class RectRectDistanceTracker(object):
 */
  __pyx_v_5pyart_3map_7ckdtree_GREATER = 2;

  /* "pyart/map/ckdtree.pyx":1206
 *     @cython.wraparound(False)
 *     def query(cKDTree self, object x, np.intp_t k=1, np.float64_t eps=0,
 *               np.float64_t p=2, np.float64_t distance_upper_bound=infinity):             # <<<<<<<<<<<<<<
 *         """query(self, x, k=1, eps=0, p=2, distance_upper_bound=np.inf)
 * 
 */
  __pyx_k__14 = __pyx_v_5pyart_3map_7ckdtree_infinity;

  /* "pyart/map/ckdtree.pyx":1
 * # ckdtree.pyx taken from file of same name from SciPy             # <<<<<<<<<<<<<<
 * # http://www.scipy.org
 * # SciPy license:
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_kp_u_cKDTree_query_ball_point_line_13, __pyx_kp_u_query_ball_point_self_x_r_p_eps) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":1014
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init pyart.map.ckdtree", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init pyart.map.ckdtree");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (__Pyx_PyFastCFunction_Check(func)) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* PyObjectFormatAndDecref */
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatSimpleAndDecref(PyObject* s, PyObject* f) {
    if (unlikely(!s)) return NULL;
    if (likely(PyUnicode_CheckExact(s))) return s;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyString_CheckExact(s))) {
        PyObject *result = PyUnicode_FromEncodedObject(s, NULL, "strict");
        Py_DECREF(s);
        return result;
    }
    #endif
    return __Pyx_PyObject_FormatAndDecref(s, f);
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatAndDecref(PyObject* s, PyObject* f) {
    PyObject *result = PyObject_Format(s, f);
    Py_DECREF(s);
    return result;
}

/* JoinPyUnicode */
static PyObject* __Pyx_PyUnicode_Join(PyObject* value_tuple, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      CYTHON_UNUSED Py_UCS4 max_char) {
#if CYTHON_USE_UNICODE_INTERNALS && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    PyObject *result_uval;
    int result_ukind;
    Py_ssize_t i, char_pos;
    void *result_udata;
#if CYTHON_PEP393_ENABLED
    result_uval = PyUnicode_New(result_ulength, max_char);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = (max_char <= 255) ? PyUnicode_1BYTE_KIND : (max_char <= 65535) ? PyUnicode_2BYTE_KIND : PyUnicode_4BYTE_KIND;
    result_udata = PyUnicode_DATA(result_uval);
#else
    result_uval = PyUnicode_FromUnicode(NULL, result_ulength);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = sizeof(Py_UNICODE);
    result_udata = PyUnicode_AS_UNICODE(result_uval);
#endif
    char_pos = 0;
    for (i=0; i < value_count; i++) {
        int ukind;
        Py_ssize_t ulength;
        void *udata;
        PyObject *uval = PyTuple_GET_ITEM(value_tuple, i);
        if (unlikely(__Pyx_PyUnicode_READY(uval)))
            goto bad;
        ulength = __Pyx_PyUnicode_GET_LENGTH(uval);
        if (unlikely(!ulength))
            continue;
        if (unlikely(char_pos + ulength < 0))
            goto overflow;
        ukind = __Pyx_PyUnicode_KIND(uval);
        udata = __Pyx_PyUnicode_DATA(uval);
        if (!CYTHON_PEP393_ENABLED || ukind == result_ukind) {
            memcpy((char *)result_udata + char_pos * result_ukind, udata, (size_t) (ulength * result_ukind));
        } else {
            #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030300F0 || defined(_PyUnicode_FastCopyCharacters)
            _PyUnicode_FastCopyCharacters(result_uval, char_pos, uval, 0, ulength);
            #else
            Py_ssize_t j;
            for (j=0; j < ulength; j++) {
                Py_UCS4 uchar = __Pyx_PyUnicode_READ(ukind, udata, j);
                __Pyx_PyUnicode_WRITE(result_ukind, result_udata, char_pos+j, uchar);
            }
            #endif
        }
        char_pos += ulength;
    }
    return result_uval;
overflow:
    PyErr_SetString(PyExc_OverflowError, "join() result is too long for a Python string");
bad:
    Py_DECREF(result_uval);
    return NULL;
#else
    result_ulength++;
    value_count++;
    return PyUnicode_Join(__pyx_empty_unicode, value_tuple);
#endif
}

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* KeywordStringCheck */
static int __Pyx_CheckKeywordStrings(
    PyObject *kwdict,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kwdict, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    while (PyDict_Next(kwdict, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if ((!kw_allowed) && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* SwapException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = *type;
    exc_info->exc_value = *value;
    exc_info->exc_traceback = *tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
    #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject* index) {
    PyObject *runerr;
    Py_ssize_t key_value;
    PySequenceMethods *m = Py_TYPE(obj)->tp_as_sequence;
    if (unlikely(!(m && m->sq_item))) {
        PyErr_Format(PyExc_TypeError, "'%.200s' object is not subscriptable", Py_TYPE(obj)->tp_name);
        return NULL;
    }
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError, "cannot fit '%.200s' into an index-sized integer", Py_TYPE(index)->tp_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key) {
    PyMappingMethods *m = Py_TYPE(obj)->tp_as_mapping;
    if (likely(m && m->mp_subscript)) {
        return m->mp_subscript(obj, key);
    }
    return __Pyx_PyObject_GetIndex(obj, key);
}
#endif

/* IsLittleEndian */
static CYTHON_INLINE int __Pyx_Is_Little_Endian(void)
{
  union {
    uint32_t u32;
    uint8_t u8[4];
  } S;
  S.u32 = 0x01020304;
  return S.u8[0] == 4;
}

/* BufferFormatCheck */
static void __Pyx_BufFmt_Init(__Pyx_BufFmt_Context* ctx,
                              __Pyx_BufFmt_StackElem* stack,
                              __Pyx_TypeInfo* type) {
  stack[0].field = &ctx->root;
  stack[0].parent_offset = 0;
  ctx->root.type = type;
  ctx->root.name = "buffer dtype";
  ctx->root.offset = 0;
  ctx->head = stack;
  ctx->head->field = &ctx->root;
  ctx->fmt_offset = 0;
  ctx->head->parent_offset = 0;
  ctx->new_packmode = '@';
  ctx->enc_packmode = '@';
  ctx->new_count = 1;
  ctx->enc_count = 0;
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  ctx->is_valid_array = 0;
  ctx->struct_alignment = 0;
  while (type->typegroup == 'S') {
    ++ctx->head;
    ctx->head->field = type->fields;
    ctx->head->parent_offset = 0;
    type = type->fields->type;
  }
}
static int __Pyx_BufFmt_ParseNumber(const char** ts) {
    int count;
    const char* t = *ts;
    if (*t < '0' || *t > '9') {
      return -1;
    } else {
        count = *t++ - '0';
        while (*t >= '0' && *t <= '9') {
            count *= 10;
            count += *t++ - '0';
        }
    }
    *ts = t;
    return count;
}
static int __Pyx_BufFmt_ExpectNumber(const char **ts) {
    int number = __Pyx_BufFmt_ParseNumber(ts);
    if (number == -1)
        PyErr_Format(PyExc_ValueError,\
                     "Does not understand character buffer dtype format string ('%c')", **ts);
    return number;
}
static void __Pyx_BufFmt_RaiseUnexpectedChar(char ch) {
  PyErr_Format(PyExc_ValueError,
               "Unexpected format string character: '%c'", ch);
}
static const char* __Pyx_BufFmt_DescribeTypeChar(char ch, int is_complex) {
  switch (ch) {
    case '?': return "'bool'";
    case 'c': return "'char'";
    case 'b': return "'signed char'";
    case 'B': return "'unsigned char'";
    case 'h': return "'short'";
    case 'H': return "'unsigned short'";
    case 'i': return "'int'";
    case 'I': return "'unsigned int'";
    case 'l': return "'long'";
    case 'L': return "'unsigned long'";
    case 'q': return "'long long'";
    case 'Q': return "'unsigned long long'";
    case 'f': return (is_complex ? "'complex float'" : "'float'");
    case 'd': return (is_complex ? "'complex double'" : "'double'");
    case 'g': return (is_complex ? "'complex long double'" : "'long double'");
    case 'T': return "a struct";
    case 'O': return "Python object";
    case 'P': return "a pointer";
    case 's': case 'p': return "a string";
    case 0: return "end";
    default: return "unparseable format string";
  }
}
static size_t __Pyx_BufFmt_TypeCharToStandardSize(char ch, int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return 2;
    case 'i': case 'I': case 'l': case 'L': return 4;
    case 'q': case 'Q': return 8;
    case 'f': return (is_complex ? 8 : 4);
    case 'd': return (is_complex ? 16 : 8);
    case 'g': {
      PyErr_SetString(PyExc_ValueError, "Python does not define a standard format string size for long double ('g')..");
      return 0;
    }
    case 'O': case 'P': return sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static size_t __Pyx_BufFmt_TypeCharToNativeSize(char ch, int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(short);
    case 'i': case 'I': return sizeof(int);
    case 'l': case 'L': return sizeof(long);
    #ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(PY_LONG_LONG);
    #endif
    case 'f': return sizeof(float) * (is_complex ? 2 : 1);
    case 'd': return sizeof(double) * (is_complex ? 2 : 1);
    case 'g': return sizeof(long double) * (is_complex ? 2 : 1);
    case 'O': case 'P': return sizeof(void*);
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
typedef struct { char c; short x; } __Pyx_st_short;
typedef struct { char c; int x; } __Pyx_st_int;
typedef struct { char c; long x; } __Pyx_st_long;
typedef struct { char c; float x; } __Pyx_st_float;
typedef struct { char c; double x; } __Pyx_st_double;
typedef struct { char c; long double x; } __Pyx_st_longdouble;
typedef struct { char c; void *x; } __Pyx_st_void_p;
#ifdef HAVE_LONG_LONG
typedef struct { char c; PY_LONG_LONG x; } __Pyx_st_longlong;
#endif
static size_t __Pyx_BufFmt_TypeCharToAlignment(char ch, CYTHON_UNUSED int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_st_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_st_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_st_long) - sizeof(long);
#ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(__Pyx_st_longlong) - sizeof(PY_LONG_LONG);
#endif
    case 'f': return sizeof(__Pyx_st_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_st_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_st_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_st_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
/* These are for computing the padding at the end of the struct to align
   on the first member of the struct. This will probably the same as above,
   but we don't have any guarantees.
 */
typedef struct { short x; char c; } __Pyx_pad_short;
typedef struct { int x; char c; } __Pyx_pad_int;
typedef struct { long x; char c; } __Pyx_pad_long;
typedef struct { float x; char c; } __Pyx_pad_float;
typedef struct { double x; char c; } __Pyx_pad_double;
typedef struct { long double x; char c; } __Pyx_pad_longdouble;
typedef struct { void *x; char c; } __Pyx_pad_void_p;
#ifdef HAVE_LONG_LONG
typedef struct { PY_LONG_LONG x; char c; } __Pyx_pad_longlong;
#endif
static size_t __Pyx_BufFmt_TypeCharToPadding(char ch, CYTHON_UNUSED int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_pad_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_pad_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_pad_long) - sizeof(long);
#ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(__Pyx_pad_longlong) - sizeof(PY_LONG_LONG);
#endif
    case 'f': return sizeof(__Pyx_pad_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_pad_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_pad_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_pad_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static char __Pyx_BufFmt_TypeCharToGroup(char ch, int is_complex) {
  switch (ch) {
    case 'c':
        return 'H';
    case 'b': case 'h': case 'i':
    case 'l': case 'q': case 's': case 'p':
        return 'I';
    case '?': case 'B': case 'H': case 'I': case 'L': case 'Q':
        return 'U';
    case 'f': case 'd': case 'g':
        return (is_complex ? 'C' : 'R');
    case 'O':
        return 'O';
    case 'P':
        return 'P';
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
static void __Pyx_BufFmt_RaiseExpected(__Pyx_BufFmt_Context* ctx) {
  if (ctx->head == NULL || ctx->head->field == &ctx->root) {
    const char* expected;
    const char* quote;
    if (ctx->head == NULL) {
      expected = "end";
      quote = "";
    } else {
      expected = ctx->head->field->type->name;
      quote = "'";
    }
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected %s%s%s but got %s",
                 quote, expected, quote,
                 __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex));
  } else {
    __Pyx_StructField* field = ctx->head->field;
    __Pyx_StructField* parent = (ctx->head - 1)->field;
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected '%s' but got %s in '%s.%s'",
                 field->type->name, __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex),
                 parent->type->name, field->name);
  }
}
static int __Pyx_BufFmt_ProcessTypeChunk(__Pyx_BufFmt_Context* ctx) {
  char group;
  size_t size, offset, arraysize = 1;
  if (ctx->enc_type == 0) return 0;
  if (ctx->head->field->type->arraysize[0]) {
    int i, ndim = 0;
    if (ctx->enc_type == 's' || ctx->enc_type == 'p') {
        ctx->is_valid_array = ctx->head->field->type->ndim == 1;
        ndim = 1;
        if (ctx->enc_count != ctx->head->field->type->arraysize[0]) {
            PyErr_Format(PyExc_ValueError,
                         "Expected a dimension of size %zu, got %zu",
                         ctx->head->field->type->arraysize[0], ctx->enc_count);
            return -1;
        }
    }
    if (!ctx->is_valid_array) {
      PyErr_Format(PyExc_ValueError, "Expected %d dimensions, got %d",
                   ctx->head->field->type->ndim, ndim);
      return -1;
    }
    for (i = 0; i < ctx->head->field->type->ndim; i++) {
      arraysize *= ctx->head->field->type->arraysize[i];
    }
    ctx->is_valid_array = 0;
    ctx->enc_count = 1;
  }
  group = __Pyx_BufFmt_TypeCharToGroup(ctx->enc_type, ctx->is_complex);
  do {
    __Pyx_StructField* field = ctx->head->field;
    __Pyx_TypeInfo* type = field->type;
    if (ctx->enc_packmode == '@' || ctx->enc_packmode == '^') {
      size = __Pyx_BufFmt_TypeCharToNativeSize(ctx->enc_type, ctx->is_complex);
    } else {
      size = __Pyx_BufFmt_TypeCharToStandardSize(ctx->enc_type, ctx->is_complex);
    }
    if (ctx->enc_packmode == '@') {
      size_t align_at = __Pyx_BufFmt_TypeCharToAlignment(ctx->enc_type, ctx->is_complex);
      size_t align_mod_offset;
      if (align_at == 0) return -1;
      align_mod_offset = ctx->fmt_offset % align_at;
      if (align_mod_offset > 0) ctx->fmt_offset += align_at - align_mod_offset;
      if (ctx->struct_alignment == 0)
          ctx->struct_alignment = __Pyx_BufFmt_TypeCharToPadding(ctx->enc_type,
                                                                 ctx->is_complex);
    }
    if (type->size != size || type->typegroup != group) {
      if (type->typegroup == 'C' && type->fields != NULL) {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        ++ctx->head;
        ctx->head->field = type->fields;
        ctx->head->parent_offset = parent_offset;
        continue;
      }
      if ((type->typegroup == 'H' || group == 'H') && type->size == size) {
      } else {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
      }
    }
    offset = ctx->head->parent_offset + field->offset;
    if (ctx->fmt_offset != offset) {
      PyErr_Format(PyExc_ValueError,
                   "Buffer dtype mismatch; next field is at offset %" CYTHON_FORMAT_SSIZE_T "d but %" CYTHON_FORMAT_SSIZE_T "d expected",
                   (Py_ssize_t)ctx->fmt_offset, (Py_ssize_t)offset);
      return -1;
    }
    ctx->fmt_offset += size;
    if (arraysize)
      ctx->fmt_offset += (arraysize - 1) * size;
    --ctx->enc_count;
    while (1) {
      if (field == &ctx->root) {
        ctx->head = NULL;
        if (ctx->enc_count != 0) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
        }
        break;
      }
      ctx->head->field = ++field;
      if (field->type == NULL) {
        --ctx->head;
        field = ctx->head->field;
        continue;
      } else if (field->type->typegroup == 'S') {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        if (field->type->fields->type == NULL) continue;
        field = field->type->fields;
        ++ctx->head;
        ctx->head->field = field;
        ctx->head->parent_offset = parent_offset;
        break;
      } else {
        break;
      }
    }
  } while (ctx->enc_count);
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  return 0;
}
static PyObject *
__pyx_buffmt_parse_array(__Pyx_BufFmt_Context* ctx, const char** tsp)
{
    const char *ts = *tsp;
    int i = 0, number, ndim;
    ++ts;
    if (ctx->new_count != 1) {
        PyErr_SetString(PyExc_ValueError,
                        "Cannot handle repeated arrays in format string");
        return NULL;
    }
    if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
    ndim = ctx->head->field->type->ndim;
    while (*ts && *ts != ')') {
        switch (*ts) {
            case ' ': case '\f': case '\r': case '\n': case '\t': case '\v':  continue;
            default:  break;
        }
        number = __Pyx_BufFmt_ExpectNumber(&ts);
        if (number == -1) return NULL;
        if (i < ndim && (size_t) number != ctx->head->field->type->arraysize[i])
            return PyErr_Format(PyExc_ValueError,
                        "Expected a dimension of size %zu, got %d",
                        ctx->head->field->type->arraysize[i], number);
        if (*ts != ',' && *ts != ')')
            return PyErr_Format(PyExc_ValueError,
                                "Expected a comma in format string, got '%c'", *ts);
        if (*ts == ',') ts++;
        i++;
    }
    if (i != ndim)
        return PyErr_Format(PyExc_ValueError, "Expected %d dimension(s), got %d",
                            ctx->head->field->type->ndim, i);
    if (!*ts) {
        PyErr_SetString(PyExc_ValueError,
                        "Unexpected end of format string, expected ')'");
        return NULL;
    }
    ctx->is_valid_array = 1;
    ctx->new_count = 1;
    *tsp = ++ts;
    return Py_None;
}
static const char* __Pyx_BufFmt_CheckString(__Pyx_BufFmt_Context* ctx, const char* ts) {
  int got_Z = 0;
  while (1) {
    switch(*ts) {
      case 0:
        if (ctx->enc_type != 0 && ctx->head == NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        if (ctx->head != NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        return ts;
      case ' ':
      case '\r':
      case '\n':
        ++ts;
        break;
      case '<':
        if (!__Pyx_Is_Little_Endian()) {
          PyErr_SetString(PyExc_ValueError, "Little-endian buffer not supported on big-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '>':
      case '!':
        if (__Pyx_Is_Little_Endian()) {
          PyErr_SetString(PyExc_ValueError, "Big-endian buffer not supported on little-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '=':
      case '@':
      case '^':
        ctx->new_packmode = *ts++;
        break;
      case 'T':
        {
          const char* ts_after_sub;
          size_t i, struct_count = ctx->new_count;
          size_t struct_alignment = ctx->struct_alignment;
          ctx->new_count = 1;
          ++ts;
          if (*ts != '{') {
            PyErr_SetString(PyExc_ValueError, "Buffer acquisition: Expected '{' after 'T'");
            return NULL;
          }
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          ctx->enc_count = 0;
          ctx->struct_alignment = 0;
          ++ts;
          ts_after_sub = ts;
          for (i = 0; i != struct_count; ++i) {
            ts_after_sub = __Pyx_BufFmt_CheckString(ctx, ts);
            if (!ts_after_sub) return NULL;
          }
          ts = ts_after_sub;
          if (struct_alignment) ctx->struct_alignment = struct_alignment;
        }
        break;
      case '}':
        {
          size_t alignment = ctx->struct_alignment;
          ++ts;
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          if (alignment && ctx->fmt_offset % alignment) {
            ctx->fmt_offset += alignment - (ctx->fmt_offset % alignment);
          }
        }
        return ts;
      case 'x':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->fmt_offset += ctx->new_count;
        ctx->new_count = 1;
        ctx->enc_count = 0;
        ctx->enc_type = 0;
        ctx->enc_packmode = ctx->new_packmode;
        ++ts;
        break;
      case 'Z':
        got_Z = 1;
        ++ts;
        if (*ts != 'f' && *ts != 'd' && *ts != 'g') {
          __Pyx_BufFmt_RaiseUnexpectedChar('Z');
          return NULL;
        }
        CYTHON_FALLTHROUGH;
      case '?': case 'c': case 'b': case 'B': case 'h': case 'H': case 'i': case 'I':
      case 'l': case 'L': case 'q': case 'Q':
      case 'f': case 'd': case 'g':
      case 'O': case 'p':
        if ((ctx->enc_type == *ts) && (got_Z == ctx->is_complex) &&
            (ctx->enc_packmode == ctx->new_packmode) && (!ctx->is_valid_array)) {
          ctx->enc_count += ctx->new_count;
          ctx->new_count = 1;
          got_Z = 0;
          ++ts;
          break;
        }
        CYTHON_FALLTHROUGH;
      case 's':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->enc_count = ctx->new_count;
        ctx->enc_packmode = ctx->new_packmode;
        ctx->enc_type = *ts;
        ctx->is_complex = got_Z;
        ++ts;
        ctx->new_count = 1;
        got_Z = 0;
        break;
      case ':':
        ++ts;
        while(*ts != ':') ++ts;
        ++ts;
        break;
      case '(':
        if (!__pyx_buffmt_parse_array(ctx, &ts)) return NULL;
        break;
      default:
        {
          int number = __Pyx_BufFmt_ExpectNumber(&ts);
          if (number == -1) return NULL;
          ctx->new_count = (size_t)number;
        }
    }
  }
}

/* BufferGetAndValidate */
  static CYTHON_INLINE void __Pyx_SafeReleaseBuffer(Py_buffer* info) {
  if (unlikely(info->buf == NULL)) return;
  if (info->suboffsets == __Pyx_minusones) info->suboffsets = NULL;
  __Pyx_ReleaseBuffer(info);
}
static void __Pyx_ZeroBuffer(Py_buffer* buf) {
  buf->buf = NULL;
  buf->obj = NULL;
  buf->strides = __Pyx_zeros;
  buf->shape = __Pyx_zeros;
  buf->suboffsets = __Pyx_minusones;
}
static int __Pyx__GetBufferAndValidate(
        Py_buffer* buf, PyObject* obj,  __Pyx_TypeInfo* dtype, int flags,
        int nd, int cast, __Pyx_BufFmt_StackElem* stack)
{
  buf->buf = NULL;
  if (unlikely(__Pyx_GetBuffer(obj, buf, flags) == -1)) {
    __Pyx_ZeroBuffer(buf);
    return -1;
  }
  if (unlikely(buf->ndim != nd)) {
    PyErr_Format(PyExc_ValueError,
                 "Buffer has wrong number of dimensions (expected %d, got %d)",
                 nd, buf->ndim);
    goto fail;
  }
  if (!cast) {
    __Pyx_BufFmt_Context ctx;
    __Pyx_BufFmt_Init(&ctx, stack, dtype);
    if (!__Pyx_BufFmt_CheckString(&ctx, buf->format)) goto fail;
  }
  if (unlikely((size_t)buf->itemsize != dtype->size)) {
    PyErr_Format(PyExc_ValueError,
      "Item size of buffer (%" CYTHON_FORMAT_SSIZE_T "d byte%s) does not match size of '%s' (%" CYTHON_FORMAT_SSIZE_T "d byte%s)",
      buf->itemsize, (buf->itemsize > 1) ? "s" : "",
      dtype->name, (Py_ssize_t)dtype->size, (dtype->size > 1) ? "s" : "");
    goto fail;
  }
  if (buf->suboffsets == NULL) buf->suboffsets = __Pyx_minusones;
  return 0;
fail:;
  __Pyx_SafeReleaseBuffer(buf);
  return -1;
}

/* BufferFallbackError */
  static void __Pyx_RaiseBufferFallbackError(void) {
  PyErr_SetString(PyExc_ValueError,
     "Buffer acquisition failed on assignment; and then reacquiring the old buffer failed too!");
}

/* PyObjectCallNoArg */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* BufferIndexError */
  static void __Pyx_RaiseBufferIndexError(int axis) {
  PyErr_Format(PyExc_IndexError,
     "Out of bounds on buffer access (axis %d)", axis);
}

/* SliceObject */
  static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_slice(obj, cstart, cstop);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object is unsliceable", Py_TYPE(obj)->tp_name);
bad:
    return NULL;
}

/* PyErrExceptionMatches */
  #if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* PyObject_GenericGetAttrNoDict */
  #if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
  #if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
  static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* PyObjectGetAttrStrNoError */
  static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
}

/* SetupReduce */
  static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* TypeImport */
  #ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* Import */
  static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* CLineInTraceback */
  #ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
  static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
  #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_srcfile = NULL;
    py_srcfile = PyString_FromString(filename);
    if (!py_srcfile) goto bad;
    #endif
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        if (!py_funcname) goto bad;
        #endif
    }
    #if PY_MAJOR_VERSION < 3
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    #else
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    #endif
    Py_XDECREF(py_funcname);  // XDECREF since it's only set on Py3 if cline
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_srcfile);
    #endif
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

#if PY_MAJOR_VERSION < 3
static int __Pyx_GetBuffer(PyObject *obj, Py_buffer *view, int flags) {
    if (PyObject_CheckBuffer(obj)) return PyObject_GetBuffer(obj, view, flags);
    PyErr_Format(PyExc_TypeError, "'%.200s' does not have the buffer interface", Py_TYPE(obj)->tp_name);
    return -1;
}
static void __Pyx_ReleaseBuffer(Py_buffer *view) {
    PyObject *obj = view->obj;
    if (!obj) return;
    if (PyObject_CheckBuffer(obj)) {
        PyBuffer_Release(view);
        return;
    }
    if ((0)) {}
    view->obj = NULL;
    Py_DECREF(obj);
}
#endif


  /* CIntFromPyVerify */
  #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* Declarations */
  #if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
  #if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabsf(b.real) >= fabsf(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                float r = b.imag / b.real;
                float s = (float)(1.0) / (b.real + b.imag * r);
                return __pyx_t_float_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            float r = b.real / b.imag;
            float s = (float)(1.0) / (b.imag + b.real * r);
            return __pyx_t_float_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            float denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_float_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_float(a, a);
                    case 3:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, a);
                    case 4:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = powf(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2f(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_float(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

/* Declarations */
  #if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
  #if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabs(b.real) >= fabs(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                double r = b.imag / b.real;
                double s = (double)(1.0) / (b.real + b.imag * r);
                return __pyx_t_double_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            double r = b.real / b.imag;
            double s = (double)(1.0) / (b.imag + b.real * r);
            return __pyx_t_double_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            double denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_double_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_double(a, a);
                    case 3:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, a);
                    case 4:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = pow(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_double(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

/* CIntFromPy */
  static CYTHON_INLINE Py_intptr_t __Pyx_PyInt_As_Py_intptr_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const Py_intptr_t neg_one = (Py_intptr_t) -1, const_zero = (Py_intptr_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(Py_intptr_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (Py_intptr_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (Py_intptr_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(Py_intptr_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(Py_intptr_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) >= 2 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((((Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(Py_intptr_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) >= 3 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((((((Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(Py_intptr_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) >= 4 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((((((((Py_intptr_t)digits[3]) << PyLong_SHIFT) | (Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (Py_intptr_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(Py_intptr_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(Py_intptr_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(Py_intptr_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(Py_intptr_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (Py_intptr_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(Py_intptr_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(Py_intptr_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(Py_intptr_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 2 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((Py_intptr_t)-1)*(((((Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(Py_intptr_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 2 * PyLong_SHIFT) {
                            return (Py_intptr_t) ((((((Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(Py_intptr_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 3 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((Py_intptr_t)-1)*(((((((Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(Py_intptr_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 3 * PyLong_SHIFT) {
                            return (Py_intptr_t) ((((((((Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(Py_intptr_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 4 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((Py_intptr_t)-1)*(((((((((Py_intptr_t)digits[3]) << PyLong_SHIFT) | (Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(Py_intptr_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 4 * PyLong_SHIFT) {
                            return (Py_intptr_t) ((((((((((Py_intptr_t)digits[3]) << PyLong_SHIFT) | (Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(Py_intptr_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(Py_intptr_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(Py_intptr_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(Py_intptr_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            Py_intptr_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (Py_intptr_t) -1;
        }
    } else {
        Py_intptr_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (Py_intptr_t) -1;
        val = __Pyx_PyInt_As_Py_intptr_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to Py_intptr_t");
    return (Py_intptr_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to Py_intptr_t");
    return (Py_intptr_t) -1;
}

/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyInt_From_Py_intptr_t(Py_intptr_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const Py_intptr_t neg_one = (Py_intptr_t) -1, const_zero = (Py_intptr_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(Py_intptr_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(Py_intptr_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(Py_intptr_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(Py_intptr_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(Py_intptr_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(Py_intptr_t),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
  static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
  static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* FastTypeChecks */
  #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
  static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* InitStrings */
  static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
#if PY_MAJOR_VERSION < 3
  } else if (likely(PyInt_CheckExact(o))) {
    return PyInt_AS_LONG(o);
#endif
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyInt_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
