/* Generated by Cython 0.29.28 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [
            "/home/zsherman/anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/core/include/numpy/arrayobject.h",
            "/home/zsherman/anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/core/include/numpy/arrayscalars.h",
            "/home/zsherman/anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/core/include/numpy/ndarrayobject.h",
            "/home/zsherman/anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/core/include/numpy/ndarraytypes.h",
            "/home/zsherman/anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/core/include/numpy/ufuncobject.h",
            "/home/zsherman/anaconda3/envs/pyart_env/include/rsl.h"
        ],
        "include_dirs": [
            "/home/zsherman/anaconda3/envs/pyart_env/include",
            "/home/zsherman/anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/core/include"
        ],
        "libraries": [
            "rsl"
        ],
        "library_dirs": [
            "/home/zsherman/anaconda3/envs/pyart_env/lib"
        ],
        "name": "pyart.io._rsl_interface",
        "runtime_library_dirs": [
            "/home/zsherman/anaconda3/envs/pyart_env/lib"
        ],
        "sources": [
            "pyart/io/_rsl_interface.pyx"
        ]
    },
    "module_name": "pyart.io._rsl_interface"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_28"
#define CYTHON_HEX_VERSION 0x001D1CF0
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0 || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 0
  #elif !defined(CYTHON_FAST_THREAD_STATE)
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL (PY_VERSION_HEX < 0x030B00A1)
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 0
  #elif !defined(CYTHON_USE_EXC_INFO_STACK)
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_MAJOR_VERSION < 3
    #include "longintrepr.h"
  #endif
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_DefaultClassType PyType_Type
#if PY_VERSION_HEX >= 0x030B00A1
    static CYTHON_INLINE PyCodeObject* __Pyx_PyCode_New(int a, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
        PyObject *kwds=NULL, *argcount=NULL, *posonlyargcount=NULL, *kwonlyargcount=NULL;
        PyObject *nlocals=NULL, *stacksize=NULL, *flags=NULL, *replace=NULL, *call_result=NULL, *empty=NULL;
        const char *fn_cstr=NULL;
        const char *name_cstr=NULL;
        PyCodeObject* co=NULL;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        if (!(kwds=PyDict_New())) goto end;
        if (!(argcount=PyLong_FromLong(a))) goto end;
        if (PyDict_SetItemString(kwds, "co_argcount", argcount) != 0) goto end;
        if (!(posonlyargcount=PyLong_FromLong(0))) goto end;
        if (PyDict_SetItemString(kwds, "co_posonlyargcount", posonlyargcount) != 0) goto end;
        if (!(kwonlyargcount=PyLong_FromLong(k))) goto end;
        if (PyDict_SetItemString(kwds, "co_kwonlyargcount", kwonlyargcount) != 0) goto end;
        if (!(nlocals=PyLong_FromLong(l))) goto end;
        if (PyDict_SetItemString(kwds, "co_nlocals", nlocals) != 0) goto end;
        if (!(stacksize=PyLong_FromLong(s))) goto end;
        if (PyDict_SetItemString(kwds, "co_stacksize", stacksize) != 0) goto end;
        if (!(flags=PyLong_FromLong(f))) goto end;
        if (PyDict_SetItemString(kwds, "co_flags", flags) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_code", code) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_consts", c) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_names", n) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_varnames", v) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_freevars", fv) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_cellvars", cell) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_linetable", lnos) != 0) goto end;
        if (!(fn_cstr=PyUnicode_AsUTF8AndSize(fn, NULL))) goto end;
        if (!(name_cstr=PyUnicode_AsUTF8AndSize(name, NULL))) goto end;
        if (!(co = PyCode_NewEmpty(fn_cstr, name_cstr, fline))) goto end;
        if (!(replace = PyObject_GetAttrString((PyObject*)co, "replace"))) goto cleanup_code_too;
        if (!(empty = PyTuple_New(0))) goto cleanup_code_too; // unfortunately __pyx_empty_tuple isn't available here
        if (!(call_result = PyObject_Call(replace, empty, kwds))) goto cleanup_code_too;
        Py_XDECREF((PyObject*)co);
        co = (PyCodeObject*)call_result;
        call_result = NULL;
        if (0) {
            cleanup_code_too:
            Py_XDECREF((PyObject*)co);
            co = NULL;
        }
        end:
        Py_XDECREF(kwds);
        Py_XDECREF(argcount);
        Py_XDECREF(posonlyargcount);
        Py_XDECREF(kwonlyargcount);
        Py_XDECREF(nlocals);
        Py_XDECREF(stacksize);
        Py_XDECREF(replace);
        Py_XDECREF(call_result);
        Py_XDECREF(empty);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return co;
    }
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if defined(PyUnicode_IS_READY)
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #else
  #define __Pyx_PyUnicode_READY(op)       (0)
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if defined(PyUnicode_IS_READY) && defined(PyUnicode_GET_SIZE)
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
  #endif
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsHash_t
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__pyart__io___rsl_interface
#define __PYX_HAVE_API__pyart__io___rsl_interface
/* Early includes */
#include "rsl.h"
#include <string.h>
#include <stdio.h>
#include "numpy/arrayobject.h"
#include "numpy/ndarrayobject.h"
#include "numpy/ndarraytypes.h"
#include "numpy/arrayscalars.h"
#include "numpy/ufuncobject.h"

    /* NumPy API declarations from "numpy/__init__.pxd" */
    
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;

/* Header.proto */
#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif defined(_Complex_I)
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif


static const char *__pyx_f[] = {
  "pyart/io/_rsl_interface.pyx",
  "stringsource",
  "__init__.pxd",
  "type.pxd",
};
/* BufferFormatStructs.proto */
#define IS_UNSIGNED(type) (((type) -1) > 0)
struct __Pyx_StructField_;
#define __PYX_BUF_FLAGS_PACKED_STRUCT (1 << 0)
typedef struct {
  const char* name;
  struct __Pyx_StructField_* fields;
  size_t size;
  size_t arraysize[8];
  int ndim;
  char typegroup;
  char is_unsigned;
  int flags;
} __Pyx_TypeInfo;
typedef struct __Pyx_StructField_ {
  __Pyx_TypeInfo* type;
  const char* name;
  size_t offset;
} __Pyx_StructField;
typedef struct {
  __Pyx_StructField* field;
  size_t parent_offset;
} __Pyx_BufFmt_StackElem;
typedef struct {
  __Pyx_StructField root;
  __Pyx_BufFmt_StackElem* head;
  size_t fmt_offset;
  size_t new_count, enc_count;
  size_t struct_alignment;
  int is_complex;
  char enc_type;
  char new_packmode;
  char enc_packmode;
  char is_valid_array;
} __Pyx_BufFmt_Context;


/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":690
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 */
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":691
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
 */
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":692
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * #ctypedef npy_int96      int96_t
 */
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":693
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_int96      int96_t
 * #ctypedef npy_int128     int128_t
 */
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":697
 * #ctypedef npy_int128     int128_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 */
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":698
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
 */
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":699
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * #ctypedef npy_uint96     uint96_t
 */
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":700
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_uint96     uint96_t
 * #ctypedef npy_uint128    uint128_t
 */
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":704
 * #ctypedef npy_uint128    uint128_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
 */
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":705
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
 */
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":714
 * # The int types are mapped a bit surprising --
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t
 */
typedef npy_long __pyx_t_5numpy_int_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":715
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   longlong_t
 * 
 */
typedef npy_longlong __pyx_t_5numpy_long_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":716
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_ulong      uint_t
 */
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":718
 * ctypedef npy_longlong   longlong_t
 * 
 * ctypedef npy_ulong      uint_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t
 */
typedef npy_ulong __pyx_t_5numpy_uint_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":719
 * 
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 */
typedef npy_ulonglong __pyx_t_5numpy_ulong_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":720
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
 */
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":722
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
 */
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":723
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
 */
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":725
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
 */
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":726
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
 */
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":727
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cfloat      cfloat_t
 */
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;

/* "pyart/io/_rsl_interface.pyx":12
 * # properly wrapped as Cython does not export the typedef from _rsl_h
 * # define NPY_NO_DEPRECATED_API NPY_1_7_API_VERSION" [-W#warnings]
 * ctypedef unsigned short Range             # <<<<<<<<<<<<<<
 * import numpy as np
 * cimport numpy as np
 */
typedef unsigned short __pyx_t_5pyart_2io_14_rsl_interface_Range;
/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif
static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);

/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif
static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);


/*--- Type declarations ---*/
struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume;
struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay;
struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep;
struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile;
struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_float____Range___to_py;
struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_Range____float___to_py;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":729
 * ctypedef npy_longdouble longdouble_t
 * 
 * ctypedef npy_cfloat      cfloat_t             # <<<<<<<<<<<<<<
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t
 */
typedef npy_cfloat __pyx_t_5numpy_cfloat_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":730
 * 
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t             # <<<<<<<<<<<<<<
 * ctypedef npy_clongdouble clongdouble_t
 * 
 */
typedef npy_cdouble __pyx_t_5numpy_cdouble_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":731
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cdouble     complex_t
 */
typedef npy_clongdouble __pyx_t_5numpy_clongdouble_t;

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":733
 * ctypedef npy_clongdouble clongdouble_t
 * 
 * ctypedef npy_cdouble     complex_t             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 */
typedef npy_cdouble __pyx_t_5numpy_complex_t;
struct __pyx_opt_args_5pyart_2io_14_rsl_interface_create_volume;

/* "pyart/io/_rsl_interface.pyx":46
 * 
 * cpdef create_volume(
 *         np.ndarray[np.float32_t, ndim=2] arr,             # <<<<<<<<<<<<<<
 *         np.ndarray[np.int32_t, ndim=1] rays_per_sweep,
 *         int vol_num=1):
 */
struct __pyx_opt_args_5pyart_2io_14_rsl_interface_create_volume {
  int __pyx_n;
  int vol_num;
};

/* "pyart/io/_rsl_interface.pxd":6
 * cimport pyart.io._rsl_h as _rsl_h
 * 
 * cdef class _RslVolume:             # <<<<<<<<<<<<<<
 *     cdef _rsl_h.Volume * _Volume
 *     cdef int _dealloc
 */
struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume {
  PyObject_HEAD
  struct __pyx_vtabstruct_5pyart_2io_14_rsl_interface__RslVolume *__pyx_vtab;
  Volume *_Volume;
  int _dealloc;
};


/* "pyart/io/_rsl_interface.pyx":192
 * 
 * 
 * cdef class _RslRay:             # <<<<<<<<<<<<<<
 *     """
 *     A object for accessing RSL Ray data and header information
 */
struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay {
  PyObject_HEAD
  struct __pyx_vtabstruct_5pyart_2io_14_rsl_interface__RslRay *__pyx_vtab;
  Ray *_Ray;
  int _dealloc;
};


/* "pyart/io/_rsl_interface.pyx":612
 * 
 * 
 * cdef class _RslSweep:             # <<<<<<<<<<<<<<
 *     """
 *     A object for accessing RSL Sweep data and header information.
 */
struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep {
  PyObject_HEAD
  struct __pyx_vtabstruct_5pyart_2io_14_rsl_interface__RslSweep *__pyx_vtab;
  Sweep *_Sweep;
  int _dealloc;
};


/* "pyart/io/_rsl_interface.pyx":1060
 * 
 * 
 * cdef class RslFile:             # <<<<<<<<<<<<<<
 *     """
 *     RslFile(filename)
 */
struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile {
  PyObject_HEAD
  Radar *_Radar;
  Volume *_Volume;
  Sweep *_Sweep;
  Ray *_Ray;
};


/* "cfunc.to_py":64
 * 
 * @cname("__Pyx_CFunc_float____Range___to_py")
 * cdef object __Pyx_CFunc_float____Range___to_py(float (*f)(Range) except *):             # <<<<<<<<<<<<<<
 *     def wrap(Range x):
 *         """wrap(x: 'Range') -> 'float'"""
 */
struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_float____Range___to_py {
  PyObject_HEAD
  float (*__pyx_v_f)(__pyx_t_5pyart_2io_14_rsl_interface_Range);
};

struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_Range____float___to_py {
  PyObject_HEAD
  __pyx_t_5pyart_2io_14_rsl_interface_Range (*__pyx_v_f)(float);
};



/* "pyart/io/_rsl_interface.pyx":754
 * 
 * 
 * cdef class _RslVolume:             # <<<<<<<<<<<<<<
 *     """
 *     A object for accessing RSL Volume data and header information.
 */

struct __pyx_vtabstruct_5pyart_2io_14_rsl_interface__RslVolume {
  PyObject *(*load)(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *, Volume *);
  PyObject *(*_prtmode)(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *, Ray_header);
};
static struct __pyx_vtabstruct_5pyart_2io_14_rsl_interface__RslVolume *__pyx_vtabptr_5pyart_2io_14_rsl_interface__RslVolume;


/* "pyart/io/_rsl_interface.pyx":192
 * 
 * 
 * cdef class _RslRay:             # <<<<<<<<<<<<<<
 *     """
 *     A object for accessing RSL Ray data and header information
 */

struct __pyx_vtabstruct_5pyart_2io_14_rsl_interface__RslRay {
  PyObject *(*load)(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *, Ray *);
};
static struct __pyx_vtabstruct_5pyart_2io_14_rsl_interface__RslRay *__pyx_vtabptr_5pyart_2io_14_rsl_interface__RslRay;


/* "pyart/io/_rsl_interface.pyx":612
 * 
 * 
 * cdef class _RslSweep:             # <<<<<<<<<<<<<<
 *     """
 *     A object for accessing RSL Sweep data and header information.
 */

struct __pyx_vtabstruct_5pyart_2io_14_rsl_interface__RslSweep {
  PyObject *(*load)(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *, Sweep *);
};
static struct __pyx_vtabstruct_5pyart_2io_14_rsl_interface__RslSweep *__pyx_vtabptr_5pyart_2io_14_rsl_interface__RslSweep;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
#if CYTHON_FAST_PYCALL
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif // CYTHON_FAST_PYCALL
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* IsLittleEndian.proto */
static CYTHON_INLINE int __Pyx_Is_Little_Endian(void);

/* BufferFormatCheck.proto */
static const char* __Pyx_BufFmt_CheckString(__Pyx_BufFmt_Context* ctx, const char* ts);
static void __Pyx_BufFmt_Init(__Pyx_BufFmt_Context* ctx,
                              __Pyx_BufFmt_StackElem* stack,
                              __Pyx_TypeInfo* type);

/* BufferGetAndValidate.proto */
#define __Pyx_GetBufferAndValidate(buf, obj, dtype, flags, nd, cast, stack)\
    ((obj == Py_None || obj == NULL) ?\
    (__Pyx_ZeroBuffer(buf), 0) :\
    __Pyx__GetBufferAndValidate(buf, obj, dtype, flags, nd, cast, stack))
static int  __Pyx__GetBufferAndValidate(Py_buffer* buf, PyObject* obj,
    __Pyx_TypeInfo* dtype, int flags, int nd, int cast, __Pyx_BufFmt_StackElem* stack);
static void __Pyx_ZeroBuffer(Py_buffer* buf);
static CYTHON_INLINE void __Pyx_SafeReleaseBuffer(Py_buffer* info);
static Py_ssize_t __Pyx_minusones[] = { -1, -1, -1, -1, -1, -1, -1, -1 };
static Py_ssize_t __Pyx_zeros[] = { 0, 0, 0, 0, 0, 0, 0, 0 };

/* BufferIndexError.proto */
static void __Pyx_RaiseBufferIndexError(int axis);

#define __Pyx_BufPtrStrided1d(type, buf, i0, s0) (type)((char*)buf + i0 * s0)
/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_AddObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* PyIntCompare.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyFloatBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyFloat_AddObjC(PyObject *op1, PyObject *op2, double floatval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyFloat_AddObjC(op1, op2, floatval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* BufferFallbackError.proto */
static void __Pyx_RaiseBufferFallbackError(void);

#define __Pyx_BufPtrStrided2d(type, buf, i0, s0, i1, s1) (type)((char*)buf + i0 * s0 + i1 * s1)
/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* py_abs.proto */
#if CYTHON_USE_PYLONG_INTERNALS
static PyObject *__Pyx_PyLong_AbsNeg(PyObject *num);
#define __Pyx_PyNumber_Absolute(x)\
    ((likely(PyLong_CheckExact(x))) ?\
         (likely(Py_SIZE(x) >= 0) ? (Py_INCREF(x), (x)) : __Pyx_PyLong_AbsNeg(x)) :\
         PyNumber_Absolute(x))
#else
#define __Pyx_PyNumber_Absolute(x)  PyNumber_Absolute(x)
#endif

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* CythonFunctionShared.proto */
#define __Pyx_CyFunction_USED 1
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#define __Pyx_CyFunction_GetClassObj(f)\
    (((__pyx_CyFunctionObject *) (f))->func_classobj)
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
    PyCFunctionObject func;
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
    PyObject *func_classobj;
    void *defaults;
    int defaults_pyobjects;
    size_t defaults_size;  // used by FusedFunction for copying defaults
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
} __pyx_CyFunctionObject;
static PyTypeObject *__pyx_CyFunctionType = 0;
#define __Pyx_CyFunction_Check(obj)  (__Pyx_TypeCheck(obj, __pyx_CyFunctionType))
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *self,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(void);

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* BufferStructDeclare.proto */
typedef struct {
  Py_ssize_t shape, strides, suboffsets;
} __Pyx_Buf_DimInfo;
typedef struct {
  size_t refcount;
  Py_buffer pybuffer;
} __Pyx_Buffer;
typedef struct {
  __Pyx_Buffer *rcbuffer;
  char *data;
  __Pyx_Buf_DimInfo diminfo[8];
} __Pyx_LocalBuf_ND;

#if PY_MAJOR_VERSION < 3
    static int __Pyx_GetBuffer(PyObject *obj, Py_buffer *view, int flags);
    static void __Pyx_ReleaseBuffer(Py_buffer *view);
#else
    #define __Pyx_GetBuffer PyObject_GetBuffer
    #define __Pyx_ReleaseBuffer PyBuffer_Release
#endif


/* GCCDiagnostics.proto */
#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

static PyObject* __pyx_convert__to_py_Radar_header(Radar_header s);
/* RealImag.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if defined(__cplusplus) && CYTHON_CCOMPLEX\
        && (defined(_WIN32) || defined(__clang__) || (defined(__GNUC__) && (__GNUC__ >= 5 || __GNUC__ == 4 && __GNUC_MINOR__ >= 4 )) || __cplusplus >= 201103)
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_float(a, b)   ((a)==(b))
    #define __Pyx_c_sum_float(a, b)  ((a)+(b))
    #define __Pyx_c_diff_float(a, b) ((a)-(b))
    #define __Pyx_c_prod_float(a, b) ((a)*(b))
    #define __Pyx_c_quot_float(a, b) ((a)/(b))
    #define __Pyx_c_neg_float(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_float(z) ((z)==(float)0)
    #define __Pyx_c_conj_float(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (::std::abs(z))
        #define __Pyx_c_pow_float(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_float(z) ((z)==0)
    #define __Pyx_c_conj_float(z)    (conjf(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (cabsf(z))
        #define __Pyx_c_pow_float(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_double(a, b)   ((a)==(b))
    #define __Pyx_c_sum_double(a, b)  ((a)+(b))
    #define __Pyx_c_diff_double(a, b) ((a)-(b))
    #define __Pyx_c_prod_double(a, b) ((a)*(b))
    #define __Pyx_c_quot_double(a, b) ((a)/(b))
    #define __Pyx_c_neg_double(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_double(z) ((z)==(double)0)
    #define __Pyx_c_conj_double(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (::std::abs(z))
        #define __Pyx_c_pow_double(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_double(z) ((z)==0)
    #define __Pyx_c_conj_double(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (cabs(z))
        #define __Pyx_c_pow_double(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE Range __Pyx_PyInt_As_Range(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_Range(Range value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_Py_intptr_t(Py_intptr_t value);

/* CIntFromPy.proto */
static CYTHON_INLINE Py_intptr_t __Pyx_PyInt_As_Py_intptr_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_npy_int32(npy_int32 value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntFromPy.proto */
static CYTHON_INLINE unsigned short __Pyx_PyInt_As_unsigned_short(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_short(unsigned short value);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static PyObject *__pyx_f_5pyart_2io_14_rsl_interface_7_RslRay_load(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, Ray *__pyx_v_Ray); /* proto*/
static PyObject *__pyx_f_5pyart_2io_14_rsl_interface_9_RslSweep_load(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self, Sweep *__pyx_v_Sweep); /* proto*/
static PyObject *__pyx_f_5pyart_2io_14_rsl_interface_10_RslVolume_load(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self, Volume *__pyx_v_Volume); /* proto*/
static PyObject *__pyx_f_5pyart_2io_14_rsl_interface_10_RslVolume__prtmode(CYTHON_UNUSED struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self, Ray_header __pyx_v_h); /* proto*/

/* Module declarations from 'pyart.io._rsl_h' */

/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'numpy' */

/* Module declarations from 'numpy' */
static PyTypeObject *__pyx_ptype_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_5numpy_flatiter = 0;
static PyTypeObject *__pyx_ptype_5numpy_broadcast = 0;
static PyTypeObject *__pyx_ptype_5numpy_ndarray = 0;
static PyTypeObject *__pyx_ptype_5numpy_generic = 0;
static PyTypeObject *__pyx_ptype_5numpy_number = 0;
static PyTypeObject *__pyx_ptype_5numpy_integer = 0;
static PyTypeObject *__pyx_ptype_5numpy_signedinteger = 0;
static PyTypeObject *__pyx_ptype_5numpy_unsignedinteger = 0;
static PyTypeObject *__pyx_ptype_5numpy_inexact = 0;
static PyTypeObject *__pyx_ptype_5numpy_floating = 0;
static PyTypeObject *__pyx_ptype_5numpy_complexfloating = 0;
static PyTypeObject *__pyx_ptype_5numpy_flexible = 0;
static PyTypeObject *__pyx_ptype_5numpy_character = 0;
static PyTypeObject *__pyx_ptype_5numpy_ufunc = 0;

/* Module declarations from 'pyart.io._rsl_interface' */
static PyTypeObject *__pyx_ptype_5pyart_2io_14_rsl_interface__RslVolume = 0;
static PyTypeObject *__pyx_ptype_5pyart_2io_14_rsl_interface__RslRay = 0;
static PyTypeObject *__pyx_ptype_5pyart_2io_14_rsl_interface__RslSweep = 0;
static PyTypeObject *__pyx_ptype_5pyart_2io_14_rsl_interface_RslFile = 0;
static PyTypeObject *__pyx_ptype___pyx_scope_struct____Pyx_CFunc_float____Range___to_py = 0;
static PyTypeObject *__pyx_ptype___pyx_scope_struct____Pyx_CFunc_Range____float___to_py = 0;
static PyObject *__pyx_f_5pyart_2io_14_rsl_interface_copy_volume(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_5pyart_2io_14_rsl_interface_create_volume(PyArrayObject *, PyArrayObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_5pyart_2io_14_rsl_interface_create_volume *__pyx_optional_args); /*proto*/
static PyObject *__pyx_f_5pyart_2io_14_rsl_interface__label_volume(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *, PyObject *, int __pyx_skip_dispatch); /*proto*/
static PyObject *__Pyx_CFunc_float____Range___to_py(float (*)(__pyx_t_5pyart_2io_14_rsl_interface_Range)); /*proto*/
static PyObject *__Pyx_CFunc_Range____float___to_py(__pyx_t_5pyart_2io_14_rsl_interface_Range (*)(float)); /*proto*/
static __Pyx_TypeInfo __Pyx_TypeInfo_nn___pyx_t_5numpy_float32_t = { "float32_t", NULL, sizeof(__pyx_t_5numpy_float32_t), { 0 }, 0, 'R', 0, 0 };
static __Pyx_TypeInfo __Pyx_TypeInfo_nn___pyx_t_5numpy_int32_t = { "int32_t", NULL, sizeof(__pyx_t_5numpy_int32_t), { 0 }, 0, IS_UNSIGNED(__pyx_t_5numpy_int32_t) ? 'U' : 'I', IS_UNSIGNED(__pyx_t_5numpy_int32_t), 0 };
#define __Pyx_MODULE_NAME "pyart.io._rsl_interface"
extern int __pyx_module_is_main_pyart__io___rsl_interface;
int __pyx_module_is_main_pyart__io___rsl_interface = 0;

/* Implementation of 'pyart.io._rsl_interface' */
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_IOError;
static PyObject *__pyx_builtin_ImportError;
static const char __pyx_k_x[] = "x";
static const char __pyx_k_np[] = "np";
static const char __pyx_k_S24[] = "|S24";
static const char __pyx_k_arr[] = "arr";
static const char __pyx_k_day[] = "day";
static const char __pyx_k_sec[] = "sec";
static const char __pyx_k_sum[] = "sum";
static const char __pyx_k_vcp[] = "vcp";
static const char __pyx_k_city[] = "city";
static const char __pyx_k_data[] = "data";
static const char __pyx_k_dual[] = "dual                    ";
static const char __pyx_k_hour[] = "hour";
static const char __pyx_k_latd[] = "latd";
static const char __pyx_k_latm[] = "latm";
static const char __pyx_k_lats[] = "lats";
static const char __pyx_k_lond[] = "lond";
static const char __pyx_k_lonm[] = "lonm";
static const char __pyx_k_lons[] = "lons";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_wrap[] = "wrap";
static const char __pyx_k_year[] = "year";
static const char __pyx_k_dtype[] = "dtype";
static const char __pyx_k_empty[] = "empty";
static const char __pyx_k_fixed[] = "fixed                   ";
static const char __pyx_k_int32[] = "int32";
static const char __pyx_k_month[] = "month";
static const char __pyx_k_numpy[] = "numpy";
static const char __pyx_k_radar[] = "radar";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_shape[] = "shape";
static const char __pyx_k_state[] = "state";
static const char __pyx_k_zeros[] = "zeros";
static const char __pyx_k_RslRay[] = "_RslRay";
static const char __pyx_k_callid[] = "callid";
static const char __pyx_k_height[] = "height";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_lpulse[] = "lpulse";
static const char __pyx_k_minute[] = "minute";
static const char __pyx_k_name_2[] = "name";
static const char __pyx_k_number[] = "number";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_spulse[] = "spulse";
static const char __pyx_k_volume[] = "volume";
static const char __pyx_k_wsr88d[] = "wsr88d";
static const char __pyx_k_IOError[] = "IOError";
static const char __pyx_k_RslFile[] = "RslFile";
static const char __pyx_k_azimuth[] = "azimuth";
static const char __pyx_k_country[] = "country";
static const char __pyx_k_float32[] = "float32";
static const char __pyx_k_nsweeps[] = "nsweeps";
static const char __pyx_k_project[] = "project";
static const char __pyx_k_seconds[] = "seconds";
static const char __pyx_k_vol_num[] = "vol_num";
static const char __pyx_k_RslSweep[] = "_RslSweep";
static const char __pyx_k_altitude[] = "altitude";
static const char __pyx_k_datetime[] = "datetime";
static const char __pyx_k_filename[] = "filename";
static const char __pyx_k_get_data[] = "get_data";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_nvolumes[] = "nvolumes";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_RslVolume[] = "_RslVolume";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_elevation[] = "elevation";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_scan_mode[] = "scan_mode";
static const char __pyx_k_timedelta[] = "timedelta";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_get_volume[] = "get_volume";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_radar_name[] = "radar_name";
static const char __pyx_k_radar_type[] = "radar_type";
static const char __pyx_k_total_rays[] = "total_rays";
static const char __pyx_k_ImportError[] = "ImportError";
static const char __pyx_k_cfunc_to_py[] = "cfunc.to_py";
static const char __pyx_k_radar_format[] = "radar_format";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_get_nray_array[] = "get_nray_array";
static const char __pyx_k_rays_per_sweep[] = "rays_per_sweep";
static const char __pyx_k_RSL_VERSION_STR[] = "_RSL_VERSION_STR";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_nyquist_velocity[] = "nyquist_velocity";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_invalid_ray_number[] = "invalid ray_number";
static const char __pyx_k_file_cannot_be_read[] = "file cannot be read.";
static const char __pyx_k_invalid_radar_format[] = "invalid radar_format:";
static const char __pyx_k_invalid_sweep_number[] = "invalid sweep_number";
static const char __pyx_k_meters_between_gates[] = "meters_between_gates";
static const char __pyx_k_instrument_parameters[] = "instrument_parameters";
static const char __pyx_k_invalid_volume_number[] = "invalid volume_number";
static const char __pyx_k_callid_must_be_provided[] = "callid must be provided.";
static const char __pyx_k_Pyx_CFunc_Range____float___to[] = "__Pyx_CFunc_Range____float___to_py.<locals>.wrap";
static const char __pyx_k_Pyx_CFunc_float____Range___to[] = "__Pyx_CFunc_float____Range___to_py.<locals>.wrap";
static const char __pyx_k_cannot_load__RslRay_with_NULL[] = "cannot load _RslRay with NULL";
static const char __pyx_k_Cython_wrapper_around_the_NASA[] = "\nCython wrapper around the NASA TRMM RSL library.\n\n";
static const char __pyx_k_meters_to_center_of_first_gate[] = "meters_to_center_of_first_gate";
static const char __pyx_k_cannot_load__RslSweep_with_NULL[] = "cannot load _RslSweep with NULL";
static const char __pyx_k_numpy_core_multiarray_failed_to[] = "numpy.core.multiarray failed to import";
static const char __pyx_k_self__Sweep_cannot_be_converted[] = "self._Sweep cannot be converted to a Python object for pickling";
static const char __pyx_k_cannot_load__RslVolume_with_NULL[] = "cannot load _RslVolume with NULL";
static const char __pyx_k_no_default___reduce___due_to_non[] = "no default __reduce__ due to non-trivial __cinit__";
static const char __pyx_k_numpy_core_umath_failed_to_impor[] = "numpy.core.umath failed to import";
static const char __pyx_k_self__Ray_cannot_be_converted_to[] = "self._Ray cannot be converted to a Python object for pickling";
static const char __pyx_k_self__Volume_cannot_be_converted[] = "self._Volume cannot be converted to a Python object for pickling";
static PyObject *__pyx_n_s_IOError;
static PyObject *__pyx_n_s_ImportError;
static PyObject *__pyx_n_s_Pyx_CFunc_Range____float___to;
static PyObject *__pyx_n_s_Pyx_CFunc_float____Range___to;
static PyObject *__pyx_n_s_RSL_VERSION_STR;
static PyObject *__pyx_n_s_RslFile;
static PyObject *__pyx_n_s_RslRay;
static PyObject *__pyx_n_s_RslSweep;
static PyObject *__pyx_n_s_RslVolume;
static PyObject *__pyx_kp_u_S24;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_n_s_altitude;
static PyObject *__pyx_n_s_arr;
static PyObject *__pyx_n_s_azimuth;
static PyObject *__pyx_n_s_callid;
static PyObject *__pyx_kp_u_callid_must_be_provided;
static PyObject *__pyx_kp_u_cannot_load__RslRay_with_NULL;
static PyObject *__pyx_kp_u_cannot_load__RslSweep_with_NULL;
static PyObject *__pyx_kp_u_cannot_load__RslVolume_with_NULL;
static PyObject *__pyx_n_s_cfunc_to_py;
static PyObject *__pyx_n_s_city;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_country;
static PyObject *__pyx_n_u_data;
static PyObject *__pyx_n_s_datetime;
static PyObject *__pyx_n_s_day;
static PyObject *__pyx_n_s_dtype;
static PyObject *__pyx_kp_u_dual;
static PyObject *__pyx_n_s_elevation;
static PyObject *__pyx_n_s_empty;
static PyObject *__pyx_kp_u_file_cannot_be_read;
static PyObject *__pyx_n_s_filename;
static PyObject *__pyx_kp_u_fixed;
static PyObject *__pyx_n_u_float32;
static PyObject *__pyx_n_s_get_data;
static PyObject *__pyx_n_s_get_nray_array;
static PyObject *__pyx_n_s_get_volume;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_height;
static PyObject *__pyx_n_s_hour;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_instrument_parameters;
static PyObject *__pyx_n_u_int32;
static PyObject *__pyx_kp_u_invalid_radar_format;
static PyObject *__pyx_kp_u_invalid_ray_number;
static PyObject *__pyx_kp_u_invalid_sweep_number;
static PyObject *__pyx_kp_u_invalid_volume_number;
static PyObject *__pyx_n_s_latd;
static PyObject *__pyx_n_s_latm;
static PyObject *__pyx_n_s_lats;
static PyObject *__pyx_n_s_lond;
static PyObject *__pyx_n_s_lonm;
static PyObject *__pyx_n_s_lons;
static PyObject *__pyx_n_s_lpulse;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_u_meters_between_gates;
static PyObject *__pyx_n_u_meters_to_center_of_first_gate;
static PyObject *__pyx_n_s_minute;
static PyObject *__pyx_n_s_month;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_name_2;
static PyObject *__pyx_kp_s_no_default___reduce___due_to_non;
static PyObject *__pyx_n_s_np;
static PyObject *__pyx_n_s_nsweeps;
static PyObject *__pyx_n_s_number;
static PyObject *__pyx_n_s_numpy;
static PyObject *__pyx_kp_u_numpy_core_multiarray_failed_to;
static PyObject *__pyx_kp_u_numpy_core_umath_failed_to_impor;
static PyObject *__pyx_n_s_nvolumes;
static PyObject *__pyx_n_u_nyquist_velocity;
static PyObject *__pyx_n_s_project;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_radar;
static PyObject *__pyx_n_s_radar_format;
static PyObject *__pyx_n_s_radar_name;
static PyObject *__pyx_n_s_radar_type;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_rays_per_sweep;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_scan_mode;
static PyObject *__pyx_n_s_sec;
static PyObject *__pyx_n_s_seconds;
static PyObject *__pyx_kp_s_self__Ray_cannot_be_converted_to;
static PyObject *__pyx_kp_s_self__Sweep_cannot_be_converted;
static PyObject *__pyx_kp_s_self__Volume_cannot_be_converted;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_shape;
static PyObject *__pyx_n_u_shape;
static PyObject *__pyx_n_s_spulse;
static PyObject *__pyx_n_s_state;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_sum;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_timedelta;
static PyObject *__pyx_n_s_total_rays;
static PyObject *__pyx_n_s_vcp;
static PyObject *__pyx_n_s_vol_num;
static PyObject *__pyx_n_s_volume;
static PyObject *__pyx_n_s_wrap;
static PyObject *__pyx_n_u_wsr88d;
static PyObject *__pyx_n_s_x;
static PyObject *__pyx_n_s_year;
static PyObject *__pyx_n_s_zeros;
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_copy_volume(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_volume); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_2create_volume(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_arr, PyArrayObject *__pyx_v_rays_per_sweep, int __pyx_v_vol_num); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_4_label_volume(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_volume, PyObject *__pyx_v_radar); /* proto */
static void __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay___dealloc__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_2get_datetime(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4get_data(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_5month___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_5month_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_month); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3day___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3day_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_day); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4year___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4year_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_year); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4hour___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4hour_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_hour); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_6minute___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_6minute_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_minute); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3sec___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3sec_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_sec); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_8unam_rng___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_8unam_rng_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_unam_rng); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7azimuth___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7azimuth_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_azimuth); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7ray_num___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7ray_num_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_ray_num); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4elev___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4elev_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_elev); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_8elev_num___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_8elev_num_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_elev_num); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10range_bin1___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10range_bin1_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_range_bin1); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9gate_size___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9gate_size_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_gate_size); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7vel_res___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7vel_res_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_vel_res); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10sweep_rate___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10sweep_rate_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_sweep_rate); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3prf___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3prf_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_prf); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4prf2___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4prf2_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_prf2); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9azim_rate___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9azim_rate_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_azim_rate); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9fix_angle___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9fix_angle_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_fix_angle); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_5pitch___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_5pitch_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_pitch); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4roll___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4roll_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_roll); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7heading___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7heading_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_heading); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10pitch_rate___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10pitch_rate_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_pitch_rate); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9roll_rate___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9roll_rate_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_roll_rate); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_12heading_rate___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_12heading_rate_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_heading_rate); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3lat___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3lat_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_lat); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3lon___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3lon_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_lon); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3alt___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3alt_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_alt); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3rvc___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3rvc_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_rvc); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_8vel_east___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_8vel_east_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_vel_east); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9vel_north___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9vel_north_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_vel_north); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_6vel_up___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_6vel_up_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_vel_up); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_11pulse_count___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_11pulse_count_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_pulse_count); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_11pulse_width___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_11pulse_width_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_pulse_width); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10beam_width___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10beam_width_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_beam_width); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9frequency___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9frequency_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_frequency); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10wavelength___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10wavelength_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_wavelength); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7nyq_vel___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7nyq_vel_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_nyq_vel); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_5nbins___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_5nbins_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_nbins); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static void __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep___dealloc__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_2get_ray(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self, int __pyx_v_ray_number); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_4get_data(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_9sweep_num___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_9sweep_num_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self, int __pyx_v_sweep_num); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_4elev___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_4elev_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self, float __pyx_v_elev); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_7azimuth___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_7azimuth_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self, float __pyx_v_azimuth); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_10beam_width___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_10beam_width_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self, float __pyx_v_beam_width); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_12vert_half_bw___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_12vert_half_bw_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self, float __pyx_v_vert_half_bw); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_12horz_half_bw___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_12horz_half_bw_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self, float __pyx_v_horz_half_bw); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_5nrays___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_5nrays_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self, int __pyx_v_nrays); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static void __pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume___dealloc__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_2total_rays(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_4get_nray_array(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_6get_sweep(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self, int __pyx_v_sweep_number); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_8get_azimuth_and_elev_array(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_10get_sweep_fix_angles(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_12get_sweep_azimuths(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_14get_sweep_elevs(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_16get_instr_params(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_18get_data(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_20is_range_bins_uniform(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_7nsweeps___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_7nsweeps_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self, int __pyx_v_nsweeps); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_12calibr_const___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_12calibr_const_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self, float __pyx_v_calibr_const); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_22__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_24__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile___cinit__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, PyObject *__pyx_v_filename, PyObject *__pyx_v_radar_format, PyObject *__pyx_v_callid); /* proto */
static void __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_2__dealloc__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4get_volume(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_volume_number); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6available_moments(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_8get_radar_header(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_10get_volume_array(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_volume_num); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_5month___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_5month_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_month); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_3day___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_3day_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_day); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4year___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4year_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_year); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4hour___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4hour_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_hour); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6minute___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6minute_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_minute); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_3sec___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_3sec_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, float __pyx_v_sec); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_8nvolumes___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_8nvolumes_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_nvolumes); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6number___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6number_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_number); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4latd___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4latd_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_latd); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4latm___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4latm_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_latm); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4lats___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4lats_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_lats); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4lond___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4lond_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_lond); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4lonm___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4lonm_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_lonm); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4lons___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4lons_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_lons); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6height___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6height_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_height); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6spulse___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6spulse_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_spulse); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6lpulse___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6lpulse_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_lpulse); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_9scan_mode___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_9scan_mode_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_scan_mode); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_3vcp___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self); /* proto */
static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_3vcp_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_vcp); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_12__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_14__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_11cfunc_dot_to_py_34__Pyx_CFunc_float____Range___to_py_wrap(PyObject *__pyx_self, __pyx_t_5pyart_2io_14_rsl_interface_Range __pyx_v_x); /* proto */
static PyObject *__pyx_pf_11cfunc_dot_to_py_34__Pyx_CFunc_Range____float___to_py_wrap(PyObject *__pyx_self, float __pyx_v_x); /* proto */
static PyObject *__pyx_tp_new_5pyart_2io_14_rsl_interface__RslVolume(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pyart_2io_14_rsl_interface__RslRay(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pyart_2io_14_rsl_interface__RslSweep(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pyart_2io_14_rsl_interface_RslFile(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new___pyx_scope_struct____Pyx_CFunc_float____Range___to_py(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new___pyx_scope_struct____Pyx_CFunc_Range____float___to_py(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_float_0_1;
static PyObject *__pyx_float_1e6;
static PyObject *__pyx_float_1_31072000e_05;
static PyObject *__pyx_float_neg_999_;
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_23;
static PyObject *__pyx_int_24;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_codeobj__20;
static PyObject *__pyx_codeobj__22;
/* Late includes */

/* "pyart/io/_rsl_interface.pyx":21
 * 
 * 
 * cpdef copy_volume(_RslVolume volume):             # <<<<<<<<<<<<<<
 *     """
 *     copy_volume(volume)
 */

static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_1copy_volume(PyObject *__pyx_self, PyObject *__pyx_v_volume); /*proto*/
static PyObject *__pyx_f_5pyart_2io_14_rsl_interface_copy_volume(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_volume, CYTHON_UNUSED int __pyx_skip_dispatch) {
  Volume *__pyx_v_volume_copy;
  struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_rslvolume = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy_volume", 0);

  /* "pyart/io/_rsl_interface.pyx":38
 * 
 *     """
 *     volume_copy = _rsl_h.RSL_copy_volume(volume._Volume)             # <<<<<<<<<<<<<<
 *     rslvolume = _RslVolume()
 *     rslvolume.load(volume_copy)
 */
  __pyx_v_volume_copy = RSL_copy_volume(__pyx_v_volume->_Volume);

  /* "pyart/io/_rsl_interface.pyx":39
 *     """
 *     volume_copy = _rsl_h.RSL_copy_volume(volume._Volume)
 *     rslvolume = _RslVolume()             # <<<<<<<<<<<<<<
 *     rslvolume.load(volume_copy)
 *     rslvolume._dealloc = 1
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_5pyart_2io_14_rsl_interface__RslVolume)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_rslvolume = ((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_rsl_interface.pyx":40
 *     volume_copy = _rsl_h.RSL_copy_volume(volume._Volume)
 *     rslvolume = _RslVolume()
 *     rslvolume.load(volume_copy)             # <<<<<<<<<<<<<<
 *     rslvolume._dealloc = 1
 *     return rslvolume
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5pyart_2io_14_rsl_interface__RslVolume *)__pyx_v_rslvolume->__pyx_vtab)->load(__pyx_v_rslvolume, __pyx_v_volume_copy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_rsl_interface.pyx":41
 *     rslvolume = _RslVolume()
 *     rslvolume.load(volume_copy)
 *     rslvolume._dealloc = 1             # <<<<<<<<<<<<<<
 *     return rslvolume
 * 
 */
  __pyx_v_rslvolume->_dealloc = 1;

  /* "pyart/io/_rsl_interface.pyx":42
 *     rslvolume.load(volume_copy)
 *     rslvolume._dealloc = 1
 *     return rslvolume             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_rslvolume));
  __pyx_r = ((PyObject *)__pyx_v_rslvolume);
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":21
 * 
 * 
 * cpdef copy_volume(_RslVolume volume):             # <<<<<<<<<<<<<<
 *     """
 *     copy_volume(volume)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface.copy_volume", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_rslvolume);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_1copy_volume(PyObject *__pyx_self, PyObject *__pyx_v_volume); /*proto*/
static char __pyx_doc_5pyart_2io_14_rsl_interface_copy_volume[] = "\n    copy_volume(volume)\n\n    Return a copy of a _RslVolume object.\n\n    Parameters\n    ----------\n    volume : _RslVolume\n        _RslVolume object to create a copy of.\n\n    Returns\n    -------\n    nvolume : _RslVolume\n        Copy of volume.\n\n    ";
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_1copy_volume(PyObject *__pyx_self, PyObject *__pyx_v_volume) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy_volume (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_volume), __pyx_ptype_5pyart_2io_14_rsl_interface__RslVolume, 1, "volume", 0))) __PYX_ERR(0, 21, __pyx_L1_error)
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_copy_volume(__pyx_self, ((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *)__pyx_v_volume));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_copy_volume(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_volume) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy_volume", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5pyart_2io_14_rsl_interface_copy_volume(__pyx_v_volume, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface.copy_volume", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":45
 * 
 * 
 * cpdef create_volume(             # <<<<<<<<<<<<<<
 *         np.ndarray[np.float32_t, ndim=2] arr,
 *         np.ndarray[np.int32_t, ndim=1] rays_per_sweep,
 */

static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_3create_volume(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_5pyart_2io_14_rsl_interface_create_volume(PyArrayObject *__pyx_v_arr, PyArrayObject *__pyx_v_rays_per_sweep, CYTHON_UNUSED int __pyx_skip_dispatch, struct __pyx_opt_args_5pyart_2io_14_rsl_interface_create_volume *__pyx_optional_args) {
  int __pyx_v_vol_num = ((int)1);
  float (**__pyx_v_RSL_F_LIST)(Range);
  Range (**__pyx_v_RSL_INVF_LIST)(float);
  Volume *__pyx_v_volume;
  Sweep *__pyx_v_sweep;
  Ray *__pyx_v_ray;
  npy_intp __pyx_v_nsweeps;
  npy_intp __pyx_v_nbins;
  PyObject *__pyx_v_ray_index = NULL;
  npy_intp __pyx_v_nsweep;
  PyObject *__pyx_v_nrays = NULL;
  PyObject *__pyx_v_nray = NULL;
  npy_intp __pyx_v_nbin;
  struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_rslvolume = NULL;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_arr;
  __Pyx_Buffer __pyx_pybuffer_arr;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_rays_per_sweep;
  __Pyx_Buffer __pyx_pybuffer_rays_per_sweep;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  float (*__pyx_t_1[42])(Range);
  Range (*__pyx_t_2[42])(float);
  npy_intp __pyx_t_3;
  npy_intp __pyx_t_4;
  npy_intp __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  PyObject *(*__pyx_t_11)(PyObject *);
  Py_ssize_t __pyx_t_12;
  npy_intp __pyx_t_13;
  npy_intp __pyx_t_14;
  npy_intp __pyx_t_15;
  PyObject *__pyx_t_16 = NULL;
  float __pyx_t_17;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("create_volume", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_vol_num = __pyx_optional_args->vol_num;
    }
  }
  __pyx_pybuffer_arr.pybuffer.buf = NULL;
  __pyx_pybuffer_arr.refcount = 0;
  __pyx_pybuffernd_arr.data = NULL;
  __pyx_pybuffernd_arr.rcbuffer = &__pyx_pybuffer_arr;
  __pyx_pybuffer_rays_per_sweep.pybuffer.buf = NULL;
  __pyx_pybuffer_rays_per_sweep.refcount = 0;
  __pyx_pybuffernd_rays_per_sweep.data = NULL;
  __pyx_pybuffernd_rays_per_sweep.rcbuffer = &__pyx_pybuffer_rays_per_sweep;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_arr.rcbuffer->pybuffer, (PyObject*)__pyx_v_arr, &__Pyx_TypeInfo_nn___pyx_t_5numpy_float32_t, PyBUF_FORMAT| PyBUF_STRIDES, 2, 0, __pyx_stack) == -1)) __PYX_ERR(0, 45, __pyx_L1_error)
  }
  __pyx_pybuffernd_arr.diminfo[0].strides = __pyx_pybuffernd_arr.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_arr.diminfo[0].shape = __pyx_pybuffernd_arr.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_arr.diminfo[1].strides = __pyx_pybuffernd_arr.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_arr.diminfo[1].shape = __pyx_pybuffernd_arr.rcbuffer->pybuffer.shape[1];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_rays_per_sweep.rcbuffer->pybuffer, (PyObject*)__pyx_v_rays_per_sweep, &__Pyx_TypeInfo_nn___pyx_t_5numpy_int32_t, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 45, __pyx_L1_error)
  }
  __pyx_pybuffernd_rays_per_sweep.diminfo[0].strides = __pyx_pybuffernd_rays_per_sweep.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_rays_per_sweep.diminfo[0].shape = __pyx_pybuffernd_rays_per_sweep.rcbuffer->pybuffer.shape[0];

  /* "pyart/io/_rsl_interface.pyx":75
 *     """
 *     # these variables can be moved to the module level if used elsewhere
 *     cdef (float (*)(_rsl_h.Range)) * RSL_F_LIST = [             # <<<<<<<<<<<<<<
 *         _rsl_h.DZ_F, _rsl_h.VR_F, _rsl_h.SW_F, _rsl_h.CZ_F, _rsl_h.ZT_F,
 *         _rsl_h.DR_F, _rsl_h.LR_F, _rsl_h.ZD_F, _rsl_h.DM_F, _rsl_h.RH_F,
 */
  __pyx_t_1[0] = DZ_F;
  __pyx_t_1[1] = VR_F;
  __pyx_t_1[2] = SW_F;
  __pyx_t_1[3] = CZ_F;
  __pyx_t_1[4] = ZT_F;
  __pyx_t_1[5] = DR_F;
  __pyx_t_1[6] = LR_F;
  __pyx_t_1[7] = ZD_F;
  __pyx_t_1[8] = DM_F;
  __pyx_t_1[9] = RH_F;
  __pyx_t_1[10] = PH_F;
  __pyx_t_1[11] = XZ_F;
  __pyx_t_1[12] = CD_F;
  __pyx_t_1[13] = MZ_F;
  __pyx_t_1[14] = MD_F;
  __pyx_t_1[15] = ZE_F;
  __pyx_t_1[16] = VE_F;
  __pyx_t_1[17] = KD_F;
  __pyx_t_1[18] = TI_F;
  __pyx_t_1[19] = DX_F;
  __pyx_t_1[20] = CH_F;
  __pyx_t_1[21] = AH_F;
  __pyx_t_1[22] = CV_F;
  __pyx_t_1[23] = AV_F;
  __pyx_t_1[24] = SQ_F;
  __pyx_t_1[25] = VS_F;
  __pyx_t_1[26] = VL_F;
  __pyx_t_1[27] = VG_F;
  __pyx_t_1[28] = VT_F;
  __pyx_t_1[29] = NP_F;
  __pyx_t_1[30] = HC_F;
  __pyx_t_1[31] = VC_F;
  __pyx_t_1[32] = VR_F;
  __pyx_t_1[33] = SW_F;
  __pyx_t_1[34] = VR_F;
  __pyx_t_1[35] = SW_F;
  __pyx_t_1[36] = DZ_F;
  __pyx_t_1[37] = CZ_F;
  __pyx_t_1[38] = PH_F;
  __pyx_t_1[39] = SD_F;
  __pyx_t_1[40] = DZ_F;
  __pyx_t_1[41] = DZ_F;
  __pyx_v_RSL_F_LIST = __pyx_t_1;

  /* "pyart/io/_rsl_interface.pyx":86
 *         _rsl_h.DZ_F, _rsl_h.DZ_F]
 * 
 *     cdef (_rsl_h.Range (*)(float)) * RSL_INVF_LIST = [             # <<<<<<<<<<<<<<
 *         _rsl_h.DZ_INVF, _rsl_h.VR_INVF, _rsl_h.SW_INVF, _rsl_h.CZ_INVF,
 *         _rsl_h.ZT_INVF, _rsl_h.DR_INVF, _rsl_h.LR_INVF, _rsl_h.ZD_INVF,
 */
  __pyx_t_2[0] = DZ_INVF;
  __pyx_t_2[1] = VR_INVF;
  __pyx_t_2[2] = SW_INVF;
  __pyx_t_2[3] = CZ_INVF;
  __pyx_t_2[4] = ZT_INVF;
  __pyx_t_2[5] = DR_INVF;
  __pyx_t_2[6] = LR_INVF;
  __pyx_t_2[7] = ZD_INVF;
  __pyx_t_2[8] = DM_INVF;
  __pyx_t_2[9] = RH_INVF;
  __pyx_t_2[10] = PH_INVF;
  __pyx_t_2[11] = XZ_INVF;
  __pyx_t_2[12] = CD_INVF;
  __pyx_t_2[13] = MZ_INVF;
  __pyx_t_2[14] = MD_INVF;
  __pyx_t_2[15] = ZE_INVF;
  __pyx_t_2[16] = VE_INVF;
  __pyx_t_2[17] = KD_INVF;
  __pyx_t_2[18] = TI_INVF;
  __pyx_t_2[19] = DX_INVF;
  __pyx_t_2[20] = CH_INVF;
  __pyx_t_2[21] = AH_INVF;
  __pyx_t_2[22] = CV_INVF;
  __pyx_t_2[23] = AV_INVF;
  __pyx_t_2[24] = SQ_INVF;
  __pyx_t_2[25] = VS_INVF;
  __pyx_t_2[26] = VL_INVF;
  __pyx_t_2[27] = VG_INVF;
  __pyx_t_2[28] = VT_INVF;
  __pyx_t_2[29] = NP_INVF;
  __pyx_t_2[30] = HC_INVF;
  __pyx_t_2[31] = VC_INVF;
  __pyx_t_2[32] = VR_INVF;
  __pyx_t_2[33] = SW_INVF;
  __pyx_t_2[34] = VR_INVF;
  __pyx_t_2[35] = SW_INVF;
  __pyx_t_2[36] = DZ_INVF;
  __pyx_t_2[37] = CZ_INVF;
  __pyx_t_2[38] = PH_INVF;
  __pyx_t_2[39] = SD_INVF;
  __pyx_t_2[40] = DZ_INVF;
  __pyx_t_2[41] = DZ_INVF;
  __pyx_v_RSL_INVF_LIST = __pyx_t_2;

  /* "pyart/io/_rsl_interface.pyx":104
 *     cdef int ray_i
 * 
 *     nsweeps = rays_per_sweep.shape[0]             # <<<<<<<<<<<<<<
 *     nbins = arr.shape[1]
 * 
 */
  __pyx_v_nsweeps = (__pyx_v_rays_per_sweep->dimensions[0]);

  /* "pyart/io/_rsl_interface.pyx":105
 * 
 *     nsweeps = rays_per_sweep.shape[0]
 *     nbins = arr.shape[1]             # <<<<<<<<<<<<<<
 * 
 *     volume = _rsl_h.RSL_new_volume(nsweeps)
 */
  __pyx_v_nbins = (__pyx_v_arr->dimensions[1]);

  /* "pyart/io/_rsl_interface.pyx":107
 *     nbins = arr.shape[1]
 * 
 *     volume = _rsl_h.RSL_new_volume(nsweeps)             # <<<<<<<<<<<<<<
 *     volume.h.nsweeps = nsweeps
 * 
 */
  __pyx_v_volume = RSL_new_volume(__pyx_v_nsweeps);

  /* "pyart/io/_rsl_interface.pyx":108
 * 
 *     volume = _rsl_h.RSL_new_volume(nsweeps)
 *     volume.h.nsweeps = nsweeps             # <<<<<<<<<<<<<<
 * 
 *     ray_index = 0
 */
  __pyx_v_volume->h.nsweeps = __pyx_v_nsweeps;

  /* "pyart/io/_rsl_interface.pyx":110
 *     volume.h.nsweeps = nsweeps
 * 
 *     ray_index = 0             # <<<<<<<<<<<<<<
 *     for nsweep in range(nsweeps):
 *         nrays = rays_per_sweep[nsweep]
 */
  __Pyx_INCREF(__pyx_int_0);
  __pyx_v_ray_index = __pyx_int_0;

  /* "pyart/io/_rsl_interface.pyx":111
 * 
 *     ray_index = 0
 *     for nsweep in range(nsweeps):             # <<<<<<<<<<<<<<
 *         nrays = rays_per_sweep[nsweep]
 *         sweep = _rsl_h.RSL_new_sweep(nrays)
 */
  __pyx_t_3 = __pyx_v_nsweeps;
  __pyx_t_4 = __pyx_t_3;
  for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
    __pyx_v_nsweep = __pyx_t_5;

    /* "pyart/io/_rsl_interface.pyx":112
 *     ray_index = 0
 *     for nsweep in range(nsweeps):
 *         nrays = rays_per_sweep[nsweep]             # <<<<<<<<<<<<<<
 *         sweep = _rsl_h.RSL_new_sweep(nrays)
 *         volume.sweep[nsweep] = sweep
 */
    __pyx_t_6 = __pyx_v_nsweep;
    __pyx_t_7 = -1;
    if (__pyx_t_6 < 0) {
      __pyx_t_6 += __pyx_pybuffernd_rays_per_sweep.diminfo[0].shape;
      if (unlikely(__pyx_t_6 < 0)) __pyx_t_7 = 0;
    } else if (unlikely(__pyx_t_6 >= __pyx_pybuffernd_rays_per_sweep.diminfo[0].shape)) __pyx_t_7 = 0;
    if (unlikely(__pyx_t_7 != -1)) {
      __Pyx_RaiseBufferIndexError(__pyx_t_7);
      __PYX_ERR(0, 112, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_PyInt_From_npy_int32((*__Pyx_BufPtrStrided1d(__pyx_t_5numpy_int32_t *, __pyx_pybuffernd_rays_per_sweep.rcbuffer->pybuffer.buf, __pyx_t_6, __pyx_pybuffernd_rays_per_sweep.diminfo[0].strides))); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 112, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_XDECREF_SET(__pyx_v_nrays, __pyx_t_8);
    __pyx_t_8 = 0;

    /* "pyart/io/_rsl_interface.pyx":113
 *     for nsweep in range(nsweeps):
 *         nrays = rays_per_sweep[nsweep]
 *         sweep = _rsl_h.RSL_new_sweep(nrays)             # <<<<<<<<<<<<<<
 *         volume.sweep[nsweep] = sweep
 *         sweep.h.nrays = nrays
 */
    __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_v_nrays); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 113, __pyx_L1_error)
    __pyx_v_sweep = RSL_new_sweep(__pyx_t_7);

    /* "pyart/io/_rsl_interface.pyx":114
 *         nrays = rays_per_sweep[nsweep]
 *         sweep = _rsl_h.RSL_new_sweep(nrays)
 *         volume.sweep[nsweep] = sweep             # <<<<<<<<<<<<<<
 *         sweep.h.nrays = nrays
 *         for nray in range(nrays):
 */
    (__pyx_v_volume->sweep[__pyx_v_nsweep]) = __pyx_v_sweep;

    /* "pyart/io/_rsl_interface.pyx":115
 *         sweep = _rsl_h.RSL_new_sweep(nrays)
 *         volume.sweep[nsweep] = sweep
 *         sweep.h.nrays = nrays             # <<<<<<<<<<<<<<
 *         for nray in range(nrays):
 *             ray = _rsl_h.RSL_new_ray(nbins)
 */
    __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_v_nrays); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 115, __pyx_L1_error)
    __pyx_v_sweep->h.nrays = __pyx_t_7;

    /* "pyart/io/_rsl_interface.pyx":116
 *         volume.sweep[nsweep] = sweep
 *         sweep.h.nrays = nrays
 *         for nray in range(nrays):             # <<<<<<<<<<<<<<
 *             ray = _rsl_h.RSL_new_ray(nbins)
 *             sweep.ray[nray] = ray
 */
    __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_v_nrays); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 116, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (likely(PyList_CheckExact(__pyx_t_8)) || PyTuple_CheckExact(__pyx_t_8)) {
      __pyx_t_9 = __pyx_t_8; __Pyx_INCREF(__pyx_t_9); __pyx_t_10 = 0;
      __pyx_t_11 = NULL;
    } else {
      __pyx_t_10 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 116, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_11 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 116, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    for (;;) {
      if (likely(!__pyx_t_11)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_10 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_8 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_10); __Pyx_INCREF(__pyx_t_8); __pyx_t_10++; if (unlikely(0 < 0)) __PYX_ERR(0, 116, __pyx_L1_error)
          #else
          __pyx_t_8 = PySequence_ITEM(__pyx_t_9, __pyx_t_10); __pyx_t_10++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 116, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          #endif
        } else {
          if (__pyx_t_10 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_8 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_10); __Pyx_INCREF(__pyx_t_8); __pyx_t_10++; if (unlikely(0 < 0)) __PYX_ERR(0, 116, __pyx_L1_error)
          #else
          __pyx_t_8 = PySequence_ITEM(__pyx_t_9, __pyx_t_10); __pyx_t_10++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 116, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          #endif
        }
      } else {
        __pyx_t_8 = __pyx_t_11(__pyx_t_9);
        if (unlikely(!__pyx_t_8)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 116, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_8);
      }
      __Pyx_XDECREF_SET(__pyx_v_nray, __pyx_t_8);
      __pyx_t_8 = 0;

      /* "pyart/io/_rsl_interface.pyx":117
 *         sweep.h.nrays = nrays
 *         for nray in range(nrays):
 *             ray = _rsl_h.RSL_new_ray(nbins)             # <<<<<<<<<<<<<<
 *             sweep.ray[nray] = ray
 *             ray.h.nbins = nbins
 */
      __pyx_v_ray = RSL_new_ray(__pyx_v_nbins);

      /* "pyart/io/_rsl_interface.pyx":118
 *         for nray in range(nrays):
 *             ray = _rsl_h.RSL_new_ray(nbins)
 *             sweep.ray[nray] = ray             # <<<<<<<<<<<<<<
 *             ray.h.nbins = nbins
 *             ray.h.f = RSL_F_LIST[vol_num]
 */
      __pyx_t_12 = __Pyx_PyIndex_AsSsize_t(__pyx_v_nray); if (unlikely((__pyx_t_12 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 118, __pyx_L1_error)
      (__pyx_v_sweep->ray[__pyx_t_12]) = __pyx_v_ray;

      /* "pyart/io/_rsl_interface.pyx":119
 *             ray = _rsl_h.RSL_new_ray(nbins)
 *             sweep.ray[nray] = ray
 *             ray.h.nbins = nbins             # <<<<<<<<<<<<<<
 *             ray.h.f = RSL_F_LIST[vol_num]
 *             ray.h.invf = RSL_INVF_LIST[vol_num]
 */
      __pyx_v_ray->h.nbins = __pyx_v_nbins;

      /* "pyart/io/_rsl_interface.pyx":120
 *             sweep.ray[nray] = ray
 *             ray.h.nbins = nbins
 *             ray.h.f = RSL_F_LIST[vol_num]             # <<<<<<<<<<<<<<
 *             ray.h.invf = RSL_INVF_LIST[vol_num]
 *             for nbin in range(nbins):
 */
      __pyx_v_ray->h.f = (__pyx_v_RSL_F_LIST[__pyx_v_vol_num]);

      /* "pyart/io/_rsl_interface.pyx":121
 *             ray.h.nbins = nbins
 *             ray.h.f = RSL_F_LIST[vol_num]
 *             ray.h.invf = RSL_INVF_LIST[vol_num]             # <<<<<<<<<<<<<<
 *             for nbin in range(nbins):
 *                 ray.range[nbin] = ray.h.invf(arr[ray_index, nbin])
 */
      __pyx_v_ray->h.invf = (__pyx_v_RSL_INVF_LIST[__pyx_v_vol_num]);

      /* "pyart/io/_rsl_interface.pyx":122
 *             ray.h.f = RSL_F_LIST[vol_num]
 *             ray.h.invf = RSL_INVF_LIST[vol_num]
 *             for nbin in range(nbins):             # <<<<<<<<<<<<<<
 *                 ray.range[nbin] = ray.h.invf(arr[ray_index, nbin])
 *             ray_index += 1
 */
      __pyx_t_13 = __pyx_v_nbins;
      __pyx_t_14 = __pyx_t_13;
      for (__pyx_t_15 = 0; __pyx_t_15 < __pyx_t_14; __pyx_t_15+=1) {
        __pyx_v_nbin = __pyx_t_15;

        /* "pyart/io/_rsl_interface.pyx":123
 *             ray.h.invf = RSL_INVF_LIST[vol_num]
 *             for nbin in range(nbins):
 *                 ray.range[nbin] = ray.h.invf(arr[ray_index, nbin])             # <<<<<<<<<<<<<<
 *             ray_index += 1
 * 
 */
        __pyx_t_8 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nbin); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 123, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_16 = PyTuple_New(2); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 123, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_v_ray_index);
        __Pyx_GIVEREF(__pyx_v_ray_index);
        PyTuple_SET_ITEM(__pyx_t_16, 0, __pyx_v_ray_index);
        __Pyx_GIVEREF(__pyx_t_8);
        PyTuple_SET_ITEM(__pyx_t_16, 1, __pyx_t_8);
        __pyx_t_8 = 0;
        __pyx_t_8 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_arr), __pyx_t_16); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 123, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        __pyx_t_17 = __pyx_PyFloat_AsFloat(__pyx_t_8); if (unlikely((__pyx_t_17 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 123, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        (__pyx_v_ray->range[__pyx_v_nbin]) = __pyx_v_ray->h.invf(__pyx_t_17);
      }

      /* "pyart/io/_rsl_interface.pyx":124
 *             for nbin in range(nbins):
 *                 ray.range[nbin] = ray.h.invf(arr[ray_index, nbin])
 *             ray_index += 1             # <<<<<<<<<<<<<<
 * 
 *     rslvolume = _RslVolume()
 */
      __pyx_t_8 = __Pyx_PyInt_AddObjC(__pyx_v_ray_index, __pyx_int_1, 1, 1, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 124, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF_SET(__pyx_v_ray_index, __pyx_t_8);
      __pyx_t_8 = 0;

      /* "pyart/io/_rsl_interface.pyx":116
 *         volume.sweep[nsweep] = sweep
 *         sweep.h.nrays = nrays
 *         for nray in range(nrays):             # <<<<<<<<<<<<<<
 *             ray = _rsl_h.RSL_new_ray(nbins)
 *             sweep.ray[nray] = ray
 */
    }
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }

  /* "pyart/io/_rsl_interface.pyx":126
 *             ray_index += 1
 * 
 *     rslvolume = _RslVolume()             # <<<<<<<<<<<<<<
 *     rslvolume.load(volume)
 *     rslvolume._dealloc = 1
 */
  __pyx_t_9 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_5pyart_2io_14_rsl_interface__RslVolume)); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_v_rslvolume = ((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *)__pyx_t_9);
  __pyx_t_9 = 0;

  /* "pyart/io/_rsl_interface.pyx":127
 * 
 *     rslvolume = _RslVolume()
 *     rslvolume.load(volume)             # <<<<<<<<<<<<<<
 *     rslvolume._dealloc = 1
 *     return rslvolume
 */
  __pyx_t_9 = ((struct __pyx_vtabstruct_5pyart_2io_14_rsl_interface__RslVolume *)__pyx_v_rslvolume->__pyx_vtab)->load(__pyx_v_rslvolume, __pyx_v_volume); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "pyart/io/_rsl_interface.pyx":128
 *     rslvolume = _RslVolume()
 *     rslvolume.load(volume)
 *     rslvolume._dealloc = 1             # <<<<<<<<<<<<<<
 *     return rslvolume
 * 
 */
  __pyx_v_rslvolume->_dealloc = 1;

  /* "pyart/io/_rsl_interface.pyx":129
 *     rslvolume.load(volume)
 *     rslvolume._dealloc = 1
 *     return rslvolume             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_rslvolume));
  __pyx_r = ((PyObject *)__pyx_v_rslvolume);
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":45
 * 
 * 
 * cpdef create_volume(             # <<<<<<<<<<<<<<
 *         np.ndarray[np.float32_t, ndim=2] arr,
 *         np.ndarray[np.int32_t, ndim=1] rays_per_sweep,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_arr.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_rays_per_sweep.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("pyart.io._rsl_interface.create_volume", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_arr.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_rays_per_sweep.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF(__pyx_v_ray_index);
  __Pyx_XDECREF(__pyx_v_nrays);
  __Pyx_XDECREF(__pyx_v_nray);
  __Pyx_XDECREF((PyObject *)__pyx_v_rslvolume);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_3create_volume(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5pyart_2io_14_rsl_interface_2create_volume[] = "\n    create_volume(arr, rays_per_sweep, vol_num=1)\n\n    Create a _RslVolume object from a 2D float32 array.\n\n    No headers parameters except nsweeps, nrays and nbins are not set in the\n    resulting _RslVolume object.\n\n    Parameters\n    ----------\n    arr : array, 2D, float32\n        Two dimensional float32 array.\n    rays_per_sweep: array, 1D, int32\n        Array listing number of rays in each sweep.\n    vol_num : int\n        Volume number used to set f and invf in the header.  The default is\n        for velocity fields.  Useful values are 0 for reflectivity and\n        1 for velocity.\n\n    Returns\n    -------\n    volumes : _RslVolume\n        _RslVolume containing array data.\n\n    ";
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_3create_volume(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyArrayObject *__pyx_v_arr = 0;
  PyArrayObject *__pyx_v_rays_per_sweep = 0;
  int __pyx_v_vol_num;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create_volume (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_arr,&__pyx_n_s_rays_per_sweep,&__pyx_n_s_vol_num,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_arr)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_rays_per_sweep)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_volume", 0, 2, 3, 1); __PYX_ERR(0, 45, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_vol_num);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "create_volume") < 0)) __PYX_ERR(0, 45, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_arr = ((PyArrayObject *)values[0]);
    __pyx_v_rays_per_sweep = ((PyArrayObject *)values[1]);
    if (values[2]) {
      __pyx_v_vol_num = __Pyx_PyInt_As_int(values[2]); if (unlikely((__pyx_v_vol_num == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 48, __pyx_L3_error)
    } else {
      __pyx_v_vol_num = ((int)1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("create_volume", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 45, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface.create_volume", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_arr), __pyx_ptype_5numpy_ndarray, 1, "arr", 0))) __PYX_ERR(0, 46, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_rays_per_sweep), __pyx_ptype_5numpy_ndarray, 1, "rays_per_sweep", 0))) __PYX_ERR(0, 47, __pyx_L1_error)
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_2create_volume(__pyx_self, __pyx_v_arr, __pyx_v_rays_per_sweep, __pyx_v_vol_num);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_2create_volume(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_arr, PyArrayObject *__pyx_v_rays_per_sweep, int __pyx_v_vol_num) {
  __Pyx_LocalBuf_ND __pyx_pybuffernd_arr;
  __Pyx_Buffer __pyx_pybuffer_arr;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_rays_per_sweep;
  __Pyx_Buffer __pyx_pybuffer_rays_per_sweep;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_5pyart_2io_14_rsl_interface_create_volume __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("create_volume", 0);
  __pyx_pybuffer_arr.pybuffer.buf = NULL;
  __pyx_pybuffer_arr.refcount = 0;
  __pyx_pybuffernd_arr.data = NULL;
  __pyx_pybuffernd_arr.rcbuffer = &__pyx_pybuffer_arr;
  __pyx_pybuffer_rays_per_sweep.pybuffer.buf = NULL;
  __pyx_pybuffer_rays_per_sweep.refcount = 0;
  __pyx_pybuffernd_rays_per_sweep.data = NULL;
  __pyx_pybuffernd_rays_per_sweep.rcbuffer = &__pyx_pybuffer_rays_per_sweep;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_arr.rcbuffer->pybuffer, (PyObject*)__pyx_v_arr, &__Pyx_TypeInfo_nn___pyx_t_5numpy_float32_t, PyBUF_FORMAT| PyBUF_STRIDES, 2, 0, __pyx_stack) == -1)) __PYX_ERR(0, 45, __pyx_L1_error)
  }
  __pyx_pybuffernd_arr.diminfo[0].strides = __pyx_pybuffernd_arr.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_arr.diminfo[0].shape = __pyx_pybuffernd_arr.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_arr.diminfo[1].strides = __pyx_pybuffernd_arr.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_arr.diminfo[1].shape = __pyx_pybuffernd_arr.rcbuffer->pybuffer.shape[1];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_rays_per_sweep.rcbuffer->pybuffer, (PyObject*)__pyx_v_rays_per_sweep, &__Pyx_TypeInfo_nn___pyx_t_5numpy_int32_t, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 45, __pyx_L1_error)
  }
  __pyx_pybuffernd_rays_per_sweep.diminfo[0].strides = __pyx_pybuffernd_rays_per_sweep.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_rays_per_sweep.diminfo[0].shape = __pyx_pybuffernd_rays_per_sweep.rcbuffer->pybuffer.shape[0];
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.vol_num = __pyx_v_vol_num;
  __pyx_t_1 = __pyx_f_5pyart_2io_14_rsl_interface_create_volume(__pyx_v_arr, __pyx_v_rays_per_sweep, 0, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_arr.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_rays_per_sweep.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("pyart.io._rsl_interface.create_volume", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_arr.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_rays_per_sweep.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":132
 * 
 * 
 * cpdef _label_volume(_RslVolume volume, radar):             # <<<<<<<<<<<<<<
 *     """
 *     _label_volume(volume, radar)
 */

static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_5_label_volume(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_5pyart_2io_14_rsl_interface__label_volume(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_volume, PyObject *__pyx_v_radar, CYTHON_UNUSED int __pyx_skip_dispatch) {
  Sweep *__pyx_v_sweep;
  Ray *__pyx_v_ray;
  int __pyx_v_ray_index;
  Volume *__pyx_v_vol;
  int __pyx_v_nsweeps;
  CYTHON_UNUSED int __pyx_v_nbins;
  PyObject *__pyx_v_gate_size = NULL;
  PyObject *__pyx_v_range_bin1 = NULL;
  PyObject *__pyx_v_alt = NULL;
  PyObject *__pyx_v_nyq_vels = NULL;
  PyObject *__pyx_v_azimuths = NULL;
  PyObject *__pyx_v_elevs = NULL;
  int __pyx_v_nsweep;
  int __pyx_v_nrays;
  int __pyx_v_nray;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Volume *__pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  float __pyx_t_12;
  int __pyx_t_13;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_label_volume", 0);

  /* "pyart/io/_rsl_interface.pyx":156
 *     cdef int ray_index
 * 
 *     vol = volume._Volume             # <<<<<<<<<<<<<<
 *     nsweeps = vol.h.nsweeps
 *     nbins = vol.sweep[0].ray[0].h.nbins
 */
  __pyx_t_1 = __pyx_v_volume->_Volume;
  __pyx_v_vol = __pyx_t_1;

  /* "pyart/io/_rsl_interface.pyx":157
 * 
 *     vol = volume._Volume
 *     nsweeps = vol.h.nsweeps             # <<<<<<<<<<<<<<
 *     nbins = vol.sweep[0].ray[0].h.nbins
 * 
 */
  __pyx_t_2 = __pyx_v_vol->h.nsweeps;
  __pyx_v_nsweeps = __pyx_t_2;

  /* "pyart/io/_rsl_interface.pyx":158
 *     vol = volume._Volume
 *     nsweeps = vol.h.nsweeps
 *     nbins = vol.sweep[0].ray[0].h.nbins             # <<<<<<<<<<<<<<
 * 
 *     gate_size = int(radar.range['meters_between_gates'])
 */
  __pyx_t_2 = ((__pyx_v_vol->sweep[0])->ray[0])->h.nbins;
  __pyx_v_nbins = __pyx_t_2;

  /* "pyart/io/_rsl_interface.pyx":160
 *     nbins = vol.sweep[0].ray[0].h.nbins
 * 
 *     gate_size = int(radar.range['meters_between_gates'])             # <<<<<<<<<<<<<<
 *     range_bin1 = int(radar.range['meters_to_center_of_first_gate'])
 *     if 'shape' in dir(radar.altitude['data']):
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_radar, __pyx_n_s_range); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_t_3, __pyx_n_u_meters_between_gates); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyNumber_Int(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_gate_size = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pyart/io/_rsl_interface.pyx":161
 * 
 *     gate_size = int(radar.range['meters_between_gates'])
 *     range_bin1 = int(radar.range['meters_to_center_of_first_gate'])             # <<<<<<<<<<<<<<
 *     if 'shape' in dir(radar.altitude['data']):
 *         if radar.altitude['data'].shape == ():
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_radar, __pyx_n_s_range); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_t_3, __pyx_n_u_meters_to_center_of_first_gate); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyNumber_Int(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_range_bin1 = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pyart/io/_rsl_interface.pyx":162
 *     gate_size = int(radar.range['meters_between_gates'])
 *     range_bin1 = int(radar.range['meters_to_center_of_first_gate'])
 *     if 'shape' in dir(radar.altitude['data']):             # <<<<<<<<<<<<<<
 *         if radar.altitude['data'].shape == ():
 *             alt = float(radar.altitude['data'])
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_radar, __pyx_n_s_altitude); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 162, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_t_3, __pyx_n_u_data); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 162, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyObject_Dir(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 162, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_5 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_shape, __pyx_t_3, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 162, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "pyart/io/_rsl_interface.pyx":163
 *     range_bin1 = int(radar.range['meters_to_center_of_first_gate'])
 *     if 'shape' in dir(radar.altitude['data']):
 *         if radar.altitude['data'].shape == ():             # <<<<<<<<<<<<<<
 *             alt = float(radar.altitude['data'])
 *         else:
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_radar, __pyx_n_s_altitude); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 163, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_t_3, __pyx_n_u_data); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 163, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_shape); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 163, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyObject_RichCompare(__pyx_t_3, __pyx_empty_tuple, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 163, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 163, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (__pyx_t_6) {

      /* "pyart/io/_rsl_interface.pyx":164
 *     if 'shape' in dir(radar.altitude['data']):
 *         if radar.altitude['data'].shape == ():
 *             alt = float(radar.altitude['data'])             # <<<<<<<<<<<<<<
 *         else:
 *             alt = radar.altitude['data'][0]
 */
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_radar, __pyx_n_s_altitude); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 164, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_t_4, __pyx_n_u_data); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 164, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyNumber_Float(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 164, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_v_alt = __pyx_t_4;
      __pyx_t_4 = 0;

      /* "pyart/io/_rsl_interface.pyx":163
 *     range_bin1 = int(radar.range['meters_to_center_of_first_gate'])
 *     if 'shape' in dir(radar.altitude['data']):
 *         if radar.altitude['data'].shape == ():             # <<<<<<<<<<<<<<
 *             alt = float(radar.altitude['data'])
 *         else:
 */
      goto __pyx_L4;
    }

    /* "pyart/io/_rsl_interface.pyx":166
 *             alt = float(radar.altitude['data'])
 *         else:
 *             alt = radar.altitude['data'][0]             # <<<<<<<<<<<<<<
 *     else:
 *         alt = radar.altitude['data']
 */
    /*else*/ {
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_radar, __pyx_n_s_altitude); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 166, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_t_4, __pyx_n_u_data); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 166, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_GetItemInt(__pyx_t_3, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 166, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_v_alt = __pyx_t_4;
      __pyx_t_4 = 0;
    }
    __pyx_L4:;

    /* "pyart/io/_rsl_interface.pyx":162
 *     gate_size = int(radar.range['meters_between_gates'])
 *     range_bin1 = int(radar.range['meters_to_center_of_first_gate'])
 *     if 'shape' in dir(radar.altitude['data']):             # <<<<<<<<<<<<<<
 *         if radar.altitude['data'].shape == ():
 *             alt = float(radar.altitude['data'])
 */
    goto __pyx_L3;
  }

  /* "pyart/io/_rsl_interface.pyx":168
 *             alt = radar.altitude['data'][0]
 *     else:
 *         alt = radar.altitude['data']             # <<<<<<<<<<<<<<
 * 
 *     nyq_vels = radar.instrument_parameters['nyquist_velocity']['data']
 */
  /*else*/ {
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_radar, __pyx_n_s_altitude); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 168, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_t_4, __pyx_n_u_data); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 168, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_alt = __pyx_t_3;
    __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "pyart/io/_rsl_interface.pyx":170
 *         alt = radar.altitude['data']
 * 
 *     nyq_vels = radar.instrument_parameters['nyquist_velocity']['data']             # <<<<<<<<<<<<<<
 *     azimuths = radar.azimuth['data']
 *     elevs = radar.elevation['data']
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_radar, __pyx_n_s_instrument_parameters); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_t_3, __pyx_n_u_nyquist_velocity); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_t_4, __pyx_n_u_data); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_nyq_vels = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pyart/io/_rsl_interface.pyx":171
 * 
 *     nyq_vels = radar.instrument_parameters['nyquist_velocity']['data']
 *     azimuths = radar.azimuth['data']             # <<<<<<<<<<<<<<
 *     elevs = radar.elevation['data']
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_radar, __pyx_n_s_azimuth); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_t_3, __pyx_n_u_data); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_azimuths = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "pyart/io/_rsl_interface.pyx":172
 *     nyq_vels = radar.instrument_parameters['nyquist_velocity']['data']
 *     azimuths = radar.azimuth['data']
 *     elevs = radar.elevation['data']             # <<<<<<<<<<<<<<
 * 
 *     # label the volume
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_radar, __pyx_n_s_elevation); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_t_4, __pyx_n_u_data); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_elevs = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pyart/io/_rsl_interface.pyx":175
 * 
 *     # label the volume
 *     ray_index = 0             # <<<<<<<<<<<<<<
 *     vol.h.nsweeps = nsweeps
 *     for nsweep in range(nsweeps):
 */
  __pyx_v_ray_index = 0;

  /* "pyart/io/_rsl_interface.pyx":176
 *     # label the volume
 *     ray_index = 0
 *     vol.h.nsweeps = nsweeps             # <<<<<<<<<<<<<<
 *     for nsweep in range(nsweeps):
 *         sweep = vol.sweep[nsweep]
 */
  __pyx_v_vol->h.nsweeps = __pyx_v_nsweeps;

  /* "pyart/io/_rsl_interface.pyx":177
 *     ray_index = 0
 *     vol.h.nsweeps = nsweeps
 *     for nsweep in range(nsweeps):             # <<<<<<<<<<<<<<
 *         sweep = vol.sweep[nsweep]
 *         nrays = sweep.h.nrays
 */
  __pyx_t_2 = __pyx_v_nsweeps;
  __pyx_t_7 = __pyx_t_2;
  for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
    __pyx_v_nsweep = __pyx_t_8;

    /* "pyart/io/_rsl_interface.pyx":178
 *     vol.h.nsweeps = nsweeps
 *     for nsweep in range(nsweeps):
 *         sweep = vol.sweep[nsweep]             # <<<<<<<<<<<<<<
 *         nrays = sweep.h.nrays
 *         for nray in range(nrays):
 */
    __pyx_v_sweep = (__pyx_v_vol->sweep[__pyx_v_nsweep]);

    /* "pyart/io/_rsl_interface.pyx":179
 *     for nsweep in range(nsweeps):
 *         sweep = vol.sweep[nsweep]
 *         nrays = sweep.h.nrays             # <<<<<<<<<<<<<<
 *         for nray in range(nrays):
 *             ray = sweep.ray[nray]
 */
    __pyx_t_9 = __pyx_v_sweep->h.nrays;
    __pyx_v_nrays = __pyx_t_9;

    /* "pyart/io/_rsl_interface.pyx":180
 *         sweep = vol.sweep[nsweep]
 *         nrays = sweep.h.nrays
 *         for nray in range(nrays):             # <<<<<<<<<<<<<<
 *             ray = sweep.ray[nray]
 *             ray.h.azimuth = azimuths[ray_index]
 */
    __pyx_t_9 = __pyx_v_nrays;
    __pyx_t_10 = __pyx_t_9;
    for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
      __pyx_v_nray = __pyx_t_11;

      /* "pyart/io/_rsl_interface.pyx":181
 *         nrays = sweep.h.nrays
 *         for nray in range(nrays):
 *             ray = sweep.ray[nray]             # <<<<<<<<<<<<<<
 *             ray.h.azimuth = azimuths[ray_index]
 *             ray.h.elev = elevs[ray_index]
 */
      __pyx_v_ray = (__pyx_v_sweep->ray[__pyx_v_nray]);

      /* "pyart/io/_rsl_interface.pyx":182
 *         for nray in range(nrays):
 *             ray = sweep.ray[nray]
 *             ray.h.azimuth = azimuths[ray_index]             # <<<<<<<<<<<<<<
 *             ray.h.elev = elevs[ray_index]
 *             ray.h.nyq_vel = nyq_vels[ray_index]
 */
      __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_azimuths, __pyx_v_ray_index, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 182, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_12 = __pyx_PyFloat_AsFloat(__pyx_t_3); if (unlikely((__pyx_t_12 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 182, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_v_ray->h.azimuth = __pyx_t_12;

      /* "pyart/io/_rsl_interface.pyx":183
 *             ray = sweep.ray[nray]
 *             ray.h.azimuth = azimuths[ray_index]
 *             ray.h.elev = elevs[ray_index]             # <<<<<<<<<<<<<<
 *             ray.h.nyq_vel = nyq_vels[ray_index]
 *             ray.h.range_bin1 = range_bin1
 */
      __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_elevs, __pyx_v_ray_index, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 183, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_12 = __pyx_PyFloat_AsFloat(__pyx_t_3); if (unlikely((__pyx_t_12 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 183, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_v_ray->h.elev = __pyx_t_12;

      /* "pyart/io/_rsl_interface.pyx":184
 *             ray.h.azimuth = azimuths[ray_index]
 *             ray.h.elev = elevs[ray_index]
 *             ray.h.nyq_vel = nyq_vels[ray_index]             # <<<<<<<<<<<<<<
 *             ray.h.range_bin1 = range_bin1
 *             ray.h.gate_size = gate_size
 */
      __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_nyq_vels, __pyx_v_ray_index, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 184, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_12 = __pyx_PyFloat_AsFloat(__pyx_t_3); if (unlikely((__pyx_t_12 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 184, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_v_ray->h.nyq_vel = __pyx_t_12;

      /* "pyart/io/_rsl_interface.pyx":185
 *             ray.h.elev = elevs[ray_index]
 *             ray.h.nyq_vel = nyq_vels[ray_index]
 *             ray.h.range_bin1 = range_bin1             # <<<<<<<<<<<<<<
 *             ray.h.gate_size = gate_size
 *             ray.h.alt = alt
 */
      __pyx_t_13 = __Pyx_PyInt_As_int(__pyx_v_range_bin1); if (unlikely((__pyx_t_13 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 185, __pyx_L1_error)
      __pyx_v_ray->h.range_bin1 = __pyx_t_13;

      /* "pyart/io/_rsl_interface.pyx":186
 *             ray.h.nyq_vel = nyq_vels[ray_index]
 *             ray.h.range_bin1 = range_bin1
 *             ray.h.gate_size = gate_size             # <<<<<<<<<<<<<<
 *             ray.h.alt = alt
 *             ray_index += 1
 */
      __pyx_t_13 = __Pyx_PyInt_As_int(__pyx_v_gate_size); if (unlikely((__pyx_t_13 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 186, __pyx_L1_error)
      __pyx_v_ray->h.gate_size = __pyx_t_13;

      /* "pyart/io/_rsl_interface.pyx":187
 *             ray.h.range_bin1 = range_bin1
 *             ray.h.gate_size = gate_size
 *             ray.h.alt = alt             # <<<<<<<<<<<<<<
 *             ray_index += 1
 *     return
 */
      __pyx_t_13 = __Pyx_PyInt_As_int(__pyx_v_alt); if (unlikely((__pyx_t_13 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 187, __pyx_L1_error)
      __pyx_v_ray->h.alt = __pyx_t_13;

      /* "pyart/io/_rsl_interface.pyx":188
 *             ray.h.gate_size = gate_size
 *             ray.h.alt = alt
 *             ray_index += 1             # <<<<<<<<<<<<<<
 *     return
 * 
 */
      __pyx_v_ray_index = (__pyx_v_ray_index + 1);
    }
  }

  /* "pyart/io/_rsl_interface.pyx":189
 *             ray.h.alt = alt
 *             ray_index += 1
 *     return             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":132
 * 
 * 
 * cpdef _label_volume(_RslVolume volume, radar):             # <<<<<<<<<<<<<<
 *     """
 *     _label_volume(volume, radar)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyart.io._rsl_interface._label_volume", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_gate_size);
  __Pyx_XDECREF(__pyx_v_range_bin1);
  __Pyx_XDECREF(__pyx_v_alt);
  __Pyx_XDECREF(__pyx_v_nyq_vels);
  __Pyx_XDECREF(__pyx_v_azimuths);
  __Pyx_XDECREF(__pyx_v_elevs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_5_label_volume(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5pyart_2io_14_rsl_interface_4_label_volume[] = "\n    _label_volume(volume, radar)\n\n    Add labels for dealiasing to a _RslVolume object from a radar object.\n\n    This function does not set all parameter in the _RslVolume suitable for\n    writing out the volume, rather it set those parameters which must be set\n    prior to using :py:func:`pyart.correct._fourdd_interface.fourdd_dealias`.\n\n    Parameters\n    ----------\n    volume : _RslVolume\n        Volume object to which parameters will be set as needed prior to\n        dealiasing.  Object is manipulated in-place.\n    radar : Radar\n        Radar object from which parameters are taken.\n\n    ";
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_5_label_volume(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_volume = 0;
  PyObject *__pyx_v_radar = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_label_volume (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_volume,&__pyx_n_s_radar,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_volume)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_radar)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_label_volume", 1, 2, 2, 1); __PYX_ERR(0, 132, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_label_volume") < 0)) __PYX_ERR(0, 132, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_volume = ((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *)values[0]);
    __pyx_v_radar = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_label_volume", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 132, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._label_volume", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_volume), __pyx_ptype_5pyart_2io_14_rsl_interface__RslVolume, 1, "volume", 0))) __PYX_ERR(0, 132, __pyx_L1_error)
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_4_label_volume(__pyx_self, __pyx_v_volume, __pyx_v_radar);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_4_label_volume(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_volume, PyObject *__pyx_v_radar) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_label_volume", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5pyart_2io_14_rsl_interface__label_volume(__pyx_v_volume, __pyx_v_radar, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._label_volume", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":289
 *     cdef int _dealloc
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self._dealloc == 1:
 *             _rsl_h.RSL_free_ray(self._Ray)
 */

/* Python wrapper */
static void __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_1__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_1__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay___dealloc__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay___dealloc__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "pyart/io/_rsl_interface.pyx":290
 * 
 *     def __dealloc__(self):
 *         if self._dealloc == 1:             # <<<<<<<<<<<<<<
 *             _rsl_h.RSL_free_ray(self._Ray)
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->_dealloc == 1) != 0);
  if (__pyx_t_1) {

    /* "pyart/io/_rsl_interface.pyx":291
 *     def __dealloc__(self):
 *         if self._dealloc == 1:
 *             _rsl_h.RSL_free_ray(self._Ray)             # <<<<<<<<<<<<<<
 * 
 *     cdef load(self, _rsl_h.Ray * Ray):
 */
    RSL_free_ray(__pyx_v_self->_Ray);

    /* "pyart/io/_rsl_interface.pyx":290
 * 
 *     def __dealloc__(self):
 *         if self._dealloc == 1:             # <<<<<<<<<<<<<<
 *             _rsl_h.RSL_free_ray(self._Ray)
 * 
 */
  }

  /* "pyart/io/_rsl_interface.pyx":289
 *     cdef int _dealloc
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self._dealloc == 1:
 *             _rsl_h.RSL_free_ray(self._Ray)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "pyart/io/_rsl_interface.pyx":293
 *             _rsl_h.RSL_free_ray(self._Ray)
 * 
 *     cdef load(self, _rsl_h.Ray * Ray):             # <<<<<<<<<<<<<<
 *         """ Load the _RslRay object, must be called after creation. """
 *         if Ray is NULL:
 */

static PyObject *__pyx_f_5pyart_2io_14_rsl_interface_7_RslRay_load(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, Ray *__pyx_v_Ray) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("load", 0);

  /* "pyart/io/_rsl_interface.pyx":295
 *     cdef load(self, _rsl_h.Ray * Ray):
 *         """ Load the _RslRay object, must be called after creation. """
 *         if Ray is NULL:             # <<<<<<<<<<<<<<
 *             raise ValueError('cannot load _RslRay with NULL')
 *         self._Ray = Ray
 */
  __pyx_t_1 = ((__pyx_v_Ray == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "pyart/io/_rsl_interface.pyx":296
 *         """ Load the _RslRay object, must be called after creation. """
 *         if Ray is NULL:
 *             raise ValueError('cannot load _RslRay with NULL')             # <<<<<<<<<<<<<<
 *         self._Ray = Ray
 *         self._dealloc = 0
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 296, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 296, __pyx_L1_error)

    /* "pyart/io/_rsl_interface.pyx":295
 *     cdef load(self, _rsl_h.Ray * Ray):
 *         """ Load the _RslRay object, must be called after creation. """
 *         if Ray is NULL:             # <<<<<<<<<<<<<<
 *             raise ValueError('cannot load _RslRay with NULL')
 *         self._Ray = Ray
 */
  }

  /* "pyart/io/_rsl_interface.pyx":297
 *         if Ray is NULL:
 *             raise ValueError('cannot load _RslRay with NULL')
 *         self._Ray = Ray             # <<<<<<<<<<<<<<
 *         self._dealloc = 0
 * 
 */
  __pyx_v_self->_Ray = __pyx_v_Ray;

  /* "pyart/io/_rsl_interface.pyx":298
 *             raise ValueError('cannot load _RslRay with NULL')
 *         self._Ray = Ray
 *         self._dealloc = 0             # <<<<<<<<<<<<<<
 * 
 *     def get_datetime(self):
 */
  __pyx_v_self->_dealloc = 0;

  /* "pyart/io/_rsl_interface.pyx":293
 *             _rsl_h.RSL_free_ray(self._Ray)
 * 
 *     cdef load(self, _rsl_h.Ray * Ray):             # <<<<<<<<<<<<<<
 *         """ Load the _RslRay object, must be called after creation. """
 *         if Ray is NULL:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.load", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":300
 *         self._dealloc = 0
 * 
 *     def get_datetime(self):             # <<<<<<<<<<<<<<
 *         """
 *         get_datetime()
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3get_datetime(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5pyart_2io_14_rsl_interface_7_RslRay_2get_datetime[] = "\n        get_datetime()\n\n        Return a datetime describing the date and time of the ray.\n        ";
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3get_datetime(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_datetime (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_2get_datetime(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_2get_datetime(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_s = NULL;
  PyObject *__pyx_v_full_seconds = NULL;
  PyObject *__pyx_v_fractional_seconds = NULL;
  PyObject *__pyx_v_microseconds = NULL;
  PyObject *__pyx_v_dt = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *(*__pyx_t_5)(PyObject *);
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_datetime", 0);

  /* "pyart/io/_rsl_interface.pyx":306
 *         Return a datetime describing the date and time of the ray.
 *         """
 *         s = self             # <<<<<<<<<<<<<<
 *         full_seconds, fractional_seconds = divmod(s.sec, 1)
 *         microseconds = int(fractional_seconds * 1e6)
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_v_s = __pyx_v_self;

  /* "pyart/io/_rsl_interface.pyx":307
 *         """
 *         s = self
 *         full_seconds, fractional_seconds = divmod(s.sec, 1)             # <<<<<<<<<<<<<<
 *         microseconds = int(fractional_seconds * 1e6)
 *         # Some UF writers incorrectly specify midnight as 24:00:00 rather
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_s), __pyx_n_s_sec); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 307, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Divmod(__pyx_t_1, __pyx_int_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 307, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
    PyObject* sequence = __pyx_t_2;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 307, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_1 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_3);
    #else
    __pyx_t_1 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 307, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 307, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_4 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 307, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_5 = Py_TYPE(__pyx_t_4)->tp_iternext;
    index = 0; __pyx_t_1 = __pyx_t_5(__pyx_t_4); if (unlikely(!__pyx_t_1)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_1);
    index = 1; __pyx_t_3 = __pyx_t_5(__pyx_t_4); if (unlikely(!__pyx_t_3)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_5(__pyx_t_4), 2) < 0) __PYX_ERR(0, 307, __pyx_L1_error)
    __pyx_t_5 = NULL;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 307, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }
  __pyx_v_full_seconds = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_v_fractional_seconds = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pyart/io/_rsl_interface.pyx":308
 *         s = self
 *         full_seconds, fractional_seconds = divmod(s.sec, 1)
 *         microseconds = int(fractional_seconds * 1e6)             # <<<<<<<<<<<<<<
 *         # Some UF writers incorrectly specify midnight as 24:00:00 rather
 *         # than 00:00:00.  Handle this case explicitly
 */
  __pyx_t_2 = PyNumber_Multiply(__pyx_v_fractional_seconds, __pyx_float_1e6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 308, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyNumber_Int(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 308, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_microseconds = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pyart/io/_rsl_interface.pyx":311
 *         # Some UF writers incorrectly specify midnight as 24:00:00 rather
 *         # than 00:00:00.  Handle this case explicitly
 *         if s.hour == 24:             # <<<<<<<<<<<<<<
 *             s.hour = 23
 *             s.minute = 0
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_s), __pyx_n_s_hour); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 311, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_t_3, __pyx_int_24, 24, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 311, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 311, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_6) {

    /* "pyart/io/_rsl_interface.pyx":312
 *         # than 00:00:00.  Handle this case explicitly
 *         if s.hour == 24:
 *             s.hour = 23             # <<<<<<<<<<<<<<
 *             s.minute = 0
 *             full_seconds = 0
 */
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_s), __pyx_n_s_hour, __pyx_int_23) < 0) __PYX_ERR(0, 312, __pyx_L1_error)

    /* "pyart/io/_rsl_interface.pyx":313
 *         if s.hour == 24:
 *             s.hour = 23
 *             s.minute = 0             # <<<<<<<<<<<<<<
 *             full_seconds = 0
 *             dt = datetime(s.year, s.month, s.day, s.hour, s.minute,
 */
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_s), __pyx_n_s_minute, __pyx_int_0) < 0) __PYX_ERR(0, 313, __pyx_L1_error)

    /* "pyart/io/_rsl_interface.pyx":314
 *             s.hour = 23
 *             s.minute = 0
 *             full_seconds = 0             # <<<<<<<<<<<<<<
 *             dt = datetime(s.year, s.month, s.day, s.hour, s.minute,
 *                           full_seconds, microseconds)
 */
    __Pyx_INCREF(__pyx_int_0);
    __Pyx_DECREF_SET(__pyx_v_full_seconds, __pyx_int_0);

    /* "pyart/io/_rsl_interface.pyx":315
 *             s.minute = 0
 *             full_seconds = 0
 *             dt = datetime(s.year, s.month, s.day, s.hour, s.minute,             # <<<<<<<<<<<<<<
 *                           full_seconds, microseconds)
 *             return dt + timedelta(seconds=1)
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_datetime); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 315, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_s), __pyx_n_s_year); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 315, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_s), __pyx_n_s_month); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 315, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_s), __pyx_n_s_day); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 315, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_s), __pyx_n_s_hour); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 315, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_s), __pyx_n_s_minute); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 315, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);

    /* "pyart/io/_rsl_interface.pyx":316
 *             full_seconds = 0
 *             dt = datetime(s.year, s.month, s.day, s.hour, s.minute,
 *                           full_seconds, microseconds)             # <<<<<<<<<<<<<<
 *             return dt + timedelta(seconds=1)
 *         return datetime(s.year, s.month, s.day, s.hour, s.minute,
 */
    __pyx_t_10 = NULL;
    __pyx_t_11 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_11 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[8] = {__pyx_t_10, __pyx_t_1, __pyx_t_4, __pyx_t_7, __pyx_t_8, __pyx_t_9, __pyx_v_full_seconds, __pyx_v_microseconds};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_11, 7+__pyx_t_11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 315, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[8] = {__pyx_t_10, __pyx_t_1, __pyx_t_4, __pyx_t_7, __pyx_t_8, __pyx_t_9, __pyx_v_full_seconds, __pyx_v_microseconds};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_11, 7+__pyx_t_11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 315, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    } else
    #endif
    {
      __pyx_t_12 = PyTuple_New(7+__pyx_t_11); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 315, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      if (__pyx_t_10) {
        __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_10); __pyx_t_10 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_1);
      PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_11, __pyx_t_1);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_11, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_7);
      PyTuple_SET_ITEM(__pyx_t_12, 2+__pyx_t_11, __pyx_t_7);
      __Pyx_GIVEREF(__pyx_t_8);
      PyTuple_SET_ITEM(__pyx_t_12, 3+__pyx_t_11, __pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_9);
      PyTuple_SET_ITEM(__pyx_t_12, 4+__pyx_t_11, __pyx_t_9);
      __Pyx_INCREF(__pyx_v_full_seconds);
      __Pyx_GIVEREF(__pyx_v_full_seconds);
      PyTuple_SET_ITEM(__pyx_t_12, 5+__pyx_t_11, __pyx_v_full_seconds);
      __Pyx_INCREF(__pyx_v_microseconds);
      __Pyx_GIVEREF(__pyx_v_microseconds);
      PyTuple_SET_ITEM(__pyx_t_12, 6+__pyx_t_11, __pyx_v_microseconds);
      __pyx_t_1 = 0;
      __pyx_t_4 = 0;
      __pyx_t_7 = 0;
      __pyx_t_8 = 0;
      __pyx_t_9 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_12, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 315, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_dt = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "pyart/io/_rsl_interface.pyx":317
 *             dt = datetime(s.year, s.month, s.day, s.hour, s.minute,
 *                           full_seconds, microseconds)
 *             return dt + timedelta(seconds=1)             # <<<<<<<<<<<<<<
 *         return datetime(s.year, s.month, s.day, s.hour, s.minute,
 *                         int(full_seconds), microseconds)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_timedelta); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 317, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 317, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_seconds, __pyx_int_1) < 0) __PYX_ERR(0, 317, __pyx_L1_error)
    __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 317, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyNumber_Add(__pyx_v_dt, __pyx_t_12); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 317, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "pyart/io/_rsl_interface.pyx":311
 *         # Some UF writers incorrectly specify midnight as 24:00:00 rather
 *         # than 00:00:00.  Handle this case explicitly
 *         if s.hour == 24:             # <<<<<<<<<<<<<<
 *             s.hour = 23
 *             s.minute = 0
 */
  }

  /* "pyart/io/_rsl_interface.pyx":318
 *                           full_seconds, microseconds)
 *             return dt + timedelta(seconds=1)
 *         return datetime(s.year, s.month, s.day, s.hour, s.minute,             # <<<<<<<<<<<<<<
 *                         int(full_seconds), microseconds)
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_n_s_datetime); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_s), __pyx_n_s_year); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_s), __pyx_n_s_month); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_s), __pyx_n_s_day); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_s), __pyx_n_s_hour); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_s), __pyx_n_s_minute); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "pyart/io/_rsl_interface.pyx":319
 *             return dt + timedelta(seconds=1)
 *         return datetime(s.year, s.month, s.day, s.hour, s.minute,
 *                         int(full_seconds), microseconds)             # <<<<<<<<<<<<<<
 * 
 *     def get_data(self):
 */
  __pyx_t_1 = __Pyx_PyNumber_Int(__pyx_v_full_seconds); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_10 = NULL;
  __pyx_t_11 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
      __pyx_t_11 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_12)) {
    PyObject *__pyx_temp[8] = {__pyx_t_10, __pyx_t_2, __pyx_t_9, __pyx_t_8, __pyx_t_7, __pyx_t_4, __pyx_t_1, __pyx_v_microseconds};
    __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_12, __pyx_temp+1-__pyx_t_11, 7+__pyx_t_11); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 318, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_12)) {
    PyObject *__pyx_temp[8] = {__pyx_t_10, __pyx_t_2, __pyx_t_9, __pyx_t_8, __pyx_t_7, __pyx_t_4, __pyx_t_1, __pyx_v_microseconds};
    __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_12, __pyx_temp+1-__pyx_t_11, 7+__pyx_t_11); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 318, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else
  #endif
  {
    __pyx_t_13 = PyTuple_New(7+__pyx_t_11); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 318, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    if (__pyx_t_10) {
      __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_10); __pyx_t_10 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_11, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_9);
    PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_11, __pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_13, 2+__pyx_t_11, __pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_13, 3+__pyx_t_11, __pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_13, 4+__pyx_t_11, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_13, 5+__pyx_t_11, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_microseconds);
    __Pyx_GIVEREF(__pyx_v_microseconds);
    PyTuple_SET_ITEM(__pyx_t_13, 6+__pyx_t_11, __pyx_v_microseconds);
    __pyx_t_2 = 0;
    __pyx_t_9 = 0;
    __pyx_t_8 = 0;
    __pyx_t_7 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_13, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 318, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  }
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":300
 *         self._dealloc = 0
 * 
 *     def get_datetime(self):             # <<<<<<<<<<<<<<
 *         """
 *         get_datetime()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.get_datetime", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_s);
  __Pyx_XDECREF(__pyx_v_full_seconds);
  __Pyx_XDECREF(__pyx_v_fractional_seconds);
  __Pyx_XDECREF(__pyx_v_microseconds);
  __Pyx_XDECREF(__pyx_v_dt);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":321
 *                         int(full_seconds), microseconds)
 * 
 *     def get_data(self):             # <<<<<<<<<<<<<<
 *         """
 *         get_data()
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_5get_data(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5pyart_2io_14_rsl_interface_7_RslRay_4get_data[] = "\n        get_data()\n\n        Return the one-dimensional data contained in the ray.\n        ";
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_5get_data(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_data (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4get_data(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4get_data(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  Range __pyx_v_raw;
  PyArrayObject *__pyx_v_data = 0;
  int __pyx_v_shape;
  int __pyx_v_nbin;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_data;
  __Pyx_Buffer __pyx_pybuffer_data;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyArrayObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_data", 0);
  __pyx_pybuffer_data.pybuffer.buf = NULL;
  __pyx_pybuffer_data.refcount = 0;
  __pyx_pybuffernd_data.data = NULL;
  __pyx_pybuffernd_data.rcbuffer = &__pyx_pybuffer_data;

  /* "pyart/io/_rsl_interface.pyx":330
 *         cdef np.ndarray[np.float32_t, ndim = 1] data
 * 
 *         shape = (self._Ray.h.nbins)             # <<<<<<<<<<<<<<
 *         data = np.zeros(shape, dtype='float32') + 1.31072000e+05
 *         for nbin in range(self._Ray.h.nbins):
 */
  __pyx_t_1 = __pyx_v_self->_Ray->h.nbins;
  __pyx_v_shape = __pyx_t_1;

  /* "pyart/io/_rsl_interface.pyx":331
 * 
 *         shape = (self._Ray.h.nbins)
 *         data = np.zeros(shape, dtype='float32') + 1.31072000e+05             # <<<<<<<<<<<<<<
 *         for nbin in range(self._Ray.h.nbins):
 *             raw = self._Ray.range[nbin]
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 331, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_zeros); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 331, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_shape); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 331, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 331, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 331, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_n_u_float32) < 0) __PYX_ERR(0, 331, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 331, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyFloat_AddObjC(__pyx_t_5, __pyx_float_1_31072000e_05, 1.31072000e+05, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 331, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 331, __pyx_L1_error)
  __pyx_t_6 = ((PyArrayObject *)__pyx_t_2);
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_data.rcbuffer->pybuffer);
    __pyx_t_1 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_data.rcbuffer->pybuffer, (PyObject*)__pyx_t_6, &__Pyx_TypeInfo_nn___pyx_t_5numpy_float32_t, PyBUF_FORMAT| PyBUF_STRIDES| PyBUF_WRITABLE, 1, 0, __pyx_stack);
    if (unlikely(__pyx_t_1 < 0)) {
      PyErr_Fetch(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9);
      if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_data.rcbuffer->pybuffer, (PyObject*)__pyx_v_data, &__Pyx_TypeInfo_nn___pyx_t_5numpy_float32_t, PyBUF_FORMAT| PyBUF_STRIDES| PyBUF_WRITABLE, 1, 0, __pyx_stack) == -1)) {
        Py_XDECREF(__pyx_t_7); Py_XDECREF(__pyx_t_8); Py_XDECREF(__pyx_t_9);
        __Pyx_RaiseBufferFallbackError();
      } else {
        PyErr_Restore(__pyx_t_7, __pyx_t_8, __pyx_t_9);
      }
      __pyx_t_7 = __pyx_t_8 = __pyx_t_9 = 0;
    }
    __pyx_pybuffernd_data.diminfo[0].strides = __pyx_pybuffernd_data.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_data.diminfo[0].shape = __pyx_pybuffernd_data.rcbuffer->pybuffer.shape[0];
    if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 331, __pyx_L1_error)
  }
  __pyx_t_6 = 0;
  __pyx_v_data = ((PyArrayObject *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyart/io/_rsl_interface.pyx":332
 *         shape = (self._Ray.h.nbins)
 *         data = np.zeros(shape, dtype='float32') + 1.31072000e+05
 *         for nbin in range(self._Ray.h.nbins):             # <<<<<<<<<<<<<<
 *             raw = self._Ray.range[nbin]
 *             data[nbin] = self._Ray.h.f(raw)
 */
  __pyx_t_1 = __pyx_v_self->_Ray->h.nbins;
  __pyx_t_10 = __pyx_t_1;
  for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
    __pyx_v_nbin = __pyx_t_11;

    /* "pyart/io/_rsl_interface.pyx":333
 *         data = np.zeros(shape, dtype='float32') + 1.31072000e+05
 *         for nbin in range(self._Ray.h.nbins):
 *             raw = self._Ray.range[nbin]             # <<<<<<<<<<<<<<
 *             data[nbin] = self._Ray.h.f(raw)
 *         return data
 */
    __pyx_v_raw = (__pyx_v_self->_Ray->range[__pyx_v_nbin]);

    /* "pyart/io/_rsl_interface.pyx":334
 *         for nbin in range(self._Ray.h.nbins):
 *             raw = self._Ray.range[nbin]
 *             data[nbin] = self._Ray.h.f(raw)             # <<<<<<<<<<<<<<
 *         return data
 * 
 */
    __pyx_t_12 = __pyx_v_nbin;
    __pyx_t_13 = -1;
    if (__pyx_t_12 < 0) {
      __pyx_t_12 += __pyx_pybuffernd_data.diminfo[0].shape;
      if (unlikely(__pyx_t_12 < 0)) __pyx_t_13 = 0;
    } else if (unlikely(__pyx_t_12 >= __pyx_pybuffernd_data.diminfo[0].shape)) __pyx_t_13 = 0;
    if (unlikely(__pyx_t_13 != -1)) {
      __Pyx_RaiseBufferIndexError(__pyx_t_13);
      __PYX_ERR(0, 334, __pyx_L1_error)
    }
    *__Pyx_BufPtrStrided1d(__pyx_t_5numpy_float32_t *, __pyx_pybuffernd_data.rcbuffer->pybuffer.buf, __pyx_t_12, __pyx_pybuffernd_data.diminfo[0].strides) = __pyx_v_self->_Ray->h.f(__pyx_v_raw);
  }

  /* "pyart/io/_rsl_interface.pyx":335
 *             raw = self._Ray.range[nbin]
 *             data[nbin] = self._Ray.h.f(raw)
 *         return data             # <<<<<<<<<<<<<<
 * 
 *     # header properties mapped to class attributes.
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_data));
  __pyx_r = ((PyObject *)__pyx_v_data);
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":321
 *                         int(full_seconds), microseconds)
 * 
 *     def get_data(self):             # <<<<<<<<<<<<<<
 *         """
 *         get_data()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_data.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.get_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_data.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_data);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":339
 *     # header properties mapped to class attributes.
 *     property month:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.month
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_5month_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_5month_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_5month___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_5month___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":340
 *     property month:
 *         def __get__(self):
 *             return self._Ray.h.month             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int month):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Ray->h.month); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":339
 *     # header properties mapped to class attributes.
 *     property month:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.month
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.month.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":342
 *             return self._Ray.h.month
 * 
 *         def __set__(self, int month):             # <<<<<<<<<<<<<<
 *             self._Ray.h.month = month
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_5month_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_month); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_5month_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_month) {
  int __pyx_v_month;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_month); {
    __pyx_v_month = __Pyx_PyInt_As_int(__pyx_arg_month); if (unlikely((__pyx_v_month == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 342, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.month.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_5month_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((int)__pyx_v_month));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_5month_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_month) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":343
 * 
 *         def __set__(self, int month):
 *             self._Ray.h.month = month             # <<<<<<<<<<<<<<
 * 
 *     property day:
 */
  __pyx_v_self->_Ray->h.month = __pyx_v_month;

  /* "pyart/io/_rsl_interface.pyx":342
 *             return self._Ray.h.month
 * 
 *         def __set__(self, int month):             # <<<<<<<<<<<<<<
 *             self._Ray.h.month = month
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":346
 * 
 *     property day:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.day
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3day_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3day_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3day___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3day___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":347
 *     property day:
 *         def __get__(self):
 *             return self._Ray.h.day             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int day):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Ray->h.day); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 347, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":346
 * 
 *     property day:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.day
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.day.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":349
 *             return self._Ray.h.day
 * 
 *         def __set__(self, int day):             # <<<<<<<<<<<<<<
 *             self._Ray.h.day = day
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3day_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_day); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3day_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_day) {
  int __pyx_v_day;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_day); {
    __pyx_v_day = __Pyx_PyInt_As_int(__pyx_arg_day); if (unlikely((__pyx_v_day == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 349, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.day.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3day_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((int)__pyx_v_day));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3day_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_day) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":350
 * 
 *         def __set__(self, int day):
 *             self._Ray.h.day = day             # <<<<<<<<<<<<<<
 * 
 *     property year:
 */
  __pyx_v_self->_Ray->h.day = __pyx_v_day;

  /* "pyart/io/_rsl_interface.pyx":349
 *             return self._Ray.h.day
 * 
 *         def __set__(self, int day):             # <<<<<<<<<<<<<<
 *             self._Ray.h.day = day
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":353
 * 
 *     property year:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.year
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4year_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4year_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4year___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4year___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":354
 *     property year:
 *         def __get__(self):
 *             return self._Ray.h.year             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int year):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Ray->h.year); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 354, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":353
 * 
 *     property year:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.year
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.year.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":356
 *             return self._Ray.h.year
 * 
 *         def __set__(self, int year):             # <<<<<<<<<<<<<<
 *             self._Ray.h.year = year
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4year_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_year); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4year_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_year) {
  int __pyx_v_year;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_year); {
    __pyx_v_year = __Pyx_PyInt_As_int(__pyx_arg_year); if (unlikely((__pyx_v_year == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 356, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.year.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4year_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((int)__pyx_v_year));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4year_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_year) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":357
 * 
 *         def __set__(self, int year):
 *             self._Ray.h.year = year             # <<<<<<<<<<<<<<
 * 
 *     property hour:
 */
  __pyx_v_self->_Ray->h.year = __pyx_v_year;

  /* "pyart/io/_rsl_interface.pyx":356
 *             return self._Ray.h.year
 * 
 *         def __set__(self, int year):             # <<<<<<<<<<<<<<
 *             self._Ray.h.year = year
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":360
 * 
 *     property hour:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.hour
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4hour_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4hour_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4hour___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4hour___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":361
 *     property hour:
 *         def __get__(self):
 *             return self._Ray.h.hour             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int hour):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Ray->h.hour); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":360
 * 
 *     property hour:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.hour
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.hour.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":363
 *             return self._Ray.h.hour
 * 
 *         def __set__(self, int hour):             # <<<<<<<<<<<<<<
 *             self._Ray.h.hour = hour
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4hour_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_hour); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4hour_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_hour) {
  int __pyx_v_hour;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_hour); {
    __pyx_v_hour = __Pyx_PyInt_As_int(__pyx_arg_hour); if (unlikely((__pyx_v_hour == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 363, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.hour.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4hour_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((int)__pyx_v_hour));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4hour_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_hour) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":364
 * 
 *         def __set__(self, int hour):
 *             self._Ray.h.hour = hour             # <<<<<<<<<<<<<<
 * 
 *     property minute:
 */
  __pyx_v_self->_Ray->h.hour = __pyx_v_hour;

  /* "pyart/io/_rsl_interface.pyx":363
 *             return self._Ray.h.hour
 * 
 *         def __set__(self, int hour):             # <<<<<<<<<<<<<<
 *             self._Ray.h.hour = hour
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":367
 * 
 *     property minute:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.minute
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_6minute_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_6minute_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_6minute___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_6minute___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":368
 *     property minute:
 *         def __get__(self):
 *             return self._Ray.h.minute             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int minute):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Ray->h.minute); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 368, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":367
 * 
 *     property minute:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.minute
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.minute.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":370
 *             return self._Ray.h.minute
 * 
 *         def __set__(self, int minute):             # <<<<<<<<<<<<<<
 *             self._Ray.h.minute = minute
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_6minute_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_minute); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_6minute_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_minute) {
  int __pyx_v_minute;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_minute); {
    __pyx_v_minute = __Pyx_PyInt_As_int(__pyx_arg_minute); if (unlikely((__pyx_v_minute == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 370, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.minute.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_6minute_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((int)__pyx_v_minute));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_6minute_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_minute) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":371
 * 
 *         def __set__(self, int minute):
 *             self._Ray.h.minute = minute             # <<<<<<<<<<<<<<
 * 
 *     property sec:
 */
  __pyx_v_self->_Ray->h.minute = __pyx_v_minute;

  /* "pyart/io/_rsl_interface.pyx":370
 *             return self._Ray.h.minute
 * 
 *         def __set__(self, int minute):             # <<<<<<<<<<<<<<
 *             self._Ray.h.minute = minute
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":374
 * 
 *     property sec:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.sec
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3sec_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3sec_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3sec___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3sec___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":375
 *     property sec:
 *         def __get__(self):
 *             return self._Ray.h.sec             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float sec):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Ray->h.sec); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 375, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":374
 * 
 *     property sec:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.sec
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.sec.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":377
 *             return self._Ray.h.sec
 * 
 *         def __set__(self, float sec):             # <<<<<<<<<<<<<<
 *             self._Ray.h.sec = sec
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3sec_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_sec); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3sec_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_sec) {
  float __pyx_v_sec;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_sec); {
    __pyx_v_sec = __pyx_PyFloat_AsFloat(__pyx_arg_sec); if (unlikely((__pyx_v_sec == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 377, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.sec.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3sec_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((float)__pyx_v_sec));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3sec_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_sec) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":378
 * 
 *         def __set__(self, float sec):
 *             self._Ray.h.sec = sec             # <<<<<<<<<<<<<<
 * 
 *     property unam_rng:
 */
  __pyx_v_self->_Ray->h.sec = __pyx_v_sec;

  /* "pyart/io/_rsl_interface.pyx":377
 *             return self._Ray.h.sec
 * 
 *         def __set__(self, float sec):             # <<<<<<<<<<<<<<
 *             self._Ray.h.sec = sec
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":381
 * 
 *     property unam_rng:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.unam_rng
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_8unam_rng_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_8unam_rng_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_8unam_rng___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_8unam_rng___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":382
 *     property unam_rng:
 *         def __get__(self):
 *             return self._Ray.h.unam_rng             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float unam_rng):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Ray->h.unam_rng); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 382, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":381
 * 
 *     property unam_rng:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.unam_rng
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.unam_rng.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":384
 *             return self._Ray.h.unam_rng
 * 
 *         def __set__(self, float unam_rng):             # <<<<<<<<<<<<<<
 *             self._Ray.h.unam_rng = unam_rng
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_8unam_rng_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_unam_rng); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_8unam_rng_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_unam_rng) {
  float __pyx_v_unam_rng;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_unam_rng); {
    __pyx_v_unam_rng = __pyx_PyFloat_AsFloat(__pyx_arg_unam_rng); if (unlikely((__pyx_v_unam_rng == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 384, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.unam_rng.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_8unam_rng_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((float)__pyx_v_unam_rng));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_8unam_rng_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_unam_rng) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":385
 * 
 *         def __set__(self, float unam_rng):
 *             self._Ray.h.unam_rng = unam_rng             # <<<<<<<<<<<<<<
 * 
 *     property azimuth:
 */
  __pyx_v_self->_Ray->h.unam_rng = __pyx_v_unam_rng;

  /* "pyart/io/_rsl_interface.pyx":384
 *             return self._Ray.h.unam_rng
 * 
 *         def __set__(self, float unam_rng):             # <<<<<<<<<<<<<<
 *             self._Ray.h.unam_rng = unam_rng
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":388
 * 
 *     property azimuth:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.azimuth
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7azimuth_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7azimuth_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7azimuth___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7azimuth___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":389
 *     property azimuth:
 *         def __get__(self):
 *             return self._Ray.h.azimuth             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float azimuth):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Ray->h.azimuth); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 389, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":388
 * 
 *     property azimuth:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.azimuth
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.azimuth.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":391
 *             return self._Ray.h.azimuth
 * 
 *         def __set__(self, float azimuth):             # <<<<<<<<<<<<<<
 *             self._Ray.h.azimuth = azimuth
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7azimuth_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_azimuth); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7azimuth_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_azimuth) {
  float __pyx_v_azimuth;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_azimuth); {
    __pyx_v_azimuth = __pyx_PyFloat_AsFloat(__pyx_arg_azimuth); if (unlikely((__pyx_v_azimuth == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 391, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.azimuth.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7azimuth_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((float)__pyx_v_azimuth));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7azimuth_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_azimuth) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":392
 * 
 *         def __set__(self, float azimuth):
 *             self._Ray.h.azimuth = azimuth             # <<<<<<<<<<<<<<
 * 
 *     property ray_num:
 */
  __pyx_v_self->_Ray->h.azimuth = __pyx_v_azimuth;

  /* "pyart/io/_rsl_interface.pyx":391
 *             return self._Ray.h.azimuth
 * 
 *         def __set__(self, float azimuth):             # <<<<<<<<<<<<<<
 *             self._Ray.h.azimuth = azimuth
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":395
 * 
 *     property ray_num:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.ray_num
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7ray_num_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7ray_num_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7ray_num___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7ray_num___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":396
 *     property ray_num:
 *         def __get__(self):
 *             return self._Ray.h.ray_num             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int ray_num):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Ray->h.ray_num); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 396, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":395
 * 
 *     property ray_num:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.ray_num
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.ray_num.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":398
 *             return self._Ray.h.ray_num
 * 
 *         def __set__(self, int ray_num):             # <<<<<<<<<<<<<<
 *             self._Ray.h.ray_num = ray_num
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7ray_num_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_ray_num); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7ray_num_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_ray_num) {
  int __pyx_v_ray_num;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_ray_num); {
    __pyx_v_ray_num = __Pyx_PyInt_As_int(__pyx_arg_ray_num); if (unlikely((__pyx_v_ray_num == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 398, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.ray_num.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7ray_num_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((int)__pyx_v_ray_num));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7ray_num_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_ray_num) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":399
 * 
 *         def __set__(self, int ray_num):
 *             self._Ray.h.ray_num = ray_num             # <<<<<<<<<<<<<<
 * 
 *     property elev:
 */
  __pyx_v_self->_Ray->h.ray_num = __pyx_v_ray_num;

  /* "pyart/io/_rsl_interface.pyx":398
 *             return self._Ray.h.ray_num
 * 
 *         def __set__(self, int ray_num):             # <<<<<<<<<<<<<<
 *             self._Ray.h.ray_num = ray_num
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":402
 * 
 *     property elev:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.elev
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4elev_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4elev_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4elev___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4elev___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":403
 *     property elev:
 *         def __get__(self):
 *             return self._Ray.h.elev             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float elev):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Ray->h.elev); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 403, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":402
 * 
 *     property elev:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.elev
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.elev.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":405
 *             return self._Ray.h.elev
 * 
 *         def __set__(self, float elev):             # <<<<<<<<<<<<<<
 *             self._Ray.h.elev = elev
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4elev_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_elev); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4elev_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_elev) {
  float __pyx_v_elev;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_elev); {
    __pyx_v_elev = __pyx_PyFloat_AsFloat(__pyx_arg_elev); if (unlikely((__pyx_v_elev == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 405, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.elev.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4elev_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((float)__pyx_v_elev));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4elev_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_elev) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":406
 * 
 *         def __set__(self, float elev):
 *             self._Ray.h.elev = elev             # <<<<<<<<<<<<<<
 * 
 *     property elev_num:
 */
  __pyx_v_self->_Ray->h.elev = __pyx_v_elev;

  /* "pyart/io/_rsl_interface.pyx":405
 *             return self._Ray.h.elev
 * 
 *         def __set__(self, float elev):             # <<<<<<<<<<<<<<
 *             self._Ray.h.elev = elev
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":409
 * 
 *     property elev_num:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.elev_num
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_8elev_num_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_8elev_num_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_8elev_num___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_8elev_num___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":410
 *     property elev_num:
 *         def __get__(self):
 *             return self._Ray.h.elev_num             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int elev_num):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Ray->h.elev_num); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 410, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":409
 * 
 *     property elev_num:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.elev_num
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.elev_num.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":412
 *             return self._Ray.h.elev_num
 * 
 *         def __set__(self, int elev_num):             # <<<<<<<<<<<<<<
 *             self._Ray.h.elev_num = elev_num
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_8elev_num_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_elev_num); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_8elev_num_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_elev_num) {
  int __pyx_v_elev_num;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_elev_num); {
    __pyx_v_elev_num = __Pyx_PyInt_As_int(__pyx_arg_elev_num); if (unlikely((__pyx_v_elev_num == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 412, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.elev_num.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_8elev_num_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((int)__pyx_v_elev_num));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_8elev_num_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_elev_num) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":413
 * 
 *         def __set__(self, int elev_num):
 *             self._Ray.h.elev_num = elev_num             # <<<<<<<<<<<<<<
 * 
 *     property range_bin1:
 */
  __pyx_v_self->_Ray->h.elev_num = __pyx_v_elev_num;

  /* "pyart/io/_rsl_interface.pyx":412
 *             return self._Ray.h.elev_num
 * 
 *         def __set__(self, int elev_num):             # <<<<<<<<<<<<<<
 *             self._Ray.h.elev_num = elev_num
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":416
 * 
 *     property range_bin1:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.range_bin1
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10range_bin1_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10range_bin1_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10range_bin1___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10range_bin1___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":417
 *     property range_bin1:
 *         def __get__(self):
 *             return self._Ray.h.range_bin1             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int range_bin1):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Ray->h.range_bin1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 417, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":416
 * 
 *     property range_bin1:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.range_bin1
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.range_bin1.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":419
 *             return self._Ray.h.range_bin1
 * 
 *         def __set__(self, int range_bin1):             # <<<<<<<<<<<<<<
 *             self._Ray.h.range_bin1 = range_bin1
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10range_bin1_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_range_bin1); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10range_bin1_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_range_bin1) {
  int __pyx_v_range_bin1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_range_bin1); {
    __pyx_v_range_bin1 = __Pyx_PyInt_As_int(__pyx_arg_range_bin1); if (unlikely((__pyx_v_range_bin1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 419, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.range_bin1.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10range_bin1_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((int)__pyx_v_range_bin1));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10range_bin1_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_range_bin1) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":420
 * 
 *         def __set__(self, int range_bin1):
 *             self._Ray.h.range_bin1 = range_bin1             # <<<<<<<<<<<<<<
 * 
 *     property gate_size:
 */
  __pyx_v_self->_Ray->h.range_bin1 = __pyx_v_range_bin1;

  /* "pyart/io/_rsl_interface.pyx":419
 *             return self._Ray.h.range_bin1
 * 
 *         def __set__(self, int range_bin1):             # <<<<<<<<<<<<<<
 *             self._Ray.h.range_bin1 = range_bin1
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":423
 * 
 *     property gate_size:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.gate_size
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9gate_size_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9gate_size_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9gate_size___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9gate_size___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":424
 *     property gate_size:
 *         def __get__(self):
 *             return self._Ray.h.gate_size             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int gate_size):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Ray->h.gate_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 424, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":423
 * 
 *     property gate_size:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.gate_size
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.gate_size.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":426
 *             return self._Ray.h.gate_size
 * 
 *         def __set__(self, int gate_size):             # <<<<<<<<<<<<<<
 *             self._Ray.h.gate_size = gate_size
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9gate_size_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_gate_size); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9gate_size_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_gate_size) {
  int __pyx_v_gate_size;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_gate_size); {
    __pyx_v_gate_size = __Pyx_PyInt_As_int(__pyx_arg_gate_size); if (unlikely((__pyx_v_gate_size == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 426, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.gate_size.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9gate_size_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((int)__pyx_v_gate_size));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9gate_size_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_gate_size) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":427
 * 
 *         def __set__(self, int gate_size):
 *             self._Ray.h.gate_size = gate_size             # <<<<<<<<<<<<<<
 * 
 *     property vel_res:
 */
  __pyx_v_self->_Ray->h.gate_size = __pyx_v_gate_size;

  /* "pyart/io/_rsl_interface.pyx":426
 *             return self._Ray.h.gate_size
 * 
 *         def __set__(self, int gate_size):             # <<<<<<<<<<<<<<
 *             self._Ray.h.gate_size = gate_size
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":430
 * 
 *     property vel_res:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.vel_res
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7vel_res_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7vel_res_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7vel_res___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7vel_res___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":431
 *     property vel_res:
 *         def __get__(self):
 *             return self._Ray.h.vel_res             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float vel_res):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Ray->h.vel_res); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 431, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":430
 * 
 *     property vel_res:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.vel_res
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.vel_res.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":433
 *             return self._Ray.h.vel_res
 * 
 *         def __set__(self, float vel_res):             # <<<<<<<<<<<<<<
 *             self._Ray.h.vel_res = vel_res
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7vel_res_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_vel_res); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7vel_res_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_vel_res) {
  float __pyx_v_vel_res;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_vel_res); {
    __pyx_v_vel_res = __pyx_PyFloat_AsFloat(__pyx_arg_vel_res); if (unlikely((__pyx_v_vel_res == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 433, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.vel_res.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7vel_res_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((float)__pyx_v_vel_res));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7vel_res_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_vel_res) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":434
 * 
 *         def __set__(self, float vel_res):
 *             self._Ray.h.vel_res = vel_res             # <<<<<<<<<<<<<<
 * 
 *     property sweep_rate:
 */
  __pyx_v_self->_Ray->h.vel_res = __pyx_v_vel_res;

  /* "pyart/io/_rsl_interface.pyx":433
 *             return self._Ray.h.vel_res
 * 
 *         def __set__(self, float vel_res):             # <<<<<<<<<<<<<<
 *             self._Ray.h.vel_res = vel_res
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":437
 * 
 *     property sweep_rate:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.sweep_rate
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10sweep_rate_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10sweep_rate_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10sweep_rate___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10sweep_rate___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":438
 *     property sweep_rate:
 *         def __get__(self):
 *             return self._Ray.h.sweep_rate             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float sweep_rate):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Ray->h.sweep_rate); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 438, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":437
 * 
 *     property sweep_rate:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.sweep_rate
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.sweep_rate.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":440
 *             return self._Ray.h.sweep_rate
 * 
 *         def __set__(self, float sweep_rate):             # <<<<<<<<<<<<<<
 *             self._Ray.h.sweep_rate = sweep_rate
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10sweep_rate_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_sweep_rate); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10sweep_rate_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_sweep_rate) {
  float __pyx_v_sweep_rate;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_sweep_rate); {
    __pyx_v_sweep_rate = __pyx_PyFloat_AsFloat(__pyx_arg_sweep_rate); if (unlikely((__pyx_v_sweep_rate == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 440, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.sweep_rate.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10sweep_rate_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((float)__pyx_v_sweep_rate));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10sweep_rate_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_sweep_rate) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":441
 * 
 *         def __set__(self, float sweep_rate):
 *             self._Ray.h.sweep_rate = sweep_rate             # <<<<<<<<<<<<<<
 * 
 *     property prf:
 */
  __pyx_v_self->_Ray->h.sweep_rate = __pyx_v_sweep_rate;

  /* "pyart/io/_rsl_interface.pyx":440
 *             return self._Ray.h.sweep_rate
 * 
 *         def __set__(self, float sweep_rate):             # <<<<<<<<<<<<<<
 *             self._Ray.h.sweep_rate = sweep_rate
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":444
 * 
 *     property prf:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.prf
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3prf_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3prf_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3prf___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3prf___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":445
 *     property prf:
 *         def __get__(self):
 *             return self._Ray.h.prf             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int prf):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Ray->h.prf); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":444
 * 
 *     property prf:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.prf
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.prf.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":447
 *             return self._Ray.h.prf
 * 
 *         def __set__(self, int prf):             # <<<<<<<<<<<<<<
 *             self._Ray.h.prf = prf
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3prf_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_prf); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3prf_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_prf) {
  int __pyx_v_prf;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_prf); {
    __pyx_v_prf = __Pyx_PyInt_As_int(__pyx_arg_prf); if (unlikely((__pyx_v_prf == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 447, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.prf.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3prf_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((int)__pyx_v_prf));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3prf_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_prf) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":448
 * 
 *         def __set__(self, int prf):
 *             self._Ray.h.prf = prf             # <<<<<<<<<<<<<<
 * 
 *     property prf2:
 */
  __pyx_v_self->_Ray->h.prf = __pyx_v_prf;

  /* "pyart/io/_rsl_interface.pyx":447
 *             return self._Ray.h.prf
 * 
 *         def __set__(self, int prf):             # <<<<<<<<<<<<<<
 *             self._Ray.h.prf = prf
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":451
 * 
 *     property prf2:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.prf2
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4prf2_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4prf2_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4prf2___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4prf2___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":452
 *     property prf2:
 *         def __get__(self):
 *             return self._Ray.h.prf2             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int prf2):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Ray->h.prf2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 452, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":451
 * 
 *     property prf2:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.prf2
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.prf2.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":454
 *             return self._Ray.h.prf2
 * 
 *         def __set__(self, int prf2):             # <<<<<<<<<<<<<<
 *             self._Ray.h.prf2 = prf2
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4prf2_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_prf2); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4prf2_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_prf2) {
  int __pyx_v_prf2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_prf2); {
    __pyx_v_prf2 = __Pyx_PyInt_As_int(__pyx_arg_prf2); if (unlikely((__pyx_v_prf2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 454, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.prf2.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4prf2_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((int)__pyx_v_prf2));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4prf2_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_prf2) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":455
 * 
 *         def __set__(self, int prf2):
 *             self._Ray.h.prf2 = prf2             # <<<<<<<<<<<<<<
 * 
 *     property azim_rate:
 */
  __pyx_v_self->_Ray->h.prf2 = __pyx_v_prf2;

  /* "pyart/io/_rsl_interface.pyx":454
 *             return self._Ray.h.prf2
 * 
 *         def __set__(self, int prf2):             # <<<<<<<<<<<<<<
 *             self._Ray.h.prf2 = prf2
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":458
 * 
 *     property azim_rate:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.azim_rate
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9azim_rate_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9azim_rate_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9azim_rate___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9azim_rate___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":459
 *     property azim_rate:
 *         def __get__(self):
 *             return self._Ray.h.azim_rate             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float azim_rate):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Ray->h.azim_rate); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 459, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":458
 * 
 *     property azim_rate:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.azim_rate
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.azim_rate.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":461
 *             return self._Ray.h.azim_rate
 * 
 *         def __set__(self, float azim_rate):             # <<<<<<<<<<<<<<
 *             self._Ray.h.azim_rate = azim_rate
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9azim_rate_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_azim_rate); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9azim_rate_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_azim_rate) {
  float __pyx_v_azim_rate;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_azim_rate); {
    __pyx_v_azim_rate = __pyx_PyFloat_AsFloat(__pyx_arg_azim_rate); if (unlikely((__pyx_v_azim_rate == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 461, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.azim_rate.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9azim_rate_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((float)__pyx_v_azim_rate));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9azim_rate_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_azim_rate) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":462
 * 
 *         def __set__(self, float azim_rate):
 *             self._Ray.h.azim_rate = azim_rate             # <<<<<<<<<<<<<<
 * 
 *     property fix_angle:
 */
  __pyx_v_self->_Ray->h.azim_rate = __pyx_v_azim_rate;

  /* "pyart/io/_rsl_interface.pyx":461
 *             return self._Ray.h.azim_rate
 * 
 *         def __set__(self, float azim_rate):             # <<<<<<<<<<<<<<
 *             self._Ray.h.azim_rate = azim_rate
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":465
 * 
 *     property fix_angle:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.fix_angle
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9fix_angle_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9fix_angle_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9fix_angle___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9fix_angle___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":466
 *     property fix_angle:
 *         def __get__(self):
 *             return self._Ray.h.fix_angle             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float fix_angle):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Ray->h.fix_angle); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 466, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":465
 * 
 *     property fix_angle:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.fix_angle
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.fix_angle.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":468
 *             return self._Ray.h.fix_angle
 * 
 *         def __set__(self, float fix_angle):             # <<<<<<<<<<<<<<
 *             self._Ray.h.fix_angle = fix_angle
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9fix_angle_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_fix_angle); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9fix_angle_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_fix_angle) {
  float __pyx_v_fix_angle;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_fix_angle); {
    __pyx_v_fix_angle = __pyx_PyFloat_AsFloat(__pyx_arg_fix_angle); if (unlikely((__pyx_v_fix_angle == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 468, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.fix_angle.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9fix_angle_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((float)__pyx_v_fix_angle));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9fix_angle_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_fix_angle) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":469
 * 
 *         def __set__(self, float fix_angle):
 *             self._Ray.h.fix_angle = fix_angle             # <<<<<<<<<<<<<<
 * 
 *     property pitch:
 */
  __pyx_v_self->_Ray->h.fix_angle = __pyx_v_fix_angle;

  /* "pyart/io/_rsl_interface.pyx":468
 *             return self._Ray.h.fix_angle
 * 
 *         def __set__(self, float fix_angle):             # <<<<<<<<<<<<<<
 *             self._Ray.h.fix_angle = fix_angle
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":472
 * 
 *     property pitch:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.pitch
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_5pitch_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_5pitch_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_5pitch___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_5pitch___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":473
 *     property pitch:
 *         def __get__(self):
 *             return self._Ray.h.pitch             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float pitch):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Ray->h.pitch); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 473, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":472
 * 
 *     property pitch:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.pitch
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.pitch.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":475
 *             return self._Ray.h.pitch
 * 
 *         def __set__(self, float pitch):             # <<<<<<<<<<<<<<
 *             self._Ray.h.pitch = pitch
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_5pitch_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_pitch); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_5pitch_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_pitch) {
  float __pyx_v_pitch;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_pitch); {
    __pyx_v_pitch = __pyx_PyFloat_AsFloat(__pyx_arg_pitch); if (unlikely((__pyx_v_pitch == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 475, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.pitch.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_5pitch_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((float)__pyx_v_pitch));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_5pitch_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_pitch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":476
 * 
 *         def __set__(self, float pitch):
 *             self._Ray.h.pitch = pitch             # <<<<<<<<<<<<<<
 * 
 *     property roll:
 */
  __pyx_v_self->_Ray->h.pitch = __pyx_v_pitch;

  /* "pyart/io/_rsl_interface.pyx":475
 *             return self._Ray.h.pitch
 * 
 *         def __set__(self, float pitch):             # <<<<<<<<<<<<<<
 *             self._Ray.h.pitch = pitch
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":479
 * 
 *     property roll:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.roll
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4roll_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4roll_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4roll___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4roll___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":480
 *     property roll:
 *         def __get__(self):
 *             return self._Ray.h.roll             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float roll):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Ray->h.roll); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 480, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":479
 * 
 *     property roll:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.roll
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.roll.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":482
 *             return self._Ray.h.roll
 * 
 *         def __set__(self, float roll):             # <<<<<<<<<<<<<<
 *             self._Ray.h.roll = roll
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4roll_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_roll); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4roll_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_roll) {
  float __pyx_v_roll;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_roll); {
    __pyx_v_roll = __pyx_PyFloat_AsFloat(__pyx_arg_roll); if (unlikely((__pyx_v_roll == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 482, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.roll.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4roll_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((float)__pyx_v_roll));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_4roll_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_roll) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":483
 * 
 *         def __set__(self, float roll):
 *             self._Ray.h.roll = roll             # <<<<<<<<<<<<<<
 * 
 *     property heading:
 */
  __pyx_v_self->_Ray->h.roll = __pyx_v_roll;

  /* "pyart/io/_rsl_interface.pyx":482
 *             return self._Ray.h.roll
 * 
 *         def __set__(self, float roll):             # <<<<<<<<<<<<<<
 *             self._Ray.h.roll = roll
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":486
 * 
 *     property heading:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.heading
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7heading_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7heading_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7heading___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7heading___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":487
 *     property heading:
 *         def __get__(self):
 *             return self._Ray.h.heading             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float heading):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Ray->h.heading); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 487, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":486
 * 
 *     property heading:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.heading
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.heading.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":489
 *             return self._Ray.h.heading
 * 
 *         def __set__(self, float heading):             # <<<<<<<<<<<<<<
 *             self._Ray.h.heading = heading
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7heading_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_heading); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7heading_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_heading) {
  float __pyx_v_heading;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_heading); {
    __pyx_v_heading = __pyx_PyFloat_AsFloat(__pyx_arg_heading); if (unlikely((__pyx_v_heading == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 489, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.heading.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7heading_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((float)__pyx_v_heading));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7heading_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_heading) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":490
 * 
 *         def __set__(self, float heading):
 *             self._Ray.h.heading = heading             # <<<<<<<<<<<<<<
 * 
 *     property pitch_rate:
 */
  __pyx_v_self->_Ray->h.heading = __pyx_v_heading;

  /* "pyart/io/_rsl_interface.pyx":489
 *             return self._Ray.h.heading
 * 
 *         def __set__(self, float heading):             # <<<<<<<<<<<<<<
 *             self._Ray.h.heading = heading
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":493
 * 
 *     property pitch_rate:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.pitch_rate
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10pitch_rate_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10pitch_rate_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10pitch_rate___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10pitch_rate___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":494
 *     property pitch_rate:
 *         def __get__(self):
 *             return self._Ray.h.pitch_rate             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float pitch_rate):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Ray->h.pitch_rate); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 494, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":493
 * 
 *     property pitch_rate:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.pitch_rate
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.pitch_rate.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":496
 *             return self._Ray.h.pitch_rate
 * 
 *         def __set__(self, float pitch_rate):             # <<<<<<<<<<<<<<
 *             self._Ray.h.pitch_rate = pitch_rate
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10pitch_rate_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_pitch_rate); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10pitch_rate_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_pitch_rate) {
  float __pyx_v_pitch_rate;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_pitch_rate); {
    __pyx_v_pitch_rate = __pyx_PyFloat_AsFloat(__pyx_arg_pitch_rate); if (unlikely((__pyx_v_pitch_rate == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 496, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.pitch_rate.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10pitch_rate_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((float)__pyx_v_pitch_rate));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10pitch_rate_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_pitch_rate) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":497
 * 
 *         def __set__(self, float pitch_rate):
 *             self._Ray.h.pitch_rate = pitch_rate             # <<<<<<<<<<<<<<
 * 
 *     property roll_rate:
 */
  __pyx_v_self->_Ray->h.pitch_rate = __pyx_v_pitch_rate;

  /* "pyart/io/_rsl_interface.pyx":496
 *             return self._Ray.h.pitch_rate
 * 
 *         def __set__(self, float pitch_rate):             # <<<<<<<<<<<<<<
 *             self._Ray.h.pitch_rate = pitch_rate
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":500
 * 
 *     property roll_rate:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.roll_rate
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9roll_rate_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9roll_rate_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9roll_rate___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9roll_rate___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":501
 *     property roll_rate:
 *         def __get__(self):
 *             return self._Ray.h.roll_rate             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float roll_rate):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Ray->h.roll_rate); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 501, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":500
 * 
 *     property roll_rate:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.roll_rate
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.roll_rate.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":503
 *             return self._Ray.h.roll_rate
 * 
 *         def __set__(self, float roll_rate):             # <<<<<<<<<<<<<<
 *             self._Ray.h.roll_rate = roll_rate
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9roll_rate_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_roll_rate); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9roll_rate_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_roll_rate) {
  float __pyx_v_roll_rate;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_roll_rate); {
    __pyx_v_roll_rate = __pyx_PyFloat_AsFloat(__pyx_arg_roll_rate); if (unlikely((__pyx_v_roll_rate == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 503, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.roll_rate.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9roll_rate_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((float)__pyx_v_roll_rate));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9roll_rate_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_roll_rate) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":504
 * 
 *         def __set__(self, float roll_rate):
 *             self._Ray.h.roll_rate = roll_rate             # <<<<<<<<<<<<<<
 * 
 *     property heading_rate:
 */
  __pyx_v_self->_Ray->h.roll_rate = __pyx_v_roll_rate;

  /* "pyart/io/_rsl_interface.pyx":503
 *             return self._Ray.h.roll_rate
 * 
 *         def __set__(self, float roll_rate):             # <<<<<<<<<<<<<<
 *             self._Ray.h.roll_rate = roll_rate
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":507
 * 
 *     property heading_rate:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.heading_rate
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_12heading_rate_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_12heading_rate_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_12heading_rate___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_12heading_rate___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":508
 *     property heading_rate:
 *         def __get__(self):
 *             return self._Ray.h.heading_rate             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float heading_rate):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Ray->h.heading_rate); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 508, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":507
 * 
 *     property heading_rate:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.heading_rate
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.heading_rate.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":510
 *             return self._Ray.h.heading_rate
 * 
 *         def __set__(self, float heading_rate):             # <<<<<<<<<<<<<<
 *             self._Ray.h.heading_rate = heading_rate
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_12heading_rate_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_heading_rate); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_12heading_rate_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_heading_rate) {
  float __pyx_v_heading_rate;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_heading_rate); {
    __pyx_v_heading_rate = __pyx_PyFloat_AsFloat(__pyx_arg_heading_rate); if (unlikely((__pyx_v_heading_rate == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 510, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.heading_rate.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_12heading_rate_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((float)__pyx_v_heading_rate));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_12heading_rate_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_heading_rate) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":511
 * 
 *         def __set__(self, float heading_rate):
 *             self._Ray.h.heading_rate = heading_rate             # <<<<<<<<<<<<<<
 * 
 *     property lat:
 */
  __pyx_v_self->_Ray->h.heading_rate = __pyx_v_heading_rate;

  /* "pyart/io/_rsl_interface.pyx":510
 *             return self._Ray.h.heading_rate
 * 
 *         def __set__(self, float heading_rate):             # <<<<<<<<<<<<<<
 *             self._Ray.h.heading_rate = heading_rate
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":514
 * 
 *     property lat:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.lat
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3lat_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3lat_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3lat___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3lat___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":515
 *     property lat:
 *         def __get__(self):
 *             return self._Ray.h.lat             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float lat):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Ray->h.lat); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 515, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":514
 * 
 *     property lat:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.lat
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.lat.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":517
 *             return self._Ray.h.lat
 * 
 *         def __set__(self, float lat):             # <<<<<<<<<<<<<<
 *             self._Ray.h.lat = lat
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3lat_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_lat); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3lat_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_lat) {
  float __pyx_v_lat;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_lat); {
    __pyx_v_lat = __pyx_PyFloat_AsFloat(__pyx_arg_lat); if (unlikely((__pyx_v_lat == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 517, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.lat.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3lat_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((float)__pyx_v_lat));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3lat_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_lat) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":518
 * 
 *         def __set__(self, float lat):
 *             self._Ray.h.lat = lat             # <<<<<<<<<<<<<<
 * 
 *     property lon:
 */
  __pyx_v_self->_Ray->h.lat = __pyx_v_lat;

  /* "pyart/io/_rsl_interface.pyx":517
 *             return self._Ray.h.lat
 * 
 *         def __set__(self, float lat):             # <<<<<<<<<<<<<<
 *             self._Ray.h.lat = lat
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":521
 * 
 *     property lon:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.lon
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3lon_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3lon_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3lon___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3lon___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":522
 *     property lon:
 *         def __get__(self):
 *             return self._Ray.h.lon             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float lon):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Ray->h.lon); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 522, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":521
 * 
 *     property lon:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.lon
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.lon.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":524
 *             return self._Ray.h.lon
 * 
 *         def __set__(self, float lon):             # <<<<<<<<<<<<<<
 *             self._Ray.h.lon = lon
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3lon_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_lon); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3lon_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_lon) {
  float __pyx_v_lon;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_lon); {
    __pyx_v_lon = __pyx_PyFloat_AsFloat(__pyx_arg_lon); if (unlikely((__pyx_v_lon == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 524, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.lon.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3lon_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((float)__pyx_v_lon));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3lon_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_lon) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":525
 * 
 *         def __set__(self, float lon):
 *             self._Ray.h.lon = lon             # <<<<<<<<<<<<<<
 * 
 *     property alt:
 */
  __pyx_v_self->_Ray->h.lon = __pyx_v_lon;

  /* "pyart/io/_rsl_interface.pyx":524
 *             return self._Ray.h.lon
 * 
 *         def __set__(self, float lon):             # <<<<<<<<<<<<<<
 *             self._Ray.h.lon = lon
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":528
 * 
 *     property alt:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.alt
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3alt_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3alt_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3alt___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3alt___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":529
 *     property alt:
 *         def __get__(self):
 *             return self._Ray.h.alt             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int alt):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Ray->h.alt); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":528
 * 
 *     property alt:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.alt
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.alt.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":531
 *             return self._Ray.h.alt
 * 
 *         def __set__(self, int alt):             # <<<<<<<<<<<<<<
 *             self._Ray.h.alt = alt
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3alt_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_alt); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3alt_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_alt) {
  int __pyx_v_alt;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_alt); {
    __pyx_v_alt = __Pyx_PyInt_As_int(__pyx_arg_alt); if (unlikely((__pyx_v_alt == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 531, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.alt.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3alt_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((int)__pyx_v_alt));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3alt_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_alt) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":532
 * 
 *         def __set__(self, int alt):
 *             self._Ray.h.alt = alt             # <<<<<<<<<<<<<<
 * 
 *     property rvc:
 */
  __pyx_v_self->_Ray->h.alt = __pyx_v_alt;

  /* "pyart/io/_rsl_interface.pyx":531
 *             return self._Ray.h.alt
 * 
 *         def __set__(self, int alt):             # <<<<<<<<<<<<<<
 *             self._Ray.h.alt = alt
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":535
 * 
 *     property rvc:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.rvc
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3rvc_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3rvc_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3rvc___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3rvc___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":536
 *     property rvc:
 *         def __get__(self):
 *             return self._Ray.h.rvc             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float rvc):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Ray->h.rvc); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 536, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":535
 * 
 *     property rvc:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.rvc
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.rvc.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":538
 *             return self._Ray.h.rvc
 * 
 *         def __set__(self, float rvc):             # <<<<<<<<<<<<<<
 *             self._Ray.h.rvc = rvc
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3rvc_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_rvc); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3rvc_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_rvc) {
  float __pyx_v_rvc;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_rvc); {
    __pyx_v_rvc = __pyx_PyFloat_AsFloat(__pyx_arg_rvc); if (unlikely((__pyx_v_rvc == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 538, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.rvc.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3rvc_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((float)__pyx_v_rvc));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_3rvc_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_rvc) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":539
 * 
 *         def __set__(self, float rvc):
 *             self._Ray.h.rvc = rvc             # <<<<<<<<<<<<<<
 * 
 *     property vel_east:
 */
  __pyx_v_self->_Ray->h.rvc = __pyx_v_rvc;

  /* "pyart/io/_rsl_interface.pyx":538
 *             return self._Ray.h.rvc
 * 
 *         def __set__(self, float rvc):             # <<<<<<<<<<<<<<
 *             self._Ray.h.rvc = rvc
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":542
 * 
 *     property vel_east:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.vel_east
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_8vel_east_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_8vel_east_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_8vel_east___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_8vel_east___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":543
 *     property vel_east:
 *         def __get__(self):
 *             return self._Ray.h.vel_east             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float vel_east):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Ray->h.vel_east); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 543, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":542
 * 
 *     property vel_east:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.vel_east
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.vel_east.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":545
 *             return self._Ray.h.vel_east
 * 
 *         def __set__(self, float vel_east):             # <<<<<<<<<<<<<<
 *             self._Ray.h.vel_east = vel_east
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_8vel_east_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_vel_east); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_8vel_east_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_vel_east) {
  float __pyx_v_vel_east;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_vel_east); {
    __pyx_v_vel_east = __pyx_PyFloat_AsFloat(__pyx_arg_vel_east); if (unlikely((__pyx_v_vel_east == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 545, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.vel_east.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_8vel_east_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((float)__pyx_v_vel_east));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_8vel_east_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_vel_east) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":546
 * 
 *         def __set__(self, float vel_east):
 *             self._Ray.h.vel_east = vel_east             # <<<<<<<<<<<<<<
 * 
 *     property vel_north:
 */
  __pyx_v_self->_Ray->h.vel_east = __pyx_v_vel_east;

  /* "pyart/io/_rsl_interface.pyx":545
 *             return self._Ray.h.vel_east
 * 
 *         def __set__(self, float vel_east):             # <<<<<<<<<<<<<<
 *             self._Ray.h.vel_east = vel_east
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":549
 * 
 *     property vel_north:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.vel_north
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9vel_north_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9vel_north_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9vel_north___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9vel_north___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":550
 *     property vel_north:
 *         def __get__(self):
 *             return self._Ray.h.vel_north             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float vel_north):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Ray->h.vel_north); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 550, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":549
 * 
 *     property vel_north:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.vel_north
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.vel_north.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":552
 *             return self._Ray.h.vel_north
 * 
 *         def __set__(self, float vel_north):             # <<<<<<<<<<<<<<
 *             self._Ray.h.vel_north = vel_north
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9vel_north_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_vel_north); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9vel_north_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_vel_north) {
  float __pyx_v_vel_north;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_vel_north); {
    __pyx_v_vel_north = __pyx_PyFloat_AsFloat(__pyx_arg_vel_north); if (unlikely((__pyx_v_vel_north == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 552, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.vel_north.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9vel_north_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((float)__pyx_v_vel_north));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9vel_north_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_vel_north) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":553
 * 
 *         def __set__(self, float vel_north):
 *             self._Ray.h.vel_north = vel_north             # <<<<<<<<<<<<<<
 * 
 *     property vel_up:
 */
  __pyx_v_self->_Ray->h.vel_north = __pyx_v_vel_north;

  /* "pyart/io/_rsl_interface.pyx":552
 *             return self._Ray.h.vel_north
 * 
 *         def __set__(self, float vel_north):             # <<<<<<<<<<<<<<
 *             self._Ray.h.vel_north = vel_north
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":556
 * 
 *     property vel_up:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.vel_up
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_6vel_up_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_6vel_up_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_6vel_up___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_6vel_up___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":557
 *     property vel_up:
 *         def __get__(self):
 *             return self._Ray.h.vel_up             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float vel_up):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Ray->h.vel_up); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 557, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":556
 * 
 *     property vel_up:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.vel_up
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.vel_up.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":559
 *             return self._Ray.h.vel_up
 * 
 *         def __set__(self, float vel_up):             # <<<<<<<<<<<<<<
 *             self._Ray.h.vel_up = vel_up
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_6vel_up_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_vel_up); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_6vel_up_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_vel_up) {
  float __pyx_v_vel_up;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_vel_up); {
    __pyx_v_vel_up = __pyx_PyFloat_AsFloat(__pyx_arg_vel_up); if (unlikely((__pyx_v_vel_up == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 559, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.vel_up.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_6vel_up_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((float)__pyx_v_vel_up));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_6vel_up_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_vel_up) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":560
 * 
 *         def __set__(self, float vel_up):
 *             self._Ray.h.vel_up = vel_up             # <<<<<<<<<<<<<<
 * 
 *     property pulse_count:
 */
  __pyx_v_self->_Ray->h.vel_up = __pyx_v_vel_up;

  /* "pyart/io/_rsl_interface.pyx":559
 *             return self._Ray.h.vel_up
 * 
 *         def __set__(self, float vel_up):             # <<<<<<<<<<<<<<
 *             self._Ray.h.vel_up = vel_up
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":563
 * 
 *     property pulse_count:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.pulse_count
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_11pulse_count_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_11pulse_count_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_11pulse_count___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_11pulse_count___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":564
 *     property pulse_count:
 *         def __get__(self):
 *             return self._Ray.h.pulse_count             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int pulse_count):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Ray->h.pulse_count); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 564, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":563
 * 
 *     property pulse_count:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.pulse_count
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.pulse_count.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":566
 *             return self._Ray.h.pulse_count
 * 
 *         def __set__(self, int pulse_count):             # <<<<<<<<<<<<<<
 *             self._Ray.h.pulse_count = pulse_count
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_11pulse_count_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_pulse_count); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_11pulse_count_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_pulse_count) {
  int __pyx_v_pulse_count;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_pulse_count); {
    __pyx_v_pulse_count = __Pyx_PyInt_As_int(__pyx_arg_pulse_count); if (unlikely((__pyx_v_pulse_count == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 566, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.pulse_count.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_11pulse_count_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((int)__pyx_v_pulse_count));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_11pulse_count_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_pulse_count) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":567
 * 
 *         def __set__(self, int pulse_count):
 *             self._Ray.h.pulse_count = pulse_count             # <<<<<<<<<<<<<<
 * 
 *     property pulse_width:
 */
  __pyx_v_self->_Ray->h.pulse_count = __pyx_v_pulse_count;

  /* "pyart/io/_rsl_interface.pyx":566
 *             return self._Ray.h.pulse_count
 * 
 *         def __set__(self, int pulse_count):             # <<<<<<<<<<<<<<
 *             self._Ray.h.pulse_count = pulse_count
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":570
 * 
 *     property pulse_width:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.pulse_width
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_11pulse_width_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_11pulse_width_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_11pulse_width___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_11pulse_width___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":571
 *     property pulse_width:
 *         def __get__(self):
 *             return self._Ray.h.pulse_width             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float pulse_width):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Ray->h.pulse_width); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 571, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":570
 * 
 *     property pulse_width:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.pulse_width
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.pulse_width.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":573
 *             return self._Ray.h.pulse_width
 * 
 *         def __set__(self, float pulse_width):             # <<<<<<<<<<<<<<
 *             self._Ray.h.pulse_width = pulse_width
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_11pulse_width_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_pulse_width); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_11pulse_width_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_pulse_width) {
  float __pyx_v_pulse_width;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_pulse_width); {
    __pyx_v_pulse_width = __pyx_PyFloat_AsFloat(__pyx_arg_pulse_width); if (unlikely((__pyx_v_pulse_width == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 573, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.pulse_width.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_11pulse_width_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((float)__pyx_v_pulse_width));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_11pulse_width_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_pulse_width) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":574
 * 
 *         def __set__(self, float pulse_width):
 *             self._Ray.h.pulse_width = pulse_width             # <<<<<<<<<<<<<<
 * 
 *     property beam_width:
 */
  __pyx_v_self->_Ray->h.pulse_width = __pyx_v_pulse_width;

  /* "pyart/io/_rsl_interface.pyx":573
 *             return self._Ray.h.pulse_width
 * 
 *         def __set__(self, float pulse_width):             # <<<<<<<<<<<<<<
 *             self._Ray.h.pulse_width = pulse_width
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":577
 * 
 *     property beam_width:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.beam_width
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10beam_width_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10beam_width_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10beam_width___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10beam_width___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":578
 *     property beam_width:
 *         def __get__(self):
 *             return self._Ray.h.beam_width             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float beam_width):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Ray->h.beam_width); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 578, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":577
 * 
 *     property beam_width:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.beam_width
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.beam_width.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":580
 *             return self._Ray.h.beam_width
 * 
 *         def __set__(self, float beam_width):             # <<<<<<<<<<<<<<
 *             self._Ray.h.beam_width = beam_width
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10beam_width_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_beam_width); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10beam_width_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_beam_width) {
  float __pyx_v_beam_width;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_beam_width); {
    __pyx_v_beam_width = __pyx_PyFloat_AsFloat(__pyx_arg_beam_width); if (unlikely((__pyx_v_beam_width == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 580, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.beam_width.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10beam_width_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((float)__pyx_v_beam_width));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10beam_width_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_beam_width) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":581
 * 
 *         def __set__(self, float beam_width):
 *             self._Ray.h.beam_width = beam_width             # <<<<<<<<<<<<<<
 * 
 *     property frequency:
 */
  __pyx_v_self->_Ray->h.beam_width = __pyx_v_beam_width;

  /* "pyart/io/_rsl_interface.pyx":580
 *             return self._Ray.h.beam_width
 * 
 *         def __set__(self, float beam_width):             # <<<<<<<<<<<<<<
 *             self._Ray.h.beam_width = beam_width
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":584
 * 
 *     property frequency:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.frequency
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9frequency_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9frequency_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9frequency___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9frequency___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":585
 *     property frequency:
 *         def __get__(self):
 *             return self._Ray.h.frequency             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float frequency):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Ray->h.frequency); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 585, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":584
 * 
 *     property frequency:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.frequency
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.frequency.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":587
 *             return self._Ray.h.frequency
 * 
 *         def __set__(self, float frequency):             # <<<<<<<<<<<<<<
 *             self._Ray.h.frequency = frequency
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9frequency_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_frequency); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9frequency_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_frequency) {
  float __pyx_v_frequency;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_frequency); {
    __pyx_v_frequency = __pyx_PyFloat_AsFloat(__pyx_arg_frequency); if (unlikely((__pyx_v_frequency == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 587, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.frequency.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9frequency_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((float)__pyx_v_frequency));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_9frequency_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_frequency) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":588
 * 
 *         def __set__(self, float frequency):
 *             self._Ray.h.frequency = frequency             # <<<<<<<<<<<<<<
 * 
 *     property wavelength:
 */
  __pyx_v_self->_Ray->h.frequency = __pyx_v_frequency;

  /* "pyart/io/_rsl_interface.pyx":587
 *             return self._Ray.h.frequency
 * 
 *         def __set__(self, float frequency):             # <<<<<<<<<<<<<<
 *             self._Ray.h.frequency = frequency
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":591
 * 
 *     property wavelength:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.wavelength
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10wavelength_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10wavelength_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10wavelength___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10wavelength___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":592
 *     property wavelength:
 *         def __get__(self):
 *             return self._Ray.h.wavelength             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float wavelength):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Ray->h.wavelength); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 592, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":591
 * 
 *     property wavelength:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.wavelength
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.wavelength.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":594
 *             return self._Ray.h.wavelength
 * 
 *         def __set__(self, float wavelength):             # <<<<<<<<<<<<<<
 *             self._Ray.h.wavelength = wavelength
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10wavelength_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_wavelength); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10wavelength_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_wavelength) {
  float __pyx_v_wavelength;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_wavelength); {
    __pyx_v_wavelength = __pyx_PyFloat_AsFloat(__pyx_arg_wavelength); if (unlikely((__pyx_v_wavelength == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 594, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.wavelength.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10wavelength_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((float)__pyx_v_wavelength));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_10wavelength_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_wavelength) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":595
 * 
 *         def __set__(self, float wavelength):
 *             self._Ray.h.wavelength = wavelength             # <<<<<<<<<<<<<<
 * 
 *     property nyq_vel:
 */
  __pyx_v_self->_Ray->h.wavelength = __pyx_v_wavelength;

  /* "pyart/io/_rsl_interface.pyx":594
 *             return self._Ray.h.wavelength
 * 
 *         def __set__(self, float wavelength):             # <<<<<<<<<<<<<<
 *             self._Ray.h.wavelength = wavelength
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":598
 * 
 *     property nyq_vel:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.nyq_vel
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7nyq_vel_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7nyq_vel_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7nyq_vel___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7nyq_vel___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":599
 *     property nyq_vel:
 *         def __get__(self):
 *             return self._Ray.h.nyq_vel             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float nyq_vel):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Ray->h.nyq_vel); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 599, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":598
 * 
 *     property nyq_vel:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.nyq_vel
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.nyq_vel.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":601
 *             return self._Ray.h.nyq_vel
 * 
 *         def __set__(self, float nyq_vel):             # <<<<<<<<<<<<<<
 *             self._Ray.h.nyq_vel = nyq_vel
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7nyq_vel_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_nyq_vel); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7nyq_vel_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_nyq_vel) {
  float __pyx_v_nyq_vel;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_nyq_vel); {
    __pyx_v_nyq_vel = __pyx_PyFloat_AsFloat(__pyx_arg_nyq_vel); if (unlikely((__pyx_v_nyq_vel == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 601, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.nyq_vel.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7nyq_vel_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((float)__pyx_v_nyq_vel));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_7nyq_vel_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, float __pyx_v_nyq_vel) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":602
 * 
 *         def __set__(self, float nyq_vel):
 *             self._Ray.h.nyq_vel = nyq_vel             # <<<<<<<<<<<<<<
 * 
 *     property nbins:
 */
  __pyx_v_self->_Ray->h.nyq_vel = __pyx_v_nyq_vel;

  /* "pyart/io/_rsl_interface.pyx":601
 *             return self._Ray.h.nyq_vel
 * 
 *         def __set__(self, float nyq_vel):             # <<<<<<<<<<<<<<
 *             self._Ray.h.nyq_vel = nyq_vel
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":605
 * 
 *     property nbins:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.nbins
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_5nbins_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_5nbins_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_5nbins___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_5nbins___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":606
 *     property nbins:
 *         def __get__(self):
 *             return self._Ray.h.nbins             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int nbins):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Ray->h.nbins); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 606, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":605
 * 
 *     property nbins:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Ray.h.nbins
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.nbins.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":608
 *             return self._Ray.h.nbins
 * 
 *         def __set__(self, int nbins):             # <<<<<<<<<<<<<<
 *             self._Ray.h.nbins = nbins
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_5nbins_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_nbins); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_5nbins_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_nbins) {
  int __pyx_v_nbins;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_nbins); {
    __pyx_v_nbins = __Pyx_PyInt_As_int(__pyx_arg_nbins); if (unlikely((__pyx_v_nbins == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 608, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.nbins.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_5nbins_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((int)__pyx_v_nbins));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_5nbins_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, int __pyx_v_nbins) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":609
 * 
 *         def __set__(self, int nbins):
 *             self._Ray.h.nbins = nbins             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_self->_Ray->h.nbins = __pyx_v_nbins;

  /* "pyart/io/_rsl_interface.pyx":608
 *             return self._Ray.h.nbins
 * 
 *         def __set__(self, int nbins):             # <<<<<<<<<<<<<<
 *             self._Ray.h.nbins = nbins
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._Ray cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_6__reduce_cython__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._Ray cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._Ray cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._Ray cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._Ray cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._Ray cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_8__setstate_cython__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7_RslRay_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self._Ray cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._Ray cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._Ray cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._Ray cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslRay.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":641
 *     cdef int _dealloc
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self._dealloc == 1:
 *             _rsl_h.RSL_free_sweep(self._Sweep)
 */

/* Python wrapper */
static void __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_1__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_1__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep___dealloc__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep___dealloc__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "pyart/io/_rsl_interface.pyx":642
 * 
 *     def __dealloc__(self):
 *         if self._dealloc == 1:             # <<<<<<<<<<<<<<
 *             _rsl_h.RSL_free_sweep(self._Sweep)
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->_dealloc == 1) != 0);
  if (__pyx_t_1) {

    /* "pyart/io/_rsl_interface.pyx":643
 *     def __dealloc__(self):
 *         if self._dealloc == 1:
 *             _rsl_h.RSL_free_sweep(self._Sweep)             # <<<<<<<<<<<<<<
 * 
 *     cdef load(self, _rsl_h.Sweep * Sweep):
 */
    RSL_free_sweep(__pyx_v_self->_Sweep);

    /* "pyart/io/_rsl_interface.pyx":642
 * 
 *     def __dealloc__(self):
 *         if self._dealloc == 1:             # <<<<<<<<<<<<<<
 *             _rsl_h.RSL_free_sweep(self._Sweep)
 * 
 */
  }

  /* "pyart/io/_rsl_interface.pyx":641
 *     cdef int _dealloc
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self._dealloc == 1:
 *             _rsl_h.RSL_free_sweep(self._Sweep)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "pyart/io/_rsl_interface.pyx":645
 *             _rsl_h.RSL_free_sweep(self._Sweep)
 * 
 *     cdef load(self, _rsl_h.Sweep * Sweep):             # <<<<<<<<<<<<<<
 *         """ Load the _RslSweep object, must be called after creation. """
 *         if Sweep is NULL:
 */

static PyObject *__pyx_f_5pyart_2io_14_rsl_interface_9_RslSweep_load(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self, Sweep *__pyx_v_Sweep) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("load", 0);

  /* "pyart/io/_rsl_interface.pyx":647
 *     cdef load(self, _rsl_h.Sweep * Sweep):
 *         """ Load the _RslSweep object, must be called after creation. """
 *         if Sweep is NULL:             # <<<<<<<<<<<<<<
 *             raise ValueError("cannot load _RslSweep with NULL")
 *         self._Sweep = Sweep
 */
  __pyx_t_1 = ((__pyx_v_Sweep == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "pyart/io/_rsl_interface.pyx":648
 *         """ Load the _RslSweep object, must be called after creation. """
 *         if Sweep is NULL:
 *             raise ValueError("cannot load _RslSweep with NULL")             # <<<<<<<<<<<<<<
 *         self._Sweep = Sweep
 *         self._dealloc = 0
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 648, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 648, __pyx_L1_error)

    /* "pyart/io/_rsl_interface.pyx":647
 *     cdef load(self, _rsl_h.Sweep * Sweep):
 *         """ Load the _RslSweep object, must be called after creation. """
 *         if Sweep is NULL:             # <<<<<<<<<<<<<<
 *             raise ValueError("cannot load _RslSweep with NULL")
 *         self._Sweep = Sweep
 */
  }

  /* "pyart/io/_rsl_interface.pyx":649
 *         if Sweep is NULL:
 *             raise ValueError("cannot load _RslSweep with NULL")
 *         self._Sweep = Sweep             # <<<<<<<<<<<<<<
 *         self._dealloc = 0
 * 
 */
  __pyx_v_self->_Sweep = __pyx_v_Sweep;

  /* "pyart/io/_rsl_interface.pyx":650
 *             raise ValueError("cannot load _RslSweep with NULL")
 *         self._Sweep = Sweep
 *         self._dealloc = 0             # <<<<<<<<<<<<<<
 * 
 *     def get_ray(self, int ray_number):
 */
  __pyx_v_self->_dealloc = 0;

  /* "pyart/io/_rsl_interface.pyx":645
 *             _rsl_h.RSL_free_sweep(self._Sweep)
 * 
 *     cdef load(self, _rsl_h.Sweep * Sweep):             # <<<<<<<<<<<<<<
 *         """ Load the _RslSweep object, must be called after creation. """
 *         if Sweep is NULL:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslSweep.load", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":652
 *         self._dealloc = 0
 * 
 *     def get_ray(self, int ray_number):             # <<<<<<<<<<<<<<
 *         """
 *         get_ray(ray_number)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_3get_ray(PyObject *__pyx_v_self, PyObject *__pyx_arg_ray_number); /*proto*/
static char __pyx_doc_5pyart_2io_14_rsl_interface_9_RslSweep_2get_ray[] = "\n        get_ray(ray_number)\n\n        Return a _RslRay for a given ray.\n\n        Parameters\n        ----------\n        ray_number : int\n            Ray number to retrieve\n\n        Returns\n        -------\n        ray : _RslRay\n            _RslRay object containing the requested ray.\n\n        ";
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_3get_ray(PyObject *__pyx_v_self, PyObject *__pyx_arg_ray_number) {
  int __pyx_v_ray_number;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_ray (wrapper)", 0);
  assert(__pyx_arg_ray_number); {
    __pyx_v_ray_number = __Pyx_PyInt_As_int(__pyx_arg_ray_number); if (unlikely((__pyx_v_ray_number == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 652, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslSweep.get_ray", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_2get_ray(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *)__pyx_v_self), ((int)__pyx_v_ray_number));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_2get_ray(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self, int __pyx_v_ray_number) {
  struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *__pyx_v_rslray = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_ray", 0);

  /* "pyart/io/_rsl_interface.pyx":669
 * 
 *         """
 *         if ray_number < 0 or ray_number >= self._Sweep.h.nrays:             # <<<<<<<<<<<<<<
 *             raise ValueError('invalid ray_number')
 *         rslray = _RslRay()
 */
  __pyx_t_2 = ((__pyx_v_ray_number < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ray_number >= __pyx_v_self->_Sweep->h.nrays) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "pyart/io/_rsl_interface.pyx":670
 *         """
 *         if ray_number < 0 or ray_number >= self._Sweep.h.nrays:
 *             raise ValueError('invalid ray_number')             # <<<<<<<<<<<<<<
 *         rslray = _RslRay()
 *         rslray.load(self._Sweep.ray[ray_number])
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 670, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 670, __pyx_L1_error)

    /* "pyart/io/_rsl_interface.pyx":669
 * 
 *         """
 *         if ray_number < 0 or ray_number >= self._Sweep.h.nrays:             # <<<<<<<<<<<<<<
 *             raise ValueError('invalid ray_number')
 *         rslray = _RslRay()
 */
  }

  /* "pyart/io/_rsl_interface.pyx":671
 *         if ray_number < 0 or ray_number >= self._Sweep.h.nrays:
 *             raise ValueError('invalid ray_number')
 *         rslray = _RslRay()             # <<<<<<<<<<<<<<
 *         rslray.load(self._Sweep.ray[ray_number])
 *         return rslray
 */
  __pyx_t_3 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_5pyart_2io_14_rsl_interface__RslRay)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 671, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_rslray = ((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "pyart/io/_rsl_interface.pyx":672
 *             raise ValueError('invalid ray_number')
 *         rslray = _RslRay()
 *         rslray.load(self._Sweep.ray[ray_number])             # <<<<<<<<<<<<<<
 *         return rslray
 * 
 */
  __pyx_t_3 = ((struct __pyx_vtabstruct_5pyart_2io_14_rsl_interface__RslRay *)__pyx_v_rslray->__pyx_vtab)->load(__pyx_v_rslray, (__pyx_v_self->_Sweep->ray[__pyx_v_ray_number])); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 672, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pyart/io/_rsl_interface.pyx":673
 *         rslray = _RslRay()
 *         rslray.load(self._Sweep.ray[ray_number])
 *         return rslray             # <<<<<<<<<<<<<<
 * 
 *     def get_data(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_rslray));
  __pyx_r = ((PyObject *)__pyx_v_rslray);
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":652
 *         self._dealloc = 0
 * 
 *     def get_ray(self, int ray_number):             # <<<<<<<<<<<<<<
 *         """
 *         get_ray(ray_number)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslSweep.get_ray", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_rslray);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":675
 *         return rslray
 * 
 *     def get_data(self):             # <<<<<<<<<<<<<<
 *         """
 *         get_data()
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_5get_data(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5pyart_2io_14_rsl_interface_9_RslSweep_4get_data[] = "\n        get_data()\n\n        Return the two-dimensional data contained in the sweep.\n\n        If a given ray has few bins than the first ray, the missing bins\n        will be filled with 131072.0\n        ";
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_5get_data(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_data (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_4get_data(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_4get_data(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self) {
  Range __pyx_v_raw;
  Ray *__pyx_v_ray;
  PyArrayObject *__pyx_v_data = 0;
  Sweep *__pyx_v_sweep;
  int __pyx_v_nrays;
  int __pyx_v_nbins;
  PyObject *__pyx_v_shape = NULL;
  int __pyx_v_nray;
  int __pyx_v_nbin;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_data;
  __Pyx_Buffer __pyx_pybuffer_data;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Sweep *__pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyArrayObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  int __pyx_t_12;
  int __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  int __pyx_t_18;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_data", 0);
  __pyx_pybuffer_data.pybuffer.buf = NULL;
  __pyx_pybuffer_data.refcount = 0;
  __pyx_pybuffernd_data.data = NULL;
  __pyx_pybuffernd_data.rcbuffer = &__pyx_pybuffer_data;

  /* "pyart/io/_rsl_interface.pyx":688
 *         cdef np.ndarray[np.float32_t, ndim = 2] data
 * 
 *         sweep = self._Sweep             # <<<<<<<<<<<<<<
 *         nrays = sweep.h.nrays
 *         nbins = sweep.ray[0].h.nbins
 */
  __pyx_t_1 = __pyx_v_self->_Sweep;
  __pyx_v_sweep = __pyx_t_1;

  /* "pyart/io/_rsl_interface.pyx":689
 * 
 *         sweep = self._Sweep
 *         nrays = sweep.h.nrays             # <<<<<<<<<<<<<<
 *         nbins = sweep.ray[0].h.nbins
 * 
 */
  __pyx_t_2 = __pyx_v_sweep->h.nrays;
  __pyx_v_nrays = __pyx_t_2;

  /* "pyart/io/_rsl_interface.pyx":690
 *         sweep = self._Sweep
 *         nrays = sweep.h.nrays
 *         nbins = sweep.ray[0].h.nbins             # <<<<<<<<<<<<<<
 * 
 *         shape = (nrays, nbins)
 */
  __pyx_t_2 = (__pyx_v_sweep->ray[0])->h.nbins;
  __pyx_v_nbins = __pyx_t_2;

  /* "pyart/io/_rsl_interface.pyx":692
 *         nbins = sweep.ray[0].h.nbins
 * 
 *         shape = (nrays, nbins)             # <<<<<<<<<<<<<<
 *         data = np.zeros(shape, dtype='float32') + 1.31072000e+05
 *         for nray in range(nrays):
 */
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_nrays); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 692, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_nbins); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 692, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 692, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_4);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_v_shape = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "pyart/io/_rsl_interface.pyx":693
 * 
 *         shape = (nrays, nbins)
 *         data = np.zeros(shape, dtype='float32') + 1.31072000e+05             # <<<<<<<<<<<<<<
 *         for nray in range(nrays):
 *             ray = sweep.ray[nray]
 */
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 693, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 693, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 693, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_v_shape);
  __Pyx_GIVEREF(__pyx_v_shape);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_shape);
  __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 693, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_n_u_float32) < 0) __PYX_ERR(0, 693, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, __pyx_t_3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 693, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyFloat_AddObjC(__pyx_t_6, __pyx_float_1_31072000e_05, 1.31072000e+05, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 693, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 693, __pyx_L1_error)
  __pyx_t_7 = ((PyArrayObject *)__pyx_t_3);
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_data.rcbuffer->pybuffer);
    __pyx_t_2 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_data.rcbuffer->pybuffer, (PyObject*)__pyx_t_7, &__Pyx_TypeInfo_nn___pyx_t_5numpy_float32_t, PyBUF_FORMAT| PyBUF_STRIDES| PyBUF_WRITABLE, 2, 0, __pyx_stack);
    if (unlikely(__pyx_t_2 < 0)) {
      PyErr_Fetch(&__pyx_t_8, &__pyx_t_9, &__pyx_t_10);
      if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_data.rcbuffer->pybuffer, (PyObject*)__pyx_v_data, &__Pyx_TypeInfo_nn___pyx_t_5numpy_float32_t, PyBUF_FORMAT| PyBUF_STRIDES| PyBUF_WRITABLE, 2, 0, __pyx_stack) == -1)) {
        Py_XDECREF(__pyx_t_8); Py_XDECREF(__pyx_t_9); Py_XDECREF(__pyx_t_10);
        __Pyx_RaiseBufferFallbackError();
      } else {
        PyErr_Restore(__pyx_t_8, __pyx_t_9, __pyx_t_10);
      }
      __pyx_t_8 = __pyx_t_9 = __pyx_t_10 = 0;
    }
    __pyx_pybuffernd_data.diminfo[0].strides = __pyx_pybuffernd_data.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_data.diminfo[0].shape = __pyx_pybuffernd_data.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_data.diminfo[1].strides = __pyx_pybuffernd_data.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_data.diminfo[1].shape = __pyx_pybuffernd_data.rcbuffer->pybuffer.shape[1];
    if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 693, __pyx_L1_error)
  }
  __pyx_t_7 = 0;
  __pyx_v_data = ((PyArrayObject *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "pyart/io/_rsl_interface.pyx":694
 *         shape = (nrays, nbins)
 *         data = np.zeros(shape, dtype='float32') + 1.31072000e+05
 *         for nray in range(nrays):             # <<<<<<<<<<<<<<
 *             ray = sweep.ray[nray]
 *             assert ray is not NULL
 */
  __pyx_t_2 = __pyx_v_nrays;
  __pyx_t_11 = __pyx_t_2;
  for (__pyx_t_12 = 0; __pyx_t_12 < __pyx_t_11; __pyx_t_12+=1) {
    __pyx_v_nray = __pyx_t_12;

    /* "pyart/io/_rsl_interface.pyx":695
 *         data = np.zeros(shape, dtype='float32') + 1.31072000e+05
 *         for nray in range(nrays):
 *             ray = sweep.ray[nray]             # <<<<<<<<<<<<<<
 *             assert ray is not NULL
 *             nbins = ray.h.nbins
 */
    __pyx_v_ray = (__pyx_v_sweep->ray[__pyx_v_nray]);

    /* "pyart/io/_rsl_interface.pyx":696
 *         for nray in range(nrays):
 *             ray = sweep.ray[nray]
 *             assert ray is not NULL             # <<<<<<<<<<<<<<
 *             nbins = ray.h.nbins
 *             for nbin in range(nbins):
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      if (unlikely(!((__pyx_v_ray != NULL) != 0))) {
        PyErr_SetNone(PyExc_AssertionError);
        __PYX_ERR(0, 696, __pyx_L1_error)
      }
    }
    #endif

    /* "pyart/io/_rsl_interface.pyx":697
 *             ray = sweep.ray[nray]
 *             assert ray is not NULL
 *             nbins = ray.h.nbins             # <<<<<<<<<<<<<<
 *             for nbin in range(nbins):
 *                 raw = ray.range[nbin]
 */
    __pyx_t_13 = __pyx_v_ray->h.nbins;
    __pyx_v_nbins = __pyx_t_13;

    /* "pyart/io/_rsl_interface.pyx":698
 *             assert ray is not NULL
 *             nbins = ray.h.nbins
 *             for nbin in range(nbins):             # <<<<<<<<<<<<<<
 *                 raw = ray.range[nbin]
 *                 data[nray, nbin] = ray.h.f(raw)
 */
    __pyx_t_13 = __pyx_v_nbins;
    __pyx_t_14 = __pyx_t_13;
    for (__pyx_t_15 = 0; __pyx_t_15 < __pyx_t_14; __pyx_t_15+=1) {
      __pyx_v_nbin = __pyx_t_15;

      /* "pyart/io/_rsl_interface.pyx":699
 *             nbins = ray.h.nbins
 *             for nbin in range(nbins):
 *                 raw = ray.range[nbin]             # <<<<<<<<<<<<<<
 *                 data[nray, nbin] = ray.h.f(raw)
 *         return data
 */
      __pyx_v_raw = (__pyx_v_ray->range[__pyx_v_nbin]);

      /* "pyart/io/_rsl_interface.pyx":700
 *             for nbin in range(nbins):
 *                 raw = ray.range[nbin]
 *                 data[nray, nbin] = ray.h.f(raw)             # <<<<<<<<<<<<<<
 *         return data
 * 
 */
      __pyx_t_16 = __pyx_v_nray;
      __pyx_t_17 = __pyx_v_nbin;
      __pyx_t_18 = -1;
      if (__pyx_t_16 < 0) {
        __pyx_t_16 += __pyx_pybuffernd_data.diminfo[0].shape;
        if (unlikely(__pyx_t_16 < 0)) __pyx_t_18 = 0;
      } else if (unlikely(__pyx_t_16 >= __pyx_pybuffernd_data.diminfo[0].shape)) __pyx_t_18 = 0;
      if (__pyx_t_17 < 0) {
        __pyx_t_17 += __pyx_pybuffernd_data.diminfo[1].shape;
        if (unlikely(__pyx_t_17 < 0)) __pyx_t_18 = 1;
      } else if (unlikely(__pyx_t_17 >= __pyx_pybuffernd_data.diminfo[1].shape)) __pyx_t_18 = 1;
      if (unlikely(__pyx_t_18 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_18);
        __PYX_ERR(0, 700, __pyx_L1_error)
      }
      *__Pyx_BufPtrStrided2d(__pyx_t_5numpy_float32_t *, __pyx_pybuffernd_data.rcbuffer->pybuffer.buf, __pyx_t_16, __pyx_pybuffernd_data.diminfo[0].strides, __pyx_t_17, __pyx_pybuffernd_data.diminfo[1].strides) = __pyx_v_ray->h.f(__pyx_v_raw);
    }
  }

  /* "pyart/io/_rsl_interface.pyx":701
 *                 raw = ray.range[nbin]
 *                 data[nray, nbin] = ray.h.f(raw)
 *         return data             # <<<<<<<<<<<<<<
 * 
 *     # header properties mapped to class attributes.
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_data));
  __pyx_r = ((PyObject *)__pyx_v_data);
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":675
 *         return rslray
 * 
 *     def get_data(self):             # <<<<<<<<<<<<<<
 *         """
 *         get_data()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_data.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslSweep.get_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_data.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_data);
  __Pyx_XDECREF(__pyx_v_shape);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":705
 *     # header properties mapped to class attributes.
 *     property sweep_num:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Sweep.h.sweep_num
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_9sweep_num_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_9sweep_num_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_9sweep_num___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_9sweep_num___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":706
 *     property sweep_num:
 *         def __get__(self):
 *             return self._Sweep.h.sweep_num             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int sweep_num):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Sweep->h.sweep_num); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 706, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":705
 *     # header properties mapped to class attributes.
 *     property sweep_num:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Sweep.h.sweep_num
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslSweep.sweep_num.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":708
 *             return self._Sweep.h.sweep_num
 * 
 *         def __set__(self, int sweep_num):             # <<<<<<<<<<<<<<
 *             self._Sweep.h.sweep_num = sweep_num
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_9sweep_num_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_sweep_num); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_9sweep_num_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_sweep_num) {
  int __pyx_v_sweep_num;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_sweep_num); {
    __pyx_v_sweep_num = __Pyx_PyInt_As_int(__pyx_arg_sweep_num); if (unlikely((__pyx_v_sweep_num == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 708, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslSweep.sweep_num.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_9sweep_num_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *)__pyx_v_self), ((int)__pyx_v_sweep_num));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_9sweep_num_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self, int __pyx_v_sweep_num) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":709
 * 
 *         def __set__(self, int sweep_num):
 *             self._Sweep.h.sweep_num = sweep_num             # <<<<<<<<<<<<<<
 * 
 *     property elev:
 */
  __pyx_v_self->_Sweep->h.sweep_num = __pyx_v_sweep_num;

  /* "pyart/io/_rsl_interface.pyx":708
 *             return self._Sweep.h.sweep_num
 * 
 *         def __set__(self, int sweep_num):             # <<<<<<<<<<<<<<
 *             self._Sweep.h.sweep_num = sweep_num
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":712
 * 
 *     property elev:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Sweep.h.elev
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_4elev_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_4elev_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_4elev___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_4elev___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":713
 *     property elev:
 *         def __get__(self):
 *             return self._Sweep.h.elev             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float elev):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Sweep->h.elev); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 713, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":712
 * 
 *     property elev:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Sweep.h.elev
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslSweep.elev.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":715
 *             return self._Sweep.h.elev
 * 
 *         def __set__(self, float elev):             # <<<<<<<<<<<<<<
 *             self._Sweep.h.elev = elev
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_4elev_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_elev); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_4elev_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_elev) {
  float __pyx_v_elev;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_elev); {
    __pyx_v_elev = __pyx_PyFloat_AsFloat(__pyx_arg_elev); if (unlikely((__pyx_v_elev == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 715, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslSweep.elev.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_4elev_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *)__pyx_v_self), ((float)__pyx_v_elev));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_4elev_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self, float __pyx_v_elev) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":716
 * 
 *         def __set__(self, float elev):
 *             self._Sweep.h.elev = elev             # <<<<<<<<<<<<<<
 * 
 *     property azimuth:
 */
  __pyx_v_self->_Sweep->h.elev = __pyx_v_elev;

  /* "pyart/io/_rsl_interface.pyx":715
 *             return self._Sweep.h.elev
 * 
 *         def __set__(self, float elev):             # <<<<<<<<<<<<<<
 *             self._Sweep.h.elev = elev
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":719
 * 
 *     property azimuth:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Sweep.h.azimuth
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_7azimuth_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_7azimuth_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_7azimuth___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_7azimuth___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":720
 *     property azimuth:
 *         def __get__(self):
 *             return self._Sweep.h.azimuth             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float azimuth):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Sweep->h.azimuth); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 720, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":719
 * 
 *     property azimuth:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Sweep.h.azimuth
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslSweep.azimuth.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":722
 *             return self._Sweep.h.azimuth
 * 
 *         def __set__(self, float azimuth):             # <<<<<<<<<<<<<<
 *             self._Sweep.h.azimuth = azimuth
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_7azimuth_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_azimuth); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_7azimuth_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_azimuth) {
  float __pyx_v_azimuth;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_azimuth); {
    __pyx_v_azimuth = __pyx_PyFloat_AsFloat(__pyx_arg_azimuth); if (unlikely((__pyx_v_azimuth == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 722, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslSweep.azimuth.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_7azimuth_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *)__pyx_v_self), ((float)__pyx_v_azimuth));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_7azimuth_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self, float __pyx_v_azimuth) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":723
 * 
 *         def __set__(self, float azimuth):
 *             self._Sweep.h.azimuth = azimuth             # <<<<<<<<<<<<<<
 * 
 *     property beam_width:
 */
  __pyx_v_self->_Sweep->h.azimuth = __pyx_v_azimuth;

  /* "pyart/io/_rsl_interface.pyx":722
 *             return self._Sweep.h.azimuth
 * 
 *         def __set__(self, float azimuth):             # <<<<<<<<<<<<<<
 *             self._Sweep.h.azimuth = azimuth
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":726
 * 
 *     property beam_width:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Sweep.h.beam_width
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_10beam_width_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_10beam_width_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_10beam_width___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_10beam_width___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":727
 *     property beam_width:
 *         def __get__(self):
 *             return self._Sweep.h.beam_width             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float beam_width):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Sweep->h.beam_width); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 727, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":726
 * 
 *     property beam_width:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Sweep.h.beam_width
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslSweep.beam_width.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":729
 *             return self._Sweep.h.beam_width
 * 
 *         def __set__(self, float beam_width):             # <<<<<<<<<<<<<<
 *             self._Sweep.h.beam_width = beam_width
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_10beam_width_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_beam_width); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_10beam_width_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_beam_width) {
  float __pyx_v_beam_width;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_beam_width); {
    __pyx_v_beam_width = __pyx_PyFloat_AsFloat(__pyx_arg_beam_width); if (unlikely((__pyx_v_beam_width == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 729, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslSweep.beam_width.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_10beam_width_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *)__pyx_v_self), ((float)__pyx_v_beam_width));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_10beam_width_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self, float __pyx_v_beam_width) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":730
 * 
 *         def __set__(self, float beam_width):
 *             self._Sweep.h.beam_width = beam_width             # <<<<<<<<<<<<<<
 * 
 *     property vert_half_bw:
 */
  __pyx_v_self->_Sweep->h.beam_width = __pyx_v_beam_width;

  /* "pyart/io/_rsl_interface.pyx":729
 *             return self._Sweep.h.beam_width
 * 
 *         def __set__(self, float beam_width):             # <<<<<<<<<<<<<<
 *             self._Sweep.h.beam_width = beam_width
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":733
 * 
 *     property vert_half_bw:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Sweep.h.vert_half_bw
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_12vert_half_bw_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_12vert_half_bw_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_12vert_half_bw___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_12vert_half_bw___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":734
 *     property vert_half_bw:
 *         def __get__(self):
 *             return self._Sweep.h.vert_half_bw             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float vert_half_bw):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Sweep->h.vert_half_bw); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 734, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":733
 * 
 *     property vert_half_bw:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Sweep.h.vert_half_bw
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslSweep.vert_half_bw.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":736
 *             return self._Sweep.h.vert_half_bw
 * 
 *         def __set__(self, float vert_half_bw):             # <<<<<<<<<<<<<<
 *             self._Sweep.h.vert_half_bw = vert_half_bw
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_12vert_half_bw_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_vert_half_bw); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_12vert_half_bw_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_vert_half_bw) {
  float __pyx_v_vert_half_bw;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_vert_half_bw); {
    __pyx_v_vert_half_bw = __pyx_PyFloat_AsFloat(__pyx_arg_vert_half_bw); if (unlikely((__pyx_v_vert_half_bw == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 736, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslSweep.vert_half_bw.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_12vert_half_bw_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *)__pyx_v_self), ((float)__pyx_v_vert_half_bw));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_12vert_half_bw_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self, float __pyx_v_vert_half_bw) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":737
 * 
 *         def __set__(self, float vert_half_bw):
 *             self._Sweep.h.vert_half_bw = vert_half_bw             # <<<<<<<<<<<<<<
 * 
 *     property horz_half_bw:
 */
  __pyx_v_self->_Sweep->h.vert_half_bw = __pyx_v_vert_half_bw;

  /* "pyart/io/_rsl_interface.pyx":736
 *             return self._Sweep.h.vert_half_bw
 * 
 *         def __set__(self, float vert_half_bw):             # <<<<<<<<<<<<<<
 *             self._Sweep.h.vert_half_bw = vert_half_bw
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":740
 * 
 *     property horz_half_bw:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Sweep.h.horz_half_bw
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_12horz_half_bw_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_12horz_half_bw_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_12horz_half_bw___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_12horz_half_bw___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":741
 *     property horz_half_bw:
 *         def __get__(self):
 *             return self._Sweep.h.horz_half_bw             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float horz_half_bw):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Sweep->h.horz_half_bw); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 741, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":740
 * 
 *     property horz_half_bw:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Sweep.h.horz_half_bw
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslSweep.horz_half_bw.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":743
 *             return self._Sweep.h.horz_half_bw
 * 
 *         def __set__(self, float horz_half_bw):             # <<<<<<<<<<<<<<
 *             self._Sweep.h.horz_half_bw = horz_half_bw
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_12horz_half_bw_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_horz_half_bw); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_12horz_half_bw_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_horz_half_bw) {
  float __pyx_v_horz_half_bw;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_horz_half_bw); {
    __pyx_v_horz_half_bw = __pyx_PyFloat_AsFloat(__pyx_arg_horz_half_bw); if (unlikely((__pyx_v_horz_half_bw == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 743, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslSweep.horz_half_bw.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_12horz_half_bw_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *)__pyx_v_self), ((float)__pyx_v_horz_half_bw));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_12horz_half_bw_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self, float __pyx_v_horz_half_bw) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":744
 * 
 *         def __set__(self, float horz_half_bw):
 *             self._Sweep.h.horz_half_bw = horz_half_bw             # <<<<<<<<<<<<<<
 * 
 *     property nrays:
 */
  __pyx_v_self->_Sweep->h.horz_half_bw = __pyx_v_horz_half_bw;

  /* "pyart/io/_rsl_interface.pyx":743
 *             return self._Sweep.h.horz_half_bw
 * 
 *         def __set__(self, float horz_half_bw):             # <<<<<<<<<<<<<<
 *             self._Sweep.h.horz_half_bw = horz_half_bw
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":747
 * 
 *     property nrays:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Sweep.h.nrays
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_5nrays_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_5nrays_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_5nrays___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_5nrays___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":748
 *     property nrays:
 *         def __get__(self):
 *             return self._Sweep.h.nrays             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int nrays):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Sweep->h.nrays); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 748, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":747
 * 
 *     property nrays:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Sweep.h.nrays
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslSweep.nrays.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":750
 *             return self._Sweep.h.nrays
 * 
 *         def __set__(self, int nrays):             # <<<<<<<<<<<<<<
 *             self._Sweep.h.nrays = nrays
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_5nrays_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_nrays); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_5nrays_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_nrays) {
  int __pyx_v_nrays;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_nrays); {
    __pyx_v_nrays = __Pyx_PyInt_As_int(__pyx_arg_nrays); if (unlikely((__pyx_v_nrays == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 750, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslSweep.nrays.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_5nrays_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *)__pyx_v_self), ((int)__pyx_v_nrays));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_5nrays_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self, int __pyx_v_nrays) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":751
 * 
 *         def __set__(self, int nrays):
 *             self._Sweep.h.nrays = nrays             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_self->_Sweep->h.nrays = __pyx_v_nrays;

  /* "pyart/io/_rsl_interface.pyx":750
 *             return self._Sweep.h.nrays
 * 
 *         def __set__(self, int nrays):             # <<<<<<<<<<<<<<
 *             self._Sweep.h.nrays = nrays
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._Sweep cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_6__reduce_cython__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._Sweep cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._Sweep cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._Sweep cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslSweep.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._Sweep cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._Sweep cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_8__setstate_cython__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_9_RslSweep_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self._Sweep cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._Sweep cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._Sweep cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._Sweep cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslSweep.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":771
 *     """
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self._dealloc == 1:
 *             _rsl_h.RSL_free_volume(self._Volume)
 */

/* Python wrapper */
static void __pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_1__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_1__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume___dealloc__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume___dealloc__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "pyart/io/_rsl_interface.pyx":772
 * 
 *     def __dealloc__(self):
 *         if self._dealloc == 1:             # <<<<<<<<<<<<<<
 *             _rsl_h.RSL_free_volume(self._Volume)
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->_dealloc == 1) != 0);
  if (__pyx_t_1) {

    /* "pyart/io/_rsl_interface.pyx":773
 *     def __dealloc__(self):
 *         if self._dealloc == 1:
 *             _rsl_h.RSL_free_volume(self._Volume)             # <<<<<<<<<<<<<<
 * 
 *     cdef load(self, _rsl_h.Volume * Volume):
 */
    RSL_free_volume(__pyx_v_self->_Volume);

    /* "pyart/io/_rsl_interface.pyx":772
 * 
 *     def __dealloc__(self):
 *         if self._dealloc == 1:             # <<<<<<<<<<<<<<
 *             _rsl_h.RSL_free_volume(self._Volume)
 * 
 */
  }

  /* "pyart/io/_rsl_interface.pyx":771
 *     """
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self._dealloc == 1:
 *             _rsl_h.RSL_free_volume(self._Volume)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "pyart/io/_rsl_interface.pyx":775
 *             _rsl_h.RSL_free_volume(self._Volume)
 * 
 *     cdef load(self, _rsl_h.Volume * Volume):             # <<<<<<<<<<<<<<
 *         """ Load the _RslVolume object, must be called after creation. """
 *         if Volume is NULL:
 */

static PyObject *__pyx_f_5pyart_2io_14_rsl_interface_10_RslVolume_load(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self, Volume *__pyx_v_Volume) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("load", 0);

  /* "pyart/io/_rsl_interface.pyx":777
 *     cdef load(self, _rsl_h.Volume * Volume):
 *         """ Load the _RslVolume object, must be called after creation. """
 *         if Volume is NULL:             # <<<<<<<<<<<<<<
 *             raise ValueError('cannot load _RslVolume with NULL')
 *         self._Volume = Volume
 */
  __pyx_t_1 = ((__pyx_v_Volume == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "pyart/io/_rsl_interface.pyx":778
 *         """ Load the _RslVolume object, must be called after creation. """
 *         if Volume is NULL:
 *             raise ValueError('cannot load _RslVolume with NULL')             # <<<<<<<<<<<<<<
 *         self._Volume = Volume
 *         self._dealloc = 0
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 778, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 778, __pyx_L1_error)

    /* "pyart/io/_rsl_interface.pyx":777
 *     cdef load(self, _rsl_h.Volume * Volume):
 *         """ Load the _RslVolume object, must be called after creation. """
 *         if Volume is NULL:             # <<<<<<<<<<<<<<
 *             raise ValueError('cannot load _RslVolume with NULL')
 *         self._Volume = Volume
 */
  }

  /* "pyart/io/_rsl_interface.pyx":779
 *         if Volume is NULL:
 *             raise ValueError('cannot load _RslVolume with NULL')
 *         self._Volume = Volume             # <<<<<<<<<<<<<<
 *         self._dealloc = 0
 * 
 */
  __pyx_v_self->_Volume = __pyx_v_Volume;

  /* "pyart/io/_rsl_interface.pyx":780
 *             raise ValueError('cannot load _RslVolume with NULL')
 *         self._Volume = Volume
 *         self._dealloc = 0             # <<<<<<<<<<<<<<
 * 
 *     def total_rays(self):
 */
  __pyx_v_self->_dealloc = 0;

  /* "pyart/io/_rsl_interface.pyx":775
 *             _rsl_h.RSL_free_volume(self._Volume)
 * 
 *     cdef load(self, _rsl_h.Volume * Volume):             # <<<<<<<<<<<<<<
 *         """ Load the _RslVolume object, must be called after creation. """
 *         if Volume is NULL:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslVolume.load", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":782
 *         self._dealloc = 0
 * 
 *     def total_rays(self):             # <<<<<<<<<<<<<<
 *         """
 *         total_rays()
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_3total_rays(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5pyart_2io_14_rsl_interface_10_RslVolume_2total_rays[] = "\n        total_rays()\n\n        Return the total number of rays present in all sweeps of the volume.\n        ";
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_3total_rays(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("total_rays (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_2total_rays(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_2total_rays(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("total_rays", 0);

  /* "pyart/io/_rsl_interface.pyx":788
 *         Return the total number of rays present in all sweeps of the volume.
 *         """
 *         return np.sum(self.get_nray_array())             # <<<<<<<<<<<<<<
 * 
 *     def get_nray_array(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 788, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_sum); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 788, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_nray_array); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 788, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 788, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 788, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":782
 *         self._dealloc = 0
 * 
 *     def total_rays(self):             # <<<<<<<<<<<<<<
 *         """
 *         total_rays()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslVolume.total_rays", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":790
 *         return np.sum(self.get_nray_array())
 * 
 *     def get_nray_array(self):             # <<<<<<<<<<<<<<
 *         """
 *         get_nray_array()
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_5get_nray_array(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5pyart_2io_14_rsl_interface_10_RslVolume_4get_nray_array[] = "\n        get_nray_array()\n\n        Return an array of the number of rays for each sweep.\n        ";
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_5get_nray_array(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_nray_array (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_4get_nray_array(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_4get_nray_array(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self) {
  Sweep *__pyx_v_sweep;
  PyObject *__pyx_v_nrays = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  Py_ssize_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_nray_array", 0);

  /* "pyart/io/_rsl_interface.pyx":797
 *         """
 *         cdef _rsl_h.Sweep * sweep
 *         nrays = np.empty((self.nsweeps), dtype='int32')             # <<<<<<<<<<<<<<
 *         for i in range(self.nsweeps):
 *             sweep = self._Volume.sweep[i]
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 797, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_empty); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 797, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nsweeps); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 797, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 797, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 797, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_n_u_int32) < 0) __PYX_ERR(0, 797, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 797, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_nrays = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "pyart/io/_rsl_interface.pyx":798
 *         cdef _rsl_h.Sweep * sweep
 *         nrays = np.empty((self.nsweeps), dtype='int32')
 *         for i in range(self.nsweeps):             # <<<<<<<<<<<<<<
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nsweeps); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 798, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 798, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_4 = __pyx_t_1; __Pyx_INCREF(__pyx_t_4); __pyx_t_5 = 0;
    __pyx_t_6 = NULL;
  } else {
    __pyx_t_5 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 798, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 798, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_6)) {
      if (likely(PyList_CheckExact(__pyx_t_4))) {
        if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_1); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 798, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 798, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_1); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 798, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 798, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_6(__pyx_t_4);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 798, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "pyart/io/_rsl_interface.pyx":799
 *         nrays = np.empty((self.nsweeps), dtype='int32')
 *         for i in range(self.nsweeps):
 *             sweep = self._Volume.sweep[i]             # <<<<<<<<<<<<<<
 *             assert sweep is not NULL
 *             nrays[i] = sweep.h.nrays
 */
    __pyx_t_7 = __Pyx_PyIndex_AsSsize_t(__pyx_v_i); if (unlikely((__pyx_t_7 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 799, __pyx_L1_error)
    __pyx_v_sweep = (__pyx_v_self->_Volume->sweep[__pyx_t_7]);

    /* "pyart/io/_rsl_interface.pyx":800
 *         for i in range(self.nsweeps):
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL             # <<<<<<<<<<<<<<
 *             nrays[i] = sweep.h.nrays
 *         return nrays
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      if (unlikely(!((__pyx_v_sweep != NULL) != 0))) {
        PyErr_SetNone(PyExc_AssertionError);
        __PYX_ERR(0, 800, __pyx_L1_error)
      }
    }
    #endif

    /* "pyart/io/_rsl_interface.pyx":801
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL
 *             nrays[i] = sweep.h.nrays             # <<<<<<<<<<<<<<
 *         return nrays
 * 
 */
    __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_sweep->h.nrays); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 801, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (unlikely(PyObject_SetItem(__pyx_v_nrays, __pyx_v_i, __pyx_t_1) < 0)) __PYX_ERR(0, 801, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyart/io/_rsl_interface.pyx":798
 *         cdef _rsl_h.Sweep * sweep
 *         nrays = np.empty((self.nsweeps), dtype='int32')
 *         for i in range(self.nsweeps):             # <<<<<<<<<<<<<<
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL
 */
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyart/io/_rsl_interface.pyx":802
 *             assert sweep is not NULL
 *             nrays[i] = sweep.h.nrays
 *         return nrays             # <<<<<<<<<<<<<<
 * 
 *     def get_sweep(self, int sweep_number):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_nrays);
  __pyx_r = __pyx_v_nrays;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":790
 *         return np.sum(self.get_nray_array())
 * 
 *     def get_nray_array(self):             # <<<<<<<<<<<<<<
 *         """
 *         get_nray_array()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslVolume.get_nray_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_nrays);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":804
 *         return nrays
 * 
 *     def get_sweep(self, int sweep_number):             # <<<<<<<<<<<<<<
 *         """
 *         get_sweep(sweep_numer)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_7get_sweep(PyObject *__pyx_v_self, PyObject *__pyx_arg_sweep_number); /*proto*/
static char __pyx_doc_5pyart_2io_14_rsl_interface_10_RslVolume_6get_sweep[] = "\n        get_sweep(sweep_numer)\n\n        Return a _RslSweep for a given sweep number.\n\n        Parameters\n        ----------\n        sweep_number : int\n            Sweep number to retrieve\n\n        Returns\n        -------\n        sweep : _RslSweep\n            _RslSweep object containing the requested sweep.\n\n        ";
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_7get_sweep(PyObject *__pyx_v_self, PyObject *__pyx_arg_sweep_number) {
  int __pyx_v_sweep_number;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_sweep (wrapper)", 0);
  assert(__pyx_arg_sweep_number); {
    __pyx_v_sweep_number = __Pyx_PyInt_As_int(__pyx_arg_sweep_number); if (unlikely((__pyx_v_sweep_number == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 804, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslVolume.get_sweep", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_6get_sweep(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *)__pyx_v_self), ((int)__pyx_v_sweep_number));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_6get_sweep(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self, int __pyx_v_sweep_number) {
  struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *__pyx_v_rslsweep = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_sweep", 0);

  /* "pyart/io/_rsl_interface.pyx":821
 * 
 *         """
 *         if sweep_number < 0 or sweep_number >= self._Volume.h.nsweeps:             # <<<<<<<<<<<<<<
 *             raise ValueError('invalid sweep_number')
 * 
 */
  __pyx_t_2 = ((__pyx_v_sweep_number < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_sweep_number >= __pyx_v_self->_Volume->h.nsweeps) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "pyart/io/_rsl_interface.pyx":822
 *         """
 *         if sweep_number < 0 or sweep_number >= self._Volume.h.nsweeps:
 *             raise ValueError('invalid sweep_number')             # <<<<<<<<<<<<<<
 * 
 *         rslsweep = _RslSweep()
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 822, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 822, __pyx_L1_error)

    /* "pyart/io/_rsl_interface.pyx":821
 * 
 *         """
 *         if sweep_number < 0 or sweep_number >= self._Volume.h.nsweeps:             # <<<<<<<<<<<<<<
 *             raise ValueError('invalid sweep_number')
 * 
 */
  }

  /* "pyart/io/_rsl_interface.pyx":824
 *             raise ValueError('invalid sweep_number')
 * 
 *         rslsweep = _RslSweep()             # <<<<<<<<<<<<<<
 *         rslsweep.load(self._Volume.sweep[sweep_number])
 *         return rslsweep
 */
  __pyx_t_3 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_5pyart_2io_14_rsl_interface__RslSweep)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 824, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_rslsweep = ((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "pyart/io/_rsl_interface.pyx":825
 * 
 *         rslsweep = _RslSweep()
 *         rslsweep.load(self._Volume.sweep[sweep_number])             # <<<<<<<<<<<<<<
 *         return rslsweep
 * 
 */
  __pyx_t_3 = ((struct __pyx_vtabstruct_5pyart_2io_14_rsl_interface__RslSweep *)__pyx_v_rslsweep->__pyx_vtab)->load(__pyx_v_rslsweep, (__pyx_v_self->_Volume->sweep[__pyx_v_sweep_number])); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 825, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pyart/io/_rsl_interface.pyx":826
 *         rslsweep = _RslSweep()
 *         rslsweep.load(self._Volume.sweep[sweep_number])
 *         return rslsweep             # <<<<<<<<<<<<<<
 * 
 *     def get_azimuth_and_elev_array(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_rslsweep));
  __pyx_r = ((PyObject *)__pyx_v_rslsweep);
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":804
 *         return nrays
 * 
 *     def get_sweep(self, int sweep_number):             # <<<<<<<<<<<<<<
 *         """
 *         get_sweep(sweep_numer)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslVolume.get_sweep", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_rslsweep);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":828
 *         return rslsweep
 * 
 *     def get_azimuth_and_elev_array(self):             # <<<<<<<<<<<<<<
 *         """
 *         get_azimuth_and_elev_array()
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_9get_azimuth_and_elev_array(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5pyart_2io_14_rsl_interface_10_RslVolume_8get_azimuth_and_elev_array[] = "\n        get_azimuth_and_elev_array()\n\n        Return azimuth and elevation array for each sweep and ray.\n        ";
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_9get_azimuth_and_elev_array(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_azimuth_and_elev_array (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_8get_azimuth_and_elev_array(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_8get_azimuth_and_elev_array(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self) {
  int __pyx_v_nrays;
  int __pyx_v_ray_count;
  Sweep *__pyx_v_sweep;
  Ray *__pyx_v_ray;
  PyObject *__pyx_v_total_rays = NULL;
  PyObject *__pyx_v_azimuth = NULL;
  PyObject *__pyx_v_elev = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_v_j = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  PyObject *(*__pyx_t_7)(PyObject *);
  Py_ssize_t __pyx_t_8;
  PyObject *(*__pyx_t_9)(PyObject *);
  Py_ssize_t __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_azimuth_and_elev_array", 0);

  /* "pyart/io/_rsl_interface.pyx":834
 *         Return azimuth and elevation array for each sweep and ray.
 *         """
 *         cdef int nrays = self._Volume.sweep[0].h.nrays             # <<<<<<<<<<<<<<
 *         cdef int ray_count
 *         cdef _rsl_h.Sweep * sweep
 */
  __pyx_t_1 = (__pyx_v_self->_Volume->sweep[0])->h.nrays;
  __pyx_v_nrays = __pyx_t_1;

  /* "pyart/io/_rsl_interface.pyx":840
 * 
 *         # create empty azimuth and elev output arrays
 *         total_rays = self.total_rays()             # <<<<<<<<<<<<<<
 *         azimuth = np.empty([total_rays], dtype='float32')
 *         elev = np.empty([total_rays], dtype='float32')
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_total_rays); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 840, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 840, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_total_rays = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pyart/io/_rsl_interface.pyx":841
 *         # create empty azimuth and elev output arrays
 *         total_rays = self.total_rays()
 *         azimuth = np.empty([total_rays], dtype='float32')             # <<<<<<<<<<<<<<
 *         elev = np.empty([total_rays], dtype='float32')
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 841, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 841, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 841, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_total_rays);
  __Pyx_GIVEREF(__pyx_v_total_rays);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_v_total_rays);
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 841, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 841, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_n_u_float32) < 0) __PYX_ERR(0, 841, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 841, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_azimuth = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "pyart/io/_rsl_interface.pyx":842
 *         total_rays = self.total_rays()
 *         azimuth = np.empty([total_rays], dtype='float32')
 *         elev = np.empty([total_rays], dtype='float32')             # <<<<<<<<<<<<<<
 * 
 *         # loop over the sweeps and rays storing azimuth and elev
 */
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 842, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_empty); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 842, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyList_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 842, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_v_total_rays);
  __Pyx_GIVEREF(__pyx_v_total_rays);
  PyList_SET_ITEM(__pyx_t_5, 0, __pyx_v_total_rays);
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 842, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 842, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_dtype, __pyx_n_u_float32) < 0) __PYX_ERR(0, 842, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 842, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_elev = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pyart/io/_rsl_interface.pyx":845
 * 
 *         # loop over the sweeps and rays storing azimuth and elev
 *         ray_count = 0             # <<<<<<<<<<<<<<
 *         for i in range(self.nsweeps):
 *             sweep = self._Volume.sweep[i]
 */
  __pyx_v_ray_count = 0;

  /* "pyart/io/_rsl_interface.pyx":846
 *         # loop over the sweeps and rays storing azimuth and elev
 *         ray_count = 0
 *         for i in range(self.nsweeps):             # <<<<<<<<<<<<<<
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nsweeps); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 846, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 846, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (likely(PyList_CheckExact(__pyx_t_5)) || PyTuple_CheckExact(__pyx_t_5)) {
    __pyx_t_3 = __pyx_t_5; __Pyx_INCREF(__pyx_t_3); __pyx_t_6 = 0;
    __pyx_t_7 = NULL;
  } else {
    __pyx_t_6 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 846, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 846, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  for (;;) {
    if (likely(!__pyx_t_7)) {
      if (likely(PyList_CheckExact(__pyx_t_3))) {
        if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_6); __Pyx_INCREF(__pyx_t_5); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 846, __pyx_L1_error)
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_3, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 846, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      } else {
        if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_6); __Pyx_INCREF(__pyx_t_5); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 846, __pyx_L1_error)
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_3, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 846, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      }
    } else {
      __pyx_t_5 = __pyx_t_7(__pyx_t_3);
      if (unlikely(!__pyx_t_5)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 846, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_5);
    }
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_5);
    __pyx_t_5 = 0;

    /* "pyart/io/_rsl_interface.pyx":847
 *         ray_count = 0
 *         for i in range(self.nsweeps):
 *             sweep = self._Volume.sweep[i]             # <<<<<<<<<<<<<<
 *             assert sweep is not NULL
 *             nrays = sweep.h.nrays
 */
    __pyx_t_8 = __Pyx_PyIndex_AsSsize_t(__pyx_v_i); if (unlikely((__pyx_t_8 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 847, __pyx_L1_error)
    __pyx_v_sweep = (__pyx_v_self->_Volume->sweep[__pyx_t_8]);

    /* "pyart/io/_rsl_interface.pyx":848
 *         for i in range(self.nsweeps):
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL             # <<<<<<<<<<<<<<
 *             nrays = sweep.h.nrays
 *             for j in range(nrays):
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      if (unlikely(!((__pyx_v_sweep != NULL) != 0))) {
        PyErr_SetNone(PyExc_AssertionError);
        __PYX_ERR(0, 848, __pyx_L1_error)
      }
    }
    #endif

    /* "pyart/io/_rsl_interface.pyx":849
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL
 *             nrays = sweep.h.nrays             # <<<<<<<<<<<<<<
 *             for j in range(nrays):
 *                 ray = sweep.ray[j]
 */
    __pyx_t_1 = __pyx_v_sweep->h.nrays;
    __pyx_v_nrays = __pyx_t_1;

    /* "pyart/io/_rsl_interface.pyx":850
 *             assert sweep is not NULL
 *             nrays = sweep.h.nrays
 *             for j in range(nrays):             # <<<<<<<<<<<<<<
 *                 ray = sweep.ray[j]
 *                 assert ray is not NULL
 */
    __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_nrays); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 850, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 850, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (likely(PyList_CheckExact(__pyx_t_4)) || PyTuple_CheckExact(__pyx_t_4)) {
      __pyx_t_5 = __pyx_t_4; __Pyx_INCREF(__pyx_t_5); __pyx_t_8 = 0;
      __pyx_t_9 = NULL;
    } else {
      __pyx_t_8 = -1; __pyx_t_5 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 850, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_9 = Py_TYPE(__pyx_t_5)->tp_iternext; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 850, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    for (;;) {
      if (likely(!__pyx_t_9)) {
        if (likely(PyList_CheckExact(__pyx_t_5))) {
          if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_5)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyList_GET_ITEM(__pyx_t_5, __pyx_t_8); __Pyx_INCREF(__pyx_t_4); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 850, __pyx_L1_error)
          #else
          __pyx_t_4 = PySequence_ITEM(__pyx_t_5, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 850, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        } else {
          if (__pyx_t_8 >= PyTuple_GET_SIZE(__pyx_t_5)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_5, __pyx_t_8); __Pyx_INCREF(__pyx_t_4); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 850, __pyx_L1_error)
          #else
          __pyx_t_4 = PySequence_ITEM(__pyx_t_5, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 850, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        }
      } else {
        __pyx_t_4 = __pyx_t_9(__pyx_t_5);
        if (unlikely(!__pyx_t_4)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 850, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_4);
      }
      __Pyx_XDECREF_SET(__pyx_v_j, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "pyart/io/_rsl_interface.pyx":851
 *             nrays = sweep.h.nrays
 *             for j in range(nrays):
 *                 ray = sweep.ray[j]             # <<<<<<<<<<<<<<
 *                 assert ray is not NULL
 *                 azimuth[ray_count + j] = ray.h.azimuth
 */
      __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_v_j); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 851, __pyx_L1_error)
      __pyx_v_ray = (__pyx_v_sweep->ray[__pyx_t_10]);

      /* "pyart/io/_rsl_interface.pyx":852
 *             for j in range(nrays):
 *                 ray = sweep.ray[j]
 *                 assert ray is not NULL             # <<<<<<<<<<<<<<
 *                 azimuth[ray_count + j] = ray.h.azimuth
 *                 elev[ray_count + j] = ray.h.elev
 */
      #ifndef CYTHON_WITHOUT_ASSERTIONS
      if (unlikely(!Py_OptimizeFlag)) {
        if (unlikely(!((__pyx_v_ray != NULL) != 0))) {
          PyErr_SetNone(PyExc_AssertionError);
          __PYX_ERR(0, 852, __pyx_L1_error)
        }
      }
      #endif

      /* "pyart/io/_rsl_interface.pyx":853
 *                 ray = sweep.ray[j]
 *                 assert ray is not NULL
 *                 azimuth[ray_count + j] = ray.h.azimuth             # <<<<<<<<<<<<<<
 *                 elev[ray_count + j] = ray.h.elev
 *             ray_count += nrays
 */
      __pyx_t_4 = PyFloat_FromDouble(__pyx_v_ray->h.azimuth); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 853, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_ray_count); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 853, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_11 = PyNumber_Add(__pyx_t_2, __pyx_v_j); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 853, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(PyObject_SetItem(__pyx_v_azimuth, __pyx_t_11, __pyx_t_4) < 0)) __PYX_ERR(0, 853, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyart/io/_rsl_interface.pyx":854
 *                 assert ray is not NULL
 *                 azimuth[ray_count + j] = ray.h.azimuth
 *                 elev[ray_count + j] = ray.h.elev             # <<<<<<<<<<<<<<
 *             ray_count += nrays
 *         return azimuth, elev
 */
      __pyx_t_4 = PyFloat_FromDouble(__pyx_v_ray->h.elev); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 854, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_11 = __Pyx_PyInt_From_int(__pyx_v_ray_count); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 854, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_2 = PyNumber_Add(__pyx_t_11, __pyx_v_j); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 854, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      if (unlikely(PyObject_SetItem(__pyx_v_elev, __pyx_t_2, __pyx_t_4) < 0)) __PYX_ERR(0, 854, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyart/io/_rsl_interface.pyx":850
 *             assert sweep is not NULL
 *             nrays = sweep.h.nrays
 *             for j in range(nrays):             # <<<<<<<<<<<<<<
 *                 ray = sweep.ray[j]
 *                 assert ray is not NULL
 */
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "pyart/io/_rsl_interface.pyx":855
 *                 azimuth[ray_count + j] = ray.h.azimuth
 *                 elev[ray_count + j] = ray.h.elev
 *             ray_count += nrays             # <<<<<<<<<<<<<<
 *         return azimuth, elev
 * 
 */
    __pyx_v_ray_count = (__pyx_v_ray_count + __pyx_v_nrays);

    /* "pyart/io/_rsl_interface.pyx":846
 *         # loop over the sweeps and rays storing azimuth and elev
 *         ray_count = 0
 *         for i in range(self.nsweeps):             # <<<<<<<<<<<<<<
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL
 */
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pyart/io/_rsl_interface.pyx":856
 *                 elev[ray_count + j] = ray.h.elev
 *             ray_count += nrays
 *         return azimuth, elev             # <<<<<<<<<<<<<<
 * 
 *     def get_sweep_fix_angles(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 856, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_azimuth);
  __Pyx_GIVEREF(__pyx_v_azimuth);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_azimuth);
  __Pyx_INCREF(__pyx_v_elev);
  __Pyx_GIVEREF(__pyx_v_elev);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_elev);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":828
 *         return rslsweep
 * 
 *     def get_azimuth_and_elev_array(self):             # <<<<<<<<<<<<<<
 *         """
 *         get_azimuth_and_elev_array()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslVolume.get_azimuth_and_elev_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_total_rays);
  __Pyx_XDECREF(__pyx_v_azimuth);
  __Pyx_XDECREF(__pyx_v_elev);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XDECREF(__pyx_v_j);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":858
 *         return azimuth, elev
 * 
 *     def get_sweep_fix_angles(self):             # <<<<<<<<<<<<<<
 *         """
 *         get_sweep_fix_angles()
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_11get_sweep_fix_angles(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5pyart_2io_14_rsl_interface_10_RslVolume_10get_sweep_fix_angles[] = "\n        get_sweep_fix_angles()\n\n        Return array of fix angle for each sweep.\n\n        Angles determined from the first ray in each sweep.\n        ";
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_11get_sweep_fix_angles(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_sweep_fix_angles (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_10get_sweep_fix_angles(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_10get_sweep_fix_angles(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self) {
  Sweep *__pyx_v_sweep;
  Ray *__pyx_v_ray;
  PyObject *__pyx_v_fix_angles = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  Py_ssize_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_sweep_fix_angles", 0);

  /* "pyart/io/_rsl_interface.pyx":868
 *         cdef _rsl_h.Sweep * sweep
 *         cdef _rsl_h.Ray * ray
 *         fix_angles = np.empty((self.nsweeps), dtype='float32')             # <<<<<<<<<<<<<<
 *         for i in range(self.nsweeps):
 *             sweep = self._Volume.sweep[i]
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 868, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_empty); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 868, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nsweeps); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 868, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 868, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 868, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_n_u_float32) < 0) __PYX_ERR(0, 868, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 868, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_fix_angles = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "pyart/io/_rsl_interface.pyx":869
 *         cdef _rsl_h.Ray * ray
 *         fix_angles = np.empty((self.nsweeps), dtype='float32')
 *         for i in range(self.nsweeps):             # <<<<<<<<<<<<<<
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nsweeps); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 869, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 869, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_4 = __pyx_t_1; __Pyx_INCREF(__pyx_t_4); __pyx_t_5 = 0;
    __pyx_t_6 = NULL;
  } else {
    __pyx_t_5 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 869, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 869, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_6)) {
      if (likely(PyList_CheckExact(__pyx_t_4))) {
        if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_1); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 869, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 869, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_1); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 869, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 869, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_6(__pyx_t_4);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 869, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "pyart/io/_rsl_interface.pyx":870
 *         fix_angles = np.empty((self.nsweeps), dtype='float32')
 *         for i in range(self.nsweeps):
 *             sweep = self._Volume.sweep[i]             # <<<<<<<<<<<<<<
 *             assert sweep is not NULL
 *             ray = sweep.ray[0]
 */
    __pyx_t_7 = __Pyx_PyIndex_AsSsize_t(__pyx_v_i); if (unlikely((__pyx_t_7 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 870, __pyx_L1_error)
    __pyx_v_sweep = (__pyx_v_self->_Volume->sweep[__pyx_t_7]);

    /* "pyart/io/_rsl_interface.pyx":871
 *         for i in range(self.nsweeps):
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL             # <<<<<<<<<<<<<<
 *             ray = sweep.ray[0]
 *             assert ray is not NULL
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      if (unlikely(!((__pyx_v_sweep != NULL) != 0))) {
        PyErr_SetNone(PyExc_AssertionError);
        __PYX_ERR(0, 871, __pyx_L1_error)
      }
    }
    #endif

    /* "pyart/io/_rsl_interface.pyx":872
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL
 *             ray = sweep.ray[0]             # <<<<<<<<<<<<<<
 *             assert ray is not NULL
 *             fix_angles[i] = ray.h.fix_angle
 */
    __pyx_v_ray = (__pyx_v_sweep->ray[0]);

    /* "pyart/io/_rsl_interface.pyx":873
 *             assert sweep is not NULL
 *             ray = sweep.ray[0]
 *             assert ray is not NULL             # <<<<<<<<<<<<<<
 *             fix_angles[i] = ray.h.fix_angle
 *         return fix_angles
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      if (unlikely(!((__pyx_v_ray != NULL) != 0))) {
        PyErr_SetNone(PyExc_AssertionError);
        __PYX_ERR(0, 873, __pyx_L1_error)
      }
    }
    #endif

    /* "pyart/io/_rsl_interface.pyx":874
 *             ray = sweep.ray[0]
 *             assert ray is not NULL
 *             fix_angles[i] = ray.h.fix_angle             # <<<<<<<<<<<<<<
 *         return fix_angles
 * 
 */
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_ray->h.fix_angle); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 874, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (unlikely(PyObject_SetItem(__pyx_v_fix_angles, __pyx_v_i, __pyx_t_1) < 0)) __PYX_ERR(0, 874, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyart/io/_rsl_interface.pyx":869
 *         cdef _rsl_h.Ray * ray
 *         fix_angles = np.empty((self.nsweeps), dtype='float32')
 *         for i in range(self.nsweeps):             # <<<<<<<<<<<<<<
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL
 */
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyart/io/_rsl_interface.pyx":875
 *             assert ray is not NULL
 *             fix_angles[i] = ray.h.fix_angle
 *         return fix_angles             # <<<<<<<<<<<<<<
 * 
 *     def get_sweep_azimuths(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_fix_angles);
  __pyx_r = __pyx_v_fix_angles;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":858
 *         return azimuth, elev
 * 
 *     def get_sweep_fix_angles(self):             # <<<<<<<<<<<<<<
 *         """
 *         get_sweep_fix_angles()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslVolume.get_sweep_fix_angles", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_fix_angles);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":877
 *         return fix_angles
 * 
 *     def get_sweep_azimuths(self):             # <<<<<<<<<<<<<<
 *         """
 *         get_sweep_azimuths()
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_13get_sweep_azimuths(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5pyart_2io_14_rsl_interface_10_RslVolume_12get_sweep_azimuths[] = "\n        get_sweep_azimuths()\n\n        Return azimuth array for each sweep.\n        ";
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_13get_sweep_azimuths(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_sweep_azimuths (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_12get_sweep_azimuths(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_12get_sweep_azimuths(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self) {
  Sweep *__pyx_v_sweep;
  PyObject *__pyx_v_azimuth = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  Py_ssize_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_sweep_azimuths", 0);

  /* "pyart/io/_rsl_interface.pyx":884
 *         """
 *         cdef _rsl_h.Sweep * sweep
 *         azimuth = np.empty((self.nsweeps), dtype='float32')             # <<<<<<<<<<<<<<
 *         for i in range(self.nsweeps):
 *             sweep = self._Volume.sweep[i]
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 884, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_empty); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 884, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nsweeps); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 884, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 884, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 884, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_n_u_float32) < 0) __PYX_ERR(0, 884, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 884, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_azimuth = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "pyart/io/_rsl_interface.pyx":885
 *         cdef _rsl_h.Sweep * sweep
 *         azimuth = np.empty((self.nsweeps), dtype='float32')
 *         for i in range(self.nsweeps):             # <<<<<<<<<<<<<<
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nsweeps); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 885, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 885, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_4 = __pyx_t_1; __Pyx_INCREF(__pyx_t_4); __pyx_t_5 = 0;
    __pyx_t_6 = NULL;
  } else {
    __pyx_t_5 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 885, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 885, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_6)) {
      if (likely(PyList_CheckExact(__pyx_t_4))) {
        if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_1); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 885, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 885, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_1); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 885, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 885, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_6(__pyx_t_4);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 885, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "pyart/io/_rsl_interface.pyx":886
 *         azimuth = np.empty((self.nsweeps), dtype='float32')
 *         for i in range(self.nsweeps):
 *             sweep = self._Volume.sweep[i]             # <<<<<<<<<<<<<<
 *             assert sweep is not NULL
 *             azimuth[i] = sweep.h.azimuth
 */
    __pyx_t_7 = __Pyx_PyIndex_AsSsize_t(__pyx_v_i); if (unlikely((__pyx_t_7 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 886, __pyx_L1_error)
    __pyx_v_sweep = (__pyx_v_self->_Volume->sweep[__pyx_t_7]);

    /* "pyart/io/_rsl_interface.pyx":887
 *         for i in range(self.nsweeps):
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL             # <<<<<<<<<<<<<<
 *             azimuth[i] = sweep.h.azimuth
 *         return azimuth
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      if (unlikely(!((__pyx_v_sweep != NULL) != 0))) {
        PyErr_SetNone(PyExc_AssertionError);
        __PYX_ERR(0, 887, __pyx_L1_error)
      }
    }
    #endif

    /* "pyart/io/_rsl_interface.pyx":888
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL
 *             azimuth[i] = sweep.h.azimuth             # <<<<<<<<<<<<<<
 *         return azimuth
 * 
 */
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_sweep->h.azimuth); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 888, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (unlikely(PyObject_SetItem(__pyx_v_azimuth, __pyx_v_i, __pyx_t_1) < 0)) __PYX_ERR(0, 888, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyart/io/_rsl_interface.pyx":885
 *         cdef _rsl_h.Sweep * sweep
 *         azimuth = np.empty((self.nsweeps), dtype='float32')
 *         for i in range(self.nsweeps):             # <<<<<<<<<<<<<<
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL
 */
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyart/io/_rsl_interface.pyx":889
 *             assert sweep is not NULL
 *             azimuth[i] = sweep.h.azimuth
 *         return azimuth             # <<<<<<<<<<<<<<
 * 
 *     def get_sweep_elevs(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_azimuth);
  __pyx_r = __pyx_v_azimuth;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":877
 *         return fix_angles
 * 
 *     def get_sweep_azimuths(self):             # <<<<<<<<<<<<<<
 *         """
 *         get_sweep_azimuths()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslVolume.get_sweep_azimuths", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_azimuth);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":891
 *         return azimuth
 * 
 *     def get_sweep_elevs(self):             # <<<<<<<<<<<<<<
 *         """
 *         get_sweep_elevs()
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_15get_sweep_elevs(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5pyart_2io_14_rsl_interface_10_RslVolume_14get_sweep_elevs[] = "\n        get_sweep_elevs()\n\n        Return elevation array for each sweep.\n        ";
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_15get_sweep_elevs(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_sweep_elevs (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_14get_sweep_elevs(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_14get_sweep_elevs(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self) {
  Sweep *__pyx_v_sweep;
  PyObject *__pyx_v_elev = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  Py_ssize_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_sweep_elevs", 0);

  /* "pyart/io/_rsl_interface.pyx":898
 *         """
 *         cdef _rsl_h.Sweep * sweep
 *         elev = np.empty((self.nsweeps), dtype='float32')             # <<<<<<<<<<<<<<
 *         for i in range(self.nsweeps):
 *             sweep = self._Volume.sweep[i]
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 898, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_empty); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 898, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nsweeps); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 898, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 898, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 898, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_n_u_float32) < 0) __PYX_ERR(0, 898, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 898, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_elev = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "pyart/io/_rsl_interface.pyx":899
 *         cdef _rsl_h.Sweep * sweep
 *         elev = np.empty((self.nsweeps), dtype='float32')
 *         for i in range(self.nsweeps):             # <<<<<<<<<<<<<<
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nsweeps); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 899, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 899, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_4 = __pyx_t_1; __Pyx_INCREF(__pyx_t_4); __pyx_t_5 = 0;
    __pyx_t_6 = NULL;
  } else {
    __pyx_t_5 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 899, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 899, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_6)) {
      if (likely(PyList_CheckExact(__pyx_t_4))) {
        if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_1); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 899, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 899, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_1); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 899, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 899, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_6(__pyx_t_4);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 899, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "pyart/io/_rsl_interface.pyx":900
 *         elev = np.empty((self.nsweeps), dtype='float32')
 *         for i in range(self.nsweeps):
 *             sweep = self._Volume.sweep[i]             # <<<<<<<<<<<<<<
 *             assert sweep is not NULL
 *             elev[i] = sweep.h.elev
 */
    __pyx_t_7 = __Pyx_PyIndex_AsSsize_t(__pyx_v_i); if (unlikely((__pyx_t_7 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 900, __pyx_L1_error)
    __pyx_v_sweep = (__pyx_v_self->_Volume->sweep[__pyx_t_7]);

    /* "pyart/io/_rsl_interface.pyx":901
 *         for i in range(self.nsweeps):
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL             # <<<<<<<<<<<<<<
 *             elev[i] = sweep.h.elev
 *         return elev
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      if (unlikely(!((__pyx_v_sweep != NULL) != 0))) {
        PyErr_SetNone(PyExc_AssertionError);
        __PYX_ERR(0, 901, __pyx_L1_error)
      }
    }
    #endif

    /* "pyart/io/_rsl_interface.pyx":902
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL
 *             elev[i] = sweep.h.elev             # <<<<<<<<<<<<<<
 *         return elev
 * 
 */
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_sweep->h.elev); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 902, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (unlikely(PyObject_SetItem(__pyx_v_elev, __pyx_v_i, __pyx_t_1) < 0)) __PYX_ERR(0, 902, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyart/io/_rsl_interface.pyx":899
 *         cdef _rsl_h.Sweep * sweep
 *         elev = np.empty((self.nsweeps), dtype='float32')
 *         for i in range(self.nsweeps):             # <<<<<<<<<<<<<<
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL
 */
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyart/io/_rsl_interface.pyx":903
 *             assert sweep is not NULL
 *             elev[i] = sweep.h.elev
 *         return elev             # <<<<<<<<<<<<<<
 * 
 *     def get_instr_params(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_elev);
  __pyx_r = __pyx_v_elev;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":891
 *         return azimuth
 * 
 *     def get_sweep_elevs(self):             # <<<<<<<<<<<<<<
 *         """
 *         get_sweep_elevs()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslVolume.get_sweep_elevs", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_elev);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":905
 *         return elev
 * 
 *     def get_instr_params(self):             # <<<<<<<<<<<<<<
 *         """
 *         get_instr_params()
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_17get_instr_params(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5pyart_2io_14_rsl_interface_10_RslVolume_16get_instr_params[] = "\n        get_instr_params()\n\n        Return instrumental parameter for the volume.\n\n        Returns\n        -------\n        pm_data : array, (nsweeps)\n            Array of prt modes.\n        nv_data : array, (total_rays)\n            Array of nyquist velocities.\n        pr_data : array, (total_rays)\n            Array of pulse repetition frequency in Hz.\n        ur_data : array, (total_rays)\n            Array of unambiguous ranges, in km.\n\n        ";
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_17get_instr_params(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_instr_params (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_16get_instr_params(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_16get_instr_params(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self) {
  int __pyx_v_nrays;
  int __pyx_v_ray_count;
  Sweep *__pyx_v_sweep;
  Ray *__pyx_v_ray;
  PyObject *__pyx_v_total_rays = NULL;
  PyObject *__pyx_v_nyq_vel = NULL;
  PyObject *__pyx_v_valid_nyq_vel = NULL;
  PyObject *__pyx_v_pm_data = NULL;
  PyObject *__pyx_v_nv_data = NULL;
  PyObject *__pyx_v_pr_data = NULL;
  PyObject *__pyx_v_ur_data = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_v_j = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  PyObject *(*__pyx_t_7)(PyObject *);
  Py_ssize_t __pyx_t_8;
  PyObject *(*__pyx_t_9)(PyObject *);
  Py_ssize_t __pyx_t_10;
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_instr_params", 0);

  /* "pyart/io/_rsl_interface.pyx":923
 * 
 *         """
 *         cdef int nrays = self._Volume.sweep[0].h.nrays             # <<<<<<<<<<<<<<
 *         cdef int ray_count
 *         cdef _rsl_h.Sweep * sweep
 */
  __pyx_t_1 = (__pyx_v_self->_Volume->sweep[0])->h.nrays;
  __pyx_v_nrays = __pyx_t_1;

  /* "pyart/io/_rsl_interface.pyx":931
 * 
 *         # initalize empty instrument parameter arrays
 *         total_rays = self.total_rays()             # <<<<<<<<<<<<<<
 *         nyq_vel = self._Volume.sweep[0].ray[0].h.nyq_vel
 *         valid_nyq_vel = abs(nyq_vel) > 0.1
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_total_rays); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 931, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 931, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_total_rays = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pyart/io/_rsl_interface.pyx":932
 *         # initalize empty instrument parameter arrays
 *         total_rays = self.total_rays()
 *         nyq_vel = self._Volume.sweep[0].ray[0].h.nyq_vel             # <<<<<<<<<<<<<<
 *         valid_nyq_vel = abs(nyq_vel) > 0.1
 *         pm_data = np.empty(self.nsweeps, dtype='|S24')
 */
  __pyx_t_2 = PyFloat_FromDouble(((__pyx_v_self->_Volume->sweep[0])->ray[0])->h.nyq_vel); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 932, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_nyq_vel = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pyart/io/_rsl_interface.pyx":933
 *         total_rays = self.total_rays()
 *         nyq_vel = self._Volume.sweep[0].ray[0].h.nyq_vel
 *         valid_nyq_vel = abs(nyq_vel) > 0.1             # <<<<<<<<<<<<<<
 *         pm_data = np.empty(self.nsweeps, dtype='|S24')
 *         nv_data = np.empty((total_rays), dtype='float32')
 */
  __pyx_t_2 = __Pyx_PyNumber_Absolute(__pyx_v_nyq_vel); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 933, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_2, __pyx_float_0_1, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 933, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_valid_nyq_vel = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pyart/io/_rsl_interface.pyx":934
 *         nyq_vel = self._Volume.sweep[0].ray[0].h.nyq_vel
 *         valid_nyq_vel = abs(nyq_vel) > 0.1
 *         pm_data = np.empty(self.nsweeps, dtype='|S24')             # <<<<<<<<<<<<<<
 *         nv_data = np.empty((total_rays), dtype='float32')
 *         pr_data = np.empty((total_rays), dtype='float32')
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 934, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_empty); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 934, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nsweeps); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 934, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 934, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 934, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_kp_u_S24) < 0) __PYX_ERR(0, 934, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 934, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_pm_data = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "pyart/io/_rsl_interface.pyx":935
 *         valid_nyq_vel = abs(nyq_vel) > 0.1
 *         pm_data = np.empty(self.nsweeps, dtype='|S24')
 *         nv_data = np.empty((total_rays), dtype='float32')             # <<<<<<<<<<<<<<
 *         pr_data = np.empty((total_rays), dtype='float32')
 *         ur_data = np.empty((total_rays), dtype='float32')
 */
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 935, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 935, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 935, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_v_total_rays);
  __Pyx_GIVEREF(__pyx_v_total_rays);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_total_rays);
  __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 935, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_n_u_float32) < 0) __PYX_ERR(0, 935, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 935, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_nv_data = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pyart/io/_rsl_interface.pyx":936
 *         pm_data = np.empty(self.nsweeps, dtype='|S24')
 *         nv_data = np.empty((total_rays), dtype='float32')
 *         pr_data = np.empty((total_rays), dtype='float32')             # <<<<<<<<<<<<<<
 *         ur_data = np.empty((total_rays), dtype='float32')
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 936, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 936, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 936, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_total_rays);
  __Pyx_GIVEREF(__pyx_v_total_rays);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_total_rays);
  __pyx_t_5 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 936, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_dtype, __pyx_n_u_float32) < 0) __PYX_ERR(0, 936, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_2, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 936, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_pr_data = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pyart/io/_rsl_interface.pyx":937
 *         nv_data = np.empty((total_rays), dtype='float32')
 *         pr_data = np.empty((total_rays), dtype='float32')
 *         ur_data = np.empty((total_rays), dtype='float32')             # <<<<<<<<<<<<<<
 * 
 *         # loop over sweeps and rays storing instrument parameters
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 937, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_empty); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 937, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 937, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_total_rays);
  __Pyx_GIVEREF(__pyx_v_total_rays);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_total_rays);
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 937, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_n_u_float32) < 0) __PYX_ERR(0, 937, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 937, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_ur_data = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "pyart/io/_rsl_interface.pyx":940
 * 
 *         # loop over sweeps and rays storing instrument parameters
 *         ray_count = 0             # <<<<<<<<<<<<<<
 *         for i in range(self.nsweeps):
 *             sweep = self._Volume.sweep[i]
 */
  __pyx_v_ray_count = 0;

  /* "pyart/io/_rsl_interface.pyx":941
 *         # loop over sweeps and rays storing instrument parameters
 *         ray_count = 0
 *         for i in range(self.nsweeps):             # <<<<<<<<<<<<<<
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nsweeps); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 941, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 941, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
    __pyx_t_4 = __pyx_t_2; __Pyx_INCREF(__pyx_t_4); __pyx_t_6 = 0;
    __pyx_t_7 = NULL;
  } else {
    __pyx_t_6 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 941, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 941, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  for (;;) {
    if (likely(!__pyx_t_7)) {
      if (likely(PyList_CheckExact(__pyx_t_4))) {
        if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_6); __Pyx_INCREF(__pyx_t_2); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 941, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_4, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 941, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_6); __Pyx_INCREF(__pyx_t_2); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 941, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_4, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 941, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_7(__pyx_t_4);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 941, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "pyart/io/_rsl_interface.pyx":942
 *         ray_count = 0
 *         for i in range(self.nsweeps):
 *             sweep = self._Volume.sweep[i]             # <<<<<<<<<<<<<<
 *             assert sweep is not NULL
 *             nrays = sweep.h.nrays
 */
    __pyx_t_8 = __Pyx_PyIndex_AsSsize_t(__pyx_v_i); if (unlikely((__pyx_t_8 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 942, __pyx_L1_error)
    __pyx_v_sweep = (__pyx_v_self->_Volume->sweep[__pyx_t_8]);

    /* "pyart/io/_rsl_interface.pyx":943
 *         for i in range(self.nsweeps):
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL             # <<<<<<<<<<<<<<
 *             nrays = sweep.h.nrays
 *             for j in range(nrays):
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      if (unlikely(!((__pyx_v_sweep != NULL) != 0))) {
        PyErr_SetNone(PyExc_AssertionError);
        __PYX_ERR(0, 943, __pyx_L1_error)
      }
    }
    #endif

    /* "pyart/io/_rsl_interface.pyx":944
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL
 *             nrays = sweep.h.nrays             # <<<<<<<<<<<<<<
 *             for j in range(nrays):
 *                 ray = sweep.ray[j]
 */
    __pyx_t_1 = __pyx_v_sweep->h.nrays;
    __pyx_v_nrays = __pyx_t_1;

    /* "pyart/io/_rsl_interface.pyx":945
 *             assert sweep is not NULL
 *             nrays = sweep.h.nrays
 *             for j in range(nrays):             # <<<<<<<<<<<<<<
 *                 ray = sweep.ray[j]
 *                 assert ray is not NULL
 */
    __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_nrays); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 945, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 945, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
      __pyx_t_2 = __pyx_t_3; __Pyx_INCREF(__pyx_t_2); __pyx_t_8 = 0;
      __pyx_t_9 = NULL;
    } else {
      __pyx_t_8 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 945, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_9 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 945, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    for (;;) {
      if (likely(!__pyx_t_9)) {
        if (likely(PyList_CheckExact(__pyx_t_2))) {
          if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_3 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_8); __Pyx_INCREF(__pyx_t_3); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 945, __pyx_L1_error)
          #else
          __pyx_t_3 = PySequence_ITEM(__pyx_t_2, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 945, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          #endif
        } else {
          if (__pyx_t_8 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_8); __Pyx_INCREF(__pyx_t_3); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 945, __pyx_L1_error)
          #else
          __pyx_t_3 = PySequence_ITEM(__pyx_t_2, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 945, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          #endif
        }
      } else {
        __pyx_t_3 = __pyx_t_9(__pyx_t_2);
        if (unlikely(!__pyx_t_3)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 945, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_XDECREF_SET(__pyx_v_j, __pyx_t_3);
      __pyx_t_3 = 0;

      /* "pyart/io/_rsl_interface.pyx":946
 *             nrays = sweep.h.nrays
 *             for j in range(nrays):
 *                 ray = sweep.ray[j]             # <<<<<<<<<<<<<<
 *                 assert ray is not NULL
 *                 if j == 0:
 */
      __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_v_j); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 946, __pyx_L1_error)
      __pyx_v_ray = (__pyx_v_sweep->ray[__pyx_t_10]);

      /* "pyart/io/_rsl_interface.pyx":947
 *             for j in range(nrays):
 *                 ray = sweep.ray[j]
 *                 assert ray is not NULL             # <<<<<<<<<<<<<<
 *                 if j == 0:
 *                     pm_data[i] = self._prtmode(ray.h)
 */
      #ifndef CYTHON_WITHOUT_ASSERTIONS
      if (unlikely(!Py_OptimizeFlag)) {
        if (unlikely(!((__pyx_v_ray != NULL) != 0))) {
          PyErr_SetNone(PyExc_AssertionError);
          __PYX_ERR(0, 947, __pyx_L1_error)
        }
      }
      #endif

      /* "pyart/io/_rsl_interface.pyx":948
 *                 ray = sweep.ray[j]
 *                 assert ray is not NULL
 *                 if j == 0:             # <<<<<<<<<<<<<<
 *                     pm_data[i] = self._prtmode(ray.h)
 * 
 */
      __pyx_t_3 = __Pyx_PyInt_EqObjC(__pyx_v_j, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 948, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 948, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_11) {

        /* "pyart/io/_rsl_interface.pyx":949
 *                 assert ray is not NULL
 *                 if j == 0:
 *                     pm_data[i] = self._prtmode(ray.h)             # <<<<<<<<<<<<<<
 * 
 *                 if valid_nyq_vel:
 */
        __pyx_t_3 = ((struct __pyx_vtabstruct_5pyart_2io_14_rsl_interface__RslVolume *)__pyx_v_self->__pyx_vtab)->_prtmode(__pyx_v_self, __pyx_v_ray->h); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 949, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        if (unlikely(PyObject_SetItem(__pyx_v_pm_data, __pyx_v_i, __pyx_t_3) < 0)) __PYX_ERR(0, 949, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

        /* "pyart/io/_rsl_interface.pyx":948
 *                 ray = sweep.ray[j]
 *                 assert ray is not NULL
 *                 if j == 0:             # <<<<<<<<<<<<<<
 *                     pm_data[i] = self._prtmode(ray.h)
 * 
 */
      }

      /* "pyart/io/_rsl_interface.pyx":951
 *                     pm_data[i] = self._prtmode(ray.h)
 * 
 *                 if valid_nyq_vel:             # <<<<<<<<<<<<<<
 *                     nv_data[ray_count + j] = ray.h.nyq_vel
 *                 else:
 */
      __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_v_valid_nyq_vel); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 951, __pyx_L1_error)
      if (__pyx_t_11) {

        /* "pyart/io/_rsl_interface.pyx":952
 * 
 *                 if valid_nyq_vel:
 *                     nv_data[ray_count + j] = ray.h.nyq_vel             # <<<<<<<<<<<<<<
 *                 else:
 *                     nv_data[ray_count + j] = (ray.h.wavelength *
 */
        __pyx_t_3 = PyFloat_FromDouble(__pyx_v_ray->h.nyq_vel); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 952, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_ray_count); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 952, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_12 = PyNumber_Add(__pyx_t_5, __pyx_v_j); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 952, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(PyObject_SetItem(__pyx_v_nv_data, __pyx_t_12, __pyx_t_3) < 0)) __PYX_ERR(0, 952, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

        /* "pyart/io/_rsl_interface.pyx":951
 *                     pm_data[i] = self._prtmode(ray.h)
 * 
 *                 if valid_nyq_vel:             # <<<<<<<<<<<<<<
 *                     nv_data[ray_count + j] = ray.h.nyq_vel
 *                 else:
 */
        goto __pyx_L8;
      }

      /* "pyart/io/_rsl_interface.pyx":955
 *                 else:
 *                     nv_data[ray_count + j] = (ray.h.wavelength *
 *                                               ray.h.prf / 4.0)             # <<<<<<<<<<<<<<
 * 
 *                 if ray.h.prf == 0:
 */
      /*else*/ {

        /* "pyart/io/_rsl_interface.pyx":954
 *                     nv_data[ray_count + j] = ray.h.nyq_vel
 *                 else:
 *                     nv_data[ray_count + j] = (ray.h.wavelength *             # <<<<<<<<<<<<<<
 *                                               ray.h.prf / 4.0)
 * 
 */
        __pyx_t_3 = PyFloat_FromDouble((((double)(__pyx_v_ray->h.wavelength * __pyx_v_ray->h.prf)) / 4.0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 955, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_12 = __Pyx_PyInt_From_int(__pyx_v_ray_count); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 954, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        __pyx_t_5 = PyNumber_Add(__pyx_t_12, __pyx_v_j); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 954, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(PyObject_SetItem(__pyx_v_nv_data, __pyx_t_5, __pyx_t_3) < 0)) __PYX_ERR(0, 954, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      }
      __pyx_L8:;

      /* "pyart/io/_rsl_interface.pyx":957
 *                                               ray.h.prf / 4.0)
 * 
 *                 if ray.h.prf == 0:             # <<<<<<<<<<<<<<
 *                     pr_data[ray_count + j] = -999.
 *                 else:
 */
      __pyx_t_11 = ((__pyx_v_ray->h.prf == 0) != 0);
      if (__pyx_t_11) {

        /* "pyart/io/_rsl_interface.pyx":958
 * 
 *                 if ray.h.prf == 0:
 *                     pr_data[ray_count + j] = -999.             # <<<<<<<<<<<<<<
 *                 else:
 *                     pr_data[ray_count + j] = 1. / ray.h.prf
 */
        __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_ray_count); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 958, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_5 = PyNumber_Add(__pyx_t_3, __pyx_v_j); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 958, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(PyObject_SetItem(__pyx_v_pr_data, __pyx_t_5, __pyx_float_neg_999_) < 0)) __PYX_ERR(0, 958, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

        /* "pyart/io/_rsl_interface.pyx":957
 *                                               ray.h.prf / 4.0)
 * 
 *                 if ray.h.prf == 0:             # <<<<<<<<<<<<<<
 *                     pr_data[ray_count + j] = -999.
 *                 else:
 */
        goto __pyx_L9;
      }

      /* "pyart/io/_rsl_interface.pyx":960
 *                     pr_data[ray_count + j] = -999.
 *                 else:
 *                     pr_data[ray_count + j] = 1. / ray.h.prf             # <<<<<<<<<<<<<<
 *                 ur_data[ray_count + j] = ray.h.unam_rng * 1000.0
 *             ray_count += nrays
 */
      /*else*/ {
        if (unlikely(__pyx_v_ray->h.prf == 0)) {
          PyErr_SetString(PyExc_ZeroDivisionError, "float division");
          __PYX_ERR(0, 960, __pyx_L1_error)
        }
        __pyx_t_5 = PyFloat_FromDouble((1. / ((double)__pyx_v_ray->h.prf))); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 960, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_ray_count); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 960, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_12 = PyNumber_Add(__pyx_t_3, __pyx_v_j); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 960, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(PyObject_SetItem(__pyx_v_pr_data, __pyx_t_12, __pyx_t_5) < 0)) __PYX_ERR(0, 960, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
      __pyx_L9:;

      /* "pyart/io/_rsl_interface.pyx":961
 *                 else:
 *                     pr_data[ray_count + j] = 1. / ray.h.prf
 *                 ur_data[ray_count + j] = ray.h.unam_rng * 1000.0             # <<<<<<<<<<<<<<
 *             ray_count += nrays
 *         return pm_data, nv_data, pr_data, ur_data
 */
      __pyx_t_5 = PyFloat_FromDouble((__pyx_v_ray->h.unam_rng * 1000.0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 961, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_12 = __Pyx_PyInt_From_int(__pyx_v_ray_count); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 961, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_3 = PyNumber_Add(__pyx_t_12, __pyx_v_j); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 961, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      if (unlikely(PyObject_SetItem(__pyx_v_ur_data, __pyx_t_3, __pyx_t_5) < 0)) __PYX_ERR(0, 961, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "pyart/io/_rsl_interface.pyx":945
 *             assert sweep is not NULL
 *             nrays = sweep.h.nrays
 *             for j in range(nrays):             # <<<<<<<<<<<<<<
 *                 ray = sweep.ray[j]
 *                 assert ray is not NULL
 */
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyart/io/_rsl_interface.pyx":962
 *                     pr_data[ray_count + j] = 1. / ray.h.prf
 *                 ur_data[ray_count + j] = ray.h.unam_rng * 1000.0
 *             ray_count += nrays             # <<<<<<<<<<<<<<
 *         return pm_data, nv_data, pr_data, ur_data
 * 
 */
    __pyx_v_ray_count = (__pyx_v_ray_count + __pyx_v_nrays);

    /* "pyart/io/_rsl_interface.pyx":941
 *         # loop over sweeps and rays storing instrument parameters
 *         ray_count = 0
 *         for i in range(self.nsweeps):             # <<<<<<<<<<<<<<
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL
 */
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyart/io/_rsl_interface.pyx":963
 *                 ur_data[ray_count + j] = ray.h.unam_rng * 1000.0
 *             ray_count += nrays
 *         return pm_data, nv_data, pr_data, ur_data             # <<<<<<<<<<<<<<
 * 
 *     def get_data(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = PyTuple_New(4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 963, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_pm_data);
  __Pyx_GIVEREF(__pyx_v_pm_data);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_pm_data);
  __Pyx_INCREF(__pyx_v_nv_data);
  __Pyx_GIVEREF(__pyx_v_nv_data);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_nv_data);
  __Pyx_INCREF(__pyx_v_pr_data);
  __Pyx_GIVEREF(__pyx_v_pr_data);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_pr_data);
  __Pyx_INCREF(__pyx_v_ur_data);
  __Pyx_GIVEREF(__pyx_v_ur_data);
  PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_v_ur_data);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":905
 *         return elev
 * 
 *     def get_instr_params(self):             # <<<<<<<<<<<<<<
 *         """
 *         get_instr_params()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslVolume.get_instr_params", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_total_rays);
  __Pyx_XDECREF(__pyx_v_nyq_vel);
  __Pyx_XDECREF(__pyx_v_valid_nyq_vel);
  __Pyx_XDECREF(__pyx_v_pm_data);
  __Pyx_XDECREF(__pyx_v_nv_data);
  __Pyx_XDECREF(__pyx_v_pr_data);
  __Pyx_XDECREF(__pyx_v_ur_data);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XDECREF(__pyx_v_j);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":965
 *         return pm_data, nv_data, pr_data, ur_data
 * 
 *     def get_data(self):             # <<<<<<<<<<<<<<
 *         """
 *         get_data()
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_19get_data(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5pyart_2io_14_rsl_interface_10_RslVolume_18get_data[] = "\n        get_data()\n\n        Return the two-dimensional data contained in the volume.\n\n        If a given ray has few bins than the first ray, the missing bins\n        will be filled with 131072.0\n        ";
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_19get_data(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_data (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_18get_data(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_18get_data(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self) {
  Range __pyx_v_raw;
  Sweep *__pyx_v_sweep;
  Ray *__pyx_v_ray;
  int __pyx_v_ray_count;
  int __pyx_v_nsweeps;
  int __pyx_v_nrays;
  int __pyx_v_nbins;
  int __pyx_v_nray;
  int __pyx_v_nsweep;
  int __pyx_v_nbin;
  PyArrayObject *__pyx_v_data = 0;
  Volume *__pyx_v_vol;
  PyObject *__pyx_v_total_rays = NULL;
  PyObject *__pyx_v_shape = NULL;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_data;
  __Pyx_Buffer __pyx_pybuffer_data;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Volume *__pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyArrayObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  int __pyx_t_12;
  int __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  int __pyx_t_16;
  int __pyx_t_17;
  int __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  int __pyx_t_21;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_data", 0);
  __pyx_pybuffer_data.pybuffer.buf = NULL;
  __pyx_pybuffer_data.refcount = 0;
  __pyx_pybuffernd_data.data = NULL;
  __pyx_pybuffernd_data.rcbuffer = &__pyx_pybuffer_data;

  /* "pyart/io/_rsl_interface.pyx":980
 *         cdef np.ndarray[np.float32_t, ndim = 2] data
 * 
 *         vol = self._Volume             # <<<<<<<<<<<<<<
 *         nbins = vol.sweep[0].ray[0].h.nbins
 *         total_rays = self.total_rays()
 */
  __pyx_t_1 = __pyx_v_self->_Volume;
  __pyx_v_vol = __pyx_t_1;

  /* "pyart/io/_rsl_interface.pyx":981
 * 
 *         vol = self._Volume
 *         nbins = vol.sweep[0].ray[0].h.nbins             # <<<<<<<<<<<<<<
 *         total_rays = self.total_rays()
 *         shape = (total_rays, nbins)
 */
  __pyx_t_2 = ((__pyx_v_vol->sweep[0])->ray[0])->h.nbins;
  __pyx_v_nbins = __pyx_t_2;

  /* "pyart/io/_rsl_interface.pyx":982
 *         vol = self._Volume
 *         nbins = vol.sweep[0].ray[0].h.nbins
 *         total_rays = self.total_rays()             # <<<<<<<<<<<<<<
 *         shape = (total_rays, nbins)
 *         data = np.zeros(shape, dtype='float32') + 1.31072000e+05
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_total_rays); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 982, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 982, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_total_rays = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pyart/io/_rsl_interface.pyx":983
 *         nbins = vol.sweep[0].ray[0].h.nbins
 *         total_rays = self.total_rays()
 *         shape = (total_rays, nbins)             # <<<<<<<<<<<<<<
 *         data = np.zeros(shape, dtype='float32') + 1.31072000e+05
 * 
 */
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_nbins); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 983, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 983, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_total_rays);
  __Pyx_GIVEREF(__pyx_v_total_rays);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_total_rays);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_v_shape = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "pyart/io/_rsl_interface.pyx":984
 *         total_rays = self.total_rays()
 *         shape = (total_rays, nbins)
 *         data = np.zeros(shape, dtype='float32') + 1.31072000e+05             # <<<<<<<<<<<<<<
 * 
 *         ray_count = 0
 */
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 984, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 984, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 984, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_shape);
  __Pyx_GIVEREF(__pyx_v_shape);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_shape);
  __pyx_t_5 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 984, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_dtype, __pyx_n_u_float32) < 0) __PYX_ERR(0, 984, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 984, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyFloat_AddObjC(__pyx_t_6, __pyx_float_1_31072000e_05, 1.31072000e+05, 0, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 984, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (!(likely(((__pyx_t_5) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_5, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 984, __pyx_L1_error)
  __pyx_t_7 = ((PyArrayObject *)__pyx_t_5);
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_data.rcbuffer->pybuffer);
    __pyx_t_2 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_data.rcbuffer->pybuffer, (PyObject*)__pyx_t_7, &__Pyx_TypeInfo_nn___pyx_t_5numpy_float32_t, PyBUF_FORMAT| PyBUF_STRIDES| PyBUF_WRITABLE, 2, 0, __pyx_stack);
    if (unlikely(__pyx_t_2 < 0)) {
      PyErr_Fetch(&__pyx_t_8, &__pyx_t_9, &__pyx_t_10);
      if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_data.rcbuffer->pybuffer, (PyObject*)__pyx_v_data, &__Pyx_TypeInfo_nn___pyx_t_5numpy_float32_t, PyBUF_FORMAT| PyBUF_STRIDES| PyBUF_WRITABLE, 2, 0, __pyx_stack) == -1)) {
        Py_XDECREF(__pyx_t_8); Py_XDECREF(__pyx_t_9); Py_XDECREF(__pyx_t_10);
        __Pyx_RaiseBufferFallbackError();
      } else {
        PyErr_Restore(__pyx_t_8, __pyx_t_9, __pyx_t_10);
      }
      __pyx_t_8 = __pyx_t_9 = __pyx_t_10 = 0;
    }
    __pyx_pybuffernd_data.diminfo[0].strides = __pyx_pybuffernd_data.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_data.diminfo[0].shape = __pyx_pybuffernd_data.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_data.diminfo[1].strides = __pyx_pybuffernd_data.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_data.diminfo[1].shape = __pyx_pybuffernd_data.rcbuffer->pybuffer.shape[1];
    if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 984, __pyx_L1_error)
  }
  __pyx_t_7 = 0;
  __pyx_v_data = ((PyArrayObject *)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "pyart/io/_rsl_interface.pyx":986
 *         data = np.zeros(shape, dtype='float32') + 1.31072000e+05
 * 
 *         ray_count = 0             # <<<<<<<<<<<<<<
 *         nsweeps = vol.h.nsweeps
 *         for nsweep in range(nsweeps):
 */
  __pyx_v_ray_count = 0;

  /* "pyart/io/_rsl_interface.pyx":987
 * 
 *         ray_count = 0
 *         nsweeps = vol.h.nsweeps             # <<<<<<<<<<<<<<
 *         for nsweep in range(nsweeps):
 *             sweep = vol.sweep[nsweep]
 */
  __pyx_t_2 = __pyx_v_vol->h.nsweeps;
  __pyx_v_nsweeps = __pyx_t_2;

  /* "pyart/io/_rsl_interface.pyx":988
 *         ray_count = 0
 *         nsweeps = vol.h.nsweeps
 *         for nsweep in range(nsweeps):             # <<<<<<<<<<<<<<
 *             sweep = vol.sweep[nsweep]
 *             assert sweep is not NULL
 */
  __pyx_t_2 = __pyx_v_nsweeps;
  __pyx_t_11 = __pyx_t_2;
  for (__pyx_t_12 = 0; __pyx_t_12 < __pyx_t_11; __pyx_t_12+=1) {
    __pyx_v_nsweep = __pyx_t_12;

    /* "pyart/io/_rsl_interface.pyx":989
 *         nsweeps = vol.h.nsweeps
 *         for nsweep in range(nsweeps):
 *             sweep = vol.sweep[nsweep]             # <<<<<<<<<<<<<<
 *             assert sweep is not NULL
 *             nrays = sweep.h.nrays
 */
    __pyx_v_sweep = (__pyx_v_vol->sweep[__pyx_v_nsweep]);

    /* "pyart/io/_rsl_interface.pyx":990
 *         for nsweep in range(nsweeps):
 *             sweep = vol.sweep[nsweep]
 *             assert sweep is not NULL             # <<<<<<<<<<<<<<
 *             nrays = sweep.h.nrays
 *             for nray in range(nrays):
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      if (unlikely(!((__pyx_v_sweep != NULL) != 0))) {
        PyErr_SetNone(PyExc_AssertionError);
        __PYX_ERR(0, 990, __pyx_L1_error)
      }
    }
    #endif

    /* "pyart/io/_rsl_interface.pyx":991
 *             sweep = vol.sweep[nsweep]
 *             assert sweep is not NULL
 *             nrays = sweep.h.nrays             # <<<<<<<<<<<<<<
 *             for nray in range(nrays):
 *                 ray = sweep.ray[nray]
 */
    __pyx_t_13 = __pyx_v_sweep->h.nrays;
    __pyx_v_nrays = __pyx_t_13;

    /* "pyart/io/_rsl_interface.pyx":992
 *             assert sweep is not NULL
 *             nrays = sweep.h.nrays
 *             for nray in range(nrays):             # <<<<<<<<<<<<<<
 *                 ray = sweep.ray[nray]
 *                 assert ray is not NULL
 */
    __pyx_t_13 = __pyx_v_nrays;
    __pyx_t_14 = __pyx_t_13;
    for (__pyx_t_15 = 0; __pyx_t_15 < __pyx_t_14; __pyx_t_15+=1) {
      __pyx_v_nray = __pyx_t_15;

      /* "pyart/io/_rsl_interface.pyx":993
 *             nrays = sweep.h.nrays
 *             for nray in range(nrays):
 *                 ray = sweep.ray[nray]             # <<<<<<<<<<<<<<
 *                 assert ray is not NULL
 *                 nbins = ray.h.nbins
 */
      __pyx_v_ray = (__pyx_v_sweep->ray[__pyx_v_nray]);

      /* "pyart/io/_rsl_interface.pyx":994
 *             for nray in range(nrays):
 *                 ray = sweep.ray[nray]
 *                 assert ray is not NULL             # <<<<<<<<<<<<<<
 *                 nbins = ray.h.nbins
 *                 for nbin in range(nbins):
 */
      #ifndef CYTHON_WITHOUT_ASSERTIONS
      if (unlikely(!Py_OptimizeFlag)) {
        if (unlikely(!((__pyx_v_ray != NULL) != 0))) {
          PyErr_SetNone(PyExc_AssertionError);
          __PYX_ERR(0, 994, __pyx_L1_error)
        }
      }
      #endif

      /* "pyart/io/_rsl_interface.pyx":995
 *                 ray = sweep.ray[nray]
 *                 assert ray is not NULL
 *                 nbins = ray.h.nbins             # <<<<<<<<<<<<<<
 *                 for nbin in range(nbins):
 *                     raw = ray.range[nbin]
 */
      __pyx_t_16 = __pyx_v_ray->h.nbins;
      __pyx_v_nbins = __pyx_t_16;

      /* "pyart/io/_rsl_interface.pyx":996
 *                 assert ray is not NULL
 *                 nbins = ray.h.nbins
 *                 for nbin in range(nbins):             # <<<<<<<<<<<<<<
 *                     raw = ray.range[nbin]
 *                     data[ray_count + nray, nbin] = ray.h.f(raw)
 */
      __pyx_t_16 = __pyx_v_nbins;
      __pyx_t_17 = __pyx_t_16;
      for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
        __pyx_v_nbin = __pyx_t_18;

        /* "pyart/io/_rsl_interface.pyx":997
 *                 nbins = ray.h.nbins
 *                 for nbin in range(nbins):
 *                     raw = ray.range[nbin]             # <<<<<<<<<<<<<<
 *                     data[ray_count + nray, nbin] = ray.h.f(raw)
 *             ray_count += nrays
 */
        __pyx_v_raw = (__pyx_v_ray->range[__pyx_v_nbin]);

        /* "pyart/io/_rsl_interface.pyx":998
 *                 for nbin in range(nbins):
 *                     raw = ray.range[nbin]
 *                     data[ray_count + nray, nbin] = ray.h.f(raw)             # <<<<<<<<<<<<<<
 *             ray_count += nrays
 *         return data
 */
        __pyx_t_19 = (__pyx_v_ray_count + __pyx_v_nray);
        __pyx_t_20 = __pyx_v_nbin;
        __pyx_t_21 = -1;
        if (__pyx_t_19 < 0) {
          __pyx_t_19 += __pyx_pybuffernd_data.diminfo[0].shape;
          if (unlikely(__pyx_t_19 < 0)) __pyx_t_21 = 0;
        } else if (unlikely(__pyx_t_19 >= __pyx_pybuffernd_data.diminfo[0].shape)) __pyx_t_21 = 0;
        if (__pyx_t_20 < 0) {
          __pyx_t_20 += __pyx_pybuffernd_data.diminfo[1].shape;
          if (unlikely(__pyx_t_20 < 0)) __pyx_t_21 = 1;
        } else if (unlikely(__pyx_t_20 >= __pyx_pybuffernd_data.diminfo[1].shape)) __pyx_t_21 = 1;
        if (unlikely(__pyx_t_21 != -1)) {
          __Pyx_RaiseBufferIndexError(__pyx_t_21);
          __PYX_ERR(0, 998, __pyx_L1_error)
        }
        *__Pyx_BufPtrStrided2d(__pyx_t_5numpy_float32_t *, __pyx_pybuffernd_data.rcbuffer->pybuffer.buf, __pyx_t_19, __pyx_pybuffernd_data.diminfo[0].strides, __pyx_t_20, __pyx_pybuffernd_data.diminfo[1].strides) = __pyx_v_ray->h.f(__pyx_v_raw);
      }
    }

    /* "pyart/io/_rsl_interface.pyx":999
 *                     raw = ray.range[nbin]
 *                     data[ray_count + nray, nbin] = ray.h.f(raw)
 *             ray_count += nrays             # <<<<<<<<<<<<<<
 *         return data
 * 
 */
    __pyx_v_ray_count = (__pyx_v_ray_count + __pyx_v_nrays);
  }

  /* "pyart/io/_rsl_interface.pyx":1000
 *                     data[ray_count + nray, nbin] = ray.h.f(raw)
 *             ray_count += nrays
 *         return data             # <<<<<<<<<<<<<<
 * 
 *     def is_range_bins_uniform(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_data));
  __pyx_r = ((PyObject *)__pyx_v_data);
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":965
 *         return pm_data, nv_data, pr_data, ur_data
 * 
 *     def get_data(self):             # <<<<<<<<<<<<<<
 *         """
 *         get_data()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_data.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslVolume.get_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_data.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_data);
  __Pyx_XDECREF(__pyx_v_total_rays);
  __Pyx_XDECREF(__pyx_v_shape);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1002
 *         return data
 * 
 *     def is_range_bins_uniform(self):             # <<<<<<<<<<<<<<
 *         """
 *         is_range_bins_uniform()
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_21is_range_bins_uniform(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5pyart_2io_14_rsl_interface_10_RslVolume_20is_range_bins_uniform[] = "\n        is_range_bins_uniform()\n\n        Return True is the locations of the range bin are identical for all\n        rays, False if locations change in one or more rays.\n        ";
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_21is_range_bins_uniform(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_range_bins_uniform (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_20is_range_bins_uniform(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_20is_range_bins_uniform(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self) {
  int __pyx_v_nrays;
  Sweep *__pyx_v_sweep;
  Ray *__pyx_v_ray;
  int __pyx_v_ref_gate_size;
  int __pyx_v_ref_range_bin1;
  PyObject *__pyx_v_i = NULL;
  int __pyx_v_j;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *(*__pyx_t_5)(PyObject *);
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_range_bins_uniform", 0);

  /* "pyart/io/_rsl_interface.pyx":1009
 *         rays, False if locations change in one or more rays.
 *         """
 *         cdef int nrays = self._Volume.sweep[0].h.nrays             # <<<<<<<<<<<<<<
 *         cdef _rsl_h.Sweep * sweep
 *         cdef _rsl_h.Ray * ray
 */
  __pyx_t_1 = (__pyx_v_self->_Volume->sweep[0])->h.nrays;
  __pyx_v_nrays = __pyx_t_1;

  /* "pyart/io/_rsl_interface.pyx":1015
 *         # loop over the sweeps and rays checking that the gate_size and
 *         # range_bin1 are the same as the that in the first ray
 *         sweep = self._Volume.sweep[0]             # <<<<<<<<<<<<<<
 *         assert sweep is not NULL
 *         ray = sweep.ray[0]
 */
  __pyx_v_sweep = (__pyx_v_self->_Volume->sweep[0]);

  /* "pyart/io/_rsl_interface.pyx":1016
 *         # range_bin1 are the same as the that in the first ray
 *         sweep = self._Volume.sweep[0]
 *         assert sweep is not NULL             # <<<<<<<<<<<<<<
 *         ray = sweep.ray[0]
 *         assert ray is not NULL
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_sweep != NULL) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 1016, __pyx_L1_error)
    }
  }
  #endif

  /* "pyart/io/_rsl_interface.pyx":1017
 *         sweep = self._Volume.sweep[0]
 *         assert sweep is not NULL
 *         ray = sweep.ray[0]             # <<<<<<<<<<<<<<
 *         assert ray is not NULL
 *         ref_gate_size = ray.h.gate_size
 */
  __pyx_v_ray = (__pyx_v_sweep->ray[0]);

  /* "pyart/io/_rsl_interface.pyx":1018
 *         assert sweep is not NULL
 *         ray = sweep.ray[0]
 *         assert ray is not NULL             # <<<<<<<<<<<<<<
 *         ref_gate_size = ray.h.gate_size
 *         ref_range_bin1 = ray.h.range_bin1
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_ray != NULL) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 1018, __pyx_L1_error)
    }
  }
  #endif

  /* "pyart/io/_rsl_interface.pyx":1019
 *         ray = sweep.ray[0]
 *         assert ray is not NULL
 *         ref_gate_size = ray.h.gate_size             # <<<<<<<<<<<<<<
 *         ref_range_bin1 = ray.h.range_bin1
 * 
 */
  __pyx_t_1 = __pyx_v_ray->h.gate_size;
  __pyx_v_ref_gate_size = __pyx_t_1;

  /* "pyart/io/_rsl_interface.pyx":1020
 *         assert ray is not NULL
 *         ref_gate_size = ray.h.gate_size
 *         ref_range_bin1 = ray.h.range_bin1             # <<<<<<<<<<<<<<
 * 
 *         for i in range(self.nsweeps):
 */
  __pyx_t_1 = __pyx_v_ray->h.range_bin1;
  __pyx_v_ref_range_bin1 = __pyx_t_1;

  /* "pyart/io/_rsl_interface.pyx":1022
 *         ref_range_bin1 = ray.h.range_bin1
 * 
 *         for i in range(self.nsweeps):             # <<<<<<<<<<<<<<
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nsweeps); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1022, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1022, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
    __pyx_t_2 = __pyx_t_3; __Pyx_INCREF(__pyx_t_2); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1022, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1022, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_3 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_4); __Pyx_INCREF(__pyx_t_3); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 1022, __pyx_L1_error)
        #else
        __pyx_t_3 = PySequence_ITEM(__pyx_t_2, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1022, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        #endif
      } else {
        if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_4); __Pyx_INCREF(__pyx_t_3); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 1022, __pyx_L1_error)
        #else
        __pyx_t_3 = PySequence_ITEM(__pyx_t_2, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1022, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        #endif
      }
    } else {
      __pyx_t_3 = __pyx_t_5(__pyx_t_2);
      if (unlikely(!__pyx_t_3)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 1022, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "pyart/io/_rsl_interface.pyx":1023
 * 
 *         for i in range(self.nsweeps):
 *             sweep = self._Volume.sweep[i]             # <<<<<<<<<<<<<<
 *             assert sweep is not NULL
 *             nrays = sweep.h.nrays
 */
    __pyx_t_6 = __Pyx_PyIndex_AsSsize_t(__pyx_v_i); if (unlikely((__pyx_t_6 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1023, __pyx_L1_error)
    __pyx_v_sweep = (__pyx_v_self->_Volume->sweep[__pyx_t_6]);

    /* "pyart/io/_rsl_interface.pyx":1024
 *         for i in range(self.nsweeps):
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL             # <<<<<<<<<<<<<<
 *             nrays = sweep.h.nrays
 *             for j in range(nrays):
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      if (unlikely(!((__pyx_v_sweep != NULL) != 0))) {
        PyErr_SetNone(PyExc_AssertionError);
        __PYX_ERR(0, 1024, __pyx_L1_error)
      }
    }
    #endif

    /* "pyart/io/_rsl_interface.pyx":1025
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL
 *             nrays = sweep.h.nrays             # <<<<<<<<<<<<<<
 *             for j in range(nrays):
 *                 ray = sweep.ray[j]
 */
    __pyx_t_1 = __pyx_v_sweep->h.nrays;
    __pyx_v_nrays = __pyx_t_1;

    /* "pyart/io/_rsl_interface.pyx":1026
 *             assert sweep is not NULL
 *             nrays = sweep.h.nrays
 *             for j in range(nrays):             # <<<<<<<<<<<<<<
 *                 ray = sweep.ray[j]
 *                 assert ray is not NULL
 */
    __pyx_t_1 = __pyx_v_nrays;
    __pyx_t_7 = __pyx_t_1;
    for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
      __pyx_v_j = __pyx_t_8;

      /* "pyart/io/_rsl_interface.pyx":1027
 *             nrays = sweep.h.nrays
 *             for j in range(nrays):
 *                 ray = sweep.ray[j]             # <<<<<<<<<<<<<<
 *                 assert ray is not NULL
 *                 if ray.h.gate_size != ref_gate_size:
 */
      __pyx_v_ray = (__pyx_v_sweep->ray[__pyx_v_j]);

      /* "pyart/io/_rsl_interface.pyx":1028
 *             for j in range(nrays):
 *                 ray = sweep.ray[j]
 *                 assert ray is not NULL             # <<<<<<<<<<<<<<
 *                 if ray.h.gate_size != ref_gate_size:
 *                     return False
 */
      #ifndef CYTHON_WITHOUT_ASSERTIONS
      if (unlikely(!Py_OptimizeFlag)) {
        if (unlikely(!((__pyx_v_ray != NULL) != 0))) {
          PyErr_SetNone(PyExc_AssertionError);
          __PYX_ERR(0, 1028, __pyx_L1_error)
        }
      }
      #endif

      /* "pyart/io/_rsl_interface.pyx":1029
 *                 ray = sweep.ray[j]
 *                 assert ray is not NULL
 *                 if ray.h.gate_size != ref_gate_size:             # <<<<<<<<<<<<<<
 *                     return False
 *                 if ray.h.range_bin1 != ref_range_bin1:
 */
      __pyx_t_9 = ((__pyx_v_ray->h.gate_size != __pyx_v_ref_gate_size) != 0);
      if (__pyx_t_9) {

        /* "pyart/io/_rsl_interface.pyx":1030
 *                 assert ray is not NULL
 *                 if ray.h.gate_size != ref_gate_size:
 *                     return False             # <<<<<<<<<<<<<<
 *                 if ray.h.range_bin1 != ref_range_bin1:
 *                     return False
 */
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(Py_False);
        __pyx_r = Py_False;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        goto __pyx_L0;

        /* "pyart/io/_rsl_interface.pyx":1029
 *                 ray = sweep.ray[j]
 *                 assert ray is not NULL
 *                 if ray.h.gate_size != ref_gate_size:             # <<<<<<<<<<<<<<
 *                     return False
 *                 if ray.h.range_bin1 != ref_range_bin1:
 */
      }

      /* "pyart/io/_rsl_interface.pyx":1031
 *                 if ray.h.gate_size != ref_gate_size:
 *                     return False
 *                 if ray.h.range_bin1 != ref_range_bin1:             # <<<<<<<<<<<<<<
 *                     return False
 *         return True
 */
      __pyx_t_9 = ((__pyx_v_ray->h.range_bin1 != __pyx_v_ref_range_bin1) != 0);
      if (__pyx_t_9) {

        /* "pyart/io/_rsl_interface.pyx":1032
 *                     return False
 *                 if ray.h.range_bin1 != ref_range_bin1:
 *                     return False             # <<<<<<<<<<<<<<
 *         return True
 * 
 */
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(Py_False);
        __pyx_r = Py_False;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        goto __pyx_L0;

        /* "pyart/io/_rsl_interface.pyx":1031
 *                 if ray.h.gate_size != ref_gate_size:
 *                     return False
 *                 if ray.h.range_bin1 != ref_range_bin1:             # <<<<<<<<<<<<<<
 *                     return False
 *         return True
 */
      }
    }

    /* "pyart/io/_rsl_interface.pyx":1022
 *         ref_range_bin1 = ray.h.range_bin1
 * 
 *         for i in range(self.nsweeps):             # <<<<<<<<<<<<<<
 *             sweep = self._Volume.sweep[i]
 *             assert sweep is not NULL
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyart/io/_rsl_interface.pyx":1033
 *                 if ray.h.range_bin1 != ref_range_bin1:
 *                     return False
 *         return True             # <<<<<<<<<<<<<<
 * 
 *     cdef _prtmode(self, _rsl_h.Ray_header h):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(Py_True);
  __pyx_r = Py_True;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1002
 *         return data
 * 
 *     def is_range_bins_uniform(self):             # <<<<<<<<<<<<<<
 *         """
 *         is_range_bins_uniform()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslVolume.is_range_bins_uniform", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1035
 *         return True
 * 
 *     cdef _prtmode(self, _rsl_h.Ray_header h):             # <<<<<<<<<<<<<<
 *         """ Return the prt mode of a given Ray header. """
 *         # TODO need to add additional logic here
 */

static PyObject *__pyx_f_5pyart_2io_14_rsl_interface_10_RslVolume__prtmode(CYTHON_UNUSED struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self, Ray_header __pyx_v_h) {
  PyObject *__pyx_v_mode = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("_prtmode", 0);

  /* "pyart/io/_rsl_interface.pyx":1038
 *         """ Return the prt mode of a given Ray header. """
 *         # TODO need to add additional logic here
 *         if h.prf2 != h.prf:             # <<<<<<<<<<<<<<
 *             mode = 'dual                    '
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_h.prf2 != __pyx_v_h.prf) != 0);
  if (__pyx_t_1) {

    /* "pyart/io/_rsl_interface.pyx":1039
 *         # TODO need to add additional logic here
 *         if h.prf2 != h.prf:
 *             mode = 'dual                    '             # <<<<<<<<<<<<<<
 *         else:
 *             mode = 'fixed                   '
 */
    __Pyx_INCREF(__pyx_kp_u_dual);
    __pyx_v_mode = __pyx_kp_u_dual;

    /* "pyart/io/_rsl_interface.pyx":1038
 *         """ Return the prt mode of a given Ray header. """
 *         # TODO need to add additional logic here
 *         if h.prf2 != h.prf:             # <<<<<<<<<<<<<<
 *             mode = 'dual                    '
 *         else:
 */
    goto __pyx_L3;
  }

  /* "pyart/io/_rsl_interface.pyx":1041
 *             mode = 'dual                    '
 *         else:
 *             mode = 'fixed                   '             # <<<<<<<<<<<<<<
 *         return mode
 * 
 */
  /*else*/ {
    __Pyx_INCREF(__pyx_kp_u_fixed);
    __pyx_v_mode = __pyx_kp_u_fixed;
  }
  __pyx_L3:;

  /* "pyart/io/_rsl_interface.pyx":1042
 *         else:
 *             mode = 'fixed                   '
 *         return mode             # <<<<<<<<<<<<<<
 * 
 *     # header properties mapped to class attributes.
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_mode);
  __pyx_r = __pyx_v_mode;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1035
 *         return True
 * 
 *     cdef _prtmode(self, _rsl_h.Ray_header h):             # <<<<<<<<<<<<<<
 *         """ Return the prt mode of a given Ray header. """
 *         # TODO need to add additional logic here
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_mode);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1046
 *     # header properties mapped to class attributes.
 *     property nsweeps:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Volume.h.nsweeps
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_7nsweeps_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_7nsweeps_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_7nsweeps___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_7nsweeps___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":1047
 *     property nsweeps:
 *         def __get__(self):
 *             return self._Volume.h.nsweeps             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int nsweeps):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Volume->h.nsweeps); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1047, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1046
 *     # header properties mapped to class attributes.
 *     property nsweeps:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Volume.h.nsweeps
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslVolume.nsweeps.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1049
 *             return self._Volume.h.nsweeps
 * 
 *         def __set__(self, int nsweeps):             # <<<<<<<<<<<<<<
 *             self._Volume.h.nsweeps = nsweeps
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_7nsweeps_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_nsweeps); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_7nsweeps_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_nsweeps) {
  int __pyx_v_nsweeps;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_nsweeps); {
    __pyx_v_nsweeps = __Pyx_PyInt_As_int(__pyx_arg_nsweeps); if (unlikely((__pyx_v_nsweeps == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1049, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslVolume.nsweeps.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_7nsweeps_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *)__pyx_v_self), ((int)__pyx_v_nsweeps));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_7nsweeps_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self, int __pyx_v_nsweeps) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":1050
 * 
 *         def __set__(self, int nsweeps):
 *             self._Volume.h.nsweeps = nsweeps             # <<<<<<<<<<<<<<
 * 
 *     property calibr_const:
 */
  __pyx_v_self->_Volume->h.nsweeps = __pyx_v_nsweeps;

  /* "pyart/io/_rsl_interface.pyx":1049
 *             return self._Volume.h.nsweeps
 * 
 *         def __set__(self, int nsweeps):             # <<<<<<<<<<<<<<
 *             self._Volume.h.nsweeps = nsweeps
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1053
 * 
 *     property calibr_const:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Volume.h.calibr_const
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_12calibr_const_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_12calibr_const_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_12calibr_const___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_12calibr_const___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":1054
 *     property calibr_const:
 *         def __get__(self):
 *             return self._Volume.h.calibr_const             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float calibr_const):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Volume->h.calibr_const); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1054, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1053
 * 
 *     property calibr_const:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Volume.h.calibr_const
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslVolume.calibr_const.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1056
 *             return self._Volume.h.calibr_const
 * 
 *         def __set__(self, float calibr_const):             # <<<<<<<<<<<<<<
 *             self._Volume.h.calibr_const = calibr_const
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_12calibr_const_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_calibr_const); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_12calibr_const_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_calibr_const) {
  float __pyx_v_calibr_const;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_calibr_const); {
    __pyx_v_calibr_const = __pyx_PyFloat_AsFloat(__pyx_arg_calibr_const); if (unlikely((__pyx_v_calibr_const == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1056, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslVolume.calibr_const.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_12calibr_const_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *)__pyx_v_self), ((float)__pyx_v_calibr_const));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_12calibr_const_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self, float __pyx_v_calibr_const) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":1057
 * 
 *         def __set__(self, float calibr_const):
 *             self._Volume.h.calibr_const = calibr_const             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_self->_Volume->h.calibr_const = __pyx_v_calibr_const;

  /* "pyart/io/_rsl_interface.pyx":1056
 *             return self._Volume.h.calibr_const
 * 
 *         def __set__(self, float calibr_const):             # <<<<<<<<<<<<<<
 *             self._Volume.h.calibr_const = calibr_const
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._Volume cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_23__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_23__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_22__reduce_cython__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_22__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._Volume cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._Volume cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._Volume cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslVolume.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._Volume cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._Volume cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_25__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_25__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_24__setstate_cython__(((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_10_RslVolume_24__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self._Volume cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._Volume cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__11, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._Volume cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._Volume cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface._RslVolume.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1110
 *     cdef _rsl_h.Ray * _Ray
 * 
 *     def __cinit__(self, filename, radar_format=None, callid=None):             # <<<<<<<<<<<<<<
 *         """ Initalize the _RslFile object. """
 *         if radar_format == 'wsr88d':
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_filename = 0;
  PyObject *__pyx_v_radar_format = 0;
  PyObject *__pyx_v_callid = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_filename,&__pyx_n_s_radar_format,&__pyx_n_s_callid,0};
    PyObject* values[3] = {0,0,0};
    values[1] = ((PyObject *)Py_None);
    values[2] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_filename)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_radar_format);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_callid);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(0, 1110, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_filename = values[0];
    __pyx_v_radar_format = values[1];
    __pyx_v_callid = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 1, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1110, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile___cinit__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self), __pyx_v_filename, __pyx_v_radar_format, __pyx_v_callid);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile___cinit__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, PyObject *__pyx_v_filename, PyObject *__pyx_v_radar_format, PyObject *__pyx_v_callid) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  char *__pyx_t_4;
  char *__pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "pyart/io/_rsl_interface.pyx":1112
 *     def __cinit__(self, filename, radar_format=None, callid=None):
 *         """ Initalize the _RslFile object. """
 *         if radar_format == 'wsr88d':             # <<<<<<<<<<<<<<
 *             if callid is None:
 *                 raise ValueError('callid must be provided.')
 */
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_radar_format, __pyx_n_u_wsr88d, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 1112, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "pyart/io/_rsl_interface.pyx":1113
 *         """ Initalize the _RslFile object. """
 *         if radar_format == 'wsr88d':
 *             if callid is None:             # <<<<<<<<<<<<<<
 *                 raise ValueError('callid must be provided.')
 *             self._Radar = _rsl_h.RSL_wsr88d_to_radar(filename, callid)
 */
    __pyx_t_1 = (__pyx_v_callid == Py_None);
    __pyx_t_2 = (__pyx_t_1 != 0);
    if (unlikely(__pyx_t_2)) {

      /* "pyart/io/_rsl_interface.pyx":1114
 *         if radar_format == 'wsr88d':
 *             if callid is None:
 *                 raise ValueError('callid must be provided.')             # <<<<<<<<<<<<<<
 *             self._Radar = _rsl_h.RSL_wsr88d_to_radar(filename, callid)
 *         elif radar_format is None:
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1114, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(0, 1114, __pyx_L1_error)

      /* "pyart/io/_rsl_interface.pyx":1113
 *         """ Initalize the _RslFile object. """
 *         if radar_format == 'wsr88d':
 *             if callid is None:             # <<<<<<<<<<<<<<
 *                 raise ValueError('callid must be provided.')
 *             self._Radar = _rsl_h.RSL_wsr88d_to_radar(filename, callid)
 */
    }

    /* "pyart/io/_rsl_interface.pyx":1115
 *             if callid is None:
 *                 raise ValueError('callid must be provided.')
 *             self._Radar = _rsl_h.RSL_wsr88d_to_radar(filename, callid)             # <<<<<<<<<<<<<<
 *         elif radar_format is None:
 *             self._Radar = _rsl_h.RSL_anyformat_to_radar(filename)
 */
    __pyx_t_4 = __Pyx_PyObject_AsWritableString(__pyx_v_filename); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 1115, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_AsWritableString(__pyx_v_callid); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(0, 1115, __pyx_L1_error)
    __pyx_v_self->_Radar = RSL_wsr88d_to_radar(__pyx_t_4, __pyx_t_5);

    /* "pyart/io/_rsl_interface.pyx":1112
 *     def __cinit__(self, filename, radar_format=None, callid=None):
 *         """ Initalize the _RslFile object. """
 *         if radar_format == 'wsr88d':             # <<<<<<<<<<<<<<
 *             if callid is None:
 *                 raise ValueError('callid must be provided.')
 */
    goto __pyx_L3;
  }

  /* "pyart/io/_rsl_interface.pyx":1116
 *                 raise ValueError('callid must be provided.')
 *             self._Radar = _rsl_h.RSL_wsr88d_to_radar(filename, callid)
 *         elif radar_format is None:             # <<<<<<<<<<<<<<
 *             self._Radar = _rsl_h.RSL_anyformat_to_radar(filename)
 *         else:
 */
  __pyx_t_2 = (__pyx_v_radar_format == Py_None);
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (likely(__pyx_t_1)) {

    /* "pyart/io/_rsl_interface.pyx":1117
 *             self._Radar = _rsl_h.RSL_wsr88d_to_radar(filename, callid)
 *         elif radar_format is None:
 *             self._Radar = _rsl_h.RSL_anyformat_to_radar(filename)             # <<<<<<<<<<<<<<
 *         else:
 *             raise ValueError('invalid radar_format:', radar_format)
 */
    __pyx_t_5 = __Pyx_PyObject_AsWritableString(__pyx_v_filename); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(0, 1117, __pyx_L1_error)
    __pyx_v_self->_Radar = RSL_anyformat_to_radar(__pyx_t_5);

    /* "pyart/io/_rsl_interface.pyx":1116
 *                 raise ValueError('callid must be provided.')
 *             self._Radar = _rsl_h.RSL_wsr88d_to_radar(filename, callid)
 *         elif radar_format is None:             # <<<<<<<<<<<<<<
 *             self._Radar = _rsl_h.RSL_anyformat_to_radar(filename)
 *         else:
 */
    goto __pyx_L3;
  }

  /* "pyart/io/_rsl_interface.pyx":1119
 *             self._Radar = _rsl_h.RSL_anyformat_to_radar(filename)
 *         else:
 *             raise ValueError('invalid radar_format:', radar_format)             # <<<<<<<<<<<<<<
 *         if self._Radar is NULL:
 *             raise IOError('file cannot be read.')
 */
  /*else*/ {
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1119, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_kp_u_invalid_radar_format);
    __Pyx_GIVEREF(__pyx_kp_u_invalid_radar_format);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_kp_u_invalid_radar_format);
    __Pyx_INCREF(__pyx_v_radar_format);
    __Pyx_GIVEREF(__pyx_v_radar_format);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_radar_format);
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_3, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1119, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 1119, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "pyart/io/_rsl_interface.pyx":1120
 *         else:
 *             raise ValueError('invalid radar_format:', radar_format)
 *         if self._Radar is NULL:             # <<<<<<<<<<<<<<
 *             raise IOError('file cannot be read.')
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->_Radar == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "pyart/io/_rsl_interface.pyx":1121
 *             raise ValueError('invalid radar_format:', radar_format)
 *         if self._Radar is NULL:
 *             raise IOError('file cannot be read.')             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_IOError, __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1121, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 1121, __pyx_L1_error)

    /* "pyart/io/_rsl_interface.pyx":1120
 *         else:
 *             raise ValueError('invalid radar_format:', radar_format)
 *         if self._Radar is NULL:             # <<<<<<<<<<<<<<
 *             raise IOError('file cannot be read.')
 * 
 */
  }

  /* "pyart/io/_rsl_interface.pyx":1110
 *     cdef _rsl_h.Ray * _Ray
 * 
 *     def __cinit__(self, filename, radar_format=None, callid=None):             # <<<<<<<<<<<<<<
 *         """ Initalize the _RslFile object. """
 *         if radar_format == 'wsr88d':
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1123
 *             raise IOError('file cannot be read.')
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         """ Free memory used by object. """
 *         _rsl_h.RSL_free_radar(self._Radar)
 */

/* Python wrapper */
static void __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_2__dealloc__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_2__dealloc__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "pyart/io/_rsl_interface.pyx":1125
 *     def __dealloc__(self):
 *         """ Free memory used by object. """
 *         _rsl_h.RSL_free_radar(self._Radar)             # <<<<<<<<<<<<<<
 * 
 *     def get_volume(self, int volume_number):
 */
  RSL_free_radar(__pyx_v_self->_Radar);

  /* "pyart/io/_rsl_interface.pyx":1123
 *             raise IOError('file cannot be read.')
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         """ Free memory used by object. """
 *         _rsl_h.RSL_free_radar(self._Radar)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "pyart/io/_rsl_interface.pyx":1127
 *         _rsl_h.RSL_free_radar(self._Radar)
 * 
 *     def get_volume(self, int volume_number):             # <<<<<<<<<<<<<<
 *         """
 *         get_volume(volume_number)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_5get_volume(PyObject *__pyx_v_self, PyObject *__pyx_arg_volume_number); /*proto*/
static char __pyx_doc_5pyart_2io_14_rsl_interface_7RslFile_4get_volume[] = "\n        get_volume(volume_number)\n\n        Return a _RslVolume for a given volume number.\n\n        Parameters\n        ----------\n        volume_number : int\n            Volume number to retrieve\n\n        Returns\n        -------\n        volume : _RslVolume\n            _RslVolume object containing requested volume.\n\n        ";
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_5get_volume(PyObject *__pyx_v_self, PyObject *__pyx_arg_volume_number) {
  int __pyx_v_volume_number;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_volume (wrapper)", 0);
  assert(__pyx_arg_volume_number); {
    __pyx_v_volume_number = __Pyx_PyInt_As_int(__pyx_arg_volume_number); if (unlikely((__pyx_v_volume_number == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1127, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.get_volume", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4get_volume(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self), ((int)__pyx_v_volume_number));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4get_volume(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_volume_number) {
  struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *__pyx_v_rslvolume = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_volume", 0);

  /* "pyart/io/_rsl_interface.pyx":1144
 * 
 *         """
 *         if volume_number < 0 or volume_number >= self._Radar.h.nvolumes:             # <<<<<<<<<<<<<<
 *             raise ValueError('invalid volume_number')
 * 
 */
  __pyx_t_2 = ((__pyx_v_volume_number < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_volume_number >= __pyx_v_self->_Radar->h.nvolumes) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "pyart/io/_rsl_interface.pyx":1145
 *         """
 *         if volume_number < 0 or volume_number >= self._Radar.h.nvolumes:
 *             raise ValueError('invalid volume_number')             # <<<<<<<<<<<<<<
 * 
 *         rslvolume = _RslVolume()
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__14, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1145, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 1145, __pyx_L1_error)

    /* "pyart/io/_rsl_interface.pyx":1144
 * 
 *         """
 *         if volume_number < 0 or volume_number >= self._Radar.h.nvolumes:             # <<<<<<<<<<<<<<
 *             raise ValueError('invalid volume_number')
 * 
 */
  }

  /* "pyart/io/_rsl_interface.pyx":1147
 *             raise ValueError('invalid volume_number')
 * 
 *         rslvolume = _RslVolume()             # <<<<<<<<<<<<<<
 *         rslvolume.load(self._Radar.v[volume_number])
 *         return rslvolume
 */
  __pyx_t_3 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_5pyart_2io_14_rsl_interface__RslVolume)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1147, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_rslvolume = ((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "pyart/io/_rsl_interface.pyx":1148
 * 
 *         rslvolume = _RslVolume()
 *         rslvolume.load(self._Radar.v[volume_number])             # <<<<<<<<<<<<<<
 *         return rslvolume
 * 
 */
  __pyx_t_3 = ((struct __pyx_vtabstruct_5pyart_2io_14_rsl_interface__RslVolume *)__pyx_v_rslvolume->__pyx_vtab)->load(__pyx_v_rslvolume, (__pyx_v_self->_Radar->v[__pyx_v_volume_number])); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1148, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pyart/io/_rsl_interface.pyx":1149
 *         rslvolume = _RslVolume()
 *         rslvolume.load(self._Radar.v[volume_number])
 *         return rslvolume             # <<<<<<<<<<<<<<
 * 
 *     def available_moments(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_rslvolume));
  __pyx_r = ((PyObject *)__pyx_v_rslvolume);
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1127
 *         _rsl_h.RSL_free_radar(self._Radar)
 * 
 *     def get_volume(self, int volume_number):             # <<<<<<<<<<<<<<
 *         """
 *         get_volume(volume_number)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.get_volume", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_rslvolume);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1151
 *         return rslvolume
 * 
 *     def available_moments(self):             # <<<<<<<<<<<<<<
 *         """
 *         available_moments()
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_7available_moments(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5pyart_2io_14_rsl_interface_7RslFile_6available_moments[] = "\n        available_moments()\n\n        Return a list of available volume moments.\n        ";
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_7available_moments(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("available_moments (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6available_moments(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6available_moments(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self) {
  PyObject *__pyx_v_av = NULL;
  int __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("available_moments", 0);

  /* "pyart/io/_rsl_interface.pyx":1157
 *         Return a list of available volume moments.
 *         """
 *         av = []             # <<<<<<<<<<<<<<
 *         for i in range(self._Radar.h.nvolumes):
 *             if self._Radar.v[i] is not NULL:
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1157, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_av = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyart/io/_rsl_interface.pyx":1158
 *         """
 *         av = []
 *         for i in range(self._Radar.h.nvolumes):             # <<<<<<<<<<<<<<
 *             if self._Radar.v[i] is not NULL:
 *                 av.append(i)
 */
  __pyx_t_2 = __pyx_v_self->_Radar->h.nvolumes;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "pyart/io/_rsl_interface.pyx":1159
 *         av = []
 *         for i in range(self._Radar.h.nvolumes):
 *             if self._Radar.v[i] is not NULL:             # <<<<<<<<<<<<<<
 *                 av.append(i)
 *         return av
 */
    __pyx_t_5 = (((__pyx_v_self->_Radar->v[__pyx_v_i]) != NULL) != 0);
    if (__pyx_t_5) {

      /* "pyart/io/_rsl_interface.pyx":1160
 *         for i in range(self._Radar.h.nvolumes):
 *             if self._Radar.v[i] is not NULL:
 *                 av.append(i)             # <<<<<<<<<<<<<<
 *         return av
 * 
 */
      __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1160, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_6 = __Pyx_PyList_Append(__pyx_v_av, __pyx_t_1); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 1160, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "pyart/io/_rsl_interface.pyx":1159
 *         av = []
 *         for i in range(self._Radar.h.nvolumes):
 *             if self._Radar.v[i] is not NULL:             # <<<<<<<<<<<<<<
 *                 av.append(i)
 *         return av
 */
    }
  }

  /* "pyart/io/_rsl_interface.pyx":1161
 *             if self._Radar.v[i] is not NULL:
 *                 av.append(i)
 *         return av             # <<<<<<<<<<<<<<
 * 
 *     def get_radar_header(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_av);
  __pyx_r = __pyx_v_av;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1151
 *         return rslvolume
 * 
 *     def available_moments(self):             # <<<<<<<<<<<<<<
 *         """
 *         available_moments()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.available_moments", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_av);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1163
 *         return av
 * 
 *     def get_radar_header(self):             # <<<<<<<<<<<<<<
 *         """
 *         get_radar_headers()
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_9get_radar_header(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5pyart_2io_14_rsl_interface_7RslFile_8get_radar_header[] = "\n        get_radar_headers()\n\n        Return a dictionary of radar header parameters.\n        ";
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_9get_radar_header(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_radar_header (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_8get_radar_header(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_8get_radar_header(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_radar_header", 0);

  /* "pyart/io/_rsl_interface.pyx":1169
 *         Return a dictionary of radar header parameters.
 *         """
 *         return self._Radar.h             # <<<<<<<<<<<<<<
 * 
 *     def get_volume_array(self, int volume_num):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_convert__to_py_Radar_header(__pyx_v_self->_Radar->h); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1169, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1163
 *         return av
 * 
 *     def get_radar_header(self):             # <<<<<<<<<<<<<<
 *         """
 *         get_radar_headers()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.get_radar_header", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1171
 *         return self._Radar.h
 * 
 *     def get_volume_array(self, int volume_num):             # <<<<<<<<<<<<<<
 *         """
 *         get_volume_array(volume_number)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_11get_volume_array(PyObject *__pyx_v_self, PyObject *__pyx_arg_volume_num); /*proto*/
static char __pyx_doc_5pyart_2io_14_rsl_interface_7RslFile_10get_volume_array[] = "\n        get_volume_array(volume_number)\n\n        Return the three-dimensional data contained in a given volume.\n\n        Parameters\n        ----------\n        volume_number : int\n\n        Returns\n        -------\n        volume : array (nsweep, nrays, nbins), float32\n            Array containing  data for the given volume.\n\n        ";
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_11get_volume_array(PyObject *__pyx_v_self, PyObject *__pyx_arg_volume_num) {
  int __pyx_v_volume_num;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_volume_array (wrapper)", 0);
  assert(__pyx_arg_volume_num); {
    __pyx_v_volume_num = __Pyx_PyInt_As_int(__pyx_arg_volume_num); if (unlikely((__pyx_v_volume_num == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1171, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.get_volume_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_10get_volume_array(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self), ((int)__pyx_v_volume_num));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_10get_volume_array(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_volume_num) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_volume_array", 0);

  /* "pyart/io/_rsl_interface.pyx":1187
 * 
 *         """
 *         return self.get_volume(volume_num).get_data()             # <<<<<<<<<<<<<<
 * 
 *     # header properties mapped to class attributes.
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_volume); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_volume_num); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_get_data); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1171
 *         return self._Radar.h
 * 
 *     def get_volume_array(self, int volume_num):             # <<<<<<<<<<<<<<
 *         """
 *         get_volume_array(volume_number)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.get_volume_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1191
 *     # header properties mapped to class attributes.
 *     property month:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.month
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_5month_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_5month_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_5month___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_5month___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":1192
 *     property month:
 *         def __get__(self):
 *             return self._Radar.h.month             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int month):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Radar->h.month); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1192, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1191
 *     # header properties mapped to class attributes.
 *     property month:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.month
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.month.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1194
 *             return self._Radar.h.month
 * 
 *         def __set__(self, int month):             # <<<<<<<<<<<<<<
 *             self._Radar.h.month = month
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_5month_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_month); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_5month_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_month) {
  int __pyx_v_month;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_month); {
    __pyx_v_month = __Pyx_PyInt_As_int(__pyx_arg_month); if (unlikely((__pyx_v_month == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1194, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.month.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_5month_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self), ((int)__pyx_v_month));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_5month_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_month) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":1195
 * 
 *         def __set__(self, int month):
 *             self._Radar.h.month = month             # <<<<<<<<<<<<<<
 * 
 *     property day:
 */
  __pyx_v_self->_Radar->h.month = __pyx_v_month;

  /* "pyart/io/_rsl_interface.pyx":1194
 *             return self._Radar.h.month
 * 
 *         def __set__(self, int month):             # <<<<<<<<<<<<<<
 *             self._Radar.h.month = month
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1198
 * 
 *     property day:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.day
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_3day_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_3day_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_3day___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_3day___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":1199
 *     property day:
 *         def __get__(self):
 *             return self._Radar.h.day             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int day):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Radar->h.day); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1198
 * 
 *     property day:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.day
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.day.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1201
 *             return self._Radar.h.day
 * 
 *         def __set__(self, int day):             # <<<<<<<<<<<<<<
 *             self._Radar.h.day = day
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_3day_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_day); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_3day_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_day) {
  int __pyx_v_day;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_day); {
    __pyx_v_day = __Pyx_PyInt_As_int(__pyx_arg_day); if (unlikely((__pyx_v_day == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1201, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.day.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_3day_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self), ((int)__pyx_v_day));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_3day_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_day) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":1202
 * 
 *         def __set__(self, int day):
 *             self._Radar.h.day = day             # <<<<<<<<<<<<<<
 * 
 *     property year:
 */
  __pyx_v_self->_Radar->h.day = __pyx_v_day;

  /* "pyart/io/_rsl_interface.pyx":1201
 *             return self._Radar.h.day
 * 
 *         def __set__(self, int day):             # <<<<<<<<<<<<<<
 *             self._Radar.h.day = day
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1205
 * 
 *     property year:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.year
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4year_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4year_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4year___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4year___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":1206
 *     property year:
 *         def __get__(self):
 *             return self._Radar.h.year             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int year):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Radar->h.year); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1205
 * 
 *     property year:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.year
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.year.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1208
 *             return self._Radar.h.year
 * 
 *         def __set__(self, int year):             # <<<<<<<<<<<<<<
 *             self._Radar.h.year = year
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4year_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_year); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4year_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_year) {
  int __pyx_v_year;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_year); {
    __pyx_v_year = __Pyx_PyInt_As_int(__pyx_arg_year); if (unlikely((__pyx_v_year == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1208, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.year.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4year_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self), ((int)__pyx_v_year));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4year_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_year) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":1209
 * 
 *         def __set__(self, int year):
 *             self._Radar.h.year = year             # <<<<<<<<<<<<<<
 * 
 *     property hour:
 */
  __pyx_v_self->_Radar->h.year = __pyx_v_year;

  /* "pyart/io/_rsl_interface.pyx":1208
 *             return self._Radar.h.year
 * 
 *         def __set__(self, int year):             # <<<<<<<<<<<<<<
 *             self._Radar.h.year = year
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1212
 * 
 *     property hour:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.hour
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4hour_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4hour_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4hour___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4hour___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":1213
 *     property hour:
 *         def __get__(self):
 *             return self._Radar.h.hour             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int hour):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Radar->h.hour); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1213, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1212
 * 
 *     property hour:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.hour
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.hour.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1215
 *             return self._Radar.h.hour
 * 
 *         def __set__(self, int hour):             # <<<<<<<<<<<<<<
 *             self._Radar.h.hour = hour
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4hour_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_hour); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4hour_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_hour) {
  int __pyx_v_hour;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_hour); {
    __pyx_v_hour = __Pyx_PyInt_As_int(__pyx_arg_hour); if (unlikely((__pyx_v_hour == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1215, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.hour.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4hour_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self), ((int)__pyx_v_hour));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4hour_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_hour) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":1216
 * 
 *         def __set__(self, int hour):
 *             self._Radar.h.hour = hour             # <<<<<<<<<<<<<<
 * 
 *     property minute:
 */
  __pyx_v_self->_Radar->h.hour = __pyx_v_hour;

  /* "pyart/io/_rsl_interface.pyx":1215
 *             return self._Radar.h.hour
 * 
 *         def __set__(self, int hour):             # <<<<<<<<<<<<<<
 *             self._Radar.h.hour = hour
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1219
 * 
 *     property minute:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.minute
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6minute_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6minute_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6minute___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6minute___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":1220
 *     property minute:
 *         def __get__(self):
 *             return self._Radar.h.minute             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int minute):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Radar->h.minute); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1220, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1219
 * 
 *     property minute:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.minute
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.minute.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1222
 *             return self._Radar.h.minute
 * 
 *         def __set__(self, int minute):             # <<<<<<<<<<<<<<
 *             self._Radar.h.minute = minute
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6minute_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_minute); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6minute_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_minute) {
  int __pyx_v_minute;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_minute); {
    __pyx_v_minute = __Pyx_PyInt_As_int(__pyx_arg_minute); if (unlikely((__pyx_v_minute == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1222, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.minute.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6minute_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self), ((int)__pyx_v_minute));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6minute_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_minute) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":1223
 * 
 *         def __set__(self, int minute):
 *             self._Radar.h.minute = minute             # <<<<<<<<<<<<<<
 * 
 *     property sec:
 */
  __pyx_v_self->_Radar->h.minute = __pyx_v_minute;

  /* "pyart/io/_rsl_interface.pyx":1222
 *             return self._Radar.h.minute
 * 
 *         def __set__(self, int minute):             # <<<<<<<<<<<<<<
 *             self._Radar.h.minute = minute
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1226
 * 
 *     property sec:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.sec
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_3sec_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_3sec_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_3sec___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_3sec___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":1227
 *     property sec:
 *         def __get__(self):
 *             return self._Radar.h.sec             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, float sec):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_Radar->h.sec); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1227, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1226
 * 
 *     property sec:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.sec
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.sec.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1229
 *             return self._Radar.h.sec
 * 
 *         def __set__(self, float sec):             # <<<<<<<<<<<<<<
 *             self._Radar.h.sec = sec
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_3sec_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_sec); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_3sec_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_sec) {
  float __pyx_v_sec;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_sec); {
    __pyx_v_sec = __pyx_PyFloat_AsFloat(__pyx_arg_sec); if (unlikely((__pyx_v_sec == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1229, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.sec.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_3sec_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self), ((float)__pyx_v_sec));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_3sec_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, float __pyx_v_sec) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":1230
 * 
 *         def __set__(self, float sec):
 *             self._Radar.h.sec = sec             # <<<<<<<<<<<<<<
 * 
 *     property nvolumes:
 */
  __pyx_v_self->_Radar->h.sec = __pyx_v_sec;

  /* "pyart/io/_rsl_interface.pyx":1229
 *             return self._Radar.h.sec
 * 
 *         def __set__(self, float sec):             # <<<<<<<<<<<<<<
 *             self._Radar.h.sec = sec
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1233
 * 
 *     property nvolumes:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.nvolumes
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_8nvolumes_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_8nvolumes_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_8nvolumes___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_8nvolumes___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":1234
 *     property nvolumes:
 *         def __get__(self):
 *             return self._Radar.h.nvolumes             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int nvolumes):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Radar->h.nvolumes); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1233
 * 
 *     property nvolumes:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.nvolumes
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.nvolumes.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1236
 *             return self._Radar.h.nvolumes
 * 
 *         def __set__(self, int nvolumes):             # <<<<<<<<<<<<<<
 *             self._Radar.h.nvolumes = nvolumes
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_8nvolumes_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_nvolumes); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_8nvolumes_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_nvolumes) {
  int __pyx_v_nvolumes;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_nvolumes); {
    __pyx_v_nvolumes = __Pyx_PyInt_As_int(__pyx_arg_nvolumes); if (unlikely((__pyx_v_nvolumes == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1236, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.nvolumes.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_8nvolumes_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self), ((int)__pyx_v_nvolumes));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_8nvolumes_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_nvolumes) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":1237
 * 
 *         def __set__(self, int nvolumes):
 *             self._Radar.h.nvolumes = nvolumes             # <<<<<<<<<<<<<<
 * 
 *     property number:
 */
  __pyx_v_self->_Radar->h.nvolumes = __pyx_v_nvolumes;

  /* "pyart/io/_rsl_interface.pyx":1236
 *             return self._Radar.h.nvolumes
 * 
 *         def __set__(self, int nvolumes):             # <<<<<<<<<<<<<<
 *             self._Radar.h.nvolumes = nvolumes
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1240
 * 
 *     property number:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.number
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6number_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6number_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6number___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6number___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":1241
 *     property number:
 *         def __get__(self):
 *             return self._Radar.h.number             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int number):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Radar->h.number); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1240
 * 
 *     property number:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.number
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.number.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1243
 *             return self._Radar.h.number
 * 
 *         def __set__(self, int number):             # <<<<<<<<<<<<<<
 *             self._Radar.h.number = number
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6number_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_number); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6number_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_number) {
  int __pyx_v_number;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_number); {
    __pyx_v_number = __Pyx_PyInt_As_int(__pyx_arg_number); if (unlikely((__pyx_v_number == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1243, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.number.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6number_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self), ((int)__pyx_v_number));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6number_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_number) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":1244
 * 
 *         def __set__(self, int number):
 *             self._Radar.h.number = number             # <<<<<<<<<<<<<<
 * 
 *     property latd:
 */
  __pyx_v_self->_Radar->h.number = __pyx_v_number;

  /* "pyart/io/_rsl_interface.pyx":1243
 *             return self._Radar.h.number
 * 
 *         def __set__(self, int number):             # <<<<<<<<<<<<<<
 *             self._Radar.h.number = number
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1247
 * 
 *     property latd:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.latd
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4latd_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4latd_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4latd___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4latd___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":1248
 *     property latd:
 *         def __get__(self):
 *             return self._Radar.h.latd             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int latd):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Radar->h.latd); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1247
 * 
 *     property latd:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.latd
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.latd.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1250
 *             return self._Radar.h.latd
 * 
 *         def __set__(self, int latd):             # <<<<<<<<<<<<<<
 *             self._Radar.h.latd = latd
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4latd_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_latd); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4latd_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_latd) {
  int __pyx_v_latd;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_latd); {
    __pyx_v_latd = __Pyx_PyInt_As_int(__pyx_arg_latd); if (unlikely((__pyx_v_latd == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1250, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.latd.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4latd_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self), ((int)__pyx_v_latd));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4latd_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_latd) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":1251
 * 
 *         def __set__(self, int latd):
 *             self._Radar.h.latd = latd             # <<<<<<<<<<<<<<
 * 
 *     property latm:
 */
  __pyx_v_self->_Radar->h.latd = __pyx_v_latd;

  /* "pyart/io/_rsl_interface.pyx":1250
 *             return self._Radar.h.latd
 * 
 *         def __set__(self, int latd):             # <<<<<<<<<<<<<<
 *             self._Radar.h.latd = latd
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1254
 * 
 *     property latm:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.latm
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4latm_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4latm_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4latm___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4latm___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":1255
 *     property latm:
 *         def __get__(self):
 *             return self._Radar.h.latm             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int latm):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Radar->h.latm); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1254
 * 
 *     property latm:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.latm
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.latm.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1257
 *             return self._Radar.h.latm
 * 
 *         def __set__(self, int latm):             # <<<<<<<<<<<<<<
 *             self._Radar.h.latm = latm
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4latm_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_latm); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4latm_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_latm) {
  int __pyx_v_latm;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_latm); {
    __pyx_v_latm = __Pyx_PyInt_As_int(__pyx_arg_latm); if (unlikely((__pyx_v_latm == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1257, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.latm.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4latm_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self), ((int)__pyx_v_latm));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4latm_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_latm) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":1258
 * 
 *         def __set__(self, int latm):
 *             self._Radar.h.latm = latm             # <<<<<<<<<<<<<<
 * 
 *     property lats:
 */
  __pyx_v_self->_Radar->h.latm = __pyx_v_latm;

  /* "pyart/io/_rsl_interface.pyx":1257
 *             return self._Radar.h.latm
 * 
 *         def __set__(self, int latm):             # <<<<<<<<<<<<<<
 *             self._Radar.h.latm = latm
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1261
 * 
 *     property lats:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.lats
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4lats_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4lats_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4lats___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4lats___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":1262
 *     property lats:
 *         def __get__(self):
 *             return self._Radar.h.lats             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int lats):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Radar->h.lats); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1261
 * 
 *     property lats:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.lats
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.lats.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1264
 *             return self._Radar.h.lats
 * 
 *         def __set__(self, int lats):             # <<<<<<<<<<<<<<
 *             self._Radar.h.lats = lats
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4lats_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_lats); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4lats_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_lats) {
  int __pyx_v_lats;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_lats); {
    __pyx_v_lats = __Pyx_PyInt_As_int(__pyx_arg_lats); if (unlikely((__pyx_v_lats == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1264, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.lats.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4lats_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self), ((int)__pyx_v_lats));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4lats_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_lats) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":1265
 * 
 *         def __set__(self, int lats):
 *             self._Radar.h.lats = lats             # <<<<<<<<<<<<<<
 * 
 *     property lond:
 */
  __pyx_v_self->_Radar->h.lats = __pyx_v_lats;

  /* "pyart/io/_rsl_interface.pyx":1264
 *             return self._Radar.h.lats
 * 
 *         def __set__(self, int lats):             # <<<<<<<<<<<<<<
 *             self._Radar.h.lats = lats
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1268
 * 
 *     property lond:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.lond
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4lond_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4lond_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4lond___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4lond___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":1269
 *     property lond:
 *         def __get__(self):
 *             return self._Radar.h.lond             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int lond):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Radar->h.lond); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1269, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1268
 * 
 *     property lond:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.lond
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.lond.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1271
 *             return self._Radar.h.lond
 * 
 *         def __set__(self, int lond):             # <<<<<<<<<<<<<<
 *             self._Radar.h.lond = lond
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4lond_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_lond); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4lond_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_lond) {
  int __pyx_v_lond;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_lond); {
    __pyx_v_lond = __Pyx_PyInt_As_int(__pyx_arg_lond); if (unlikely((__pyx_v_lond == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1271, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.lond.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4lond_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self), ((int)__pyx_v_lond));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4lond_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_lond) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":1272
 * 
 *         def __set__(self, int lond):
 *             self._Radar.h.lond = lond             # <<<<<<<<<<<<<<
 * 
 *     property lonm:
 */
  __pyx_v_self->_Radar->h.lond = __pyx_v_lond;

  /* "pyart/io/_rsl_interface.pyx":1271
 *             return self._Radar.h.lond
 * 
 *         def __set__(self, int lond):             # <<<<<<<<<<<<<<
 *             self._Radar.h.lond = lond
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1275
 * 
 *     property lonm:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.lonm
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4lonm_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4lonm_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4lonm___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4lonm___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":1276
 *     property lonm:
 *         def __get__(self):
 *             return self._Radar.h.lonm             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int lonm):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Radar->h.lonm); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1275
 * 
 *     property lonm:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.lonm
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.lonm.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1278
 *             return self._Radar.h.lonm
 * 
 *         def __set__(self, int lonm):             # <<<<<<<<<<<<<<
 *             self._Radar.h.lonm = lonm
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4lonm_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_lonm); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4lonm_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_lonm) {
  int __pyx_v_lonm;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_lonm); {
    __pyx_v_lonm = __Pyx_PyInt_As_int(__pyx_arg_lonm); if (unlikely((__pyx_v_lonm == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1278, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.lonm.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4lonm_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self), ((int)__pyx_v_lonm));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4lonm_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_lonm) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":1279
 * 
 *         def __set__(self, int lonm):
 *             self._Radar.h.lonm = lonm             # <<<<<<<<<<<<<<
 * 
 *     property lons:
 */
  __pyx_v_self->_Radar->h.lonm = __pyx_v_lonm;

  /* "pyart/io/_rsl_interface.pyx":1278
 *             return self._Radar.h.lonm
 * 
 *         def __set__(self, int lonm):             # <<<<<<<<<<<<<<
 *             self._Radar.h.lonm = lonm
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1282
 * 
 *     property lons:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.lons
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4lons_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4lons_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4lons___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4lons___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":1283
 *     property lons:
 *         def __get__(self):
 *             return self._Radar.h.lons             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int lons):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Radar->h.lons); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1283, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1282
 * 
 *     property lons:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.lons
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.lons.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1285
 *             return self._Radar.h.lons
 * 
 *         def __set__(self, int lons):             # <<<<<<<<<<<<<<
 *             self._Radar.h.lons = lons
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4lons_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_lons); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4lons_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_lons) {
  int __pyx_v_lons;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_lons); {
    __pyx_v_lons = __Pyx_PyInt_As_int(__pyx_arg_lons); if (unlikely((__pyx_v_lons == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1285, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.lons.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4lons_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self), ((int)__pyx_v_lons));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_4lons_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_lons) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":1286
 * 
 *         def __set__(self, int lons):
 *             self._Radar.h.lons = lons             # <<<<<<<<<<<<<<
 * 
 *     property height:
 */
  __pyx_v_self->_Radar->h.lons = __pyx_v_lons;

  /* "pyart/io/_rsl_interface.pyx":1285
 *             return self._Radar.h.lons
 * 
 *         def __set__(self, int lons):             # <<<<<<<<<<<<<<
 *             self._Radar.h.lons = lons
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1289
 * 
 *     property height:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.height
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6height_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6height_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6height___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6height___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":1290
 *     property height:
 *         def __get__(self):
 *             return self._Radar.h.height             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int height):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Radar->h.height); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1290, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1289
 * 
 *     property height:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.height
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.height.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1292
 *             return self._Radar.h.height
 * 
 *         def __set__(self, int height):             # <<<<<<<<<<<<<<
 *             self._Radar.h.height = height
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6height_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_height); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6height_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_height) {
  int __pyx_v_height;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_height); {
    __pyx_v_height = __Pyx_PyInt_As_int(__pyx_arg_height); if (unlikely((__pyx_v_height == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1292, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.height.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6height_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self), ((int)__pyx_v_height));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6height_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_height) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":1293
 * 
 *         def __set__(self, int height):
 *             self._Radar.h.height = height             # <<<<<<<<<<<<<<
 * 
 *     property spulse:
 */
  __pyx_v_self->_Radar->h.height = __pyx_v_height;

  /* "pyart/io/_rsl_interface.pyx":1292
 *             return self._Radar.h.height
 * 
 *         def __set__(self, int height):             # <<<<<<<<<<<<<<
 *             self._Radar.h.height = height
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1296
 * 
 *     property spulse:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.spulse
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6spulse_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6spulse_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6spulse___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6spulse___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":1297
 *     property spulse:
 *         def __get__(self):
 *             return self._Radar.h.spulse             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int spulse):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Radar->h.spulse); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1297, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1296
 * 
 *     property spulse:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.spulse
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.spulse.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1299
 *             return self._Radar.h.spulse
 * 
 *         def __set__(self, int spulse):             # <<<<<<<<<<<<<<
 *             self._Radar.h.spulse = spulse
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6spulse_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_spulse); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6spulse_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_spulse) {
  int __pyx_v_spulse;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_spulse); {
    __pyx_v_spulse = __Pyx_PyInt_As_int(__pyx_arg_spulse); if (unlikely((__pyx_v_spulse == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1299, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.spulse.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6spulse_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self), ((int)__pyx_v_spulse));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6spulse_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_spulse) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":1300
 * 
 *         def __set__(self, int spulse):
 *             self._Radar.h.spulse = spulse             # <<<<<<<<<<<<<<
 * 
 *     property lpulse:
 */
  __pyx_v_self->_Radar->h.spulse = __pyx_v_spulse;

  /* "pyart/io/_rsl_interface.pyx":1299
 *             return self._Radar.h.spulse
 * 
 *         def __set__(self, int spulse):             # <<<<<<<<<<<<<<
 *             self._Radar.h.spulse = spulse
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1303
 * 
 *     property lpulse:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.lpulse
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6lpulse_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6lpulse_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6lpulse___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6lpulse___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":1304
 *     property lpulse:
 *         def __get__(self):
 *             return self._Radar.h.lpulse             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int lpulse):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Radar->h.lpulse); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1303
 * 
 *     property lpulse:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.lpulse
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.lpulse.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1306
 *             return self._Radar.h.lpulse
 * 
 *         def __set__(self, int lpulse):             # <<<<<<<<<<<<<<
 *             self._Radar.h.lpulse = lpulse
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6lpulse_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_lpulse); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6lpulse_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_lpulse) {
  int __pyx_v_lpulse;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_lpulse); {
    __pyx_v_lpulse = __Pyx_PyInt_As_int(__pyx_arg_lpulse); if (unlikely((__pyx_v_lpulse == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1306, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.lpulse.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6lpulse_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self), ((int)__pyx_v_lpulse));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_6lpulse_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_lpulse) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":1307
 * 
 *         def __set__(self, int lpulse):
 *             self._Radar.h.lpulse = lpulse             # <<<<<<<<<<<<<<
 * 
 *     property scan_mode:
 */
  __pyx_v_self->_Radar->h.lpulse = __pyx_v_lpulse;

  /* "pyart/io/_rsl_interface.pyx":1306
 *             return self._Radar.h.lpulse
 * 
 *         def __set__(self, int lpulse):             # <<<<<<<<<<<<<<
 *             self._Radar.h.lpulse = lpulse
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1310
 * 
 *     property scan_mode:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.scan_mode
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_9scan_mode_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_9scan_mode_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_9scan_mode___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_9scan_mode___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":1311
 *     property scan_mode:
 *         def __get__(self):
 *             return self._Radar.h.scan_mode             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int scan_mode):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Radar->h.scan_mode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1311, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1310
 * 
 *     property scan_mode:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.scan_mode
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.scan_mode.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1313
 *             return self._Radar.h.scan_mode
 * 
 *         def __set__(self, int scan_mode):             # <<<<<<<<<<<<<<
 *             self._Radar.h.scan_mode = scan_mode
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_9scan_mode_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_scan_mode); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_9scan_mode_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_scan_mode) {
  int __pyx_v_scan_mode;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_scan_mode); {
    __pyx_v_scan_mode = __Pyx_PyInt_As_int(__pyx_arg_scan_mode); if (unlikely((__pyx_v_scan_mode == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1313, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.scan_mode.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_9scan_mode_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self), ((int)__pyx_v_scan_mode));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_9scan_mode_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_scan_mode) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":1314
 * 
 *         def __set__(self, int scan_mode):
 *             self._Radar.h.scan_mode = scan_mode             # <<<<<<<<<<<<<<
 * 
 *     property vcp:
 */
  __pyx_v_self->_Radar->h.scan_mode = __pyx_v_scan_mode;

  /* "pyart/io/_rsl_interface.pyx":1313
 *             return self._Radar.h.scan_mode
 * 
 *         def __set__(self, int scan_mode):             # <<<<<<<<<<<<<<
 *             self._Radar.h.scan_mode = scan_mode
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1317
 * 
 *     property vcp:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.vcp
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_3vcp_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_3vcp_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_3vcp___get__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_3vcp___get__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyart/io/_rsl_interface.pyx":1318
 *     property vcp:
 *         def __get__(self):
 *             return self._Radar.h.vcp             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, int vcp):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_Radar->h.vcp); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyart/io/_rsl_interface.pyx":1317
 * 
 *     property vcp:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._Radar.h.vcp
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.vcp.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyart/io/_rsl_interface.pyx":1320
 *             return self._Radar.h.vcp
 * 
 *         def __set__(self, int vcp):             # <<<<<<<<<<<<<<
 *             self._Radar.h.vcp = vcp
 */

/* Python wrapper */
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_3vcp_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_vcp); /*proto*/
static int __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_3vcp_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_vcp) {
  int __pyx_v_vcp;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_vcp); {
    __pyx_v_vcp = __Pyx_PyInt_As_int(__pyx_arg_vcp); if (unlikely((__pyx_v_vcp == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1320, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.vcp.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_3vcp_2__set__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self), ((int)__pyx_v_vcp));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_3vcp_2__set__(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, int __pyx_v_vcp) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pyart/io/_rsl_interface.pyx":1321
 * 
 *         def __set__(self, int vcp):
 *             self._Radar.h.vcp = vcp             # <<<<<<<<<<<<<<
 */
  __pyx_v_self->_Radar->h.vcp = __pyx_v_vcp;

  /* "pyart/io/_rsl_interface.pyx":1320
 *             return self._Radar.h.vcp
 * 
 *         def __set__(self, int vcp):             # <<<<<<<<<<<<<<
 *             self._Radar.h.vcp = vcp
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_13__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_13__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_12__reduce_cython__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_12__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__15, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_15__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_15__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_14__setstate_cython__(((struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pyart_2io_14_rsl_interface_7RslFile_14__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyart.io._rsl_interface.RslFile.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":735
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":736
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 736, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":735
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":738
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":739
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 739, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":738
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":741
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":742
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 742, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":741
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":744
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":745
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 745, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":744
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":747
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":748
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 748, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":747
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":750
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyDataType_SHAPE(PyArray_Descr *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("PyDataType_SHAPE", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":751
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  __pyx_t_1 = (PyDataType_HASSUBARRAY(__pyx_v_d) != 0);
  if (__pyx_t_1) {

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":752
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape             # <<<<<<<<<<<<<<
 *     else:
 *         return ()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject*)__pyx_v_d->subarray->shape));
    __pyx_r = ((PyObject*)__pyx_v_d->subarray->shape);
    goto __pyx_L0;

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":751
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  }

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":754
 *         return <tuple>d.subarray.shape
 *     else:
 *         return ()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_empty_tuple);
    __pyx_r = __pyx_empty_tuple;
    goto __pyx_L0;
  }

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":750
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":929
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_array_base", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":930
 * 
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!             # <<<<<<<<<<<<<<
 *     PyArray_SetBaseObject(arr, base)
 * 
 */
  Py_INCREF(__pyx_v_base);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":931
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
 */
  (void)(PyArray_SetBaseObject(__pyx_v_arr, __pyx_v_base));

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":929
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":933
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_v_base;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_array_base", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":934
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)             # <<<<<<<<<<<<<<
 *     if base is NULL:
 *         return None
 */
  __pyx_v_base = PyArray_BASE(__pyx_v_arr);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":935
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  __pyx_t_1 = ((__pyx_v_base == NULL) != 0);
  if (__pyx_t_1) {

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":936
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     return <object>base
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":935
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  }

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":937
 *     if base is NULL:
 *         return None
 *     return <object>base             # <<<<<<<<<<<<<<
 * 
 * # Versions of the import_* functions which are more suitable for
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_base));
  __pyx_r = ((PyObject *)__pyx_v_base);
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":933
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":941
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_array(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_array", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":942
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":943
 * cdef inline int import_array() except -1:
 *     try:
 *         __pyx_import_array()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")
 */
      __pyx_t_4 = _import_array(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 943, __pyx_L3_error)

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":942
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":944
 *     try:
 *         __pyx_import_array()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 944, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":945
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__17, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 945, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 945, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":942
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":941
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":947
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_umath(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_umath", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":948
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":949
 * cdef inline int import_umath() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 949, __pyx_L3_error)

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":948
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":950
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 950, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":951
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__18, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 951, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 951, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":948
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":947
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":953
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_ufunc(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_ufunc", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":954
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":955
 * cdef inline int import_ufunc() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 955, __pyx_L3_error)

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":954
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":956
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 956, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":957
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef extern from *:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__18, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 957, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 957, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":954
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":953
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":967
 * 
 * 
 * cdef inline bint is_timedelta64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.timedelta64)`
 */

static CYTHON_INLINE int __pyx_f_5numpy_is_timedelta64_object(PyObject *__pyx_v_obj) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_timedelta64_object", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":979
 *     bool
 *     """
 *     return PyObject_TypeCheck(obj, &PyTimedeltaArrType_Type)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = PyObject_TypeCheck(__pyx_v_obj, (&PyTimedeltaArrType_Type));
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":967
 * 
 * 
 * cdef inline bint is_timedelta64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.timedelta64)`
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":982
 * 
 * 
 * cdef inline bint is_datetime64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.datetime64)`
 */

static CYTHON_INLINE int __pyx_f_5numpy_is_datetime64_object(PyObject *__pyx_v_obj) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_datetime64_object", 0);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":994
 *     bool
 *     """
 *     return PyObject_TypeCheck(obj, &PyDatetimeArrType_Type)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = PyObject_TypeCheck(__pyx_v_obj, (&PyDatetimeArrType_Type));
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":982
 * 
 * 
 * cdef inline bint is_datetime64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.datetime64)`
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":997
 * 
 * 
 * cdef inline npy_datetime get_datetime64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy datetime64 object
 */

static CYTHON_INLINE npy_datetime __pyx_f_5numpy_get_datetime64_value(PyObject *__pyx_v_obj) {
  npy_datetime __pyx_r;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":1004
 *     also needed.  That can be found using `get_datetime64_unit`.
 *     """
 *     return (<PyDatetimeScalarObject*>obj).obval             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((PyDatetimeScalarObject *)__pyx_v_obj)->obval;
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":997
 * 
 * 
 * cdef inline npy_datetime get_datetime64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy datetime64 object
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":1007
 * 
 * 
 * cdef inline npy_timedelta get_timedelta64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy timedelta64 object
 */

static CYTHON_INLINE npy_timedelta __pyx_f_5numpy_get_timedelta64_value(PyObject *__pyx_v_obj) {
  npy_timedelta __pyx_r;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":1011
 *     returns the int64 value underlying scalar numpy timedelta64 object
 *     """
 *     return (<PyTimedeltaScalarObject*>obj).obval             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((PyTimedeltaScalarObject *)__pyx_v_obj)->obval;
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":1007
 * 
 * 
 * cdef inline npy_timedelta get_timedelta64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy timedelta64 object
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":1014
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
 */

static CYTHON_INLINE NPY_DATETIMEUNIT __pyx_f_5numpy_get_datetime64_unit(PyObject *__pyx_v_obj) {
  NPY_DATETIMEUNIT __pyx_r;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":1018
 *     returns the unit part of the dtype for a numpy datetime64 object.
 *     """
 *     return <NPY_DATETIMEUNIT>(<PyDatetimeScalarObject*>obj).obmeta.base             # <<<<<<<<<<<<<<
 */
  __pyx_r = ((NPY_DATETIMEUNIT)((PyDatetimeScalarObject *)__pyx_v_obj)->obmeta.base);
  goto __pyx_L0;

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":1014
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "cfunc.to_py":65
 * @cname("__Pyx_CFunc_float____Range___to_py")
 * cdef object __Pyx_CFunc_float____Range___to_py(float (*f)(Range) except *):
 *     def wrap(Range x):             # <<<<<<<<<<<<<<
 *         """wrap(x: 'Range') -> 'float'"""
 *         return f(x)
 */

/* Python wrapper */
static PyObject *__pyx_pw_11cfunc_dot_to_py_34__Pyx_CFunc_float____Range___to_py_1wrap(PyObject *__pyx_self, PyObject *__pyx_arg_x); /*proto*/
static char __pyx_doc_11cfunc_dot_to_py_34__Pyx_CFunc_float____Range___to_py_wrap[] = "wrap(x: 'Range') -> 'float'";
static PyMethodDef __pyx_mdef_11cfunc_dot_to_py_34__Pyx_CFunc_float____Range___to_py_1wrap = {"wrap", (PyCFunction)__pyx_pw_11cfunc_dot_to_py_34__Pyx_CFunc_float____Range___to_py_1wrap, METH_O, __pyx_doc_11cfunc_dot_to_py_34__Pyx_CFunc_float____Range___to_py_wrap};
static PyObject *__pyx_pw_11cfunc_dot_to_py_34__Pyx_CFunc_float____Range___to_py_1wrap(PyObject *__pyx_self, PyObject *__pyx_arg_x) {
  __pyx_t_5pyart_2io_14_rsl_interface_Range __pyx_v_x;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("wrap (wrapper)", 0);
  assert(__pyx_arg_x); {
    __pyx_v_x = __Pyx_PyInt_As_unsigned_short(__pyx_arg_x); if (unlikely((__pyx_v_x == (unsigned short)-1) && PyErr_Occurred())) __PYX_ERR(1, 65, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("cfunc.to_py.__Pyx_CFunc_float____Range___to_py.wrap", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_11cfunc_dot_to_py_34__Pyx_CFunc_float____Range___to_py_wrap(__pyx_self, ((__pyx_t_5pyart_2io_14_rsl_interface_Range)__pyx_v_x));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11cfunc_dot_to_py_34__Pyx_CFunc_float____Range___to_py_wrap(PyObject *__pyx_self, __pyx_t_5pyart_2io_14_rsl_interface_Range __pyx_v_x) {
  struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_float____Range___to_py *__pyx_cur_scope;
  struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_float____Range___to_py *__pyx_outer_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  float __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("wrap", 0);
  __pyx_outer_scope = (struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_float____Range___to_py *) __Pyx_CyFunction_GetClosure(__pyx_self);
  __pyx_cur_scope = __pyx_outer_scope;

  /* "cfunc.to_py":67
 *     def wrap(Range x):
 *         """wrap(x: 'Range') -> 'float'"""
 *         return f(x)             # <<<<<<<<<<<<<<
 *     return wrap
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_cur_scope->__pyx_v_f(__pyx_v_x); if (unlikely(__pyx_t_1 == ((float)-1) && PyErr_Occurred())) __PYX_ERR(1, 67, __pyx_L1_error)
  __pyx_t_2 = PyFloat_FromDouble(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 67, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "cfunc.to_py":65
 * @cname("__Pyx_CFunc_float____Range___to_py")
 * cdef object __Pyx_CFunc_float____Range___to_py(float (*f)(Range) except *):
 *     def wrap(Range x):             # <<<<<<<<<<<<<<
 *         """wrap(x: 'Range') -> 'float'"""
 *         return f(x)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cfunc.to_py.__Pyx_CFunc_float____Range___to_py.wrap", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cfunc.to_py":64
 * 
 * @cname("__Pyx_CFunc_float____Range___to_py")
 * cdef object __Pyx_CFunc_float____Range___to_py(float (*f)(Range) except *):             # <<<<<<<<<<<<<<
 *     def wrap(Range x):
 *         """wrap(x: 'Range') -> 'float'"""
 */

static PyObject *__Pyx_CFunc_float____Range___to_py(float (*__pyx_v_f)(__pyx_t_5pyart_2io_14_rsl_interface_Range)) {
  struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_float____Range___to_py *__pyx_cur_scope;
  PyObject *__pyx_v_wrap = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_CFunc_float____Range___to_py", 0);
  __pyx_cur_scope = (struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_float____Range___to_py *)__pyx_tp_new___pyx_scope_struct____Pyx_CFunc_float____Range___to_py(__pyx_ptype___pyx_scope_struct____Pyx_CFunc_float____Range___to_py, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_float____Range___to_py *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(1, 64, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_f = __pyx_v_f;

  /* "cfunc.to_py":65
 * @cname("__Pyx_CFunc_float____Range___to_py")
 * cdef object __Pyx_CFunc_float____Range___to_py(float (*f)(Range) except *):
 *     def wrap(Range x):             # <<<<<<<<<<<<<<
 *         """wrap(x: 'Range') -> 'float'"""
 *         return f(x)
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_11cfunc_dot_to_py_34__Pyx_CFunc_float____Range___to_py_1wrap, 0, __pyx_n_s_Pyx_CFunc_float____Range___to, ((PyObject*)__pyx_cur_scope), __pyx_n_s_cfunc_to_py, __pyx_d, ((PyObject *)__pyx_codeobj__20)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_wrap = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cfunc.to_py":68
 *         """wrap(x: 'Range') -> 'float'"""
 *         return f(x)
 *     return wrap             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_wrap);
  __pyx_r = __pyx_v_wrap;
  goto __pyx_L0;

  /* "cfunc.to_py":64
 * 
 * @cname("__Pyx_CFunc_float____Range___to_py")
 * cdef object __Pyx_CFunc_float____Range___to_py(float (*f)(Range) except *):             # <<<<<<<<<<<<<<
 *     def wrap(Range x):
 *         """wrap(x: 'Range') -> 'float'"""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cfunc.to_py.__Pyx_CFunc_float____Range___to_py", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_wrap);
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cfunc.to_py":65
 * @cname("__Pyx_CFunc_Range____float___to_py")
 * cdef object __Pyx_CFunc_Range____float___to_py(Range (*f)(float) except *):
 *     def wrap(float x):             # <<<<<<<<<<<<<<
 *         """wrap(x: 'float') -> 'Range'"""
 *         return f(x)
 */

/* Python wrapper */
static PyObject *__pyx_pw_11cfunc_dot_to_py_34__Pyx_CFunc_Range____float___to_py_1wrap(PyObject *__pyx_self, PyObject *__pyx_arg_x); /*proto*/
static char __pyx_doc_11cfunc_dot_to_py_34__Pyx_CFunc_Range____float___to_py_wrap[] = "wrap(x: 'float') -> 'Range'";
static PyMethodDef __pyx_mdef_11cfunc_dot_to_py_34__Pyx_CFunc_Range____float___to_py_1wrap = {"wrap", (PyCFunction)__pyx_pw_11cfunc_dot_to_py_34__Pyx_CFunc_Range____float___to_py_1wrap, METH_O, __pyx_doc_11cfunc_dot_to_py_34__Pyx_CFunc_Range____float___to_py_wrap};
static PyObject *__pyx_pw_11cfunc_dot_to_py_34__Pyx_CFunc_Range____float___to_py_1wrap(PyObject *__pyx_self, PyObject *__pyx_arg_x) {
  float __pyx_v_x;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("wrap (wrapper)", 0);
  assert(__pyx_arg_x); {
    __pyx_v_x = __pyx_PyFloat_AsFloat(__pyx_arg_x); if (unlikely((__pyx_v_x == (float)-1) && PyErr_Occurred())) __PYX_ERR(1, 65, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("cfunc.to_py.__Pyx_CFunc_Range____float___to_py.wrap", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_11cfunc_dot_to_py_34__Pyx_CFunc_Range____float___to_py_wrap(__pyx_self, ((float)__pyx_v_x));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11cfunc_dot_to_py_34__Pyx_CFunc_Range____float___to_py_wrap(PyObject *__pyx_self, float __pyx_v_x) {
  struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_Range____float___to_py *__pyx_cur_scope;
  struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_Range____float___to_py *__pyx_outer_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __pyx_t_5pyart_2io_14_rsl_interface_Range __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("wrap", 0);
  __pyx_outer_scope = (struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_Range____float___to_py *) __Pyx_CyFunction_GetClosure(__pyx_self);
  __pyx_cur_scope = __pyx_outer_scope;

  /* "cfunc.to_py":67
 *     def wrap(float x):
 *         """wrap(x: 'float') -> 'Range'"""
 *         return f(x)             # <<<<<<<<<<<<<<
 *     return wrap
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_cur_scope->__pyx_v_f(__pyx_v_x); if (unlikely(__pyx_t_1 == ((__pyx_t_5pyart_2io_14_rsl_interface_Range)-1) && PyErr_Occurred())) __PYX_ERR(1, 67, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_From_unsigned_short(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 67, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "cfunc.to_py":65
 * @cname("__Pyx_CFunc_Range____float___to_py")
 * cdef object __Pyx_CFunc_Range____float___to_py(Range (*f)(float) except *):
 *     def wrap(float x):             # <<<<<<<<<<<<<<
 *         """wrap(x: 'float') -> 'Range'"""
 *         return f(x)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cfunc.to_py.__Pyx_CFunc_Range____float___to_py.wrap", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cfunc.to_py":64
 * 
 * @cname("__Pyx_CFunc_Range____float___to_py")
 * cdef object __Pyx_CFunc_Range____float___to_py(Range (*f)(float) except *):             # <<<<<<<<<<<<<<
 *     def wrap(float x):
 *         """wrap(x: 'float') -> 'Range'"""
 */

static PyObject *__Pyx_CFunc_Range____float___to_py(__pyx_t_5pyart_2io_14_rsl_interface_Range (*__pyx_v_f)(float)) {
  struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_Range____float___to_py *__pyx_cur_scope;
  PyObject *__pyx_v_wrap = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_CFunc_Range____float___to_py", 0);
  __pyx_cur_scope = (struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_Range____float___to_py *)__pyx_tp_new___pyx_scope_struct____Pyx_CFunc_Range____float___to_py(__pyx_ptype___pyx_scope_struct____Pyx_CFunc_Range____float___to_py, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_Range____float___to_py *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(1, 64, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_f = __pyx_v_f;

  /* "cfunc.to_py":65
 * @cname("__Pyx_CFunc_Range____float___to_py")
 * cdef object __Pyx_CFunc_Range____float___to_py(Range (*f)(float) except *):
 *     def wrap(float x):             # <<<<<<<<<<<<<<
 *         """wrap(x: 'float') -> 'Range'"""
 *         return f(x)
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_11cfunc_dot_to_py_34__Pyx_CFunc_Range____float___to_py_1wrap, 0, __pyx_n_s_Pyx_CFunc_Range____float___to, ((PyObject*)__pyx_cur_scope), __pyx_n_s_cfunc_to_py, __pyx_d, ((PyObject *)__pyx_codeobj__22)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_wrap = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cfunc.to_py":68
 *         """wrap(x: 'float') -> 'Range'"""
 *         return f(x)
 *     return wrap             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_wrap);
  __pyx_r = __pyx_v_wrap;
  goto __pyx_L0;

  /* "cfunc.to_py":64
 * 
 * @cname("__Pyx_CFunc_Range____float___to_py")
 * cdef object __Pyx_CFunc_Range____float___to_py(Range (*f)(float) except *):             # <<<<<<<<<<<<<<
 *     def wrap(float x):
 *         """wrap(x: 'float') -> 'Range'"""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cfunc.to_py.__Pyx_CFunc_Range____float___to_py", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_wrap);
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_5pyart_2io_14_rsl_interface__RslVolume __pyx_vtable_5pyart_2io_14_rsl_interface__RslVolume;

static PyObject *__pyx_tp_new_5pyart_2io_14_rsl_interface__RslVolume(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *)o);
  p->__pyx_vtab = __pyx_vtabptr_5pyart_2io_14_rsl_interface__RslVolume;
  return o;
}

static void __pyx_tp_dealloc_5pyart_2io_14_rsl_interface__RslVolume(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_1__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  (*Py_TYPE(o)->tp_free)(o);
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_10_RslVolume_nsweeps(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_7nsweeps_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_10_RslVolume_nsweeps(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_7nsweeps_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_10_RslVolume_calibr_const(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_12calibr_const_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_10_RslVolume_calibr_const(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_12calibr_const_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_5pyart_2io_14_rsl_interface__RslVolume[] = {
  {"total_rays", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_3total_rays, METH_NOARGS, __pyx_doc_5pyart_2io_14_rsl_interface_10_RslVolume_2total_rays},
  {"get_nray_array", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_5get_nray_array, METH_NOARGS, __pyx_doc_5pyart_2io_14_rsl_interface_10_RslVolume_4get_nray_array},
  {"get_sweep", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_7get_sweep, METH_O, __pyx_doc_5pyart_2io_14_rsl_interface_10_RslVolume_6get_sweep},
  {"get_azimuth_and_elev_array", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_9get_azimuth_and_elev_array, METH_NOARGS, __pyx_doc_5pyart_2io_14_rsl_interface_10_RslVolume_8get_azimuth_and_elev_array},
  {"get_sweep_fix_angles", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_11get_sweep_fix_angles, METH_NOARGS, __pyx_doc_5pyart_2io_14_rsl_interface_10_RslVolume_10get_sweep_fix_angles},
  {"get_sweep_azimuths", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_13get_sweep_azimuths, METH_NOARGS, __pyx_doc_5pyart_2io_14_rsl_interface_10_RslVolume_12get_sweep_azimuths},
  {"get_sweep_elevs", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_15get_sweep_elevs, METH_NOARGS, __pyx_doc_5pyart_2io_14_rsl_interface_10_RslVolume_14get_sweep_elevs},
  {"get_instr_params", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_17get_instr_params, METH_NOARGS, __pyx_doc_5pyart_2io_14_rsl_interface_10_RslVolume_16get_instr_params},
  {"get_data", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_19get_data, METH_NOARGS, __pyx_doc_5pyart_2io_14_rsl_interface_10_RslVolume_18get_data},
  {"is_range_bins_uniform", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_21is_range_bins_uniform, METH_NOARGS, __pyx_doc_5pyart_2io_14_rsl_interface_10_RslVolume_20is_range_bins_uniform},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_23__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_10_RslVolume_25__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5pyart_2io_14_rsl_interface__RslVolume[] = {
  {(char *)"nsweeps", __pyx_getprop_5pyart_2io_14_rsl_interface_10_RslVolume_nsweeps, __pyx_setprop_5pyart_2io_14_rsl_interface_10_RslVolume_nsweeps, (char *)0, 0},
  {(char *)"calibr_const", __pyx_getprop_5pyart_2io_14_rsl_interface_10_RslVolume_calibr_const, __pyx_setprop_5pyart_2io_14_rsl_interface_10_RslVolume_calibr_const, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pyart_2io_14_rsl_interface__RslVolume = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyart.io._rsl_interface._RslVolume", /*tp_name*/
  sizeof(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pyart_2io_14_rsl_interface__RslVolume, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  "\n    A object for accessing RSL Volume data and header information.\n\n    This class should not be initalized from within Python.  _RslVolume\n    objects are returned from the :py:func:`RslFile.get_volume` and other\n    functions/methods.\n\n    Attributes\n    ----------\n    nsweeps : int\n        Sweep number.\n    calibr_const : float\n        Calibration constant.\n\n    ", /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pyart_2io_14_rsl_interface__RslVolume, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5pyart_2io_14_rsl_interface__RslVolume, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pyart_2io_14_rsl_interface__RslVolume, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_5pyart_2io_14_rsl_interface__RslRay __pyx_vtable_5pyart_2io_14_rsl_interface__RslRay;

static PyObject *__pyx_tp_new_5pyart_2io_14_rsl_interface__RslRay(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *)o);
  p->__pyx_vtab = __pyx_vtabptr_5pyart_2io_14_rsl_interface__RslRay;
  return o;
}

static void __pyx_tp_dealloc_5pyart_2io_14_rsl_interface__RslRay(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_1__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  (*Py_TYPE(o)->tp_free)(o);
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_month(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_5month_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_month(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_5month_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_day(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3day_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_day(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3day_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_year(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4year_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_year(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4year_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_hour(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4hour_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_hour(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4hour_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_minute(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_6minute_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_minute(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_6minute_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_sec(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3sec_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_sec(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3sec_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_unam_rng(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_8unam_rng_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_unam_rng(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_8unam_rng_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_azimuth(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7azimuth_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_azimuth(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7azimuth_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_ray_num(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7ray_num_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_ray_num(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7ray_num_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_elev(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4elev_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_elev(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4elev_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_elev_num(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_8elev_num_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_elev_num(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_8elev_num_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_range_bin1(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10range_bin1_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_range_bin1(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10range_bin1_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_gate_size(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9gate_size_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_gate_size(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9gate_size_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_vel_res(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7vel_res_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_vel_res(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7vel_res_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_sweep_rate(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10sweep_rate_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_sweep_rate(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10sweep_rate_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_prf(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3prf_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_prf(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3prf_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_prf2(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4prf2_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_prf2(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4prf2_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_azim_rate(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9azim_rate_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_azim_rate(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9azim_rate_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_fix_angle(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9fix_angle_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_fix_angle(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9fix_angle_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_pitch(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_5pitch_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_pitch(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_5pitch_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_roll(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4roll_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_roll(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_4roll_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_heading(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7heading_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_heading(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7heading_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_pitch_rate(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10pitch_rate_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_pitch_rate(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10pitch_rate_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_roll_rate(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9roll_rate_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_roll_rate(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9roll_rate_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_heading_rate(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_12heading_rate_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_heading_rate(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_12heading_rate_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_lat(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3lat_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_lat(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3lat_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_lon(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3lon_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_lon(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3lon_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_alt(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3alt_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_alt(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3alt_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_rvc(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3rvc_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_rvc(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3rvc_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_vel_east(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_8vel_east_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_vel_east(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_8vel_east_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_vel_north(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9vel_north_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_vel_north(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9vel_north_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_vel_up(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_6vel_up_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_vel_up(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_6vel_up_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_pulse_count(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_11pulse_count_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_pulse_count(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_11pulse_count_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_pulse_width(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_11pulse_width_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_pulse_width(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_11pulse_width_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_beam_width(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10beam_width_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_beam_width(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10beam_width_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_frequency(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9frequency_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_frequency(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9frequency_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_wavelength(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10wavelength_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_wavelength(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_10wavelength_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_nyq_vel(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7nyq_vel_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_nyq_vel(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7nyq_vel_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_nbins(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_5nbins_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_nbins(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_5nbins_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_5pyart_2io_14_rsl_interface__RslRay[] = {
  {"get_datetime", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_3get_datetime, METH_NOARGS, __pyx_doc_5pyart_2io_14_rsl_interface_7_RslRay_2get_datetime},
  {"get_data", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_5get_data, METH_NOARGS, __pyx_doc_5pyart_2io_14_rsl_interface_7_RslRay_4get_data},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_7_RslRay_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5pyart_2io_14_rsl_interface__RslRay[] = {
  {(char *)"month", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_month, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_month, (char *)0, 0},
  {(char *)"day", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_day, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_day, (char *)0, 0},
  {(char *)"year", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_year, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_year, (char *)0, 0},
  {(char *)"hour", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_hour, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_hour, (char *)0, 0},
  {(char *)"minute", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_minute, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_minute, (char *)0, 0},
  {(char *)"sec", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_sec, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_sec, (char *)0, 0},
  {(char *)"unam_rng", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_unam_rng, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_unam_rng, (char *)0, 0},
  {(char *)"azimuth", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_azimuth, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_azimuth, (char *)0, 0},
  {(char *)"ray_num", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_ray_num, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_ray_num, (char *)0, 0},
  {(char *)"elev", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_elev, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_elev, (char *)0, 0},
  {(char *)"elev_num", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_elev_num, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_elev_num, (char *)0, 0},
  {(char *)"range_bin1", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_range_bin1, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_range_bin1, (char *)0, 0},
  {(char *)"gate_size", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_gate_size, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_gate_size, (char *)0, 0},
  {(char *)"vel_res", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_vel_res, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_vel_res, (char *)0, 0},
  {(char *)"sweep_rate", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_sweep_rate, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_sweep_rate, (char *)0, 0},
  {(char *)"prf", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_prf, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_prf, (char *)0, 0},
  {(char *)"prf2", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_prf2, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_prf2, (char *)0, 0},
  {(char *)"azim_rate", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_azim_rate, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_azim_rate, (char *)0, 0},
  {(char *)"fix_angle", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_fix_angle, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_fix_angle, (char *)0, 0},
  {(char *)"pitch", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_pitch, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_pitch, (char *)0, 0},
  {(char *)"roll", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_roll, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_roll, (char *)0, 0},
  {(char *)"heading", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_heading, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_heading, (char *)0, 0},
  {(char *)"pitch_rate", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_pitch_rate, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_pitch_rate, (char *)0, 0},
  {(char *)"roll_rate", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_roll_rate, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_roll_rate, (char *)0, 0},
  {(char *)"heading_rate", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_heading_rate, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_heading_rate, (char *)0, 0},
  {(char *)"lat", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_lat, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_lat, (char *)0, 0},
  {(char *)"lon", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_lon, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_lon, (char *)0, 0},
  {(char *)"alt", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_alt, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_alt, (char *)0, 0},
  {(char *)"rvc", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_rvc, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_rvc, (char *)0, 0},
  {(char *)"vel_east", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_vel_east, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_vel_east, (char *)0, 0},
  {(char *)"vel_north", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_vel_north, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_vel_north, (char *)0, 0},
  {(char *)"vel_up", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_vel_up, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_vel_up, (char *)0, 0},
  {(char *)"pulse_count", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_pulse_count, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_pulse_count, (char *)0, 0},
  {(char *)"pulse_width", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_pulse_width, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_pulse_width, (char *)0, 0},
  {(char *)"beam_width", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_beam_width, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_beam_width, (char *)0, 0},
  {(char *)"frequency", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_frequency, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_frequency, (char *)0, 0},
  {(char *)"wavelength", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_wavelength, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_wavelength, (char *)0, 0},
  {(char *)"nyq_vel", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_nyq_vel, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_nyq_vel, (char *)0, 0},
  {(char *)"nbins", __pyx_getprop_5pyart_2io_14_rsl_interface_7_RslRay_nbins, __pyx_setprop_5pyart_2io_14_rsl_interface_7_RslRay_nbins, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pyart_2io_14_rsl_interface__RslRay = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyart.io._rsl_interface._RslRay", /*tp_name*/
  sizeof(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pyart_2io_14_rsl_interface__RslRay, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  "\n    A object for accessing RSL Ray data and header information\n\n    This class should not be initalized from within Python.  _RslRay object are\n    returned from the :py:func:`_RslSweep.get_ray` method.\n\n    Attributes\n    ----------\n    month : int\n        Date for this ray, month (1-12).\n    day : int\n        Date for this ray, day (1-31).\n    year : int\n        Date for this ray, year (eg. 1993).\n    hour : int\n        Time for this ray, hour (0-23).\n    minute : int\n        Time for this ray, minute (0-59).\n    sec : float\n        Time for this ray, second + fractor of second.\n    unam_rng : float\n        Unambiguous range in km.\n    azimuth : float\n        Mean azimuth angle in degrees for the ray, must be positive.\n        0 for north, 90 for east, 270 for west.\n    ray_num : int\n        Ray number within a scan.\n    elev : float\n        Elevation angle in degrees.\n    elev_num : int\n        Elevation number within the volume scan.\n    range_bin1 : int\n        Range to first gate in meters.\n    gate_size : int\n        Gate size in meters.\n    vel_res : float\n        Doppler velocity resolution.\n    sweep_rate : float\n        Sweep rate, full sweeps / minute.\n    prf : int\n        Pulse repetition frequency in Hz.\n    prf2 : int\n        Second pulse repition frequenct for dual PRF data.\n    azim_rate : float\n        Sweep rate in degrees / second.\n    fix_angle : float\n        Elevation angle for the sweep in degrees.\n    pitch : float\n        Pitch angle.\n    roll : float\n        Roll angle.\n    heading : float\n        Heading.\n    pitch_rate : float\n        Pitch rate in angle / sec.\n    roll_rate : float\n        Roll rate in angle / sec.\n    heading_rate : float\n        Heading rate in angle / sec.\n    lat : float\n        Latitude in degrees.\n    lon : float\n        Longitude in degrees.\n    alt : int\n        Altitude in meters.\n    rvs : float\n        Radial velocity correction in meters / s""econd.\n    vel_east : float\n        Platform velocity to the east in meters / second.  Negative values for\n        velocity to the west.\n    vel_north : float\n        Platform velocity to the north in meters / second.  Negative values for\n        velocity to the south.\n    vel_up : float\n        Platform velocity upward in meters / second.  Negative values for\n        velocity downward.\n    pulse_count : int\n        Pulses used in a single dwell time.\n    pulse_width : float\n        Pulse width in microseconds.\n    beam_width : float\n        Beamwidth in degrees.\n    frequency : float\n        Carrier frequency in GHz.\n    wavelength : float\n        Wavelength in meters.\n    nyq_vel : float\n        Nyquist velocity in meters / second.\n    nbins : int\n        Number of array elements in ray data.\n\n    ", /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pyart_2io_14_rsl_interface__RslRay, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5pyart_2io_14_rsl_interface__RslRay, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pyart_2io_14_rsl_interface__RslRay, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_5pyart_2io_14_rsl_interface__RslSweep __pyx_vtable_5pyart_2io_14_rsl_interface__RslSweep;

static PyObject *__pyx_tp_new_5pyart_2io_14_rsl_interface__RslSweep(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *)o);
  p->__pyx_vtab = __pyx_vtabptr_5pyart_2io_14_rsl_interface__RslSweep;
  return o;
}

static void __pyx_tp_dealloc_5pyart_2io_14_rsl_interface__RslSweep(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_1__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  (*Py_TYPE(o)->tp_free)(o);
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_9_RslSweep_sweep_num(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_9sweep_num_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_9_RslSweep_sweep_num(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_9sweep_num_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_9_RslSweep_elev(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_4elev_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_9_RslSweep_elev(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_4elev_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_9_RslSweep_azimuth(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_7azimuth_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_9_RslSweep_azimuth(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_7azimuth_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_9_RslSweep_beam_width(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_10beam_width_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_9_RslSweep_beam_width(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_10beam_width_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_9_RslSweep_vert_half_bw(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_12vert_half_bw_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_9_RslSweep_vert_half_bw(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_12vert_half_bw_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_9_RslSweep_horz_half_bw(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_12horz_half_bw_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_9_RslSweep_horz_half_bw(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_12horz_half_bw_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_9_RslSweep_nrays(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_5nrays_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_9_RslSweep_nrays(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_5nrays_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_5pyart_2io_14_rsl_interface__RslSweep[] = {
  {"get_ray", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_3get_ray, METH_O, __pyx_doc_5pyart_2io_14_rsl_interface_9_RslSweep_2get_ray},
  {"get_data", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_5get_data, METH_NOARGS, __pyx_doc_5pyart_2io_14_rsl_interface_9_RslSweep_4get_data},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_9_RslSweep_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5pyart_2io_14_rsl_interface__RslSweep[] = {
  {(char *)"sweep_num", __pyx_getprop_5pyart_2io_14_rsl_interface_9_RslSweep_sweep_num, __pyx_setprop_5pyart_2io_14_rsl_interface_9_RslSweep_sweep_num, (char *)0, 0},
  {(char *)"elev", __pyx_getprop_5pyart_2io_14_rsl_interface_9_RslSweep_elev, __pyx_setprop_5pyart_2io_14_rsl_interface_9_RslSweep_elev, (char *)0, 0},
  {(char *)"azimuth", __pyx_getprop_5pyart_2io_14_rsl_interface_9_RslSweep_azimuth, __pyx_setprop_5pyart_2io_14_rsl_interface_9_RslSweep_azimuth, (char *)0, 0},
  {(char *)"beam_width", __pyx_getprop_5pyart_2io_14_rsl_interface_9_RslSweep_beam_width, __pyx_setprop_5pyart_2io_14_rsl_interface_9_RslSweep_beam_width, (char *)0, 0},
  {(char *)"vert_half_bw", __pyx_getprop_5pyart_2io_14_rsl_interface_9_RslSweep_vert_half_bw, __pyx_setprop_5pyart_2io_14_rsl_interface_9_RslSweep_vert_half_bw, (char *)0, 0},
  {(char *)"horz_half_bw", __pyx_getprop_5pyart_2io_14_rsl_interface_9_RslSweep_horz_half_bw, __pyx_setprop_5pyart_2io_14_rsl_interface_9_RslSweep_horz_half_bw, (char *)0, 0},
  {(char *)"nrays", __pyx_getprop_5pyart_2io_14_rsl_interface_9_RslSweep_nrays, __pyx_setprop_5pyart_2io_14_rsl_interface_9_RslSweep_nrays, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pyart_2io_14_rsl_interface__RslSweep = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyart.io._rsl_interface._RslSweep", /*tp_name*/
  sizeof(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pyart_2io_14_rsl_interface__RslSweep, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  "\n    A object for accessing RSL Sweep data and header information.\n\n    This class should not be initalized from within Python.  _RslSweep objects\n    are returned from the :py:func:`_RslVolume.get_sweep` method.\n\n    Attributes\n    ----------\n    sweep_num : int\n        Interger sweep number.\n    elev : float\n        Mean elevation angle for thr sweep.  -999.0 for RHI sweeps.\n    azimuth : float\n        Azumuth for the sweep.  -999.0 for PPI scans.\n    beam_width : float\n        Beam width in degrees.  Can also be found in _RslRay objects.\n    vert_half_bw : float\n        Vertical beam width divided by 2.\n    horz_half_bw : float\n        Horizontal beam width divided by 2.\n    nrays : int\n        Number of rays in the sweep.\n\n    ", /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pyart_2io_14_rsl_interface__RslSweep, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5pyart_2io_14_rsl_interface__RslSweep, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pyart_2io_14_rsl_interface__RslSweep, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};

static PyObject *__pyx_tp_new_5pyart_2io_14_rsl_interface_RslFile(PyTypeObject *t, PyObject *a, PyObject *k) {
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  if (unlikely(__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_5pyart_2io_14_rsl_interface_RslFile(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_3__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  (*Py_TYPE(o)->tp_free)(o);
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_month(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_5month_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_month(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_5month_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_day(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_3day_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_day(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_3day_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_year(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4year_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_year(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4year_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_hour(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4hour_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_hour(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4hour_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_minute(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6minute_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_minute(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6minute_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_sec(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_3sec_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_sec(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_3sec_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_nvolumes(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_8nvolumes_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_nvolumes(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_8nvolumes_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_number(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6number_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_number(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6number_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_latd(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4latd_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_latd(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4latd_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_latm(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4latm_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_latm(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4latm_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_lats(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4lats_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_lats(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4lats_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_lond(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4lond_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_lond(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4lond_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_lonm(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4lonm_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_lonm(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4lonm_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_lons(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4lons_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_lons(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_4lons_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_height(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6height_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_height(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6height_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_spulse(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6spulse_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_spulse(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6spulse_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_lpulse(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6lpulse_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_lpulse(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_6lpulse_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_scan_mode(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_9scan_mode_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_scan_mode(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_9scan_mode_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_vcp(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_3vcp_1__get__(o);
}

static int __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_vcp(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_3vcp_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_5pyart_2io_14_rsl_interface_RslFile[] = {
  {"get_volume", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_5get_volume, METH_O, __pyx_doc_5pyart_2io_14_rsl_interface_7RslFile_4get_volume},
  {"available_moments", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_7available_moments, METH_NOARGS, __pyx_doc_5pyart_2io_14_rsl_interface_7RslFile_6available_moments},
  {"get_radar_header", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_9get_radar_header, METH_NOARGS, __pyx_doc_5pyart_2io_14_rsl_interface_7RslFile_8get_radar_header},
  {"get_volume_array", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_11get_volume_array, METH_O, __pyx_doc_5pyart_2io_14_rsl_interface_7RslFile_10get_volume_array},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_13__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_7RslFile_15__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5pyart_2io_14_rsl_interface_RslFile[] = {
  {(char *)"month", __pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_month, __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_month, (char *)0, 0},
  {(char *)"day", __pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_day, __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_day, (char *)0, 0},
  {(char *)"year", __pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_year, __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_year, (char *)0, 0},
  {(char *)"hour", __pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_hour, __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_hour, (char *)0, 0},
  {(char *)"minute", __pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_minute, __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_minute, (char *)0, 0},
  {(char *)"sec", __pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_sec, __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_sec, (char *)0, 0},
  {(char *)"nvolumes", __pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_nvolumes, __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_nvolumes, (char *)0, 0},
  {(char *)"number", __pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_number, __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_number, (char *)0, 0},
  {(char *)"latd", __pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_latd, __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_latd, (char *)0, 0},
  {(char *)"latm", __pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_latm, __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_latm, (char *)0, 0},
  {(char *)"lats", __pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_lats, __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_lats, (char *)0, 0},
  {(char *)"lond", __pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_lond, __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_lond, (char *)0, 0},
  {(char *)"lonm", __pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_lonm, __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_lonm, (char *)0, 0},
  {(char *)"lons", __pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_lons, __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_lons, (char *)0, 0},
  {(char *)"height", __pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_height, __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_height, (char *)0, 0},
  {(char *)"spulse", __pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_spulse, __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_spulse, (char *)0, 0},
  {(char *)"lpulse", __pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_lpulse, __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_lpulse, (char *)0, 0},
  {(char *)"scan_mode", __pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_scan_mode, __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_scan_mode, (char *)0, 0},
  {(char *)"vcp", __pyx_getprop_5pyart_2io_14_rsl_interface_7RslFile_vcp, __pyx_setprop_5pyart_2io_14_rsl_interface_7RslFile_vcp, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pyart_2io_14_rsl_interface_RslFile = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyart.io._rsl_interface.RslFile", /*tp_name*/
  sizeof(struct __pyx_obj_5pyart_2io_14_rsl_interface_RslFile), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pyart_2io_14_rsl_interface_RslFile, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  "\n    RslFile(filename)\n\n    A object for accessing Radar data and parameter using the RSL library.\n\n    Parameters\n    ----------\n    filename : str\n        Radar file to read.\n\n    Attributes\n    ----------\n    month : int\n        Date, month (1-12).\n    day : int\n        Date, day (1-31).\n    year : int\n        Date, year (eg. 1993).\n    hour : int\n        Time, hour (0-23).\n    minute : int\n        Time, minute (0-59).\n    sec : float\n        Time, second + fractions of second.\n    nvolumes : int\n        Number of volume slots in the file.\n    number : int\n        Arbitrary number for this radar site.\n    latd, latm, lats : int\n        Latitude degrees, minutes and seconds for the site.\n    lond, lonm, lons : int\n        Longitude degrees, minutes and seconds for the site.\n    height : int\n        Height of site in meters above sea level.\n    spulse : int\n        Length of short pulse in ns.\n    lpulse : int\n        Length of long pulse in ns.\n    scan_mode : int\n        Scan mode, 0 for PPI, 1 for RHI.\n    vcp : int\n        Volume coverage pattern, WSR-88D only.\n\n    ", /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pyart_2io_14_rsl_interface_RslFile, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5pyart_2io_14_rsl_interface_RslFile, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pyart_2io_14_rsl_interface_RslFile, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};

static struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_float____Range___to_py *__pyx_freelist___pyx_scope_struct____Pyx_CFunc_float____Range___to_py[8];
static int __pyx_freecount___pyx_scope_struct____Pyx_CFunc_float____Range___to_py = 0;

static PyObject *__pyx_tp_new___pyx_scope_struct____Pyx_CFunc_float____Range___to_py(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount___pyx_scope_struct____Pyx_CFunc_float____Range___to_py > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_float____Range___to_py)))) {
    o = (PyObject*)__pyx_freelist___pyx_scope_struct____Pyx_CFunc_float____Range___to_py[--__pyx_freecount___pyx_scope_struct____Pyx_CFunc_float____Range___to_py];
    memset(o, 0, sizeof(struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_float____Range___to_py));
    (void) PyObject_INIT(o, t);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc___pyx_scope_struct____Pyx_CFunc_float____Range___to_py(PyObject *o) {
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount___pyx_scope_struct____Pyx_CFunc_float____Range___to_py < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_float____Range___to_py)))) {
    __pyx_freelist___pyx_scope_struct____Pyx_CFunc_float____Range___to_py[__pyx_freecount___pyx_scope_struct____Pyx_CFunc_float____Range___to_py++] = ((struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_float____Range___to_py *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static PyTypeObject __pyx_scope_struct____Pyx_CFunc_float____Range___to_py = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyart.io._rsl_interface.__pyx_scope_struct____Pyx_CFunc_float____Range___to_py", /*tp_name*/
  sizeof(struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_float____Range___to_py), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc___pyx_scope_struct____Pyx_CFunc_float____Range___to_py, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new___pyx_scope_struct____Pyx_CFunc_float____Range___to_py, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};

static struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_Range____float___to_py *__pyx_freelist___pyx_scope_struct____Pyx_CFunc_Range____float___to_py[8];
static int __pyx_freecount___pyx_scope_struct____Pyx_CFunc_Range____float___to_py = 0;

static PyObject *__pyx_tp_new___pyx_scope_struct____Pyx_CFunc_Range____float___to_py(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount___pyx_scope_struct____Pyx_CFunc_Range____float___to_py > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_Range____float___to_py)))) {
    o = (PyObject*)__pyx_freelist___pyx_scope_struct____Pyx_CFunc_Range____float___to_py[--__pyx_freecount___pyx_scope_struct____Pyx_CFunc_Range____float___to_py];
    memset(o, 0, sizeof(struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_Range____float___to_py));
    (void) PyObject_INIT(o, t);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc___pyx_scope_struct____Pyx_CFunc_Range____float___to_py(PyObject *o) {
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount___pyx_scope_struct____Pyx_CFunc_Range____float___to_py < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_Range____float___to_py)))) {
    __pyx_freelist___pyx_scope_struct____Pyx_CFunc_Range____float___to_py[__pyx_freecount___pyx_scope_struct____Pyx_CFunc_Range____float___to_py++] = ((struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_Range____float___to_py *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static PyTypeObject __pyx_scope_struct____Pyx_CFunc_Range____float___to_py = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyart.io._rsl_interface.__pyx_scope_struct____Pyx_CFunc_Range____float___to_py", /*tp_name*/
  sizeof(struct __pyx_obj___pyx_scope_struct____Pyx_CFunc_Range____float___to_py), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc___pyx_scope_struct____Pyx_CFunc_Range____float___to_py, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new___pyx_scope_struct____Pyx_CFunc_Range____float___to_py, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {"copy_volume", (PyCFunction)__pyx_pw_5pyart_2io_14_rsl_interface_1copy_volume, METH_O, __pyx_doc_5pyart_2io_14_rsl_interface_copy_volume},
  {"create_volume", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5pyart_2io_14_rsl_interface_3create_volume, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5pyart_2io_14_rsl_interface_2create_volume},
  {"_label_volume", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5pyart_2io_14_rsl_interface_5_label_volume, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5pyart_2io_14_rsl_interface_4_label_volume},
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec__rsl_interface(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec__rsl_interface},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "_rsl_interface",
    __pyx_k_Cython_wrapper_around_the_NASA, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_IOError, __pyx_k_IOError, sizeof(__pyx_k_IOError), 0, 0, 1, 1},
  {&__pyx_n_s_ImportError, __pyx_k_ImportError, sizeof(__pyx_k_ImportError), 0, 0, 1, 1},
  {&__pyx_n_s_Pyx_CFunc_Range____float___to, __pyx_k_Pyx_CFunc_Range____float___to, sizeof(__pyx_k_Pyx_CFunc_Range____float___to), 0, 0, 1, 1},
  {&__pyx_n_s_Pyx_CFunc_float____Range___to, __pyx_k_Pyx_CFunc_float____Range___to, sizeof(__pyx_k_Pyx_CFunc_float____Range___to), 0, 0, 1, 1},
  {&__pyx_n_s_RSL_VERSION_STR, __pyx_k_RSL_VERSION_STR, sizeof(__pyx_k_RSL_VERSION_STR), 0, 0, 1, 1},
  {&__pyx_n_s_RslFile, __pyx_k_RslFile, sizeof(__pyx_k_RslFile), 0, 0, 1, 1},
  {&__pyx_n_s_RslRay, __pyx_k_RslRay, sizeof(__pyx_k_RslRay), 0, 0, 1, 1},
  {&__pyx_n_s_RslSweep, __pyx_k_RslSweep, sizeof(__pyx_k_RslSweep), 0, 0, 1, 1},
  {&__pyx_n_s_RslVolume, __pyx_k_RslVolume, sizeof(__pyx_k_RslVolume), 0, 0, 1, 1},
  {&__pyx_kp_u_S24, __pyx_k_S24, sizeof(__pyx_k_S24), 0, 1, 0, 0},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_n_s_altitude, __pyx_k_altitude, sizeof(__pyx_k_altitude), 0, 0, 1, 1},
  {&__pyx_n_s_arr, __pyx_k_arr, sizeof(__pyx_k_arr), 0, 0, 1, 1},
  {&__pyx_n_s_azimuth, __pyx_k_azimuth, sizeof(__pyx_k_azimuth), 0, 0, 1, 1},
  {&__pyx_n_s_callid, __pyx_k_callid, sizeof(__pyx_k_callid), 0, 0, 1, 1},
  {&__pyx_kp_u_callid_must_be_provided, __pyx_k_callid_must_be_provided, sizeof(__pyx_k_callid_must_be_provided), 0, 1, 0, 0},
  {&__pyx_kp_u_cannot_load__RslRay_with_NULL, __pyx_k_cannot_load__RslRay_with_NULL, sizeof(__pyx_k_cannot_load__RslRay_with_NULL), 0, 1, 0, 0},
  {&__pyx_kp_u_cannot_load__RslSweep_with_NULL, __pyx_k_cannot_load__RslSweep_with_NULL, sizeof(__pyx_k_cannot_load__RslSweep_with_NULL), 0, 1, 0, 0},
  {&__pyx_kp_u_cannot_load__RslVolume_with_NULL, __pyx_k_cannot_load__RslVolume_with_NULL, sizeof(__pyx_k_cannot_load__RslVolume_with_NULL), 0, 1, 0, 0},
  {&__pyx_n_s_cfunc_to_py, __pyx_k_cfunc_to_py, sizeof(__pyx_k_cfunc_to_py), 0, 0, 1, 1},
  {&__pyx_n_s_city, __pyx_k_city, sizeof(__pyx_k_city), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_country, __pyx_k_country, sizeof(__pyx_k_country), 0, 0, 1, 1},
  {&__pyx_n_u_data, __pyx_k_data, sizeof(__pyx_k_data), 0, 1, 0, 1},
  {&__pyx_n_s_datetime, __pyx_k_datetime, sizeof(__pyx_k_datetime), 0, 0, 1, 1},
  {&__pyx_n_s_day, __pyx_k_day, sizeof(__pyx_k_day), 0, 0, 1, 1},
  {&__pyx_n_s_dtype, __pyx_k_dtype, sizeof(__pyx_k_dtype), 0, 0, 1, 1},
  {&__pyx_kp_u_dual, __pyx_k_dual, sizeof(__pyx_k_dual), 0, 1, 0, 0},
  {&__pyx_n_s_elevation, __pyx_k_elevation, sizeof(__pyx_k_elevation), 0, 0, 1, 1},
  {&__pyx_n_s_empty, __pyx_k_empty, sizeof(__pyx_k_empty), 0, 0, 1, 1},
  {&__pyx_kp_u_file_cannot_be_read, __pyx_k_file_cannot_be_read, sizeof(__pyx_k_file_cannot_be_read), 0, 1, 0, 0},
  {&__pyx_n_s_filename, __pyx_k_filename, sizeof(__pyx_k_filename), 0, 0, 1, 1},
  {&__pyx_kp_u_fixed, __pyx_k_fixed, sizeof(__pyx_k_fixed), 0, 1, 0, 0},
  {&__pyx_n_u_float32, __pyx_k_float32, sizeof(__pyx_k_float32), 0, 1, 0, 1},
  {&__pyx_n_s_get_data, __pyx_k_get_data, sizeof(__pyx_k_get_data), 0, 0, 1, 1},
  {&__pyx_n_s_get_nray_array, __pyx_k_get_nray_array, sizeof(__pyx_k_get_nray_array), 0, 0, 1, 1},
  {&__pyx_n_s_get_volume, __pyx_k_get_volume, sizeof(__pyx_k_get_volume), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_height, __pyx_k_height, sizeof(__pyx_k_height), 0, 0, 1, 1},
  {&__pyx_n_s_hour, __pyx_k_hour, sizeof(__pyx_k_hour), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_instrument_parameters, __pyx_k_instrument_parameters, sizeof(__pyx_k_instrument_parameters), 0, 0, 1, 1},
  {&__pyx_n_u_int32, __pyx_k_int32, sizeof(__pyx_k_int32), 0, 1, 0, 1},
  {&__pyx_kp_u_invalid_radar_format, __pyx_k_invalid_radar_format, sizeof(__pyx_k_invalid_radar_format), 0, 1, 0, 0},
  {&__pyx_kp_u_invalid_ray_number, __pyx_k_invalid_ray_number, sizeof(__pyx_k_invalid_ray_number), 0, 1, 0, 0},
  {&__pyx_kp_u_invalid_sweep_number, __pyx_k_invalid_sweep_number, sizeof(__pyx_k_invalid_sweep_number), 0, 1, 0, 0},
  {&__pyx_kp_u_invalid_volume_number, __pyx_k_invalid_volume_number, sizeof(__pyx_k_invalid_volume_number), 0, 1, 0, 0},
  {&__pyx_n_s_latd, __pyx_k_latd, sizeof(__pyx_k_latd), 0, 0, 1, 1},
  {&__pyx_n_s_latm, __pyx_k_latm, sizeof(__pyx_k_latm), 0, 0, 1, 1},
  {&__pyx_n_s_lats, __pyx_k_lats, sizeof(__pyx_k_lats), 0, 0, 1, 1},
  {&__pyx_n_s_lond, __pyx_k_lond, sizeof(__pyx_k_lond), 0, 0, 1, 1},
  {&__pyx_n_s_lonm, __pyx_k_lonm, sizeof(__pyx_k_lonm), 0, 0, 1, 1},
  {&__pyx_n_s_lons, __pyx_k_lons, sizeof(__pyx_k_lons), 0, 0, 1, 1},
  {&__pyx_n_s_lpulse, __pyx_k_lpulse, sizeof(__pyx_k_lpulse), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_u_meters_between_gates, __pyx_k_meters_between_gates, sizeof(__pyx_k_meters_between_gates), 0, 1, 0, 1},
  {&__pyx_n_u_meters_to_center_of_first_gate, __pyx_k_meters_to_center_of_first_gate, sizeof(__pyx_k_meters_to_center_of_first_gate), 0, 1, 0, 1},
  {&__pyx_n_s_minute, __pyx_k_minute, sizeof(__pyx_k_minute), 0, 0, 1, 1},
  {&__pyx_n_s_month, __pyx_k_month, sizeof(__pyx_k_month), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_name_2, __pyx_k_name_2, sizeof(__pyx_k_name_2), 0, 0, 1, 1},
  {&__pyx_kp_s_no_default___reduce___due_to_non, __pyx_k_no_default___reduce___due_to_non, sizeof(__pyx_k_no_default___reduce___due_to_non), 0, 0, 1, 0},
  {&__pyx_n_s_np, __pyx_k_np, sizeof(__pyx_k_np), 0, 0, 1, 1},
  {&__pyx_n_s_nsweeps, __pyx_k_nsweeps, sizeof(__pyx_k_nsweeps), 0, 0, 1, 1},
  {&__pyx_n_s_number, __pyx_k_number, sizeof(__pyx_k_number), 0, 0, 1, 1},
  {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
  {&__pyx_kp_u_numpy_core_multiarray_failed_to, __pyx_k_numpy_core_multiarray_failed_to, sizeof(__pyx_k_numpy_core_multiarray_failed_to), 0, 1, 0, 0},
  {&__pyx_kp_u_numpy_core_umath_failed_to_impor, __pyx_k_numpy_core_umath_failed_to_impor, sizeof(__pyx_k_numpy_core_umath_failed_to_impor), 0, 1, 0, 0},
  {&__pyx_n_s_nvolumes, __pyx_k_nvolumes, sizeof(__pyx_k_nvolumes), 0, 0, 1, 1},
  {&__pyx_n_u_nyquist_velocity, __pyx_k_nyquist_velocity, sizeof(__pyx_k_nyquist_velocity), 0, 1, 0, 1},
  {&__pyx_n_s_project, __pyx_k_project, sizeof(__pyx_k_project), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_radar, __pyx_k_radar, sizeof(__pyx_k_radar), 0, 0, 1, 1},
  {&__pyx_n_s_radar_format, __pyx_k_radar_format, sizeof(__pyx_k_radar_format), 0, 0, 1, 1},
  {&__pyx_n_s_radar_name, __pyx_k_radar_name, sizeof(__pyx_k_radar_name), 0, 0, 1, 1},
  {&__pyx_n_s_radar_type, __pyx_k_radar_type, sizeof(__pyx_k_radar_type), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_rays_per_sweep, __pyx_k_rays_per_sweep, sizeof(__pyx_k_rays_per_sweep), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_scan_mode, __pyx_k_scan_mode, sizeof(__pyx_k_scan_mode), 0, 0, 1, 1},
  {&__pyx_n_s_sec, __pyx_k_sec, sizeof(__pyx_k_sec), 0, 0, 1, 1},
  {&__pyx_n_s_seconds, __pyx_k_seconds, sizeof(__pyx_k_seconds), 0, 0, 1, 1},
  {&__pyx_kp_s_self__Ray_cannot_be_converted_to, __pyx_k_self__Ray_cannot_be_converted_to, sizeof(__pyx_k_self__Ray_cannot_be_converted_to), 0, 0, 1, 0},
  {&__pyx_kp_s_self__Sweep_cannot_be_converted, __pyx_k_self__Sweep_cannot_be_converted, sizeof(__pyx_k_self__Sweep_cannot_be_converted), 0, 0, 1, 0},
  {&__pyx_kp_s_self__Volume_cannot_be_converted, __pyx_k_self__Volume_cannot_be_converted, sizeof(__pyx_k_self__Volume_cannot_be_converted), 0, 0, 1, 0},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_shape, __pyx_k_shape, sizeof(__pyx_k_shape), 0, 0, 1, 1},
  {&__pyx_n_u_shape, __pyx_k_shape, sizeof(__pyx_k_shape), 0, 1, 0, 1},
  {&__pyx_n_s_spulse, __pyx_k_spulse, sizeof(__pyx_k_spulse), 0, 0, 1, 1},
  {&__pyx_n_s_state, __pyx_k_state, sizeof(__pyx_k_state), 0, 0, 1, 1},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_sum, __pyx_k_sum, sizeof(__pyx_k_sum), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_timedelta, __pyx_k_timedelta, sizeof(__pyx_k_timedelta), 0, 0, 1, 1},
  {&__pyx_n_s_total_rays, __pyx_k_total_rays, sizeof(__pyx_k_total_rays), 0, 0, 1, 1},
  {&__pyx_n_s_vcp, __pyx_k_vcp, sizeof(__pyx_k_vcp), 0, 0, 1, 1},
  {&__pyx_n_s_vol_num, __pyx_k_vol_num, sizeof(__pyx_k_vol_num), 0, 0, 1, 1},
  {&__pyx_n_s_volume, __pyx_k_volume, sizeof(__pyx_k_volume), 0, 0, 1, 1},
  {&__pyx_n_s_wrap, __pyx_k_wrap, sizeof(__pyx_k_wrap), 0, 0, 1, 1},
  {&__pyx_n_u_wsr88d, __pyx_k_wsr88d, sizeof(__pyx_k_wsr88d), 0, 1, 0, 1},
  {&__pyx_n_s_x, __pyx_k_x, sizeof(__pyx_k_x), 0, 0, 1, 1},
  {&__pyx_n_s_year, __pyx_k_year, sizeof(__pyx_k_year), 0, 0, 1, 1},
  {&__pyx_n_s_zeros, __pyx_k_zeros, sizeof(__pyx_k_zeros), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 111, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 296, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(1, 2, __pyx_L1_error)
  __pyx_builtin_IOError = __Pyx_GetBuiltinName(__pyx_n_s_IOError); if (!__pyx_builtin_IOError) __PYX_ERR(0, 1121, __pyx_L1_error)
  __pyx_builtin_ImportError = __Pyx_GetBuiltinName(__pyx_n_s_ImportError); if (!__pyx_builtin_ImportError) __PYX_ERR(2, 945, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "pyart/io/_rsl_interface.pyx":296
 *         """ Load the _RslRay object, must be called after creation. """
 *         if Ray is NULL:
 *             raise ValueError('cannot load _RslRay with NULL')             # <<<<<<<<<<<<<<
 *         self._Ray = Ray
 *         self._dealloc = 0
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_u_cannot_load__RslRay_with_NULL); if (unlikely(!__pyx_tuple_)) __PYX_ERR(0, 296, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._Ray cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._Ray cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_s_self__Ray_cannot_be_converted_to); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "(tree fragment)":4
 *     raise TypeError("self._Ray cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._Ray cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_s_self__Ray_cannot_be_converted_to); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "pyart/io/_rsl_interface.pyx":648
 *         """ Load the _RslSweep object, must be called after creation. """
 *         if Sweep is NULL:
 *             raise ValueError("cannot load _RslSweep with NULL")             # <<<<<<<<<<<<<<
 *         self._Sweep = Sweep
 *         self._dealloc = 0
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_u_cannot_load__RslSweep_with_NULL); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 648, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "pyart/io/_rsl_interface.pyx":670
 *         """
 *         if ray_number < 0 or ray_number >= self._Sweep.h.nrays:
 *             raise ValueError('invalid ray_number')             # <<<<<<<<<<<<<<
 *         rslray = _RslRay()
 *         rslray.load(self._Sweep.ray[ray_number])
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_u_invalid_ray_number); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(0, 670, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._Sweep cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._Sweep cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_s_self__Sweep_cannot_be_converted); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "(tree fragment)":4
 *     raise TypeError("self._Sweep cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._Sweep cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_kp_s_self__Sweep_cannot_be_converted); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "pyart/io/_rsl_interface.pyx":778
 *         """ Load the _RslVolume object, must be called after creation. """
 *         if Volume is NULL:
 *             raise ValueError('cannot load _RslVolume with NULL')             # <<<<<<<<<<<<<<
 *         self._Volume = Volume
 *         self._dealloc = 0
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_kp_u_cannot_load__RslVolume_with_NULL); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(0, 778, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "pyart/io/_rsl_interface.pyx":822
 *         """
 *         if sweep_number < 0 or sweep_number >= self._Volume.h.nsweeps:
 *             raise ValueError('invalid sweep_number')             # <<<<<<<<<<<<<<
 * 
 *         rslsweep = _RslSweep()
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_kp_u_invalid_sweep_number); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(0, 822, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._Volume cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._Volume cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_kp_s_self__Volume_cannot_be_converted); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "(tree fragment)":4
 *     raise TypeError("self._Volume cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._Volume cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_kp_s_self__Volume_cannot_be_converted); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "pyart/io/_rsl_interface.pyx":1114
 *         if radar_format == 'wsr88d':
 *             if callid is None:
 *                 raise ValueError('callid must be provided.')             # <<<<<<<<<<<<<<
 *             self._Radar = _rsl_h.RSL_wsr88d_to_radar(filename, callid)
 *         elif radar_format is None:
 */
  __pyx_tuple__12 = PyTuple_Pack(1, __pyx_kp_u_callid_must_be_provided); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(0, 1114, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "pyart/io/_rsl_interface.pyx":1121
 *             raise ValueError('invalid radar_format:', radar_format)
 *         if self._Radar is NULL:
 *             raise IOError('file cannot be read.')             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_kp_u_file_cannot_be_read); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(0, 1121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "pyart/io/_rsl_interface.pyx":1145
 *         """
 *         if volume_number < 0 or volume_number >= self._Radar.h.nvolumes:
 *             raise ValueError('invalid volume_number')             # <<<<<<<<<<<<<<
 * 
 *         rslvolume = _RslVolume()
 */
  __pyx_tuple__14 = PyTuple_Pack(1, __pyx_kp_u_invalid_volume_number); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(0, 1145, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__16 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":945
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
  __pyx_tuple__17 = PyTuple_Pack(1, __pyx_kp_u_numpy_core_multiarray_failed_to); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(2, 945, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);

  /* "../../anaconda3/envs/pyart_dist/lib/python3.9/site-packages/numpy/__init__.pxd":951
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
  __pyx_tuple__18 = PyTuple_Pack(1, __pyx_kp_u_numpy_core_umath_failed_to_impor); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(2, 951, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);

  /* "cfunc.to_py":65
 * @cname("__Pyx_CFunc_float____Range___to_py")
 * cdef object __Pyx_CFunc_float____Range___to_py(float (*f)(Range) except *):
 *     def wrap(Range x):             # <<<<<<<<<<<<<<
 *         """wrap(x: 'Range') -> 'float'"""
 *         return f(x)
 */
  __pyx_tuple__19 = PyTuple_Pack(2, __pyx_n_s_x, __pyx_n_s_x); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(1, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);
  __pyx_codeobj__20 = (PyObject*)__Pyx_PyCode_New(1, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__19, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_wrap, 65, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__20)) __PYX_ERR(1, 65, __pyx_L1_error)
  __pyx_tuple__21 = PyTuple_Pack(2, __pyx_n_s_x, __pyx_n_s_x); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(1, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);
  __pyx_codeobj__22 = (PyObject*)__Pyx_PyCode_New(1, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__21, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_wrap, 65, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__22)) __PYX_ERR(1, 65, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_float_0_1 = PyFloat_FromDouble(0.1); if (unlikely(!__pyx_float_0_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_1e6 = PyFloat_FromDouble(1e6); if (unlikely(!__pyx_float_1e6)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_1_31072000e_05 = PyFloat_FromDouble(1.31072000e+05); if (unlikely(!__pyx_float_1_31072000e_05)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_neg_999_ = PyFloat_FromDouble(-999.); if (unlikely(!__pyx_float_neg_999_)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_23 = PyInt_FromLong(23); if (unlikely(!__pyx_int_23)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_24 = PyInt_FromLong(24); if (unlikely(!__pyx_int_24)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_5pyart_2io_14_rsl_interface__RslVolume = &__pyx_vtable_5pyart_2io_14_rsl_interface__RslVolume;
  __pyx_vtable_5pyart_2io_14_rsl_interface__RslVolume.load = (PyObject *(*)(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *, Volume *))__pyx_f_5pyart_2io_14_rsl_interface_10_RslVolume_load;
  __pyx_vtable_5pyart_2io_14_rsl_interface__RslVolume._prtmode = (PyObject *(*)(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslVolume *, Ray_header))__pyx_f_5pyart_2io_14_rsl_interface_10_RslVolume__prtmode;
  if (PyType_Ready(&__pyx_type_5pyart_2io_14_rsl_interface__RslVolume) < 0) __PYX_ERR(0, 754, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_5pyart_2io_14_rsl_interface__RslVolume.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_5pyart_2io_14_rsl_interface__RslVolume.tp_dictoffset && __pyx_type_5pyart_2io_14_rsl_interface__RslVolume.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_5pyart_2io_14_rsl_interface__RslVolume.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_5pyart_2io_14_rsl_interface__RslVolume.tp_dict, __pyx_vtabptr_5pyart_2io_14_rsl_interface__RslVolume) < 0) __PYX_ERR(0, 754, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_RslVolume, (PyObject *)&__pyx_type_5pyart_2io_14_rsl_interface__RslVolume) < 0) __PYX_ERR(0, 754, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5pyart_2io_14_rsl_interface__RslVolume) < 0) __PYX_ERR(0, 754, __pyx_L1_error)
  __pyx_ptype_5pyart_2io_14_rsl_interface__RslVolume = &__pyx_type_5pyart_2io_14_rsl_interface__RslVolume;
  __pyx_vtabptr_5pyart_2io_14_rsl_interface__RslRay = &__pyx_vtable_5pyart_2io_14_rsl_interface__RslRay;
  __pyx_vtable_5pyart_2io_14_rsl_interface__RslRay.load = (PyObject *(*)(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslRay *, Ray *))__pyx_f_5pyart_2io_14_rsl_interface_7_RslRay_load;
  if (PyType_Ready(&__pyx_type_5pyart_2io_14_rsl_interface__RslRay) < 0) __PYX_ERR(0, 192, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_5pyart_2io_14_rsl_interface__RslRay.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_5pyart_2io_14_rsl_interface__RslRay.tp_dictoffset && __pyx_type_5pyart_2io_14_rsl_interface__RslRay.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_5pyart_2io_14_rsl_interface__RslRay.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_5pyart_2io_14_rsl_interface__RslRay.tp_dict, __pyx_vtabptr_5pyart_2io_14_rsl_interface__RslRay) < 0) __PYX_ERR(0, 192, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_RslRay, (PyObject *)&__pyx_type_5pyart_2io_14_rsl_interface__RslRay) < 0) __PYX_ERR(0, 192, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5pyart_2io_14_rsl_interface__RslRay) < 0) __PYX_ERR(0, 192, __pyx_L1_error)
  __pyx_ptype_5pyart_2io_14_rsl_interface__RslRay = &__pyx_type_5pyart_2io_14_rsl_interface__RslRay;
  __pyx_vtabptr_5pyart_2io_14_rsl_interface__RslSweep = &__pyx_vtable_5pyart_2io_14_rsl_interface__RslSweep;
  __pyx_vtable_5pyart_2io_14_rsl_interface__RslSweep.load = (PyObject *(*)(struct __pyx_obj_5pyart_2io_14_rsl_interface__RslSweep *, Sweep *))__pyx_f_5pyart_2io_14_rsl_interface_9_RslSweep_load;
  if (PyType_Ready(&__pyx_type_5pyart_2io_14_rsl_interface__RslSweep) < 0) __PYX_ERR(0, 612, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_5pyart_2io_14_rsl_interface__RslSweep.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_5pyart_2io_14_rsl_interface__RslSweep.tp_dictoffset && __pyx_type_5pyart_2io_14_rsl_interface__RslSweep.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_5pyart_2io_14_rsl_interface__RslSweep.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_5pyart_2io_14_rsl_interface__RslSweep.tp_dict, __pyx_vtabptr_5pyart_2io_14_rsl_interface__RslSweep) < 0) __PYX_ERR(0, 612, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_RslSweep, (PyObject *)&__pyx_type_5pyart_2io_14_rsl_interface__RslSweep) < 0) __PYX_ERR(0, 612, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5pyart_2io_14_rsl_interface__RslSweep) < 0) __PYX_ERR(0, 612, __pyx_L1_error)
  __pyx_ptype_5pyart_2io_14_rsl_interface__RslSweep = &__pyx_type_5pyart_2io_14_rsl_interface__RslSweep;
  if (PyType_Ready(&__pyx_type_5pyart_2io_14_rsl_interface_RslFile) < 0) __PYX_ERR(0, 1060, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_5pyart_2io_14_rsl_interface_RslFile.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_5pyart_2io_14_rsl_interface_RslFile.tp_dictoffset && __pyx_type_5pyart_2io_14_rsl_interface_RslFile.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_5pyart_2io_14_rsl_interface_RslFile.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_RslFile, (PyObject *)&__pyx_type_5pyart_2io_14_rsl_interface_RslFile) < 0) __PYX_ERR(0, 1060, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5pyart_2io_14_rsl_interface_RslFile) < 0) __PYX_ERR(0, 1060, __pyx_L1_error)
  __pyx_ptype_5pyart_2io_14_rsl_interface_RslFile = &__pyx_type_5pyart_2io_14_rsl_interface_RslFile;
  if (PyType_Ready(&__pyx_scope_struct____Pyx_CFunc_float____Range___to_py) < 0) __PYX_ERR(1, 64, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_scope_struct____Pyx_CFunc_float____Range___to_py.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_scope_struct____Pyx_CFunc_float____Range___to_py.tp_dictoffset && __pyx_scope_struct____Pyx_CFunc_float____Range___to_py.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_scope_struct____Pyx_CFunc_float____Range___to_py.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype___pyx_scope_struct____Pyx_CFunc_float____Range___to_py = &__pyx_scope_struct____Pyx_CFunc_float____Range___to_py;
  if (PyType_Ready(&__pyx_scope_struct____Pyx_CFunc_Range____float___to_py) < 0) __PYX_ERR(1, 64, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_scope_struct____Pyx_CFunc_Range____float___to_py.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_scope_struct____Pyx_CFunc_Range____float___to_py.tp_dictoffset && __pyx_scope_struct____Pyx_CFunc_Range____float___to_py.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_scope_struct____Pyx_CFunc_Range____float___to_py.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype___pyx_scope_struct____Pyx_CFunc_Range____float___to_py = &__pyx_scope_struct____Pyx_CFunc_Range____float___to_py;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("numpy"); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_5numpy_dtype = __Pyx_ImportType(__pyx_t_1, "numpy", "dtype", sizeof(PyArray_Descr), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_dtype) __PYX_ERR(2, 200, __pyx_L1_error)
  __pyx_ptype_5numpy_flatiter = __Pyx_ImportType(__pyx_t_1, "numpy", "flatiter", sizeof(PyArrayIterObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_flatiter) __PYX_ERR(2, 223, __pyx_L1_error)
  __pyx_ptype_5numpy_broadcast = __Pyx_ImportType(__pyx_t_1, "numpy", "broadcast", sizeof(PyArrayMultiIterObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_broadcast) __PYX_ERR(2, 227, __pyx_L1_error)
  __pyx_ptype_5numpy_ndarray = __Pyx_ImportType(__pyx_t_1, "numpy", "ndarray", sizeof(PyArrayObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_ndarray) __PYX_ERR(2, 239, __pyx_L1_error)
  __pyx_ptype_5numpy_generic = __Pyx_ImportType(__pyx_t_1, "numpy", "generic", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_generic) __PYX_ERR(2, 771, __pyx_L1_error)
  __pyx_ptype_5numpy_number = __Pyx_ImportType(__pyx_t_1, "numpy", "number", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_number) __PYX_ERR(2, 773, __pyx_L1_error)
  __pyx_ptype_5numpy_integer = __Pyx_ImportType(__pyx_t_1, "numpy", "integer", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_integer) __PYX_ERR(2, 775, __pyx_L1_error)
  __pyx_ptype_5numpy_signedinteger = __Pyx_ImportType(__pyx_t_1, "numpy", "signedinteger", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_signedinteger) __PYX_ERR(2, 777, __pyx_L1_error)
  __pyx_ptype_5numpy_unsignedinteger = __Pyx_ImportType(__pyx_t_1, "numpy", "unsignedinteger", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_unsignedinteger) __PYX_ERR(2, 779, __pyx_L1_error)
  __pyx_ptype_5numpy_inexact = __Pyx_ImportType(__pyx_t_1, "numpy", "inexact", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_inexact) __PYX_ERR(2, 781, __pyx_L1_error)
  __pyx_ptype_5numpy_floating = __Pyx_ImportType(__pyx_t_1, "numpy", "floating", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_floating) __PYX_ERR(2, 783, __pyx_L1_error)
  __pyx_ptype_5numpy_complexfloating = __Pyx_ImportType(__pyx_t_1, "numpy", "complexfloating", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_complexfloating) __PYX_ERR(2, 785, __pyx_L1_error)
  __pyx_ptype_5numpy_flexible = __Pyx_ImportType(__pyx_t_1, "numpy", "flexible", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_flexible) __PYX_ERR(2, 787, __pyx_L1_error)
  __pyx_ptype_5numpy_character = __Pyx_ImportType(__pyx_t_1, "numpy", "character", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_character) __PYX_ERR(2, 789, __pyx_L1_error)
  __pyx_ptype_5numpy_ufunc = __Pyx_ImportType(__pyx_t_1, "numpy", "ufunc", sizeof(PyUFuncObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_ufunc) __PYX_ERR(2, 827, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC init_rsl_interface(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC init_rsl_interface(void)
#else
__Pyx_PyMODINIT_FUNC PyInit__rsl_interface(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit__rsl_interface(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec__rsl_interface(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module '_rsl_interface' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit__rsl_interface(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("_rsl_interface", __pyx_methods, __pyx_k_Cython_wrapper_around_the_NASA, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_pyart__io___rsl_interface) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "pyart.io._rsl_interface")) {
      if (unlikely(PyDict_SetItemString(modules, "pyart.io._rsl_interface", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely(__Pyx_modinit_type_init_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_type_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "pyart/io/_rsl_interface.pyx":13
 * # define NPY_NO_DEPRECATED_API NPY_1_7_API_VERSION" [-W#warnings]
 * ctypedef unsigned short Range
 * import numpy as np             # <<<<<<<<<<<<<<
 * cimport numpy as np
 * from datetime import datetime, timedelta
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_np, __pyx_t_1) < 0) __PYX_ERR(0, 13, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyart/io/_rsl_interface.pyx":15
 * import numpy as np
 * cimport numpy as np
 * from datetime import datetime, timedelta             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_datetime);
  __Pyx_GIVEREF(__pyx_n_s_datetime);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_datetime);
  __Pyx_INCREF(__pyx_n_s_timedelta);
  __Pyx_GIVEREF(__pyx_n_s_timedelta);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_timedelta);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_datetime, __pyx_t_1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_datetime); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_datetime, __pyx_t_1) < 0) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_timedelta); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_timedelta, __pyx_t_1) < 0) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyart/io/_rsl_interface.pyx":18
 * 
 * 
 * _RSL_VERSION_STR = _rsl_h._RSL_VERSION_STR             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_FromString(RSL_VERSION_STR); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_RSL_VERSION_STR, __pyx_t_2) < 0) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyart/io/_rsl_interface.pyx":1
 * """             # <<<<<<<<<<<<<<
 * Cython wrapper around the NASA TRMM RSL library.
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cfunc.to_py":64
 * 
 * @cname("__Pyx_CFunc_Range____float___to_py")
 * cdef object __Pyx_CFunc_Range____float___to_py(Range (*f)(float) except *):             # <<<<<<<<<<<<<<
 *     def wrap(float x):
 *         """wrap(x: 'float') -> 'Range'"""
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init pyart.io._rsl_interface", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init pyart.io._rsl_interface");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* IsLittleEndian */
static CYTHON_INLINE int __Pyx_Is_Little_Endian(void)
{
  union {
    uint32_t u32;
    uint8_t u8[4];
  } S;
  S.u32 = 0x01020304;
  return S.u8[0] == 4;
}

/* BufferFormatCheck */
static void __Pyx_BufFmt_Init(__Pyx_BufFmt_Context* ctx,
                              __Pyx_BufFmt_StackElem* stack,
                              __Pyx_TypeInfo* type) {
  stack[0].field = &ctx->root;
  stack[0].parent_offset = 0;
  ctx->root.type = type;
  ctx->root.name = "buffer dtype";
  ctx->root.offset = 0;
  ctx->head = stack;
  ctx->head->field = &ctx->root;
  ctx->fmt_offset = 0;
  ctx->head->parent_offset = 0;
  ctx->new_packmode = '@';
  ctx->enc_packmode = '@';
  ctx->new_count = 1;
  ctx->enc_count = 0;
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  ctx->is_valid_array = 0;
  ctx->struct_alignment = 0;
  while (type->typegroup == 'S') {
    ++ctx->head;
    ctx->head->field = type->fields;
    ctx->head->parent_offset = 0;
    type = type->fields->type;
  }
}
static int __Pyx_BufFmt_ParseNumber(const char** ts) {
    int count;
    const char* t = *ts;
    if (*t < '0' || *t > '9') {
      return -1;
    } else {
        count = *t++ - '0';
        while (*t >= '0' && *t <= '9') {
            count *= 10;
            count += *t++ - '0';
        }
    }
    *ts = t;
    return count;
}
static int __Pyx_BufFmt_ExpectNumber(const char **ts) {
    int number = __Pyx_BufFmt_ParseNumber(ts);
    if (number == -1)
        PyErr_Format(PyExc_ValueError,\
                     "Does not understand character buffer dtype format string ('%c')", **ts);
    return number;
}
static void __Pyx_BufFmt_RaiseUnexpectedChar(char ch) {
  PyErr_Format(PyExc_ValueError,
               "Unexpected format string character: '%c'", ch);
}
static const char* __Pyx_BufFmt_DescribeTypeChar(char ch, int is_complex) {
  switch (ch) {
    case '?': return "'bool'";
    case 'c': return "'char'";
    case 'b': return "'signed char'";
    case 'B': return "'unsigned char'";
    case 'h': return "'short'";
    case 'H': return "'unsigned short'";
    case 'i': return "'int'";
    case 'I': return "'unsigned int'";
    case 'l': return "'long'";
    case 'L': return "'unsigned long'";
    case 'q': return "'long long'";
    case 'Q': return "'unsigned long long'";
    case 'f': return (is_complex ? "'complex float'" : "'float'");
    case 'd': return (is_complex ? "'complex double'" : "'double'");
    case 'g': return (is_complex ? "'complex long double'" : "'long double'");
    case 'T': return "a struct";
    case 'O': return "Python object";
    case 'P': return "a pointer";
    case 's': case 'p': return "a string";
    case 0: return "end";
    default: return "unparseable format string";
  }
}
static size_t __Pyx_BufFmt_TypeCharToStandardSize(char ch, int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return 2;
    case 'i': case 'I': case 'l': case 'L': return 4;
    case 'q': case 'Q': return 8;
    case 'f': return (is_complex ? 8 : 4);
    case 'd': return (is_complex ? 16 : 8);
    case 'g': {
      PyErr_SetString(PyExc_ValueError, "Python does not define a standard format string size for long double ('g')..");
      return 0;
    }
    case 'O': case 'P': return sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static size_t __Pyx_BufFmt_TypeCharToNativeSize(char ch, int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(short);
    case 'i': case 'I': return sizeof(int);
    case 'l': case 'L': return sizeof(long);
    #ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(PY_LONG_LONG);
    #endif
    case 'f': return sizeof(float) * (is_complex ? 2 : 1);
    case 'd': return sizeof(double) * (is_complex ? 2 : 1);
    case 'g': return sizeof(long double) * (is_complex ? 2 : 1);
    case 'O': case 'P': return sizeof(void*);
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
typedef struct { char c; short x; } __Pyx_st_short;
typedef struct { char c; int x; } __Pyx_st_int;
typedef struct { char c; long x; } __Pyx_st_long;
typedef struct { char c; float x; } __Pyx_st_float;
typedef struct { char c; double x; } __Pyx_st_double;
typedef struct { char c; long double x; } __Pyx_st_longdouble;
typedef struct { char c; void *x; } __Pyx_st_void_p;
#ifdef HAVE_LONG_LONG
typedef struct { char c; PY_LONG_LONG x; } __Pyx_st_longlong;
#endif
static size_t __Pyx_BufFmt_TypeCharToAlignment(char ch, CYTHON_UNUSED int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_st_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_st_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_st_long) - sizeof(long);
#ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(__Pyx_st_longlong) - sizeof(PY_LONG_LONG);
#endif
    case 'f': return sizeof(__Pyx_st_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_st_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_st_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_st_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
/* These are for computing the padding at the end of the struct to align
   on the first member of the struct. This will probably the same as above,
   but we don't have any guarantees.
 */
typedef struct { short x; char c; } __Pyx_pad_short;
typedef struct { int x; char c; } __Pyx_pad_int;
typedef struct { long x; char c; } __Pyx_pad_long;
typedef struct { float x; char c; } __Pyx_pad_float;
typedef struct { double x; char c; } __Pyx_pad_double;
typedef struct { long double x; char c; } __Pyx_pad_longdouble;
typedef struct { void *x; char c; } __Pyx_pad_void_p;
#ifdef HAVE_LONG_LONG
typedef struct { PY_LONG_LONG x; char c; } __Pyx_pad_longlong;
#endif
static size_t __Pyx_BufFmt_TypeCharToPadding(char ch, CYTHON_UNUSED int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_pad_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_pad_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_pad_long) - sizeof(long);
#ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(__Pyx_pad_longlong) - sizeof(PY_LONG_LONG);
#endif
    case 'f': return sizeof(__Pyx_pad_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_pad_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_pad_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_pad_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static char __Pyx_BufFmt_TypeCharToGroup(char ch, int is_complex) {
  switch (ch) {
    case 'c':
        return 'H';
    case 'b': case 'h': case 'i':
    case 'l': case 'q': case 's': case 'p':
        return 'I';
    case '?': case 'B': case 'H': case 'I': case 'L': case 'Q':
        return 'U';
    case 'f': case 'd': case 'g':
        return (is_complex ? 'C' : 'R');
    case 'O':
        return 'O';
    case 'P':
        return 'P';
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
static void __Pyx_BufFmt_RaiseExpected(__Pyx_BufFmt_Context* ctx) {
  if (ctx->head == NULL || ctx->head->field == &ctx->root) {
    const char* expected;
    const char* quote;
    if (ctx->head == NULL) {
      expected = "end";
      quote = "";
    } else {
      expected = ctx->head->field->type->name;
      quote = "'";
    }
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected %s%s%s but got %s",
                 quote, expected, quote,
                 __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex));
  } else {
    __Pyx_StructField* field = ctx->head->field;
    __Pyx_StructField* parent = (ctx->head - 1)->field;
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected '%s' but got %s in '%s.%s'",
                 field->type->name, __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex),
                 parent->type->name, field->name);
  }
}
static int __Pyx_BufFmt_ProcessTypeChunk(__Pyx_BufFmt_Context* ctx) {
  char group;
  size_t size, offset, arraysize = 1;
  if (ctx->enc_type == 0) return 0;
  if (ctx->head->field->type->arraysize[0]) {
    int i, ndim = 0;
    if (ctx->enc_type == 's' || ctx->enc_type == 'p') {
        ctx->is_valid_array = ctx->head->field->type->ndim == 1;
        ndim = 1;
        if (ctx->enc_count != ctx->head->field->type->arraysize[0]) {
            PyErr_Format(PyExc_ValueError,
                         "Expected a dimension of size %zu, got %zu",
                         ctx->head->field->type->arraysize[0], ctx->enc_count);
            return -1;
        }
    }
    if (!ctx->is_valid_array) {
      PyErr_Format(PyExc_ValueError, "Expected %d dimensions, got %d",
                   ctx->head->field->type->ndim, ndim);
      return -1;
    }
    for (i = 0; i < ctx->head->field->type->ndim; i++) {
      arraysize *= ctx->head->field->type->arraysize[i];
    }
    ctx->is_valid_array = 0;
    ctx->enc_count = 1;
  }
  group = __Pyx_BufFmt_TypeCharToGroup(ctx->enc_type, ctx->is_complex);
  do {
    __Pyx_StructField* field = ctx->head->field;
    __Pyx_TypeInfo* type = field->type;
    if (ctx->enc_packmode == '@' || ctx->enc_packmode == '^') {
      size = __Pyx_BufFmt_TypeCharToNativeSize(ctx->enc_type, ctx->is_complex);
    } else {
      size = __Pyx_BufFmt_TypeCharToStandardSize(ctx->enc_type, ctx->is_complex);
    }
    if (ctx->enc_packmode == '@') {
      size_t align_at = __Pyx_BufFmt_TypeCharToAlignment(ctx->enc_type, ctx->is_complex);
      size_t align_mod_offset;
      if (align_at == 0) return -1;
      align_mod_offset = ctx->fmt_offset % align_at;
      if (align_mod_offset > 0) ctx->fmt_offset += align_at - align_mod_offset;
      if (ctx->struct_alignment == 0)
          ctx->struct_alignment = __Pyx_BufFmt_TypeCharToPadding(ctx->enc_type,
                                                                 ctx->is_complex);
    }
    if (type->size != size || type->typegroup != group) {
      if (type->typegroup == 'C' && type->fields != NULL) {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        ++ctx->head;
        ctx->head->field = type->fields;
        ctx->head->parent_offset = parent_offset;
        continue;
      }
      if ((type->typegroup == 'H' || group == 'H') && type->size == size) {
      } else {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
      }
    }
    offset = ctx->head->parent_offset + field->offset;
    if (ctx->fmt_offset != offset) {
      PyErr_Format(PyExc_ValueError,
                   "Buffer dtype mismatch; next field is at offset %" CYTHON_FORMAT_SSIZE_T "d but %" CYTHON_FORMAT_SSIZE_T "d expected",
                   (Py_ssize_t)ctx->fmt_offset, (Py_ssize_t)offset);
      return -1;
    }
    ctx->fmt_offset += size;
    if (arraysize)
      ctx->fmt_offset += (arraysize - 1) * size;
    --ctx->enc_count;
    while (1) {
      if (field == &ctx->root) {
        ctx->head = NULL;
        if (ctx->enc_count != 0) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
        }
        break;
      }
      ctx->head->field = ++field;
      if (field->type == NULL) {
        --ctx->head;
        field = ctx->head->field;
        continue;
      } else if (field->type->typegroup == 'S') {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        if (field->type->fields->type == NULL) continue;
        field = field->type->fields;
        ++ctx->head;
        ctx->head->field = field;
        ctx->head->parent_offset = parent_offset;
        break;
      } else {
        break;
      }
    }
  } while (ctx->enc_count);
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  return 0;
}
static PyObject *
__pyx_buffmt_parse_array(__Pyx_BufFmt_Context* ctx, const char** tsp)
{
    const char *ts = *tsp;
    int i = 0, number, ndim;
    ++ts;
    if (ctx->new_count != 1) {
        PyErr_SetString(PyExc_ValueError,
                        "Cannot handle repeated arrays in format string");
        return NULL;
    }
    if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
    ndim = ctx->head->field->type->ndim;
    while (*ts && *ts != ')') {
        switch (*ts) {
            case ' ': case '\f': case '\r': case '\n': case '\t': case '\v':  continue;
            default:  break;
        }
        number = __Pyx_BufFmt_ExpectNumber(&ts);
        if (number == -1) return NULL;
        if (i < ndim && (size_t) number != ctx->head->field->type->arraysize[i])
            return PyErr_Format(PyExc_ValueError,
                        "Expected a dimension of size %zu, got %d",
                        ctx->head->field->type->arraysize[i], number);
        if (*ts != ',' && *ts != ')')
            return PyErr_Format(PyExc_ValueError,
                                "Expected a comma in format string, got '%c'", *ts);
        if (*ts == ',') ts++;
        i++;
    }
    if (i != ndim)
        return PyErr_Format(PyExc_ValueError, "Expected %d dimension(s), got %d",
                            ctx->head->field->type->ndim, i);
    if (!*ts) {
        PyErr_SetString(PyExc_ValueError,
                        "Unexpected end of format string, expected ')'");
        return NULL;
    }
    ctx->is_valid_array = 1;
    ctx->new_count = 1;
    *tsp = ++ts;
    return Py_None;
}
static const char* __Pyx_BufFmt_CheckString(__Pyx_BufFmt_Context* ctx, const char* ts) {
  int got_Z = 0;
  while (1) {
    switch(*ts) {
      case 0:
        if (ctx->enc_type != 0 && ctx->head == NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        if (ctx->head != NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        return ts;
      case ' ':
      case '\r':
      case '\n':
        ++ts;
        break;
      case '<':
        if (!__Pyx_Is_Little_Endian()) {
          PyErr_SetString(PyExc_ValueError, "Little-endian buffer not supported on big-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '>':
      case '!':
        if (__Pyx_Is_Little_Endian()) {
          PyErr_SetString(PyExc_ValueError, "Big-endian buffer not supported on little-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '=':
      case '@':
      case '^':
        ctx->new_packmode = *ts++;
        break;
      case 'T':
        {
          const char* ts_after_sub;
          size_t i, struct_count = ctx->new_count;
          size_t struct_alignment = ctx->struct_alignment;
          ctx->new_count = 1;
          ++ts;
          if (*ts != '{') {
            PyErr_SetString(PyExc_ValueError, "Buffer acquisition: Expected '{' after 'T'");
            return NULL;
          }
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          ctx->enc_count = 0;
          ctx->struct_alignment = 0;
          ++ts;
          ts_after_sub = ts;
          for (i = 0; i != struct_count; ++i) {
            ts_after_sub = __Pyx_BufFmt_CheckString(ctx, ts);
            if (!ts_after_sub) return NULL;
          }
          ts = ts_after_sub;
          if (struct_alignment) ctx->struct_alignment = struct_alignment;
        }
        break;
      case '}':
        {
          size_t alignment = ctx->struct_alignment;
          ++ts;
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          if (alignment && ctx->fmt_offset % alignment) {
            ctx->fmt_offset += alignment - (ctx->fmt_offset % alignment);
          }
        }
        return ts;
      case 'x':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->fmt_offset += ctx->new_count;
        ctx->new_count = 1;
        ctx->enc_count = 0;
        ctx->enc_type = 0;
        ctx->enc_packmode = ctx->new_packmode;
        ++ts;
        break;
      case 'Z':
        got_Z = 1;
        ++ts;
        if (*ts != 'f' && *ts != 'd' && *ts != 'g') {
          __Pyx_BufFmt_RaiseUnexpectedChar('Z');
          return NULL;
        }
        CYTHON_FALLTHROUGH;
      case '?': case 'c': case 'b': case 'B': case 'h': case 'H': case 'i': case 'I':
      case 'l': case 'L': case 'q': case 'Q':
      case 'f': case 'd': case 'g':
      case 'O': case 'p':
        if ((ctx->enc_type == *ts) && (got_Z == ctx->is_complex) &&
            (ctx->enc_packmode == ctx->new_packmode) && (!ctx->is_valid_array)) {
          ctx->enc_count += ctx->new_count;
          ctx->new_count = 1;
          got_Z = 0;
          ++ts;
          break;
        }
        CYTHON_FALLTHROUGH;
      case 's':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->enc_count = ctx->new_count;
        ctx->enc_packmode = ctx->new_packmode;
        ctx->enc_type = *ts;
        ctx->is_complex = got_Z;
        ++ts;
        ctx->new_count = 1;
        got_Z = 0;
        break;
      case ':':
        ++ts;
        while(*ts != ':') ++ts;
        ++ts;
        break;
      case '(':
        if (!__pyx_buffmt_parse_array(ctx, &ts)) return NULL;
        break;
      default:
        {
          int number = __Pyx_BufFmt_ExpectNumber(&ts);
          if (number == -1) return NULL;
          ctx->new_count = (size_t)number;
        }
    }
  }
}

/* BufferGetAndValidate */
  static CYTHON_INLINE void __Pyx_SafeReleaseBuffer(Py_buffer* info) {
  if (unlikely(info->buf == NULL)) return;
  if (info->suboffsets == __Pyx_minusones) info->suboffsets = NULL;
  __Pyx_ReleaseBuffer(info);
}
static void __Pyx_ZeroBuffer(Py_buffer* buf) {
  buf->buf = NULL;
  buf->obj = NULL;
  buf->strides = __Pyx_zeros;
  buf->shape = __Pyx_zeros;
  buf->suboffsets = __Pyx_minusones;
}
static int __Pyx__GetBufferAndValidate(
        Py_buffer* buf, PyObject* obj,  __Pyx_TypeInfo* dtype, int flags,
        int nd, int cast, __Pyx_BufFmt_StackElem* stack)
{
  buf->buf = NULL;
  if (unlikely(__Pyx_GetBuffer(obj, buf, flags) == -1)) {
    __Pyx_ZeroBuffer(buf);
    return -1;
  }
  if (unlikely(buf->ndim != nd)) {
    PyErr_Format(PyExc_ValueError,
                 "Buffer has wrong number of dimensions (expected %d, got %d)",
                 nd, buf->ndim);
    goto fail;
  }
  if (!cast) {
    __Pyx_BufFmt_Context ctx;
    __Pyx_BufFmt_Init(&ctx, stack, dtype);
    if (!__Pyx_BufFmt_CheckString(&ctx, buf->format)) goto fail;
  }
  if (unlikely((size_t)buf->itemsize != dtype->size)) {
    PyErr_Format(PyExc_ValueError,
      "Item size of buffer (%" CYTHON_FORMAT_SSIZE_T "d byte%s) does not match size of '%s' (%" CYTHON_FORMAT_SSIZE_T "d byte%s)",
      buf->itemsize, (buf->itemsize > 1) ? "s" : "",
      dtype->name, (Py_ssize_t)dtype->size, (dtype->size > 1) ? "s" : "");
    goto fail;
  }
  if (buf->suboffsets == NULL) buf->suboffsets = __Pyx_minusones;
  return 0;
fail:;
  __Pyx_SafeReleaseBuffer(buf);
  return -1;
}

/* BufferIndexError */
  static void __Pyx_RaiseBufferIndexError(int axis) {
  PyErr_Format(PyExc_IndexError,
     "Out of bounds on buffer access (axis %d)", axis);
}

/* PyCFunctionFastCall */
  #if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyObjectCallOneArg */
  #if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (__Pyx_PyFastCFunction_Check(func)) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* GetItemInt */
  static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* ObjectGetItem */
  #if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject* index) {
    PyObject *runerr;
    Py_ssize_t key_value;
    PySequenceMethods *m = Py_TYPE(obj)->tp_as_sequence;
    if (unlikely(!(m && m->sq_item))) {
        PyErr_Format(PyExc_TypeError, "'%.200s' object is not subscriptable", Py_TYPE(obj)->tp_name);
        return NULL;
    }
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError, "cannot fit '%.200s' into an index-sized integer", Py_TYPE(index)->tp_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key) {
    PyMappingMethods *m = Py_TYPE(obj)->tp_as_mapping;
    if (likely(m && m->mp_subscript)) {
        return m->mp_subscript(obj, key);
    }
    return __Pyx_PyObject_GetIndex(obj, key);
}
#endif

/* PyIntBinop */
  #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a + b);
            if (likely((x^a) >= 0 || (x^b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_add(op1, op2);
            }
        }
                x = a + b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla + llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("add", return NULL)
            result = ((double)a) + (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#endif

/* PyErrFetchRestore */
  #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseArgTupleInvalid */
  static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
  static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
  static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* DictGetItem */
  #if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            if (unlikely(PyTuple_Check(key))) {
                PyObject* args = PyTuple_Pack(1, key);
                if (likely(args)) {
                    PyErr_SetObject(PyExc_KeyError, args);
                    Py_DECREF(args);
                }
            } else {
                PyErr_SetObject(PyExc_KeyError, key);
            }
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#endif

/* RaiseException */
  #if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* RaiseTooManyValuesToUnpack */
  static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
  static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* IterFinish */
  static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* UnpackItemEndCheck */
  static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* PyIntCompare */
  static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED long inplace) {
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a == b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = Py_SIZE(op1);
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        if (intval == 0) {
            if (size == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
        } else if (intval < 0) {
            if (size >= 0)
                Py_RETURN_FALSE;
            intval = -intval;
            size = -size;
        } else {
            if (size <= 0)
                Py_RETURN_FALSE;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        if (unequal == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
        if ((double)a == (double)b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    return (
        PyObject_RichCompare(op1, op2, Py_EQ));
}

/* PyObjectSetAttrStr */
  #if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* PyDictVersioning */
  #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
  #if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyFloatBinop */
  #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyFloat_AddObjC(PyObject *op1, PyObject *op2, double floatval, int inplace, int zerodivision_check) {
    const double b = floatval;
    double a, result;
    (void)inplace;
    (void)zerodivision_check;
    if (likely(PyFloat_CheckExact(op1))) {
        a = PyFloat_AS_DOUBLE(op1);
        
    } else
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        a = (double) PyInt_AS_LONG(op1);
        
    } else
    #endif
    if (likely(PyLong_CheckExact(op1))) {
        #if CYTHON_USE_PYLONG_INTERNALS
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        switch (size) {
            case  0: a = 0.0; break;
            case -1: a = -(double) digits[0]; break;
            case  1: a = (double) digits[0]; break;
            case -2:
            case 2:
                if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (1 * PyLong_SHIFT < 53))) {
                    a = (double) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53) || (a < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -2)
                            a = -a;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            case -3:
            case 3:
                if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53))) {
                    a = (double) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53) || (a < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -3)
                            a = -a;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            case -4:
            case 4:
                if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53))) {
                    a = (double) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (4 * PyLong_SHIFT < 53) || (a < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -4)
                            a = -a;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            default:
        #else
        {
        #endif
            a = PyLong_AsDouble(op1);
            if (unlikely(a == -1.0 && PyErr_Occurred())) return NULL;
            
        }
    } else {
        return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
    }
        
        PyFPE_START_PROTECT("add", return NULL)
        result = a + b;
        PyFPE_END_PROTECT(result)
        return PyFloat_FromDouble(result);
}
#endif

/* ExtTypeTest */
    static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* BufferFallbackError */
    static void __Pyx_RaiseBufferFallbackError(void) {
  PyErr_SetString(PyExc_ValueError,
     "Buffer acquisition failed on assignment; and then reacquiring the old buffer failed too!");
}

/* PyObjectCall2Args */
    static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* py_abs */
    #if CYTHON_USE_PYLONG_INTERNALS
static PyObject *__Pyx_PyLong_AbsNeg(PyObject *n) {
    if (likely(Py_SIZE(n) == -1)) {
        return PyLong_FromLong(((PyLongObject*)n)->ob_digit[0]);
    }
#if CYTHON_COMPILING_IN_CPYTHON
    {
        PyObject *copy = _PyLong_Copy((PyLongObject*)n);
        if (likely(copy)) {
            __Pyx_SET_SIZE(copy, -Py_SIZE(copy));
        }
        return copy;
    }
#else
    return PyNumber_Negative(n);
#endif
}
#endif

/* BytesEquals */
    static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
    static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* GetTopmostException */
    #if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
    #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* PyErrExceptionMatches */
    #if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetException */
    #if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* FetchCommonType */
    static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CythonFunctionShared */
    #include <structmember.h>
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *closure)
{
    if (unlikely(op->func_doc == NULL)) {
        if (op->func.m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(op->func.m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(op->func.m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp = op->func_doc;
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    op->func_doc = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(op->func.m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(op->func.m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = op->func_name;
    Py_INCREF(value);
    op->func_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = op->func_qualname;
    Py_INCREF(value);
    op->func_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_self(__pyx_CyFunctionObject *m, CYTHON_UNUSED void *closure)
{
    PyObject *self;
    self = m->func_closure;
    if (self == NULL)
        self = Py_None;
    Py_INCREF(self);
    return self;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    tmp = op->func_dict;
    Py_INCREF(value);
    op->func_dict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(CYTHON_UNUSED __pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyTuple_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_tuple;
    op->defaults_tuple = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_kwdict;
    op->defaults_kwdict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value || value == Py_None) {
        value = NULL;
    } else if (!PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    tmp = op->func_annotations;
    op->func_annotations = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "__self__", (getter)__Pyx_CyFunction_get_self, 0, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), PY_WRITE_RESTRICTED, 0},
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, CYTHON_UNUSED PyObject *args)
{
#if PY_MAJOR_VERSION >= 3
    Py_INCREF(m->func_qualname);
    return m->func_qualname;
#else
    return PyString_FromString(m->func.m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func.m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    if (unlikely(op == NULL))
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    op->func.m_ml = ml;
    op->func.m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    op->func.m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
    op->func_classobj = NULL;
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults_size = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(m->func.m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
    Py_CLEAR(m->func_classobj);
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    PyObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(m->func.m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(m->func_classobj);
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject *__Pyx_CyFunction_descr_get(PyObject *func, PyObject *obj, PyObject *type)
{
#if PY_MAJOR_VERSION < 3
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    if (m->flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(func);
        return func;
    }
    if (m->flags & __Pyx_CYFUNCTION_CLASSMETHOD) {
        if (type == NULL)
            type = (PyObject *)(Py_TYPE(obj));
        return __Pyx_PyMethod_New(func, type, (PyObject *)(Py_TYPE(type)));
    }
    if (obj == Py_None)
        obj = NULL;
#endif
    return __Pyx_PyMethod_New(func, obj, type);
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void*)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags in "
                        "__Pyx_CyFunction_Call. METH_OLDARGS is no "
                        "longer supported!");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    return __Pyx_CyFunction_CallMethod(func, ((PyCFunctionObject*)func)->m_self, arg, kw);
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
        argc = PyTuple_GET_SIZE(args);
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_CyFunction_descr_get,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
    0,
#endif
};
static int __pyx_CyFunction_init(void) {
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    m->defaults_size = size;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
    static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* PyObject_GenericGetAttrNoDict */
    #if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
    #if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
    static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* PyObjectGetAttrStrNoError */
    static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
}

/* SetupReduce */
    static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* TypeImport */
    #ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* Import */
    static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
    static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* CLineInTraceback */
    #ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
    static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
    #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_srcfile = NULL;
    py_srcfile = PyString_FromString(filename);
    if (!py_srcfile) goto bad;
    #endif
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        if (!py_funcname) goto bad;
        #endif
    }
    #if PY_MAJOR_VERSION < 3
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    #else
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    #endif
    Py_XDECREF(py_funcname);  // XDECREF since it's only set on Py3 if cline
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_srcfile);
    #endif
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

#if PY_MAJOR_VERSION < 3
static int __Pyx_GetBuffer(PyObject *obj, Py_buffer *view, int flags) {
    if (PyObject_CheckBuffer(obj)) return PyObject_GetBuffer(obj, view, flags);
    PyErr_Format(PyExc_TypeError, "'%.200s' does not have the buffer interface", Py_TYPE(obj)->tp_name);
    return -1;
}
static void __Pyx_ReleaseBuffer(Py_buffer *view) {
    PyObject *obj = view->obj;
    if (!obj) return;
    if (PyObject_CheckBuffer(obj)) {
        PyBuffer_Release(view);
        return;
    }
    if ((0)) {}
    view->obj = NULL;
    Py_DECREF(obj);
}
#endif


    /* CIntFromPyVerify */
    #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

static PyObject* __pyx_convert__to_py_Radar_header(Radar_header s) {
      PyObject* res;
      PyObject* member;
      res = __Pyx_PyDict_NewPresized(26); if (unlikely(!res)) return NULL;
      member = __Pyx_PyInt_From_int(s.month); if (unlikely(!member)) goto bad;
      if (unlikely(PyDict_SetItem(res, __pyx_n_s_month, member) < 0)) goto bad;
      Py_DECREF(member);
      member = __Pyx_PyInt_From_int(s.day); if (unlikely(!member)) goto bad;
      if (unlikely(PyDict_SetItem(res, __pyx_n_s_day, member) < 0)) goto bad;
      Py_DECREF(member);
      member = __Pyx_PyInt_From_int(s.year); if (unlikely(!member)) goto bad;
      if (unlikely(PyDict_SetItem(res, __pyx_n_s_year, member) < 0)) goto bad;
      Py_DECREF(member);
      member = __Pyx_PyInt_From_int(s.hour); if (unlikely(!member)) goto bad;
      if (unlikely(PyDict_SetItem(res, __pyx_n_s_hour, member) < 0)) goto bad;
      Py_DECREF(member);
      member = __Pyx_PyInt_From_int(s.minute); if (unlikely(!member)) goto bad;
      if (unlikely(PyDict_SetItem(res, __pyx_n_s_minute, member) < 0)) goto bad;
      Py_DECREF(member);
      member = PyFloat_FromDouble(s.sec); if (unlikely(!member)) goto bad;
      if (unlikely(PyDict_SetItem(res, __pyx_n_s_sec, member) < 0)) goto bad;
      Py_DECREF(member);
      member = __Pyx_PyObject_FromString(s.radar_type); if (unlikely(!member)) goto bad;
      if (unlikely(PyDict_SetItem(res, __pyx_n_s_radar_type, member) < 0)) goto bad;
      Py_DECREF(member);
      member = __Pyx_PyInt_From_int(s.nvolumes); if (unlikely(!member)) goto bad;
      if (unlikely(PyDict_SetItem(res, __pyx_n_s_nvolumes, member) < 0)) goto bad;
      Py_DECREF(member);
      member = __Pyx_PyInt_From_int(s.number); if (unlikely(!member)) goto bad;
      if (unlikely(PyDict_SetItem(res, __pyx_n_s_number, member) < 0)) goto bad;
      Py_DECREF(member);
      member = __Pyx_PyObject_FromString(s.name); if (unlikely(!member)) goto bad;
      if (unlikely(PyDict_SetItem(res, __pyx_n_s_name_2, member) < 0)) goto bad;
      Py_DECREF(member);
      member = __Pyx_PyObject_FromString(s.radar_name); if (unlikely(!member)) goto bad;
      if (unlikely(PyDict_SetItem(res, __pyx_n_s_radar_name, member) < 0)) goto bad;
      Py_DECREF(member);
      member = __Pyx_PyObject_FromString(s.project); if (unlikely(!member)) goto bad;
      if (unlikely(PyDict_SetItem(res, __pyx_n_s_project, member) < 0)) goto bad;
      Py_DECREF(member);
      member = __Pyx_PyObject_FromString(s.city); if (unlikely(!member)) goto bad;
      if (unlikely(PyDict_SetItem(res, __pyx_n_s_city, member) < 0)) goto bad;
      Py_DECREF(member);
      member = __Pyx_PyObject_FromString(s.state); if (unlikely(!member)) goto bad;
      if (unlikely(PyDict_SetItem(res, __pyx_n_s_state, member) < 0)) goto bad;
      Py_DECREF(member);
      member = __Pyx_PyObject_FromString(s.country); if (unlikely(!member)) goto bad;
      if (unlikely(PyDict_SetItem(res, __pyx_n_s_country, member) < 0)) goto bad;
      Py_DECREF(member);
      member = __Pyx_PyInt_From_int(s.latd); if (unlikely(!member)) goto bad;
      if (unlikely(PyDict_SetItem(res, __pyx_n_s_latd, member) < 0)) goto bad;
      Py_DECREF(member);
      member = __Pyx_PyInt_From_int(s.latm); if (unlikely(!member)) goto bad;
      if (unlikely(PyDict_SetItem(res, __pyx_n_s_latm, member) < 0)) goto bad;
      Py_DECREF(member);
      member = __Pyx_PyInt_From_int(s.lats); if (unlikely(!member)) goto bad;
      if (unlikely(PyDict_SetItem(res, __pyx_n_s_lats, member) < 0)) goto bad;
      Py_DECREF(member);
      member = __Pyx_PyInt_From_int(s.lond); if (unlikely(!member)) goto bad;
      if (unlikely(PyDict_SetItem(res, __pyx_n_s_lond, member) < 0)) goto bad;
      Py_DECREF(member);
      member = __Pyx_PyInt_From_int(s.lonm); if (unlikely(!member)) goto bad;
      if (unlikely(PyDict_SetItem(res, __pyx_n_s_lonm, member) < 0)) goto bad;
      Py_DECREF(member);
      member = __Pyx_PyInt_From_int(s.lons); if (unlikely(!member)) goto bad;
      if (unlikely(PyDict_SetItem(res, __pyx_n_s_lons, member) < 0)) goto bad;
      Py_DECREF(member);
      member = __Pyx_PyInt_From_int(s.height); if (unlikely(!member)) goto bad;
      if (unlikely(PyDict_SetItem(res, __pyx_n_s_height, member) < 0)) goto bad;
      Py_DECREF(member);
      member = __Pyx_PyInt_From_int(s.spulse); if (unlikely(!member)) goto bad;
      if (unlikely(PyDict_SetItem(res, __pyx_n_s_spulse, member) < 0)) goto bad;
      Py_DECREF(member);
      member = __Pyx_PyInt_From_int(s.lpulse); if (unlikely(!member)) goto bad;
      if (unlikely(PyDict_SetItem(res, __pyx_n_s_lpulse, member) < 0)) goto bad;
      Py_DECREF(member);
      member = __Pyx_PyInt_From_int(s.scan_mode); if (unlikely(!member)) goto bad;
      if (unlikely(PyDict_SetItem(res, __pyx_n_s_scan_mode, member) < 0)) goto bad;
      Py_DECREF(member);
      member = __Pyx_PyInt_From_int(s.vcp); if (unlikely(!member)) goto bad;
      if (unlikely(PyDict_SetItem(res, __pyx_n_s_vcp, member) < 0)) goto bad;
      Py_DECREF(member);
      return res;
      bad:
      Py_XDECREF(member);
      Py_DECREF(res);
      return NULL;
    }
    /* Declarations */
    #if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
    #if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabsf(b.real) >= fabsf(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                float r = b.imag / b.real;
                float s = (float)(1.0) / (b.real + b.imag * r);
                return __pyx_t_float_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            float r = b.real / b.imag;
            float s = (float)(1.0) / (b.imag + b.real * r);
            return __pyx_t_float_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            float denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_float_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_float(a, a);
                    case 3:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, a);
                    case 4:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = powf(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2f(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_float(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

/* Declarations */
    #if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
    #if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabs(b.real) >= fabs(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                double r = b.imag / b.real;
                double s = (double)(1.0) / (b.real + b.imag * r);
                return __pyx_t_double_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            double r = b.real / b.imag;
            double s = (double)(1.0) / (b.imag + b.real * r);
            return __pyx_t_double_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            double denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_double_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_double(a, a);
                    case 3:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, a);
                    case 4:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = pow(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_double(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

/* CIntFromPy */
    static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE Range __Pyx_PyInt_As_Range(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const Range neg_one = (Range) -1, const_zero = (Range) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(Range) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(Range, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (Range) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (Range) 0;
                case  1: __PYX_VERIFY_RETURN_INT(Range, digit, digits[0])
                case 2:
                    if (8 * sizeof(Range) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Range, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Range) >= 2 * PyLong_SHIFT) {
                            return (Range) (((((Range)digits[1]) << PyLong_SHIFT) | (Range)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(Range) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Range, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Range) >= 3 * PyLong_SHIFT) {
                            return (Range) (((((((Range)digits[2]) << PyLong_SHIFT) | (Range)digits[1]) << PyLong_SHIFT) | (Range)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(Range) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Range, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Range) >= 4 * PyLong_SHIFT) {
                            return (Range) (((((((((Range)digits[3]) << PyLong_SHIFT) | (Range)digits[2]) << PyLong_SHIFT) | (Range)digits[1]) << PyLong_SHIFT) | (Range)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (Range) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(Range) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(Range, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(Range) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(Range, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (Range) 0;
                case -1: __PYX_VERIFY_RETURN_INT(Range, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(Range,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(Range) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Range, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Range) - 1 > 2 * PyLong_SHIFT) {
                            return (Range) (((Range)-1)*(((((Range)digits[1]) << PyLong_SHIFT) | (Range)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(Range) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Range, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Range) - 1 > 2 * PyLong_SHIFT) {
                            return (Range) ((((((Range)digits[1]) << PyLong_SHIFT) | (Range)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(Range) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Range, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Range) - 1 > 3 * PyLong_SHIFT) {
                            return (Range) (((Range)-1)*(((((((Range)digits[2]) << PyLong_SHIFT) | (Range)digits[1]) << PyLong_SHIFT) | (Range)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(Range) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Range, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Range) - 1 > 3 * PyLong_SHIFT) {
                            return (Range) ((((((((Range)digits[2]) << PyLong_SHIFT) | (Range)digits[1]) << PyLong_SHIFT) | (Range)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(Range) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Range, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Range) - 1 > 4 * PyLong_SHIFT) {
                            return (Range) (((Range)-1)*(((((((((Range)digits[3]) << PyLong_SHIFT) | (Range)digits[2]) << PyLong_SHIFT) | (Range)digits[1]) << PyLong_SHIFT) | (Range)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(Range) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Range, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Range) - 1 > 4 * PyLong_SHIFT) {
                            return (Range) ((((((((((Range)digits[3]) << PyLong_SHIFT) | (Range)digits[2]) << PyLong_SHIFT) | (Range)digits[1]) << PyLong_SHIFT) | (Range)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(Range) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(Range, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(Range) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(Range, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            Range val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (Range) -1;
        }
    } else {
        Range val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (Range) -1;
        val = __Pyx_PyInt_As_Range(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to Range");
    return (Range) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to Range");
    return (Range) -1;
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_Range(Range value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const Range neg_one = (Range) -1, const_zero = (Range) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(Range) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(Range) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(Range) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(Range) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(Range) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(Range),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_Py_intptr_t(Py_intptr_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const Py_intptr_t neg_one = (Py_intptr_t) -1, const_zero = (Py_intptr_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(Py_intptr_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(Py_intptr_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(Py_intptr_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(Py_intptr_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(Py_intptr_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(Py_intptr_t),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
    static CYTHON_INLINE Py_intptr_t __Pyx_PyInt_As_Py_intptr_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const Py_intptr_t neg_one = (Py_intptr_t) -1, const_zero = (Py_intptr_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(Py_intptr_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (Py_intptr_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (Py_intptr_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(Py_intptr_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(Py_intptr_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) >= 2 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((((Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(Py_intptr_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) >= 3 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((((((Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(Py_intptr_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) >= 4 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((((((((Py_intptr_t)digits[3]) << PyLong_SHIFT) | (Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (Py_intptr_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(Py_intptr_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(Py_intptr_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(Py_intptr_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(Py_intptr_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (Py_intptr_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(Py_intptr_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(Py_intptr_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(Py_intptr_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 2 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((Py_intptr_t)-1)*(((((Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(Py_intptr_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 2 * PyLong_SHIFT) {
                            return (Py_intptr_t) ((((((Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(Py_intptr_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 3 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((Py_intptr_t)-1)*(((((((Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(Py_intptr_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 3 * PyLong_SHIFT) {
                            return (Py_intptr_t) ((((((((Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(Py_intptr_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 4 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((Py_intptr_t)-1)*(((((((((Py_intptr_t)digits[3]) << PyLong_SHIFT) | (Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(Py_intptr_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 4 * PyLong_SHIFT) {
                            return (Py_intptr_t) ((((((((((Py_intptr_t)digits[3]) << PyLong_SHIFT) | (Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(Py_intptr_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(Py_intptr_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(Py_intptr_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(Py_intptr_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            Py_intptr_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (Py_intptr_t) -1;
        }
    } else {
        Py_intptr_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (Py_intptr_t) -1;
        val = __Pyx_PyInt_As_Py_intptr_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to Py_intptr_t");
    return (Py_intptr_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to Py_intptr_t");
    return (Py_intptr_t) -1;
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_npy_int32(npy_int32 value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const npy_int32 neg_one = (npy_int32) -1, const_zero = (npy_int32) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(npy_int32) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(npy_int32) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(npy_int32) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(npy_int32) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(npy_int32) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(npy_int32),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
    static CYTHON_INLINE unsigned short __Pyx_PyInt_As_unsigned_short(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const unsigned short neg_one = (unsigned short) -1, const_zero = (unsigned short) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(unsigned short) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(unsigned short, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (unsigned short) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned short) 0;
                case  1: __PYX_VERIFY_RETURN_INT(unsigned short, digit, digits[0])
                case 2:
                    if (8 * sizeof(unsigned short) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned short, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned short) >= 2 * PyLong_SHIFT) {
                            return (unsigned short) (((((unsigned short)digits[1]) << PyLong_SHIFT) | (unsigned short)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned short) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned short, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned short) >= 3 * PyLong_SHIFT) {
                            return (unsigned short) (((((((unsigned short)digits[2]) << PyLong_SHIFT) | (unsigned short)digits[1]) << PyLong_SHIFT) | (unsigned short)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned short) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned short, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned short) >= 4 * PyLong_SHIFT) {
                            return (unsigned short) (((((((((unsigned short)digits[3]) << PyLong_SHIFT) | (unsigned short)digits[2]) << PyLong_SHIFT) | (unsigned short)digits[1]) << PyLong_SHIFT) | (unsigned short)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (unsigned short) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(unsigned short) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned short, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned short) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned short, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned short) 0;
                case -1: __PYX_VERIFY_RETURN_INT(unsigned short, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(unsigned short,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(unsigned short) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned short, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned short) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned short) (((unsigned short)-1)*(((((unsigned short)digits[1]) << PyLong_SHIFT) | (unsigned short)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(unsigned short) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned short, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned short) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned short) ((((((unsigned short)digits[1]) << PyLong_SHIFT) | (unsigned short)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(unsigned short) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned short, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned short) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned short) (((unsigned short)-1)*(((((((unsigned short)digits[2]) << PyLong_SHIFT) | (unsigned short)digits[1]) << PyLong_SHIFT) | (unsigned short)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned short) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned short, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned short) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned short) ((((((((unsigned short)digits[2]) << PyLong_SHIFT) | (unsigned short)digits[1]) << PyLong_SHIFT) | (unsigned short)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(unsigned short) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned short, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned short) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned short) (((unsigned short)-1)*(((((((((unsigned short)digits[3]) << PyLong_SHIFT) | (unsigned short)digits[2]) << PyLong_SHIFT) | (unsigned short)digits[1]) << PyLong_SHIFT) | (unsigned short)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned short) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned short, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned short) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned short) ((((((((((unsigned short)digits[3]) << PyLong_SHIFT) | (unsigned short)digits[2]) << PyLong_SHIFT) | (unsigned short)digits[1]) << PyLong_SHIFT) | (unsigned short)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(unsigned short) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned short, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned short) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned short, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            unsigned short val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (unsigned short) -1;
        }
    } else {
        unsigned short val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (unsigned short) -1;
        val = __Pyx_PyInt_As_unsigned_short(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to unsigned short");
    return (unsigned short) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to unsigned short");
    return (unsigned short) -1;
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_short(unsigned short value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const unsigned short neg_one = (unsigned short) -1, const_zero = (unsigned short) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(unsigned short) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(unsigned short) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned short) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(unsigned short) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned short) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(unsigned short),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
    static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FastTypeChecks */
    #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
    static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* InitStrings */
    static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
#if PY_MAJOR_VERSION < 3
  } else if (likely(PyInt_CheckExact(o))) {
    return PyInt_AS_LONG(o);
#endif
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyInt_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
